# Generated by Django 3.1.7 on 2021-10-19 16:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_nonlocalizedevents', '0007_eventcandidate_viable'),
    ]

    operations = [
        migrations.AddField(
            model_name='eventcandidate',
            name='priority',
            field=models.IntegerField(default=1),
        ),
    ]
