# Generated by Django 4.2 on 2023-04-28 20:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tom_nonlocalizedevents', '0016_externalcoincidence_alter_eventsequence_options_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventsequence',
            name='external_coincidence',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sequences', to='tom_nonlocalizedevents.externalcoincidence'),
        ),
        migrations.AlterField(
            model_name='eventsequence',
            name='ingestor_source',
            field=models.CharField(default='', help_text='The source of this alert - i.e. which stream is it from?', max_length=64),
        ),
    ]
