# Generated by Django 3.1.5 on 2021-02-12 19:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_nonlocalizedevents', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Superevent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('superevent_id', models.CharField(max_length=64)),
                ('superevent_url', models.URLField()),
            ],
        ),
    ]
