# Generated by Django 3.1.7 on 2021-10-05 20:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_nonlocalizedevents', '0006_auto_20210827_2346'),
    ]

    operations = [
        migrations.AddField(
            model_name='eventcandidate',
            name='viable',
            field=models.BooleanField(default=True),
        ),
    ]
