# Generated by Django 3.2.6 on 2021-08-27 23:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tom_targets', '0019_auto_20210811_0018'),
        ('tom_nonlocalizedevents', '0005_auto_20210319_2241'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventCandidate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('superevent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='tom_nonlocalizedevents.superevent')),
                ('target', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='tom_targets.target')),
            ],
        ),
        migrations.AddConstraint(
            model_name='eventcandidate',
            constraint=models.UniqueConstraint(fields=('target', 'superevent'), name='Unique Target/Superevent'),
        ),
    ]
