# Generated by Django 3.1.7 on 2021-03-19 22:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_nonlocalizedevents', '0004_superevent_superevent_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='superevent',
            name='superevent_type',
            field=models.CharField(choices=[('GW', 'Gravitational Wave'), ('GRB', 'Gamma-ray Burst'), ('NU', 'Neutrino'), ('UNK', 'Unknown')], default='GW', max_length=3),
        ),
    ]
