{% extends 'tom_common/base.html' %}
{% load render_bundle from webpack_loader %}


{% block content %}

    <div id="superevent_identifier" style="display: none">{{ superevent_id }}</div>

    <div id="superevent-sequences">
        <superevent-sequences superevent="{{ superevent_data }}" tom_api_url="{{ tom_api_url }}" hermes_api_url="{{ hermes_api_url }}"></superevent-sequences>
    </div>

    {% render_bundle 'superevent_vue_app' config='TOM_NONLOCALIZEDEVENTS' %}

{% endblock %}
