# Generated by Django 4.1.1 on 2022-10-13 00:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tom_nonlocalizedevents', '0008_eventcandidate_priority'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Superevent',
            new_name='NonLocalizedEvent',
        ),
    ]
