# Generated by Django 4.1.1 on 2022-10-13 00:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tom_nonlocalizedevents', '0009_rename_superevent_nonlocalizedevent'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='eventcandidate',
            name='Unique Target/Superevent',
        ),
        migrations.RenameField(
            model_name='eventcandidate',
            old_name='superevent',
            new_name='nonlocalizedevent',
        ),
        migrations.RenameField(
            model_name='nonlocalizedevent',
            old_name='superevent_id',
            new_name='event_id',
        ),
        migrations.RenameField(
            model_name='nonlocalizedevent',
            old_name='superevent_type',
            new_name='event_type',
        ),
        migrations.RenameField(
            model_name='nonlocalizedevent',
            old_name='superevent_url',
            new_name='skymap_file_url',
        ),
        migrations.AddField(
            model_name='nonlocalizedevent',
            name='sequence_id',
            field=models.PositiveIntegerField(default=1),
        ),
        migrations.AddConstraint(
            model_name='eventcandidate',
            constraint=models.UniqueConstraint(fields=('target', 'nonlocalizedevent'), name='Unique Target/NonLocalizedEvent'),
        ),
    ]
