(function(e){var t={};function a(n){if(t[n])return t[n].exports;var r=t[n]={i:n,l:!1,exports:{}};return e[n].call(r.exports,r,r.exports,a),r.l=!0,r.exports}a.m=e,a.c=t,a.d=function(e,t,n){a.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:n})},a.r=function(e){"undefined"!==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},a.t=function(e,t){if(1&t&&(e=a(e)),8&t)return e;if(4&t&&"object"===typeof e&&e&&e.__esModule)return e;var n=Object.create(null);if(a.r(n),Object.defineProperty(n,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var r in e)a.d(n,r,function(t){return e[t]}.bind(null,r));return n},a.n=function(e){var t=e&&e.__esModule?function(){return e["default"]}:function(){return e};return a.d(t,"a",t),t},a.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},a.p="",a(a.s=0)})({0:function(e,t,a){e.exports=a("3172")},"010e":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("uz-latn",{months:"Yanvar_Fevral_Mart_Aprel_May_Iyun_Iyul_Avgust_Sentabr_Oktabr_Noyabr_Dekabr".split("_"),monthsShort:"Yan_Fev_Mar_Apr_May_Iyun_Iyul_Avg_Sen_Okt_Noy_Dek".split("_"),weekdays:"Yakshanba_Dushanba_Seshanba_Chorshanba_Payshanba_Juma_Shanba".split("_"),weekdaysShort:"Yak_Dush_Sesh_Chor_Pay_Jum_Shan".split("_"),weekdaysMin:"Ya_Du_Se_Cho_Pa_Ju_Sha".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Bugun soat] LT [da]",nextDay:"[Ertaga] LT [da]",nextWeek:"dddd [kuni soat] LT [da]",lastDay:"[Kecha soat] LT [da]",lastWeek:"[O'tgan] dddd [kuni soat] LT [da]",sameElse:"L"},relativeTime:{future:"Yaqin %s ichida",past:"Bir necha %s oldin",s:"soniya",ss:"%d soniya",m:"bir daqiqa",mm:"%d daqiqa",h:"bir soat",hh:"%d soat",d:"bir kun",dd:"%d kun",M:"bir oy",MM:"%d oy",y:"bir yil",yy:"%d yil"},week:{dow:1,doy:7}});return t}))},"02fb":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ml",{months:"ജനുവരി_ഫെബ്രുവരി_മാർച്ച്_ഏപ്രിൽ_മേയ്_ജൂൺ_ജൂലൈ_ഓഗസ്റ്റ്_സെപ്റ്റംബർ_ഒക്ടോബർ_നവംബർ_ഡിസംബർ".split("_"),monthsShort:"ജനു._ഫെബ്രു._മാർ._ഏപ്രി._മേയ്_ജൂൺ_ജൂലൈ._ഓഗ._സെപ്റ്റ._ഒക്ടോ._നവം._ഡിസം.".split("_"),monthsParseExact:!0,weekdays:"ഞായറാഴ്ച_തിങ്കളാഴ്ച_ചൊവ്വാഴ്ച_ബുധനാഴ്ച_വ്യാഴാഴ്ച_വെള്ളിയാഴ്ച_ശനിയാഴ്ച".split("_"),weekdaysShort:"ഞായർ_തിങ്കൾ_ചൊവ്വ_ബുധൻ_വ്യാഴം_വെള്ളി_ശനി".split("_"),weekdaysMin:"ഞാ_തി_ചൊ_ബു_വ്യാ_വെ_ശ".split("_"),longDateFormat:{LT:"A h:mm -നു",LTS:"A h:mm:ss -നു",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm -നു",LLLL:"dddd, D MMMM YYYY, A h:mm -നു"},calendar:{sameDay:"[ഇന്ന്] LT",nextDay:"[നാളെ] LT",nextWeek:"dddd, LT",lastDay:"[ഇന്നലെ] LT",lastWeek:"[കഴിഞ്ഞ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s കഴിഞ്ഞ്",past:"%s മുൻപ്",s:"അൽപ നിമിഷങ്ങൾ",ss:"%d സെക്കൻഡ്",m:"ഒരു മിനിറ്റ്",mm:"%d മിനിറ്റ്",h:"ഒരു മണിക്കൂർ",hh:"%d മണിക്കൂർ",d:"ഒരു ദിവസം",dd:"%d ദിവസം",M:"ഒരു മാസം",MM:"%d മാസം",y:"ഒരു വർഷം",yy:"%d വർഷം"},meridiemParse:/രാത്രി|രാവിലെ|ഉച്ച കഴിഞ്ഞ്|വൈകുന്നേരം|രാത്രി/i,meridiemHour:function(e,t){return 12===e&&(e=0),"രാത്രി"===t&&e>=4||"ഉച്ച കഴിഞ്ഞ്"===t||"വൈകുന്നേരം"===t?e+12:e},meridiem:function(e,t,a){return e<4?"രാത്രി":e<12?"രാവിലെ":e<17?"ഉച്ച കഴിഞ്ഞ്":e<20?"വൈകുന്നേരം":"രാത്രി"}});return t}))},"03ec":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("cv",{months:"кӑрлач_нарӑс_пуш_ака_май_ҫӗртме_утӑ_ҫурла_авӑн_юпа_чӳк_раштав".split("_"),monthsShort:"кӑр_нар_пуш_ака_май_ҫӗр_утӑ_ҫур_авн_юпа_чӳк_раш".split("_"),weekdays:"вырсарникун_тунтикун_ытларикун_юнкун_кӗҫнерникун_эрнекун_шӑматкун".split("_"),weekdaysShort:"выр_тун_ытл_юн_кӗҫ_эрн_шӑм".split("_"),weekdaysMin:"вр_тн_ыт_юн_кҫ_эр_шм".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ]",LLL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm",LLLL:"dddd, YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm"},calendar:{sameDay:"[Паян] LT [сехетре]",nextDay:"[Ыран] LT [сехетре]",lastDay:"[Ӗнер] LT [сехетре]",nextWeek:"[Ҫитес] dddd LT [сехетре]",lastWeek:"[Иртнӗ] dddd LT [сехетре]",sameElse:"L"},relativeTime:{future:function(e){var t=/сехет$/i.exec(e)?"рен":/ҫул$/i.exec(e)?"тан":"ран";return e+t},past:"%s каялла",s:"пӗр-ик ҫеккунт",ss:"%d ҫеккунт",m:"пӗр минут",mm:"%d минут",h:"пӗр сехет",hh:"%d сехет",d:"пӗр кун",dd:"%d кун",M:"пӗр уйӑх",MM:"%d уйӑх",y:"пӗр ҫул",yy:"%d ҫул"},dayOfMonthOrdinalParse:/\d{1,2}-мӗш/,ordinal:"%d-мӗш",week:{dow:1,doy:7}});return t}))},"04f8":function(e,t,a){var n=a("2d00"),r=a("d039");e.exports=!!Object.getOwnPropertySymbols&&!r((function(){var e=Symbol();return!String(e)||!(Object(e)instanceof Symbol)||!Symbol.sham&&n&&n<41}))},"0558":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e){return e%100===11||e%10!==1}function a(e,a,n,r){var i=e+" ";switch(n){case"s":return a||r?"nokkrar sekúndur":"nokkrum sekúndum";case"ss":return t(e)?i+(a||r?"sekúndur":"sekúndum"):i+"sekúnda";case"m":return a?"mínúta":"mínútu";case"mm":return t(e)?i+(a||r?"mínútur":"mínútum"):a?i+"mínúta":i+"mínútu";case"hh":return t(e)?i+(a||r?"klukkustundir":"klukkustundum"):i+"klukkustund";case"d":return a?"dagur":r?"dag":"degi";case"dd":return t(e)?a?i+"dagar":i+(r?"daga":"dögum"):a?i+"dagur":i+(r?"dag":"degi");case"M":return a?"mánuður":r?"mánuð":"mánuði";case"MM":return t(e)?a?i+"mánuðir":i+(r?"mánuði":"mánuðum"):a?i+"mánuður":i+(r?"mánuð":"mánuði");case"y":return a||r?"ár":"ári";case"yy":return t(e)?i+(a||r?"ár":"árum"):i+(a||r?"ár":"ári")}}var n=e.defineLocale("is",{months:"janúar_febrúar_mars_apríl_maí_júní_júlí_ágúst_september_október_nóvember_desember".split("_"),monthsShort:"jan_feb_mar_apr_maí_jún_júl_ágú_sep_okt_nóv_des".split("_"),weekdays:"sunnudagur_mánudagur_þriðjudagur_miðvikudagur_fimmtudagur_föstudagur_laugardagur".split("_"),weekdaysShort:"sun_mán_þri_mið_fim_fös_lau".split("_"),weekdaysMin:"Su_Má_Þr_Mi_Fi_Fö_La".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd, D. MMMM YYYY [kl.] H:mm"},calendar:{sameDay:"[í dag kl.] LT",nextDay:"[á morgun kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[í gær kl.] LT",lastWeek:"[síðasta] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"eftir %s",past:"fyrir %s síðan",s:a,ss:a,m:a,mm:a,h:"klukkustund",hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return n}))},"06cf":function(e,t,a){var n=a("83ab"),r=a("c65b"),i=a("d1e7"),o=a("5c6c"),s=a("fc6a"),l=a("a04b"),c=a("1a2d"),u=a("0cfb"),d=Object.getOwnPropertyDescriptor;t.f=n?d:function(e,t){if(e=s(e),t=l(t),u)try{return d(e,t)}catch(a){}if(c(e,t))return o(!r(i.f,e,t),e[t])}},"0721":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fo",{months:"januar_februar_mars_apríl_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sunnudagur_mánadagur_týsdagur_mikudagur_hósdagur_fríggjadagur_leygardagur".split("_"),weekdaysShort:"sun_mán_týs_mik_hós_frí_ley".split("_"),weekdaysMin:"su_má_tý_mi_hó_fr_le".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D. MMMM, YYYY HH:mm"},calendar:{sameDay:"[Í dag kl.] LT",nextDay:"[Í morgin kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[Í gjár kl.] LT",lastWeek:"[síðstu] dddd [kl] LT",sameElse:"L"},relativeTime:{future:"um %s",past:"%s síðani",s:"fá sekund",ss:"%d sekundir",m:"ein minuttur",mm:"%d minuttir",h:"ein tími",hh:"%d tímar",d:"ein dagur",dd:"%d dagar",M:"ein mánaður",MM:"%d mánaðir",y:"eitt ár",yy:"%d ár"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},"079e":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ja",{eras:[{since:"2019-05-01",offset:1,name:"令和",narrow:"㋿",abbr:"R"},{since:"1989-01-08",until:"2019-04-30",offset:1,name:"平成",narrow:"㍻",abbr:"H"},{since:"1926-12-25",until:"1989-01-07",offset:1,name:"昭和",narrow:"㍼",abbr:"S"},{since:"1912-07-30",until:"1926-12-24",offset:1,name:"大正",narrow:"㍽",abbr:"T"},{since:"1873-01-01",until:"1912-07-29",offset:6,name:"明治",narrow:"㍾",abbr:"M"},{since:"0001-01-01",until:"1873-12-31",offset:1,name:"西暦",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"紀元前",narrow:"BC",abbr:"BC"}],eraYearOrdinalRegex:/(元|\d+)年/,eraYearOrdinalParse:function(e,t){return"元"===t[1]?1:parseInt(t[1]||e,10)},months:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"日曜日_月曜日_火曜日_水曜日_木曜日_金曜日_土曜日".split("_"),weekdaysShort:"日_月_火_水_木_金_土".split("_"),weekdaysMin:"日_月_火_水_木_金_土".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日 dddd HH:mm",l:"YYYY/MM/DD",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日(ddd) HH:mm"},meridiemParse:/午前|午後/i,isPM:function(e){return"午後"===e},meridiem:function(e,t,a){return e<12?"午前":"午後"},calendar:{sameDay:"[今日] LT",nextDay:"[明日] LT",nextWeek:function(e){return e.week()!==this.week()?"[来週]dddd LT":"dddd LT"},lastDay:"[昨日] LT",lastWeek:function(e){return this.week()!==e.week()?"[先週]dddd LT":"dddd LT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}日/,ordinal:function(e,t){switch(t){case"y":return 1===e?"元年":e+"年";case"d":case"D":case"DDD":return e+"日";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"数秒",ss:"%d秒",m:"1分",mm:"%d分",h:"1時間",hh:"%d時間",d:"1日",dd:"%d日",M:"1ヶ月",MM:"%dヶ月",y:"1年",yy:"%d年"}});return t}))},"07fa":function(e,t,a){var n=a("50c4");e.exports=function(e){return n(e.length)}},"0a06":function(e,t,a){"use strict";var n=a("c532"),r=a("30b5"),i=a("f6b49"),o=a("5270"),s=a("4a7b"),l=a("848b"),c=l.validators;function u(e){this.defaults=e,this.interceptors={request:new i,response:new i}}u.prototype.request=function(e){"string"===typeof e?(e=arguments[1]||{},e.url=arguments[0]):e=e||{},e=s(this.defaults,e),e.method?e.method=e.method.toLowerCase():this.defaults.method?e.method=this.defaults.method.toLowerCase():e.method="get";var t=e.transitional;void 0!==t&&l.assertOptions(t,{silentJSONParsing:c.transitional(c.boolean,"1.0.0"),forcedJSONParsing:c.transitional(c.boolean,"1.0.0"),clarifyTimeoutError:c.transitional(c.boolean,"1.0.0")},!1);var a=[],n=!0;this.interceptors.request.forEach((function(t){"function"===typeof t.runWhen&&!1===t.runWhen(e)||(n=n&&t.synchronous,a.unshift(t.fulfilled,t.rejected))}));var r,i=[];if(this.interceptors.response.forEach((function(e){i.push(e.fulfilled,e.rejected)})),!n){var u=[o,void 0];Array.prototype.unshift.apply(u,a),u=u.concat(i),r=Promise.resolve(e);while(u.length)r=r.then(u.shift(),u.shift());return r}var d=e;while(a.length){var h=a.shift(),f=a.shift();try{d=h(d)}catch(p){f(p);break}}try{r=o(d)}catch(p){return Promise.reject(p)}while(i.length)r=r.then(i.shift(),i.shift());return r},u.prototype.getUri=function(e){return e=s(this.defaults,e),r(e.url,e.params,e.paramsSerializer).replace(/^\?/,"")},n.forEach(["delete","get","head","options"],(function(e){u.prototype[e]=function(t,a){return this.request(s(a||{},{method:e,url:t,data:(a||{}).data}))}})),n.forEach(["post","put","patch"],(function(e){u.prototype[e]=function(t,a,n){return this.request(s(n||{},{method:e,url:t,data:a}))}})),e.exports=u},"0a3c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),a="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),n=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],r=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,i=e.defineLocale("es-do",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,n){return e?/-MMM-/.test(n)?a[e.month()]:t[e.month()]:t},monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return i}))},"0a84":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ar-ma",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اثنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,doy:4}});return t}))},"0caa":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r={s:["thoddea sekondamni","thodde sekond"],ss:[e+" sekondamni",e+" sekond"],m:["eka mintan","ek minut"],mm:[e+" mintamni",e+" mintam"],h:["eka voran","ek vor"],hh:[e+" voramni",e+" voram"],d:["eka disan","ek dis"],dd:[e+" disamni",e+" dis"],M:["eka mhoinean","ek mhoino"],MM:[e+" mhoineamni",e+" mhoine"],y:["eka vorsan","ek voros"],yy:[e+" vorsamni",e+" vorsam"]};return n?r[a][0]:r[a][1]}var a=e.defineLocale("gom-latn",{months:{standalone:"Janer_Febrer_Mars_Abril_Mai_Jun_Julai_Agost_Setembr_Otubr_Novembr_Dezembr".split("_"),format:"Janerachea_Febrerachea_Marsachea_Abrilachea_Maiachea_Junachea_Julaiachea_Agostachea_Setembrachea_Otubrachea_Novembrachea_Dezembrachea".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"Jan._Feb._Mars_Abr._Mai_Jun_Jul._Ago._Set._Otu._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Aitar_Somar_Mongllar_Budhvar_Birestar_Sukrar_Son'var".split("_"),weekdaysShort:"Ait._Som._Mon._Bud._Bre._Suk._Son.".split("_"),weekdaysMin:"Ai_Sm_Mo_Bu_Br_Su_Sn".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [vazta]",LTS:"A h:mm:ss [vazta]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [vazta]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [vazta]",llll:"ddd, D MMM YYYY, A h:mm [vazta]"},calendar:{sameDay:"[Aiz] LT",nextDay:"[Faleam] LT",nextWeek:"[Fuddlo] dddd[,] LT",lastDay:"[Kal] LT",lastWeek:"[Fattlo] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s adim",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}(er)/,ordinal:function(e,t){switch(t){case"D":return e+"er";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return e}},week:{dow:0,doy:3},meridiemParse:/rati|sokallim|donparam|sanje/,meridiemHour:function(e,t){return 12===e&&(e=0),"rati"===t?e<4?e:e+12:"sokallim"===t?e:"donparam"===t?e>12?e:e+12:"sanje"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"rati":e<12?"sokallim":e<16?"donparam":e<20?"sanje":"rati"}});return a}))},"0cfb":function(e,t,a){var n=a("83ab"),r=a("d039"),i=a("cc12");e.exports=!n&&!r((function(){return 7!=Object.defineProperty(i("div"),"a",{get:function(){return 7}}).a}))},"0d51":function(e,t){var a=String;e.exports=function(e){try{return a(e)}catch(t){return"Object"}}},"0df6":function(e,t,a){"use strict";e.exports=function(e){return function(t){return e.apply(null,t)}}},"0e49":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fr-ch",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(e,t){switch(t){default:case"M":case"Q":case"D":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}},week:{dow:1,doy:4}});return t}))},"0e6b":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-au",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:0,doy:4}});return t}))},"0e81":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"'inci",5:"'inci",8:"'inci",70:"'inci",80:"'inci",2:"'nci",7:"'nci",20:"'nci",50:"'nci",3:"'üncü",4:"'üncü",100:"'üncü",6:"'ncı",9:"'uncu",10:"'uncu",30:"'uncu",60:"'ıncı",90:"'ıncı"},a=e.defineLocale("tr",{months:"Ocak_Şubat_Mart_Nisan_Mayıs_Haziran_Temmuz_Ağustos_Eylül_Ekim_Kasım_Aralık".split("_"),monthsShort:"Oca_Şub_Mar_Nis_May_Haz_Tem_Ağu_Eyl_Eki_Kas_Ara".split("_"),weekdays:"Pazar_Pazartesi_Salı_Çarşamba_Perşembe_Cuma_Cumartesi".split("_"),weekdaysShort:"Paz_Pzt_Sal_Çar_Per_Cum_Cmt".split("_"),weekdaysMin:"Pz_Pt_Sa_Ça_Pe_Cu_Ct".split("_"),meridiem:function(e,t,a){return e<12?a?"öö":"ÖÖ":a?"ös":"ÖS"},meridiemParse:/öö|ÖÖ|ös|ÖS/,isPM:function(e){return"ös"===e||"ÖS"===e},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[yarın saat] LT",nextWeek:"[gelecek] dddd [saat] LT",lastDay:"[dün] LT",lastWeek:"[geçen] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s önce",s:"birkaç saniye",ss:"%d saniye",m:"bir dakika",mm:"%d dakika",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",w:"bir hafta",ww:"%d hafta",M:"bir ay",MM:"%d ay",y:"bir yıl",yy:"%d yıl"},ordinal:function(e,a){switch(a){case"d":case"D":case"Do":case"DD":return e;default:if(0===e)return e+"'ıncı";var n=e%10,r=e%100-n,i=e>=100?100:null;return e+(t[n]||t[r]||t[i])}},week:{dow:1,doy:7}});return a}))},"0f14":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("da",{months:"januar_februar_marts_april_maj_juni_juli_august_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"søn_man_tir_ons_tor_fre_lør".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd [d.] D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"på dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[i] dddd[s kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"få sekunder",ss:"%d sekunder",m:"et minut",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dage",M:"en måned",MM:"%d måneder",y:"et år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},"0f38":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tl-ph",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}});return t}))},"0ff2":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("eu",{months:"urtarrila_otsaila_martxoa_apirila_maiatza_ekaina_uztaila_abuztua_iraila_urria_azaroa_abendua".split("_"),monthsShort:"urt._ots._mar._api._mai._eka._uzt._abu._ira._urr._aza._abe.".split("_"),monthsParseExact:!0,weekdays:"igandea_astelehena_asteartea_asteazkena_osteguna_ostirala_larunbata".split("_"),weekdaysShort:"ig._al._ar._az._og._ol._lr.".split("_"),weekdaysMin:"ig_al_ar_az_og_ol_lr".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY[ko] MMMM[ren] D[a]",LLL:"YYYY[ko] MMMM[ren] D[a] HH:mm",LLLL:"dddd, YYYY[ko] MMMM[ren] D[a] HH:mm",l:"YYYY-M-D",ll:"YYYY[ko] MMM D[a]",lll:"YYYY[ko] MMM D[a] HH:mm",llll:"ddd, YYYY[ko] MMM D[a] HH:mm"},calendar:{sameDay:"[gaur] LT[etan]",nextDay:"[bihar] LT[etan]",nextWeek:"dddd LT[etan]",lastDay:"[atzo] LT[etan]",lastWeek:"[aurreko] dddd LT[etan]",sameElse:"L"},relativeTime:{future:"%s barru",past:"duela %s",s:"segundo batzuk",ss:"%d segundo",m:"minutu bat",mm:"%d minutu",h:"ordu bat",hh:"%d ordu",d:"egun bat",dd:"%d egun",M:"hilabete bat",MM:"%d hilabete",y:"urte bat",yy:"%d urte"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return t}))},"10e8":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("th",{months:"มกราคม_กุมภาพันธ์_มีนาคม_เมษายน_พฤษภาคม_มิถุนายน_กรกฎาคม_สิงหาคม_กันยายน_ตุลาคม_พฤศจิกายน_ธันวาคม".split("_"),monthsShort:"ม.ค._ก.พ._มี.ค._เม.ย._พ.ค._มิ.ย._ก.ค._ส.ค._ก.ย._ต.ค._พ.ย._ธ.ค.".split("_"),monthsParseExact:!0,weekdays:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัสบดี_ศุกร์_เสาร์".split("_"),weekdaysShort:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัส_ศุกร์_เสาร์".split("_"),weekdaysMin:"อา._จ._อ._พ._พฤ._ศ._ส.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY เวลา H:mm",LLLL:"วันddddที่ D MMMM YYYY เวลา H:mm"},meridiemParse:/ก่อนเที่ยง|หลังเที่ยง/,isPM:function(e){return"หลังเที่ยง"===e},meridiem:function(e,t,a){return e<12?"ก่อนเที่ยง":"หลังเที่ยง"},calendar:{sameDay:"[วันนี้ เวลา] LT",nextDay:"[พรุ่งนี้ เวลา] LT",nextWeek:"dddd[หน้า เวลา] LT",lastDay:"[เมื่อวานนี้ เวลา] LT",lastWeek:"[วัน]dddd[ที่แล้ว เวลา] LT",sameElse:"L"},relativeTime:{future:"อีก %s",past:"%sที่แล้ว",s:"ไม่กี่วินาที",ss:"%d วินาที",m:"1 นาที",mm:"%d นาที",h:"1 ชั่วโมง",hh:"%d ชั่วโมง",d:"1 วัน",dd:"%d วัน",w:"1 สัปดาห์",ww:"%d สัปดาห์",M:"1 เดือน",MM:"%d เดือน",y:"1 ปี",yy:"%d ปี"}});return t}))},"13d2":function(e,t,a){var n=a("d039"),r=a("1626"),i=a("1a2d"),o=a("83ab"),s=a("5e77").CONFIGURABLE,l=a("8925"),c=a("69f3"),u=c.enforce,d=c.get,h=Object.defineProperty,f=o&&!n((function(){return 8!==h((function(){}),"length",{value:8}).length})),p=String(String).split("String"),m=e.exports=function(e,t,a){"Symbol("===String(t).slice(0,7)&&(t="["+String(t).replace(/^Symbol\(([^)]*)\)/,"$1")+"]"),a&&a.getter&&(t="get "+t),a&&a.setter&&(t="set "+t),(!i(e,"name")||s&&e.name!==t)&&(o?h(e,"name",{value:t,configurable:!0}):e.name=t),f&&a&&i(a,"arity")&&e.length!==a.arity&&h(e,"length",{value:a.arity});try{a&&i(a,"constructor")&&a.constructor?o&&h(e,"prototype",{writable:!1}):e.prototype&&(e.prototype=void 0)}catch(r){}var n=u(e);return i(n,"source")||(n.source=p.join("string"==typeof t?t:"")),e};Function.prototype.toString=m((function(){return r(this)&&d(this).source||l(this)}),"toString")},"13e9":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["секунда","секунде","секунди"],m:["један минут","једног минута"],mm:["минут","минута","минута"],h:["један сат","једног сата"],hh:["сат","сата","сати"],d:["један дан","једног дана"],dd:["дан","дана","дана"],M:["један месец","једног месеца"],MM:["месец","месеца","месеци"],y:["једну годину","једне године"],yy:["годину","године","година"]},correctGrammaticalCase:function(e,t){return e%10>=1&&e%10<=4&&(e%100<10||e%100>=20)?e%10===1?t[0]:t[1]:t[2]},translate:function(e,a,n,r){var i,o=t.words[n];return 1===n.length?"y"===n&&a?"једна година":r||a?o[0]:o[1]:(i=t.correctGrammaticalCase(e,o),"yy"===n&&a&&"годину"===i?e+" година":e+" "+i)}},a=e.defineLocale("sr-cyrl",{months:"јануар_фебруар_март_април_мај_јун_јул_август_септембар_октобар_новембар_децембар".split("_"),monthsShort:"јан._феб._мар._апр._мај_јун_јул_авг._сеп._окт._нов._дец.".split("_"),monthsParseExact:!0,weekdays:"недеља_понедељак_уторак_среда_четвртак_петак_субота".split("_"),weekdaysShort:"нед._пон._уто._сре._чет._пет._суб.".split("_"),weekdaysMin:"не_по_ут_ср_че_пе_су".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[данас у] LT",nextDay:"[сутра у] LT",nextWeek:function(){switch(this.day()){case 0:return"[у] [недељу] [у] LT";case 3:return"[у] [среду] [у] LT";case 6:return"[у] [суботу] [у] LT";case 1:case 2:case 4:case 5:return"[у] dddd [у] LT"}},lastDay:"[јуче у] LT",lastWeek:function(){var e=["[прошле] [недеље] [у] LT","[прошлог] [понедељка] [у] LT","[прошлог] [уторка] [у] LT","[прошле] [среде] [у] LT","[прошлог] [четвртка] [у] LT","[прошлог] [петка] [у] LT","[прошле] [суботе] [у] LT"];return e[this.day()]},sameElse:"L"},relativeTime:{future:"за %s",past:"пре %s",s:"неколико секунди",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:t.translate,dd:t.translate,M:t.translate,MM:t.translate,y:t.translate,yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return a}))},"14d9":function(e,t,a){"use strict";var n=a("23e7"),r=a("7b0b"),i=a("07fa"),o=a("3a34"),s=a("3511"),l=a("d039"),c=l((function(){return 4294967297!==[].push.call({length:4294967296},1)})),u=!function(){try{Object.defineProperty([],"length",{writable:!1}).push()}catch(e){return e instanceof TypeError}}();n({target:"Array",proto:!0,arity:1,forced:c||u},{push:function(e){var t=r(this),a=i(t),n=arguments.length;s(a+n);for(var l=0;l<n;l++)t[a]=arguments[l],a++;return o(t,a),a}})},1626:function(e,t,a){var n=a("8ea1"),r=n.all;e.exports=n.IS_HTMLDDA?function(e){return"function"==typeof e||e===r}:function(e){return"function"==typeof e}},"167b":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("oc-lnc",{months:{standalone:"genièr_febrièr_març_abril_mai_junh_julhet_agost_setembre_octòbre_novembre_decembre".split("_"),format:"de genièr_de febrièr_de març_d'abril_de mai_de junh_de julhet_d'agost_de setembre_d'octòbre_de novembre_de decembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._març_abr._mai_junh_julh._ago._set._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"dimenge_diluns_dimars_dimècres_dijòus_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dm._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dm_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:"[uèi a] LT",nextDay:"[deman a] LT",nextWeek:"dddd [a] LT",lastDay:"[ièr a] LT",lastWeek:"dddd [passat a] LT",sameElse:"L"},relativeTime:{future:"d'aquí %s",past:"fa %s",s:"unas segondas",ss:"%d segondas",m:"una minuta",mm:"%d minutas",h:"una ora",hh:"%d oras",d:"un jorn",dd:"%d jorns",M:"un mes",MM:"%d meses",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(e,t){var a=1===e?"r":2===e?"n":3===e?"r":4===e?"t":"è";return"w"!==t&&"W"!==t||(a="a"),e+a},week:{dow:1,doy:4}});return t}))},"16d2":function(e,t,a){},"1a2d":function(e,t,a){var n=a("e330"),r=a("7b0b"),i=n({}.hasOwnProperty);e.exports=Object.hasOwn||function(e,t){return i(r(e),t)}},"1b45":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("mt",{months:"Jannar_Frar_Marzu_April_Mejju_Ġunju_Lulju_Awwissu_Settembru_Ottubru_Novembru_Diċembru".split("_"),monthsShort:"Jan_Fra_Mar_Apr_Mej_Ġun_Lul_Aww_Set_Ott_Nov_Diċ".split("_"),weekdays:"Il-Ħadd_It-Tnejn_It-Tlieta_L-Erbgħa_Il-Ħamis_Il-Ġimgħa_Is-Sibt".split("_"),weekdaysShort:"Ħad_Tne_Tli_Erb_Ħam_Ġim_Sib".split("_"),weekdaysMin:"Ħa_Tn_Tl_Er_Ħa_Ġi_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Illum fil-]LT",nextDay:"[Għada fil-]LT",nextWeek:"dddd [fil-]LT",lastDay:"[Il-bieraħ fil-]LT",lastWeek:"dddd [li għadda] [fil-]LT",sameElse:"L"},relativeTime:{future:"f’ %s",past:"%s ilu",s:"ftit sekondi",ss:"%d sekondi",m:"minuta",mm:"%d minuti",h:"siegħa",hh:"%d siegħat",d:"ġurnata",dd:"%d ġranet",M:"xahar",MM:"%d xhur",y:"sena",yy:"%d sni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},"1cfd":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"1",2:"2",3:"3",4:"4",5:"5",6:"6",7:"7",8:"8",9:"9",0:"0"},a=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},n={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},r=function(e){return function(t,r,i,o){var s=a(t),l=n[e][a(t)];return 2===s&&(l=l[r?0:1]),l.replace(/%d/i,t)}},i=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"],o=e.defineLocale("ar-ly",{months:i,monthsShort:i,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,a){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:r("s"),ss:r("s"),m:r("m"),mm:r("m"),h:r("h"),hh:r("h"),d:r("d"),dd:r("d"),M:r("M"),MM:r("M"),y:r("y"),yy:r("y")},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}});return o}))},"1d2b":function(e,t,a){"use strict";e.exports=function(e,t){return function(){for(var a=new Array(arguments.length),n=0;n<a.length;n++)a[n]=arguments[n];return e.apply(t,a)}}},"1d80":function(e,t,a){var n=a("7234"),r=TypeError;e.exports=function(e){if(n(e))throw r("Can't call method on "+e);return e}},"1fc1":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t){var a=e.split("_");return t%10===1&&t%100!==11?a[0]:t%10>=2&&t%10<=4&&(t%100<10||t%100>=20)?a[1]:a[2]}function a(e,a,n){var r={ss:a?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:a?"хвіліна_хвіліны_хвілін":"хвіліну_хвіліны_хвілін",hh:a?"гадзіна_гадзіны_гадзін":"гадзіну_гадзіны_гадзін",dd:"дзень_дні_дзён",MM:"месяц_месяцы_месяцаў",yy:"год_гады_гадоў"};return"m"===n?a?"хвіліна":"хвіліну":"h"===n?a?"гадзіна":"гадзіну":e+" "+t(r[n],+e)}var n=e.defineLocale("be",{months:{format:"студзеня_лютага_сакавіка_красавіка_траўня_чэрвеня_ліпеня_жніўня_верасня_кастрычніка_лістапада_снежня".split("_"),standalone:"студзень_люты_сакавік_красавік_травень_чэрвень_ліпень_жнівень_верасень_кастрычнік_лістапад_снежань".split("_")},monthsShort:"студ_лют_сак_крас_трав_чэрв_ліп_жнів_вер_каст_ліст_снеж".split("_"),weekdays:{format:"нядзелю_панядзелак_аўторак_сераду_чацвер_пятніцу_суботу".split("_"),standalone:"нядзеля_панядзелак_аўторак_серада_чацвер_пятніца_субота".split("_"),isFormat:/\[ ?[Ууў] ?(?:мінулую|наступную)? ?\] ?dddd/},weekdaysShort:"нд_пн_ат_ср_чц_пт_сб".split("_"),weekdaysMin:"нд_пн_ат_ср_чц_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., HH:mm",LLLL:"dddd, D MMMM YYYY г., HH:mm"},calendar:{sameDay:"[Сёння ў] LT",nextDay:"[Заўтра ў] LT",lastDay:"[Учора ў] LT",nextWeek:function(){return"[У] dddd [ў] LT"},lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return"[У мінулую] dddd [ў] LT";case 1:case 2:case 4:return"[У мінулы] dddd [ў] LT"}},sameElse:"L"},relativeTime:{future:"праз %s",past:"%s таму",s:"некалькі секунд",m:a,mm:a,h:a,hh:a,d:"дзень",dd:a,M:"месяц",MM:a,y:"год",yy:a},meridiemParse:/ночы|раніцы|дня|вечара/,isPM:function(e){return/^(дня|вечара)$/.test(e)},meridiem:function(e,t,a){return e<4?"ночы":e<12?"раніцы":e<17?"дня":"вечара"},dayOfMonthOrdinalParse:/\d{1,2}-(і|ы|га)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":case"w":case"W":return e%10!==2&&e%10!==3||e%100===12||e%100===13?e+"-ы":e+"-і";case"D":return e+"-га";default:return e}},week:{dow:1,doy:7}});return n}))},"201b":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ka",{months:"იანვარი_თებერვალი_მარტი_აპრილი_მაისი_ივნისი_ივლისი_აგვისტო_სექტემბერი_ოქტომბერი_ნოემბერი_დეკემბერი".split("_"),monthsShort:"იან_თებ_მარ_აპრ_მაი_ივნ_ივლ_აგვ_სექ_ოქტ_ნოე_დეკ".split("_"),weekdays:{standalone:"კვირა_ორშაბათი_სამშაბათი_ოთხშაბათი_ხუთშაბათი_პარასკევი_შაბათი".split("_"),format:"კვირას_ორშაბათს_სამშაბათს_ოთხშაბათს_ხუთშაბათს_პარასკევს_შაბათს".split("_"),isFormat:/(წინა|შემდეგ)/},weekdaysShort:"კვი_ორშ_სამ_ოთხ_ხუთ_პარ_შაბ".split("_"),weekdaysMin:"კვ_ორ_სა_ოთ_ხუ_პა_შა".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[დღეს] LT[-ზე]",nextDay:"[ხვალ] LT[-ზე]",lastDay:"[გუშინ] LT[-ზე]",nextWeek:"[შემდეგ] dddd LT[-ზე]",lastWeek:"[წინა] dddd LT-ზე",sameElse:"L"},relativeTime:{future:function(e){return e.replace(/(წამ|წუთ|საათ|წელ|დღ|თვ)(ი|ე)/,(function(e,t,a){return"ი"===a?t+"ში":t+a+"ში"}))},past:function(e){return/(წამი|წუთი|საათი|დღე|თვე)/.test(e)?e.replace(/(ი|ე)$/,"ის წინ"):/წელი/.test(e)?e.replace(/წელი$/,"წლის წინ"):e},s:"რამდენიმე წამი",ss:"%d წამი",m:"წუთი",mm:"%d წუთი",h:"საათი",hh:"%d საათი",d:"დღე",dd:"%d დღე",M:"თვე",MM:"%d თვე",y:"წელი",yy:"%d წელი"},dayOfMonthOrdinalParse:/0|1-ლი|მე-\d{1,2}|\d{1,2}-ე/,ordinal:function(e){return 0===e?e:1===e?e+"-ლი":e<20||e<=100&&e%20===0||e%100===0?"მე-"+e:e+"-ე"},week:{dow:1,doy:7}});return t}))},"22f8":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ko",{months:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),monthsShort:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),weekdays:"일요일_월요일_화요일_수요일_목요일_금요일_토요일".split("_"),weekdaysShort:"일_월_화_수_목_금_토".split("_"),weekdaysMin:"일_월_화_수_목_금_토".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY년 MMMM D일",LLL:"YYYY년 MMMM D일 A h:mm",LLLL:"YYYY년 MMMM D일 dddd A h:mm",l:"YYYY.MM.DD.",ll:"YYYY년 MMMM D일",lll:"YYYY년 MMMM D일 A h:mm",llll:"YYYY년 MMMM D일 dddd A h:mm"},calendar:{sameDay:"오늘 LT",nextDay:"내일 LT",nextWeek:"dddd LT",lastDay:"어제 LT",lastWeek:"지난주 dddd LT",sameElse:"L"},relativeTime:{future:"%s 후",past:"%s 전",s:"몇 초",ss:"%d초",m:"1분",mm:"%d분",h:"한 시간",hh:"%d시간",d:"하루",dd:"%d일",M:"한 달",MM:"%d달",y:"일 년",yy:"%d년"},dayOfMonthOrdinalParse:/\d{1,2}(일|월|주)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"일";case"M":return e+"월";case"w":case"W":return e+"주";default:return e}},meridiemParse:/오전|오후/,isPM:function(e){return"오후"===e},meridiem:function(e,t,a){return e<12?"오전":"오후"}});return t}))},"23cb":function(e,t,a){var n=a("5926"),r=Math.max,i=Math.min;e.exports=function(e,t){var a=n(e);return a<0?r(a+t,0):i(a,t)}},"23e7":function(e,t,a){var n=a("da84"),r=a("06cf").f,i=a("9112"),o=a("cb2d"),s=a("6374"),l=a("e893"),c=a("94ca");e.exports=function(e,t){var a,u,d,h,f,p,m=e.target,v=e.global,_=e.stat;if(u=v?n:_?n[m]||s(m,{}):(n[m]||{}).prototype,u)for(d in t){if(f=t[d],e.dontCallGetSet?(p=r(u,d),h=p&&p.value):h=u[d],a=c(v?d:m+(_?".":"#")+d,e.forced),!a&&void 0!==h){if(typeof f==typeof h)continue;l(f,h)}(e.sham||h&&h.sham)&&i(f,"sham",!0),o(u,d,f,e)}}},"241c":function(e,t,a){var n=a("ca84"),r=a("7839"),i=r.concat("length","prototype");t.f=Object.getOwnPropertyNames||function(e){return n(e,i)}},2421:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},a={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},n=["کانونی دووەم","شوبات","ئازار","نیسان","ئایار","حوزەیران","تەمموز","ئاب","ئەیلوول","تشرینی یەكەم","تشرینی دووەم","كانونی یەکەم"],r=e.defineLocale("ku",{months:n,monthsShort:n,weekdays:"یه‌كشه‌ممه‌_دووشه‌ممه‌_سێشه‌ممه‌_چوارشه‌ممه‌_پێنجشه‌ممه‌_هه‌ینی_شه‌ممه‌".split("_"),weekdaysShort:"یه‌كشه‌م_دووشه‌م_سێشه‌م_چوارشه‌م_پێنجشه‌م_هه‌ینی_شه‌ممه‌".split("_"),weekdaysMin:"ی_د_س_چ_پ_ه_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ئێواره‌|به‌یانی/,isPM:function(e){return/ئێواره‌/.test(e)},meridiem:function(e,t,a){return e<12?"به‌یانی":"ئێواره‌"},calendar:{sameDay:"[ئه‌مرۆ كاتژمێر] LT",nextDay:"[به‌یانی كاتژمێر] LT",nextWeek:"dddd [كاتژمێر] LT",lastDay:"[دوێنێ كاتژمێر] LT",lastWeek:"dddd [كاتژمێر] LT",sameElse:"L"},relativeTime:{future:"له‌ %s",past:"%s",s:"چه‌ند چركه‌یه‌ك",ss:"چركه‌ %d",m:"یه‌ك خوله‌ك",mm:"%d خوله‌ك",h:"یه‌ك كاتژمێر",hh:"%d كاتژمێر",d:"یه‌ك ڕۆژ",dd:"%d ڕۆژ",M:"یه‌ك مانگ",MM:"%d مانگ",y:"یه‌ك ساڵ",yy:"%d ساڵ"},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return a[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}});return r}))},2444:function(e,t,a){"use strict";(function(t){var n=a("c532"),r=a("c8af"),i=a("387f"),o={"Content-Type":"application/x-www-form-urlencoded"};function s(e,t){!n.isUndefined(e)&&n.isUndefined(e["Content-Type"])&&(e["Content-Type"]=t)}function l(){var e;return("undefined"!==typeof XMLHttpRequest||"undefined"!==typeof t&&"[object process]"===Object.prototype.toString.call(t))&&(e=a("b50d")),e}function c(e,t,a){if(n.isString(e))try{return(t||JSON.parse)(e),n.trim(e)}catch(r){if("SyntaxError"!==r.name)throw r}return(a||JSON.stringify)(e)}var u={transitional:{silentJSONParsing:!0,forcedJSONParsing:!0,clarifyTimeoutError:!1},adapter:l(),transformRequest:[function(e,t){return r(t,"Accept"),r(t,"Content-Type"),n.isFormData(e)||n.isArrayBuffer(e)||n.isBuffer(e)||n.isStream(e)||n.isFile(e)||n.isBlob(e)?e:n.isArrayBufferView(e)?e.buffer:n.isURLSearchParams(e)?(s(t,"application/x-www-form-urlencoded;charset=utf-8"),e.toString()):n.isObject(e)||t&&"application/json"===t["Content-Type"]?(s(t,"application/json"),c(e)):e}],transformResponse:[function(e){var t=this.transitional,a=t&&t.silentJSONParsing,r=t&&t.forcedJSONParsing,o=!a&&"json"===this.responseType;if(o||r&&n.isString(e)&&e.length)try{return JSON.parse(e)}catch(s){if(o){if("SyntaxError"===s.name)throw i(s,this,"E_JSON_PARSE");throw s}}return e}],timeout:0,xsrfCookieName:"XSRF-TOKEN",xsrfHeaderName:"X-XSRF-TOKEN",maxContentLength:-1,maxBodyLength:-1,validateStatus:function(e){return e>=200&&e<300},headers:{common:{Accept:"application/json, text/plain, */*"}}};n.forEach(["delete","get","head"],(function(e){u.headers[e]={}})),n.forEach(["post","put","patch"],(function(e){u.headers[e]=n.merge(o)})),e.exports=u}).call(this,a("4362"))},2554:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a){var n=e+" ";switch(a){case"ss":return n+=1===e?"sekunda":2===e||3===e||4===e?"sekunde":"sekundi",n;case"m":return t?"jedna minuta":"jedne minute";case"mm":return n+=1===e?"minuta":2===e||3===e||4===e?"minute":"minuta",n;case"h":return t?"jedan sat":"jednog sata";case"hh":return n+=1===e?"sat":2===e||3===e||4===e?"sata":"sati",n;case"dd":return n+=1===e?"dan":"dana",n;case"MM":return n+=1===e?"mjesec":2===e||3===e||4===e?"mjeseca":"mjeseci",n;case"yy":return n+=1===e?"godina":2===e||3===e||4===e?"godine":"godina",n}}var a=e.defineLocale("bs",{months:"januar_februar_mart_april_maj_juni_juli_august_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._aug._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[prošlu] dddd [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:t,m:t,mm:t,h:t,hh:t,d:"dan",dd:t,M:"mjesec",MM:t,y:"godinu",yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return a}))},"26f9":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={ss:"sekundė_sekundžių_sekundes",m:"minutė_minutės_minutę",mm:"minutės_minučių_minutes",h:"valanda_valandos_valandą",hh:"valandos_valandų_valandas",d:"diena_dienos_dieną",dd:"dienos_dienų_dienas",M:"mėnuo_mėnesio_mėnesį",MM:"mėnesiai_mėnesių_mėnesius",y:"metai_metų_metus",yy:"metai_metų_metus"};function a(e,t,a,n){return t?"kelios sekundės":n?"kelių sekundžių":"kelias sekundes"}function n(e,t,a,n){return t?i(a)[0]:n?i(a)[1]:i(a)[2]}function r(e){return e%10===0||e>10&&e<20}function i(e){return t[e].split("_")}function o(e,t,a,o){var s=e+" ";return 1===e?s+n(e,t,a[0],o):t?s+(r(e)?i(a)[1]:i(a)[0]):o?s+i(a)[1]:s+(r(e)?i(a)[1]:i(a)[2])}var s=e.defineLocale("lt",{months:{format:"sausio_vasario_kovo_balandžio_gegužės_birželio_liepos_rugpjūčio_rugsėjo_spalio_lapkričio_gruodžio".split("_"),standalone:"sausis_vasaris_kovas_balandis_gegužė_birželis_liepa_rugpjūtis_rugsėjis_spalis_lapkritis_gruodis".split("_"),isFormat:/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?|MMMM?(\[[^\[\]]*\]|\s)+D[oD]?/},monthsShort:"sau_vas_kov_bal_geg_bir_lie_rgp_rgs_spa_lap_grd".split("_"),weekdays:{format:"sekmadienį_pirmadienį_antradienį_trečiadienį_ketvirtadienį_penktadienį_šeštadienį".split("_"),standalone:"sekmadienis_pirmadienis_antradienis_trečiadienis_ketvirtadienis_penktadienis_šeštadienis".split("_"),isFormat:/dddd HH:mm/},weekdaysShort:"Sek_Pir_Ant_Tre_Ket_Pen_Šeš".split("_"),weekdaysMin:"S_P_A_T_K_Pn_Š".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY [m.] MMMM D [d.]",LLL:"YYYY [m.] MMMM D [d.], HH:mm [val.]",LLLL:"YYYY [m.] MMMM D [d.], dddd, HH:mm [val.]",l:"YYYY-MM-DD",ll:"YYYY [m.] MMMM D [d.]",lll:"YYYY [m.] MMMM D [d.], HH:mm [val.]",llll:"YYYY [m.] MMMM D [d.], ddd, HH:mm [val.]"},calendar:{sameDay:"[Šiandien] LT",nextDay:"[Rytoj] LT",nextWeek:"dddd LT",lastDay:"[Vakar] LT",lastWeek:"[Praėjusį] dddd LT",sameElse:"L"},relativeTime:{future:"po %s",past:"prieš %s",s:a,ss:o,m:n,mm:o,h:n,hh:o,d:n,dd:o,M:n,MM:o,y:n,yy:o},dayOfMonthOrdinalParse:/\d{1,2}-oji/,ordinal:function(e){return e+"-oji"},week:{dow:1,doy:4}});return s}))},2921:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("vi",{months:"tháng 1_tháng 2_tháng 3_tháng 4_tháng 5_tháng 6_tháng 7_tháng 8_tháng 9_tháng 10_tháng 11_tháng 12".split("_"),monthsShort:"Thg 01_Thg 02_Thg 03_Thg 04_Thg 05_Thg 06_Thg 07_Thg 08_Thg 09_Thg 10_Thg 11_Thg 12".split("_"),monthsParseExact:!0,weekdays:"chủ nhật_thứ hai_thứ ba_thứ tư_thứ năm_thứ sáu_thứ bảy".split("_"),weekdaysShort:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysMin:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysParseExact:!0,meridiemParse:/sa|ch/i,isPM:function(e){return/^ch$/i.test(e)},meridiem:function(e,t,a){return e<12?a?"sa":"SA":a?"ch":"CH"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [năm] YYYY",LLL:"D MMMM [năm] YYYY HH:mm",LLLL:"dddd, D MMMM [năm] YYYY HH:mm",l:"DD/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[Hôm nay lúc] LT",nextDay:"[Ngày mai lúc] LT",nextWeek:"dddd [tuần tới lúc] LT",lastDay:"[Hôm qua lúc] LT",lastWeek:"dddd [tuần trước lúc] LT",sameElse:"L"},relativeTime:{future:"%s tới",past:"%s trước",s:"vài giây",ss:"%d giây",m:"một phút",mm:"%d phút",h:"một giờ",hh:"%d giờ",d:"một ngày",dd:"%d ngày",w:"một tuần",ww:"%d tuần",M:"một tháng",MM:"%d tháng",y:"một năm",yy:"%d năm"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}});return t}))},"293c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["sekund","sekunda","sekundi"],m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mjesec","mjeseca","mjeseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(e,t){return 1===e?t[0]:e>=2&&e<=4?t[1]:t[2]},translate:function(e,a,n){var r=t.words[n];return 1===n.length?a?r[0]:r[1]:e+" "+t.correctGrammaticalCase(e,r)}},a=e.defineLocale("me",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sjutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){var e=["[prošle] [nedjelje] [u] LT","[prošlog] [ponedjeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srijede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"];return e[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"nekoliko sekundi",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:"dan",dd:t.translate,M:"mjesec",MM:t.translate,y:"godinu",yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return a}))},"2b88":function(e,t,a){"use strict";
/*! 
  * portal-vue © Thorsten Lünborg, 2019 
  * 
  * Version: 2.1.7
  * 
  * LICENCE: MIT 
  * 
  * https://github.com/linusborg/portal-vue
  * 
 */function n(e){return e&&"object"===typeof e&&"default"in e?e["default"]:e}Object.defineProperty(t,"__esModule",{value:!0});var r=n(a("a026"));function i(e){return i="function"===typeof Symbol&&"symbol"===typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"===typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},i(e)}function o(e){return s(e)||l(e)||c()}function s(e){if(Array.isArray(e)){for(var t=0,a=new Array(e.length);t<e.length;t++)a[t]=e[t];return a}}function l(e){if(Symbol.iterator in Object(e)||"[object Arguments]"===Object.prototype.toString.call(e))return Array.from(e)}function c(){throw new TypeError("Invalid attempt to spread non-iterable instance")}var u="undefined"!==typeof window;function d(e){return Array.isArray(e)||"object"===i(e)?Object.freeze(e):e}function h(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return e.reduce((function(e,a){var n=a.passengers[0],r="function"===typeof n?n(t):a.passengers;return e.concat(r)}),[])}function f(e,t){return e.map((function(e,t){return[t,e]})).sort((function(e,a){return t(e[1],a[1])||e[0]-a[0]})).map((function(e){return e[1]}))}function p(e,t){return t.reduce((function(t,a){return e.hasOwnProperty(a)&&(t[a]=e[a]),t}),{})}var m={},v={},_={},g=r.extend({data:function(){return{transports:m,targets:v,sources:_,trackInstances:u}},methods:{open:function(e){if(u){var t=e.to,a=e.from,n=e.passengers,i=e.order,o=void 0===i?1/0:i;if(t&&a&&n){var s={to:t,from:a,passengers:d(n),order:o},l=Object.keys(this.transports);-1===l.indexOf(t)&&r.set(this.transports,t,[]);var c=this.$_getTransportIndex(s),h=this.transports[t].slice(0);-1===c?h.push(s):h[c]=s,this.transports[t]=f(h,(function(e,t){return e.order-t.order}))}}},close:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],a=e.to,n=e.from;if(a&&(n||!1!==t)&&this.transports[a])if(t)this.transports[a]=[];else{var r=this.$_getTransportIndex(e);if(r>=0){var i=this.transports[a].slice(0);i.splice(r,1),this.transports[a]=i}}},registerTarget:function(e,t,a){u&&(this.trackInstances&&!a&&this.targets[e]&&console.warn("[portal-vue]: Target ".concat(e," already exists")),this.$set(this.targets,e,Object.freeze([t])))},unregisterTarget:function(e){this.$delete(this.targets,e)},registerSource:function(e,t,a){u&&(this.trackInstances&&!a&&this.sources[e]&&console.warn("[portal-vue]: source ".concat(e," already exists")),this.$set(this.sources,e,Object.freeze([t])))},unregisterSource:function(e){this.$delete(this.sources,e)},hasTarget:function(e){return!(!this.targets[e]||!this.targets[e][0])},hasSource:function(e){return!(!this.sources[e]||!this.sources[e][0])},hasContentFor:function(e){return!!this.transports[e]&&!!this.transports[e].length},$_getTransportIndex:function(e){var t=e.to,a=e.from;for(var n in this.transports[t])if(this.transports[t][n].from===a)return+n;return-1}}}),y=new g(m),b=1,M=r.extend({name:"portal",props:{disabled:{type:Boolean},name:{type:String,default:function(){return String(b++)}},order:{type:Number,default:0},slim:{type:Boolean},slotProps:{type:Object,default:function(){return{}}},tag:{type:String,default:"DIV"},to:{type:String,default:function(){return String(Math.round(1e7*Math.random()))}}},created:function(){var e=this;this.$nextTick((function(){y.registerSource(e.name,e)}))},mounted:function(){this.disabled||this.sendUpdate()},updated:function(){this.disabled?this.clear():this.sendUpdate()},beforeDestroy:function(){y.unregisterSource(this.name),this.clear()},watch:{to:function(e,t){t&&t!==e&&this.clear(t),this.sendUpdate()}},methods:{clear:function(e){var t={from:this.name,to:e||this.to};y.close(t)},normalizeSlots:function(){return this.$scopedSlots.default?[this.$scopedSlots.default]:this.$slots.default},normalizeOwnChildren:function(e){return"function"===typeof e?e(this.slotProps):e},sendUpdate:function(){var e=this.normalizeSlots();if(e){var t={from:this.name,to:this.to,passengers:o(e),order:this.order};y.open(t)}else this.clear()}},render:function(e){var t=this.$slots.default||this.$scopedSlots.default||[],a=this.tag;return t&&this.disabled?t.length<=1&&this.slim?this.normalizeOwnChildren(t)[0]:e(a,[this.normalizeOwnChildren(t)]):this.slim?e():e(a,{class:{"v-portal":!0},style:{display:"none"},key:"v-portal-placeholder"})}}),z=r.extend({name:"portalTarget",props:{multiple:{type:Boolean,default:!1},name:{type:String,required:!0},slim:{type:Boolean,default:!1},slotProps:{type:Object,default:function(){return{}}},tag:{type:String,default:"div"},transition:{type:[String,Object,Function]}},data:function(){return{transports:y.transports,firstRender:!0}},created:function(){var e=this;this.$nextTick((function(){y.registerTarget(e.name,e)}))},watch:{ownTransports:function(){this.$emit("change",this.children().length>0)},name:function(e,t){y.unregisterTarget(t),y.registerTarget(e,this)}},mounted:function(){var e=this;this.transition&&this.$nextTick((function(){e.firstRender=!1}))},beforeDestroy:function(){y.unregisterTarget(this.name)},computed:{ownTransports:function(){var e=this.transports[this.name]||[];return this.multiple?e:0===e.length?[]:[e[e.length-1]]},passengers:function(){return h(this.ownTransports,this.slotProps)}},methods:{children:function(){return 0!==this.passengers.length?this.passengers:this.$scopedSlots.default?this.$scopedSlots.default(this.slotProps):this.$slots.default||[]},noWrapper:function(){var e=this.slim&&!this.transition;return e&&this.children().length>1&&console.warn("[portal-vue]: PortalTarget with `slim` option received more than one child element."),e}},render:function(e){var t=this.noWrapper(),a=this.children(),n=this.transition||this.tag;return t?a[0]:this.slim&&!n?e():e(n,{props:{tag:this.transition&&this.tag?this.tag:void 0},class:{"vue-portal-target":!0}},a)}}),L=0,w=["disabled","name","order","slim","slotProps","tag","to"],H=["multiple","transition"],k=r.extend({name:"MountingPortal",inheritAttrs:!1,props:{append:{type:[Boolean,String]},bail:{type:Boolean},mountTo:{type:String,required:!0},disabled:{type:Boolean},name:{type:String,default:function(){return"mounted_"+String(L++)}},order:{type:Number,default:0},slim:{type:Boolean},slotProps:{type:Object,default:function(){return{}}},tag:{type:String,default:"DIV"},to:{type:String,default:function(){return String(Math.round(1e7*Math.random()))}},multiple:{type:Boolean,default:!1},targetSlim:{type:Boolean},targetSlotProps:{type:Object,default:function(){return{}}},targetTag:{type:String,default:"div"},transition:{type:[String,Object,Function]}},created:function(){if("undefined"!==typeof document){var e=document.querySelector(this.mountTo);if(e){var t=this.$props;if(y.targets[t.name])t.bail?console.warn("[portal-vue]: Target ".concat(t.name," is already mounted.\n        Aborting because 'bail: true' is set")):this.portalTarget=y.targets[t.name];else{var a=t.append;if(a){var n="string"===typeof a?a:"DIV",r=document.createElement(n);e.appendChild(r),e=r}var i=p(this.$props,H);i.slim=this.targetSlim,i.tag=this.targetTag,i.slotProps=this.targetSlotProps,i.name=this.to,this.portalTarget=new z({el:e,parent:this.$parent||this,propsData:i})}}else console.error("[portal-vue]: Mount Point '".concat(this.mountTo,"' not found in document"))}},beforeDestroy:function(){var e=this.portalTarget;if(this.append){var t=e.$el;t.parentNode.removeChild(t)}e.$destroy()},render:function(e){if(!this.portalTarget)return console.warn("[portal-vue] Target wasn't mounted"),e();if(!this.$scopedSlots.manual){var t=p(this.$props,w);return e(M,{props:t,attrs:this.$attrs,on:this.$listeners,scopedSlots:this.$scopedSlots},this.$slots.default)}var a=this.$scopedSlots.manual({to:this.to});return Array.isArray(a)&&(a=a[0]),a||e()}});function A(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};e.component(t.portalName||"Portal",M),e.component(t.portalTargetName||"PortalTarget",z),e.component(t.MountingPortalName||"MountingPortal",k)}var S={install:A};t.default=S,t.Portal=M,t.PortalTarget=z,t.MountingPortal=k,t.Wormhole=y},"2bfb":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("af",{months:"Januarie_Februarie_Maart_April_Mei_Junie_Julie_Augustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mrt_Apr_Mei_Jun_Jul_Aug_Sep_Okt_Nov_Des".split("_"),weekdays:"Sondag_Maandag_Dinsdag_Woensdag_Donderdag_Vrydag_Saterdag".split("_"),weekdaysShort:"Son_Maa_Din_Woe_Don_Vry_Sat".split("_"),weekdaysMin:"So_Ma_Di_Wo_Do_Vr_Sa".split("_"),meridiemParse:/vm|nm/i,isPM:function(e){return/^nm$/i.test(e)},meridiem:function(e,t,a){return e<12?a?"vm":"VM":a?"nm":"NM"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Vandag om] LT",nextDay:"[Môre om] LT",nextWeek:"dddd [om] LT",lastDay:"[Gister om] LT",lastWeek:"[Laas] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oor %s",past:"%s gelede",s:"'n paar sekondes",ss:"%d sekondes",m:"'n minuut",mm:"%d minute",h:"'n uur",hh:"%d ure",d:"'n dag",dd:"%d dae",M:"'n maand",MM:"%d maande",y:"'n jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return t}))},"2d00":function(e,t,a){var n,r,i=a("da84"),o=a("342f"),s=i.process,l=i.Deno,c=s&&s.versions||l&&l.version,u=c&&c.v8;u&&(n=u.split("."),r=n[0]>0&&n[0]<4?1:+(n[0]+n[1])),!r&&o&&(n=o.match(/Edge\/(\d+)/),(!n||n[1]>=74)&&(n=o.match(/Chrome\/(\d+)/),n&&(r=+n[1]))),e.exports=r},"2d83":function(e,t,a){"use strict";var n=a("387f");e.exports=function(e,t,a,r,i){var o=new Error(e);return n(o,t,a,r,i)}},"2dd8":function(e,t,a){},"2e67":function(e,t,a){"use strict";e.exports=function(e){return!(!e||!e.__CANCEL__)}},"2e8c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("uz",{months:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_"),monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"Якшанба_Душанба_Сешанба_Чоршанба_Пайшанба_Жума_Шанба".split("_"),weekdaysShort:"Якш_Душ_Сеш_Чор_Пай_Жум_Шан".split("_"),weekdaysMin:"Як_Ду_Се_Чо_Па_Жу_Ша".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Бугун соат] LT [да]",nextDay:"[Эртага] LT [да]",nextWeek:"dddd [куни соат] LT [да]",lastDay:"[Кеча соат] LT [да]",lastWeek:"[Утган] dddd [куни соат] LT [да]",sameElse:"L"},relativeTime:{future:"Якин %s ичида",past:"Бир неча %s олдин",s:"фурсат",ss:"%d фурсат",m:"бир дакика",mm:"%d дакика",h:"бир соат",hh:"%d соат",d:"бир кун",dd:"%d кун",M:"бир ой",MM:"%d ой",y:"бир йил",yy:"%d йил"},week:{dow:1,doy:7}});return t}))},"2ef0":function(e,t,a){(function(e,n){var r;
/**
 * @license
 * Lodash <https://lodash.com/>
 * Copyright OpenJS Foundation and other contributors <https://openjsf.org/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */(function(){var i,o="4.17.21",s=200,l="Unsupported core-js use. Try https://npms.io/search?q=ponyfill.",c="Expected a function",u="Invalid `variable` option passed into `_.template`",d="__lodash_hash_undefined__",h=500,f="__lodash_placeholder__",p=1,m=2,v=4,_=1,g=2,y=1,b=2,M=4,z=8,L=16,w=32,H=64,k=128,A=256,S=512,V=30,O="...",T=800,D=16,C=1,j=2,x=3,B=1/0,Y=9007199254740991,P=17976931348623157e292,I=NaN,F=4294967295,E=F-1,$=F>>>1,R=[["ary",k],["bind",y],["bindKey",b],["curry",z],["curryRight",L],["flip",S],["partial",w],["partialRight",H],["rearg",A]],N="[object Arguments]",W="[object Array]",U="[object AsyncFunction]",G="[object Boolean]",q="[object Date]",J="[object DOMException]",K="[object Error]",X="[object Function]",Z="[object GeneratorFunction]",Q="[object Map]",ee="[object Number]",te="[object Null]",ae="[object Object]",ne="[object Promise]",re="[object Proxy]",ie="[object RegExp]",oe="[object Set]",se="[object String]",le="[object Symbol]",ce="[object Undefined]",ue="[object WeakMap]",de="[object WeakSet]",he="[object ArrayBuffer]",fe="[object DataView]",pe="[object Float32Array]",me="[object Float64Array]",ve="[object Int8Array]",_e="[object Int16Array]",ge="[object Int32Array]",ye="[object Uint8Array]",be="[object Uint8ClampedArray]",Me="[object Uint16Array]",ze="[object Uint32Array]",Le=/\b__p \+= '';/g,we=/\b(__p \+=) '' \+/g,He=/(__e\(.*?\)|\b__t\)) \+\n'';/g,ke=/&(?:amp|lt|gt|quot|#39);/g,Ae=/[&<>"']/g,Se=RegExp(ke.source),Ve=RegExp(Ae.source),Oe=/<%-([\s\S]+?)%>/g,Te=/<%([\s\S]+?)%>/g,De=/<%=([\s\S]+?)%>/g,Ce=/\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,je=/^\w*$/,xe=/[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g,Be=/[\\^$.*+?()[\]{}|]/g,Ye=RegExp(Be.source),Pe=/^\s+/,Ie=/\s/,Fe=/\{(?:\n\/\* \[wrapped with .+\] \*\/)?\n?/,Ee=/\{\n\/\* \[wrapped with (.+)\] \*/,$e=/,? & /,Re=/[^\x00-\x2f\x3a-\x40\x5b-\x60\x7b-\x7f]+/g,Ne=/[()=,{}\[\]\/\s]/,We=/\\(\\)?/g,Ue=/\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g,Ge=/\w*$/,qe=/^[-+]0x[0-9a-f]+$/i,Je=/^0b[01]+$/i,Ke=/^\[object .+?Constructor\]$/,Xe=/^0o[0-7]+$/i,Ze=/^(?:0|[1-9]\d*)$/,Qe=/[\xc0-\xd6\xd8-\xf6\xf8-\xff\u0100-\u017f]/g,et=/($^)/,tt=/['\n\r\u2028\u2029\\]/g,at="\\ud800-\\udfff",nt="\\u0300-\\u036f",rt="\\ufe20-\\ufe2f",it="\\u20d0-\\u20ff",ot=nt+rt+it,st="\\u2700-\\u27bf",lt="a-z\\xdf-\\xf6\\xf8-\\xff",ct="\\xac\\xb1\\xd7\\xf7",ut="\\x00-\\x2f\\x3a-\\x40\\x5b-\\x60\\x7b-\\xbf",dt="\\u2000-\\u206f",ht=" \\t\\x0b\\f\\xa0\\ufeff\\n\\r\\u2028\\u2029\\u1680\\u180e\\u2000\\u2001\\u2002\\u2003\\u2004\\u2005\\u2006\\u2007\\u2008\\u2009\\u200a\\u202f\\u205f\\u3000",ft="A-Z\\xc0-\\xd6\\xd8-\\xde",pt="\\ufe0e\\ufe0f",mt=ct+ut+dt+ht,vt="['’]",_t="["+at+"]",gt="["+mt+"]",yt="["+ot+"]",bt="\\d+",Mt="["+st+"]",zt="["+lt+"]",Lt="[^"+at+mt+bt+st+lt+ft+"]",wt="\\ud83c[\\udffb-\\udfff]",Ht="(?:"+yt+"|"+wt+")",kt="[^"+at+"]",At="(?:\\ud83c[\\udde6-\\uddff]){2}",St="[\\ud800-\\udbff][\\udc00-\\udfff]",Vt="["+ft+"]",Ot="\\u200d",Tt="(?:"+zt+"|"+Lt+")",Dt="(?:"+Vt+"|"+Lt+")",Ct="(?:"+vt+"(?:d|ll|m|re|s|t|ve))?",jt="(?:"+vt+"(?:D|LL|M|RE|S|T|VE))?",xt=Ht+"?",Bt="["+pt+"]?",Yt="(?:"+Ot+"(?:"+[kt,At,St].join("|")+")"+Bt+xt+")*",Pt="\\d*(?:1st|2nd|3rd|(?![123])\\dth)(?=\\b|[A-Z_])",It="\\d*(?:1ST|2ND|3RD|(?![123])\\dTH)(?=\\b|[a-z_])",Ft=Bt+xt+Yt,Et="(?:"+[Mt,At,St].join("|")+")"+Ft,$t="(?:"+[kt+yt+"?",yt,At,St,_t].join("|")+")",Rt=RegExp(vt,"g"),Nt=RegExp(yt,"g"),Wt=RegExp(wt+"(?="+wt+")|"+$t+Ft,"g"),Ut=RegExp([Vt+"?"+zt+"+"+Ct+"(?="+[gt,Vt,"$"].join("|")+")",Dt+"+"+jt+"(?="+[gt,Vt+Tt,"$"].join("|")+")",Vt+"?"+Tt+"+"+Ct,Vt+"+"+jt,It,Pt,bt,Et].join("|"),"g"),Gt=RegExp("["+Ot+at+ot+pt+"]"),qt=/[a-z][A-Z]|[A-Z]{2}[a-z]|[0-9][a-zA-Z]|[a-zA-Z][0-9]|[^a-zA-Z0-9 ]/,Jt=["Array","Buffer","DataView","Date","Error","Float32Array","Float64Array","Function","Int8Array","Int16Array","Int32Array","Map","Math","Object","Promise","RegExp","Set","String","Symbol","TypeError","Uint8Array","Uint8ClampedArray","Uint16Array","Uint32Array","WeakMap","_","clearTimeout","isFinite","parseInt","setTimeout"],Kt=-1,Xt={};Xt[pe]=Xt[me]=Xt[ve]=Xt[_e]=Xt[ge]=Xt[ye]=Xt[be]=Xt[Me]=Xt[ze]=!0,Xt[N]=Xt[W]=Xt[he]=Xt[G]=Xt[fe]=Xt[q]=Xt[K]=Xt[X]=Xt[Q]=Xt[ee]=Xt[ae]=Xt[ie]=Xt[oe]=Xt[se]=Xt[ue]=!1;var Zt={};Zt[N]=Zt[W]=Zt[he]=Zt[fe]=Zt[G]=Zt[q]=Zt[pe]=Zt[me]=Zt[ve]=Zt[_e]=Zt[ge]=Zt[Q]=Zt[ee]=Zt[ae]=Zt[ie]=Zt[oe]=Zt[se]=Zt[le]=Zt[ye]=Zt[be]=Zt[Me]=Zt[ze]=!0,Zt[K]=Zt[X]=Zt[ue]=!1;var Qt={"À":"A","Á":"A","Â":"A","Ã":"A","Ä":"A","Å":"A","à":"a","á":"a","â":"a","ã":"a","ä":"a","å":"a","Ç":"C","ç":"c","Ð":"D","ð":"d","È":"E","É":"E","Ê":"E","Ë":"E","è":"e","é":"e","ê":"e","ë":"e","Ì":"I","Í":"I","Î":"I","Ï":"I","ì":"i","í":"i","î":"i","ï":"i","Ñ":"N","ñ":"n","Ò":"O","Ó":"O","Ô":"O","Õ":"O","Ö":"O","Ø":"O","ò":"o","ó":"o","ô":"o","õ":"o","ö":"o","ø":"o","Ù":"U","Ú":"U","Û":"U","Ü":"U","ù":"u","ú":"u","û":"u","ü":"u","Ý":"Y","ý":"y","ÿ":"y","Æ":"Ae","æ":"ae","Þ":"Th","þ":"th","ß":"ss","Ā":"A","Ă":"A","Ą":"A","ā":"a","ă":"a","ą":"a","Ć":"C","Ĉ":"C","Ċ":"C","Č":"C","ć":"c","ĉ":"c","ċ":"c","č":"c","Ď":"D","Đ":"D","ď":"d","đ":"d","Ē":"E","Ĕ":"E","Ė":"E","Ę":"E","Ě":"E","ē":"e","ĕ":"e","ė":"e","ę":"e","ě":"e","Ĝ":"G","Ğ":"G","Ġ":"G","Ģ":"G","ĝ":"g","ğ":"g","ġ":"g","ģ":"g","Ĥ":"H","Ħ":"H","ĥ":"h","ħ":"h","Ĩ":"I","Ī":"I","Ĭ":"I","Į":"I","İ":"I","ĩ":"i","ī":"i","ĭ":"i","į":"i","ı":"i","Ĵ":"J","ĵ":"j","Ķ":"K","ķ":"k","ĸ":"k","Ĺ":"L","Ļ":"L","Ľ":"L","Ŀ":"L","Ł":"L","ĺ":"l","ļ":"l","ľ":"l","ŀ":"l","ł":"l","Ń":"N","Ņ":"N","Ň":"N","Ŋ":"N","ń":"n","ņ":"n","ň":"n","ŋ":"n","Ō":"O","Ŏ":"O","Ő":"O","ō":"o","ŏ":"o","ő":"o","Ŕ":"R","Ŗ":"R","Ř":"R","ŕ":"r","ŗ":"r","ř":"r","Ś":"S","Ŝ":"S","Ş":"S","Š":"S","ś":"s","ŝ":"s","ş":"s","š":"s","Ţ":"T","Ť":"T","Ŧ":"T","ţ":"t","ť":"t","ŧ":"t","Ũ":"U","Ū":"U","Ŭ":"U","Ů":"U","Ű":"U","Ų":"U","ũ":"u","ū":"u","ŭ":"u","ů":"u","ű":"u","ų":"u","Ŵ":"W","ŵ":"w","Ŷ":"Y","ŷ":"y","Ÿ":"Y","Ź":"Z","Ż":"Z","Ž":"Z","ź":"z","ż":"z","ž":"z","Ĳ":"IJ","ĳ":"ij","Œ":"Oe","œ":"oe","ŉ":"'n","ſ":"s"},ea={"&":"&amp;","<":"&lt;",">":"&gt;",'"':"&quot;","'":"&#39;"},ta={"&amp;":"&","&lt;":"<","&gt;":">","&quot;":'"',"&#39;":"'"},aa={"\\":"\\","'":"'","\n":"n","\r":"r","\u2028":"u2028","\u2029":"u2029"},na=parseFloat,ra=parseInt,ia="object"==typeof e&&e&&e.Object===Object&&e,oa="object"==typeof self&&self&&self.Object===Object&&self,sa=ia||oa||Function("return this")(),la=t&&!t.nodeType&&t,ca=la&&"object"==typeof n&&n&&!n.nodeType&&n,ua=ca&&ca.exports===la,da=ua&&ia.process,ha=function(){try{var e=ca&&ca.require&&ca.require("util").types;return e||da&&da.binding&&da.binding("util")}catch(t){}}(),fa=ha&&ha.isArrayBuffer,pa=ha&&ha.isDate,ma=ha&&ha.isMap,va=ha&&ha.isRegExp,_a=ha&&ha.isSet,ga=ha&&ha.isTypedArray;function ya(e,t,a){switch(a.length){case 0:return e.call(t);case 1:return e.call(t,a[0]);case 2:return e.call(t,a[0],a[1]);case 3:return e.call(t,a[0],a[1],a[2])}return e.apply(t,a)}function ba(e,t,a,n){var r=-1,i=null==e?0:e.length;while(++r<i){var o=e[r];t(n,o,a(o),e)}return n}function Ma(e,t){var a=-1,n=null==e?0:e.length;while(++a<n)if(!1===t(e[a],a,e))break;return e}function za(e,t){var a=null==e?0:e.length;while(a--)if(!1===t(e[a],a,e))break;return e}function La(e,t){var a=-1,n=null==e?0:e.length;while(++a<n)if(!t(e[a],a,e))return!1;return!0}function wa(e,t){var a=-1,n=null==e?0:e.length,r=0,i=[];while(++a<n){var o=e[a];t(o,a,e)&&(i[r++]=o)}return i}function Ha(e,t){var a=null==e?0:e.length;return!!a&&Ya(e,t,0)>-1}function ka(e,t,a){var n=-1,r=null==e?0:e.length;while(++n<r)if(a(t,e[n]))return!0;return!1}function Aa(e,t){var a=-1,n=null==e?0:e.length,r=Array(n);while(++a<n)r[a]=t(e[a],a,e);return r}function Sa(e,t){var a=-1,n=t.length,r=e.length;while(++a<n)e[r+a]=t[a];return e}function Va(e,t,a,n){var r=-1,i=null==e?0:e.length;n&&i&&(a=e[++r]);while(++r<i)a=t(a,e[r],r,e);return a}function Oa(e,t,a,n){var r=null==e?0:e.length;n&&r&&(a=e[--r]);while(r--)a=t(a,e[r],r,e);return a}function Ta(e,t){var a=-1,n=null==e?0:e.length;while(++a<n)if(t(e[a],a,e))return!0;return!1}var Da=Ea("length");function Ca(e){return e.split("")}function ja(e){return e.match(Re)||[]}function xa(e,t,a){var n;return a(e,(function(e,a,r){if(t(e,a,r))return n=a,!1})),n}function Ba(e,t,a,n){var r=e.length,i=a+(n?1:-1);while(n?i--:++i<r)if(t(e[i],i,e))return i;return-1}function Ya(e,t,a){return t===t?pn(e,t,a):Ba(e,Ia,a)}function Pa(e,t,a,n){var r=a-1,i=e.length;while(++r<i)if(n(e[r],t))return r;return-1}function Ia(e){return e!==e}function Fa(e,t){var a=null==e?0:e.length;return a?Wa(e,t)/a:I}function Ea(e){return function(t){return null==t?i:t[e]}}function $a(e){return function(t){return null==e?i:e[t]}}function Ra(e,t,a,n,r){return r(e,(function(e,r,i){a=n?(n=!1,e):t(a,e,r,i)})),a}function Na(e,t){var a=e.length;e.sort(t);while(a--)e[a]=e[a].value;return e}function Wa(e,t){var a,n=-1,r=e.length;while(++n<r){var o=t(e[n]);o!==i&&(a=a===i?o:a+o)}return a}function Ua(e,t){var a=-1,n=Array(e);while(++a<e)n[a]=t(a);return n}function Ga(e,t){return Aa(t,(function(t){return[t,e[t]]}))}function qa(e){return e?e.slice(0,gn(e)+1).replace(Pe,""):e}function Ja(e){return function(t){return e(t)}}function Ka(e,t){return Aa(t,(function(t){return e[t]}))}function Xa(e,t){return e.has(t)}function Za(e,t){var a=-1,n=e.length;while(++a<n&&Ya(t,e[a],0)>-1);return a}function Qa(e,t){var a=e.length;while(a--&&Ya(t,e[a],0)>-1);return a}function en(e,t){var a=e.length,n=0;while(a--)e[a]===t&&++n;return n}var tn=$a(Qt),an=$a(ea);function nn(e){return"\\"+aa[e]}function rn(e,t){return null==e?i:e[t]}function on(e){return Gt.test(e)}function sn(e){return qt.test(e)}function ln(e){var t,a=[];while(!(t=e.next()).done)a.push(t.value);return a}function cn(e){var t=-1,a=Array(e.size);return e.forEach((function(e,n){a[++t]=[n,e]})),a}function un(e,t){return function(a){return e(t(a))}}function dn(e,t){var a=-1,n=e.length,r=0,i=[];while(++a<n){var o=e[a];o!==t&&o!==f||(e[a]=f,i[r++]=a)}return i}function hn(e){var t=-1,a=Array(e.size);return e.forEach((function(e){a[++t]=e})),a}function fn(e){var t=-1,a=Array(e.size);return e.forEach((function(e){a[++t]=[e,e]})),a}function pn(e,t,a){var n=a-1,r=e.length;while(++n<r)if(e[n]===t)return n;return-1}function mn(e,t,a){var n=a+1;while(n--)if(e[n]===t)return n;return n}function vn(e){return on(e)?bn(e):Da(e)}function _n(e){return on(e)?Mn(e):Ca(e)}function gn(e){var t=e.length;while(t--&&Ie.test(e.charAt(t)));return t}var yn=$a(ta);function bn(e){var t=Wt.lastIndex=0;while(Wt.test(e))++t;return t}function Mn(e){return e.match(Wt)||[]}function zn(e){return e.match(Ut)||[]}var Ln=function e(t){t=null==t?sa:wn.defaults(sa.Object(),t,wn.pick(sa,Jt));var a=t.Array,n=t.Date,r=t.Error,Ie=t.Function,Re=t.Math,at=t.Object,nt=t.RegExp,rt=t.String,it=t.TypeError,ot=a.prototype,st=Ie.prototype,lt=at.prototype,ct=t["__core-js_shared__"],ut=st.toString,dt=lt.hasOwnProperty,ht=0,ft=function(){var e=/[^.]+$/.exec(ct&&ct.keys&&ct.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}(),pt=lt.toString,mt=ut.call(at),vt=sa._,_t=nt("^"+ut.call(dt).replace(Be,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$"),gt=ua?t.Buffer:i,yt=t.Symbol,bt=t.Uint8Array,Mt=gt?gt.allocUnsafe:i,zt=un(at.getPrototypeOf,at),Lt=at.create,wt=lt.propertyIsEnumerable,Ht=ot.splice,kt=yt?yt.isConcatSpreadable:i,At=yt?yt.iterator:i,St=yt?yt.toStringTag:i,Vt=function(){try{var e=Go(at,"defineProperty");return e({},"",{}),e}catch(t){}}(),Ot=t.clearTimeout!==sa.clearTimeout&&t.clearTimeout,Tt=n&&n.now!==sa.Date.now&&n.now,Dt=t.setTimeout!==sa.setTimeout&&t.setTimeout,Ct=Re.ceil,jt=Re.floor,xt=at.getOwnPropertySymbols,Bt=gt?gt.isBuffer:i,Yt=t.isFinite,Pt=ot.join,It=un(at.keys,at),Ft=Re.max,Et=Re.min,$t=n.now,Wt=t.parseInt,Ut=Re.random,Gt=ot.reverse,qt=Go(t,"DataView"),Qt=Go(t,"Map"),ea=Go(t,"Promise"),ta=Go(t,"Set"),aa=Go(t,"WeakMap"),ia=Go(at,"create"),oa=aa&&new aa,la={},ca=Ds(qt),da=Ds(Qt),ha=Ds(ea),Da=Ds(ta),Ca=Ds(aa),$a=yt?yt.prototype:i,pn=$a?$a.valueOf:i,bn=$a?$a.toString:i;function Mn(e){if(Hu(e)&&!lu(e)&&!(e instanceof An)){if(e instanceof kn)return e;if(dt.call(e,"__wrapped__"))return js(e)}return new kn(e)}var Ln=function(){function e(){}return function(t){if(!wu(t))return{};if(Lt)return Lt(t);e.prototype=t;var a=new e;return e.prototype=i,a}}();function Hn(){}function kn(e,t){this.__wrapped__=e,this.__actions__=[],this.__chain__=!!t,this.__index__=0,this.__values__=i}function An(e){this.__wrapped__=e,this.__actions__=[],this.__dir__=1,this.__filtered__=!1,this.__iteratees__=[],this.__takeCount__=F,this.__views__=[]}function Sn(){var e=new An(this.__wrapped__);return e.__actions__=no(this.__actions__),e.__dir__=this.__dir__,e.__filtered__=this.__filtered__,e.__iteratees__=no(this.__iteratees__),e.__takeCount__=this.__takeCount__,e.__views__=no(this.__views__),e}function Vn(){if(this.__filtered__){var e=new An(this);e.__dir__=-1,e.__filtered__=!0}else e=this.clone(),e.__dir__*=-1;return e}function On(){var e=this.__wrapped__.value(),t=this.__dir__,a=lu(e),n=t<0,r=a?e.length:0,i=Zo(0,r,this.__views__),o=i.start,s=i.end,l=s-o,c=n?s:o-1,u=this.__iteratees__,d=u.length,h=0,f=Et(l,this.__takeCount__);if(!a||!n&&r==l&&f==l)return Pi(e,this.__actions__);var p=[];e:while(l--&&h<f){c+=t;var m=-1,v=e[c];while(++m<d){var _=u[m],g=_.iteratee,y=_.type,b=g(v);if(y==j)v=b;else if(!b){if(y==C)continue e;break e}}p[h++]=v}return p}function Tn(e){var t=-1,a=null==e?0:e.length;this.clear();while(++t<a){var n=e[t];this.set(n[0],n[1])}}function Dn(){this.__data__=ia?ia(null):{},this.size=0}function Cn(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}function jn(e){var t=this.__data__;if(ia){var a=t[e];return a===d?i:a}return dt.call(t,e)?t[e]:i}function xn(e){var t=this.__data__;return ia?t[e]!==i:dt.call(t,e)}function Bn(e,t){var a=this.__data__;return this.size+=this.has(e)?0:1,a[e]=ia&&t===i?d:t,this}function Yn(e){var t=-1,a=null==e?0:e.length;this.clear();while(++t<a){var n=e[t];this.set(n[0],n[1])}}function Pn(){this.__data__=[],this.size=0}function In(e){var t=this.__data__,a=ur(t,e);if(a<0)return!1;var n=t.length-1;return a==n?t.pop():Ht.call(t,a,1),--this.size,!0}function Fn(e){var t=this.__data__,a=ur(t,e);return a<0?i:t[a][1]}function En(e){return ur(this.__data__,e)>-1}function $n(e,t){var a=this.__data__,n=ur(a,e);return n<0?(++this.size,a.push([e,t])):a[n][1]=t,this}function Rn(e){var t=-1,a=null==e?0:e.length;this.clear();while(++t<a){var n=e[t];this.set(n[0],n[1])}}function Nn(){this.size=0,this.__data__={hash:new Tn,map:new(Qt||Yn),string:new Tn}}function Wn(e){var t=Wo(this,e)["delete"](e);return this.size-=t?1:0,t}function Un(e){return Wo(this,e).get(e)}function Gn(e){return Wo(this,e).has(e)}function qn(e,t){var a=Wo(this,e),n=a.size;return a.set(e,t),this.size+=a.size==n?0:1,this}function Jn(e){var t=-1,a=null==e?0:e.length;this.__data__=new Rn;while(++t<a)this.add(e[t])}function Kn(e){return this.__data__.set(e,d),this}function Xn(e){return this.__data__.has(e)}function Zn(e){var t=this.__data__=new Yn(e);this.size=t.size}function Qn(){this.__data__=new Yn,this.size=0}function er(e){var t=this.__data__,a=t["delete"](e);return this.size=t.size,a}function tr(e){return this.__data__.get(e)}function ar(e){return this.__data__.has(e)}function nr(e,t){var a=this.__data__;if(a instanceof Yn){var n=a.__data__;if(!Qt||n.length<s-1)return n.push([e,t]),this.size=++a.size,this;a=this.__data__=new Rn(n)}return a.set(e,t),this.size=a.size,this}function rr(e,t){var a=lu(e),n=!a&&su(e),r=!a&&!n&&fu(e),i=!a&&!n&&!r&&Fu(e),o=a||n||r||i,s=o?Ua(e.length,rt):[],l=s.length;for(var c in e)!t&&!dt.call(e,c)||o&&("length"==c||r&&("offset"==c||"parent"==c)||i&&("buffer"==c||"byteLength"==c||"byteOffset"==c)||os(c,l))||s.push(c);return s}function ir(e){var t=e.length;return t?e[_i(0,t-1)]:i}function or(e,t){return Vs(no(e),vr(t,0,e.length))}function sr(e){return Vs(no(e))}function lr(e,t,a){(a!==i&&!ru(e[t],a)||a===i&&!(t in e))&&pr(e,t,a)}function cr(e,t,a){var n=e[t];dt.call(e,t)&&ru(n,a)&&(a!==i||t in e)||pr(e,t,a)}function ur(e,t){var a=e.length;while(a--)if(ru(e[a][0],t))return a;return-1}function dr(e,t,a,n){return zr(e,(function(e,r,i){t(n,e,a(e),i)})),n}function hr(e,t){return e&&ro(t,zd(t),e)}function fr(e,t){return e&&ro(t,Ld(t),e)}function pr(e,t,a){"__proto__"==t&&Vt?Vt(e,t,{configurable:!0,enumerable:!0,value:a,writable:!0}):e[t]=a}function mr(e,t){var n=-1,r=t.length,o=a(r),s=null==e;while(++n<r)o[n]=s?i:vd(e,t[n]);return o}function vr(e,t,a){return e===e&&(a!==i&&(e=e<=a?e:a),t!==i&&(e=e>=t?e:t)),e}function _r(e,t,a,n,r,o){var s,l=t&p,c=t&m,u=t&v;if(a&&(s=r?a(e,n,r,o):a(e)),s!==i)return s;if(!wu(e))return e;var d=lu(e);if(d){if(s=ts(e),!l)return no(e,s)}else{var h=Xo(e),f=h==X||h==Z;if(fu(e))return Gi(e,l);if(h==ae||h==N||f&&!r){if(s=c||f?{}:as(e),!l)return c?oo(e,fr(s,e)):io(e,hr(s,e))}else{if(!Zt[h])return r?e:{};s=ns(e,h,l)}}o||(o=new Zn);var _=o.get(e);if(_)return _;o.set(e,s),Yu(e)?e.forEach((function(n){s.add(_r(n,t,a,n,e,o))})):ku(e)&&e.forEach((function(n,r){s.set(r,_r(n,t,a,r,e,o))}));var g=u?c?Fo:Io:c?Ld:zd,y=d?i:g(e);return Ma(y||e,(function(n,r){y&&(r=n,n=e[r]),cr(s,r,_r(n,t,a,r,e,o))})),s}function gr(e){var t=zd(e);return function(a){return yr(a,e,t)}}function yr(e,t,a){var n=a.length;if(null==e)return!n;e=at(e);while(n--){var r=a[n],o=t[r],s=e[r];if(s===i&&!(r in e)||!o(s))return!1}return!0}function br(e,t,a){if("function"!=typeof e)throw new it(c);return Hs((function(){e.apply(i,a)}),t)}function Mr(e,t,a,n){var r=-1,i=Ha,o=!0,l=e.length,c=[],u=t.length;if(!l)return c;a&&(t=Aa(t,Ja(a))),n?(i=ka,o=!1):t.length>=s&&(i=Xa,o=!1,t=new Jn(t));e:while(++r<l){var d=e[r],h=null==a?d:a(d);if(d=n||0!==d?d:0,o&&h===h){var f=u;while(f--)if(t[f]===h)continue e;c.push(d)}else i(t,h,n)||c.push(d)}return c}Mn.templateSettings={escape:Oe,evaluate:Te,interpolate:De,variable:"",imports:{_:Mn}},Mn.prototype=Hn.prototype,Mn.prototype.constructor=Mn,kn.prototype=Ln(Hn.prototype),kn.prototype.constructor=kn,An.prototype=Ln(Hn.prototype),An.prototype.constructor=An,Tn.prototype.clear=Dn,Tn.prototype["delete"]=Cn,Tn.prototype.get=jn,Tn.prototype.has=xn,Tn.prototype.set=Bn,Yn.prototype.clear=Pn,Yn.prototype["delete"]=In,Yn.prototype.get=Fn,Yn.prototype.has=En,Yn.prototype.set=$n,Rn.prototype.clear=Nn,Rn.prototype["delete"]=Wn,Rn.prototype.get=Un,Rn.prototype.has=Gn,Rn.prototype.set=qn,Jn.prototype.add=Jn.prototype.push=Kn,Jn.prototype.has=Xn,Zn.prototype.clear=Qn,Zn.prototype["delete"]=er,Zn.prototype.get=tr,Zn.prototype.has=ar,Zn.prototype.set=nr;var zr=co(Tr),Lr=co(Dr,!0);function wr(e,t){var a=!0;return zr(e,(function(e,n,r){return a=!!t(e,n,r),a})),a}function Hr(e,t,a){var n=-1,r=e.length;while(++n<r){var o=e[n],s=t(o);if(null!=s&&(l===i?s===s&&!Iu(s):a(s,l)))var l=s,c=o}return c}function kr(e,t,a,n){var r=e.length;a=qu(a),a<0&&(a=-a>r?0:r+a),n=n===i||n>r?r:qu(n),n<0&&(n+=r),n=a>n?0:Ju(n);while(a<n)e[a++]=t;return e}function Ar(e,t){var a=[];return zr(e,(function(e,n,r){t(e,n,r)&&a.push(e)})),a}function Sr(e,t,a,n,r){var i=-1,o=e.length;a||(a=is),r||(r=[]);while(++i<o){var s=e[i];t>0&&a(s)?t>1?Sr(s,t-1,a,n,r):Sa(r,s):n||(r[r.length]=s)}return r}var Vr=uo(),Or=uo(!0);function Tr(e,t){return e&&Vr(e,t,zd)}function Dr(e,t){return e&&Or(e,t,zd)}function Cr(e,t){return wa(t,(function(t){return Mu(e[t])}))}function jr(e,t){t=Ri(t,e);var a=0,n=t.length;while(null!=e&&a<n)e=e[Ts(t[a++])];return a&&a==n?e:i}function xr(e,t,a){var n=t(e);return lu(e)?n:Sa(n,a(e))}function Br(e){return null==e?e===i?ce:te:St&&St in at(e)?qo(e):ys(e)}function Yr(e,t){return e>t}function Pr(e,t){return null!=e&&dt.call(e,t)}function Ir(e,t){return null!=e&&t in at(e)}function Fr(e,t,a){return e>=Et(t,a)&&e<Ft(t,a)}function Er(e,t,n){var r=n?ka:Ha,o=e[0].length,s=e.length,l=s,c=a(s),u=1/0,d=[];while(l--){var h=e[l];l&&t&&(h=Aa(h,Ja(t))),u=Et(h.length,u),c[l]=!n&&(t||o>=120&&h.length>=120)?new Jn(l&&h):i}h=e[0];var f=-1,p=c[0];e:while(++f<o&&d.length<u){var m=h[f],v=t?t(m):m;if(m=n||0!==m?m:0,!(p?Xa(p,v):r(d,v,n))){l=s;while(--l){var _=c[l];if(!(_?Xa(_,v):r(e[l],v,n)))continue e}p&&p.push(v),d.push(m)}}return d}function $r(e,t,a,n){return Tr(e,(function(e,r,i){t(n,a(e),r,i)})),n}function Rr(e,t,a){t=Ri(t,e),e=Ms(e,t);var n=null==e?e:e[Ts(il(t))];return null==n?i:ya(n,e,a)}function Nr(e){return Hu(e)&&Br(e)==N}function Wr(e){return Hu(e)&&Br(e)==he}function Ur(e){return Hu(e)&&Br(e)==q}function Gr(e,t,a,n,r){return e===t||(null==e||null==t||!Hu(e)&&!Hu(t)?e!==e&&t!==t:qr(e,t,a,n,Gr,r))}function qr(e,t,a,n,r,i){var o=lu(e),s=lu(t),l=o?W:Xo(e),c=s?W:Xo(t);l=l==N?ae:l,c=c==N?ae:c;var u=l==ae,d=c==ae,h=l==c;if(h&&fu(e)){if(!fu(t))return!1;o=!0,u=!1}if(h&&!u)return i||(i=new Zn),o||Fu(e)?xo(e,t,a,n,r,i):Bo(e,t,l,a,n,r,i);if(!(a&_)){var f=u&&dt.call(e,"__wrapped__"),p=d&&dt.call(t,"__wrapped__");if(f||p){var m=f?e.value():e,v=p?t.value():t;return i||(i=new Zn),r(m,v,a,n,i)}}return!!h&&(i||(i=new Zn),Yo(e,t,a,n,r,i))}function Jr(e){return Hu(e)&&Xo(e)==Q}function Kr(e,t,a,n){var r=a.length,o=r,s=!n;if(null==e)return!o;e=at(e);while(r--){var l=a[r];if(s&&l[2]?l[1]!==e[l[0]]:!(l[0]in e))return!1}while(++r<o){l=a[r];var c=l[0],u=e[c],d=l[1];if(s&&l[2]){if(u===i&&!(c in e))return!1}else{var h=new Zn;if(n)var f=n(u,d,c,e,t,h);if(!(f===i?Gr(d,u,_|g,n,h):f))return!1}}return!0}function Xr(e){if(!wu(e)||ds(e))return!1;var t=Mu(e)?_t:Ke;return t.test(Ds(e))}function Zr(e){return Hu(e)&&Br(e)==ie}function Qr(e){return Hu(e)&&Xo(e)==oe}function ei(e){return Hu(e)&&Lu(e.length)&&!!Xt[Br(e)]}function ti(e){return"function"==typeof e?e:null==e?Oh:"object"==typeof e?lu(e)?si(e[0],e[1]):oi(e):Rh(e)}function ai(e){if(!fs(e))return It(e);var t=[];for(var a in at(e))dt.call(e,a)&&"constructor"!=a&&t.push(a);return t}function ni(e){if(!wu(e))return gs(e);var t=fs(e),a=[];for(var n in e)("constructor"!=n||!t&&dt.call(e,n))&&a.push(n);return a}function ri(e,t){return e<t}function ii(e,t){var n=-1,r=uu(e)?a(e.length):[];return zr(e,(function(e,a,i){r[++n]=t(e,a,i)})),r}function oi(e){var t=Uo(e);return 1==t.length&&t[0][2]?ms(t[0][0],t[0][1]):function(a){return a===e||Kr(a,e,t)}}function si(e,t){return ls(e)&&ps(t)?ms(Ts(e),t):function(a){var n=vd(a,e);return n===i&&n===t?gd(a,e):Gr(t,n,_|g)}}function li(e,t,a,n,r){e!==t&&Vr(t,(function(o,s){if(r||(r=new Zn),wu(o))ci(e,t,s,a,li,n,r);else{var l=n?n(Ls(e,s),o,s+"",e,t,r):i;l===i&&(l=o),lr(e,s,l)}}),Ld)}function ci(e,t,a,n,r,o,s){var l=Ls(e,a),c=Ls(t,a),u=s.get(c);if(u)lr(e,a,u);else{var d=o?o(l,c,a+"",e,t,s):i,h=d===i;if(h){var f=lu(c),p=!f&&fu(c),m=!f&&!p&&Fu(c);d=c,f||p||m?lu(l)?d=l:du(l)?d=no(l):p?(h=!1,d=Gi(c,!0)):m?(h=!1,d=Zi(c,!0)):d=[]:ju(c)||su(c)?(d=l,su(l)?d=Xu(l):wu(l)&&!Mu(l)||(d=as(c))):h=!1}h&&(s.set(c,d),r(d,c,n,o,s),s["delete"](c)),lr(e,a,d)}}function ui(e,t){var a=e.length;if(a)return t+=t<0?a:0,os(t,a)?e[t]:i}function di(e,t,a){t=t.length?Aa(t,(function(e){return lu(e)?function(t){return jr(t,1===e.length?e[0]:e)}:e})):[Oh];var n=-1;t=Aa(t,Ja(No()));var r=ii(e,(function(e,a,r){var i=Aa(t,(function(t){return t(e)}));return{criteria:i,index:++n,value:e}}));return Na(r,(function(e,t){return eo(e,t,a)}))}function hi(e,t){return fi(e,t,(function(t,a){return gd(e,a)}))}function fi(e,t,a){var n=-1,r=t.length,i={};while(++n<r){var o=t[n],s=jr(e,o);a(s,o)&&Li(i,Ri(o,e),s)}return i}function pi(e){return function(t){return jr(t,e)}}function mi(e,t,a,n){var r=n?Pa:Ya,i=-1,o=t.length,s=e;e===t&&(t=no(t)),a&&(s=Aa(e,Ja(a)));while(++i<o){var l=0,c=t[i],u=a?a(c):c;while((l=r(s,u,l,n))>-1)s!==e&&Ht.call(s,l,1),Ht.call(e,l,1)}return e}function vi(e,t){var a=e?t.length:0,n=a-1;while(a--){var r=t[a];if(a==n||r!==i){var i=r;os(r)?Ht.call(e,r,1):xi(e,r)}}return e}function _i(e,t){return e+jt(Ut()*(t-e+1))}function gi(e,t,n,r){var i=-1,o=Ft(Ct((t-e)/(n||1)),0),s=a(o);while(o--)s[r?o:++i]=e,e+=n;return s}function yi(e,t){var a="";if(!e||t<1||t>Y)return a;do{t%2&&(a+=e),t=jt(t/2),t&&(e+=e)}while(t);return a}function bi(e,t){return ks(bs(e,t,Oh),e+"")}function Mi(e){return ir(Ed(e))}function zi(e,t){var a=Ed(e);return Vs(a,vr(t,0,a.length))}function Li(e,t,a,n){if(!wu(e))return e;t=Ri(t,e);var r=-1,o=t.length,s=o-1,l=e;while(null!=l&&++r<o){var c=Ts(t[r]),u=a;if("__proto__"===c||"constructor"===c||"prototype"===c)return e;if(r!=s){var d=l[c];u=n?n(d,c,l):i,u===i&&(u=wu(d)?d:os(t[r+1])?[]:{})}cr(l,c,u),l=l[c]}return e}var wi=oa?function(e,t){return oa.set(e,t),e}:Oh,Hi=Vt?function(e,t){return Vt(e,"toString",{configurable:!0,enumerable:!1,value:kh(t),writable:!0})}:Oh;function ki(e){return Vs(Ed(e))}function Ai(e,t,n){var r=-1,i=e.length;t<0&&(t=-t>i?0:i+t),n=n>i?i:n,n<0&&(n+=i),i=t>n?0:n-t>>>0,t>>>=0;var o=a(i);while(++r<i)o[r]=e[r+t];return o}function Si(e,t){var a;return zr(e,(function(e,n,r){return a=t(e,n,r),!a})),!!a}function Vi(e,t,a){var n=0,r=null==e?n:e.length;if("number"==typeof t&&t===t&&r<=$){while(n<r){var i=n+r>>>1,o=e[i];null!==o&&!Iu(o)&&(a?o<=t:o<t)?n=i+1:r=i}return r}return Oi(e,t,Oh,a)}function Oi(e,t,a,n){var r=0,o=null==e?0:e.length;if(0===o)return 0;t=a(t);var s=t!==t,l=null===t,c=Iu(t),u=t===i;while(r<o){var d=jt((r+o)/2),h=a(e[d]),f=h!==i,p=null===h,m=h===h,v=Iu(h);if(s)var _=n||m;else _=u?m&&(n||f):l?m&&f&&(n||!p):c?m&&f&&!p&&(n||!v):!p&&!v&&(n?h<=t:h<t);_?r=d+1:o=d}return Et(o,E)}function Ti(e,t){var a=-1,n=e.length,r=0,i=[];while(++a<n){var o=e[a],s=t?t(o):o;if(!a||!ru(s,l)){var l=s;i[r++]=0===o?0:o}}return i}function Di(e){return"number"==typeof e?e:Iu(e)?I:+e}function Ci(e){if("string"==typeof e)return e;if(lu(e))return Aa(e,Ci)+"";if(Iu(e))return bn?bn.call(e):"";var t=e+"";return"0"==t&&1/e==-B?"-0":t}function ji(e,t,a){var n=-1,r=Ha,i=e.length,o=!0,l=[],c=l;if(a)o=!1,r=ka;else if(i>=s){var u=t?null:Vo(e);if(u)return hn(u);o=!1,r=Xa,c=new Jn}else c=t?[]:l;e:while(++n<i){var d=e[n],h=t?t(d):d;if(d=a||0!==d?d:0,o&&h===h){var f=c.length;while(f--)if(c[f]===h)continue e;t&&c.push(h),l.push(d)}else r(c,h,a)||(c!==l&&c.push(h),l.push(d))}return l}function xi(e,t){return t=Ri(t,e),e=Ms(e,t),null==e||delete e[Ts(il(t))]}function Bi(e,t,a,n){return Li(e,t,a(jr(e,t)),n)}function Yi(e,t,a,n){var r=e.length,i=n?r:-1;while((n?i--:++i<r)&&t(e[i],i,e));return a?Ai(e,n?0:i,n?i+1:r):Ai(e,n?i+1:0,n?r:i)}function Pi(e,t){var a=e;return a instanceof An&&(a=a.value()),Va(t,(function(e,t){return t.func.apply(t.thisArg,Sa([e],t.args))}),a)}function Ii(e,t,n){var r=e.length;if(r<2)return r?ji(e[0]):[];var i=-1,o=a(r);while(++i<r){var s=e[i],l=-1;while(++l<r)l!=i&&(o[i]=Mr(o[i]||s,e[l],t,n))}return ji(Sr(o,1),t,n)}function Fi(e,t,a){var n=-1,r=e.length,o=t.length,s={};while(++n<r){var l=n<o?t[n]:i;a(s,e[n],l)}return s}function Ei(e){return du(e)?e:[]}function $i(e){return"function"==typeof e?e:Oh}function Ri(e,t){return lu(e)?e:ls(e,t)?[e]:Os(Qu(e))}var Ni=bi;function Wi(e,t,a){var n=e.length;return a=a===i?n:a,!t&&a>=n?e:Ai(e,t,a)}var Ui=Ot||function(e){return sa.clearTimeout(e)};function Gi(e,t){if(t)return e.slice();var a=e.length,n=Mt?Mt(a):new e.constructor(a);return e.copy(n),n}function qi(e){var t=new e.constructor(e.byteLength);return new bt(t).set(new bt(e)),t}function Ji(e,t){var a=t?qi(e.buffer):e.buffer;return new e.constructor(a,e.byteOffset,e.byteLength)}function Ki(e){var t=new e.constructor(e.source,Ge.exec(e));return t.lastIndex=e.lastIndex,t}function Xi(e){return pn?at(pn.call(e)):{}}function Zi(e,t){var a=t?qi(e.buffer):e.buffer;return new e.constructor(a,e.byteOffset,e.length)}function Qi(e,t){if(e!==t){var a=e!==i,n=null===e,r=e===e,o=Iu(e),s=t!==i,l=null===t,c=t===t,u=Iu(t);if(!l&&!u&&!o&&e>t||o&&s&&c&&!l&&!u||n&&s&&c||!a&&c||!r)return 1;if(!n&&!o&&!u&&e<t||u&&a&&r&&!n&&!o||l&&a&&r||!s&&r||!c)return-1}return 0}function eo(e,t,a){var n=-1,r=e.criteria,i=t.criteria,o=r.length,s=a.length;while(++n<o){var l=Qi(r[n],i[n]);if(l){if(n>=s)return l;var c=a[n];return l*("desc"==c?-1:1)}}return e.index-t.index}function to(e,t,n,r){var i=-1,o=e.length,s=n.length,l=-1,c=t.length,u=Ft(o-s,0),d=a(c+u),h=!r;while(++l<c)d[l]=t[l];while(++i<s)(h||i<o)&&(d[n[i]]=e[i]);while(u--)d[l++]=e[i++];return d}function ao(e,t,n,r){var i=-1,o=e.length,s=-1,l=n.length,c=-1,u=t.length,d=Ft(o-l,0),h=a(d+u),f=!r;while(++i<d)h[i]=e[i];var p=i;while(++c<u)h[p+c]=t[c];while(++s<l)(f||i<o)&&(h[p+n[s]]=e[i++]);return h}function no(e,t){var n=-1,r=e.length;t||(t=a(r));while(++n<r)t[n]=e[n];return t}function ro(e,t,a,n){var r=!a;a||(a={});var o=-1,s=t.length;while(++o<s){var l=t[o],c=n?n(a[l],e[l],l,a,e):i;c===i&&(c=e[l]),r?pr(a,l,c):cr(a,l,c)}return a}function io(e,t){return ro(e,Jo(e),t)}function oo(e,t){return ro(e,Ko(e),t)}function so(e,t){return function(a,n){var r=lu(a)?ba:dr,i=t?t():{};return r(a,e,No(n,2),i)}}function lo(e){return bi((function(t,a){var n=-1,r=a.length,o=r>1?a[r-1]:i,s=r>2?a[2]:i;o=e.length>3&&"function"==typeof o?(r--,o):i,s&&ss(a[0],a[1],s)&&(o=r<3?i:o,r=1),t=at(t);while(++n<r){var l=a[n];l&&e(t,l,n,o)}return t}))}function co(e,t){return function(a,n){if(null==a)return a;if(!uu(a))return e(a,n);var r=a.length,i=t?r:-1,o=at(a);while(t?i--:++i<r)if(!1===n(o[i],i,o))break;return a}}function uo(e){return function(t,a,n){var r=-1,i=at(t),o=n(t),s=o.length;while(s--){var l=o[e?s:++r];if(!1===a(i[l],l,i))break}return t}}function ho(e,t,a){var n=t&y,r=mo(e);function i(){var t=this&&this!==sa&&this instanceof i?r:e;return t.apply(n?a:this,arguments)}return i}function fo(e){return function(t){t=Qu(t);var a=on(t)?_n(t):i,n=a?a[0]:t.charAt(0),r=a?Wi(a,1).join(""):t.slice(1);return n[e]()+r}}function po(e){return function(t){return Va(Mh(qd(t).replace(Rt,"")),e,"")}}function mo(e){return function(){var t=arguments;switch(t.length){case 0:return new e;case 1:return new e(t[0]);case 2:return new e(t[0],t[1]);case 3:return new e(t[0],t[1],t[2]);case 4:return new e(t[0],t[1],t[2],t[3]);case 5:return new e(t[0],t[1],t[2],t[3],t[4]);case 6:return new e(t[0],t[1],t[2],t[3],t[4],t[5]);case 7:return new e(t[0],t[1],t[2],t[3],t[4],t[5],t[6])}var a=Ln(e.prototype),n=e.apply(a,t);return wu(n)?n:a}}function vo(e,t,n){var r=mo(e);function o(){var s=arguments.length,l=a(s),c=s,u=Ro(o);while(c--)l[c]=arguments[c];var d=s<3&&l[0]!==u&&l[s-1]!==u?[]:dn(l,u);if(s-=d.length,s<n)return Ao(e,t,yo,o.placeholder,i,l,d,i,i,n-s);var h=this&&this!==sa&&this instanceof o?r:e;return ya(h,this,l)}return o}function _o(e){return function(t,a,n){var r=at(t);if(!uu(t)){var o=No(a,3);t=zd(t),a=function(e){return o(r[e],e,r)}}var s=e(t,a,n);return s>-1?r[o?t[s]:s]:i}}function go(e){return Po((function(t){var a=t.length,n=a,r=kn.prototype.thru;e&&t.reverse();while(n--){var o=t[n];if("function"!=typeof o)throw new it(c);if(r&&!s&&"wrapper"==$o(o))var s=new kn([],!0)}n=s?n:a;while(++n<a){o=t[n];var l=$o(o),u="wrapper"==l?Eo(o):i;s=u&&us(u[0])&&u[1]==(k|z|w|A)&&!u[4].length&&1==u[9]?s[$o(u[0])].apply(s,u[3]):1==o.length&&us(o)?s[l]():s.thru(o)}return function(){var e=arguments,n=e[0];if(s&&1==e.length&&lu(n))return s.plant(n).value();var r=0,i=a?t[r].apply(this,e):n;while(++r<a)i=t[r].call(this,i);return i}}))}function yo(e,t,n,r,o,s,l,c,u,d){var h=t&k,f=t&y,p=t&b,m=t&(z|L),v=t&S,_=p?i:mo(e);function g(){var i=arguments.length,y=a(i),b=i;while(b--)y[b]=arguments[b];if(m)var M=Ro(g),z=en(y,M);if(r&&(y=to(y,r,o,m)),s&&(y=ao(y,s,l,m)),i-=z,m&&i<d){var L=dn(y,M);return Ao(e,t,yo,g.placeholder,n,y,L,c,u,d-i)}var w=f?n:this,H=p?w[e]:e;return i=y.length,c?y=zs(y,c):v&&i>1&&y.reverse(),h&&u<i&&(y.length=u),this&&this!==sa&&this instanceof g&&(H=_||mo(H)),H.apply(w,y)}return g}function bo(e,t){return function(a,n){return $r(a,e,t(n),{})}}function Mo(e,t){return function(a,n){var r;if(a===i&&n===i)return t;if(a!==i&&(r=a),n!==i){if(r===i)return n;"string"==typeof a||"string"==typeof n?(a=Ci(a),n=Ci(n)):(a=Di(a),n=Di(n)),r=e(a,n)}return r}}function zo(e){return Po((function(t){return t=Aa(t,Ja(No())),bi((function(a){var n=this;return e(t,(function(e){return ya(e,n,a)}))}))}))}function Lo(e,t){t=t===i?" ":Ci(t);var a=t.length;if(a<2)return a?yi(t,e):t;var n=yi(t,Ct(e/vn(t)));return on(t)?Wi(_n(n),0,e).join(""):n.slice(0,e)}function wo(e,t,n,r){var i=t&y,o=mo(e);function s(){var t=-1,l=arguments.length,c=-1,u=r.length,d=a(u+l),h=this&&this!==sa&&this instanceof s?o:e;while(++c<u)d[c]=r[c];while(l--)d[c++]=arguments[++t];return ya(h,i?n:this,d)}return s}function Ho(e){return function(t,a,n){return n&&"number"!=typeof n&&ss(t,a,n)&&(a=n=i),t=Gu(t),a===i?(a=t,t=0):a=Gu(a),n=n===i?t<a?1:-1:Gu(n),gi(t,a,n,e)}}function ko(e){return function(t,a){return"string"==typeof t&&"string"==typeof a||(t=Ku(t),a=Ku(a)),e(t,a)}}function Ao(e,t,a,n,r,o,s,l,c,u){var d=t&z,h=d?s:i,f=d?i:s,p=d?o:i,m=d?i:o;t|=d?w:H,t&=~(d?H:w),t&M||(t&=~(y|b));var v=[e,t,r,p,h,m,f,l,c,u],_=a.apply(i,v);return us(e)&&ws(_,v),_.placeholder=n,As(_,e,t)}function So(e){var t=Re[e];return function(e,a){if(e=Ku(e),a=null==a?0:Et(qu(a),292),a&&Yt(e)){var n=(Qu(e)+"e").split("e"),r=t(n[0]+"e"+(+n[1]+a));return n=(Qu(r)+"e").split("e"),+(n[0]+"e"+(+n[1]-a))}return t(e)}}var Vo=ta&&1/hn(new ta([,-0]))[1]==B?function(e){return new ta(e)}:Ph;function Oo(e){return function(t){var a=Xo(t);return a==Q?cn(t):a==oe?fn(t):Ga(t,e(t))}}function To(e,t,a,n,r,o,s,l){var u=t&b;if(!u&&"function"!=typeof e)throw new it(c);var d=n?n.length:0;if(d||(t&=~(w|H),n=r=i),s=s===i?s:Ft(qu(s),0),l=l===i?l:qu(l),d-=r?r.length:0,t&H){var h=n,f=r;n=r=i}var p=u?i:Eo(e),m=[e,t,a,n,r,h,f,o,s,l];if(p&&_s(m,p),e=m[0],t=m[1],a=m[2],n=m[3],r=m[4],l=m[9]=m[9]===i?u?0:e.length:Ft(m[9]-d,0),!l&&t&(z|L)&&(t&=~(z|L)),t&&t!=y)v=t==z||t==L?vo(e,t,l):t!=w&&t!=(y|w)||r.length?yo.apply(i,m):wo(e,t,a,n);else var v=ho(e,t,a);var _=p?wi:ws;return As(_(v,m),e,t)}function Do(e,t,a,n){return e===i||ru(e,lt[a])&&!dt.call(n,a)?t:e}function Co(e,t,a,n,r,o){return wu(e)&&wu(t)&&(o.set(t,e),li(e,t,i,Co,o),o["delete"](t)),e}function jo(e){return ju(e)?i:e}function xo(e,t,a,n,r,o){var s=a&_,l=e.length,c=t.length;if(l!=c&&!(s&&c>l))return!1;var u=o.get(e),d=o.get(t);if(u&&d)return u==t&&d==e;var h=-1,f=!0,p=a&g?new Jn:i;o.set(e,t),o.set(t,e);while(++h<l){var m=e[h],v=t[h];if(n)var y=s?n(v,m,h,t,e,o):n(m,v,h,e,t,o);if(y!==i){if(y)continue;f=!1;break}if(p){if(!Ta(t,(function(e,t){if(!Xa(p,t)&&(m===e||r(m,e,a,n,o)))return p.push(t)}))){f=!1;break}}else if(m!==v&&!r(m,v,a,n,o)){f=!1;break}}return o["delete"](e),o["delete"](t),f}function Bo(e,t,a,n,r,i,o){switch(a){case fe:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case he:return!(e.byteLength!=t.byteLength||!i(new bt(e),new bt(t)));case G:case q:case ee:return ru(+e,+t);case K:return e.name==t.name&&e.message==t.message;case ie:case se:return e==t+"";case Q:var s=cn;case oe:var l=n&_;if(s||(s=hn),e.size!=t.size&&!l)return!1;var c=o.get(e);if(c)return c==t;n|=g,o.set(e,t);var u=xo(s(e),s(t),n,r,i,o);return o["delete"](e),u;case le:if(pn)return pn.call(e)==pn.call(t)}return!1}function Yo(e,t,a,n,r,o){var s=a&_,l=Io(e),c=l.length,u=Io(t),d=u.length;if(c!=d&&!s)return!1;var h=c;while(h--){var f=l[h];if(!(s?f in t:dt.call(t,f)))return!1}var p=o.get(e),m=o.get(t);if(p&&m)return p==t&&m==e;var v=!0;o.set(e,t),o.set(t,e);var g=s;while(++h<c){f=l[h];var y=e[f],b=t[f];if(n)var M=s?n(b,y,f,t,e,o):n(y,b,f,e,t,o);if(!(M===i?y===b||r(y,b,a,n,o):M)){v=!1;break}g||(g="constructor"==f)}if(v&&!g){var z=e.constructor,L=t.constructor;z==L||!("constructor"in e)||!("constructor"in t)||"function"==typeof z&&z instanceof z&&"function"==typeof L&&L instanceof L||(v=!1)}return o["delete"](e),o["delete"](t),v}function Po(e){return ks(bs(e,i,qs),e+"")}function Io(e){return xr(e,zd,Jo)}function Fo(e){return xr(e,Ld,Ko)}var Eo=oa?function(e){return oa.get(e)}:Ph;function $o(e){var t=e.name+"",a=la[t],n=dt.call(la,t)?a.length:0;while(n--){var r=a[n],i=r.func;if(null==i||i==e)return r.name}return t}function Ro(e){var t=dt.call(Mn,"placeholder")?Mn:e;return t.placeholder}function No(){var e=Mn.iteratee||Th;return e=e===Th?ti:e,arguments.length?e(arguments[0],arguments[1]):e}function Wo(e,t){var a=e.__data__;return cs(t)?a["string"==typeof t?"string":"hash"]:a.map}function Uo(e){var t=zd(e),a=t.length;while(a--){var n=t[a],r=e[n];t[a]=[n,r,ps(r)]}return t}function Go(e,t){var a=rn(e,t);return Xr(a)?a:i}function qo(e){var t=dt.call(e,St),a=e[St];try{e[St]=i;var n=!0}catch(o){}var r=pt.call(e);return n&&(t?e[St]=a:delete e[St]),r}var Jo=xt?function(e){return null==e?[]:(e=at(e),wa(xt(e),(function(t){return wt.call(e,t)})))}:Gh,Ko=xt?function(e){var t=[];while(e)Sa(t,Jo(e)),e=zt(e);return t}:Gh,Xo=Br;function Zo(e,t,a){var n=-1,r=a.length;while(++n<r){var i=a[n],o=i.size;switch(i.type){case"drop":e+=o;break;case"dropRight":t-=o;break;case"take":t=Et(t,e+o);break;case"takeRight":e=Ft(e,t-o);break}}return{start:e,end:t}}function Qo(e){var t=e.match(Ee);return t?t[1].split($e):[]}function es(e,t,a){t=Ri(t,e);var n=-1,r=t.length,i=!1;while(++n<r){var o=Ts(t[n]);if(!(i=null!=e&&a(e,o)))break;e=e[o]}return i||++n!=r?i:(r=null==e?0:e.length,!!r&&Lu(r)&&os(o,r)&&(lu(e)||su(e)))}function ts(e){var t=e.length,a=new e.constructor(t);return t&&"string"==typeof e[0]&&dt.call(e,"index")&&(a.index=e.index,a.input=e.input),a}function as(e){return"function"!=typeof e.constructor||fs(e)?{}:Ln(zt(e))}function ns(e,t,a){var n=e.constructor;switch(t){case he:return qi(e);case G:case q:return new n(+e);case fe:return Ji(e,a);case pe:case me:case ve:case _e:case ge:case ye:case be:case Me:case ze:return Zi(e,a);case Q:return new n;case ee:case se:return new n(e);case ie:return Ki(e);case oe:return new n;case le:return Xi(e)}}function rs(e,t){var a=t.length;if(!a)return e;var n=a-1;return t[n]=(a>1?"& ":"")+t[n],t=t.join(a>2?", ":" "),e.replace(Fe,"{\n/* [wrapped with "+t+"] */\n")}function is(e){return lu(e)||su(e)||!!(kt&&e&&e[kt])}function os(e,t){var a=typeof e;return t=null==t?Y:t,!!t&&("number"==a||"symbol"!=a&&Ze.test(e))&&e>-1&&e%1==0&&e<t}function ss(e,t,a){if(!wu(a))return!1;var n=typeof t;return!!("number"==n?uu(a)&&os(t,a.length):"string"==n&&t in a)&&ru(a[t],e)}function ls(e,t){if(lu(e))return!1;var a=typeof e;return!("number"!=a&&"symbol"!=a&&"boolean"!=a&&null!=e&&!Iu(e))||(je.test(e)||!Ce.test(e)||null!=t&&e in at(t))}function cs(e){var t=typeof e;return"string"==t||"number"==t||"symbol"==t||"boolean"==t?"__proto__"!==e:null===e}function us(e){var t=$o(e),a=Mn[t];if("function"!=typeof a||!(t in An.prototype))return!1;if(e===a)return!0;var n=Eo(a);return!!n&&e===n[0]}function ds(e){return!!ft&&ft in e}(qt&&Xo(new qt(new ArrayBuffer(1)))!=fe||Qt&&Xo(new Qt)!=Q||ea&&Xo(ea.resolve())!=ne||ta&&Xo(new ta)!=oe||aa&&Xo(new aa)!=ue)&&(Xo=function(e){var t=Br(e),a=t==ae?e.constructor:i,n=a?Ds(a):"";if(n)switch(n){case ca:return fe;case da:return Q;case ha:return ne;case Da:return oe;case Ca:return ue}return t});var hs=ct?Mu:qh;function fs(e){var t=e&&e.constructor,a="function"==typeof t&&t.prototype||lt;return e===a}function ps(e){return e===e&&!wu(e)}function ms(e,t){return function(a){return null!=a&&(a[e]===t&&(t!==i||e in at(a)))}}function vs(e){var t=Fc(e,(function(e){return a.size===h&&a.clear(),e})),a=t.cache;return t}function _s(e,t){var a=e[1],n=t[1],r=a|n,i=r<(y|b|k),o=n==k&&a==z||n==k&&a==A&&e[7].length<=t[8]||n==(k|A)&&t[7].length<=t[8]&&a==z;if(!i&&!o)return e;n&y&&(e[2]=t[2],r|=a&y?0:M);var s=t[3];if(s){var l=e[3];e[3]=l?to(l,s,t[4]):s,e[4]=l?dn(e[3],f):t[4]}return s=t[5],s&&(l=e[5],e[5]=l?ao(l,s,t[6]):s,e[6]=l?dn(e[5],f):t[6]),s=t[7],s&&(e[7]=s),n&k&&(e[8]=null==e[8]?t[8]:Et(e[8],t[8])),null==e[9]&&(e[9]=t[9]),e[0]=t[0],e[1]=r,e}function gs(e){var t=[];if(null!=e)for(var a in at(e))t.push(a);return t}function ys(e){return pt.call(e)}function bs(e,t,n){return t=Ft(t===i?e.length-1:t,0),function(){var r=arguments,i=-1,o=Ft(r.length-t,0),s=a(o);while(++i<o)s[i]=r[t+i];i=-1;var l=a(t+1);while(++i<t)l[i]=r[i];return l[t]=n(s),ya(e,this,l)}}function Ms(e,t){return t.length<2?e:jr(e,Ai(t,0,-1))}function zs(e,t){var a=e.length,n=Et(t.length,a),r=no(e);while(n--){var o=t[n];e[n]=os(o,a)?r[o]:i}return e}function Ls(e,t){if(("constructor"!==t||"function"!==typeof e[t])&&"__proto__"!=t)return e[t]}var ws=Ss(wi),Hs=Dt||function(e,t){return sa.setTimeout(e,t)},ks=Ss(Hi);function As(e,t,a){var n=t+"";return ks(e,rs(n,Cs(Qo(n),a)))}function Ss(e){var t=0,a=0;return function(){var n=$t(),r=D-(n-a);if(a=n,r>0){if(++t>=T)return arguments[0]}else t=0;return e.apply(i,arguments)}}function Vs(e,t){var a=-1,n=e.length,r=n-1;t=t===i?n:t;while(++a<t){var o=_i(a,r),s=e[o];e[o]=e[a],e[a]=s}return e.length=t,e}var Os=vs((function(e){var t=[];return 46===e.charCodeAt(0)&&t.push(""),e.replace(xe,(function(e,a,n,r){t.push(n?r.replace(We,"$1"):a||e)})),t}));function Ts(e){if("string"==typeof e||Iu(e))return e;var t=e+"";return"0"==t&&1/e==-B?"-0":t}function Ds(e){if(null!=e){try{return ut.call(e)}catch(t){}try{return e+""}catch(t){}}return""}function Cs(e,t){return Ma(R,(function(a){var n="_."+a[0];t&a[1]&&!Ha(e,n)&&e.push(n)})),e.sort()}function js(e){if(e instanceof An)return e.clone();var t=new kn(e.__wrapped__,e.__chain__);return t.__actions__=no(e.__actions__),t.__index__=e.__index__,t.__values__=e.__values__,t}function xs(e,t,n){t=(n?ss(e,t,n):t===i)?1:Ft(qu(t),0);var r=null==e?0:e.length;if(!r||t<1)return[];var o=0,s=0,l=a(Ct(r/t));while(o<r)l[s++]=Ai(e,o,o+=t);return l}function Bs(e){var t=-1,a=null==e?0:e.length,n=0,r=[];while(++t<a){var i=e[t];i&&(r[n++]=i)}return r}function Ys(){var e=arguments.length;if(!e)return[];var t=a(e-1),n=arguments[0],r=e;while(r--)t[r-1]=arguments[r];return Sa(lu(n)?no(n):[n],Sr(t,1))}var Ps=bi((function(e,t){return du(e)?Mr(e,Sr(t,1,du,!0)):[]})),Is=bi((function(e,t){var a=il(t);return du(a)&&(a=i),du(e)?Mr(e,Sr(t,1,du,!0),No(a,2)):[]})),Fs=bi((function(e,t){var a=il(t);return du(a)&&(a=i),du(e)?Mr(e,Sr(t,1,du,!0),i,a):[]}));function Es(e,t,a){var n=null==e?0:e.length;return n?(t=a||t===i?1:qu(t),Ai(e,t<0?0:t,n)):[]}function $s(e,t,a){var n=null==e?0:e.length;return n?(t=a||t===i?1:qu(t),t=n-t,Ai(e,0,t<0?0:t)):[]}function Rs(e,t){return e&&e.length?Yi(e,No(t,3),!0,!0):[]}function Ns(e,t){return e&&e.length?Yi(e,No(t,3),!0):[]}function Ws(e,t,a,n){var r=null==e?0:e.length;return r?(a&&"number"!=typeof a&&ss(e,t,a)&&(a=0,n=r),kr(e,t,a,n)):[]}function Us(e,t,a){var n=null==e?0:e.length;if(!n)return-1;var r=null==a?0:qu(a);return r<0&&(r=Ft(n+r,0)),Ba(e,No(t,3),r)}function Gs(e,t,a){var n=null==e?0:e.length;if(!n)return-1;var r=n-1;return a!==i&&(r=qu(a),r=a<0?Ft(n+r,0):Et(r,n-1)),Ba(e,No(t,3),r,!0)}function qs(e){var t=null==e?0:e.length;return t?Sr(e,1):[]}function Js(e){var t=null==e?0:e.length;return t?Sr(e,B):[]}function Ks(e,t){var a=null==e?0:e.length;return a?(t=t===i?1:qu(t),Sr(e,t)):[]}function Xs(e){var t=-1,a=null==e?0:e.length,n={};while(++t<a){var r=e[t];n[r[0]]=r[1]}return n}function Zs(e){return e&&e.length?e[0]:i}function Qs(e,t,a){var n=null==e?0:e.length;if(!n)return-1;var r=null==a?0:qu(a);return r<0&&(r=Ft(n+r,0)),Ya(e,t,r)}function el(e){var t=null==e?0:e.length;return t?Ai(e,0,-1):[]}var tl=bi((function(e){var t=Aa(e,Ei);return t.length&&t[0]===e[0]?Er(t):[]})),al=bi((function(e){var t=il(e),a=Aa(e,Ei);return t===il(a)?t=i:a.pop(),a.length&&a[0]===e[0]?Er(a,No(t,2)):[]})),nl=bi((function(e){var t=il(e),a=Aa(e,Ei);return t="function"==typeof t?t:i,t&&a.pop(),a.length&&a[0]===e[0]?Er(a,i,t):[]}));function rl(e,t){return null==e?"":Pt.call(e,t)}function il(e){var t=null==e?0:e.length;return t?e[t-1]:i}function ol(e,t,a){var n=null==e?0:e.length;if(!n)return-1;var r=n;return a!==i&&(r=qu(a),r=r<0?Ft(n+r,0):Et(r,n-1)),t===t?mn(e,t,r):Ba(e,Ia,r,!0)}function sl(e,t){return e&&e.length?ui(e,qu(t)):i}var ll=bi(cl);function cl(e,t){return e&&e.length&&t&&t.length?mi(e,t):e}function ul(e,t,a){return e&&e.length&&t&&t.length?mi(e,t,No(a,2)):e}function dl(e,t,a){return e&&e.length&&t&&t.length?mi(e,t,i,a):e}var hl=Po((function(e,t){var a=null==e?0:e.length,n=mr(e,t);return vi(e,Aa(t,(function(e){return os(e,a)?+e:e})).sort(Qi)),n}));function fl(e,t){var a=[];if(!e||!e.length)return a;var n=-1,r=[],i=e.length;t=No(t,3);while(++n<i){var o=e[n];t(o,n,e)&&(a.push(o),r.push(n))}return vi(e,r),a}function pl(e){return null==e?e:Gt.call(e)}function ml(e,t,a){var n=null==e?0:e.length;return n?(a&&"number"!=typeof a&&ss(e,t,a)?(t=0,a=n):(t=null==t?0:qu(t),a=a===i?n:qu(a)),Ai(e,t,a)):[]}function vl(e,t){return Vi(e,t)}function _l(e,t,a){return Oi(e,t,No(a,2))}function gl(e,t){var a=null==e?0:e.length;if(a){var n=Vi(e,t);if(n<a&&ru(e[n],t))return n}return-1}function yl(e,t){return Vi(e,t,!0)}function bl(e,t,a){return Oi(e,t,No(a,2),!0)}function Ml(e,t){var a=null==e?0:e.length;if(a){var n=Vi(e,t,!0)-1;if(ru(e[n],t))return n}return-1}function zl(e){return e&&e.length?Ti(e):[]}function Ll(e,t){return e&&e.length?Ti(e,No(t,2)):[]}function wl(e){var t=null==e?0:e.length;return t?Ai(e,1,t):[]}function Hl(e,t,a){return e&&e.length?(t=a||t===i?1:qu(t),Ai(e,0,t<0?0:t)):[]}function kl(e,t,a){var n=null==e?0:e.length;return n?(t=a||t===i?1:qu(t),t=n-t,Ai(e,t<0?0:t,n)):[]}function Al(e,t){return e&&e.length?Yi(e,No(t,3),!1,!0):[]}function Sl(e,t){return e&&e.length?Yi(e,No(t,3)):[]}var Vl=bi((function(e){return ji(Sr(e,1,du,!0))})),Ol=bi((function(e){var t=il(e);return du(t)&&(t=i),ji(Sr(e,1,du,!0),No(t,2))})),Tl=bi((function(e){var t=il(e);return t="function"==typeof t?t:i,ji(Sr(e,1,du,!0),i,t)}));function Dl(e){return e&&e.length?ji(e):[]}function Cl(e,t){return e&&e.length?ji(e,No(t,2)):[]}function jl(e,t){return t="function"==typeof t?t:i,e&&e.length?ji(e,i,t):[]}function xl(e){if(!e||!e.length)return[];var t=0;return e=wa(e,(function(e){if(du(e))return t=Ft(e.length,t),!0})),Ua(t,(function(t){return Aa(e,Ea(t))}))}function Bl(e,t){if(!e||!e.length)return[];var a=xl(e);return null==t?a:Aa(a,(function(e){return ya(t,i,e)}))}var Yl=bi((function(e,t){return du(e)?Mr(e,t):[]})),Pl=bi((function(e){return Ii(wa(e,du))})),Il=bi((function(e){var t=il(e);return du(t)&&(t=i),Ii(wa(e,du),No(t,2))})),Fl=bi((function(e){var t=il(e);return t="function"==typeof t?t:i,Ii(wa(e,du),i,t)})),El=bi(xl);function $l(e,t){return Fi(e||[],t||[],cr)}function Rl(e,t){return Fi(e||[],t||[],Li)}var Nl=bi((function(e){var t=e.length,a=t>1?e[t-1]:i;return a="function"==typeof a?(e.pop(),a):i,Bl(e,a)}));function Wl(e){var t=Mn(e);return t.__chain__=!0,t}function Ul(e,t){return t(e),e}function Gl(e,t){return t(e)}var ql=Po((function(e){var t=e.length,a=t?e[0]:0,n=this.__wrapped__,r=function(t){return mr(t,e)};return!(t>1||this.__actions__.length)&&n instanceof An&&os(a)?(n=n.slice(a,+a+(t?1:0)),n.__actions__.push({func:Gl,args:[r],thisArg:i}),new kn(n,this.__chain__).thru((function(e){return t&&!e.length&&e.push(i),e}))):this.thru(r)}));function Jl(){return Wl(this)}function Kl(){return new kn(this.value(),this.__chain__)}function Xl(){this.__values__===i&&(this.__values__=Uu(this.value()));var e=this.__index__>=this.__values__.length,t=e?i:this.__values__[this.__index__++];return{done:e,value:t}}function Zl(){return this}function Ql(e){var t,a=this;while(a instanceof Hn){var n=js(a);n.__index__=0,n.__values__=i,t?r.__wrapped__=n:t=n;var r=n;a=a.__wrapped__}return r.__wrapped__=e,t}function ec(){var e=this.__wrapped__;if(e instanceof An){var t=e;return this.__actions__.length&&(t=new An(this)),t=t.reverse(),t.__actions__.push({func:Gl,args:[pl],thisArg:i}),new kn(t,this.__chain__)}return this.thru(pl)}function tc(){return Pi(this.__wrapped__,this.__actions__)}var ac=so((function(e,t,a){dt.call(e,a)?++e[a]:pr(e,a,1)}));function nc(e,t,a){var n=lu(e)?La:wr;return a&&ss(e,t,a)&&(t=i),n(e,No(t,3))}function rc(e,t){var a=lu(e)?wa:Ar;return a(e,No(t,3))}var ic=_o(Us),oc=_o(Gs);function sc(e,t){return Sr(vc(e,t),1)}function lc(e,t){return Sr(vc(e,t),B)}function cc(e,t,a){return a=a===i?1:qu(a),Sr(vc(e,t),a)}function uc(e,t){var a=lu(e)?Ma:zr;return a(e,No(t,3))}function dc(e,t){var a=lu(e)?za:Lr;return a(e,No(t,3))}var hc=so((function(e,t,a){dt.call(e,a)?e[a].push(t):pr(e,a,[t])}));function fc(e,t,a,n){e=uu(e)?e:Ed(e),a=a&&!n?qu(a):0;var r=e.length;return a<0&&(a=Ft(r+a,0)),Pu(e)?a<=r&&e.indexOf(t,a)>-1:!!r&&Ya(e,t,a)>-1}var pc=bi((function(e,t,n){var r=-1,i="function"==typeof t,o=uu(e)?a(e.length):[];return zr(e,(function(e){o[++r]=i?ya(t,e,n):Rr(e,t,n)})),o})),mc=so((function(e,t,a){pr(e,a,t)}));function vc(e,t){var a=lu(e)?Aa:ii;return a(e,No(t,3))}function _c(e,t,a,n){return null==e?[]:(lu(t)||(t=null==t?[]:[t]),a=n?i:a,lu(a)||(a=null==a?[]:[a]),di(e,t,a))}var gc=so((function(e,t,a){e[a?0:1].push(t)}),(function(){return[[],[]]}));function yc(e,t,a){var n=lu(e)?Va:Ra,r=arguments.length<3;return n(e,No(t,4),a,r,zr)}function bc(e,t,a){var n=lu(e)?Oa:Ra,r=arguments.length<3;return n(e,No(t,4),a,r,Lr)}function Mc(e,t){var a=lu(e)?wa:Ar;return a(e,Ec(No(t,3)))}function zc(e){var t=lu(e)?ir:Mi;return t(e)}function Lc(e,t,a){t=(a?ss(e,t,a):t===i)?1:qu(t);var n=lu(e)?or:zi;return n(e,t)}function wc(e){var t=lu(e)?sr:ki;return t(e)}function Hc(e){if(null==e)return 0;if(uu(e))return Pu(e)?vn(e):e.length;var t=Xo(e);return t==Q||t==oe?e.size:ai(e).length}function kc(e,t,a){var n=lu(e)?Ta:Si;return a&&ss(e,t,a)&&(t=i),n(e,No(t,3))}var Ac=bi((function(e,t){if(null==e)return[];var a=t.length;return a>1&&ss(e,t[0],t[1])?t=[]:a>2&&ss(t[0],t[1],t[2])&&(t=[t[0]]),di(e,Sr(t,1),[])})),Sc=Tt||function(){return sa.Date.now()};function Vc(e,t){if("function"!=typeof t)throw new it(c);return e=qu(e),function(){if(--e<1)return t.apply(this,arguments)}}function Oc(e,t,a){return t=a?i:t,t=e&&null==t?e.length:t,To(e,k,i,i,i,i,t)}function Tc(e,t){var a;if("function"!=typeof t)throw new it(c);return e=qu(e),function(){return--e>0&&(a=t.apply(this,arguments)),e<=1&&(t=i),a}}var Dc=bi((function(e,t,a){var n=y;if(a.length){var r=dn(a,Ro(Dc));n|=w}return To(e,n,t,a,r)})),Cc=bi((function(e,t,a){var n=y|b;if(a.length){var r=dn(a,Ro(Cc));n|=w}return To(t,n,e,a,r)}));function jc(e,t,a){t=a?i:t;var n=To(e,z,i,i,i,i,i,t);return n.placeholder=jc.placeholder,n}function xc(e,t,a){t=a?i:t;var n=To(e,L,i,i,i,i,i,t);return n.placeholder=xc.placeholder,n}function Bc(e,t,a){var n,r,o,s,l,u,d=0,h=!1,f=!1,p=!0;if("function"!=typeof e)throw new it(c);function m(t){var a=n,o=r;return n=r=i,d=t,s=e.apply(o,a),s}function v(e){return d=e,l=Hs(y,t),h?m(e):s}function _(e){var a=e-u,n=e-d,r=t-a;return f?Et(r,o-n):r}function g(e){var a=e-u,n=e-d;return u===i||a>=t||a<0||f&&n>=o}function y(){var e=Sc();if(g(e))return b(e);l=Hs(y,_(e))}function b(e){return l=i,p&&n?m(e):(n=r=i,s)}function M(){l!==i&&Ui(l),d=0,n=u=r=l=i}function z(){return l===i?s:b(Sc())}function L(){var e=Sc(),a=g(e);if(n=arguments,r=this,u=e,a){if(l===i)return v(u);if(f)return Ui(l),l=Hs(y,t),m(u)}return l===i&&(l=Hs(y,t)),s}return t=Ku(t)||0,wu(a)&&(h=!!a.leading,f="maxWait"in a,o=f?Ft(Ku(a.maxWait)||0,t):o,p="trailing"in a?!!a.trailing:p),L.cancel=M,L.flush=z,L}var Yc=bi((function(e,t){return br(e,1,t)})),Pc=bi((function(e,t,a){return br(e,Ku(t)||0,a)}));function Ic(e){return To(e,S)}function Fc(e,t){if("function"!=typeof e||null!=t&&"function"!=typeof t)throw new it(c);var a=function(){var n=arguments,r=t?t.apply(this,n):n[0],i=a.cache;if(i.has(r))return i.get(r);var o=e.apply(this,n);return a.cache=i.set(r,o)||i,o};return a.cache=new(Fc.Cache||Rn),a}function Ec(e){if("function"!=typeof e)throw new it(c);return function(){var t=arguments;switch(t.length){case 0:return!e.call(this);case 1:return!e.call(this,t[0]);case 2:return!e.call(this,t[0],t[1]);case 3:return!e.call(this,t[0],t[1],t[2])}return!e.apply(this,t)}}function $c(e){return Tc(2,e)}Fc.Cache=Rn;var Rc=Ni((function(e,t){t=1==t.length&&lu(t[0])?Aa(t[0],Ja(No())):Aa(Sr(t,1),Ja(No()));var a=t.length;return bi((function(n){var r=-1,i=Et(n.length,a);while(++r<i)n[r]=t[r].call(this,n[r]);return ya(e,this,n)}))})),Nc=bi((function(e,t){var a=dn(t,Ro(Nc));return To(e,w,i,t,a)})),Wc=bi((function(e,t){var a=dn(t,Ro(Wc));return To(e,H,i,t,a)})),Uc=Po((function(e,t){return To(e,A,i,i,i,t)}));function Gc(e,t){if("function"!=typeof e)throw new it(c);return t=t===i?t:qu(t),bi(e,t)}function qc(e,t){if("function"!=typeof e)throw new it(c);return t=null==t?0:Ft(qu(t),0),bi((function(a){var n=a[t],r=Wi(a,0,t);return n&&Sa(r,n),ya(e,this,r)}))}function Jc(e,t,a){var n=!0,r=!0;if("function"!=typeof e)throw new it(c);return wu(a)&&(n="leading"in a?!!a.leading:n,r="trailing"in a?!!a.trailing:r),Bc(e,t,{leading:n,maxWait:t,trailing:r})}function Kc(e){return Oc(e,1)}function Xc(e,t){return Nc($i(t),e)}function Zc(){if(!arguments.length)return[];var e=arguments[0];return lu(e)?e:[e]}function Qc(e){return _r(e,v)}function eu(e,t){return t="function"==typeof t?t:i,_r(e,v,t)}function tu(e){return _r(e,p|v)}function au(e,t){return t="function"==typeof t?t:i,_r(e,p|v,t)}function nu(e,t){return null==t||yr(e,t,zd(t))}function ru(e,t){return e===t||e!==e&&t!==t}var iu=ko(Yr),ou=ko((function(e,t){return e>=t})),su=Nr(function(){return arguments}())?Nr:function(e){return Hu(e)&&dt.call(e,"callee")&&!wt.call(e,"callee")},lu=a.isArray,cu=fa?Ja(fa):Wr;function uu(e){return null!=e&&Lu(e.length)&&!Mu(e)}function du(e){return Hu(e)&&uu(e)}function hu(e){return!0===e||!1===e||Hu(e)&&Br(e)==G}var fu=Bt||qh,pu=pa?Ja(pa):Ur;function mu(e){return Hu(e)&&1===e.nodeType&&!ju(e)}function vu(e){if(null==e)return!0;if(uu(e)&&(lu(e)||"string"==typeof e||"function"==typeof e.splice||fu(e)||Fu(e)||su(e)))return!e.length;var t=Xo(e);if(t==Q||t==oe)return!e.size;if(fs(e))return!ai(e).length;for(var a in e)if(dt.call(e,a))return!1;return!0}function _u(e,t){return Gr(e,t)}function gu(e,t,a){a="function"==typeof a?a:i;var n=a?a(e,t):i;return n===i?Gr(e,t,i,a):!!n}function yu(e){if(!Hu(e))return!1;var t=Br(e);return t==K||t==J||"string"==typeof e.message&&"string"==typeof e.name&&!ju(e)}function bu(e){return"number"==typeof e&&Yt(e)}function Mu(e){if(!wu(e))return!1;var t=Br(e);return t==X||t==Z||t==U||t==re}function zu(e){return"number"==typeof e&&e==qu(e)}function Lu(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=Y}function wu(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}function Hu(e){return null!=e&&"object"==typeof e}var ku=ma?Ja(ma):Jr;function Au(e,t){return e===t||Kr(e,t,Uo(t))}function Su(e,t,a){return a="function"==typeof a?a:i,Kr(e,t,Uo(t),a)}function Vu(e){return Cu(e)&&e!=+e}function Ou(e){if(hs(e))throw new r(l);return Xr(e)}function Tu(e){return null===e}function Du(e){return null==e}function Cu(e){return"number"==typeof e||Hu(e)&&Br(e)==ee}function ju(e){if(!Hu(e)||Br(e)!=ae)return!1;var t=zt(e);if(null===t)return!0;var a=dt.call(t,"constructor")&&t.constructor;return"function"==typeof a&&a instanceof a&&ut.call(a)==mt}var xu=va?Ja(va):Zr;function Bu(e){return zu(e)&&e>=-Y&&e<=Y}var Yu=_a?Ja(_a):Qr;function Pu(e){return"string"==typeof e||!lu(e)&&Hu(e)&&Br(e)==se}function Iu(e){return"symbol"==typeof e||Hu(e)&&Br(e)==le}var Fu=ga?Ja(ga):ei;function Eu(e){return e===i}function $u(e){return Hu(e)&&Xo(e)==ue}function Ru(e){return Hu(e)&&Br(e)==de}var Nu=ko(ri),Wu=ko((function(e,t){return e<=t}));function Uu(e){if(!e)return[];if(uu(e))return Pu(e)?_n(e):no(e);if(At&&e[At])return ln(e[At]());var t=Xo(e),a=t==Q?cn:t==oe?hn:Ed;return a(e)}function Gu(e){if(!e)return 0===e?e:0;if(e=Ku(e),e===B||e===-B){var t=e<0?-1:1;return t*P}return e===e?e:0}function qu(e){var t=Gu(e),a=t%1;return t===t?a?t-a:t:0}function Ju(e){return e?vr(qu(e),0,F):0}function Ku(e){if("number"==typeof e)return e;if(Iu(e))return I;if(wu(e)){var t="function"==typeof e.valueOf?e.valueOf():e;e=wu(t)?t+"":t}if("string"!=typeof e)return 0===e?e:+e;e=qa(e);var a=Je.test(e);return a||Xe.test(e)?ra(e.slice(2),a?2:8):qe.test(e)?I:+e}function Xu(e){return ro(e,Ld(e))}function Zu(e){return e?vr(qu(e),-Y,Y):0===e?e:0}function Qu(e){return null==e?"":Ci(e)}var ed=lo((function(e,t){if(fs(t)||uu(t))ro(t,zd(t),e);else for(var a in t)dt.call(t,a)&&cr(e,a,t[a])})),td=lo((function(e,t){ro(t,Ld(t),e)})),ad=lo((function(e,t,a,n){ro(t,Ld(t),e,n)})),nd=lo((function(e,t,a,n){ro(t,zd(t),e,n)})),rd=Po(mr);function id(e,t){var a=Ln(e);return null==t?a:hr(a,t)}var od=bi((function(e,t){e=at(e);var a=-1,n=t.length,r=n>2?t[2]:i;r&&ss(t[0],t[1],r)&&(n=1);while(++a<n){var o=t[a],s=Ld(o),l=-1,c=s.length;while(++l<c){var u=s[l],d=e[u];(d===i||ru(d,lt[u])&&!dt.call(e,u))&&(e[u]=o[u])}}return e})),sd=bi((function(e){return e.push(i,Co),ya(Ad,i,e)}));function ld(e,t){return xa(e,No(t,3),Tr)}function cd(e,t){return xa(e,No(t,3),Dr)}function ud(e,t){return null==e?e:Vr(e,No(t,3),Ld)}function dd(e,t){return null==e?e:Or(e,No(t,3),Ld)}function hd(e,t){return e&&Tr(e,No(t,3))}function fd(e,t){return e&&Dr(e,No(t,3))}function pd(e){return null==e?[]:Cr(e,zd(e))}function md(e){return null==e?[]:Cr(e,Ld(e))}function vd(e,t,a){var n=null==e?i:jr(e,t);return n===i?a:n}function _d(e,t){return null!=e&&es(e,t,Pr)}function gd(e,t){return null!=e&&es(e,t,Ir)}var yd=bo((function(e,t,a){null!=t&&"function"!=typeof t.toString&&(t=pt.call(t)),e[t]=a}),kh(Oh)),bd=bo((function(e,t,a){null!=t&&"function"!=typeof t.toString&&(t=pt.call(t)),dt.call(e,t)?e[t].push(a):e[t]=[a]}),No),Md=bi(Rr);function zd(e){return uu(e)?rr(e):ai(e)}function Ld(e){return uu(e)?rr(e,!0):ni(e)}function wd(e,t){var a={};return t=No(t,3),Tr(e,(function(e,n,r){pr(a,t(e,n,r),e)})),a}function Hd(e,t){var a={};return t=No(t,3),Tr(e,(function(e,n,r){pr(a,n,t(e,n,r))})),a}var kd=lo((function(e,t,a){li(e,t,a)})),Ad=lo((function(e,t,a,n){li(e,t,a,n)})),Sd=Po((function(e,t){var a={};if(null==e)return a;var n=!1;t=Aa(t,(function(t){return t=Ri(t,e),n||(n=t.length>1),t})),ro(e,Fo(e),a),n&&(a=_r(a,p|m|v,jo));var r=t.length;while(r--)xi(a,t[r]);return a}));function Vd(e,t){return Td(e,Ec(No(t)))}var Od=Po((function(e,t){return null==e?{}:hi(e,t)}));function Td(e,t){if(null==e)return{};var a=Aa(Fo(e),(function(e){return[e]}));return t=No(t),fi(e,a,(function(e,a){return t(e,a[0])}))}function Dd(e,t,a){t=Ri(t,e);var n=-1,r=t.length;r||(r=1,e=i);while(++n<r){var o=null==e?i:e[Ts(t[n])];o===i&&(n=r,o=a),e=Mu(o)?o.call(e):o}return e}function Cd(e,t,a){return null==e?e:Li(e,t,a)}function jd(e,t,a,n){return n="function"==typeof n?n:i,null==e?e:Li(e,t,a,n)}var xd=Oo(zd),Bd=Oo(Ld);function Yd(e,t,a){var n=lu(e),r=n||fu(e)||Fu(e);if(t=No(t,4),null==a){var i=e&&e.constructor;a=r?n?new i:[]:wu(e)&&Mu(i)?Ln(zt(e)):{}}return(r?Ma:Tr)(e,(function(e,n,r){return t(a,e,n,r)})),a}function Pd(e,t){return null==e||xi(e,t)}function Id(e,t,a){return null==e?e:Bi(e,t,$i(a))}function Fd(e,t,a,n){return n="function"==typeof n?n:i,null==e?e:Bi(e,t,$i(a),n)}function Ed(e){return null==e?[]:Ka(e,zd(e))}function $d(e){return null==e?[]:Ka(e,Ld(e))}function Rd(e,t,a){return a===i&&(a=t,t=i),a!==i&&(a=Ku(a),a=a===a?a:0),t!==i&&(t=Ku(t),t=t===t?t:0),vr(Ku(e),t,a)}function Nd(e,t,a){return t=Gu(t),a===i?(a=t,t=0):a=Gu(a),e=Ku(e),Fr(e,t,a)}function Wd(e,t,a){if(a&&"boolean"!=typeof a&&ss(e,t,a)&&(t=a=i),a===i&&("boolean"==typeof t?(a=t,t=i):"boolean"==typeof e&&(a=e,e=i)),e===i&&t===i?(e=0,t=1):(e=Gu(e),t===i?(t=e,e=0):t=Gu(t)),e>t){var n=e;e=t,t=n}if(a||e%1||t%1){var r=Ut();return Et(e+r*(t-e+na("1e-"+((r+"").length-1))),t)}return _i(e,t)}var Ud=po((function(e,t,a){return t=t.toLowerCase(),e+(a?Gd(t):t)}));function Gd(e){return bh(Qu(e).toLowerCase())}function qd(e){return e=Qu(e),e&&e.replace(Qe,tn).replace(Nt,"")}function Jd(e,t,a){e=Qu(e),t=Ci(t);var n=e.length;a=a===i?n:vr(qu(a),0,n);var r=a;return a-=t.length,a>=0&&e.slice(a,r)==t}function Kd(e){return e=Qu(e),e&&Ve.test(e)?e.replace(Ae,an):e}function Xd(e){return e=Qu(e),e&&Ye.test(e)?e.replace(Be,"\\$&"):e}var Zd=po((function(e,t,a){return e+(a?"-":"")+t.toLowerCase()})),Qd=po((function(e,t,a){return e+(a?" ":"")+t.toLowerCase()})),eh=fo("toLowerCase");function th(e,t,a){e=Qu(e),t=qu(t);var n=t?vn(e):0;if(!t||n>=t)return e;var r=(t-n)/2;return Lo(jt(r),a)+e+Lo(Ct(r),a)}function ah(e,t,a){e=Qu(e),t=qu(t);var n=t?vn(e):0;return t&&n<t?e+Lo(t-n,a):e}function nh(e,t,a){e=Qu(e),t=qu(t);var n=t?vn(e):0;return t&&n<t?Lo(t-n,a)+e:e}function rh(e,t,a){return a||null==t?t=0:t&&(t=+t),Wt(Qu(e).replace(Pe,""),t||0)}function ih(e,t,a){return t=(a?ss(e,t,a):t===i)?1:qu(t),yi(Qu(e),t)}function oh(){var e=arguments,t=Qu(e[0]);return e.length<3?t:t.replace(e[1],e[2])}var sh=po((function(e,t,a){return e+(a?"_":"")+t.toLowerCase()}));function lh(e,t,a){return a&&"number"!=typeof a&&ss(e,t,a)&&(t=a=i),a=a===i?F:a>>>0,a?(e=Qu(e),e&&("string"==typeof t||null!=t&&!xu(t))&&(t=Ci(t),!t&&on(e))?Wi(_n(e),0,a):e.split(t,a)):[]}var ch=po((function(e,t,a){return e+(a?" ":"")+bh(t)}));function uh(e,t,a){return e=Qu(e),a=null==a?0:vr(qu(a),0,e.length),t=Ci(t),e.slice(a,a+t.length)==t}function dh(e,t,a){var n=Mn.templateSettings;a&&ss(e,t,a)&&(t=i),e=Qu(e),t=ad({},t,n,Do);var o,s,l=ad({},t.imports,n.imports,Do),c=zd(l),d=Ka(l,c),h=0,f=t.interpolate||et,p="__p += '",m=nt((t.escape||et).source+"|"+f.source+"|"+(f===De?Ue:et).source+"|"+(t.evaluate||et).source+"|$","g"),v="//# sourceURL="+(dt.call(t,"sourceURL")?(t.sourceURL+"").replace(/\s/g," "):"lodash.templateSources["+ ++Kt+"]")+"\n";e.replace(m,(function(t,a,n,r,i,l){return n||(n=r),p+=e.slice(h,l).replace(tt,nn),a&&(o=!0,p+="' +\n__e("+a+") +\n'"),i&&(s=!0,p+="';\n"+i+";\n__p += '"),n&&(p+="' +\n((__t = ("+n+")) == null ? '' : __t) +\n'"),h=l+t.length,t})),p+="';\n";var _=dt.call(t,"variable")&&t.variable;if(_){if(Ne.test(_))throw new r(u)}else p="with (obj) {\n"+p+"\n}\n";p=(s?p.replace(Le,""):p).replace(we,"$1").replace(He,"$1;"),p="function("+(_||"obj")+") {\n"+(_?"":"obj || (obj = {});\n")+"var __t, __p = ''"+(o?", __e = _.escape":"")+(s?", __j = Array.prototype.join;\nfunction print() { __p += __j.call(arguments, '') }\n":";\n")+p+"return __p\n}";var g=zh((function(){return Ie(c,v+"return "+p).apply(i,d)}));if(g.source=p,yu(g))throw g;return g}function hh(e){return Qu(e).toLowerCase()}function fh(e){return Qu(e).toUpperCase()}function ph(e,t,a){if(e=Qu(e),e&&(a||t===i))return qa(e);if(!e||!(t=Ci(t)))return e;var n=_n(e),r=_n(t),o=Za(n,r),s=Qa(n,r)+1;return Wi(n,o,s).join("")}function mh(e,t,a){if(e=Qu(e),e&&(a||t===i))return e.slice(0,gn(e)+1);if(!e||!(t=Ci(t)))return e;var n=_n(e),r=Qa(n,_n(t))+1;return Wi(n,0,r).join("")}function vh(e,t,a){if(e=Qu(e),e&&(a||t===i))return e.replace(Pe,"");if(!e||!(t=Ci(t)))return e;var n=_n(e),r=Za(n,_n(t));return Wi(n,r).join("")}function _h(e,t){var a=V,n=O;if(wu(t)){var r="separator"in t?t.separator:r;a="length"in t?qu(t.length):a,n="omission"in t?Ci(t.omission):n}e=Qu(e);var o=e.length;if(on(e)){var s=_n(e);o=s.length}if(a>=o)return e;var l=a-vn(n);if(l<1)return n;var c=s?Wi(s,0,l).join(""):e.slice(0,l);if(r===i)return c+n;if(s&&(l+=c.length-l),xu(r)){if(e.slice(l).search(r)){var u,d=c;r.global||(r=nt(r.source,Qu(Ge.exec(r))+"g")),r.lastIndex=0;while(u=r.exec(d))var h=u.index;c=c.slice(0,h===i?l:h)}}else if(e.indexOf(Ci(r),l)!=l){var f=c.lastIndexOf(r);f>-1&&(c=c.slice(0,f))}return c+n}function gh(e){return e=Qu(e),e&&Se.test(e)?e.replace(ke,yn):e}var yh=po((function(e,t,a){return e+(a?" ":"")+t.toUpperCase()})),bh=fo("toUpperCase");function Mh(e,t,a){return e=Qu(e),t=a?i:t,t===i?sn(e)?zn(e):ja(e):e.match(t)||[]}var zh=bi((function(e,t){try{return ya(e,i,t)}catch(a){return yu(a)?a:new r(a)}})),Lh=Po((function(e,t){return Ma(t,(function(t){t=Ts(t),pr(e,t,Dc(e[t],e))})),e}));function wh(e){var t=null==e?0:e.length,a=No();return e=t?Aa(e,(function(e){if("function"!=typeof e[1])throw new it(c);return[a(e[0]),e[1]]})):[],bi((function(a){var n=-1;while(++n<t){var r=e[n];if(ya(r[0],this,a))return ya(r[1],this,a)}}))}function Hh(e){return gr(_r(e,p))}function kh(e){return function(){return e}}function Ah(e,t){return null==e||e!==e?t:e}var Sh=go(),Vh=go(!0);function Oh(e){return e}function Th(e){return ti("function"==typeof e?e:_r(e,p))}function Dh(e){return oi(_r(e,p))}function Ch(e,t){return si(e,_r(t,p))}var jh=bi((function(e,t){return function(a){return Rr(a,e,t)}})),xh=bi((function(e,t){return function(a){return Rr(e,a,t)}}));function Bh(e,t,a){var n=zd(t),r=Cr(t,n);null!=a||wu(t)&&(r.length||!n.length)||(a=t,t=e,e=this,r=Cr(t,zd(t)));var i=!(wu(a)&&"chain"in a)||!!a.chain,o=Mu(e);return Ma(r,(function(a){var n=t[a];e[a]=n,o&&(e.prototype[a]=function(){var t=this.__chain__;if(i||t){var a=e(this.__wrapped__),r=a.__actions__=no(this.__actions__);return r.push({func:n,args:arguments,thisArg:e}),a.__chain__=t,a}return n.apply(e,Sa([this.value()],arguments))})})),e}function Yh(){return sa._===this&&(sa._=vt),this}function Ph(){}function Ih(e){return e=qu(e),bi((function(t){return ui(t,e)}))}var Fh=zo(Aa),Eh=zo(La),$h=zo(Ta);function Rh(e){return ls(e)?Ea(Ts(e)):pi(e)}function Nh(e){return function(t){return null==e?i:jr(e,t)}}var Wh=Ho(),Uh=Ho(!0);function Gh(){return[]}function qh(){return!1}function Jh(){return{}}function Kh(){return""}function Xh(){return!0}function Zh(e,t){if(e=qu(e),e<1||e>Y)return[];var a=F,n=Et(e,F);t=No(t),e-=F;var r=Ua(n,t);while(++a<e)t(a);return r}function Qh(e){return lu(e)?Aa(e,Ts):Iu(e)?[e]:no(Os(Qu(e)))}function ef(e){var t=++ht;return Qu(e)+t}var tf=Mo((function(e,t){return e+t}),0),af=So("ceil"),nf=Mo((function(e,t){return e/t}),1),rf=So("floor");function of(e){return e&&e.length?Hr(e,Oh,Yr):i}function sf(e,t){return e&&e.length?Hr(e,No(t,2),Yr):i}function lf(e){return Fa(e,Oh)}function cf(e,t){return Fa(e,No(t,2))}function uf(e){return e&&e.length?Hr(e,Oh,ri):i}function df(e,t){return e&&e.length?Hr(e,No(t,2),ri):i}var hf=Mo((function(e,t){return e*t}),1),ff=So("round"),pf=Mo((function(e,t){return e-t}),0);function mf(e){return e&&e.length?Wa(e,Oh):0}function vf(e,t){return e&&e.length?Wa(e,No(t,2)):0}return Mn.after=Vc,Mn.ary=Oc,Mn.assign=ed,Mn.assignIn=td,Mn.assignInWith=ad,Mn.assignWith=nd,Mn.at=rd,Mn.before=Tc,Mn.bind=Dc,Mn.bindAll=Lh,Mn.bindKey=Cc,Mn.castArray=Zc,Mn.chain=Wl,Mn.chunk=xs,Mn.compact=Bs,Mn.concat=Ys,Mn.cond=wh,Mn.conforms=Hh,Mn.constant=kh,Mn.countBy=ac,Mn.create=id,Mn.curry=jc,Mn.curryRight=xc,Mn.debounce=Bc,Mn.defaults=od,Mn.defaultsDeep=sd,Mn.defer=Yc,Mn.delay=Pc,Mn.difference=Ps,Mn.differenceBy=Is,Mn.differenceWith=Fs,Mn.drop=Es,Mn.dropRight=$s,Mn.dropRightWhile=Rs,Mn.dropWhile=Ns,Mn.fill=Ws,Mn.filter=rc,Mn.flatMap=sc,Mn.flatMapDeep=lc,Mn.flatMapDepth=cc,Mn.flatten=qs,Mn.flattenDeep=Js,Mn.flattenDepth=Ks,Mn.flip=Ic,Mn.flow=Sh,Mn.flowRight=Vh,Mn.fromPairs=Xs,Mn.functions=pd,Mn.functionsIn=md,Mn.groupBy=hc,Mn.initial=el,Mn.intersection=tl,Mn.intersectionBy=al,Mn.intersectionWith=nl,Mn.invert=yd,Mn.invertBy=bd,Mn.invokeMap=pc,Mn.iteratee=Th,Mn.keyBy=mc,Mn.keys=zd,Mn.keysIn=Ld,Mn.map=vc,Mn.mapKeys=wd,Mn.mapValues=Hd,Mn.matches=Dh,Mn.matchesProperty=Ch,Mn.memoize=Fc,Mn.merge=kd,Mn.mergeWith=Ad,Mn.method=jh,Mn.methodOf=xh,Mn.mixin=Bh,Mn.negate=Ec,Mn.nthArg=Ih,Mn.omit=Sd,Mn.omitBy=Vd,Mn.once=$c,Mn.orderBy=_c,Mn.over=Fh,Mn.overArgs=Rc,Mn.overEvery=Eh,Mn.overSome=$h,Mn.partial=Nc,Mn.partialRight=Wc,Mn.partition=gc,Mn.pick=Od,Mn.pickBy=Td,Mn.property=Rh,Mn.propertyOf=Nh,Mn.pull=ll,Mn.pullAll=cl,Mn.pullAllBy=ul,Mn.pullAllWith=dl,Mn.pullAt=hl,Mn.range=Wh,Mn.rangeRight=Uh,Mn.rearg=Uc,Mn.reject=Mc,Mn.remove=fl,Mn.rest=Gc,Mn.reverse=pl,Mn.sampleSize=Lc,Mn.set=Cd,Mn.setWith=jd,Mn.shuffle=wc,Mn.slice=ml,Mn.sortBy=Ac,Mn.sortedUniq=zl,Mn.sortedUniqBy=Ll,Mn.split=lh,Mn.spread=qc,Mn.tail=wl,Mn.take=Hl,Mn.takeRight=kl,Mn.takeRightWhile=Al,Mn.takeWhile=Sl,Mn.tap=Ul,Mn.throttle=Jc,Mn.thru=Gl,Mn.toArray=Uu,Mn.toPairs=xd,Mn.toPairsIn=Bd,Mn.toPath=Qh,Mn.toPlainObject=Xu,Mn.transform=Yd,Mn.unary=Kc,Mn.union=Vl,Mn.unionBy=Ol,Mn.unionWith=Tl,Mn.uniq=Dl,Mn.uniqBy=Cl,Mn.uniqWith=jl,Mn.unset=Pd,Mn.unzip=xl,Mn.unzipWith=Bl,Mn.update=Id,Mn.updateWith=Fd,Mn.values=Ed,Mn.valuesIn=$d,Mn.without=Yl,Mn.words=Mh,Mn.wrap=Xc,Mn.xor=Pl,Mn.xorBy=Il,Mn.xorWith=Fl,Mn.zip=El,Mn.zipObject=$l,Mn.zipObjectDeep=Rl,Mn.zipWith=Nl,Mn.entries=xd,Mn.entriesIn=Bd,Mn.extend=td,Mn.extendWith=ad,Bh(Mn,Mn),Mn.add=tf,Mn.attempt=zh,Mn.camelCase=Ud,Mn.capitalize=Gd,Mn.ceil=af,Mn.clamp=Rd,Mn.clone=Qc,Mn.cloneDeep=tu,Mn.cloneDeepWith=au,Mn.cloneWith=eu,Mn.conformsTo=nu,Mn.deburr=qd,Mn.defaultTo=Ah,Mn.divide=nf,Mn.endsWith=Jd,Mn.eq=ru,Mn.escape=Kd,Mn.escapeRegExp=Xd,Mn.every=nc,Mn.find=ic,Mn.findIndex=Us,Mn.findKey=ld,Mn.findLast=oc,Mn.findLastIndex=Gs,Mn.findLastKey=cd,Mn.floor=rf,Mn.forEach=uc,Mn.forEachRight=dc,Mn.forIn=ud,Mn.forInRight=dd,Mn.forOwn=hd,Mn.forOwnRight=fd,Mn.get=vd,Mn.gt=iu,Mn.gte=ou,Mn.has=_d,Mn.hasIn=gd,Mn.head=Zs,Mn.identity=Oh,Mn.includes=fc,Mn.indexOf=Qs,Mn.inRange=Nd,Mn.invoke=Md,Mn.isArguments=su,Mn.isArray=lu,Mn.isArrayBuffer=cu,Mn.isArrayLike=uu,Mn.isArrayLikeObject=du,Mn.isBoolean=hu,Mn.isBuffer=fu,Mn.isDate=pu,Mn.isElement=mu,Mn.isEmpty=vu,Mn.isEqual=_u,Mn.isEqualWith=gu,Mn.isError=yu,Mn.isFinite=bu,Mn.isFunction=Mu,Mn.isInteger=zu,Mn.isLength=Lu,Mn.isMap=ku,Mn.isMatch=Au,Mn.isMatchWith=Su,Mn.isNaN=Vu,Mn.isNative=Ou,Mn.isNil=Du,Mn.isNull=Tu,Mn.isNumber=Cu,Mn.isObject=wu,Mn.isObjectLike=Hu,Mn.isPlainObject=ju,Mn.isRegExp=xu,Mn.isSafeInteger=Bu,Mn.isSet=Yu,Mn.isString=Pu,Mn.isSymbol=Iu,Mn.isTypedArray=Fu,Mn.isUndefined=Eu,Mn.isWeakMap=$u,Mn.isWeakSet=Ru,Mn.join=rl,Mn.kebabCase=Zd,Mn.last=il,Mn.lastIndexOf=ol,Mn.lowerCase=Qd,Mn.lowerFirst=eh,Mn.lt=Nu,Mn.lte=Wu,Mn.max=of,Mn.maxBy=sf,Mn.mean=lf,Mn.meanBy=cf,Mn.min=uf,Mn.minBy=df,Mn.stubArray=Gh,Mn.stubFalse=qh,Mn.stubObject=Jh,Mn.stubString=Kh,Mn.stubTrue=Xh,Mn.multiply=hf,Mn.nth=sl,Mn.noConflict=Yh,Mn.noop=Ph,Mn.now=Sc,Mn.pad=th,Mn.padEnd=ah,Mn.padStart=nh,Mn.parseInt=rh,Mn.random=Wd,Mn.reduce=yc,Mn.reduceRight=bc,Mn.repeat=ih,Mn.replace=oh,Mn.result=Dd,Mn.round=ff,Mn.runInContext=e,Mn.sample=zc,Mn.size=Hc,Mn.snakeCase=sh,Mn.some=kc,Mn.sortedIndex=vl,Mn.sortedIndexBy=_l,Mn.sortedIndexOf=gl,Mn.sortedLastIndex=yl,Mn.sortedLastIndexBy=bl,Mn.sortedLastIndexOf=Ml,Mn.startCase=ch,Mn.startsWith=uh,Mn.subtract=pf,Mn.sum=mf,Mn.sumBy=vf,Mn.template=dh,Mn.times=Zh,Mn.toFinite=Gu,Mn.toInteger=qu,Mn.toLength=Ju,Mn.toLower=hh,Mn.toNumber=Ku,Mn.toSafeInteger=Zu,Mn.toString=Qu,Mn.toUpper=fh,Mn.trim=ph,Mn.trimEnd=mh,Mn.trimStart=vh,Mn.truncate=_h,Mn.unescape=gh,Mn.uniqueId=ef,Mn.upperCase=yh,Mn.upperFirst=bh,Mn.each=uc,Mn.eachRight=dc,Mn.first=Zs,Bh(Mn,function(){var e={};return Tr(Mn,(function(t,a){dt.call(Mn.prototype,a)||(e[a]=t)})),e}(),{chain:!1}),Mn.VERSION=o,Ma(["bind","bindKey","curry","curryRight","partial","partialRight"],(function(e){Mn[e].placeholder=Mn})),Ma(["drop","take"],(function(e,t){An.prototype[e]=function(a){a=a===i?1:Ft(qu(a),0);var n=this.__filtered__&&!t?new An(this):this.clone();return n.__filtered__?n.__takeCount__=Et(a,n.__takeCount__):n.__views__.push({size:Et(a,F),type:e+(n.__dir__<0?"Right":"")}),n},An.prototype[e+"Right"]=function(t){return this.reverse()[e](t).reverse()}})),Ma(["filter","map","takeWhile"],(function(e,t){var a=t+1,n=a==C||a==x;An.prototype[e]=function(e){var t=this.clone();return t.__iteratees__.push({iteratee:No(e,3),type:a}),t.__filtered__=t.__filtered__||n,t}})),Ma(["head","last"],(function(e,t){var a="take"+(t?"Right":"");An.prototype[e]=function(){return this[a](1).value()[0]}})),Ma(["initial","tail"],(function(e,t){var a="drop"+(t?"":"Right");An.prototype[e]=function(){return this.__filtered__?new An(this):this[a](1)}})),An.prototype.compact=function(){return this.filter(Oh)},An.prototype.find=function(e){return this.filter(e).head()},An.prototype.findLast=function(e){return this.reverse().find(e)},An.prototype.invokeMap=bi((function(e,t){return"function"==typeof e?new An(this):this.map((function(a){return Rr(a,e,t)}))})),An.prototype.reject=function(e){return this.filter(Ec(No(e)))},An.prototype.slice=function(e,t){e=qu(e);var a=this;return a.__filtered__&&(e>0||t<0)?new An(a):(e<0?a=a.takeRight(-e):e&&(a=a.drop(e)),t!==i&&(t=qu(t),a=t<0?a.dropRight(-t):a.take(t-e)),a)},An.prototype.takeRightWhile=function(e){return this.reverse().takeWhile(e).reverse()},An.prototype.toArray=function(){return this.take(F)},Tr(An.prototype,(function(e,t){var a=/^(?:filter|find|map|reject)|While$/.test(t),n=/^(?:head|last)$/.test(t),r=Mn[n?"take"+("last"==t?"Right":""):t],o=n||/^find/.test(t);r&&(Mn.prototype[t]=function(){var t=this.__wrapped__,s=n?[1]:arguments,l=t instanceof An,c=s[0],u=l||lu(t),d=function(e){var t=r.apply(Mn,Sa([e],s));return n&&h?t[0]:t};u&&a&&"function"==typeof c&&1!=c.length&&(l=u=!1);var h=this.__chain__,f=!!this.__actions__.length,p=o&&!h,m=l&&!f;if(!o&&u){t=m?t:new An(this);var v=e.apply(t,s);return v.__actions__.push({func:Gl,args:[d],thisArg:i}),new kn(v,h)}return p&&m?e.apply(this,s):(v=this.thru(d),p?n?v.value()[0]:v.value():v)})})),Ma(["pop","push","shift","sort","splice","unshift"],(function(e){var t=ot[e],a=/^(?:push|sort|unshift)$/.test(e)?"tap":"thru",n=/^(?:pop|shift)$/.test(e);Mn.prototype[e]=function(){var e=arguments;if(n&&!this.__chain__){var r=this.value();return t.apply(lu(r)?r:[],e)}return this[a]((function(a){return t.apply(lu(a)?a:[],e)}))}})),Tr(An.prototype,(function(e,t){var a=Mn[t];if(a){var n=a.name+"";dt.call(la,n)||(la[n]=[]),la[n].push({name:t,func:a})}})),la[yo(i,b).name]=[{name:"wrapper",func:i}],An.prototype.clone=Sn,An.prototype.reverse=Vn,An.prototype.value=On,Mn.prototype.at=ql,Mn.prototype.chain=Jl,Mn.prototype.commit=Kl,Mn.prototype.next=Xl,Mn.prototype.plant=Ql,Mn.prototype.reverse=ec,Mn.prototype.toJSON=Mn.prototype.valueOf=Mn.prototype.value=tc,Mn.prototype.first=Mn.prototype.head,At&&(Mn.prototype[At]=Zl),Mn},wn=Ln();sa._=wn,r=function(){return wn}.call(t,a,t,n),r===i||(n.exports=r)}).call(this)}).call(this,a("c8ba"),a("62e4")(e))},"2f62":function(e,t,a){"use strict";(function(e){
/*!
 * vuex v3.6.2
 * (c) 2021 Evan You
 * @license MIT
 */
function a(e){var t=Number(e.version.split(".")[0]);if(t>=2)e.mixin({beforeCreate:n});else{var a=e.prototype._init;e.prototype._init=function(e){void 0===e&&(e={}),e.init=e.init?[n].concat(e.init):n,a.call(this,e)}}function n(){var e=this.$options;e.store?this.$store="function"===typeof e.store?e.store():e.store:e.parent&&e.parent.$store&&(this.$store=e.parent.$store)}}var n="undefined"!==typeof window?window:"undefined"!==typeof e?e:{},r=n.__VUE_DEVTOOLS_GLOBAL_HOOK__;function i(e){r&&(e._devtoolHook=r,r.emit("vuex:init",e),r.on("vuex:travel-to-state",(function(t){e.replaceState(t)})),e.subscribe((function(e,t){r.emit("vuex:mutation",e,t)}),{prepend:!0}),e.subscribeAction((function(e,t){r.emit("vuex:action",e,t)}),{prepend:!0}))}function o(e,t){return e.filter(t)[0]}function s(e,t){if(void 0===t&&(t=[]),null===e||"object"!==typeof e)return e;var a=o(t,(function(t){return t.original===e}));if(a)return a.copy;var n=Array.isArray(e)?[]:{};return t.push({original:e,copy:n}),Object.keys(e).forEach((function(a){n[a]=s(e[a],t)})),n}function l(e,t){Object.keys(e).forEach((function(a){return t(e[a],a)}))}function c(e){return null!==e&&"object"===typeof e}function u(e){return e&&"function"===typeof e.then}function d(e,t){return function(){return e(t)}}var h=function(e,t){this.runtime=t,this._children=Object.create(null),this._rawModule=e;var a=e.state;this.state=("function"===typeof a?a():a)||{}},f={namespaced:{configurable:!0}};f.namespaced.get=function(){return!!this._rawModule.namespaced},h.prototype.addChild=function(e,t){this._children[e]=t},h.prototype.removeChild=function(e){delete this._children[e]},h.prototype.getChild=function(e){return this._children[e]},h.prototype.hasChild=function(e){return e in this._children},h.prototype.update=function(e){this._rawModule.namespaced=e.namespaced,e.actions&&(this._rawModule.actions=e.actions),e.mutations&&(this._rawModule.mutations=e.mutations),e.getters&&(this._rawModule.getters=e.getters)},h.prototype.forEachChild=function(e){l(this._children,e)},h.prototype.forEachGetter=function(e){this._rawModule.getters&&l(this._rawModule.getters,e)},h.prototype.forEachAction=function(e){this._rawModule.actions&&l(this._rawModule.actions,e)},h.prototype.forEachMutation=function(e){this._rawModule.mutations&&l(this._rawModule.mutations,e)},Object.defineProperties(h.prototype,f);var p=function(e){this.register([],e,!1)};function m(e,t,a){if(t.update(a),a.modules)for(var n in a.modules){if(!t.getChild(n))return void 0;m(e.concat(n),t.getChild(n),a.modules[n])}}p.prototype.get=function(e){return e.reduce((function(e,t){return e.getChild(t)}),this.root)},p.prototype.getNamespace=function(e){var t=this.root;return e.reduce((function(e,a){return t=t.getChild(a),e+(t.namespaced?a+"/":"")}),"")},p.prototype.update=function(e){m([],this.root,e)},p.prototype.register=function(e,t,a){var n=this;void 0===a&&(a=!0);var r=new h(t,a);if(0===e.length)this.root=r;else{var i=this.get(e.slice(0,-1));i.addChild(e[e.length-1],r)}t.modules&&l(t.modules,(function(t,r){n.register(e.concat(r),t,a)}))},p.prototype.unregister=function(e){var t=this.get(e.slice(0,-1)),a=e[e.length-1],n=t.getChild(a);n&&n.runtime&&t.removeChild(a)},p.prototype.isRegistered=function(e){var t=this.get(e.slice(0,-1)),a=e[e.length-1];return!!t&&t.hasChild(a)};var v;var _=function(e){var t=this;void 0===e&&(e={}),!v&&"undefined"!==typeof window&&window.Vue&&T(window.Vue);var a=e.plugins;void 0===a&&(a=[]);var n=e.strict;void 0===n&&(n=!1),this._committing=!1,this._actions=Object.create(null),this._actionSubscribers=[],this._mutations=Object.create(null),this._wrappedGetters=Object.create(null),this._modules=new p(e),this._modulesNamespaceMap=Object.create(null),this._subscribers=[],this._watcherVM=new v,this._makeLocalGettersCache=Object.create(null);var r=this,o=this,s=o.dispatch,l=o.commit;this.dispatch=function(e,t){return s.call(r,e,t)},this.commit=function(e,t,a){return l.call(r,e,t,a)},this.strict=n;var c=this._modules.root.state;z(this,c,[],this._modules.root),M(this,c),a.forEach((function(e){return e(t)}));var u=void 0!==e.devtools?e.devtools:v.config.devtools;u&&i(this)},g={state:{configurable:!0}};function y(e,t,a){return t.indexOf(e)<0&&(a&&a.prepend?t.unshift(e):t.push(e)),function(){var a=t.indexOf(e);a>-1&&t.splice(a,1)}}function b(e,t){e._actions=Object.create(null),e._mutations=Object.create(null),e._wrappedGetters=Object.create(null),e._modulesNamespaceMap=Object.create(null);var a=e.state;z(e,a,[],e._modules.root,!0),M(e,a,t)}function M(e,t,a){var n=e._vm;e.getters={},e._makeLocalGettersCache=Object.create(null);var r=e._wrappedGetters,i={};l(r,(function(t,a){i[a]=d(t,e),Object.defineProperty(e.getters,a,{get:function(){return e._vm[a]},enumerable:!0})}));var o=v.config.silent;v.config.silent=!0,e._vm=new v({data:{$$state:t},computed:i}),v.config.silent=o,e.strict&&S(e),n&&(a&&e._withCommit((function(){n._data.$$state=null})),v.nextTick((function(){return n.$destroy()})))}function z(e,t,a,n,r){var i=!a.length,o=e._modules.getNamespace(a);if(n.namespaced&&(e._modulesNamespaceMap[o],e._modulesNamespaceMap[o]=n),!i&&!r){var s=V(t,a.slice(0,-1)),l=a[a.length-1];e._withCommit((function(){v.set(s,l,n.state)}))}var c=n.context=L(e,o,a);n.forEachMutation((function(t,a){var n=o+a;H(e,n,t,c)})),n.forEachAction((function(t,a){var n=t.root?a:o+a,r=t.handler||t;k(e,n,r,c)})),n.forEachGetter((function(t,a){var n=o+a;A(e,n,t,c)})),n.forEachChild((function(n,i){z(e,t,a.concat(i),n,r)}))}function L(e,t,a){var n=""===t,r={dispatch:n?e.dispatch:function(a,n,r){var i=O(a,n,r),o=i.payload,s=i.options,l=i.type;return s&&s.root||(l=t+l),e.dispatch(l,o)},commit:n?e.commit:function(a,n,r){var i=O(a,n,r),o=i.payload,s=i.options,l=i.type;s&&s.root||(l=t+l),e.commit(l,o,s)}};return Object.defineProperties(r,{getters:{get:n?function(){return e.getters}:function(){return w(e,t)}},state:{get:function(){return V(e.state,a)}}}),r}function w(e,t){if(!e._makeLocalGettersCache[t]){var a={},n=t.length;Object.keys(e.getters).forEach((function(r){if(r.slice(0,n)===t){var i=r.slice(n);Object.defineProperty(a,i,{get:function(){return e.getters[r]},enumerable:!0})}})),e._makeLocalGettersCache[t]=a}return e._makeLocalGettersCache[t]}function H(e,t,a,n){var r=e._mutations[t]||(e._mutations[t]=[]);r.push((function(t){a.call(e,n.state,t)}))}function k(e,t,a,n){var r=e._actions[t]||(e._actions[t]=[]);r.push((function(t){var r=a.call(e,{dispatch:n.dispatch,commit:n.commit,getters:n.getters,state:n.state,rootGetters:e.getters,rootState:e.state},t);return u(r)||(r=Promise.resolve(r)),e._devtoolHook?r.catch((function(t){throw e._devtoolHook.emit("vuex:error",t),t})):r}))}function A(e,t,a,n){e._wrappedGetters[t]||(e._wrappedGetters[t]=function(e){return a(n.state,n.getters,e.state,e.getters)})}function S(e){e._vm.$watch((function(){return this._data.$$state}),(function(){0}),{deep:!0,sync:!0})}function V(e,t){return t.reduce((function(e,t){return e[t]}),e)}function O(e,t,a){return c(e)&&e.type&&(a=t,t=e,e=e.type),{type:e,payload:t,options:a}}function T(e){v&&e===v||(v=e,a(v))}g.state.get=function(){return this._vm._data.$$state},g.state.set=function(e){0},_.prototype.commit=function(e,t,a){var n=this,r=O(e,t,a),i=r.type,o=r.payload,s=(r.options,{type:i,payload:o}),l=this._mutations[i];l&&(this._withCommit((function(){l.forEach((function(e){e(o)}))})),this._subscribers.slice().forEach((function(e){return e(s,n.state)})))},_.prototype.dispatch=function(e,t){var a=this,n=O(e,t),r=n.type,i=n.payload,o={type:r,payload:i},s=this._actions[r];if(s){try{this._actionSubscribers.slice().filter((function(e){return e.before})).forEach((function(e){return e.before(o,a.state)}))}catch(c){0}var l=s.length>1?Promise.all(s.map((function(e){return e(i)}))):s[0](i);return new Promise((function(e,t){l.then((function(t){try{a._actionSubscribers.filter((function(e){return e.after})).forEach((function(e){return e.after(o,a.state)}))}catch(c){0}e(t)}),(function(e){try{a._actionSubscribers.filter((function(e){return e.error})).forEach((function(t){return t.error(o,a.state,e)}))}catch(c){0}t(e)}))}))}},_.prototype.subscribe=function(e,t){return y(e,this._subscribers,t)},_.prototype.subscribeAction=function(e,t){var a="function"===typeof e?{before:e}:e;return y(a,this._actionSubscribers,t)},_.prototype.watch=function(e,t,a){var n=this;return this._watcherVM.$watch((function(){return e(n.state,n.getters)}),t,a)},_.prototype.replaceState=function(e){var t=this;this._withCommit((function(){t._vm._data.$$state=e}))},_.prototype.registerModule=function(e,t,a){void 0===a&&(a={}),"string"===typeof e&&(e=[e]),this._modules.register(e,t),z(this,this.state,e,this._modules.get(e),a.preserveState),M(this,this.state)},_.prototype.unregisterModule=function(e){var t=this;"string"===typeof e&&(e=[e]),this._modules.unregister(e),this._withCommit((function(){var a=V(t.state,e.slice(0,-1));v.delete(a,e[e.length-1])})),b(this)},_.prototype.hasModule=function(e){return"string"===typeof e&&(e=[e]),this._modules.isRegistered(e)},_.prototype.hotUpdate=function(e){this._modules.update(e),b(this,!0)},_.prototype._withCommit=function(e){var t=this._committing;this._committing=!0,e(),this._committing=t},Object.defineProperties(_.prototype,g);var D=I((function(e,t){var a={};return Y(t).forEach((function(t){var n=t.key,r=t.val;a[n]=function(){var t=this.$store.state,a=this.$store.getters;if(e){var n=F(this.$store,"mapState",e);if(!n)return;t=n.context.state,a=n.context.getters}return"function"===typeof r?r.call(this,t,a):t[r]},a[n].vuex=!0})),a})),C=I((function(e,t){var a={};return Y(t).forEach((function(t){var n=t.key,r=t.val;a[n]=function(){var t=[],a=arguments.length;while(a--)t[a]=arguments[a];var n=this.$store.commit;if(e){var i=F(this.$store,"mapMutations",e);if(!i)return;n=i.context.commit}return"function"===typeof r?r.apply(this,[n].concat(t)):n.apply(this.$store,[r].concat(t))}})),a})),j=I((function(e,t){var a={};return Y(t).forEach((function(t){var n=t.key,r=t.val;r=e+r,a[n]=function(){if(!e||F(this.$store,"mapGetters",e))return this.$store.getters[r]},a[n].vuex=!0})),a})),x=I((function(e,t){var a={};return Y(t).forEach((function(t){var n=t.key,r=t.val;a[n]=function(){var t=[],a=arguments.length;while(a--)t[a]=arguments[a];var n=this.$store.dispatch;if(e){var i=F(this.$store,"mapActions",e);if(!i)return;n=i.context.dispatch}return"function"===typeof r?r.apply(this,[n].concat(t)):n.apply(this.$store,[r].concat(t))}})),a})),B=function(e){return{mapState:D.bind(null,e),mapGetters:j.bind(null,e),mapMutations:C.bind(null,e),mapActions:x.bind(null,e)}};function Y(e){return P(e)?Array.isArray(e)?e.map((function(e){return{key:e,val:e}})):Object.keys(e).map((function(t){return{key:t,val:e[t]}})):[]}function P(e){return Array.isArray(e)||c(e)}function I(e){return function(t,a){return"string"!==typeof t?(a=t,t=""):"/"!==t.charAt(t.length-1)&&(t+="/"),e(t,a)}}function F(e,t,a){var n=e._modulesNamespaceMap[a];return n}function E(e){void 0===e&&(e={});var t=e.collapsed;void 0===t&&(t=!0);var a=e.filter;void 0===a&&(a=function(e,t,a){return!0});var n=e.transformer;void 0===n&&(n=function(e){return e});var r=e.mutationTransformer;void 0===r&&(r=function(e){return e});var i=e.actionFilter;void 0===i&&(i=function(e,t){return!0});var o=e.actionTransformer;void 0===o&&(o=function(e){return e});var l=e.logMutations;void 0===l&&(l=!0);var c=e.logActions;void 0===c&&(c=!0);var u=e.logger;return void 0===u&&(u=console),function(e){var d=s(e.state);"undefined"!==typeof u&&(l&&e.subscribe((function(e,i){var o=s(i);if(a(e,d,o)){var l=N(),c=r(e),h="mutation "+e.type+l;$(u,h,t),u.log("%c prev state","color: #9E9E9E; font-weight: bold",n(d)),u.log("%c mutation","color: #03A9F4; font-weight: bold",c),u.log("%c next state","color: #4CAF50; font-weight: bold",n(o)),R(u)}d=o})),c&&e.subscribeAction((function(e,a){if(i(e,a)){var n=N(),r=o(e),s="action "+e.type+n;$(u,s,t),u.log("%c action","color: #03A9F4; font-weight: bold",r),R(u)}})))}}function $(e,t,a){var n=a?e.groupCollapsed:e.group;try{n.call(e,t)}catch(r){e.log(t)}}function R(e){try{e.groupEnd()}catch(t){e.log("—— log end ——")}}function N(){var e=new Date;return" @ "+U(e.getHours(),2)+":"+U(e.getMinutes(),2)+":"+U(e.getSeconds(),2)+"."+U(e.getMilliseconds(),3)}function W(e,t){return new Array(t+1).join(e)}function U(e,t){return W("0",t-e.toString().length)+e}var G={Store:_,install:T,version:"3.6.2",mapState:D,mapMutations:C,mapGetters:j,mapActions:x,createNamespacedHelpers:B,createLogger:E};t["a"]=G}).call(this,a("c8ba"))},"30b5":function(e,t,a){"use strict";var n=a("c532");function r(e){return encodeURIComponent(e).replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",").replace(/%20/g,"+").replace(/%5B/gi,"[").replace(/%5D/gi,"]")}e.exports=function(e,t,a){if(!t)return e;var i;if(a)i=a(t);else if(n.isURLSearchParams(t))i=t.toString();else{var o=[];n.forEach(t,(function(e,t){null!==e&&"undefined"!==typeof e&&(n.isArray(e)?t+="[]":e=[e],n.forEach(e,(function(e){n.isDate(e)?e=e.toISOString():n.isObject(e)&&(e=JSON.stringify(e)),o.push(r(t)+"="+r(e))})))})),i=o.join("&")}if(i){var s=e.indexOf("#");-1!==s&&(e=e.slice(0,s)),e+=(-1===e.indexOf("?")?"?":"&")+i}return e}},3172:function(e,t,a){"use strict";a.r(t);var n=a("bc3a"),r=a.n(n),i=a("a026"),o=a("2f62"),s=(a("14d9"),a("ba4c")),l=a.n(s),c=function(e){return function(e){return!!e&&"object"==typeof e}(e)&&!function(e){var t=Object.prototype.toString.call(e);return"[object RegExp]"===t||"[object Date]"===t||function(e){return e.$$typeof===u}(e)}(e)},u="function"==typeof Symbol&&Symbol.for?Symbol.for("react.element"):60103;function d(e,t){return!1!==t.clone&&t.isMergeableObject(e)?m(Array.isArray(e)?[]:{},e,t):e}function h(e,t,a){return e.concat(t).map((function(e){return d(e,a)}))}function f(e){return Object.keys(e).concat(function(e){return Object.getOwnPropertySymbols?Object.getOwnPropertySymbols(e).filter((function(t){return e.propertyIsEnumerable(t)})):[]}(e))}function p(e,t){try{return t in e}catch(e){return!1}}function m(e,t,a){(a=a||{}).arrayMerge=a.arrayMerge||h,a.isMergeableObject=a.isMergeableObject||c,a.cloneUnlessOtherwiseSpecified=d;var n=Array.isArray(t);return n===Array.isArray(e)?n?a.arrayMerge(e,t,a):function(e,t,a){var n={};return a.isMergeableObject(e)&&f(e).forEach((function(t){n[t]=d(e[t],a)})),f(t).forEach((function(r){(function(e,t){return p(e,t)&&!(Object.hasOwnProperty.call(e,t)&&Object.propertyIsEnumerable.call(e,t))})(e,r)||(n[r]=p(e,r)&&a.isMergeableObject(t[r])?function(e,t){if(!t.customMerge)return m;var a=t.customMerge(e);return"function"==typeof a?a:m}(r,a)(e[r],t[r],a):d(t[r],a))})),n}(e,t,a):d(t,a)}m.all=function(e,t){if(!Array.isArray(e))throw new Error("first argument should be an array");return e.reduce((function(e,a){return m(e,a,t)}),{})};var v=m;function _(e){var t=(e=e||{}).storage||window&&window.localStorage,a=e.key||"vuex";function n(e,t){var a=t.getItem(e);try{return"string"==typeof a?JSON.parse(a):"object"==typeof a?a:void 0}catch(e){}}function r(){return!0}function i(e,t,a){return a.setItem(e,JSON.stringify(t))}function o(e,t){return Array.isArray(t)?t.reduce((function(t,a){return function(e,t,a,n){return!/^(__proto__|constructor|prototype)$/.test(t)&&((t=t.split?t.split("."):t.slice(0)).slice(0,-1).reduce((function(e,t){return e[t]=e[t]||{}}),e)[t.pop()]=a),e}(t,a,(n=e,void 0===(n=((r=a).split?r.split("."):r).reduce((function(e,t){return e&&e[t]}),n))?void 0:n));var n,r}),{}):e}function s(e){return function(t){return e.subscribe(t)}}(e.assertStorage||function(){t.setItem("@@",1),t.removeItem("@@")})(t);var l,c=function(){return(e.getState||n)(a,t)};return e.fetchBeforeUse&&(l=c()),function(n){e.fetchBeforeUse||(l=c()),"object"==typeof l&&null!==l&&(n.replaceState(e.overwrite?l:v(n.state,l,{arrayMerge:e.arrayMerger||function(e,t){return t},clone:!1})),(e.rehydrated||function(){})(n)),(e.subscriber||s)(n)((function(n,s){(e.filter||r)(n)&&(e.setState||i)(a,(e.reducer||o)(s,e.paths),t)}))}}var g=_,y={state:{count:0},mutations:{increment:e=>e.count++,decrement:e=>e.count--}};l.a.use(o["a"]);let b=[];b.push(g({paths:["counter.count"]}));let M=new o["a"].Store({state:{tomApiBaseUrl:"http://localhost:8000",hermesApiBaseUrl:"https://hermes.lco.global",tomAxiosConfig:{},hermesAxiosConfig:{}},plugins:b,modules:{counter:y},mutations:{setHermesApiBaseUrl(e,t){console.log("Setting url to "+t),e.hermesApiBaseUrl=t},setTomApiBaseUrl(e,t){e.tomApiBaseUrl=t},setHermesAxiosConfig(e,t){e.hermesAxiosConfig=t},setTomAxiosConfig(e,t){e.tomAxiosConfig=t}},actions:{},strict:!1});var z={store:M,install(e){e.prototype.$store=M}},L=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",{staticClass:"row superevent-top-level-details"},[a("div",{staticClass:"col-md-12"},[a("b-card",{attrs:{"no-body":""}},[a("b-card-title",{staticClass:"text-center font-weight-bold superevent-banner",staticStyle:{"font-size":"xxx-large"}},[e._v(e._s(e.superevent_data.event_id))]),a("b-tabs",{attrs:{card:"",pills:"","active-nav-item-class":"font-weight-bold"}},e._l(e.expandedSequences,(function(t,n){return a("b-tab",{key:t.sequence_id,attrs:{"no-body":"",title:e.getTabTitle(t),active:n+1==e.expandedSequences.length,lazy:""},on:{click:function(a){e.tab=t.sequence_id}}},[a("superevent-detail",{ref:"sequence_"+t.sequence_id.toString(),refInFor:!0,attrs:{supereventPk:e.superevent_data.id,supereventId:e.superevent_data.event_id,sequence:t,candidates:e.superevent_data.candidates,alerts:e.references}})],1)})),1)],1)],1)])},w=[],H=a("2ef0"),k=a.n(H),A=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",[a("div",e._l(e.messages,(function(t){return a("b-alert",{key:t,attrs:{show:"",dismissable:""}},[e._v(" "+e._s(t)+" ")])})),1),a("div",{directives:[{name:"show",rawName:"v-show",value:void 0!==e.sequence,expression:"sequence !== undefined"}]},[a("gravitational-wave-banner",{attrs:{sequence:e.sequence,supereventId:e.supereventId}})],1),a("b-row",{staticStyle:{"margin-top":"6px"}},[a("b-col",{attrs:{cols:"8"}},[a("alerts-table",{attrs:{alerts:e.alerts},on:{"selected-alert":e.onSelectAlert}})],1),a("b-col",{attrs:{cols:"4"}},[a("h3",[e._v("GraceDB BAYESTAR Images")]),a("b-row",[a("b-img-lazy",{attrs:{src:e.getVolumeImageUrl(e.volume_image=!0),fluid:""}})],1),a("b-row",[a("b-img-lazy",{attrs:{src:e.getVolumeImageUrl(e.volume_image=!1),fluid:""}})],1)],1)],1),a("b-row",{staticClass:"my-3"},[a("b-col",{staticClass:"col-md-auto"},[a("add-candidate-modal",{attrs:{supereventPk:this.supereventPk,existingEventCandidates:this.eventCandidates},on:{"created-candidates":e.onCreatedCandidates}})],1),a("b-col",{staticClass:"col-md-auto"},[a("create-target-modal",{attrs:{alerts:this.selectedAlerts,supereventPk:this.supereventPk},on:{"created-target-candidates":e.onCreatedCandidates}})],1)],1),a("hr"),a("b-row",[a("b-col",{attrs:{cols:"12"}},[a("h3",[e._v("Active Candidates")]),a("candidate-target-table",{attrs:{candidates:this.viableCandidates,sequenceId:this.sequence.sequence_id},on:{"toggle-viability":e.onToggleViability,"change-priority":e.onChangePriority}})],1)],1),a("b-row",[a("b-col",{attrs:{cols:"12"}},[a("h3",[e._v("Retired Candidates")]),a("candidate-target-table",{attrs:{candidates:this.nonViableCandidates,sequenceId:this.sequence.sequence_id},on:{"toggle-viability":e.onToggleViability,"change-priority":e.onChangePriority}})],1)],1),a("hr")],1)},S=[],V=(a("bf37"),function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",[a("b-button",{directives:[{name:"b-modal",rawName:"v-b-modal.candidate-form-modal",modifiers:{"candidate-form-modal":!0}}],staticClass:"float-left",attrs:{variant:"outline-primary"}},[e._v("Add Candidate from Existing Target")]),a("b-modal",{attrs:{id:"candidate-form-modal",size:"xl",title:"Add Event Candidate"},scopedSlots:e._u([{key:"modal-footer",fn:function(t){var n=t.cancel;return[a("b-button",{staticClass:"float-right",attrs:{variant:"primary",disabled:0===e.selectedTargets.length},on:{click:e.onCreateCandidates}},[e._v("Add Candidates")]),a("b-button",{staticClass:"float-right",on:{click:function(e){return n()}}},[e._v("Cancel")])]}}])},[a("b-container",[a("b-form",{on:{submit:e.onCreateNewTarget}},[a("b-form-row",[a("b-col",[a("b-form-input",{attrs:{id:"target-name-input",placeholder:"Target Name",required:""},on:{input:e.targetSearch},model:{value:e.form.name,callback:function(t){e.$set(e.form,"name",t)},expression:"form.name"}})],1),a("b-col",[a("b-form-input",{attrs:{id:"target-ra-input",placeholder:"Right Ascension",required:""},on:{input:e.targetSearch},model:{value:e.form.ra,callback:function(t){e.$set(e.form,"ra",t)},expression:"form.ra"}})],1),a("b-col",[a("b-form-input",{attrs:{id:"target-dec-input",placeholder:"Declination",required:""},on:{input:e.targetSearch},model:{value:e.form.dec,callback:function(t){e.$set(e.form,"dec",t)},expression:"form.dec"}})],1)],1),a("b-form-group",{directives:[{name:"show",rawName:"v-show",value:e.targetFormValid(),expression:"targetFormValid()"}],staticClass:"my-3",attrs:{label:"Choose groups for new target:"}},[a("b-form-checkbox-group",e._l(e.userGroups,(function(t){return a("b-form-checkbox",{key:t.name,attrs:{value:t.id},on:{change:e.onSelectGroup}},[e._v(" "+e._s(t.name)+" ")])})),1)],1),a("b-form-row",{staticClass:"my-2 pl-1"},[a("b-button",{staticClass:"float-right mr-2",attrs:{variant:"primary",disabled:!e.targetFormValid()},on:{click:e.onCreateNewTarget}},[e._v("Create New Target")]),a("b-button",{staticClass:"float-right ml-2",attrs:{variant:"outline-primary"},on:{click:e.onClearSearch}},[e._v("Clear Search")])],1)],1),a("hr"),a("ttk-target-table",{attrs:{targets:e.matches},on:{"selected-target":e.onSelectTarget}})],1)],1)],1)}),O=[],T={props:{supereventPk:{type:Number,required:!0},existingEventCandidates:{type:Array,required:!1}},data(){return{form:{name:"",ra:"",dec:""},matches:[],selectedTargets:[],userGroups:[]}},created(){this.$root.$on("bv::modal::show",(e,t)=>{this.getTargets(),r.a.get(this.$store.state.tomApiBaseUrl+"/api/groups/",this.$store.state.tomAxiosConfig).then(e=>{this.userGroups=e["data"]["results"]}).catch(e=>{console.log(`Unable to retrieve groups: ${e}.`)})})},methods:{getTargets(e,t,a){let n="";e&&(n+="name="+e),t&&a&&(n+=`&cone_search=${t},${a},5`),r.a.get(`${this.$store.state.tomApiBaseUrl}/api/targets/?${n}`,this.$store.state.tomAxiosConfig).then(e=>{this.matches=e["data"]["results"],this.matches=this.matches.filter(e=>{let t=!1;return this.existingEventCandidates.forEach(a=>{e.id===a.id&&(t=!0)}),!t})}).catch(e=>{console.log(`Unable to retrieve targets: ${e}.`)})},targetFormValid(){return!!(this.form.name&&this.form.ra&&this.form.dec)},targetSearch(){this.getTargets(this.form.name,this.form.ra,this.form.dec)},createEventCandidates(e,t){r.a.post(this.$store.state.tomApiBaseUrl+"/api/eventcandidates/",e).then(e=>{let a;a=Array.isArray(e.data)?e.data.length:1,t(a)}).catch(e=>{console.log("Unable to create event candidates: "+e)})},onClearSearch(){this.form={name:"",ra:"",dec:""},this.getTargets()},onCreateCandidates(){if(0!==this.selectedTargets.length){let e=[];this.selectedTargets.forEach(t=>e.push({nonlocalizedevent:this.supereventPk,target:t})),this.createEventCandidates(e,e=>{this.$bvModal.hide("candidate-form-modal"),this.$emit("created-candidates",e)})}else if(""!==this.form.name&&""!==this.form.ra&&""!==this.form.dec){let e={name:this.form.name,ra:this.form.ra,dec:this.form.dec,type:"SIDEREAL",aliases:[],targetextra_set:[]};r.a.post(this.$store.state.tomApiBaseUrl+"/api/targets/",e).then(e=>{let t=e.data.id;this.createEventCandidates({nonlocalizedevent:this.supereventPk,target:t},e=>{this.$bvModal.hide("candidate-form-modal"),this.$emit("created-candidates",e)})}).catch(e=>{console.log("Unable to create target: "+e)}),this.form={name:"",ra:"",dec:""}}this.selectedTargets=[]},onCreateNewTarget(){r.a.get(this.$store.state.tomApiBaseUrl+"/api/groups/",this.$store.state.tomAxiosConfig).then(e=>{console.log(e);let t=e["data"]["results"].map(e=>({id:e.id})),a={name:this.form.name,ra:this.form.ra,dec:this.form.dec,type:"SIDEREAL",aliases:[],targetextra_set:[],groups:t};r.a.post(this.$store.state.tomApiBaseUrl+"/api/targets/",a,this.$store.state.tomAxiosConfig).then(e=>{this.getTargets(this.form.name,this.form.ra,this.form.dec)}).catch(e=>{console.log("Unable to create target: "+e)})}).catch(e=>{console.log("Unable to retrieve groups: "+e)})},onSelectTarget(e,t){!0===t?k.a.includes(this.selectedTargets,e.item.id)||this.selectedTargets.push(e.item.id):this.selectedTargets=this.selectedTargets.filter((function(t){return t!==e.item.id}))}}},D=T;function C(e,t,a,n,r,i,o,s){var l,c="function"===typeof e?e.options:e;if(t&&(c.render=t,c.staticRenderFns=a,c._compiled=!0),n&&(c.functional=!0),i&&(c._scopeId="data-v-"+i),o?(l=function(e){e=e||this.$vnode&&this.$vnode.ssrContext||this.parent&&this.parent.$vnode&&this.parent.$vnode.ssrContext,e||"undefined"===typeof __VUE_SSR_CONTEXT__||(e=__VUE_SSR_CONTEXT__),r&&r.call(this,e),e&&e._registeredComponents&&e._registeredComponents.add(o)},c._ssrRegister=l):r&&(l=s?function(){r.call(this,(c.functional?this.parent:this).$root.$options.shadowRoot)}:r),l)if(c.functional){c._injectStyles=l;var u=c.render;c.render=function(e,t){return l.call(t),u(e,t)}}else{var d=c.beforeCreate;c.beforeCreate=d?[].concat(d,l):[l]}return{exports:e,options:c}}var j=C(D,V,O,!1,null,null,null),x=j.exports,B=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",[a("b-form-group",[a("b-form-input",{attrs:{id:"filter-input",type:"search",placeholder:"Search Alerts"},model:{value:e.filter,callback:function(t){e.filter=t},expression:"filter"}})],1),a("b-table",{attrs:{id:"alerts-table","sticky-header":"600px",hover:"","head-variant":"light","foot-clone":"",filter:e.filter,items:e.getAlertsFromAlertData(),fields:e.alert_fields},on:{"row-clicked":e.showRowDetails},scopedSlots:e._u([{key:"cell(selected)",fn:function(t){return[t.item.targets&&t.item.targets.length>0?a("div",[a("b-form-checkbox",{on:{change:function(a){return e.$emit("selected-alert",t,a)}}})],1):e._e()]}},{key:"cell(show_details)",fn:function(e){return[e.detailsShowing?a("b-link",{on:{click:e.toggleDetails}},[a("b-icon-caret-down")],1):a("b-link",{on:{click:e.toggleDetails}},[a("b-icon-caret-right")],1)]}},{key:"row-details",fn:function(t){return[t.item.topic.toUpperCase().includes("circular")?a("span",[e._v(e._s(t.item.message_text))]):t.item.topic.toUpperCase().includes("LVC_COUNTERPART")?a("div",e._l(Object.entries(t.item.data),(function(t){var n=t[0],r=t[1];return a("dl",{key:[n,r],staticClass:"row"},[a("dt",{staticClass:"col-md-3"},[e._v(e._s(n)+": ")]),a("dd",{staticClass:"col-md-9"},[e._v(e._s(r))])])})),0):e._e()]}},{key:"cell(identifier)",fn:function(t){return[t.item.targets&&t.item.targets.length>0?a("span",[a("b-link",{attrs:{href:e.getAlertUrl(t.item)}},[e._v(e._s(t.item.targets[0].name))])],1):a("span",[a("b-link",{attrs:{href:e.getAlertUrl(t.item)}},[e._v(e._s(t.item.id))])],1)]}},{key:"cell(timestamp)",fn:function(t){return[e._v(" "+e._s(e.getAlertDate(t.item))+" ")]}},{key:"cell(from)",fn:function(t){return[a("span",[e._v(" "+e._s(e.getSimplifiedFromField(t.item.submitter))+" ")])]}},{key:"cell(subject)",fn:function(t){return[t.item.title?a("span",[e._v(e._s(t.item.title))]):t.item.targets&&t.item.targets.length>0?a("span",[e._v(" Right Ascension: "+e._s(t.item.targets[0].right_ascension_sexagesimal)),a("br"),e._v(" Declination: "+e._s(t.item.targets[0].declination_sexagesimal)+" ")]):e._e()]}}])})],1)},Y=[],P=a("c1df"),I=a.n(P),F={name:"AlertsTable",components:{},data:function(){return{alert_data:[],alert_fields:[{key:"selected",label:""},{key:"show_details",label:""},{key:"identifier"},{key:"timestamp",sortable:!0},{key:"from"},{key:"subject"}],filter:null}},props:{alerts:{type:Array,required:!0}},mounted(){},methods:{getAlertUrl(e){return`${this.$store.state.hermesApiBaseUrl}/api/v0/messages/${e.id}`},getAlertsFromAlertData(){return this.alerts.filter(e=>"GCN/LVC NOTICE"!==e.title)},getAlertDate(e){return I()(e.published).format("YYYY-MM-DD hh:mm:ss")},getSimplifiedFromField(e){return e.split("<")[0].trim()},showRowDetails(e,t,a){e._showDetails=!e._showDetails}}},E=F,$=(a("620a"),C(E,B,Y,!1,null,"0acbab5a",null)),R=$.exports,N=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("b-container",[a("b-table",{attrs:{striped:"","primary-key":"id","sort-by":"credible_region",fields:e.candidateFields,items:e.candidates},scopedSlots:e._u([{key:"cell(target-link)",fn:function(t){return[a("b-link",{attrs:{href:e.getTargetDetailUrl(t.item.target)}},[e._v(e._s(t.item.target.name))])]}},{key:"cell(active)",fn:function(t){return[a("b-button",{attrs:{size:"sm"},on:{click:function(a){e.$bvModal.show(e.getId("viable-modal-",t,""))}}},[e._v(" "+e._s(t.item.viable?"Retire":"Activate")+" ")]),a("b-modal",{attrs:{id:e.getId("viable-modal-",t,""),title:e.modalTitle(t)},on:{ok:function(a){return e.$emit("toggle-viability",t,a)}}},[a("b-container",{attrs:{fluid:""}},[a("b-row",[a("b-col",{attrs:{sm:"2"}},[a("label",{attrs:{for:e.getId("textarea-",t,"")}},[e._v("Viability Reason:")])]),a("b-col",{attrs:{sm:"10"}},[a("b-form-textarea",{attrs:{id:e.getId("textarea-",t,""),placeholder:"Enter reason for viability decision here",rows:"2","max-rows":"4"},model:{value:t.item.viability_reason,callback:function(a){e.$set(t.item,"viability_reason",a)},expression:"row.item.viability_reason"}})],1)],1)],1)],1)]}}])})],1)},W=[],U={name:"CandidateTargetTable",components:{},props:{candidates:{type:Array,required:!0},sequenceId:Number},computed:{},data(){return{candidateFields:[{key:"target-link",label:"Candidate",sortable:!0},{key:"target.ra",label:"RA",formatter:(e,t,a)=>e.toLocaleString()},{key:"target.dec",label:"DEC",formatter:(e,t,a)=>e.toLocaleString()},{key:"credible_region",label:"CR %",sortable:!0},{key:"active"},{key:"viability_reason",label:"Viability Reason"}]}},methods:{getTargetDetailUrl(e){return`${this.$store.state.tomApiBaseUrl}/targets/${e.id}`},modalTitle(e){return(e.item.viable?"Retire":"Activate")+" Candidate"},getId(e,t,a){return e+t.item.id.toString()+a}}},G=U,q=C(G,N,W,!1,null,null,null),J=q.exports,K=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",[a("b-button",{directives:[{name:"b-modal",rawName:"v-b-modal.candidate-from-target-modal",modifiers:{"candidate-from-target-modal":!0}}],staticClass:"float-left",attrs:{variant:"outline-primary",disabled:0===e.modalAlerts.length}},[e._v("Add Candidates from Alerts")]),a("b-modal",{attrs:{id:"candidate-from-target-modal",size:"xl",title:"Create Candidate(s) from Alerts"},scopedSlots:e._u([{key:"modal-footer",fn:function(t){var n=t.cancel;return[e.submissionError?a("span",{staticClass:"float-left text-danger"},[e._v(e._s(JSON.stringify(e.submissionError)))]):e._e(),a("b-button",{staticClass:"float-right",attrs:{variant:"primary"},on:{click:e.onCandidateFromAlert}},[e._v("Add Candidates")]),a("b-button",{staticClass:"float-right",on:{click:function(e){return n()}}},[e._v("Cancel")])]}}])},[a("b-container",[a("div",{staticClass:"my-2"},[a("span",[e._v("Targets to be created:")])]),a("ttk-target-table",{attrs:{selectable:!1,targets:this.modalAlerts}}),a("hr"),a("b-form",{on:{submit:e.onCandidateFromAlert}},[a("b-form-group",{attrs:{label:"Choose groups for new targets:"}},[a("b-form-checkbox-group",e._l(e.userGroups,(function(t){return a("b-form-checkbox",{key:t.name,attrs:{value:t.id},on:{change:e.onSelectGroup}},[e._v(" "+e._s(t.name)+" ")])})),1)],1)],1)],1)],1)],1)},X=[],Z={props:{alerts:{type:Array,required:!1},supereventPk:{type:Number,required:!0}},data(){return{selectedGroups:[],userGroups:[],modalAlerts:this.alerts,submissionError:null}},created(){this.$root.$on("bv::modal::show",(e,t)=>{this.submissionError=null,this.modalAlerts=this.alerts,this.modalAlerts=this.modalAlerts.map(e=>{let t=e;return t.identifier=e.targets[0].name,t.name=e.targets[0].name,t.ra=e.targets[0].right_ascension,t.dec=e.targets[0].declination,t}),r.a.get(this.$store.state.tomApiBaseUrl+"/api/groups/",this.$store.state.tomAxiosConfig).then(e=>{this.userGroups=e["data"]["results"]}).catch(e=>{console.log(`Unable to retrieve groups: ${e}.`)})})},methods:{onCandidateFromAlert(){this.submissionError=null,this.modalAlerts.forEach(e=>{let t={name:e["name"],ra:e["ra"],dec:e["dec"],type:"SIDEREAL",aliases:[],targetextra_set:[],groups:this.selectedGroups.map(e=>({id:e}))},a={nonlocalizedevent:this.supereventPk,target_fields:t};r.a.post(this.$store.state.tomApiBaseUrl+"/api/eventcandidates/",a).then(e=>{this.$bvModal.hide("candidate-from-target-modal"),this.$emit("created-target-candidates",e.data.length)}).catch(e=>{console.log("Unable to create eventcandidates for new targets: "+e),this.submissionError=e.response.data})})},onSelectGroup(e,t){!0===t?k.a.includes(this.selectedGroups,e.item)||this.selectedGroups.push(e.item):this.selectedGroups=this.selectedGroups.filter((function(t){return t!==e.item}))}}},Q=Z,ee=C(Q,K,X,!1,null,null,null),te=ee.exports,ae=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",[a("b-jumbotron",{staticClass:"py-4 pl-3 text-white superevent-banner",attrs:{id:"superevent-banner"}},[a("b-row",[a("b-col",["gcn"===e.sequence.ingestor_source.toLowerCase()?a("h3",[a("span",[e._v("NS: "+e._s(e.safeGetEventAttributes("details.prob_ns",-1,!0).toPrecision(3)))])]):a("h3",[a("span",[e._v("Instruments: "+e._s(e.safeGetEventAttributes("details.instruments",[]).join(",")))])])]),a("b-col",[a("h3",[a("span",[e._v("NSBH: "+e._s(e.safeGetEventAttributes(this.nsbhPath,-1,!0).toPrecision(3)))])])]),a("b-col",[a("h3",[a("span",[e._v("FAR: "+e._s(e.safeGetEventAttributes("details.far",-1,!0).toPrecision(3)))])])]),a("b-col",[a("h3",[a("span",[e._v("90%: "+e._s(parseFloat(e.safeGetEventAttributes(this.area90Path)).toPrecision(4)))])])]),e.hasExternalCoincidence?a("b-col",[a("h3",[a("span",[e._v("GCN #: "+e._s(e.safeGetEventAttributes("external_coincidence.details.gcn_notice_id",-1)))])])]):e._e()],1),a("b-row",[a("b-col",[a("h3",[a("span",[e._v("BNS: "+e._s(e.safeGetEventAttributes(this.bnsPath,-1,!0).toPrecision(3)))])])]),a("b-col",[a("h3",[a("span",[e._v("Terrestrial: "+e._s(e.safeGetEventAttributes(this.terrestrialPath,-1,!0).toPrecision(3)))])])]),a("b-col",[a("h3",[a("span",[e._v("BBH: "+e._s(e.safeGetEventAttributes(this.bbhPath,-1,!0).toPrecision(3)))])])]),a("b-col",[a("h3",[a("span",[e._v("50%: "+e._s(parseFloat(e.safeGetEventAttributes(this.area50Path)).toPrecision(4)))])])]),e.hasExternalCoincidence?a("b-col",[a("h3",[a("span",[e._v("Observatory: "+e._s(e.safeGetEventAttributes("external_coincidence.details.observatory","")))])])]):e._e()],1)],1)],1)},ne=[],re={name:"GravitationalWaveBanner",props:{sequence:{type:Object,required:!0},supereventId:String},computed:{hasExternalCoincidence(){return!k.a.isNil(this.sequence.external_coincidence)&&!k.a.isEmpty(this.sequence.external_coincidence)},nsbhPath(){return"gcn"===this.sequence.ingestor_source.toLowerCase()?"details.prob_nsbh":"details.classification.NSBH"},bbhPath(){return"gcn"===this.sequence.ingestor_source.toLowerCase()?"details.prob_bbh":"details.classification.BBH"},bnsPath(){return"gcn"===this.sequence.ingestor_source.toLowerCase()?"details.prob_bns":"details.classification.BNS"},terrestrialPath(){return"gcn"===this.sequence.ingestor_source.toLowerCase()?"details.prob_terres":"details.classification.Terrestrial"},area50Path(){return this.hasExternalCoincidence?"external_coincidence.localization.area_50":"localization.area_50"},area90Path(){return this.hasExternalCoincidence?"external_coincidence.localization.area_90":"localization.area_90"}},methods:{safeGetEventAttributes(e,t=0,a=!1){if(k.a.isObjectLike(this.sequence)){var n=k.a.get(this.sequence,e,t);return a&&"string"===typeof n&&(n=n.split(" ")[0]),isNaN(parseFloat(n))?n:parseFloat(n)}return t}}},ie=re,oe=C(ie,ae,ne,!1,null,null,null),se=oe.exports,le={name:"SupereventDetail",props:{supereventPk:Number,supereventId:String,sequence:{type:Object,required:!0},alerts:{type:Array,required:!0},candidates:{type:Array,required:!0}},components:{AddCandidateModal:x,AlertsTable:R,CreateTargetModal:te,CandidateTargetTable:J,GravitationalWaveBanner:se},data:function(){return{alert_fields:[{key:"identifier"},{key:"timestamp",sortable:!0},{key:"from"},{key:"subject"}],messages:[],eventCandidates:[],selectedAlerts:[],superevent_data:{}}},computed:{hasExternalCoincidence(){return!k.a.isNil(this.sequence.external_coincidence)&&!k.a.isEmpty(this.sequence.external_coincidence)},skymapVersion(){return this.hasExternalCoincidence?k.a.get(this.sequence,"external_coincidence.localization.skymap_version",void 0):k.a.get(this.sequence,"localization.skymap_version",void 0)},skymapUrl(){return this.hasExternalCoincidence?k.a.get(this.sequence,"external_coincidence.localization.skymap_url",void 0):k.a.get(this.sequence,"localization.skymap_url",void 0)},viableCandidates(){return this.eventCandidates.filter(e=>e.viable)},nonViableCandidates(){return this.eventCandidates.filter(e=>!e.viable)}},created(){this.processEventCandidates()},methods:{processEventCandidates(){this.eventCandidates=[],this.candidates.forEach(e=>{this.hasExternalCoincidence?this.sequence.external_coincidence.localization.id.toString()in e["credible_regions"]?e["credible_region"]=e["credible_regions"][this.sequence.external_coincidence.localization.id.toString()]:e["credible_region"]=100:this.sequence.localization.id.toString()in e["credible_regions"]?e["credible_region"]=e["credible_regions"][this.sequence.localization.id.toString()]:e["credible_region"]=100,this.eventCandidates.push(e)})},getSupereventData(){let e=this.eventCandidates.slice();this.eventCandidates=[],r.a.get(`${this.$store.state.tomApiBaseUrl}/api/nonlocalizedevents/${this.supereventPk}/`).then(e=>{e["data"]["candidates"].forEach(e=>{this.sequence.sequence_id.toString()in e["credible_regions"]?e["credible_region"]=e["credible_regions"][this.sequence.sequence_id.toString()]:e["credible_region"]=100,this.eventCandidates.push(e)})}).catch(t=>{console.log(`getSupereventData: Error getting database data for pk ${this.supereventPk}: ${t}`),this.eventCandidates=e})},getVolumeImageUrl(e=!0){let t="";t=e?"bayestar.volume.png":this.hasExternalCoincidence?"combined-ext.png":"bayestar.png";let a="https://gracedb.ligo.org/api/superevents/"+this.supereventId+"/files/"+t;return void 0!==this.skymapVersion&&(a=a+","+this.skymapVersion),a},onCreatedCandidates(e){this.messages.push(`Successfully added ${e} candidates.`),this.getSupereventData()},onSelectAlert(e,t){!0===t?k.a.includes(this.selectedAlerts,e.item)||this.selectedAlerts.push(e.item):this.selectedAlerts=this.selectedAlerts.filter((function(t){return t!==e.item}))},onToggleViability(e,t){const a=e.item,n=`${this.$store.state.tomApiBaseUrl}/api/eventcandidates/${a.id}/`,i=!a.viable,o={viable:i,viability_reason:a.viability_reason};r.a.patch(n,o).then(t=>{e.item.viable=t["data"].viable}).catch(e=>{console.error("onToggleViability - url: "+n),console.error("onToggleViability - patch: "+JSON.stringify(o)),console.error(`onToggleViability: Error getting database data for ${JSON.stringify(a)}: ${e}`)})},onChangePriority(e,t){const a=e.item,n=`${this.$store.state.tomApiBaseUrl}/api/eventcandidates/${a.id}`,i=t,o={priority:i};console.log("onChangePriority -        row.item.id: "+e.item.id),console.log("onChangePriority -  row.item.priority: "+e.item.priority),console.log("onChangePriority - candidate.priority: "+a.priority),console.log("onChangePriority -              event: "+t),r.a.patch(n,o).then(e=>{}).catch(e=>{console.error("onChangePriority - url: "+n),console.error("onChangePriority - patch: "+JSON.stringify(o)),console.error(`onChangePriority: Error getting database data for ${JSON.stringify(a)}: ${e}`)}),console.log("after PATCH..."),console.log("onChangePriority - candidate.priority: "+a.priority),console.log("onChangePriority -              event: "+t),console.log()}}},ce=le,ue=C(ce,A,S,!1,null,"32b2a06c",null),de=ue.exports,he={name:"SupereventSequences",props:{superevent:String,hermes_api_url:String,tom_api_url:String},data:function(){return{superevent_data:JSON.parse(this.superevent),references:[]}},components:{SupereventDetail:de},created(){this.$store.commit("setTomApiBaseUrl",this.tom_api_url),this.$store.commit("setHermesApiBaseUrl",this.hermes_api_url),this.getHermesDBData()},computed:{expandedSequences(){let e=[];return this.superevent_data.sequences.forEach(t=>{let a=k.a.cloneDeep(t);a.external_coincidence=null,e.push(a),k.a.isNil(t.external_coincidence)||k.a.isEmpty(t.external_coincidence)||e.push(t)}),e}},methods:{getTabTitle(e){let t="";return t+=e.sequence_id.toString()+": "+k.a.capitalize(e.event_subtype),k.a.isNil(e.external_coincidence)||k.a.isEmpty(e.external_coincidence)||(t+=" Combined"),t},getHermesDBData(){r.a.get(`${this.$store.state.hermesApiBaseUrl}/api/v0/nonlocalizedevents/${this.superevent_data.event_id}/`,this.$store.state.hermesAxiosConfig).then(e=>{this.references=k.a.get(e["data"],"references",[])}).catch(e=>{console.log(`Error getting references for superevent ${this.superevent_data.event_id}: ${e}`)})}}},fe=he,pe=C(fe,L,w,!1,null,null,null),me=pe.exports,ve="undefined"!==typeof window,_e="undefined"!==typeof document,ge="undefined"!==typeof navigator,ye="undefined"!==typeof Promise,be="undefined"!==typeof MutationObserver||"undefined"!==typeof WebKitMutationObserver||"undefined"!==typeof MozMutationObserver,Me=ve&&_e&&ge,ze=ve?window:{},Le=_e?document:{},we=ge?navigator:{},He=(we.userAgent||"").toLowerCase(),ke=He.indexOf("jsdom")>0,Ae=(/msie|trident/.test(He),function(){var e=!1;if(Me)try{var t={get passive(){e=!0}};ze.addEventListener("test",t,t),ze.removeEventListener("test",t,t)}catch(a){e=!1}return e}()),Se=Me&&("ontouchstart"in Le.documentElement||we.maxTouchPoints>0),Ve=Me&&Boolean(ze.PointerEvent||ze.MSPointerEvent),Oe=Me&&"IntersectionObserver"in ze&&"IntersectionObserverEntry"in ze&&"intersectionRatio"in ze.IntersectionObserverEntry.prototype,Te="BvConfig",De="$bvConfig",Ce=["xs","sm","md","lg","xl"],je=/\[(\d+)]/g,xe=/^(BV?)/,Be=/^\d+$/,Ye=/^\..+/,Pe=/^#/,Ie=/^#[A-Za-z]+[\w\-:.]*$/,Fe=/(<([^>]+)>)/gi,Ee=/\B([A-Z])/g,$e=/([a-z])([A-Z])/g,Re=/^[0-9]*\.?[0-9]+$/,Ne=/\+/g,We=/[-/\\^$*+?.()|[\]{}]/g,Ue=/[\s\uFEFF\xA0]+/g,Ge=/\s+/,qe=/\/\*$/,Je=/(\s|^)(\w)/g,Ke=/^\s+/,Xe=/_/g,Ze=/-(\w)/g,Qe=/^\d+-\d\d?-\d\d?(?:\s|T|$)/,et=/-|\s|T/,tt=/^([0-1]?[0-9]|2[0-3]):[0-5]?[0-9](:[0-5]?[0-9])?$/,at=/^.*(#[^#]+)$/,nt=/%2C/g,rt=/[!'()*]/g,it=/^(\?|#|&)/,ot=/^\d+(\.\d*)?[/:]\d+(\.\d*)?$/,st=/[/:]/,lt=/^col-/,ct=/^BIcon/,ut=/-u-.+/;function dt(e){return dt="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},dt(e)}function ht(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function ft(e,t){if("function"!==typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");Object.defineProperty(e,"prototype",{value:Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),writable:!1}),t&&Mt(e,t)}function pt(e){var t=yt();return function(){var a,n=zt(e);if(t){var r=zt(this).constructor;a=Reflect.construct(n,arguments,r)}else a=n.apply(this,arguments);return mt(this,a)}}function mt(e,t){if(t&&("object"===dt(t)||"function"===typeof t))return t;if(void 0!==t)throw new TypeError("Derived constructors may only return object or undefined");return vt(e)}function vt(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function _t(e){var t="function"===typeof Map?new Map:void 0;return _t=function(e){if(null===e||!bt(e))return e;if("function"!==typeof e)throw new TypeError("Super expression must either be null or a function");if("undefined"!==typeof t){if(t.has(e))return t.get(e);t.set(e,a)}function a(){return gt(e,arguments,zt(this).constructor)}return a.prototype=Object.create(e.prototype,{constructor:{value:a,enumerable:!1,writable:!0,configurable:!0}}),Mt(a,e)},_t(e)}function gt(e,t,a){return gt=yt()?Reflect.construct:function(e,t,a){var n=[null];n.push.apply(n,t);var r=Function.bind.apply(e,n),i=new r;return a&&Mt(i,a.prototype),i},gt.apply(null,arguments)}function yt(){if("undefined"===typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"===typeof Proxy)return!0;try{return Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],(function(){}))),!0}catch(e){return!1}}function bt(e){return-1!==Function.toString.call(e).indexOf("[native code]")}function Mt(e,t){return Mt=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e},Mt(e,t)}function zt(e){return zt=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)},zt(e)}var Lt=ve?ze.Element:function(e){ft(a,e);var t=pt(a);function a(){return ht(this,a),t.apply(this,arguments)}return a}(_t(Object)),wt=ve?ze.HTMLElement:function(e){ft(a,e);var t=pt(a);function a(){return ht(this,a),t.apply(this,arguments)}return a}(Lt),Ht=ve?ze.SVGElement:function(e){ft(a,e);var t=pt(a);function a(){return ht(this,a),t.apply(this,arguments)}return a}(Lt),kt=ve?ze.File:function(e){ft(a,e);var t=pt(a);function a(){return ht(this,a),t.apply(this,arguments)}return a}(_t(Object));function At(e){return At="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},At(e)}var St=function(e){return At(e)},Vt=function(e){return Object.prototype.toString.call(e).slice(8,-1)},Ot=function(e){return void 0===e},Tt=function(e){return null===e},Dt=function(e){return Ot(e)||Tt(e)},Ct=function(e){return"function"===St(e)},jt=function(e){return"boolean"===St(e)},xt=function(e){return"string"===St(e)},Bt=function(e){return"number"===St(e)},Yt=function(e){return Re.test(String(e))},Pt=function(e){return Array.isArray(e)},It=function(e){return null!==e&&"object"===At(e)},Ft=function(e){return"[object Object]"===Object.prototype.toString.call(e)},Et=function(e){return e instanceof Date},$t=function(e){return e instanceof Event},Rt=function(e){return e instanceof kt},Nt=function(e){return"RegExp"===Vt(e)},Wt=function(e){return!Dt(e)&&Ct(e.then)&&Ct(e.catch)};function Ut(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function Gt(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?Ut(Object(a),!0).forEach((function(t){qt(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):Ut(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function qt(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Jt=function(){return Object.assign.apply(Object,arguments)},Kt=function(e,t){return Object.create(e,t)},Xt=function(e,t){return Object.defineProperties(e,t)},Zt=function(e,t,a){return Object.defineProperty(e,t,a)},Qt=function(e){return Object.getOwnPropertyNames(e)},ea=function(e){return Object.keys(e)},ta=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},aa=function(e){return Object.prototype.toString.call(e)},na=function(e){return Gt({},e)},ra=function(e,t){return ea(e).filter((function(e){return-1!==t.indexOf(e)})).reduce((function(t,a){return Gt(Gt({},t),{},qt({},a,e[a]))}),{})},ia=function(e,t){return ea(e).filter((function(e){return-1===t.indexOf(e)})).reduce((function(t,a){return Gt(Gt({},t),{},qt({},a,e[a]))}),{})},oa=function e(t,a){return It(t)&&It(a)&&ea(a).forEach((function(n){It(a[n])?(t[n]&&It(t[n])||(t[n]=a[n]),e(t[n],a[n])):Jt(t,qt({},n,a[n]))})),t},sa=function(e){return ea(e).sort().reduce((function(t,a){return Gt(Gt({},t),{},qt({},a,e[a]))}),{})},la=function(){return{enumerable:!0,configurable:!1,writable:!1}};function ca(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function ua(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?ca(Object(a),!0).forEach((function(t){da(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):ca(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function da(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}function ha(e){return va(e)||ma(e)||pa(e)||fa()}function fa(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function pa(e,t){if(e){if("string"===typeof e)return _a(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?_a(e,t):void 0}}function ma(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function va(e){if(Array.isArray(e))return _a(e)}function _a(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}var ga=function e(t){var a=arguments.length>1&&void 0!==arguments[1]?arguments[1]:t;return Pt(t)?t.reduce((function(t,a){return[].concat(ha(t),[e(a,a)])}),[]):Ft(t)?ea(t).reduce((function(a,n){return ua(ua({},a),{},da({},n,e(t[n],t[n])))}),{}):a},ya=function(e){return e},ba=function(e,t){var a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0;if(t=Pt(t)?t.join("."):t,!t||!It(e))return a;if(t in e)return e[t];t=String(t).replace(je,".$1");var n=t.split(".").filter(ya);return 0===n.length?a:n.every((function(t){return It(e)&&t in e&&!Dt(e=e[t])}))?e:Tt(e)?null:a},Ma=function(e,t){var a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:null,n=ba(e,t);return Dt(n)?a:n},za=a("938d"),La=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null;Object(za["a"])()||console.warn("[BootstrapVue warn]: ".concat(t?"".concat(t," - "):"").concat(e))},wa=function(e){return!Me&&(La("".concat(e,": Can not be called during SSR.")),!0)},Ha=function(e){return!ye&&(La("".concat(e,": Requires Promise support.")),!0)},ka=function(e){return!be&&(La("".concat(e,": Requires MutationObserver support.")),!0)};function Aa(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function Sa(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function Va(e,t,a){return t&&Sa(e.prototype,t),a&&Sa(e,a),Object.defineProperty(e,"prototype",{writable:!1}),e}var Oa=function(){function e(){Aa(this,e),this.$_config={}}return Va(e,[{key:"setConfig",value:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{};if(Ft(t)){var a=Qt(t);a.forEach((function(a){var n=t[a];"breakpoints"===a?!Pt(n)||n.length<2||n.some((function(e){return!xt(e)||0===e.length}))?La('"breakpoints" must be an array of at least 2 breakpoint names',Te):e.$_config[a]=ga(n):Ft(n)&&(e.$_config[a]=Qt(n).reduce((function(e,t){return Ot(n[t])||(e[t]=ga(n[t])),e}),e.$_config[a]||{}))}))}}},{key:"resetConfig",value:function(){this.$_config={}}},{key:"getConfig",value:function(){return ga(this.$_config)}},{key:"getConfigValue",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:void 0;return ga(ba(this.$_config,e,t))}}]),e}(),Ta=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:i["default"];t.prototype[De]=i["default"].prototype[De]=t.prototype[De]||i["default"].prototype[De]||new Oa,t.prototype[De].setConfig(e)};function Da(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function Ca(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?Da(Object(a),!0).forEach((function(t){ja(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):Da(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function ja(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var xa=function(){var e=!1,t=["Multiple instances of Vue detected!","You may need to set up an alias for Vue in your bundler config.","See: https://bootstrap-vue.org/docs#using-module-bundlers"].join("\n");return function(a){e||i["default"]===a||ke||La(t),e=!0}}(),Ba=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.components,a=e.directives,n=e.plugins,r=function e(r){var i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};e.installed||(e.installed=!0,xa(r),Ta(i,r),$a(r,t),Na(r,a),Fa(r,n))};return r.installed=!1,r},Ya=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.components,a=e.directives,n=e.plugins,r=function e(r){e.installed||(e.installed=!0,xa(r),$a(r,t),Na(r,a),Fa(r,n))};return r.installed=!1,r},Pa=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return Ca(Ca({},t),{},{install:Ba(e)})},Ia=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return Ca(Ca({},t),{},{install:Ya(e)})},Fa=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};for(var a in t)a&&t[a]&&e.use(t[a])},Ea=function(e,t,a){e&&t&&a&&e.component(t,a)},$a=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};for(var a in t)Ea(e,a,t[a])},Ra=function(e,t,a){e&&t&&a&&e.directive(t.replace(/^VB/,"B"),a)},Na=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};for(var a in t)Ra(e,a,t[a])},Wa=function(){return(Wa=Object.assign||function(e){for(var t,a=1,n=arguments.length;a<n;a++)for(var r in t=arguments[a])Object.prototype.hasOwnProperty.call(t,r)&&(e[r]=t[r]);return e}).apply(this,arguments)},Ua={kebab:/-(\w)/g,styleProp:/:(.*)/,styleList:/;(?![^(]*\))/g};function Ga(e,t){return t?t.toUpperCase():""}function qa(e){for(var t,a={},n=0,r=e.split(Ua.styleList);n<r.length;n++){var i=r[n].split(Ua.styleProp),o=i[0],s=i[1];(o=o.trim())&&("string"==typeof s&&(s=s.trim()),a[(t=o,t.replace(Ua.kebab,Ga))]=s)}return a}function Ja(){for(var e,t,a={},n=arguments.length;n--;)for(var r=0,i=Object.keys(arguments[n]);r<i.length;r++)switch(e=i[r]){case"class":case"style":case"directives":if(Array.isArray(a[e])||(a[e]=[]),"style"===e){var o=void 0;o=Array.isArray(arguments[n].style)?arguments[n].style:[arguments[n].style];for(var s=0;s<o.length;s++){var l=o[s];"string"==typeof l&&(o[s]=qa(l))}arguments[n].style=o}a[e]=a[e].concat(arguments[n][e]);break;case"staticClass":if(!arguments[n][e])break;void 0===a[e]&&(a[e]=""),a[e]&&(a[e]+=" "),a[e]+=arguments[n][e].trim();break;case"on":case"nativeOn":a[e]||(a[e]={});for(var c=0,u=Object.keys(arguments[n][e]||{});c<u.length;c++)t=u[c],a[e][t]?a[e][t]=[].concat(a[e][t],arguments[n][e][t]):a[e][t]=arguments[n][e][t];break;case"attrs":case"props":case"domProps":case"scopedSlots":case"staticStyle":case"hook":case"transition":a[e]||(a[e]={}),a[e]=Wa({},arguments[n][e],a[e]);break;case"slot":case"key":case"ref":case"tag":case"show":case"keepAlive":default:a[e]||(a[e]=arguments[n][e])}return a}var Ka="_uid",Xa="BAlert",Za="BAspect",Qa="BAvatar",en="BAvatarGroup",tn="BBadge",an="BBreadcrumb",nn="BBreadcrumbItem",rn="BBreadcrumbLink",on="BButton",sn="BButtonClose",ln="BButtonGroup",cn="BButtonToolbar",un="BCalendar",dn="BCard",hn="BCardBody",fn="BCardFooter",pn="BCardGroup",mn="BCardHeader",vn="BCardImg",_n="BCardImgLazy",gn="BCardSubTitle",yn="BCardText",bn="BCardTitle",Mn="BCarousel",zn="BCarouselSlide",Ln="BCol",wn="BCollapse",Hn="BContainer",kn="BDropdown",An="BDropdownDivider",Sn="BDropdownForm",Vn="BDropdownGroup",On="BDropdownHeader",Tn="BDropdownItem",Dn="BDropdownItemButton",Cn="BDropdownText",jn="BEmbed",xn="BForm",Bn="BFormCheckbox",Yn="BFormCheckboxGroup",Pn="BFormDatalist",In="BFormDatepicker",Fn="BFormFile",En="BFormGroup",$n="BFormInput",Rn="BFormInvalidFeedback",Nn="BFormRadio",Wn="BFormRadioGroup",Un="BFormRating",Gn="BFormRow",qn="BFormSelect",Jn="BFormSelectOption",Kn="BFormSelectOptionGroup",Xn="BFormSpinbutton",Zn="BFormTag",Qn="BFormTags",er="BFormText",tr="BFormTextarea",ar="BFormTimepicker",nr="BFormValidFeedback",rr="BIcon",ir="BIconstack",or="BIconBase",sr="BImg",lr="BImgLazy",cr="BInputGroup",ur="BInputGroupAddon",dr="BInputGroupAppend",hr="BInputGroupPrepend",fr="BInputGroupText",pr="BJumbotron",mr="BLink",vr="BListGroup",_r="BListGroupItem",gr="BMedia",yr="BMediaAside",br="BMediaBody",Mr="BModal",zr="BMsgBox",Lr="BNav",wr="BNavbar",Hr="BNavbarBrand",kr="BNavbarNav",Ar="BNavbarToggle",Sr="BNavForm",Vr="BNavItem",Or="BNavItemDropdown",Tr="BNavText",Dr="BOverlay",Cr="BPagination",jr="BPaginationNav",xr="BPopover",Br="BProgress",Yr="BProgressBar",Pr="BRow",Ir="BSidebar",Fr="BSkeleton",Er="BSkeletonIcon",$r="BSkeletonImg",Rr="BSkeletonTable",Nr="BSkeletonWrapper",Wr="BSpinner",Ur="BTab",Gr="BTable",qr="BTableCell",Jr="BTableLite",Kr="BTableSimple",Xr="BTabs",Zr="BTbody",Qr="BTfoot",ei="BTh",ti="BThead",ai="BTime",ni="BToast",ri="BToaster",ii="BTooltip",oi="BTr",si="BVCollapse",li="BVFormBtnLabelControl",ci="BVFormRatingStar",ui="BVPopover",di="BVPopoverTemplate",hi="BVPopper",fi="BVTabButton",pi="BVToastPop",mi="BVTooltip",vi="BVTooltipTemplate",_i="BVTransition",gi="BVTransporter",yi="BVTransporterTarget",bi="activate-tab",Mi="blur",zi="cancel",Li="change",wi="changed",Hi="click",ki="close",Ai="context",Si="context-changed",Vi="destroyed",Oi="disable",Ti="disabled",Di="dismissed",Ci="dismiss-count-down",ji="enable",xi="enabled",Bi="filtered",Yi="first",Pi="focus",Ii="focusin",Fi="focusout",Ei="head-clicked",$i="hidden",Ri="hide",Ni="img-error",Wi="input",Ui="last",Gi="mouseenter",qi="mouseleave",Ji="next",Ki="ok",Xi="open",Zi="page-click",Qi="paused",eo="prev",to="refresh",ao="refreshed",no="remove",ro="row-clicked",io="row-contextmenu",oo="row-dblclicked",so="row-hovered",lo="row-middle-clicked",co="row-selected",uo="row-unhovered",ho="selected",fo="show",po="shown",mo="sliding-end",vo="sliding-start",_o="sort-changed",go="tag-state",yo="toggle",bo="unpaused",Mo="update",zo="hook:beforeDestroy",Lo="hook:destroyed",wo="update:",Ho="bv",ko="::",Ao={passive:!0},So={passive:!0,capture:!1},Vo=void 0,Oo=Array,To=Boolean,Do=Date,Co=Function,jo=Number,xo=Object,Bo=RegExp,Yo=String,Po=[Oo,Co],Io=[Oo,xo],Fo=[Oo,xo,Yo],Eo=[Oo,Yo],$o=[To,jo],Ro=[To,jo,Yo],No=[To,Yo],Wo=[Do,Yo],Uo=[Co,Yo],Go=[jo,Yo],qo=[jo,xo,Yo],Jo=[xo,Co],Ko=[xo,Yo],Xo="add-button-text",Zo="append",Qo="aside",es="badge",ts="bottom-row",as="button-content",ns="custom-foot",rs="decrement",is="default",os="description",ss="dismiss",ls="drop-placeholder",cs="ellipsis-text",us="empty",ds="emptyfiltered",hs="file-name",fs="first",ps="first-text",ms="footer",vs="header",_s="header-close",gs="icon-clear",ys="icon-empty",bs="icon-full",Ms="icon-half",zs="img",Ls="increment",ws="invalid-feedback",Hs="label",ks="last-text",As="lead",Ss="loading",Vs="modal-backdrop",Os="modal-cancel",Ts="modal-footer",Ds="modal-header",Cs="modal-header-close",js="modal-ok",xs="modal-title",Bs="nav-next-decade",Ys="nav-next-month",Ps="nav-next-year",Is="nav-prev-decade",Fs="nav-prev-month",Es="nav-prev-year",$s="nav-this-month",Rs="next-text",Ns="overlay",Ws="page",Us="placeholder",Gs="prepend",qs="prev-text",Js="row-details",Ks="table-busy",Xs="table-caption",Zs="table-colgroup",Qs="tabs-end",el="tabs-start",tl="text",al="thead-top",nl="title",rl="toast-title",il="top-row",ol="valid-feedback",sl=function(){return Array.from.apply(Array,arguments)},ll=function(e,t){return-1!==e.indexOf(t)},cl=function(){for(var e=arguments.length,t=new Array(e),a=0;a<e;a++)t[a]=arguments[a];return Array.prototype.concat.apply([],t)},ul=function(e,t){var a=Ct(t)?t:function(){return t};return Array.apply(null,{length:e}).map(a)},dl=function(e){return e.reduce((function(e,t){return cl(e,t)}),[])},hl=function e(t){return t.reduce((function(t,a){return cl(t,Array.isArray(a)?e(a):a)}),[])},fl=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:NaN,a=parseInt(e,10);return isNaN(a)?t:a},pl=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:NaN,a=parseFloat(e);return isNaN(a)?t:a},ml=function(e,t){return pl(e).toFixed(fl(t,0))},vl=function(e){return e.replace(Ee,"-$1").toLowerCase()},_l=function(e){return e=vl(e).replace(Ze,(function(e,t){return t?t.toUpperCase():""})),e.charAt(0).toUpperCase()+e.slice(1)},gl=function(e){return e.replace(Xe," ").replace($e,(function(e,t,a){return t+" "+a})).replace(Je,(function(e,t,a){return t+a.toUpperCase()}))},yl=function(e){return e=xt(e)?e.trim():String(e),e.charAt(0).toLowerCase()+e.slice(1)},bl=function(e){return e=xt(e)?e.trim():String(e),e.charAt(0).toUpperCase()+e.slice(1)},Ml=function(e){return e.replace(We,"\\$&")},zl=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:2;return Dt(e)?"":Pt(e)||Ft(e)&&e.toString===Object.prototype.toString?JSON.stringify(e,null,t):String(e)},Ll=function(e){return zl(e).replace(Ke,"")},wl=function(e){return zl(e).trim()},Hl=function(e){return zl(e).toLowerCase()},kl=Lt.prototype,Al=["button","[href]:not(.disabled)","input","select","textarea","[tabindex]","[contenteditable]"].map((function(e){return"".concat(e,":not(:disabled):not([disabled])")})).join(", "),Sl=kl.matches||kl.msMatchesSelector||kl.webkitMatchesSelector,Vl=kl.closest||function(e){var t=this;do{if($l(t,e))return t;t=t.parentElement||t.parentNode}while(!Tt(t)&&t.nodeType===Node.ELEMENT_NODE);return null},Ol=(ze.requestAnimationFrame||ze.webkitRequestAnimationFrame||ze.mozRequestAnimationFrame||ze.msRequestAnimationFrame||ze.oRequestAnimationFrame||function(e){return setTimeout(e,16)}).bind(ze),Tl=ze.MutationObserver||ze.WebKitMutationObserver||ze.MozMutationObserver||null,Dl=function(e){return e&&e.parentNode&&e.parentNode.removeChild(e)},Cl=function(e){return!(!e||e.nodeType!==Node.ELEMENT_NODE)},jl=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[],t=Le.activeElement;return t&&!e.some((function(e){return e===t}))?t:null},xl=function(e,t){return zl(e).toLowerCase()===zl(t).toLowerCase()},Bl=function(e){return Cl(e)&&e===jl()},Yl=function(e){if(!Cl(e)||!e.parentNode||!Nl(Le.body,e))return!1;if("none"===tc(e,"display"))return!1;var t=ac(e);return!!(t&&t.height>0&&t.width>0)},Pl=function(e){return!Cl(e)||e.disabled||Zl(e,"disabled")||ql(e,"disabled")},Il=function(e){return Cl(e)&&e.offsetHeight},Fl=function(e,t){return sl((Cl(t)?t:Le).querySelectorAll(e))},El=function(e,t){return(Cl(t)?t:Le).querySelector(e)||null},$l=function(e,t){return!!Cl(e)&&Sl.call(e,t)},Rl=function(e,t){var a=arguments.length>2&&void 0!==arguments[2]&&arguments[2];if(!Cl(t))return null;var n=Vl.call(t,e);return a?n:n===t?null:n},Nl=function(e,t){return!(!e||!Ct(e.contains))&&e.contains(t)},Wl=function(e){return Le.getElementById(/^#/.test(e)?e.slice(1):e)||null},Ul=function(e,t){t&&Cl(e)&&e.classList&&e.classList.add(t)},Gl=function(e,t){t&&Cl(e)&&e.classList&&e.classList.remove(t)},ql=function(e,t){return!!(t&&Cl(e)&&e.classList)&&e.classList.contains(t)},Jl=function(e,t,a){t&&Cl(e)&&e.setAttribute(t,a)},Kl=function(e,t){t&&Cl(e)&&e.removeAttribute(t)},Xl=function(e,t){return t&&Cl(e)?e.getAttribute(t):null},Zl=function(e,t){return t&&Cl(e)?e.hasAttribute(t):null},Ql=function(e,t,a){t&&Cl(e)&&(e.style[t]=a)},ec=function(e,t){t&&Cl(e)&&(e.style[t]="")},tc=function(e,t){return t&&Cl(e)&&e.style[t]||null},ac=function(e){return Cl(e)?e.getBoundingClientRect():null},nc=function(e){var t=ze.getComputedStyle;return t&&Cl(e)?t(e):{}},rc=function(){var e=ze.getSelection;return e?ze.getSelection():null},ic=function(e){var t={top:0,left:0};if(!Cl(e)||0===e.getClientRects().length)return t;var a=ac(e);if(a){var n=e.ownerDocument.defaultView;t.top=a.top+n.pageYOffset,t.left=a.left+n.pageXOffset}return t},oc=function(e){var t={top:0,left:0};if(!Cl(e))return t;var a={top:0,left:0},n=nc(e);if("fixed"===n.position)t=ac(e)||t;else{t=ic(e);var r=e.ownerDocument,i=e.offsetParent||r.documentElement;while(i&&(i===r.body||i===r.documentElement)&&"static"===nc(i).position)i=i.parentNode;if(i&&i!==e&&i.nodeType===Node.ELEMENT_NODE){a=ic(i);var o=nc(i);a.top+=pl(o.borderTopWidth,0),a.left+=pl(o.borderLeftWidth,0)}}return{top:t.top-a.top-pl(n.marginTop,0),left:t.left-a.left-pl(n.marginLeft,0)}},sc=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:document;return Fl(Al,e).filter(Yl).filter((function(e){return e.tabIndex>-1&&!e.disabled}))},lc=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};try{e.focus(t)}catch(a){}return Bl(e)},cc=function(e){try{e.blur()}catch(t){}return!Bl(e)},uc=function(e){var t=Kt(null);return function(){for(var a=arguments.length,n=new Array(a),r=0;r<a;r++)n[r]=arguments[r];var i=JSON.stringify(n);return t[i]=t[i]||e.apply(null,n)}},dc=i["default"].prototype,hc=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:void 0,a=dc[De];return a?a.getConfigValue(e,t):ga(t)},fc=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0;return t?hc("".concat(e,".").concat(t),a):hc(e,{})},pc=function(){return hc("breakpoints",Ce)},mc=uc((function(){return pc()})),vc=function(){return ga(mc())},_c=uc((function(){var e=vc();return e[0]="",e}));function gc(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function yc(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?gc(Object(a),!0).forEach((function(t){bc(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):gc(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function bc(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Mc=function(e,t){return e+bl(t)},zc=function(e,t){return yl(t.replace(e,""))},Lc=function(e,t){return t+(e?bl(e):"")},wc=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:Vo,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:void 0,a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:void 0,n=arguments.length>3&&void 0!==arguments[3]?arguments[3]:void 0,r=!0===a;return n=r?n:a,yc(yc(yc({},e?{type:e}:{}),r?{required:r}:Ot(t)?{}:{default:It(t)?function(){return t}:t}),Ot(n)?{}:{validator:n})},Hc=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:ya;if(Pt(e))return e.map(t);var a={};for(var n in e)ta(e,n)&&(a[t(n)]=It(e[n])?na(e[n]):e[n]);return a},kc=function(e,t){var a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:ya;return(Pt(e)?e.slice():ea(e)).reduce((function(e,n){return e[a(n)]=t[n],e}),{})},Ac=function(e,t,a){return yc(yc({},ga(e)),{},{default:function(){var n=fc(a,t,e.default);return Ct(n)?n():n}})},Sc=function(e,t){return ea(e).reduce((function(a,n){return yc(yc({},a),{},bc({},n,Ac(e[n],n,t)))}),{})},Vc=Ac({},"","").default.name,Oc=function(e){return Ct(e)&&e.name&&e.name!==Vc};function Tc(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Dc=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},a=t.type,n=void 0===a?Vo:a,r=t.defaultValue,o=void 0===r?void 0:r,s=t.validator,l=void 0===s?void 0:s,c=t.event,u=void 0===c?Wi:c,d=Tc({},e,wc(n,o,l)),h=i["default"].extend({model:{prop:e,event:u},props:d});return{mixin:h,props:d,prop:e,event:u}},Cc=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};return e=cl(e).filter(ya),e.some((function(e){return t[e]||a[e]}))},jc=function(e){var t,a=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{};e=cl(e).filter(ya);for(var i=0;i<e.length&&!t;i++){var o=e[i];t=n[o]||r[o]}return Ct(t)?t(a):t},xc=i["default"].extend({methods:{hasNormalizedSlot:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:is,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:this.$scopedSlots,a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.$slots;return Cc(e,t,a)},normalizeSlot:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:is,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:this.$scopedSlots,n=arguments.length>3&&void 0!==arguments[3]?arguments[3]:this.$slots,r=jc(e,t,a,n);return r?cl(r):r}}}),Bc=function(e){return Ae?It(e)?e:{capture:!!e||!1}:!!(It(e)?e.capture:e)},Yc=function(e,t,a,n){e&&e.addEventListener&&e.addEventListener(t,a,Bc(n))},Pc=function(e,t,a,n){e&&e.removeEventListener&&e.removeEventListener(t,a,Bc(n))},Ic=function(e){for(var t=e?Yc:Pc,a=arguments.length,n=new Array(a>1?a-1:0),r=1;r<a;r++)n[r-1]=arguments[r];t.apply(void 0,n)},Fc=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},a=t.preventDefault,n=void 0===a||a,r=t.propagation,i=void 0===r||r,o=t.immediatePropagation,s=void 0!==o&&o;n&&e.preventDefault(),i&&e.stopPropagation(),s&&e.stopImmediatePropagation()},Ec=function(e){return vl(e.replace(xe,""))},$c=function(e,t){return[Ho,Ec(e),t].join(ko)},Rc=function(e,t){return[Ho,t,Ec(e)].join(ko)};function Nc(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Wc=Sc({ariaLabel:wc(Yo,"Close"),content:wc(Yo,"&times;"),disabled:wc(To,!1),textVariant:wc(Yo)},sn),Uc=i["default"].extend({name:sn,functional:!0,props:Wc,render:function(e,t){var a=t.props,n=t.data,r=t.slots,i=t.scopedSlots,o=r(),s=i||{},l={staticClass:"close",class:Nc({},"text-".concat(a.textVariant),a.textVariant),attrs:{type:"button",disabled:a.disabled,"aria-label":a.ariaLabel?String(a.ariaLabel):null},on:{click:function(e){a.disabled&&$t(e)&&Fc(e)}}};return Cc(is,s,o)||(l.domProps={innerHTML:a.content}),e("button",Ja(n,l),jc(is,{},s,o))}});function Gc(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function qc(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?Gc(Object(a),!0).forEach((function(t){Jc(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):Gc(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function Jc(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Kc,Xc={name:"",enterClass:"",enterActiveClass:"",enterToClass:"show",leaveClass:"show",leaveActiveClass:"",leaveToClass:""},Zc=qc(qc({},Xc),{},{enterActiveClass:"fade",leaveActiveClass:"fade"}),Qc={appear:wc(To,!1),mode:wc(Yo),noFade:wc(To,!1),transProps:wc(xo)},eu=i["default"].extend({name:_i,functional:!0,props:Qc,render:function(e,t){var a=t.children,n=t.data,r=t.props,i=r.transProps;return Ft(i)||(i=r.noFade?Xc:Zc,r.appear&&(i=qc(qc({},i),{},{appear:!0,appearClass:i.enterClass,appearActiveClass:i.enterActiveClass,appearToClass:i.enterToClass}))),i=qc(qc({mode:r.mode},i),{},{css:!0}),e("transition",Ja(n,{props:i}),a)}});function tu(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function au(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?tu(Object(a),!0).forEach((function(t){nu(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):tu(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function nu(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var ru=Dc("show",{type:Ro,defaultValue:!1}),iu=ru.mixin,ou=ru.props,su=ru.prop,lu=ru.event,cu=function(e){return""===e||jt(e)?0:(e=fl(e,0),e>0?e:0)},uu=function(e){return""===e||!0===e||!(fl(e,0)<1)&&!!e},du=Sc(sa(au(au({},ou),{},{dismissLabel:wc(Yo,"Close"),dismissible:wc(To,!1),fade:wc(To,!1),variant:wc(Yo,"info")})),Xa),hu=i["default"].extend({name:Xa,mixins:[iu,xc],props:du,data:function(){return{countDown:0,localShow:uu(this[su])}},watch:(Kc={},nu(Kc,su,(function(e){this.countDown=cu(e),this.localShow=uu(e)})),nu(Kc,"countDown",(function(e){var t=this;this.clearCountDownInterval();var a=this[su];Yt(a)&&(this.$emit(Ci,e),a!==e&&this.$emit(lu,e),e>0?(this.localShow=!0,this.$_countDownTimeout=setTimeout((function(){t.countDown--}),1e3)):this.$nextTick((function(){Ol((function(){t.localShow=!1}))})))})),nu(Kc,"localShow",(function(e){var t=this[su];e||!this.dismissible&&!Yt(t)||this.$emit(Di),Yt(t)||t===e||this.$emit(lu,e)})),Kc),created:function(){this.$_filterTimer=null;var e=this[su];this.countDown=cu(e),this.localShow=uu(e)},beforeDestroy:function(){this.clearCountDownInterval()},methods:{dismiss:function(){this.clearCountDownInterval(),this.countDown=0,this.localShow=!1},clearCountDownInterval:function(){clearTimeout(this.$_countDownTimeout),this.$_countDownTimeout=null}},render:function(e){var t=e();if(this.localShow){var a=this.dismissible,n=this.variant,r=e();a&&(r=e(Uc,{attrs:{"aria-label":this.dismissLabel},on:{click:this.dismiss}},[this.normalizeSlot(ss)])),t=e("div",{staticClass:"alert",class:nu({"alert-dismissible":a},"alert-".concat(n),n),attrs:{role:"alert","aria-live":"polite","aria-atomic":!0},key:this[Ka]},[r,this.normalizeSlot()])}return e(eu,{props:{noFade:!this.fade}},[t])}}),fu=Pa({components:{BAlert:hu}}),pu=Math.min,mu=Math.max,vu=Math.abs,_u=Math.ceil,gu=Math.floor,yu=Math.pow,bu=Math.round;function Mu(e,t){return ku(e)||Hu(e,t)||Lu(e,t)||zu()}function zu(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function Lu(e,t){if(e){if("string"===typeof e)return wu(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?wu(e,t):void 0}}function wu(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}function Hu(e,t){var a=null==e?null:"undefined"!==typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=a){var n,r,i=[],o=!0,s=!1;try{for(a=a.call(e);!(o=(n=a.next()).done);o=!0)if(i.push(n.value),t&&i.length===t)break}catch(l){s=!0,r=l}finally{try{o||null==a["return"]||a["return"]()}finally{if(s)throw r}}return i}}function ku(e){if(Array.isArray(e))return e}var Au="b-aspect",Su=Sc({aspect:wc(Go,"1:1"),tag:wc(Yo,"div")},Za),Vu=i["default"].extend({name:Za,mixins:[xc],props:Su,computed:{padding:function(){var e=this.aspect,t=1;if(ot.test(e)){var a=e.split(st).map((function(e){return pl(e)||1})),n=Mu(a,2),r=n[0],i=n[1];t=r/i}else t=pl(e)||1;return"".concat(100/vu(t),"%")}},render:function(e){var t=e("div",{staticClass:"".concat(Au,"-sizer flex-grow-1"),style:{paddingBottom:this.padding,height:0}}),a=e("div",{staticClass:"".concat(Au,"-content flex-grow-1 w-100 mw-100"),style:{marginLeft:"-100%"}},this.normalizeSlot());return e(this.tag,{staticClass:"".concat(Au," d-flex")},[t,a])}}),Ou=Pa({components:{BAspect:Vu}}),Tu="a",Du=function(e){return"%"+e.charCodeAt(0).toString(16)},Cu=function(e){return encodeURIComponent(zl(e)).replace(rt,Du).replace(nt,",")},ju=decodeURIComponent,xu=function(e){if(!Ft(e))return"";var t=ea(e).map((function(t){var a=e[t];return Ot(a)?"":Tt(a)?Cu(t):Pt(a)?a.reduce((function(e,a){return Tt(a)?e.push(Cu(t)):Ot(a)||e.push(Cu(t)+"="+Cu(a)),e}),[]).join("&"):Cu(t)+"="+Cu(a)})).filter((function(e){return e.length>0})).join("&");return t?"?".concat(t):""},Bu=function(e){var t={};return e=zl(e).trim().replace(it,""),e?(e.split("&").forEach((function(e){var a=e.replace(Ne," ").split("="),n=ju(a.shift()),r=a.length>0?ju(a.join("=")):null;Ot(t[n])?t[n]=r:Pt(t[n])?t[n].push(r):t[n]=[t[n],r]})),t):t},Yu=function(e){return!(!e.href&&!e.to)},Pu=function(e){return!(!e||xl(e,"a"))},Iu=function(e,t){var a=e.to,n=e.disabled,r=e.routerComponentName,i=!!t.$router;return!i||i&&(n||!a)?Tu:r||(t.$nuxt?"nuxt-link":"router-link")},Fu=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.target,a=e.rel;return"_blank"===t&&Tt(a)?"noopener":a||null},Eu=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.href,a=e.to,n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:Tu,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"#",i=arguments.length>3&&void 0!==arguments[3]?arguments[3]:"/";if(t)return t;if(Pu(n))return null;if(xt(a))return a||i;if(Ft(a)&&(a.path||a.query||a.hash)){var o=zl(a.path),s=xu(a.query),l=zl(a.hash);return l=l&&"#"!==l.charAt(0)?"#".concat(l):l,"".concat(o).concat(s).concat(l)||i}return r};function $u(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Ru={viewBox:"0 0 16 16",width:"1em",height:"1em",focusable:"false",role:"img","aria-label":"icon"},Nu={width:null,height:null,focusable:null,role:null,"aria-label":null},Wu={animation:wc(Yo),content:wc(Yo),flipH:wc(To,!1),flipV:wc(To,!1),fontScale:wc(Go,1),rotate:wc(Go,0),scale:wc(Go,1),shiftH:wc(Go,0),shiftV:wc(Go,0),stacked:wc(To,!1),title:wc(Yo),variant:wc(Yo)},Uu=i["default"].extend({name:or,functional:!0,props:Wu,render:function(e,t){var a,n=t.data,r=t.props,i=t.children,o=r.animation,s=r.content,l=r.flipH,c=r.flipV,u=r.stacked,d=r.title,h=r.variant,f=mu(pl(r.fontScale,1),0)||1,p=mu(pl(r.scale,1),0)||1,m=pl(r.rotate,0),v=pl(r.shiftH,0),_=pl(r.shiftV,0),g=l||c||1!==p,y=g||m,b=v||_,M=!Dt(s),z=[y?"translate(8 8)":null,g?"scale(".concat((l?-1:1)*p," ").concat((c?-1:1)*p,")"):null,m?"rotate(".concat(m,")"):null,y?"translate(-8 -8)":null].filter(ya),L=e("g",{attrs:{transform:z.join(" ")||null},domProps:M?{innerHTML:s||""}:{}},i);b&&(L=e("g",{attrs:{transform:"translate(".concat(16*v/16," ").concat(-16*_/16,")")}},[L])),u&&(L=e("g",[L]));var w=d?e("title",d):null,H=[w,L].filter(ya);return e("svg",Ja({staticClass:"b-icon bi",class:(a={},$u(a,"text-".concat(h),h),$u(a,"b-icon-animation-".concat(o),o),a),attrs:Ru,style:u?{}:{fontSize:1===f?null:"".concat(100*f,"%")}},n,u?{attrs:Nu}:{},{attrs:{xmlns:u?null:"http://www.w3.org/2000/svg",fill:"currentColor"}}),H)}});function Gu(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function qu(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?Gu(Object(a),!0).forEach((function(t){Ju(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):Gu(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function Ju(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Ku=ia(Wu,["content"]),Xu=function(e,t){var a=vl(e),n="BIcon".concat(_l(e)),r="bi-".concat(a),o=a.replace(/-/g," "),s=wl(t||"");return i["default"].extend({name:n,functional:!0,props:Ku,render:function(e,t){var a=t.data,n=t.props;return e(Uu,Ja({props:{title:o},attrs:{"aria-label":o}},a,{staticClass:r,props:qu(qu({},n),{},{content:s})}))}})},Zu=Xu("Blank",""),Qu=Xu("Alarm",'<path d="M8.5 5.5a.5.5 0 0 0-1 0v3.362l-1.429 2.38a.5.5 0 1 0 .858.515l1.5-2.5A.5.5 0 0 0 8.5 9V5.5z"/><path d="M6.5 0a.5.5 0 0 0 0 1H7v1.07a7.001 7.001 0 0 0-3.273 12.474l-.602.602a.5.5 0 0 0 .707.708l.746-.746A6.97 6.97 0 0 0 8 16a6.97 6.97 0 0 0 3.422-.892l.746.746a.5.5 0 0 0 .707-.708l-.601-.602A7.001 7.001 0 0 0 9 2.07V1h.5a.5.5 0 0 0 0-1h-3zm1.038 3.018a6.093 6.093 0 0 1 .924 0 6 6 0 1 1-.924 0zM0 3.5c0 .753.333 1.429.86 1.887A8.035 8.035 0 0 1 4.387 1.86 2.5 2.5 0 0 0 0 3.5zM13.5 1c-.753 0-1.429.333-1.887.86a8.035 8.035 0 0 1 3.527 3.527A2.5 2.5 0 0 0 13.5 1z"/>'),ed=Xu("AlarmFill",'<path d="M6 .5a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1H9v1.07a7.001 7.001 0 0 1 3.274 12.474l.601.602a.5.5 0 0 1-.707.708l-.746-.746A6.97 6.97 0 0 1 8 16a6.97 6.97 0 0 1-3.422-.892l-.746.746a.5.5 0 0 1-.707-.708l.602-.602A7.001 7.001 0 0 1 7 2.07V1h-.5A.5.5 0 0 1 6 .5zm2.5 5a.5.5 0 0 0-1 0v3.362l-1.429 2.38a.5.5 0 1 0 .858.515l1.5-2.5A.5.5 0 0 0 8.5 9V5.5zM.86 5.387A2.5 2.5 0 1 1 4.387 1.86 8.035 8.035 0 0 0 .86 5.387zM11.613 1.86a2.5 2.5 0 1 1 3.527 3.527 8.035 8.035 0 0 0-3.527-3.527z"/>'),td=Xu("AlignBottom",'<rect width="4" height="12" x="6" y="1" rx="1"/><path d="M1.5 14a.5.5 0 0 0 0 1v-1zm13 1a.5.5 0 0 0 0-1v1zm-13 0h13v-1h-13v1z"/>'),ad=Xu("AlignCenter",'<path d="M8 1a.5.5 0 0 1 .5.5V6h-1V1.5A.5.5 0 0 1 8 1zm0 14a.5.5 0 0 1-.5-.5V10h1v4.5a.5.5 0 0 1-.5.5zM2 7a1 1 0 0 1 1-1h10a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V7z"/>'),nd=Xu("AlignEnd",'<path fill-rule="evenodd" d="M14.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 1 0v-13a.5.5 0 0 0-.5-.5z"/><path d="M13 7a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V7z"/>'),rd=Xu("AlignMiddle",'<path d="M6 13a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1v10zM1 8a.5.5 0 0 0 .5.5H6v-1H1.5A.5.5 0 0 0 1 8zm14 0a.5.5 0 0 1-.5.5H10v-1h4.5a.5.5 0 0 1 .5.5z"/>'),id=Xu("AlignStart",'<path fill-rule="evenodd" d="M1.5 1a.5.5 0 0 1 .5.5v13a.5.5 0 0 1-1 0v-13a.5.5 0 0 1 .5-.5z"/><path d="M3 7a1 1 0 0 1 1-1h10a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V7z"/>'),od=Xu("AlignTop",'<rect width="4" height="12" rx="1" transform="matrix(1 0 0 -1 6 15)"/><path d="M1.5 2a.5.5 0 0 1 0-1v1zm13-1a.5.5 0 0 1 0 1V1zm-13 0h13v1h-13V1z"/>'),sd=Xu("Alt",'<path d="M1 13.5a.5.5 0 0 0 .5.5h3.797a.5.5 0 0 0 .439-.26L11 3h3.5a.5.5 0 0 0 0-1h-3.797a.5.5 0 0 0-.439.26L5 13H1.5a.5.5 0 0 0-.5.5zm10 0a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 0-1h-3a.5.5 0 0 0-.5.5z"/>'),ld=Xu("App",'<path d="M11 2a3 3 0 0 1 3 3v6a3 3 0 0 1-3 3H5a3 3 0 0 1-3-3V5a3 3 0 0 1 3-3h6zM5 1a4 4 0 0 0-4 4v6a4 4 0 0 0 4 4h6a4 4 0 0 0 4-4V5a4 4 0 0 0-4-4H5z"/>'),cd=Xu("AppIndicator",'<path d="M5.5 2A3.5 3.5 0 0 0 2 5.5v5A3.5 3.5 0 0 0 5.5 14h5a3.5 3.5 0 0 0 3.5-3.5V8a.5.5 0 0 1 1 0v2.5a4.5 4.5 0 0 1-4.5 4.5h-5A4.5 4.5 0 0 1 1 10.5v-5A4.5 4.5 0 0 1 5.5 1H8a.5.5 0 0 1 0 1H5.5z"/><path d="M16 3a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/>'),ud=Xu("Archive",'<path d="M0 2a1 1 0 0 1 1-1h14a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1v7.5a2.5 2.5 0 0 1-2.5 2.5h-9A2.5 2.5 0 0 1 1 12.5V5a1 1 0 0 1-1-1V2zm2 3v7.5A1.5 1.5 0 0 0 3.5 14h9a1.5 1.5 0 0 0 1.5-1.5V5H2zm13-3H1v2h14V2zM5 7.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5z"/>'),dd=Xu("ArchiveFill",'<path d="M12.643 15C13.979 15 15 13.845 15 12.5V5H1v7.5C1 13.845 2.021 15 3.357 15h9.286zM5.5 7h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1zM.8 1a.8.8 0 0 0-.8.8V3a.8.8 0 0 0 .8.8h14.4A.8.8 0 0 0 16 3V1.8a.8.8 0 0 0-.8-.8H.8z"/>'),hd=Xu("Arrow90degDown",'<path fill-rule="evenodd" d="M4.854 14.854a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 .708-.708L4 13.293V3.5A2.5 2.5 0 0 1 6.5 1h8a.5.5 0 0 1 0 1h-8A1.5 1.5 0 0 0 5 3.5v9.793l3.146-3.147a.5.5 0 0 1 .708.708l-4 4z"/>'),fd=Xu("Arrow90degLeft",'<path fill-rule="evenodd" d="M1.146 4.854a.5.5 0 0 1 0-.708l4-4a.5.5 0 1 1 .708.708L2.707 4H12.5A2.5 2.5 0 0 1 15 6.5v8a.5.5 0 0 1-1 0v-8A1.5 1.5 0 0 0 12.5 5H2.707l3.147 3.146a.5.5 0 1 1-.708.708l-4-4z"/>'),pd=Xu("Arrow90degRight",'<path fill-rule="evenodd" d="M14.854 4.854a.5.5 0 0 0 0-.708l-4-4a.5.5 0 0 0-.708.708L13.293 4H3.5A2.5 2.5 0 0 0 1 6.5v8a.5.5 0 0 0 1 0v-8A1.5 1.5 0 0 1 3.5 5h9.793l-3.147 3.146a.5.5 0 0 0 .708.708l4-4z"/>'),md=Xu("Arrow90degUp",'<path fill-rule="evenodd" d="M4.854 1.146a.5.5 0 0 0-.708 0l-4 4a.5.5 0 1 0 .708.708L4 2.707V12.5A2.5 2.5 0 0 0 6.5 15h8a.5.5 0 0 0 0-1h-8A1.5 1.5 0 0 1 5 12.5V2.707l3.146 3.147a.5.5 0 1 0 .708-.708l-4-4z"/>'),vd=Xu("ArrowBarDown",'<path fill-rule="evenodd" d="M1 3.5a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13a.5.5 0 0 1-.5-.5zM8 6a.5.5 0 0 1 .5.5v5.793l2.146-2.147a.5.5 0 0 1 .708.708l-3 3a.5.5 0 0 1-.708 0l-3-3a.5.5 0 0 1 .708-.708L7.5 12.293V6.5A.5.5 0 0 1 8 6z"/>'),_d=Xu("ArrowBarLeft",'<path fill-rule="evenodd" d="M12.5 15a.5.5 0 0 1-.5-.5v-13a.5.5 0 0 1 1 0v13a.5.5 0 0 1-.5.5zM10 8a.5.5 0 0 1-.5.5H3.707l2.147 2.146a.5.5 0 0 1-.708.708l-3-3a.5.5 0 0 1 0-.708l3-3a.5.5 0 1 1 .708.708L3.707 7.5H9.5a.5.5 0 0 1 .5.5z"/>'),gd=Xu("ArrowBarRight",'<path fill-rule="evenodd" d="M6 8a.5.5 0 0 0 .5.5h5.793l-2.147 2.146a.5.5 0 0 0 .708.708l3-3a.5.5 0 0 0 0-.708l-3-3a.5.5 0 0 0-.708.708L12.293 7.5H6.5A.5.5 0 0 0 6 8zm-2.5 7a.5.5 0 0 1-.5-.5v-13a.5.5 0 0 1 1 0v13a.5.5 0 0 1-.5.5z"/>'),yd=Xu("ArrowBarUp",'<path fill-rule="evenodd" d="M8 10a.5.5 0 0 0 .5-.5V3.707l2.146 2.147a.5.5 0 0 0 .708-.708l-3-3a.5.5 0 0 0-.708 0l-3 3a.5.5 0 1 0 .708.708L7.5 3.707V9.5a.5.5 0 0 0 .5.5zm-7 2.5a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13a.5.5 0 0 1-.5-.5z"/>'),bd=Xu("ArrowClockwise",'<path fill-rule="evenodd" d="M8 3a5 5 0 1 0 4.546 2.914.5.5 0 0 1 .908-.417A6 6 0 1 1 8 2v1z"/><path d="M8 4.466V.534a.25.25 0 0 1 .41-.192l2.36 1.966c.12.1.12.284 0 .384L8.41 4.658A.25.25 0 0 1 8 4.466z"/>'),Md=Xu("ArrowCounterclockwise",'<path fill-rule="evenodd" d="M8 3a5 5 0 1 1-4.546 2.914.5.5 0 0 0-.908-.417A6 6 0 1 0 8 2v1z"/><path d="M8 4.466V.534a.25.25 0 0 0-.41-.192L5.23 2.308a.25.25 0 0 0 0 .384l2.36 1.966A.25.25 0 0 0 8 4.466z"/>'),zd=Xu("ArrowDown",'<path fill-rule="evenodd" d="M8 1a.5.5 0 0 1 .5.5v11.793l3.146-3.147a.5.5 0 0 1 .708.708l-4 4a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 .708-.708L7.5 13.293V1.5A.5.5 0 0 1 8 1z"/>'),Ld=Xu("ArrowDownCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM8.5 4.5a.5.5 0 0 0-1 0v5.793L5.354 8.146a.5.5 0 1 0-.708.708l3 3a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 10.293V4.5z"/>'),wd=Xu("ArrowDownCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM8.5 4.5a.5.5 0 0 0-1 0v5.793L5.354 8.146a.5.5 0 1 0-.708.708l3 3a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 10.293V4.5z"/>'),Hd=Xu("ArrowDownLeft",'<path fill-rule="evenodd" d="M2 13.5a.5.5 0 0 0 .5.5h6a.5.5 0 0 0 0-1H3.707L13.854 2.854a.5.5 0 0 0-.708-.708L3 12.293V7.5a.5.5 0 0 0-1 0v6z"/>'),kd=Xu("ArrowDownLeftCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-5.904-2.854a.5.5 0 1 1 .707.708L6.707 9.95h2.768a.5.5 0 1 1 0 1H5.5a.5.5 0 0 1-.5-.5V6.475a.5.5 0 1 1 1 0v2.768l4.096-4.097z"/>'),Ad=Xu("ArrowDownLeftCircleFill",'<path d="M16 8A8 8 0 1 0 0 8a8 8 0 0 0 16 0zm-5.904-2.803a.5.5 0 1 1 .707.707L6.707 10h2.768a.5.5 0 0 1 0 1H5.5a.5.5 0 0 1-.5-.5V6.525a.5.5 0 0 1 1 0v2.768l4.096-4.096z"/>'),Sd=Xu("ArrowDownLeftSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm10.096 3.146a.5.5 0 1 1 .707.708L6.707 9.95h2.768a.5.5 0 1 1 0 1H5.5a.5.5 0 0 1-.5-.5V6.475a.5.5 0 1 1 1 0v2.768l4.096-4.097z"/>'),Vd=Xu("ArrowDownLeftSquareFill",'<path d="M2 16a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2zm8.096-10.803L6 9.293V6.525a.5.5 0 0 0-1 0V10.5a.5.5 0 0 0 .5.5h3.975a.5.5 0 0 0 0-1H6.707l4.096-4.096a.5.5 0 1 0-.707-.707z"/>'),Od=Xu("ArrowDownRight",'<path fill-rule="evenodd" d="M14 13.5a.5.5 0 0 1-.5.5h-6a.5.5 0 0 1 0-1h4.793L2.146 2.854a.5.5 0 1 1 .708-.708L13 12.293V7.5a.5.5 0 0 1 1 0v6z"/>'),Td=Xu("ArrowDownRightCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM5.854 5.146a.5.5 0 1 0-.708.708L9.243 9.95H6.475a.5.5 0 1 0 0 1h3.975a.5.5 0 0 0 .5-.5V6.475a.5.5 0 1 0-1 0v2.768L5.854 5.146z"/>'),Dd=Xu("ArrowDownRightCircleFill",'<path d="M0 8a8 8 0 1 1 16 0A8 8 0 0 1 0 8zm5.904-2.803a.5.5 0 1 0-.707.707L9.293 10H6.525a.5.5 0 0 0 0 1H10.5a.5.5 0 0 0 .5-.5V6.525a.5.5 0 0 0-1 0v2.768L5.904 5.197z"/>'),Cd=Xu("ArrowDownRightSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm5.854 3.146a.5.5 0 1 0-.708.708L9.243 9.95H6.475a.5.5 0 1 0 0 1h3.975a.5.5 0 0 0 .5-.5V6.475a.5.5 0 1 0-1 0v2.768L5.854 5.146z"/>'),jd=Xu("ArrowDownRightSquareFill",'<path d="M14 16a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12zM5.904 5.197 10 9.293V6.525a.5.5 0 0 1 1 0V10.5a.5.5 0 0 1-.5.5H6.525a.5.5 0 0 1 0-1h2.768L5.197 5.904a.5.5 0 0 1 .707-.707z"/>'),xd=Xu("ArrowDownShort",'<path fill-rule="evenodd" d="M8 4a.5.5 0 0 1 .5.5v5.793l2.146-2.147a.5.5 0 0 1 .708.708l-3 3a.5.5 0 0 1-.708 0l-3-3a.5.5 0 1 1 .708-.708L7.5 10.293V4.5A.5.5 0 0 1 8 4z"/>'),Bd=Xu("ArrowDownSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm8.5 2.5a.5.5 0 0 0-1 0v5.793L5.354 8.146a.5.5 0 1 0-.708.708l3 3a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 10.293V4.5z"/>'),Yd=Xu("ArrowDownSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm6.5 4.5v5.793l2.146-2.147a.5.5 0 0 1 .708.708l-3 3a.5.5 0 0 1-.708 0l-3-3a.5.5 0 1 1 .708-.708L7.5 10.293V4.5a.5.5 0 0 1 1 0z"/>'),Pd=Xu("ArrowDownUp",'<path fill-rule="evenodd" d="M11.5 15a.5.5 0 0 0 .5-.5V2.707l3.146 3.147a.5.5 0 0 0 .708-.708l-4-4a.5.5 0 0 0-.708 0l-4 4a.5.5 0 1 0 .708.708L11 2.707V14.5a.5.5 0 0 0 .5.5zm-7-14a.5.5 0 0 1 .5.5v11.793l3.146-3.147a.5.5 0 0 1 .708.708l-4 4a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 .708-.708L4 13.293V1.5a.5.5 0 0 1 .5-.5z"/>'),Id=Xu("ArrowLeft",'<path fill-rule="evenodd" d="M15 8a.5.5 0 0 0-.5-.5H2.707l3.147-3.146a.5.5 0 1 0-.708-.708l-4 4a.5.5 0 0 0 0 .708l4 4a.5.5 0 0 0 .708-.708L2.707 8.5H14.5A.5.5 0 0 0 15 8z"/>'),Fd=Xu("ArrowLeftCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-4.5-.5a.5.5 0 0 1 0 1H5.707l2.147 2.146a.5.5 0 0 1-.708.708l-3-3a.5.5 0 0 1 0-.708l3-3a.5.5 0 1 1 .708.708L5.707 7.5H11.5z"/>'),Ed=Xu("ArrowLeftCircleFill",'<path d="M8 0a8 8 0 1 0 0 16A8 8 0 0 0 8 0zm3.5 7.5a.5.5 0 0 1 0 1H5.707l2.147 2.146a.5.5 0 0 1-.708.708l-3-3a.5.5 0 0 1 0-.708l3-3a.5.5 0 1 1 .708.708L5.707 7.5H11.5z"/>'),$d=Xu("ArrowLeftRight",'<path fill-rule="evenodd" d="M1 11.5a.5.5 0 0 0 .5.5h11.793l-3.147 3.146a.5.5 0 0 0 .708.708l4-4a.5.5 0 0 0 0-.708l-4-4a.5.5 0 0 0-.708.708L13.293 11H1.5a.5.5 0 0 0-.5.5zm14-7a.5.5 0 0 1-.5.5H2.707l3.147 3.146a.5.5 0 1 1-.708.708l-4-4a.5.5 0 0 1 0-.708l4-4a.5.5 0 1 1 .708.708L2.707 4H14.5a.5.5 0 0 1 .5.5z"/>'),Rd=Xu("ArrowLeftShort",'<path fill-rule="evenodd" d="M12 8a.5.5 0 0 1-.5.5H5.707l2.147 2.146a.5.5 0 0 1-.708.708l-3-3a.5.5 0 0 1 0-.708l3-3a.5.5 0 1 1 .708.708L5.707 7.5H11.5a.5.5 0 0 1 .5.5z"/>'),Nd=Xu("ArrowLeftSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm11.5 5.5a.5.5 0 0 1 0 1H5.707l2.147 2.146a.5.5 0 0 1-.708.708l-3-3a.5.5 0 0 1 0-.708l3-3a.5.5 0 1 1 .708.708L5.707 7.5H11.5z"/>'),Wd=Xu("ArrowLeftSquareFill",'<path d="M16 14a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12zm-4.5-6.5H5.707l2.147-2.146a.5.5 0 1 0-.708-.708l-3 3a.5.5 0 0 0 0 .708l3 3a.5.5 0 0 0 .708-.708L5.707 8.5H11.5a.5.5 0 0 0 0-1z"/>'),Ud=Xu("ArrowRepeat",'<path d="M11.534 7h3.932a.25.25 0 0 1 .192.41l-1.966 2.36a.25.25 0 0 1-.384 0l-1.966-2.36a.25.25 0 0 1 .192-.41zm-11 2h3.932a.25.25 0 0 0 .192-.41L2.692 6.23a.25.25 0 0 0-.384 0L.342 8.59A.25.25 0 0 0 .534 9z"/><path fill-rule="evenodd" d="M8 3c-1.552 0-2.94.707-3.857 1.818a.5.5 0 1 1-.771-.636A6.002 6.002 0 0 1 13.917 7H12.9A5.002 5.002 0 0 0 8 3zM3.1 9a5.002 5.002 0 0 0 8.757 2.182.5.5 0 1 1 .771.636A6.002 6.002 0 0 1 2.083 9H3.1z"/>'),Gd=Xu("ArrowReturnLeft",'<path fill-rule="evenodd" d="M14.5 1.5a.5.5 0 0 1 .5.5v4.8a2.5 2.5 0 0 1-2.5 2.5H2.707l3.347 3.346a.5.5 0 0 1-.708.708l-4.2-4.2a.5.5 0 0 1 0-.708l4-4a.5.5 0 1 1 .708.708L2.707 8.3H12.5A1.5 1.5 0 0 0 14 6.8V2a.5.5 0 0 1 .5-.5z"/>'),qd=Xu("ArrowReturnRight",'<path fill-rule="evenodd" d="M1.5 1.5A.5.5 0 0 0 1 2v4.8a2.5 2.5 0 0 0 2.5 2.5h9.793l-3.347 3.346a.5.5 0 0 0 .708.708l4.2-4.2a.5.5 0 0 0 0-.708l-4-4a.5.5 0 0 0-.708.708L13.293 8.3H3.5A1.5 1.5 0 0 1 2 6.8V2a.5.5 0 0 0-.5-.5z"/>'),Jd=Xu("ArrowRight",'<path fill-rule="evenodd" d="M1 8a.5.5 0 0 1 .5-.5h11.793l-3.147-3.146a.5.5 0 0 1 .708-.708l4 4a.5.5 0 0 1 0 .708l-4 4a.5.5 0 0 1-.708-.708L13.293 8.5H1.5A.5.5 0 0 1 1 8z"/>'),Kd=Xu("ArrowRightCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM4.5 7.5a.5.5 0 0 0 0 1h5.793l-2.147 2.146a.5.5 0 0 0 .708.708l3-3a.5.5 0 0 0 0-.708l-3-3a.5.5 0 1 0-.708.708L10.293 7.5H4.5z"/>'),Xd=Xu("ArrowRightCircleFill",'<path d="M8 0a8 8 0 1 1 0 16A8 8 0 0 1 8 0zM4.5 7.5a.5.5 0 0 0 0 1h5.793l-2.147 2.146a.5.5 0 0 0 .708.708l3-3a.5.5 0 0 0 0-.708l-3-3a.5.5 0 1 0-.708.708L10.293 7.5H4.5z"/>'),Zd=Xu("ArrowRightShort",'<path fill-rule="evenodd" d="M4 8a.5.5 0 0 1 .5-.5h5.793L8.146 5.354a.5.5 0 1 1 .708-.708l3 3a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708-.708L10.293 8.5H4.5A.5.5 0 0 1 4 8z"/>'),Qd=Xu("ArrowRightSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm4.5 5.5a.5.5 0 0 0 0 1h5.793l-2.147 2.146a.5.5 0 0 0 .708.708l3-3a.5.5 0 0 0 0-.708l-3-3a.5.5 0 1 0-.708.708L10.293 7.5H4.5z"/>'),eh=Xu("ArrowRightSquareFill",'<path d="M0 14a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v12zm4.5-6.5h5.793L8.146 5.354a.5.5 0 1 1 .708-.708l3 3a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708-.708L10.293 8.5H4.5a.5.5 0 0 1 0-1z"/>'),th=Xu("ArrowUp",'<path fill-rule="evenodd" d="M8 15a.5.5 0 0 0 .5-.5V2.707l3.146 3.147a.5.5 0 0 0 .708-.708l-4-4a.5.5 0 0 0-.708 0l-4 4a.5.5 0 1 0 .708.708L7.5 2.707V14.5a.5.5 0 0 0 .5.5z"/>'),ah=Xu("ArrowUpCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-7.5 3.5a.5.5 0 0 1-1 0V5.707L5.354 7.854a.5.5 0 1 1-.708-.708l3-3a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 5.707V11.5z"/>'),nh=Xu("ArrowUpCircleFill",'<path d="M16 8A8 8 0 1 0 0 8a8 8 0 0 0 16 0zm-7.5 3.5a.5.5 0 0 1-1 0V5.707L5.354 7.854a.5.5 0 1 1-.708-.708l3-3a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 5.707V11.5z"/>'),rh=Xu("ArrowUpLeft",'<path fill-rule="evenodd" d="M2 2.5a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1H3.707l10.147 10.146a.5.5 0 0 1-.708.708L3 3.707V8.5a.5.5 0 0 1-1 0v-6z"/>'),ih=Xu("ArrowUpLeftCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-5.904 2.803a.5.5 0 1 0 .707-.707L6.707 6h2.768a.5.5 0 1 0 0-1H5.5a.5.5 0 0 0-.5.5v3.975a.5.5 0 0 0 1 0V6.707l4.096 4.096z"/>'),oh=Xu("ArrowUpLeftCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-5.904 2.803a.5.5 0 1 0 .707-.707L6.707 6h2.768a.5.5 0 1 0 0-1H5.5a.5.5 0 0 0-.5.5v3.975a.5.5 0 0 0 1 0V6.707l4.096 4.096z"/>'),sh=Xu("ArrowUpLeftSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm10.096 8.803a.5.5 0 1 0 .707-.707L6.707 6h2.768a.5.5 0 1 0 0-1H5.5a.5.5 0 0 0-.5.5v3.975a.5.5 0 0 0 1 0V6.707l4.096 4.096z"/>'),lh=Xu("ArrowUpLeftSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm8.096 10.803L6 6.707v2.768a.5.5 0 0 1-1 0V5.5a.5.5 0 0 1 .5-.5h3.975a.5.5 0 1 1 0 1H6.707l4.096 4.096a.5.5 0 1 1-.707.707z"/>'),ch=Xu("ArrowUpRight",'<path fill-rule="evenodd" d="M14 2.5a.5.5 0 0 0-.5-.5h-6a.5.5 0 0 0 0 1h4.793L2.146 13.146a.5.5 0 0 0 .708.708L13 3.707V8.5a.5.5 0 0 0 1 0v-6z"/>'),uh=Xu("ArrowUpRightCircle",'<path fill-rule="evenodd" d="M1 8a7 7 0 1 0 14 0A7 7 0 0 0 1 8zm15 0A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM5.854 10.803a.5.5 0 1 1-.708-.707L9.243 6H6.475a.5.5 0 1 1 0-1h3.975a.5.5 0 0 1 .5.5v3.975a.5.5 0 1 1-1 0V6.707l-4.096 4.096z"/>'),dh=Xu("ArrowUpRightCircleFill",'<path d="M0 8a8 8 0 1 0 16 0A8 8 0 0 0 0 8zm5.904 2.803a.5.5 0 1 1-.707-.707L9.293 6H6.525a.5.5 0 1 1 0-1H10.5a.5.5 0 0 1 .5.5v3.975a.5.5 0 0 1-1 0V6.707l-4.096 4.096z"/>'),hh=Xu("ArrowUpRightSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm5.854 8.803a.5.5 0 1 1-.708-.707L9.243 6H6.475a.5.5 0 1 1 0-1h3.975a.5.5 0 0 1 .5.5v3.975a.5.5 0 1 1-1 0V6.707l-4.096 4.096z"/>'),fh=Xu("ArrowUpRightSquareFill",'<path d="M14 0a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12zM5.904 10.803 10 6.707v2.768a.5.5 0 0 0 1 0V5.5a.5.5 0 0 0-.5-.5H6.525a.5.5 0 1 0 0 1h2.768l-4.096 4.096a.5.5 0 0 0 .707.707z"/>'),ph=Xu("ArrowUpShort",'<path fill-rule="evenodd" d="M8 12a.5.5 0 0 0 .5-.5V5.707l2.146 2.147a.5.5 0 0 0 .708-.708l-3-3a.5.5 0 0 0-.708 0l-3 3a.5.5 0 1 0 .708.708L7.5 5.707V11.5a.5.5 0 0 0 .5.5z"/>'),mh=Xu("ArrowUpSquare",'<path fill-rule="evenodd" d="M15 2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2zM0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm8.5 9.5a.5.5 0 0 1-1 0V5.707L5.354 7.854a.5.5 0 1 1-.708-.708l3-3a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 5.707V11.5z"/>'),vh=Xu("ArrowUpSquareFill",'<path d="M2 16a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2zm6.5-4.5V5.707l2.146 2.147a.5.5 0 0 0 .708-.708l-3-3a.5.5 0 0 0-.708 0l-3 3a.5.5 0 1 0 .708.708L7.5 5.707V11.5a.5.5 0 0 0 1 0z"/>'),_h=Xu("ArrowsAngleContract",'<path fill-rule="evenodd" d="M.172 15.828a.5.5 0 0 0 .707 0l4.096-4.096V14.5a.5.5 0 1 0 1 0v-3.975a.5.5 0 0 0-.5-.5H1.5a.5.5 0 0 0 0 1h2.768L.172 15.121a.5.5 0 0 0 0 .707zM15.828.172a.5.5 0 0 0-.707 0l-4.096 4.096V1.5a.5.5 0 1 0-1 0v3.975a.5.5 0 0 0 .5.5H14.5a.5.5 0 0 0 0-1h-2.768L15.828.879a.5.5 0 0 0 0-.707z"/>'),gh=Xu("ArrowsAngleExpand",'<path fill-rule="evenodd" d="M5.828 10.172a.5.5 0 0 0-.707 0l-4.096 4.096V11.5a.5.5 0 0 0-1 0v3.975a.5.5 0 0 0 .5.5H4.5a.5.5 0 0 0 0-1H1.732l4.096-4.096a.5.5 0 0 0 0-.707zm4.344-4.344a.5.5 0 0 0 .707 0l4.096-4.096V4.5a.5.5 0 1 0 1 0V.525a.5.5 0 0 0-.5-.5H11.5a.5.5 0 0 0 0 1h2.768l-4.096 4.096a.5.5 0 0 0 0 .707z"/>'),yh=Xu("ArrowsCollapse",'<path fill-rule="evenodd" d="M1 8a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13A.5.5 0 0 1 1 8zm7-8a.5.5 0 0 1 .5.5v3.793l1.146-1.147a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 1 1 .708-.708L7.5 4.293V.5A.5.5 0 0 1 8 0zm-.5 11.707-1.146 1.147a.5.5 0 0 1-.708-.708l2-2a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1-.708.708L8.5 11.707V15.5a.5.5 0 0 1-1 0v-3.793z"/>'),bh=Xu("ArrowsExpand",'<path fill-rule="evenodd" d="M1 8a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13A.5.5 0 0 1 1 8zM7.646.146a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1-.708.708L8.5 1.707V5.5a.5.5 0 0 1-1 0V1.707L6.354 2.854a.5.5 0 1 1-.708-.708l2-2zM8 10a.5.5 0 0 1 .5.5v3.793l1.146-1.147a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 0 1 .708-.708L7.5 14.293V10.5A.5.5 0 0 1 8 10z"/>'),Mh=Xu("ArrowsFullscreen",'<path fill-rule="evenodd" d="M5.828 10.172a.5.5 0 0 0-.707 0l-4.096 4.096V11.5a.5.5 0 0 0-1 0v3.975a.5.5 0 0 0 .5.5H4.5a.5.5 0 0 0 0-1H1.732l4.096-4.096a.5.5 0 0 0 0-.707zm4.344 0a.5.5 0 0 1 .707 0l4.096 4.096V11.5a.5.5 0 1 1 1 0v3.975a.5.5 0 0 1-.5.5H11.5a.5.5 0 0 1 0-1h2.768l-4.096-4.096a.5.5 0 0 1 0-.707zm0-4.344a.5.5 0 0 0 .707 0l4.096-4.096V4.5a.5.5 0 1 0 1 0V.525a.5.5 0 0 0-.5-.5H11.5a.5.5 0 0 0 0 1h2.768l-4.096 4.096a.5.5 0 0 0 0 .707zm-4.344 0a.5.5 0 0 1-.707 0L1.025 1.732V4.5a.5.5 0 0 1-1 0V.525a.5.5 0 0 1 .5-.5H4.5a.5.5 0 0 1 0 1H1.732l4.096 4.096a.5.5 0 0 1 0 .707z"/>'),zh=Xu("ArrowsMove",'<path fill-rule="evenodd" d="M7.646.146a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1-.708.708L8.5 1.707V5.5a.5.5 0 0 1-1 0V1.707L6.354 2.854a.5.5 0 1 1-.708-.708l2-2zM8 10a.5.5 0 0 1 .5.5v3.793l1.146-1.147a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 0 1 .708-.708L7.5 14.293V10.5A.5.5 0 0 1 8 10zM.146 8.354a.5.5 0 0 1 0-.708l2-2a.5.5 0 1 1 .708.708L1.707 7.5H5.5a.5.5 0 0 1 0 1H1.707l1.147 1.146a.5.5 0 0 1-.708.708l-2-2zM10 8a.5.5 0 0 1 .5-.5h3.793l-1.147-1.146a.5.5 0 0 1 .708-.708l2 2a.5.5 0 0 1 0 .708l-2 2a.5.5 0 0 1-.708-.708L14.293 8.5H10.5A.5.5 0 0 1 10 8z"/>'),Lh=Xu("AspectRatio",'<path d="M0 3.5A1.5 1.5 0 0 1 1.5 2h13A1.5 1.5 0 0 1 16 3.5v9a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 12.5v-9zM1.5 3a.5.5 0 0 0-.5.5v9a.5.5 0 0 0 .5.5h13a.5.5 0 0 0 .5-.5v-9a.5.5 0 0 0-.5-.5h-13z"/><path d="M2 4.5a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1H3v2.5a.5.5 0 0 1-1 0v-3zm12 7a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1 0-1H13V8.5a.5.5 0 0 1 1 0v3z"/>'),wh=Xu("AspectRatioFill",'<path d="M0 12.5v-9A1.5 1.5 0 0 1 1.5 2h13A1.5 1.5 0 0 1 16 3.5v9a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 12.5zM2.5 4a.5.5 0 0 0-.5.5v3a.5.5 0 0 0 1 0V5h2.5a.5.5 0 0 0 0-1h-3zm11 8a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 0-1 0V11h-2.5a.5.5 0 0 0 0 1h3z"/>'),Hh=Xu("Asterisk",'<path d="M8 0a1 1 0 0 1 1 1v5.268l4.562-2.634a1 1 0 1 1 1 1.732L10 8l4.562 2.634a1 1 0 1 1-1 1.732L9 9.732V15a1 1 0 1 1-2 0V9.732l-4.562 2.634a1 1 0 1 1-1-1.732L6 8 1.438 5.366a1 1 0 0 1 1-1.732L7 6.268V1a1 1 0 0 1 1-1z"/>'),kh=Xu("At",'<path d="M13.106 7.222c0-2.967-2.249-5.032-5.482-5.032-3.35 0-5.646 2.318-5.646 5.702 0 3.493 2.235 5.708 5.762 5.708.862 0 1.689-.123 2.304-.335v-.862c-.43.199-1.354.328-2.29.328-2.926 0-4.813-1.88-4.813-4.798 0-2.844 1.921-4.881 4.594-4.881 2.735 0 4.608 1.688 4.608 4.156 0 1.682-.554 2.769-1.416 2.769-.492 0-.772-.28-.772-.76V5.206H8.923v.834h-.11c-.266-.595-.881-.964-1.6-.964-1.4 0-2.378 1.162-2.378 2.823 0 1.737.957 2.906 2.379 2.906.8 0 1.415-.39 1.709-1.087h.11c.081.67.703 1.148 1.503 1.148 1.572 0 2.57-1.415 2.57-3.643zm-7.177.704c0-1.197.54-1.907 1.456-1.907.93 0 1.524.738 1.524 1.907S8.308 9.84 7.371 9.84c-.895 0-1.442-.725-1.442-1.914z"/>'),Ah=Xu("Award",'<path d="M9.669.864 8 0 6.331.864l-1.858.282-.842 1.68-1.337 1.32L2.6 6l-.306 1.854 1.337 1.32.842 1.68 1.858.282L8 12l1.669-.864 1.858-.282.842-1.68 1.337-1.32L13.4 6l.306-1.854-1.337-1.32-.842-1.68L9.669.864zm1.196 1.193.684 1.365 1.086 1.072L12.387 6l.248 1.506-1.086 1.072-.684 1.365-1.51.229L8 10.874l-1.355-.702-1.51-.229-.684-1.365-1.086-1.072L3.614 6l-.25-1.506 1.087-1.072.684-1.365 1.51-.229L8 1.126l1.356.702 1.509.229z"/><path d="M4 11.794V16l4-1 4 1v-4.206l-2.018.306L8 13.126 6.018 12.1 4 11.794z"/>'),Sh=Xu("AwardFill",'<path d="m8 0 1.669.864 1.858.282.842 1.68 1.337 1.32L13.4 6l.306 1.854-1.337 1.32-.842 1.68-1.858.282L8 12l-1.669-.864-1.858-.282-.842-1.68-1.337-1.32L2.6 6l-.306-1.854 1.337-1.32.842-1.68L6.331.864 8 0z"/><path d="M4 11.794V16l4-1 4 1v-4.206l-2.018.306L8 13.126 6.018 12.1 4 11.794z"/>'),Vh=Xu("Back",'<path d="M0 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2h2a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2v-2H2a2 2 0 0 1-2-2V2zm2-1a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H2z"/>'),Oh=Xu("Backspace",'<path d="M5.83 5.146a.5.5 0 0 0 0 .708L7.975 8l-2.147 2.146a.5.5 0 0 0 .707.708l2.147-2.147 2.146 2.147a.5.5 0 0 0 .707-.708L9.39 8l2.146-2.146a.5.5 0 0 0-.707-.708L8.683 7.293 6.536 5.146a.5.5 0 0 0-.707 0z"/><path d="M13.683 1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2h-7.08a2 2 0 0 1-1.519-.698L.241 8.65a1 1 0 0 1 0-1.302L5.084 1.7A2 2 0 0 1 6.603 1h7.08zm-7.08 1a1 1 0 0 0-.76.35L1 8l4.844 5.65a1 1 0 0 0 .759.35h7.08a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1h-7.08z"/>'),Th=Xu("BackspaceFill",'<path d="M15.683 3a2 2 0 0 0-2-2h-7.08a2 2 0 0 0-1.519.698L.241 7.35a1 1 0 0 0 0 1.302l4.843 5.65A2 2 0 0 0 6.603 15h7.08a2 2 0 0 0 2-2V3zM5.829 5.854a.5.5 0 1 1 .707-.708l2.147 2.147 2.146-2.147a.5.5 0 1 1 .707.708L9.39 8l2.146 2.146a.5.5 0 0 1-.707.708L8.683 8.707l-2.147 2.147a.5.5 0 0 1-.707-.708L7.976 8 5.829 5.854z"/>'),Dh=Xu("BackspaceReverse",'<path d="M9.854 5.146a.5.5 0 0 1 0 .708L7.707 8l2.147 2.146a.5.5 0 0 1-.708.708L7 8.707l-2.146 2.147a.5.5 0 0 1-.708-.708L6.293 8 4.146 5.854a.5.5 0 1 1 .708-.708L7 7.293l2.146-2.147a.5.5 0 0 1 .708 0z"/><path d="M2 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h7.08a2 2 0 0 0 1.519-.698l4.843-5.651a1 1 0 0 0 0-1.302L10.6 1.7A2 2 0 0 0 9.08 1H2zm7.08 1a1 1 0 0 1 .76.35L14.682 8l-4.844 5.65a1 1 0 0 1-.759.35H2a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1h7.08z"/>'),Ch=Xu("BackspaceReverseFill",'<path d="M0 3a2 2 0 0 1 2-2h7.08a2 2 0 0 1 1.519.698l4.843 5.651a1 1 0 0 1 0 1.302L10.6 14.3a2 2 0 0 1-1.52.7H2a2 2 0 0 1-2-2V3zm9.854 2.854a.5.5 0 0 0-.708-.708L7 7.293 4.854 5.146a.5.5 0 1 0-.708.708L6.293 8l-2.147 2.146a.5.5 0 0 0 .708.708L7 8.707l2.146 2.147a.5.5 0 0 0 .708-.708L7.707 8l2.147-2.146z"/>'),jh=Xu("Badge3d",'<path d="M4.52 8.368h.664c.646 0 1.055.378 1.06.9.008.537-.427.919-1.086.919-.598-.004-1.037-.325-1.068-.756H3c.03.914.791 1.688 2.153 1.688 1.24 0 2.285-.66 2.272-1.798-.013-.953-.747-1.38-1.292-1.432v-.062c.44-.07 1.125-.527 1.108-1.375-.013-.906-.8-1.57-2.053-1.565-1.31.005-2.043.734-2.074 1.67h1.103c.022-.391.383-.751.936-.751.532 0 .928.33.928.813.004.479-.383.835-.928.835h-.632v.914zm3.606-3.367V11h2.189C12.125 11 13 9.893 13 7.985c0-1.894-.861-2.984-2.685-2.984H8.126zm1.187.967h.844c1.112 0 1.621.686 1.621 2.04 0 1.353-.505 2.02-1.621 2.02h-.844v-4.06z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),xh=Xu("Badge3dFill",'<path d="M10.157 5.968h-.844v4.06h.844c1.116 0 1.621-.667 1.621-2.02 0-1.354-.51-2.04-1.621-2.04z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm5.184 4.368c.646 0 1.055.378 1.06.9.008.537-.427.919-1.086.919-.598-.004-1.037-.325-1.068-.756H3c.03.914.791 1.688 2.153 1.688 1.24 0 2.285-.66 2.272-1.798-.013-.953-.747-1.38-1.292-1.432v-.062c.44-.07 1.125-.527 1.108-1.375-.013-.906-.8-1.57-2.053-1.565-1.31.005-2.043.734-2.074 1.67h1.103c.022-.391.383-.751.936-.751.532 0 .928.33.928.813.004.479-.383.835-.928.835h-.632v.914h.663zM8.126 11h2.189C12.125 11 13 9.893 13 7.985c0-1.894-.861-2.984-2.685-2.984H8.126V11z"/>'),Bh=Xu("Badge4k",'<path d="M4.807 5.001C4.021 6.298 3.203 7.6 2.5 8.917v.971h2.905V11h1.112V9.888h.733V8.93h-.733V5.001h-1.71zm-1.23 3.93v-.032a46.781 46.781 0 0 1 1.766-3.001h.062V8.93H3.577zm9.831-3.93h-1.306L9.835 7.687h-.057V5H8.59v6h1.187V9.075l.615-.699L12.072 11H13.5l-2.232-3.415 2.14-2.584z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Yh=Xu("Badge4kFill",'<path d="M3.577 8.9v.03h1.828V5.898h-.062a46.781 46.781 0 0 0-1.766 3.001z"/><path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm2.372 3.715.435-.714h1.71v3.93h.733v.957h-.733V11H5.405V9.888H2.5v-.971c.574-1.077 1.225-2.142 1.872-3.202zm7.73-.714h1.306l-2.14 2.584L13.5 11h-1.428l-1.679-2.624-.615.7V11H8.59V5.001h1.187v2.686h.057L12.102 5z"/>'),Ph=Xu("Badge8k",'<path d="M4.837 11.114c1.406 0 2.333-.725 2.333-1.766 0-.945-.712-1.38-1.256-1.49v-.053c.496-.15 1.02-.55 1.02-1.331 0-.914-.831-1.587-2.084-1.587-1.257 0-2.087.673-2.087 1.587 0 .773.51 1.177 1.02 1.331v.053c-.546.11-1.258.54-1.258 1.494 0 1.042.906 1.762 2.312 1.762zm.013-3.643c-.545 0-.95-.356-.95-.866s.405-.852.95-.852c.545 0 .945.343.945.852 0 .51-.4.866-.945.866zm0 2.786c-.65 0-1.142-.395-1.142-.984S4.2 8.28 4.85 8.28c.646 0 1.143.404 1.143.993s-.497.984-1.143.984zM13.408 5h-1.306L9.835 7.685h-.057V5H8.59v5.998h1.187V9.075l.615-.699 1.679 2.623H13.5l-2.232-3.414L13.408 5z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Ih=Xu("Badge8kFill",'<path d="M3.9 6.605c0 .51.405.866.95.866.545 0 .945-.356.945-.866s-.4-.852-.945-.852c-.545 0-.95.343-.95.852zm-.192 2.668c0 .589.492.984 1.142.984.646 0 1.143-.395 1.143-.984S5.496 8.28 4.85 8.28c-.65 0-1.142.404-1.142.993z"/><path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm5.17 7.348c0 1.041-.927 1.766-2.333 1.766-1.406 0-2.312-.72-2.312-1.762 0-.954.712-1.384 1.257-1.494v-.053c-.51-.154-1.02-.558-1.02-1.331 0-.914.831-1.587 2.088-1.587 1.253 0 2.083.673 2.083 1.587 0 .782-.523 1.182-1.02 1.331v.053c.545.11 1.257.545 1.257 1.49zM12.102 5h1.306l-2.14 2.584 2.232 3.415h-1.428l-1.679-2.624-.615.699v1.925H8.59V5h1.187v2.685h.057L12.102 5z"/>'),Fh=Xu("BadgeAd",'<path d="m3.7 11 .47-1.542h2.004L6.644 11h1.261L5.901 5.001H4.513L2.5 11h1.2zm1.503-4.852.734 2.426H4.416l.734-2.426h.053zm4.759.128c-1.059 0-1.753.765-1.753 2.043v.695c0 1.279.685 2.043 1.74 2.043.677 0 1.222-.33 1.367-.804h.057V11h1.138V4.685h-1.16v2.36h-.053c-.18-.475-.68-.77-1.336-.77zm.387.923c.58 0 1.002.44 1.002 1.138v.602c0 .76-.396 1.2-.984 1.2-.598 0-.972-.449-.972-1.248v-.453c0-.795.37-1.24.954-1.24z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Eh=Xu("BadgeAdFill",'<path d="M11.35 8.337c0-.699-.42-1.138-1.001-1.138-.584 0-.954.444-.954 1.239v.453c0 .8.374 1.248.972 1.248.588 0 .984-.44.984-1.2v-.602zm-5.413.237-.734-2.426H5.15l-.734 2.426h1.52z"/><path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm6.209 6.32c0-1.28.694-2.044 1.753-2.044.655 0 1.156.294 1.336.769h.053v-2.36h1.16V11h-1.138v-.747h-.057c-.145.474-.69.804-1.367.804-1.055 0-1.74-.764-1.74-2.043v-.695zm-4.04 1.138L3.7 11H2.5l2.013-5.999H5.9L7.905 11H6.644l-.47-1.542H4.17z"/>'),$h=Xu("BadgeAr",'<path d="m3.794 11 .47-1.542H6.27L6.739 11H8L5.996 5.001H4.607L2.595 11h1.2zm1.503-4.852.734 2.426h-1.52l.734-2.426h.052zm5.598-1.147H8.5V11h1.173V8.763h1.064L11.787 11h1.327L11.91 8.583C12.455 8.373 13 7.779 13 6.9c0-1.147-.773-1.9-2.105-1.9zm-1.222 2.87V5.933h1.05c.63 0 1.05.347 1.05.989 0 .633-.408.95-1.067.95H9.673z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Rh=Xu("BadgeArFill",'<path d="m6.031 8.574-.734-2.426h-.052L4.51 8.574h1.52zm3.642-2.641v1.938h1.033c.66 0 1.068-.316 1.068-.95 0-.64-.422-.988-1.05-.988h-1.05z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm4.265 5.458h2.004L6.739 11H8L5.996 5.001H4.607L2.595 11h1.2l.47-1.542zM8.5 5v6h1.173V8.763h1.064L11.787 11h1.327L11.91 8.583C12.455 8.373 13 7.779 13 6.9c0-1.147-.773-1.9-2.105-1.9H8.5z"/>'),Nh=Xu("BadgeCc",'<path d="M3.708 7.755c0-1.111.488-1.753 1.319-1.753.681 0 1.138.47 1.186 1.107H7.36V7c-.052-1.186-1.024-2-2.342-2C3.414 5 2.5 6.05 2.5 7.751v.747c0 1.7.905 2.73 2.518 2.73 1.314 0 2.285-.792 2.342-1.939v-.114H6.213c-.048.615-.496 1.05-1.186 1.05-.84 0-1.319-.62-1.319-1.727v-.743zm6.14 0c0-1.111.488-1.753 1.318-1.753.682 0 1.139.47 1.187 1.107H13.5V7c-.053-1.186-1.024-2-2.342-2C9.554 5 8.64 6.05 8.64 7.751v.747c0 1.7.905 2.73 2.518 2.73 1.314 0 2.285-.792 2.342-1.939v-.114h-1.147c-.048.615-.497 1.05-1.187 1.05-.839 0-1.318-.62-1.318-1.727v-.743z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Wh=Xu("BadgeCcFill",'<path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm3.027 4.002c-.83 0-1.319.642-1.319 1.753v.743c0 1.107.48 1.727 1.319 1.727.69 0 1.138-.435 1.186-1.05H7.36v.114c-.057 1.147-1.028 1.938-2.342 1.938-1.613 0-2.518-1.028-2.518-2.729v-.747C2.5 6.051 3.414 5 5.018 5c1.318 0 2.29.813 2.342 2v.11H6.213c-.048-.638-.505-1.108-1.186-1.108zm6.14 0c-.831 0-1.319.642-1.319 1.753v.743c0 1.107.48 1.727 1.318 1.727.69 0 1.139-.435 1.187-1.05H13.5v.114c-.057 1.147-1.028 1.938-2.342 1.938-1.613 0-2.518-1.028-2.518-2.729v-.747c0-1.7.914-2.751 2.518-2.751 1.318 0 2.29.813 2.342 2v.11h-1.147c-.048-.638-.505-1.108-1.187-1.108z"/>'),Uh=Xu("BadgeHd",'<path d="M7.396 11V5.001H6.209v2.44H3.687V5H2.5v6h1.187V8.43h2.522V11h1.187zM8.5 5.001V11h2.188c1.811 0 2.685-1.107 2.685-3.015 0-1.894-.86-2.984-2.684-2.984H8.5zm1.187.967h.843c1.112 0 1.622.686 1.622 2.04 0 1.353-.505 2.02-1.622 2.02h-.843v-4.06z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Gh=Xu("BadgeHdFill",'<path d="M10.53 5.968h-.843v4.06h.843c1.117 0 1.622-.667 1.622-2.02 0-1.354-.51-2.04-1.622-2.04z"/><path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm5.396 3.001V11H6.209V8.43H3.687V11H2.5V5.001h1.187v2.44h2.522V5h1.187zM8.5 11V5.001h2.188c1.824 0 2.685 1.09 2.685 2.984C13.373 9.893 12.5 11 10.69 11H8.5z"/>'),qh=Xu("BadgeTm",'<path d="M5.295 11V5.995H7V5H2.403v.994h1.701V11h1.19zm3.397 0V7.01h.058l1.428 3.239h.773l1.42-3.24h.057V11H13.5V5.001h-1.2l-1.71 3.894h-.039l-1.71-3.894H7.634V11h1.06z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Jh=Xu("BadgeTmFill",'<path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm3.295 3.995V11H4.104V5.995h-1.7V5H7v.994H5.295zM8.692 7.01V11H7.633V5.001h1.209l1.71 3.894h.039l1.71-3.894H13.5V11h-1.072V7.01h-.057l-1.42 3.239h-.773L8.75 7.008h-.058z"/>'),Kh=Xu("BadgeVo",'<path d="M4.508 11h1.429l1.99-5.999H6.61L5.277 9.708H5.22L3.875 5.001H2.5L4.508 11zM13.5 8.39v-.77c0-1.696-.962-2.733-2.566-2.733-1.604 0-2.571 1.029-2.571 2.734v.769c0 1.691.967 2.724 2.57 2.724 1.605 0 2.567-1.033 2.567-2.724zm-1.204-.778v.782c0 1.156-.571 1.732-1.362 1.732-.796 0-1.363-.576-1.363-1.732v-.782c0-1.156.567-1.736 1.363-1.736.79 0 1.362.58 1.362 1.736z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),Xh=Xu("BadgeVoFill",'<path d="M12.296 8.394v-.782c0-1.156-.571-1.736-1.362-1.736-.796 0-1.363.58-1.363 1.736v.782c0 1.156.567 1.732 1.363 1.732.79 0 1.362-.576 1.362-1.732z"/><path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm11.5 5.62v.77c0 1.691-.962 2.724-2.566 2.724-1.604 0-2.571-1.033-2.571-2.724v-.77c0-1.704.967-2.733 2.57-2.733 1.605 0 2.567 1.037 2.567 2.734zM5.937 11H4.508L2.5 5.001h1.375L5.22 9.708h.057L6.61 5.001h1.318L5.937 11z"/>'),Zh=Xu("BadgeVr",'<path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/><path d="M4.508 11h1.429l1.99-5.999H6.61L5.277 9.708H5.22L3.875 5.001H2.5L4.508 11zm6.387-5.999H8.5V11h1.173V8.763h1.064L11.787 11h1.327L11.91 8.583C12.455 8.373 13 7.779 13 6.9c0-1.147-.773-1.9-2.105-1.9zm-1.222 2.87V5.933h1.05c.63 0 1.05.347 1.05.989 0 .633-.408.95-1.067.95H9.673z"/>'),Qh=Xu("BadgeVrFill",'<path d="M9.673 5.933v1.938h1.033c.66 0 1.068-.316 1.068-.95 0-.64-.422-.988-1.05-.988h-1.05z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm5.937 7 1.99-5.999H6.61L5.277 9.708H5.22L3.875 5.001H2.5L4.508 11h1.429zM8.5 5.001V11h1.173V8.763h1.064L11.787 11h1.327L11.91 8.583C12.455 8.373 13 7.779 13 6.9c0-1.147-.773-1.9-2.105-1.9H8.5z"/>'),ef=Xu("BadgeWc",'<path d="M10.348 7.643c0-1.112.488-1.754 1.318-1.754.682 0 1.139.47 1.187 1.108H14v-.11c-.053-1.187-1.024-2-2.342-2-1.604 0-2.518 1.05-2.518 2.751v.747c0 1.7.905 2.73 2.518 2.73 1.314 0 2.285-.792 2.342-1.939v-.114h-1.147c-.048.615-.497 1.05-1.187 1.05-.839 0-1.318-.62-1.318-1.727v-.742zM4.457 11l1.02-4.184h.045L6.542 11h1.006L9 5.001H7.818l-.82 4.355h-.056L5.97 5.001h-.94l-.972 4.355h-.053l-.827-4.355H2L3.452 11h1.005z"/><path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/>'),tf=Xu("BadgeWcFill",'<path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm11.666 1.89c.682 0 1.139.47 1.187 1.107H14v-.11c-.053-1.187-1.024-2-2.342-2-1.604 0-2.518 1.05-2.518 2.751v.747c0 1.7.905 2.73 2.518 2.73 1.314 0 2.285-.792 2.342-1.939v-.114h-1.147c-.048.615-.497 1.05-1.187 1.05-.839 0-1.318-.62-1.318-1.727v-.742c0-1.112.488-1.754 1.318-1.754zm-6.188.926h.044L6.542 11h1.006L9 5.001H7.818l-.82 4.355h-.056L5.97 5.001h-.94l-.972 4.355h-.053l-.827-4.355H2L3.452 11h1.005l1.02-4.184z"/>'),af=Xu("Bag",'<path d="M8 1a2.5 2.5 0 0 1 2.5 2.5V4h-5v-.5A2.5 2.5 0 0 1 8 1zm3.5 3v-.5a3.5 3.5 0 1 0-7 0V4H1v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V4h-3.5zM2 5h12v9a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V5z"/>'),nf=Xu("BagCheck",'<path fill-rule="evenodd" d="M10.854 8.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 0 1 .708-.708L7.5 10.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M8 1a2.5 2.5 0 0 1 2.5 2.5V4h-5v-.5A2.5 2.5 0 0 1 8 1zm3.5 3v-.5a3.5 3.5 0 1 0-7 0V4H1v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V4h-3.5zM2 5h12v9a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V5z"/>'),rf=Xu("BagCheckFill",'<path fill-rule="evenodd" d="M10.5 3.5a2.5 2.5 0 0 0-5 0V4h5v-.5zm1 0V4H15v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V4h3.5v-.5a3.5 3.5 0 1 1 7 0zm-.646 5.354a.5.5 0 0 0-.708-.708L7.5 10.793 6.354 9.646a.5.5 0 1 0-.708.708l1.5 1.5a.5.5 0 0 0 .708 0l3-3z"/>'),of=Xu("BagDash",'<path fill-rule="evenodd" d="M5.5 10a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M8 1a2.5 2.5 0 0 1 2.5 2.5V4h-5v-.5A2.5 2.5 0 0 1 8 1zm3.5 3v-.5a3.5 3.5 0 1 0-7 0V4H1v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V4h-3.5zM2 5h12v9a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V5z"/>'),sf=Xu("BagDashFill",'<path fill-rule="evenodd" d="M10.5 3.5a2.5 2.5 0 0 0-5 0V4h5v-.5zm1 0V4H15v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V4h3.5v-.5a3.5 3.5 0 1 1 7 0zM6 9.5a.5.5 0 0 0 0 1h4a.5.5 0 0 0 0-1H6z"/>'),lf=Xu("BagFill",'<path d="M8 1a2.5 2.5 0 0 1 2.5 2.5V4h-5v-.5A2.5 2.5 0 0 1 8 1zm3.5 3v-.5a3.5 3.5 0 1 0-7 0V4H1v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V4h-3.5z"/>'),cf=Xu("BagPlus",'<path fill-rule="evenodd" d="M8 7.5a.5.5 0 0 1 .5.5v1.5H10a.5.5 0 0 1 0 1H8.5V12a.5.5 0 0 1-1 0v-1.5H6a.5.5 0 0 1 0-1h1.5V8a.5.5 0 0 1 .5-.5z"/><path d="M8 1a2.5 2.5 0 0 1 2.5 2.5V4h-5v-.5A2.5 2.5 0 0 1 8 1zm3.5 3v-.5a3.5 3.5 0 1 0-7 0V4H1v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V4h-3.5zM2 5h12v9a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V5z"/>'),uf=Xu("BagPlusFill",'<path fill-rule="evenodd" d="M10.5 3.5a2.5 2.5 0 0 0-5 0V4h5v-.5zm1 0V4H15v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V4h3.5v-.5a3.5 3.5 0 1 1 7 0zM8.5 8a.5.5 0 0 0-1 0v1.5H6a.5.5 0 0 0 0 1h1.5V12a.5.5 0 0 0 1 0v-1.5H10a.5.5 0 0 0 0-1H8.5V8z"/>'),df=Xu("BagX",'<path fill-rule="evenodd" d="M6.146 8.146a.5.5 0 0 1 .708 0L8 9.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 10l1.147 1.146a.5.5 0 0 1-.708.708L8 10.707l-1.146 1.147a.5.5 0 0 1-.708-.708L7.293 10 6.146 8.854a.5.5 0 0 1 0-.708z"/><path d="M8 1a2.5 2.5 0 0 1 2.5 2.5V4h-5v-.5A2.5 2.5 0 0 1 8 1zm3.5 3v-.5a3.5 3.5 0 1 0-7 0V4H1v10a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V4h-3.5zM2 5h12v9a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V5z"/>'),hf=Xu("BagXFill",'<path fill-rule="evenodd" d="M10.5 3.5a2.5 2.5 0 0 0-5 0V4h5v-.5zm1 0V4H15v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V4h3.5v-.5a3.5 3.5 0 1 1 7 0zM6.854 8.146a.5.5 0 1 0-.708.708L7.293 10l-1.147 1.146a.5.5 0 0 0 .708.708L8 10.707l1.146 1.147a.5.5 0 0 0 .708-.708L8.707 10l1.147-1.146a.5.5 0 0 0-.708-.708L8 9.293 6.854 8.146z"/>'),ff=Xu("Bank",'<path d="M8 .95 14.61 4h.89a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5H15v7a.5.5 0 0 1 .485.379l.5 2A.5.5 0 0 1 15.5 17H.5a.5.5 0 0 1-.485-.621l.5-2A.5.5 0 0 1 1 14V7H.5a.5.5 0 0 1-.5-.5v-2A.5.5 0 0 1 .5 4h.89L8 .95zM3.776 4h8.447L8 2.05 3.776 4zM2 7v7h1V7H2zm2 0v7h2.5V7H4zm3.5 0v7h1V7h-1zm2 0v7H12V7H9.5zM13 7v7h1V7h-1zm2-1V5H1v1h14zm-.39 9H1.39l-.25 1h13.72l-.25-1z"/>'),pf=Xu("Bank2",'<path d="M8.277.084a.5.5 0 0 0-.554 0l-7.5 5A.5.5 0 0 0 .5 6h1.875v7H1.5a.5.5 0 0 0 0 1h13a.5.5 0 1 0 0-1h-.875V6H15.5a.5.5 0 0 0 .277-.916l-7.5-5zM12.375 6v7h-1.25V6h1.25zm-2.5 0v7h-1.25V6h1.25zm-2.5 0v7h-1.25V6h1.25zm-2.5 0v7h-1.25V6h1.25zM8 4a1 1 0 1 1 0-2 1 1 0 0 1 0 2zM.5 15a.5.5 0 0 0 0 1h15a.5.5 0 1 0 0-1H.5z"/>'),mf=Xu("BarChart",'<path d="M4 11H2v3h2v-3zm5-4H7v7h2V7zm5-5v12h-2V2h2zm-2-1a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1h-2zM6 7a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v7a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V7zm-5 4a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1v-3z"/>'),vf=Xu("BarChartFill",'<path d="M1 11a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1v-3zm5-4a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v7a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V7zm5-5a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1V2z"/>'),_f=Xu("BarChartLine",'<path d="M11 2a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v12h.5a.5.5 0 0 1 0 1H.5a.5.5 0 0 1 0-1H1v-3a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v3h1V7a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v7h1V2zm1 12h2V2h-2v12zm-3 0V7H7v7h2zm-5 0v-3H2v3h2z"/>'),gf=Xu("BarChartLineFill",'<path d="M11 2a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v12h.5a.5.5 0 0 1 0 1H.5a.5.5 0 0 1 0-1H1v-3a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v3h1V7a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v7h1V2z"/>'),yf=Xu("BarChartSteps",'<path d="M.5 0a.5.5 0 0 1 .5.5v15a.5.5 0 0 1-1 0V.5A.5.5 0 0 1 .5 0zM2 1.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-4a.5.5 0 0 1-.5-.5v-1zm2 4a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-1zm2 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-6a.5.5 0 0 1-.5-.5v-1zm2 4a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-1z"/>'),bf=Xu("Basket",'<path d="M5.757 1.071a.5.5 0 0 1 .172.686L3.383 6h9.234L10.07 1.757a.5.5 0 1 1 .858-.514L13.783 6H15a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1v4.5a2.5 2.5 0 0 1-2.5 2.5h-9A2.5 2.5 0 0 1 1 13.5V9a1 1 0 0 1-1-1V7a1 1 0 0 1 1-1h1.217L5.07 1.243a.5.5 0 0 1 .686-.172zM2 9v4.5A1.5 1.5 0 0 0 3.5 15h9a1.5 1.5 0 0 0 1.5-1.5V9H2zM1 7v1h14V7H1zm3 3a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-1 0v-3A.5.5 0 0 1 4 10zm2 0a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-1 0v-3A.5.5 0 0 1 6 10zm2 0a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-1 0v-3A.5.5 0 0 1 8 10zm2 0a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-1 0v-3a.5.5 0 0 1 .5-.5zm2 0a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-1 0v-3a.5.5 0 0 1 .5-.5z"/>'),Mf=Xu("Basket2",'<path d="M4 10a1 1 0 0 1 2 0v2a1 1 0 0 1-2 0v-2zm3 0a1 1 0 0 1 2 0v2a1 1 0 0 1-2 0v-2zm3 0a1 1 0 1 1 2 0v2a1 1 0 0 1-2 0v-2z"/><path d="M5.757 1.071a.5.5 0 0 1 .172.686L3.383 6h9.234L10.07 1.757a.5.5 0 1 1 .858-.514L13.783 6H15.5a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-.623l-1.844 6.456a.75.75 0 0 1-.722.544H3.69a.75.75 0 0 1-.722-.544L1.123 8H.5a.5.5 0 0 1-.5-.5v-1A.5.5 0 0 1 .5 6h1.717L5.07 1.243a.5.5 0 0 1 .686-.172zM2.163 8l1.714 6h8.246l1.714-6H2.163z"/>'),zf=Xu("Basket2Fill",'<path d="M5.929 1.757a.5.5 0 1 0-.858-.514L2.217 6H.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h.623l1.844 6.456A.75.75 0 0 0 3.69 15h8.622a.75.75 0 0 0 .722-.544L14.877 8h.623a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1.717L10.93 1.243a.5.5 0 1 0-.858.514L12.617 6H3.383L5.93 1.757zM4 10a1 1 0 0 1 2 0v2a1 1 0 1 1-2 0v-2zm3 0a1 1 0 0 1 2 0v2a1 1 0 1 1-2 0v-2zm4-1a1 1 0 0 1 1 1v2a1 1 0 1 1-2 0v-2a1 1 0 0 1 1-1z"/>'),Lf=Xu("Basket3",'<path d="M5.757 1.071a.5.5 0 0 1 .172.686L3.383 6h9.234L10.07 1.757a.5.5 0 1 1 .858-.514L13.783 6H15.5a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H.5a.5.5 0 0 1-.5-.5v-1A.5.5 0 0 1 .5 6h1.717L5.07 1.243a.5.5 0 0 1 .686-.172zM3.394 15l-1.48-6h-.97l1.525 6.426a.75.75 0 0 0 .729.574h9.606a.75.75 0 0 0 .73-.574L15.056 9h-.972l-1.479 6h-9.21z"/>'),wf=Xu("Basket3Fill",'<path d="M5.757 1.071a.5.5 0 0 1 .172.686L3.383 6h9.234L10.07 1.757a.5.5 0 1 1 .858-.514L13.783 6H15.5a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H.5a.5.5 0 0 1-.5-.5v-1A.5.5 0 0 1 .5 6h1.717L5.07 1.243a.5.5 0 0 1 .686-.172zM2.468 15.426.943 9h14.114l-1.525 6.426a.75.75 0 0 1-.729.574H3.197a.75.75 0 0 1-.73-.574z"/>'),Hf=Xu("BasketFill",'<path d="M5.071 1.243a.5.5 0 0 1 .858.514L3.383 6h9.234L10.07 1.757a.5.5 0 1 1 .858-.514L13.783 6H15.5a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5H15v5a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V9H.5a.5.5 0 0 1-.5-.5v-2A.5.5 0 0 1 .5 6h1.717L5.07 1.243zM3.5 10.5a.5.5 0 1 0-1 0v3a.5.5 0 0 0 1 0v-3zm2.5 0a.5.5 0 1 0-1 0v3a.5.5 0 0 0 1 0v-3zm2.5 0a.5.5 0 1 0-1 0v3a.5.5 0 0 0 1 0v-3zm2.5 0a.5.5 0 1 0-1 0v3a.5.5 0 0 0 1 0v-3zm2.5 0a.5.5 0 1 0-1 0v3a.5.5 0 0 0 1 0v-3z"/>'),kf=Xu("Battery",'<path d="M0 6a2 2 0 0 1 2-2h10a2 2 0 0 1 2 2v4a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V6zm2-1a1 1 0 0 0-1 1v4a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V6a1 1 0 0 0-1-1H2zm14 3a1.5 1.5 0 0 1-1.5 1.5v-3A1.5 1.5 0 0 1 16 8z"/>'),Af=Xu("BatteryCharging",'<path d="M9.585 2.568a.5.5 0 0 1 .226.58L8.677 6.832h1.99a.5.5 0 0 1 .364.843l-5.334 5.667a.5.5 0 0 1-.842-.49L5.99 9.167H4a.5.5 0 0 1-.364-.843l5.333-5.667a.5.5 0 0 1 .616-.09z"/><path d="M2 4h4.332l-.94 1H2a1 1 0 0 0-1 1v4a1 1 0 0 0 1 1h2.38l-.308 1H2a2 2 0 0 1-2-2V6a2 2 0 0 1 2-2z"/><path d="M2 6h2.45L2.908 7.639A1.5 1.5 0 0 0 3.313 10H2V6zm8.595-2-.308 1H12a1 1 0 0 1 1 1v4a1 1 0 0 1-1 1H9.276l-.942 1H12a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2h-1.405z"/><path d="M12 10h-1.783l1.542-1.639c.097-.103.178-.218.241-.34V10zm0-3.354V6h-.646a1.5 1.5 0 0 1 .646.646zM16 8a1.5 1.5 0 0 1-1.5 1.5v-3A1.5 1.5 0 0 1 16 8z"/>'),Sf=Xu("BatteryFull",'<path d="M2 6h10v4H2V6z"/><path d="M2 4a2 2 0 0 0-2 2v4a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2H2zm10 1a1 1 0 0 1 1 1v4a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h10zm4 3a1.5 1.5 0 0 1-1.5 1.5v-3A1.5 1.5 0 0 1 16 8z"/>'),Vf=Xu("BatteryHalf",'<path d="M2 6h5v4H2V6z"/><path d="M2 4a2 2 0 0 0-2 2v4a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2H2zm10 1a1 1 0 0 1 1 1v4a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h10zm4 3a1.5 1.5 0 0 1-1.5 1.5v-3A1.5 1.5 0 0 1 16 8z"/>'),Of=Xu("Bell",'<path d="M8 16a2 2 0 0 0 2-2H6a2 2 0 0 0 2 2zM8 1.918l-.797.161A4.002 4.002 0 0 0 4 6c0 .628-.134 2.197-.459 3.742-.16.767-.376 1.566-.663 2.258h10.244c-.287-.692-.502-1.49-.663-2.258C12.134 8.197 12 6.628 12 6a4.002 4.002 0 0 0-3.203-3.92L8 1.917zM14.22 12c.223.447.481.801.78 1H1c.299-.199.557-.553.78-1C2.68 10.2 3 6.88 3 6c0-2.42 1.72-4.44 4.005-4.901a1 1 0 1 1 1.99 0A5.002 5.002 0 0 1 13 6c0 .88.32 4.2 1.22 6z"/>'),Tf=Xu("BellFill",'<path d="M8 16a2 2 0 0 0 2-2H6a2 2 0 0 0 2 2zm.995-14.901a1 1 0 1 0-1.99 0A5.002 5.002 0 0 0 3 6c0 1.098-.5 6-2 7h14c-1.5-1-2-5.902-2-7 0-2.42-1.72-4.44-4.005-4.901z"/>'),Df=Xu("BellSlash",'<path d="M5.164 14H15c-.299-.199-.557-.553-.78-1-.9-1.8-1.22-5.12-1.22-6 0-.264-.02-.523-.06-.776l-.938.938c.02.708.157 2.154.457 3.58.161.767.377 1.566.663 2.258H6.164l-1 1zm5.581-9.91a3.986 3.986 0 0 0-1.948-1.01L8 2.917l-.797.161A4.002 4.002 0 0 0 4 7c0 .628-.134 2.197-.459 3.742-.05.238-.105.479-.166.718l-1.653 1.653c.02-.037.04-.074.059-.113C2.679 11.2 3 7.88 3 7c0-2.42 1.72-4.44 4.005-4.901a1 1 0 1 1 1.99 0c.942.19 1.788.645 2.457 1.284l-.707.707zM10 15a2 2 0 1 1-4 0h4zm-9.375.625a.53.53 0 0 0 .75.75l14.75-14.75a.53.53 0 0 0-.75-.75L.625 15.625z"/>'),Cf=Xu("BellSlashFill",'<path d="M5.164 14H15c-1.5-1-2-5.902-2-7 0-.264-.02-.523-.06-.776L5.164 14zm6.288-10.617A4.988 4.988 0 0 0 8.995 2.1a1 1 0 1 0-1.99 0A5.002 5.002 0 0 0 3 7c0 .898-.335 4.342-1.278 6.113l9.73-9.73zM10 15a2 2 0 1 1-4 0h4zm-9.375.625a.53.53 0 0 0 .75.75l14.75-14.75a.53.53 0 0 0-.75-.75L.625 15.625z"/>'),jf=Xu("Bezier",'<path fill-rule="evenodd" d="M0 10.5A1.5 1.5 0 0 1 1.5 9h1A1.5 1.5 0 0 1 4 10.5v1A1.5 1.5 0 0 1 2.5 13h-1A1.5 1.5 0 0 1 0 11.5v-1zm1.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zm10.5.5A1.5 1.5 0 0 1 13.5 9h1a1.5 1.5 0 0 1 1.5 1.5v1a1.5 1.5 0 0 1-1.5 1.5h-1a1.5 1.5 0 0 1-1.5-1.5v-1zm1.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zM6 4.5A1.5 1.5 0 0 1 7.5 3h1A1.5 1.5 0 0 1 10 4.5v1A1.5 1.5 0 0 1 8.5 7h-1A1.5 1.5 0 0 1 6 5.5v-1zM7.5 4a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1z"/><path d="M6 4.5H1.866a1 1 0 1 0 0 1h2.668A6.517 6.517 0 0 0 1.814 9H2.5c.123 0 .244.015.358.043a5.517 5.517 0 0 1 3.185-3.185A1.503 1.503 0 0 1 6 5.5v-1zm3.957 1.358A1.5 1.5 0 0 0 10 5.5v-1h4.134a1 1 0 1 1 0 1h-2.668a6.517 6.517 0 0 1 2.72 3.5H13.5c-.123 0-.243.015-.358.043a5.517 5.517 0 0 0-3.185-3.185z"/>'),xf=Xu("Bezier2",'<path fill-rule="evenodd" d="M1 2.5A1.5 1.5 0 0 1 2.5 1h1A1.5 1.5 0 0 1 5 2.5h4.134a1 1 0 1 1 0 1h-2.01c.18.18.34.381.484.605.638.992.892 2.354.892 3.895 0 1.993.257 3.092.713 3.7.356.476.895.721 1.787.784A1.5 1.5 0 0 1 12.5 11h1a1.5 1.5 0 0 1 1.5 1.5v1a1.5 1.5 0 0 1-1.5 1.5h-1a1.5 1.5 0 0 1-1.5-1.5H6.866a1 1 0 1 1 0-1h1.711a2.839 2.839 0 0 1-.165-.2C7.743 11.407 7.5 10.007 7.5 8c0-1.46-.246-2.597-.733-3.355-.39-.605-.952-1-1.767-1.112A1.5 1.5 0 0 1 3.5 5h-1A1.5 1.5 0 0 1 1 3.5v-1zM2.5 2a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zm10 10a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1z"/>'),Bf=Xu("Bicycle",'<path d="M4 4.5a.5.5 0 0 1 .5-.5H6a.5.5 0 0 1 0 1v.5h4.14l.386-1.158A.5.5 0 0 1 11 4h1a.5.5 0 0 1 0 1h-.64l-.311.935.807 1.29a3 3 0 1 1-.848.53l-.508-.812-2.076 3.322A.5.5 0 0 1 8 10.5H5.959a3 3 0 1 1-1.815-3.274L5 5.856V5h-.5a.5.5 0 0 1-.5-.5zm1.5 2.443-.508.814c.5.444.85 1.054.967 1.743h1.139L5.5 6.943zM8 9.057 9.598 6.5H6.402L8 9.057zM4.937 9.5a1.997 1.997 0 0 0-.487-.877l-.548.877h1.035zM3.603 8.092A2 2 0 1 0 4.937 10.5H3a.5.5 0 0 1-.424-.765l1.027-1.643zm7.947.53a2 2 0 1 0 .848-.53l1.026 1.643a.5.5 0 1 1-.848.53L11.55 8.623z"/>'),Yf=Xu("Binoculars",'<path d="M3 2.5A1.5 1.5 0 0 1 4.5 1h1A1.5 1.5 0 0 1 7 2.5V5h2V2.5A1.5 1.5 0 0 1 10.5 1h1A1.5 1.5 0 0 1 13 2.5v2.382a.5.5 0 0 0 .276.447l.895.447A1.5 1.5 0 0 1 15 7.118V14.5a1.5 1.5 0 0 1-1.5 1.5h-3A1.5 1.5 0 0 1 9 14.5v-3a.5.5 0 0 1 .146-.354l.854-.853V9.5a.5.5 0 0 0-.5-.5h-3a.5.5 0 0 0-.5.5v.793l.854.853A.5.5 0 0 1 7 11.5v3A1.5 1.5 0 0 1 5.5 16h-3A1.5 1.5 0 0 1 1 14.5V7.118a1.5 1.5 0 0 1 .83-1.342l.894-.447A.5.5 0 0 0 3 4.882V2.5zM4.5 2a.5.5 0 0 0-.5.5V3h2v-.5a.5.5 0 0 0-.5-.5h-1zM6 4H4v.882a1.5 1.5 0 0 1-.83 1.342l-.894.447A.5.5 0 0 0 2 7.118V13h4v-1.293l-.854-.853A.5.5 0 0 1 5 10.5v-1A1.5 1.5 0 0 1 6.5 8h3A1.5 1.5 0 0 1 11 9.5v1a.5.5 0 0 1-.146.354l-.854.853V13h4V7.118a.5.5 0 0 0-.276-.447l-.895-.447A1.5 1.5 0 0 1 12 4.882V4h-2v1.5a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5V4zm4-1h2v-.5a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5V3zm4 11h-4v.5a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5V14zm-8 0H2v.5a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5V14z"/>'),Pf=Xu("BinocularsFill",'<path d="M4.5 1A1.5 1.5 0 0 0 3 2.5V3h4v-.5A1.5 1.5 0 0 0 5.5 1h-1zM7 4v1h2V4h4v.882a.5.5 0 0 0 .276.447l.895.447A1.5 1.5 0 0 1 15 7.118V13H9v-1.5a.5.5 0 0 1 .146-.354l.854-.853V9.5a.5.5 0 0 0-.5-.5h-3a.5.5 0 0 0-.5.5v.793l.854.853A.5.5 0 0 1 7 11.5V13H1V7.118a1.5 1.5 0 0 1 .83-1.342l.894-.447A.5.5 0 0 0 3 4.882V4h4zM1 14v.5A1.5 1.5 0 0 0 2.5 16h3A1.5 1.5 0 0 0 7 14.5V14H1zm8 0v.5a1.5 1.5 0 0 0 1.5 1.5h3a1.5 1.5 0 0 0 1.5-1.5V14H9zm4-11H9v-.5A1.5 1.5 0 0 1 10.5 1h1A1.5 1.5 0 0 1 13 2.5V3z"/>'),If=Xu("BlockquoteLeft",'<path d="M2.5 3a.5.5 0 0 0 0 1h11a.5.5 0 0 0 0-1h-11zm5 3a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1h-6zm0 3a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1h-6zm-5 3a.5.5 0 0 0 0 1h11a.5.5 0 0 0 0-1h-11zm.79-5.373c.112-.078.26-.17.444-.275L3.524 6c-.122.074-.272.17-.452.287-.18.117-.35.26-.51.428a2.425 2.425 0 0 0-.398.562c-.11.207-.164.438-.164.692 0 .36.072.65.217.873.144.219.385.328.72.328.215 0 .383-.07.504-.211a.697.697 0 0 0 .188-.463c0-.23-.07-.404-.211-.521-.137-.121-.326-.182-.568-.182h-.282c.024-.203.065-.37.123-.498a1.38 1.38 0 0 1 .252-.37 1.94 1.94 0 0 1 .346-.298zm2.167 0c.113-.078.262-.17.445-.275L5.692 6c-.122.074-.272.17-.452.287-.18.117-.35.26-.51.428a2.425 2.425 0 0 0-.398.562c-.11.207-.164.438-.164.692 0 .36.072.65.217.873.144.219.385.328.72.328.215 0 .383-.07.504-.211a.697.697 0 0 0 .188-.463c0-.23-.07-.404-.211-.521-.137-.121-.326-.182-.568-.182h-.282a1.75 1.75 0 0 1 .118-.492c.058-.13.144-.254.257-.375a1.94 1.94 0 0 1 .346-.3z"/>'),Ff=Xu("BlockquoteRight",'<path d="M2.5 3a.5.5 0 0 0 0 1h11a.5.5 0 0 0 0-1h-11zm0 3a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1h-6zm0 3a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1h-6zm0 3a.5.5 0 0 0 0 1h11a.5.5 0 0 0 0-1h-11zm10.113-5.373a6.59 6.59 0 0 0-.445-.275l.21-.352c.122.074.272.17.452.287.18.117.35.26.51.428.156.164.289.351.398.562.11.207.164.438.164.692 0 .36-.072.65-.216.873-.145.219-.385.328-.721.328-.215 0-.383-.07-.504-.211a.697.697 0 0 1-.188-.463c0-.23.07-.404.211-.521.137-.121.326-.182.569-.182h.281a1.686 1.686 0 0 0-.123-.498 1.379 1.379 0 0 0-.252-.37 1.94 1.94 0 0 0-.346-.298zm-2.168 0A6.59 6.59 0 0 0 10 6.352L10.21 6c.122.074.272.17.452.287.18.117.35.26.51.428.156.164.289.351.398.562.11.207.164.438.164.692 0 .36-.072.65-.216.873-.145.219-.385.328-.721.328-.215 0-.383-.07-.504-.211a.697.697 0 0 1-.188-.463c0-.23.07-.404.211-.521.137-.121.327-.182.569-.182h.281a1.749 1.749 0 0 0-.117-.492 1.402 1.402 0 0 0-.258-.375 1.94 1.94 0 0 0-.346-.3z"/>'),Ef=Xu("Book",'<path d="M1 2.828c.885-.37 2.154-.769 3.388-.893 1.33-.134 2.458.063 3.112.752v9.746c-.935-.53-2.12-.603-3.213-.493-1.18.12-2.37.461-3.287.811V2.828zm7.5-.141c.654-.689 1.782-.886 3.112-.752 1.234.124 2.503.523 3.388.893v9.923c-.918-.35-2.107-.692-3.287-.81-1.094-.111-2.278-.039-3.213.492V2.687zM8 1.783C7.015.936 5.587.81 4.287.94c-1.514.153-3.042.672-3.994 1.105A.5.5 0 0 0 0 2.5v11a.5.5 0 0 0 .707.455c.882-.4 2.303-.881 3.68-1.02 1.409-.142 2.59.087 3.223.877a.5.5 0 0 0 .78 0c.633-.79 1.814-1.019 3.222-.877 1.378.139 2.8.62 3.681 1.02A.5.5 0 0 0 16 13.5v-11a.5.5 0 0 0-.293-.455c-.952-.433-2.48-.952-3.994-1.105C10.413.809 8.985.936 8 1.783z"/>'),$f=Xu("BookFill",'<path d="M8 1.783C7.015.936 5.587.81 4.287.94c-1.514.153-3.042.672-3.994 1.105A.5.5 0 0 0 0 2.5v11a.5.5 0 0 0 .707.455c.882-.4 2.303-.881 3.68-1.02 1.409-.142 2.59.087 3.223.877a.5.5 0 0 0 .78 0c.633-.79 1.814-1.019 3.222-.877 1.378.139 2.8.62 3.681 1.02A.5.5 0 0 0 16 13.5v-11a.5.5 0 0 0-.293-.455c-.952-.433-2.48-.952-3.994-1.105C10.413.809 8.985.936 8 1.783z"/>'),Rf=Xu("BookHalf",'<path d="M8.5 2.687c.654-.689 1.782-.886 3.112-.752 1.234.124 2.503.523 3.388.893v9.923c-.918-.35-2.107-.692-3.287-.81-1.094-.111-2.278-.039-3.213.492V2.687zM8 1.783C7.015.936 5.587.81 4.287.94c-1.514.153-3.042.672-3.994 1.105A.5.5 0 0 0 0 2.5v11a.5.5 0 0 0 .707.455c.882-.4 2.303-.881 3.68-1.02 1.409-.142 2.59.087 3.223.877a.5.5 0 0 0 .78 0c.633-.79 1.814-1.019 3.222-.877 1.378.139 2.8.62 3.681 1.02A.5.5 0 0 0 16 13.5v-11a.5.5 0 0 0-.293-.455c-.952-.433-2.48-.952-3.994-1.105C10.413.809 8.985.936 8 1.783z"/>'),Nf=Xu("Bookmark",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.777.416L8 13.101l-5.223 2.815A.5.5 0 0 1 2 15.5V2zm2-1a1 1 0 0 0-1 1v12.566l4.723-2.482a.5.5 0 0 1 .554 0L13 14.566V2a1 1 0 0 0-1-1H4z"/>'),Wf=Xu("BookmarkCheck",'<path fill-rule="evenodd" d="M10.854 5.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 7.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.777.416L8 13.101l-5.223 2.815A.5.5 0 0 1 2 15.5V2zm2-1a1 1 0 0 0-1 1v12.566l4.723-2.482a.5.5 0 0 1 .554 0L13 14.566V2a1 1 0 0 0-1-1H4z"/>'),Uf=Xu("BookmarkCheckFill",'<path fill-rule="evenodd" d="M2 15.5V2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.74.439L8 13.069l-5.26 2.87A.5.5 0 0 1 2 15.5zm8.854-9.646a.5.5 0 0 0-.708-.708L7.5 7.793 6.354 6.646a.5.5 0 1 0-.708.708l1.5 1.5a.5.5 0 0 0 .708 0l3-3z"/>'),Gf=Xu("BookmarkDash",'<path fill-rule="evenodd" d="M5.5 6.5A.5.5 0 0 1 6 6h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.777.416L8 13.101l-5.223 2.815A.5.5 0 0 1 2 15.5V2zm2-1a1 1 0 0 0-1 1v12.566l4.723-2.482a.5.5 0 0 1 .554 0L13 14.566V2a1 1 0 0 0-1-1H4z"/>'),qf=Xu("BookmarkDashFill",'<path fill-rule="evenodd" d="M2 15.5V2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.74.439L8 13.069l-5.26 2.87A.5.5 0 0 1 2 15.5zM6 6a.5.5 0 0 0 0 1h4a.5.5 0 0 0 0-1H6z"/>'),Jf=Xu("BookmarkFill",'<path d="M2 2v13.5a.5.5 0 0 0 .74.439L8 13.069l5.26 2.87A.5.5 0 0 0 14 15.5V2a2 2 0 0 0-2-2H4a2 2 0 0 0-2 2z"/>'),Kf=Xu("BookmarkHeart",'<path fill-rule="evenodd" d="M8 4.41c1.387-1.425 4.854 1.07 0 4.277C3.146 5.48 6.613 2.986 8 4.412z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.777.416L8 13.101l-5.223 2.815A.5.5 0 0 1 2 15.5V2zm2-1a1 1 0 0 0-1 1v12.566l4.723-2.482a.5.5 0 0 1 .554 0L13 14.566V2a1 1 0 0 0-1-1H4z"/>'),Xf=Xu("BookmarkHeartFill",'<path d="M2 15.5a.5.5 0 0 0 .74.439L8 13.069l5.26 2.87A.5.5 0 0 0 14 15.5V2a2 2 0 0 0-2-2H4a2 2 0 0 0-2 2v13.5zM8 4.41c1.387-1.425 4.854 1.07 0 4.277C3.146 5.48 6.613 2.986 8 4.412z"/>'),Zf=Xu("BookmarkPlus",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.777.416L8 13.101l-5.223 2.815A.5.5 0 0 1 2 15.5V2zm2-1a1 1 0 0 0-1 1v12.566l4.723-2.482a.5.5 0 0 1 .554 0L13 14.566V2a1 1 0 0 0-1-1H4z"/><path d="M8 4a.5.5 0 0 1 .5.5V6H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V7H6a.5.5 0 0 1 0-1h1.5V4.5A.5.5 0 0 1 8 4z"/>'),Qf=Xu("BookmarkPlusFill",'<path fill-rule="evenodd" d="M2 15.5V2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.74.439L8 13.069l-5.26 2.87A.5.5 0 0 1 2 15.5zm6.5-11a.5.5 0 0 0-1 0V6H6a.5.5 0 0 0 0 1h1.5v1.5a.5.5 0 0 0 1 0V7H10a.5.5 0 0 0 0-1H8.5V4.5z"/>'),ep=Xu("BookmarkStar",'<path d="M7.84 4.1a.178.178 0 0 1 .32 0l.634 1.285a.178.178 0 0 0 .134.098l1.42.206c.145.021.204.2.098.303L9.42 6.993a.178.178 0 0 0-.051.158l.242 1.414a.178.178 0 0 1-.258.187l-1.27-.668a.178.178 0 0 0-.165 0l-1.27.668a.178.178 0 0 1-.257-.187l.242-1.414a.178.178 0 0 0-.05-.158l-1.03-1.001a.178.178 0 0 1 .098-.303l1.42-.206a.178.178 0 0 0 .134-.098L7.84 4.1z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.777.416L8 13.101l-5.223 2.815A.5.5 0 0 1 2 15.5V2zm2-1a1 1 0 0 0-1 1v12.566l4.723-2.482a.5.5 0 0 1 .554 0L13 14.566V2a1 1 0 0 0-1-1H4z"/>'),tp=Xu("BookmarkStarFill",'<path fill-rule="evenodd" d="M2 15.5V2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.74.439L8 13.069l-5.26 2.87A.5.5 0 0 1 2 15.5zM8.16 4.1a.178.178 0 0 0-.32 0l-.634 1.285a.178.178 0 0 1-.134.098l-1.42.206a.178.178 0 0 0-.098.303L6.58 6.993c.042.041.061.1.051.158L6.39 8.565a.178.178 0 0 0 .258.187l1.27-.668a.178.178 0 0 1 .165 0l1.27.668a.178.178 0 0 0 .257-.187L9.368 7.15a.178.178 0 0 1 .05-.158l1.028-1.001a.178.178 0 0 0-.098-.303l-1.42-.206a.178.178 0 0 1-.134-.098L8.16 4.1z"/>'),ap=Xu("BookmarkX",'<path fill-rule="evenodd" d="M6.146 5.146a.5.5 0 0 1 .708 0L8 6.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 7l1.147 1.146a.5.5 0 0 1-.708.708L8 7.707 6.854 8.854a.5.5 0 1 1-.708-.708L7.293 7 6.146 5.854a.5.5 0 0 1 0-.708z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.777.416L8 13.101l-5.223 2.815A.5.5 0 0 1 2 15.5V2zm2-1a1 1 0 0 0-1 1v12.566l4.723-2.482a.5.5 0 0 1 .554 0L13 14.566V2a1 1 0 0 0-1-1H4z"/>'),np=Xu("BookmarkXFill",'<path fill-rule="evenodd" d="M2 15.5V2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v13.5a.5.5 0 0 1-.74.439L8 13.069l-5.26 2.87A.5.5 0 0 1 2 15.5zM6.854 5.146a.5.5 0 1 0-.708.708L7.293 7 6.146 8.146a.5.5 0 1 0 .708.708L8 7.707l1.146 1.147a.5.5 0 1 0 .708-.708L8.707 7l1.147-1.146a.5.5 0 0 0-.708-.708L8 6.293 6.854 5.146z"/>'),rp=Xu("Bookmarks",'<path d="M2 4a2 2 0 0 1 2-2h6a2 2 0 0 1 2 2v11.5a.5.5 0 0 1-.777.416L7 13.101l-4.223 2.815A.5.5 0 0 1 2 15.5V4zm2-1a1 1 0 0 0-1 1v10.566l3.723-2.482a.5.5 0 0 1 .554 0L11 14.566V4a1 1 0 0 0-1-1H4z"/><path d="M4.268 1H12a1 1 0 0 1 1 1v11.768l.223.148A.5.5 0 0 0 14 13.5V2a2 2 0 0 0-2-2H6a2 2 0 0 0-1.732 1z"/>'),ip=Xu("BookmarksFill",'<path d="M2 4a2 2 0 0 1 2-2h6a2 2 0 0 1 2 2v11.5a.5.5 0 0 1-.777.416L7 13.101l-4.223 2.815A.5.5 0 0 1 2 15.5V4z"/><path d="M4.268 1A2 2 0 0 1 6 0h6a2 2 0 0 1 2 2v11.5a.5.5 0 0 1-.777.416L13 13.768V2a1 1 0 0 0-1-1H4.268z"/>'),op=Xu("Bookshelf",'<path d="M2.5 0a.5.5 0 0 1 .5.5V2h10V.5a.5.5 0 0 1 1 0v15a.5.5 0 0 1-1 0V15H3v.5a.5.5 0 0 1-1 0V.5a.5.5 0 0 1 .5-.5zM3 14h10v-3H3v3zm0-4h10V7H3v3zm0-4h10V3H3v3z"/>'),sp=Xu("Bootstrap",'<path d="M5.062 12h3.475c1.804 0 2.888-.908 2.888-2.396 0-1.102-.761-1.916-1.904-2.034v-.1c.832-.14 1.482-.93 1.482-1.816 0-1.3-.955-2.11-2.542-2.11H5.062V12zm1.313-4.875V4.658h1.78c.973 0 1.542.457 1.542 1.237 0 .802-.604 1.23-1.764 1.23H6.375zm0 3.762V8.162h1.822c1.236 0 1.887.463 1.887 1.348 0 .896-.627 1.377-1.811 1.377H6.375z"/><path d="M0 4a4 4 0 0 1 4-4h8a4 4 0 0 1 4 4v8a4 4 0 0 1-4 4H4a4 4 0 0 1-4-4V4zm4-3a3 3 0 0 0-3 3v8a3 3 0 0 0 3 3h8a3 3 0 0 0 3-3V4a3 3 0 0 0-3-3H4z"/>'),lp=Xu("BootstrapFill",'<path d="M6.375 7.125V4.658h1.78c.973 0 1.542.457 1.542 1.237 0 .802-.604 1.23-1.764 1.23H6.375zm0 3.762h1.898c1.184 0 1.81-.48 1.81-1.377 0-.885-.65-1.348-1.886-1.348H6.375v2.725z"/><path d="M4.002 0a4 4 0 0 0-4 4v8a4 4 0 0 0 4 4h8a4 4 0 0 0 4-4V4a4 4 0 0 0-4-4h-8zm1.06 12V3.545h3.399c1.587 0 2.543.809 2.543 2.11 0 .884-.65 1.675-1.483 1.816v.1c1.143.117 1.904.931 1.904 2.033 0 1.488-1.084 2.396-2.888 2.396H5.062z"/>'),cp=Xu("BootstrapReboot",'<path d="M1.161 8a6.84 6.84 0 1 0 6.842-6.84.58.58 0 1 1 0-1.16 8 8 0 1 1-6.556 3.412l-.663-.577a.58.58 0 0 1 .227-.997l2.52-.69a.58.58 0 0 1 .728.633l-.332 2.592a.58.58 0 0 1-.956.364l-.643-.56A6.812 6.812 0 0 0 1.16 8z"/><path d="M6.641 11.671V8.843h1.57l1.498 2.828h1.314L9.377 8.665c.897-.3 1.427-1.106 1.427-2.1 0-1.37-.943-2.246-2.456-2.246H5.5v7.352h1.141zm0-3.75V5.277h1.57c.881 0 1.416.499 1.416 1.32 0 .84-.504 1.324-1.386 1.324h-1.6z"/>'),up=Xu("Border",'<path d="M0 0h.969v.5H1v.469H.969V1H.5V.969H0V0zm2.844 1h-.938V0h.938v1zm1.875 0H3.78V0h.938v1zm1.875 0h-.938V0h.938v1zm.937 0V.969H7.5V.5h.031V0h.938v.5H8.5v.469h-.031V1H7.53zm2.813 0h-.938V0h.938v1zm1.875 0h-.938V0h.938v1zm1.875 0h-.938V0h.938v1zM15.5 1h-.469V.969H15V.5h.031V0H16v.969h-.5V1zM1 1.906v.938H0v-.938h1zm6.5.938v-.938h1v.938h-1zm7.5 0v-.938h1v.938h-1zM1 3.78v.938H0V3.78h1zm6.5.938V3.78h1v.938h-1zm7.5 0V3.78h1v.938h-1zM1 5.656v.938H0v-.938h1zm6.5.938v-.938h1v.938h-1zm7.5 0v-.938h1v.938h-1zM.969 8.5H.5v-.031H0V7.53h.5V7.5h.469v.031H1v.938H.969V8.5zm1.875 0h-.938v-1h.938v1zm1.875 0H3.78v-1h.938v1zm1.875 0h-.938v-1h.938v1zm1.875-.031V8.5H7.53v-.031H7.5V7.53h.031V7.5h.938v.031H8.5v.938h-.031zm1.875.031h-.938v-1h.938v1zm1.875 0h-.938v-1h.938v1zm1.875 0h-.938v-1h.938v1zm1.406 0h-.469v-.031H15V7.53h.031V7.5h.469v.031h.5v.938h-.5V8.5zM0 10.344v-.938h1v.938H0zm7.5 0v-.938h1v.938h-1zm8.5-.938v.938h-1v-.938h1zM0 12.22v-.938h1v.938H0zm7.5 0v-.938h1v.938h-1zm8.5-.938v.938h-1v-.938h1zM0 14.094v-.938h1v.938H0zm7.5 0v-.938h1v.938h-1zm8.5-.938v.938h-1v-.938h1zM.969 16H0v-.969h.5V15h.469v.031H1v.469H.969v.5zm1.875 0h-.938v-1h.938v1zm1.875 0H3.78v-1h.938v1zm1.875 0h-.938v-1h.938v1zm.937 0v-.5H7.5v-.469h.031V15h.938v.031H8.5v.469h-.031v.5H7.53zm2.813 0h-.938v-1h.938v1zm1.875 0h-.938v-1h.938v1zm1.875 0h-.938v-1h.938v1zm.937 0v-.5H15v-.469h.031V15h.469v.031h.5V16h-.969z"/>'),dp=Xu("BorderAll",'<path d="M0 0h16v16H0V0zm1 1v6.5h6.5V1H1zm7.5 0v6.5H15V1H8.5zM15 8.5H8.5V15H15V8.5zM7.5 15V8.5H1V15h6.5z"/>'),hp=Xu("BorderBottom",'<path d="M.969 0H0v.969h.5V1h.469V.969H1V.5H.969V0zm.937 1h.938V0h-.938v1zm1.875 0h.938V0H3.78v1zm1.875 0h.938V0h-.938v1zM7.531.969V1h.938V.969H8.5V.5h-.031V0H7.53v.5H7.5v.469h.031zM9.406 1h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.469V.969h.5V0h-.969v.5H15v.469h.031V1zM1 2.844v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM1 4.719V3.78H0v.938h1zm6.5-.938v.938h1V3.78h-1zm7.5 0v.938h1V3.78h-1zM1 6.594v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM.5 8.5h.469v-.031H1V7.53H.969V7.5H.5v.031H0v.938h.5V8.5zm1.406 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm2.813 0v-.031H8.5V7.53h-.031V7.5H7.53v.031H7.5v.938h.031V8.5h.938zm.937 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.469v-.031h.5V7.53h-.5V7.5h-.469v.031H15v.938h.031V8.5zM0 9.406v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zM0 15h16v1H0v-1z"/>'),fp=Xu("BorderCenter",'<path d="M.969 0H0v.969h.5V1h.469V.969H1V.5H.969V0zm.937 1h.938V0h-.938v1zm1.875 0h.938V0H3.78v1zm1.875 0h.938V0h-.938v1zM7.531.969V1h.938V.969H8.5V.5h-.031V0H7.53v.5H7.5v.469h.031zM9.406 1h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.469V.969h.5V0h-.969v.5H15v.469h.031V1zM1 2.844v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM1 4.719V3.78H0v.938h1zm6.5-.938v.938h1V3.78h-1zm7.5 0v.938h1V3.78h-1zM1 6.594v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM0 8.5v-1h16v1H0zm0 .906v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zM0 16h.969v-.5H1v-.469H.969V15H.5v.031H0V16zm1.906 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5h.938v-.5H8.5v-.469h-.031V15H7.53v.031H7.5v.469h.031zm1.875.5h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5H16v-.969h-.5V15h-.469v.031H15v.469h.031z"/>'),pp=Xu("BorderInner",'<path d="M.969 0H0v.969h.5V1h.469V.969H1V.5H.969V0zm.937 1h.938V0h-.938v1zm1.875 0h.938V0H3.78v1zm1.875 0h.938V0h-.938v1z"/><path d="M8.5 7.5H16v1H8.5V16h-1V8.5H0v-1h7.5V0h1v7.5z"/><path d="M9.406 1h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.469V.969h.5V0h-.969v.5H15v.469h.031V1zM1 2.844v-.938H0v.938h1zm14-.938v.938h1v-.938h-1zM1 4.719V3.78H0v.938h1zm14-.938v.938h1V3.78h-1zM1 6.594v-.938H0v.938h1zm14-.938v.938h1v-.938h-1zM0 9.406v.938h1v-.938H0zm16 .938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm16 .938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm16 .938v-.938h-1v.938h1zM0 16h.969v-.5H1v-.469H.969V15H.5v.031H0V16zm1.906 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm3.75 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5H16v-.969h-.5V15h-.469v.031H15v.469h.031z"/>'),mp=Xu("BorderLeft",'<path d="M0 0v16h1V0H0zm1.906 1h.938V0h-.938v1zm1.875 0h.938V0H3.78v1zm1.875 0h.938V0h-.938v1zM7.531.969V1h.938V.969H8.5V.5h-.031V0H7.53v.5H7.5v.469h.031zM9.406 1h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.469V.969h.5V0h-.969v.5H15v.469h.031V1zM7.5 1.906v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM7.5 3.781v.938h1V3.78h-1zm7.5 0v.938h1V3.78h-1zM7.5 5.656v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM1.906 8.5h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm2.813 0v-.031H8.5V7.53h-.031V7.5H7.53v.031H7.5v.938h.031V8.5h.938zm.937 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.469v-.031h.5V7.53h-.5V7.5h-.469v.031H15v.938h.031V8.5zM7.5 9.406v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-8.5.937v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-8.5.937v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zM1.906 16h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5h.938v-.5H8.5v-.469h-.031V15H7.53v.031H7.5v.469h.031zm1.875.5h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5H16v-.969h-.5V15h-.469v.031H15v.469h.031z"/>'),vp=Xu("BorderMiddle",'<path d="M.969 0H0v.969h.5V1h.469V.969H1V.5H.969V0zm.937 1h.938V0h-.938v1zm1.875 0h.938V0H3.78v1zm1.875 0h.938V0h-.938v1zM8.5 16h-1V0h1v16zm.906-15h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.469V.969h.5V0h-.969v.5H15v.469h.031V1zM1 2.844v-.938H0v.938h1zm14-.938v.938h1v-.938h-1zM1 4.719V3.78H0v.938h1zm14-.938v.938h1V3.78h-1zM1 6.594v-.938H0v.938h1zm14-.938v.938h1v-.938h-1zM.5 8.5h.469v-.031H1V7.53H.969V7.5H.5v.031H0v.938h.5V8.5zm1.406 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm3.75 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.469v-.031h.5V7.53h-.5V7.5h-.469v.031H15v.938h.031V8.5zM0 9.406v.938h1v-.938H0zm16 .938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm16 .938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm16 .938v-.938h-1v.938h1zM0 16h.969v-.5H1v-.469H.969V15H.5v.031H0V16zm1.906 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm3.75 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5H16v-.969h-.5V15h-.469v.031H15v.469h.031z"/>'),_p=Xu("BorderOuter",'<path d="M7.5 1.906v.938h1v-.938h-1zm0 1.875v.938h1V3.78h-1zm0 1.875v.938h1v-.938h-1zM1.906 8.5h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm2.813 0v-.031H8.5V7.53h-.031V7.5H7.53v.031H7.5v.938h.031V8.5h.938zm.937 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zM7.5 9.406v.938h1v-.938h-1zm0 1.875v.938h1v-.938h-1zm0 1.875v.938h1v-.938h-1z"/><path d="M0 0v16h16V0H0zm1 1h14v14H1V1z"/>'),gp=Xu("BorderRight",'<path d="M.969 0H0v.969h.5V1h.469V.969H1V.5H.969V0zm.937 1h.938V0h-.938v1zm1.875 0h.938V0H3.78v1zm1.875 0h.938V0h-.938v1zM7.531.969V1h.938V.969H8.5V.5h-.031V0H7.53v.5H7.5v.469h.031zM9.406 1h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zm1.875 0h.938V0h-.938v1zM16 0h-1v16h1V0zM1 2.844v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zM1 4.719V3.78H0v.938h1zm6.5-.938v.938h1V3.78h-1zM1 6.594v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zM.5 8.5h.469v-.031H1V7.53H.969V7.5H.5v.031H0v.938h.5V8.5zm1.406 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm2.813 0v-.031H8.5V7.53h-.031V7.5H7.53v.031H7.5v.938h.031V8.5h.938zm.937 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zM0 9.406v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zM0 11.281v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zM0 13.156v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zM0 16h.969v-.5H1v-.469H.969V15H.5v.031H0V16zm1.906 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5h.938v-.5H8.5v-.469h-.031V15H7.53v.031H7.5v.469h.031zm1.875.5h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1z"/>'),yp=Xu("BorderStyle",'<path d="M1 3.5a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-13a.5.5 0 0 1-.5-.5v-1zm0 4a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-1zm0 4a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm8 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-4 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm8 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-4-4a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-1z"/>'),bp=Xu("BorderTop",'<path d="M0 0v1h16V0H0zm1 2.844v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM1 4.719V3.78H0v.938h1zm6.5-.938v.938h1V3.78h-1zm7.5 0v.938h1V3.78h-1zM1 6.594v-.938H0v.938h1zm6.5-.938v.938h1v-.938h-1zm7.5 0v.938h1v-.938h-1zM.5 8.5h.469v-.031H1V7.53H.969V7.5H.5v.031H0v.938h.5V8.5zm1.406 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm2.813 0v-.031H8.5V7.53h-.031V7.5H7.53v.031H7.5v.938h.031V8.5h.938zm.937 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.469v-.031h.5V7.53h-.5V7.5h-.469v.031H15v.938h.031V8.5zM0 9.406v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zm-16 .937v.938h1v-.938H0zm7.5 0v.938h1v-.938h-1zm8.5.938v-.938h-1v.938h1zM0 16h.969v-.5H1v-.469H.969V15H.5v.031H0V16zm1.906 0h.938v-1h-.938v1zm1.875 0h.938v-1H3.78v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5h.938v-.5H8.5v-.469h-.031V15H7.53v.031H7.5v.469h.031zm1.875.5h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875 0h.938v-1h-.938v1zm1.875-.5v.5H16v-.969h-.5V15h-.469v.031H15v.469h.031z"/>'),Mp=Xu("BorderWidth",'<path d="M0 3.5A.5.5 0 0 1 .5 3h15a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5H.5a.5.5 0 0 1-.5-.5v-2zm0 5A.5.5 0 0 1 .5 8h15a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H.5a.5.5 0 0 1-.5-.5v-1zm0 4a.5.5 0 0 1 .5-.5h15a.5.5 0 0 1 0 1H.5a.5.5 0 0 1-.5-.5z"/>'),zp=Xu("BoundingBox",'<path d="M5 2V0H0v5h2v6H0v5h5v-2h6v2h5v-5h-2V5h2V0h-5v2H5zm6 1v2h2v6h-2v2H5v-2H3V5h2V3h6zm1-2h3v3h-3V1zm3 11v3h-3v-3h3zM4 15H1v-3h3v3zM1 4V1h3v3H1z"/>'),Lp=Xu("BoundingBoxCircles",'<path d="M2 1a1 1 0 1 0 0 2 1 1 0 0 0 0-2zM0 2a2 2 0 0 1 3.937-.5h8.126A2 2 0 1 1 14.5 3.937v8.126a2 2 0 1 1-2.437 2.437H3.937A2 2 0 1 1 1.5 12.063V3.937A2 2 0 0 1 0 2zm2.5 1.937v8.126c.703.18 1.256.734 1.437 1.437h8.126a2.004 2.004 0 0 1 1.437-1.437V3.937A2.004 2.004 0 0 1 12.063 2.5H3.937A2.004 2.004 0 0 1 2.5 3.937zM14 1a1 1 0 1 0 0 2 1 1 0 0 0 0-2zM2 13a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm12 0a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/>'),wp=Xu("Box",'<path d="M8.186 1.113a.5.5 0 0 0-.372 0L1.846 3.5 8 5.961 14.154 3.5 8.186 1.113zM15 4.239l-6.5 2.6v7.922l6.5-2.6V4.24zM7.5 14.762V6.838L1 4.239v7.923l6.5 2.6zM7.443.184a1.5 1.5 0 0 1 1.114 0l7.129 2.852A.5.5 0 0 1 16 3.5v8.662a1 1 0 0 1-.629.928l-7.185 2.874a.5.5 0 0 1-.372 0L.63 13.09a1 1 0 0 1-.63-.928V3.5a.5.5 0 0 1 .314-.464L7.443.184z"/>'),Hp=Xu("BoxArrowDown",'<path fill-rule="evenodd" d="M3.5 10a.5.5 0 0 1-.5-.5v-8a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 0 0 1h2A1.5 1.5 0 0 0 14 9.5v-8A1.5 1.5 0 0 0 12.5 0h-9A1.5 1.5 0 0 0 2 1.5v8A1.5 1.5 0 0 0 3.5 11h2a.5.5 0 0 0 0-1h-2z"/><path fill-rule="evenodd" d="M7.646 15.854a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 14.293V5.5a.5.5 0 0 0-1 0v8.793l-2.146-2.147a.5.5 0 0 0-.708.708l3 3z"/>'),kp=Xu("BoxArrowDownLeft",'<path fill-rule="evenodd" d="M7.364 12.5a.5.5 0 0 0 .5.5H14.5a1.5 1.5 0 0 0 1.5-1.5v-10A1.5 1.5 0 0 0 14.5 0h-10A1.5 1.5 0 0 0 3 1.5v6.636a.5.5 0 1 0 1 0V1.5a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v10a.5.5 0 0 1-.5.5H7.864a.5.5 0 0 0-.5.5z"/><path fill-rule="evenodd" d="M0 15.5a.5.5 0 0 0 .5.5h5a.5.5 0 0 0 0-1H1.707l8.147-8.146a.5.5 0 0 0-.708-.708L1 14.293V10.5a.5.5 0 0 0-1 0v5z"/>'),Ap=Xu("BoxArrowDownRight",'<path fill-rule="evenodd" d="M8.636 12.5a.5.5 0 0 1-.5.5H1.5A1.5 1.5 0 0 1 0 11.5v-10A1.5 1.5 0 0 1 1.5 0h10A1.5 1.5 0 0 1 13 1.5v6.636a.5.5 0 0 1-1 0V1.5a.5.5 0 0 0-.5-.5h-10a.5.5 0 0 0-.5.5v10a.5.5 0 0 0 .5.5h6.636a.5.5 0 0 1 .5.5z"/><path fill-rule="evenodd" d="M16 15.5a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1 0-1h3.793L6.146 6.854a.5.5 0 1 1 .708-.708L15 14.293V10.5a.5.5 0 0 1 1 0v5z"/>'),Sp=Xu("BoxArrowInDown",'<path fill-rule="evenodd" d="M3.5 6a.5.5 0 0 0-.5.5v8a.5.5 0 0 0 .5.5h9a.5.5 0 0 0 .5-.5v-8a.5.5 0 0 0-.5-.5h-2a.5.5 0 0 1 0-1h2A1.5 1.5 0 0 1 14 6.5v8a1.5 1.5 0 0 1-1.5 1.5h-9A1.5 1.5 0 0 1 2 14.5v-8A1.5 1.5 0 0 1 3.5 5h2a.5.5 0 0 1 0 1h-2z"/><path fill-rule="evenodd" d="M7.646 11.854a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 10.293V1.5a.5.5 0 0 0-1 0v8.793L5.354 8.146a.5.5 0 1 0-.708.708l3 3z"/>'),Vp=Xu("BoxArrowInDownLeft",'<path fill-rule="evenodd" d="M9.636 2.5a.5.5 0 0 0-.5-.5H2.5A1.5 1.5 0 0 0 1 3.5v10A1.5 1.5 0 0 0 2.5 15h10a1.5 1.5 0 0 0 1.5-1.5V6.864a.5.5 0 0 0-1 0V13.5a.5.5 0 0 1-.5.5h-10a.5.5 0 0 1-.5-.5v-10a.5.5 0 0 1 .5-.5h6.636a.5.5 0 0 0 .5-.5z"/><path fill-rule="evenodd" d="M5 10.5a.5.5 0 0 0 .5.5h5a.5.5 0 0 0 0-1H6.707l8.147-8.146a.5.5 0 0 0-.708-.708L6 9.293V5.5a.5.5 0 0 0-1 0v5z"/>'),Op=Xu("BoxArrowInDownRight",'<path fill-rule="evenodd" d="M6.364 2.5a.5.5 0 0 1 .5-.5H13.5A1.5 1.5 0 0 1 15 3.5v10a1.5 1.5 0 0 1-1.5 1.5h-10A1.5 1.5 0 0 1 2 13.5V6.864a.5.5 0 1 1 1 0V13.5a.5.5 0 0 0 .5.5h10a.5.5 0 0 0 .5-.5v-10a.5.5 0 0 0-.5-.5H6.864a.5.5 0 0 1-.5-.5z"/><path fill-rule="evenodd" d="M11 10.5a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1 0-1h3.793L1.146 1.854a.5.5 0 1 1 .708-.708L10 9.293V5.5a.5.5 0 0 1 1 0v5z"/>'),Tp=Xu("BoxArrowInLeft",'<path fill-rule="evenodd" d="M10 3.5a.5.5 0 0 0-.5-.5h-8a.5.5 0 0 0-.5.5v9a.5.5 0 0 0 .5.5h8a.5.5 0 0 0 .5-.5v-2a.5.5 0 0 1 1 0v2A1.5 1.5 0 0 1 9.5 14h-8A1.5 1.5 0 0 1 0 12.5v-9A1.5 1.5 0 0 1 1.5 2h8A1.5 1.5 0 0 1 11 3.5v2a.5.5 0 0 1-1 0v-2z"/><path fill-rule="evenodd" d="M4.146 8.354a.5.5 0 0 1 0-.708l3-3a.5.5 0 1 1 .708.708L5.707 7.5H14.5a.5.5 0 0 1 0 1H5.707l2.147 2.146a.5.5 0 0 1-.708.708l-3-3z"/>'),Dp=Xu("BoxArrowInRight",'<path fill-rule="evenodd" d="M6 3.5a.5.5 0 0 1 .5-.5h8a.5.5 0 0 1 .5.5v9a.5.5 0 0 1-.5.5h-8a.5.5 0 0 1-.5-.5v-2a.5.5 0 0 0-1 0v2A1.5 1.5 0 0 0 6.5 14h8a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 2h-8A1.5 1.5 0 0 0 5 3.5v2a.5.5 0 0 0 1 0v-2z"/><path fill-rule="evenodd" d="M11.854 8.354a.5.5 0 0 0 0-.708l-3-3a.5.5 0 1 0-.708.708L10.293 7.5H1.5a.5.5 0 0 0 0 1h8.793l-2.147 2.146a.5.5 0 0 0 .708.708l3-3z"/>'),Cp=Xu("BoxArrowInUp",'<path fill-rule="evenodd" d="M3.5 10a.5.5 0 0 1-.5-.5v-8a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 0 0 1h2A1.5 1.5 0 0 0 14 9.5v-8A1.5 1.5 0 0 0 12.5 0h-9A1.5 1.5 0 0 0 2 1.5v8A1.5 1.5 0 0 0 3.5 11h2a.5.5 0 0 0 0-1h-2z"/><path fill-rule="evenodd" d="M7.646 4.146a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 5.707V14.5a.5.5 0 0 1-1 0V5.707L5.354 7.854a.5.5 0 1 1-.708-.708l3-3z"/>'),jp=Xu("BoxArrowInUpLeft",'<path fill-rule="evenodd" d="M9.636 13.5a.5.5 0 0 1-.5.5H2.5A1.5 1.5 0 0 1 1 12.5v-10A1.5 1.5 0 0 1 2.5 1h10A1.5 1.5 0 0 1 14 2.5v6.636a.5.5 0 0 1-1 0V2.5a.5.5 0 0 0-.5-.5h-10a.5.5 0 0 0-.5.5v10a.5.5 0 0 0 .5.5h6.636a.5.5 0 0 1 .5.5z"/><path fill-rule="evenodd" d="M5 5.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1H6.707l8.147 8.146a.5.5 0 0 1-.708.708L6 6.707V10.5a.5.5 0 0 1-1 0v-5z"/>'),xp=Xu("BoxArrowInUpRight",'<path fill-rule="evenodd" d="M6.364 13.5a.5.5 0 0 0 .5.5H13.5a1.5 1.5 0 0 0 1.5-1.5v-10A1.5 1.5 0 0 0 13.5 1h-10A1.5 1.5 0 0 0 2 2.5v6.636a.5.5 0 1 0 1 0V2.5a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v10a.5.5 0 0 1-.5.5H6.864a.5.5 0 0 0-.5.5z"/><path fill-rule="evenodd" d="M11 5.5a.5.5 0 0 0-.5-.5h-5a.5.5 0 0 0 0 1h3.793l-8.147 8.146a.5.5 0 0 0 .708.708L10 6.707V10.5a.5.5 0 0 0 1 0v-5z"/>'),Bp=Xu("BoxArrowLeft",'<path fill-rule="evenodd" d="M6 12.5a.5.5 0 0 0 .5.5h8a.5.5 0 0 0 .5-.5v-9a.5.5 0 0 0-.5-.5h-8a.5.5 0 0 0-.5.5v2a.5.5 0 0 1-1 0v-2A1.5 1.5 0 0 1 6.5 2h8A1.5 1.5 0 0 1 16 3.5v9a1.5 1.5 0 0 1-1.5 1.5h-8A1.5 1.5 0 0 1 5 12.5v-2a.5.5 0 0 1 1 0v2z"/><path fill-rule="evenodd" d="M.146 8.354a.5.5 0 0 1 0-.708l3-3a.5.5 0 1 1 .708.708L1.707 7.5H10.5a.5.5 0 0 1 0 1H1.707l2.147 2.146a.5.5 0 0 1-.708.708l-3-3z"/>'),Yp=Xu("BoxArrowRight",'<path fill-rule="evenodd" d="M10 12.5a.5.5 0 0 1-.5.5h-8a.5.5 0 0 1-.5-.5v-9a.5.5 0 0 1 .5-.5h8a.5.5 0 0 1 .5.5v2a.5.5 0 0 0 1 0v-2A1.5 1.5 0 0 0 9.5 2h-8A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h8a1.5 1.5 0 0 0 1.5-1.5v-2a.5.5 0 0 0-1 0v2z"/><path fill-rule="evenodd" d="M15.854 8.354a.5.5 0 0 0 0-.708l-3-3a.5.5 0 0 0-.708.708L14.293 7.5H5.5a.5.5 0 0 0 0 1h8.793l-2.147 2.146a.5.5 0 0 0 .708.708l3-3z"/>'),Pp=Xu("BoxArrowUp",'<path fill-rule="evenodd" d="M3.5 6a.5.5 0 0 0-.5.5v8a.5.5 0 0 0 .5.5h9a.5.5 0 0 0 .5-.5v-8a.5.5 0 0 0-.5-.5h-2a.5.5 0 0 1 0-1h2A1.5 1.5 0 0 1 14 6.5v8a1.5 1.5 0 0 1-1.5 1.5h-9A1.5 1.5 0 0 1 2 14.5v-8A1.5 1.5 0 0 1 3.5 5h2a.5.5 0 0 1 0 1h-2z"/><path fill-rule="evenodd" d="M7.646.146a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 1.707V10.5a.5.5 0 0 1-1 0V1.707L5.354 3.854a.5.5 0 1 1-.708-.708l3-3z"/>'),Ip=Xu("BoxArrowUpLeft",'<path fill-rule="evenodd" d="M7.364 3.5a.5.5 0 0 1 .5-.5H14.5A1.5 1.5 0 0 1 16 4.5v10a1.5 1.5 0 0 1-1.5 1.5h-10A1.5 1.5 0 0 1 3 14.5V7.864a.5.5 0 1 1 1 0V14.5a.5.5 0 0 0 .5.5h10a.5.5 0 0 0 .5-.5v-10a.5.5 0 0 0-.5-.5H7.864a.5.5 0 0 1-.5-.5z"/><path fill-rule="evenodd" d="M0 .5A.5.5 0 0 1 .5 0h5a.5.5 0 0 1 0 1H1.707l8.147 8.146a.5.5 0 0 1-.708.708L1 1.707V5.5a.5.5 0 0 1-1 0v-5z"/>'),Fp=Xu("BoxArrowUpRight",'<path fill-rule="evenodd" d="M8.636 3.5a.5.5 0 0 0-.5-.5H1.5A1.5 1.5 0 0 0 0 4.5v10A1.5 1.5 0 0 0 1.5 16h10a1.5 1.5 0 0 0 1.5-1.5V7.864a.5.5 0 0 0-1 0V14.5a.5.5 0 0 1-.5.5h-10a.5.5 0 0 1-.5-.5v-10a.5.5 0 0 1 .5-.5h6.636a.5.5 0 0 0 .5-.5z"/><path fill-rule="evenodd" d="M16 .5a.5.5 0 0 0-.5-.5h-5a.5.5 0 0 0 0 1h3.793L6.146 9.146a.5.5 0 1 0 .708.708L15 1.707V5.5a.5.5 0 0 0 1 0v-5z"/>'),Ep=Xu("BoxSeam",'<path d="M8.186 1.113a.5.5 0 0 0-.372 0L1.846 3.5l2.404.961L10.404 2l-2.218-.887zm3.564 1.426L5.596 5 8 5.961 14.154 3.5l-2.404-.961zm3.25 1.7-6.5 2.6v7.922l6.5-2.6V4.24zM7.5 14.762V6.838L1 4.239v7.923l6.5 2.6zM7.443.184a1.5 1.5 0 0 1 1.114 0l7.129 2.852A.5.5 0 0 1 16 3.5v8.662a1 1 0 0 1-.629.928l-7.185 2.874a.5.5 0 0 1-.372 0L.63 13.09a1 1 0 0 1-.63-.928V3.5a.5.5 0 0 1 .314-.464L7.443.184z"/>'),$p=Xu("Braces",'<path d="M2.114 8.063V7.9c1.005-.102 1.497-.615 1.497-1.6V4.503c0-1.094.39-1.538 1.354-1.538h.273V2h-.376C3.25 2 2.49 2.759 2.49 4.352v1.524c0 1.094-.376 1.456-1.49 1.456v1.299c1.114 0 1.49.362 1.49 1.456v1.524c0 1.593.759 2.352 2.372 2.352h.376v-.964h-.273c-.964 0-1.354-.444-1.354-1.538V9.663c0-.984-.492-1.497-1.497-1.6zM13.886 7.9v.163c-1.005.103-1.497.616-1.497 1.6v1.798c0 1.094-.39 1.538-1.354 1.538h-.273v.964h.376c1.613 0 2.372-.759 2.372-2.352v-1.524c0-1.094.376-1.456 1.49-1.456V7.332c-1.114 0-1.49-.362-1.49-1.456V4.352C13.51 2.759 12.75 2 11.138 2h-.376v.964h.273c.964 0 1.354.444 1.354 1.538V6.3c0 .984.492 1.497 1.497 1.6z"/>'),Rp=Xu("Bricks",'<path d="M0 .5A.5.5 0 0 1 .5 0h15a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-.5.5H14v2h1.5a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-.5.5H14v2h1.5a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-.5.5H.5a.5.5 0 0 1-.5-.5v-3a.5.5 0 0 1 .5-.5H2v-2H.5a.5.5 0 0 1-.5-.5v-3A.5.5 0 0 1 .5 6H2V4H.5a.5.5 0 0 1-.5-.5v-3zM3 4v2h4.5V4H3zm5.5 0v2H13V4H8.5zM3 10v2h4.5v-2H3zm5.5 0v2H13v-2H8.5zM1 1v2h3.5V1H1zm4.5 0v2h5V1h-5zm6 0v2H15V1h-3.5zM1 7v2h3.5V7H1zm4.5 0v2h5V7h-5zm6 0v2H15V7h-3.5zM1 13v2h3.5v-2H1zm4.5 0v2h5v-2h-5zm6 0v2H15v-2h-3.5z"/>'),Np=Xu("Briefcase",'<path d="M6.5 1A1.5 1.5 0 0 0 5 2.5V3H1.5A1.5 1.5 0 0 0 0 4.5v8A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-8A1.5 1.5 0 0 0 14.5 3H11v-.5A1.5 1.5 0 0 0 9.5 1h-3zm0 1h3a.5.5 0 0 1 .5.5V3H6v-.5a.5.5 0 0 1 .5-.5zm1.886 6.914L15 7.151V12.5a.5.5 0 0 1-.5.5h-13a.5.5 0 0 1-.5-.5V7.15l6.614 1.764a1.5 1.5 0 0 0 .772 0zM1.5 4h13a.5.5 0 0 1 .5.5v1.616L8.129 7.948a.5.5 0 0 1-.258 0L1 6.116V4.5a.5.5 0 0 1 .5-.5z"/>'),Wp=Xu("BriefcaseFill",'<path d="M6.5 1A1.5 1.5 0 0 0 5 2.5V3H1.5A1.5 1.5 0 0 0 0 4.5v1.384l7.614 2.03a1.5 1.5 0 0 0 .772 0L16 5.884V4.5A1.5 1.5 0 0 0 14.5 3H11v-.5A1.5 1.5 0 0 0 9.5 1h-3zm0 1h3a.5.5 0 0 1 .5.5V3H6v-.5a.5.5 0 0 1 .5-.5z"/><path d="M0 12.5A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5V6.85L8.129 8.947a.5.5 0 0 1-.258 0L0 6.85v5.65z"/>'),Up=Xu("BrightnessAltHigh",'<path d="M8 3a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 3zm8 8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2a.5.5 0 0 1 .5.5zm-13.5.5a.5.5 0 0 0 0-1h-2a.5.5 0 0 0 0 1h2zm11.157-6.157a.5.5 0 0 1 0 .707l-1.414 1.414a.5.5 0 1 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zm-9.9 2.121a.5.5 0 0 0 .707-.707L3.05 5.343a.5.5 0 1 0-.707.707l1.414 1.414zM8 7a4 4 0 0 0-4 4 .5.5 0 0 0 .5.5h7a.5.5 0 0 0 .5-.5 4 4 0 0 0-4-4zm0 1a3 3 0 0 1 2.959 2.5H5.04A3 3 0 0 1 8 8z"/>'),Gp=Xu("BrightnessAltHighFill",'<path d="M8 3a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 3zm8 8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2a.5.5 0 0 1 .5.5zm-13.5.5a.5.5 0 0 0 0-1h-2a.5.5 0 0 0 0 1h2zm11.157-6.157a.5.5 0 0 1 0 .707l-1.414 1.414a.5.5 0 1 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zm-9.9 2.121a.5.5 0 0 0 .707-.707L3.05 5.343a.5.5 0 1 0-.707.707l1.414 1.414zM8 7a4 4 0 0 0-4 4 .5.5 0 0 0 .5.5h7a.5.5 0 0 0 .5-.5 4 4 0 0 0-4-4z"/>'),qp=Xu("BrightnessAltLow",'<path d="M8.5 5.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm5 6a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zM2 11a.5.5 0 1 0 1 0 .5.5 0 0 0-1 0zm10.243-3.536a.5.5 0 1 1-.707-.707.5.5 0 0 1 .707.707zm-8.486-.707a.5.5 0 1 0 .707.707.5.5 0 0 0-.707-.707zM8 7a4 4 0 0 0-4 4 .5.5 0 0 0 .5.5h7a.5.5 0 0 0 .5-.5 4 4 0 0 0-4-4zm0 1a3 3 0 0 1 2.959 2.5H5.04A3 3 0 0 1 8 8z"/>'),Jp=Xu("BrightnessAltLowFill",'<path d="M8.5 5.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm5 6a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zM2 11a.5.5 0 1 0 1 0 .5.5 0 0 0-1 0zm10.243-3.536a.5.5 0 1 1-.707-.707.5.5 0 0 1 .707.707zm-8.486-.707a.5.5 0 1 0 .707.707.5.5 0 0 0-.707-.707zM8 7a4 4 0 0 0-4 4 .5.5 0 0 0 .5.5h7a.5.5 0 0 0 .5-.5 4 4 0 0 0-4-4z"/>'),Kp=Xu("BrightnessHigh",'<path d="M8 11a3 3 0 1 1 0-6 3 3 0 0 1 0 6zm0 1a4 4 0 1 0 0-8 4 4 0 0 0 0 8zM8 0a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 0zm0 13a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 13zm8-5a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2a.5.5 0 0 1 .5.5zM3 8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2A.5.5 0 0 1 3 8zm10.657-5.657a.5.5 0 0 1 0 .707l-1.414 1.415a.5.5 0 1 1-.707-.708l1.414-1.414a.5.5 0 0 1 .707 0zm-9.193 9.193a.5.5 0 0 1 0 .707L3.05 13.657a.5.5 0 0 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zm9.193 2.121a.5.5 0 0 1-.707 0l-1.414-1.414a.5.5 0 0 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .707zM4.464 4.465a.5.5 0 0 1-.707 0L2.343 3.05a.5.5 0 1 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .708z"/>'),Xp=Xu("BrightnessHighFill",'<path d="M12 8a4 4 0 1 1-8 0 4 4 0 0 1 8 0zM8 0a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 0zm0 13a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 13zm8-5a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2a.5.5 0 0 1 .5.5zM3 8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2A.5.5 0 0 1 3 8zm10.657-5.657a.5.5 0 0 1 0 .707l-1.414 1.415a.5.5 0 1 1-.707-.708l1.414-1.414a.5.5 0 0 1 .707 0zm-9.193 9.193a.5.5 0 0 1 0 .707L3.05 13.657a.5.5 0 0 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zm9.193 2.121a.5.5 0 0 1-.707 0l-1.414-1.414a.5.5 0 0 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .707zM4.464 4.465a.5.5 0 0 1-.707 0L2.343 3.05a.5.5 0 1 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .708z"/>'),Zp=Xu("BrightnessLow",'<path d="M8 11a3 3 0 1 1 0-6 3 3 0 0 1 0 6zm0 1a4 4 0 1 0 0-8 4 4 0 0 0 0 8zm.5-9.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm0 11a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm5-5a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm-11 0a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm9.743-4.036a.5.5 0 1 1-.707-.707.5.5 0 0 1 .707.707zm-7.779 7.779a.5.5 0 1 1-.707-.707.5.5 0 0 1 .707.707zm7.072 0a.5.5 0 1 1 .707-.707.5.5 0 0 1-.707.707zM3.757 4.464a.5.5 0 1 1 .707-.707.5.5 0 0 1-.707.707z"/>'),Qp=Xu("BrightnessLowFill",'<path d="M12 8a4 4 0 1 1-8 0 4 4 0 0 1 8 0zM8.5 2.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm0 11a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm5-5a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm-11 0a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm9.743-4.036a.5.5 0 1 1-.707-.707.5.5 0 0 1 .707.707zm-7.779 7.779a.5.5 0 1 1-.707-.707.5.5 0 0 1 .707.707zm7.072 0a.5.5 0 1 1 .707-.707.5.5 0 0 1-.707.707zM3.757 4.464a.5.5 0 1 1 .707-.707.5.5 0 0 1-.707.707z"/>'),em=Xu("Broadcast",'<path d="M3.05 3.05a7 7 0 0 0 0 9.9.5.5 0 0 1-.707.707 8 8 0 0 1 0-11.314.5.5 0 0 1 .707.707zm2.122 2.122a4 4 0 0 0 0 5.656.5.5 0 1 1-.708.708 5 5 0 0 1 0-7.072.5.5 0 0 1 .708.708zm5.656-.708a.5.5 0 0 1 .708 0 5 5 0 0 1 0 7.072.5.5 0 1 1-.708-.708 4 4 0 0 0 0-5.656.5.5 0 0 1 0-.708zm2.122-2.12a.5.5 0 0 1 .707 0 8 8 0 0 1 0 11.313.5.5 0 0 1-.707-.707 7 7 0 0 0 0-9.9.5.5 0 0 1 0-.707zM10 8a2 2 0 1 1-4 0 2 2 0 0 1 4 0z"/>'),tm=Xu("BroadcastPin",'<path d="M3.05 3.05a7 7 0 0 0 0 9.9.5.5 0 0 1-.707.707 8 8 0 0 1 0-11.314.5.5 0 0 1 .707.707zm2.122 2.122a4 4 0 0 0 0 5.656.5.5 0 1 1-.708.708 5 5 0 0 1 0-7.072.5.5 0 0 1 .708.708zm5.656-.708a.5.5 0 0 1 .708 0 5 5 0 0 1 0 7.072.5.5 0 1 1-.708-.708 4 4 0 0 0 0-5.656.5.5 0 0 1 0-.708zm2.122-2.12a.5.5 0 0 1 .707 0 8 8 0 0 1 0 11.313.5.5 0 0 1-.707-.707 7 7 0 0 0 0-9.9.5.5 0 0 1 0-.707zM6 8a2 2 0 1 1 2.5 1.937V15.5a.5.5 0 0 1-1 0V9.937A2 2 0 0 1 6 8z"/>'),am=Xu("Brush",'<path d="M15.825.12a.5.5 0 0 1 .132.584c-1.53 3.43-4.743 8.17-7.095 10.64a6.067 6.067 0 0 1-2.373 1.534c-.018.227-.06.538-.16.868-.201.659-.667 1.479-1.708 1.74a8.118 8.118 0 0 1-3.078.132 3.659 3.659 0 0 1-.562-.135 1.382 1.382 0 0 1-.466-.247.714.714 0 0 1-.204-.288.622.622 0 0 1 .004-.443c.095-.245.316-.38.461-.452.394-.197.625-.453.867-.826.095-.144.184-.297.287-.472l.117-.198c.151-.255.326-.54.546-.848.528-.739 1.201-.925 1.746-.896.126.007.243.025.348.048.062-.172.142-.38.238-.608.261-.619.658-1.419 1.187-2.069 2.176-2.67 6.18-6.206 9.117-8.104a.5.5 0 0 1 .596.04zM4.705 11.912a1.23 1.23 0 0 0-.419-.1c-.246-.013-.573.05-.879.479-.197.275-.355.532-.5.777l-.105.177c-.106.181-.213.362-.32.528a3.39 3.39 0 0 1-.76.861c.69.112 1.736.111 2.657-.12.559-.139.843-.569.993-1.06a3.122 3.122 0 0 0 .126-.75l-.793-.792zm1.44.026c.12-.04.277-.1.458-.183a5.068 5.068 0 0 0 1.535-1.1c1.9-1.996 4.412-5.57 6.052-8.631-2.59 1.927-5.566 4.66-7.302 6.792-.442.543-.795 1.243-1.042 1.826-.121.288-.214.54-.275.72v.001l.575.575zm-4.973 3.04.007-.005a.031.031 0 0 1-.007.004zm3.582-3.043.002.001h-.002z"/>'),nm=Xu("BrushFill",'<path d="M15.825.12a.5.5 0 0 1 .132.584c-1.53 3.43-4.743 8.17-7.095 10.64a6.067 6.067 0 0 1-2.373 1.534c-.018.227-.06.538-.16.868-.201.659-.667 1.479-1.708 1.74a8.118 8.118 0 0 1-3.078.132 3.659 3.659 0 0 1-.562-.135 1.382 1.382 0 0 1-.466-.247.714.714 0 0 1-.204-.288.622.622 0 0 1 .004-.443c.095-.245.316-.38.461-.452.394-.197.625-.453.867-.826.095-.144.184-.297.287-.472l.117-.198c.151-.255.326-.54.546-.848.528-.739 1.201-.925 1.746-.896.126.007.243.025.348.048.062-.172.142-.38.238-.608.261-.619.658-1.419 1.187-2.069 2.176-2.67 6.18-6.206 9.117-8.104a.5.5 0 0 1 .596.04z"/>'),rm=Xu("Bucket",'<path d="M2.522 5H2a.5.5 0 0 0-.494.574l1.372 9.149A1.5 1.5 0 0 0 4.36 16h7.278a1.5 1.5 0 0 0 1.483-1.277l1.373-9.149A.5.5 0 0 0 14 5h-.522A5.5 5.5 0 0 0 2.522 5zm1.005 0a4.5 4.5 0 0 1 8.945 0H3.527zm9.892 1-1.286 8.574a.5.5 0 0 1-.494.426H4.36a.5.5 0 0 1-.494-.426L2.58 6h10.838z"/>'),im=Xu("BucketFill",'<path d="M2.522 5H2a.5.5 0 0 0-.494.574l1.372 9.149A1.5 1.5 0 0 0 4.36 16h7.278a1.5 1.5 0 0 0 1.483-1.277l1.373-9.149A.5.5 0 0 0 14 5h-.522A5.5 5.5 0 0 0 2.522 5zm1.005 0a4.5 4.5 0 0 1 8.945 0H3.527z"/>'),om=Xu("Bug",'<path d="M4.355.522a.5.5 0 0 1 .623.333l.291.956A4.979 4.979 0 0 1 8 1c1.007 0 1.946.298 2.731.811l.29-.956a.5.5 0 1 1 .957.29l-.41 1.352A4.985 4.985 0 0 1 13 6h.5a.5.5 0 0 0 .5-.5V5a.5.5 0 0 1 1 0v.5A1.5 1.5 0 0 1 13.5 7H13v1h1.5a.5.5 0 0 1 0 1H13v1h.5a1.5 1.5 0 0 1 1.5 1.5v.5a.5.5 0 1 1-1 0v-.5a.5.5 0 0 0-.5-.5H13a5 5 0 0 1-10 0h-.5a.5.5 0 0 0-.5.5v.5a.5.5 0 1 1-1 0v-.5A1.5 1.5 0 0 1 2.5 10H3V9H1.5a.5.5 0 0 1 0-1H3V7h-.5A1.5 1.5 0 0 1 1 5.5V5a.5.5 0 0 1 1 0v.5a.5.5 0 0 0 .5.5H3c0-1.364.547-2.601 1.432-3.503l-.41-1.352a.5.5 0 0 1 .333-.623zM4 7v4a4 4 0 0 0 3.5 3.97V7H4zm4.5 0v7.97A4 4 0 0 0 12 11V7H8.5zM12 6a3.989 3.989 0 0 0-1.334-2.982A3.983 3.983 0 0 0 8 2a3.983 3.983 0 0 0-2.667 1.018A3.989 3.989 0 0 0 4 6h8z"/>'),sm=Xu("BugFill",'<path d="M4.978.855a.5.5 0 1 0-.956.29l.41 1.352A4.985 4.985 0 0 0 3 6h10a4.985 4.985 0 0 0-1.432-3.503l.41-1.352a.5.5 0 1 0-.956-.29l-.291.956A4.978 4.978 0 0 0 8 1a4.979 4.979 0 0 0-2.731.811l-.29-.956z"/><path d="M13 6v1H8.5v8.975A5 5 0 0 0 13 11h.5a.5.5 0 0 1 .5.5v.5a.5.5 0 1 0 1 0v-.5a1.5 1.5 0 0 0-1.5-1.5H13V9h1.5a.5.5 0 0 0 0-1H13V7h.5A1.5 1.5 0 0 0 15 5.5V5a.5.5 0 0 0-1 0v.5a.5.5 0 0 1-.5.5H13zm-5.5 9.975V7H3V6h-.5a.5.5 0 0 1-.5-.5V5a.5.5 0 0 0-1 0v.5A1.5 1.5 0 0 0 2.5 7H3v1H1.5a.5.5 0 0 0 0 1H3v1h-.5A1.5 1.5 0 0 0 1 11.5v.5a.5.5 0 1 0 1 0v-.5a.5.5 0 0 1 .5-.5H3a5 5 0 0 0 4.5 4.975z"/>'),lm=Xu("Building",'<path fill-rule="evenodd" d="M14.763.075A.5.5 0 0 1 15 .5v15a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5V14h-1v1.5a.5.5 0 0 1-.5.5h-9a.5.5 0 0 1-.5-.5V10a.5.5 0 0 1 .342-.474L6 7.64V4.5a.5.5 0 0 1 .276-.447l8-4a.5.5 0 0 1 .487.022zM6 8.694 1 10.36V15h5V8.694zM7 15h2v-1.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5V15h2V1.309l-7 3.5V15z"/><path d="M2 11h1v1H2v-1zm2 0h1v1H4v-1zm-2 2h1v1H2v-1zm2 0h1v1H4v-1zm4-4h1v1H8V9zm2 0h1v1h-1V9zm-2 2h1v1H8v-1zm2 0h1v1h-1v-1zm2-2h1v1h-1V9zm0 2h1v1h-1v-1zM8 7h1v1H8V7zm2 0h1v1h-1V7zm2 0h1v1h-1V7zM8 5h1v1H8V5zm2 0h1v1h-1V5zm2 0h1v1h-1V5zm0-2h1v1h-1V3z"/>'),cm=Xu("Bullseye",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M8 13A5 5 0 1 1 8 3a5 5 0 0 1 0 10zm0 1A6 6 0 1 0 8 2a6 6 0 0 0 0 12z"/><path d="M8 11a3 3 0 1 1 0-6 3 3 0 0 1 0 6zm0 1a4 4 0 1 0 0-8 4 4 0 0 0 0 8z"/><path d="M9.5 8a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),um=Xu("Calculator",'<path d="M12 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h8zM4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4z"/><path d="M4 2.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-2zm0 4a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm0 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm0 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm3-6a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm0 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm0 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm3-6a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm0 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-4z"/>'),dm=Xu("CalculatorFill",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm2 .5v2a.5.5 0 0 0 .5.5h7a.5.5 0 0 0 .5-.5v-2a.5.5 0 0 0-.5-.5h-7a.5.5 0 0 0-.5.5zm0 4v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5zM4.5 9a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zM4 12.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5zM7.5 6a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zM7 9.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5zm.5 2.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zM10 6.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5zm.5 2.5a.5.5 0 0 0-.5.5v4a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-4a.5.5 0 0 0-.5-.5h-1z"/>'),hm=Xu("Calendar",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),fm=Xu("Calendar2",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),pm=Xu("Calendar2Check",'<path d="M10.854 8.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 0 1 .708-.708L7.5 10.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),mm=Xu("Calendar2CheckFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zm-2.6 5.854a.5.5 0 0 0-.708-.708L7.5 10.793 6.354 9.646a.5.5 0 1 0-.708.708l1.5 1.5a.5.5 0 0 0 .708 0l3-3z"/>'),vm=Xu("Calendar2Date",'<path d="M6.445 12.688V7.354h-.633A12.6 12.6 0 0 0 4.5 8.16v.695c.375-.257.969-.62 1.258-.777h.012v4.61h.675zm1.188-1.305c.047.64.594 1.406 1.703 1.406 1.258 0 2-1.066 2-2.871 0-1.934-.781-2.668-1.953-2.668-.926 0-1.797.672-1.797 1.809 0 1.16.824 1.77 1.676 1.77.746 0 1.23-.376 1.383-.79h.027c-.004 1.316-.461 2.164-1.305 2.164-.664 0-1.008-.45-1.05-.82h-.684zm2.953-2.317c0 .696-.559 1.18-1.184 1.18-.601 0-1.144-.383-1.144-1.2 0-.823.582-1.21 1.168-1.21.633 0 1.16.398 1.16 1.23z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),_m=Xu("Calendar2DateFill",'<path d="M9.402 10.246c.625 0 1.184-.484 1.184-1.18 0-.832-.527-1.23-1.16-1.23-.586 0-1.168.387-1.168 1.21 0 .817.543 1.2 1.144 1.2z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zm-4.118 9.79c1.258 0 2-1.067 2-2.872 0-1.934-.781-2.668-1.953-2.668-.926 0-1.797.672-1.797 1.809 0 1.16.824 1.77 1.676 1.77.746 0 1.23-.376 1.383-.79h.027c-.004 1.316-.461 2.164-1.305 2.164-.664 0-1.008-.45-1.05-.82h-.684c.047.64.594 1.406 1.703 1.406zm-2.89-5.435h-.633A12.6 12.6 0 0 0 4.5 8.16v.695c.375-.257.969-.62 1.258-.777h.012v4.61h.675V7.354z"/>'),gm=Xu("Calendar2Day",'<path d="M4.684 12.523v-2.3h2.261v-.61H4.684V7.801h2.464v-.61H4v5.332h.684zm3.296 0h.676V9.98c0-.554.227-1.007.953-1.007.125 0 .258.004.329.015v-.613a1.806 1.806 0 0 0-.254-.02c-.582 0-.891.32-1.012.567h-.02v-.504H7.98v4.105zm2.805-5.093c0 .238.192.425.43.425a.428.428 0 1 0 0-.855.426.426 0 0 0-.43.43zm.094 5.093h.672V8.418h-.672v4.105z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),ym=Xu("Calendar2DayFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zm-2.24 4.855a.428.428 0 1 0 0-.855.426.426 0 0 0-.429.43c0 .238.192.425.43.425zm.337.563h-.672v4.105h.672V8.418zm-6.867 4.105v-2.3h2.261v-.61H4.684V7.801h2.464v-.61H4v5.332h.684zm3.296 0h.676V9.98c0-.554.227-1.007.953-1.007.125 0 .258.004.329.015v-.613a1.806 1.806 0 0 0-.254-.02c-.582 0-.891.32-1.012.567h-.02v-.504H7.98v4.105z"/>'),bm=Xu("Calendar2Event",'<path d="M11 7.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),Mm=Xu("Calendar2EventFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zM11.5 7a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1z"/>'),zm=Xu("Calendar2Fill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v11a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM2.545 3h10.91c.3 0 .545.224.545.5v1c0 .276-.244.5-.546.5H2.545C2.245 5 2 4.776 2 4.5v-1c0-.276.244-.5.545-.5z"/>'),Lm=Xu("Calendar2Minus",'<path d="M5.5 10.5A.5.5 0 0 1 6 10h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),wm=Xu("Calendar2MinusFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zM6 10a.5.5 0 0 0 0 1h4a.5.5 0 0 0 0-1H6z"/>'),Hm=Xu("Calendar2Month",'<path d="m2.56 12.332.54-1.602h1.984l.54 1.602h.718L4.444 7h-.696L1.85 12.332h.71zm1.544-4.527L4.9 10.18H3.284l.8-2.375h.02zm5.746.422h-.676v2.543c0 .652-.414 1.023-1.004 1.023-.539 0-.98-.246-.98-1.012V8.227h-.676v2.746c0 .941.606 1.425 1.453 1.425.656 0 1.043-.28 1.188-.605h.027v.539h.668V8.227zm2.258 5.046c-.563 0-.91-.304-.985-.636h-.687c.094.683.625 1.199 1.668 1.199.93 0 1.746-.527 1.746-1.578V8.227h-.649v.578h-.019c-.191-.348-.637-.64-1.195-.64-.965 0-1.64.679-1.64 1.886v.34c0 1.23.683 1.902 1.64 1.902.558 0 1.008-.293 1.172-.648h.02v.605c0 .645-.423 1.023-1.071 1.023zm.008-4.53c.648 0 1.062.527 1.062 1.359v.253c0 .848-.39 1.364-1.062 1.364-.692 0-1.098-.512-1.098-1.364v-.253c0-.868.406-1.36 1.098-1.36z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),km=Xu("Calendar2MonthFill",'<path d="M4.104 7.805 4.9 10.18H3.284l.8-2.375h.02zm9.074 2.297c0-.832-.414-1.36-1.062-1.36-.692 0-1.098.492-1.098 1.36v.253c0 .852.406 1.364 1.098 1.364.671 0 1.062-.516 1.062-1.364v-.253z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zM2.561 12.332 3.1 10.73h1.984l.54 1.602h.718L4.444 7h-.696L1.85 12.332h.71zM9.85 8.227h-.676v2.543c0 .652-.414 1.023-1.004 1.023-.539 0-.98-.246-.98-1.012V8.227h-.676v2.746c0 .941.606 1.425 1.453 1.425.656 0 1.043-.28 1.188-.605h.027v.539h.668V8.227zm1.273 4.41h-.687c.094.683.625 1.199 1.668 1.199.93 0 1.746-.527 1.746-1.578V8.227h-.649v.578h-.019c-.191-.348-.637-.64-1.195-.64-.965 0-1.64.679-1.64 1.886v.34c0 1.23.683 1.902 1.64 1.902.558 0 1.008-.293 1.172-.648h.02v.605c0 .645-.423 1.023-1.071 1.023-.563 0-.91-.304-.985-.636z"/>'),Am=Xu("Calendar2Plus",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4zM8 8a.5.5 0 0 1 .5.5V10H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V11H6a.5.5 0 0 1 0-1h1.5V8.5A.5.5 0 0 1 8 8z"/>'),Sm=Xu("Calendar2PlusFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 3.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5H2.545c-.3 0-.545.224-.545.5zm6.5 5a.5.5 0 0 0-1 0V10H6a.5.5 0 0 0 0 1h1.5v1.5a.5.5 0 0 0 1 0V11H10a.5.5 0 0 0 0-1H8.5V8.5z"/>'),Vm=Xu("Calendar2Range",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4zM9 8a1 1 0 0 1 1-1h5v2h-5a1 1 0 0 1-1-1zm-8 2h4a1 1 0 1 1 0 2H1v-2z"/>'),Om=Xu("Calendar2RangeFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zM10 7a1 1 0 0 0 0 2h5V7h-5zm-4 4a1 1 0 0 0-1-1H1v2h4a1 1 0 0 0 1-1z"/>'),Tm=Xu("Calendar2Week",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4zM11 7.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-5 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1z"/>'),Dm=Xu("Calendar2WeekFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zM8.5 7a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zm3 0a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zM3 10.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5zm3.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1z"/>'),Cm=Xu("Calendar2X",'<path d="M6.146 8.146a.5.5 0 0 1 .708 0L8 9.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 10l1.147 1.146a.5.5 0 0 1-.708.708L8 10.707l-1.146 1.147a.5.5 0 0 1-.708-.708L7.293 10 6.146 8.854a.5.5 0 0 1 0-.708z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2z"/><path d="M2.5 4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H3a.5.5 0 0 1-.5-.5V4z"/>'),jm=Xu("Calendar2XFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zm9.954 3H2.545c-.3 0-.545.224-.545.5v1c0 .276.244.5.545.5h10.91c.3 0 .545-.224.545-.5v-1c0-.276-.244-.5-.546-.5zm-6.6 5.146a.5.5 0 1 0-.708.708L7.293 10l-1.147 1.146a.5.5 0 0 0 .708.708L8 10.707l1.146 1.147a.5.5 0 0 0 .708-.708L8.707 10l1.147-1.146a.5.5 0 0 0-.708-.708L8 9.293 6.854 8.146z"/>'),xm=Xu("Calendar3",'<path d="M14 0H2a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM1 3.857C1 3.384 1.448 3 2 3h12c.552 0 1 .384 1 .857v10.286c0 .473-.448.857-1 .857H2c-.552 0-1-.384-1-.857V3.857z"/><path d="M6.5 7a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm3 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm3 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm-9 3a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm3 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm3 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm3 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm-9 3a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm3 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm3 0a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),Bm=Xu("Calendar3Event",'<path d="M14 0H2a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM1 3.857C1 3.384 1.448 3 2 3h12c.552 0 1 .384 1 .857v10.286c0 .473-.448.857-1 .857H2c-.552 0-1-.384-1-.857V3.857z"/><path d="M12 7a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),Ym=Xu("Calendar3EventFill",'<path fill-rule="evenodd" d="M2 0a2 2 0 0 0-2 2h16a2 2 0 0 0-2-2H2zM0 14V3h16v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm12-8a1 1 0 1 0 2 0 1 1 0 0 0-2 0z"/>'),Pm=Xu("Calendar3Fill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2H0zm0 1v11a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3H0z"/>'),Im=Xu("Calendar3Range",'<path d="M14 0H2a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM1 3.857C1 3.384 1.448 3 2 3h12c.552 0 1 .384 1 .857v10.286c0 .473-.448.857-1 .857H2c-.552 0-1-.384-1-.857V3.857z"/><path d="M7 10a1 1 0 0 0 0-2H1v2h6zm2-3h6V5H9a1 1 0 0 0 0 2z"/>'),Fm=Xu("Calendar3RangeFill",'<path fill-rule="evenodd" d="M2 0a2 2 0 0 0-2 2h16a2 2 0 0 0-2-2H2zM0 8V3h16v2h-6a1 1 0 1 0 0 2h6v7a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2v-4h6a1 1 0 1 0 0-2H0z"/>'),Em=Xu("Calendar3Week",'<path d="M14 0H2a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM1 3.857C1 3.384 1.448 3 2 3h12c.552 0 1 .384 1 .857v10.286c0 .473-.448.857-1 .857H2c-.552 0-1-.384-1-.857V3.857z"/><path d="M12 7a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm-5 3a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm2-3a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm-5 3a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),$m=Xu("Calendar3WeekFill",'<path fill-rule="evenodd" d="M2 0a2 2 0 0 0-2 2h16a2 2 0 0 0-2-2H2zM0 14V3h16v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm12-8a1 1 0 1 0 2 0 1 1 0 0 0-2 0zM5 9a1 1 0 1 0 2 0 1 1 0 0 0-2 0zm5-2a1 1 0 1 1 0-2 1 1 0 0 1 0 2zM2 9a1 1 0 1 0 2 0 1 1 0 0 0-2 0z"/>'),Rm=Xu("Calendar4",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v1h14V3a1 1 0 0 0-1-1H2zm13 3H1v9a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V5z"/>'),Nm=Xu("Calendar4Event",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v1h14V3a1 1 0 0 0-1-1H2zm13 3H1v9a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V5z"/><path d="M11 7.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1z"/>'),Wm=Xu("Calendar4Range",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v1h14V3a1 1 0 0 0-1-1H2zm13 3H1v9a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V5z"/><path d="M9 7.5a.5.5 0 0 1 .5-.5H15v2H9.5a.5.5 0 0 1-.5-.5v-1zm-2 3v1a.5.5 0 0 1-.5.5H1v-2h5.5a.5.5 0 0 1 .5.5z"/>'),Um=Xu("Calendar4Week",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM2 2a1 1 0 0 0-1 1v1h14V3a1 1 0 0 0-1-1H2zm13 3H1v9a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V5z"/><path d="M11 7.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-2 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1z"/>'),Gm=Xu("CalendarCheck",'<path d="M10.854 7.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 9.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),qm=Xu("CalendarCheckFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zm-5.146-5.146-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 0 1 .708-.708L7.5 10.793l2.646-2.647a.5.5 0 0 1 .708.708z"/>'),Jm=Xu("CalendarDate",'<path d="M6.445 11.688V6.354h-.633A12.6 12.6 0 0 0 4.5 7.16v.695c.375-.257.969-.62 1.258-.777h.012v4.61h.675zm1.188-1.305c.047.64.594 1.406 1.703 1.406 1.258 0 2-1.066 2-2.871 0-1.934-.781-2.668-1.953-2.668-.926 0-1.797.672-1.797 1.809 0 1.16.824 1.77 1.676 1.77.746 0 1.23-.376 1.383-.79h.027c-.004 1.316-.461 2.164-1.305 2.164-.664 0-1.008-.45-1.05-.82h-.684zm2.953-2.317c0 .696-.559 1.18-1.184 1.18-.601 0-1.144-.383-1.144-1.2 0-.823.582-1.21 1.168-1.21.633 0 1.16.398 1.16 1.23z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),Km=Xu("CalendarDateFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zm5.402 9.746c.625 0 1.184-.484 1.184-1.18 0-.832-.527-1.23-1.16-1.23-.586 0-1.168.387-1.168 1.21 0 .817.543 1.2 1.144 1.2z"/><path d="M16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zm-6.664-1.21c-1.11 0-1.656-.767-1.703-1.407h.683c.043.37.387.82 1.051.82.844 0 1.301-.848 1.305-2.164h-.027c-.153.414-.637.79-1.383.79-.852 0-1.676-.61-1.676-1.77 0-1.137.871-1.809 1.797-1.809 1.172 0 1.953.734 1.953 2.668 0 1.805-.742 2.871-2 2.871zm-2.89-5.435v5.332H5.77V8.079h-.012c-.29.156-.883.52-1.258.777V8.16a12.6 12.6 0 0 1 1.313-.805h.632z"/>'),Xm=Xu("CalendarDay",'<path d="M4.684 11.523v-2.3h2.261v-.61H4.684V6.801h2.464v-.61H4v5.332h.684zm3.296 0h.676V8.98c0-.554.227-1.007.953-1.007.125 0 .258.004.329.015v-.613a1.806 1.806 0 0 0-.254-.02c-.582 0-.891.32-1.012.567h-.02v-.504H7.98v4.105zm2.805-5.093c0 .238.192.425.43.425a.428.428 0 1 0 0-.855.426.426 0 0 0-.43.43zm.094 5.093h.672V7.418h-.672v4.105z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),Zm=Xu("CalendarDayFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 14a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V5h16v9zm-4.785-6.145a.428.428 0 1 0 0-.855.426.426 0 0 0-.43.43c0 .238.192.425.43.425zm.336.563h-.672v4.105h.672V8.418zm-6.867 4.105v-2.3h2.261v-.61H4.684V7.801h2.464v-.61H4v5.332h.684zm3.296 0h.676V9.98c0-.554.227-1.007.953-1.007.125 0 .258.004.329.015v-.613a1.806 1.806 0 0 0-.254-.02c-.582 0-.891.32-1.012.567h-.02v-.504H7.98v4.105z"/>'),Qm=Xu("CalendarEvent",'<path d="M11 6.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),ev=Xu("CalendarEventFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zm-3.5-7h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5z"/>'),tv=Xu("CalendarFill",'<path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V5h16V4H0V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5z"/>'),av=Xu("CalendarMinus",'<path d="M5.5 9.5A.5.5 0 0 1 6 9h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),nv=Xu("CalendarMinusFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zM6 10h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1 0-1z"/>'),rv=Xu("CalendarMonth",'<path d="M2.56 11.332 3.1 9.73h1.984l.54 1.602h.718L4.444 6h-.696L1.85 11.332h.71zm1.544-4.527L4.9 9.18H3.284l.8-2.375h.02zm5.746.422h-.676V9.77c0 .652-.414 1.023-1.004 1.023-.539 0-.98-.246-.98-1.012V7.227h-.676v2.746c0 .941.606 1.425 1.453 1.425.656 0 1.043-.28 1.188-.605h.027v.539h.668V7.227zm2.258 5.046c-.563 0-.91-.304-.985-.636h-.687c.094.683.625 1.199 1.668 1.199.93 0 1.746-.527 1.746-1.578V7.227h-.649v.578h-.019c-.191-.348-.637-.64-1.195-.64-.965 0-1.64.679-1.64 1.886v.34c0 1.23.683 1.902 1.64 1.902.558 0 1.008-.293 1.172-.648h.02v.605c0 .645-.423 1.023-1.071 1.023zm.008-4.53c.648 0 1.062.527 1.062 1.359v.253c0 .848-.39 1.364-1.062 1.364-.692 0-1.098-.512-1.098-1.364v-.253c0-.868.406-1.36 1.098-1.36z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),iv=Xu("CalendarMonthFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zm.104 7.305L4.9 10.18H3.284l.8-2.375h.02zm9.074 2.297c0-.832-.414-1.36-1.062-1.36-.692 0-1.098.492-1.098 1.36v.253c0 .852.406 1.364 1.098 1.364.671 0 1.062-.516 1.062-1.364v-.253z"/><path d="M16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zM2.56 12.332h-.71L3.748 7h.696l1.898 5.332h-.719l-.539-1.602H3.1l-.54 1.602zm7.29-4.105v4.105h-.668v-.539h-.027c-.145.324-.532.605-1.188.605-.847 0-1.453-.484-1.453-1.425V8.227h.676v2.554c0 .766.441 1.012.98 1.012.59 0 1.004-.371 1.004-1.023V8.227h.676zm1.273 4.41c.075.332.422.636.985.636.648 0 1.07-.378 1.07-1.023v-.605h-.02c-.163.355-.613.648-1.171.648-.957 0-1.64-.672-1.64-1.902v-.34c0-1.207.675-1.887 1.64-1.887.558 0 1.004.293 1.195.64h.02v-.577h.648v4.03c0 1.052-.816 1.579-1.746 1.579-1.043 0-1.574-.516-1.668-1.2h.687z"/>'),ov=Xu("CalendarPlus",'<path d="M8 7a.5.5 0 0 1 .5.5V9H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V10H6a.5.5 0 0 1 0-1h1.5V7.5A.5.5 0 0 1 8 7z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),sv=Xu("CalendarPlusFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zM8.5 8.5V10H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V11H6a.5.5 0 0 1 0-1h1.5V8.5a.5.5 0 0 1 1 0z"/>'),lv=Xu("CalendarRange",'<path d="M9 7a1 1 0 0 1 1-1h5v2h-5a1 1 0 0 1-1-1zM1 9h4a1 1 0 0 1 0 2H1V9z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),cv=Xu("CalendarRangeFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 7V5H0v5h5a1 1 0 1 1 0 2H0v2a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V9h-6a1 1 0 1 1 0-2h6z"/>'),uv=Xu("CalendarWeek",'<path d="M11 6.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm-5 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),dv=Xu("CalendarWeekFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zM9.5 7h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5zm3 0h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5zM2 10.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm3.5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5z"/>'),hv=Xu("CalendarX",'<path d="M6.146 7.146a.5.5 0 0 1 .708 0L8 8.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 9l1.147 1.146a.5.5 0 0 1-.708.708L8 9.707l-1.146 1.147a.5.5 0 0 1-.708-.708L7.293 9 6.146 7.854a.5.5 0 0 1 0-.708z"/><path d="M3.5 0a.5.5 0 0 1 .5.5V1h8V.5a.5.5 0 0 1 1 0V1h1a2 2 0 0 1 2 2v11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h1V.5a.5.5 0 0 1 .5-.5zM1 4v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4H1z"/>'),fv=Xu("CalendarXFill",'<path d="M4 .5a.5.5 0 0 0-1 0V1H2a2 2 0 0 0-2 2v1h16V3a2 2 0 0 0-2-2h-1V.5a.5.5 0 0 0-1 0V1H4V.5zM16 14V5H0v9a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2zM6.854 8.146 8 9.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 10l1.147 1.146a.5.5 0 0 1-.708.708L8 10.707l-1.146 1.147a.5.5 0 0 1-.708-.708L7.293 10 6.146 8.854a.5.5 0 1 1 .708-.708z"/>'),pv=Xu("Camera",'<path d="M15 12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h1.172a3 3 0 0 0 2.12-.879l.83-.828A1 1 0 0 1 6.827 3h2.344a1 1 0 0 1 .707.293l.828.828A3 3 0 0 0 12.828 5H14a1 1 0 0 1 1 1v6zM2 4a2 2 0 0 0-2 2v6a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2h-1.172a2 2 0 0 1-1.414-.586l-.828-.828A2 2 0 0 0 9.172 2H6.828a2 2 0 0 0-1.414.586l-.828.828A2 2 0 0 1 3.172 4H2z"/><path d="M8 11a2.5 2.5 0 1 1 0-5 2.5 2.5 0 0 1 0 5zm0 1a3.5 3.5 0 1 0 0-7 3.5 3.5 0 0 0 0 7zM3 6.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0z"/>'),mv=Xu("Camera2",'<path d="M5 8c0-1.657 2.343-3 4-3V4a4 4 0 0 0-4 4z"/><path d="M12.318 3h2.015C15.253 3 16 3.746 16 4.667v6.666c0 .92-.746 1.667-1.667 1.667h-2.015A5.97 5.97 0 0 1 9 14a5.972 5.972 0 0 1-3.318-1H1.667C.747 13 0 12.254 0 11.333V4.667C0 3.747.746 3 1.667 3H2a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1h.682A5.97 5.97 0 0 1 9 2c1.227 0 2.367.368 3.318 1zM2 4.5a.5.5 0 1 0-1 0 .5.5 0 0 0 1 0zM14 8A5 5 0 1 0 4 8a5 5 0 0 0 10 0z"/>'),vv=Xu("CameraFill",'<path d="M10.5 8.5a2.5 2.5 0 1 1-5 0 2.5 2.5 0 0 1 5 0z"/><path d="M2 4a2 2 0 0 0-2 2v6a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2h-1.172a2 2 0 0 1-1.414-.586l-.828-.828A2 2 0 0 0 9.172 2H6.828a2 2 0 0 0-1.414.586l-.828.828A2 2 0 0 1 3.172 4H2zm.5 2a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm9 2.5a3.5 3.5 0 1 1-7 0 3.5 3.5 0 0 1 7 0z"/>'),_v=Xu("CameraReels",'<path d="M6 3a3 3 0 1 1-6 0 3 3 0 0 1 6 0zM1 3a2 2 0 1 0 4 0 2 2 0 0 0-4 0z"/><path d="M9 6h.5a2 2 0 0 1 1.983 1.738l3.11-1.382A1 1 0 0 1 16 7.269v7.462a1 1 0 0 1-1.406.913l-3.111-1.382A2 2 0 0 1 9.5 16H2a2 2 0 0 1-2-2V8a2 2 0 0 1 2-2h7zm6 8.73V7.27l-3.5 1.555v4.35l3.5 1.556zM1 8v6a1 1 0 0 0 1 1h7.5a1 1 0 0 0 1-1V8a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1z"/><path d="M9 6a3 3 0 1 0 0-6 3 3 0 0 0 0 6zM7 3a2 2 0 1 1 4 0 2 2 0 0 1-4 0z"/>'),gv=Xu("CameraReelsFill",'<path d="M6 3a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/><path d="M9 6a3 3 0 1 1 0-6 3 3 0 0 1 0 6z"/><path d="M9 6h.5a2 2 0 0 1 1.983 1.738l3.11-1.382A1 1 0 0 1 16 7.269v7.462a1 1 0 0 1-1.406.913l-3.111-1.382A2 2 0 0 1 9.5 16H2a2 2 0 0 1-2-2V8a2 2 0 0 1 2-2h7z"/>'),yv=Xu("CameraVideo",'<path fill-rule="evenodd" d="M0 5a2 2 0 0 1 2-2h7.5a2 2 0 0 1 1.983 1.738l3.11-1.382A1 1 0 0 1 16 4.269v7.462a1 1 0 0 1-1.406.913l-3.111-1.382A2 2 0 0 1 9.5 13H2a2 2 0 0 1-2-2V5zm11.5 5.175 3.5 1.556V4.269l-3.5 1.556v4.35zM2 4a1 1 0 0 0-1 1v6a1 1 0 0 0 1 1h7.5a1 1 0 0 0 1-1V5a1 1 0 0 0-1-1H2z"/>'),bv=Xu("CameraVideoFill",'<path fill-rule="evenodd" d="M0 5a2 2 0 0 1 2-2h7.5a2 2 0 0 1 1.983 1.738l3.11-1.382A1 1 0 0 1 16 4.269v7.462a1 1 0 0 1-1.406.913l-3.111-1.382A2 2 0 0 1 9.5 13H2a2 2 0 0 1-2-2V5z"/>'),Mv=Xu("CameraVideoOff",'<path fill-rule="evenodd" d="M10.961 12.365a1.99 1.99 0 0 0 .522-1.103l3.11 1.382A1 1 0 0 0 16 11.731V4.269a1 1 0 0 0-1.406-.913l-3.111 1.382A2 2 0 0 0 9.5 3H4.272l.714 1H9.5a1 1 0 0 1 1 1v6a1 1 0 0 1-.144.518l.605.847zM1.428 4.18A.999.999 0 0 0 1 5v6a1 1 0 0 0 1 1h5.014l.714 1H2a2 2 0 0 1-2-2V5c0-.675.334-1.272.847-1.634l.58.814zM15 11.73l-3.5-1.555v-4.35L15 4.269v7.462zm-4.407 3.56-10-14 .814-.58 10 14-.814.58z"/>'),zv=Xu("CameraVideoOffFill",'<path fill-rule="evenodd" d="M10.961 12.365a1.99 1.99 0 0 0 .522-1.103l3.11 1.382A1 1 0 0 0 16 11.731V4.269a1 1 0 0 0-1.406-.913l-3.111 1.382A2 2 0 0 0 9.5 3H4.272l6.69 9.365zm-10.114-9A2.001 2.001 0 0 0 0 5v6a2 2 0 0 0 2 2h5.728L.847 3.366zm9.746 11.925-10-14 .814-.58 10 14-.814.58z"/>'),Lv=Xu("Capslock",'<path fill-rule="evenodd" d="M7.27 1.047a1 1 0 0 1 1.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H11.5v1a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1v-1H1.654C.78 9.5.326 8.455.924 7.816L7.27 1.047zM14.346 8.5 8 1.731 1.654 8.5H4.5a1 1 0 0 1 1 1v1h5v-1a1 1 0 0 1 1-1h2.846zm-9.846 5a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1v-1zm6 0h-5v1h5v-1z"/>'),wv=Xu("CapslockFill",'<path d="M7.27 1.047a1 1 0 0 1 1.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H11.5v1a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1v-1H1.654C.78 9.5.326 8.455.924 7.816L7.27 1.047zM4.5 13.5a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1v-1z"/>'),Hv=Xu("CardChecklist",'<path d="M14.5 3a.5.5 0 0 1 .5.5v9a.5.5 0 0 1-.5.5h-13a.5.5 0 0 1-.5-.5v-9a.5.5 0 0 1 .5-.5h13zm-13-1A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 2h-13z"/><path d="M7 5.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm-1.496-.854a.5.5 0 0 1 0 .708l-1.5 1.5a.5.5 0 0 1-.708 0l-.5-.5a.5.5 0 1 1 .708-.708l.146.147 1.146-1.147a.5.5 0 0 1 .708 0zM7 9.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm-1.496-.854a.5.5 0 0 1 0 .708l-1.5 1.5a.5.5 0 0 1-.708 0l-.5-.5a.5.5 0 0 1 .708-.708l.146.147 1.146-1.147a.5.5 0 0 1 .708 0z"/>'),kv=Xu("CardHeading",'<path d="M14.5 3a.5.5 0 0 1 .5.5v9a.5.5 0 0 1-.5.5h-13a.5.5 0 0 1-.5-.5v-9a.5.5 0 0 1 .5-.5h13zm-13-1A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 2h-13z"/><path d="M3 8.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zm0-5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-9a.5.5 0 0 1-.5-.5v-1z"/>'),Av=Xu("CardImage",'<path d="M6.002 5.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/><path d="M1.5 2A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 2h-13zm13 1a.5.5 0 0 1 .5.5v6l-3.775-1.947a.5.5 0 0 0-.577.093l-3.71 3.71-2.66-1.772a.5.5 0 0 0-.63.062L1.002 12v.54A.505.505 0 0 1 1 12.5v-9a.5.5 0 0 1 .5-.5h13z"/>'),Sv=Xu("CardList",'<path d="M14.5 3a.5.5 0 0 1 .5.5v9a.5.5 0 0 1-.5.5h-13a.5.5 0 0 1-.5-.5v-9a.5.5 0 0 1 .5-.5h13zm-13-1A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 2h-13z"/><path d="M5 8a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7A.5.5 0 0 1 5 8zm0-2.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0 5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-1-5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zM4 8a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm0 2.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0z"/>'),Vv=Xu("CardText",'<path d="M14.5 3a.5.5 0 0 1 .5.5v9a.5.5 0 0 1-.5.5h-13a.5.5 0 0 1-.5-.5v-9a.5.5 0 0 1 .5-.5h13zm-13-1A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 2h-13z"/><path d="M3 5.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM3 8a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9A.5.5 0 0 1 3 8zm0 2.5a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),Ov=Xu("CaretDown",'<path d="M3.204 5h9.592L8 10.481 3.204 5zm-.753.659 4.796 5.48a1 1 0 0 0 1.506 0l4.796-5.48c.566-.647.106-1.659-.753-1.659H3.204a1 1 0 0 0-.753 1.659z"/>'),Tv=Xu("CaretDownFill",'<path d="M7.247 11.14 2.451 5.658C1.885 5.013 2.345 4 3.204 4h9.592a1 1 0 0 1 .753 1.659l-4.796 5.48a1 1 0 0 1-1.506 0z"/>'),Dv=Xu("CaretDownSquare",'<path d="M3.626 6.832A.5.5 0 0 1 4 6h8a.5.5 0 0 1 .374.832l-4 4.5a.5.5 0 0 1-.748 0l-4-4.5z"/><path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2z"/>'),Cv=Xu("CaretDownSquareFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm4 4a.5.5 0 0 0-.374.832l4 4.5a.5.5 0 0 0 .748 0l4-4.5A.5.5 0 0 0 12 6H4z"/>'),jv=Xu("CaretLeft",'<path d="M10 12.796V3.204L4.519 8 10 12.796zm-.659.753-5.48-4.796a1 1 0 0 1 0-1.506l5.48-4.796A1 1 0 0 1 11 3.204v9.592a1 1 0 0 1-1.659.753z"/>'),xv=Xu("CaretLeftFill",'<path d="m3.86 8.753 5.482 4.796c.646.566 1.658.106 1.658-.753V3.204a1 1 0 0 0-1.659-.753l-5.48 4.796a1 1 0 0 0 0 1.506z"/>'),Bv=Xu("CaretLeftSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M10.205 12.456A.5.5 0 0 0 10.5 12V4a.5.5 0 0 0-.832-.374l-4.5 4a.5.5 0 0 0 0 .748l4.5 4a.5.5 0 0 0 .537.082z"/>'),Yv=Xu("CaretLeftSquareFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm10.5 10V4a.5.5 0 0 0-.832-.374l-4.5 4a.5.5 0 0 0 0 .748l4.5 4A.5.5 0 0 0 10.5 12z"/>'),Pv=Xu("CaretRight",'<path d="M6 12.796V3.204L11.481 8 6 12.796zm.659.753 5.48-4.796a1 1 0 0 0 0-1.506L6.66 2.451C6.011 1.885 5 2.345 5 3.204v9.592a1 1 0 0 0 1.659.753z"/>'),Iv=Xu("CaretRightFill",'<path d="m12.14 8.753-5.482 4.796c-.646.566-1.658.106-1.658-.753V3.204a1 1 0 0 1 1.659-.753l5.48 4.796a1 1 0 0 1 0 1.506z"/>'),Fv=Xu("CaretRightSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M5.795 12.456A.5.5 0 0 1 5.5 12V4a.5.5 0 0 1 .832-.374l4.5 4a.5.5 0 0 1 0 .748l-4.5 4a.5.5 0 0 1-.537.082z"/>'),Ev=Xu("CaretRightSquareFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm5.5 10a.5.5 0 0 0 .832.374l4.5-4a.5.5 0 0 0 0-.748l-4.5-4A.5.5 0 0 0 5.5 4v8z"/>'),$v=Xu("CaretUp",'<path d="M3.204 11h9.592L8 5.519 3.204 11zm-.753-.659 4.796-5.48a1 1 0 0 1 1.506 0l4.796 5.48c.566.647.106 1.659-.753 1.659H3.204a1 1 0 0 1-.753-1.659z"/>'),Rv=Xu("CaretUpFill",'<path d="m7.247 4.86-4.796 5.481c-.566.647-.106 1.659.753 1.659h9.592a1 1 0 0 0 .753-1.659l-4.796-5.48a1 1 0 0 0-1.506 0z"/>'),Nv=Xu("CaretUpSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M3.544 10.705A.5.5 0 0 0 4 11h8a.5.5 0 0 0 .374-.832l-4-4.5a.5.5 0 0 0-.748 0l-4 4.5a.5.5 0 0 0-.082.537z"/>'),Wv=Xu("CaretUpSquareFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm4 9h8a.5.5 0 0 0 .374-.832l-4-4.5a.5.5 0 0 0-.748 0l-4 4.5A.5.5 0 0 0 4 11z"/>'),Uv=Xu("Cart",'<path d="M0 1.5A.5.5 0 0 1 .5 1H2a.5.5 0 0 1 .485.379L2.89 3H14.5a.5.5 0 0 1 .491.592l-1.5 8A.5.5 0 0 1 13 12H4a.5.5 0 0 1-.491-.408L2.01 3.607 1.61 2H.5a.5.5 0 0 1-.5-.5zM3.102 4l1.313 7h8.17l1.313-7H3.102zM5 12a2 2 0 1 0 0 4 2 2 0 0 0 0-4zm7 0a2 2 0 1 0 0 4 2 2 0 0 0 0-4zm-7 1a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm7 0a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),Gv=Xu("Cart2",'<path d="M0 2.5A.5.5 0 0 1 .5 2H2a.5.5 0 0 1 .485.379L2.89 4H14.5a.5.5 0 0 1 .485.621l-1.5 6A.5.5 0 0 1 13 11H4a.5.5 0 0 1-.485-.379L1.61 3H.5a.5.5 0 0 1-.5-.5zM3.14 5l1.25 5h8.22l1.25-5H3.14zM5 13a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm-2 1a2 2 0 1 1 4 0 2 2 0 0 1-4 0zm9-1a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm-2 1a2 2 0 1 1 4 0 2 2 0 0 1-4 0z"/>'),qv=Xu("Cart3",'<path d="M0 1.5A.5.5 0 0 1 .5 1H2a.5.5 0 0 1 .485.379L2.89 3H14.5a.5.5 0 0 1 .49.598l-1 5a.5.5 0 0 1-.465.401l-9.397.472L4.415 11H13a.5.5 0 0 1 0 1H4a.5.5 0 0 1-.491-.408L2.01 3.607 1.61 2H.5a.5.5 0 0 1-.5-.5zM3.102 4l.84 4.479 9.144-.459L13.89 4H3.102zM5 12a2 2 0 1 0 0 4 2 2 0 0 0 0-4zm7 0a2 2 0 1 0 0 4 2 2 0 0 0 0-4zm-7 1a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm7 0a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),Jv=Xu("Cart4",'<path d="M0 2.5A.5.5 0 0 1 .5 2H2a.5.5 0 0 1 .485.379L2.89 4H14.5a.5.5 0 0 1 .485.621l-1.5 6A.5.5 0 0 1 13 11H4a.5.5 0 0 1-.485-.379L1.61 3H.5a.5.5 0 0 1-.5-.5zM3.14 5l.5 2H5V5H3.14zM6 5v2h2V5H6zm3 0v2h2V5H9zm3 0v2h1.36l.5-2H12zm1.11 3H12v2h.61l.5-2zM11 8H9v2h2V8zM8 8H6v2h2V8zM5 8H3.89l.5 2H5V8zm0 5a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm-2 1a2 2 0 1 1 4 0 2 2 0 0 1-4 0zm9-1a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm-2 1a2 2 0 1 1 4 0 2 2 0 0 1-4 0z"/>'),Kv=Xu("CartCheck",'<path d="M11.354 6.354a.5.5 0 0 0-.708-.708L8 8.293 6.854 7.146a.5.5 0 1 0-.708.708l1.5 1.5a.5.5 0 0 0 .708 0l3-3z"/><path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zm3.915 10L3.102 4h10.796l-1.313 7h-8.17zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),Xv=Xu("CartCheckFill",'<path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm-1.646-7.646-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L8 8.293l2.646-2.647a.5.5 0 0 1 .708.708z"/>'),Zv=Xu("CartDash",'<path d="M6.5 7a.5.5 0 0 0 0 1h4a.5.5 0 0 0 0-1h-4z"/><path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zm3.915 10L3.102 4h10.796l-1.313 7h-8.17zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),Qv=Xu("CartDashFill",'<path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zM6.5 7h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 1 0-1z"/>'),e_=Xu("CartFill",'<path d="M0 1.5A.5.5 0 0 1 .5 1H2a.5.5 0 0 1 .485.379L2.89 3H14.5a.5.5 0 0 1 .491.592l-1.5 8A.5.5 0 0 1 13 12H4a.5.5 0 0 1-.491-.408L2.01 3.607 1.61 2H.5a.5.5 0 0 1-.5-.5zM5 12a2 2 0 1 0 0 4 2 2 0 0 0 0-4zm7 0a2 2 0 1 0 0 4 2 2 0 0 0 0-4zm-7 1a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm7 0a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),t_=Xu("CartPlus",'<path d="M9 5.5a.5.5 0 0 0-1 0V7H6.5a.5.5 0 0 0 0 1H8v1.5a.5.5 0 0 0 1 0V8h1.5a.5.5 0 0 0 0-1H9V5.5z"/><path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zm3.915 10L3.102 4h10.796l-1.313 7h-8.17zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),a_=Xu("CartPlusFill",'<path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zM9 5.5V7h1.5a.5.5 0 0 1 0 1H9v1.5a.5.5 0 0 1-1 0V8H6.5a.5.5 0 0 1 0-1H8V5.5a.5.5 0 0 1 1 0z"/>'),n_=Xu("CartX",'<path d="M7.354 5.646a.5.5 0 1 0-.708.708L7.793 7.5 6.646 8.646a.5.5 0 1 0 .708.708L8.5 8.207l1.146 1.147a.5.5 0 0 0 .708-.708L9.207 7.5l1.147-1.146a.5.5 0 0 0-.708-.708L8.5 6.793 7.354 5.646z"/><path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zm3.915 10L3.102 4h10.796l-1.313 7h-8.17zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),r_=Xu("CartXFill",'<path d="M.5 1a.5.5 0 0 0 0 1h1.11l.401 1.607 1.498 7.985A.5.5 0 0 0 4 12h1a2 2 0 1 0 0 4 2 2 0 0 0 0-4h7a2 2 0 1 0 0 4 2 2 0 0 0 0-4h1a.5.5 0 0 0 .491-.408l1.5-8A.5.5 0 0 0 14.5 3H2.89l-.405-1.621A.5.5 0 0 0 2 1H.5zM6 14a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm7 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zM7.354 5.646 8.5 6.793l1.146-1.147a.5.5 0 0 1 .708.708L9.207 7.5l1.147 1.146a.5.5 0 0 1-.708.708L8.5 8.207 7.354 9.354a.5.5 0 1 1-.708-.708L7.793 7.5 6.646 6.354a.5.5 0 1 1 .708-.708z"/>'),i_=Xu("Cash",'<path d="M8 10a2 2 0 1 0 0-4 2 2 0 0 0 0 4z"/><path d="M0 4a1 1 0 0 1 1-1h14a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V4zm3 0a2 2 0 0 1-2 2v4a2 2 0 0 1 2 2h10a2 2 0 0 1 2-2V6a2 2 0 0 1-2-2H3z"/>'),o_=Xu("CashCoin",'<path fill-rule="evenodd" d="M11 15a4 4 0 1 0 0-8 4 4 0 0 0 0 8zm5-4a5 5 0 1 1-10 0 5 5 0 0 1 10 0z"/><path d="M9.438 11.944c.047.596.518 1.06 1.363 1.116v.44h.375v-.443c.875-.061 1.386-.529 1.386-1.207 0-.618-.39-.936-1.09-1.1l-.296-.07v-1.2c.376.043.614.248.671.532h.658c-.047-.575-.54-1.024-1.329-1.073V8.5h-.375v.45c-.747.073-1.255.522-1.255 1.158 0 .562.378.92 1.007 1.066l.248.061v1.272c-.384-.058-.639-.27-.696-.563h-.668zm1.36-1.354c-.369-.085-.569-.26-.569-.522 0-.294.216-.514.572-.578v1.1h-.003zm.432.746c.449.104.655.272.655.569 0 .339-.257.571-.709.614v-1.195l.054.012z"/><path d="M1 0a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h4.083c.058-.344.145-.678.258-1H3a2 2 0 0 0-2-2V3a2 2 0 0 0 2-2h10a2 2 0 0 0 2 2v3.528c.38.34.717.728 1 1.154V1a1 1 0 0 0-1-1H1z"/><path d="M9.998 5.083 10 5a2 2 0 1 0-3.132 1.65 5.982 5.982 0 0 1 3.13-1.567z"/>'),s_=Xu("CashStack",'<path d="M1 3a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1H1zm7 8a2 2 0 1 0 0-4 2 2 0 0 0 0 4z"/><path d="M0 5a1 1 0 0 1 1-1h14a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V5zm3 0a2 2 0 0 1-2 2v4a2 2 0 0 1 2 2h10a2 2 0 0 1 2-2V7a2 2 0 0 1-2-2H3z"/>'),l_=Xu("Cast",'<path d="m7.646 9.354-3.792 3.792a.5.5 0 0 0 .353.854h7.586a.5.5 0 0 0 .354-.854L8.354 9.354a.5.5 0 0 0-.708 0z"/><path d="M11.414 11H14.5a.5.5 0 0 0 .5-.5v-7a.5.5 0 0 0-.5-.5h-13a.5.5 0 0 0-.5.5v7a.5.5 0 0 0 .5.5h3.086l-1 1H1.5A1.5 1.5 0 0 1 0 10.5v-7A1.5 1.5 0 0 1 1.5 2h13A1.5 1.5 0 0 1 16 3.5v7a1.5 1.5 0 0 1-1.5 1.5h-2.086l-1-1z"/>'),c_=Xu("Chat",'<path d="M2.678 11.894a1 1 0 0 1 .287.801 10.97 10.97 0 0 1-.398 2c1.395-.323 2.247-.697 2.634-.893a1 1 0 0 1 .71-.074A8.06 8.06 0 0 0 8 14c3.996 0 7-2.807 7-6 0-3.192-3.004-6-7-6S1 4.808 1 8c0 1.468.617 2.83 1.678 3.894zm-.493 3.905a21.682 21.682 0 0 1-.713.129c-.2.032-.352-.176-.273-.362a9.68 9.68 0 0 0 .244-.637l.003-.01c.248-.72.45-1.548.524-2.319C.743 11.37 0 9.76 0 8c0-3.866 3.582-7 8-7s8 3.134 8 7-3.582 7-8 7a9.06 9.06 0 0 1-2.347-.306c-.52.263-1.639.742-3.468 1.105z"/>'),u_=Xu("ChatDots",'<path d="M5 8a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm3 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/><path d="m2.165 15.803.02-.004c1.83-.363 2.948-.842 3.468-1.105A9.06 9.06 0 0 0 8 15c4.418 0 8-3.134 8-7s-3.582-7-8-7-8 3.134-8 7c0 1.76.743 3.37 1.97 4.6a10.437 10.437 0 0 1-.524 2.318l-.003.011a10.722 10.722 0 0 1-.244.637c-.079.186.074.394.273.362a21.673 21.673 0 0 0 .693-.125zm.8-3.108a1 1 0 0 0-.287-.801C1.618 10.83 1 9.468 1 8c0-3.192 3.004-6 7-6s7 2.808 7 6c0 3.193-3.004 6-7 6a8.06 8.06 0 0 1-2.088-.272 1 1 0 0 0-.711.074c-.387.196-1.24.57-2.634.893a10.97 10.97 0 0 0 .398-2z"/>'),d_=Xu("ChatDotsFill",'<path d="M16 8c0 3.866-3.582 7-8 7a9.06 9.06 0 0 1-2.347-.306c-.584.296-1.925.864-4.181 1.234-.2.032-.352-.176-.273-.362.354-.836.674-1.95.77-2.966C.744 11.37 0 9.76 0 8c0-3.866 3.582-7 8-7s8 3.134 8 7zM5 8a1 1 0 1 0-2 0 1 1 0 0 0 2 0zm4 0a1 1 0 1 0-2 0 1 1 0 0 0 2 0zm3 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),h_=Xu("ChatFill",'<path d="M8 15c4.418 0 8-3.134 8-7s-3.582-7-8-7-8 3.134-8 7c0 1.76.743 3.37 1.97 4.6-.097 1.016-.417 2.13-.771 2.966-.079.186.074.394.273.362 2.256-.37 3.597-.938 4.18-1.234A9.06 9.06 0 0 0 8 15z"/>'),f_=Xu("ChatLeft",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H4.414A2 2 0 0 0 3 11.586l-2 2V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12.793a.5.5 0 0 0 .854.353l2.853-2.853A1 1 0 0 1 4.414 12H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/>'),p_=Xu("ChatLeftDots",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H4.414A2 2 0 0 0 3 11.586l-2 2V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12.793a.5.5 0 0 0 .854.353l2.853-2.853A1 1 0 0 1 4.414 12H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M5 6a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),m_=Xu("ChatLeftDotsFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H4.414a1 1 0 0 0-.707.293L.854 15.146A.5.5 0 0 1 0 14.793V2zm5 4a1 1 0 1 0-2 0 1 1 0 0 0 2 0zm4 0a1 1 0 1 0-2 0 1 1 0 0 0 2 0zm3 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),v_=Xu("ChatLeftFill",'<path d="M2 0a2 2 0 0 0-2 2v12.793a.5.5 0 0 0 .854.353l2.853-2.853A1 1 0 0 1 4.414 12H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/>'),__=Xu("ChatLeftQuote",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H4.414A2 2 0 0 0 3 11.586l-2 2V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12.793a.5.5 0 0 0 .854.353l2.853-2.853A1 1 0 0 1 4.414 12H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M7.066 4.76A1.665 1.665 0 0 0 4 5.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 1 0 .6.58c1.486-1.54 1.293-3.214.682-4.112zm4 0A1.665 1.665 0 0 0 8 5.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 1 0 .6.58c1.486-1.54 1.293-3.214.682-4.112z"/>'),g_=Xu("ChatLeftQuoteFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H4.414a1 1 0 0 0-.707.293L.854 15.146A.5.5 0 0 1 0 14.793V2zm7.194 2.766a1.688 1.688 0 0 0-.227-.272 1.467 1.467 0 0 0-.469-.324l-.008-.004A1.785 1.785 0 0 0 5.734 4C4.776 4 4 4.746 4 5.667c0 .92.776 1.666 1.734 1.666.343 0 .662-.095.931-.26-.137.389-.39.804-.81 1.22a.405.405 0 0 0 .011.59c.173.16.447.155.614-.01 1.334-1.329 1.37-2.758.941-3.706a2.461 2.461 0 0 0-.227-.4zM11 7.073c-.136.389-.39.804-.81 1.22a.405.405 0 0 0 .012.59c.172.16.446.155.613-.01 1.334-1.329 1.37-2.758.942-3.706a2.466 2.466 0 0 0-.228-.4 1.686 1.686 0 0 0-.227-.273 1.466 1.466 0 0 0-.469-.324l-.008-.004A1.785 1.785 0 0 0 10.07 4c-.957 0-1.734.746-1.734 1.667 0 .92.777 1.666 1.734 1.666.343 0 .662-.095.931-.26z"/>'),y_=Xu("ChatLeftText",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H4.414A2 2 0 0 0 3 11.586l-2 2V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12.793a.5.5 0 0 0 .854.353l2.853-2.853A1 1 0 0 1 4.414 12H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M3 3.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM3 6a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9A.5.5 0 0 1 3 6zm0 2.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5z"/>'),b_=Xu("ChatLeftTextFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H4.414a1 1 0 0 0-.707.293L.854 15.146A.5.5 0 0 1 0 14.793V2zm3.5 1a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9zm0 2.5a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9zm0 2.5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5z"/>'),M_=Xu("ChatQuote",'<path d="M2.678 11.894a1 1 0 0 1 .287.801 10.97 10.97 0 0 1-.398 2c1.395-.323 2.247-.697 2.634-.893a1 1 0 0 1 .71-.074A8.06 8.06 0 0 0 8 14c3.996 0 7-2.807 7-6 0-3.192-3.004-6-7-6S1 4.808 1 8c0 1.468.617 2.83 1.678 3.894zm-.493 3.905a21.682 21.682 0 0 1-.713.129c-.2.032-.352-.176-.273-.362a9.68 9.68 0 0 0 .244-.637l.003-.01c.248-.72.45-1.548.524-2.319C.743 11.37 0 9.76 0 8c0-3.866 3.582-7 8-7s8 3.134 8 7-3.582 7-8 7a9.06 9.06 0 0 1-2.347-.306c-.52.263-1.639.742-3.468 1.105z"/><path d="M7.066 6.76A1.665 1.665 0 0 0 4 7.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 0 0 .6.58c1.486-1.54 1.293-3.214.682-4.112zm4 0A1.665 1.665 0 0 0 8 7.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 0 0 .6.58c1.486-1.54 1.293-3.214.682-4.112z"/>'),z_=Xu("ChatQuoteFill",'<path d="M16 8c0 3.866-3.582 7-8 7a9.06 9.06 0 0 1-2.347-.306c-.584.296-1.925.864-4.181 1.234-.2.032-.352-.176-.273-.362.354-.836.674-1.95.77-2.966C.744 11.37 0 9.76 0 8c0-3.866 3.582-7 8-7s8 3.134 8 7zM7.194 6.766a1.688 1.688 0 0 0-.227-.272 1.467 1.467 0 0 0-.469-.324l-.008-.004A1.785 1.785 0 0 0 5.734 6C4.776 6 4 6.746 4 7.667c0 .92.776 1.666 1.734 1.666.343 0 .662-.095.931-.26-.137.389-.39.804-.81 1.22a.405.405 0 0 0 .011.59c.173.16.447.155.614-.01 1.334-1.329 1.37-2.758.941-3.706a2.461 2.461 0 0 0-.227-.4zM11 9.073c-.136.389-.39.804-.81 1.22a.405.405 0 0 0 .012.59c.172.16.446.155.613-.01 1.334-1.329 1.37-2.758.942-3.706a2.466 2.466 0 0 0-.228-.4 1.686 1.686 0 0 0-.227-.273 1.466 1.466 0 0 0-.469-.324l-.008-.004A1.785 1.785 0 0 0 10.07 6c-.957 0-1.734.746-1.734 1.667 0 .92.777 1.666 1.734 1.666.343 0 .662-.095.931-.26z"/>'),L_=Xu("ChatRight",'<path d="M2 1a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h9.586a2 2 0 0 1 1.414.586l2 2V2a1 1 0 0 0-1-1H2zm12-1a2 2 0 0 1 2 2v12.793a.5.5 0 0 1-.854.353l-2.853-2.853a1 1 0 0 0-.707-.293H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12z"/>'),w_=Xu("ChatRightDots",'<path d="M2 1a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h9.586a2 2 0 0 1 1.414.586l2 2V2a1 1 0 0 0-1-1H2zm12-1a2 2 0 0 1 2 2v12.793a.5.5 0 0 1-.854.353l-2.853-2.853a1 1 0 0 0-.707-.293H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12z"/><path d="M5 6a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),H_=Xu("ChatRightDotsFill",'<path d="M16 2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h9.586a1 1 0 0 1 .707.293l2.853 2.853a.5.5 0 0 0 .854-.353V2zM5 6a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm3 1a1 1 0 1 1 0-2 1 1 0 0 1 0 2z"/>'),k_=Xu("ChatRightFill",'<path d="M14 0a2 2 0 0 1 2 2v12.793a.5.5 0 0 1-.854.353l-2.853-2.853a1 1 0 0 0-.707-.293H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12z"/>'),A_=Xu("ChatRightQuote",'<path d="M2 1a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h9.586a2 2 0 0 1 1.414.586l2 2V2a1 1 0 0 0-1-1H2zm12-1a2 2 0 0 1 2 2v12.793a.5.5 0 0 1-.854.353l-2.853-2.853a1 1 0 0 0-.707-.293H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12z"/><path d="M7.066 4.76A1.665 1.665 0 0 0 4 5.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 1 0 .6.58c1.486-1.54 1.293-3.214.682-4.112zm4 0A1.665 1.665 0 0 0 8 5.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 1 0 .6.58c1.486-1.54 1.293-3.214.682-4.112z"/>'),S_=Xu("ChatRightQuoteFill",'<path d="M16 2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h9.586a1 1 0 0 1 .707.293l2.853 2.853a.5.5 0 0 0 .854-.353V2zM7.194 4.766c.087.124.163.26.227.401.428.948.393 2.377-.942 3.706a.446.446 0 0 1-.612.01.405.405 0 0 1-.011-.59c.419-.416.672-.831.809-1.22-.269.165-.588.26-.93.26C4.775 7.333 4 6.587 4 5.667 4 4.747 4.776 4 5.734 4c.271 0 .528.06.756.166l.008.004c.169.07.327.182.469.324.085.083.161.174.227.272zM11 7.073c-.269.165-.588.26-.93.26-.958 0-1.735-.746-1.735-1.666 0-.92.777-1.667 1.734-1.667.271 0 .528.06.756.166l.008.004c.17.07.327.182.469.324.085.083.161.174.227.272.087.124.164.26.228.401.428.948.392 2.377-.942 3.706a.446.446 0 0 1-.613.01.405.405 0 0 1-.011-.59c.42-.416.672-.831.81-1.22z"/>'),V_=Xu("ChatRightText",'<path d="M2 1a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h9.586a2 2 0 0 1 1.414.586l2 2V2a1 1 0 0 0-1-1H2zm12-1a2 2 0 0 1 2 2v12.793a.5.5 0 0 1-.854.353l-2.853-2.853a1 1 0 0 0-.707-.293H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12z"/><path d="M3 3.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM3 6a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9A.5.5 0 0 1 3 6zm0 2.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5z"/>'),O_=Xu("ChatRightTextFill",'<path d="M16 2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h9.586a1 1 0 0 1 .707.293l2.853 2.853a.5.5 0 0 0 .854-.353V2zM3.5 3h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1 0-1zm0 2.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1 0-1zm0 2.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1z"/>'),T_=Xu("ChatSquare",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1h-2.5a2 2 0 0 0-1.6.8L8 14.333 6.1 11.8a2 2 0 0 0-1.6-.8H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h2.5a1 1 0 0 1 .8.4l1.9 2.533a1 1 0 0 0 1.6 0l1.9-2.533a1 1 0 0 1 .8-.4H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/>'),D_=Xu("ChatSquareDots",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1h-2.5a2 2 0 0 0-1.6.8L8 14.333 6.1 11.8a2 2 0 0 0-1.6-.8H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h2.5a1 1 0 0 1 .8.4l1.9 2.533a1 1 0 0 0 1.6 0l1.9-2.533a1 1 0 0 1 .8-.4H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M5 6a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm4 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),C_=Xu("ChatSquareDotsFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2h-2.5a1 1 0 0 0-.8.4l-1.9 2.533a1 1 0 0 1-1.6 0L5.3 12.4a1 1 0 0 0-.8-.4H2a2 2 0 0 1-2-2V2zm5 4a1 1 0 1 0-2 0 1 1 0 0 0 2 0zm4 0a1 1 0 1 0-2 0 1 1 0 0 0 2 0zm3 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),j_=Xu("ChatSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h2.5a1 1 0 0 1 .8.4l1.9 2.533a1 1 0 0 0 1.6 0l1.9-2.533a1 1 0 0 1 .8-.4H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/>'),x_=Xu("ChatSquareQuote",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1h-2.5a2 2 0 0 0-1.6.8L8 14.333 6.1 11.8a2 2 0 0 0-1.6-.8H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h2.5a1 1 0 0 1 .8.4l1.9 2.533a1 1 0 0 0 1.6 0l1.9-2.533a1 1 0 0 1 .8-.4H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M7.066 4.76A1.665 1.665 0 0 0 4 5.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 1 0 .6.58c1.486-1.54 1.293-3.214.682-4.112zm4 0A1.665 1.665 0 0 0 8 5.668a1.667 1.667 0 0 0 2.561 1.406c-.131.389-.375.804-.777 1.22a.417.417 0 1 0 .6.58c1.486-1.54 1.293-3.214.682-4.112z"/>'),B_=Xu("ChatSquareQuoteFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2h-2.5a1 1 0 0 0-.8.4l-1.9 2.533a1 1 0 0 1-1.6 0L5.3 12.4a1 1 0 0 0-.8-.4H2a2 2 0 0 1-2-2V2zm7.194 2.766a1.688 1.688 0 0 0-.227-.272 1.467 1.467 0 0 0-.469-.324l-.008-.004A1.785 1.785 0 0 0 5.734 4C4.776 4 4 4.746 4 5.667c0 .92.776 1.666 1.734 1.666.343 0 .662-.095.931-.26-.137.389-.39.804-.81 1.22a.405.405 0 0 0 .011.59c.173.16.447.155.614-.01 1.334-1.329 1.37-2.758.941-3.706a2.461 2.461 0 0 0-.227-.4zM11 7.073c-.136.389-.39.804-.81 1.22a.405.405 0 0 0 .012.59c.172.16.446.155.613-.01 1.334-1.329 1.37-2.758.942-3.706a2.466 2.466 0 0 0-.228-.4 1.686 1.686 0 0 0-.227-.273 1.466 1.466 0 0 0-.469-.324l-.008-.004A1.785 1.785 0 0 0 10.07 4c-.957 0-1.734.746-1.734 1.667 0 .92.777 1.666 1.734 1.666.343 0 .662-.095.931-.26z"/>'),Y_=Xu("ChatSquareText",'<path d="M14 1a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1h-2.5a2 2 0 0 0-1.6.8L8 14.333 6.1 11.8a2 2 0 0 0-1.6-.8H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h2.5a1 1 0 0 1 .8.4l1.9 2.533a1 1 0 0 0 1.6 0l1.9-2.533a1 1 0 0 1 .8-.4H14a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M3 3.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM3 6a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9A.5.5 0 0 1 3 6zm0 2.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5z"/>'),P_=Xu("ChatSquareTextFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2h-2.5a1 1 0 0 0-.8.4l-1.9 2.533a1 1 0 0 1-1.6 0L5.3 12.4a1 1 0 0 0-.8-.4H2a2 2 0 0 1-2-2V2zm3.5 1a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9zm0 2.5a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9zm0 2.5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5z"/>'),I_=Xu("ChatText",'<path d="M2.678 11.894a1 1 0 0 1 .287.801 10.97 10.97 0 0 1-.398 2c1.395-.323 2.247-.697 2.634-.893a1 1 0 0 1 .71-.074A8.06 8.06 0 0 0 8 14c3.996 0 7-2.807 7-6 0-3.192-3.004-6-7-6S1 4.808 1 8c0 1.468.617 2.83 1.678 3.894zm-.493 3.905a21.682 21.682 0 0 1-.713.129c-.2.032-.352-.176-.273-.362a9.68 9.68 0 0 0 .244-.637l.003-.01c.248-.72.45-1.548.524-2.319C.743 11.37 0 9.76 0 8c0-3.866 3.582-7 8-7s8 3.134 8 7-3.582 7-8 7a9.06 9.06 0 0 1-2.347-.306c-.52.263-1.639.742-3.468 1.105z"/><path d="M4 5.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zM4 8a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7A.5.5 0 0 1 4 8zm0 2.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5z"/>'),F_=Xu("ChatTextFill",'<path d="M16 8c0 3.866-3.582 7-8 7a9.06 9.06 0 0 1-2.347-.306c-.584.296-1.925.864-4.181 1.234-.2.032-.352-.176-.273-.362.354-.836.674-1.95.77-2.966C.744 11.37 0 9.76 0 8c0-3.866 3.582-7 8-7s8 3.134 8 7zM4.5 5a.5.5 0 0 0 0 1h7a.5.5 0 0 0 0-1h-7zm0 2.5a.5.5 0 0 0 0 1h7a.5.5 0 0 0 0-1h-7zm0 2.5a.5.5 0 0 0 0 1h4a.5.5 0 0 0 0-1h-4z"/>'),E_=Xu("Check",'<path d="M10.97 4.97a.75.75 0 0 1 1.07 1.05l-3.99 4.99a.75.75 0 0 1-1.08.02L4.324 8.384a.75.75 0 1 1 1.06-1.06l2.094 2.093 3.473-4.425a.267.267 0 0 1 .02-.022z"/>'),$_=Xu("Check2",'<path d="M13.854 3.646a.5.5 0 0 1 0 .708l-7 7a.5.5 0 0 1-.708 0l-3.5-3.5a.5.5 0 1 1 .708-.708L6.5 10.293l6.646-6.647a.5.5 0 0 1 .708 0z"/>'),R_=Xu("Check2All",'<path d="M12.354 4.354a.5.5 0 0 0-.708-.708L5 10.293 1.854 7.146a.5.5 0 1 0-.708.708l3.5 3.5a.5.5 0 0 0 .708 0l7-7zm-4.208 7-.896-.897.707-.707.543.543 6.646-6.647a.5.5 0 0 1 .708.708l-7 7a.5.5 0 0 1-.708 0z"/><path d="m5.354 7.146.896.897-.707.707-.897-.896a.5.5 0 1 1 .708-.708z"/>'),N_=Xu("Check2Circle",'<path d="M2.5 8a5.5 5.5 0 0 1 8.25-4.764.5.5 0 0 0 .5-.866A6.5 6.5 0 1 0 14.5 8a.5.5 0 0 0-1 0 5.5 5.5 0 1 1-11 0z"/><path d="M15.354 3.354a.5.5 0 0 0-.708-.708L8 9.293 5.354 6.646a.5.5 0 1 0-.708.708l3 3a.5.5 0 0 0 .708 0l7-7z"/>'),W_=Xu("Check2Square",'<path d="M3 14.5A1.5 1.5 0 0 1 1.5 13V3A1.5 1.5 0 0 1 3 1.5h8a.5.5 0 0 1 0 1H3a.5.5 0 0 0-.5.5v10a.5.5 0 0 0 .5.5h10a.5.5 0 0 0 .5-.5V8a.5.5 0 0 1 1 0v5a1.5 1.5 0 0 1-1.5 1.5H3z"/><path d="m8.354 10.354 7-7a.5.5 0 0 0-.708-.708L8 9.293 5.354 6.646a.5.5 0 1 0-.708.708l3 3a.5.5 0 0 0 .708 0z"/>'),U_=Xu("CheckAll",'<path d="M8.97 4.97a.75.75 0 0 1 1.07 1.05l-3.99 4.99a.75.75 0 0 1-1.08.02L2.324 8.384a.75.75 0 1 1 1.06-1.06l2.094 2.093L8.95 4.992a.252.252 0 0 1 .02-.022zm-.92 5.14.92.92a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 1 0-1.091-1.028L9.477 9.417l-.485-.486-.943 1.179z"/>'),G_=Xu("CheckCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.97 4.97a.235.235 0 0 0-.02.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-1.071-1.05z"/>'),q_=Xu("CheckCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-3.97-3.03a.75.75 0 0 0-1.08.022L7.477 9.417 5.384 7.323a.75.75 0 0 0-1.06 1.06L6.97 11.03a.75.75 0 0 0 1.079-.02l3.992-4.99a.75.75 0 0 0-.01-1.05z"/>'),J_=Xu("CheckLg",'<path d="M13.485 1.431a1.473 1.473 0 0 1 2.104 2.062l-7.84 9.801a1.473 1.473 0 0 1-2.12.04L.431 8.138a1.473 1.473 0 0 1 2.084-2.083l4.111 4.112 6.82-8.69a.486.486 0 0 1 .04-.045z"/>'),K_=Xu("CheckSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M10.97 4.97a.75.75 0 0 1 1.071 1.05l-3.992 4.99a.75.75 0 0 1-1.08.02L4.324 8.384a.75.75 0 1 1 1.06-1.06l2.094 2.093 3.473-4.425a.235.235 0 0 1 .02-.022z"/>'),X_=Xu("CheckSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm10.03 4.97a.75.75 0 0 1 .011 1.05l-3.992 4.99a.75.75 0 0 1-1.08.02L4.324 8.384a.75.75 0 1 1 1.06-1.06l2.094 2.093 3.473-4.425a.75.75 0 0 1 1.08-.022z"/>'),Z_=Xu("ChevronBarContract",'<path fill-rule="evenodd" d="M3.646 14.854a.5.5 0 0 0 .708 0L8 11.207l3.646 3.647a.5.5 0 0 0 .708-.708l-4-4a.5.5 0 0 0-.708 0l-4 4a.5.5 0 0 0 0 .708zm0-13.708a.5.5 0 0 1 .708 0L8 4.793l3.646-3.647a.5.5 0 0 1 .708.708l-4 4a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 0-.708zM1 8a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13A.5.5 0 0 1 1 8z"/>'),Q_=Xu("ChevronBarDown",'<path fill-rule="evenodd" d="M3.646 4.146a.5.5 0 0 1 .708 0L8 7.793l3.646-3.647a.5.5 0 0 1 .708.708l-4 4a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 0-.708zM1 11.5a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13a.5.5 0 0 1-.5-.5z"/>'),eg=Xu("ChevronBarExpand",'<path fill-rule="evenodd" d="M3.646 10.146a.5.5 0 0 1 .708 0L8 13.793l3.646-3.647a.5.5 0 0 1 .708.708l-4 4a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 0-.708zm0-4.292a.5.5 0 0 0 .708 0L8 2.207l3.646 3.647a.5.5 0 0 0 .708-.708l-4-4a.5.5 0 0 0-.708 0l-4 4a.5.5 0 0 0 0 .708zM1 8a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13A.5.5 0 0 1 1 8z"/>'),tg=Xu("ChevronBarLeft",'<path fill-rule="evenodd" d="M11.854 3.646a.5.5 0 0 1 0 .708L8.207 8l3.647 3.646a.5.5 0 0 1-.708.708l-4-4a.5.5 0 0 1 0-.708l4-4a.5.5 0 0 1 .708 0zM4.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 1 0v-13a.5.5 0 0 0-.5-.5z"/>'),ag=Xu("ChevronBarRight",'<path fill-rule="evenodd" d="M4.146 3.646a.5.5 0 0 0 0 .708L7.793 8l-3.647 3.646a.5.5 0 0 0 .708.708l4-4a.5.5 0 0 0 0-.708l-4-4a.5.5 0 0 0-.708 0zM11.5 1a.5.5 0 0 1 .5.5v13a.5.5 0 0 1-1 0v-13a.5.5 0 0 1 .5-.5z"/>'),ng=Xu("ChevronBarUp",'<path fill-rule="evenodd" d="M3.646 11.854a.5.5 0 0 0 .708 0L8 8.207l3.646 3.647a.5.5 0 0 0 .708-.708l-4-4a.5.5 0 0 0-.708 0l-4 4a.5.5 0 0 0 0 .708zM2.4 5.2c0 .22.18.4.4.4h10.4a.4.4 0 0 0 0-.8H2.8a.4.4 0 0 0-.4.4z"/>'),rg=Xu("ChevronCompactDown",'<path fill-rule="evenodd" d="M1.553 6.776a.5.5 0 0 1 .67-.223L8 9.44l5.776-2.888a.5.5 0 1 1 .448.894l-6 3a.5.5 0 0 1-.448 0l-6-3a.5.5 0 0 1-.223-.67z"/>'),ig=Xu("ChevronCompactLeft",'<path fill-rule="evenodd" d="M9.224 1.553a.5.5 0 0 1 .223.67L6.56 8l2.888 5.776a.5.5 0 1 1-.894.448l-3-6a.5.5 0 0 1 0-.448l3-6a.5.5 0 0 1 .67-.223z"/>'),og=Xu("ChevronCompactRight",'<path fill-rule="evenodd" d="M6.776 1.553a.5.5 0 0 1 .671.223l3 6a.5.5 0 0 1 0 .448l-3 6a.5.5 0 1 1-.894-.448L9.44 8 6.553 2.224a.5.5 0 0 1 .223-.671z"/>'),sg=Xu("ChevronCompactUp",'<path fill-rule="evenodd" d="M7.776 5.553a.5.5 0 0 1 .448 0l6 3a.5.5 0 1 1-.448.894L8 6.56 2.224 9.447a.5.5 0 1 1-.448-.894l6-3z"/>'),lg=Xu("ChevronContract",'<path fill-rule="evenodd" d="M3.646 13.854a.5.5 0 0 0 .708 0L8 10.207l3.646 3.647a.5.5 0 0 0 .708-.708l-4-4a.5.5 0 0 0-.708 0l-4 4a.5.5 0 0 0 0 .708zm0-11.708a.5.5 0 0 1 .708 0L8 5.793l3.646-3.647a.5.5 0 0 1 .708.708l-4 4a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 0-.708z"/>'),cg=Xu("ChevronDoubleDown",'<path fill-rule="evenodd" d="M1.646 6.646a.5.5 0 0 1 .708 0L8 12.293l5.646-5.647a.5.5 0 0 1 .708.708l-6 6a.5.5 0 0 1-.708 0l-6-6a.5.5 0 0 1 0-.708z"/><path fill-rule="evenodd" d="M1.646 2.646a.5.5 0 0 1 .708 0L8 8.293l5.646-5.647a.5.5 0 0 1 .708.708l-6 6a.5.5 0 0 1-.708 0l-6-6a.5.5 0 0 1 0-.708z"/>'),ug=Xu("ChevronDoubleLeft",'<path fill-rule="evenodd" d="M8.354 1.646a.5.5 0 0 1 0 .708L2.707 8l5.647 5.646a.5.5 0 0 1-.708.708l-6-6a.5.5 0 0 1 0-.708l6-6a.5.5 0 0 1 .708 0z"/><path fill-rule="evenodd" d="M12.354 1.646a.5.5 0 0 1 0 .708L6.707 8l5.647 5.646a.5.5 0 0 1-.708.708l-6-6a.5.5 0 0 1 0-.708l6-6a.5.5 0 0 1 .708 0z"/>'),dg=Xu("ChevronDoubleRight",'<path fill-rule="evenodd" d="M3.646 1.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708L9.293 8 3.646 2.354a.5.5 0 0 1 0-.708z"/><path fill-rule="evenodd" d="M7.646 1.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708L13.293 8 7.646 2.354a.5.5 0 0 1 0-.708z"/>'),hg=Xu("ChevronDoubleUp",'<path fill-rule="evenodd" d="M7.646 2.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1-.708.708L8 3.707 2.354 9.354a.5.5 0 1 1-.708-.708l6-6z"/><path fill-rule="evenodd" d="M7.646 6.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1-.708.708L8 7.707l-5.646 5.647a.5.5 0 0 1-.708-.708l6-6z"/>'),fg=Xu("ChevronDown",'<path fill-rule="evenodd" d="M1.646 4.646a.5.5 0 0 1 .708 0L8 10.293l5.646-5.647a.5.5 0 0 1 .708.708l-6 6a.5.5 0 0 1-.708 0l-6-6a.5.5 0 0 1 0-.708z"/>'),pg=Xu("ChevronExpand",'<path fill-rule="evenodd" d="M3.646 9.146a.5.5 0 0 1 .708 0L8 12.793l3.646-3.647a.5.5 0 0 1 .708.708l-4 4a.5.5 0 0 1-.708 0l-4-4a.5.5 0 0 1 0-.708zm0-2.292a.5.5 0 0 0 .708 0L8 3.207l3.646 3.647a.5.5 0 0 0 .708-.708l-4-4a.5.5 0 0 0-.708 0l-4 4a.5.5 0 0 0 0 .708z"/>'),mg=Xu("ChevronLeft",'<path fill-rule="evenodd" d="M11.354 1.646a.5.5 0 0 1 0 .708L5.707 8l5.647 5.646a.5.5 0 0 1-.708.708l-6-6a.5.5 0 0 1 0-.708l6-6a.5.5 0 0 1 .708 0z"/>'),vg=Xu("ChevronRight",'<path fill-rule="evenodd" d="M4.646 1.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708L10.293 8 4.646 2.354a.5.5 0 0 1 0-.708z"/>'),_g=Xu("ChevronUp",'<path fill-rule="evenodd" d="M7.646 4.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1-.708.708L8 5.707l-5.646 5.647a.5.5 0 0 1-.708-.708l6-6z"/>'),gg=Xu("Circle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/>'),yg=Xu("CircleFill",'<circle cx="8" cy="8" r="8"/>'),bg=Xu("CircleHalf",'<path d="M8 15A7 7 0 1 0 8 1v14zm0 1A8 8 0 1 1 8 0a8 8 0 0 1 0 16z"/>'),Mg=Xu("CircleSquare",'<path d="M0 6a6 6 0 1 1 12 0A6 6 0 0 1 0 6z"/><path d="M12.93 5h1.57a.5.5 0 0 1 .5.5v9a.5.5 0 0 1-.5.5h-9a.5.5 0 0 1-.5-.5v-1.57a6.953 6.953 0 0 1-1-.22v1.79A1.5 1.5 0 0 0 5.5 16h9a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 4h-1.79c.097.324.17.658.22 1z"/>'),zg=Xu("Clipboard",'<path d="M4 1.5H3a2 2 0 0 0-2 2V14a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V3.5a2 2 0 0 0-2-2h-1v1h1a1 1 0 0 1 1 1V14a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3.5a1 1 0 0 1 1-1h1v-1z"/><path d="M9.5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5h3zm-3-1A1.5 1.5 0 0 0 5 1.5v1A1.5 1.5 0 0 0 6.5 4h3A1.5 1.5 0 0 0 11 2.5v-1A1.5 1.5 0 0 0 9.5 0h-3z"/>'),Lg=Xu("ClipboardCheck",'<path fill-rule="evenodd" d="M10.854 7.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 9.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M4 1.5H3a2 2 0 0 0-2 2V14a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V3.5a2 2 0 0 0-2-2h-1v1h1a1 1 0 0 1 1 1V14a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3.5a1 1 0 0 1 1-1h1v-1z"/><path d="M9.5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5h3zm-3-1A1.5 1.5 0 0 0 5 1.5v1A1.5 1.5 0 0 0 6.5 4h3A1.5 1.5 0 0 0 11 2.5v-1A1.5 1.5 0 0 0 9.5 0h-3z"/>'),wg=Xu("ClipboardData",'<path d="M4 11a1 1 0 1 1 2 0v1a1 1 0 1 1-2 0v-1zm6-4a1 1 0 1 1 2 0v5a1 1 0 1 1-2 0V7zM7 9a1 1 0 0 1 2 0v3a1 1 0 1 1-2 0V9z"/><path d="M4 1.5H3a2 2 0 0 0-2 2V14a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V3.5a2 2 0 0 0-2-2h-1v1h1a1 1 0 0 1 1 1V14a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3.5a1 1 0 0 1 1-1h1v-1z"/><path d="M9.5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5h3zm-3-1A1.5 1.5 0 0 0 5 1.5v1A1.5 1.5 0 0 0 6.5 4h3A1.5 1.5 0 0 0 11 2.5v-1A1.5 1.5 0 0 0 9.5 0h-3z"/>'),Hg=Xu("ClipboardMinus",'<path fill-rule="evenodd" d="M5.5 9.5A.5.5 0 0 1 6 9h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M4 1.5H3a2 2 0 0 0-2 2V14a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V3.5a2 2 0 0 0-2-2h-1v1h1a1 1 0 0 1 1 1V14a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3.5a1 1 0 0 1 1-1h1v-1z"/><path d="M9.5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5h3zm-3-1A1.5 1.5 0 0 0 5 1.5v1A1.5 1.5 0 0 0 6.5 4h3A1.5 1.5 0 0 0 11 2.5v-1A1.5 1.5 0 0 0 9.5 0h-3z"/>'),kg=Xu("ClipboardPlus",'<path fill-rule="evenodd" d="M8 7a.5.5 0 0 1 .5.5V9H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V10H6a.5.5 0 0 1 0-1h1.5V7.5A.5.5 0 0 1 8 7z"/><path d="M4 1.5H3a2 2 0 0 0-2 2V14a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V3.5a2 2 0 0 0-2-2h-1v1h1a1 1 0 0 1 1 1V14a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3.5a1 1 0 0 1 1-1h1v-1z"/><path d="M9.5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5h3zm-3-1A1.5 1.5 0 0 0 5 1.5v1A1.5 1.5 0 0 0 6.5 4h3A1.5 1.5 0 0 0 11 2.5v-1A1.5 1.5 0 0 0 9.5 0h-3z"/>'),Ag=Xu("ClipboardX",'<path fill-rule="evenodd" d="M6.146 7.146a.5.5 0 0 1 .708 0L8 8.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 9l1.147 1.146a.5.5 0 0 1-.708.708L8 9.707l-1.146 1.147a.5.5 0 0 1-.708-.708L7.293 9 6.146 7.854a.5.5 0 0 1 0-.708z"/><path d="M4 1.5H3a2 2 0 0 0-2 2V14a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V3.5a2 2 0 0 0-2-2h-1v1h1a1 1 0 0 1 1 1V14a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3.5a1 1 0 0 1 1-1h1v-1z"/><path d="M9.5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5h3zm-3-1A1.5 1.5 0 0 0 5 1.5v1A1.5 1.5 0 0 0 6.5 4h3A1.5 1.5 0 0 0 11 2.5v-1A1.5 1.5 0 0 0 9.5 0h-3z"/>'),Sg=Xu("Clock",'<path d="M8 3.5a.5.5 0 0 0-1 0V9a.5.5 0 0 0 .252.434l3.5 2a.5.5 0 0 0 .496-.868L8 8.71V3.5z"/><path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm7-8A7 7 0 1 1 1 8a7 7 0 0 1 14 0z"/>'),Vg=Xu("ClockFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM8 3.5a.5.5 0 0 0-1 0V9a.5.5 0 0 0 .252.434l3.5 2a.5.5 0 0 0 .496-.868L8 8.71V3.5z"/>'),Og=Xu("ClockHistory",'<path d="M8.515 1.019A7 7 0 0 0 8 1V0a8 8 0 0 1 .589.022l-.074.997zm2.004.45a7.003 7.003 0 0 0-.985-.299l.219-.976c.383.086.76.2 1.126.342l-.36.933zm1.37.71a7.01 7.01 0 0 0-.439-.27l.493-.87a8.025 8.025 0 0 1 .979.654l-.615.789a6.996 6.996 0 0 0-.418-.302zm1.834 1.79a6.99 6.99 0 0 0-.653-.796l.724-.69c.27.285.52.59.747.91l-.818.576zm.744 1.352a7.08 7.08 0 0 0-.214-.468l.893-.45a7.976 7.976 0 0 1 .45 1.088l-.95.313a7.023 7.023 0 0 0-.179-.483zm.53 2.507a6.991 6.991 0 0 0-.1-1.025l.985-.17c.067.386.106.778.116 1.17l-1 .025zm-.131 1.538c.033-.17.06-.339.081-.51l.993.123a7.957 7.957 0 0 1-.23 1.155l-.964-.267c.046-.165.086-.332.12-.501zm-.952 2.379c.184-.29.346-.594.486-.908l.914.405c-.16.36-.345.706-.555 1.038l-.845-.535zm-.964 1.205c.122-.122.239-.248.35-.378l.758.653a8.073 8.073 0 0 1-.401.432l-.707-.707z"/><path d="M8 1a7 7 0 1 0 4.95 11.95l.707.707A8.001 8.001 0 1 1 8 0v1z"/><path d="M7.5 3a.5.5 0 0 1 .5.5v5.21l3.248 1.856a.5.5 0 0 1-.496.868l-3.5-2A.5.5 0 0 1 7 9V3.5a.5.5 0 0 1 .5-.5z"/>'),Tg=Xu("Cloud",'<path d="M4.406 3.342A5.53 5.53 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773 16 11.569 14.502 13 12.687 13H3.781C1.708 13 0 11.366 0 9.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383zm.653.757c-.757.653-1.153 1.44-1.153 2.056v.448l-.445.049C2.064 6.805 1 7.952 1 9.318 1 10.785 2.23 12 3.781 12h8.906C13.98 12 15 10.988 15 9.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 4.825 10.328 3 8 3a4.53 4.53 0 0 0-2.941 1.1z"/>'),Dg=Xu("CloudArrowDown",'<path fill-rule="evenodd" d="M7.646 10.854a.5.5 0 0 0 .708 0l2-2a.5.5 0 0 0-.708-.708L8.5 9.293V5.5a.5.5 0 0 0-1 0v3.793L6.354 8.146a.5.5 0 1 0-.708.708l2 2z"/><path d="M4.406 3.342A5.53 5.53 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773 16 11.569 14.502 13 12.687 13H3.781C1.708 13 0 11.366 0 9.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383zm.653.757c-.757.653-1.153 1.44-1.153 2.056v.448l-.445.049C2.064 6.805 1 7.952 1 9.318 1 10.785 2.23 12 3.781 12h8.906C13.98 12 15 10.988 15 9.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 4.825 10.328 3 8 3a4.53 4.53 0 0 0-2.941 1.1z"/>'),Cg=Xu("CloudArrowDownFill",'<path d="M8 2a5.53 5.53 0 0 0-3.594 1.342c-.766.66-1.321 1.52-1.464 2.383C1.266 6.095 0 7.555 0 9.318 0 11.366 1.708 13 3.781 13h8.906C14.502 13 16 11.57 16 9.773c0-1.636-1.242-2.969-2.834-3.194C12.923 3.999 10.69 2 8 2zm2.354 6.854-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 1 1 .708-.708L7.5 9.293V5.5a.5.5 0 0 1 1 0v3.793l1.146-1.147a.5.5 0 0 1 .708.708z"/>'),jg=Xu("CloudArrowUp",'<path fill-rule="evenodd" d="M7.646 5.146a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1-.708.708L8.5 6.707V10.5a.5.5 0 0 1-1 0V6.707L6.354 7.854a.5.5 0 1 1-.708-.708l2-2z"/><path d="M4.406 3.342A5.53 5.53 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773 16 11.569 14.502 13 12.687 13H3.781C1.708 13 0 11.366 0 9.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383zm.653.757c-.757.653-1.153 1.44-1.153 2.056v.448l-.445.049C2.064 6.805 1 7.952 1 9.318 1 10.785 2.23 12 3.781 12h8.906C13.98 12 15 10.988 15 9.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 4.825 10.328 3 8 3a4.53 4.53 0 0 0-2.941 1.1z"/>'),xg=Xu("CloudArrowUpFill",'<path d="M8 2a5.53 5.53 0 0 0-3.594 1.342c-.766.66-1.321 1.52-1.464 2.383C1.266 6.095 0 7.555 0 9.318 0 11.366 1.708 13 3.781 13h8.906C14.502 13 16 11.57 16 9.773c0-1.636-1.242-2.969-2.834-3.194C12.923 3.999 10.69 2 8 2zm2.354 5.146a.5.5 0 0 1-.708.708L8.5 6.707V10.5a.5.5 0 0 1-1 0V6.707L6.354 7.854a.5.5 0 1 1-.708-.708l2-2a.5.5 0 0 1 .708 0l2 2z"/>'),Bg=Xu("CloudCheck",'<path fill-rule="evenodd" d="M10.354 6.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7 8.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M4.406 3.342A5.53 5.53 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773 16 11.569 14.502 13 12.687 13H3.781C1.708 13 0 11.366 0 9.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383zm.653.757c-.757.653-1.153 1.44-1.153 2.056v.448l-.445.049C2.064 6.805 1 7.952 1 9.318 1 10.785 2.23 12 3.781 12h8.906C13.98 12 15 10.988 15 9.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 4.825 10.328 3 8 3a4.53 4.53 0 0 0-2.941 1.1z"/>'),Yg=Xu("CloudCheckFill",'<path d="M8 2a5.53 5.53 0 0 0-3.594 1.342c-.766.66-1.321 1.52-1.464 2.383C1.266 6.095 0 7.555 0 9.318 0 11.366 1.708 13 3.781 13h8.906C14.502 13 16 11.57 16 9.773c0-1.636-1.242-2.969-2.834-3.194C12.923 3.999 10.69 2 8 2zm2.354 4.854-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7 8.793l2.646-2.647a.5.5 0 0 1 .708.708z"/>'),Pg=Xu("CloudDownload",'<path d="M4.406 1.342A5.53 5.53 0 0 1 8 0c2.69 0 4.923 2 5.166 4.579C14.758 4.804 16 6.137 16 7.773 16 9.569 14.502 11 12.687 11H10a.5.5 0 0 1 0-1h2.688C13.979 10 15 8.988 15 7.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 2.825 10.328 1 8 1a4.53 4.53 0 0 0-2.941 1.1c-.757.652-1.153 1.438-1.153 2.055v.448l-.445.049C2.064 4.805 1 5.952 1 7.318 1 8.785 2.23 10 3.781 10H6a.5.5 0 0 1 0 1H3.781C1.708 11 0 9.366 0 7.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383z"/><path d="M7.646 15.854a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 14.293V5.5a.5.5 0 0 0-1 0v8.793l-2.146-2.147a.5.5 0 0 0-.708.708l3 3z"/>'),Ig=Xu("CloudDownloadFill",'<path fill-rule="evenodd" d="M8 0a5.53 5.53 0 0 0-3.594 1.342c-.766.66-1.321 1.52-1.464 2.383C1.266 4.095 0 5.555 0 7.318 0 9.366 1.708 11 3.781 11H7.5V5.5a.5.5 0 0 1 1 0V11h4.188C14.502 11 16 9.57 16 7.773c0-1.636-1.242-2.969-2.834-3.194C12.923 1.999 10.69 0 8 0zm-.354 15.854a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 14.293V11h-1v3.293l-2.146-2.147a.5.5 0 0 0-.708.708l3 3z"/>'),Fg=Xu("CloudDrizzle",'<path d="M4.158 12.025a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm6 0a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm-3.5 1.5a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm6 0a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm.747-8.498a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 11H13a3 3 0 0 0 .405-5.973zM8.5 2a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 2z"/>'),Eg=Xu("CloudDrizzleFill",'<path d="M4.158 12.025a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm6 0a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm-3.5 1.5a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm6 0a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm.747-8.498a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 11H13a3 3 0 0 0 .405-5.973z"/>'),$g=Xu("CloudFill",'<path d="M4.406 3.342A5.53 5.53 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773 16 11.569 14.502 13 12.687 13H3.781C1.708 13 0 11.366 0 9.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383z"/>'),Rg=Xu("CloudFog",'<path d="M3 13.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm10.405-9.473a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 12H13a3 3 0 0 0 .405-5.973zM8.5 3a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 3z"/>'),Ng=Xu("CloudFog2",'<path d="M8.5 4a4.002 4.002 0 0 0-3.8 2.745.5.5 0 1 1-.949-.313 5.002 5.002 0 0 1 9.654.595A3 3 0 0 1 13 13H.5a.5.5 0 0 1 0-1H13a2 2 0 0 0 .001-4h-.026a.5.5 0 0 1-.5-.445A4 4 0 0 0 8.5 4zM0 8.5A.5.5 0 0 1 .5 8h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5z"/>'),Wg=Xu("CloudFog2Fill",'<path d="M8.5 3a5.001 5.001 0 0 1 4.905 4.027A3 3 0 0 1 13 13h-1.5a.5.5 0 0 0 0-1H1.05a3.51 3.51 0 0 1-.713-1H9.5a.5.5 0 0 0 0-1H.035a3.53 3.53 0 0 1 0-1H7.5a.5.5 0 0 0 0-1H.337a3.5 3.5 0 0 1 3.57-1.977A5.001 5.001 0 0 1 8.5 3z"/>'),Ug=Xu("CloudFogFill",'<path d="M3 13.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm10.405-9.473a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 12H13a3 3 0 0 0 .405-5.973z"/>'),Gg=Xu("CloudHail",'<path d="M13.405 4.527a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10.5H13a3 3 0 0 0 .405-5.973zM8.5 1.5a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1-.001 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 1.5zM3.75 15.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm.408-3.724a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zM7.75 15.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm.408-3.724a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm3.592 3.724a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm.408-3.724a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316z"/>'),qg=Xu("CloudHailFill",'<path d="M3.75 15.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm.408-3.724a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zM7.75 15.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm.408-3.724a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm3.592 3.724a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm.408-3.724a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm1.247-6.999a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10.5H13a3 3 0 0 0 .405-5.973z"/>'),Jg=Xu("CloudHaze",'<path d="M8.5 3a4.002 4.002 0 0 0-3.8 2.745.5.5 0 1 1-.949-.313 5.002 5.002 0 0 1 9.654.595A3 3 0 0 1 13 12H4.5a.5.5 0 0 1 0-1H13a2 2 0 0 0 .001-4h-.026a.5.5 0 0 1-.5-.445A4 4 0 0 0 8.5 3zM0 7.5A.5.5 0 0 1 .5 7h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm2 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm-2 4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5z"/>'),Kg=Xu("CloudHaze1",'<path d="M4 11.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm-3 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm2 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM13.405 4.027a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973zM8.5 1a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 1z"/>'),Xg=Xu("CloudHaze2Fill",'<path d="M8.5 2a5.001 5.001 0 0 1 4.905 4.027A3 3 0 0 1 13 12H3.5A3.5 3.5 0 0 1 .035 9H5.5a.5.5 0 0 0 0-1H.035a3.5 3.5 0 0 1 3.871-2.977A5.001 5.001 0 0 1 8.5 2zm-6 8a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9zM0 13.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5z"/>'),Zg=Xu("CloudHazeFill",'<path d="M4 11.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm-3 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm2 2a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM13.405 4.027a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973z"/>'),Qg=Xu("CloudLightning",'<path d="M13.405 4.027a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973zM8.5 1a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 1zM7.053 11.276A.5.5 0 0 1 7.5 11h1a.5.5 0 0 1 .474.658l-.28.842H9.5a.5.5 0 0 1 .39.812l-2 2.5a.5.5 0 0 1-.875-.433L7.36 14H6.5a.5.5 0 0 1-.447-.724l1-2z"/>'),ey=Xu("CloudLightningFill",'<path d="M7.053 11.276A.5.5 0 0 1 7.5 11h1a.5.5 0 0 1 .474.658l-.28.842H9.5a.5.5 0 0 1 .39.812l-2 2.5a.5.5 0 0 1-.875-.433L7.36 14H6.5a.5.5 0 0 1-.447-.724l1-2zm6.352-7.249a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973z"/>'),ty=Xu("CloudLightningRain",'<path d="M2.658 11.026a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm9.5 0a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm-7.5 1.5a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm9.5 0a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm-.753-8.499a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973zM8.5 1a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 1zM7.053 11.276A.5.5 0 0 1 7.5 11h1a.5.5 0 0 1 .474.658l-.28.842H9.5a.5.5 0 0 1 .39.812l-2 2.5a.5.5 0 0 1-.875-.433L7.36 14H6.5a.5.5 0 0 1-.447-.724l1-2z"/>'),ay=Xu("CloudLightningRainFill",'<path d="M2.658 11.026a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm9.5 0a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm-7.5 1.5a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm9.5 0a.5.5 0 0 1 .316.632l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.316zm-7.105-1.25A.5.5 0 0 1 7.5 11h1a.5.5 0 0 1 .474.658l-.28.842H9.5a.5.5 0 0 1 .39.812l-2 2.5a.5.5 0 0 1-.875-.433L7.36 14H6.5a.5.5 0 0 1-.447-.724l1-2zm6.352-7.249a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973z"/>'),ny=Xu("CloudMinus",'<path d="M4.406 3.342A5.53 5.53 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773 16 11.569 14.502 13 12.687 13H3.781C1.708 13 0 11.366 0 9.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383zm.653.757c-.757.653-1.153 1.44-1.153 2.056v.448l-.445.049C2.064 6.805 1 7.952 1 9.318 1 10.785 2.23 12 3.781 12h8.906C13.98 12 15 10.988 15 9.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 4.825 10.328 3 8 3a4.53 4.53 0 0 0-2.941 1.1z"/>'),ry=Xu("CloudMinusFill",'<path d="M8 2a5.53 5.53 0 0 0-3.594 1.342c-.766.66-1.321 1.52-1.464 2.383C1.266 6.095 0 7.555 0 9.318 0 11.366 1.708 13 3.781 13h8.906C14.502 13 16 11.57 16 9.773c0-1.636-1.242-2.969-2.834-3.194C12.923 3.999 10.69 2 8 2zM6 7.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1 0-1z"/>'),iy=Xu("CloudMoon",'<path d="M7 8a3.5 3.5 0 0 1 3.5 3.555.5.5 0 0 0 .625.492A1.503 1.503 0 0 1 13 13.5a1.5 1.5 0 0 1-1.5 1.5H3a2 2 0 1 1 .1-3.998.5.5 0 0 0 .509-.375A3.502 3.502 0 0 1 7 8zm4.473 3a4.5 4.5 0 0 0-8.72-.99A3 3 0 0 0 3 16h8.5a2.5 2.5 0 0 0 0-5h-.027z"/><path d="M11.286 1.778a.5.5 0 0 0-.565-.755 4.595 4.595 0 0 0-3.18 5.003 5.46 5.46 0 0 1 1.055.209A3.603 3.603 0 0 1 9.83 2.617a4.593 4.593 0 0 0 4.31 5.744 3.576 3.576 0 0 1-2.241.634c.162.317.295.652.394 1a4.59 4.59 0 0 0 3.624-2.04.5.5 0 0 0-.565-.755 3.593 3.593 0 0 1-4.065-5.422z"/>'),oy=Xu("CloudMoonFill",'<path d="M11.473 11a4.5 4.5 0 0 0-8.72-.99A3 3 0 0 0 3 16h8.5a2.5 2.5 0 0 0 0-5h-.027z"/><path d="M11.286 1.778a.5.5 0 0 0-.565-.755 4.595 4.595 0 0 0-3.18 5.003 5.46 5.46 0 0 1 1.055.209A3.603 3.603 0 0 1 9.83 2.617a4.593 4.593 0 0 0 4.31 5.744 3.576 3.576 0 0 1-2.241.634c.162.317.295.652.394 1a4.59 4.59 0 0 0 3.624-2.04.5.5 0 0 0-.565-.755 3.593 3.593 0 0 1-4.065-5.422z"/>'),sy=Xu("CloudPlus",'<path fill-rule="evenodd" d="M8 5.5a.5.5 0 0 1 .5.5v1.5H10a.5.5 0 0 1 0 1H8.5V10a.5.5 0 0 1-1 0V8.5H6a.5.5 0 0 1 0-1h1.5V6a.5.5 0 0 1 .5-.5z"/><path d="M4.406 3.342A5.53 5.53 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773 16 11.569 14.502 13 12.687 13H3.781C1.708 13 0 11.366 0 9.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383zm.653.757c-.757.653-1.153 1.44-1.153 2.056v.448l-.445.049C2.064 6.805 1 7.952 1 9.318 1 10.785 2.23 12 3.781 12h8.906C13.98 12 15 10.988 15 9.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 4.825 10.328 3 8 3a4.53 4.53 0 0 0-2.941 1.1z"/>'),ly=Xu("CloudPlusFill",'<path d="M8 2a5.53 5.53 0 0 0-3.594 1.342c-.766.66-1.321 1.52-1.464 2.383C1.266 6.095 0 7.555 0 9.318 0 11.366 1.708 13 3.781 13h8.906C14.502 13 16 11.57 16 9.773c0-1.636-1.242-2.969-2.834-3.194C12.923 3.999 10.69 2 8 2zm.5 4v1.5H10a.5.5 0 0 1 0 1H8.5V10a.5.5 0 0 1-1 0V8.5H6a.5.5 0 0 1 0-1h1.5V6a.5.5 0 0 1 1 0z"/>'),cy=Xu("CloudRain",'<path d="M4.158 12.025a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm3 0a.5.5 0 0 1 .316.633l-1 3a.5.5 0 0 1-.948-.316l1-3a.5.5 0 0 1 .632-.317zm3 0a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 0 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm3 0a.5.5 0 0 1 .316.633l-1 3a.5.5 0 1 1-.948-.316l1-3a.5.5 0 0 1 .632-.317zm.247-6.998a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 11H13a3 3 0 0 0 .405-5.973zM8.5 2a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 2z"/>'),uy=Xu("CloudRainFill",'<path d="M4.158 12.025a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm3 0a.5.5 0 0 1 .316.633l-1 3a.5.5 0 1 1-.948-.316l1-3a.5.5 0 0 1 .632-.317zm3 0a.5.5 0 0 1 .316.633l-.5 1.5a.5.5 0 1 1-.948-.316l.5-1.5a.5.5 0 0 1 .632-.317zm3 0a.5.5 0 0 1 .316.633l-1 3a.5.5 0 1 1-.948-.316l1-3a.5.5 0 0 1 .632-.317zm.247-6.998a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 11H13a3 3 0 0 0 .405-5.973z"/>'),dy=Xu("CloudRainHeavy",'<path d="M4.176 11.032a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 1 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm3 0a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 1 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm3 0a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 1 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm3 0a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 0 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm.229-7.005a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973zM8.5 1a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 1z"/>'),hy=Xu("CloudRainHeavyFill",'<path d="M4.176 11.032a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 0 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm3 0a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 0 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm3 0a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 0 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm3 0a.5.5 0 0 1 .292.643l-1.5 4a.5.5 0 0 1-.936-.35l1.5-4a.5.5 0 0 1 .644-.293zm.229-7.005a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973z"/>'),fy=Xu("CloudSlash",'<path fill-rule="evenodd" d="M3.112 5.112a3.125 3.125 0 0 0-.17.613C1.266 6.095 0 7.555 0 9.318 0 11.366 1.708 13 3.781 13H11l-1-1H3.781C2.231 12 1 10.785 1 9.318c0-1.365 1.064-2.513 2.46-2.666l.446-.05v-.447c0-.075.006-.152.018-.231l-.812-.812zm2.55-1.45-.725-.725A5.512 5.512 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773a3.2 3.2 0 0 1-1.516 2.711l-.733-.733C14.498 11.378 15 10.626 15 9.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 4.825 10.328 3 8 3c-.875 0-1.678.26-2.339.661z"/><path d="m13.646 14.354-12-12 .708-.708 12 12-.707.707z"/>'),py=Xu("CloudSlashFill",'<path fill-rule="evenodd" d="M3.112 5.112a3.125 3.125 0 0 0-.17.613C1.266 6.095 0 7.555 0 9.318 0 11.366 1.708 13 3.781 13H11L3.112 5.112zm11.372 7.372L4.937 2.937A5.512 5.512 0 0 1 8 2c2.69 0 4.923 2 5.166 4.579C14.758 6.804 16 8.137 16 9.773a3.2 3.2 0 0 1-1.516 2.711zm-.838 1.87-12-12 .708-.708 12 12-.707.707z"/>'),my=Xu("CloudSleet",'<path d="M13.405 4.027a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973zM8.5 1a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1 0 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 1zM2.375 13.5a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm1.849-2.447a.5.5 0 0 1 .223.67l-.5 1a.5.5 0 1 1-.894-.447l.5-1a.5.5 0 0 1 .67-.223zM6.375 13.5a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm1.849-2.447a.5.5 0 0 1 .223.67l-.5 1a.5.5 0 1 1-.894-.447l.5-1a.5.5 0 0 1 .67-.223zm2.151 2.447a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm1.849-2.447a.5.5 0 0 1 .223.67l-.5 1a.5.5 0 1 1-.894-.447l.5-1a.5.5 0 0 1 .67-.223z"/>'),vy=Xu("CloudSleetFill",'<path d="M2.375 13.5a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 1 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 1 1-.248-.434l.495-.283-.495-.283a.25.25 0 1 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm1.849-2.447a.5.5 0 0 1 .223.67l-.5 1a.5.5 0 0 1-.894-.447l.5-1a.5.5 0 0 1 .67-.223zM6.375 13.5a.25.25 0 0 1 .25.25v.57l.5-.287a.25.25 0 0 1 .249.434l-.495.283.495.283a.25.25 0 1 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 1 1-.248-.434l.495-.283-.495-.283a.25.25 0 1 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm1.849-2.447a.5.5 0 0 1 .223.67l-.5 1a.5.5 0 0 1-.894-.447l.5-1a.5.5 0 0 1 .67-.223zm2.151 2.447a.25.25 0 0 1 .25.25v.57l.5-.287a.25.25 0 0 1 .249.434l-.495.283.495.283a.25.25 0 1 1-.248.434l-.501-.286v.569a.25.25 0 0 1-.5 0v-.57l-.501.287a.25.25 0 1 1-.248-.434l.495-.283-.495-.283a.25.25 0 1 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm1.849-2.447a.5.5 0 0 1 .223.67l-.5 1a.5.5 0 1 1-.894-.447l.5-1a.5.5 0 0 1 .67-.223zm1.181-7.026a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10H13a3 3 0 0 0 .405-5.973z"/>'),_y=Xu("CloudSnow",'<path d="M13.405 4.277a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10.25H13a3 3 0 0 0 .405-5.973zM8.5 1.25a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1-.001 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 1.25zM2.625 11.5a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm2.75 2a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm5.5 0a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm-2.75-2a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm5.5 0a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25z"/>'),gy=Xu("CloudSnowFill",'<path d="M2.625 11.5a.25.25 0 0 1 .25.25v.57l.501-.287a.25.25 0 0 1 .248.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm2.75 2a.25.25 0 0 1 .25.25v.57l.5-.287a.25.25 0 0 1 .249.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm5.5 0a.25.25 0 0 1 .25.25v.57l.5-.287a.25.25 0 0 1 .249.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 0 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm-2.75-2a.25.25 0 0 1 .25.25v.57l.5-.287a.25.25 0 0 1 .249.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 1 1-.5 0v-.57l-.501.287a.25.25 0 0 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm5.5 0a.25.25 0 0 1 .25.25v.57l.5-.287a.25.25 0 0 1 .249.434l-.495.283.495.283a.25.25 0 0 1-.248.434l-.501-.286v.569a.25.25 0 0 1-.5 0v-.57l-.501.287a.25.25 0 1 1-.248-.434l.495-.283-.495-.283a.25.25 0 0 1 .248-.434l.501.286v-.569a.25.25 0 0 1 .25-.25zm-.22-7.223a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 10.25H13a3 3 0 0 0 .405-5.973z"/>'),yy=Xu("CloudSun",'<path d="M7 8a3.5 3.5 0 0 1 3.5 3.555.5.5 0 0 0 .624.492A1.503 1.503 0 0 1 13 13.5a1.5 1.5 0 0 1-1.5 1.5H3a2 2 0 1 1 .1-3.998.5.5 0 0 0 .51-.375A3.502 3.502 0 0 1 7 8zm4.473 3a4.5 4.5 0 0 0-8.72-.99A3 3 0 0 0 3 16h8.5a2.5 2.5 0 0 0 0-5h-.027z"/><path d="M10.5 1.5a.5.5 0 0 0-1 0v1a.5.5 0 0 0 1 0v-1zm3.743 1.964a.5.5 0 1 0-.707-.707l-.708.707a.5.5 0 0 0 .708.708l.707-.708zm-7.779-.707a.5.5 0 0 0-.707.707l.707.708a.5.5 0 1 0 .708-.708l-.708-.707zm1.734 3.374a2 2 0 1 1 3.296 2.198c.199.281.372.582.516.898a3 3 0 1 0-4.84-3.225c.352.011.696.055 1.028.129zm4.484 4.074c.6.215 1.125.59 1.522 1.072a.5.5 0 0 0 .039-.742l-.707-.707a.5.5 0 0 0-.854.377zM14.5 6.5a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1z"/>'),by=Xu("CloudSunFill",'<path d="M11.473 11a4.5 4.5 0 0 0-8.72-.99A3 3 0 0 0 3 16h8.5a2.5 2.5 0 0 0 0-5h-.027z"/><path d="M10.5 1.5a.5.5 0 0 0-1 0v1a.5.5 0 0 0 1 0v-1zm3.743 1.964a.5.5 0 1 0-.707-.707l-.708.707a.5.5 0 0 0 .708.708l.707-.708zm-7.779-.707a.5.5 0 0 0-.707.707l.707.708a.5.5 0 1 0 .708-.708l-.708-.707zm1.734 3.374a2 2 0 1 1 3.296 2.198c.199.281.372.582.516.898a3 3 0 1 0-4.84-3.225c.352.011.696.055 1.028.129zm4.484 4.074c.6.215 1.125.59 1.522 1.072a.5.5 0 0 0 .039-.742l-.707-.707a.5.5 0 0 0-.854.377zM14.5 6.5a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1z"/>'),My=Xu("CloudUpload",'<path fill-rule="evenodd" d="M4.406 1.342A5.53 5.53 0 0 1 8 0c2.69 0 4.923 2 5.166 4.579C14.758 4.804 16 6.137 16 7.773 16 9.569 14.502 11 12.687 11H10a.5.5 0 0 1 0-1h2.688C13.979 10 15 8.988 15 7.773c0-1.216-1.02-2.228-2.313-2.228h-.5v-.5C12.188 2.825 10.328 1 8 1a4.53 4.53 0 0 0-2.941 1.1c-.757.652-1.153 1.438-1.153 2.055v.448l-.445.049C2.064 4.805 1 5.952 1 7.318 1 8.785 2.23 10 3.781 10H6a.5.5 0 0 1 0 1H3.781C1.708 11 0 9.366 0 7.318c0-1.763 1.266-3.223 2.942-3.593.143-.863.698-1.723 1.464-2.383z"/><path fill-rule="evenodd" d="M7.646 4.146a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 5.707V14.5a.5.5 0 0 1-1 0V5.707L5.354 7.854a.5.5 0 1 1-.708-.708l3-3z"/>'),zy=Xu("CloudUploadFill",'<path fill-rule="evenodd" d="M8 0a5.53 5.53 0 0 0-3.594 1.342c-.766.66-1.321 1.52-1.464 2.383C1.266 4.095 0 5.555 0 7.318 0 9.366 1.708 11 3.781 11H7.5V5.707L5.354 7.854a.5.5 0 1 1-.708-.708l3-3a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 5.707V11h4.188C14.502 11 16 9.57 16 7.773c0-1.636-1.242-2.969-2.834-3.194C12.923 1.999 10.69 0 8 0zm-.5 14.5V11h1v3.5a.5.5 0 0 1-1 0z"/>'),Ly=Xu("Clouds",'<path d="M16 7.5a2.5 2.5 0 0 1-1.456 2.272 3.513 3.513 0 0 0-.65-.824 1.5 1.5 0 0 0-.789-2.896.5.5 0 0 1-.627-.421 3 3 0 0 0-5.22-1.625 5.587 5.587 0 0 0-1.276.088 4.002 4.002 0 0 1 7.392.91A2.5 2.5 0 0 1 16 7.5z"/><path d="M7 5a4.5 4.5 0 0 1 4.473 4h.027a2.5 2.5 0 0 1 0 5H3a3 3 0 0 1-.247-5.99A4.502 4.502 0 0 1 7 5zm3.5 4.5a3.5 3.5 0 0 0-6.89-.873.5.5 0 0 1-.51.375A2 2 0 1 0 3 13h8.5a1.5 1.5 0 1 0-.376-2.953.5.5 0 0 1-.624-.492V9.5z"/>'),wy=Xu("CloudsFill",'<path d="M11.473 9a4.5 4.5 0 0 0-8.72-.99A3 3 0 0 0 3 14h8.5a2.5 2.5 0 1 0-.027-5z"/><path d="M14.544 9.772a3.506 3.506 0 0 0-2.225-1.676 5.502 5.502 0 0 0-6.337-4.002 4.002 4.002 0 0 1 7.392.91 2.5 2.5 0 0 1 1.17 4.769z"/>'),Hy=Xu("Cloudy",'<path d="M13.405 8.527a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 14.5H13a3 3 0 0 0 .405-5.973zM8.5 5.5a4 4 0 0 1 3.976 3.555.5.5 0 0 0 .5.445H13a2 2 0 0 1-.001 4H3.5a2.5 2.5 0 1 1 .605-4.926.5.5 0 0 0 .596-.329A4.002 4.002 0 0 1 8.5 5.5z"/>'),ky=Xu("CloudyFill",'<path d="M13.405 7.027a5.001 5.001 0 0 0-9.499-1.004A3.5 3.5 0 1 0 3.5 13H13a3 3 0 0 0 .405-5.973z"/>'),Ay=Xu("Code",'<path d="M5.854 4.854a.5.5 0 1 0-.708-.708l-3.5 3.5a.5.5 0 0 0 0 .708l3.5 3.5a.5.5 0 0 0 .708-.708L2.707 8l3.147-3.146zm4.292 0a.5.5 0 0 1 .708-.708l3.5 3.5a.5.5 0 0 1 0 .708l-3.5 3.5a.5.5 0 0 1-.708-.708L13.293 8l-3.147-3.146z"/>'),Sy=Xu("CodeSlash",'<path d="M10.478 1.647a.5.5 0 1 0-.956-.294l-4 13a.5.5 0 0 0 .956.294l4-13zM4.854 4.146a.5.5 0 0 1 0 .708L1.707 8l3.147 3.146a.5.5 0 0 1-.708.708l-3.5-3.5a.5.5 0 0 1 0-.708l3.5-3.5a.5.5 0 0 1 .708 0zm6.292 0a.5.5 0 0 0 0 .708L14.293 8l-3.147 3.146a.5.5 0 0 0 .708.708l3.5-3.5a.5.5 0 0 0 0-.708l-3.5-3.5a.5.5 0 0 0-.708 0z"/>'),Vy=Xu("CodeSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M6.854 4.646a.5.5 0 0 1 0 .708L4.207 8l2.647 2.646a.5.5 0 0 1-.708.708l-3-3a.5.5 0 0 1 0-.708l3-3a.5.5 0 0 1 .708 0zm2.292 0a.5.5 0 0 0 0 .708L11.793 8l-2.647 2.646a.5.5 0 0 0 .708.708l3-3a.5.5 0 0 0 0-.708l-3-3a.5.5 0 0 0-.708 0z"/>'),Oy=Xu("Coin",'<path d="M5.5 9.511c.076.954.83 1.697 2.182 1.785V12h.6v-.709c1.4-.098 2.218-.846 2.218-1.932 0-.987-.626-1.496-1.745-1.76l-.473-.112V5.57c.6.068.982.396 1.074.85h1.052c-.076-.919-.864-1.638-2.126-1.716V4h-.6v.719c-1.195.117-2.01.836-2.01 1.853 0 .9.606 1.472 1.613 1.707l.397.098v2.034c-.615-.093-1.022-.43-1.114-.9H5.5zm2.177-2.166c-.59-.137-.91-.416-.91-.836 0-.47.345-.822.915-.925v1.76h-.005zm.692 1.193c.717.166 1.048.435 1.048.91 0 .542-.412.914-1.135.982V8.518l.087.02z"/><path fill-rule="evenodd" d="M8 15A7 7 0 1 0 8 1a7 7 0 0 0 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path fill-rule="evenodd" d="M8 13.5a5.5 5.5 0 1 0 0-11 5.5 5.5 0 0 0 0 11zm0 .5A6 6 0 1 0 8 2a6 6 0 0 0 0 12z"/>'),Ty=Xu("Collection",'<path d="M2.5 3.5a.5.5 0 0 1 0-1h11a.5.5 0 0 1 0 1h-11zm2-2a.5.5 0 0 1 0-1h7a.5.5 0 0 1 0 1h-7zM0 13a1.5 1.5 0 0 0 1.5 1.5h13A1.5 1.5 0 0 0 16 13V6a1.5 1.5 0 0 0-1.5-1.5h-13A1.5 1.5 0 0 0 0 6v7zm1.5.5A.5.5 0 0 1 1 13V6a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-.5.5h-13z"/>'),Dy=Xu("CollectionFill",'<path d="M0 13a1.5 1.5 0 0 0 1.5 1.5h13A1.5 1.5 0 0 0 16 13V6a1.5 1.5 0 0 0-1.5-1.5h-13A1.5 1.5 0 0 0 0 6v7zM2 3a.5.5 0 0 0 .5.5h11a.5.5 0 0 0 0-1h-11A.5.5 0 0 0 2 3zm2-2a.5.5 0 0 0 .5.5h7a.5.5 0 0 0 0-1h-7A.5.5 0 0 0 4 1z"/>'),Cy=Xu("CollectionPlay",'<path d="M2 3a.5.5 0 0 0 .5.5h11a.5.5 0 0 0 0-1h-11A.5.5 0 0 0 2 3zm2-2a.5.5 0 0 0 .5.5h7a.5.5 0 0 0 0-1h-7A.5.5 0 0 0 4 1zm2.765 5.576A.5.5 0 0 0 6 7v5a.5.5 0 0 0 .765.424l4-2.5a.5.5 0 0 0 0-.848l-4-2.5z"/><path d="M1.5 14.5A1.5 1.5 0 0 1 0 13V6a1.5 1.5 0 0 1 1.5-1.5h13A1.5 1.5 0 0 1 16 6v7a1.5 1.5 0 0 1-1.5 1.5h-13zm13-1a.5.5 0 0 0 .5-.5V6a.5.5 0 0 0-.5-.5h-13A.5.5 0 0 0 1 6v7a.5.5 0 0 0 .5.5h13z"/>'),jy=Xu("CollectionPlayFill",'<path d="M2.5 3.5a.5.5 0 0 1 0-1h11a.5.5 0 0 1 0 1h-11zm2-2a.5.5 0 0 1 0-1h7a.5.5 0 0 1 0 1h-7zM0 13a1.5 1.5 0 0 0 1.5 1.5h13A1.5 1.5 0 0 0 16 13V6a1.5 1.5 0 0 0-1.5-1.5h-13A1.5 1.5 0 0 0 0 6v7zm6.258-6.437a.5.5 0 0 1 .507.013l4 2.5a.5.5 0 0 1 0 .848l-4 2.5A.5.5 0 0 1 6 12V7a.5.5 0 0 1 .258-.437z"/>'),xy=Xu("Columns",'<path d="M0 2a1 1 0 0 1 1-1h14a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V2zm8.5 0v8H15V2H8.5zm0 9v3H15v-3H8.5zm-1-9H1v3h6.5V2zM1 14h6.5V6H1v8z"/>'),By=Xu("ColumnsGap",'<path d="M6 1v3H1V1h5zM1 0a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1V1a1 1 0 0 0-1-1H1zm14 12v3h-5v-3h5zm-5-1a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1h-5zM6 8v7H1V8h5zM1 7a1 1 0 0 0-1 1v7a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1V8a1 1 0 0 0-1-1H1zm14-6v7h-5V1h5zm-5-1a1 1 0 0 0-1 1v7a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1V1a1 1 0 0 0-1-1h-5z"/>'),Yy=Xu("Command",'<path d="M3.5 2A1.5 1.5 0 0 1 5 3.5V5H3.5a1.5 1.5 0 1 1 0-3zM6 5V3.5A2.5 2.5 0 1 0 3.5 6H5v4H3.5A2.5 2.5 0 1 0 6 12.5V11h4v1.5a2.5 2.5 0 1 0 2.5-2.5H11V6h1.5A2.5 2.5 0 1 0 10 3.5V5H6zm4 1v4H6V6h4zm1-1V3.5A1.5 1.5 0 1 1 12.5 5H11zm0 6h1.5a1.5 1.5 0 1 1-1.5 1.5V11zm-6 0v1.5A1.5 1.5 0 1 1 3.5 11H5z"/>'),Py=Xu("Compass",'<path d="M8 16.016a7.5 7.5 0 0 0 1.962-14.74A1 1 0 0 0 9 0H7a1 1 0 0 0-.962 1.276A7.5 7.5 0 0 0 8 16.016zm6.5-7.5a6.5 6.5 0 1 1-13 0 6.5 6.5 0 0 1 13 0z"/><path d="m6.94 7.44 4.95-2.83-2.83 4.95-4.949 2.83 2.828-4.95z"/>'),Iy=Xu("CompassFill",'<path d="M15.5 8.516a7.5 7.5 0 1 1-9.462-7.24A1 1 0 0 1 7 0h2a1 1 0 0 1 .962 1.276 7.503 7.503 0 0 1 5.538 7.24zm-3.61-3.905L6.94 7.439 4.11 12.39l4.95-2.828 2.828-4.95z"/>'),Fy=Xu("Cone",'<path d="M7.03 1.88c.252-1.01 1.688-1.01 1.94 0l2.905 11.62H14a.5.5 0 0 1 0 1H2a.5.5 0 0 1 0-1h2.125L7.03 1.88z"/>'),Ey=Xu("ConeStriped",'<path d="m9.97 4.88.953 3.811C10.159 8.878 9.14 9 8 9c-1.14 0-2.158-.122-2.923-.309L6.03 4.88C6.635 4.957 7.3 5 8 5s1.365-.043 1.97-.12zm-.245-.978L8.97.88C8.718-.13 7.282-.13 7.03.88L6.275 3.9C6.8 3.965 7.382 4 8 4c.618 0 1.2-.036 1.725-.098zm4.396 8.613a.5.5 0 0 1 .037.96l-6 2a.5.5 0 0 1-.316 0l-6-2a.5.5 0 0 1 .037-.96l2.391-.598.565-2.257c.862.212 1.964.339 3.165.339s2.303-.127 3.165-.339l.565 2.257 2.391.598z"/>'),$y=Xu("Controller",'<path d="M11.5 6.027a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm-1.5 1.5a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zm2.5-.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm-1.5 1.5a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zm-6.5-3h1v1h1v1h-1v1h-1v-1h-1v-1h1v-1z"/><path d="M3.051 3.26a.5.5 0 0 1 .354-.613l1.932-.518a.5.5 0 0 1 .62.39c.655-.079 1.35-.117 2.043-.117.72 0 1.443.041 2.12.126a.5.5 0 0 1 .622-.399l1.932.518a.5.5 0 0 1 .306.729c.14.09.266.19.373.297.408.408.78 1.05 1.095 1.772.32.733.599 1.591.805 2.466.206.875.34 1.78.364 2.606.024.816-.059 1.602-.328 2.21a1.42 1.42 0 0 1-1.445.83c-.636-.067-1.115-.394-1.513-.773-.245-.232-.496-.526-.739-.808-.126-.148-.25-.292-.368-.423-.728-.804-1.597-1.527-3.224-1.527-1.627 0-2.496.723-3.224 1.527-.119.131-.242.275-.368.423-.243.282-.494.575-.739.808-.398.38-.877.706-1.513.773a1.42 1.42 0 0 1-1.445-.83c-.27-.608-.352-1.395-.329-2.21.024-.826.16-1.73.365-2.606.206-.875.486-1.733.805-2.466.315-.722.687-1.364 1.094-1.772a2.34 2.34 0 0 1 .433-.335.504.504 0 0 1-.028-.079zm2.036.412c-.877.185-1.469.443-1.733.708-.276.276-.587.783-.885 1.465a13.748 13.748 0 0 0-.748 2.295 12.351 12.351 0 0 0-.339 2.406c-.022.755.062 1.368.243 1.776a.42.42 0 0 0 .426.24c.327-.034.61-.199.929-.502.212-.202.4-.423.615-.674.133-.156.276-.323.44-.504C4.861 9.969 5.978 9.027 8 9.027s3.139.942 3.965 1.855c.164.181.307.348.44.504.214.251.403.472.615.674.318.303.601.468.929.503a.42.42 0 0 0 .426-.241c.18-.408.265-1.02.243-1.776a12.354 12.354 0 0 0-.339-2.406 13.753 13.753 0 0 0-.748-2.295c-.298-.682-.61-1.19-.885-1.465-.264-.265-.856-.523-1.733-.708-.85-.179-1.877-.27-2.913-.27-1.036 0-2.063.091-2.913.27z"/>'),Ry=Xu("Cpu",'<path d="M5 0a.5.5 0 0 1 .5.5V2h1V.5a.5.5 0 0 1 1 0V2h1V.5a.5.5 0 0 1 1 0V2h1V.5a.5.5 0 0 1 1 0V2A2.5 2.5 0 0 1 14 4.5h1.5a.5.5 0 0 1 0 1H14v1h1.5a.5.5 0 0 1 0 1H14v1h1.5a.5.5 0 0 1 0 1H14v1h1.5a.5.5 0 0 1 0 1H14a2.5 2.5 0 0 1-2.5 2.5v1.5a.5.5 0 0 1-1 0V14h-1v1.5a.5.5 0 0 1-1 0V14h-1v1.5a.5.5 0 0 1-1 0V14h-1v1.5a.5.5 0 0 1-1 0V14A2.5 2.5 0 0 1 2 11.5H.5a.5.5 0 0 1 0-1H2v-1H.5a.5.5 0 0 1 0-1H2v-1H.5a.5.5 0 0 1 0-1H2v-1H.5a.5.5 0 0 1 0-1H2A2.5 2.5 0 0 1 4.5 2V.5A.5.5 0 0 1 5 0zm-.5 3A1.5 1.5 0 0 0 3 4.5v7A1.5 1.5 0 0 0 4.5 13h7a1.5 1.5 0 0 0 1.5-1.5v-7A1.5 1.5 0 0 0 11.5 3h-7zM5 6.5A1.5 1.5 0 0 1 6.5 5h3A1.5 1.5 0 0 1 11 6.5v3A1.5 1.5 0 0 1 9.5 11h-3A1.5 1.5 0 0 1 5 9.5v-3zM6.5 6a.5.5 0 0 0-.5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 0-.5-.5h-3z"/>'),Ny=Xu("CpuFill",'<path d="M6.5 6a.5.5 0 0 0-.5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 0-.5-.5h-3z"/><path d="M5.5.5a.5.5 0 0 0-1 0V2A2.5 2.5 0 0 0 2 4.5H.5a.5.5 0 0 0 0 1H2v1H.5a.5.5 0 0 0 0 1H2v1H.5a.5.5 0 0 0 0 1H2v1H.5a.5.5 0 0 0 0 1H2A2.5 2.5 0 0 0 4.5 14v1.5a.5.5 0 0 0 1 0V14h1v1.5a.5.5 0 0 0 1 0V14h1v1.5a.5.5 0 0 0 1 0V14h1v1.5a.5.5 0 0 0 1 0V14a2.5 2.5 0 0 0 2.5-2.5h1.5a.5.5 0 0 0 0-1H14v-1h1.5a.5.5 0 0 0 0-1H14v-1h1.5a.5.5 0 0 0 0-1H14v-1h1.5a.5.5 0 0 0 0-1H14A2.5 2.5 0 0 0 11.5 2V.5a.5.5 0 0 0-1 0V2h-1V.5a.5.5 0 0 0-1 0V2h-1V.5a.5.5 0 0 0-1 0V2h-1V.5zm1 4.5h3A1.5 1.5 0 0 1 11 6.5v3A1.5 1.5 0 0 1 9.5 11h-3A1.5 1.5 0 0 1 5 9.5v-3A1.5 1.5 0 0 1 6.5 5z"/>'),Wy=Xu("CreditCard",'<path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm2-1a1 1 0 0 0-1 1v1h14V4a1 1 0 0 0-1-1H2zm13 4H1v5a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V7z"/><path d="M2 10a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1v-1z"/>'),Uy=Xu("CreditCard2Back",'<path d="M11 5.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-1z"/><path d="M2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm13 2v5H1V4a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1zm-1 9H2a1 1 0 0 1-1-1v-1h14v1a1 1 0 0 1-1 1z"/>'),Gy=Xu("CreditCard2BackFill",'<path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5H0V4zm11.5 1a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h2a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-2zM0 11v1a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-1H0z"/>'),qy=Xu("CreditCard2Front",'<path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/><path d="M2 5.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-1zm0 3a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5zm3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5zm3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5zm3 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5z"/>'),Jy=Xu("CreditCard2FrontFill",'<path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm2.5 1a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h2a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-2zm0 3a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zm0 2a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm3 0a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm3 0a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm3 0a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1z"/>'),Ky=Xu("CreditCardFill",'<path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v1H0V4zm0 3v5a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V7H0zm3 2h1a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1v-1a1 1 0 0 1 1-1z"/>'),Xy=Xu("Crop",'<path d="M3.5.5A.5.5 0 0 1 4 1v13h13a.5.5 0 0 1 0 1h-2v2a.5.5 0 0 1-1 0v-2H3.5a.5.5 0 0 1-.5-.5V4H1a.5.5 0 0 1 0-1h2V1a.5.5 0 0 1 .5-.5zm2.5 3a.5.5 0 0 1 .5-.5h8a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-1 0V4H6.5a.5.5 0 0 1-.5-.5z"/>'),Zy=Xu("Cup",'<path d="M1 2a1 1 0 0 1 1-1h11a1 1 0 0 1 1 1v1h.5A1.5 1.5 0 0 1 16 4.5v7a1.5 1.5 0 0 1-1.5 1.5h-.55a2.5 2.5 0 0 1-2.45 2h-8A2.5 2.5 0 0 1 1 12.5V2zm13 10h.5a.5.5 0 0 0 .5-.5v-7a.5.5 0 0 0-.5-.5H14v8zM13 2H2v10.5A1.5 1.5 0 0 0 3.5 14h8a1.5 1.5 0 0 0 1.5-1.5V2z"/>'),Qy=Xu("CupFill",'<path d="M1 2a1 1 0 0 1 1-1h11a1 1 0 0 1 1 1v1h.5A1.5 1.5 0 0 1 16 4.5v7a1.5 1.5 0 0 1-1.5 1.5h-.55a2.5 2.5 0 0 1-2.45 2h-8A2.5 2.5 0 0 1 1 12.5V2zm13 10h.5a.5.5 0 0 0 .5-.5v-7a.5.5 0 0 0-.5-.5H14v8z"/>'),eb=Xu("CupStraw",'<path d="M13.902.334a.5.5 0 0 1-.28.65l-2.254.902-.4 1.927c.376.095.715.215.972.367.228.135.56.396.56.82 0 .046-.004.09-.011.132l-.962 9.068a1.28 1.28 0 0 1-.524.93c-.488.34-1.494.87-3.01.87-1.516 0-2.522-.53-3.01-.87a1.28 1.28 0 0 1-.524-.93L3.51 5.132A.78.78 0 0 1 3.5 5c0-.424.332-.685.56-.82.262-.154.607-.276.99-.372C5.824 3.614 6.867 3.5 8 3.5c.712 0 1.389.045 1.985.127l.464-2.215a.5.5 0 0 1 .303-.356l2.5-1a.5.5 0 0 1 .65.278zM9.768 4.607A13.991 13.991 0 0 0 8 4.5c-1.076 0-2.033.11-2.707.278A3.284 3.284 0 0 0 4.645 5c.146.073.362.15.648.222C5.967 5.39 6.924 5.5 8 5.5c.571 0 1.109-.03 1.588-.085l.18-.808zm.292 1.756C9.445 6.45 8.742 6.5 8 6.5c-1.133 0-2.176-.114-2.95-.308a5.514 5.514 0 0 1-.435-.127l.838 8.03c.013.121.06.186.102.215.357.249 1.168.69 2.438.69 1.27 0 2.081-.441 2.438-.69.042-.029.09-.094.102-.215l.852-8.03a5.517 5.517 0 0 1-.435.127 8.88 8.88 0 0 1-.89.17zM4.467 4.884s.003.002.005.006l-.005-.006zm7.066 0-.005.006c.002-.004.005-.006.005-.006zM11.354 5a3.174 3.174 0 0 0-.604-.21l-.099.445.055-.013c.286-.072.502-.149.648-.222z"/>'),tb=Xu("CurrencyBitcoin",'<path d="M5.5 13v1.25c0 .138.112.25.25.25h1a.25.25 0 0 0 .25-.25V13h.5v1.25c0 .138.112.25.25.25h1a.25.25 0 0 0 .25-.25V13h.084c1.992 0 3.416-1.033 3.416-2.82 0-1.502-1.007-2.323-2.186-2.44v-.088c.97-.242 1.683-.974 1.683-2.19C11.997 3.93 10.847 3 9.092 3H9V1.75a.25.25 0 0 0-.25-.25h-1a.25.25 0 0 0-.25.25V3h-.573V1.75a.25.25 0 0 0-.25-.25H5.75a.25.25 0 0 0-.25.25V3l-1.998.011a.25.25 0 0 0-.25.25v.989c0 .137.11.25.248.25l.755-.005a.75.75 0 0 1 .745.75v5.505a.75.75 0 0 1-.75.75l-.748.011a.25.25 0 0 0-.25.25v1c0 .138.112.25.25.25L5.5 13zm1.427-8.513h1.719c.906 0 1.438.498 1.438 1.312 0 .871-.575 1.362-1.877 1.362h-1.28V4.487zm0 4.051h1.84c1.137 0 1.756.58 1.756 1.524 0 .953-.626 1.45-2.158 1.45H6.927V8.539z"/>'),ab=Xu("CurrencyDollar",'<path d="M4 10.781c.148 1.667 1.513 2.85 3.591 3.003V15h1.043v-1.216c2.27-.179 3.678-1.438 3.678-3.3 0-1.59-.947-2.51-2.956-3.028l-.722-.187V3.467c1.122.11 1.879.714 2.07 1.616h1.47c-.166-1.6-1.54-2.748-3.54-2.875V1H7.591v1.233c-1.939.23-3.27 1.472-3.27 3.156 0 1.454.966 2.483 2.661 2.917l.61.162v4.031c-1.149-.17-1.94-.8-2.131-1.718H4zm3.391-3.836c-1.043-.263-1.6-.825-1.6-1.616 0-.944.704-1.641 1.8-1.828v3.495l-.2-.05zm1.591 1.872c1.287.323 1.852.859 1.852 1.769 0 1.097-.826 1.828-2.2 1.939V8.73l.348.086z"/>'),nb=Xu("CurrencyEuro",'<path d="M4 9.42h1.063C5.4 12.323 7.317 14 10.34 14c.622 0 1.167-.068 1.659-.185v-1.3c-.484.119-1.045.17-1.659.17-2.1 0-3.455-1.198-3.775-3.264h4.017v-.928H6.497v-.936c0-.11 0-.219.008-.329h4.078v-.927H6.618c.388-1.898 1.719-2.985 3.723-2.985.614 0 1.175.05 1.659.177V2.194A6.617 6.617 0 0 0 10.341 2c-2.928 0-4.82 1.569-5.244 4.3H4v.928h1.01v1.265H4v.928z"/>'),rb=Xu("CurrencyExchange",'<path d="M0 5a5.002 5.002 0 0 0 4.027 4.905 6.46 6.46 0 0 1 .544-2.073C3.695 7.536 3.132 6.864 3 5.91h-.5v-.426h.466V5.05c0-.046 0-.093.004-.135H2.5v-.427h.511C3.236 3.24 4.213 2.5 5.681 2.5c.316 0 .59.031.819.085v.733a3.46 3.46 0 0 0-.815-.082c-.919 0-1.538.466-1.734 1.252h1.917v.427h-1.98c-.003.046-.003.097-.003.147v.422h1.983v.427H3.93c.118.602.468 1.03 1.005 1.229a6.5 6.5 0 0 1 4.97-3.113A5.002 5.002 0 0 0 0 5zm16 5.5a5.5 5.5 0 1 1-11 0 5.5 5.5 0 0 1 11 0zm-7.75 1.322c.069.835.746 1.485 1.964 1.562V14h.54v-.62c1.259-.086 1.996-.74 1.996-1.69 0-.865-.563-1.31-1.57-1.54l-.426-.1V8.374c.54.06.884.347.966.745h.948c-.07-.804-.779-1.433-1.914-1.502V7h-.54v.629c-1.076.103-1.808.732-1.808 1.622 0 .787.544 1.288 1.45 1.493l.358.085v1.78c-.554-.08-.92-.376-1.003-.787H8.25zm1.96-1.895c-.532-.12-.82-.364-.82-.732 0-.41.311-.719.824-.809v1.54h-.005zm.622 1.044c.645.145.943.38.943.796 0 .474-.37.8-1.02.86v-1.674l.077.018z"/>'),ib=Xu("CurrencyPound",'<path d="M4 8.585h1.969c.115.465.186.939.186 1.43 0 1.385-.736 2.496-2.075 2.771V14H12v-1.24H6.492v-.129c.825-.525 1.135-1.446 1.135-2.694 0-.465-.07-.913-.168-1.352h3.29v-.972H7.22c-.186-.723-.372-1.455-.372-2.247 0-1.274 1.047-2.066 2.58-2.066a5.32 5.32 0 0 1 2.103.465V2.456A5.629 5.629 0 0 0 9.348 2C6.865 2 5.322 3.291 5.322 5.366c0 .775.195 1.515.399 2.247H4v.972z"/>'),ob=Xu("CurrencyYen",'<path d="M8.75 14v-2.629h2.446v-.967H8.75v-1.31h2.445v-.967H9.128L12.5 2h-1.699L8.047 7.327h-.086L5.207 2H3.5l3.363 6.127H4.778v.968H7.25v1.31H4.78v.966h2.47V14h1.502z"/>'),sb=Xu("Cursor",'<path d="M14.082 2.182a.5.5 0 0 1 .103.557L8.528 15.467a.5.5 0 0 1-.917-.007L5.57 10.694.803 8.652a.5.5 0 0 1-.006-.916l12.728-5.657a.5.5 0 0 1 .556.103zM2.25 8.184l3.897 1.67a.5.5 0 0 1 .262.263l1.67 3.897L12.743 3.52 2.25 8.184z"/>'),lb=Xu("CursorFill",'<path d="M14.082 2.182a.5.5 0 0 1 .103.557L8.528 15.467a.5.5 0 0 1-.917-.007L5.57 10.694.803 8.652a.5.5 0 0 1-.006-.916l12.728-5.657a.5.5 0 0 1 .556.103z"/>'),cb=Xu("CursorText",'<path d="M5 2a.5.5 0 0 1 .5-.5c.862 0 1.573.287 2.06.566.174.099.321.198.44.286.119-.088.266-.187.44-.286A4.165 4.165 0 0 1 10.5 1.5a.5.5 0 0 1 0 1c-.638 0-1.177.213-1.564.434a3.49 3.49 0 0 0-.436.294V7.5H9a.5.5 0 0 1 0 1h-.5v4.272c.1.08.248.187.436.294.387.221.926.434 1.564.434a.5.5 0 0 1 0 1 4.165 4.165 0 0 1-2.06-.566A4.561 4.561 0 0 1 8 13.65a4.561 4.561 0 0 1-.44.285 4.165 4.165 0 0 1-2.06.566.5.5 0 0 1 0-1c.638 0 1.177-.213 1.564-.434.188-.107.335-.214.436-.294V8.5H7a.5.5 0 0 1 0-1h.5V3.228a3.49 3.49 0 0 0-.436-.294A3.166 3.166 0 0 0 5.5 2.5.5.5 0 0 1 5 2zm3.352 1.355zm-.704 9.29z"/>'),ub=Xu("Dash",'<path d="M4 8a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7A.5.5 0 0 1 4 8z"/>'),db=Xu("DashCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4 8a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7A.5.5 0 0 1 4 8z"/>'),hb=Xu("DashCircleDotted",'<path d="M8 0c-.176 0-.35.006-.523.017l.064.998a7.117 7.117 0 0 1 .918 0l.064-.998A8.113 8.113 0 0 0 8 0zM6.44.152c-.346.069-.684.16-1.012.27l.321.948c.287-.098.582-.177.884-.237L6.44.153zm4.132.271a7.946 7.946 0 0 0-1.011-.27l-.194.98c.302.06.597.14.884.237l.321-.947zm1.873.925a8 8 0 0 0-.906-.524l-.443.896c.275.136.54.29.793.459l.556-.831zM4.46.824c-.314.155-.616.33-.905.524l.556.83a7.07 7.07 0 0 1 .793-.458L4.46.824zM2.725 1.985c-.262.23-.51.478-.74.74l.752.66c.202-.23.418-.446.648-.648l-.66-.752zm11.29.74a8.058 8.058 0 0 0-.74-.74l-.66.752c.23.202.447.418.648.648l.752-.66zm1.161 1.735a7.98 7.98 0 0 0-.524-.905l-.83.556c.169.253.322.518.458.793l.896-.443zM1.348 3.555c-.194.289-.37.591-.524.906l.896.443c.136-.275.29-.54.459-.793l-.831-.556zM.423 5.428a7.945 7.945 0 0 0-.27 1.011l.98.194c.06-.302.14-.597.237-.884l-.947-.321zM15.848 6.44a7.943 7.943 0 0 0-.27-1.012l-.948.321c.098.287.177.582.237.884l.98-.194zM.017 7.477a8.113 8.113 0 0 0 0 1.046l.998-.064a7.117 7.117 0 0 1 0-.918l-.998-.064zM16 8a8.1 8.1 0 0 0-.017-.523l-.998.064a7.11 7.11 0 0 1 0 .918l.998.064A8.1 8.1 0 0 0 16 8zM.152 9.56c.069.346.16.684.27 1.012l.948-.321a6.944 6.944 0 0 1-.237-.884l-.98.194zm15.425 1.012c.112-.328.202-.666.27-1.011l-.98-.194c-.06.302-.14.597-.237.884l.947.321zM.824 11.54a8 8 0 0 0 .524.905l.83-.556a6.999 6.999 0 0 1-.458-.793l-.896.443zm13.828.905c.194-.289.37-.591.524-.906l-.896-.443c-.136.275-.29.54-.459.793l.831.556zm-12.667.83c.23.262.478.51.74.74l.66-.752a7.047 7.047 0 0 1-.648-.648l-.752.66zm11.29.74c.262-.23.51-.478.74-.74l-.752-.66c-.201.23-.418.447-.648.648l.66.752zm-1.735 1.161c.314-.155.616-.33.905-.524l-.556-.83a7.07 7.07 0 0 1-.793.458l.443.896zm-7.985-.524c.289.194.591.37.906.524l.443-.896a6.998 6.998 0 0 1-.793-.459l-.556.831zm1.873.925c.328.112.666.202 1.011.27l.194-.98a6.953 6.953 0 0 1-.884-.237l-.321.947zm4.132.271a7.944 7.944 0 0 0 1.012-.27l-.321-.948a6.954 6.954 0 0 1-.884.237l.194.98zm-2.083.135a8.1 8.1 0 0 0 1.046 0l-.064-.998a7.11 7.11 0 0 1-.918 0l-.064.998zM4.5 7.5a.5.5 0 0 0 0 1h7a.5.5 0 0 0 0-1h-7z"/>'),fb=Xu("DashCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM4.5 7.5a.5.5 0 0 0 0 1h7a.5.5 0 0 0 0-1h-7z"/>'),pb=Xu("DashLg",'<path d="M0 8a1 1 0 0 1 1-1h14a1 1 0 1 1 0 2H1a1 1 0 0 1-1-1z"/>'),mb=Xu("DashSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M4 8a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7A.5.5 0 0 1 4 8z"/>'),vb=Xu("DashSquareDotted",'<path d="M2.5 0c-.166 0-.33.016-.487.048l.194.98A1.51 1.51 0 0 1 2.5 1h.458V0H2.5zm2.292 0h-.917v1h.917V0zm1.833 0h-.917v1h.917V0zm1.833 0h-.916v1h.916V0zm1.834 0h-.917v1h.917V0zm1.833 0h-.917v1h.917V0zM13.5 0h-.458v1h.458c.1 0 .199.01.293.029l.194-.981A2.51 2.51 0 0 0 13.5 0zm2.079 1.11a2.511 2.511 0 0 0-.69-.689l-.556.831c.164.11.305.251.415.415l.83-.556zM1.11.421a2.511 2.511 0 0 0-.689.69l.831.556c.11-.164.251-.305.415-.415L1.11.422zM16 2.5c0-.166-.016-.33-.048-.487l-.98.194c.018.094.028.192.028.293v.458h1V2.5zM.048 2.013A2.51 2.51 0 0 0 0 2.5v.458h1V2.5c0-.1.01-.199.029-.293l-.981-.194zM0 3.875v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zM0 5.708v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zM0 7.542v.916h1v-.916H0zm15 .916h1v-.916h-1v.916zM0 9.375v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zm-16 .916v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zm-16 .917v.458c0 .166.016.33.048.487l.98-.194A1.51 1.51 0 0 1 1 13.5v-.458H0zm16 .458v-.458h-1v.458c0 .1-.01.199-.029.293l.981.194c.032-.158.048-.32.048-.487zM.421 14.89c.183.272.417.506.69.689l.556-.831a1.51 1.51 0 0 1-.415-.415l-.83.556zm14.469.689c.272-.183.506-.417.689-.69l-.831-.556c-.11.164-.251.305-.415.415l.556.83zm-12.877.373c.158.032.32.048.487.048h.458v-1H2.5c-.1 0-.199-.01-.293-.029l-.194.981zM13.5 16c.166 0 .33-.016.487-.048l-.194-.98A1.51 1.51 0 0 1 13.5 15h-.458v1h.458zm-9.625 0h.917v-1h-.917v1zm1.833 0h.917v-1h-.917v1zm1.834 0h.916v-1h-.916v1zm1.833 0h.917v-1h-.917v1zm1.833 0h.917v-1h-.917v1zM4.5 7.5a.5.5 0 0 0 0 1h7a.5.5 0 0 0 0-1h-7z"/>'),_b=Xu("DashSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm2.5 7.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1 0-1z"/>'),gb=Xu("Diagram2",'<path fill-rule="evenodd" d="M6 3.5A1.5 1.5 0 0 1 7.5 2h1A1.5 1.5 0 0 1 10 3.5v1A1.5 1.5 0 0 1 8.5 6v1H11a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-1 0V8h-5v.5a.5.5 0 0 1-1 0v-1A.5.5 0 0 1 5 7h2.5V6A1.5 1.5 0 0 1 6 4.5v-1zM8.5 5a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1zM3 11.5A1.5 1.5 0 0 1 4.5 10h1A1.5 1.5 0 0 1 7 11.5v1A1.5 1.5 0 0 1 5.5 14h-1A1.5 1.5 0 0 1 3 12.5v-1zm1.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zm4.5.5a1.5 1.5 0 0 1 1.5-1.5h1a1.5 1.5 0 0 1 1.5 1.5v1a1.5 1.5 0 0 1-1.5 1.5h-1A1.5 1.5 0 0 1 9 12.5v-1zm1.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1z"/>'),yb=Xu("Diagram2Fill",'<path fill-rule="evenodd" d="M6 3.5A1.5 1.5 0 0 1 7.5 2h1A1.5 1.5 0 0 1 10 3.5v1A1.5 1.5 0 0 1 8.5 6v1H11a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-1 0V8h-5v.5a.5.5 0 0 1-1 0v-1A.5.5 0 0 1 5 7h2.5V6A1.5 1.5 0 0 1 6 4.5v-1zm-3 8A1.5 1.5 0 0 1 4.5 10h1A1.5 1.5 0 0 1 7 11.5v1A1.5 1.5 0 0 1 5.5 14h-1A1.5 1.5 0 0 1 3 12.5v-1zm6 0a1.5 1.5 0 0 1 1.5-1.5h1a1.5 1.5 0 0 1 1.5 1.5v1a1.5 1.5 0 0 1-1.5 1.5h-1A1.5 1.5 0 0 1 9 12.5v-1z"/>'),bb=Xu("Diagram3",'<path fill-rule="evenodd" d="M6 3.5A1.5 1.5 0 0 1 7.5 2h1A1.5 1.5 0 0 1 10 3.5v1A1.5 1.5 0 0 1 8.5 6v1H14a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-1 0V8h-5v.5a.5.5 0 0 1-1 0V8h-5v.5a.5.5 0 0 1-1 0v-1A.5.5 0 0 1 2 7h5.5V6A1.5 1.5 0 0 1 6 4.5v-1zM8.5 5a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1zM0 11.5A1.5 1.5 0 0 1 1.5 10h1A1.5 1.5 0 0 1 4 11.5v1A1.5 1.5 0 0 1 2.5 14h-1A1.5 1.5 0 0 1 0 12.5v-1zm1.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zm4.5.5A1.5 1.5 0 0 1 7.5 10h1a1.5 1.5 0 0 1 1.5 1.5v1A1.5 1.5 0 0 1 8.5 14h-1A1.5 1.5 0 0 1 6 12.5v-1zm1.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zm4.5.5a1.5 1.5 0 0 1 1.5-1.5h1a1.5 1.5 0 0 1 1.5 1.5v1a1.5 1.5 0 0 1-1.5 1.5h-1a1.5 1.5 0 0 1-1.5-1.5v-1zm1.5-.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1z"/>'),Mb=Xu("Diagram3Fill",'<path fill-rule="evenodd" d="M6 3.5A1.5 1.5 0 0 1 7.5 2h1A1.5 1.5 0 0 1 10 3.5v1A1.5 1.5 0 0 1 8.5 6v1H14a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-1 0V8h-5v.5a.5.5 0 0 1-1 0V8h-5v.5a.5.5 0 0 1-1 0v-1A.5.5 0 0 1 2 7h5.5V6A1.5 1.5 0 0 1 6 4.5v-1zm-6 8A1.5 1.5 0 0 1 1.5 10h1A1.5 1.5 0 0 1 4 11.5v1A1.5 1.5 0 0 1 2.5 14h-1A1.5 1.5 0 0 1 0 12.5v-1zm6 0A1.5 1.5 0 0 1 7.5 10h1a1.5 1.5 0 0 1 1.5 1.5v1A1.5 1.5 0 0 1 8.5 14h-1A1.5 1.5 0 0 1 6 12.5v-1zm6 0a1.5 1.5 0 0 1 1.5-1.5h1a1.5 1.5 0 0 1 1.5 1.5v1a1.5 1.5 0 0 1-1.5 1.5h-1a1.5 1.5 0 0 1-1.5-1.5v-1z"/>'),zb=Xu("Diamond",'<path d="M6.95.435c.58-.58 1.52-.58 2.1 0l6.515 6.516c.58.58.58 1.519 0 2.098L9.05 15.565c-.58.58-1.519.58-2.098 0L.435 9.05a1.482 1.482 0 0 1 0-2.098L6.95.435zm1.4.7a.495.495 0 0 0-.7 0L1.134 7.65a.495.495 0 0 0 0 .7l6.516 6.516a.495.495 0 0 0 .7 0l6.516-6.516a.495.495 0 0 0 0-.7L8.35 1.134z"/>'),Lb=Xu("DiamondFill",'<path fill-rule="evenodd" d="M6.95.435c.58-.58 1.52-.58 2.1 0l6.515 6.516c.58.58.58 1.519 0 2.098L9.05 15.565c-.58.58-1.519.58-2.098 0L.435 9.05a1.482 1.482 0 0 1 0-2.098L6.95.435z"/>'),wb=Xu("DiamondHalf",'<path d="M9.05.435c-.58-.58-1.52-.58-2.1 0L.436 6.95c-.58.58-.58 1.519 0 2.098l6.516 6.516c.58.58 1.519.58 2.098 0l6.516-6.516c.58-.58.58-1.519 0-2.098L9.05.435zM8 .989c.127 0 .253.049.35.145l6.516 6.516a.495.495 0 0 1 0 .7L8.35 14.866a.493.493 0 0 1-.35.145V.989z"/>'),Hb=Xu("Dice1",'<circle cx="8" cy="8" r="1.5"/><path d="M13 1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h10zM3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3z"/>'),kb=Xu("Dice1Fill",'<path d="M3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3zm5 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),Ab=Xu("Dice2",'<path d="M13 1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h10zM3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3z"/><path d="M5.5 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm8 8a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),Sb=Xu("Dice2Fill",'<path d="M0 3a3 3 0 0 1 3-3h10a3 3 0 0 1 3 3v10a3 3 0 0 1-3 3H3a3 3 0 0 1-3-3V3zm5.5 1a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0zm6.5 9.5a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/>'),Vb=Xu("Dice3",'<path d="M13 1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h10zM3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3z"/><path d="M5.5 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm8 8a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm-4-4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),Ob=Xu("Dice3Fill",'<path d="M3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3zm2.5 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm8 8a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zM8 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),Tb=Xu("Dice4",'<path d="M13 1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h10zM3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3z"/><path d="M5.5 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm8 0a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0 8a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm-8 0a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),Db=Xu("Dice4Fill",'<path d="M3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3zm1 5.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm8 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm1.5 6.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zM4 13.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),Cb=Xu("Dice5",'<path d="M13 1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h10zM3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3z"/><path d="M5.5 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm8 0a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0 8a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm-8 0a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm4-4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),jb=Xu("Dice5Fill",'<path d="M3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3zm2.5 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm8 0a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zM12 13.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zM5.5 12a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zM8 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),xb=Xu("Dice6",'<path d="M13 1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h10zM3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3z"/><path d="M5.5 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm8 0a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0 8a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0-4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm-8 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0-4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),Bb=Xu("Dice6Fill",'<path d="M3 0a3 3 0 0 0-3 3v10a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V3a3 3 0 0 0-3-3H3zm1 5.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm8 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm1.5 6.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zM12 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zM5.5 12a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zM4 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),Yb=Xu("Disc",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10 8a2 2 0 1 1-4 0 2 2 0 0 1 4 0zM8 4a4 4 0 0 0-4 4 .5.5 0 0 1-1 0 5 5 0 0 1 5-5 .5.5 0 0 1 0 1zm4.5 3.5a.5.5 0 0 1 .5.5 5 5 0 0 1-5 5 .5.5 0 0 1 0-1 4 4 0 0 0 4-4 .5.5 0 0 1 .5-.5z"/>'),Pb=Xu("DiscFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-6 0a2 2 0 1 0-4 0 2 2 0 0 0 4 0zM4 8a4 4 0 0 1 4-4 .5.5 0 0 0 0-1 5 5 0 0 0-5 5 .5.5 0 0 0 1 0zm9 0a.5.5 0 1 0-1 0 4 4 0 0 1-4 4 .5.5 0 0 0 0 1 5 5 0 0 0 5-5z"/>'),Ib=Xu("Discord",'<path d="M6.552 6.712c-.456 0-.816.4-.816.888s.368.888.816.888c.456 0 .816-.4.816-.888.008-.488-.36-.888-.816-.888zm2.92 0c-.456 0-.816.4-.816.888s.368.888.816.888c.456 0 .816-.4.816-.888s-.36-.888-.816-.888z"/><path d="M13.36 0H2.64C1.736 0 1 .736 1 1.648v10.816c0 .912.736 1.648 1.64 1.648h9.072l-.424-1.48 1.024.952.968.896L15 16V1.648C15 .736 14.264 0 13.36 0zm-3.088 10.448s-.288-.344-.528-.648c1.048-.296 1.448-.952 1.448-.952-.328.216-.64.368-.92.472-.4.168-.784.28-1.16.344a5.604 5.604 0 0 1-2.072-.008 6.716 6.716 0 0 1-1.176-.344 4.688 4.688 0 0 1-.584-.272c-.024-.016-.048-.024-.072-.04-.016-.008-.024-.016-.032-.024-.144-.08-.224-.136-.224-.136s.384.64 1.4.944c-.24.304-.536.664-.536.664-1.768-.056-2.44-1.216-2.44-1.216 0-2.576 1.152-4.664 1.152-4.664 1.152-.864 2.248-.84 2.248-.84l.08.096c-1.44.416-2.104 1.048-2.104 1.048s.176-.096.472-.232c.856-.376 1.536-.48 1.816-.504.048-.008.088-.016.136-.016a6.521 6.521 0 0 1 4.024.752s-.632-.6-1.992-1.016l.112-.128s1.096-.024 2.248.84c0 0 1.152 2.088 1.152 4.664 0 0-.68 1.16-2.448 1.216z"/>'),Fb=Xu("Display",'<path d="M0 4s0-2 2-2h12s2 0 2 2v6s0 2-2 2h-4c0 .667.083 1.167.25 1.5H11a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1h.75c.167-.333.25-.833.25-1.5H2s-2 0-2-2V4zm1.398-.855a.758.758 0 0 0-.254.302A1.46 1.46 0 0 0 1 4.01V10c0 .325.078.502.145.602.07.105.17.188.302.254a1.464 1.464 0 0 0 .538.143L2.01 11H14c.325 0 .502-.078.602-.145a.758.758 0 0 0 .254-.302 1.464 1.464 0 0 0 .143-.538L15 9.99V4c0-.325-.078-.502-.145-.602a.757.757 0 0 0-.302-.254A1.46 1.46 0 0 0 13.99 3H2c-.325 0-.502.078-.602.145z"/>'),Eb=Xu("DisplayFill",'<path d="M6 12c0 .667-.083 1.167-.25 1.5H5a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1h-.75c-.167-.333-.25-.833-.25-1.5h4c2 0 2-2 2-2V4c0-2-2-2-2-2H2C0 2 0 4 0 4v6c0 2 2 2 2 2h4z"/>'),$b=Xu("DistributeHorizontal",'<path fill-rule="evenodd" d="M14.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 1 0v-13a.5.5 0 0 0-.5-.5zm-13 0a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 1 0v-13a.5.5 0 0 0-.5-.5z"/><path d="M6 13a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1v10z"/>'),Rb=Xu("DistributeVertical",'<path fill-rule="evenodd" d="M1 1.5a.5.5 0 0 0 .5.5h13a.5.5 0 0 0 0-1h-13a.5.5 0 0 0-.5.5zm0 13a.5.5 0 0 0 .5.5h13a.5.5 0 0 0 0-1h-13a.5.5 0 0 0-.5.5z"/><path d="M2 7a1 1 0 0 1 1-1h10a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V7z"/>'),Nb=Xu("DoorClosed",'<path d="M3 2a1 1 0 0 1 1-1h8a1 1 0 0 1 1 1v13h1.5a.5.5 0 0 1 0 1h-13a.5.5 0 0 1 0-1H3V2zm1 13h8V2H4v13z"/><path d="M9 9a1 1 0 1 0 2 0 1 1 0 0 0-2 0z"/>'),Wb=Xu("DoorClosedFill",'<path d="M12 1a1 1 0 0 1 1 1v13h1.5a.5.5 0 0 1 0 1h-13a.5.5 0 0 1 0-1H3V2a1 1 0 0 1 1-1h8zm-2 9a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),Ub=Xu("DoorOpen",'<path d="M8.5 10c-.276 0-.5-.448-.5-1s.224-1 .5-1 .5.448.5 1-.224 1-.5 1z"/><path d="M10.828.122A.5.5 0 0 1 11 .5V1h.5A1.5 1.5 0 0 1 13 2.5V15h1.5a.5.5 0 0 1 0 1h-13a.5.5 0 0 1 0-1H3V1.5a.5.5 0 0 1 .43-.495l7-1a.5.5 0 0 1 .398.117zM11.5 2H11v13h1V2.5a.5.5 0 0 0-.5-.5zM4 1.934V15h6V1.077l-6 .857z"/>'),Gb=Xu("DoorOpenFill",'<path d="M1.5 15a.5.5 0 0 0 0 1h13a.5.5 0 0 0 0-1H13V2.5A1.5 1.5 0 0 0 11.5 1H11V.5a.5.5 0 0 0-.57-.495l-7 1A.5.5 0 0 0 3 1.5V15H1.5zM11 2h.5a.5.5 0 0 1 .5.5V15h-1V2zm-2.5 8c-.276 0-.5-.448-.5-1s.224-1 .5-1 .5.448.5 1-.224 1-.5 1z"/>'),qb=Xu("Dot",'<path d="M8 9.5a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/>'),Jb=Xu("Download",'<path d="M.5 9.9a.5.5 0 0 1 .5.5v2.5a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2.5a.5.5 0 0 1 1 0v2.5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2v-2.5a.5.5 0 0 1 .5-.5z"/><path d="M7.646 11.854a.5.5 0 0 0 .708 0l3-3a.5.5 0 0 0-.708-.708L8.5 10.293V1.5a.5.5 0 0 0-1 0v8.793L5.354 8.146a.5.5 0 1 0-.708.708l3 3z"/>'),Kb=Xu("Droplet",'<path fill-rule="evenodd" d="M7.21.8C7.69.295 8 0 8 0c.109.363.234.708.371 1.038.812 1.946 2.073 3.35 3.197 4.6C12.878 7.096 14 8.345 14 10a6 6 0 0 1-12 0C2 6.668 5.58 2.517 7.21.8zm.413 1.021A31.25 31.25 0 0 0 5.794 3.99c-.726.95-1.436 2.008-1.96 3.07C3.304 8.133 3 9.138 3 10a5 5 0 0 0 10 0c0-1.201-.796-2.157-2.181-3.7l-.03-.032C9.75 5.11 8.5 3.72 7.623 1.82z"/><path fill-rule="evenodd" d="M4.553 7.776c.82-1.641 1.717-2.753 2.093-3.13l.708.708c-.29.29-1.128 1.311-1.907 2.87l-.894-.448z"/>'),Xb=Xu("DropletFill",'<path fill-rule="evenodd" d="M8 16a6 6 0 0 0 6-6c0-1.655-1.122-2.904-2.432-4.362C10.254 4.176 8.75 2.503 8 0c0 0-6 5.686-6 10a6 6 0 0 0 6 6zM6.646 4.646c-.376.377-1.272 1.489-2.093 3.13l.894.448c.78-1.559 1.616-2.58 1.907-2.87l-.708-.708z"/>'),Zb=Xu("DropletHalf",'<path fill-rule="evenodd" d="M7.21.8C7.69.295 8 0 8 0c.109.363.234.708.371 1.038.812 1.946 2.073 3.35 3.197 4.6C12.878 7.096 14 8.345 14 10a6 6 0 0 1-12 0C2 6.668 5.58 2.517 7.21.8zm.413 1.021A31.25 31.25 0 0 0 5.794 3.99c-.726.95-1.436 2.008-1.96 3.07C3.304 8.133 3 9.138 3 10c0 0 2.5 1.5 5 .5s5-.5 5-.5c0-1.201-.796-2.157-2.181-3.7l-.03-.032C9.75 5.11 8.5 3.72 7.623 1.82z"/><path fill-rule="evenodd" d="M4.553 7.776c.82-1.641 1.717-2.753 2.093-3.13l.708.708c-.29.29-1.128 1.311-1.907 2.87l-.894-.448z"/>'),Qb=Xu("Earbuds",'<path fill-rule="evenodd" d="M6.825 4.138c.596 2.141-.36 3.593-2.389 4.117a4.432 4.432 0 0 1-2.018.054c-.048-.01.9 2.778 1.522 4.61l.41 1.205a.52.52 0 0 1-.346.659l-.593.19a.548.548 0 0 1-.69-.34L.184 6.99c-.696-2.137.662-4.309 2.564-4.8 2.029-.523 3.402 0 4.076 1.948zm-.868 2.221c.43-.112.561-.993.292-1.969-.269-.975-.836-1.675-1.266-1.563-.43.112-.561.994-.292 1.969.269.975.836 1.675 1.266 1.563zm3.218-2.221c-.596 2.141.36 3.593 2.389 4.117a4.434 4.434 0 0 0 2.018.054c.048-.01-.9 2.778-1.522 4.61l-.41 1.205a.52.52 0 0 0 .346.659l.593.19c.289.092.6-.06.69-.34l2.536-7.643c.696-2.137-.662-4.309-2.564-4.8-2.029-.523-3.402 0-4.076 1.948zm.868 2.221c-.43-.112-.561-.993-.292-1.969.269-.975.836-1.675 1.266-1.563.43.112.561.994.292 1.969-.269.975-.836 1.675-1.266 1.563z"/>'),eM=Xu("Easel",'<path d="M8 0a.5.5 0 0 1 .473.337L9.046 2H14a1 1 0 0 1 1 1v7a1 1 0 0 1-1 1h-1.85l1.323 3.837a.5.5 0 1 1-.946.326L11.092 11H8.5v3a.5.5 0 0 1-1 0v-3H4.908l-1.435 4.163a.5.5 0 1 1-.946-.326L3.85 11H2a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1h4.954L7.527.337A.5.5 0 0 1 8 0zM2 3v7h12V3H2z"/>'),tM=Xu("EaselFill",'<path d="M8.473.337a.5.5 0 0 0-.946 0L6.954 2H2a1 1 0 0 0-1 1v7a1 1 0 0 0 1 1h1.85l-1.323 3.837a.5.5 0 1 0 .946.326L4.908 11H7.5v2.5a.5.5 0 0 0 1 0V11h2.592l1.435 4.163a.5.5 0 0 0 .946-.326L12.15 11H14a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H9.046L8.473.337z"/>'),aM=Xu("Egg",'<path d="M8 15a5 5 0 0 1-5-5c0-1.956.69-4.286 1.742-6.12.524-.913 1.112-1.658 1.704-2.164C7.044 1.206 7.572 1 8 1c.428 0 .956.206 1.554.716.592.506 1.18 1.251 1.704 2.164C12.31 5.714 13 8.044 13 10a5 5 0 0 1-5 5zm0 1a6 6 0 0 0 6-6c0-4.314-3-10-6-10S2 5.686 2 10a6 6 0 0 0 6 6z"/>'),nM=Xu("EggFill",'<path d="M14 10a6 6 0 0 1-12 0C2 5.686 5 0 8 0s6 5.686 6 10z"/>'),rM=Xu("EggFried",'<path d="M8 11a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/><path d="M13.997 5.17a5 5 0 0 0-8.101-4.09A5 5 0 0 0 1.28 9.342a5 5 0 0 0 8.336 5.109 3.5 3.5 0 0 0 5.201-4.065 3.001 3.001 0 0 0-.822-5.216zm-1-.034a1 1 0 0 0 .668.977 2.001 2.001 0 0 1 .547 3.478 1 1 0 0 0-.341 1.113 2.5 2.5 0 0 1-3.715 2.905 1 1 0 0 0-1.262.152 4 4 0 0 1-6.67-4.087 1 1 0 0 0-.2-1 4 4 0 0 1 3.693-6.61 1 1 0 0 0 .8-.2 4 4 0 0 1 6.48 3.273z"/>'),iM=Xu("Eject",'<path d="M7.27 1.047a1 1 0 0 1 1.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H1.656C.78 9.5.326 8.455.926 7.816L7.27 1.047zM14.346 8.5 8 1.731 1.654 8.5h12.692zM.5 11.5a1 1 0 0 1 1-1h13a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1h-13a1 1 0 0 1-1-1v-1zm14 0h-13v1h13v-1z"/>'),oM=Xu("EjectFill",'<path d="M7.27 1.047a1 1 0 0 1 1.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H1.656C.78 9.5.326 8.455.926 7.816L7.27 1.047zM.5 11.5a1 1 0 0 1 1-1h13a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1h-13a1 1 0 0 1-1-1v-1z"/>'),sM=Xu("EmojiAngry",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4.285 12.433a.5.5 0 0 0 .683-.183A3.498 3.498 0 0 1 8 10.5c1.295 0 2.426.703 3.032 1.75a.5.5 0 0 0 .866-.5A4.498 4.498 0 0 0 8 9.5a4.5 4.5 0 0 0-3.898 2.25.5.5 0 0 0 .183.683zm6.991-8.38a.5.5 0 1 1 .448.894l-1.009.504c.176.27.285.64.285 1.049 0 .828-.448 1.5-1 1.5s-1-.672-1-1.5c0-.247.04-.48.11-.686a.502.502 0 0 1 .166-.761l2-1zm-6.552 0a.5.5 0 0 0-.448.894l1.009.504A1.94 1.94 0 0 0 5 6.5C5 7.328 5.448 8 6 8s1-.672 1-1.5c0-.247-.04-.48-.11-.686a.502.502 0 0 0-.166-.761l-2-1z"/>'),lM=Xu("EmojiAngryFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM4.053 4.276a.5.5 0 0 1 .67-.223l2 1a.5.5 0 0 1 .166.76c.071.206.111.44.111.687C7 7.328 6.552 8 6 8s-1-.672-1-1.5c0-.408.109-.778.285-1.049l-1.009-.504a.5.5 0 0 1-.223-.67zm.232 8.157a.5.5 0 0 1-.183-.683A4.498 4.498 0 0 1 8 9.5a4.5 4.5 0 0 1 3.898 2.25.5.5 0 1 1-.866.5A3.498 3.498 0 0 0 8 10.5a3.498 3.498 0 0 0-3.032 1.75.5.5 0 0 1-.683.183zM10 8c-.552 0-1-.672-1-1.5 0-.247.04-.48.11-.686a.502.502 0 0 1 .166-.761l2-1a.5.5 0 1 1 .448.894l-1.009.504c.176.27.285.64.285 1.049 0 .828-.448 1.5-1 1.5z"/>'),cM=Xu("EmojiDizzy",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M9.146 5.146a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 0 1 .708.708l-.647.646.647.646a.5.5 0 0 1-.708.708l-.646-.647-.646.647a.5.5 0 1 1-.708-.708l.647-.646-.647-.646a.5.5 0 0 1 0-.708zm-5 0a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 1 1 .708.708l-.647.646.647.646a.5.5 0 1 1-.708.708L5.5 7.207l-.646.647a.5.5 0 1 1-.708-.708l.647-.646-.647-.646a.5.5 0 0 1 0-.708zM10 11a2 2 0 1 1-4 0 2 2 0 0 1 4 0z"/>'),uM=Xu("EmojiDizzyFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM4.146 5.146a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 1 1 .708.708l-.647.646.647.646a.5.5 0 1 1-.708.708L5.5 7.207l-.646.647a.5.5 0 1 1-.708-.708l.647-.646-.647-.646a.5.5 0 0 1 0-.708zm5 0a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 0 1 .708.708l-.647.646.647.646a.5.5 0 0 1-.708.708l-.646-.647-.646.647a.5.5 0 1 1-.708-.708l.647-.646-.647-.646a.5.5 0 0 1 0-.708zM8 13a2 2 0 1 1 0-4 2 2 0 0 1 0 4z"/>'),dM=Xu("EmojiExpressionless",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4 10.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm5 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),hM=Xu("EmojiExpressionlessFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM4.5 6h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1zm5 0h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1zm-5 4h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1 0-1z"/>'),fM=Xu("EmojiFrown",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4.285 12.433a.5.5 0 0 0 .683-.183A3.498 3.498 0 0 1 8 10.5c1.295 0 2.426.703 3.032 1.75a.5.5 0 0 0 .866-.5A4.498 4.498 0 0 0 8 9.5a4.5 4.5 0 0 0-3.898 2.25.5.5 0 0 0 .183.683zM7 6.5C7 7.328 6.552 8 6 8s-1-.672-1-1.5S5.448 5 6 5s1 .672 1 1.5zm4 0c0 .828-.448 1.5-1 1.5s-1-.672-1-1.5S9.448 5 10 5s1 .672 1 1.5z"/>'),pM=Xu("EmojiFrownFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM7 6.5C7 7.328 6.552 8 6 8s-1-.672-1-1.5S5.448 5 6 5s1 .672 1 1.5zm-2.715 5.933a.5.5 0 0 1-.183-.683A4.498 4.498 0 0 1 8 9.5a4.5 4.5 0 0 1 3.898 2.25.5.5 0 0 1-.866.5A3.498 3.498 0 0 0 8 10.5a3.498 3.498 0 0 0-3.032 1.75.5.5 0 0 1-.683.183zM10 8c-.552 0-1-.672-1-1.5S9.448 5 10 5s1 .672 1 1.5S10.552 8 10 8z"/>'),mM=Xu("EmojiHeartEyes",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M11.315 10.014a.5.5 0 0 1 .548.736A4.498 4.498 0 0 1 7.965 13a4.498 4.498 0 0 1-3.898-2.25.5.5 0 0 1 .548-.736h.005l.017.005.067.015.252.055c.215.046.515.108.857.169.693.124 1.522.242 2.152.242.63 0 1.46-.118 2.152-.242a26.58 26.58 0 0 0 1.109-.224l.067-.015.017-.004.005-.002zM4.756 4.566c.763-1.424 4.02-.12.952 3.434-4.496-1.596-2.35-4.298-.952-3.434zm6.488 0c1.398-.864 3.544 1.838-.952 3.434-3.067-3.554.19-4.858.952-3.434z"/>'),vM=Xu("EmojiHeartEyesFill",'<path d="M8 0a8 8 0 1 0 0 16A8 8 0 0 0 8 0zM4.756 4.566c.763-1.424 4.02-.12.952 3.434-4.496-1.596-2.35-4.298-.952-3.434zm6.559 5.448a.5.5 0 0 1 .548.736A4.498 4.498 0 0 1 7.965 13a4.498 4.498 0 0 1-3.898-2.25.5.5 0 0 1 .548-.736h.005l.017.005.067.015.252.055c.215.046.515.108.857.169.693.124 1.522.242 2.152.242.63 0 1.46-.118 2.152-.242a26.58 26.58 0 0 0 1.109-.224l.067-.015.017-.004.005-.002zm-.07-5.448c1.397-.864 3.543 1.838-.953 3.434-3.067-3.554.19-4.858.952-3.434z"/>'),_M=Xu("EmojiLaughing",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M12.331 9.5a1 1 0 0 1 0 1A4.998 4.998 0 0 1 8 13a4.998 4.998 0 0 1-4.33-2.5A1 1 0 0 1 4.535 9h6.93a1 1 0 0 1 .866.5zM7 6.5c0 .828-.448 0-1 0s-1 .828-1 0S5.448 5 6 5s1 .672 1 1.5zm4 0c0 .828-.448 0-1 0s-1 .828-1 0S9.448 5 10 5s1 .672 1 1.5z"/>'),gM=Xu("EmojiLaughingFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM7 6.5c0 .501-.164.396-.415.235C6.42 6.629 6.218 6.5 6 6.5c-.218 0-.42.13-.585.235C5.164 6.896 5 7 5 6.5 5 5.672 5.448 5 6 5s1 .672 1 1.5zm5.331 3a1 1 0 0 1 0 1A4.998 4.998 0 0 1 8 13a4.998 4.998 0 0 1-4.33-2.5A1 1 0 0 1 4.535 9h6.93a1 1 0 0 1 .866.5zm-1.746-2.765C10.42 6.629 10.218 6.5 10 6.5c-.218 0-.42.13-.585.235C9.164 6.896 9 7 9 6.5c0-.828.448-1.5 1-1.5s1 .672 1 1.5c0 .501-.164.396-.415.235z"/>'),yM=Xu("EmojiNeutral",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4 10.5a.5.5 0 0 0 .5.5h7a.5.5 0 0 0 0-1h-7a.5.5 0 0 0-.5.5zm3-4C7 5.672 6.552 5 6 5s-1 .672-1 1.5S5.448 8 6 8s1-.672 1-1.5zm4 0c0-.828-.448-1.5-1-1.5s-1 .672-1 1.5S9.448 8 10 8s1-.672 1-1.5z"/>'),bM=Xu("EmojiNeutralFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM7 6.5C7 7.328 6.552 8 6 8s-1-.672-1-1.5S5.448 5 6 5s1 .672 1 1.5zm-3 4a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zM10 8c-.552 0-1-.672-1-1.5S9.448 5 10 5s1 .672 1 1.5S10.552 8 10 8z"/>'),MM=Xu("EmojiSmile",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4.285 9.567a.5.5 0 0 1 .683.183A3.498 3.498 0 0 0 8 11.5a3.498 3.498 0 0 0 3.032-1.75.5.5 0 1 1 .866.5A4.498 4.498 0 0 1 8 12.5a4.498 4.498 0 0 1-3.898-2.25.5.5 0 0 1 .183-.683zM7 6.5C7 7.328 6.552 8 6 8s-1-.672-1-1.5S5.448 5 6 5s1 .672 1 1.5zm4 0c0 .828-.448 1.5-1 1.5s-1-.672-1-1.5S9.448 5 10 5s1 .672 1 1.5z"/>'),zM=Xu("EmojiSmileFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM7 6.5C7 7.328 6.552 8 6 8s-1-.672-1-1.5S5.448 5 6 5s1 .672 1 1.5zM4.285 9.567a.5.5 0 0 1 .683.183A3.498 3.498 0 0 0 8 11.5a3.498 3.498 0 0 0 3.032-1.75.5.5 0 1 1 .866.5A4.498 4.498 0 0 1 8 12.5a4.498 4.498 0 0 1-3.898-2.25.5.5 0 0 1 .183-.683zM10 8c-.552 0-1-.672-1-1.5S9.448 5 10 5s1 .672 1 1.5S10.552 8 10 8z"/>'),LM=Xu("EmojiSmileUpsideDown",'<path d="M8 1a7 7 0 1 0 0 14A7 7 0 0 0 8 1zm0-1a8 8 0 1 1 0 16A8 8 0 0 1 8 0z"/><path d="M4.285 6.433a.5.5 0 0 0 .683-.183A3.498 3.498 0 0 1 8 4.5c1.295 0 2.426.703 3.032 1.75a.5.5 0 0 0 .866-.5A4.498 4.498 0 0 0 8 3.5a4.5 4.5 0 0 0-3.898 2.25.5.5 0 0 0 .183.683zM7 9.5C7 8.672 6.552 8 6 8s-1 .672-1 1.5.448 1.5 1 1.5 1-.672 1-1.5zm4 0c0-.828-.448-1.5-1-1.5s-1 .672-1 1.5.448 1.5 1 1.5 1-.672 1-1.5z"/>'),wM=Xu("EmojiSmileUpsideDownFill",'<path d="M8 0a8 8 0 1 1 0 16A8 8 0 0 1 8 0zM7 9.5C7 8.672 6.552 8 6 8s-1 .672-1 1.5.448 1.5 1 1.5 1-.672 1-1.5zM4.285 6.433a.5.5 0 0 0 .683-.183A3.498 3.498 0 0 1 8 4.5c1.295 0 2.426.703 3.032 1.75a.5.5 0 0 0 .866-.5A4.498 4.498 0 0 0 8 3.5a4.5 4.5 0 0 0-3.898 2.25.5.5 0 0 0 .183.683zM10 8c-.552 0-1 .672-1 1.5s.448 1.5 1 1.5 1-.672 1-1.5S10.552 8 10 8z"/>'),HM=Xu("EmojiSunglasses",'<path d="M4.968 9.75a.5.5 0 1 0-.866.5A4.498 4.498 0 0 0 8 12.5a4.5 4.5 0 0 0 3.898-2.25.5.5 0 1 0-.866-.5A3.498 3.498 0 0 1 8 11.5a3.498 3.498 0 0 1-3.032-1.75zM7 5.116V5a1 1 0 0 0-1-1H3.28a1 1 0 0 0-.97 1.243l.311 1.242A2 2 0 0 0 4.561 8H5a2 2 0 0 0 1.994-1.839A2.99 2.99 0 0 1 8 6c.393 0 .74.064 1.006.161A2 2 0 0 0 11 8h.438a2 2 0 0 0 1.94-1.515l.311-1.242A1 1 0 0 0 12.72 4H10a1 1 0 0 0-1 1v.116A4.22 4.22 0 0 0 8 5c-.35 0-.69.04-1 .116z"/><path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-1 0A7 7 0 1 0 1 8a7 7 0 0 0 14 0z"/>'),kM=Xu("EmojiSunglassesFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM2.31 5.243A1 1 0 0 1 3.28 4H6a1 1 0 0 1 1 1v.116A4.22 4.22 0 0 1 8 5c.35 0 .69.04 1 .116V5a1 1 0 0 1 1-1h2.72a1 1 0 0 1 .97 1.243l-.311 1.242A2 2 0 0 1 11.439 8H11a2 2 0 0 1-1.994-1.839A2.99 2.99 0 0 0 8 6c-.393 0-.74.064-1.006.161A2 2 0 0 1 5 8h-.438a2 2 0 0 1-1.94-1.515L2.31 5.243zM4.969 9.75A3.498 3.498 0 0 0 8 11.5a3.498 3.498 0 0 0 3.032-1.75.5.5 0 1 1 .866.5A4.498 4.498 0 0 1 8 12.5a4.498 4.498 0 0 1-3.898-2.25.5.5 0 0 1 .866-.5z"/>'),AM=Xu("EmojiWink",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4.285 9.567a.5.5 0 0 1 .683.183A3.498 3.498 0 0 0 8 11.5a3.498 3.498 0 0 0 3.032-1.75.5.5 0 1 1 .866.5A4.498 4.498 0 0 1 8 12.5a4.498 4.498 0 0 1-3.898-2.25.5.5 0 0 1 .183-.683zM7 6.5C7 7.328 6.552 8 6 8s-1-.672-1-1.5S5.448 5 6 5s1 .672 1 1.5zm1.757-.437a.5.5 0 0 1 .68.194.934.934 0 0 0 .813.493c.339 0 .645-.19.813-.493a.5.5 0 1 1 .874.486A1.934 1.934 0 0 1 10.25 7.75c-.73 0-1.356-.412-1.687-1.007a.5.5 0 0 1 .194-.68z"/>'),SM=Xu("EmojiWinkFill",'<path d="M8 0a8 8 0 1 1 0 16A8 8 0 0 1 8 0zM7 6.5C7 5.672 6.552 5 6 5s-1 .672-1 1.5S5.448 8 6 8s1-.672 1-1.5zM4.285 9.567a.5.5 0 0 0-.183.683A4.498 4.498 0 0 0 8 12.5a4.5 4.5 0 0 0 3.898-2.25.5.5 0 1 0-.866-.5A3.498 3.498 0 0 1 8 11.5a3.498 3.498 0 0 1-3.032-1.75.5.5 0 0 0-.683-.183zm5.152-3.31a.5.5 0 0 0-.874.486c.33.595.958 1.007 1.687 1.007.73 0 1.356-.412 1.687-1.007a.5.5 0 0 0-.874-.486.934.934 0 0 1-.813.493.934.934 0 0 1-.813-.493z"/>'),VM=Xu("Envelope",'<path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm2-1a1 1 0 0 0-1 1v.217l7 4.2 7-4.2V4a1 1 0 0 0-1-1H2zm13 2.383-4.758 2.855L15 11.114v-5.73zm-.034 6.878L9.271 8.82 8 9.583 6.728 8.82l-5.694 3.44A1 1 0 0 0 2 13h12a1 1 0 0 0 .966-.739zM1 11.114l4.758-2.876L1 5.383v5.73z"/>'),OM=Xu("EnvelopeFill",'<path d="M.05 3.555A2 2 0 0 1 2 2h12a2 2 0 0 1 1.95 1.555L8 8.414.05 3.555zM0 4.697v7.104l5.803-3.558L0 4.697zM6.761 8.83l-6.57 4.027A2 2 0 0 0 2 14h12a2 2 0 0 0 1.808-1.144l-6.57-4.027L8 9.586l-1.239-.757zm3.436-.586L16 11.801V4.697l-5.803 3.546z"/>'),TM=Xu("EnvelopeOpen",'<path d="M8.47 1.318a1 1 0 0 0-.94 0l-6 3.2A1 1 0 0 0 1 5.4v.818l5.724 3.465L8 8.917l1.276.766L15 6.218V5.4a1 1 0 0 0-.53-.882l-6-3.2zM15 7.388l-4.754 2.877L15 13.117v-5.73zm-.035 6.874L8 10.083l-6.965 4.18A1 1 0 0 0 2 15h12a1 1 0 0 0 .965-.738zM1 13.117l4.754-2.852L1 7.387v5.73zM7.059.435a2 2 0 0 1 1.882 0l6 3.2A2 2 0 0 1 16 5.4V14a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V5.4a2 2 0 0 1 1.059-1.765l6-3.2z"/>'),DM=Xu("EnvelopeOpenFill",'<path d="M8.941.435a2 2 0 0 0-1.882 0l-6 3.2A2 2 0 0 0 0 5.4v.313l6.709 3.933L8 8.928l1.291.717L16 5.715V5.4a2 2 0 0 0-1.059-1.765l-6-3.2zM16 6.873l-5.693 3.337L16 13.372v-6.5zm-.059 7.611L8 10.072.059 14.484A2 2 0 0 0 2 16h12a2 2 0 0 0 1.941-1.516zM0 13.373l5.693-3.163L0 6.873v6.5z"/>'),CM=Xu("Eraser",'<path d="M8.086 2.207a2 2 0 0 1 2.828 0l3.879 3.879a2 2 0 0 1 0 2.828l-5.5 5.5A2 2 0 0 1 7.879 15H5.12a2 2 0 0 1-1.414-.586l-2.5-2.5a2 2 0 0 1 0-2.828l6.879-6.879zm2.121.707a1 1 0 0 0-1.414 0L4.16 7.547l5.293 5.293 4.633-4.633a1 1 0 0 0 0-1.414l-3.879-3.879zM8.746 13.547 3.453 8.254 1.914 9.793a1 1 0 0 0 0 1.414l2.5 2.5a1 1 0 0 0 .707.293H7.88a1 1 0 0 0 .707-.293l.16-.16z"/>'),jM=Xu("EraserFill",'<path d="M8.086 2.207a2 2 0 0 1 2.828 0l3.879 3.879a2 2 0 0 1 0 2.828l-5.5 5.5A2 2 0 0 1 7.879 15H5.12a2 2 0 0 1-1.414-.586l-2.5-2.5a2 2 0 0 1 0-2.828l6.879-6.879zm.66 11.34L3.453 8.254 1.914 9.793a1 1 0 0 0 0 1.414l2.5 2.5a1 1 0 0 0 .707.293H7.88a1 1 0 0 0 .707-.293l.16-.16z"/>'),xM=Xu("Exclamation",'<path d="M7.002 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.553.553 0 0 1-1.1 0L7.1 4.995z"/>'),BM=Xu("ExclamationCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M7.002 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995z"/>'),YM=Xu("ExclamationCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM8 4a.905.905 0 0 0-.9.995l.35 3.507a.552.552 0 0 0 1.1 0l.35-3.507A.905.905 0 0 0 8 4zm.002 6a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/>'),PM=Xu("ExclamationDiamond",'<path d="M6.95.435c.58-.58 1.52-.58 2.1 0l6.515 6.516c.58.58.58 1.519 0 2.098L9.05 15.565c-.58.58-1.519.58-2.098 0L.435 9.05a1.482 1.482 0 0 1 0-2.098L6.95.435zm1.4.7a.495.495 0 0 0-.7 0L1.134 7.65a.495.495 0 0 0 0 .7l6.516 6.516a.495.495 0 0 0 .7 0l6.516-6.516a.495.495 0 0 0 0-.7L8.35 1.134z"/><path d="M7.002 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995z"/>'),IM=Xu("ExclamationDiamondFill",'<path d="M9.05.435c-.58-.58-1.52-.58-2.1 0L.436 6.95c-.58.58-.58 1.519 0 2.098l6.516 6.516c.58.58 1.519.58 2.098 0l6.516-6.516c.58-.58.58-1.519 0-2.098L9.05.435zM8 4c.535 0 .954.462.9.995l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995A.905.905 0 0 1 8 4zm.002 6a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),FM=Xu("ExclamationLg",'<path d="M6.002 14a2 2 0 1 1 4 0 2 2 0 0 1-4 0zm.195-12.01a1.81 1.81 0 1 1 3.602 0l-.701 7.015a1.105 1.105 0 0 1-2.2 0l-.7-7.015z"/>'),EM=Xu("ExclamationOctagon",'<path d="M4.54.146A.5.5 0 0 1 4.893 0h6.214a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353L4.54.146zM5.1 1 1 5.1v5.8L5.1 15h5.8l4.1-4.1V5.1L10.9 1H5.1z"/><path d="M7.002 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995z"/>'),$M=Xu("ExclamationOctagonFill",'<path d="M11.46.146A.5.5 0 0 0 11.107 0H4.893a.5.5 0 0 0-.353.146L.146 4.54A.5.5 0 0 0 0 4.893v6.214a.5.5 0 0 0 .146.353l4.394 4.394a.5.5 0 0 0 .353.146h6.214a.5.5 0 0 0 .353-.146l4.394-4.394a.5.5 0 0 0 .146-.353V4.893a.5.5 0 0 0-.146-.353L11.46.146zM8 4c.535 0 .954.462.9.995l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995A.905.905 0 0 1 8 4zm.002 6a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),RM=Xu("ExclamationSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M7.002 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995z"/>'),NM=Xu("ExclamationSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm6 4c.535 0 .954.462.9.995l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995A.905.905 0 0 1 8 4zm.002 6a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),WM=Xu("ExclamationTriangle",'<path d="M7.938 2.016A.13.13 0 0 1 8.002 2a.13.13 0 0 1 .063.016.146.146 0 0 1 .054.057l6.857 11.667c.036.06.035.124.002.183a.163.163 0 0 1-.054.06.116.116 0 0 1-.066.017H1.146a.115.115 0 0 1-.066-.017.163.163 0 0 1-.054-.06.176.176 0 0 1 .002-.183L7.884 2.073a.147.147 0 0 1 .054-.057zm1.044-.45a1.13 1.13 0 0 0-1.96 0L.165 13.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L8.982 1.566z"/><path d="M7.002 12a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 5.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 5.995z"/>'),UM=Xu("ExclamationTriangleFill",'<path d="M8.982 1.566a1.13 1.13 0 0 0-1.96 0L.165 13.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L8.982 1.566zM8 5c.535 0 .954.462.9.995l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 5.995A.905.905 0 0 1 8 5zm.002 6a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),GM=Xu("Exclude",'<path d="M0 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2h2a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2v-2H2a2 2 0 0 1-2-2V2zm12 2H5a1 1 0 0 0-1 1v7h7a1 1 0 0 0 1-1V4z"/>'),qM=Xu("Eye",'<path d="M16 8s-3-5.5-8-5.5S0 8 0 8s3 5.5 8 5.5S16 8 16 8zM1.173 8a13.133 13.133 0 0 1 1.66-2.043C4.12 4.668 5.88 3.5 8 3.5c2.12 0 3.879 1.168 5.168 2.457A13.133 13.133 0 0 1 14.828 8c-.058.087-.122.183-.195.288-.335.48-.83 1.12-1.465 1.755C11.879 11.332 10.119 12.5 8 12.5c-2.12 0-3.879-1.168-5.168-2.457A13.134 13.134 0 0 1 1.172 8z"/><path d="M8 5.5a2.5 2.5 0 1 0 0 5 2.5 2.5 0 0 0 0-5zM4.5 8a3.5 3.5 0 1 1 7 0 3.5 3.5 0 0 1-7 0z"/>'),JM=Xu("EyeFill",'<path d="M10.5 8a2.5 2.5 0 1 1-5 0 2.5 2.5 0 0 1 5 0z"/><path d="M0 8s3-5.5 8-5.5S16 8 16 8s-3 5.5-8 5.5S0 8 0 8zm8 3.5a3.5 3.5 0 1 0 0-7 3.5 3.5 0 0 0 0 7z"/>'),KM=Xu("EyeSlash",'<path d="M13.359 11.238C15.06 9.72 16 8 16 8s-3-5.5-8-5.5a7.028 7.028 0 0 0-2.79.588l.77.771A5.944 5.944 0 0 1 8 3.5c2.12 0 3.879 1.168 5.168 2.457A13.134 13.134 0 0 1 14.828 8c-.058.087-.122.183-.195.288-.335.48-.83 1.12-1.465 1.755-.165.165-.337.328-.517.486l.708.709z"/><path d="M11.297 9.176a3.5 3.5 0 0 0-4.474-4.474l.823.823a2.5 2.5 0 0 1 2.829 2.829l.822.822zm-2.943 1.299.822.822a3.5 3.5 0 0 1-4.474-4.474l.823.823a2.5 2.5 0 0 0 2.829 2.829z"/><path d="M3.35 5.47c-.18.16-.353.322-.518.487A13.134 13.134 0 0 0 1.172 8l.195.288c.335.48.83 1.12 1.465 1.755C4.121 11.332 5.881 12.5 8 12.5c.716 0 1.39-.133 2.02-.36l.77.772A7.029 7.029 0 0 1 8 13.5C3 13.5 0 8 0 8s.939-1.721 2.641-3.238l.708.709zm10.296 8.884-12-12 .708-.708 12 12-.708.708z"/>'),XM=Xu("EyeSlashFill",'<path d="m10.79 12.912-1.614-1.615a3.5 3.5 0 0 1-4.474-4.474l-2.06-2.06C.938 6.278 0 8 0 8s3 5.5 8 5.5a7.029 7.029 0 0 0 2.79-.588zM5.21 3.088A7.028 7.028 0 0 1 8 2.5c5 0 8 5.5 8 5.5s-.939 1.721-2.641 3.238l-2.062-2.062a3.5 3.5 0 0 0-4.474-4.474L5.21 3.089z"/><path d="M5.525 7.646a2.5 2.5 0 0 0 2.829 2.829l-2.83-2.829zm4.95.708-2.829-2.83a2.5 2.5 0 0 1 2.829 2.829zm3.171 6-12-12 .708-.708 12 12-.708.708z"/>'),ZM=Xu("Eyedropper",'<path d="M13.354.646a1.207 1.207 0 0 0-1.708 0L8.5 3.793l-.646-.647a.5.5 0 1 0-.708.708L8.293 5l-7.147 7.146A.5.5 0 0 0 1 12.5v1.793l-.854.853a.5.5 0 1 0 .708.707L1.707 15H3.5a.5.5 0 0 0 .354-.146L11 7.707l1.146 1.147a.5.5 0 0 0 .708-.708l-.647-.646 3.147-3.146a1.207 1.207 0 0 0 0-1.708l-2-2zM2 12.707l7-7L10.293 7l-7 7H2v-1.293z"/>'),QM=Xu("Eyeglasses",'<path d="M4 6a2 2 0 1 1 0 4 2 2 0 0 1 0-4zm2.625.547a3 3 0 0 0-5.584.953H.5a.5.5 0 0 0 0 1h.541A3 3 0 0 0 7 8a1 1 0 0 1 2 0 3 3 0 0 0 5.959.5h.541a.5.5 0 0 0 0-1h-.541a3 3 0 0 0-5.584-.953A1.993 1.993 0 0 0 8 6c-.532 0-1.016.208-1.375.547zM14 8a2 2 0 1 1-4 0 2 2 0 0 1 4 0z"/>'),ez=Xu("Facebook",'<path d="M16 8.049c0-4.446-3.582-8.05-8-8.05C3.58 0-.002 3.603-.002 8.05c0 4.017 2.926 7.347 6.75 7.951v-5.625h-2.03V8.05H6.75V6.275c0-2.017 1.195-3.131 3.022-3.131.876 0 1.791.157 1.791.157v1.98h-1.009c-.993 0-1.303.621-1.303 1.258v1.51h2.218l-.354 2.326H9.25V16c3.824-.604 6.75-3.934 6.75-7.951z"/>'),tz=Xu("File",'<path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),az=Xu("FileArrowDown",'<path d="M8 5a.5.5 0 0 1 .5.5v3.793l1.146-1.147a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 1 1 .708-.708L7.5 9.293V5.5A.5.5 0 0 1 8 5z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),nz=Xu("FileArrowDownFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM8 5a.5.5 0 0 1 .5.5v3.793l1.146-1.147a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 1 1 .708-.708L7.5 9.293V5.5A.5.5 0 0 1 8 5z"/>'),rz=Xu("FileArrowUp",'<path d="M8 11a.5.5 0 0 0 .5-.5V6.707l1.146 1.147a.5.5 0 0 0 .708-.708l-2-2a.5.5 0 0 0-.708 0l-2 2a.5.5 0 1 0 .708.708L7.5 6.707V10.5a.5.5 0 0 0 .5.5z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),iz=Xu("FileArrowUpFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM7.5 6.707 6.354 7.854a.5.5 0 1 1-.708-.708l2-2a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1-.708.708L8.5 6.707V10.5a.5.5 0 0 1-1 0V6.707z"/>'),oz=Xu("FileBarGraph",'<path d="M4.5 12a.5.5 0 0 1-.5-.5v-2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-1zm3 0a.5.5 0 0 1-.5-.5v-4a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-.5.5h-1zm3 0a.5.5 0 0 1-.5-.5v-6a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v6a.5.5 0 0 1-.5.5h-1z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),sz=Xu("FileBarGraphFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zm-2 11.5v-6a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v6a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5zm-2.5.5a.5.5 0 0 1-.5-.5v-4a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-.5.5h-1zm-3 0a.5.5 0 0 1-.5-.5v-2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-1z"/>'),lz=Xu("FileBinary",'<path d="M5.526 13.09c.976 0 1.524-.79 1.524-2.205 0-1.412-.548-2.203-1.524-2.203-.978 0-1.526.79-1.526 2.203 0 1.415.548 2.206 1.526 2.206zm-.832-2.205c0-1.05.29-1.612.832-1.612.358 0 .607.247.733.721L4.7 11.137a6.749 6.749 0 0 1-.006-.252zm.832 1.614c-.36 0-.606-.246-.732-.718l1.556-1.145c.003.079.005.164.005.249 0 1.052-.29 1.614-.829 1.614zm5.329.501v-.595H9.73V8.772h-.69l-1.19.786v.688L8.986 9.5h.05v2.906h-1.18V13h3z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),cz=Xu("FileBinaryFill",'<path d="M5.526 9.273c-.542 0-.832.563-.832 1.612 0 .088.003.173.006.252l1.56-1.143c-.126-.474-.375-.72-.733-.72zm-.732 2.508c.126.472.372.718.732.718.54 0 .83-.563.83-1.614 0-.085-.003-.17-.006-.25l-1.556 1.146z"/><path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM7.05 10.885c0 1.415-.548 2.206-1.524 2.206C4.548 13.09 4 12.3 4 10.885c0-1.412.548-2.203 1.526-2.203.976 0 1.524.79 1.524 2.203zm3.805 1.52V13h-3v-.595h1.181V9.5h-.05l-1.136.747v-.688l1.19-.786h.69v3.633h1.125z"/>'),uz=Xu("FileBreak",'<path d="M0 10.5a.5.5 0 0 1 .5-.5h15a.5.5 0 0 1 0 1H.5a.5.5 0 0 1-.5-.5zM12 0H4a2 2 0 0 0-2 2v7h1V2a1 1 0 0 1 1-1h8a1 1 0 0 1 1 1v7h1V2a2 2 0 0 0-2-2zm2 12h-1v2a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1v-2H2v2a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2v-2z"/>'),dz=Xu("FileBreakFill",'<path d="M4 0h8a2 2 0 0 1 2 2v7H2V2a2 2 0 0 1 2-2zM2 12h12v2a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2v-2zM.5 10a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1H.5z"/>'),hz=Xu("FileCheck",'<path d="M10.854 6.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 8.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),fz=Xu("FileCheckFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zm-1.146 6.854-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 8.793l2.646-2.647a.5.5 0 0 1 .708.708z"/>'),pz=Xu("FileCode",'<path d="M6.646 5.646a.5.5 0 1 1 .708.708L5.707 8l1.647 1.646a.5.5 0 0 1-.708.708l-2-2a.5.5 0 0 1 0-.708l2-2zm2.708 0a.5.5 0 1 0-.708.708L10.293 8 8.646 9.646a.5.5 0 0 0 .708.708l2-2a.5.5 0 0 0 0-.708l-2-2z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),mz=Xu("FileCodeFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM6.646 5.646a.5.5 0 1 1 .708.708L5.707 8l1.647 1.646a.5.5 0 0 1-.708.708l-2-2a.5.5 0 0 1 0-.708l2-2zm2.708 0 2 2a.5.5 0 0 1 0 .708l-2 2a.5.5 0 0 1-.708-.708L10.293 8 8.646 6.354a.5.5 0 1 1 .708-.708z"/>'),vz=Xu("FileDiff",'<path d="M8 4a.5.5 0 0 1 .5.5V6H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V7H6a.5.5 0 0 1 0-1h1.5V4.5A.5.5 0 0 1 8 4zm-2.5 6.5A.5.5 0 0 1 6 10h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),_z=Xu("FileDiffFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM8.5 4.5V6H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V7H6a.5.5 0 0 1 0-1h1.5V4.5a.5.5 0 0 1 1 0zM6 10h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1 0-1z"/>'),gz=Xu("FileEarmark",'<path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/>'),yz=Xu("FileEarmarkArrowDown",'<path d="M8.5 6.5a.5.5 0 0 0-1 0v3.793L6.354 9.146a.5.5 0 1 0-.708.708l2 2a.5.5 0 0 0 .708 0l2-2a.5.5 0 0 0-.708-.708L8.5 10.293V6.5z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),bz=Xu("FileEarmarkArrowDownFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zm-1 4v3.793l1.146-1.147a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 0 1 .708-.708L7.5 11.293V7.5a.5.5 0 0 1 1 0z"/>'),Mz=Xu("FileEarmarkArrowUp",'<path d="M8.5 11.5a.5.5 0 0 1-1 0V7.707L6.354 8.854a.5.5 0 1 1-.708-.708l2-2a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1-.708.708L8.5 7.707V11.5z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),zz=Xu("FileEarmarkArrowUpFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM6.354 9.854a.5.5 0 0 1-.708-.708l2-2a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1-.708.708L8.5 8.707V12.5a.5.5 0 0 1-1 0V8.707L6.354 9.854z"/>'),Lz=Xu("FileEarmarkBarGraph",'<path d="M10 13.5a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-6a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5v6zm-2.5.5a.5.5 0 0 1-.5-.5v-4a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-.5.5h-1zm-3 0a.5.5 0 0 1-.5-.5v-2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-1z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),wz=Xu("FileEarmarkBarGraphFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zm.5 10v-6a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v6a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5zm-2.5.5a.5.5 0 0 1-.5-.5v-4a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-.5.5h-1zm-3 0a.5.5 0 0 1-.5-.5v-2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-1z"/>'),Hz=Xu("FileEarmarkBinary",'<path d="M7.05 11.885c0 1.415-.548 2.206-1.524 2.206C4.548 14.09 4 13.3 4 11.885c0-1.412.548-2.203 1.526-2.203.976 0 1.524.79 1.524 2.203zm-1.524-1.612c-.542 0-.832.563-.832 1.612 0 .088.003.173.006.252l1.559-1.143c-.126-.474-.375-.72-.733-.72zm-.732 2.508c.126.472.372.718.732.718.54 0 .83-.563.83-1.614 0-.085-.003-.17-.006-.25l-1.556 1.146zm6.061.624V14h-3v-.595h1.181V10.5h-.05l-1.136.747v-.688l1.19-.786h.69v3.633h1.125z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),kz=Xu("FileEarmarkBinaryFill",'<path d="M5.526 10.273c-.542 0-.832.563-.832 1.612 0 .088.003.173.006.252l1.559-1.143c-.126-.474-.375-.72-.733-.72zm-.732 2.508c.126.472.372.718.732.718.54 0 .83-.563.83-1.614 0-.085-.003-.17-.006-.25l-1.556 1.146z"/><path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zm-2.45 8.385c0 1.415-.548 2.206-1.524 2.206C4.548 14.09 4 13.3 4 11.885c0-1.412.548-2.203 1.526-2.203.976 0 1.524.79 1.524 2.203zm3.805 1.52V14h-3v-.595h1.181V10.5h-.05l-1.136.747v-.688l1.19-.786h.69v3.633h1.125z"/>'),Az=Xu("FileEarmarkBreak",'<path d="M14 4.5V9h-1V4.5h-2A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v7H2V2a2 2 0 0 1 2-2h5.5L14 4.5zM13 12h1v2a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2v-2h1v2a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1v-2zM.5 10a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1H.5z"/>'),Sz=Xu("FileEarmarkBreakFill",'<path d="M4 0h5.293A1 1 0 0 1 10 .293L13.707 4a1 1 0 0 1 .293.707V9H2V2a2 2 0 0 1 2-2zm5.5 1.5v2a1 1 0 0 0 1 1h2l-3-3zM2 12h12v2a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2v-2zM.5 10a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1H.5z"/>'),Vz=Xu("FileEarmarkCheck",'<path d="M10.854 7.854a.5.5 0 0 0-.708-.708L7.5 9.793 6.354 8.646a.5.5 0 1 0-.708.708l1.5 1.5a.5.5 0 0 0 .708 0l3-3z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),Oz=Xu("FileEarmarkCheckFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zm1.354 4.354-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 9.793l2.646-2.647a.5.5 0 0 1 .708.708z"/>'),Tz=Xu("FileEarmarkCode",'<path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/><path d="M8.646 6.646a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1 0 .708l-2 2a.5.5 0 0 1-.708-.708L10.293 9 8.646 7.354a.5.5 0 0 1 0-.708zm-1.292 0a.5.5 0 0 0-.708 0l-2 2a.5.5 0 0 0 0 .708l2 2a.5.5 0 0 0 .708-.708L5.707 9l1.647-1.646a.5.5 0 0 0 0-.708z"/>'),Dz=Xu("FileEarmarkCodeFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM6.646 7.646a.5.5 0 1 1 .708.708L5.707 10l1.647 1.646a.5.5 0 0 1-.708.708l-2-2a.5.5 0 0 1 0-.708l2-2zm2.708 0 2 2a.5.5 0 0 1 0 .708l-2 2a.5.5 0 0 1-.708-.708L10.293 10 8.646 8.354a.5.5 0 1 1 .708-.708z"/>'),Cz=Xu("FileEarmarkDiff",'<path d="M8 5a.5.5 0 0 1 .5.5V7H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V8H6a.5.5 0 0 1 0-1h1.5V5.5A.5.5 0 0 1 8 5zm-2.5 6.5A.5.5 0 0 1 6 11h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),jz=Xu("FileEarmarkDiffFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM8 6a.5.5 0 0 1 .5.5V8H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V9H6a.5.5 0 0 1 0-1h1.5V6.5A.5.5 0 0 1 8 6zm-2.5 6.5A.5.5 0 0 1 6 12h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/>'),xz=Xu("FileEarmarkEasel",'<path d="M8.5 6a.5.5 0 1 0-1 0h-2A1.5 1.5 0 0 0 4 7.5v2A1.5 1.5 0 0 0 5.5 11h.473l-.447 1.342a.5.5 0 1 0 .948.316L7.027 11H7.5v1a.5.5 0 0 0 1 0v-1h.473l.553 1.658a.5.5 0 1 0 .948-.316L10.027 11h.473A1.5 1.5 0 0 0 12 9.5v-2A1.5 1.5 0 0 0 10.5 6h-2zM5 7.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-2z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),Bz=Xu("FileEarmarkEaselFill",'<path d="M5 7.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-2z"/><path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM8.5 6h2A1.5 1.5 0 0 1 12 7.5v2a1.5 1.5 0 0 1-1.5 1.5h-.473l.447 1.342a.5.5 0 0 1-.948.316L8.973 11H8.5v1a.5.5 0 0 1-1 0v-1h-.473l-.553 1.658a.5.5 0 1 1-.948-.316L5.973 11H5.5A1.5 1.5 0 0 1 4 9.5v-2A1.5 1.5 0 0 1 5.5 6h2a.5.5 0 0 1 1 0z"/>'),Yz=Xu("FileEarmarkExcel",'<path d="M5.884 6.68a.5.5 0 1 0-.768.64L7.349 10l-2.233 2.68a.5.5 0 0 0 .768.64L8 10.781l2.116 2.54a.5.5 0 0 0 .768-.641L8.651 10l2.233-2.68a.5.5 0 0 0-.768-.64L8 9.219l-2.116-2.54z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),Pz=Xu("FileEarmarkExcelFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM5.884 6.68 8 9.219l2.116-2.54a.5.5 0 1 1 .768.641L8.651 10l2.233 2.68a.5.5 0 0 1-.768.64L8 10.781l-2.116 2.54a.5.5 0 0 1-.768-.641L7.349 10 5.116 7.32a.5.5 0 1 1 .768-.64z"/>'),Iz=Xu("FileEarmarkFill",'<path d="M4 0h5.293A1 1 0 0 1 10 .293L13.707 4a1 1 0 0 1 .293.707V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2zm5.5 1.5v2a1 1 0 0 0 1 1h2l-3-3z"/>'),Fz=Xu("FileEarmarkFont",'<path d="M10.943 6H5.057L5 8h.5c.18-1.096.356-1.192 1.694-1.235l.293-.01v5.09c0 .47-.1.582-.898.655v.5H9.41v-.5c-.803-.073-.903-.184-.903-.654V6.755l.298.01c1.338.043 1.514.14 1.694 1.235h.5l-.057-2z"/><path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/>'),Ez=Xu("FileEarmarkFontFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM5.057 6h5.886L11 8h-.5c-.18-1.096-.356-1.192-1.694-1.235l-.298-.01v5.09c0 .47.1.582.903.655v.5H6.59v-.5c.799-.073.898-.184.898-.654V6.755l-.293.01C5.856 6.808 5.68 6.905 5.5 8H5l.057-2z"/>'),$z=Xu("FileEarmarkImage",'<path d="M6.502 7a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/><path d="M14 14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5V14zM4 1a1 1 0 0 0-1 1v10l2.224-2.224a.5.5 0 0 1 .61-.075L8 11l2.157-3.02a.5.5 0 0 1 .76-.063L13 10V4.5h-2A1.5 1.5 0 0 1 9.5 3V1H4z"/>'),Rz=Xu("FileEarmarkImageFill",'<path d="M4 0h5.293A1 1 0 0 1 10 .293L13.707 4a1 1 0 0 1 .293.707v5.586l-2.73-2.73a1 1 0 0 0-1.52.127l-1.889 2.644-1.769-1.062a1 1 0 0 0-1.222.15L2 12.292V2a2 2 0 0 1 2-2zm5.5 1.5v2a1 1 0 0 0 1 1h2l-3-3zm-1.498 4a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0z"/><path d="M10.564 8.27 14 11.708V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2v-.293l3.578-3.577 2.56 1.536 2.426-3.395z"/>'),Nz=Xu("FileEarmarkLock",'<path d="M10 7v1.076c.54.166 1 .597 1 1.224v2.4c0 .816-.781 1.3-1.5 1.3h-3c-.719 0-1.5-.484-1.5-1.3V9.3c0-.627.46-1.058 1-1.224V7a2 2 0 1 1 4 0zM7 7v1h2V7a1 1 0 0 0-2 0zM6 9.3v2.4c0 .042.02.107.105.175A.637.637 0 0 0 6.5 12h3a.64.64 0 0 0 .395-.125c.085-.068.105-.133.105-.175V9.3c0-.042-.02-.107-.105-.175A.637.637 0 0 0 9.5 9h-3a.637.637 0 0 0-.395.125C6.02 9.193 6 9.258 6 9.3z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),Wz=Xu("FileEarmarkLock2",'<path d="M10 7v1.076c.54.166 1 .597 1 1.224v2.4c0 .816-.781 1.3-1.5 1.3h-3c-.719 0-1.5-.484-1.5-1.3V9.3c0-.627.46-1.058 1-1.224V7a2 2 0 1 1 4 0zM7 7v1h2V7a1 1 0 0 0-2 0z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),Uz=Xu("FileEarmarkLock2Fill",'<path d="M7 7a1 1 0 0 1 2 0v1H7V7z"/><path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM10 7v1.076c.54.166 1 .597 1 1.224v2.4c0 .816-.781 1.3-1.5 1.3h-3c-.719 0-1.5-.484-1.5-1.3V9.3c0-.627.46-1.058 1-1.224V7a2 2 0 1 1 4 0z"/>'),Gz=Xu("FileEarmarkLockFill",'<path d="M7 7a1 1 0 0 1 2 0v1H7V7zM6 9.3c0-.042.02-.107.105-.175A.637.637 0 0 1 6.5 9h3a.64.64 0 0 1 .395.125c.085.068.105.133.105.175v2.4c0 .042-.02.107-.105.175A.637.637 0 0 1 9.5 12h-3a.637.637 0 0 1-.395-.125C6.02 11.807 6 11.742 6 11.7V9.3z"/><path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM10 7v1.076c.54.166 1 .597 1 1.224v2.4c0 .816-.781 1.3-1.5 1.3h-3c-.719 0-1.5-.484-1.5-1.3V9.3c0-.627.46-1.058 1-1.224V7a2 2 0 1 1 4 0z"/>'),qz=Xu("FileEarmarkMedical",'<path d="M7.5 5.5a.5.5 0 0 0-1 0v.634l-.549-.317a.5.5 0 1 0-.5.866L6 7l-.549.317a.5.5 0 1 0 .5.866l.549-.317V8.5a.5.5 0 1 0 1 0v-.634l.549.317a.5.5 0 1 0 .5-.866L8 7l.549-.317a.5.5 0 1 0-.5-.866l-.549.317V5.5zm-2 4.5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zm0 2a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),Jz=Xu("FileEarmarkMedicalFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zm-3 2v.634l.549-.317a.5.5 0 1 1 .5.866L7 7l.549.317a.5.5 0 1 1-.5.866L6.5 7.866V8.5a.5.5 0 0 1-1 0v-.634l-.549.317a.5.5 0 1 1-.5-.866L5 7l-.549-.317a.5.5 0 0 1 .5-.866l.549.317V5.5a.5.5 0 1 1 1 0zm-2 4.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1zm0 2h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1z"/>'),Kz=Xu("FileEarmarkMinus",'<path d="M5.5 9a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/>'),Xz=Xu("FileEarmarkMinusFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM6 8.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1 0-1z"/>'),Zz=Xu("FileEarmarkMusic",'<path d="M11 6.64a1 1 0 0 0-1.243-.97l-1 .25A1 1 0 0 0 8 6.89v4.306A2.572 2.572 0 0 0 7 11c-.5 0-.974.134-1.338.377-.36.24-.662.628-.662 1.123s.301.883.662 1.123c.364.243.839.377 1.338.377.5 0 .974-.134 1.338-.377.36-.24.662-.628.662-1.123V8.89l2-.5V6.64z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),Qz=Xu("FileEarmarkMusicFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM11 6.64v1.75l-2 .5v3.61c0 .495-.301.883-.662 1.123C7.974 13.866 7.499 14 7 14c-.5 0-.974-.134-1.338-.377-.36-.24-.662-.628-.662-1.123s.301-.883.662-1.123C6.026 11.134 6.501 11 7 11c.356 0 .7.068 1 .196V6.89a1 1 0 0 1 .757-.97l1-.25A1 1 0 0 1 11 6.64z"/>'),eL=Xu("FileEarmarkPdf",'<path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/><path d="M4.603 14.087a.81.81 0 0 1-.438-.42c-.195-.388-.13-.776.08-1.102.198-.307.526-.568.897-.787a7.68 7.68 0 0 1 1.482-.645 19.697 19.697 0 0 0 1.062-2.227 7.269 7.269 0 0 1-.43-1.295c-.086-.4-.119-.796-.046-1.136.075-.354.274-.672.65-.823.192-.077.4-.12.602-.077a.7.7 0 0 1 .477.365c.088.164.12.356.127.538.007.188-.012.396-.047.614-.084.51-.27 1.134-.52 1.794a10.954 10.954 0 0 0 .98 1.686 5.753 5.753 0 0 1 1.334.05c.364.066.734.195.96.465.12.144.193.32.2.518.007.192-.047.382-.138.563a1.04 1.04 0 0 1-.354.416.856.856 0 0 1-.51.138c-.331-.014-.654-.196-.933-.417a5.712 5.712 0 0 1-.911-.95 11.651 11.651 0 0 0-1.997.406 11.307 11.307 0 0 1-1.02 1.51c-.292.35-.609.656-.927.787a.793.793 0 0 1-.58.029zm1.379-1.901c-.166.076-.32.156-.459.238-.328.194-.541.383-.647.547-.094.145-.096.25-.04.361.01.022.02.036.026.044a.266.266 0 0 0 .035-.012c.137-.056.355-.235.635-.572a8.18 8.18 0 0 0 .45-.606zm1.64-1.33a12.71 12.71 0 0 1 1.01-.193 11.744 11.744 0 0 1-.51-.858 20.801 20.801 0 0 1-.5 1.05zm2.446.45c.15.163.296.3.435.41.24.19.407.253.498.256a.107.107 0 0 0 .07-.015.307.307 0 0 0 .094-.125.436.436 0 0 0 .059-.2.095.095 0 0 0-.026-.063c-.052-.062-.2-.152-.518-.209a3.876 3.876 0 0 0-.612-.053zM8.078 7.8a6.7 6.7 0 0 0 .2-.828c.031-.188.043-.343.038-.465a.613.613 0 0 0-.032-.198.517.517 0 0 0-.145.04c-.087.035-.158.106-.196.283-.04.192-.03.469.046.822.024.111.054.227.09.346z"/>'),tL=Xu("FileEarmarkPdfFill",'<path d="M5.523 12.424c.14-.082.293-.162.459-.238a7.878 7.878 0 0 1-.45.606c-.28.337-.498.516-.635.572a.266.266 0 0 1-.035.012.282.282 0 0 1-.026-.044c-.056-.11-.054-.216.04-.36.106-.165.319-.354.647-.548zm2.455-1.647c-.119.025-.237.05-.356.078a21.148 21.148 0 0 0 .5-1.05 12.045 12.045 0 0 0 .51.858c-.217.032-.436.07-.654.114zm2.525.939a3.881 3.881 0 0 1-.435-.41c.228.005.434.022.612.054.317.057.466.147.518.209a.095.095 0 0 1 .026.064.436.436 0 0 1-.06.2.307.307 0 0 1-.094.124.107.107 0 0 1-.069.015c-.09-.003-.258-.066-.498-.256zM8.278 6.97c-.04.244-.108.524-.2.829a4.86 4.86 0 0 1-.089-.346c-.076-.353-.087-.63-.046-.822.038-.177.11-.248.196-.283a.517.517 0 0 1 .145-.04c.013.03.028.092.032.198.005.122-.007.277-.038.465z"/><path fill-rule="evenodd" d="M4 0h5.293A1 1 0 0 1 10 .293L13.707 4a1 1 0 0 1 .293.707V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2zm5.5 1.5v2a1 1 0 0 0 1 1h2l-3-3zM4.165 13.668c.09.18.23.343.438.419.207.075.412.04.58-.03.318-.13.635-.436.926-.786.333-.401.683-.927 1.021-1.51a11.651 11.651 0 0 1 1.997-.406c.3.383.61.713.91.95.28.22.603.403.934.417a.856.856 0 0 0 .51-.138c.155-.101.27-.247.354-.416.09-.181.145-.37.138-.563a.844.844 0 0 0-.2-.518c-.226-.27-.596-.4-.96-.465a5.76 5.76 0 0 0-1.335-.05 10.954 10.954 0 0 1-.98-1.686c.25-.66.437-1.284.52-1.794.036-.218.055-.426.048-.614a1.238 1.238 0 0 0-.127-.538.7.7 0 0 0-.477-.365c-.202-.043-.41 0-.601.077-.377.15-.576.47-.651.823-.073.34-.04.736.046 1.136.088.406.238.848.43 1.295a19.697 19.697 0 0 1-1.062 2.227 7.662 7.662 0 0 0-1.482.645c-.37.22-.699.48-.897.787-.21.326-.275.714-.08 1.103z"/>'),aL=Xu("FileEarmarkPerson",'<path d="M11 8a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2v9.255S12 12 8 12s-5 1.755-5 1.755V2a1 1 0 0 1 1-1h5.5v2z"/>'),nL=Xu("FileEarmarkPersonFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM11 8a3 3 0 1 1-6 0 3 3 0 0 1 6 0zm2 5.755V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1v-.245S4 12 8 12s5 1.755 5 1.755z"/>'),rL=Xu("FileEarmarkPlay",'<path d="M6 6.883v4.234a.5.5 0 0 0 .757.429l3.528-2.117a.5.5 0 0 0 0-.858L6.757 6.454a.5.5 0 0 0-.757.43z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),iL=Xu("FileEarmarkPlayFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM6 6.883a.5.5 0 0 1 .757-.429l3.528 2.117a.5.5 0 0 1 0 .858l-3.528 2.117a.5.5 0 0 1-.757-.43V6.884z"/>'),oL=Xu("FileEarmarkPlus",'<path d="M8 6.5a.5.5 0 0 1 .5.5v1.5H10a.5.5 0 0 1 0 1H8.5V11a.5.5 0 0 1-1 0V9.5H6a.5.5 0 0 1 0-1h1.5V7a.5.5 0 0 1 .5-.5z"/><path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/>'),sL=Xu("FileEarmarkPlusFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM8.5 7v1.5H10a.5.5 0 0 1 0 1H8.5V11a.5.5 0 0 1-1 0V9.5H6a.5.5 0 0 1 0-1h1.5V7a.5.5 0 0 1 1 0z"/>'),lL=Xu("FileEarmarkPost",'<path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/><path d="M4 6.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-7zm0-3a.5.5 0 0 1 .5-.5H7a.5.5 0 0 1 0 1H4.5a.5.5 0 0 1-.5-.5z"/>'),cL=Xu("FileEarmarkPostFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zm-5-.5H7a.5.5 0 0 1 0 1H4.5a.5.5 0 0 1 0-1zm0 3h7a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-7a.5.5 0 0 1 .5-.5z"/>'),uL=Xu("FileEarmarkPpt",'<path d="M7 5.5a1 1 0 0 0-1 1V13a.5.5 0 0 0 1 0v-2h1.188a2.75 2.75 0 0 0 0-5.5H7zM8.188 10H7V6.5h1.188a1.75 1.75 0 1 1 0 3.5z"/><path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/>'),dL=Xu("FileEarmarkPptFill",'<path d="M8.188 10H7V6.5h1.188a1.75 1.75 0 1 1 0 3.5z"/><path d="M4 0h5.293A1 1 0 0 1 10 .293L13.707 4a1 1 0 0 1 .293.707V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2zm5.5 1.5v2a1 1 0 0 0 1 1h2l-3-3zM7 5.5a1 1 0 0 0-1 1V13a.5.5 0 0 0 1 0v-2h1.188a2.75 2.75 0 0 0 0-5.5H7z"/>'),hL=Xu("FileEarmarkRichtext",'<path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/><path d="M4.5 12.5A.5.5 0 0 1 5 12h3a.5.5 0 0 1 0 1H5a.5.5 0 0 1-.5-.5zm0-2A.5.5 0 0 1 5 10h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1-.5-.5zm1.639-3.708 1.33.886 1.854-1.855a.25.25 0 0 1 .289-.047l1.888.974V8.5a.5.5 0 0 1-.5.5H5a.5.5 0 0 1-.5-.5V8s1.54-1.274 1.639-1.208zM6.25 6a.75.75 0 1 0 0-1.5.75.75 0 0 0 0 1.5z"/>'),fL=Xu("FileEarmarkRichtextFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM7 6.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm-.861 1.542 1.33.886 1.854-1.855a.25.25 0 0 1 .289-.047l1.888.974V9.5a.5.5 0 0 1-.5.5H5a.5.5 0 0 1-.5-.5V9s1.54-1.274 1.639-1.208zM5 11h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1zm0 2h3a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1z"/>'),pL=Xu("FileEarmarkRuled",'<path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V9H3V2a1 1 0 0 1 1-1h5.5v2zM3 12v-2h2v2H3zm0 1h2v2H4a1 1 0 0 1-1-1v-1zm3 2v-2h7v1a1 1 0 0 1-1 1H6zm7-3H6v-2h7v2z"/>'),mL=Xu("FileEarmarkRuledFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM3 9h10v1H6v2h7v1H6v2H5v-2H3v-1h2v-2H3V9z"/>'),vL=Xu("FileEarmarkSlides",'<path d="M5 6a.5.5 0 0 0-.496.438l-.5 4A.5.5 0 0 0 4.5 11h3v2.016c-.863.055-1.5.251-1.5.484 0 .276.895.5 2 .5s2-.224 2-.5c0-.233-.637-.429-1.5-.484V11h3a.5.5 0 0 0 .496-.562l-.5-4A.5.5 0 0 0 11 6H5zm2 3.78V7.22c0-.096.106-.156.19-.106l2.13 1.279a.125.125 0 0 1 0 .214l-2.13 1.28A.125.125 0 0 1 7 9.778z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),_L=Xu("FileEarmarkSlidesFill",'<path d="M7 9.78V7.22c0-.096.106-.156.19-.106l2.13 1.279a.125.125 0 0 1 0 .214l-2.13 1.28A.125.125 0 0 1 7 9.778z"/><path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM5 6h6a.5.5 0 0 1 .496.438l.5 4A.5.5 0 0 1 11.5 11h-3v2.016c.863.055 1.5.251 1.5.484 0 .276-.895.5-2 .5s-2-.224-2-.5c0-.233.637-.429 1.5-.484V11h-3a.5.5 0 0 1-.496-.562l.5-4A.5.5 0 0 1 5 6z"/>'),gL=Xu("FileEarmarkSpreadsheet",'<path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V9H3V2a1 1 0 0 1 1-1h5.5v2zM3 12v-2h2v2H3zm0 1h2v2H4a1 1 0 0 1-1-1v-1zm3 2v-2h3v2H6zm4 0v-2h3v1a1 1 0 0 1-1 1h-2zm3-3h-3v-2h3v2zm-7 0v-2h3v2H6z"/>'),yL=Xu("FileEarmarkSpreadsheetFill",'<path d="M6 12v-2h3v2H6z"/><path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM3 9h10v1h-3v2h3v1h-3v2H9v-2H6v2H5v-2H3v-1h2v-2H3V9z"/>'),bL=Xu("FileEarmarkText",'<path d="M5.5 7a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zM5 9.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/><path d="M9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.5L9.5 0zm0 1v2A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5z"/>'),ML=Xu("FileEarmarkTextFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM4.5 9a.5.5 0 0 1 0-1h7a.5.5 0 0 1 0 1h-7zM4 10.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm.5 2.5a.5.5 0 0 1 0-1h4a.5.5 0 0 1 0 1h-4z"/>'),zL=Xu("FileEarmarkWord",'<path d="M5.485 6.879a.5.5 0 1 0-.97.242l1.5 6a.5.5 0 0 0 .967.01L8 9.402l1.018 3.73a.5.5 0 0 0 .967-.01l1.5-6a.5.5 0 0 0-.97-.242l-1.036 4.144-.997-3.655a.5.5 0 0 0-.964 0l-.997 3.655L5.485 6.88z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),LL=Xu("FileEarmarkWordFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM5.485 6.879l1.036 4.144.997-3.655a.5.5 0 0 1 .964 0l.997 3.655 1.036-4.144a.5.5 0 0 1 .97.242l-1.5 6a.5.5 0 0 1-.967.01L8 9.402l-1.018 3.73a.5.5 0 0 1-.967-.01l-1.5-6a.5.5 0 1 1 .97-.242z"/>'),wL=Xu("FileEarmarkX",'<path d="M6.854 7.146a.5.5 0 1 0-.708.708L7.293 9l-1.147 1.146a.5.5 0 0 0 .708.708L8 9.707l1.146 1.147a.5.5 0 0 0 .708-.708L8.707 9l1.147-1.146a.5.5 0 0 0-.708-.708L8 8.293 6.854 7.146z"/><path d="M14 14V4.5L9.5 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2zM9.5 3A1.5 1.5 0 0 0 11 4.5h2V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h5.5v2z"/>'),HL=Xu("FileEarmarkXFill",'<path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zM6.854 7.146 8 8.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 9l1.147 1.146a.5.5 0 0 1-.708.708L8 9.707l-1.146 1.147a.5.5 0 0 1-.708-.708L7.293 9 6.146 7.854a.5.5 0 1 1 .708-.708z"/>'),kL=Xu("FileEarmarkZip",'<path d="M5 7.5a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v.938l.4 1.599a1 1 0 0 1-.416 1.074l-.93.62a1 1 0 0 1-1.11 0l-.929-.62a1 1 0 0 1-.415-1.074L5 8.438V7.5zm2 0H6v.938a1 1 0 0 1-.03.243l-.4 1.598.93.62.929-.62-.4-1.598A1 1 0 0 1 7 8.438V7.5z"/><path d="M14 4.5V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h5.5L14 4.5zm-3 0A1.5 1.5 0 0 1 9.5 3V1h-2v1h-1v1h1v1h-1v1h1v1H6V5H5V4h1V3H5V2h1V1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4.5h-2z"/>'),AL=Xu("FileEarmarkZipFill",'<path d="M5.5 9.438V8.5h1v.938a1 1 0 0 0 .03.243l.4 1.598-.93.62-.93-.62.4-1.598a1 1 0 0 0 .03-.243z"/><path d="M9.293 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V4.707A1 1 0 0 0 13.707 4L10 .293A1 1 0 0 0 9.293 0zM9.5 3.5v-2l3 3h-2a1 1 0 0 1-1-1zm-4-.5V2h-1V1H6v1h1v1H6v1h1v1H6v1h1v1H5.5V6h-1V5h1V4h-1V3h1zm0 4.5h1a1 1 0 0 1 1 1v.938l.4 1.599a1 1 0 0 1-.416 1.074l-.93.62a1 1 0 0 1-1.109 0l-.93-.62a1 1 0 0 1-.415-1.074l.4-1.599V8.5a1 1 0 0 1 1-1z"/>'),SL=Xu("FileEasel",'<path d="M8.5 5a.5.5 0 1 0-1 0h-2A1.5 1.5 0 0 0 4 6.5v2A1.5 1.5 0 0 0 5.5 10h.473l-.447 1.342a.5.5 0 1 0 .948.316L7.027 10H7.5v1a.5.5 0 0 0 1 0v-1h.473l.553 1.658a.5.5 0 1 0 .948-.316L10.027 10h.473A1.5 1.5 0 0 0 12 8.5v-2A1.5 1.5 0 0 0 10.5 5h-2zM5 6.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-2z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),VL=Xu("FileEaselFill",'<path d="M5 6.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-2z"/><path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM8.5 5h2A1.5 1.5 0 0 1 12 6.5v2a1.5 1.5 0 0 1-1.5 1.5h-.473l.447 1.342a.5.5 0 0 1-.948.316L8.973 10H8.5v1a.5.5 0 0 1-1 0v-1h-.473l-.553 1.658a.5.5 0 1 1-.948-.316L5.973 10H5.5A1.5 1.5 0 0 1 4 8.5v-2A1.5 1.5 0 0 1 5.5 5h2a.5.5 0 0 1 1 0z"/>'),OL=Xu("FileExcel",'<path d="M5.18 4.616a.5.5 0 0 1 .704.064L8 7.219l2.116-2.54a.5.5 0 1 1 .768.641L8.651 8l2.233 2.68a.5.5 0 0 1-.768.64L8 8.781l-2.116 2.54a.5.5 0 0 1-.768-.641L7.349 8 5.116 5.32a.5.5 0 0 1 .064-.704z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),TL=Xu("FileExcelFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM5.884 4.68 8 7.219l2.116-2.54a.5.5 0 1 1 .768.641L8.651 8l2.233 2.68a.5.5 0 0 1-.768.64L8 8.781l-2.116 2.54a.5.5 0 0 1-.768-.641L7.349 8 5.116 5.32a.5.5 0 1 1 .768-.64z"/>'),DL=Xu("FileFill",'<path fill-rule="evenodd" d="M4 0h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2z"/>'),CL=Xu("FileFont",'<path d="M10.943 4H5.057L5 6h.5c.18-1.096.356-1.192 1.694-1.235l.293-.01v6.09c0 .47-.1.582-.898.655v.5H9.41v-.5c-.803-.073-.903-.184-.903-.654V4.755l.298.01c1.338.043 1.514.14 1.694 1.235h.5l-.057-2z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),jL=Xu("FileFontFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM5.057 4h5.886L11 6h-.5c-.18-1.096-.356-1.192-1.694-1.235l-.298-.01v6.09c0 .47.1.582.903.655v.5H6.59v-.5c.799-.073.898-.184.898-.654V4.755l-.293.01C5.856 4.808 5.68 4.905 5.5 6H5l.057-2z"/>'),xL=Xu("FileImage",'<path d="M8.002 5.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/><path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM3 2a1 1 0 0 1 1-1h8a1 1 0 0 1 1 1v8l-2.083-2.083a.5.5 0 0 0-.76.063L8 11 5.835 9.7a.5.5 0 0 0-.611.076L3 12V2z"/>'),BL=Xu("FileImageFill",'<path d="M4 0h8a2 2 0 0 1 2 2v8.293l-2.73-2.73a1 1 0 0 0-1.52.127l-1.889 2.644-1.769-1.062a1 1 0 0 0-1.222.15L2 12.292V2a2 2 0 0 1 2-2zm4.002 5.5a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0z"/><path d="M10.564 8.27 14 11.708V14a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2v-.293l3.578-3.577 2.56 1.536 2.426-3.395z"/>'),YL=Xu("FileLock",'<path d="M8 5a1 1 0 0 1 1 1v1H7V6a1 1 0 0 1 1-1zm2 2.076V6a2 2 0 1 0-4 0v1.076c-.54.166-1 .597-1 1.224v2.4c0 .816.781 1.3 1.5 1.3h3c.719 0 1.5-.484 1.5-1.3V8.3c0-.627-.46-1.058-1-1.224zM6.105 8.125A.637.637 0 0 1 6.5 8h3a.64.64 0 0 1 .395.125c.085.068.105.133.105.175v2.4c0 .042-.02.107-.105.175A.637.637 0 0 1 9.5 11h-3a.637.637 0 0 1-.395-.125C6.02 10.807 6 10.742 6 10.7V8.3c0-.042.02-.107.105-.175z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),PL=Xu("FileLock2",'<path d="M8 5a1 1 0 0 1 1 1v1H7V6a1 1 0 0 1 1-1zm2 2.076V6a2 2 0 1 0-4 0v1.076c-.54.166-1 .597-1 1.224v2.4c0 .816.781 1.3 1.5 1.3h3c.719 0 1.5-.484 1.5-1.3V8.3c0-.627-.46-1.058-1-1.224z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),IL=Xu("FileLock2Fill",'<path d="M7 6a1 1 0 0 1 2 0v1H7V6z"/><path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zm-2 6v1.076c.54.166 1 .597 1 1.224v2.4c0 .816-.781 1.3-1.5 1.3h-3c-.719 0-1.5-.484-1.5-1.3V8.3c0-.627.46-1.058 1-1.224V6a2 2 0 1 1 4 0z"/>'),FL=Xu("FileLockFill",'<path d="M7 6a1 1 0 0 1 2 0v1H7V6zM6 8.3c0-.042.02-.107.105-.175A.637.637 0 0 1 6.5 8h3a.64.64 0 0 1 .395.125c.085.068.105.133.105.175v2.4c0 .042-.02.107-.105.175A.637.637 0 0 1 9.5 11h-3a.637.637 0 0 1-.395-.125C6.02 10.807 6 10.742 6 10.7V8.3z"/><path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zm-2 6v1.076c.54.166 1 .597 1 1.224v2.4c0 .816-.781 1.3-1.5 1.3h-3c-.719 0-1.5-.484-1.5-1.3V8.3c0-.627.46-1.058 1-1.224V6a2 2 0 1 1 4 0z"/>'),EL=Xu("FileMedical",'<path d="M8.5 4.5a.5.5 0 0 0-1 0v.634l-.549-.317a.5.5 0 1 0-.5.866L7 6l-.549.317a.5.5 0 1 0 .5.866l.549-.317V7.5a.5.5 0 1 0 1 0v-.634l.549.317a.5.5 0 1 0 .5-.866L9 6l.549-.317a.5.5 0 1 0-.5-.866l-.549.317V4.5zM5.5 9a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zm0 2a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),$L=Xu("FileMedicalFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM8.5 4.5v.634l.549-.317a.5.5 0 1 1 .5.866L9 6l.549.317a.5.5 0 1 1-.5.866L8.5 6.866V7.5a.5.5 0 0 1-1 0v-.634l-.549.317a.5.5 0 1 1-.5-.866L7 6l-.549-.317a.5.5 0 0 1 .5-.866l.549.317V4.5a.5.5 0 1 1 1 0zM5.5 9h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1zm0 2h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1z"/>'),RL=Xu("FileMinus",'<path d="M5.5 8a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),NL=Xu("FileMinusFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM6 7.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1 0-1z"/>'),WL=Xu("FileMusic",'<path d="M10.304 3.13a1 1 0 0 1 1.196.98v1.8l-2.5.5v5.09c0 .495-.301.883-.662 1.123C7.974 12.866 7.499 13 7 13c-.5 0-.974-.134-1.338-.377-.36-.24-.662-.628-.662-1.123s.301-.883.662-1.123C6.026 10.134 6.501 10 7 10c.356 0 .7.068 1 .196V4.41a1 1 0 0 1 .804-.98l1.5-.3z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),UL=Xu("FileMusicFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zm-.5 4.11v1.8l-2.5.5v5.09c0 .495-.301.883-.662 1.123C7.974 12.866 7.499 13 7 13c-.5 0-.974-.134-1.338-.377-.36-.24-.662-.628-.662-1.123s.301-.883.662-1.123C6.026 10.134 6.501 10 7 10c.356 0 .7.068 1 .196V4.41a1 1 0 0 1 .804-.98l1.5-.3a1 1 0 0 1 1.196.98z"/>'),GL=Xu("FilePdf",'<path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/><path d="M4.603 12.087a.81.81 0 0 1-.438-.42c-.195-.388-.13-.776.08-1.102.198-.307.526-.568.897-.787a7.68 7.68 0 0 1 1.482-.645 19.701 19.701 0 0 0 1.062-2.227 7.269 7.269 0 0 1-.43-1.295c-.086-.4-.119-.796-.046-1.136.075-.354.274-.672.65-.823.192-.077.4-.12.602-.077a.7.7 0 0 1 .477.365c.088.164.12.356.127.538.007.187-.012.395-.047.614-.084.51-.27 1.134-.52 1.794a10.954 10.954 0 0 0 .98 1.686 5.753 5.753 0 0 1 1.334.05c.364.065.734.195.96.465.12.144.193.32.2.518.007.192-.047.382-.138.563a1.04 1.04 0 0 1-.354.416.856.856 0 0 1-.51.138c-.331-.014-.654-.196-.933-.417a5.716 5.716 0 0 1-.911-.95 11.642 11.642 0 0 0-1.997.406 11.311 11.311 0 0 1-1.021 1.51c-.29.35-.608.655-.926.787a.793.793 0 0 1-.58.029zm1.379-1.901c-.166.076-.32.156-.459.238-.328.194-.541.383-.647.547-.094.145-.096.25-.04.361.01.022.02.036.026.044a.27.27 0 0 0 .035-.012c.137-.056.355-.235.635-.572a8.18 8.18 0 0 0 .45-.606zm1.64-1.33a12.647 12.647 0 0 1 1.01-.193 11.666 11.666 0 0 1-.51-.858 20.741 20.741 0 0 1-.5 1.05zm2.446.45c.15.162.296.3.435.41.24.19.407.253.498.256a.107.107 0 0 0 .07-.015.307.307 0 0 0 .094-.125.436.436 0 0 0 .059-.2.095.095 0 0 0-.026-.063c-.052-.062-.2-.152-.518-.209a3.881 3.881 0 0 0-.612-.053zM8.078 5.8a6.7 6.7 0 0 0 .2-.828c.031-.188.043-.343.038-.465a.613.613 0 0 0-.032-.198.517.517 0 0 0-.145.04c-.087.035-.158.106-.196.283-.04.192-.03.469.046.822.024.111.054.227.09.346z"/>'),qL=Xu("FilePdfFill",'<path d="M5.523 10.424c.14-.082.293-.162.459-.238a7.878 7.878 0 0 1-.45.606c-.28.337-.498.516-.635.572a.266.266 0 0 1-.035.012.282.282 0 0 1-.026-.044c-.056-.11-.054-.216.04-.36.106-.165.319-.354.647-.548zm2.455-1.647c-.119.025-.237.05-.356.078a21.035 21.035 0 0 0 .5-1.05 11.96 11.96 0 0 0 .51.858c-.217.032-.436.07-.654.114zm2.525.939a3.888 3.888 0 0 1-.435-.41c.228.005.434.022.612.054.317.057.466.147.518.209a.095.095 0 0 1 .026.064.436.436 0 0 1-.06.2.307.307 0 0 1-.094.124.107.107 0 0 1-.069.015c-.09-.003-.258-.066-.498-.256zM8.278 4.97c-.04.244-.108.524-.2.829a4.86 4.86 0 0 1-.089-.346c-.076-.353-.087-.63-.046-.822.038-.177.11-.248.196-.283a.517.517 0 0 1 .145-.04c.013.03.028.092.032.198.005.122-.007.277-.038.465z"/><path fill-rule="evenodd" d="M4 0h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2zm.165 11.668c.09.18.23.343.438.419.207.075.412.04.58-.03.318-.13.635-.436.926-.786.333-.401.683-.927 1.021-1.51a11.64 11.64 0 0 1 1.997-.406c.3.383.61.713.91.95.28.22.603.403.934.417a.856.856 0 0 0 .51-.138c.155-.101.27-.247.354-.416.09-.181.145-.37.138-.563a.844.844 0 0 0-.2-.518c-.226-.27-.596-.4-.96-.465a5.76 5.76 0 0 0-1.335-.05 10.954 10.954 0 0 1-.98-1.686c.25-.66.437-1.284.52-1.794.036-.218.055-.426.048-.614a1.238 1.238 0 0 0-.127-.538.7.7 0 0 0-.477-.365c-.202-.043-.41 0-.601.077-.377.15-.576.47-.651.823-.073.34-.04.736.046 1.136.088.406.238.848.43 1.295a19.707 19.707 0 0 1-1.062 2.227 7.662 7.662 0 0 0-1.482.645c-.37.22-.699.48-.897.787-.21.326-.275.714-.08 1.103z"/>'),JL=Xu("FilePerson",'<path d="M12 1a1 1 0 0 1 1 1v10.755S12 11 8 11s-5 1.755-5 1.755V2a1 1 0 0 1 1-1h8zM4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4z"/><path d="M8 10a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/>'),KL=Xu("FilePersonFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zm-1 7a3 3 0 1 1-6 0 3 3 0 0 1 6 0zm-3 4c2.623 0 4.146.826 5 1.755V14a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1v-1.245C3.854 11.825 5.377 11 8 11z"/>'),XL=Xu("FilePlay",'<path d="M6 10.117V5.883a.5.5 0 0 1 .757-.429l3.528 2.117a.5.5 0 0 1 0 .858l-3.528 2.117a.5.5 0 0 1-.757-.43z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),ZL=Xu("FilePlayFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM6 5.883a.5.5 0 0 1 .757-.429l3.528 2.117a.5.5 0 0 1 0 .858l-3.528 2.117a.5.5 0 0 1-.757-.43V5.884z"/>'),QL=Xu("FilePlus",'<path d="M8.5 6a.5.5 0 0 0-1 0v1.5H6a.5.5 0 0 0 0 1h1.5V10a.5.5 0 0 0 1 0V8.5H10a.5.5 0 0 0 0-1H8.5V6z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),ew=Xu("FilePlusFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM8.5 6v1.5H10a.5.5 0 0 1 0 1H8.5V10a.5.5 0 0 1-1 0V8.5H6a.5.5 0 0 1 0-1h1.5V6a.5.5 0 0 1 1 0z"/>'),tw=Xu("FilePost",'<path d="M4 3.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-8z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),aw=Xu("FilePostFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM4.5 3h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1zm0 2h7a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-8a.5.5 0 0 1 .5-.5z"/>'),nw=Xu("FilePpt",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/><path d="M6 5a1 1 0 0 1 1-1h1.188a2.75 2.75 0 0 1 0 5.5H7v2a.5.5 0 0 1-1 0V5zm1 3.5h1.188a1.75 1.75 0 1 0 0-3.5H7v3.5z"/>'),rw=Xu("FilePptFill",'<path d="M8.188 8.5H7V5h1.188a1.75 1.75 0 1 1 0 3.5z"/><path d="M4 0h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2zm3 4a1 1 0 0 0-1 1v6.5a.5.5 0 0 0 1 0v-2h1.188a2.75 2.75 0 0 0 0-5.5H7z"/>'),iw=Xu("FileRichtext",'<path d="M7 4.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm-.861 1.542 1.33.886 1.854-1.855a.25.25 0 0 1 .289-.047l1.888.974V7.5a.5.5 0 0 1-.5.5H5a.5.5 0 0 1-.5-.5V7s1.54-1.274 1.639-1.208zM5 9a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1H5zm0 2a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1H5z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),ow=Xu("FileRichtextFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM7 4.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm-.861 1.542 1.33.886 1.854-1.855a.25.25 0 0 1 .289-.047l1.888.974V7.5a.5.5 0 0 1-.5.5H5a.5.5 0 0 1-.5-.5V7s1.54-1.274 1.639-1.208zM5 9h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1zm0 2h3a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1z"/>'),sw=Xu("FileRuled",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm2-1a1 1 0 0 0-1 1v4h10V2a1 1 0 0 0-1-1H4zm9 6H6v2h7V7zm0 3H6v2h7v-2zm0 3H6v2h6a1 1 0 0 0 1-1v-1zm-8 2v-2H3v1a1 1 0 0 0 1 1h1zm-2-3h2v-2H3v2zm0-3h2V7H3v2z"/>'),lw=Xu("FileRuledFill",'<path d="M12 0H4a2 2 0 0 0-2 2v4h12V2a2 2 0 0 0-2-2zm2 7H6v2h8V7zm0 3H6v2h8v-2zm0 3H6v3h6a2 2 0 0 0 2-2v-1zm-9 3v-3H2v1a2 2 0 0 0 2 2h1zm-3-4h3v-2H2v2zm0-3h3V7H2v2z"/>'),cw=Xu("FileSlides",'<path d="M5 4a.5.5 0 0 0-.496.438l-.5 4A.5.5 0 0 0 4.5 9h3v2.016c-.863.055-1.5.251-1.5.484 0 .276.895.5 2 .5s2-.224 2-.5c0-.233-.637-.429-1.5-.484V9h3a.5.5 0 0 0 .496-.562l-.5-4A.5.5 0 0 0 11 4H5zm2 3.78V5.22c0-.096.106-.156.19-.106l2.13 1.279a.125.125 0 0 1 0 .214l-2.13 1.28A.125.125 0 0 1 7 7.778z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),uw=Xu("FileSlidesFill",'<path d="M7 7.78V5.22c0-.096.106-.156.19-.106l2.13 1.279a.125.125 0 0 1 0 .214l-2.13 1.28A.125.125 0 0 1 7 7.778z"/><path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM5 4h6a.5.5 0 0 1 .496.438l.5 4A.5.5 0 0 1 11.5 9h-3v2.016c.863.055 1.5.251 1.5.484 0 .276-.895.5-2 .5s-2-.224-2-.5c0-.233.637-.429 1.5-.484V9h-3a.5.5 0 0 1-.496-.562l.5-4A.5.5 0 0 1 5 4z"/>'),dw=Xu("FileSpreadsheet",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm2-1a1 1 0 0 0-1 1v4h10V2a1 1 0 0 0-1-1H4zm9 6h-3v2h3V7zm0 3h-3v2h3v-2zm0 3h-3v2h2a1 1 0 0 0 1-1v-1zm-4 2v-2H6v2h3zm-4 0v-2H3v1a1 1 0 0 0 1 1h1zm-2-3h2v-2H3v2zm0-3h2V7H3v2zm3-2v2h3V7H6zm3 3H6v2h3v-2z"/>'),hw=Xu("FileSpreadsheetFill",'<path d="M12 0H4a2 2 0 0 0-2 2v4h12V2a2 2 0 0 0-2-2zm2 7h-4v2h4V7zm0 3h-4v2h4v-2zm0 3h-4v3h2a2 2 0 0 0 2-2v-1zm-5 3v-3H6v3h3zm-4 0v-3H2v1a2 2 0 0 0 2 2h1zm-3-4h3v-2H2v2zm0-3h3V7H2v2zm4 0V7h3v2H6zm0 1h3v2H6v-2z"/>'),fw=Xu("FileText",'<path d="M5 4a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1H5zm-.5 2.5A.5.5 0 0 1 5 6h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1-.5-.5zM5 8a.5.5 0 0 0 0 1h6a.5.5 0 0 0 0-1H5zm0 2a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1H5z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm10-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1z"/>'),pw=Xu("FileTextFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM5 4h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1zm-.5 2.5A.5.5 0 0 1 5 6h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1-.5-.5zM5 8h6a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1zm0 2h3a.5.5 0 0 1 0 1H5a.5.5 0 0 1 0-1z"/>'),mw=Xu("FileWord",'<path d="M4.879 4.515a.5.5 0 0 1 .606.364l1.036 4.144.997-3.655a.5.5 0 0 1 .964 0l.997 3.655 1.036-4.144a.5.5 0 0 1 .97.242l-1.5 6a.5.5 0 0 1-.967.01L8 7.402l-1.018 3.73a.5.5 0 0 1-.967-.01l-1.5-6a.5.5 0 0 1 .364-.606z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),vw=Xu("FileWordFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM5.485 4.879l1.036 4.144.997-3.655a.5.5 0 0 1 .964 0l.997 3.655 1.036-4.144a.5.5 0 0 1 .97.242l-1.5 6a.5.5 0 0 1-.967.01L8 7.402l-1.018 3.73a.5.5 0 0 1-.967-.01l-1.5-6a.5.5 0 1 1 .97-.242z"/>'),_w=Xu("FileX",'<path d="M6.146 6.146a.5.5 0 0 1 .708 0L8 7.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 8l1.147 1.146a.5.5 0 0 1-.708.708L8 8.707 6.854 9.854a.5.5 0 0 1-.708-.708L7.293 8 6.146 6.854a.5.5 0 0 1 0-.708z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm0 1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1z"/>'),gw=Xu("FileXFill",'<path d="M12 0H4a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zM6.854 6.146 8 7.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 8l1.147 1.146a.5.5 0 0 1-.708.708L8 8.707 6.854 9.854a.5.5 0 0 1-.708-.708L7.293 8 6.146 6.854a.5.5 0 1 1 .708-.708z"/>'),yw=Xu("FileZip",'<path d="M6.5 7.5a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v.938l.4 1.599a1 1 0 0 1-.416 1.074l-.93.62a1 1 0 0 1-1.109 0l-.93-.62a1 1 0 0 1-.415-1.074l.4-1.599V7.5zm2 0h-1v.938a1 1 0 0 1-.03.243l-.4 1.598.93.62.93-.62-.4-1.598a1 1 0 0 1-.03-.243V7.5z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm5.5-1H4a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H9v1H8v1h1v1H8v1h1v1H7.5V5h-1V4h1V3h-1V2h1V1z"/>'),bw=Xu("FileZipFill",'<path d="M8.5 9.438V8.5h-1v.938a1 1 0 0 1-.03.243l-.4 1.598.93.62.93-.62-.4-1.598a1 1 0 0 1-.03-.243z"/><path d="M4 0h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2zm2.5 8.5v.938l-.4 1.599a1 1 0 0 0 .416 1.074l.93.62a1 1 0 0 0 1.109 0l.93-.62a1 1 0 0 0 .415-1.074l-.4-1.599V8.5a1 1 0 0 0-1-1h-1a1 1 0 0 0-1 1zm1-5.5h-1v1h1v1h-1v1h1v1H9V6H8V5h1V4H8V3h1V2H8V1H6.5v1h1v1z"/>'),Mw=Xu("Files",'<path d="M13 0H6a2 2 0 0 0-2 2 2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h7a2 2 0 0 0 2-2 2 2 0 0 0 2-2V2a2 2 0 0 0-2-2zm0 13V4a2 2 0 0 0-2-2H5a1 1 0 0 1 1-1h7a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1zM3 4a1 1 0 0 1 1-1h7a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V4z"/>'),zw=Xu("FilesAlt",'<path d="M11 0H3a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2 2 2 0 0 0 2-2V4a2 2 0 0 0-2-2 2 2 0 0 0-2-2zm2 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1V3zM2 2a1 1 0 0 1 1-1h8a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V2z"/>'),Lw=Xu("Film",'<path d="M0 1a1 1 0 0 1 1-1h14a1 1 0 0 1 1 1v14a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V1zm4 0v6h8V1H4zm8 8H4v6h8V9zM1 1v2h2V1H1zm2 3H1v2h2V4zM1 7v2h2V7H1zm2 3H1v2h2v-2zm-2 3v2h2v-2H1zM15 1h-2v2h2V1zm-2 3v2h2V4h-2zm2 3h-2v2h2V7zm-2 3v2h2v-2h-2zm2 3h-2v2h2v-2z"/>'),ww=Xu("Filter",'<path d="M6 10.5a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1h-3a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),Hw=Xu("FilterCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M7 11.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5z"/>'),kw=Xu("FilterCircleFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zM3.5 5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1 0-1zM5 8.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm2 3a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5z"/>'),Aw=Xu("FilterLeft",'<path d="M2 10.5a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1h-3a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),Sw=Xu("FilterRight",'<path d="M14 10.5a.5.5 0 0 0-.5-.5h-3a.5.5 0 0 0 0 1h3a.5.5 0 0 0 .5-.5zm0-3a.5.5 0 0 0-.5-.5h-7a.5.5 0 0 0 0 1h7a.5.5 0 0 0 .5-.5zm0-3a.5.5 0 0 0-.5-.5h-11a.5.5 0 0 0 0 1h11a.5.5 0 0 0 .5-.5z"/>'),Vw=Xu("FilterSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M6 11.5a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1h-3a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),Ow=Xu("FilterSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm.5 5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1 0-1zM4 8.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm2 3a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1h-3a.5.5 0 0 1-.5-.5z"/>'),Tw=Xu("Flag",'<path d="M14.778.085A.5.5 0 0 1 15 .5V8a.5.5 0 0 1-.314.464L14.5 8l.186.464-.003.001-.006.003-.023.009a12.435 12.435 0 0 1-.397.15c-.264.095-.631.223-1.047.35-.816.252-1.879.523-2.71.523-.847 0-1.548-.28-2.158-.525l-.028-.01C7.68 8.71 7.14 8.5 6.5 8.5c-.7 0-1.638.23-2.437.477A19.626 19.626 0 0 0 3 9.342V15.5a.5.5 0 0 1-1 0V.5a.5.5 0 0 1 1 0v.282c.226-.079.496-.17.79-.26C4.606.272 5.67 0 6.5 0c.84 0 1.524.277 2.121.519l.043.018C9.286.788 9.828 1 10.5 1c.7 0 1.638-.23 2.437-.477a19.587 19.587 0 0 0 1.349-.476l.019-.007.004-.002h.001M14 1.221c-.22.078-.48.167-.766.255-.81.252-1.872.523-2.734.523-.886 0-1.592-.286-2.203-.534l-.008-.003C7.662 1.21 7.139 1 6.5 1c-.669 0-1.606.229-2.415.478A21.294 21.294 0 0 0 3 1.845v6.433c.22-.078.48-.167.766-.255C4.576 7.77 5.638 7.5 6.5 7.5c.847 0 1.548.28 2.158.525l.028.01C9.32 8.29 9.86 8.5 10.5 8.5c.668 0 1.606-.229 2.415-.478A21.317 21.317 0 0 0 14 7.655V1.222z"/>'),Dw=Xu("FlagFill",'<path d="M14.778.085A.5.5 0 0 1 15 .5V8a.5.5 0 0 1-.314.464L14.5 8l.186.464-.003.001-.006.003-.023.009a12.435 12.435 0 0 1-.397.15c-.264.095-.631.223-1.047.35-.816.252-1.879.523-2.71.523-.847 0-1.548-.28-2.158-.525l-.028-.01C7.68 8.71 7.14 8.5 6.5 8.5c-.7 0-1.638.23-2.437.477A19.626 19.626 0 0 0 3 9.342V15.5a.5.5 0 0 1-1 0V.5a.5.5 0 0 1 1 0v.282c.226-.079.496-.17.79-.26C4.606.272 5.67 0 6.5 0c.84 0 1.524.277 2.121.519l.043.018C9.286.788 9.828 1 10.5 1c.7 0 1.638-.23 2.437-.477a19.587 19.587 0 0 0 1.349-.476l.019-.007.004-.002h.001"/>'),Cw=Xu("Flower1",'<path d="M6.174 1.184a2 2 0 0 1 3.652 0A2 2 0 0 1 12.99 3.01a2 2 0 0 1 1.826 3.164 2 2 0 0 1 0 3.652 2 2 0 0 1-1.826 3.164 2 2 0 0 1-3.164 1.826 2 2 0 0 1-3.652 0A2 2 0 0 1 3.01 12.99a2 2 0 0 1-1.826-3.164 2 2 0 0 1 0-3.652A2 2 0 0 1 3.01 3.01a2 2 0 0 1 3.164-1.826zM8 1a1 1 0 0 0-.998 1.03l.01.091c.012.077.029.176.054.296.049.241.122.542.213.887.182.688.428 1.513.676 2.314L8 5.762l.045-.144c.248-.8.494-1.626.676-2.314.091-.345.164-.646.213-.887a4.997 4.997 0 0 0 .064-.386L9 2a1 1 0 0 0-1-1zM2 9l.03-.002.091-.01a4.99 4.99 0 0 0 .296-.054c.241-.049.542-.122.887-.213a60.59 60.59 0 0 0 2.314-.676L5.762 8l-.144-.045a60.59 60.59 0 0 0-2.314-.676 16.705 16.705 0 0 0-.887-.213 4.99 4.99 0 0 0-.386-.064L2 7a1 1 0 1 0 0 2zm7 5-.002-.03a5.005 5.005 0 0 0-.064-.386 16.398 16.398 0 0 0-.213-.888 60.582 60.582 0 0 0-.676-2.314L8 10.238l-.045.144c-.248.8-.494 1.626-.676 2.314-.091.345-.164.646-.213.887a4.996 4.996 0 0 0-.064.386L7 14a1 1 0 1 0 2 0zm-5.696-2.134.025-.017a5.001 5.001 0 0 0 .303-.248c.184-.164.408-.377.661-.629A60.614 60.614 0 0 0 5.96 9.23l.103-.111-.147.033a60.88 60.88 0 0 0-2.343.572c-.344.093-.64.18-.874.258a5.063 5.063 0 0 0-.367.138l-.027.014a1 1 0 1 0 1 1.732zM4.5 14.062a1 1 0 0 0 1.366-.366l.014-.027c.01-.02.021-.048.036-.084a5.09 5.09 0 0 0 .102-.283c.078-.233.165-.53.258-.874a60.6 60.6 0 0 0 .572-2.343l.033-.147-.11.102a60.848 60.848 0 0 0-1.743 1.667 17.07 17.07 0 0 0-.629.66 5.06 5.06 0 0 0-.248.304l-.017.025a1 1 0 0 0 .366 1.366zm9.196-8.196a1 1 0 0 0-1-1.732l-.025.017a4.951 4.951 0 0 0-.303.248 16.69 16.69 0 0 0-.661.629A60.72 60.72 0 0 0 10.04 6.77l-.102.111.147-.033a60.6 60.6 0 0 0 2.342-.572c.345-.093.642-.18.875-.258a4.993 4.993 0 0 0 .367-.138.53.53 0 0 0 .027-.014zM11.5 1.938a1 1 0 0 0-1.366.366l-.014.027c-.01.02-.021.048-.036.084a5.09 5.09 0 0 0-.102.283c-.078.233-.165.53-.258.875a60.62 60.62 0 0 0-.572 2.342l-.033.147.11-.102a60.848 60.848 0 0 0 1.743-1.667c.252-.253.465-.477.629-.66a5.001 5.001 0 0 0 .248-.304l.017-.025a1 1 0 0 0-.366-1.366zM14 9a1 1 0 0 0 0-2l-.03.002a4.996 4.996 0 0 0-.386.064c-.242.049-.543.122-.888.213-.688.182-1.513.428-2.314.676L10.238 8l.144.045c.8.248 1.626.494 2.314.676.345.091.646.164.887.213a4.996 4.996 0 0 0 .386.064L14 9zM1.938 4.5a1 1 0 0 0 .393 1.38l.084.035c.072.03.166.064.283.103.233.078.53.165.874.258a60.88 60.88 0 0 0 2.343.572l.147.033-.103-.111a60.584 60.584 0 0 0-1.666-1.742 16.705 16.705 0 0 0-.66-.629 4.996 4.996 0 0 0-.304-.248l-.025-.017a1 1 0 0 0-1.366.366zm2.196-1.196.017.025a4.996 4.996 0 0 0 .248.303c.164.184.377.408.629.661A60.597 60.597 0 0 0 6.77 5.96l.111.102-.033-.147a60.602 60.602 0 0 0-.572-2.342c-.093-.345-.18-.642-.258-.875a5.006 5.006 0 0 0-.138-.367l-.014-.027a1 1 0 1 0-1.732 1zm9.928 8.196a1 1 0 0 0-.366-1.366l-.027-.014a5 5 0 0 0-.367-.138c-.233-.078-.53-.165-.875-.258a60.619 60.619 0 0 0-2.342-.572l-.147-.033.102.111a60.73 60.73 0 0 0 1.667 1.742c.253.252.477.465.66.629a4.946 4.946 0 0 0 .304.248l.025.017a1 1 0 0 0 1.366-.366zm-3.928 2.196a1 1 0 0 0 1.732-1l-.017-.025a5.065 5.065 0 0 0-.248-.303 16.705 16.705 0 0 0-.629-.661A60.462 60.462 0 0 0 9.23 10.04l-.111-.102.033.147a60.6 60.6 0 0 0 .572 2.342c.093.345.18.642.258.875a4.985 4.985 0 0 0 .138.367.575.575 0 0 0 .014.027zM8 9.5a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/>'),jw=Xu("Flower2",'<path d="M8 16a4 4 0 0 0 4-4 4 4 0 0 0 0-8 4 4 0 0 0-8 0 4 4 0 1 0 0 8 4 4 0 0 0 4 4zm3-12c0 .073-.01.155-.03.247-.544.241-1.091.638-1.598 1.084A2.987 2.987 0 0 0 8 5c-.494 0-.96.12-1.372.331-.507-.446-1.054-.843-1.597-1.084A1.117 1.117 0 0 1 5 4a3 3 0 0 1 6 0zm-.812 6.052A2.99 2.99 0 0 0 11 8a2.99 2.99 0 0 0-.812-2.052c.215-.18.432-.346.647-.487C11.34 5.131 11.732 5 12 5a3 3 0 1 1 0 6c-.268 0-.66-.13-1.165-.461a6.833 6.833 0 0 1-.647-.487zm-3.56.617a3.001 3.001 0 0 0 2.744 0c.507.446 1.054.842 1.598 1.084.02.091.03.174.03.247a3 3 0 1 1-6 0c0-.073.01-.155.03-.247.544-.242 1.091-.638 1.598-1.084zm-.816-4.721A2.99 2.99 0 0 0 5 8c0 .794.308 1.516.812 2.052a6.83 6.83 0 0 1-.647.487C4.66 10.869 4.268 11 4 11a3 3 0 0 1 0-6c.268 0 .66.13 1.165.461.215.141.432.306.647.487zM8 9a1 1 0 1 1 0-2 1 1 0 0 1 0 2z"/>'),xw=Xu("Flower3",'<path d="M11.424 8c.437-.052.811-.136 1.04-.268a2 2 0 0 0-2-3.464c-.229.132-.489.414-.752.767C9.886 4.63 10 4.264 10 4a2 2 0 1 0-4 0c0 .264.114.63.288 1.035-.263-.353-.523-.635-.752-.767a2 2 0 0 0-2 3.464c.229.132.603.216 1.04.268-.437.052-.811.136-1.04.268a2 2 0 1 0 2 3.464c.229-.132.489-.414.752-.767C6.114 11.37 6 11.736 6 12a2 2 0 1 0 4 0c0-.264-.114-.63-.288-1.035.263.353.523.635.752.767a2 2 0 1 0 2-3.464c-.229-.132-.603-.216-1.04-.268zM9 4a1.468 1.468 0 0 1-.045.205c-.039.132-.1.295-.183.484a12.88 12.88 0 0 1-.637 1.223L8 6.142a21.73 21.73 0 0 1-.135-.23 12.88 12.88 0 0 1-.637-1.223 4.216 4.216 0 0 1-.183-.484A1.473 1.473 0 0 1 7 4a1 1 0 1 1 2 0zM3.67 5.5a1 1 0 0 1 1.366-.366 1.472 1.472 0 0 1 .156.142c.094.1.204.233.326.4.245.333.502.747.742 1.163l.13.232a21.86 21.86 0 0 1-.265.002 12.88 12.88 0 0 1-1.379-.06 4.214 4.214 0 0 1-.51-.083 1.47 1.47 0 0 1-.2-.064A1 1 0 0 1 3.67 5.5zm1.366 5.366a1 1 0 0 1-1-1.732c.001 0 .016-.008.047-.02.037-.013.087-.028.153-.044.134-.032.305-.06.51-.083a12.88 12.88 0 0 1 1.379-.06c.09 0 .178 0 .266.002a21.82 21.82 0 0 1-.131.232c-.24.416-.497.83-.742 1.163a4.1 4.1 0 0 1-.327.4 1.483 1.483 0 0 1-.155.142zM9 12a1 1 0 0 1-2 0 1.476 1.476 0 0 1 .045-.206c.039-.131.1-.294.183-.483.166-.378.396-.808.637-1.223L8 9.858l.135.23c.241.415.47.845.637 1.223.083.19.144.352.183.484A1.338 1.338 0 0 1 9 12zm3.33-6.5a1 1 0 0 1-.366 1.366 1.478 1.478 0 0 1-.2.064c-.134.032-.305.06-.51.083-.412.045-.898.061-1.379.06-.09 0-.178 0-.266-.002l.131-.232c.24-.416.497-.83.742-1.163a4.1 4.1 0 0 1 .327-.4c.046-.05.085-.086.114-.11.026-.022.04-.03.041-.032a1 1 0 0 1 1.366.366zm-1.366 5.366a1.494 1.494 0 0 1-.155-.141 4.225 4.225 0 0 1-.327-.4A12.88 12.88 0 0 1 9.74 9.16a22 22 0 0 1-.13-.232l.265-.002c.48-.001.967.015 1.379.06.205.023.376.051.51.083.066.016.116.031.153.044l.048.02a1 1 0 1 1-1 1.732zM8 9a1 1 0 1 1 0-2 1 1 0 0 1 0 2z"/>'),Bw=Xu("Folder",'<path d="M.54 3.87.5 3a2 2 0 0 1 2-2h3.672a2 2 0 0 1 1.414.586l.828.828A2 2 0 0 0 9.828 3h3.982a2 2 0 0 1 1.992 2.181l-.637 7A2 2 0 0 1 13.174 14H2.826a2 2 0 0 1-1.991-1.819l-.637-7a1.99 1.99 0 0 1 .342-1.31zM2.19 4a1 1 0 0 0-.996 1.09l.637 7a1 1 0 0 0 .995.91h10.348a1 1 0 0 0 .995-.91l.637-7A1 1 0 0 0 13.81 4H2.19zm4.69-1.707A1 1 0 0 0 6.172 2H2.5a1 1 0 0 0-1 .981l.006.139C1.72 3.042 1.95 3 2.19 3h5.396l-.707-.707z"/>'),Yw=Xu("Folder2",'<path d="M1 3.5A1.5 1.5 0 0 1 2.5 2h2.764c.958 0 1.76.56 2.311 1.184C7.985 3.648 8.48 4 9 4h4.5A1.5 1.5 0 0 1 15 5.5v7a1.5 1.5 0 0 1-1.5 1.5h-11A1.5 1.5 0 0 1 1 12.5v-9zM2.5 3a.5.5 0 0 0-.5.5V6h12v-.5a.5.5 0 0 0-.5-.5H9c-.964 0-1.71-.629-2.174-1.154C6.374 3.334 5.82 3 5.264 3H2.5zM14 7H2v5.5a.5.5 0 0 0 .5.5h11a.5.5 0 0 0 .5-.5V7z"/>'),Pw=Xu("Folder2Open",'<path d="M1 3.5A1.5 1.5 0 0 1 2.5 2h2.764c.958 0 1.76.56 2.311 1.184C7.985 3.648 8.48 4 9 4h4.5A1.5 1.5 0 0 1 15 5.5v.64c.57.265.94.876.856 1.546l-.64 5.124A2.5 2.5 0 0 1 12.733 15H3.266a2.5 2.5 0 0 1-2.481-2.19l-.64-5.124A1.5 1.5 0 0 1 1 6.14V3.5zM2 6h12v-.5a.5.5 0 0 0-.5-.5H9c-.964 0-1.71-.629-2.174-1.154C6.374 3.334 5.82 3 5.264 3H2.5a.5.5 0 0 0-.5.5V6zm-.367 1a.5.5 0 0 0-.496.562l.64 5.124A1.5 1.5 0 0 0 3.266 14h9.468a1.5 1.5 0 0 0 1.489-1.314l.64-5.124A.5.5 0 0 0 14.367 7H1.633z"/>'),Iw=Xu("FolderCheck",'<path d="m.5 3 .04.87a1.99 1.99 0 0 0-.342 1.311l.637 7A2 2 0 0 0 2.826 14H9v-1H2.826a1 1 0 0 1-.995-.91l-.637-7A1 1 0 0 1 2.19 4h11.62a1 1 0 0 1 .996 1.09L14.54 8h1.005l.256-2.819A2 2 0 0 0 13.81 3H9.828a2 2 0 0 1-1.414-.586l-.828-.828A2 2 0 0 0 6.172 1H2.5a2 2 0 0 0-2 2zm5.672-1a1 1 0 0 1 .707.293L7.586 3H2.19c-.24 0-.47.042-.683.12L1.5 2.98a1 1 0 0 1 1-.98h3.672z"/><path d="M15.854 10.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.707 0l-1.5-1.5a.5.5 0 0 1 .707-.708l1.146 1.147 2.646-2.647a.5.5 0 0 1 .708 0z"/>'),Fw=Xu("FolderFill",'<path d="M9.828 3h3.982a2 2 0 0 1 1.992 2.181l-.637 7A2 2 0 0 1 13.174 14H2.825a2 2 0 0 1-1.991-1.819l-.637-7a1.99 1.99 0 0 1 .342-1.31L.5 3a2 2 0 0 1 2-2h3.672a2 2 0 0 1 1.414.586l.828.828A2 2 0 0 0 9.828 3zm-8.322.12C1.72 3.042 1.95 3 2.19 3h5.396l-.707-.707A1 1 0 0 0 6.172 2H2.5a1 1 0 0 0-1 .981l.006.139z"/>'),Ew=Xu("FolderMinus",'<path d="m.5 3 .04.87a1.99 1.99 0 0 0-.342 1.311l.637 7A2 2 0 0 0 2.826 14H9v-1H2.826a1 1 0 0 1-.995-.91l-.637-7A1 1 0 0 1 2.19 4h11.62a1 1 0 0 1 .996 1.09L14.54 8h1.005l.256-2.819A2 2 0 0 0 13.81 3H9.828a2 2 0 0 1-1.414-.586l-.828-.828A2 2 0 0 0 6.172 1H2.5a2 2 0 0 0-2 2zm5.672-1a1 1 0 0 1 .707.293L7.586 3H2.19c-.24 0-.47.042-.683.12L1.5 2.98a1 1 0 0 1 1-.98h3.672z"/><path d="M11 11.5a.5.5 0 0 1 .5-.5h4a.5.5 0 1 1 0 1h-4a.5.5 0 0 1-.5-.5z"/>'),$w=Xu("FolderPlus",'<path d="m.5 3 .04.87a1.99 1.99 0 0 0-.342 1.311l.637 7A2 2 0 0 0 2.826 14H9v-1H2.826a1 1 0 0 1-.995-.91l-.637-7A1 1 0 0 1 2.19 4h11.62a1 1 0 0 1 .996 1.09L14.54 8h1.005l.256-2.819A2 2 0 0 0 13.81 3H9.828a2 2 0 0 1-1.414-.586l-.828-.828A2 2 0 0 0 6.172 1H2.5a2 2 0 0 0-2 2zm5.672-1a1 1 0 0 1 .707.293L7.586 3H2.19c-.24 0-.47.042-.683.12L1.5 2.98a1 1 0 0 1 1-.98h3.672z"/><path d="M13.5 10a.5.5 0 0 1 .5.5V12h1.5a.5.5 0 1 1 0 1H14v1.5a.5.5 0 1 1-1 0V13h-1.5a.5.5 0 0 1 0-1H13v-1.5a.5.5 0 0 1 .5-.5z"/>'),Rw=Xu("FolderSymlink",'<path d="m11.798 8.271-3.182 1.97c-.27.166-.616-.036-.616-.372V9.1s-2.571-.3-4 2.4c.571-4.8 3.143-4.8 4-4.8v-.769c0-.336.346-.538.616-.371l3.182 1.969c.27.166.27.576 0 .742z"/><path d="m.5 3 .04.87a1.99 1.99 0 0 0-.342 1.311l.637 7A2 2 0 0 0 2.826 14h10.348a2 2 0 0 0 1.991-1.819l.637-7A2 2 0 0 0 13.81 3H9.828a2 2 0 0 1-1.414-.586l-.828-.828A2 2 0 0 0 6.172 1H2.5a2 2 0 0 0-2 2zm.694 2.09A1 1 0 0 1 2.19 4h11.62a1 1 0 0 1 .996 1.09l-.636 7a1 1 0 0 1-.996.91H2.826a1 1 0 0 1-.995-.91l-.637-7zM6.172 2a1 1 0 0 1 .707.293L7.586 3H2.19c-.24 0-.47.042-.683.12L1.5 2.98a1 1 0 0 1 1-.98h3.672z"/>'),Nw=Xu("FolderSymlinkFill",'<path d="M13.81 3H9.828a2 2 0 0 1-1.414-.586l-.828-.828A2 2 0 0 0 6.172 1H2.5a2 2 0 0 0-2 2l.04.87a1.99 1.99 0 0 0-.342 1.311l.637 7A2 2 0 0 0 2.826 14h10.348a2 2 0 0 0 1.991-1.819l.637-7A2 2 0 0 0 13.81 3zM2.19 3c-.24 0-.47.042-.683.12L1.5 2.98a1 1 0 0 1 1-.98h3.672a1 1 0 0 1 .707.293L7.586 3H2.19zm9.608 5.271-3.182 1.97c-.27.166-.616-.036-.616-.372V9.1s-2.571-.3-4 2.4c.571-4.8 3.143-4.8 4-4.8v-.769c0-.336.346-.538.616-.371l3.182 1.969c.27.166.27.576 0 .742z"/>'),Ww=Xu("FolderX",'<path d="M.54 3.87.5 3a2 2 0 0 1 2-2h3.672a2 2 0 0 1 1.414.586l.828.828A2 2 0 0 0 9.828 3h3.982a2 2 0 0 1 1.992 2.181L15.546 8H14.54l.265-2.91A1 1 0 0 0 13.81 4H2.19a1 1 0 0 0-.996 1.09l.637 7a1 1 0 0 0 .995.91H9v1H2.826a2 2 0 0 1-1.991-1.819l-.637-7a1.99 1.99 0 0 1 .342-1.31zm6.339-1.577A1 1 0 0 0 6.172 2H2.5a1 1 0 0 0-1 .981l.006.139C1.72 3.042 1.95 3 2.19 3h5.396l-.707-.707z"/><path d="M11.854 10.146a.5.5 0 0 0-.707.708L12.293 12l-1.146 1.146a.5.5 0 0 0 .707.708L13 12.707l1.146 1.147a.5.5 0 0 0 .708-.708L13.707 12l1.147-1.146a.5.5 0 0 0-.707-.708L13 11.293l-1.146-1.147z"/>'),Uw=Xu("Fonts",'<path d="M12.258 3h-8.51l-.083 2.46h.479c.26-1.544.758-1.783 2.693-1.845l.424-.013v7.827c0 .663-.144.82-1.3.923v.52h4.082v-.52c-1.162-.103-1.306-.26-1.306-.923V3.602l.431.013c1.934.062 2.434.301 2.693 1.846h.479L12.258 3z"/>'),Gw=Xu("Forward",'<path d="M9.502 5.513a.144.144 0 0 0-.202.134V6.65a.5.5 0 0 1-.5.5H2.5v2.9h6.3a.5.5 0 0 1 .5.5v1.003c0 .108.11.176.202.134l3.984-2.933a.51.51 0 0 1 .042-.028.147.147 0 0 0 0-.252.51.51 0 0 1-.042-.028L9.502 5.513zM8.3 5.647a1.144 1.144 0 0 1 1.767-.96l3.994 2.94a1.147 1.147 0 0 1 0 1.946l-3.994 2.94a1.144 1.144 0 0 1-1.767-.96v-.503H2a.5.5 0 0 1-.5-.5v-3.9a.5.5 0 0 1 .5-.5h6.3v-.503z"/>'),qw=Xu("ForwardFill",'<path d="m9.77 12.11 4.012-2.953a.647.647 0 0 0 0-1.114L9.771 5.09a.644.644 0 0 0-.971.557V6.65H2v3.9h6.8v1.003c0 .505.545.808.97.557z"/>'),Jw=Xu("Front",'<path d="M0 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2h2a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2v-2H2a2 2 0 0 1-2-2V2zm5 10v2a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V6a1 1 0 0 0-1-1h-2v5a2 2 0 0 1-2 2H5z"/>'),Kw=Xu("Fullscreen",'<path d="M1.5 1a.5.5 0 0 0-.5.5v4a.5.5 0 0 1-1 0v-4A1.5 1.5 0 0 1 1.5 0h4a.5.5 0 0 1 0 1h-4zM10 .5a.5.5 0 0 1 .5-.5h4A1.5 1.5 0 0 1 16 1.5v4a.5.5 0 0 1-1 0v-4a.5.5 0 0 0-.5-.5h-4a.5.5 0 0 1-.5-.5zM.5 10a.5.5 0 0 1 .5.5v4a.5.5 0 0 0 .5.5h4a.5.5 0 0 1 0 1h-4A1.5 1.5 0 0 1 0 14.5v-4a.5.5 0 0 1 .5-.5zm15 0a.5.5 0 0 1 .5.5v4a1.5 1.5 0 0 1-1.5 1.5h-4a.5.5 0 0 1 0-1h4a.5.5 0 0 0 .5-.5v-4a.5.5 0 0 1 .5-.5z"/>'),Xw=Xu("FullscreenExit",'<path d="M5.5 0a.5.5 0 0 1 .5.5v4A1.5 1.5 0 0 1 4.5 6h-4a.5.5 0 0 1 0-1h4a.5.5 0 0 0 .5-.5v-4a.5.5 0 0 1 .5-.5zm5 0a.5.5 0 0 1 .5.5v4a.5.5 0 0 0 .5.5h4a.5.5 0 0 1 0 1h-4A1.5 1.5 0 0 1 10 4.5v-4a.5.5 0 0 1 .5-.5zM0 10.5a.5.5 0 0 1 .5-.5h4A1.5 1.5 0 0 1 6 11.5v4a.5.5 0 0 1-1 0v-4a.5.5 0 0 0-.5-.5h-4a.5.5 0 0 1-.5-.5zm10 1a1.5 1.5 0 0 1 1.5-1.5h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 0-.5.5v4a.5.5 0 0 1-1 0v-4z"/>'),Zw=Xu("Funnel",'<path d="M1.5 1.5A.5.5 0 0 1 2 1h12a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.128.334L10 8.692V13.5a.5.5 0 0 1-.342.474l-3 1A.5.5 0 0 1 6 14.5V8.692L1.628 3.834A.5.5 0 0 1 1.5 3.5v-2zm1 .5v1.308l4.372 4.858A.5.5 0 0 1 7 8.5v5.306l2-.666V8.5a.5.5 0 0 1 .128-.334L13.5 3.308V2h-11z"/>'),Qw=Xu("FunnelFill",'<path d="M1.5 1.5A.5.5 0 0 1 2 1h12a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.128.334L10 8.692V13.5a.5.5 0 0 1-.342.474l-3 1A.5.5 0 0 1 6 14.5V8.692L1.628 3.834A.5.5 0 0 1 1.5 3.5v-2z"/>'),eH=Xu("Gear",'<path d="M8 4.754a3.246 3.246 0 1 0 0 6.492 3.246 3.246 0 0 0 0-6.492zM5.754 8a2.246 2.246 0 1 1 4.492 0 2.246 2.246 0 0 1-4.492 0z"/><path d="M9.796 1.343c-.527-1.79-3.065-1.79-3.592 0l-.094.319a.873.873 0 0 1-1.255.52l-.292-.16c-1.64-.892-3.433.902-2.54 2.541l.159.292a.873.873 0 0 1-.52 1.255l-.319.094c-1.79.527-1.79 3.065 0 3.592l.319.094a.873.873 0 0 1 .52 1.255l-.16.292c-.892 1.64.901 3.434 2.541 2.54l.292-.159a.873.873 0 0 1 1.255.52l.094.319c.527 1.79 3.065 1.79 3.592 0l.094-.319a.873.873 0 0 1 1.255-.52l.292.16c1.64.893 3.434-.902 2.54-2.541l-.159-.292a.873.873 0 0 1 .52-1.255l.319-.094c1.79-.527 1.79-3.065 0-3.592l-.319-.094a.873.873 0 0 1-.52-1.255l.16-.292c.893-1.64-.902-3.433-2.541-2.54l-.292.159a.873.873 0 0 1-1.255-.52l-.094-.319zm-2.633.283c.246-.835 1.428-.835 1.674 0l.094.319a1.873 1.873 0 0 0 2.693 1.115l.291-.16c.764-.415 1.6.42 1.184 1.185l-.159.292a1.873 1.873 0 0 0 1.116 2.692l.318.094c.835.246.835 1.428 0 1.674l-.319.094a1.873 1.873 0 0 0-1.115 2.693l.16.291c.415.764-.42 1.6-1.185 1.184l-.291-.159a1.873 1.873 0 0 0-2.693 1.116l-.094.318c-.246.835-1.428.835-1.674 0l-.094-.319a1.873 1.873 0 0 0-2.692-1.115l-.292.16c-.764.415-1.6-.42-1.184-1.185l.159-.291A1.873 1.873 0 0 0 1.945 8.93l-.319-.094c-.835-.246-.835-1.428 0-1.674l.319-.094A1.873 1.873 0 0 0 3.06 4.377l-.16-.292c-.415-.764.42-1.6 1.185-1.184l.292.159a1.873 1.873 0 0 0 2.692-1.115l.094-.319z"/>'),tH=Xu("GearFill",'<path d="M9.405 1.05c-.413-1.4-2.397-1.4-2.81 0l-.1.34a1.464 1.464 0 0 1-2.105.872l-.31-.17c-1.283-.698-2.686.705-1.987 1.987l.169.311c.446.82.023 1.841-.872 2.105l-.34.1c-1.4.413-1.4 2.397 0 2.81l.34.1a1.464 1.464 0 0 1 .872 2.105l-.17.31c-.698 1.283.705 2.686 1.987 1.987l.311-.169a1.464 1.464 0 0 1 2.105.872l.1.34c.413 1.4 2.397 1.4 2.81 0l.1-.34a1.464 1.464 0 0 1 2.105-.872l.31.17c1.283.698 2.686-.705 1.987-1.987l-.169-.311a1.464 1.464 0 0 1 .872-2.105l.34-.1c1.4-.413 1.4-2.397 0-2.81l-.34-.1a1.464 1.464 0 0 1-.872-2.105l.17-.31c.698-1.283-.705-2.686-1.987-1.987l-.311.169a1.464 1.464 0 0 1-2.105-.872l-.1-.34zM8 10.93a2.929 2.929 0 1 1 0-5.86 2.929 2.929 0 0 1 0 5.858z"/>'),aH=Xu("GearWide",'<path d="M8.932.727c-.243-.97-1.62-.97-1.864 0l-.071.286a.96.96 0 0 1-1.622.434l-.205-.211c-.695-.719-1.888-.03-1.613.931l.08.284a.96.96 0 0 1-1.186 1.187l-.284-.081c-.96-.275-1.65.918-.931 1.613l.211.205a.96.96 0 0 1-.434 1.622l-.286.071c-.97.243-.97 1.62 0 1.864l.286.071a.96.96 0 0 1 .434 1.622l-.211.205c-.719.695-.03 1.888.931 1.613l.284-.08a.96.96 0 0 1 1.187 1.187l-.081.283c-.275.96.918 1.65 1.613.931l.205-.211a.96.96 0 0 1 1.622.434l.071.286c.243.97 1.62.97 1.864 0l.071-.286a.96.96 0 0 1 1.622-.434l.205.211c.695.719 1.888.03 1.613-.931l-.08-.284a.96.96 0 0 1 1.187-1.187l.283.081c.96.275 1.65-.918.931-1.613l-.211-.205a.96.96 0 0 1 .434-1.622l.286-.071c.97-.243.97-1.62 0-1.864l-.286-.071a.96.96 0 0 1-.434-1.622l.211-.205c.719-.695.03-1.888-.931-1.613l-.284.08a.96.96 0 0 1-1.187-1.186l.081-.284c.275-.96-.918-1.65-1.613-.931l-.205.211a.96.96 0 0 1-1.622-.434L8.932.727zM8 12.997a4.998 4.998 0 1 1 0-9.995 4.998 4.998 0 0 1 0 9.996z"/>'),nH=Xu("GearWideConnected",'<path d="M7.068.727c.243-.97 1.62-.97 1.864 0l.071.286a.96.96 0 0 0 1.622.434l.205-.211c.695-.719 1.888-.03 1.613.931l-.08.284a.96.96 0 0 0 1.187 1.187l.283-.081c.96-.275 1.65.918.931 1.613l-.211.205a.96.96 0 0 0 .434 1.622l.286.071c.97.243.97 1.62 0 1.864l-.286.071a.96.96 0 0 0-.434 1.622l.211.205c.719.695.03 1.888-.931 1.613l-.284-.08a.96.96 0 0 0-1.187 1.187l.081.283c.275.96-.918 1.65-1.613.931l-.205-.211a.96.96 0 0 0-1.622.434l-.071.286c-.243.97-1.62.97-1.864 0l-.071-.286a.96.96 0 0 0-1.622-.434l-.205.211c-.695.719-1.888.03-1.613-.931l.08-.284a.96.96 0 0 0-1.186-1.187l-.284.081c-.96.275-1.65-.918-.931-1.613l.211-.205a.96.96 0 0 0-.434-1.622l-.286-.071c-.97-.243-.97-1.62 0-1.864l.286-.071a.96.96 0 0 0 .434-1.622l-.211-.205c-.719-.695-.03-1.888.931-1.613l.284.08a.96.96 0 0 0 1.187-1.186l-.081-.284c-.275-.96.918-1.65 1.613-.931l.205.211a.96.96 0 0 0 1.622-.434l.071-.286zM12.973 8.5H8.25l-2.834 3.779A4.998 4.998 0 0 0 12.973 8.5zm0-1a4.998 4.998 0 0 0-7.557-3.779l2.834 3.78h4.723zM5.048 3.967c-.03.021-.058.043-.087.065l.087-.065zm-.431.355A4.984 4.984 0 0 0 3.002 8c0 1.455.622 2.765 1.615 3.678L7.375 8 4.617 4.322zm.344 7.646.087.065-.087-.065z"/>'),rH=Xu("Gem",'<path d="M3.1.7a.5.5 0 0 1 .4-.2h9a.5.5 0 0 1 .4.2l2.976 3.974c.149.185.156.45.01.644L8.4 15.3a.5.5 0 0 1-.8 0L.1 5.3a.5.5 0 0 1 0-.6l3-4zm11.386 3.785-1.806-2.41-.776 2.413 2.582-.003zm-3.633.004.961-2.989H4.186l.963 2.995 5.704-.006zM5.47 5.495 8 13.366l2.532-7.876-5.062.005zm-1.371-.999-.78-2.422-1.818 2.425 2.598-.003zM1.499 5.5l5.113 6.817-2.192-6.82L1.5 5.5zm7.889 6.817 5.123-6.83-2.928.002-2.195 6.828z"/>'),iH=Xu("GenderAmbiguous",'<path fill-rule="evenodd" d="M11.5 1a.5.5 0 0 1 0-1h4a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0V1.707l-3.45 3.45A4 4 0 0 1 8.5 10.97V13H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V14H6a.5.5 0 0 1 0-1h1.5v-2.03a4 4 0 1 1 3.471-6.648L14.293 1H11.5zm-.997 4.346a3 3 0 1 0-5.006 3.309 3 3 0 0 0 5.006-3.31z"/>'),oH=Xu("GenderFemale",'<path fill-rule="evenodd" d="M8 1a4 4 0 1 0 0 8 4 4 0 0 0 0-8zM3 5a5 5 0 1 1 5.5 4.975V12h2a.5.5 0 0 1 0 1h-2v2.5a.5.5 0 0 1-1 0V13h-2a.5.5 0 0 1 0-1h2V9.975A5 5 0 0 1 3 5z"/>'),sH=Xu("GenderMale",'<path fill-rule="evenodd" d="M9.5 2a.5.5 0 0 1 0-1h5a.5.5 0 0 1 .5.5v5a.5.5 0 0 1-1 0V2.707L9.871 6.836a5 5 0 1 1-.707-.707L13.293 2H9.5zM6 6a4 4 0 1 0 0 8 4 4 0 0 0 0-8z"/>'),lH=Xu("GenderTrans",'<path fill-rule="evenodd" d="M0 .5A.5.5 0 0 1 .5 0h3a.5.5 0 0 1 0 1H1.707L3.5 2.793l.646-.647a.5.5 0 1 1 .708.708l-.647.646.822.822A3.99 3.99 0 0 1 8 3c1.18 0 2.239.51 2.971 1.322L14.293 1H11.5a.5.5 0 0 1 0-1h4a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0V1.707l-3.45 3.45A4 4 0 0 1 8.5 10.97V13H10a.5.5 0 0 1 0 1H8.5v1.5a.5.5 0 0 1-1 0V14H6a.5.5 0 0 1 0-1h1.5v-2.03a4 4 0 0 1-3.05-5.814l-.95-.949-.646.647a.5.5 0 1 1-.708-.708l.647-.646L1 1.707V3.5a.5.5 0 0 1-1 0v-3zm5.49 4.856a3 3 0 1 0 5.02 3.288 3 3 0 0 0-5.02-3.288z"/>'),cH=Xu("Geo",'<path fill-rule="evenodd" d="M8 1a3 3 0 1 0 0 6 3 3 0 0 0 0-6zM4 4a4 4 0 1 1 4.5 3.969V13.5a.5.5 0 0 1-1 0V7.97A4 4 0 0 1 4 3.999zm2.493 8.574a.5.5 0 0 1-.411.575c-.712.118-1.28.295-1.655.493a1.319 1.319 0 0 0-.37.265.301.301 0 0 0-.057.09V14l.002.008a.147.147 0 0 0 .016.033.617.617 0 0 0 .145.15c.165.13.435.27.813.395.751.25 1.82.414 3.024.414s2.273-.163 3.024-.414c.378-.126.648-.265.813-.395a.619.619 0 0 0 .146-.15.148.148 0 0 0 .015-.033L12 14v-.004a.301.301 0 0 0-.057-.09 1.318 1.318 0 0 0-.37-.264c-.376-.198-.943-.375-1.655-.493a.5.5 0 1 1 .164-.986c.77.127 1.452.328 1.957.594C12.5 13 13 13.4 13 14c0 .426-.26.752-.544.977-.29.228-.68.413-1.116.558-.878.293-2.059.465-3.34.465-1.281 0-2.462-.172-3.34-.465-.436-.145-.826-.33-1.116-.558C3.26 14.752 3 14.426 3 14c0-.599.5-1 .961-1.243.505-.266 1.187-.467 1.957-.594a.5.5 0 0 1 .575.411z"/>'),uH=Xu("GeoAlt",'<path d="M12.166 8.94c-.524 1.062-1.234 2.12-1.96 3.07A31.493 31.493 0 0 1 8 14.58a31.481 31.481 0 0 1-2.206-2.57c-.726-.95-1.436-2.008-1.96-3.07C3.304 7.867 3 6.862 3 6a5 5 0 0 1 10 0c0 .862-.305 1.867-.834 2.94zM8 16s6-5.686 6-10A6 6 0 0 0 2 6c0 4.314 6 10 6 10z"/><path d="M8 8a2 2 0 1 1 0-4 2 2 0 0 1 0 4zm0 1a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/>'),dH=Xu("GeoAltFill",'<path d="M8 16s6-5.686 6-10A6 6 0 0 0 2 6c0 4.314 6 10 6 10zm0-7a3 3 0 1 1 0-6 3 3 0 0 1 0 6z"/>'),hH=Xu("GeoFill",'<path fill-rule="evenodd" d="M4 4a4 4 0 1 1 4.5 3.969V13.5a.5.5 0 0 1-1 0V7.97A4 4 0 0 1 4 3.999zm2.493 8.574a.5.5 0 0 1-.411.575c-.712.118-1.28.295-1.655.493a1.319 1.319 0 0 0-.37.265.301.301 0 0 0-.057.09V14l.002.008a.147.147 0 0 0 .016.033.617.617 0 0 0 .145.15c.165.13.435.27.813.395.751.25 1.82.414 3.024.414s2.273-.163 3.024-.414c.378-.126.648-.265.813-.395a.619.619 0 0 0 .146-.15.148.148 0 0 0 .015-.033L12 14v-.004a.301.301 0 0 0-.057-.09 1.318 1.318 0 0 0-.37-.264c-.376-.198-.943-.375-1.655-.493a.5.5 0 1 1 .164-.986c.77.127 1.452.328 1.957.594C12.5 13 13 13.4 13 14c0 .426-.26.752-.544.977-.29.228-.68.413-1.116.558-.878.293-2.059.465-3.34.465-1.281 0-2.462-.172-3.34-.465-.436-.145-.826-.33-1.116-.558C3.26 14.752 3 14.426 3 14c0-.599.5-1 .961-1.243.505-.266 1.187-.467 1.957-.594a.5.5 0 0 1 .575.411z"/>'),fH=Xu("Gift",'<path d="M3 2.5a2.5 2.5 0 0 1 5 0 2.5 2.5 0 0 1 5 0v.006c0 .07 0 .27-.038.494H15a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1v7.5a1.5 1.5 0 0 1-1.5 1.5h-11A1.5 1.5 0 0 1 1 14.5V7a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h2.038A2.968 2.968 0 0 1 3 2.506V2.5zm1.068.5H7v-.5a1.5 1.5 0 1 0-3 0c0 .085.002.274.045.43a.522.522 0 0 0 .023.07zM9 3h2.932a.56.56 0 0 0 .023-.07c.043-.156.045-.345.045-.43a1.5 1.5 0 0 0-3 0V3zM1 4v2h6V4H1zm8 0v2h6V4H9zm5 3H9v8h4.5a.5.5 0 0 0 .5-.5V7zm-7 8V7H2v7.5a.5.5 0 0 0 .5.5H7z"/>'),pH=Xu("GiftFill",'<path d="M3 2.5a2.5 2.5 0 0 1 5 0 2.5 2.5 0 0 1 5 0v.006c0 .07 0 .27-.038.494H15a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h2.038A2.968 2.968 0 0 1 3 2.506V2.5zm1.068.5H7v-.5a1.5 1.5 0 1 0-3 0c0 .085.002.274.045.43a.522.522 0 0 0 .023.07zM9 3h2.932a.56.56 0 0 0 .023-.07c.043-.156.045-.345.045-.43a1.5 1.5 0 0 0-3 0V3zm6 4v7.5a1.5 1.5 0 0 1-1.5 1.5H9V7h6zM2.5 16A1.5 1.5 0 0 1 1 14.5V7h6v9H2.5z"/>'),mH=Xu("Github",'<path d="M8 0C3.58 0 0 3.58 0 8c0 3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 .67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46.55.38A8.012 8.012 0 0 0 16 8c0-4.42-3.58-8-8-8z"/>'),vH=Xu("Globe",'<path d="M0 8a8 8 0 1 1 16 0A8 8 0 0 1 0 8zm7.5-6.923c-.67.204-1.335.82-1.887 1.855A7.97 7.97 0 0 0 5.145 4H7.5V1.077zM4.09 4a9.267 9.267 0 0 1 .64-1.539 6.7 6.7 0 0 1 .597-.933A7.025 7.025 0 0 0 2.255 4H4.09zm-.582 3.5c.03-.877.138-1.718.312-2.5H1.674a6.958 6.958 0 0 0-.656 2.5h2.49zM4.847 5a12.5 12.5 0 0 0-.338 2.5H7.5V5H4.847zM8.5 5v2.5h2.99a12.495 12.495 0 0 0-.337-2.5H8.5zM4.51 8.5a12.5 12.5 0 0 0 .337 2.5H7.5V8.5H4.51zm3.99 0V11h2.653c.187-.765.306-1.608.338-2.5H8.5zM5.145 12c.138.386.295.744.468 1.068.552 1.035 1.218 1.65 1.887 1.855V12H5.145zm.182 2.472a6.696 6.696 0 0 1-.597-.933A9.268 9.268 0 0 1 4.09 12H2.255a7.024 7.024 0 0 0 3.072 2.472zM3.82 11a13.652 13.652 0 0 1-.312-2.5h-2.49c.062.89.291 1.733.656 2.5H3.82zm6.853 3.472A7.024 7.024 0 0 0 13.745 12H11.91a9.27 9.27 0 0 1-.64 1.539 6.688 6.688 0 0 1-.597.933zM8.5 12v2.923c.67-.204 1.335-.82 1.887-1.855.173-.324.33-.682.468-1.068H8.5zm3.68-1h2.146c.365-.767.594-1.61.656-2.5h-2.49a13.65 13.65 0 0 1-.312 2.5zm2.802-3.5a6.959 6.959 0 0 0-.656-2.5H12.18c.174.782.282 1.623.312 2.5h2.49zM11.27 2.461c.247.464.462.98.64 1.539h1.835a7.024 7.024 0 0 0-3.072-2.472c.218.284.418.598.597.933zM10.855 4a7.966 7.966 0 0 0-.468-1.068C9.835 1.897 9.17 1.282 8.5 1.077V4h2.355z"/>'),_H=Xu("Globe2",'<path d="M0 8a8 8 0 1 1 16 0A8 8 0 0 1 0 8zm7.5-6.923c-.67.204-1.335.82-1.887 1.855-.143.268-.276.56-.395.872.705.157 1.472.257 2.282.287V1.077zM4.249 3.539c.142-.384.304-.744.481-1.078a6.7 6.7 0 0 1 .597-.933A7.01 7.01 0 0 0 3.051 3.05c.362.184.763.349 1.198.49zM3.509 7.5c.036-1.07.188-2.087.436-3.008a9.124 9.124 0 0 1-1.565-.667A6.964 6.964 0 0 0 1.018 7.5h2.49zm1.4-2.741a12.344 12.344 0 0 0-.4 2.741H7.5V5.091c-.91-.03-1.783-.145-2.591-.332zM8.5 5.09V7.5h2.99a12.342 12.342 0 0 0-.399-2.741c-.808.187-1.681.301-2.591.332zM4.51 8.5c.035.987.176 1.914.399 2.741A13.612 13.612 0 0 1 7.5 10.91V8.5H4.51zm3.99 0v2.409c.91.03 1.783.145 2.591.332.223-.827.364-1.754.4-2.741H8.5zm-3.282 3.696c.12.312.252.604.395.872.552 1.035 1.218 1.65 1.887 1.855V11.91c-.81.03-1.577.13-2.282.287zm.11 2.276a6.696 6.696 0 0 1-.598-.933 8.853 8.853 0 0 1-.481-1.079 8.38 8.38 0 0 0-1.198.49 7.01 7.01 0 0 0 2.276 1.522zm-1.383-2.964A13.36 13.36 0 0 1 3.508 8.5h-2.49a6.963 6.963 0 0 0 1.362 3.675c.47-.258.995-.482 1.565-.667zm6.728 2.964a7.009 7.009 0 0 0 2.275-1.521 8.376 8.376 0 0 0-1.197-.49 8.853 8.853 0 0 1-.481 1.078 6.688 6.688 0 0 1-.597.933zM8.5 11.909v3.014c.67-.204 1.335-.82 1.887-1.855.143-.268.276-.56.395-.872A12.63 12.63 0 0 0 8.5 11.91zm3.555-.401c.57.185 1.095.409 1.565.667A6.963 6.963 0 0 0 14.982 8.5h-2.49a13.36 13.36 0 0 1-.437 3.008zM14.982 7.5a6.963 6.963 0 0 0-1.362-3.675c-.47.258-.995.482-1.565.667.248.92.4 1.938.437 3.008h2.49zM11.27 2.461c.177.334.339.694.482 1.078a8.368 8.368 0 0 0 1.196-.49 7.01 7.01 0 0 0-2.275-1.52c.218.283.418.597.597.932zm-.488 1.343a7.765 7.765 0 0 0-.395-.872C9.835 1.897 9.17 1.282 8.5 1.077V4.09c.81-.03 1.577-.13 2.282-.287z"/>'),gH=Xu("Google",'<path d="M15.545 6.558a9.42 9.42 0 0 1 .139 1.626c0 2.434-.87 4.492-2.384 5.885h.002C11.978 15.292 10.158 16 8 16A8 8 0 1 1 8 0a7.689 7.689 0 0 1 5.352 2.082l-2.284 2.284A4.347 4.347 0 0 0 8 3.166c-2.087 0-3.86 1.408-4.492 3.304a4.792 4.792 0 0 0 0 3.063h.003c.635 1.893 2.405 3.301 4.492 3.301 1.078 0 2.004-.276 2.722-.764h-.003a3.702 3.702 0 0 0 1.599-2.431H8v-3.08h7.545z"/>'),yH=Xu("GraphDown",'<path fill-rule="evenodd" d="M0 0h1v15h15v1H0V0zm10 11.5a.5.5 0 0 0 .5.5h4a.5.5 0 0 0 .5-.5v-4a.5.5 0 0 0-1 0v2.6l-3.613-4.417a.5.5 0 0 0-.74-.037L7.06 8.233 3.404 3.206a.5.5 0 0 0-.808.588l4 5.5a.5.5 0 0 0 .758.06l2.609-2.61L13.445 11H10.5a.5.5 0 0 0-.5.5z"/>'),bH=Xu("GraphUp",'<path fill-rule="evenodd" d="M0 0h1v15h15v1H0V0zm10 3.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0V4.9l-3.613 4.417a.5.5 0 0 1-.74.037L7.06 6.767l-3.656 5.027a.5.5 0 0 1-.808-.588l4-5.5a.5.5 0 0 1 .758-.06l2.609 2.61L13.445 4H10.5a.5.5 0 0 1-.5-.5z"/>'),MH=Xu("Grid",'<path d="M1 2.5A1.5 1.5 0 0 1 2.5 1h3A1.5 1.5 0 0 1 7 2.5v3A1.5 1.5 0 0 1 5.5 7h-3A1.5 1.5 0 0 1 1 5.5v-3zM2.5 2a.5.5 0 0 0-.5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 0-.5-.5h-3zm6.5.5A1.5 1.5 0 0 1 10.5 1h3A1.5 1.5 0 0 1 15 2.5v3A1.5 1.5 0 0 1 13.5 7h-3A1.5 1.5 0 0 1 9 5.5v-3zm1.5-.5a.5.5 0 0 0-.5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 0-.5-.5h-3zM1 10.5A1.5 1.5 0 0 1 2.5 9h3A1.5 1.5 0 0 1 7 10.5v3A1.5 1.5 0 0 1 5.5 15h-3A1.5 1.5 0 0 1 1 13.5v-3zm1.5-.5a.5.5 0 0 0-.5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 0-.5-.5h-3zm6.5.5A1.5 1.5 0 0 1 10.5 9h3a1.5 1.5 0 0 1 1.5 1.5v3a1.5 1.5 0 0 1-1.5 1.5h-3A1.5 1.5 0 0 1 9 13.5v-3zm1.5-.5a.5.5 0 0 0-.5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 0-.5-.5h-3z"/>'),zH=Xu("Grid1x2",'<path d="M6 1H1v14h5V1zm9 0h-5v5h5V1zm0 9v5h-5v-5h5zM0 1a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v14a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V1zm9 0a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v5a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1V1zm1 8a1 1 0 0 0-1 1v5a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1v-5a1 1 0 0 0-1-1h-5z"/>'),LH=Xu("Grid1x2Fill",'<path d="M0 1a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v14a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V1zm9 0a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v5a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1V1zm0 9a1 1 0 0 1 1-1h5a1 1 0 0 1 1 1v5a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1v-5z"/>'),wH=Xu("Grid3x2",'<path d="M0 3.5A1.5 1.5 0 0 1 1.5 2h13A1.5 1.5 0 0 1 16 3.5v8a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 11.5v-8zM1.5 3a.5.5 0 0 0-.5.5V7h4V3H1.5zM5 8H1v3.5a.5.5 0 0 0 .5.5H5V8zm1 0v4h4V8H6zm4-1V3H6v4h4zm1 1v4h3.5a.5.5 0 0 0 .5-.5V8h-4zm0-1h4V3.5a.5.5 0 0 0-.5-.5H11v4z"/>'),HH=Xu("Grid3x2Gap",'<path d="M4 4v2H2V4h2zm1 7V9a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm0-5V4a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm5 5V9a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm0-5V4a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zM9 4v2H7V4h2zm5 0h-2v2h2V4zM4 9v2H2V9h2zm5 0v2H7V9h2zm5 0v2h-2V9h2zm-3-5a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1V4zm1 4a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1V9a1 1 0 0 0-1-1h-2z"/>'),kH=Xu("Grid3x2GapFill",'<path d="M1 4a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V4zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1V4zM1 9a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V9zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V9zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1V9z"/>'),AH=Xu("Grid3x3",'<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h13A1.5 1.5 0 0 1 16 1.5v13a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 14.5v-13zM1.5 1a.5.5 0 0 0-.5.5V5h4V1H1.5zM5 6H1v4h4V6zm1 4h4V6H6v4zm-1 1H1v3.5a.5.5 0 0 0 .5.5H5v-4zm1 0v4h4v-4H6zm5 0v4h3.5a.5.5 0 0 0 .5-.5V11h-4zm0-1h4V6h-4v4zm0-5h4V1.5a.5.5 0 0 0-.5-.5H11v4zm-1 0V1H6v4h4z"/>'),SH=Xu("Grid3x3Gap",'<path d="M4 2v2H2V2h2zm1 12v-2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm0-5V7a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm0-5V2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm5 10v-2a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm0-5V7a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zm0-5V2a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1zM9 2v2H7V2h2zm5 0v2h-2V2h2zM4 7v2H2V7h2zm5 0v2H7V7h2zm5 0h-2v2h2V7zM4 12v2H2v-2h2zm5 0v2H7v-2h2zm5 0v2h-2v-2h2zM12 1a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1h-2zm-1 6a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1V7zm1 4a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1v-2a1 1 0 0 0-1-1h-2z"/>'),VH=Xu("Grid3x3GapFill",'<path d="M1 2a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V2zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1V2zM1 7a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V7zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V7zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1V7zM1 12a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1v-2zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1v-2zm5 0a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1v-2z"/>'),OH=Xu("GridFill",'<path d="M1 2.5A1.5 1.5 0 0 1 2.5 1h3A1.5 1.5 0 0 1 7 2.5v3A1.5 1.5 0 0 1 5.5 7h-3A1.5 1.5 0 0 1 1 5.5v-3zm8 0A1.5 1.5 0 0 1 10.5 1h3A1.5 1.5 0 0 1 15 2.5v3A1.5 1.5 0 0 1 13.5 7h-3A1.5 1.5 0 0 1 9 5.5v-3zm-8 8A1.5 1.5 0 0 1 2.5 9h3A1.5 1.5 0 0 1 7 10.5v3A1.5 1.5 0 0 1 5.5 15h-3A1.5 1.5 0 0 1 1 13.5v-3zm8 0A1.5 1.5 0 0 1 10.5 9h3a1.5 1.5 0 0 1 1.5 1.5v3a1.5 1.5 0 0 1-1.5 1.5h-3A1.5 1.5 0 0 1 9 13.5v-3z"/>'),TH=Xu("GripHorizontal",'<path d="M2 8a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm0-3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm3 3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm0-3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm3 3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm0-3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm3 3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm0-3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm3 3a1 1 0 1 1 0 2 1 1 0 0 1 0-2zm0-3a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),DH=Xu("GripVertical",'<path d="M7 2a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm3 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zM7 5a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm3 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zM7 8a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm3 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm-3 3a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm3 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm-3 3a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm3 0a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),CH=Xu("Hammer",'<path d="M9.972 2.508a.5.5 0 0 0-.16-.556l-.178-.129a5.009 5.009 0 0 0-2.076-.783C6.215.862 4.504 1.229 2.84 3.133H1.786a.5.5 0 0 0-.354.147L.146 4.567a.5.5 0 0 0 0 .706l2.571 2.579a.5.5 0 0 0 .708 0l1.286-1.29a.5.5 0 0 0 .146-.353V5.57l8.387 8.873A.5.5 0 0 0 14 14.5l1.5-1.5a.5.5 0 0 0 .017-.689l-9.129-8.63c.747-.456 1.772-.839 3.112-.839a.5.5 0 0 0 .472-.334z"/>'),jH=Xu("HandIndex",'<path d="M6.75 1a.75.75 0 0 1 .75.75V8a.5.5 0 0 0 1 0V5.467l.086-.004c.317-.012.637-.008.816.027.134.027.294.096.448.182.077.042.15.147.15.314V8a.5.5 0 1 0 1 0V6.435a4.9 4.9 0 0 1 .106-.01c.316-.024.584-.01.708.04.118.046.3.207.486.43.081.096.15.19.2.259V8.5a.5.5 0 0 0 1 0v-1h.342a1 1 0 0 1 .995 1.1l-.271 2.715a2.5 2.5 0 0 1-.317.991l-1.395 2.442a.5.5 0 0 1-.434.252H6.035a.5.5 0 0 1-.416-.223l-1.433-2.15a1.5 1.5 0 0 1-.243-.666l-.345-3.105a.5.5 0 0 1 .399-.546L5 8.11V9a.5.5 0 0 0 1 0V1.75A.75.75 0 0 1 6.75 1zM8.5 4.466V1.75a1.75 1.75 0 1 0-3.5 0v5.34l-1.2.24a1.5 1.5 0 0 0-1.196 1.636l.345 3.106a2.5 2.5 0 0 0 .405 1.11l1.433 2.15A1.5 1.5 0 0 0 6.035 16h6.385a1.5 1.5 0 0 0 1.302-.756l1.395-2.441a3.5 3.5 0 0 0 .444-1.389l.271-2.715a2 2 0 0 0-1.99-2.199h-.581a5.114 5.114 0 0 0-.195-.248c-.191-.229-.51-.568-.88-.716-.364-.146-.846-.132-1.158-.108l-.132.012a1.26 1.26 0 0 0-.56-.642 2.632 2.632 0 0 0-.738-.288c-.31-.062-.739-.058-1.05-.046l-.048.002zm2.094 2.025z"/>'),xH=Xu("HandIndexFill",'<path d="M8.5 4.466V1.75a1.75 1.75 0 1 0-3.5 0v5.34l-1.2.24a1.5 1.5 0 0 0-1.196 1.636l.345 3.106a2.5 2.5 0 0 0 .405 1.11l1.433 2.15A1.5 1.5 0 0 0 6.035 16h6.385a1.5 1.5 0 0 0 1.302-.756l1.395-2.441a3.5 3.5 0 0 0 .444-1.389l.271-2.715a2 2 0 0 0-1.99-2.199h-.581a5.114 5.114 0 0 0-.195-.248c-.191-.229-.51-.568-.88-.716-.364-.146-.846-.132-1.158-.108l-.132.012a1.26 1.26 0 0 0-.56-.642 2.632 2.632 0 0 0-.738-.288c-.31-.062-.739-.058-1.05-.046l-.048.002z"/>'),BH=Xu("HandIndexThumb",'<path d="M6.75 1a.75.75 0 0 1 .75.75V8a.5.5 0 0 0 1 0V5.467l.086-.004c.317-.012.637-.008.816.027.134.027.294.096.448.182.077.042.15.147.15.314V8a.5.5 0 0 0 1 0V6.435l.106-.01c.316-.024.584-.01.708.04.118.046.3.207.486.43.081.096.15.19.2.259V8.5a.5.5 0 1 0 1 0v-1h.342a1 1 0 0 1 .995 1.1l-.271 2.715a2.5 2.5 0 0 1-.317.991l-1.395 2.442a.5.5 0 0 1-.434.252H6.118a.5.5 0 0 1-.447-.276l-1.232-2.465-2.512-4.185a.517.517 0 0 1 .809-.631l2.41 2.41A.5.5 0 0 0 6 9.5V1.75A.75.75 0 0 1 6.75 1zM8.5 4.466V1.75a1.75 1.75 0 1 0-3.5 0v6.543L3.443 6.736A1.517 1.517 0 0 0 1.07 8.588l2.491 4.153 1.215 2.43A1.5 1.5 0 0 0 6.118 16h6.302a1.5 1.5 0 0 0 1.302-.756l1.395-2.441a3.5 3.5 0 0 0 .444-1.389l.271-2.715a2 2 0 0 0-1.99-2.199h-.581a5.114 5.114 0 0 0-.195-.248c-.191-.229-.51-.568-.88-.716-.364-.146-.846-.132-1.158-.108l-.132.012a1.26 1.26 0 0 0-.56-.642 2.632 2.632 0 0 0-.738-.288c-.31-.062-.739-.058-1.05-.046l-.048.002zm2.094 2.025z"/>'),YH=Xu("HandIndexThumbFill",'<path d="M8.5 1.75v2.716l.047-.002c.312-.012.742-.016 1.051.046.28.056.543.18.738.288.273.152.456.385.56.642l.132-.012c.312-.024.794-.038 1.158.108.37.148.689.487.88.716.075.09.141.175.195.248h.582a2 2 0 0 1 1.99 2.199l-.272 2.715a3.5 3.5 0 0 1-.444 1.389l-1.395 2.441A1.5 1.5 0 0 1 12.42 16H6.118a1.5 1.5 0 0 1-1.342-.83l-1.215-2.43L1.07 8.589a1.517 1.517 0 0 1 2.373-1.852L5 8.293V1.75a1.75 1.75 0 0 1 3.5 0z"/>'),PH=Xu("HandThumbsDown",'<path d="M8.864 15.674c-.956.24-1.843-.484-1.908-1.42-.072-1.05-.23-2.015-.428-2.59-.125-.36-.479-1.012-1.04-1.638-.557-.624-1.282-1.179-2.131-1.41C2.685 8.432 2 7.85 2 7V3c0-.845.682-1.464 1.448-1.546 1.07-.113 1.564-.415 2.068-.723l.048-.029c.272-.166.578-.349.97-.484C6.931.08 7.395 0 8 0h3.5c.937 0 1.599.478 1.934 1.064.164.287.254.607.254.913 0 .152-.023.312-.077.464.201.262.38.577.488.9.11.33.172.762.004 1.15.069.13.12.268.159.403.077.27.113.567.113.856 0 .289-.036.586-.113.856-.035.12-.08.244-.138.363.394.571.418 1.2.234 1.733-.206.592-.682 1.1-1.2 1.272-.847.283-1.803.276-2.516.211a9.877 9.877 0 0 1-.443-.05 9.364 9.364 0 0 1-.062 4.51c-.138.508-.55.848-1.012.964l-.261.065zM11.5 1H8c-.51 0-.863.068-1.14.163-.281.097-.506.229-.776.393l-.04.025c-.555.338-1.198.73-2.49.868-.333.035-.554.29-.554.55V7c0 .255.226.543.62.65 1.095.3 1.977.997 2.614 1.709.635.71 1.064 1.475 1.238 1.977.243.7.407 1.768.482 2.85.025.362.36.595.667.518l.262-.065c.16-.04.258-.144.288-.255a8.34 8.34 0 0 0-.145-4.726.5.5 0 0 1 .595-.643h.003l.014.004.058.013a8.912 8.912 0 0 0 1.036.157c.663.06 1.457.054 2.11-.163.175-.059.45-.301.57-.651.107-.308.087-.67-.266-1.021L12.793 7l.353-.354c.043-.042.105-.14.154-.315.048-.167.075-.37.075-.581 0-.211-.027-.414-.075-.581-.05-.174-.111-.273-.154-.315l-.353-.354.353-.354c.047-.047.109-.176.005-.488a2.224 2.224 0 0 0-.505-.804l-.353-.354.353-.354c.006-.005.041-.05.041-.17a.866.866 0 0 0-.121-.415C12.4 1.272 12.063 1 11.5 1z"/>'),IH=Xu("HandThumbsDownFill",'<path d="M6.956 14.534c.065.936.952 1.659 1.908 1.42l.261-.065a1.378 1.378 0 0 0 1.012-.965c.22-.816.533-2.512.062-4.51.136.02.285.037.443.051.713.065 1.669.071 2.516-.211.518-.173.994-.68 1.2-1.272a1.896 1.896 0 0 0-.234-1.734c.058-.118.103-.242.138-.362.077-.27.113-.568.113-.856 0-.29-.036-.586-.113-.857a2.094 2.094 0 0 0-.16-.403c.169-.387.107-.82-.003-1.149a3.162 3.162 0 0 0-.488-.9c.054-.153.076-.313.076-.465a1.86 1.86 0 0 0-.253-.912C13.1.757 12.437.28 11.5.28H8c-.605 0-1.07.08-1.466.217a4.823 4.823 0 0 0-.97.485l-.048.029c-.504.308-.999.61-2.068.723C2.682 1.815 2 2.434 2 3.279v4c0 .851.685 1.433 1.357 1.616.849.232 1.574.787 2.132 1.41.56.626.914 1.28 1.039 1.638.199.575.356 1.54.428 2.591z"/>'),FH=Xu("HandThumbsUp",'<path d="M8.864.046C7.908-.193 7.02.53 6.956 1.466c-.072 1.051-.23 2.016-.428 2.59-.125.36-.479 1.013-1.04 1.639-.557.623-1.282 1.178-2.131 1.41C2.685 7.288 2 7.87 2 8.72v4.001c0 .845.682 1.464 1.448 1.545 1.07.114 1.564.415 2.068.723l.048.03c.272.165.578.348.97.484.397.136.861.217 1.466.217h3.5c.937 0 1.599-.477 1.934-1.064a1.86 1.86 0 0 0 .254-.912c0-.152-.023-.312-.077-.464.201-.263.38-.578.488-.901.11-.33.172-.762.004-1.149.069-.13.12-.269.159-.403.077-.27.113-.568.113-.857 0-.288-.036-.585-.113-.856a2.144 2.144 0 0 0-.138-.362 1.9 1.9 0 0 0 .234-1.734c-.206-.592-.682-1.1-1.2-1.272-.847-.282-1.803-.276-2.516-.211a9.84 9.84 0 0 0-.443.05 9.365 9.365 0 0 0-.062-4.509A1.38 1.38 0 0 0 9.125.111L8.864.046zM11.5 14.721H8c-.51 0-.863-.069-1.14-.164-.281-.097-.506-.228-.776-.393l-.04-.024c-.555-.339-1.198-.731-2.49-.868-.333-.036-.554-.29-.554-.55V8.72c0-.254.226-.543.62-.65 1.095-.3 1.977-.996 2.614-1.708.635-.71 1.064-1.475 1.238-1.978.243-.7.407-1.768.482-2.85.025-.362.36-.594.667-.518l.262.066c.16.04.258.143.288.255a8.34 8.34 0 0 1-.145 4.725.5.5 0 0 0 .595.644l.003-.001.014-.003.058-.014a8.908 8.908 0 0 1 1.036-.157c.663-.06 1.457-.054 2.11.164.175.058.45.3.57.65.107.308.087.67-.266 1.022l-.353.353.353.354c.043.043.105.141.154.315.048.167.075.37.075.581 0 .212-.027.414-.075.582-.05.174-.111.272-.154.315l-.353.353.353.354c.047.047.109.177.005.488a2.224 2.224 0 0 1-.505.805l-.353.353.353.354c.006.005.041.05.041.17a.866.866 0 0 1-.121.416c-.165.288-.503.56-1.066.56z"/>'),EH=Xu("HandThumbsUpFill",'<path d="M6.956 1.745C7.021.81 7.908.087 8.864.325l.261.066c.463.116.874.456 1.012.965.22.816.533 2.511.062 4.51a9.84 9.84 0 0 1 .443-.051c.713-.065 1.669-.072 2.516.21.518.173.994.681 1.2 1.273.184.532.16 1.162-.234 1.733.058.119.103.242.138.363.077.27.113.567.113.856 0 .289-.036.586-.113.856-.039.135-.09.273-.16.404.169.387.107.819-.003 1.148a3.163 3.163 0 0 1-.488.901c.054.152.076.312.076.465 0 .305-.089.625-.253.912C13.1 15.522 12.437 16 11.5 16H8c-.605 0-1.07-.081-1.466-.218a4.82 4.82 0 0 1-.97-.484l-.048-.03c-.504-.307-.999-.609-2.068-.722C2.682 14.464 2 13.846 2 13V9c0-.85.685-1.432 1.357-1.615.849-.232 1.574-.787 2.132-1.41.56-.627.914-1.28 1.039-1.639.199-.575.356-1.539.428-2.59z"/>'),$H=Xu("Handbag",'<path d="M8 1a2 2 0 0 1 2 2v2H6V3a2 2 0 0 1 2-2zm3 4V3a3 3 0 1 0-6 0v2H3.36a1.5 1.5 0 0 0-1.483 1.277L.85 13.13A2.5 2.5 0 0 0 3.322 16h9.355a2.5 2.5 0 0 0 2.473-2.87l-1.028-6.853A1.5 1.5 0 0 0 12.64 5H11zm-1 1v1.5a.5.5 0 0 0 1 0V6h1.639a.5.5 0 0 1 .494.426l1.028 6.851A1.5 1.5 0 0 1 12.678 15H3.322a1.5 1.5 0 0 1-1.483-1.723l1.028-6.851A.5.5 0 0 1 3.36 6H5v1.5a.5.5 0 1 0 1 0V6h4z"/>'),RH=Xu("HandbagFill",'<path d="M8 1a2 2 0 0 0-2 2v2H5V3a3 3 0 1 1 6 0v2h-1V3a2 2 0 0 0-2-2zM5 5H3.36a1.5 1.5 0 0 0-1.483 1.277L.85 13.13A2.5 2.5 0 0 0 3.322 16h9.355a2.5 2.5 0 0 0 2.473-2.87l-1.028-6.853A1.5 1.5 0 0 0 12.64 5H11v1.5a.5.5 0 0 1-1 0V5H6v1.5a.5.5 0 0 1-1 0V5z"/>'),NH=Xu("Hash",'<path d="M8.39 12.648a1.32 1.32 0 0 0-.015.18c0 .305.21.508.5.508.266 0 .492-.172.555-.477l.554-2.703h1.204c.421 0 .617-.234.617-.547 0-.312-.188-.53-.617-.53h-.985l.516-2.524h1.265c.43 0 .618-.227.618-.547 0-.313-.188-.524-.618-.524h-1.046l.476-2.304a1.06 1.06 0 0 0 .016-.164.51.51 0 0 0-.516-.516.54.54 0 0 0-.539.43l-.523 2.554H7.617l.477-2.304c.008-.04.015-.118.015-.164a.512.512 0 0 0-.523-.516.539.539 0 0 0-.531.43L6.53 5.484H5.414c-.43 0-.617.22-.617.532 0 .312.187.539.617.539h.906l-.515 2.523H4.609c-.421 0-.609.219-.609.531 0 .313.188.547.61.547h.976l-.516 2.492c-.008.04-.015.125-.015.18 0 .305.21.508.5.508.265 0 .492-.172.554-.477l.555-2.703h2.242l-.515 2.492zm-1-6.109h2.266l-.515 2.563H6.859l.532-2.563z"/>'),WH=Xu("Hdd",'<path d="M4.5 11a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zM3 10.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0z"/><path d="M16 11a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V9.51c0-.418.105-.83.305-1.197l2.472-4.531A1.5 1.5 0 0 1 4.094 3h7.812a1.5 1.5 0 0 1 1.317.782l2.472 4.53c.2.368.305.78.305 1.198V11zM3.655 4.26 1.592 8.043C1.724 8.014 1.86 8 2 8h12c.14 0 .276.014.408.042L12.345 4.26a.5.5 0 0 0-.439-.26H4.094a.5.5 0 0 0-.44.26zM1 10v1a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-1a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1z"/>'),UH=Xu("HddFill",'<path d="M0 10a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v1a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2v-1zm2.5 1a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zm2 0a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zM.91 7.204A2.993 2.993 0 0 1 2 7h12c.384 0 .752.072 1.09.204l-1.867-3.422A1.5 1.5 0 0 0 11.906 3H4.094a1.5 1.5 0 0 0-1.317.782L.91 7.204z"/>'),GH=Xu("HddNetwork",'<path d="M4.5 5a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zM3 4.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v1a2 2 0 0 1-2 2H8.5v3a1.5 1.5 0 0 1 1.5 1.5h5.5a.5.5 0 0 1 0 1H10A1.5 1.5 0 0 1 8.5 14h-1A1.5 1.5 0 0 1 6 12.5H.5a.5.5 0 0 1 0-1H6A1.5 1.5 0 0 1 7.5 10V7H2a2 2 0 0 1-2-2V4zm1 0v1a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1zm6 7.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5z"/>'),qH=Xu("HddNetworkFill",'<path d="M2 2a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h5.5v3A1.5 1.5 0 0 0 6 11.5H.5a.5.5 0 0 0 0 1H6A1.5 1.5 0 0 0 7.5 14h1a1.5 1.5 0 0 0 1.5-1.5h5.5a.5.5 0 0 0 0-1H10A1.5 1.5 0 0 0 8.5 10V7H14a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm.5 3a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm2 0a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1z"/>'),JH=Xu("HddRack",'<path d="M4.5 5a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zM3 4.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm2 7a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm-2.5.5a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1z"/><path d="M2 2a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h1v2H2a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-1a2 2 0 0 0-2-2h-1V7h1a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm13 2v1a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1zm0 7v1a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1v-1a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1zm-3-4v2H4V7h8z"/>'),KH=Xu("HddRackFill",'<path d="M2 2a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h1v2H2a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-1a2 2 0 0 0-2-2h-1V7h1a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm.5 3a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm2 0a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm-2 7a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm2 0a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zM12 7v2H4V7h8z"/>'),XH=Xu("HddStack",'<path d="M14 10a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1v-1a1 1 0 0 1 1-1h12zM2 9a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-1a2 2 0 0 0-2-2H2z"/><path d="M5 11.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm-2 0a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zM14 3a1 1 0 0 1 1 1v1a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/><path d="M5 4.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm-2 0a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0z"/>'),ZH=Xu("HddStackFill",'<path d="M2 9a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-1a2 2 0 0 0-2-2H2zm.5 3a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm2 0a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zM2 2a2 2 0 0 0-2 2v1a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2zm.5 3a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm2 0a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1z"/>'),QH=Xu("Headphones",'<path d="M8 3a5 5 0 0 0-5 5v1h1a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V8a6 6 0 1 1 12 0v5a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1v-3a1 1 0 0 1 1-1h1V8a5 5 0 0 0-5-5z"/>'),ek=Xu("Headset",'<path d="M8 1a5 5 0 0 0-5 5v1h1a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V6a6 6 0 1 1 12 0v6a2.5 2.5 0 0 1-2.5 2.5H9.366a1 1 0 0 1-.866.5h-1a1 1 0 1 1 0-2h1a1 1 0 0 1 .866.5H11.5A1.5 1.5 0 0 0 13 12h-1a1 1 0 0 1-1-1V8a1 1 0 0 1 1-1h1V6a5 5 0 0 0-5-5z"/>'),tk=Xu("HeadsetVr",'<path d="M8 1.248c1.857 0 3.526.641 4.65 1.794a4.978 4.978 0 0 1 2.518 1.09C13.907 1.482 11.295 0 8 0 4.75 0 2.12 1.48.844 4.122a4.979 4.979 0 0 1 2.289-1.047C4.236 1.872 5.974 1.248 8 1.248z"/><path d="M12 12a3.988 3.988 0 0 1-2.786-1.13l-.002-.002a1.612 1.612 0 0 0-.276-.167A2.164 2.164 0 0 0 8 10.5c-.414 0-.729.103-.935.201a1.612 1.612 0 0 0-.277.167l-.002.002A4 4 0 1 1 4 4h8a4 4 0 0 1 0 8z"/>'),ak=Xu("Heart",'<path d="m8 2.748-.717-.737C5.6.281 2.514.878 1.4 3.053c-.523 1.023-.641 2.5.314 4.385.92 1.815 2.834 3.989 6.286 6.357 3.452-2.368 5.365-4.542 6.286-6.357.955-1.886.838-3.362.314-4.385C13.486.878 10.4.28 8.717 2.01L8 2.748zM8 15C-7.333 4.868 3.279-3.04 7.824 1.143c.06.055.119.112.176.171a3.12 3.12 0 0 1 .176-.17C12.72-3.042 23.333 4.867 8 15z"/>'),nk=Xu("HeartFill",'<path fill-rule="evenodd" d="M8 1.314C12.438-3.248 23.534 4.735 8 15-7.534 4.736 3.562-3.248 8 1.314z"/>'),rk=Xu("HeartHalf",'<path d="M8 2.748v11.047c3.452-2.368 5.365-4.542 6.286-6.357.955-1.886.838-3.362.314-4.385C13.486.878 10.4.28 8.717 2.01L8 2.748zM8 15C-7.333 4.868 3.279-3.04 7.824 1.143c.06.055.119.112.176.171a3.12 3.12 0 0 1 .176-.17C12.72-3.042 23.333 4.867 8 15z"/>'),ik=Xu("Heptagon",'<path d="M7.779.052a.5.5 0 0 1 .442 0l6.015 2.97a.5.5 0 0 1 .267.34l1.485 6.676a.5.5 0 0 1-.093.415l-4.162 5.354a.5.5 0 0 1-.395.193H4.662a.5.5 0 0 1-.395-.193L.105 10.453a.5.5 0 0 1-.093-.415l1.485-6.676a.5.5 0 0 1 .267-.34L7.779.053zM2.422 3.813l-1.383 6.212L4.907 15h6.186l3.868-4.975-1.383-6.212L8 1.058 2.422 3.813z"/>'),ok=Xu("HeptagonFill",'<path fill-rule="evenodd" d="M7.779.052a.5.5 0 0 1 .442 0l6.015 2.97a.5.5 0 0 1 .267.34l1.485 6.676a.5.5 0 0 1-.093.415l-4.162 5.354a.5.5 0 0 1-.395.193H4.662a.5.5 0 0 1-.395-.193L.105 10.453a.5.5 0 0 1-.093-.415l1.485-6.676a.5.5 0 0 1 .267-.34L7.779.053z"/>'),sk=Xu("HeptagonHalf",'<path d="M7.779.052a.5.5 0 0 1 .442 0l6.015 2.97a.5.5 0 0 1 .267.34l1.485 6.676a.5.5 0 0 1-.093.415l-4.162 5.354a.5.5 0 0 1-.395.193H4.662a.5.5 0 0 1-.395-.193L.105 10.453a.5.5 0 0 1-.093-.415l1.485-6.676a.5.5 0 0 1 .267-.34L7.779.053zM8 15h3.093l3.868-4.975-1.383-6.212L8 1.058V15z"/>'),lk=Xu("Hexagon",'<path d="M14 4.577v6.846L8 15l-6-3.577V4.577L8 1l6 3.577zM8.5.134a1 1 0 0 0-1 0l-6 3.577a1 1 0 0 0-.5.866v6.846a1 1 0 0 0 .5.866l6 3.577a1 1 0 0 0 1 0l6-3.577a1 1 0 0 0 .5-.866V4.577a1 1 0 0 0-.5-.866L8.5.134z"/>'),ck=Xu("HexagonFill",'<path fill-rule="evenodd" d="M8.5.134a1 1 0 0 0-1 0l-6 3.577a1 1 0 0 0-.5.866v6.846a1 1 0 0 0 .5.866l6 3.577a1 1 0 0 0 1 0l6-3.577a1 1 0 0 0 .5-.866V4.577a1 1 0 0 0-.5-.866L8.5.134z"/>'),uk=Xu("HexagonHalf",'<path d="M14 4.577v6.846L8 15V1l6 3.577zM8.5.134a1 1 0 0 0-1 0l-6 3.577a1 1 0 0 0-.5.866v6.846a1 1 0 0 0 .5.866l6 3.577a1 1 0 0 0 1 0l6-3.577a1 1 0 0 0 .5-.866V4.577a1 1 0 0 0-.5-.866L8.5.134z"/>'),dk=Xu("Hourglass",'<path d="M2 1.5a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-1v1a4.5 4.5 0 0 1-2.557 4.06c-.29.139-.443.377-.443.59v.7c0 .213.154.451.443.59A4.5 4.5 0 0 1 12.5 13v1h1a.5.5 0 0 1 0 1h-11a.5.5 0 1 1 0-1h1v-1a4.5 4.5 0 0 1 2.557-4.06c.29-.139.443-.377.443-.59v-.7c0-.213-.154-.451-.443-.59A4.5 4.5 0 0 1 3.5 3V2h-1a.5.5 0 0 1-.5-.5zm2.5.5v1a3.5 3.5 0 0 0 1.989 3.158c.533.256 1.011.791 1.011 1.491v.702c0 .7-.478 1.235-1.011 1.491A3.5 3.5 0 0 0 4.5 13v1h7v-1a3.5 3.5 0 0 0-1.989-3.158C8.978 9.586 8.5 9.052 8.5 8.351v-.702c0-.7.478-1.235 1.011-1.491A3.5 3.5 0 0 0 11.5 3V2h-7z"/>'),hk=Xu("HourglassBottom",'<path d="M2 1.5a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-1v1a4.5 4.5 0 0 1-2.557 4.06c-.29.139-.443.377-.443.59v.7c0 .213.154.451.443.59A4.5 4.5 0 0 1 12.5 13v1h1a.5.5 0 0 1 0 1h-11a.5.5 0 1 1 0-1h1v-1a4.5 4.5 0 0 1 2.557-4.06c.29-.139.443-.377.443-.59v-.7c0-.213-.154-.451-.443-.59A4.5 4.5 0 0 1 3.5 3V2h-1a.5.5 0 0 1-.5-.5zm2.5.5v1a3.5 3.5 0 0 0 1.989 3.158c.533.256 1.011.791 1.011 1.491v.702s.18.149.5.149.5-.15.5-.15v-.7c0-.701.478-1.236 1.011-1.492A3.5 3.5 0 0 0 11.5 3V2h-7z"/>'),fk=Xu("HourglassSplit",'<path d="M2.5 15a.5.5 0 1 1 0-1h1v-1a4.5 4.5 0 0 1 2.557-4.06c.29-.139.443-.377.443-.59v-.7c0-.213-.154-.451-.443-.59A4.5 4.5 0 0 1 3.5 3V2h-1a.5.5 0 0 1 0-1h11a.5.5 0 0 1 0 1h-1v1a4.5 4.5 0 0 1-2.557 4.06c-.29.139-.443.377-.443.59v.7c0 .213.154.451.443.59A4.5 4.5 0 0 1 12.5 13v1h1a.5.5 0 0 1 0 1h-11zm2-13v1c0 .537.12 1.045.337 1.5h6.326c.216-.455.337-.963.337-1.5V2h-7zm3 6.35c0 .701-.478 1.236-1.011 1.492A3.5 3.5 0 0 0 4.5 13s.866-1.299 3-1.48V8.35zm1 0v3.17c2.134.181 3 1.48 3 1.48a3.5 3.5 0 0 0-1.989-3.158C8.978 9.586 8.5 9.052 8.5 8.351z"/>'),pk=Xu("HourglassTop",'<path d="M2 14.5a.5.5 0 0 0 .5.5h11a.5.5 0 1 0 0-1h-1v-1a4.5 4.5 0 0 0-2.557-4.06c-.29-.139-.443-.377-.443-.59v-.7c0-.213.154-.451.443-.59A4.5 4.5 0 0 0 12.5 3V2h1a.5.5 0 0 0 0-1h-11a.5.5 0 0 0 0 1h1v1a4.5 4.5 0 0 0 2.557 4.06c.29.139.443.377.443.59v.7c0 .213-.154.451-.443.59A4.5 4.5 0 0 0 3.5 13v1h-1a.5.5 0 0 0-.5.5zm2.5-.5v-1a3.5 3.5 0 0 1 1.989-3.158c.533-.256 1.011-.79 1.011-1.491v-.702s.18.101.5.101.5-.1.5-.1v.7c0 .701.478 1.236 1.011 1.492A3.5 3.5 0 0 1 11.5 13v1h-7z"/>'),mk=Xu("House",'<path fill-rule="evenodd" d="M2 13.5V7h1v6.5a.5.5 0 0 0 .5.5h9a.5.5 0 0 0 .5-.5V7h1v6.5a1.5 1.5 0 0 1-1.5 1.5h-9A1.5 1.5 0 0 1 2 13.5zm11-11V6l-2-2V2.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5z"/><path fill-rule="evenodd" d="M7.293 1.5a1 1 0 0 1 1.414 0l6.647 6.646a.5.5 0 0 1-.708.708L8 2.207 1.354 8.854a.5.5 0 1 1-.708-.708L7.293 1.5z"/>'),vk=Xu("HouseDoor",'<path d="M8.354 1.146a.5.5 0 0 0-.708 0l-6 6A.5.5 0 0 0 1.5 7.5v7a.5.5 0 0 0 .5.5h4.5a.5.5 0 0 0 .5-.5v-4h2v4a.5.5 0 0 0 .5.5H14a.5.5 0 0 0 .5-.5v-7a.5.5 0 0 0-.146-.354L13 5.793V2.5a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5v1.293L8.354 1.146zM2.5 14V7.707l5.5-5.5 5.5 5.5V14H10v-4a.5.5 0 0 0-.5-.5h-3a.5.5 0 0 0-.5.5v4H2.5z"/>'),_k=Xu("HouseDoorFill",'<path d="M6.5 14.5v-3.505c0-.245.25-.495.5-.495h2c.25 0 .5.25.5.5v3.5a.5.5 0 0 0 .5.5h4a.5.5 0 0 0 .5-.5v-7a.5.5 0 0 0-.146-.354L13 5.793V2.5a.5.5 0 0 0-.5-.5h-1a.5.5 0 0 0-.5.5v1.293L8.354 1.146a.5.5 0 0 0-.708 0l-6 6A.5.5 0 0 0 1.5 7.5v7a.5.5 0 0 0 .5.5h4a.5.5 0 0 0 .5-.5z"/>'),gk=Xu("HouseFill",'<path fill-rule="evenodd" d="m8 3.293 6 6V13.5a1.5 1.5 0 0 1-1.5 1.5h-9A1.5 1.5 0 0 1 2 13.5V9.293l6-6zm5-.793V6l-2-2V2.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5z"/><path fill-rule="evenodd" d="M7.293 1.5a1 1 0 0 1 1.414 0l6.647 6.646a.5.5 0 0 1-.708.708L8 2.207 1.354 8.854a.5.5 0 1 1-.708-.708L7.293 1.5z"/>'),yk=Xu("Hr",'<path d="M12 3H4a1 1 0 0 0-1 1v2.5H2V4a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2.5h-1V4a1 1 0 0 0-1-1zM2 9.5h1V12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V9.5h1V12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V9.5zm-1.5-2a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1H.5z"/>'),bk=Xu("Hurricane",'<path d="M6.999 2.6A5.5 5.5 0 0 1 15 7.5a.5.5 0 0 0 1 0 6.5 6.5 0 1 0-13 0 5 5 0 0 0 6.001 4.9A5.5 5.5 0 0 1 1 7.5a.5.5 0 0 0-1 0 6.5 6.5 0 1 0 13 0 5 5 0 0 0-6.001-4.9zM10 7.5a2 2 0 1 1-4 0 2 2 0 0 1 4 0z"/>'),Mk=Xu("Image",'<path d="M6.002 5.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/><path d="M2.002 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2h-12zm12 1a1 1 0 0 1 1 1v6.5l-3.777-1.947a.5.5 0 0 0-.577.093l-3.71 3.71-2.66-1.772a.5.5 0 0 0-.63.062L1.002 12V3a1 1 0 0 1 1-1h12z"/>'),zk=Xu("ImageAlt",'<path d="M7 2.5a2.5 2.5 0 1 1-5 0 2.5 2.5 0 0 1 5 0zm4.225 4.053a.5.5 0 0 0-.577.093l-3.71 4.71-2.66-2.772a.5.5 0 0 0-.63.062L.002 13v2a1 1 0 0 0 1 1h14a1 1 0 0 0 1-1v-4.5l-4.777-3.947z"/>'),Lk=Xu("ImageFill",'<path d="M.002 3a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2h-12a2 2 0 0 1-2-2V3zm1 9v1a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V9.5l-3.777-1.947a.5.5 0 0 0-.577.093l-3.71 3.71-2.66-1.772a.5.5 0 0 0-.63.062L1.002 12zm5-6.5a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0z"/>'),wk=Xu("Images",'<path d="M4.502 9a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/><path d="M14.002 13a2 2 0 0 1-2 2h-10a2 2 0 0 1-2-2V5A2 2 0 0 1 2 3a2 2 0 0 1 2-2h10a2 2 0 0 1 2 2v8a2 2 0 0 1-1.998 2zM14 2H4a1 1 0 0 0-1 1h9.002a2 2 0 0 1 2 2v7A1 1 0 0 0 15 11V3a1 1 0 0 0-1-1zM2.002 4a1 1 0 0 0-1 1v8l2.646-2.354a.5.5 0 0 1 .63-.062l2.66 1.773 3.71-3.71a.5.5 0 0 1 .577-.094l1.777 1.947V5a1 1 0 0 0-1-1h-10z"/>'),Hk=Xu("Inbox",'<path d="M4.98 4a.5.5 0 0 0-.39.188L1.54 8H6a.5.5 0 0 1 .5.5 1.5 1.5 0 1 0 3 0A.5.5 0 0 1 10 8h4.46l-3.05-3.812A.5.5 0 0 0 11.02 4H4.98zm9.954 5H10.45a2.5 2.5 0 0 1-4.9 0H1.066l.32 2.562a.5.5 0 0 0 .497.438h12.234a.5.5 0 0 0 .496-.438L14.933 9zM3.809 3.563A1.5 1.5 0 0 1 4.981 3h6.038a1.5 1.5 0 0 1 1.172.563l3.7 4.625a.5.5 0 0 1 .105.374l-.39 3.124A1.5 1.5 0 0 1 14.117 13H1.883a1.5 1.5 0 0 1-1.489-1.314l-.39-3.124a.5.5 0 0 1 .106-.374l3.7-4.625z"/>'),kk=Xu("InboxFill",'<path d="M4.98 4a.5.5 0 0 0-.39.188L1.54 8H6a.5.5 0 0 1 .5.5 1.5 1.5 0 1 0 3 0A.5.5 0 0 1 10 8h4.46l-3.05-3.812A.5.5 0 0 0 11.02 4H4.98zm-1.17-.437A1.5 1.5 0 0 1 4.98 3h6.04a1.5 1.5 0 0 1 1.17.563l3.7 4.625a.5.5 0 0 1 .106.374l-.39 3.124A1.5 1.5 0 0 1 14.117 13H1.883a1.5 1.5 0 0 1-1.489-1.314l-.39-3.124a.5.5 0 0 1 .106-.374l3.7-4.625z"/>'),Ak=Xu("Inboxes",'<path d="M4.98 1a.5.5 0 0 0-.39.188L1.54 5H6a.5.5 0 0 1 .5.5 1.5 1.5 0 0 0 3 0A.5.5 0 0 1 10 5h4.46l-3.05-3.812A.5.5 0 0 0 11.02 1H4.98zm9.954 5H10.45a2.5 2.5 0 0 1-4.9 0H1.066l.32 2.562A.5.5 0 0 0 1.884 9h12.234a.5.5 0 0 0 .496-.438L14.933 6zM3.809.563A1.5 1.5 0 0 1 4.981 0h6.038a1.5 1.5 0 0 1 1.172.563l3.7 4.625a.5.5 0 0 1 .105.374l-.39 3.124A1.5 1.5 0 0 1 14.117 10H1.883A1.5 1.5 0 0 1 .394 8.686l-.39-3.124a.5.5 0 0 1 .106-.374L3.81.563zM.125 11.17A.5.5 0 0 1 .5 11H6a.5.5 0 0 1 .5.5 1.5 1.5 0 0 0 3 0 .5.5 0 0 1 .5-.5h5.5a.5.5 0 0 1 .496.562l-.39 3.124A1.5 1.5 0 0 1 14.117 16H1.883a1.5 1.5 0 0 1-1.489-1.314l-.39-3.124a.5.5 0 0 1 .121-.393zm.941.83.32 2.562a.5.5 0 0 0 .497.438h12.234a.5.5 0 0 0 .496-.438l.32-2.562H10.45a2.5 2.5 0 0 1-4.9 0H1.066z"/>'),Sk=Xu("InboxesFill",'<path d="M4.98 1a.5.5 0 0 0-.39.188L1.54 5H6a.5.5 0 0 1 .5.5 1.5 1.5 0 0 0 3 0A.5.5 0 0 1 10 5h4.46l-3.05-3.812A.5.5 0 0 0 11.02 1H4.98zM3.81.563A1.5 1.5 0 0 1 4.98 0h6.04a1.5 1.5 0 0 1 1.17.563l3.7 4.625a.5.5 0 0 1 .106.374l-.39 3.124A1.5 1.5 0 0 1 14.117 10H1.883A1.5 1.5 0 0 1 .394 8.686l-.39-3.124a.5.5 0 0 1 .106-.374L3.81.563zM.125 11.17A.5.5 0 0 1 .5 11H6a.5.5 0 0 1 .5.5 1.5 1.5 0 0 0 3 0 .5.5 0 0 1 .5-.5h5.5a.5.5 0 0 1 .496.562l-.39 3.124A1.5 1.5 0 0 1 14.117 16H1.883a1.5 1.5 0 0 1-1.489-1.314l-.39-3.124a.5.5 0 0 1 .121-.393z"/>'),Vk=Xu("Info",'<path d="m8.93 6.588-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM9 4.5a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),Ok=Xu("InfoCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="m8.93 6.588-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM9 4.5a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),Tk=Xu("InfoCircleFill",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm.93-9.412-1 4.705c-.07.34.029.533.304.533.194 0 .487-.07.686-.246l-.088.416c-.287.346-.92.598-1.465.598-.703 0-1.002-.422-.808-1.319l.738-3.468c.064-.293.006-.399-.287-.47l-.451-.081.082-.381 2.29-.287zM8 5.5a1 1 0 1 1 0-2 1 1 0 0 1 0 2z"/>'),Dk=Xu("InfoLg",'<path d="m10.277 5.433-4.031.505-.145.67.794.145c.516.123.619.309.505.824L6.101 13.68c-.34 1.578.186 2.32 1.423 2.32.959 0 2.072-.443 2.577-1.052l.155-.732c-.35.31-.866.434-1.206.434-.485 0-.66-.34-.536-.939l1.763-8.278zm.122-3.673a1.76 1.76 0 1 1-3.52 0 1.76 1.76 0 0 1 3.52 0z"/>'),Ck=Xu("InfoSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="m8.93 6.588-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM9 4.5a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),jk=Xu("InfoSquareFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm8.93 4.588-2.29.287-.082.38.45.083c.294.07.352.176.288.469l-.738 3.468c-.194.897.105 1.319.808 1.319.545 0 1.178-.252 1.465-.598l.088-.416c-.2.176-.492.246-.686.246-.275 0-.375-.193-.304-.533L8.93 6.588zM8 5.5a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),xk=Xu("InputCursor",'<path d="M10 5h4a1 1 0 0 1 1 1v4a1 1 0 0 1-1 1h-4v1h4a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2h-4v1zM6 5V4H2a2 2 0 0 0-2 2v4a2 2 0 0 0 2 2h4v-1H2a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h4z"/><path fill-rule="evenodd" d="M8 1a.5.5 0 0 1 .5.5v13a.5.5 0 0 1-1 0v-13A.5.5 0 0 1 8 1z"/>'),Bk=Xu("InputCursorText",'<path fill-rule="evenodd" d="M5 2a.5.5 0 0 1 .5-.5c.862 0 1.573.287 2.06.566.174.099.321.198.44.286.119-.088.266-.187.44-.286A4.165 4.165 0 0 1 10.5 1.5a.5.5 0 0 1 0 1c-.638 0-1.177.213-1.564.434a3.49 3.49 0 0 0-.436.294V7.5H9a.5.5 0 0 1 0 1h-.5v4.272c.1.08.248.187.436.294.387.221.926.434 1.564.434a.5.5 0 0 1 0 1 4.165 4.165 0 0 1-2.06-.566A4.561 4.561 0 0 1 8 13.65a4.561 4.561 0 0 1-.44.285 4.165 4.165 0 0 1-2.06.566.5.5 0 0 1 0-1c.638 0 1.177-.213 1.564-.434.188-.107.335-.214.436-.294V8.5H7a.5.5 0 0 1 0-1h.5V3.228a3.49 3.49 0 0 0-.436-.294A3.166 3.166 0 0 0 5.5 2.5.5.5 0 0 1 5 2z"/><path d="M10 5h4a1 1 0 0 1 1 1v4a1 1 0 0 1-1 1h-4v1h4a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2h-4v1zM6 5V4H2a2 2 0 0 0-2 2v4a2 2 0 0 0 2 2h4v-1H2a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h4z"/>'),Yk=Xu("Instagram",'<path d="M8 0C5.829 0 5.556.01 4.703.048 3.85.088 3.269.222 2.76.42a3.917 3.917 0 0 0-1.417.923A3.927 3.927 0 0 0 .42 2.76C.222 3.268.087 3.85.048 4.7.01 5.555 0 5.827 0 8.001c0 2.172.01 2.444.048 3.297.04.852.174 1.433.372 1.942.205.526.478.972.923 1.417.444.445.89.719 1.416.923.51.198 1.09.333 1.942.372C5.555 15.99 5.827 16 8 16s2.444-.01 3.298-.048c.851-.04 1.434-.174 1.943-.372a3.916 3.916 0 0 0 1.416-.923c.445-.445.718-.891.923-1.417.197-.509.332-1.09.372-1.942C15.99 10.445 16 10.173 16 8s-.01-2.445-.048-3.299c-.04-.851-.175-1.433-.372-1.941a3.926 3.926 0 0 0-.923-1.417A3.911 3.911 0 0 0 13.24.42c-.51-.198-1.092-.333-1.943-.372C10.443.01 10.172 0 7.998 0h.003zm-.717 1.442h.718c2.136 0 2.389.007 3.232.046.78.035 1.204.166 1.486.275.373.145.64.319.92.599.28.28.453.546.598.92.11.281.24.705.275 1.485.039.843.047 1.096.047 3.231s-.008 2.389-.047 3.232c-.035.78-.166 1.203-.275 1.485a2.47 2.47 0 0 1-.599.919c-.28.28-.546.453-.92.598-.28.11-.704.24-1.485.276-.843.038-1.096.047-3.232.047s-2.39-.009-3.233-.047c-.78-.036-1.203-.166-1.485-.276a2.478 2.478 0 0 1-.92-.598 2.48 2.48 0 0 1-.6-.92c-.109-.281-.24-.705-.275-1.485-.038-.843-.046-1.096-.046-3.233 0-2.136.008-2.388.046-3.231.036-.78.166-1.204.276-1.486.145-.373.319-.64.599-.92.28-.28.546-.453.92-.598.282-.11.705-.24 1.485-.276.738-.034 1.024-.044 2.515-.045v.002zm4.988 1.328a.96.96 0 1 0 0 1.92.96.96 0 0 0 0-1.92zm-4.27 1.122a4.109 4.109 0 1 0 0 8.217 4.109 4.109 0 0 0 0-8.217zm0 1.441a2.667 2.667 0 1 1 0 5.334 2.667 2.667 0 0 1 0-5.334z"/>'),Pk=Xu("Intersect",'<path d="M0 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2h2a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2v-2H2a2 2 0 0 1-2-2V2zm5 10v2a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V6a1 1 0 0 0-1-1h-2v5a2 2 0 0 1-2 2H5zm6-8V2a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h2V6a2 2 0 0 1 2-2h5z"/>'),Ik=Xu("Journal",'<path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Fk=Xu("JournalAlbum",'<path d="M5.5 4a.5.5 0 0 0-.5.5v5a.5.5 0 0 0 .5.5h5a.5.5 0 0 0 .5-.5v-5a.5.5 0 0 0-.5-.5h-5zm1 7a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1h-3z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Ek=Xu("JournalArrowDown",'<path fill-rule="evenodd" d="M8 5a.5.5 0 0 1 .5.5v3.793l1.146-1.147a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 1 1 .708-.708L7.5 9.293V5.5A.5.5 0 0 1 8 5z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),$k=Xu("JournalArrowUp",'<path fill-rule="evenodd" d="M8 11a.5.5 0 0 0 .5-.5V6.707l1.146 1.147a.5.5 0 0 0 .708-.708l-2-2a.5.5 0 0 0-.708 0l-2 2a.5.5 0 1 0 .708.708L7.5 6.707V10.5a.5.5 0 0 0 .5.5z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Rk=Xu("JournalBookmark",'<path fill-rule="evenodd" d="M6 8V1h1v6.117L8.743 6.07a.5.5 0 0 1 .514 0L11 7.117V1h1v7a.5.5 0 0 1-.757.429L9 7.083 6.757 8.43A.5.5 0 0 1 6 8z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Nk=Xu("JournalBookmarkFill",'<path fill-rule="evenodd" d="M6 1h6v7a.5.5 0 0 1-.757.429L9 7.083 6.757 8.43A.5.5 0 0 1 6 8V1z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Wk=Xu("JournalCheck",'<path fill-rule="evenodd" d="M10.854 6.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 8.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Uk=Xu("JournalCode",'<path fill-rule="evenodd" d="M8.646 5.646a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1 0 .708l-2 2a.5.5 0 0 1-.708-.708L10.293 8 8.646 6.354a.5.5 0 0 1 0-.708zm-1.292 0a.5.5 0 0 0-.708 0l-2 2a.5.5 0 0 0 0 .708l2 2a.5.5 0 0 0 .708-.708L5.707 8l1.647-1.646a.5.5 0 0 0 0-.708z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Gk=Xu("JournalMedical",'<path fill-rule="evenodd" d="M8 4a.5.5 0 0 1 .5.5v.634l.549-.317a.5.5 0 1 1 .5.866L9 6l.549.317a.5.5 0 1 1-.5.866L8.5 6.866V7.5a.5.5 0 0 1-1 0v-.634l-.549.317a.5.5 0 1 1-.5-.866L7 6l-.549-.317a.5.5 0 0 1 .5-.866l.549.317V4.5A.5.5 0 0 1 8 4zM5 9.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),qk=Xu("JournalMinus",'<path fill-rule="evenodd" d="M5.5 8a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Jk=Xu("JournalPlus",'<path fill-rule="evenodd" d="M8 5.5a.5.5 0 0 1 .5.5v1.5H10a.5.5 0 0 1 0 1H8.5V10a.5.5 0 0 1-1 0V8.5H6a.5.5 0 0 1 0-1h1.5V6a.5.5 0 0 1 .5-.5z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Kk=Xu("JournalRichtext",'<path d="M7.5 3.75a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm-.861 1.542 1.33.886 1.854-1.855a.25.25 0 0 1 .289-.047L11 4.75V7a.5.5 0 0 1-.5.5h-5A.5.5 0 0 1 5 7v-.5s1.54-1.274 1.639-1.208zM5 9.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Xk=Xu("JournalText",'<path d="M5 10.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm0-2a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0-2a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0-2a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Zk=Xu("JournalX",'<path fill-rule="evenodd" d="M6.146 6.146a.5.5 0 0 1 .708 0L8 7.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 8l1.147 1.146a.5.5 0 0 1-.708.708L8 8.707 6.854 9.854a.5.5 0 0 1-.708-.708L7.293 8 6.146 6.854a.5.5 0 0 1 0-.708z"/><path d="M3 0h10a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-1h1v1a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1v1H1V2a2 2 0 0 1 2-2z"/><path d="M1 5v-.5a.5.5 0 0 1 1 0V5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V8h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0v.5h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1z"/>'),Qk=Xu("Journals",'<path d="M5 0h8a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2 2 2 0 0 1-2 2H3a2 2 0 0 1-2-2h1a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V4a1 1 0 0 0-1-1H3a1 1 0 0 0-1 1H1a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v9a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H5a1 1 0 0 0-1 1H3a2 2 0 0 1 2-2z"/><path d="M1 6v-.5a.5.5 0 0 1 1 0V6h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 3v-.5a.5.5 0 0 1 1 0V9h.5a.5.5 0 0 1 0 1h-2a.5.5 0 0 1 0-1H1zm0 2.5v.5H.5a.5.5 0 0 0 0 1h2a.5.5 0 0 0 0-1H2v-.5a.5.5 0 0 0-1 0z"/>'),eA=Xu("Joystick",'<path d="M10 2a2 2 0 0 1-1.5 1.937v5.087c.863.083 1.5.377 1.5.726 0 .414-.895.75-2 .75s-2-.336-2-.75c0-.35.637-.643 1.5-.726V3.937A2 2 0 1 1 10 2z"/><path d="M0 9.665v1.717a1 1 0 0 0 .553.894l6.553 3.277a2 2 0 0 0 1.788 0l6.553-3.277a1 1 0 0 0 .553-.894V9.665c0-.1-.06-.19-.152-.23L9.5 6.715v.993l5.227 2.178a.125.125 0 0 1 .001.23l-5.94 2.546a2 2 0 0 1-1.576 0l-5.94-2.546a.125.125 0 0 1 .001-.23L6.5 7.708l-.013-.988L.152 9.435a.25.25 0 0 0-.152.23z"/>'),tA=Xu("Justify",'<path fill-rule="evenodd" d="M2 12.5a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),aA=Xu("JustifyLeft",'<path fill-rule="evenodd" d="M2 12.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),nA=Xu("JustifyRight",'<path fill-rule="evenodd" d="M6 12.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-4-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),rA=Xu("Kanban",'<path d="M13.5 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1h-11a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h11zm-11-1a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h11a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2h-11z"/><path d="M6.5 3a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1V3zm-4 0a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v7a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1V3zm8 0a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1V3z"/>'),iA=Xu("KanbanFill",'<path d="M2.5 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h11a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2h-11zm5 2h1a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1zm-5 1a1 1 0 0 1 1-1h1a1 1 0 0 1 1 1v7a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1V3zm9-1h1a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1h-1a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1z"/>'),oA=Xu("Key",'<path d="M0 8a4 4 0 0 1 7.465-2H14a.5.5 0 0 1 .354.146l1.5 1.5a.5.5 0 0 1 0 .708l-1.5 1.5a.5.5 0 0 1-.708 0L13 9.207l-.646.647a.5.5 0 0 1-.708 0L11 9.207l-.646.647a.5.5 0 0 1-.708 0L9 9.207l-.646.647A.5.5 0 0 1 8 10h-.535A4 4 0 0 1 0 8zm4-3a3 3 0 1 0 2.712 4.285A.5.5 0 0 1 7.163 9h.63l.853-.854a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 0 1 .708 0l.646.647.793-.793-1-1h-6.63a.5.5 0 0 1-.451-.285A3 3 0 0 0 4 5z"/><path d="M4 8a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),sA=Xu("KeyFill",'<path d="M3.5 11.5a3.5 3.5 0 1 1 3.163-5H14L15.5 8 14 9.5l-1-1-1 1-1-1-1 1-1-1-1 1H6.663a3.5 3.5 0 0 1-3.163 2zM2.5 9a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),lA=Xu("Keyboard",'<path d="M14 5a1 1 0 0 1 1 1v5a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h12zM2 4a2 2 0 0 0-2 2v5a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V6a2 2 0 0 0-2-2H2z"/><path d="M13 10.25a.25.25 0 0 1 .25-.25h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5a.25.25 0 0 1-.25-.25v-.5zm0-2a.25.25 0 0 1 .25-.25h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5a.25.25 0 0 1-.25-.25v-.5zm-5 0A.25.25 0 0 1 8.25 8h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5A.25.25 0 0 1 8 8.75v-.5zm2 0a.25.25 0 0 1 .25-.25h1.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-1.5a.25.25 0 0 1-.25-.25v-.5zm1 2a.25.25 0 0 1 .25-.25h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5a.25.25 0 0 1-.25-.25v-.5zm-5-2A.25.25 0 0 1 6.25 8h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5A.25.25 0 0 1 6 8.75v-.5zm-2 0A.25.25 0 0 1 4.25 8h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5A.25.25 0 0 1 4 8.75v-.5zm-2 0A.25.25 0 0 1 2.25 8h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5A.25.25 0 0 1 2 8.75v-.5zm11-2a.25.25 0 0 1 .25-.25h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5a.25.25 0 0 1-.25-.25v-.5zm-2 0a.25.25 0 0 1 .25-.25h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5a.25.25 0 0 1-.25-.25v-.5zm-2 0A.25.25 0 0 1 9.25 6h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5A.25.25 0 0 1 9 6.75v-.5zm-2 0A.25.25 0 0 1 7.25 6h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5A.25.25 0 0 1 7 6.75v-.5zm-2 0A.25.25 0 0 1 5.25 6h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5A.25.25 0 0 1 5 6.75v-.5zm-3 0A.25.25 0 0 1 2.25 6h1.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-1.5A.25.25 0 0 1 2 6.75v-.5zm0 4a.25.25 0 0 1 .25-.25h.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-.5a.25.25 0 0 1-.25-.25v-.5zm2 0a.25.25 0 0 1 .25-.25h5.5a.25.25 0 0 1 .25.25v.5a.25.25 0 0 1-.25.25h-5.5a.25.25 0 0 1-.25-.25v-.5z"/>'),cA=Xu("KeyboardFill",'<path d="M0 6a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V6zm13 .25v.5c0 .138.112.25.25.25h.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-.5a.25.25 0 0 0-.25.25zM2.25 8a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h.5A.25.25 0 0 0 3 8.75v-.5A.25.25 0 0 0 2.75 8h-.5zM4 8.25v.5c0 .138.112.25.25.25h.5A.25.25 0 0 0 5 8.75v-.5A.25.25 0 0 0 4.75 8h-.5a.25.25 0 0 0-.25.25zM6.25 8a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h.5A.25.25 0 0 0 7 8.75v-.5A.25.25 0 0 0 6.75 8h-.5zM8 8.25v.5c0 .138.112.25.25.25h.5A.25.25 0 0 0 9 8.75v-.5A.25.25 0 0 0 8.75 8h-.5a.25.25 0 0 0-.25.25zM13.25 8a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-.5zm0 2a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-.5zm-3-2a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h1.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-1.5zm.75 2.25v.5c0 .138.112.25.25.25h.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-.5a.25.25 0 0 0-.25.25zM11.25 6a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-.5zM9 6.25v.5c0 .138.112.25.25.25h.5a.25.25 0 0 0 .25-.25v-.5A.25.25 0 0 0 9.75 6h-.5a.25.25 0 0 0-.25.25zM7.25 6a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h.5A.25.25 0 0 0 8 6.75v-.5A.25.25 0 0 0 7.75 6h-.5zM5 6.25v.5c0 .138.112.25.25.25h.5A.25.25 0 0 0 6 6.75v-.5A.25.25 0 0 0 5.75 6h-.5a.25.25 0 0 0-.25.25zM2.25 6a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h1.5A.25.25 0 0 0 4 6.75v-.5A.25.25 0 0 0 3.75 6h-1.5zM2 10.25v.5c0 .138.112.25.25.25h.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-.5a.25.25 0 0 0-.25.25zM4.25 10a.25.25 0 0 0-.25.25v.5c0 .138.112.25.25.25h5.5a.25.25 0 0 0 .25-.25v-.5a.25.25 0 0 0-.25-.25h-5.5z"/>'),uA=Xu("Ladder",'<path d="M4.5 1a.5.5 0 0 1 .5.5V2h6v-.5a.5.5 0 0 1 1 0v14a.5.5 0 0 1-1 0V15H5v.5a.5.5 0 0 1-1 0v-14a.5.5 0 0 1 .5-.5zM5 14h6v-2H5v2zm0-3h6V9H5v2zm0-3h6V6H5v2zm0-3h6V3H5v2z"/>'),dA=Xu("Lamp",'<path d="M13 3v4H3V3h10zM3 2a1 1 0 0 0-1 1v4a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H3zm4.5-1 .276-.553a.25.25 0 0 1 .448 0L8.5 1h-1zm-.012 9h1.024c.337.646.677 1.33.95 1.949.176.396.318.75.413 1.042.048.146.081.266.102.36A1.347 1.347 0 0 1 10 13.5c0 .665-.717 1.5-2 1.5s-2-.835-2-1.5c0 0 0-.013.004-.039.003-.027.01-.063.02-.11.02-.094.053-.214.1-.36.096-.291.238-.646.413-1.042.274-.62.614-1.303.95-1.949zm1.627-1h-2.23C6.032 10.595 5 12.69 5 13.5 5 14.88 6.343 16 8 16s3-1.12 3-2.5c0-.81-1.032-2.905-1.885-4.5z"/>'),hA=Xu("LampFill",'<path d="M2 3a1 1 0 0 1 1-1h10a1 1 0 0 1 1 1v4a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3zm5.5-2 .276-.553a.25.25 0 0 1 .448 0L8.5 1h-1zm-.615 8h2.23C9.968 10.595 11 12.69 11 13.5c0 1.38-1.343 2.5-3 2.5s-3-1.12-3-2.5c0-.81 1.032-2.905 1.885-4.5z"/>'),fA=Xu("Laptop",'<path d="M13.5 3a.5.5 0 0 1 .5.5V11H2V3.5a.5.5 0 0 1 .5-.5h11zm-11-1A1.5 1.5 0 0 0 1 3.5V12h14V3.5A1.5 1.5 0 0 0 13.5 2h-11zM0 12.5h16a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 12.5z"/>'),pA=Xu("LaptopFill",'<path d="M2.5 2A1.5 1.5 0 0 0 1 3.5V12h14V3.5A1.5 1.5 0 0 0 13.5 2h-11zM0 12.5h16a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 12.5z"/>'),mA=Xu("LayerBackward",'<path d="M8.354 15.854a.5.5 0 0 1-.708 0l-3-3a.5.5 0 0 1 0-.708l1-1a.5.5 0 0 1 .708 0l.646.647V4H1a1 1 0 0 1-1-1V1a1 1 0 0 1 1-1h14a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1H9v7.793l.646-.647a.5.5 0 0 1 .708 0l1 1a.5.5 0 0 1 0 .708l-3 3z"/><path d="M1 9a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h4.5a.5.5 0 0 1 0 1H1v2h4.5a.5.5 0 0 1 0 1H1zm9.5 0a.5.5 0 0 1 0-1H15V6h-4.5a.5.5 0 0 1 0-1H15a1 1 0 0 1 1 1v2a1 1 0 0 1-1 1h-4.5z"/>'),vA=Xu("LayerForward",'<path d="M8.354.146a.5.5 0 0 0-.708 0l-3 3a.5.5 0 0 0 0 .708l1 1a.5.5 0 0 0 .708 0L7 4.207V12H1a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h14a1 1 0 0 0 1-1v-2a1 1 0 0 0-1-1H9V4.207l.646.647a.5.5 0 0 0 .708 0l1-1a.5.5 0 0 0 0-.708l-3-3z"/><path d="M1 7a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h4.5a.5.5 0 0 0 0-1H1V8h4.5a.5.5 0 0 0 0-1H1zm9.5 0a.5.5 0 0 0 0 1H15v2h-4.5a.5.5 0 0 0 0 1H15a1 1 0 0 0 1-1V8a1 1 0 0 0-1-1h-4.5z"/>'),_A=Xu("Layers",'<path d="M8.235 1.559a.5.5 0 0 0-.47 0l-7.5 4a.5.5 0 0 0 0 .882L3.188 8 .264 9.559a.5.5 0 0 0 0 .882l7.5 4a.5.5 0 0 0 .47 0l7.5-4a.5.5 0 0 0 0-.882L12.813 8l2.922-1.559a.5.5 0 0 0 0-.882l-7.5-4zm3.515 7.008L14.438 10 8 13.433 1.562 10 4.25 8.567l3.515 1.874a.5.5 0 0 0 .47 0l3.515-1.874zM8 9.433 1.562 6 8 2.567 14.438 6 8 9.433z"/>'),gA=Xu("LayersFill",'<path d="M7.765 1.559a.5.5 0 0 1 .47 0l7.5 4a.5.5 0 0 1 0 .882l-7.5 4a.5.5 0 0 1-.47 0l-7.5-4a.5.5 0 0 1 0-.882l7.5-4z"/><path d="m2.125 8.567-1.86.992a.5.5 0 0 0 0 .882l7.5 4a.5.5 0 0 0 .47 0l7.5-4a.5.5 0 0 0 0-.882l-1.86-.992-5.17 2.756a1.5 1.5 0 0 1-1.41 0l-5.17-2.756z"/>'),yA=Xu("LayersHalf",'<path d="M8.235 1.559a.5.5 0 0 0-.47 0l-7.5 4a.5.5 0 0 0 0 .882L3.188 8 .264 9.559a.5.5 0 0 0 0 .882l7.5 4a.5.5 0 0 0 .47 0l7.5-4a.5.5 0 0 0 0-.882L12.813 8l2.922-1.559a.5.5 0 0 0 0-.882l-7.5-4zM8 9.433 1.562 6 8 2.567 14.438 6 8 9.433z"/>'),bA=Xu("LayoutSidebar",'<path d="M0 3a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3zm5-1v12h9a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H5zM4 2H2a1 1 0 0 0-1 1v10a1 1 0 0 0 1 1h2V2z"/>'),MA=Xu("LayoutSidebarInset",'<path d="M14 2a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1h12zM2 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2H2z"/><path d="M3 4a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V4z"/>'),zA=Xu("LayoutSidebarInsetReverse",'<path d="M2 2a1 1 0 0 0-1 1v10a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H2zm12-1a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3a2 2 0 0 1 2-2h12z"/><path d="M13 4a1 1 0 0 0-1-1h-2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1V4z"/>'),LA=Xu("LayoutSidebarReverse",'<path d="M16 3a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3zm-5-1v12H2a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1h9zm1 0h2a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1h-2V2z"/>'),wA=Xu("LayoutSplit",'<path d="M0 3a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3zm8.5-1v12H14a1 1 0 0 0 1-1V3a1 1 0 0 0-1-1H8.5zm-1 0H2a1 1 0 0 0-1 1v10a1 1 0 0 0 1 1h5.5V2z"/>'),HA=Xu("LayoutTextSidebar",'<path d="M3.5 3a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zm0 3a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zM3 9.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm.5 2.5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5z"/><path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm12-1v14h2a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1h-2zm-1 0H2a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h9V1z"/>'),kA=Xu("LayoutTextSidebarReverse",'<path d="M12.5 3a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1h5zm0 3a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1h5zm.5 3.5a.5.5 0 0 0-.5-.5h-5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 .5-.5zm-.5 2.5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1h5z"/><path d="M16 2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2zM4 1v14H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h2zm1 0h9a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H5V1z"/>'),AA=Xu("LayoutTextWindow",'<path d="M3 6.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 3a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm.5 2.5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5z"/><path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm12 1a1 1 0 0 1 1 1v1H1V2a1 1 0 0 1 1-1h12zm1 3v10a1 1 0 0 1-1 1h-2V4h3zm-4 0v11H2a1 1 0 0 1-1-1V4h10z"/>'),SA=Xu("LayoutTextWindowReverse",'<path d="M13 6.5a.5.5 0 0 0-.5-.5h-5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 .5-.5zm0 3a.5.5 0 0 0-.5-.5h-5a.5.5 0 0 0 0 1h5a.5.5 0 0 0 .5-.5zm-.5 2.5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1 0-1h5z"/><path d="M14 0a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h12zM2 1a1 1 0 0 0-1 1v1h14V2a1 1 0 0 0-1-1H2zM1 4v10a1 1 0 0 0 1 1h2V4H1zm4 0v11h9a1 1 0 0 0 1-1V4H5z"/>'),VA=Xu("LayoutThreeColumns",'<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h13A1.5 1.5 0 0 1 16 1.5v13a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 14.5v-13zM1.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 .5.5H5V1H1.5zM10 15V1H6v14h4zm1 0h3.5a.5.5 0 0 0 .5-.5v-13a.5.5 0 0 0-.5-.5H11v14z"/>'),OA=Xu("LayoutWtf",'<path d="M5 1v8H1V1h4zM1 0a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h4a1 1 0 0 0 1-1V1a1 1 0 0 0-1-1H1zm13 2v5H9V2h5zM9 1a1 1 0 0 0-1 1v5a1 1 0 0 0 1 1h5a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H9zM5 13v2H3v-2h2zm-2-1a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h2a1 1 0 0 0 1-1v-2a1 1 0 0 0-1-1H3zm12-1v2H9v-2h6zm-6-1a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h6a1 1 0 0 0 1-1v-2a1 1 0 0 0-1-1H9z"/>'),TA=Xu("LifePreserver",'<path d="M8 16A8 8 0 1 0 8 0a8 8 0 0 0 0 16zm6.43-5.228a7.025 7.025 0 0 1-3.658 3.658l-1.115-2.788a4.015 4.015 0 0 0 1.985-1.985l2.788 1.115zM5.228 14.43a7.025 7.025 0 0 1-3.658-3.658l2.788-1.115a4.015 4.015 0 0 0 1.985 1.985L5.228 14.43zm9.202-9.202-2.788 1.115a4.015 4.015 0 0 0-1.985-1.985l1.115-2.788a7.025 7.025 0 0 1 3.658 3.658zm-8.087-.87a4.015 4.015 0 0 0-1.985 1.985L1.57 5.228A7.025 7.025 0 0 1 5.228 1.57l1.115 2.788zM8 11a3 3 0 1 1 0-6 3 3 0 0 1 0 6z"/>'),DA=Xu("Lightbulb",'<path d="M2 6a6 6 0 1 1 10.174 4.31c-.203.196-.359.4-.453.619l-.762 1.769A.5.5 0 0 1 10.5 13a.5.5 0 0 1 0 1 .5.5 0 0 1 0 1l-.224.447a1 1 0 0 1-.894.553H6.618a1 1 0 0 1-.894-.553L5.5 15a.5.5 0 0 1 0-1 .5.5 0 0 1 0-1 .5.5 0 0 1-.46-.302l-.761-1.77a1.964 1.964 0 0 0-.453-.618A5.984 5.984 0 0 1 2 6zm6-5a5 5 0 0 0-3.479 8.592c.263.254.514.564.676.941L5.83 12h4.342l.632-1.467c.162-.377.413-.687.676-.941A5 5 0 0 0 8 1z"/>'),CA=Xu("LightbulbFill",'<path d="M2 6a6 6 0 1 1 10.174 4.31c-.203.196-.359.4-.453.619l-.762 1.769A.5.5 0 0 1 10.5 13h-5a.5.5 0 0 1-.46-.302l-.761-1.77a1.964 1.964 0 0 0-.453-.618A5.984 5.984 0 0 1 2 6zm3 8.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1l-.224.447a1 1 0 0 1-.894.553H6.618a1 1 0 0 1-.894-.553L5.5 15a.5.5 0 0 1-.5-.5z"/>'),jA=Xu("LightbulbOff",'<path fill-rule="evenodd" d="M2.23 4.35A6.004 6.004 0 0 0 2 6c0 1.691.7 3.22 1.826 4.31.203.196.359.4.453.619l.762 1.769A.5.5 0 0 0 5.5 13a.5.5 0 0 0 0 1 .5.5 0 0 0 0 1l.224.447a1 1 0 0 0 .894.553h2.764a1 1 0 0 0 .894-.553L10.5 15a.5.5 0 0 0 0-1 .5.5 0 0 0 0-1 .5.5 0 0 0 .288-.091L9.878 12H5.83l-.632-1.467a2.954 2.954 0 0 0-.676-.941 4.984 4.984 0 0 1-1.455-4.405l-.837-.836zm1.588-2.653.708.707a5 5 0 0 1 7.07 7.07l.707.707a6 6 0 0 0-8.484-8.484zm-2.172-.051a.5.5 0 0 1 .708 0l12 12a.5.5 0 0 1-.708.708l-12-12a.5.5 0 0 1 0-.708z"/>'),xA=Xu("LightbulbOffFill",'<path d="M2 6c0-.572.08-1.125.23-1.65l8.558 8.559A.5.5 0 0 1 10.5 13h-5a.5.5 0 0 1-.46-.302l-.761-1.77a1.964 1.964 0 0 0-.453-.618A5.984 5.984 0 0 1 2 6zm10.303 4.181L3.818 1.697a6 6 0 0 1 8.484 8.484zM5 14.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1l-.224.447a1 1 0 0 1-.894.553H6.618a1 1 0 0 1-.894-.553L5.5 15a.5.5 0 0 1-.5-.5zM2.354 1.646a.5.5 0 1 0-.708.708l12 12a.5.5 0 0 0 .708-.708l-12-12z"/>'),BA=Xu("Lightning",'<path d="M5.52.359A.5.5 0 0 1 6 0h4a.5.5 0 0 1 .474.658L8.694 6H12.5a.5.5 0 0 1 .395.807l-7 9a.5.5 0 0 1-.873-.454L6.823 9.5H3.5a.5.5 0 0 1-.48-.641l2.5-8.5zM6.374 1 4.168 8.5H7.5a.5.5 0 0 1 .478.647L6.78 13.04 11.478 7H8a.5.5 0 0 1-.474-.658L9.306 1H6.374z"/>'),YA=Xu("LightningCharge",'<path d="M11.251.068a.5.5 0 0 1 .227.58L9.677 6.5H13a.5.5 0 0 1 .364.843l-8 8.5a.5.5 0 0 1-.842-.49L6.323 9.5H3a.5.5 0 0 1-.364-.843l8-8.5a.5.5 0 0 1 .615-.09zM4.157 8.5H7a.5.5 0 0 1 .478.647L6.11 13.59l5.732-6.09H9a.5.5 0 0 1-.478-.647L9.89 2.41 4.157 8.5z"/>'),PA=Xu("LightningChargeFill",'<path d="M11.251.068a.5.5 0 0 1 .227.58L9.677 6.5H13a.5.5 0 0 1 .364.843l-8 8.5a.5.5 0 0 1-.842-.49L6.323 9.5H3a.5.5 0 0 1-.364-.843l8-8.5a.5.5 0 0 1 .615-.09z"/>'),IA=Xu("LightningFill",'<path d="M5.52.359A.5.5 0 0 1 6 0h4a.5.5 0 0 1 .474.658L8.694 6H12.5a.5.5 0 0 1 .395.807l-7 9a.5.5 0 0 1-.873-.454L6.823 9.5H3.5a.5.5 0 0 1-.48-.641l2.5-8.5z"/>'),FA=Xu("Link",'<path d="M6.354 5.5H4a3 3 0 0 0 0 6h3a3 3 0 0 0 2.83-4H9c-.086 0-.17.01-.25.031A2 2 0 0 1 7 10.5H4a2 2 0 1 1 0-4h1.535c.218-.376.495-.714.82-1z"/><path d="M9 5.5a3 3 0 0 0-2.83 4h1.098A2 2 0 0 1 9 6.5h3a2 2 0 1 1 0 4h-1.535a4.02 4.02 0 0 1-.82 1H12a3 3 0 1 0 0-6H9z"/>'),EA=Xu("Link45deg",'<path d="M4.715 6.542 3.343 7.914a3 3 0 1 0 4.243 4.243l1.828-1.829A3 3 0 0 0 8.586 5.5L8 6.086a1.002 1.002 0 0 0-.154.199 2 2 0 0 1 .861 3.337L6.88 11.45a2 2 0 1 1-2.83-2.83l.793-.792a4.018 4.018 0 0 1-.128-1.287z"/><path d="M6.586 4.672A3 3 0 0 0 7.414 9.5l.775-.776a2 2 0 0 1-.896-3.346L9.12 3.55a2 2 0 1 1 2.83 2.83l-.793.792c.112.42.155.855.128 1.287l1.372-1.372a3 3 0 1 0-4.243-4.243L6.586 4.672z"/>'),$A=Xu("Linkedin",'<path d="M0 1.146C0 .513.526 0 1.175 0h13.65C15.474 0 16 .513 16 1.146v13.708c0 .633-.526 1.146-1.175 1.146H1.175C.526 16 0 15.487 0 14.854V1.146zm4.943 12.248V6.169H2.542v7.225h2.401zm-1.2-8.212c.837 0 1.358-.554 1.358-1.248-.015-.709-.52-1.248-1.342-1.248-.822 0-1.359.54-1.359 1.248 0 .694.521 1.248 1.327 1.248h.016zm4.908 8.212V9.359c0-.216.016-.432.08-.586.173-.431.568-.878 1.232-.878.869 0 1.216.662 1.216 1.634v3.865h2.401V9.25c0-2.22-1.184-3.252-2.764-3.252-1.274 0-1.845.7-2.165 1.193v.025h-.016a5.54 5.54 0 0 1 .016-.025V6.169h-2.4c.03.678 0 7.225 0 7.225h2.4z"/>'),RA=Xu("List",'<path fill-rule="evenodd" d="M2.5 12a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 0 1H3a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 0 1H3a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h10a.5.5 0 0 1 0 1H3a.5.5 0 0 1-.5-.5z"/>'),NA=Xu("ListCheck",'<path fill-rule="evenodd" d="M5 11.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM3.854 2.146a.5.5 0 0 1 0 .708l-1.5 1.5a.5.5 0 0 1-.708 0l-.5-.5a.5.5 0 1 1 .708-.708L2 3.293l1.146-1.147a.5.5 0 0 1 .708 0zm0 4a.5.5 0 0 1 0 .708l-1.5 1.5a.5.5 0 0 1-.708 0l-.5-.5a.5.5 0 1 1 .708-.708L2 7.293l1.146-1.147a.5.5 0 0 1 .708 0zm0 4a.5.5 0 0 1 0 .708l-1.5 1.5a.5.5 0 0 1-.708 0l-.5-.5a.5.5 0 0 1 .708-.708l.146.147 1.146-1.147a.5.5 0 0 1 .708 0z"/>'),WA=Xu("ListNested",'<path fill-rule="evenodd" d="M4.5 11.5A.5.5 0 0 1 5 11h10a.5.5 0 0 1 0 1H5a.5.5 0 0 1-.5-.5zm-2-4A.5.5 0 0 1 3 7h10a.5.5 0 0 1 0 1H3a.5.5 0 0 1-.5-.5zm-2-4A.5.5 0 0 1 1 3h10a.5.5 0 0 1 0 1H1a.5.5 0 0 1-.5-.5z"/>'),UA=Xu("ListOl",'<path fill-rule="evenodd" d="M5 11.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5z"/><path d="M1.713 11.865v-.474H2c.217 0 .363-.137.363-.317 0-.185-.158-.31-.361-.31-.223 0-.367.152-.373.31h-.59c.016-.467.373-.787.986-.787.588-.002.954.291.957.703a.595.595 0 0 1-.492.594v.033a.615.615 0 0 1 .569.631c.003.533-.502.8-1.051.8-.656 0-1-.37-1.008-.794h.582c.008.178.186.306.422.309.254 0 .424-.145.422-.35-.002-.195-.155-.348-.414-.348h-.3zm-.004-4.699h-.604v-.035c0-.408.295-.844.958-.844.583 0 .96.326.96.756 0 .389-.257.617-.476.848l-.537.572v.03h1.054V9H1.143v-.395l.957-.99c.138-.142.293-.304.293-.508 0-.18-.147-.32-.342-.32a.33.33 0 0 0-.342.338v.041zM2.564 5h-.635V2.924h-.031l-.598.42v-.567l.629-.443h.635V5z"/>'),GA=Xu("ListStars",'<path fill-rule="evenodd" d="M5 11.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5z"/><path d="M2.242 2.194a.27.27 0 0 1 .516 0l.162.53c.035.115.14.194.258.194h.551c.259 0 .37.333.164.493l-.468.363a.277.277 0 0 0-.094.3l.173.569c.078.256-.213.462-.423.3l-.417-.324a.267.267 0 0 0-.328 0l-.417.323c-.21.163-.5-.043-.423-.299l.173-.57a.277.277 0 0 0-.094-.299l-.468-.363c-.206-.16-.095-.493.164-.493h.55a.271.271 0 0 0 .259-.194l.162-.53zm0 4a.27.27 0 0 1 .516 0l.162.53c.035.115.14.194.258.194h.551c.259 0 .37.333.164.493l-.468.363a.277.277 0 0 0-.094.3l.173.569c.078.255-.213.462-.423.3l-.417-.324a.267.267 0 0 0-.328 0l-.417.323c-.21.163-.5-.043-.423-.299l.173-.57a.277.277 0 0 0-.094-.299l-.468-.363c-.206-.16-.095-.493.164-.493h.55a.271.271 0 0 0 .259-.194l.162-.53zm0 4a.27.27 0 0 1 .516 0l.162.53c.035.115.14.194.258.194h.551c.259 0 .37.333.164.493l-.468.363a.277.277 0 0 0-.094.3l.173.569c.078.255-.213.462-.423.3l-.417-.324a.267.267 0 0 0-.328 0l-.417.323c-.21.163-.5-.043-.423-.299l.173-.57a.277.277 0 0 0-.094-.299l-.468-.363c-.206-.16-.095-.493.164-.493h.55a.271.271 0 0 0 .259-.194l.162-.53z"/>'),qA=Xu("ListTask",'<path fill-rule="evenodd" d="M2 2.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5V3a.5.5 0 0 0-.5-.5H2zM3 3H2v1h1V3z"/><path d="M5 3.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zM5.5 7a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9zm0 4a.5.5 0 0 0 0 1h9a.5.5 0 0 0 0-1h-9z"/><path fill-rule="evenodd" d="M1.5 7a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5H2a.5.5 0 0 1-.5-.5V7zM2 7h1v1H2V7zm0 3.5a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5H2zm1 .5H2v1h1v-1z"/>'),JA=Xu("ListUl",'<path fill-rule="evenodd" d="M5 11.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0-4a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm-3 1a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm0 4a1 1 0 1 0 0-2 1 1 0 0 0 0 2zm0 4a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),KA=Xu("Lock",'<path d="M8 1a2 2 0 0 1 2 2v4H6V3a2 2 0 0 1 2-2zm3 6V3a3 3 0 0 0-6 0v4a2 2 0 0 0-2 2v5a2 2 0 0 0 2 2h6a2 2 0 0 0 2-2V9a2 2 0 0 0-2-2zM5 8h6a1 1 0 0 1 1 1v5a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1V9a1 1 0 0 1 1-1z"/>'),XA=Xu("LockFill",'<path d="M8 1a2 2 0 0 1 2 2v4H6V3a2 2 0 0 1 2-2zm3 6V3a3 3 0 0 0-6 0v4a2 2 0 0 0-2 2v5a2 2 0 0 0 2 2h6a2 2 0 0 0 2-2V9a2 2 0 0 0-2-2z"/>'),ZA=Xu("Mailbox",'<path d="M4 4a3 3 0 0 0-3 3v6h6V7a3 3 0 0 0-3-3zm0-1h8a4 4 0 0 1 4 4v6a1 1 0 0 1-1 1H1a1 1 0 0 1-1-1V7a4 4 0 0 1 4-4zm2.646 1A3.99 3.99 0 0 1 8 7v6h7V7a3 3 0 0 0-3-3H6.646z"/><path d="M11.793 8.5H9v-1h5a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.354-.146l-.853-.854zM5 7c0 .552-.448 0-1 0s-1 .552-1 0a1 1 0 0 1 2 0z"/>'),QA=Xu("Mailbox2",'<path d="M9 8.5h2.793l.853.854A.5.5 0 0 0 13 9.5h1a.5.5 0 0 0 .5-.5V8a.5.5 0 0 0-.5-.5H9v1z"/><path d="M12 3H4a4 4 0 0 0-4 4v6a1 1 0 0 0 1 1h14a1 1 0 0 0 1-1V7a4 4 0 0 0-4-4zM8 7a3.99 3.99 0 0 0-1.354-3H12a3 3 0 0 1 3 3v6H8V7zm-3.415.157C4.42 7.087 4.218 7 4 7c-.218 0-.42.086-.585.157C3.164 7.264 3 7.334 3 7a1 1 0 0 1 2 0c0 .334-.164.264-.415.157z"/>'),eS=Xu("Map",'<path fill-rule="evenodd" d="M15.817.113A.5.5 0 0 1 16 .5v14a.5.5 0 0 1-.402.49l-5 1a.502.502 0 0 1-.196 0L5.5 15.01l-4.902.98A.5.5 0 0 1 0 15.5v-14a.5.5 0 0 1 .402-.49l5-1a.5.5 0 0 1 .196 0L10.5.99l4.902-.98a.5.5 0 0 1 .415.103zM10 1.91l-4-.8v12.98l4 .8V1.91zm1 12.98 4-.8V1.11l-4 .8v12.98zm-6-.8V1.11l-4 .8v12.98l4-.8z"/>'),tS=Xu("MapFill",'<path fill-rule="evenodd" d="M16 .5a.5.5 0 0 0-.598-.49L10.5.99 5.598.01a.5.5 0 0 0-.196 0l-5 1A.5.5 0 0 0 0 1.5v14a.5.5 0 0 0 .598.49l4.902-.98 4.902.98a.502.502 0 0 0 .196 0l5-1A.5.5 0 0 0 16 14.5V.5zM5 14.09V1.11l.5-.1.5.1v12.98l-.402-.08a.498.498 0 0 0-.196 0L5 14.09zm5 .8V1.91l.402.08a.5.5 0 0 0 .196 0L11 1.91v12.98l-.5.1-.5-.1z"/>'),aS=Xu("Markdown",'<path d="M14 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h12zM2 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2z"/><path fill-rule="evenodd" d="M9.146 8.146a.5.5 0 0 1 .708 0L11.5 9.793l1.646-1.647a.5.5 0 0 1 .708.708l-2 2a.5.5 0 0 1-.708 0l-2-2a.5.5 0 0 1 0-.708z"/><path fill-rule="evenodd" d="M11.5 5a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0v-4a.5.5 0 0 1 .5-.5z"/><path d="M3.56 11V7.01h.056l1.428 3.239h.774l1.42-3.24h.056V11h1.073V5.001h-1.2l-1.71 3.894h-.039l-1.71-3.894H2.5V11h1.06z"/>'),nS=Xu("MarkdownFill",'<path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm11.5 1a.5.5 0 0 0-.5.5v3.793L9.854 8.146a.5.5 0 1 0-.708.708l2 2a.5.5 0 0 0 .708 0l2-2a.5.5 0 0 0-.708-.708L12 9.293V5.5a.5.5 0 0 0-.5-.5zM3.56 7.01h.056l1.428 3.239h.774l1.42-3.24h.056V11h1.073V5.001h-1.2l-1.71 3.894h-.039l-1.71-3.894H2.5V11h1.06V7.01z"/>'),rS=Xu("Mask",'<path d="M6.225 1.227A7.5 7.5 0 0 1 10.5 8a7.5 7.5 0 0 1-4.275 6.773 7 7 0 1 0 0-13.546zM4.187.966a8 8 0 1 1 7.627 14.069A8 8 0 0 1 4.186.964z"/>'),iS=Xu("Mastodon",'<path d="M11.19 12.195c2.016-.24 3.77-1.475 3.99-2.603.348-1.778.32-4.339.32-4.339 0-3.47-2.286-4.488-2.286-4.488C12.062.238 10.083.017 8.027 0h-.05C5.92.017 3.942.238 2.79.765c0 0-2.285 1.017-2.285 4.488l-.002.662c-.004.64-.007 1.35.011 2.091.083 3.394.626 6.74 3.78 7.57 1.454.383 2.703.463 3.709.408 1.823-.1 2.847-.647 2.847-.647l-.06-1.317s-1.303.41-2.767.36c-1.45-.05-2.98-.156-3.215-1.928a3.614 3.614 0 0 1-.033-.496s1.424.346 3.228.428c1.103.05 2.137-.064 3.188-.189zm1.613-2.47H11.13v-4.08c0-.859-.364-1.295-1.091-1.295-.804 0-1.207.517-1.207 1.541v2.233H7.168V5.89c0-1.024-.403-1.541-1.207-1.541-.727 0-1.091.436-1.091 1.296v4.079H3.197V5.522c0-.859.22-1.541.66-2.046.456-.505 1.052-.764 1.793-.764.856 0 1.504.328 1.933.983L8 4.39l.417-.695c.429-.655 1.077-.983 1.934-.983.74 0 1.336.259 1.791.764.442.505.661 1.187.661 2.046v4.203z"/>'),oS=Xu("Megaphone",'<path d="M13 2.5a1.5 1.5 0 0 1 3 0v11a1.5 1.5 0 0 1-3 0v-.214c-2.162-1.241-4.49-1.843-6.912-2.083l.405 2.712A1 1 0 0 1 5.51 15.1h-.548a1 1 0 0 1-.916-.599l-1.85-3.49a68.14 68.14 0 0 0-.202-.003A2.014 2.014 0 0 1 0 9V7a2.02 2.02 0 0 1 1.992-2.013 74.663 74.663 0 0 0 2.483-.075c3.043-.154 6.148-.849 8.525-2.199V2.5zm1 0v11a.5.5 0 0 0 1 0v-11a.5.5 0 0 0-1 0zm-1 1.35c-2.344 1.205-5.209 1.842-8 2.033v4.233c.18.01.359.022.537.036 2.568.189 5.093.744 7.463 1.993V3.85zm-9 6.215v-4.13a95.09 95.09 0 0 1-1.992.052A1.02 1.02 0 0 0 1 7v2c0 .55.448 1.002 1.006 1.009A60.49 60.49 0 0 1 4 10.065zm-.657.975 1.609 3.037.01.024h.548l-.002-.014-.443-2.966a68.019 68.019 0 0 0-1.722-.082z"/>'),sS=Xu("MegaphoneFill",'<path d="M13 2.5a1.5 1.5 0 0 1 3 0v11a1.5 1.5 0 0 1-3 0v-11zm-1 .724c-2.067.95-4.539 1.481-7 1.656v6.237a25.222 25.222 0 0 1 1.088.085c2.053.204 4.038.668 5.912 1.56V3.224zm-8 7.841V4.934c-.68.027-1.399.043-2.008.053A2.02 2.02 0 0 0 0 7v2c0 1.106.896 1.996 1.994 2.009a68.14 68.14 0 0 1 .496.008 64 64 0 0 1 1.51.048zm1.39 1.081c.285.021.569.047.85.078l.253 1.69a1 1 0 0 1-.983 1.187h-.548a1 1 0 0 1-.916-.599l-1.314-2.48a65.81 65.81 0 0 1 1.692.064c.327.017.65.037.966.06z"/>'),lS=Xu("MenuApp",'<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h2A1.5 1.5 0 0 1 5 1.5v2A1.5 1.5 0 0 1 3.5 5h-2A1.5 1.5 0 0 1 0 3.5v-2zM1.5 1a.5.5 0 0 0-.5.5v2a.5.5 0 0 0 .5.5h2a.5.5 0 0 0 .5-.5v-2a.5.5 0 0 0-.5-.5h-2zM0 8a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V8zm1 3v2a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2H1zm14-1V8a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2h14zM2 8.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),cS=Xu("MenuAppFill",'<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h2A1.5 1.5 0 0 1 5 1.5v2A1.5 1.5 0 0 1 3.5 5h-2A1.5 1.5 0 0 1 0 3.5v-2zM0 8a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V8zm1 3v2a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2H1zm14-1V8a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2h14zM2 8.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),uS=Xu("MenuButton",'<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h8A1.5 1.5 0 0 1 11 1.5v2A1.5 1.5 0 0 1 9.5 5h-8A1.5 1.5 0 0 1 0 3.5v-2zM1.5 1a.5.5 0 0 0-.5.5v2a.5.5 0 0 0 .5.5h8a.5.5 0 0 0 .5-.5v-2a.5.5 0 0 0-.5-.5h-8z"/><path d="m7.823 2.823-.396-.396A.25.25 0 0 1 7.604 2h.792a.25.25 0 0 1 .177.427l-.396.396a.25.25 0 0 1-.354 0zM0 8a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V8zm1 3v2a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2H1zm14-1V8a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2h14zM2 8.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),dS=Xu("MenuButtonFill",'<path d="M1.5 0A1.5 1.5 0 0 0 0 1.5v2A1.5 1.5 0 0 0 1.5 5h8A1.5 1.5 0 0 0 11 3.5v-2A1.5 1.5 0 0 0 9.5 0h-8zm5.927 2.427A.25.25 0 0 1 7.604 2h.792a.25.25 0 0 1 .177.427l-.396.396a.25.25 0 0 1-.354 0l-.396-.396zM0 8a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V8zm1 3v2a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2H1zm14-1V8a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2h14zM2 8.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),hS=Xu("MenuButtonWide",'<path d="M0 1.5A1.5 1.5 0 0 1 1.5 0h13A1.5 1.5 0 0 1 16 1.5v2A1.5 1.5 0 0 1 14.5 5h-13A1.5 1.5 0 0 1 0 3.5v-2zM1.5 1a.5.5 0 0 0-.5.5v2a.5.5 0 0 0 .5.5h13a.5.5 0 0 0 .5-.5v-2a.5.5 0 0 0-.5-.5h-13z"/><path d="M2 2.5a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1h-3a.5.5 0 0 1-.5-.5zm10.823.323-.396-.396A.25.25 0 0 1 12.604 2h.792a.25.25 0 0 1 .177.427l-.396.396a.25.25 0 0 1-.354 0zM0 8a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V8zm1 3v2a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2H1zm14-1V8a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2h14zM2 8.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),fS=Xu("MenuButtonWideFill",'<path d="M1.5 0A1.5 1.5 0 0 0 0 1.5v2A1.5 1.5 0 0 0 1.5 5h13A1.5 1.5 0 0 0 16 3.5v-2A1.5 1.5 0 0 0 14.5 0h-13zm1 2h3a.5.5 0 0 1 0 1h-3a.5.5 0 0 1 0-1zm9.927.427A.25.25 0 0 1 12.604 2h.792a.25.25 0 0 1 .177.427l-.396.396a.25.25 0 0 1-.354 0l-.396-.396zM0 8a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V8zm1 3v2a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2H1zm14-1V8a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v2h14zM2 8.5a.5.5 0 0 1 .5-.5h9a.5.5 0 0 1 0 1h-9a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),pS=Xu("MenuDown",'<path d="M7.646.146a.5.5 0 0 1 .708 0L10.207 2H14a2 2 0 0 1 2 2v9a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h3.793L7.646.146zM1 7v3h14V7H1zm14-1V4a1 1 0 0 0-1-1h-3.793a1 1 0 0 1-.707-.293L8 1.207l-1.5 1.5A1 1 0 0 1 5.793 3H2a1 1 0 0 0-1 1v2h14zm0 5H1v2a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2zM2 4.5a.5.5 0 0 1 .5-.5h8a.5.5 0 0 1 0 1h-8a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0 4a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>'),mS=Xu("MenuUp",'<path d="M7.646 15.854a.5.5 0 0 0 .708 0L10.207 14H14a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v9a2 2 0 0 0 2 2h3.793l1.853 1.854zM1 9V6h14v3H1zm14 1v2a1 1 0 0 1-1 1h-3.793a1 1 0 0 0-.707.293l-1.5 1.5-1.5-1.5A1 1 0 0 0 5.793 13H2a1 1 0 0 1-1-1v-2h14zm0-5H1V3a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1v2zM2 11.5a.5.5 0 0 0 .5.5h8a.5.5 0 0 0 0-1h-8a.5.5 0 0 0-.5.5zm0-4a.5.5 0 0 0 .5.5h11a.5.5 0 0 0 0-1h-11a.5.5 0 0 0-.5.5zm0-4a.5.5 0 0 0 .5.5h6a.5.5 0 0 0 0-1h-6a.5.5 0 0 0-.5.5z"/>'),vS=Xu("Messenger",'<path d="M0 7.76C0 3.301 3.493 0 8 0s8 3.301 8 7.76-3.493 7.76-8 7.76c-.81 0-1.586-.107-2.316-.307a.639.639 0 0 0-.427.03l-1.588.702a.64.64 0 0 1-.898-.566l-.044-1.423a.639.639 0 0 0-.215-.456C.956 12.108 0 10.092 0 7.76zm5.546-1.459-2.35 3.728c-.225.358.214.761.551.506l2.525-1.916a.48.48 0 0 1 .578-.002l1.869 1.402a1.2 1.2 0 0 0 1.735-.32l2.35-3.728c.226-.358-.214-.761-.551-.506L9.728 7.381a.48.48 0 0 1-.578.002L7.281 5.98a1.2 1.2 0 0 0-1.735.32z"/>'),_S=Xu("Mic",'<path d="M3.5 6.5A.5.5 0 0 1 4 7v1a4 4 0 0 0 8 0V7a.5.5 0 0 1 1 0v1a5 5 0 0 1-4.5 4.975V15h3a.5.5 0 0 1 0 1h-7a.5.5 0 0 1 0-1h3v-2.025A5 5 0 0 1 3 8V7a.5.5 0 0 1 .5-.5z"/><path d="M10 8a2 2 0 1 1-4 0V3a2 2 0 1 1 4 0v5zM8 0a3 3 0 0 0-3 3v5a3 3 0 0 0 6 0V3a3 3 0 0 0-3-3z"/>'),gS=Xu("MicFill",'<path d="M5 3a3 3 0 0 1 6 0v5a3 3 0 0 1-6 0V3z"/><path d="M3.5 6.5A.5.5 0 0 1 4 7v1a4 4 0 0 0 8 0V7a.5.5 0 0 1 1 0v1a5 5 0 0 1-4.5 4.975V15h3a.5.5 0 0 1 0 1h-7a.5.5 0 0 1 0-1h3v-2.025A5 5 0 0 1 3 8V7a.5.5 0 0 1 .5-.5z"/>'),yS=Xu("MicMute",'<path d="M13 8c0 .564-.094 1.107-.266 1.613l-.814-.814A4.02 4.02 0 0 0 12 8V7a.5.5 0 0 1 1 0v1zm-5 4c.818 0 1.578-.245 2.212-.667l.718.719a4.973 4.973 0 0 1-2.43.923V15h3a.5.5 0 0 1 0 1h-7a.5.5 0 0 1 0-1h3v-2.025A5 5 0 0 1 3 8V7a.5.5 0 0 1 1 0v1a4 4 0 0 0 4 4zm3-9v4.879l-1-1V3a2 2 0 0 0-3.997-.118l-.845-.845A3.001 3.001 0 0 1 11 3z"/><path d="m9.486 10.607-.748-.748A2 2 0 0 1 6 8v-.878l-1-1V8a3 3 0 0 0 4.486 2.607zm-7.84-9.253 12 12 .708-.708-12-12-.708.708z"/>'),bS=Xu("MicMuteFill",'<path d="M13 8c0 .564-.094 1.107-.266 1.613l-.814-.814A4.02 4.02 0 0 0 12 8V7a.5.5 0 0 1 1 0v1zm-5 4c.818 0 1.578-.245 2.212-.667l.718.719a4.973 4.973 0 0 1-2.43.923V15h3a.5.5 0 0 1 0 1h-7a.5.5 0 0 1 0-1h3v-2.025A5 5 0 0 1 3 8V7a.5.5 0 0 1 1 0v1a4 4 0 0 0 4 4zm3-9v4.879L5.158 2.037A3.001 3.001 0 0 1 11 3z"/><path d="M9.486 10.607 5 6.12V8a3 3 0 0 0 4.486 2.607zm-7.84-9.253 12 12 .708-.708-12-12-.708.708z"/>'),MS=Xu("Minecart",'<path d="M4 15a1 1 0 1 1 0-2 1 1 0 0 1 0 2zm0 1a2 2 0 1 0 0-4 2 2 0 0 0 0 4zm8-1a1 1 0 1 1 0-2 1 1 0 0 1 0 2zm0 1a2 2 0 1 0 0-4 2 2 0 0 0 0 4zM.115 3.18A.5.5 0 0 1 .5 3h15a.5.5 0 0 1 .491.592l-1.5 8A.5.5 0 0 1 14 12H2a.5.5 0 0 1-.491-.408l-1.5-8a.5.5 0 0 1 .106-.411zm.987.82 1.313 7h11.17l1.313-7H1.102z"/>'),zS=Xu("MinecartLoaded",'<path d="M4 15a1 1 0 1 1 0-2 1 1 0 0 1 0 2zm0 1a2 2 0 1 0 0-4 2 2 0 0 0 0 4zm8-1a1 1 0 1 1 0-2 1 1 0 0 1 0 2zm0 1a2 2 0 1 0 0-4 2 2 0 0 0 0 4zM.115 3.18A.5.5 0 0 1 .5 3h15a.5.5 0 0 1 .491.592l-1.5 8A.5.5 0 0 1 14 12H2a.5.5 0 0 1-.491-.408l-1.5-8a.5.5 0 0 1 .106-.411zm.987.82 1.313 7h11.17l1.313-7H1.102z"/><path fill-rule="evenodd" d="M6 1a2.498 2.498 0 0 1 4 0c.818 0 1.545.394 2 1 .67 0 1.552.57 2 1h-2c-.314 0-.611-.15-.8-.4-.274-.365-.71-.6-1.2-.6-.314 0-.611-.15-.8-.4a1.497 1.497 0 0 0-2.4 0c-.189.25-.486.4-.8.4-.507 0-.955.251-1.228.638-.09.13-.194.25-.308.362H3c.13-.147.401-.432.562-.545a1.63 1.63 0 0 0 .393-.393A2.498 2.498 0 0 1 6 1z"/>'),LS=Xu("Moisture",'<path d="M13.5 0a.5.5 0 0 0 0 1H15v2.75h-.5a.5.5 0 0 0 0 1h.5V7.5h-1.5a.5.5 0 0 0 0 1H15v2.75h-.5a.5.5 0 0 0 0 1h.5V15h-1.5a.5.5 0 0 0 0 1h2a.5.5 0 0 0 .5-.5V.5a.5.5 0 0 0-.5-.5h-2zM7 1.5l.364-.343a.5.5 0 0 0-.728 0l-.002.002-.006.007-.022.023-.08.088a28.458 28.458 0 0 0-1.274 1.517c-.769.983-1.714 2.325-2.385 3.727C2.368 7.564 2 8.682 2 9.733 2 12.614 4.212 15 7 15s5-2.386 5-5.267c0-1.05-.368-2.169-.867-3.212-.671-1.402-1.616-2.744-2.385-3.727a28.458 28.458 0 0 0-1.354-1.605l-.022-.023-.006-.007-.002-.001L7 1.5zm0 0-.364-.343L7 1.5zm-.016.766L7 2.247l.016.019c.24.274.572.667.944 1.144.611.781 1.32 1.776 1.901 2.827H4.14c.58-1.051 1.29-2.046 1.9-2.827.373-.477.706-.87.945-1.144zM3 9.733c0-.755.244-1.612.638-2.496h6.724c.395.884.638 1.741.638 2.496C11 12.117 9.182 14 7 14s-4-1.883-4-4.267z"/>'),wS=Xu("Moon",'<path d="M6 .278a.768.768 0 0 1 .08.858 7.208 7.208 0 0 0-.878 3.46c0 4.021 3.278 7.277 7.318 7.277.527 0 1.04-.055 1.533-.16a.787.787 0 0 1 .81.316.733.733 0 0 1-.031.893A8.349 8.349 0 0 1 8.344 16C3.734 16 0 12.286 0 7.71 0 4.266 2.114 1.312 5.124.06A.752.752 0 0 1 6 .278zM4.858 1.311A7.269 7.269 0 0 0 1.025 7.71c0 4.02 3.279 7.276 7.319 7.276a7.316 7.316 0 0 0 5.205-2.162c-.337.042-.68.063-1.029.063-4.61 0-8.343-3.714-8.343-8.29 0-1.167.242-2.278.681-3.286z"/>'),HS=Xu("MoonFill",'<path d="M6 .278a.768.768 0 0 1 .08.858 7.208 7.208 0 0 0-.878 3.46c0 4.021 3.278 7.277 7.318 7.277.527 0 1.04-.055 1.533-.16a.787.787 0 0 1 .81.316.733.733 0 0 1-.031.893A8.349 8.349 0 0 1 8.344 16C3.734 16 0 12.286 0 7.71 0 4.266 2.114 1.312 5.124.06A.752.752 0 0 1 6 .278z"/>'),kS=Xu("MoonStars",'<path d="M6 .278a.768.768 0 0 1 .08.858 7.208 7.208 0 0 0-.878 3.46c0 4.021 3.278 7.277 7.318 7.277.527 0 1.04-.055 1.533-.16a.787.787 0 0 1 .81.316.733.733 0 0 1-.031.893A8.349 8.349 0 0 1 8.344 16C3.734 16 0 12.286 0 7.71 0 4.266 2.114 1.312 5.124.06A.752.752 0 0 1 6 .278zM4.858 1.311A7.269 7.269 0 0 0 1.025 7.71c0 4.02 3.279 7.276 7.319 7.276a7.316 7.316 0 0 0 5.205-2.162c-.337.042-.68.063-1.029.063-4.61 0-8.343-3.714-8.343-8.29 0-1.167.242-2.278.681-3.286z"/><path d="M10.794 3.148a.217.217 0 0 1 .412 0l.387 1.162c.173.518.579.924 1.097 1.097l1.162.387a.217.217 0 0 1 0 .412l-1.162.387a1.734 1.734 0 0 0-1.097 1.097l-.387 1.162a.217.217 0 0 1-.412 0l-.387-1.162A1.734 1.734 0 0 0 9.31 6.593l-1.162-.387a.217.217 0 0 1 0-.412l1.162-.387a1.734 1.734 0 0 0 1.097-1.097l.387-1.162zM13.863.099a.145.145 0 0 1 .274 0l.258.774c.115.346.386.617.732.732l.774.258a.145.145 0 0 1 0 .274l-.774.258a1.156 1.156 0 0 0-.732.732l-.258.774a.145.145 0 0 1-.274 0l-.258-.774a1.156 1.156 0 0 0-.732-.732l-.774-.258a.145.145 0 0 1 0-.274l.774-.258c.346-.115.617-.386.732-.732L13.863.1z"/>'),AS=Xu("MoonStarsFill",'<path d="M6 .278a.768.768 0 0 1 .08.858 7.208 7.208 0 0 0-.878 3.46c0 4.021 3.278 7.277 7.318 7.277.527 0 1.04-.055 1.533-.16a.787.787 0 0 1 .81.316.733.733 0 0 1-.031.893A8.349 8.349 0 0 1 8.344 16C3.734 16 0 12.286 0 7.71 0 4.266 2.114 1.312 5.124.06A.752.752 0 0 1 6 .278z"/><path d="M10.794 3.148a.217.217 0 0 1 .412 0l.387 1.162c.173.518.579.924 1.097 1.097l1.162.387a.217.217 0 0 1 0 .412l-1.162.387a1.734 1.734 0 0 0-1.097 1.097l-.387 1.162a.217.217 0 0 1-.412 0l-.387-1.162A1.734 1.734 0 0 0 9.31 6.593l-1.162-.387a.217.217 0 0 1 0-.412l1.162-.387a1.734 1.734 0 0 0 1.097-1.097l.387-1.162zM13.863.099a.145.145 0 0 1 .274 0l.258.774c.115.346.386.617.732.732l.774.258a.145.145 0 0 1 0 .274l-.774.258a1.156 1.156 0 0 0-.732.732l-.258.774a.145.145 0 0 1-.274 0l-.258-.774a1.156 1.156 0 0 0-.732-.732l-.774-.258a.145.145 0 0 1 0-.274l.774-.258c.346-.115.617-.386.732-.732L13.863.1z"/>'),SS=Xu("Mouse",'<path d="M8 3a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 3zm4 8a4 4 0 0 1-8 0V5a4 4 0 1 1 8 0v6zM8 0a5 5 0 0 0-5 5v6a5 5 0 0 0 10 0V5a5 5 0 0 0-5-5z"/>'),VS=Xu("Mouse2",'<path d="M3 5.188C3 2.341 5.22 0 8 0s5 2.342 5 5.188v5.625C13 13.658 10.78 16 8 16s-5-2.342-5-5.188V5.189zm4.5-4.155C5.541 1.289 4 3.035 4 5.188V5.5h3.5V1.033zm1 0V5.5H12v-.313c0-2.152-1.541-3.898-3.5-4.154zM12 6.5H4v4.313C4 13.145 5.81 15 8 15s4-1.855 4-4.188V6.5z"/>'),OS=Xu("Mouse2Fill",'<path d="M7.5.026C4.958.286 3 2.515 3 5.188V5.5h4.5V.026zm1 0V5.5H13v-.312C13 2.515 11.042.286 8.5.026zM13 6.5H3v4.313C3 13.658 5.22 16 8 16s5-2.342 5-5.188V6.5z"/>'),TS=Xu("Mouse3",'<path d="M7 0c-.593 0-1.104.157-1.527.463-.418.302-.717.726-.93 1.208C4.123 2.619 4 3.879 4 5.187v.504L3.382 6A2.5 2.5 0 0 0 2 8.236v2.576C2 13.659 4.22 16 7 16h2c2.78 0 5-2.342 5-5.188V7.51a.71.71 0 0 0 0-.02V5.186c0-1.13-.272-2.044-.748-2.772-.474-.726-1.13-1.235-1.849-1.59C9.981.123 8.26 0 7 0zm2.5 6.099V1.232c.51.11 1.008.267 1.46.49.596.293 1.099.694 1.455 1.24.355.543.585 1.262.585 2.225v1.69l-3.5-.778zm-1-5.025v4.803L5 5.099c.006-1.242.134-2.293.457-3.024.162-.366.363-.63.602-.801C6.292 1.105 6.593 1 7 1c.468 0 .98.018 1.5.074zM5 6.124 13 7.9v2.912C13 13.145 11.19 15 9 15H7c-2.19 0-4-1.855-4-4.188V8.236a1.5 1.5 0 0 1 .83-1.342l.187-.093c.01.265.024.58.047.92.062.938.19 2.12.462 2.937a.5.5 0 1 0 .948-.316c-.227-.683-.35-1.75-.413-2.688a29.17 29.17 0 0 1-.06-1.528v-.002z"/>'),DS=Xu("Mouse3Fill",'<path d="M8.5.069A15.328 15.328 0 0 0 7 0c-.593 0-1.104.157-1.527.463-.418.302-.717.726-.93 1.208-.386.873-.522 2.01-.54 3.206l4.497 1V.069zM3.71 5.836 3.381 6A2.5 2.5 0 0 0 2 8.236v2.576C2 13.659 4.22 16 7 16h2c2.78 0 5-2.342 5-5.188V8.123l-9-2v.003l.008.353c.007.3.023.715.053 1.175.063.937.186 2.005.413 2.688a.5.5 0 1 1-.948.316c-.273-.817-.4-2-.462-2.937A30.16 30.16 0 0 1 4 6.003c0-.034.003-.067.01-.1l-.3-.067zM14 7.1V5.187c0-1.13-.272-2.044-.748-2.772-.474-.726-1.13-1.235-1.849-1.59A7.495 7.495 0 0 0 9.5.212v5.887l4.5 1z"/>'),CS=Xu("MouseFill",'<path d="M3 5a5 5 0 0 1 10 0v6a5 5 0 0 1-10 0V5zm5.5-1.5a.5.5 0 0 0-1 0v2a.5.5 0 0 0 1 0v-2z"/>'),jS=Xu("MusicNote",'<path d="M9 13c0 1.105-1.12 2-2.5 2S4 14.105 4 13s1.12-2 2.5-2 2.5.895 2.5 2z"/><path fill-rule="evenodd" d="M9 3v10H8V3h1z"/><path d="M8 2.82a1 1 0 0 1 .804-.98l3-.6A1 1 0 0 1 13 2.22V4L8 5V2.82z"/>'),xS=Xu("MusicNoteBeamed",'<path d="M6 13c0 1.105-1.12 2-2.5 2S1 14.105 1 13c0-1.104 1.12-2 2.5-2s2.5.896 2.5 2zm9-2c0 1.105-1.12 2-2.5 2s-2.5-.895-2.5-2 1.12-2 2.5-2 2.5.895 2.5 2z"/><path fill-rule="evenodd" d="M14 11V2h1v9h-1zM6 3v10H5V3h1z"/><path d="M5 2.905a1 1 0 0 1 .9-.995l8-.8a1 1 0 0 1 1.1.995V3L5 4V2.905z"/>'),BS=Xu("MusicNoteList",'<path d="M12 13c0 1.105-1.12 2-2.5 2S7 14.105 7 13s1.12-2 2.5-2 2.5.895 2.5 2z"/><path fill-rule="evenodd" d="M12 3v10h-1V3h1z"/><path d="M11 2.82a1 1 0 0 1 .804-.98l3-.6A1 1 0 0 1 16 2.22V4l-5 1V2.82z"/><path fill-rule="evenodd" d="M0 11.5a.5.5 0 0 1 .5-.5H4a.5.5 0 0 1 0 1H.5a.5.5 0 0 1-.5-.5zm0-4A.5.5 0 0 1 .5 7H8a.5.5 0 0 1 0 1H.5a.5.5 0 0 1-.5-.5zm0-4A.5.5 0 0 1 .5 3H8a.5.5 0 0 1 0 1H.5a.5.5 0 0 1-.5-.5z"/>'),YS=Xu("MusicPlayer",'<path d="M4 3a1 1 0 0 1 1-1h6a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1V3zm1 0v3h6V3H5zm3 9a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/><path d="M11 11a3 3 0 1 1-6 0 3 3 0 0 1 6 0zm-3 2a2 2 0 1 0 0-4 2 2 0 0 0 0 4z"/><path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm2-1a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H4z"/>'),PS=Xu("MusicPlayerFill",'<path d="M8 12a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm1 2h6a1 1 0 0 1 1 1v2.5a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1zm3 12a3 3 0 1 1 0-6 3 3 0 0 1 0 6z"/>'),IS=Xu("Newspaper",'<path d="M0 2.5A1.5 1.5 0 0 1 1.5 1h11A1.5 1.5 0 0 1 14 2.5v10.528c0 .3-.05.654-.238.972h.738a.5.5 0 0 0 .5-.5v-9a.5.5 0 0 1 1 0v9a1.5 1.5 0 0 1-1.5 1.5H1.497A1.497 1.497 0 0 1 0 13.5v-11zM12 14c.37 0 .654-.211.853-.441.092-.106.147-.279.147-.531V2.5a.5.5 0 0 0-.5-.5h-11a.5.5 0 0 0-.5.5v11c0 .278.223.5.497.5H12z"/><path d="M2 3h10v2H2V3zm0 3h4v3H2V6zm0 4h4v1H2v-1zm0 2h4v1H2v-1zm5-6h2v1H7V6zm3 0h2v1h-2V6zM7 8h2v1H7V8zm3 0h2v1h-2V8zm-3 2h2v1H7v-1zm3 0h2v1h-2v-1zm-3 2h2v1H7v-1zm3 0h2v1h-2v-1z"/>'),FS=Xu("NodeMinus",'<path fill-rule="evenodd" d="M11 4a4 4 0 1 0 0 8 4 4 0 0 0 0-8zM6.025 7.5a5 5 0 1 1 0 1H4A1.5 1.5 0 0 1 2.5 10h-1A1.5 1.5 0 0 1 0 8.5v-1A1.5 1.5 0 0 1 1.5 6h1A1.5 1.5 0 0 1 4 7.5h2.025zM1.5 7a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1zM8 8a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5A.5.5 0 0 1 8 8z"/>'),ES=Xu("NodeMinusFill",'<path fill-rule="evenodd" d="M16 8a5 5 0 0 1-9.975.5H4A1.5 1.5 0 0 1 2.5 10h-1A1.5 1.5 0 0 1 0 8.5v-1A1.5 1.5 0 0 1 1.5 6h1A1.5 1.5 0 0 1 4 7.5h2.025A5 5 0 0 1 16 8zm-2 0a.5.5 0 0 0-.5-.5h-5a.5.5 0 0 0 0 1h5A.5.5 0 0 0 14 8z"/>'),$S=Xu("NodePlus",'<path fill-rule="evenodd" d="M11 4a4 4 0 1 0 0 8 4 4 0 0 0 0-8zM6.025 7.5a5 5 0 1 1 0 1H4A1.5 1.5 0 0 1 2.5 10h-1A1.5 1.5 0 0 1 0 8.5v-1A1.5 1.5 0 0 1 1.5 6h1A1.5 1.5 0 0 1 4 7.5h2.025zM11 5a.5.5 0 0 1 .5.5v2h2a.5.5 0 0 1 0 1h-2v2a.5.5 0 0 1-1 0v-2h-2a.5.5 0 0 1 0-1h2v-2A.5.5 0 0 1 11 5zM1.5 7a.5.5 0 0 0-.5.5v1a.5.5 0 0 0 .5.5h1a.5.5 0 0 0 .5-.5v-1a.5.5 0 0 0-.5-.5h-1z"/>'),RS=Xu("NodePlusFill",'<path d="M11 13a5 5 0 1 0-4.975-5.5H4A1.5 1.5 0 0 0 2.5 6h-1A1.5 1.5 0 0 0 0 7.5v1A1.5 1.5 0 0 0 1.5 10h1A1.5 1.5 0 0 0 4 8.5h2.025A5 5 0 0 0 11 13zm.5-7.5v2h2a.5.5 0 0 1 0 1h-2v2a.5.5 0 0 1-1 0v-2h-2a.5.5 0 0 1 0-1h2v-2a.5.5 0 0 1 1 0z"/>'),NS=Xu("Nut",'<path d="m11.42 2 3.428 6-3.428 6H4.58L1.152 8 4.58 2h6.84zM4.58 1a1 1 0 0 0-.868.504l-3.428 6a1 1 0 0 0 0 .992l3.428 6A1 1 0 0 0 4.58 15h6.84a1 1 0 0 0 .868-.504l3.429-6a1 1 0 0 0 0-.992l-3.429-6A1 1 0 0 0 11.42 1H4.58z"/><path d="M6.848 5.933a2.5 2.5 0 1 0 2.5 4.33 2.5 2.5 0 0 0-2.5-4.33zm-1.78 3.915a3.5 3.5 0 1 1 6.061-3.5 3.5 3.5 0 0 1-6.062 3.5z"/>'),WS=Xu("NutFill",'<path d="M4.58 1a1 1 0 0 0-.868.504l-3.428 6a1 1 0 0 0 0 .992l3.428 6A1 1 0 0 0 4.58 15h6.84a1 1 0 0 0 .868-.504l3.429-6a1 1 0 0 0 0-.992l-3.429-6A1 1 0 0 0 11.42 1H4.58zm5.018 9.696a3 3 0 1 1-3-5.196 3 3 0 0 1 3 5.196z"/>'),US=Xu("Octagon",'<path d="M4.54.146A.5.5 0 0 1 4.893 0h6.214a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353L4.54.146zM5.1 1 1 5.1v5.8L5.1 15h5.8l4.1-4.1V5.1L10.9 1H5.1z"/>'),GS=Xu("OctagonFill",'<path d="M11.107 0a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353L4.54.146A.5.5 0 0 1 4.893 0h6.214z"/>'),qS=Xu("OctagonHalf",'<path d="M4.54.146A.5.5 0 0 1 4.893 0h6.214a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353L4.54.146zM8 15h2.9l4.1-4.1V5.1L10.9 1H8v14z"/>'),JS=Xu("Option",'<path d="M1 2.5a.5.5 0 0 1 .5-.5h3.797a.5.5 0 0 1 .439.26L11 13h3.5a.5.5 0 0 1 0 1h-3.797a.5.5 0 0 1-.439-.26L5 3H1.5a.5.5 0 0 1-.5-.5zm10 0a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1h-3a.5.5 0 0 1-.5-.5z"/>'),KS=Xu("Outlet",'<path d="M3.34 2.994c.275-.338.68-.494 1.074-.494h7.172c.393 0 .798.156 1.074.494.578.708 1.84 2.534 1.84 5.006 0 2.472-1.262 4.297-1.84 5.006-.276.338-.68.494-1.074.494H4.414c-.394 0-.799-.156-1.074-.494C2.762 12.297 1.5 10.472 1.5 8c0-2.472 1.262-4.297 1.84-5.006zm1.074.506a.376.376 0 0 0-.299.126C3.599 4.259 2.5 5.863 2.5 8c0 2.137 1.099 3.74 1.615 4.374.06.073.163.126.3.126h7.17c.137 0 .24-.053.3-.126.516-.633 1.615-2.237 1.615-4.374 0-2.137-1.099-3.74-1.615-4.374a.376.376 0 0 0-.3-.126h-7.17z"/><path d="M6 5.5a.5.5 0 0 1 .5.5v1.5a.5.5 0 0 1-1 0V6a.5.5 0 0 1 .5-.5zm4 0a.5.5 0 0 1 .5.5v1.5a.5.5 0 0 1-1 0V6a.5.5 0 0 1 .5-.5zM7 10v1h2v-1a1 1 0 0 0-2 0z"/>'),XS=Xu("PaintBucket",'<path d="M6.192 2.78c-.458-.677-.927-1.248-1.35-1.643a2.972 2.972 0 0 0-.71-.515c-.217-.104-.56-.205-.882-.02-.367.213-.427.63-.43.896-.003.304.064.664.173 1.044.196.687.556 1.528 1.035 2.402L.752 8.22c-.277.277-.269.656-.218.918.055.283.187.593.36.903.348.627.92 1.361 1.626 2.068.707.707 1.441 1.278 2.068 1.626.31.173.62.305.903.36.262.05.64.059.918-.218l5.615-5.615c.118.257.092.512.05.939-.03.292-.068.665-.073 1.176v.123h.003a1 1 0 0 0 1.993 0H14v-.057a1.01 1.01 0 0 0-.004-.117c-.055-1.25-.7-2.738-1.86-3.494a4.322 4.322 0 0 0-.211-.434c-.349-.626-.92-1.36-1.627-2.067-.707-.707-1.441-1.279-2.068-1.627-.31-.172-.62-.304-.903-.36-.262-.05-.64-.058-.918.219l-.217.216zM4.16 1.867c.381.356.844.922 1.311 1.632l-.704.705c-.382-.727-.66-1.402-.813-1.938a3.283 3.283 0 0 1-.131-.673c.091.061.204.15.337.274zm.394 3.965c.54.852 1.107 1.567 1.607 2.033a.5.5 0 1 0 .682-.732c-.453-.422-1.017-1.136-1.564-2.027l1.088-1.088c.054.12.115.243.183.365.349.627.92 1.361 1.627 2.068.706.707 1.44 1.278 2.068 1.626.122.068.244.13.365.183l-4.861 4.862a.571.571 0 0 1-.068-.01c-.137-.027-.342-.104-.608-.252-.524-.292-1.186-.8-1.846-1.46-.66-.66-1.168-1.32-1.46-1.846-.147-.265-.225-.47-.251-.607a.573.573 0 0 1-.01-.068l3.048-3.047zm2.87-1.935a2.44 2.44 0 0 1-.241-.561c.135.033.324.11.562.241.524.292 1.186.8 1.846 1.46.45.45.83.901 1.118 1.31a3.497 3.497 0 0 0-1.066.091 11.27 11.27 0 0 1-.76-.694c-.66-.66-1.167-1.322-1.458-1.847z"/>'),ZS=Xu("Palette",'<path d="M8 5a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3zm4 3a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3zM5.5 7a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm.5 6a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/><path d="M16 8c0 3.15-1.866 2.585-3.567 2.07C11.42 9.763 10.465 9.473 10 10c-.603.683-.475 1.819-.351 2.92C9.826 14.495 9.996 16 8 16a8 8 0 1 1 8-8zm-8 7c.611 0 .654-.171.655-.176.078-.146.124-.464.07-1.119-.014-.168-.037-.37-.061-.591-.052-.464-.112-1.005-.118-1.462-.01-.707.083-1.61.704-2.314.369-.417.845-.578 1.272-.618.404-.038.812.026 1.16.104.343.077.702.186 1.025.284l.028.008c.346.105.658.199.953.266.653.148.904.083.991.024C14.717 9.38 15 9.161 15 8a7 7 0 1 0-7 7z"/>'),QS=Xu("Palette2",'<path d="M0 .5A.5.5 0 0 1 .5 0h5a.5.5 0 0 1 .5.5v5.277l4.147-4.131a.5.5 0 0 1 .707 0l3.535 3.536a.5.5 0 0 1 0 .708L10.261 10H15.5a.5.5 0 0 1 .5.5v5a.5.5 0 0 1-.5.5H3a2.99 2.99 0 0 1-2.121-.879A2.99 2.99 0 0 1 0 13.044m6-.21 7.328-7.3-2.829-2.828L6 7.188v5.647zM4.5 13a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0zM15 15v-4H9.258l-4.015 4H15zM0 .5v12.495V.5z"/><path d="M0 12.995V13a3.07 3.07 0 0 0 0-.005z"/>'),eV=Xu("PaletteFill",'<path d="M12.433 10.07C14.133 10.585 16 11.15 16 8a8 8 0 1 0-8 8c1.996 0 1.826-1.504 1.649-3.08-.124-1.101-.252-2.237.351-2.92.465-.527 1.42-.237 2.433.07zM8 5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm4.5 3a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zM5 6.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm.5 6.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),tV=Xu("Paperclip",'<path d="M4.5 3a2.5 2.5 0 0 1 5 0v9a1.5 1.5 0 0 1-3 0V5a.5.5 0 0 1 1 0v7a.5.5 0 0 0 1 0V3a1.5 1.5 0 1 0-3 0v9a2.5 2.5 0 0 0 5 0V5a.5.5 0 0 1 1 0v7a3.5 3.5 0 1 1-7 0V3z"/>'),aV=Xu("Paragraph",'<path d="M10.5 15a.5.5 0 0 1-.5-.5V2H9v12.5a.5.5 0 0 1-1 0V9H7a4 4 0 1 1 0-8h5.5a.5.5 0 0 1 0 1H11v12.5a.5.5 0 0 1-.5.5z"/>'),nV=Xu("PatchCheck",'<path fill-rule="evenodd" d="M10.354 6.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7 8.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="m10.273 2.513-.921-.944.715-.698.622.637.89-.011a2.89 2.89 0 0 1 2.924 2.924l-.01.89.636.622a2.89 2.89 0 0 1 0 4.134l-.637.622.011.89a2.89 2.89 0 0 1-2.924 2.924l-.89-.01-.622.636a2.89 2.89 0 0 1-4.134 0l-.622-.637-.89.011a2.89 2.89 0 0 1-2.924-2.924l.01-.89-.636-.622a2.89 2.89 0 0 1 0-4.134l.637-.622-.011-.89a2.89 2.89 0 0 1 2.924-2.924l.89.01.622-.636a2.89 2.89 0 0 1 4.134 0l-.715.698a1.89 1.89 0 0 0-2.704 0l-.92.944-1.32-.016a1.89 1.89 0 0 0-1.911 1.912l.016 1.318-.944.921a1.89 1.89 0 0 0 0 2.704l.944.92-.016 1.32a1.89 1.89 0 0 0 1.912 1.911l1.318-.016.921.944a1.89 1.89 0 0 0 2.704 0l.92-.944 1.32.016a1.89 1.89 0 0 0 1.911-1.912l-.016-1.318.944-.921a1.89 1.89 0 0 0 0-2.704l-.944-.92.016-1.32a1.89 1.89 0 0 0-1.912-1.911l-1.318.016z"/>'),rV=Xu("PatchCheckFill",'<path d="M10.067.87a2.89 2.89 0 0 0-4.134 0l-.622.638-.89-.011a2.89 2.89 0 0 0-2.924 2.924l.01.89-.636.622a2.89 2.89 0 0 0 0 4.134l.637.622-.011.89a2.89 2.89 0 0 0 2.924 2.924l.89-.01.622.636a2.89 2.89 0 0 0 4.134 0l.622-.637.89.011a2.89 2.89 0 0 0 2.924-2.924l-.01-.89.636-.622a2.89 2.89 0 0 0 0-4.134l-.637-.622.011-.89a2.89 2.89 0 0 0-2.924-2.924l-.89.01-.622-.636zm.287 5.984-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7 8.793l2.646-2.647a.5.5 0 0 1 .708.708z"/>'),iV=Xu("PatchExclamation",'<path d="M7.001 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.553.553 0 0 1-1.1 0L7.1 4.995z"/><path d="m10.273 2.513-.921-.944.715-.698.622.637.89-.011a2.89 2.89 0 0 1 2.924 2.924l-.01.89.636.622a2.89 2.89 0 0 1 0 4.134l-.637.622.011.89a2.89 2.89 0 0 1-2.924 2.924l-.89-.01-.622.636a2.89 2.89 0 0 1-4.134 0l-.622-.637-.89.011a2.89 2.89 0 0 1-2.924-2.924l.01-.89-.636-.622a2.89 2.89 0 0 1 0-4.134l.637-.622-.011-.89a2.89 2.89 0 0 1 2.924-2.924l.89.01.622-.636a2.89 2.89 0 0 1 4.134 0l-.715.698a1.89 1.89 0 0 0-2.704 0l-.92.944-1.32-.016a1.89 1.89 0 0 0-1.911 1.912l.016 1.318-.944.921a1.89 1.89 0 0 0 0 2.704l.944.92-.016 1.32a1.89 1.89 0 0 0 1.912 1.911l1.318-.016.921.944a1.89 1.89 0 0 0 2.704 0l.92-.944 1.32.016a1.89 1.89 0 0 0 1.911-1.912l-.016-1.318.944-.921a1.89 1.89 0 0 0 0-2.704l-.944-.92.016-1.32a1.89 1.89 0 0 0-1.912-1.911l-1.318.016z"/>'),oV=Xu("PatchExclamationFill",'<path d="M10.067.87a2.89 2.89 0 0 0-4.134 0l-.622.638-.89-.011a2.89 2.89 0 0 0-2.924 2.924l.01.89-.636.622a2.89 2.89 0 0 0 0 4.134l.637.622-.011.89a2.89 2.89 0 0 0 2.924 2.924l.89-.01.622.636a2.89 2.89 0 0 0 4.134 0l.622-.637.89.011a2.89 2.89 0 0 0 2.924-2.924l-.01-.89.636-.622a2.89 2.89 0 0 0 0-4.134l-.637-.622.011-.89a2.89 2.89 0 0 0-2.924-2.924l-.89.01-.622-.636zM8 4c.535 0 .954.462.9.995l-.35 3.507a.552.552 0 0 1-1.1 0L7.1 4.995A.905.905 0 0 1 8 4zm.002 6a1 1 0 1 1 0 2 1 1 0 0 1 0-2z"/>'),sV=Xu("PatchMinus",'<path fill-rule="evenodd" d="M5.5 8a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/><path d="m10.273 2.513-.921-.944.715-.698.622.637.89-.011a2.89 2.89 0 0 1 2.924 2.924l-.01.89.636.622a2.89 2.89 0 0 1 0 4.134l-.637.622.011.89a2.89 2.89 0 0 1-2.924 2.924l-.89-.01-.622.636a2.89 2.89 0 0 1-4.134 0l-.622-.637-.89.011a2.89 2.89 0 0 1-2.924-2.924l.01-.89-.636-.622a2.89 2.89 0 0 1 0-4.134l.637-.622-.011-.89a2.89 2.89 0 0 1 2.924-2.924l.89.01.622-.636a2.89 2.89 0 0 1 4.134 0l-.715.698a1.89 1.89 0 0 0-2.704 0l-.92.944-1.32-.016a1.89 1.89 0 0 0-1.911 1.912l.016 1.318-.944.921a1.89 1.89 0 0 0 0 2.704l.944.92-.016 1.32a1.89 1.89 0 0 0 1.912 1.911l1.318-.016.921.944a1.89 1.89 0 0 0 2.704 0l.92-.944 1.32.016a1.89 1.89 0 0 0 1.911-1.912l-.016-1.318.944-.921a1.89 1.89 0 0 0 0-2.704l-.944-.92.016-1.32a1.89 1.89 0 0 0-1.912-1.911l-1.318.016z"/>'),lV=Xu("PatchMinusFill",'<path d="M10.067.87a2.89 2.89 0 0 0-4.134 0l-.622.638-.89-.011a2.89 2.89 0 0 0-2.924 2.924l.01.89-.636.622a2.89 2.89 0 0 0 0 4.134l.637.622-.011.89a2.89 2.89 0 0 0 2.924 2.924l.89-.01.622.636a2.89 2.89 0 0 0 4.134 0l.622-.637.89.011a2.89 2.89 0 0 0 2.924-2.924l-.01-.89.636-.622a2.89 2.89 0 0 0 0-4.134l-.637-.622.011-.89a2.89 2.89 0 0 0-2.924-2.924l-.89.01-.622-.636zM6 7.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1 0-1z"/>'),cV=Xu("PatchPlus",'<path fill-rule="evenodd" d="M8 5.5a.5.5 0 0 1 .5.5v1.5H10a.5.5 0 0 1 0 1H8.5V10a.5.5 0 0 1-1 0V8.5H6a.5.5 0 0 1 0-1h1.5V6a.5.5 0 0 1 .5-.5z"/><path d="m10.273 2.513-.921-.944.715-.698.622.637.89-.011a2.89 2.89 0 0 1 2.924 2.924l-.01.89.636.622a2.89 2.89 0 0 1 0 4.134l-.637.622.011.89a2.89 2.89 0 0 1-2.924 2.924l-.89-.01-.622.636a2.89 2.89 0 0 1-4.134 0l-.622-.637-.89.011a2.89 2.89 0 0 1-2.924-2.924l.01-.89-.636-.622a2.89 2.89 0 0 1 0-4.134l.637-.622-.011-.89a2.89 2.89 0 0 1 2.924-2.924l.89.01.622-.636a2.89 2.89 0 0 1 4.134 0l-.715.698a1.89 1.89 0 0 0-2.704 0l-.92.944-1.32-.016a1.89 1.89 0 0 0-1.911 1.912l.016 1.318-.944.921a1.89 1.89 0 0 0 0 2.704l.944.92-.016 1.32a1.89 1.89 0 0 0 1.912 1.911l1.318-.016.921.944a1.89 1.89 0 0 0 2.704 0l.92-.944 1.32.016a1.89 1.89 0 0 0 1.911-1.912l-.016-1.318.944-.921a1.89 1.89 0 0 0 0-2.704l-.944-.92.016-1.32a1.89 1.89 0 0 0-1.912-1.911l-1.318.016z"/>'),uV=Xu("PatchPlusFill",'<path d="M10.067.87a2.89 2.89 0 0 0-4.134 0l-.622.638-.89-.011a2.89 2.89 0 0 0-2.924 2.924l.01.89-.636.622a2.89 2.89 0 0 0 0 4.134l.637.622-.011.89a2.89 2.89 0 0 0 2.924 2.924l.89-.01.622.636a2.89 2.89 0 0 0 4.134 0l.622-.637.89.011a2.89 2.89 0 0 0 2.924-2.924l-.01-.89.636-.622a2.89 2.89 0 0 0 0-4.134l-.637-.622.011-.89a2.89 2.89 0 0 0-2.924-2.924l-.89.01-.622-.636zM8.5 6v1.5H10a.5.5 0 0 1 0 1H8.5V10a.5.5 0 0 1-1 0V8.5H6a.5.5 0 0 1 0-1h1.5V6a.5.5 0 0 1 1 0z"/>'),dV=Xu("PatchQuestion",'<path d="M8.05 9.6c.336 0 .504-.24.554-.627.04-.534.198-.815.847-1.26.673-.475 1.049-1.09 1.049-1.986 0-1.325-.92-2.227-2.262-2.227-1.02 0-1.792.492-2.1 1.29A1.71 1.71 0 0 0 6 5.48c0 .393.203.64.545.64.272 0 .455-.147.564-.51.158-.592.525-.915 1.074-.915.61 0 1.03.446 1.03 1.084 0 .563-.208.885-.822 1.325-.619.433-.926.914-.926 1.64v.111c0 .428.208.745.585.745z"/><path d="m10.273 2.513-.921-.944.715-.698.622.637.89-.011a2.89 2.89 0 0 1 2.924 2.924l-.01.89.636.622a2.89 2.89 0 0 1 0 4.134l-.637.622.011.89a2.89 2.89 0 0 1-2.924 2.924l-.89-.01-.622.636a2.89 2.89 0 0 1-4.134 0l-.622-.637-.89.011a2.89 2.89 0 0 1-2.924-2.924l.01-.89-.636-.622a2.89 2.89 0 0 1 0-4.134l.637-.622-.011-.89a2.89 2.89 0 0 1 2.924-2.924l.89.01.622-.636a2.89 2.89 0 0 1 4.134 0l-.715.698a1.89 1.89 0 0 0-2.704 0l-.92.944-1.32-.016a1.89 1.89 0 0 0-1.911 1.912l.016 1.318-.944.921a1.89 1.89 0 0 0 0 2.704l.944.92-.016 1.32a1.89 1.89 0 0 0 1.912 1.911l1.318-.016.921.944a1.89 1.89 0 0 0 2.704 0l.92-.944 1.32.016a1.89 1.89 0 0 0 1.911-1.912l-.016-1.318.944-.921a1.89 1.89 0 0 0 0-2.704l-.944-.92.016-1.32a1.89 1.89 0 0 0-1.912-1.911l-1.318.016z"/><path d="M7.001 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0z"/>'),hV=Xu("PatchQuestionFill",'<path d="M5.933.87a2.89 2.89 0 0 1 4.134 0l.622.638.89-.011a2.89 2.89 0 0 1 2.924 2.924l-.01.89.636.622a2.89 2.89 0 0 1 0 4.134l-.637.622.011.89a2.89 2.89 0 0 1-2.924 2.924l-.89-.01-.622.636a2.89 2.89 0 0 1-4.134 0l-.622-.637-.89.011a2.89 2.89 0 0 1-2.924-2.924l.01-.89-.636-.622a2.89 2.89 0 0 1 0-4.134l.637-.622-.011-.89a2.89 2.89 0 0 1 2.924-2.924l.89.01.622-.636zM7.002 11a1 1 0 1 0 2 0 1 1 0 0 0-2 0zm1.602-2.027c.04-.534.198-.815.846-1.26.674-.475 1.05-1.09 1.05-1.986 0-1.325-.92-2.227-2.262-2.227-1.02 0-1.792.492-2.1 1.29A1.71 1.71 0 0 0 6 5.48c0 .393.203.64.545.64.272 0 .455-.147.564-.51.158-.592.525-.915 1.074-.915.61 0 1.03.446 1.03 1.084 0 .563-.208.885-.822 1.325-.619.433-.926.914-.926 1.64v.111c0 .428.208.745.585.745.336 0 .504-.24.554-.627z"/>'),fV=Xu("Pause",'<path d="M6 3.5a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-1 0V4a.5.5 0 0 1 .5-.5zm4 0a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-1 0V4a.5.5 0 0 1 .5-.5z"/>'),pV=Xu("PauseBtn",'<path d="M6.25 5C5.56 5 5 5.56 5 6.25v3.5a1.25 1.25 0 1 0 2.5 0v-3.5C7.5 5.56 6.94 5 6.25 5zm3.5 0c-.69 0-1.25.56-1.25 1.25v3.5a1.25 1.25 0 1 0 2.5 0v-3.5C11 5.56 10.44 5 9.75 5z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),mV=Xu("PauseBtnFill",'<path d="M0 12V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm6.25-7C5.56 5 5 5.56 5 6.25v3.5a1.25 1.25 0 1 0 2.5 0v-3.5C7.5 5.56 6.94 5 6.25 5zm3.5 0c-.69 0-1.25.56-1.25 1.25v3.5a1.25 1.25 0 1 0 2.5 0v-3.5C11 5.56 10.44 5 9.75 5z"/>'),vV=Xu("PauseCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M5 6.25a1.25 1.25 0 1 1 2.5 0v3.5a1.25 1.25 0 1 1-2.5 0v-3.5zm3.5 0a1.25 1.25 0 1 1 2.5 0v3.5a1.25 1.25 0 1 1-2.5 0v-3.5z"/>'),_V=Xu("PauseCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM6.25 5C5.56 5 5 5.56 5 6.25v3.5a1.25 1.25 0 1 0 2.5 0v-3.5C7.5 5.56 6.94 5 6.25 5zm3.5 0c-.69 0-1.25.56-1.25 1.25v3.5a1.25 1.25 0 1 0 2.5 0v-3.5C11 5.56 10.44 5 9.75 5z"/>'),gV=Xu("PauseFill",'<path d="M5.5 3.5A1.5 1.5 0 0 1 7 5v6a1.5 1.5 0 0 1-3 0V5a1.5 1.5 0 0 1 1.5-1.5zm5 0A1.5 1.5 0 0 1 12 5v6a1.5 1.5 0 0 1-3 0V5a1.5 1.5 0 0 1 1.5-1.5z"/>'),yV=Xu("Peace",'<path d="M7.5 1.018a7 7 0 0 0-4.79 11.566L7.5 7.793V1.018zm1 0v6.775l4.79 4.79A7 7 0 0 0 8.5 1.018zm4.084 12.273L8.5 9.207v5.775a6.97 6.97 0 0 0 4.084-1.691zM7.5 14.982V9.207l-4.084 4.084A6.97 6.97 0 0 0 7.5 14.982zM0 8a8 8 0 1 1 16 0A8 8 0 0 1 0 8z"/>'),bV=Xu("PeaceFill",'<path d="M14 13.292A8 8 0 0 0 8.5.015v7.778l5.5 5.5zm-.708.708L8.5 9.206v6.778a7.967 7.967 0 0 0 4.792-1.986zM7.5 15.985V9.207L2.708 14A7.967 7.967 0 0 0 7.5 15.985zM2 13.292A8 8 0 0 1 7.5.015v7.778l-5.5 5.5z"/>'),MV=Xu("Pen",'<path d="m13.498.795.149-.149a1.207 1.207 0 1 1 1.707 1.708l-.149.148a1.5 1.5 0 0 1-.059 2.059L4.854 14.854a.5.5 0 0 1-.233.131l-4 1a.5.5 0 0 1-.606-.606l1-4a.5.5 0 0 1 .131-.232l9.642-9.642a.5.5 0 0 0-.642.056L6.854 4.854a.5.5 0 1 1-.708-.708L9.44.854A1.5 1.5 0 0 1 11.5.796a1.5 1.5 0 0 1 1.998-.001zm-.644.766a.5.5 0 0 0-.707 0L1.95 11.756l-.764 3.057 3.057-.764L14.44 3.854a.5.5 0 0 0 0-.708l-1.585-1.585z"/>'),zV=Xu("PenFill",'<path d="m13.498.795.149-.149a1.207 1.207 0 1 1 1.707 1.708l-.149.148a1.5 1.5 0 0 1-.059 2.059L4.854 14.854a.5.5 0 0 1-.233.131l-4 1a.5.5 0 0 1-.606-.606l1-4a.5.5 0 0 1 .131-.232l9.642-9.642a.5.5 0 0 0-.642.056L6.854 4.854a.5.5 0 1 1-.708-.708L9.44.854A1.5 1.5 0 0 1 11.5.796a1.5 1.5 0 0 1 1.998-.001z"/>'),LV=Xu("Pencil",'<path d="M12.146.146a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1 0 .708l-10 10a.5.5 0 0 1-.168.11l-5 2a.5.5 0 0 1-.65-.65l2-5a.5.5 0 0 1 .11-.168l10-10zM11.207 2.5 13.5 4.793 14.793 3.5 12.5 1.207 11.207 2.5zm1.586 3L10.5 3.207 4 9.707V10h.5a.5.5 0 0 1 .5.5v.5h.5a.5.5 0 0 1 .5.5v.5h.293l6.5-6.5zm-9.761 5.175-.106.106-1.528 3.821 3.821-1.528.106-.106A.5.5 0 0 1 5 12.5V12h-.5a.5.5 0 0 1-.5-.5V11h-.5a.5.5 0 0 1-.468-.325z"/>'),wV=Xu("PencilFill",'<path d="M12.854.146a.5.5 0 0 0-.707 0L10.5 1.793 14.207 5.5l1.647-1.646a.5.5 0 0 0 0-.708l-3-3zm.646 6.061L9.793 2.5 3.293 9H3.5a.5.5 0 0 1 .5.5v.5h.5a.5.5 0 0 1 .5.5v.5h.5a.5.5 0 0 1 .5.5v.5h.5a.5.5 0 0 1 .5.5v.207l6.5-6.5zm-7.468 7.468A.5.5 0 0 1 6 13.5V13h-.5a.5.5 0 0 1-.5-.5V12h-.5a.5.5 0 0 1-.5-.5V11h-.5a.5.5 0 0 1-.5-.5V10h-.5a.499.499 0 0 1-.175-.032l-.179.178a.5.5 0 0 0-.11.168l-2 5a.5.5 0 0 0 .65.65l5-2a.5.5 0 0 0 .168-.11l.178-.178z"/>'),HV=Xu("PencilSquare",'<path d="M15.502 1.94a.5.5 0 0 1 0 .706L14.459 3.69l-2-2L13.502.646a.5.5 0 0 1 .707 0l1.293 1.293zm-1.75 2.456-2-2L4.939 9.21a.5.5 0 0 0-.121.196l-.805 2.414a.25.25 0 0 0 .316.316l2.414-.805a.5.5 0 0 0 .196-.12l6.813-6.814z"/><path fill-rule="evenodd" d="M1 13.5A1.5 1.5 0 0 0 2.5 15h11a1.5 1.5 0 0 0 1.5-1.5v-6a.5.5 0 0 0-1 0v6a.5.5 0 0 1-.5.5h-11a.5.5 0 0 1-.5-.5v-11a.5.5 0 0 1 .5-.5H9a.5.5 0 0 0 0-1H2.5A1.5 1.5 0 0 0 1 2.5v11z"/>'),kV=Xu("Pentagon",'<path d="m8 1.288 6.842 5.56L12.267 15H3.733L1.158 6.847 8 1.288zM16 6.5 8 0 0 6.5 3 16h10l3-9.5z"/>'),AV=Xu("PentagonFill",'<path d="m8 0 8 6.5-3 9.5H3L0 6.5 8 0z"/>'),SV=Xu("PentagonHalf",'<path d="m8 1.288 6.842 5.56L12.267 15H8V1.288zM16 6.5 8 0 0 6.5 3 16h10l3-9.5z"/>'),VV=Xu("People",'<path d="M15 14s1 0 1-1-1-4-5-4-5 3-5 4 1 1 1 1h8zm-7.978-1A.261.261 0 0 1 7 12.996c.001-.264.167-1.03.76-1.72C8.312 10.629 9.282 10 11 10c1.717 0 2.687.63 3.24 1.276.593.69.758 1.457.76 1.72l-.008.002a.274.274 0 0 1-.014.002H7.022zM11 7a2 2 0 1 0 0-4 2 2 0 0 0 0 4zm3-2a3 3 0 1 1-6 0 3 3 0 0 1 6 0zM6.936 9.28a5.88 5.88 0 0 0-1.23-.247A7.35 7.35 0 0 0 5 9c-4 0-5 3-5 4 0 .667.333 1 1 1h4.216A2.238 2.238 0 0 1 5 13c0-1.01.377-2.042 1.09-2.904.243-.294.526-.569.846-.816zM4.92 10A5.493 5.493 0 0 0 4 13H1c0-.26.164-1.03.76-1.724.545-.636 1.492-1.256 3.16-1.275zM1.5 5.5a3 3 0 1 1 6 0 3 3 0 0 1-6 0zm3-2a2 2 0 1 0 0 4 2 2 0 0 0 0-4z"/>'),OV=Xu("PeopleFill",'<path d="M7 14s-1 0-1-1 1-4 5-4 5 3 5 4-1 1-1 1H7zm4-6a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/><path fill-rule="evenodd" d="M5.216 14A2.238 2.238 0 0 1 5 13c0-1.355.68-2.75 1.936-3.72A6.325 6.325 0 0 0 5 9c-4 0-5 3-5 4s1 1 1 1h4.216z"/><path d="M4.5 8a2.5 2.5 0 1 0 0-5 2.5 2.5 0 0 0 0 5z"/>'),TV=Xu("Percent",'<path d="M13.442 2.558a.625.625 0 0 1 0 .884l-10 10a.625.625 0 1 1-.884-.884l10-10a.625.625 0 0 1 .884 0zM4.5 6a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm0 1a2.5 2.5 0 1 0 0-5 2.5 2.5 0 0 0 0 5zm7 6a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm0 1a2.5 2.5 0 1 0 0-5 2.5 2.5 0 0 0 0 5z"/>'),DV=Xu("Person",'<path d="M8 8a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm2-3a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm4 8c0 1-1 1-1 1H3s-1 0-1-1 1-4 6-4 6 3 6 4zm-1-.004c-.001-.246-.154-.986-.832-1.664C11.516 10.68 10.289 10 8 10c-2.29 0-3.516.68-4.168 1.332-.678.678-.83 1.418-.832 1.664h10z"/>'),CV=Xu("PersonBadge",'<path d="M6.5 2a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1h-3zM11 8a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/><path d="M4.5 0A2.5 2.5 0 0 0 2 2.5V14a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2.5A2.5 2.5 0 0 0 11.5 0h-7zM3 2.5A1.5 1.5 0 0 1 4.5 1h7A1.5 1.5 0 0 1 13 2.5v10.795a4.2 4.2 0 0 0-.776-.492C11.392 12.387 10.063 12 8 12s-3.392.387-4.224.803a4.2 4.2 0 0 0-.776.492V2.5z"/>'),jV=Xu("PersonBadgeFill",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm4.5 0a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1h-3zM8 11a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm5 2.755C12.146 12.825 10.623 12 8 12s-4.146.826-5 1.755V14a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1v-.245z"/>'),xV=Xu("PersonBoundingBox",'<path d="M1.5 1a.5.5 0 0 0-.5.5v3a.5.5 0 0 1-1 0v-3A1.5 1.5 0 0 1 1.5 0h3a.5.5 0 0 1 0 1h-3zM11 .5a.5.5 0 0 1 .5-.5h3A1.5 1.5 0 0 1 16 1.5v3a.5.5 0 0 1-1 0v-3a.5.5 0 0 0-.5-.5h-3a.5.5 0 0 1-.5-.5zM.5 11a.5.5 0 0 1 .5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 1 0 1h-3A1.5 1.5 0 0 1 0 14.5v-3a.5.5 0 0 1 .5-.5zm15 0a.5.5 0 0 1 .5.5v3a1.5 1.5 0 0 1-1.5 1.5h-3a.5.5 0 0 1 0-1h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 1 .5-.5z"/><path d="M3 14s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H3zm8-9a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/>'),BV=Xu("PersonCheck",'<path d="M6 8a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm2-3a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm4 8c0 1-1 1-1 1H1s-1 0-1-1 1-4 6-4 6 3 6 4zm-1-.004c-.001-.246-.154-.986-.832-1.664C9.516 10.68 8.289 10 6 10c-2.29 0-3.516.68-4.168 1.332-.678.678-.83 1.418-.832 1.664h10z"/><path fill-rule="evenodd" d="M15.854 5.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 0 1 .708-.708L12.5 7.793l2.646-2.647a.5.5 0 0 1 .708 0z"/>'),YV=Xu("PersonCheckFill",'<path fill-rule="evenodd" d="M15.854 5.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 0 1 .708-.708L12.5 7.793l2.646-2.647a.5.5 0 0 1 .708 0z"/><path d="M1 14s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H1zm5-6a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/>'),PV=Xu("PersonCircle",'<path d="M11 6a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/><path fill-rule="evenodd" d="M0 8a8 8 0 1 1 16 0A8 8 0 0 1 0 8zm8-7a7 7 0 0 0-5.468 11.37C3.242 11.226 4.805 10 8 10s4.757 1.225 5.468 2.37A7 7 0 0 0 8 1z"/>'),IV=Xu("PersonDash",'<path d="M6 8a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm2-3a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm4 8c0 1-1 1-1 1H1s-1 0-1-1 1-4 6-4 6 3 6 4zm-1-.004c-.001-.246-.154-.986-.832-1.664C9.516 10.68 8.289 10 6 10c-2.29 0-3.516.68-4.168 1.332-.678.678-.83 1.418-.832 1.664h10z"/><path fill-rule="evenodd" d="M11 7.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5z"/>'),FV=Xu("PersonDashFill",'<path fill-rule="evenodd" d="M11 7.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5z"/><path d="M1 14s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H1zm5-6a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/>'),EV=Xu("PersonFill",'<path d="M3 14s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H3zm5-6a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/>'),$V=Xu("PersonLinesFill",'<path d="M6 8a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm-5 6s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H1zM11 3.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5zm.5 2.5a.5.5 0 0 0 0 1h4a.5.5 0 0 0 0-1h-4zm2 3a.5.5 0 0 0 0 1h2a.5.5 0 0 0 0-1h-2zm0 3a.5.5 0 0 0 0 1h2a.5.5 0 0 0 0-1h-2z"/>'),RV=Xu("PersonPlus",'<path d="M6 8a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm2-3a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm4 8c0 1-1 1-1 1H1s-1 0-1-1 1-4 6-4 6 3 6 4zm-1-.004c-.001-.246-.154-.986-.832-1.664C9.516 10.68 8.289 10 6 10c-2.29 0-3.516.68-4.168 1.332-.678.678-.83 1.418-.832 1.664h10z"/><path fill-rule="evenodd" d="M13.5 5a.5.5 0 0 1 .5.5V7h1.5a.5.5 0 0 1 0 1H14v1.5a.5.5 0 0 1-1 0V8h-1.5a.5.5 0 0 1 0-1H13V5.5a.5.5 0 0 1 .5-.5z"/>'),NV=Xu("PersonPlusFill",'<path d="M1 14s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H1zm5-6a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/><path fill-rule="evenodd" d="M13.5 5a.5.5 0 0 1 .5.5V7h1.5a.5.5 0 0 1 0 1H14v1.5a.5.5 0 0 1-1 0V8h-1.5a.5.5 0 0 1 0-1H13V5.5a.5.5 0 0 1 .5-.5z"/>'),WV=Xu("PersonSquare",'<path d="M11 6a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/><path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm12 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1v-1c0-1-1-4-6-4s-6 3-6 4v1a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12z"/>'),UV=Xu("PersonX",'<path d="M6 8a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm2-3a2 2 0 1 1-4 0 2 2 0 0 1 4 0zm4 8c0 1-1 1-1 1H1s-1 0-1-1 1-4 6-4 6 3 6 4zm-1-.004c-.001-.246-.154-.986-.832-1.664C9.516 10.68 8.289 10 6 10c-2.29 0-3.516.68-4.168 1.332-.678.678-.83 1.418-.832 1.664h10z"/><path fill-rule="evenodd" d="M12.146 5.146a.5.5 0 0 1 .708 0L14 6.293l1.146-1.147a.5.5 0 0 1 .708.708L14.707 7l1.147 1.146a.5.5 0 0 1-.708.708L14 7.707l-1.146 1.147a.5.5 0 0 1-.708-.708L13.293 7l-1.147-1.146a.5.5 0 0 1 0-.708z"/>'),GV=Xu("PersonXFill",'<path fill-rule="evenodd" d="M1 14s-1 0-1-1 1-4 6-4 6 3 6 4-1 1-1 1H1zm5-6a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm6.146-2.854a.5.5 0 0 1 .708 0L14 6.293l1.146-1.147a.5.5 0 0 1 .708.708L14.707 7l1.147 1.146a.5.5 0 0 1-.708.708L14 7.707l-1.146 1.147a.5.5 0 0 1-.708-.708L13.293 7l-1.147-1.146a.5.5 0 0 1 0-.708z"/>'),qV=Xu("Phone",'<path d="M11 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h6zM5 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h6a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H5z"/><path d="M8 14a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),JV=Xu("PhoneFill",'<path d="M3 2a2 2 0 0 1 2-2h6a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V2zm6 11a1 1 0 1 0-2 0 1 1 0 0 0 2 0z"/>'),KV=Xu("PhoneLandscape",'<path d="M1 4.5a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1v6a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1v-6zm-1 6a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2v-6a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v6z"/><path d="M14 7.5a1 1 0 1 0-2 0 1 1 0 0 0 2 0z"/>'),XV=Xu("PhoneLandscapeFill",'<path d="M2 12.5a2 2 0 0 1-2-2v-6a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v6a2 2 0 0 1-2 2H2zm11-6a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/>'),ZV=Xu("PhoneVibrate",'<path d="M10 3a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H6a1 1 0 0 1-1-1V4a1 1 0 0 1 1-1h4zM6 2a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h4a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H6z"/><path d="M8 12a1 1 0 1 0 0-2 1 1 0 0 0 0 2zM1.599 4.058a.5.5 0 0 1 .208.676A6.967 6.967 0 0 0 1 8c0 1.18.292 2.292.807 3.266a.5.5 0 0 1-.884.468A7.968 7.968 0 0 1 0 8c0-1.347.334-2.619.923-3.734a.5.5 0 0 1 .676-.208zm12.802 0a.5.5 0 0 1 .676.208A7.967 7.967 0 0 1 16 8a7.967 7.967 0 0 1-.923 3.734.5.5 0 0 1-.884-.468A6.967 6.967 0 0 0 15 8c0-1.18-.292-2.292-.807-3.266a.5.5 0 0 1 .208-.676zM3.057 5.534a.5.5 0 0 1 .284.648A4.986 4.986 0 0 0 3 8c0 .642.12 1.255.34 1.818a.5.5 0 1 1-.93.364A5.986 5.986 0 0 1 2 8c0-.769.145-1.505.41-2.182a.5.5 0 0 1 .647-.284zm9.886 0a.5.5 0 0 1 .648.284C13.855 6.495 14 7.231 14 8c0 .769-.145 1.505-.41 2.182a.5.5 0 0 1-.93-.364C12.88 9.255 13 8.642 13 8c0-.642-.12-1.255-.34-1.818a.5.5 0 0 1 .283-.648z"/>'),QV=Xu("PhoneVibrateFill",'<path d="M4 4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2V4zm5 7a1 1 0 1 0-2 0 1 1 0 0 0 2 0zM1.807 4.734a.5.5 0 1 0-.884-.468A7.967 7.967 0 0 0 0 8c0 1.347.334 2.618.923 3.734a.5.5 0 1 0 .884-.468A6.967 6.967 0 0 1 1 8c0-1.18.292-2.292.807-3.266zm13.27-.468a.5.5 0 0 0-.884.468C14.708 5.708 15 6.819 15 8c0 1.18-.292 2.292-.807 3.266a.5.5 0 0 0 .884.468A7.967 7.967 0 0 0 16 8a7.967 7.967 0 0 0-.923-3.734zM3.34 6.182a.5.5 0 1 0-.93-.364A5.986 5.986 0 0 0 2 8c0 .769.145 1.505.41 2.182a.5.5 0 1 0 .93-.364A4.986 4.986 0 0 1 3 8c0-.642.12-1.255.34-1.818zm10.25-.364a.5.5 0 0 0-.93.364c.22.563.34 1.176.34 1.818 0 .642-.12 1.255-.34 1.818a.5.5 0 0 0 .93.364C13.856 9.505 14 8.769 14 8c0-.769-.145-1.505-.41-2.182z"/>'),eO=Xu("PieChart",'<path d="M7.5 1.018a7 7 0 0 0-4.79 11.566L7.5 7.793V1.018zm1 0V7.5h6.482A7.001 7.001 0 0 0 8.5 1.018zM14.982 8.5H8.207l-4.79 4.79A7 7 0 0 0 14.982 8.5zM0 8a8 8 0 1 1 16 0A8 8 0 0 1 0 8z"/>'),tO=Xu("PieChartFill",'<path d="M15.985 8.5H8.207l-5.5 5.5a8 8 0 0 0 13.277-5.5zM2 13.292A8 8 0 0 1 7.5.015v7.778l-5.5 5.5zM8.5.015V7.5h7.485A8.001 8.001 0 0 0 8.5.015z"/>'),aO=Xu("PiggyBank",'<path d="M5 6.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0zm1.138-1.496A6.613 6.613 0 0 1 7.964 4.5c.666 0 1.303.097 1.893.273a.5.5 0 0 0 .286-.958A7.602 7.602 0 0 0 7.964 3.5c-.734 0-1.441.103-2.102.292a.5.5 0 1 0 .276.962z"/><path fill-rule="evenodd" d="M7.964 1.527c-2.977 0-5.571 1.704-6.32 4.125h-.55A1 1 0 0 0 .11 6.824l.254 1.46a1.5 1.5 0 0 0 1.478 1.243h.263c.3.513.688.978 1.145 1.382l-.729 2.477a.5.5 0 0 0 .48.641h2a.5.5 0 0 0 .471-.332l.482-1.351c.635.173 1.31.267 2.011.267.707 0 1.388-.095 2.028-.272l.543 1.372a.5.5 0 0 0 .465.316h2a.5.5 0 0 0 .478-.645l-.761-2.506C13.81 9.895 14.5 8.559 14.5 7.069c0-.145-.007-.29-.02-.431.261-.11.508-.266.705-.444.315.306.815.306.815-.417 0 .223-.5.223-.461-.026a.95.95 0 0 0 .09-.255.7.7 0 0 0-.202-.645.58.58 0 0 0-.707-.098.735.735 0 0 0-.375.562c-.024.243.082.48.32.654a2.112 2.112 0 0 1-.259.153c-.534-2.664-3.284-4.595-6.442-4.595zM2.516 6.26c.455-2.066 2.667-3.733 5.448-3.733 3.146 0 5.536 2.114 5.536 4.542 0 1.254-.624 2.41-1.67 3.248a.5.5 0 0 0-.165.535l.66 2.175h-.985l-.59-1.487a.5.5 0 0 0-.629-.288c-.661.23-1.39.359-2.157.359a6.558 6.558 0 0 1-2.157-.359.5.5 0 0 0-.635.304l-.525 1.471h-.979l.633-2.15a.5.5 0 0 0-.17-.534 4.649 4.649 0 0 1-1.284-1.541.5.5 0 0 0-.446-.275h-.56a.5.5 0 0 1-.492-.414l-.254-1.46h.933a.5.5 0 0 0 .488-.393zm12.621-.857a.565.565 0 0 1-.098.21.704.704 0 0 1-.044-.025c-.146-.09-.157-.175-.152-.223a.236.236 0 0 1 .117-.173c.049-.027.08-.021.113.012a.202.202 0 0 1 .064.199z"/>'),nO=Xu("PiggyBankFill",'<path fill-rule="evenodd" d="M7.964 1.527c-2.977 0-5.571 1.704-6.32 4.125h-.55A1 1 0 0 0 .11 6.824l.254 1.46a1.5 1.5 0 0 0 1.478 1.243h.263c.3.513.688.978 1.145 1.382l-.729 2.477a.5.5 0 0 0 .48.641h2a.5.5 0 0 0 .471-.332l.482-1.351c.635.173 1.31.267 2.011.267.707 0 1.388-.095 2.028-.272l.543 1.372a.5.5 0 0 0 .465.316h2a.5.5 0 0 0 .478-.645l-.761-2.506C13.81 9.895 14.5 8.559 14.5 7.069c0-.145-.007-.29-.02-.431.261-.11.508-.266.705-.444.315.306.815.306.815-.417 0 .223-.5.223-.461-.026a.95.95 0 0 0 .09-.255.7.7 0 0 0-.202-.645.58.58 0 0 0-.707-.098.735.735 0 0 0-.375.562c-.024.243.082.48.32.654a2.112 2.112 0 0 1-.259.153c-.534-2.664-3.284-4.595-6.442-4.595zm7.173 3.876a.565.565 0 0 1-.098.21.704.704 0 0 1-.044-.025c-.146-.09-.157-.175-.152-.223a.236.236 0 0 1 .117-.173c.049-.027.08-.021.113.012a.202.202 0 0 1 .064.199zm-8.999-.65A6.613 6.613 0 0 1 7.964 4.5c.666 0 1.303.097 1.893.273a.5.5 0 1 0 .286-.958A7.601 7.601 0 0 0 7.964 3.5c-.734 0-1.441.103-2.102.292a.5.5 0 1 0 .276.962zM5 6.25a.75.75 0 1 1-1.5 0 .75.75 0 0 1 1.5 0z"/>'),rO=Xu("Pin",'<path d="M4.146.146A.5.5 0 0 1 4.5 0h7a.5.5 0 0 1 .5.5c0 .68-.342 1.174-.646 1.479-.126.125-.25.224-.354.298v4.431l.078.048c.203.127.476.314.751.555C12.36 7.775 13 8.527 13 9.5a.5.5 0 0 1-.5.5h-4v4.5c0 .276-.224 1.5-.5 1.5s-.5-1.224-.5-1.5V10h-4a.5.5 0 0 1-.5-.5c0-.973.64-1.725 1.17-2.189A5.921 5.921 0 0 1 5 6.708V2.277a2.77 2.77 0 0 1-.354-.298C4.342 1.674 4 1.179 4 .5a.5.5 0 0 1 .146-.354zm1.58 1.408-.002-.001.002.001zm-.002-.001.002.001A.5.5 0 0 1 6 2v5a.5.5 0 0 1-.276.447h-.002l-.012.007-.054.03a4.922 4.922 0 0 0-.827.58c-.318.278-.585.596-.725.936h7.792c-.14-.34-.407-.658-.725-.936a4.915 4.915 0 0 0-.881-.61l-.012-.006h-.002A.5.5 0 0 1 10 7V2a.5.5 0 0 1 .295-.458 1.775 1.775 0 0 0 .351-.271c.08-.08.155-.17.214-.271H5.14c.06.1.133.191.214.271a1.78 1.78 0 0 0 .37.282z"/>'),iO=Xu("PinAngle",'<path d="M9.828.722a.5.5 0 0 1 .354.146l4.95 4.95a.5.5 0 0 1 0 .707c-.48.48-1.072.588-1.503.588-.177 0-.335-.018-.46-.039l-3.134 3.134a5.927 5.927 0 0 1 .16 1.013c.046.702-.032 1.687-.72 2.375a.5.5 0 0 1-.707 0l-2.829-2.828-3.182 3.182c-.195.195-1.219.902-1.414.707-.195-.195.512-1.22.707-1.414l3.182-3.182-2.828-2.829a.5.5 0 0 1 0-.707c.688-.688 1.673-.767 2.375-.72a5.922 5.922 0 0 1 1.013.16l3.134-3.133a2.772 2.772 0 0 1-.04-.461c0-.43.108-1.022.589-1.503a.5.5 0 0 1 .353-.146zm.122 2.112v-.002.002zm0-.002v.002a.5.5 0 0 1-.122.51L6.293 6.878a.5.5 0 0 1-.511.12H5.78l-.014-.004a4.507 4.507 0 0 0-.288-.076 4.922 4.922 0 0 0-.765-.116c-.422-.028-.836.008-1.175.15l5.51 5.509c.141-.34.177-.753.149-1.175a4.924 4.924 0 0 0-.192-1.054l-.004-.013v-.001a.5.5 0 0 1 .12-.512l3.536-3.535a.5.5 0 0 1 .532-.115l.096.022c.087.017.208.034.344.034.114 0 .23-.011.343-.04L9.927 2.028c-.029.113-.04.23-.04.343a1.779 1.779 0 0 0 .062.46z"/>'),oO=Xu("PinAngleFill",'<path d="M9.828.722a.5.5 0 0 1 .354.146l4.95 4.95a.5.5 0 0 1 0 .707c-.48.48-1.072.588-1.503.588-.177 0-.335-.018-.46-.039l-3.134 3.134a5.927 5.927 0 0 1 .16 1.013c.046.702-.032 1.687-.72 2.375a.5.5 0 0 1-.707 0l-2.829-2.828-3.182 3.182c-.195.195-1.219.902-1.414.707-.195-.195.512-1.22.707-1.414l3.182-3.182-2.828-2.829a.5.5 0 0 1 0-.707c.688-.688 1.673-.767 2.375-.72a5.922 5.922 0 0 1 1.013.16l3.134-3.133a2.772 2.772 0 0 1-.04-.461c0-.43.108-1.022.589-1.503a.5.5 0 0 1 .353-.146z"/>'),sO=Xu("PinFill",'<path d="M4.146.146A.5.5 0 0 1 4.5 0h7a.5.5 0 0 1 .5.5c0 .68-.342 1.174-.646 1.479-.126.125-.25.224-.354.298v4.431l.078.048c.203.127.476.314.751.555C12.36 7.775 13 8.527 13 9.5a.5.5 0 0 1-.5.5h-4v4.5c0 .276-.224 1.5-.5 1.5s-.5-1.224-.5-1.5V10h-4a.5.5 0 0 1-.5-.5c0-.973.64-1.725 1.17-2.189A5.921 5.921 0 0 1 5 6.708V2.277a2.77 2.77 0 0 1-.354-.298C4.342 1.674 4 1.179 4 .5a.5.5 0 0 1 .146-.354z"/>'),lO=Xu("PinMap",'<path fill-rule="evenodd" d="M3.1 11.2a.5.5 0 0 1 .4-.2H6a.5.5 0 0 1 0 1H3.75L1.5 15h13l-2.25-3H10a.5.5 0 0 1 0-1h2.5a.5.5 0 0 1 .4.2l3 4a.5.5 0 0 1-.4.8H.5a.5.5 0 0 1-.4-.8l3-4z"/><path fill-rule="evenodd" d="M8 1a3 3 0 1 0 0 6 3 3 0 0 0 0-6zM4 4a4 4 0 1 1 4.5 3.969V13.5a.5.5 0 0 1-1 0V7.97A4 4 0 0 1 4 3.999z"/>'),cO=Xu("PinMapFill",'<path fill-rule="evenodd" d="M3.1 11.2a.5.5 0 0 1 .4-.2H6a.5.5 0 0 1 0 1H3.75L1.5 15h13l-2.25-3H10a.5.5 0 0 1 0-1h2.5a.5.5 0 0 1 .4.2l3 4a.5.5 0 0 1-.4.8H.5a.5.5 0 0 1-.4-.8l3-4z"/><path fill-rule="evenodd" d="M4 4a4 4 0 1 1 4.5 3.969V13.5a.5.5 0 0 1-1 0V7.97A4 4 0 0 1 4 3.999z"/>'),uO=Xu("Pip",'<path d="M0 3.5A1.5 1.5 0 0 1 1.5 2h13A1.5 1.5 0 0 1 16 3.5v9a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 12.5v-9zM1.5 3a.5.5 0 0 0-.5.5v9a.5.5 0 0 0 .5.5h13a.5.5 0 0 0 .5-.5v-9a.5.5 0 0 0-.5-.5h-13z"/><path d="M8 8.5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-3z"/>'),dO=Xu("PipFill",'<path d="M1.5 2A1.5 1.5 0 0 0 0 3.5v9A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-9A1.5 1.5 0 0 0 14.5 2h-13zm7 6h5a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-.5.5h-5a.5.5 0 0 1-.5-.5v-3a.5.5 0 0 1 .5-.5z"/>'),hO=Xu("Play",'<path d="M10.804 8 5 4.633v6.734L10.804 8zm.792-.696a.802.802 0 0 1 0 1.392l-6.363 3.692C4.713 12.69 4 12.345 4 11.692V4.308c0-.653.713-.998 1.233-.696l6.363 3.692z"/>'),fO=Xu("PlayBtn",'<path d="M6.79 5.093A.5.5 0 0 0 6 5.5v5a.5.5 0 0 0 .79.407l3.5-2.5a.5.5 0 0 0 0-.814l-3.5-2.5z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),pO=Xu("PlayBtnFill",'<path d="M0 12V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm6.79-6.907A.5.5 0 0 0 6 5.5v5a.5.5 0 0 0 .79.407l3.5-2.5a.5.5 0 0 0 0-.814l-3.5-2.5z"/>'),mO=Xu("PlayCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M6.271 5.055a.5.5 0 0 1 .52.038l3.5 2.5a.5.5 0 0 1 0 .814l-3.5 2.5A.5.5 0 0 1 6 10.5v-5a.5.5 0 0 1 .271-.445z"/>'),vO=Xu("PlayCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM6.79 5.093A.5.5 0 0 0 6 5.5v5a.5.5 0 0 0 .79.407l3.5-2.5a.5.5 0 0 0 0-.814l-3.5-2.5z"/>'),_O=Xu("PlayFill",'<path d="m11.596 8.697-6.363 3.692c-.54.313-1.233-.066-1.233-.697V4.308c0-.63.692-1.01 1.233-.696l6.363 3.692a.802.802 0 0 1 0 1.393z"/>'),gO=Xu("Plug",'<path d="M6 0a.5.5 0 0 1 .5.5V3h3V.5a.5.5 0 0 1 1 0V3h1a.5.5 0 0 1 .5.5v3A3.5 3.5 0 0 1 8.5 10c-.002.434-.01.845-.04 1.22-.041.514-.126 1.003-.317 1.424a2.083 2.083 0 0 1-.97 1.028C6.725 13.9 6.169 14 5.5 14c-.998 0-1.61.33-1.974.718A1.922 1.922 0 0 0 3 16H2c0-.616.232-1.367.797-1.968C3.374 13.42 4.261 13 5.5 13c.581 0 .962-.088 1.218-.219.241-.123.4-.3.514-.55.121-.266.193-.621.23-1.09.027-.34.035-.718.037-1.141A3.5 3.5 0 0 1 4 6.5v-3a.5.5 0 0 1 .5-.5h1V.5A.5.5 0 0 1 6 0zM5 4v2.5A2.5 2.5 0 0 0 7.5 9h1A2.5 2.5 0 0 0 11 6.5V4H5z"/>'),yO=Xu("PlugFill",'<path d="M6 0a.5.5 0 0 1 .5.5V3h3V.5a.5.5 0 0 1 1 0V3h1a.5.5 0 0 1 .5.5v3A3.5 3.5 0 0 1 8.5 10c-.002.434-.01.845-.04 1.22-.041.514-.126 1.003-.317 1.424a2.083 2.083 0 0 1-.97 1.028C6.725 13.9 6.169 14 5.5 14c-.998 0-1.61.33-1.974.718A1.922 1.922 0 0 0 3 16H2c0-.616.232-1.367.797-1.968C3.374 13.42 4.261 13 5.5 13c.581 0 .962-.088 1.218-.219.241-.123.4-.3.514-.55.121-.266.193-.621.23-1.09.027-.34.035-.718.037-1.141A3.5 3.5 0 0 1 4 6.5v-3a.5.5 0 0 1 .5-.5h1V.5A.5.5 0 0 1 6 0z"/>'),bO=Xu("Plus",'<path d="M8 4a.5.5 0 0 1 .5.5v3h3a.5.5 0 0 1 0 1h-3v3a.5.5 0 0 1-1 0v-3h-3a.5.5 0 0 1 0-1h3v-3A.5.5 0 0 1 8 4z"/>'),MO=Xu("PlusCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M8 4a.5.5 0 0 1 .5.5v3h3a.5.5 0 0 1 0 1h-3v3a.5.5 0 0 1-1 0v-3h-3a.5.5 0 0 1 0-1h3v-3A.5.5 0 0 1 8 4z"/>'),zO=Xu("PlusCircleDotted",'<path d="M8 0c-.176 0-.35.006-.523.017l.064.998a7.117 7.117 0 0 1 .918 0l.064-.998A8.113 8.113 0 0 0 8 0zM6.44.152c-.346.069-.684.16-1.012.27l.321.948c.287-.098.582-.177.884-.237L6.44.153zm4.132.271a7.946 7.946 0 0 0-1.011-.27l-.194.98c.302.06.597.14.884.237l.321-.947zm1.873.925a8 8 0 0 0-.906-.524l-.443.896c.275.136.54.29.793.459l.556-.831zM4.46.824c-.314.155-.616.33-.905.524l.556.83a7.07 7.07 0 0 1 .793-.458L4.46.824zM2.725 1.985c-.262.23-.51.478-.74.74l.752.66c.202-.23.418-.446.648-.648l-.66-.752zm11.29.74a8.058 8.058 0 0 0-.74-.74l-.66.752c.23.202.447.418.648.648l.752-.66zm1.161 1.735a7.98 7.98 0 0 0-.524-.905l-.83.556c.169.253.322.518.458.793l.896-.443zM1.348 3.555c-.194.289-.37.591-.524.906l.896.443c.136-.275.29-.54.459-.793l-.831-.556zM.423 5.428a7.945 7.945 0 0 0-.27 1.011l.98.194c.06-.302.14-.597.237-.884l-.947-.321zM15.848 6.44a7.943 7.943 0 0 0-.27-1.012l-.948.321c.098.287.177.582.237.884l.98-.194zM.017 7.477a8.113 8.113 0 0 0 0 1.046l.998-.064a7.117 7.117 0 0 1 0-.918l-.998-.064zM16 8a8.1 8.1 0 0 0-.017-.523l-.998.064a7.11 7.11 0 0 1 0 .918l.998.064A8.1 8.1 0 0 0 16 8zM.152 9.56c.069.346.16.684.27 1.012l.948-.321a6.944 6.944 0 0 1-.237-.884l-.98.194zm15.425 1.012c.112-.328.202-.666.27-1.011l-.98-.194c-.06.302-.14.597-.237.884l.947.321zM.824 11.54a8 8 0 0 0 .524.905l.83-.556a6.999 6.999 0 0 1-.458-.793l-.896.443zm13.828.905c.194-.289.37-.591.524-.906l-.896-.443c-.136.275-.29.54-.459.793l.831.556zm-12.667.83c.23.262.478.51.74.74l.66-.752a7.047 7.047 0 0 1-.648-.648l-.752.66zm11.29.74c.262-.23.51-.478.74-.74l-.752-.66c-.201.23-.418.447-.648.648l.66.752zm-1.735 1.161c.314-.155.616-.33.905-.524l-.556-.83a7.07 7.07 0 0 1-.793.458l.443.896zm-7.985-.524c.289.194.591.37.906.524l.443-.896a6.998 6.998 0 0 1-.793-.459l-.556.831zm1.873.925c.328.112.666.202 1.011.27l.194-.98a6.953 6.953 0 0 1-.884-.237l-.321.947zm4.132.271a7.944 7.944 0 0 0 1.012-.27l-.321-.948a6.954 6.954 0 0 1-.884.237l.194.98zm-2.083.135a8.1 8.1 0 0 0 1.046 0l-.064-.998a7.11 7.11 0 0 1-.918 0l-.064.998zM8.5 4.5a.5.5 0 0 0-1 0v3h-3a.5.5 0 0 0 0 1h3v3a.5.5 0 0 0 1 0v-3h3a.5.5 0 0 0 0-1h-3v-3z"/>'),LO=Xu("PlusCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM8.5 4.5a.5.5 0 0 0-1 0v3h-3a.5.5 0 0 0 0 1h3v3a.5.5 0 0 0 1 0v-3h3a.5.5 0 0 0 0-1h-3v-3z"/>'),wO=Xu("PlusLg",'<path d="M8 0a1 1 0 0 1 1 1v6h6a1 1 0 1 1 0 2H9v6a1 1 0 1 1-2 0V9H1a1 1 0 0 1 0-2h6V1a1 1 0 0 1 1-1z"/>'),HO=Xu("PlusSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M8 4a.5.5 0 0 1 .5.5v3h3a.5.5 0 0 1 0 1h-3v3a.5.5 0 0 1-1 0v-3h-3a.5.5 0 0 1 0-1h3v-3A.5.5 0 0 1 8 4z"/>'),kO=Xu("PlusSquareDotted",'<path d="M2.5 0c-.166 0-.33.016-.487.048l.194.98A1.51 1.51 0 0 1 2.5 1h.458V0H2.5zm2.292 0h-.917v1h.917V0zm1.833 0h-.917v1h.917V0zm1.833 0h-.916v1h.916V0zm1.834 0h-.917v1h.917V0zm1.833 0h-.917v1h.917V0zM13.5 0h-.458v1h.458c.1 0 .199.01.293.029l.194-.981A2.51 2.51 0 0 0 13.5 0zm2.079 1.11a2.511 2.511 0 0 0-.69-.689l-.556.831c.164.11.305.251.415.415l.83-.556zM1.11.421a2.511 2.511 0 0 0-.689.69l.831.556c.11-.164.251-.305.415-.415L1.11.422zM16 2.5c0-.166-.016-.33-.048-.487l-.98.194c.018.094.028.192.028.293v.458h1V2.5zM.048 2.013A2.51 2.51 0 0 0 0 2.5v.458h1V2.5c0-.1.01-.199.029-.293l-.981-.194zM0 3.875v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zM0 5.708v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zM0 7.542v.916h1v-.916H0zm15 .916h1v-.916h-1v.916zM0 9.375v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zm-16 .916v.917h1v-.917H0zm16 .917v-.917h-1v.917h1zm-16 .917v.458c0 .166.016.33.048.487l.98-.194A1.51 1.51 0 0 1 1 13.5v-.458H0zm16 .458v-.458h-1v.458c0 .1-.01.199-.029.293l.981.194c.032-.158.048-.32.048-.487zM.421 14.89c.183.272.417.506.69.689l.556-.831a1.51 1.51 0 0 1-.415-.415l-.83.556zm14.469.689c.272-.183.506-.417.689-.69l-.831-.556c-.11.164-.251.305-.415.415l.556.83zm-12.877.373c.158.032.32.048.487.048h.458v-1H2.5c-.1 0-.199-.01-.293-.029l-.194.981zM13.5 16c.166 0 .33-.016.487-.048l-.194-.98A1.51 1.51 0 0 1 13.5 15h-.458v1h.458zm-9.625 0h.917v-1h-.917v1zm1.833 0h.917v-1h-.917v1zm1.834-1v1h.916v-1h-.916zm1.833 1h.917v-1h-.917v1zm1.833 0h.917v-1h-.917v1zM8.5 4.5a.5.5 0 0 0-1 0v3h-3a.5.5 0 0 0 0 1h3v3a.5.5 0 0 0 1 0v-3h3a.5.5 0 0 0 0-1h-3v-3z"/>'),AO=Xu("PlusSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm6.5 4.5v3h3a.5.5 0 0 1 0 1h-3v3a.5.5 0 0 1-1 0v-3h-3a.5.5 0 0 1 0-1h3v-3a.5.5 0 0 1 1 0z"/>'),SO=Xu("Power",'<path d="M7.5 1v7h1V1h-1z"/><path d="M3 8.812a4.999 4.999 0 0 1 2.578-4.375l-.485-.874A6 6 0 1 0 11 3.616l-.501.865A5 5 0 1 1 3 8.812z"/>'),VO=Xu("Printer",'<path d="M2.5 8a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1z"/><path d="M5 1a2 2 0 0 0-2 2v2H2a2 2 0 0 0-2 2v3a2 2 0 0 0 2 2h1v1a2 2 0 0 0 2 2h6a2 2 0 0 0 2-2v-1h1a2 2 0 0 0 2-2V7a2 2 0 0 0-2-2h-1V3a2 2 0 0 0-2-2H5zM4 3a1 1 0 0 1 1-1h6a1 1 0 0 1 1 1v2H4V3zm1 5a2 2 0 0 0-2 2v1H2a1 1 0 0 1-1-1V7a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1h-1v-1a2 2 0 0 0-2-2H5zm7 2v3a1 1 0 0 1-1 1H5a1 1 0 0 1-1-1v-3a1 1 0 0 1 1-1h6a1 1 0 0 1 1 1z"/>'),OO=Xu("PrinterFill",'<path d="M5 1a2 2 0 0 0-2 2v1h10V3a2 2 0 0 0-2-2H5zm6 8H5a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h6a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1z"/><path d="M0 7a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v3a2 2 0 0 1-2 2h-1v-2a2 2 0 0 0-2-2H5a2 2 0 0 0-2 2v2H2a2 2 0 0 1-2-2V7zm2.5 1a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1z"/>'),TO=Xu("Puzzle",'<path d="M3.112 3.645A1.5 1.5 0 0 1 4.605 2H7a.5.5 0 0 1 .5.5v.382c0 .696-.497 1.182-.872 1.469a.459.459 0 0 0-.115.118.113.113 0 0 0-.012.025L6.5 4.5v.003l.003.01c.004.01.014.028.036.053a.86.86 0 0 0 .27.194C7.09 4.9 7.51 5 8 5c.492 0 .912-.1 1.19-.24a.86.86 0 0 0 .271-.194.213.213 0 0 0 .039-.063v-.009a.112.112 0 0 0-.012-.025.459.459 0 0 0-.115-.118c-.375-.287-.872-.773-.872-1.469V2.5A.5.5 0 0 1 9 2h2.395a1.5 1.5 0 0 1 1.493 1.645L12.645 6.5h.237c.195 0 .42-.147.675-.48.21-.274.528-.52.943-.52.568 0 .947.447 1.154.862C15.877 6.807 16 7.387 16 8s-.123 1.193-.346 1.638c-.207.415-.586.862-1.154.862-.415 0-.733-.246-.943-.52-.255-.333-.48-.48-.675-.48h-.237l.243 2.855A1.5 1.5 0 0 1 11.395 14H9a.5.5 0 0 1-.5-.5v-.382c0-.696.497-1.182.872-1.469a.459.459 0 0 0 .115-.118.113.113 0 0 0 .012-.025L9.5 11.5v-.003a.214.214 0 0 0-.039-.064.859.859 0 0 0-.27-.193C8.91 11.1 8.49 11 8 11c-.491 0-.912.1-1.19.24a.859.859 0 0 0-.271.194.214.214 0 0 0-.039.063v.003l.001.006a.113.113 0 0 0 .012.025c.016.027.05.068.115.118.375.287.872.773.872 1.469v.382a.5.5 0 0 1-.5.5H4.605a1.5 1.5 0 0 1-1.493-1.645L3.356 9.5h-.238c-.195 0-.42.147-.675.48-.21.274-.528.52-.943.52-.568 0-.947-.447-1.154-.862C.123 9.193 0 8.613 0 8s.123-1.193.346-1.638C.553 5.947.932 5.5 1.5 5.5c.415 0 .733.246.943.52.255.333.48.48.675.48h.238l-.244-2.855zM4.605 3a.5.5 0 0 0-.498.55l.001.007.29 3.4A.5.5 0 0 1 3.9 7.5h-.782c-.696 0-1.182-.497-1.469-.872a.459.459 0 0 0-.118-.115.112.112 0 0 0-.025-.012L1.5 6.5h-.003a.213.213 0 0 0-.064.039.86.86 0 0 0-.193.27C1.1 7.09 1 7.51 1 8c0 .491.1.912.24 1.19.07.14.14.225.194.271a.213.213 0 0 0 .063.039H1.5l.006-.001a.112.112 0 0 0 .025-.012.459.459 0 0 0 .118-.115c.287-.375.773-.872 1.469-.872H3.9a.5.5 0 0 1 .498.542l-.29 3.408a.5.5 0 0 0 .497.55h1.878c-.048-.166-.195-.352-.463-.557-.274-.21-.52-.528-.52-.943 0-.568.447-.947.862-1.154C6.807 10.123 7.387 10 8 10s1.193.123 1.638.346c.415.207.862.586.862 1.154 0 .415-.246.733-.52.943-.268.205-.415.39-.463.557h1.878a.5.5 0 0 0 .498-.55l-.001-.007-.29-3.4A.5.5 0 0 1 12.1 8.5h.782c.696 0 1.182.497 1.469.872.05.065.091.099.118.115.013.008.021.01.025.012a.02.02 0 0 0 .006.001h.003a.214.214 0 0 0 .064-.039.86.86 0 0 0 .193-.27c.14-.28.24-.7.24-1.191 0-.492-.1-.912-.24-1.19a.86.86 0 0 0-.194-.271.215.215 0 0 0-.063-.039H14.5l-.006.001a.113.113 0 0 0-.025.012.459.459 0 0 0-.118.115c-.287.375-.773.872-1.469.872H12.1a.5.5 0 0 1-.498-.543l.29-3.407a.5.5 0 0 0-.497-.55H9.517c.048.166.195.352.463.557.274.21.52.528.52.943 0 .568-.447.947-.862 1.154C9.193 5.877 8.613 6 8 6s-1.193-.123-1.638-.346C5.947 5.447 5.5 5.068 5.5 4.5c0-.415.246-.733.52-.943.268-.205.415-.39.463-.557H4.605z"/>'),DO=Xu("PuzzleFill",'<path d="M3.112 3.645A1.5 1.5 0 0 1 4.605 2H7a.5.5 0 0 1 .5.5v.382c0 .696-.497 1.182-.872 1.469a.459.459 0 0 0-.115.118.113.113 0 0 0-.012.025L6.5 4.5v.003l.003.01c.004.01.014.028.036.053a.86.86 0 0 0 .27.194C7.09 4.9 7.51 5 8 5c.492 0 .912-.1 1.19-.24a.86.86 0 0 0 .271-.194.213.213 0 0 0 .036-.054l.003-.01v-.008a.112.112 0 0 0-.012-.025.459.459 0 0 0-.115-.118c-.375-.287-.872-.773-.872-1.469V2.5A.5.5 0 0 1 9 2h2.395a1.5 1.5 0 0 1 1.493 1.645L12.645 6.5h.237c.195 0 .42-.147.675-.48.21-.274.528-.52.943-.52.568 0 .947.447 1.154.862C15.877 6.807 16 7.387 16 8s-.123 1.193-.346 1.638c-.207.415-.586.862-1.154.862-.415 0-.733-.246-.943-.52-.255-.333-.48-.48-.675-.48h-.237l.243 2.855A1.5 1.5 0 0 1 11.395 14H9a.5.5 0 0 1-.5-.5v-.382c0-.696.497-1.182.872-1.469a.459.459 0 0 0 .115-.118.113.113 0 0 0 .012-.025L9.5 11.5v-.003l-.003-.01a.214.214 0 0 0-.036-.053.859.859 0 0 0-.27-.194C8.91 11.1 8.49 11 8 11c-.491 0-.912.1-1.19.24a.859.859 0 0 0-.271.194.214.214 0 0 0-.036.054l-.003.01v.002l.001.006a.113.113 0 0 0 .012.025c.016.027.05.068.115.118.375.287.872.773.872 1.469v.382a.5.5 0 0 1-.5.5H4.605a1.5 1.5 0 0 1-1.493-1.645L3.356 9.5h-.238c-.195 0-.42.147-.675.48-.21.274-.528.52-.943.52-.568 0-.947-.447-1.154-.862C.123 9.193 0 8.613 0 8s.123-1.193.346-1.638C.553 5.947.932 5.5 1.5 5.5c.415 0 .733.246.943.52.255.333.48.48.675.48h.238l-.244-2.855z"/>'),CO=Xu("Question",'<path d="M5.255 5.786a.237.237 0 0 0 .241.247h.825c.138 0 .248-.113.266-.25.09-.656.54-1.134 1.342-1.134.686 0 1.314.343 1.314 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.003.217a.25.25 0 0 0 .25.246h.811a.25.25 0 0 0 .25-.25v-.105c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.267 0-2.655.59-2.75 2.286zm1.557 5.763c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"/>'),jO=Xu("QuestionCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M5.255 5.786a.237.237 0 0 0 .241.247h.825c.138 0 .248-.113.266-.25.09-.656.54-1.134 1.342-1.134.686 0 1.314.343 1.314 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.003.217a.25.25 0 0 0 .25.246h.811a.25.25 0 0 0 .25-.25v-.105c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.267 0-2.655.59-2.75 2.286zm1.557 5.763c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"/>'),xO=Xu("QuestionCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM5.496 6.033h.825c.138 0 .248-.113.266-.25.09-.656.54-1.134 1.342-1.134.686 0 1.314.343 1.314 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.003.217a.25.25 0 0 0 .25.246h.811a.25.25 0 0 0 .25-.25v-.105c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.267 0-2.655.59-2.75 2.286a.237.237 0 0 0 .241.247zm2.325 6.443c.61 0 1.029-.394 1.029-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94 0 .533.425.927 1.01.927z"/>'),BO=Xu("QuestionDiamond",'<path d="M6.95.435c.58-.58 1.52-.58 2.1 0l6.515 6.516c.58.58.58 1.519 0 2.098L9.05 15.565c-.58.58-1.519.58-2.098 0L.435 9.05a1.482 1.482 0 0 1 0-2.098L6.95.435zm1.4.7a.495.495 0 0 0-.7 0L1.134 7.65a.495.495 0 0 0 0 .7l6.516 6.516a.495.495 0 0 0 .7 0l6.516-6.516a.495.495 0 0 0 0-.7L8.35 1.134z"/><path d="M5.255 5.786a.237.237 0 0 0 .241.247h.825c.138 0 .248-.113.266-.25.09-.656.54-1.134 1.342-1.134.686 0 1.314.343 1.314 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.003.217a.25.25 0 0 0 .25.246h.811a.25.25 0 0 0 .25-.25v-.105c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.267 0-2.655.59-2.75 2.286zm1.557 5.763c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"/>'),YO=Xu("QuestionDiamondFill",'<path d="M9.05.435c-.58-.58-1.52-.58-2.1 0L.436 6.95c-.58.58-.58 1.519 0 2.098l6.516 6.516c.58.58 1.519.58 2.098 0l6.516-6.516c.58-.58.58-1.519 0-2.098L9.05.435zM5.495 6.033a.237.237 0 0 1-.24-.247C5.35 4.091 6.737 3.5 8.005 3.5c1.396 0 2.672.73 2.672 2.24 0 1.08-.635 1.594-1.244 2.057-.737.559-1.01.768-1.01 1.486v.105a.25.25 0 0 1-.25.25h-.81a.25.25 0 0 1-.25-.246l-.004-.217c-.038-.927.495-1.498 1.168-1.987.59-.444.965-.736.965-1.371 0-.825-.628-1.168-1.314-1.168-.803 0-1.253.478-1.342 1.134-.018.137-.128.25-.266.25h-.825zm2.325 6.443c-.584 0-1.009-.394-1.009-.927 0-.552.425-.94 1.01-.94.609 0 1.028.388 1.028.94 0 .533-.42.927-1.029.927z"/>'),PO=Xu("QuestionLg",'<path d="M3 4.075a.423.423 0 0 0 .43.44H4.9c.247 0 .442-.2.475-.445.159-1.17.962-2.022 2.393-2.022 1.222 0 2.342.611 2.342 2.082 0 1.132-.668 1.652-1.72 2.444-1.2.872-2.15 1.89-2.082 3.542l.005.386c.003.244.202.44.446.44h1.445c.247 0 .446-.2.446-.446v-.188c0-1.278.487-1.652 1.8-2.647 1.086-.826 2.217-1.743 2.217-3.667C12.667 1.301 10.393 0 7.903 0 5.645 0 3.17 1.053 3.001 4.075zm2.776 10.273c0 .95.758 1.652 1.8 1.652 1.085 0 1.832-.702 1.832-1.652 0-.985-.747-1.675-1.833-1.675-1.04 0-1.799.69-1.799 1.675z"/>'),IO=Xu("QuestionOctagon",'<path d="M4.54.146A.5.5 0 0 1 4.893 0h6.214a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353L4.54.146zM5.1 1 1 5.1v5.8L5.1 15h5.8l4.1-4.1V5.1L10.9 1H5.1z"/><path d="M5.255 5.786a.237.237 0 0 0 .241.247h.825c.138 0 .248-.113.266-.25.09-.656.54-1.134 1.342-1.134.686 0 1.314.343 1.314 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.003.217a.25.25 0 0 0 .25.246h.811a.25.25 0 0 0 .25-.25v-.105c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.267 0-2.655.59-2.75 2.286zm1.557 5.763c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"/>'),FO=Xu("QuestionOctagonFill",'<path d="M11.46.146A.5.5 0 0 0 11.107 0H4.893a.5.5 0 0 0-.353.146L.146 4.54A.5.5 0 0 0 0 4.893v6.214a.5.5 0 0 0 .146.353l4.394 4.394a.5.5 0 0 0 .353.146h6.214a.5.5 0 0 0 .353-.146l4.394-4.394a.5.5 0 0 0 .146-.353V4.893a.5.5 0 0 0-.146-.353L11.46.146zM5.496 6.033a.237.237 0 0 1-.24-.247C5.35 4.091 6.737 3.5 8.005 3.5c1.396 0 2.672.73 2.672 2.24 0 1.08-.635 1.594-1.244 2.057-.737.559-1.01.768-1.01 1.486v.105a.25.25 0 0 1-.25.25h-.81a.25.25 0 0 1-.25-.246l-.004-.217c-.038-.927.495-1.498 1.168-1.987.59-.444.965-.736.965-1.371 0-.825-.628-1.168-1.314-1.168-.803 0-1.253.478-1.342 1.134-.018.137-.128.25-.266.25h-.825zm2.325 6.443c-.584 0-1.009-.394-1.009-.927 0-.552.425-.94 1.01-.94.609 0 1.028.388 1.028.94 0 .533-.42.927-1.029.927z"/>'),EO=Xu("QuestionSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M5.255 5.786a.237.237 0 0 0 .241.247h.825c.138 0 .248-.113.266-.25.09-.656.54-1.134 1.342-1.134.686 0 1.314.343 1.314 1.168 0 .635-.374.927-.965 1.371-.673.489-1.206 1.06-1.168 1.987l.003.217a.25.25 0 0 0 .25.246h.811a.25.25 0 0 0 .25-.25v-.105c0-.718.273-.927 1.01-1.486.609-.463 1.244-.977 1.244-2.056 0-1.511-1.276-2.241-2.673-2.241-1.267 0-2.655.59-2.75 2.286zm1.557 5.763c0 .533.425.927 1.01.927.609 0 1.028-.394 1.028-.927 0-.552-.42-.94-1.029-.94-.584 0-1.009.388-1.009.94z"/>'),$O=Xu("QuestionSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm3.496 6.033a.237.237 0 0 1-.24-.247C5.35 4.091 6.737 3.5 8.005 3.5c1.396 0 2.672.73 2.672 2.24 0 1.08-.635 1.594-1.244 2.057-.737.559-1.01.768-1.01 1.486v.105a.25.25 0 0 1-.25.25h-.81a.25.25 0 0 1-.25-.246l-.004-.217c-.038-.927.495-1.498 1.168-1.987.59-.444.965-.736.965-1.371 0-.825-.628-1.168-1.314-1.168-.803 0-1.253.478-1.342 1.134-.018.137-.128.25-.266.25h-.825zm2.325 6.443c-.584 0-1.009-.394-1.009-.927 0-.552.425-.94 1.01-.94.609 0 1.028.388 1.028.94 0 .533-.42.927-1.029.927z"/>'),RO=Xu("Rainbow",'<path d="M8 4.5a7 7 0 0 0-7 7 .5.5 0 0 1-1 0 8 8 0 1 1 16 0 .5.5 0 0 1-1 0 7 7 0 0 0-7-7zm0 2a5 5 0 0 0-5 5 .5.5 0 0 1-1 0 6 6 0 1 1 12 0 .5.5 0 0 1-1 0 5 5 0 0 0-5-5zm0 2a3 3 0 0 0-3 3 .5.5 0 0 1-1 0 4 4 0 1 1 8 0 .5.5 0 0 1-1 0 3 3 0 0 0-3-3zm0 2a1 1 0 0 0-1 1 .5.5 0 0 1-1 0 2 2 0 1 1 4 0 .5.5 0 0 1-1 0 1 1 0 0 0-1-1z"/>'),NO=Xu("Receipt",'<path d="M1.92.506a.5.5 0 0 1 .434.14L3 1.293l.646-.647a.5.5 0 0 1 .708 0L5 1.293l.646-.647a.5.5 0 0 1 .708 0L7 1.293l.646-.647a.5.5 0 0 1 .708 0L9 1.293l.646-.647a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 0 1 .708 0l.646.647.646-.647a.5.5 0 0 1 .801.13l.5 1A.5.5 0 0 1 15 2v12a.5.5 0 0 1-.053.224l-.5 1a.5.5 0 0 1-.8.13L13 14.707l-.646.647a.5.5 0 0 1-.708 0L11 14.707l-.646.647a.5.5 0 0 1-.708 0L9 14.707l-.646.647a.5.5 0 0 1-.708 0L7 14.707l-.646.647a.5.5 0 0 1-.708 0L5 14.707l-.646.647a.5.5 0 0 1-.708 0L3 14.707l-.646.647a.5.5 0 0 1-.801-.13l-.5-1A.5.5 0 0 1 1 14V2a.5.5 0 0 1 .053-.224l.5-1a.5.5 0 0 1 .367-.27zm.217 1.338L2 2.118v11.764l.137.274.51-.51a.5.5 0 0 1 .707 0l.646.647.646-.646a.5.5 0 0 1 .708 0l.646.646.646-.646a.5.5 0 0 1 .708 0l.646.646.646-.646a.5.5 0 0 1 .708 0l.646.646.646-.646a.5.5 0 0 1 .708 0l.646.646.646-.646a.5.5 0 0 1 .708 0l.509.509.137-.274V2.118l-.137-.274-.51.51a.5.5 0 0 1-.707 0L12 1.707l-.646.647a.5.5 0 0 1-.708 0L10 1.707l-.646.647a.5.5 0 0 1-.708 0L8 1.707l-.646.647a.5.5 0 0 1-.708 0L6 1.707l-.646.647a.5.5 0 0 1-.708 0L4 1.707l-.646.647a.5.5 0 0 1-.708 0l-.509-.51z"/><path d="M3 4.5a.5.5 0 0 1 .5-.5h6a.5.5 0 1 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 1 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 1 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zm8-6a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 0 1h-1a.5.5 0 0 1-.5-.5z"/>'),WO=Xu("ReceiptCutoff",'<path d="M3 4.5a.5.5 0 0 1 .5-.5h6a.5.5 0 1 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 1 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 1 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 2a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zM11.5 4a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm0 2a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm0 2a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm0 2a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1zm0 2a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1z"/><path d="M2.354.646a.5.5 0 0 0-.801.13l-.5 1A.5.5 0 0 0 1 2v13H.5a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1H15V2a.5.5 0 0 0-.053-.224l-.5-1a.5.5 0 0 0-.8-.13L13 1.293l-.646-.647a.5.5 0 0 0-.708 0L11 1.293l-.646-.647a.5.5 0 0 0-.708 0L9 1.293 8.354.646a.5.5 0 0 0-.708 0L7 1.293 6.354.646a.5.5 0 0 0-.708 0L5 1.293 4.354.646a.5.5 0 0 0-.708 0L3 1.293 2.354.646zm-.217 1.198.51.51a.5.5 0 0 0 .707 0L4 1.707l.646.647a.5.5 0 0 0 .708 0L6 1.707l.646.647a.5.5 0 0 0 .708 0L8 1.707l.646.647a.5.5 0 0 0 .708 0L10 1.707l.646.647a.5.5 0 0 0 .708 0L12 1.707l.646.647a.5.5 0 0 0 .708 0l.509-.51.137.274V15H2V2.118l.137-.274z"/>'),UO=Xu("Reception0",'<path d="M0 13.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm4 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm4 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm4 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),GO=Xu("Reception1",'<path d="M0 11.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-2zm4 2a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm4 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm4 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),qO=Xu("Reception2",'<path d="M0 11.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-2zm4-3a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v5a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-5zm4 5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5zm4 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),JO=Xu("Reception3",'<path d="M0 11.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-2zm4-3a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v5a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-5zm4-3a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-8zm4 8a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),KO=Xu("Reception4",'<path d="M0 11.5a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-2zm4-3a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v5a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-5zm4-3a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-8zm4-3a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 .5.5v11a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1-.5-.5v-11z"/>'),XO=Xu("Record",'<path d="M8 12a4 4 0 1 1 0-8 4 4 0 0 1 0 8zm0 1A5 5 0 1 0 8 3a5 5 0 0 0 0 10z"/>'),ZO=Xu("Record2",'<path d="M8 12a4 4 0 1 1 0-8 4 4 0 0 1 0 8zm0 1A5 5 0 1 0 8 3a5 5 0 0 0 0 10z"/><path d="M10 8a2 2 0 1 1-4 0 2 2 0 0 1 4 0z"/>'),QO=Xu("Record2Fill",'<path d="M10 8a2 2 0 1 1-4 0 2 2 0 0 1 4 0z"/><path d="M8 13A5 5 0 1 0 8 3a5 5 0 0 0 0 10zm0-2a3 3 0 1 1 0-6 3 3 0 0 1 0 6z"/>'),eT=Xu("RecordBtn",'<path d="M8 11a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),tT=Xu("RecordBtnFill",'<path d="M0 12V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm8-1a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/>'),aT=Xu("RecordCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M11 8a3 3 0 1 1-6 0 3 3 0 0 1 6 0z"/>'),nT=Xu("RecordCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-8 3a3 3 0 1 0 0-6 3 3 0 0 0 0 6z"/>'),rT=Xu("RecordFill",'<path fill-rule="evenodd" d="M8 13A5 5 0 1 0 8 3a5 5 0 0 0 0 10z"/>'),iT=Xu("Recycle",'<path d="M9.302 1.256a1.5 1.5 0 0 0-2.604 0l-1.704 2.98a.5.5 0 0 0 .869.497l1.703-2.981a.5.5 0 0 1 .868 0l2.54 4.444-1.256-.337a.5.5 0 1 0-.26.966l2.415.647a.5.5 0 0 0 .613-.353l.647-2.415a.5.5 0 1 0-.966-.259l-.333 1.242-2.532-4.431zM2.973 7.773l-1.255.337a.5.5 0 1 1-.26-.966l2.416-.647a.5.5 0 0 1 .612.353l.647 2.415a.5.5 0 0 1-.966.259l-.333-1.242-2.545 4.454a.5.5 0 0 0 .434.748H5a.5.5 0 0 1 0 1H1.723A1.5 1.5 0 0 1 .421 12.24l2.552-4.467zm10.89 1.463a.5.5 0 1 0-.868.496l1.716 3.004a.5.5 0 0 1-.434.748h-5.57l.647-.646a.5.5 0 1 0-.708-.707l-1.5 1.5a.498.498 0 0 0 0 .707l1.5 1.5a.5.5 0 1 0 .708-.707l-.647-.647h5.57a1.5 1.5 0 0 0 1.302-2.244l-1.716-3.004z"/>'),oT=Xu("Reddit",'<path d="M6.167 8a.831.831 0 0 0-.83.83c0 .459.372.84.83.831a.831.831 0 0 0 0-1.661zm1.843 3.647c.315 0 1.403-.038 1.976-.611a.232.232 0 0 0 0-.306.213.213 0 0 0-.306 0c-.353.363-1.126.487-1.67.487-.545 0-1.308-.124-1.671-.487a.213.213 0 0 0-.306 0 .213.213 0 0 0 0 .306c.564.563 1.652.61 1.977.61zm.992-2.807c0 .458.373.83.831.83.458 0 .83-.381.83-.83a.831.831 0 0 0-1.66 0z"/><path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-3.828-1.165c-.315 0-.602.124-.812.325-.801-.573-1.9-.945-3.121-.993l.534-2.501 1.738.372a.83.83 0 1 0 .83-.869.83.83 0 0 0-.744.468l-1.938-.41a.203.203 0 0 0-.153.028.186.186 0 0 0-.086.134l-.592 2.788c-1.24.038-2.358.41-3.17.992-.21-.2-.496-.324-.81-.324a1.163 1.163 0 0 0-.478 2.224c-.02.115-.029.23-.029.353 0 1.795 2.091 3.256 4.669 3.256 2.577 0 4.668-1.451 4.668-3.256 0-.114-.01-.238-.029-.353.401-.181.688-.592.688-1.069 0-.65-.525-1.165-1.165-1.165z"/>'),sT=Xu("Reply",'<path d="M6.598 5.013a.144.144 0 0 1 .202.134V6.3a.5.5 0 0 0 .5.5c.667 0 2.013.005 3.3.822.984.624 1.99 1.76 2.595 3.876-1.02-.983-2.185-1.516-3.205-1.799a8.74 8.74 0 0 0-1.921-.306 7.404 7.404 0 0 0-.798.008h-.013l-.005.001h-.001L7.3 9.9l-.05-.498a.5.5 0 0 0-.45.498v1.153c0 .108-.11.176-.202.134L2.614 8.254a.503.503 0 0 0-.042-.028.147.147 0 0 1 0-.252.499.499 0 0 0 .042-.028l3.984-2.933zM7.8 10.386c.068 0 .143.003.223.006.434.02 1.034.086 1.7.271 1.326.368 2.896 1.202 3.94 3.08a.5.5 0 0 0 .933-.305c-.464-3.71-1.886-5.662-3.46-6.66-1.245-.79-2.527-.942-3.336-.971v-.66a1.144 1.144 0 0 0-1.767-.96l-3.994 2.94a1.147 1.147 0 0 0 0 1.946l3.994 2.94a1.144 1.144 0 0 0 1.767-.96v-.667z"/>'),lT=Xu("ReplyAll",'<path d="M8.098 5.013a.144.144 0 0 1 .202.134V6.3a.5.5 0 0 0 .5.5c.667 0 2.013.005 3.3.822.984.624 1.99 1.76 2.595 3.876-1.02-.983-2.185-1.516-3.205-1.799a8.74 8.74 0 0 0-1.921-.306 7.404 7.404 0 0 0-.798.008h-.013l-.005.001h-.001L8.8 9.9l-.05-.498a.5.5 0 0 0-.45.498v1.153c0 .108-.11.176-.202.134L4.114 8.254a.502.502 0 0 0-.042-.028.147.147 0 0 1 0-.252.497.497 0 0 0 .042-.028l3.984-2.933zM9.3 10.386c.068 0 .143.003.223.006.434.02 1.034.086 1.7.271 1.326.368 2.896 1.202 3.94 3.08a.5.5 0 0 0 .933-.305c-.464-3.71-1.886-5.662-3.46-6.66-1.245-.79-2.527-.942-3.336-.971v-.66a1.144 1.144 0 0 0-1.767-.96l-3.994 2.94a1.147 1.147 0 0 0 0 1.946l3.994 2.94a1.144 1.144 0 0 0 1.767-.96v-.667z"/><path d="M5.232 4.293a.5.5 0 0 0-.7-.106L.54 7.127a1.147 1.147 0 0 0 0 1.946l3.994 2.94a.5.5 0 1 0 .593-.805L1.114 8.254a.503.503 0 0 0-.042-.028.147.147 0 0 1 0-.252.5.5 0 0 0 .042-.028l4.012-2.954a.5.5 0 0 0 .106-.699z"/>'),cT=Xu("ReplyAllFill",'<path d="M8.021 11.9 3.453 8.62a.719.719 0 0 1 0-1.238L8.021 4.1a.716.716 0 0 1 1.079.619V6c1.5 0 6 0 7 8-2.5-4.5-7-4-7-4v1.281c0 .56-.606.898-1.079.62z"/><path d="M5.232 4.293a.5.5 0 0 1-.106.7L1.114 7.945a.5.5 0 0 1-.042.028.147.147 0 0 0 0 .252.503.503 0 0 1 .042.028l4.012 2.954a.5.5 0 1 1-.593.805L.539 9.073a1.147 1.147 0 0 1 0-1.946l3.994-2.94a.5.5 0 0 1 .699.106z"/>'),uT=Xu("ReplyFill",'<path d="M5.921 11.9 1.353 8.62a.719.719 0 0 1 0-1.238L5.921 4.1A.716.716 0 0 1 7 4.719V6c1.5 0 6 0 7 8-2.5-4.5-7-4-7-4v1.281c0 .56-.606.898-1.079.62z"/>'),dT=Xu("Rss",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M5.5 12a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm-3-8.5a1 1 0 0 1 1-1c5.523 0 10 4.477 10 10a1 1 0 1 1-2 0 8 8 0 0 0-8-8 1 1 0 0 1-1-1zm0 4a1 1 0 0 1 1-1 6 6 0 0 1 6 6 1 1 0 1 1-2 0 4 4 0 0 0-4-4 1 1 0 0 1-1-1z"/>'),hT=Xu("RssFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm1.5 2.5c5.523 0 10 4.477 10 10a1 1 0 1 1-2 0 8 8 0 0 0-8-8 1 1 0 0 1 0-2zm0 4a6 6 0 0 1 6 6 1 1 0 1 1-2 0 4 4 0 0 0-4-4 1 1 0 0 1 0-2zm.5 7a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),fT=Xu("Rulers",'<path d="M1 0a1 1 0 0 0-1 1v14a1 1 0 0 0 1 1h5v-1H2v-1h4v-1H4v-1h2v-1H2v-1h4V9H4V8h2V7H2V6h4V2h1v4h1V4h1v2h1V2h1v4h1V4h1v2h1V2h1v4h1V1a1 1 0 0 0-1-1H1z"/>'),pT=Xu("Safe",'<path d="M1 1.5A1.5 1.5 0 0 1 2.5 0h12A1.5 1.5 0 0 1 16 1.5v13a1.5 1.5 0 0 1-1.5 1.5h-12A1.5 1.5 0 0 1 1 14.5V13H.5a.5.5 0 0 1 0-1H1V8.5H.5a.5.5 0 0 1 0-1H1V4H.5a.5.5 0 0 1 0-1H1V1.5zM2.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 .5.5h12a.5.5 0 0 0 .5-.5v-13a.5.5 0 0 0-.5-.5h-12z"/><path d="M13.5 6a.5.5 0 0 1 .5.5v3a.5.5 0 0 1-1 0v-3a.5.5 0 0 1 .5-.5zM4.828 4.464a.5.5 0 0 1 .708 0l1.09 1.09a3.003 3.003 0 0 1 3.476 0l1.09-1.09a.5.5 0 1 1 .707.708l-1.09 1.09c.74 1.037.74 2.44 0 3.476l1.09 1.09a.5.5 0 1 1-.707.708l-1.09-1.09a3.002 3.002 0 0 1-3.476 0l-1.09 1.09a.5.5 0 1 1-.708-.708l1.09-1.09a3.003 3.003 0 0 1 0-3.476l-1.09-1.09a.5.5 0 0 1 0-.708zM6.95 6.586a2 2 0 1 0 2.828 2.828A2 2 0 0 0 6.95 6.586z"/>'),mT=Xu("Safe2",'<path d="M1 2.5A1.5 1.5 0 0 1 2.5 1h12A1.5 1.5 0 0 1 16 2.5v12a1.5 1.5 0 0 1-1.5 1.5h-12A1.5 1.5 0 0 1 1 14.5V14H.5a.5.5 0 0 1 0-1H1V9H.5a.5.5 0 0 1 0-1H1V4H.5a.5.5 0 0 1 0-1H1v-.5zM2.5 2a.5.5 0 0 0-.5.5v12a.5.5 0 0 0 .5.5h12a.5.5 0 0 0 .5-.5v-12a.5.5 0 0 0-.5-.5h-12z"/><path d="M5.035 8h1.528c.047-.184.12-.357.214-.516l-1.08-1.08A3.482 3.482 0 0 0 5.035 8zm1.369-2.303 1.08 1.08c.16-.094.332-.167.516-.214V5.035a3.482 3.482 0 0 0-1.596.662zM9 5.035v1.528c.184.047.357.12.516.214l1.08-1.08A3.482 3.482 0 0 0 9 5.035zm2.303 1.369-1.08 1.08c.094.16.167.332.214.516h1.528a3.483 3.483 0 0 0-.662-1.596zM11.965 9h-1.528c-.047.184-.12.357-.214.516l1.08 1.08A3.483 3.483 0 0 0 11.965 9zm-1.369 2.303-1.08-1.08c-.16.094-.332.167-.516.214v1.528a3.483 3.483 0 0 0 1.596-.662zM8 11.965v-1.528a1.989 1.989 0 0 1-.516-.214l-1.08 1.08A3.483 3.483 0 0 0 8 11.965zm-2.303-1.369 1.08-1.08A1.988 1.988 0 0 1 6.563 9H5.035c.085.593.319 1.138.662 1.596zM4 8.5a4.5 4.5 0 1 1 9 0 4.5 4.5 0 0 1-9 0zm4.5-1a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/>'),vT=Xu("Safe2Fill",'<path d="M6.563 8H5.035a3.482 3.482 0 0 1 .662-1.596l1.08 1.08c-.094.16-.167.332-.214.516zm.921-1.223-1.08-1.08A3.482 3.482 0 0 1 8 5.035v1.528c-.184.047-.357.12-.516.214zM9 6.563V5.035a3.482 3.482 0 0 1 1.596.662l-1.08 1.08A1.988 1.988 0 0 0 9 6.563zm1.223.921 1.08-1.08c.343.458.577 1.003.662 1.596h-1.528a1.989 1.989 0 0 0-.214-.516zM10.437 9h1.528a3.483 3.483 0 0 1-.662 1.596l-1.08-1.08c.094-.16.167-.332.214-.516zm-.921 1.223 1.08 1.08A3.483 3.483 0 0 1 9 11.965v-1.528c.184-.047.357-.12.516-.214zM8 10.437v1.528a3.483 3.483 0 0 1-1.596-.662l1.08-1.08c.16.094.332.167.516.214zm-1.223-.921-1.08 1.08A3.482 3.482 0 0 1 5.035 9h1.528c.047.184.12.357.214.516zM7.5 8.5a1 1 0 1 1 2 0 1 1 0 0 1-2 0z"/><path d="M2.5 1A1.5 1.5 0 0 0 1 2.5V3H.5a.5.5 0 0 0 0 1H1v4H.5a.5.5 0 0 0 0 1H1v4H.5a.5.5 0 0 0 0 1H1v.5A1.5 1.5 0 0 0 2.5 16h12a1.5 1.5 0 0 0 1.5-1.5v-12A1.5 1.5 0 0 0 14.5 1h-12zm6 3a4.5 4.5 0 1 1 0 9 4.5 4.5 0 0 1 0-9z"/>'),_T=Xu("SafeFill",'<path d="M9.778 9.414A2 2 0 1 1 6.95 6.586a2 2 0 0 1 2.828 2.828z"/><path d="M2.5 0A1.5 1.5 0 0 0 1 1.5V3H.5a.5.5 0 0 0 0 1H1v3.5H.5a.5.5 0 0 0 0 1H1V12H.5a.5.5 0 0 0 0 1H1v1.5A1.5 1.5 0 0 0 2.5 16h12a1.5 1.5 0 0 0 1.5-1.5v-13A1.5 1.5 0 0 0 14.5 0h-12zm3.036 4.464 1.09 1.09a3.003 3.003 0 0 1 3.476 0l1.09-1.09a.5.5 0 1 1 .707.708l-1.09 1.09c.74 1.037.74 2.44 0 3.476l1.09 1.09a.5.5 0 1 1-.707.708l-1.09-1.09a3.002 3.002 0 0 1-3.476 0l-1.09 1.09a.5.5 0 1 1-.708-.708l1.09-1.09a3.003 3.003 0 0 1 0-3.476l-1.09-1.09a.5.5 0 1 1 .708-.708zM14 6.5v3a.5.5 0 0 1-1 0v-3a.5.5 0 0 1 1 0z"/>'),gT=Xu("Save",'<path d="M2 1a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H9.5a1 1 0 0 0-1 1v7.293l2.646-2.647a.5.5 0 0 1 .708.708l-3.5 3.5a.5.5 0 0 1-.708 0l-3.5-3.5a.5.5 0 1 1 .708-.708L7.5 9.293V2a2 2 0 0 1 2-2H14a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h2.5a.5.5 0 0 1 0 1H2z"/>'),yT=Xu("Save2",'<path d="M2 1a1 1 0 0 0-1 1v12a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H9.5a1 1 0 0 0-1 1v4.5h2a.5.5 0 0 1 .354.854l-2.5 2.5a.5.5 0 0 1-.708 0l-2.5-2.5A.5.5 0 0 1 5.5 6.5h2V2a2 2 0 0 1 2-2H14a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h2.5a.5.5 0 0 1 0 1H2z"/>'),bT=Xu("Save2Fill",'<path d="M8.5 1.5A1.5 1.5 0 0 1 10 0h4a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h6c-.314.418-.5.937-.5 1.5v6h-2a.5.5 0 0 0-.354.854l2.5 2.5a.5.5 0 0 0 .708 0l2.5-2.5A.5.5 0 0 0 10.5 7.5h-2v-6z"/>'),MT=Xu("SaveFill",'<path d="M8.5 1.5A1.5 1.5 0 0 1 10 0h4a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2h6c-.314.418-.5.937-.5 1.5v7.793L4.854 6.646a.5.5 0 1 0-.708.708l3.5 3.5a.5.5 0 0 0 .708 0l3.5-3.5a.5.5 0 0 0-.708-.708L8.5 9.293V1.5z"/>'),zT=Xu("Scissors",'<path d="M3.5 3.5c-.614-.884-.074-1.962.858-2.5L8 7.226 11.642 1c.932.538 1.472 1.616.858 2.5L8.81 8.61l1.556 2.661a2.5 2.5 0 1 1-.794.637L8 9.73l-1.572 2.177a2.5 2.5 0 1 1-.794-.637L7.19 8.61 3.5 3.5zm2.5 10a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0zm7 0a1.5 1.5 0 1 0-3 0 1.5 1.5 0 0 0 3 0z"/>'),LT=Xu("Screwdriver",'<path d="m0 1 1-1 3.081 2.2a1 1 0 0 1 .419.815v.07a1 1 0 0 0 .293.708L10.5 9.5l.914-.305a1 1 0 0 1 1.023.242l3.356 3.356a1 1 0 0 1 0 1.414l-1.586 1.586a1 1 0 0 1-1.414 0l-3.356-3.356a1 1 0 0 1-.242-1.023L9.5 10.5 3.793 4.793a1 1 0 0 0-.707-.293h-.071a1 1 0 0 1-.814-.419L0 1zm11.354 9.646a.5.5 0 0 0-.708.708l3 3a.5.5 0 0 0 .708-.708l-3-3z"/>'),wT=Xu("SdCard",'<path d="M6.25 3.5a.75.75 0 0 0-1.5 0v2a.75.75 0 0 0 1.5 0v-2zm2 0a.75.75 0 0 0-1.5 0v2a.75.75 0 0 0 1.5 0v-2zm2 0a.75.75 0 0 0-1.5 0v2a.75.75 0 0 0 1.5 0v-2zm2 0a.75.75 0 0 0-1.5 0v2a.75.75 0 0 0 1.5 0v-2z"/><path fill-rule="evenodd" d="M5.914 0H12.5A1.5 1.5 0 0 1 14 1.5v13a1.5 1.5 0 0 1-1.5 1.5h-9A1.5 1.5 0 0 1 2 14.5V3.914c0-.398.158-.78.44-1.06L4.853.439A1.5 1.5 0 0 1 5.914 0zM13 1.5a.5.5 0 0 0-.5-.5H5.914a.5.5 0 0 0-.353.146L3.146 3.561A.5.5 0 0 0 3 3.914V14.5a.5.5 0 0 0 .5.5h9a.5.5 0 0 0 .5-.5v-13z"/>'),HT=Xu("SdCardFill",'<path fill-rule="evenodd" d="M12.5 0H5.914a1.5 1.5 0 0 0-1.06.44L2.439 2.853A1.5 1.5 0 0 0 2 3.914V14.5A1.5 1.5 0 0 0 3.5 16h9a1.5 1.5 0 0 0 1.5-1.5v-13A1.5 1.5 0 0 0 12.5 0zm-7 2.75a.75.75 0 0 1 .75.75v2a.75.75 0 0 1-1.5 0v-2a.75.75 0 0 1 .75-.75zm2 0a.75.75 0 0 1 .75.75v2a.75.75 0 0 1-1.5 0v-2a.75.75 0 0 1 .75-.75zm2.75.75a.75.75 0 0 0-1.5 0v2a.75.75 0 0 0 1.5 0v-2zm1.25-.75a.75.75 0 0 1 .75.75v2a.75.75 0 0 1-1.5 0v-2a.75.75 0 0 1 .75-.75z"/>'),kT=Xu("Search",'<path d="M11.742 10.344a6.5 6.5 0 1 0-1.397 1.398h-.001c.03.04.062.078.098.115l3.85 3.85a1 1 0 0 0 1.415-1.414l-3.85-3.85a1.007 1.007 0 0 0-.115-.1zM12 6.5a5.5 5.5 0 1 1-11 0 5.5 5.5 0 0 1 11 0z"/>'),AT=Xu("SegmentedNav",'<path d="M0 6a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v2a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V6zm6 3h4V5H6v4zm9-1V6a1 1 0 0 0-1-1h-3v4h3a1 1 0 0 0 1-1z"/>'),ST=Xu("Server",'<path d="M1.333 2.667C1.333 1.194 4.318 0 8 0s6.667 1.194 6.667 2.667V4c0 1.473-2.985 2.667-6.667 2.667S1.333 5.473 1.333 4V2.667z"/><path d="M1.333 6.334v3C1.333 10.805 4.318 12 8 12s6.667-1.194 6.667-2.667V6.334a6.51 6.51 0 0 1-1.458.79C11.81 7.684 9.967 8 8 8c-1.966 0-3.809-.317-5.208-.876a6.508 6.508 0 0 1-1.458-.79z"/><path d="M14.667 11.668a6.51 6.51 0 0 1-1.458.789c-1.4.56-3.242.876-5.21.876-1.966 0-3.809-.316-5.208-.876a6.51 6.51 0 0 1-1.458-.79v1.666C1.333 14.806 4.318 16 8 16s6.667-1.194 6.667-2.667v-1.665z"/>'),VT=Xu("Share",'<path d="M13.5 1a1.5 1.5 0 1 0 0 3 1.5 1.5 0 0 0 0-3zM11 2.5a2.5 2.5 0 1 1 .603 1.628l-6.718 3.12a2.499 2.499 0 0 1 0 1.504l6.718 3.12a2.5 2.5 0 1 1-.488.876l-6.718-3.12a2.5 2.5 0 1 1 0-3.256l6.718-3.12A2.5 2.5 0 0 1 11 2.5zm-8.5 4a1.5 1.5 0 1 0 0 3 1.5 1.5 0 0 0 0-3zm11 5.5a1.5 1.5 0 1 0 0 3 1.5 1.5 0 0 0 0-3z"/>'),OT=Xu("ShareFill",'<path d="M11 2.5a2.5 2.5 0 1 1 .603 1.628l-6.718 3.12a2.499 2.499 0 0 1 0 1.504l6.718 3.12a2.5 2.5 0 1 1-.488.876l-6.718-3.12a2.5 2.5 0 1 1 0-3.256l6.718-3.12A2.5 2.5 0 0 1 11 2.5z"/>'),TT=Xu("Shield",'<path d="M5.338 1.59a61.44 61.44 0 0 0-2.837.856.481.481 0 0 0-.328.39c-.554 4.157.726 7.19 2.253 9.188a10.725 10.725 0 0 0 2.287 2.233c.346.244.652.42.893.533.12.057.218.095.293.118a.55.55 0 0 0 .101.025.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/>'),DT=Xu("ShieldCheck",'<path d="M5.338 1.59a61.44 61.44 0 0 0-2.837.856.481.481 0 0 0-.328.39c-.554 4.157.726 7.19 2.253 9.188a10.725 10.725 0 0 0 2.287 2.233c.346.244.652.42.893.533.12.057.218.095.293.118a.55.55 0 0 0 .101.025.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/><path d="M10.854 5.146a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 7.793l2.646-2.647a.5.5 0 0 1 .708 0z"/>'),CT=Xu("ShieldExclamation",'<path d="M5.338 1.59a61.44 61.44 0 0 0-2.837.856.481.481 0 0 0-.328.39c-.554 4.157.726 7.19 2.253 9.188a10.725 10.725 0 0 0 2.287 2.233c.346.244.652.42.893.533.12.057.218.095.293.118a.55.55 0 0 0 .101.025.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/><path d="M7.001 11a1 1 0 1 1 2 0 1 1 0 0 1-2 0zM7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.553.553 0 0 1-1.1 0L7.1 4.995z"/>'),jT=Xu("ShieldFill",'<path d="M5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/>'),xT=Xu("ShieldFillCheck",'<path fill-rule="evenodd" d="M8 0c-.69 0-1.843.265-2.928.56-1.11.3-2.229.655-2.887.87a1.54 1.54 0 0 0-1.044 1.262c-.596 4.477.787 7.795 2.465 9.99a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.775 11.775 0 0 0 2.517-2.453c1.678-2.195 3.061-5.513 2.465-9.99a1.541 1.541 0 0 0-1.044-1.263 62.467 62.467 0 0 0-2.887-.87C9.843.266 8.69 0 8 0zm2.146 5.146a.5.5 0 0 1 .708.708l-3 3a.5.5 0 0 1-.708 0l-1.5-1.5a.5.5 0 1 1 .708-.708L7.5 7.793l2.646-2.647z"/>'),BT=Xu("ShieldFillExclamation",'<path fill-rule="evenodd" d="M8 0c-.69 0-1.843.265-2.928.56-1.11.3-2.229.655-2.887.87a1.54 1.54 0 0 0-1.044 1.262c-.596 4.477.787 7.795 2.465 9.99a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.775 11.775 0 0 0 2.517-2.453c1.678-2.195 3.061-5.513 2.465-9.99a1.541 1.541 0 0 0-1.044-1.263 62.467 62.467 0 0 0-2.887-.87C9.843.266 8.69 0 8 0zm-.55 8.502L7.1 4.995a.905.905 0 1 1 1.8 0l-.35 3.507a.552.552 0 0 1-1.1 0zM8.002 12a1 1 0 1 1 0-2 1 1 0 0 1 0 2z"/>'),YT=Xu("ShieldFillMinus",'<path fill-rule="evenodd" d="M8 0c-.69 0-1.843.265-2.928.56-1.11.3-2.229.655-2.887.87a1.54 1.54 0 0 0-1.044 1.262c-.596 4.477.787 7.795 2.465 9.99a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.775 11.775 0 0 0 2.517-2.453c1.678-2.195 3.061-5.513 2.465-9.99a1.541 1.541 0 0 0-1.044-1.263 62.467 62.467 0 0 0-2.887-.87C9.843.266 8.69 0 8 0zM6 7.5a.5.5 0 0 1 0-1h4a.5.5 0 0 1 0 1H6z"/>'),PT=Xu("ShieldFillPlus",'<path fill-rule="evenodd" d="M8 0c-.69 0-1.843.265-2.928.56-1.11.3-2.229.655-2.887.87a1.54 1.54 0 0 0-1.044 1.262c-.596 4.477.787 7.795 2.465 9.99a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.775 11.775 0 0 0 2.517-2.453c1.678-2.195 3.061-5.513 2.465-9.99a1.541 1.541 0 0 0-1.044-1.263 62.467 62.467 0 0 0-2.887-.87C9.843.266 8.69 0 8 0zm-.5 5a.5.5 0 0 1 1 0v1.5H10a.5.5 0 0 1 0 1H8.5V9a.5.5 0 0 1-1 0V7.5H6a.5.5 0 0 1 0-1h1.5V5z"/>'),IT=Xu("ShieldFillX",'<path d="M8 0c-.69 0-1.843.265-2.928.56-1.11.3-2.229.655-2.887.87a1.54 1.54 0 0 0-1.044 1.262c-.596 4.477.787 7.795 2.465 9.99a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.775 11.775 0 0 0 2.517-2.453c1.678-2.195 3.061-5.513 2.465-9.99a1.541 1.541 0 0 0-1.044-1.263 62.467 62.467 0 0 0-2.887-.87C9.843.266 8.69 0 8 0zM6.854 5.146 8 6.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 7l1.147 1.146a.5.5 0 0 1-.708.708L8 7.707 6.854 8.854a.5.5 0 1 1-.708-.708L7.293 7 6.146 5.854a.5.5 0 1 1 .708-.708z"/>'),FT=Xu("ShieldLock",'<path d="M5.338 1.59a61.44 61.44 0 0 0-2.837.856.481.481 0 0 0-.328.39c-.554 4.157.726 7.19 2.253 9.188a10.725 10.725 0 0 0 2.287 2.233c.346.244.652.42.893.533.12.057.218.095.293.118a.55.55 0 0 0 .101.025.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/><path d="M9.5 6.5a1.5 1.5 0 0 1-1 1.415l.385 1.99a.5.5 0 0 1-.491.595h-.788a.5.5 0 0 1-.49-.595l.384-1.99a1.5 1.5 0 1 1 2-1.415z"/>'),ET=Xu("ShieldLockFill",'<path fill-rule="evenodd" d="M8 0c-.69 0-1.843.265-2.928.56-1.11.3-2.229.655-2.887.87a1.54 1.54 0 0 0-1.044 1.262c-.596 4.477.787 7.795 2.465 9.99a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.775 11.775 0 0 0 2.517-2.453c1.678-2.195 3.061-5.513 2.465-9.99a1.541 1.541 0 0 0-1.044-1.263 62.467 62.467 0 0 0-2.887-.87C9.843.266 8.69 0 8 0zm0 5a1.5 1.5 0 0 1 .5 2.915l.385 1.99a.5.5 0 0 1-.491.595h-.788a.5.5 0 0 1-.49-.595l.384-1.99A1.5 1.5 0 0 1 8 5z"/>'),$T=Xu("ShieldMinus",'<path d="M5.338 1.59a61.44 61.44 0 0 0-2.837.856.481.481 0 0 0-.328.39c-.554 4.157.726 7.19 2.253 9.188a10.725 10.725 0 0 0 2.287 2.233c.346.244.652.42.893.533.12.057.218.095.293.118a.55.55 0 0 0 .101.025.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/><path d="M5.5 7a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1H6a.5.5 0 0 1-.5-.5z"/>'),RT=Xu("ShieldPlus",'<path d="M5.338 1.59a61.44 61.44 0 0 0-2.837.856.481.481 0 0 0-.328.39c-.554 4.157.726 7.19 2.253 9.188a10.725 10.725 0 0 0 2.287 2.233c.346.244.652.42.893.533.12.057.218.095.293.118a.55.55 0 0 0 .101.025.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/><path d="M8 4.5a.5.5 0 0 1 .5.5v1.5H10a.5.5 0 0 1 0 1H8.5V9a.5.5 0 0 1-1 0V7.5H6a.5.5 0 0 1 0-1h1.5V5a.5.5 0 0 1 .5-.5z"/>'),NT=Xu("ShieldShaded",'<path fill-rule="evenodd" d="M8 14.933a.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067v13.866zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/>'),WT=Xu("ShieldSlash",'<path fill-rule="evenodd" d="M1.093 3.093c-.465 4.275.885 7.46 2.513 9.589a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.32 11.32 0 0 0 1.733-1.525l-.745-.745a10.27 10.27 0 0 1-1.578 1.392c-.346.244-.652.42-.893.533-.12.057-.218.095-.293.118a.55.55 0 0 1-.101.025.615.615 0 0 1-.1-.025 2.348 2.348 0 0 1-.294-.118 6.141 6.141 0 0 1-.893-.533 10.725 10.725 0 0 1-2.287-2.233C3.053 10.228 1.879 7.594 2.06 4.06l-.967-.967zM3.98 1.98l-.852-.852A58.935 58.935 0 0 1 5.072.559C6.157.266 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.483 3.626-.332 6.491-1.551 8.616l-.77-.77c1.042-1.915 1.72-4.469 1.29-7.702a.48.48 0 0 0-.33-.39c-.65-.213-1.75-.56-2.836-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524a49.7 49.7 0 0 0-1.357.39zm9.666 12.374-13-13 .708-.708 13 13-.707.707z"/>'),UT=Xu("ShieldSlashFill",'<path fill-rule="evenodd" d="M1.093 3.093c-.465 4.275.885 7.46 2.513 9.589a11.777 11.777 0 0 0 2.517 2.453c.386.273.744.482 1.048.625.28.132.581.24.829.24s.548-.108.829-.24a7.159 7.159 0 0 0 1.048-.625 11.32 11.32 0 0 0 1.733-1.525L1.093 3.093zm12.215 8.215L3.128 1.128A61.369 61.369 0 0 1 5.073.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.483 3.626-.332 6.491-1.551 8.616zm.338 3.046-13-13 .708-.708 13 13-.707.707z"/>'),GT=Xu("ShieldX",'<path d="M5.338 1.59a61.44 61.44 0 0 0-2.837.856.481.481 0 0 0-.328.39c-.554 4.157.726 7.19 2.253 9.188a10.725 10.725 0 0 0 2.287 2.233c.346.244.652.42.893.533.12.057.218.095.293.118a.55.55 0 0 0 .101.025.615.615 0 0 0 .1-.025c.076-.023.174-.061.294-.118.24-.113.547-.29.893-.533a10.726 10.726 0 0 0 2.287-2.233c1.527-1.997 2.807-5.031 2.253-9.188a.48.48 0 0 0-.328-.39c-.651-.213-1.75-.56-2.837-.855C9.552 1.29 8.531 1.067 8 1.067c-.53 0-1.552.223-2.662.524zM5.072.56C6.157.265 7.31 0 8 0s1.843.265 2.928.56c1.11.3 2.229.655 2.887.87a1.54 1.54 0 0 1 1.044 1.262c.596 4.477-.787 7.795-2.465 9.99a11.775 11.775 0 0 1-2.517 2.453 7.159 7.159 0 0 1-1.048.625c-.28.132-.581.24-.829.24s-.548-.108-.829-.24a7.158 7.158 0 0 1-1.048-.625 11.777 11.777 0 0 1-2.517-2.453C1.928 10.487.545 7.169 1.141 2.692A1.54 1.54 0 0 1 2.185 1.43 62.456 62.456 0 0 1 5.072.56z"/><path d="M6.146 5.146a.5.5 0 0 1 .708 0L8 6.293l1.146-1.147a.5.5 0 1 1 .708.708L8.707 7l1.147 1.146a.5.5 0 0 1-.708.708L8 7.707 6.854 8.854a.5.5 0 1 1-.708-.708L7.293 7 6.146 5.854a.5.5 0 0 1 0-.708z"/>'),qT=Xu("Shift",'<path d="M7.27 2.047a1 1 0 0 1 1.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H11.5v3a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1v-3H1.654C.78 10.5.326 9.455.924 8.816L7.27 2.047zM14.346 9.5 8 2.731 1.654 9.5H4.5a1 1 0 0 1 1 1v3h5v-3a1 1 0 0 1 1-1h2.846z"/>'),JT=Xu("ShiftFill",'<path d="M7.27 2.047a1 1 0 0 1 1.46 0l6.345 6.77c.6.638.146 1.683-.73 1.683H11.5v3a1 1 0 0 1-1 1h-5a1 1 0 0 1-1-1v-3H1.654C.78 10.5.326 9.455.924 8.816L7.27 2.047z"/>'),KT=Xu("Shop",'<path d="M2.97 1.35A1 1 0 0 1 3.73 1h8.54a1 1 0 0 1 .76.35l2.609 3.044A1.5 1.5 0 0 1 16 5.37v.255a2.375 2.375 0 0 1-4.25 1.458A2.371 2.371 0 0 1 9.875 8 2.37 2.37 0 0 1 8 7.083 2.37 2.37 0 0 1 6.125 8a2.37 2.37 0 0 1-1.875-.917A2.375 2.375 0 0 1 0 5.625V5.37a1.5 1.5 0 0 1 .361-.976l2.61-3.045zm1.78 4.275a1.375 1.375 0 0 0 2.75 0 .5.5 0 0 1 1 0 1.375 1.375 0 0 0 2.75 0 .5.5 0 0 1 1 0 1.375 1.375 0 1 0 2.75 0V5.37a.5.5 0 0 0-.12-.325L12.27 2H3.73L1.12 5.045A.5.5 0 0 0 1 5.37v.255a1.375 1.375 0 0 0 2.75 0 .5.5 0 0 1 1 0zM1.5 8.5A.5.5 0 0 1 2 9v6h1v-5a1 1 0 0 1 1-1h3a1 1 0 0 1 1 1v5h6V9a.5.5 0 0 1 1 0v6h.5a.5.5 0 0 1 0 1H.5a.5.5 0 0 1 0-1H1V9a.5.5 0 0 1 .5-.5zM4 15h3v-5H4v5zm5-5a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1h-2a1 1 0 0 1-1-1v-3zm3 0h-2v3h2v-3z"/>'),XT=Xu("ShopWindow",'<path d="M2.97 1.35A1 1 0 0 1 3.73 1h8.54a1 1 0 0 1 .76.35l2.609 3.044A1.5 1.5 0 0 1 16 5.37v.255a2.375 2.375 0 0 1-4.25 1.458A2.371 2.371 0 0 1 9.875 8 2.37 2.37 0 0 1 8 7.083 2.37 2.37 0 0 1 6.125 8a2.37 2.37 0 0 1-1.875-.917A2.375 2.375 0 0 1 0 5.625V5.37a1.5 1.5 0 0 1 .361-.976l2.61-3.045zm1.78 4.275a1.375 1.375 0 0 0 2.75 0 .5.5 0 0 1 1 0 1.375 1.375 0 0 0 2.75 0 .5.5 0 0 1 1 0 1.375 1.375 0 1 0 2.75 0V5.37a.5.5 0 0 0-.12-.325L12.27 2H3.73L1.12 5.045A.5.5 0 0 0 1 5.37v.255a1.375 1.375 0 0 0 2.75 0 .5.5 0 0 1 1 0zM1.5 8.5A.5.5 0 0 1 2 9v6h12V9a.5.5 0 0 1 1 0v6h.5a.5.5 0 0 1 0 1H.5a.5.5 0 0 1 0-1H1V9a.5.5 0 0 1 .5-.5zm2 .5a.5.5 0 0 1 .5.5V13h8V9.5a.5.5 0 0 1 1 0V13a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V9.5a.5.5 0 0 1 .5-.5z"/>'),ZT=Xu("Shuffle",'<path fill-rule="evenodd" d="M0 3.5A.5.5 0 0 1 .5 3H1c2.202 0 3.827 1.24 4.874 2.418.49.552.865 1.102 1.126 1.532.26-.43.636-.98 1.126-1.532C9.173 4.24 10.798 3 13 3v1c-1.798 0-3.173 1.01-4.126 2.082A9.624 9.624 0 0 0 7.556 8a9.624 9.624 0 0 0 1.317 1.918C9.828 10.99 11.204 12 13 12v1c-2.202 0-3.827-1.24-4.874-2.418A10.595 10.595 0 0 1 7 9.05c-.26.43-.636.98-1.126 1.532C4.827 11.76 3.202 13 1 13H.5a.5.5 0 0 1 0-1H1c1.798 0 3.173-1.01 4.126-2.082A9.624 9.624 0 0 0 6.444 8a9.624 9.624 0 0 0-1.317-1.918C4.172 5.01 2.796 4 1 4H.5a.5.5 0 0 1-.5-.5z"/><path d="M13 5.466V1.534a.25.25 0 0 1 .41-.192l2.36 1.966c.12.1.12.284 0 .384l-2.36 1.966a.25.25 0 0 1-.41-.192zm0 9v-3.932a.25.25 0 0 1 .41-.192l2.36 1.966c.12.1.12.284 0 .384l-2.36 1.966a.25.25 0 0 1-.41-.192z"/>'),QT=Xu("Signpost",'<path d="M7 1.414V4H2a1 1 0 0 0-1 1v4a1 1 0 0 0 1 1h5v6h2v-6h3.532a1 1 0 0 0 .768-.36l1.933-2.32a.5.5 0 0 0 0-.64L13.3 4.36a1 1 0 0 0-.768-.36H9V1.414a1 1 0 0 0-2 0zM12.532 5l1.666 2-1.666 2H2V5h10.532z"/>'),eD=Xu("Signpost2",'<path d="M7 1.414V2H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h5v1H2.5a1 1 0 0 0-.8.4L.725 8.7a.5.5 0 0 0 0 .6l.975 1.3a1 1 0 0 0 .8.4H7v5h2v-5h5a1 1 0 0 0 1-1V8a1 1 0 0 0-1-1H9V6h4.5a1 1 0 0 0 .8-.4l.975-1.3a.5.5 0 0 0 0-.6L14.3 2.4a1 1 0 0 0-.8-.4H9v-.586a1 1 0 0 0-2 0zM13.5 3l.75 1-.75 1H2V3h11.5zm.5 5v2H2.5l-.75-1 .75-1H14z"/>'),tD=Xu("Signpost2Fill",'<path d="M7.293.707A1 1 0 0 0 7 1.414V2H2a1 1 0 0 0-1 1v2a1 1 0 0 0 1 1h5v1H2.5a1 1 0 0 0-.8.4L.725 8.7a.5.5 0 0 0 0 .6l.975 1.3a1 1 0 0 0 .8.4H7v5h2v-5h5a1 1 0 0 0 1-1V8a1 1 0 0 0-1-1H9V6h4.5a1 1 0 0 0 .8-.4l.975-1.3a.5.5 0 0 0 0-.6L14.3 2.4a1 1 0 0 0-.8-.4H9v-.586A1 1 0 0 0 7.293.707z"/>'),aD=Xu("SignpostFill",'<path d="M7.293.707A1 1 0 0 0 7 1.414V4H2a1 1 0 0 0-1 1v4a1 1 0 0 0 1 1h5v6h2v-6h3.532a1 1 0 0 0 .768-.36l1.933-2.32a.5.5 0 0 0 0-.64L13.3 4.36a1 1 0 0 0-.768-.36H9V1.414A1 1 0 0 0 7.293.707z"/>'),nD=Xu("SignpostSplit",'<path d="M7 7V1.414a1 1 0 0 1 2 0V2h5a1 1 0 0 1 .8.4l.975 1.3a.5.5 0 0 1 0 .6L14.8 5.6a1 1 0 0 1-.8.4H9v10H7v-5H2a1 1 0 0 1-.8-.4L.225 9.3a.5.5 0 0 1 0-.6L1.2 7.4A1 1 0 0 1 2 7h5zm1 3V8H2l-.75 1L2 10h6zm0-5h6l.75-1L14 3H8v2z"/>'),rD=Xu("SignpostSplitFill",'<path d="M7 16h2V6h5a1 1 0 0 0 .8-.4l.975-1.3a.5.5 0 0 0 0-.6L14.8 2.4A1 1 0 0 0 14 2H9v-.586a1 1 0 0 0-2 0V7H2a1 1 0 0 0-.8.4L.225 8.7a.5.5 0 0 0 0 .6l.975 1.3a1 1 0 0 0 .8.4h5v5z"/>'),iD=Xu("Sim",'<path d="M2 1.5A1.5 1.5 0 0 1 3.5 0h7.086a1.5 1.5 0 0 1 1.06.44l1.915 1.914A1.5 1.5 0 0 1 14 3.414V14.5a1.5 1.5 0 0 1-1.5 1.5h-9A1.5 1.5 0 0 1 2 14.5v-13zM3.5 1a.5.5 0 0 0-.5.5v13a.5.5 0 0 0 .5.5h9a.5.5 0 0 0 .5-.5V3.414a.5.5 0 0 0-.146-.353l-1.915-1.915A.5.5 0 0 0 10.586 1H3.5z"/><path d="M5.5 4a.5.5 0 0 0-.5.5V6h2.5V4h-2zm3 0v2H11V4.5a.5.5 0 0 0-.5-.5h-2zM11 7H5v2h6V7zm0 3H8.5v2h2a.5.5 0 0 0 .5-.5V10zm-3.5 2v-2H5v1.5a.5.5 0 0 0 .5.5h2zM4 4.5A1.5 1.5 0 0 1 5.5 3h5A1.5 1.5 0 0 1 12 4.5v7a1.5 1.5 0 0 1-1.5 1.5h-5A1.5 1.5 0 0 1 4 11.5v-7z"/>'),oD=Xu("SimFill",'<path d="M5 4.5a.5.5 0 0 1 .5-.5h2v2H5V4.5zM8.5 6V4h2a.5.5 0 0 1 .5.5V6H8.5zM5 7h6v2H5V7zm3.5 3H11v1.5a.5.5 0 0 1-.5.5h-2v-2zm-1 0v2h-2a.5.5 0 0 1-.5-.5V10h2.5z"/><path d="M3.5 0A1.5 1.5 0 0 0 2 1.5v13A1.5 1.5 0 0 0 3.5 16h9a1.5 1.5 0 0 0 1.5-1.5V3.414a1.5 1.5 0 0 0-.44-1.06L11.647.439A1.5 1.5 0 0 0 10.586 0H3.5zm2 3h5A1.5 1.5 0 0 1 12 4.5v7a1.5 1.5 0 0 1-1.5 1.5h-5A1.5 1.5 0 0 1 4 11.5v-7A1.5 1.5 0 0 1 5.5 3z"/>'),sD=Xu("SkipBackward",'<path d="M.5 3.5A.5.5 0 0 1 1 4v3.248l6.267-3.636c.52-.302 1.233.043 1.233.696v2.94l6.267-3.636c.52-.302 1.233.043 1.233.696v7.384c0 .653-.713.998-1.233.696L8.5 8.752v2.94c0 .653-.713.998-1.233.696L1 8.752V12a.5.5 0 0 1-1 0V4a.5.5 0 0 1 .5-.5zm7 1.133L1.696 8 7.5 11.367V4.633zm7.5 0L9.196 8 15 11.367V4.633z"/>'),lD=Xu("SkipBackwardBtn",'<path d="M11.21 5.093A.5.5 0 0 1 12 5.5v5a.5.5 0 0 1-.79.407L8.5 8.972V10.5a.5.5 0 0 1-.79.407L5 8.972V10.5a.5.5 0 0 1-1 0v-5a.5.5 0 0 1 1 0v1.528l2.71-1.935a.5.5 0 0 1 .79.407v1.528l2.71-1.935z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),cD=Xu("SkipBackwardBtnFill",'<path d="M0 12V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm11.21-6.907L8.5 7.028V5.5a.5.5 0 0 0-.79-.407L5 7.028V5.5a.5.5 0 0 0-1 0v5a.5.5 0 0 0 1 0V8.972l2.71 1.935a.5.5 0 0 0 .79-.407V8.972l2.71 1.935A.5.5 0 0 0 12 10.5v-5a.5.5 0 0 0-.79-.407z"/>'),uD=Xu("SkipBackwardCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M11.729 5.055a.5.5 0 0 0-.52.038L8.5 7.028V5.5a.5.5 0 0 0-.79-.407L5 7.028V5.5a.5.5 0 0 0-1 0v5a.5.5 0 0 0 1 0V8.972l2.71 1.935a.5.5 0 0 0 .79-.407V8.972l2.71 1.935A.5.5 0 0 0 12 10.5v-5a.5.5 0 0 0-.271-.445z"/>'),dD=Xu("SkipBackwardCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-4.79-2.907L8.5 7.028V5.5a.5.5 0 0 0-.79-.407L5 7.028V5.5a.5.5 0 0 0-1 0v5a.5.5 0 0 0 1 0V8.972l2.71 1.935a.5.5 0 0 0 .79-.407V8.972l2.71 1.935A.5.5 0 0 0 12 10.5v-5a.5.5 0 0 0-.79-.407z"/>'),hD=Xu("SkipBackwardFill",'<path d="M.5 3.5A.5.5 0 0 0 0 4v8a.5.5 0 0 0 1 0V8.753l6.267 3.636c.54.313 1.233-.066 1.233-.697v-2.94l6.267 3.636c.54.314 1.233-.065 1.233-.696V4.308c0-.63-.693-1.01-1.233-.696L8.5 7.248v-2.94c0-.63-.692-1.01-1.233-.696L1 7.248V4a.5.5 0 0 0-.5-.5z"/>'),fD=Xu("SkipEnd",'<path d="M12.5 4a.5.5 0 0 0-1 0v3.248L5.233 3.612C4.713 3.31 4 3.655 4 4.308v7.384c0 .653.713.998 1.233.696L11.5 8.752V12a.5.5 0 0 0 1 0V4zM5 4.633 10.804 8 5 11.367V4.633z"/>'),pD=Xu("SkipEndBtn",'<path d="M6.79 5.093 9.5 7.028V5.5a.5.5 0 0 1 1 0v5a.5.5 0 0 1-1 0V8.972l-2.71 1.935A.5.5 0 0 1 6 10.5v-5a.5.5 0 0 1 .79-.407z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),mD=Xu("SkipEndBtnFill",'<path d="M0 12V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm6.79-6.907A.5.5 0 0 0 6 5.5v5a.5.5 0 0 0 .79.407L9.5 8.972V10.5a.5.5 0 0 0 1 0v-5a.5.5 0 0 0-1 0v1.528L6.79 5.093z"/>'),vD=Xu("SkipEndCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M6.271 5.055a.5.5 0 0 1 .52.038L9.5 7.028V5.5a.5.5 0 0 1 1 0v5a.5.5 0 0 1-1 0V8.972l-2.71 1.935A.5.5 0 0 1 6 10.5v-5a.5.5 0 0 1 .271-.445z"/>'),_D=Xu("SkipEndCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM6.79 5.093A.5.5 0 0 0 6 5.5v5a.5.5 0 0 0 .79.407L9.5 8.972V10.5a.5.5 0 0 0 1 0v-5a.5.5 0 0 0-1 0v1.528L6.79 5.093z"/>'),gD=Xu("SkipEndFill",'<path d="M12.5 4a.5.5 0 0 0-1 0v3.248L5.233 3.612C4.693 3.3 4 3.678 4 4.308v7.384c0 .63.692 1.01 1.233.697L11.5 8.753V12a.5.5 0 0 0 1 0V4z"/>'),yD=Xu("SkipForward",'<path d="M15.5 3.5a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-1 0V8.752l-6.267 3.636c-.52.302-1.233-.043-1.233-.696v-2.94l-6.267 3.636C.713 12.69 0 12.345 0 11.692V4.308c0-.653.713-.998 1.233-.696L7.5 7.248v-2.94c0-.653.713-.998 1.233-.696L15 7.248V4a.5.5 0 0 1 .5-.5zM1 4.633v6.734L6.804 8 1 4.633zm7.5 0v6.734L14.304 8 8.5 4.633z"/>'),bD=Xu("SkipForwardBtn",'<path d="M4.79 5.093A.5.5 0 0 0 4 5.5v5a.5.5 0 0 0 .79.407L7.5 8.972V10.5a.5.5 0 0 0 .79.407L11 8.972V10.5a.5.5 0 0 0 1 0v-5a.5.5 0 0 0-1 0v1.528L8.29 5.093a.5.5 0 0 0-.79.407v1.528L4.79 5.093z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),MD=Xu("SkipForwardBtnFill",'<path d="M0 10V2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm4.79-6.907A.5.5 0 0 0 4 3.5v5a.5.5 0 0 0 .79.407L7.5 6.972V8.5a.5.5 0 0 0 .79.407L11 6.972V8.5a.5.5 0 0 0 1 0v-5a.5.5 0 0 0-1 0v1.528L8.29 3.093a.5.5 0 0 0-.79.407v1.528L4.79 3.093z"/>'),zD=Xu("SkipForwardCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4.271 5.055a.5.5 0 0 1 .52.038L7.5 7.028V5.5a.5.5 0 0 1 .79-.407L11 7.028V5.5a.5.5 0 0 1 1 0v5a.5.5 0 0 1-1 0V8.972l-2.71 1.935a.5.5 0 0 1-.79-.407V8.972l-2.71 1.935A.5.5 0 0 1 4 10.5v-5a.5.5 0 0 1 .271-.445z"/>'),LD=Xu("SkipForwardCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM4.79 5.093A.5.5 0 0 0 4 5.5v5a.5.5 0 0 0 .79.407L7.5 8.972V10.5a.5.5 0 0 0 .79.407L11 8.972V10.5a.5.5 0 0 0 1 0v-5a.5.5 0 0 0-1 0v1.528L8.29 5.093a.5.5 0 0 0-.79.407v1.528L4.79 5.093z"/>'),wD=Xu("SkipForwardFill",'<path d="M15.5 3.5a.5.5 0 0 1 .5.5v8a.5.5 0 0 1-1 0V8.753l-6.267 3.636c-.54.313-1.233-.066-1.233-.697v-2.94l-6.267 3.636C.693 12.703 0 12.324 0 11.693V4.308c0-.63.693-1.01 1.233-.696L7.5 7.248v-2.94c0-.63.693-1.01 1.233-.696L15 7.248V4a.5.5 0 0 1 .5-.5z"/>'),HD=Xu("SkipStart",'<path d="M4 4a.5.5 0 0 1 1 0v3.248l6.267-3.636c.52-.302 1.233.043 1.233.696v7.384c0 .653-.713.998-1.233.696L5 8.752V12a.5.5 0 0 1-1 0V4zm7.5.633L5.696 8l5.804 3.367V4.633z"/>'),kD=Xu("SkipStartBtn",'<path d="M9.71 5.093a.5.5 0 0 1 .79.407v5a.5.5 0 0 1-.79.407L7 8.972V10.5a.5.5 0 0 1-1 0v-5a.5.5 0 0 1 1 0v1.528l2.71-1.935z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),AD=Xu("SkipStartBtnFill",'<path d="M0 12V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm9.71-6.907L7 7.028V5.5a.5.5 0 0 0-1 0v5a.5.5 0 0 0 1 0V8.972l2.71 1.935a.5.5 0 0 0 .79-.407v-5a.5.5 0 0 0-.79-.407z"/>'),SD=Xu("SkipStartCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M10.229 5.055a.5.5 0 0 0-.52.038L7 7.028V5.5a.5.5 0 0 0-1 0v5a.5.5 0 0 0 1 0V8.972l2.71 1.935a.5.5 0 0 0 .79-.407v-5a.5.5 0 0 0-.271-.445z"/>'),VD=Xu("SkipStartCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM9.71 5.093 7 7.028V5.5a.5.5 0 0 0-1 0v5a.5.5 0 0 0 1 0V8.972l2.71 1.935a.5.5 0 0 0 .79-.407v-5a.5.5 0 0 0-.79-.407z"/>'),OD=Xu("SkipStartFill",'<path d="M4 4a.5.5 0 0 1 1 0v3.248l6.267-3.636c.54-.313 1.232.066 1.232.696v7.384c0 .63-.692 1.01-1.232.697L5 8.753V12a.5.5 0 0 1-1 0V4z"/>'),TD=Xu("Skype",'<path d="M4.671 0c.88 0 1.733.247 2.468.702a7.423 7.423 0 0 1 6.02 2.118 7.372 7.372 0 0 1 2.167 5.215c0 .344-.024.687-.072 1.026a4.662 4.662 0 0 1 .6 2.281 4.645 4.645 0 0 1-1.37 3.294A4.673 4.673 0 0 1 11.18 16c-.84 0-1.658-.226-2.37-.644a7.423 7.423 0 0 1-6.114-2.107A7.374 7.374 0 0 1 .529 8.035c0-.363.026-.724.08-1.081a4.644 4.644 0 0 1 .76-5.59A4.68 4.68 0 0 1 4.67 0zm.447 7.01c.18.309.43.572.729.769a7.07 7.07 0 0 0 1.257.653c.492.205.873.38 1.145.523.229.112.437.264.615.448.135.142.21.331.21.528a.872.872 0 0 1-.335.723c-.291.196-.64.289-.99.264a2.618 2.618 0 0 1-1.048-.206 11.44 11.44 0 0 1-.532-.253 1.284 1.284 0 0 0-.587-.15.717.717 0 0 0-.501.176.63.63 0 0 0-.195.491.796.796 0 0 0 .148.482 1.2 1.2 0 0 0 .456.354 5.113 5.113 0 0 0 2.212.419 4.554 4.554 0 0 0 1.624-.265 2.296 2.296 0 0 0 1.08-.801c.267-.39.402-.855.386-1.327a2.09 2.09 0 0 0-.279-1.101 2.53 2.53 0 0 0-.772-.792A7.198 7.198 0 0 0 8.486 7.3a1.05 1.05 0 0 0-.145-.058 18.182 18.182 0 0 1-1.013-.447 1.827 1.827 0 0 1-.54-.387.727.727 0 0 1-.2-.508.805.805 0 0 1 .385-.723 1.76 1.76 0 0 1 .968-.247c.26-.003.52.03.772.096.274.079.542.177.802.293.105.049.22.075.336.076a.6.6 0 0 0 .453-.19.69.69 0 0 0 .18-.496.717.717 0 0 0-.17-.476 1.374 1.374 0 0 0-.556-.354 3.69 3.69 0 0 0-.708-.183 5.963 5.963 0 0 0-1.022-.078 4.53 4.53 0 0 0-1.536.258 2.71 2.71 0 0 0-1.174.784 1.91 1.91 0 0 0-.45 1.287c-.01.37.076.736.25 1.063z"/>'),DD=Xu("Slack",'<path d="M3.362 10.11c0 .926-.756 1.681-1.681 1.681S0 11.036 0 10.111C0 9.186.756 8.43 1.68 8.43h1.682v1.68zm.846 0c0-.924.756-1.68 1.681-1.68s1.681.756 1.681 1.68v4.21c0 .924-.756 1.68-1.68 1.68a1.685 1.685 0 0 1-1.682-1.68v-4.21zM5.89 3.362c-.926 0-1.682-.756-1.682-1.681S4.964 0 5.89 0s1.68.756 1.68 1.68v1.682H5.89zm0 .846c.924 0 1.68.756 1.68 1.681S6.814 7.57 5.89 7.57H1.68C.757 7.57 0 6.814 0 5.89c0-.926.756-1.682 1.68-1.682h4.21zm6.749 1.682c0-.926.755-1.682 1.68-1.682.925 0 1.681.756 1.681 1.681s-.756 1.681-1.68 1.681h-1.681V5.89zm-.848 0c0 .924-.755 1.68-1.68 1.68A1.685 1.685 0 0 1 8.43 5.89V1.68C8.43.757 9.186 0 10.11 0c.926 0 1.681.756 1.681 1.68v4.21zm-1.681 6.748c.926 0 1.682.756 1.682 1.681S11.036 16 10.11 16s-1.681-.756-1.681-1.68v-1.682h1.68zm0-.847c-.924 0-1.68-.755-1.68-1.68 0-.925.756-1.681 1.68-1.681h4.21c.924 0 1.68.756 1.68 1.68 0 .926-.756 1.681-1.68 1.681h-4.21z"/>'),CD=Xu("Slash",'<path d="M11.354 4.646a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708l6-6a.5.5 0 0 1 .708 0z"/>'),jD=Xu("SlashCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M11.354 4.646a.5.5 0 0 0-.708 0l-6 6a.5.5 0 0 0 .708.708l6-6a.5.5 0 0 0 0-.708z"/>'),xD=Xu("SlashCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zm-4.646-2.646a.5.5 0 0 0-.708-.708l-6 6a.5.5 0 0 0 .708.708l6-6z"/>'),BD=Xu("SlashLg",'<path d="M14.707 1.293a1 1 0 0 1 0 1.414l-12 12a1 1 0 0 1-1.414-1.414l12-12a1 1 0 0 1 1.414 0z"/>'),YD=Xu("SlashSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M11.354 4.646a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708l6-6a.5.5 0 0 1 .708 0z"/>'),PD=Xu("SlashSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm9.354 5.354-6 6a.5.5 0 0 1-.708-.708l6-6a.5.5 0 0 1 .708.708z"/>'),ID=Xu("Sliders",'<path fill-rule="evenodd" d="M11.5 2a1.5 1.5 0 1 0 0 3 1.5 1.5 0 0 0 0-3zM9.05 3a2.5 2.5 0 0 1 4.9 0H16v1h-2.05a2.5 2.5 0 0 1-4.9 0H0V3h9.05zM4.5 7a1.5 1.5 0 1 0 0 3 1.5 1.5 0 0 0 0-3zM2.05 8a2.5 2.5 0 0 1 4.9 0H16v1H6.95a2.5 2.5 0 0 1-4.9 0H0V8h2.05zm9.45 4a1.5 1.5 0 1 0 0 3 1.5 1.5 0 0 0 0-3zm-2.45 1a2.5 2.5 0 0 1 4.9 0H16v1h-2.05a2.5 2.5 0 0 1-4.9 0H0v-1h9.05z"/>'),FD=Xu("Smartwatch",'<path d="M9 5a.5.5 0 0 0-1 0v3H6a.5.5 0 0 0 0 1h2.5a.5.5 0 0 0 .5-.5V5z"/><path d="M4 1.667v.383A2.5 2.5 0 0 0 2 4.5v7a2.5 2.5 0 0 0 2 2.45v.383C4 15.253 4.746 16 5.667 16h4.666c.92 0 1.667-.746 1.667-1.667v-.383a2.5 2.5 0 0 0 2-2.45V8h.5a.5.5 0 0 0 .5-.5v-2a.5.5 0 0 0-.5-.5H14v-.5a2.5 2.5 0 0 0-2-2.45v-.383C12 .747 11.254 0 10.333 0H5.667C4.747 0 4 .746 4 1.667zM4.5 3h7A1.5 1.5 0 0 1 13 4.5v7a1.5 1.5 0 0 1-1.5 1.5h-7A1.5 1.5 0 0 1 3 11.5v-7A1.5 1.5 0 0 1 4.5 3z"/>'),ED=Xu("Snow",'<path d="M8 16a.5.5 0 0 1-.5-.5v-1.293l-.646.647a.5.5 0 0 1-.707-.708L7.5 12.793V8.866l-3.4 1.963-.496 1.85a.5.5 0 1 1-.966-.26l.237-.882-1.12.646a.5.5 0 0 1-.5-.866l1.12-.646-.884-.237a.5.5 0 1 1 .26-.966l1.848.495L7 8 3.6 6.037l-1.85.495a.5.5 0 0 1-.258-.966l.883-.237-1.12-.646a.5.5 0 1 1 .5-.866l1.12.646-.237-.883a.5.5 0 1 1 .966-.258l.495 1.849L7.5 7.134V3.207L6.147 1.854a.5.5 0 1 1 .707-.708l.646.647V.5a.5.5 0 1 1 1 0v1.293l.647-.647a.5.5 0 1 1 .707.708L8.5 3.207v3.927l3.4-1.963.496-1.85a.5.5 0 1 1 .966.26l-.236.882 1.12-.646a.5.5 0 0 1 .5.866l-1.12.646.883.237a.5.5 0 1 1-.26.966l-1.848-.495L9 8l3.4 1.963 1.849-.495a.5.5 0 0 1 .259.966l-.883.237 1.12.646a.5.5 0 0 1-.5.866l-1.12-.646.236.883a.5.5 0 1 1-.966.258l-.495-1.849-3.4-1.963v3.927l1.353 1.353a.5.5 0 0 1-.707.708l-.647-.647V15.5a.5.5 0 0 1-.5.5z"/>'),$D=Xu("Snow2",'<path d="M8 16a.5.5 0 0 1-.5-.5v-1.293l-.646.647a.5.5 0 0 1-.707-.708L7.5 12.793v-1.086l-.646.647a.5.5 0 0 1-.707-.708L7.5 10.293V8.866l-1.236.713-.495 1.85a.5.5 0 1 1-.966-.26l.237-.882-.94.542-.496 1.85a.5.5 0 1 1-.966-.26l.237-.882-1.12.646a.5.5 0 0 1-.5-.866l1.12-.646-.884-.237a.5.5 0 1 1 .26-.966l1.848.495.94-.542-.882-.237a.5.5 0 1 1 .258-.966l1.85.495L7 8l-1.236-.713-1.849.495a.5.5 0 1 1-.258-.966l.883-.237-.94-.542-1.85.495a.5.5 0 0 1-.258-.966l.883-.237-1.12-.646a.5.5 0 1 1 .5-.866l1.12.646-.237-.883a.5.5 0 0 1 .966-.258l.495 1.849.94.542-.236-.883a.5.5 0 0 1 .966-.258l.495 1.849 1.236.713V5.707L6.147 4.354a.5.5 0 1 1 .707-.708l.646.647V3.207L6.147 1.854a.5.5 0 1 1 .707-.708l.646.647V.5a.5.5 0 0 1 1 0v1.293l.647-.647a.5.5 0 1 1 .707.708L8.5 3.207v1.086l.647-.647a.5.5 0 1 1 .707.708L8.5 5.707v1.427l1.236-.713.495-1.85a.5.5 0 1 1 .966.26l-.236.882.94-.542.495-1.85a.5.5 0 1 1 .966.26l-.236.882 1.12-.646a.5.5 0 0 1 .5.866l-1.12.646.883.237a.5.5 0 1 1-.26.966l-1.848-.495-.94.542.883.237a.5.5 0 1 1-.26.966l-1.848-.495L9 8l1.236.713 1.849-.495a.5.5 0 0 1 .259.966l-.883.237.94.542 1.849-.495a.5.5 0 0 1 .259.966l-.883.237 1.12.646a.5.5 0 0 1-.5.866l-1.12-.646.236.883a.5.5 0 1 1-.966.258l-.495-1.849-.94-.542.236.883a.5.5 0 0 1-.966.258L9.736 9.58 8.5 8.866v1.427l1.354 1.353a.5.5 0 0 1-.707.708l-.647-.647v1.086l1.354 1.353a.5.5 0 0 1-.707.708l-.647-.647V15.5a.5.5 0 0 1-.5.5z"/>'),RD=Xu("Snow3",'<path d="M8 7.5a.5.5 0 1 0 0 1 .5.5 0 0 0 0-1z"/><path d="M8 16a.5.5 0 0 1-.5-.5v-1.293l-.646.647a.5.5 0 0 1-.707-.708L7.5 12.793v-1.51l-2.053-1.232-1.348.778-.495 1.85a.5.5 0 1 1-.966-.26l.237-.882-1.12.646a.5.5 0 0 1-.5-.866l1.12-.646-.883-.237a.5.5 0 1 1 .258-.966l1.85.495L5 9.155v-2.31l-1.4-.808-1.85.495a.5.5 0 1 1-.259-.966l.884-.237-1.12-.646a.5.5 0 0 1 .5-.866l1.12.646-.237-.883a.5.5 0 1 1 .966-.258l.495 1.849 1.348.778L7.5 4.717v-1.51L6.147 1.854a.5.5 0 1 1 .707-.708l.646.647V.5a.5.5 0 0 1 1 0v1.293l.647-.647a.5.5 0 1 1 .707.708L8.5 3.207v1.51l2.053 1.232 1.348-.778.495-1.85a.5.5 0 1 1 .966.26l-.236.882 1.12-.646a.5.5 0 0 1 .5.866l-1.12.646.883.237a.5.5 0 1 1-.26.966l-1.848-.495-1.4.808v2.31l1.4.808 1.849-.495a.5.5 0 1 1 .259.966l-.883.237 1.12.646a.5.5 0 0 1-.5.866l-1.12-.646.236.883a.5.5 0 1 1-.966.258l-.495-1.849-1.348-.778L8.5 11.283v1.51l1.354 1.353a.5.5 0 0 1-.707.708l-.647-.647V15.5a.5.5 0 0 1-.5.5zm2-6.783V6.783l-2-1.2-2 1.2v2.434l2 1.2 2-1.2z"/>'),ND=Xu("SortAlphaDown",'<path fill-rule="evenodd" d="M10.082 5.629 9.664 7H8.598l1.789-5.332h1.234L13.402 7h-1.12l-.419-1.371h-1.781zm1.57-.785L11 2.687h-.047l-.652 2.157h1.351z"/><path d="M12.96 14H9.028v-.691l2.579-3.72v-.054H9.098v-.867h3.785v.691l-2.567 3.72v.054h2.645V14zM4.5 2.5a.5.5 0 0 0-1 0v9.793l-1.146-1.147a.5.5 0 0 0-.708.708l2 1.999.007.007a.497.497 0 0 0 .7-.006l2-2a.5.5 0 0 0-.707-.708L4.5 12.293V2.5z"/>'),WD=Xu("SortAlphaDownAlt",'<path d="M12.96 7H9.028v-.691l2.579-3.72v-.054H9.098v-.867h3.785v.691l-2.567 3.72v.054h2.645V7z"/><path fill-rule="evenodd" d="M10.082 12.629 9.664 14H8.598l1.789-5.332h1.234L13.402 14h-1.12l-.419-1.371h-1.781zm1.57-.785L11 9.688h-.047l-.652 2.156h1.351z"/><path d="M4.5 2.5a.5.5 0 0 0-1 0v9.793l-1.146-1.147a.5.5 0 0 0-.708.708l2 1.999.007.007a.497.497 0 0 0 .7-.006l2-2a.5.5 0 0 0-.707-.708L4.5 12.293V2.5z"/>'),UD=Xu("SortAlphaUp",'<path fill-rule="evenodd" d="M10.082 5.629 9.664 7H8.598l1.789-5.332h1.234L13.402 7h-1.12l-.419-1.371h-1.781zm1.57-.785L11 2.687h-.047l-.652 2.157h1.351z"/><path d="M12.96 14H9.028v-.691l2.579-3.72v-.054H9.098v-.867h3.785v.691l-2.567 3.72v.054h2.645V14zm-8.46-.5a.5.5 0 0 1-1 0V3.707L2.354 4.854a.5.5 0 1 1-.708-.708l2-1.999.007-.007a.498.498 0 0 1 .7.006l2 2a.5.5 0 1 1-.707.708L4.5 3.707V13.5z"/>'),GD=Xu("SortAlphaUpAlt",'<path d="M12.96 7H9.028v-.691l2.579-3.72v-.054H9.098v-.867h3.785v.691l-2.567 3.72v.054h2.645V7z"/><path fill-rule="evenodd" d="M10.082 12.629 9.664 14H8.598l1.789-5.332h1.234L13.402 14h-1.12l-.419-1.371h-1.781zm1.57-.785L11 9.688h-.047l-.652 2.156h1.351z"/><path d="M4.5 13.5a.5.5 0 0 1-1 0V3.707L2.354 4.854a.5.5 0 1 1-.708-.708l2-1.999.007-.007a.498.498 0 0 1 .7.006l2 2a.5.5 0 1 1-.707.708L4.5 3.707V13.5z"/>'),qD=Xu("SortDown",'<path d="M3.5 2.5a.5.5 0 0 0-1 0v8.793l-1.146-1.147a.5.5 0 0 0-.708.708l2 1.999.007.007a.497.497 0 0 0 .7-.006l2-2a.5.5 0 0 0-.707-.708L3.5 11.293V2.5zm3.5 1a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zM7.5 6a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zm0 3a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1h-3zm0 3a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1z"/>'),JD=Xu("SortDownAlt",'<path d="M3.5 3.5a.5.5 0 0 0-1 0v8.793l-1.146-1.147a.5.5 0 0 0-.708.708l2 1.999.007.007a.497.497 0 0 0 .7-.006l2-2a.5.5 0 0 0-.707-.708L3.5 12.293V3.5zm4 .5a.5.5 0 0 1 0-1h1a.5.5 0 0 1 0 1h-1zm0 3a.5.5 0 0 1 0-1h3a.5.5 0 0 1 0 1h-3zm0 3a.5.5 0 0 1 0-1h5a.5.5 0 0 1 0 1h-5zM7 12.5a.5.5 0 0 0 .5.5h7a.5.5 0 0 0 0-1h-7a.5.5 0 0 0-.5.5z"/>'),KD=Xu("SortNumericDown",'<path d="M12.438 1.668V7H11.39V2.684h-.051l-1.211.859v-.969l1.262-.906h1.046z"/><path fill-rule="evenodd" d="M11.36 14.098c-1.137 0-1.708-.657-1.762-1.278h1.004c.058.223.343.45.773.45.824 0 1.164-.829 1.133-1.856h-.059c-.148.39-.57.742-1.261.742-.91 0-1.72-.613-1.72-1.758 0-1.148.848-1.835 1.973-1.835 1.09 0 2.063.636 2.063 2.687 0 1.867-.723 2.848-2.145 2.848zm.062-2.735c.504 0 .933-.336.933-.972 0-.633-.398-1.008-.94-1.008-.52 0-.927.375-.927 1 0 .64.418.98.934.98z"/><path d="M4.5 2.5a.5.5 0 0 0-1 0v9.793l-1.146-1.147a.5.5 0 0 0-.708.708l2 1.999.007.007a.497.497 0 0 0 .7-.006l2-2a.5.5 0 0 0-.707-.708L4.5 12.293V2.5z"/>'),XD=Xu("SortNumericDownAlt",'<path fill-rule="evenodd" d="M11.36 7.098c-1.137 0-1.708-.657-1.762-1.278h1.004c.058.223.343.45.773.45.824 0 1.164-.829 1.133-1.856h-.059c-.148.39-.57.742-1.261.742-.91 0-1.72-.613-1.72-1.758 0-1.148.848-1.836 1.973-1.836 1.09 0 2.063.637 2.063 2.688 0 1.867-.723 2.848-2.145 2.848zm.062-2.735c.504 0 .933-.336.933-.972 0-.633-.398-1.008-.94-1.008-.52 0-.927.375-.927 1 0 .64.418.98.934.98z"/><path d="M12.438 8.668V14H11.39V9.684h-.051l-1.211.859v-.969l1.262-.906h1.046zM4.5 2.5a.5.5 0 0 0-1 0v9.793l-1.146-1.147a.5.5 0 0 0-.708.708l2 1.999.007.007a.497.497 0 0 0 .7-.006l2-2a.5.5 0 0 0-.707-.708L4.5 12.293V2.5z"/>'),ZD=Xu("SortNumericUp",'<path d="M12.438 1.668V7H11.39V2.684h-.051l-1.211.859v-.969l1.262-.906h1.046z"/><path fill-rule="evenodd" d="M11.36 14.098c-1.137 0-1.708-.657-1.762-1.278h1.004c.058.223.343.45.773.45.824 0 1.164-.829 1.133-1.856h-.059c-.148.39-.57.742-1.261.742-.91 0-1.72-.613-1.72-1.758 0-1.148.848-1.835 1.973-1.835 1.09 0 2.063.636 2.063 2.687 0 1.867-.723 2.848-2.145 2.848zm.062-2.735c.504 0 .933-.336.933-.972 0-.633-.398-1.008-.94-1.008-.52 0-.927.375-.927 1 0 .64.418.98.934.98z"/><path d="M4.5 13.5a.5.5 0 0 1-1 0V3.707L2.354 4.854a.5.5 0 1 1-.708-.708l2-1.999.007-.007a.498.498 0 0 1 .7.006l2 2a.5.5 0 1 1-.707.708L4.5 3.707V13.5z"/>'),QD=Xu("SortNumericUpAlt",'<path fill-rule="evenodd" d="M11.36 7.098c-1.137 0-1.708-.657-1.762-1.278h1.004c.058.223.343.45.773.45.824 0 1.164-.829 1.133-1.856h-.059c-.148.39-.57.742-1.261.742-.91 0-1.72-.613-1.72-1.758 0-1.148.848-1.836 1.973-1.836 1.09 0 2.063.637 2.063 2.688 0 1.867-.723 2.848-2.145 2.848zm.062-2.735c.504 0 .933-.336.933-.972 0-.633-.398-1.008-.94-1.008-.52 0-.927.375-.927 1 0 .64.418.98.934.98z"/><path d="M12.438 8.668V14H11.39V9.684h-.051l-1.211.859v-.969l1.262-.906h1.046zM4.5 13.5a.5.5 0 0 1-1 0V3.707L2.354 4.854a.5.5 0 1 1-.708-.708l2-1.999.007-.007a.498.498 0 0 1 .7.006l2 2a.5.5 0 1 1-.707.708L4.5 3.707V13.5z"/>'),eC=Xu("SortUp",'<path d="M3.5 12.5a.5.5 0 0 1-1 0V3.707L1.354 4.854a.5.5 0 1 1-.708-.708l2-1.999.007-.007a.498.498 0 0 1 .7.006l2 2a.5.5 0 1 1-.707.708L3.5 3.707V12.5zm3.5-9a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zM7.5 6a.5.5 0 0 0 0 1h5a.5.5 0 0 0 0-1h-5zm0 3a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1h-3zm0 3a.5.5 0 0 0 0 1h1a.5.5 0 0 0 0-1h-1z"/>'),tC=Xu("SortUpAlt",'<path d="M3.5 13.5a.5.5 0 0 1-1 0V4.707L1.354 5.854a.5.5 0 1 1-.708-.708l2-1.999.007-.007a.498.498 0 0 1 .7.006l2 2a.5.5 0 1 1-.707.708L3.5 4.707V13.5zm4-9.5a.5.5 0 0 1 0-1h1a.5.5 0 0 1 0 1h-1zm0 3a.5.5 0 0 1 0-1h3a.5.5 0 0 1 0 1h-3zm0 3a.5.5 0 0 1 0-1h5a.5.5 0 0 1 0 1h-5zM7 12.5a.5.5 0 0 0 .5.5h7a.5.5 0 0 0 0-1h-7a.5.5 0 0 0-.5.5z"/>'),aC=Xu("Soundwave",'<path fill-rule="evenodd" d="M8.5 2a.5.5 0 0 1 .5.5v11a.5.5 0 0 1-1 0v-11a.5.5 0 0 1 .5-.5zm-2 2a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-1 0v-7a.5.5 0 0 1 .5-.5zm4 0a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-1 0v-7a.5.5 0 0 1 .5-.5zm-6 1.5A.5.5 0 0 1 5 6v4a.5.5 0 0 1-1 0V6a.5.5 0 0 1 .5-.5zm8 0a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0V6a.5.5 0 0 1 .5-.5zm-10 1A.5.5 0 0 1 3 7v2a.5.5 0 0 1-1 0V7a.5.5 0 0 1 .5-.5zm12 0a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0V7a.5.5 0 0 1 .5-.5z"/>'),nC=Xu("Speaker",'<path d="M12 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h8zM4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4z"/><path d="M8 4.75a.75.75 0 1 1 0-1.5.75.75 0 0 1 0 1.5zM8 6a2 2 0 1 0 0-4 2 2 0 0 0 0 4zm0 3a1.5 1.5 0 1 0 0 3 1.5 1.5 0 0 0 0-3zm-3.5 1.5a3.5 3.5 0 1 1 7 0 3.5 3.5 0 0 1-7 0z"/>'),rC=Xu("SpeakerFill",'<path d="M9 4a1 1 0 1 1-2 0 1 1 0 0 1 2 0zm-2.5 6.5a1.5 1.5 0 1 1 3 0 1.5 1.5 0 0 1-3 0z"/><path d="M4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4zm6 4a2 2 0 1 1-4 0 2 2 0 0 1 4 0zM8 7a3.5 3.5 0 1 1 0 7 3.5 3.5 0 0 1 0-7z"/>'),iC=Xu("Speedometer",'<path d="M8 2a.5.5 0 0 1 .5.5V4a.5.5 0 0 1-1 0V2.5A.5.5 0 0 1 8 2zM3.732 3.732a.5.5 0 0 1 .707 0l.915.914a.5.5 0 1 1-.708.708l-.914-.915a.5.5 0 0 1 0-.707zM2 8a.5.5 0 0 1 .5-.5h1.586a.5.5 0 0 1 0 1H2.5A.5.5 0 0 1 2 8zm9.5 0a.5.5 0 0 1 .5-.5h1.5a.5.5 0 0 1 0 1H12a.5.5 0 0 1-.5-.5zm.754-4.246a.389.389 0 0 0-.527-.02L7.547 7.31A.91.91 0 1 0 8.85 8.569l3.434-4.297a.389.389 0 0 0-.029-.518z"/><path fill-rule="evenodd" d="M6.664 15.889A8 8 0 1 1 9.336.11a8 8 0 0 1-2.672 15.78zm-4.665-4.283A11.945 11.945 0 0 1 8 10c2.186 0 4.236.585 6.001 1.606a7 7 0 1 0-12.002 0z"/>'),oC=Xu("Speedometer2",'<path d="M8 4a.5.5 0 0 1 .5.5V6a.5.5 0 0 1-1 0V4.5A.5.5 0 0 1 8 4zM3.732 5.732a.5.5 0 0 1 .707 0l.915.914a.5.5 0 1 1-.708.708l-.914-.915a.5.5 0 0 1 0-.707zM2 10a.5.5 0 0 1 .5-.5h1.586a.5.5 0 0 1 0 1H2.5A.5.5 0 0 1 2 10zm9.5 0a.5.5 0 0 1 .5-.5h1.5a.5.5 0 0 1 0 1H12a.5.5 0 0 1-.5-.5zm.754-4.246a.389.389 0 0 0-.527-.02L7.547 9.31a.91.91 0 1 0 1.302 1.258l3.434-4.297a.389.389 0 0 0-.029-.518z"/><path fill-rule="evenodd" d="M0 10a8 8 0 1 1 15.547 2.661c-.442 1.253-1.845 1.602-2.932 1.25C11.309 13.488 9.475 13 8 13c-1.474 0-3.31.488-4.615.911-1.087.352-2.49.003-2.932-1.25A7.988 7.988 0 0 1 0 10zm8-7a7 7 0 0 0-6.603 9.329c.203.575.923.876 1.68.63C4.397 12.533 6.358 12 8 12s3.604.532 4.923.96c.757.245 1.477-.056 1.68-.631A7 7 0 0 0 8 3z"/>'),sC=Xu("Spellcheck",'<path d="M8.217 11.068c1.216 0 1.948-.869 1.948-2.31v-.702c0-1.44-.727-2.305-1.929-2.305-.742 0-1.328.347-1.499.889h-.063V3.983h-1.29V11h1.27v-.791h.064c.21.532.776.86 1.499.86zm-.43-1.025c-.66 0-1.113-.518-1.113-1.28V8.12c0-.825.42-1.343 1.098-1.343.684 0 1.075.518 1.075 1.416v.45c0 .888-.386 1.401-1.06 1.401zm-5.583 1.035c.767 0 1.201-.356 1.406-.737h.059V11h1.216V7.519c0-1.314-.947-1.783-2.11-1.783C1.355 5.736.75 6.42.69 7.27h1.216c.064-.323.313-.552.84-.552.527 0 .864.249.864.771v.464H2.346C1.145 7.953.5 8.568.5 9.496c0 .977.693 1.582 1.704 1.582zm.42-.947c-.44 0-.845-.235-.845-.718 0-.395.269-.684.84-.684h.991v.538c0 .503-.444.864-.986.864zm8.897.567c-.577-.4-.9-1.088-.9-1.983v-.65c0-1.42.894-2.338 2.305-2.338 1.352 0 2.119.82 2.139 1.806h-1.187c-.04-.351-.283-.776-.918-.776-.674 0-1.045.517-1.045 1.328v.625c0 .468.121.834.343 1.067l-.737.92z"/><path d="M14.469 9.414a.75.75 0 0 1 .117 1.055l-4 5a.75.75 0 0 1-1.116.061l-2.5-2.5a.75.75 0 1 1 1.06-1.06l1.908 1.907 3.476-4.346a.75.75 0 0 1 1.055-.117z"/>'),lC=Xu("Square",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/>'),cC=Xu("SquareFill",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2z"/>'),uC=Xu("SquareHalf",'<path d="M8 15V1h6a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H8zm6 1a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12z"/>'),dC=Xu("Stack",'<path d="m14.12 10.163 1.715.858c.22.11.22.424 0 .534L8.267 15.34a.598.598 0 0 1-.534 0L.165 11.555a.299.299 0 0 1 0-.534l1.716-.858 5.317 2.659c.505.252 1.1.252 1.604 0l5.317-2.66zM7.733.063a.598.598 0 0 1 .534 0l7.568 3.784a.3.3 0 0 1 0 .535L8.267 8.165a.598.598 0 0 1-.534 0L.165 4.382a.299.299 0 0 1 0-.535L7.733.063z"/><path d="m14.12 6.576 1.715.858c.22.11.22.424 0 .534l-7.568 3.784a.598.598 0 0 1-.534 0L.165 7.968a.299.299 0 0 1 0-.534l1.716-.858 5.317 2.659c.505.252 1.1.252 1.604 0l5.317-2.659z"/>'),hC=Xu("Star",'<path d="M2.866 14.85c-.078.444.36.791.746.593l4.39-2.256 4.389 2.256c.386.198.824-.149.746-.592l-.83-4.73 3.522-3.356c.33-.314.16-.888-.282-.95l-4.898-.696L8.465.792a.513.513 0 0 0-.927 0L5.354 5.12l-4.898.696c-.441.062-.612.636-.283.95l3.523 3.356-.83 4.73zm4.905-2.767-3.686 1.894.694-3.957a.565.565 0 0 0-.163-.505L1.71 6.745l4.052-.576a.525.525 0 0 0 .393-.288L8 2.223l1.847 3.658a.525.525 0 0 0 .393.288l4.052.575-2.906 2.77a.565.565 0 0 0-.163.506l.694 3.957-3.686-1.894a.503.503 0 0 0-.461 0z"/>'),fC=Xu("StarFill",'<path d="M3.612 15.443c-.386.198-.824-.149-.746-.592l.83-4.73L.173 6.765c-.329-.314-.158-.888.283-.95l4.898-.696L7.538.792c.197-.39.73-.39.927 0l2.184 4.327 4.898.696c.441.062.612.636.282.95l-3.522 3.356.83 4.73c.078.443-.36.79-.746.592L8 13.187l-4.389 2.256z"/>'),pC=Xu("StarHalf",'<path d="M5.354 5.119 7.538.792A.516.516 0 0 1 8 .5c.183 0 .366.097.465.292l2.184 4.327 4.898.696A.537.537 0 0 1 16 6.32a.548.548 0 0 1-.17.445l-3.523 3.356.83 4.73c.078.443-.36.79-.746.592L8 13.187l-4.389 2.256a.52.52 0 0 1-.146.05c-.342.06-.668-.254-.6-.642l.83-4.73L.173 6.765a.55.55 0 0 1-.172-.403.58.58 0 0 1 .085-.302.513.513 0 0 1 .37-.245l4.898-.696zM8 12.027a.5.5 0 0 1 .232.056l3.686 1.894-.694-3.957a.565.565 0 0 1 .162-.505l2.907-2.77-4.052-.576a.525.525 0 0 1-.393-.288L8.001 2.223 8 2.226v9.8z"/>'),mC=Xu("Stars",'<path d="M7.657 6.247c.11-.33.576-.33.686 0l.645 1.937a2.89 2.89 0 0 0 1.829 1.828l1.936.645c.33.11.33.576 0 .686l-1.937.645a2.89 2.89 0 0 0-1.828 1.829l-.645 1.936a.361.361 0 0 1-.686 0l-.645-1.937a2.89 2.89 0 0 0-1.828-1.828l-1.937-.645a.361.361 0 0 1 0-.686l1.937-.645a2.89 2.89 0 0 0 1.828-1.828l.645-1.937zM3.794 1.148a.217.217 0 0 1 .412 0l.387 1.162c.173.518.579.924 1.097 1.097l1.162.387a.217.217 0 0 1 0 .412l-1.162.387A1.734 1.734 0 0 0 4.593 5.69l-.387 1.162a.217.217 0 0 1-.412 0L3.407 5.69A1.734 1.734 0 0 0 2.31 4.593l-1.162-.387a.217.217 0 0 1 0-.412l1.162-.387A1.734 1.734 0 0 0 3.407 2.31l.387-1.162zM10.863.099a.145.145 0 0 1 .274 0l.258.774c.115.346.386.617.732.732l.774.258a.145.145 0 0 1 0 .274l-.774.258a1.156 1.156 0 0 0-.732.732l-.258.774a.145.145 0 0 1-.274 0l-.258-.774a1.156 1.156 0 0 0-.732-.732L9.1 2.137a.145.145 0 0 1 0-.274l.774-.258c.346-.115.617-.386.732-.732L10.863.1z"/>'),vC=Xu("Stickies",'<path d="M1.5 0A1.5 1.5 0 0 0 0 1.5V13a1 1 0 0 0 1 1V1.5a.5.5 0 0 1 .5-.5H14a1 1 0 0 0-1-1H1.5z"/><path d="M3.5 2A1.5 1.5 0 0 0 2 3.5v11A1.5 1.5 0 0 0 3.5 16h6.086a1.5 1.5 0 0 0 1.06-.44l4.915-4.914A1.5 1.5 0 0 0 16 9.586V3.5A1.5 1.5 0 0 0 14.5 2h-11zM3 3.5a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 .5.5V9h-4.5A1.5 1.5 0 0 0 9 10.5V15H3.5a.5.5 0 0 1-.5-.5v-11zm7 11.293V10.5a.5.5 0 0 1 .5-.5h4.293L10 14.793z"/>'),_C=Xu("StickiesFill",'<path d="M0 1.5V13a1 1 0 0 0 1 1V1.5a.5.5 0 0 1 .5-.5H14a1 1 0 0 0-1-1H1.5A1.5 1.5 0 0 0 0 1.5z"/><path d="M3.5 2A1.5 1.5 0 0 0 2 3.5v11A1.5 1.5 0 0 0 3.5 16h6.086a1.5 1.5 0 0 0 1.06-.44l4.915-4.914A1.5 1.5 0 0 0 16 9.586V3.5A1.5 1.5 0 0 0 14.5 2h-11zm6 8.5a1 1 0 0 1 1-1h4.396a.25.25 0 0 1 .177.427l-5.146 5.146a.25.25 0 0 1-.427-.177V10.5z"/>'),gC=Xu("Sticky",'<path d="M2.5 1A1.5 1.5 0 0 0 1 2.5v11A1.5 1.5 0 0 0 2.5 15h6.086a1.5 1.5 0 0 0 1.06-.44l4.915-4.914A1.5 1.5 0 0 0 15 8.586V2.5A1.5 1.5 0 0 0 13.5 1h-11zM2 2.5a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 .5.5V8H9.5A1.5 1.5 0 0 0 8 9.5V14H2.5a.5.5 0 0 1-.5-.5v-11zm7 11.293V9.5a.5.5 0 0 1 .5-.5h4.293L9 13.793z"/>'),yC=Xu("StickyFill",'<path d="M2.5 1A1.5 1.5 0 0 0 1 2.5v11A1.5 1.5 0 0 0 2.5 15h6.086a1.5 1.5 0 0 0 1.06-.44l4.915-4.914A1.5 1.5 0 0 0 15 8.586V2.5A1.5 1.5 0 0 0 13.5 1h-11zm6 8.5a1 1 0 0 1 1-1h4.396a.25.25 0 0 1 .177.427l-5.146 5.146a.25.25 0 0 1-.427-.177V9.5z"/>'),bC=Xu("Stop",'<path d="M3.5 5A1.5 1.5 0 0 1 5 3.5h6A1.5 1.5 0 0 1 12.5 5v6a1.5 1.5 0 0 1-1.5 1.5H5A1.5 1.5 0 0 1 3.5 11V5zM5 4.5a.5.5 0 0 0-.5.5v6a.5.5 0 0 0 .5.5h6a.5.5 0 0 0 .5-.5V5a.5.5 0 0 0-.5-.5H5z"/>'),MC=Xu("StopBtn",'<path d="M6.5 5A1.5 1.5 0 0 0 5 6.5v3A1.5 1.5 0 0 0 6.5 11h3A1.5 1.5 0 0 0 11 9.5v-3A1.5 1.5 0 0 0 9.5 5h-3z"/><path d="M0 4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V4zm15 0a1 1 0 0 0-1-1H2a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V4z"/>'),zC=Xu("StopBtnFill",'<path d="M0 12V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2zm6.5-7A1.5 1.5 0 0 0 5 6.5v3A1.5 1.5 0 0 0 6.5 11h3A1.5 1.5 0 0 0 11 9.5v-3A1.5 1.5 0 0 0 9.5 5h-3z"/>'),LC=Xu("StopCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M5 6.5A1.5 1.5 0 0 1 6.5 5h3A1.5 1.5 0 0 1 11 6.5v3A1.5 1.5 0 0 1 9.5 11h-3A1.5 1.5 0 0 1 5 9.5v-3z"/>'),wC=Xu("StopCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM6.5 5A1.5 1.5 0 0 0 5 6.5v3A1.5 1.5 0 0 0 6.5 11h3A1.5 1.5 0 0 0 11 9.5v-3A1.5 1.5 0 0 0 9.5 5h-3z"/>'),HC=Xu("StopFill",'<path d="M5 3.5h6A1.5 1.5 0 0 1 12.5 5v6a1.5 1.5 0 0 1-1.5 1.5H5A1.5 1.5 0 0 1 3.5 11V5A1.5 1.5 0 0 1 5 3.5z"/>'),kC=Xu("Stoplights",'<path d="M8 5a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3zm0 4a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3zm1.5 2.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/><path d="M4 2a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2h2c-.167.5-.8 1.6-2 2v2h2c-.167.5-.8 1.6-2 2v2h2c-.167.5-.8 1.6-2 2v1a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2v-1c-1.2-.4-1.833-1.5-2-2h2V8c-1.2-.4-1.833-1.5-2-2h2V4c-1.2-.4-1.833-1.5-2-2h2zm2-1a1 1 0 0 0-1 1v11a1 1 0 0 0 1 1h4a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H6z"/>'),AC=Xu("StoplightsFill",'<path fill-rule="evenodd" d="M6 0a2 2 0 0 0-2 2H2c.167.5.8 1.6 2 2v2H2c.167.5.8 1.6 2 2v2H2c.167.5.8 1.6 2 2v1a2 2 0 0 0 2 2h4a2 2 0 0 0 2-2v-1c1.2-.4 1.833-1.5 2-2h-2V8c1.2-.4 1.833-1.5 2-2h-2V4c1.2-.4 1.833-1.5 2-2h-2a2 2 0 0 0-2-2H6zm3.5 3.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0 4a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zM8 13a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),SC=Xu("Stopwatch",'<path d="M8.5 5.6a.5.5 0 1 0-1 0v2.9h-3a.5.5 0 0 0 0 1H8a.5.5 0 0 0 .5-.5V5.6z"/><path d="M6.5 1A.5.5 0 0 1 7 .5h2a.5.5 0 0 1 0 1v.57c1.36.196 2.594.78 3.584 1.64a.715.715 0 0 1 .012-.013l.354-.354-.354-.353a.5.5 0 0 1 .707-.708l1.414 1.415a.5.5 0 1 1-.707.707l-.353-.354-.354.354a.512.512 0 0 1-.013.012A7 7 0 1 1 7 2.071V1.5a.5.5 0 0 1-.5-.5zM8 3a6 6 0 1 0 .001 12A6 6 0 0 0 8 3z"/>'),VC=Xu("StopwatchFill",'<path d="M6.5 0a.5.5 0 0 0 0 1H7v1.07A7.001 7.001 0 0 0 8 16a7 7 0 0 0 5.29-11.584.531.531 0 0 0 .013-.012l.354-.354.353.354a.5.5 0 1 0 .707-.707l-1.414-1.415a.5.5 0 1 0-.707.707l.354.354-.354.354a.717.717 0 0 0-.012.012A6.973 6.973 0 0 0 9 2.071V1h.5a.5.5 0 0 0 0-1h-3zm2 5.6V9a.5.5 0 0 1-.5.5H4.5a.5.5 0 0 1 0-1h3V5.6a.5.5 0 1 1 1 0z"/>'),OC=Xu("Subtract",'<path d="M0 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2h2a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2v-2H2a2 2 0 0 1-2-2V2zm2-1a1 1 0 0 0-1 1v8a1 1 0 0 0 1 1h8a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H2z"/>'),TC=Xu("SuitClub",'<path d="M8 1a3.25 3.25 0 0 0-3.25 3.25c0 .186 0 .29.016.41.014.12.045.27.12.527l.19.665-.692-.028a3.25 3.25 0 1 0 2.357 5.334.5.5 0 0 1 .844.518l-.003.005-.006.015-.024.055a21.893 21.893 0 0 1-.438.92 22.38 22.38 0 0 1-1.266 2.197c-.013.018-.02.05.001.09.01.02.021.03.03.036A.036.036 0 0 0 5.9 15h4.2c.01 0 .016-.002.022-.006a.092.092 0 0 0 .029-.035c.02-.04.014-.073.001-.091a22.875 22.875 0 0 1-1.704-3.117l-.024-.054-.006-.015-.002-.004a.5.5 0 0 1 .838-.524c.601.7 1.516 1.168 2.496 1.168a3.25 3.25 0 1 0-.139-6.498l-.699.03.199-.671c.14-.47.14-.745.139-.927V4.25A3.25 3.25 0 0 0 8 1zm2.207 12.024c.225.405.487.848.78 1.294C11.437 15 10.975 16 10.1 16H5.9c-.876 0-1.338-1-.887-1.683.291-.442.552-.88.776-1.283a4.25 4.25 0 1 1-2.007-8.187 2.79 2.79 0 0 1-.009-.064c-.023-.187-.023-.348-.023-.52V4.25a4.25 4.25 0 0 1 8.5 0c0 .14 0 .333-.04.596a4.25 4.25 0 0 1-.46 8.476 4.186 4.186 0 0 1-1.543-.298z"/>'),DC=Xu("SuitClubFill",'<path d="M11.5 12.5a3.493 3.493 0 0 1-2.684-1.254 19.92 19.92 0 0 0 1.582 2.907c.231.35-.02.847-.438.847H6.04c-.419 0-.67-.497-.438-.847a19.919 19.919 0 0 0 1.582-2.907 3.5 3.5 0 1 1-2.538-5.743 3.5 3.5 0 1 1 6.708 0A3.5 3.5 0 1 1 11.5 12.5z"/>'),CC=Xu("SuitDiamond",'<path d="M8.384 1.226a.463.463 0 0 0-.768 0l-4.56 6.468a.537.537 0 0 0 0 .612l4.56 6.469a.463.463 0 0 0 .768 0l4.56-6.469a.537.537 0 0 0 0-.612l-4.56-6.468zM6.848.613a1.39 1.39 0 0 1 2.304 0l4.56 6.468a1.61 1.61 0 0 1 0 1.838l-4.56 6.468a1.39 1.39 0 0 1-2.304 0L2.288 8.92a1.61 1.61 0 0 1 0-1.838L6.848.613z"/>'),jC=Xu("SuitDiamondFill",'<path d="M2.45 7.4 7.2 1.067a1 1 0 0 1 1.6 0L13.55 7.4a1 1 0 0 1 0 1.2L8.8 14.933a1 1 0 0 1-1.6 0L2.45 8.6a1 1 0 0 1 0-1.2z"/>'),xC=Xu("SuitHeart",'<path d="m8 6.236-.894-1.789c-.222-.443-.607-1.08-1.152-1.595C5.418 2.345 4.776 2 4 2 2.324 2 1 3.326 1 4.92c0 1.211.554 2.066 1.868 3.37.337.334.721.695 1.146 1.093C5.122 10.423 6.5 11.717 8 13.447c1.5-1.73 2.878-3.024 3.986-4.064.425-.398.81-.76 1.146-1.093C14.446 6.986 15 6.131 15 4.92 15 3.326 13.676 2 12 2c-.777 0-1.418.345-1.954.852-.545.515-.93 1.152-1.152 1.595L8 6.236zm.392 8.292a.513.513 0 0 1-.784 0c-1.601-1.902-3.05-3.262-4.243-4.381C1.3 8.208 0 6.989 0 4.92 0 2.755 1.79 1 4 1c1.6 0 2.719 1.05 3.404 2.008.26.365.458.716.596.992a7.55 7.55 0 0 1 .596-.992C9.281 2.049 10.4 1 12 1c2.21 0 4 1.755 4 3.92 0 2.069-1.3 3.288-3.365 5.227-1.193 1.12-2.642 2.48-4.243 4.38z"/>'),BC=Xu("SuitHeartFill",'<path d="M4 1c2.21 0 4 1.755 4 3.92C8 2.755 9.79 1 12 1s4 1.755 4 3.92c0 3.263-3.234 4.414-7.608 9.608a.513.513 0 0 1-.784 0C3.234 9.334 0 8.183 0 4.92 0 2.755 1.79 1 4 1z"/>'),YC=Xu("SuitSpade",'<path d="M8 0a.5.5 0 0 1 .429.243c1.359 2.265 2.925 3.682 4.25 4.882.096.086.19.17.282.255C14.308 6.604 15.5 7.747 15.5 9.5a4 4 0 0 1-5.406 3.746c.235.39.491.782.722 1.131.434.659-.01 1.623-.856 1.623H6.04c-.845 0-1.29-.964-.856-1.623.263-.397.51-.777.728-1.134A4 4 0 0 1 .5 9.5c0-1.753 1.192-2.896 2.539-4.12l.281-.255c1.326-1.2 2.892-2.617 4.251-4.882A.5.5 0 0 1 8 0zM3.711 6.12C2.308 7.396 1.5 8.253 1.5 9.5a3 3 0 0 0 5.275 1.956.5.5 0 0 1 .868.43c-.094.438-.33.932-.611 1.428a29.247 29.247 0 0 1-1.013 1.614.03.03 0 0 0-.005.018.074.074 0 0 0 .024.054h3.924a.074.074 0 0 0 .024-.054.03.03 0 0 0-.005-.018c-.3-.455-.658-1.005-.96-1.535-.294-.514-.57-1.064-.664-1.507a.5.5 0 0 1 .868-.43A3 3 0 0 0 14.5 9.5c0-1.247-.808-2.104-2.211-3.38L12 5.86c-1.196-1.084-2.668-2.416-4-4.424-1.332 2.008-2.804 3.34-4 4.422l-.289.261z"/>'),PC=Xu("SuitSpadeFill",'<path d="M7.184 11.246A3.5 3.5 0 0 1 1 9c0-1.602 1.14-2.633 2.66-4.008C4.986 3.792 6.602 2.33 8 0c1.398 2.33 3.014 3.792 4.34 4.992C13.86 6.367 15 7.398 15 9a3.5 3.5 0 0 1-6.184 2.246 19.92 19.92 0 0 0 1.582 2.907c.231.35-.02.847-.438.847H6.04c-.419 0-.67-.497-.438-.847a19.919 19.919 0 0 0 1.582-2.907z"/>'),IC=Xu("Sun",'<path d="M8 11a3 3 0 1 1 0-6 3 3 0 0 1 0 6zm0 1a4 4 0 1 0 0-8 4 4 0 0 0 0 8zM8 0a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 0zm0 13a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 13zm8-5a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2a.5.5 0 0 1 .5.5zM3 8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2A.5.5 0 0 1 3 8zm10.657-5.657a.5.5 0 0 1 0 .707l-1.414 1.415a.5.5 0 1 1-.707-.708l1.414-1.414a.5.5 0 0 1 .707 0zm-9.193 9.193a.5.5 0 0 1 0 .707L3.05 13.657a.5.5 0 0 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zm9.193 2.121a.5.5 0 0 1-.707 0l-1.414-1.414a.5.5 0 0 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .707zM4.464 4.465a.5.5 0 0 1-.707 0L2.343 3.05a.5.5 0 1 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .708z"/>'),FC=Xu("SunFill",'<path d="M8 12a4 4 0 1 0 0-8 4 4 0 0 0 0 8zM8 0a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 0zm0 13a.5.5 0 0 1 .5.5v2a.5.5 0 0 1-1 0v-2A.5.5 0 0 1 8 13zm8-5a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2a.5.5 0 0 1 .5.5zM3 8a.5.5 0 0 1-.5.5h-2a.5.5 0 0 1 0-1h2A.5.5 0 0 1 3 8zm10.657-5.657a.5.5 0 0 1 0 .707l-1.414 1.415a.5.5 0 1 1-.707-.708l1.414-1.414a.5.5 0 0 1 .707 0zm-9.193 9.193a.5.5 0 0 1 0 .707L3.05 13.657a.5.5 0 0 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zm9.193 2.121a.5.5 0 0 1-.707 0l-1.414-1.414a.5.5 0 0 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .707zM4.464 4.465a.5.5 0 0 1-.707 0L2.343 3.05a.5.5 0 1 1 .707-.707l1.414 1.414a.5.5 0 0 1 0 .708z"/>'),EC=Xu("Sunglasses",'<path d="M3 5a2 2 0 0 0-2 2v.5H.5a.5.5 0 0 0 0 1H1V9a2 2 0 0 0 2 2h1a3 3 0 0 0 3-3 1 1 0 1 1 2 0 3 3 0 0 0 3 3h1a2 2 0 0 0 2-2v-.5h.5a.5.5 0 0 0 0-1H15V7a2 2 0 0 0-2-2h-2a2 2 0 0 0-1.888 1.338A1.99 1.99 0 0 0 8 6a1.99 1.99 0 0 0-1.112.338A2 2 0 0 0 5 5H3zm0 1h.941c.264 0 .348.356.112.474l-.457.228a2 2 0 0 0-.894.894l-.228.457C2.356 8.289 2 8.205 2 7.94V7a1 1 0 0 1 1-1z"/>'),$C=Xu("Sunrise",'<path d="M7.646 1.146a.5.5 0 0 1 .708 0l1.5 1.5a.5.5 0 0 1-.708.708L8.5 2.707V4.5a.5.5 0 0 1-1 0V2.707l-.646.647a.5.5 0 1 1-.708-.708l1.5-1.5zM2.343 4.343a.5.5 0 0 1 .707 0l1.414 1.414a.5.5 0 0 1-.707.707L2.343 5.05a.5.5 0 0 1 0-.707zm11.314 0a.5.5 0 0 1 0 .707l-1.414 1.414a.5.5 0 1 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zM8 7a3 3 0 0 1 2.599 4.5H5.4A3 3 0 0 1 8 7zm3.71 4.5a4 4 0 1 0-7.418 0H.499a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1h-3.79zM0 10a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2A.5.5 0 0 1 0 10zm13 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),RC=Xu("SunriseFill",'<path d="M7.646 1.146a.5.5 0 0 1 .708 0l1.5 1.5a.5.5 0 0 1-.708.708L8.5 2.707V4.5a.5.5 0 0 1-1 0V2.707l-.646.647a.5.5 0 1 1-.708-.708l1.5-1.5zM2.343 4.343a.5.5 0 0 1 .707 0l1.414 1.414a.5.5 0 0 1-.707.707L2.343 5.05a.5.5 0 0 1 0-.707zm11.314 0a.5.5 0 0 1 0 .707l-1.414 1.414a.5.5 0 1 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zM11.709 11.5a4 4 0 1 0-7.418 0H.5a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1h-3.79zM0 10a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2A.5.5 0 0 1 0 10zm13 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),NC=Xu("Sunset",'<path d="M7.646 4.854a.5.5 0 0 0 .708 0l1.5-1.5a.5.5 0 0 0-.708-.708l-.646.647V1.5a.5.5 0 0 0-1 0v1.793l-.646-.647a.5.5 0 1 0-.708.708l1.5 1.5zm-5.303-.51a.5.5 0 0 1 .707 0l1.414 1.413a.5.5 0 0 1-.707.707L2.343 5.05a.5.5 0 0 1 0-.707zm11.314 0a.5.5 0 0 1 0 .706l-1.414 1.414a.5.5 0 1 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zM8 7a3 3 0 0 1 2.599 4.5H5.4A3 3 0 0 1 8 7zm3.71 4.5a4 4 0 1 0-7.418 0H.499a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1h-3.79zM0 10a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2A.5.5 0 0 1 0 10zm13 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),WC=Xu("SunsetFill",'<path d="M7.646 4.854a.5.5 0 0 0 .708 0l1.5-1.5a.5.5 0 0 0-.708-.708l-.646.647V1.5a.5.5 0 0 0-1 0v1.793l-.646-.647a.5.5 0 1 0-.708.708l1.5 1.5zm-5.303-.51a.5.5 0 0 1 .707 0l1.414 1.413a.5.5 0 0 1-.707.707L2.343 5.05a.5.5 0 0 1 0-.707zm11.314 0a.5.5 0 0 1 0 .706l-1.414 1.414a.5.5 0 1 1-.707-.707l1.414-1.414a.5.5 0 0 1 .707 0zM11.709 11.5a4 4 0 1 0-7.418 0H.5a.5.5 0 0 0 0 1h15a.5.5 0 0 0 0-1h-3.79zM0 10a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2A.5.5 0 0 1 0 10zm13 0a.5.5 0 0 1 .5-.5h2a.5.5 0 0 1 0 1h-2a.5.5 0 0 1-.5-.5z"/>'),UC=Xu("SymmetryHorizontal",'<path d="M13.5 7a.5.5 0 0 0 .24-.939l-11-6A.5.5 0 0 0 2 .5v6a.5.5 0 0 0 .5.5h11zm.485 2.376a.5.5 0 0 1-.246.563l-11 6A.5.5 0 0 1 2 15.5v-6a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 .485.376zM11.539 10H3v4.658L11.54 10z"/>'),GC=Xu("SymmetryVertical",'<path d="M7 2.5a.5.5 0 0 0-.939-.24l-6 11A.5.5 0 0 0 .5 14h6a.5.5 0 0 0 .5-.5v-11zm2.376-.484a.5.5 0 0 1 .563.245l6 11A.5.5 0 0 1 15.5 14h-6a.5.5 0 0 1-.5-.5v-11a.5.5 0 0 1 .376-.484zM10 4.46V13h4.658L10 4.46z"/>'),qC=Xu("Table",'<path d="M0 2a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm15 2h-4v3h4V4zm0 4h-4v3h4V8zm0 4h-4v3h3a1 1 0 0 0 1-1v-2zm-5 3v-3H6v3h4zm-5 0v-3H1v2a1 1 0 0 0 1 1h3zm-4-4h4V8H1v3zm0-4h4V4H1v3zm5-3v3h4V4H6zm4 4H6v3h4V8z"/>'),JC=Xu("Tablet",'<path d="M12 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H4a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h8zM4 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h8a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H4z"/><path d="M8 14a1 1 0 1 0 0-2 1 1 0 0 0 0 2z"/>'),KC=Xu("TabletFill",'<path d="M2 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v12a2 2 0 0 1-2 2H4a2 2 0 0 1-2-2V2zm7 11a1 1 0 1 0-2 0 1 1 0 0 0 2 0z"/>'),XC=Xu("TabletLandscape",'<path d="M1 4a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V4zm-1 8a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v8z"/><path d="M14 8a1 1 0 1 0-2 0 1 1 0 0 0 2 0z"/>'),ZC=Xu("TabletLandscapeFill",'<path d="M2 14a2 2 0 0 1-2-2V4a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H2zm11-7a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/>'),QC=Xu("Tag",'<path d="M6 4.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm-1 0a.5.5 0 1 0-1 0 .5.5 0 0 0 1 0z"/><path d="M2 1h4.586a1 1 0 0 1 .707.293l7 7a1 1 0 0 1 0 1.414l-4.586 4.586a1 1 0 0 1-1.414 0l-7-7A1 1 0 0 1 1 6.586V2a1 1 0 0 1 1-1zm0 5.586 7 7L13.586 9l-7-7H2v4.586z"/>'),ej=Xu("TagFill",'<path d="M2 1a1 1 0 0 0-1 1v4.586a1 1 0 0 0 .293.707l7 7a1 1 0 0 0 1.414 0l4.586-4.586a1 1 0 0 0 0-1.414l-7-7A1 1 0 0 0 6.586 1H2zm4 3.5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),tj=Xu("Tags",'<path d="M3 2v4.586l7 7L14.586 9l-7-7H3zM2 2a1 1 0 0 1 1-1h4.586a1 1 0 0 1 .707.293l7 7a1 1 0 0 1 0 1.414l-4.586 4.586a1 1 0 0 1-1.414 0l-7-7A1 1 0 0 1 2 6.586V2z"/><path d="M5.5 5a.5.5 0 1 1 0-1 .5.5 0 0 1 0 1zm0 1a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3zM1 7.086a1 1 0 0 0 .293.707L8.75 15.25l-.043.043a1 1 0 0 1-1.414 0l-7-7A1 1 0 0 1 0 7.586V3a1 1 0 0 1 1-1v5.086z"/>'),aj=Xu("TagsFill",'<path d="M2 2a1 1 0 0 1 1-1h4.586a1 1 0 0 1 .707.293l7 7a1 1 0 0 1 0 1.414l-4.586 4.586a1 1 0 0 1-1.414 0l-7-7A1 1 0 0 1 2 6.586V2zm3.5 4a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/><path d="M1.293 7.793A1 1 0 0 1 1 7.086V2a1 1 0 0 0-1 1v4.586a1 1 0 0 0 .293.707l7 7a1 1 0 0 0 1.414 0l.043-.043-7.457-7.457z"/>'),nj=Xu("Telegram",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM8.287 5.906c-.778.324-2.334.994-4.666 2.01-.378.15-.577.298-.595.442-.03.243.275.339.69.47l.175.055c.408.133.958.288 1.243.294.26.006.549-.1.868-.32 2.179-1.471 3.304-2.214 3.374-2.23.05-.012.12-.026.166.016.047.041.042.12.037.141-.03.129-1.227 1.241-1.846 1.817-.193.18-.33.307-.358.336a8.154 8.154 0 0 1-.188.186c-.38.366-.664.64.015 1.088.327.216.589.393.85.571.284.194.568.387.936.629.093.06.183.125.27.187.331.236.63.448.997.414.214-.02.435-.22.547-.82.265-1.417.786-4.486.906-5.751a1.426 1.426 0 0 0-.013-.315.337.337 0 0 0-.114-.217.526.526 0 0 0-.31-.093c-.3.005-.763.166-2.984 1.09z"/>'),rj=Xu("Telephone",'<path d="M3.654 1.328a.678.678 0 0 0-1.015-.063L1.605 2.3c-.483.484-.661 1.169-.45 1.77a17.568 17.568 0 0 0 4.168 6.608 17.569 17.569 0 0 0 6.608 4.168c.601.211 1.286.033 1.77-.45l1.034-1.034a.678.678 0 0 0-.063-1.015l-2.307-1.794a.678.678 0 0 0-.58-.122l-2.19.547a1.745 1.745 0 0 1-1.657-.459L5.482 8.062a1.745 1.745 0 0 1-.46-1.657l.548-2.19a.678.678 0 0 0-.122-.58L3.654 1.328zM1.884.511a1.745 1.745 0 0 1 2.612.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511z"/>'),ij=Xu("TelephoneFill",'<path fill-rule="evenodd" d="M1.885.511a1.745 1.745 0 0 1 2.61.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511z"/>'),oj=Xu("TelephoneForward",'<path d="M3.654 1.328a.678.678 0 0 0-1.015-.063L1.605 2.3c-.483.484-.661 1.169-.45 1.77a17.568 17.568 0 0 0 4.168 6.608 17.569 17.569 0 0 0 6.608 4.168c.601.211 1.286.033 1.77-.45l1.034-1.034a.678.678 0 0 0-.063-1.015l-2.307-1.794a.678.678 0 0 0-.58-.122l-2.19.547a1.745 1.745 0 0 1-1.657-.459L5.482 8.062a1.745 1.745 0 0 1-.46-1.657l.548-2.19a.678.678 0 0 0-.122-.58L3.654 1.328zM1.884.511a1.745 1.745 0 0 1 2.612.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zm10.762.135a.5.5 0 0 1 .708 0l2.5 2.5a.5.5 0 0 1 0 .708l-2.5 2.5a.5.5 0 0 1-.708-.708L14.293 4H9.5a.5.5 0 0 1 0-1h4.793l-1.647-1.646a.5.5 0 0 1 0-.708z"/>'),sj=Xu("TelephoneForwardFill",'<path fill-rule="evenodd" d="M1.885.511a1.745 1.745 0 0 1 2.61.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zm10.761.135a.5.5 0 0 1 .708 0l2.5 2.5a.5.5 0 0 1 0 .708l-2.5 2.5a.5.5 0 0 1-.708-.708L14.293 4H9.5a.5.5 0 0 1 0-1h4.793l-1.647-1.646a.5.5 0 0 1 0-.708z"/>'),lj=Xu("TelephoneInbound",'<path d="M15.854.146a.5.5 0 0 1 0 .708L11.707 5H14.5a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5v-4a.5.5 0 0 1 1 0v2.793L15.146.146a.5.5 0 0 1 .708 0zm-12.2 1.182a.678.678 0 0 0-1.015-.063L1.605 2.3c-.483.484-.661 1.169-.45 1.77a17.568 17.568 0 0 0 4.168 6.608 17.569 17.569 0 0 0 6.608 4.168c.601.211 1.286.033 1.77-.45l1.034-1.034a.678.678 0 0 0-.063-1.015l-2.307-1.794a.678.678 0 0 0-.58-.122l-2.19.547a1.745 1.745 0 0 1-1.657-.459L5.482 8.062a1.745 1.745 0 0 1-.46-1.657l.548-2.19a.678.678 0 0 0-.122-.58L3.654 1.328zM1.884.511a1.745 1.745 0 0 1 2.612.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511z"/>'),cj=Xu("TelephoneInboundFill",'<path fill-rule="evenodd" d="M1.885.511a1.745 1.745 0 0 1 2.61.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zM15.854.146a.5.5 0 0 1 0 .708L11.707 5H14.5a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5v-4a.5.5 0 0 1 1 0v2.793L15.146.146a.5.5 0 0 1 .708 0z"/>'),uj=Xu("TelephoneMinus",'<path fill-rule="evenodd" d="M10 3.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5z"/><path d="M3.654 1.328a.678.678 0 0 0-1.015-.063L1.605 2.3c-.483.484-.661 1.169-.45 1.77a17.568 17.568 0 0 0 4.168 6.608 17.569 17.569 0 0 0 6.608 4.168c.601.211 1.286.033 1.77-.45l1.034-1.034a.678.678 0 0 0-.063-1.015l-2.307-1.794a.678.678 0 0 0-.58-.122l-2.19.547a1.745 1.745 0 0 1-1.657-.459L5.482 8.062a1.745 1.745 0 0 1-.46-1.657l.548-2.19a.678.678 0 0 0-.122-.58L3.654 1.328zM1.884.511a1.745 1.745 0 0 1 2.612.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511z"/>'),dj=Xu("TelephoneMinusFill",'<path fill-rule="evenodd" d="M1.885.511a1.745 1.745 0 0 1 2.61.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zM10 3.5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 0 1h-4a.5.5 0 0 1-.5-.5z"/>'),hj=Xu("TelephoneOutbound",'<path d="M3.654 1.328a.678.678 0 0 0-1.015-.063L1.605 2.3c-.483.484-.661 1.169-.45 1.77a17.568 17.568 0 0 0 4.168 6.608 17.569 17.569 0 0 0 6.608 4.168c.601.211 1.286.033 1.77-.45l1.034-1.034a.678.678 0 0 0-.063-1.015l-2.307-1.794a.678.678 0 0 0-.58-.122l-2.19.547a1.745 1.745 0 0 1-1.657-.459L5.482 8.062a1.745 1.745 0 0 1-.46-1.657l.548-2.19a.678.678 0 0 0-.122-.58L3.654 1.328zM1.884.511a1.745 1.745 0 0 1 2.612.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zM11 .5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0V1.707l-4.146 4.147a.5.5 0 0 1-.708-.708L14.293 1H11.5a.5.5 0 0 1-.5-.5z"/>'),fj=Xu("TelephoneOutboundFill",'<path fill-rule="evenodd" d="M1.885.511a1.745 1.745 0 0 1 2.61.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zM11 .5a.5.5 0 0 1 .5-.5h4a.5.5 0 0 1 .5.5v4a.5.5 0 0 1-1 0V1.707l-4.146 4.147a.5.5 0 0 1-.708-.708L14.293 1H11.5a.5.5 0 0 1-.5-.5z"/>'),pj=Xu("TelephonePlus",'<path d="M3.654 1.328a.678.678 0 0 0-1.015-.063L1.605 2.3c-.483.484-.661 1.169-.45 1.77a17.568 17.568 0 0 0 4.168 6.608 17.569 17.569 0 0 0 6.608 4.168c.601.211 1.286.033 1.77-.45l1.034-1.034a.678.678 0 0 0-.063-1.015l-2.307-1.794a.678.678 0 0 0-.58-.122l-2.19.547a1.745 1.745 0 0 1-1.657-.459L5.482 8.062a1.745 1.745 0 0 1-.46-1.657l.548-2.19a.678.678 0 0 0-.122-.58L3.654 1.328zM1.884.511a1.745 1.745 0 0 1 2.612.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511z"/><path fill-rule="evenodd" d="M12.5 1a.5.5 0 0 1 .5.5V3h1.5a.5.5 0 0 1 0 1H13v1.5a.5.5 0 0 1-1 0V4h-1.5a.5.5 0 0 1 0-1H12V1.5a.5.5 0 0 1 .5-.5z"/>'),mj=Xu("TelephonePlusFill",'<path fill-rule="evenodd" d="M1.885.511a1.745 1.745 0 0 1 2.61.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zM12.5 1a.5.5 0 0 1 .5.5V3h1.5a.5.5 0 0 1 0 1H13v1.5a.5.5 0 0 1-1 0V4h-1.5a.5.5 0 0 1 0-1H12V1.5a.5.5 0 0 1 .5-.5z"/>'),vj=Xu("TelephoneX",'<path d="M3.654 1.328a.678.678 0 0 0-1.015-.063L1.605 2.3c-.483.484-.661 1.169-.45 1.77a17.568 17.568 0 0 0 4.168 6.608 17.569 17.569 0 0 0 6.608 4.168c.601.211 1.286.033 1.77-.45l1.034-1.034a.678.678 0 0 0-.063-1.015l-2.307-1.794a.678.678 0 0 0-.58-.122l-2.19.547a1.745 1.745 0 0 1-1.657-.459L5.482 8.062a1.745 1.745 0 0 1-.46-1.657l.548-2.19a.678.678 0 0 0-.122-.58L3.654 1.328zM1.884.511a1.745 1.745 0 0 1 2.612.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511z"/><path fill-rule="evenodd" d="M11.146 1.646a.5.5 0 0 1 .708 0L13 2.793l1.146-1.147a.5.5 0 0 1 .708.708L13.707 3.5l1.147 1.146a.5.5 0 0 1-.708.708L13 4.207l-1.146 1.147a.5.5 0 0 1-.708-.708L12.293 3.5l-1.147-1.146a.5.5 0 0 1 0-.708z"/>'),_j=Xu("TelephoneXFill",'<path fill-rule="evenodd" d="M1.885.511a1.745 1.745 0 0 1 2.61.163L6.29 2.98c.329.423.445.974.315 1.494l-.547 2.19a.678.678 0 0 0 .178.643l2.457 2.457a.678.678 0 0 0 .644.178l2.189-.547a1.745 1.745 0 0 1 1.494.315l2.306 1.794c.829.645.905 1.87.163 2.611l-1.034 1.034c-.74.74-1.846 1.065-2.877.702a18.634 18.634 0 0 1-7.01-4.42 18.634 18.634 0 0 1-4.42-7.009c-.362-1.03-.037-2.137.703-2.877L1.885.511zm9.261 1.135a.5.5 0 0 1 .708 0L13 2.793l1.146-1.147a.5.5 0 0 1 .708.708L13.707 3.5l1.147 1.146a.5.5 0 0 1-.708.708L13 4.207l-1.146 1.147a.5.5 0 0 1-.708-.708L12.293 3.5l-1.147-1.146a.5.5 0 0 1 0-.708z"/>'),gj=Xu("Terminal",'<path d="M6 9a.5.5 0 0 1 .5-.5h3a.5.5 0 0 1 0 1h-3A.5.5 0 0 1 6 9zM3.854 4.146a.5.5 0 1 0-.708.708L4.793 6.5 3.146 8.146a.5.5 0 1 0 .708.708l2-2a.5.5 0 0 0 0-.708l-2-2z"/><path d="M2 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2H2zm12 1a1 1 0 0 1 1 1v10a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V3a1 1 0 0 1 1-1h12z"/>'),yj=Xu("TerminalFill",'<path d="M0 3a2 2 0 0 1 2-2h12a2 2 0 0 1 2 2v10a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V3zm9.5 5.5h-3a.5.5 0 0 0 0 1h3a.5.5 0 0 0 0-1zm-6.354-.354a.5.5 0 1 0 .708.708l2-2a.5.5 0 0 0 0-.708l-2-2a.5.5 0 1 0-.708.708L4.793 6.5 3.146 8.146z"/>'),bj=Xu("TextCenter",'<path fill-rule="evenodd" d="M4 12.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm2-3a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-2-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),Mj=Xu("TextIndentLeft",'<path d="M2 3.5a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm.646 2.146a.5.5 0 0 1 .708 0l2 2a.5.5 0 0 1 0 .708l-2 2a.5.5 0 0 1-.708-.708L4.293 8 2.646 6.354a.5.5 0 0 1 0-.708zM7 6.5a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 3a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zm-5 3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),zj=Xu("TextIndentRight",'<path d="M2 3.5a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm10.646 2.146a.5.5 0 0 1 .708.708L11.707 8l1.647 1.646a.5.5 0 0 1-.708.708l-2-2a.5.5 0 0 1 0-.708l2-2zM2 6.5a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 3a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5zm0 3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),Lj=Xu("TextLeft",'<path fill-rule="evenodd" d="M2 12.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),wj=Xu("TextParagraph",'<path fill-rule="evenodd" d="M2 12.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm0-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm4-3a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5z"/>'),Hj=Xu("TextRight",'<path fill-rule="evenodd" d="M6 12.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-4-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5zm4-3a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 0 1h-7a.5.5 0 0 1-.5-.5zm-4-3a.5.5 0 0 1 .5-.5h11a.5.5 0 0 1 0 1h-11a.5.5 0 0 1-.5-.5z"/>'),kj=Xu("Textarea",'<path d="M1.5 2.5A1.5 1.5 0 0 1 3 1h10a1.5 1.5 0 0 1 1.5 1.5v3.563a2 2 0 0 1 0 3.874V13.5A1.5 1.5 0 0 1 13 15H3a1.5 1.5 0 0 1-1.5-1.5V9.937a2 2 0 0 1 0-3.874V2.5zm1 3.563a2 2 0 0 1 0 3.874V13.5a.5.5 0 0 0 .5.5h10a.5.5 0 0 0 .5-.5V9.937a2 2 0 0 1 0-3.874V2.5A.5.5 0 0 0 13 2H3a.5.5 0 0 0-.5.5v3.563zM2 7a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm12 0a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/>'),Aj=Xu("TextareaResize",'<path d="M.5 4A2.5 2.5 0 0 1 3 1.5h12A2.5 2.5 0 0 1 17.5 4v8a2.5 2.5 0 0 1-2.5 2.5H3A2.5 2.5 0 0 1 .5 12V4zM3 2.5A1.5 1.5 0 0 0 1.5 4v8A1.5 1.5 0 0 0 3 13.5h12a1.5 1.5 0 0 0 1.5-1.5V4A1.5 1.5 0 0 0 15 2.5H3zm11.854 5.646a.5.5 0 0 1 0 .708l-3 3a.5.5 0 0 1-.708-.708l3-3a.5.5 0 0 1 .708 0zm0 2.5a.5.5 0 0 1 0 .708l-.5.5a.5.5 0 0 1-.708-.708l.5-.5a.5.5 0 0 1 .708 0z"/>'),Sj=Xu("TextareaT",'<path d="M1.5 2.5A1.5 1.5 0 0 1 3 1h10a1.5 1.5 0 0 1 1.5 1.5v3.563a2 2 0 0 1 0 3.874V13.5A1.5 1.5 0 0 1 13 15H3a1.5 1.5 0 0 1-1.5-1.5V9.937a2 2 0 0 1 0-3.874V2.5zm1 3.563a2 2 0 0 1 0 3.874V13.5a.5.5 0 0 0 .5.5h10a.5.5 0 0 0 .5-.5V9.937a2 2 0 0 1 0-3.874V2.5A.5.5 0 0 0 13 2H3a.5.5 0 0 0-.5.5v3.563zM2 7a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm12 0a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/><path d="M11.434 4H4.566L4.5 5.994h.386c.21-1.252.612-1.446 2.173-1.495l.343-.011v6.343c0 .537-.116.665-1.049.748V12h3.294v-.421c-.938-.083-1.054-.21-1.054-.748V4.488l.348.01c1.56.05 1.963.244 2.173 1.496h.386L11.434 4z"/>'),Vj=Xu("Thermometer",'<path d="M8 14a1.5 1.5 0 1 0 0-3 1.5 1.5 0 0 0 0 3z"/><path d="M8 0a2.5 2.5 0 0 0-2.5 2.5v7.55a3.5 3.5 0 1 0 5 0V2.5A2.5 2.5 0 0 0 8 0zM6.5 2.5a1.5 1.5 0 1 1 3 0v7.987l.167.15a2.5 2.5 0 1 1-3.333 0l.166-.15V2.5z"/>'),Oj=Xu("ThermometerHalf",'<path d="M9.5 12.5a1.5 1.5 0 1 1-2-1.415V6.5a.5.5 0 0 1 1 0v4.585a1.5 1.5 0 0 1 1 1.415z"/><path d="M5.5 2.5a2.5 2.5 0 0 1 5 0v7.55a3.5 3.5 0 1 1-5 0V2.5zM8 1a1.5 1.5 0 0 0-1.5 1.5v7.987l-.167.15a2.5 2.5 0 1 0 3.333 0l-.166-.15V2.5A1.5 1.5 0 0 0 8 1z"/>'),Tj=Xu("ThermometerHigh",'<path d="M9.5 12.5a1.5 1.5 0 1 1-2-1.415V2.5a.5.5 0 0 1 1 0v8.585a1.5 1.5 0 0 1 1 1.415z"/><path d="M5.5 2.5a2.5 2.5 0 0 1 5 0v7.55a3.5 3.5 0 1 1-5 0V2.5zM8 1a1.5 1.5 0 0 0-1.5 1.5v7.987l-.167.15a2.5 2.5 0 1 0 3.333 0l-.166-.15V2.5A1.5 1.5 0 0 0 8 1z"/>'),Dj=Xu("ThermometerLow",'<path d="M9.5 12.5a1.5 1.5 0 1 1-2-1.415V9.5a.5.5 0 0 1 1 0v1.585a1.5 1.5 0 0 1 1 1.415z"/><path d="M5.5 2.5a2.5 2.5 0 0 1 5 0v7.55a3.5 3.5 0 1 1-5 0V2.5zM8 1a1.5 1.5 0 0 0-1.5 1.5v7.987l-.167.15a2.5 2.5 0 1 0 3.333 0l-.166-.15V2.5A1.5 1.5 0 0 0 8 1z"/>'),Cj=Xu("ThermometerSnow",'<path d="M5 12.5a1.5 1.5 0 1 1-2-1.415V9.5a.5.5 0 0 1 1 0v1.585A1.5 1.5 0 0 1 5 12.5z"/><path d="M1 2.5a2.5 2.5 0 0 1 5 0v7.55a3.5 3.5 0 1 1-5 0V2.5zM3.5 1A1.5 1.5 0 0 0 2 2.5v7.987l-.167.15a2.5 2.5 0 1 0 3.333 0L5 10.486V2.5A1.5 1.5 0 0 0 3.5 1zm5 1a.5.5 0 0 1 .5.5v1.293l.646-.647a.5.5 0 0 1 .708.708L9 5.207v1.927l1.669-.963.495-1.85a.5.5 0 1 1 .966.26l-.237.882 1.12-.646a.5.5 0 0 1 .5.866l-1.12.646.884.237a.5.5 0 1 1-.26.966l-1.848-.495L9.5 8l1.669.963 1.849-.495a.5.5 0 1 1 .258.966l-.883.237 1.12.646a.5.5 0 0 1-.5.866l-1.12-.646.237.883a.5.5 0 1 1-.966.258L10.67 9.83 9 8.866v1.927l1.354 1.353a.5.5 0 0 1-.708.708L9 12.207V13.5a.5.5 0 0 1-1 0v-11a.5.5 0 0 1 .5-.5z"/>'),jj=Xu("ThermometerSun",'<path d="M5 12.5a1.5 1.5 0 1 1-2-1.415V2.5a.5.5 0 0 1 1 0v8.585A1.5 1.5 0 0 1 5 12.5z"/><path d="M1 2.5a2.5 2.5 0 0 1 5 0v7.55a3.5 3.5 0 1 1-5 0V2.5zM3.5 1A1.5 1.5 0 0 0 2 2.5v7.987l-.167.15a2.5 2.5 0 1 0 3.333 0L5 10.486V2.5A1.5 1.5 0 0 0 3.5 1zm5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-1 0v-1a.5.5 0 0 1 .5-.5zm4.243 1.757a.5.5 0 0 1 0 .707l-.707.708a.5.5 0 1 1-.708-.708l.708-.707a.5.5 0 0 1 .707 0zM8 5.5a.5.5 0 0 1 .5-.5 3 3 0 1 1 0 6 .5.5 0 0 1 0-1 2 2 0 0 0 0-4 .5.5 0 0 1-.5-.5zM12.5 8a.5.5 0 0 1 .5-.5h1a.5.5 0 1 1 0 1h-1a.5.5 0 0 1-.5-.5zm-1.172 2.828a.5.5 0 0 1 .708 0l.707.708a.5.5 0 0 1-.707.707l-.708-.707a.5.5 0 0 1 0-.708zM8.5 12a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-1 0v-1a.5.5 0 0 1 .5-.5z"/>'),xj=Xu("ThreeDots",'<path d="M3 9.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm5 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm5 0a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),Bj=Xu("ThreeDotsVertical",'<path d="M9.5 13a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0-5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0zm0-5a1.5 1.5 0 1 1-3 0 1.5 1.5 0 0 1 3 0z"/>'),Yj=Xu("Toggle2Off",'<path d="M9 11c.628-.836 1-1.874 1-3a4.978 4.978 0 0 0-1-3h4a3 3 0 1 1 0 6H9z"/><path d="M5 12a4 4 0 1 1 0-8 4 4 0 0 1 0 8zm0 1A5 5 0 1 0 5 3a5 5 0 0 0 0 10z"/>'),Pj=Xu("Toggle2On",'<path d="M7 5H3a3 3 0 0 0 0 6h4a4.995 4.995 0 0 1-.584-1H3a2 2 0 1 1 0-4h3.416c.156-.357.352-.692.584-1z"/><path d="M16 8A5 5 0 1 1 6 8a5 5 0 0 1 10 0z"/>'),Ij=Xu("ToggleOff",'<path d="M11 4a4 4 0 0 1 0 8H8a4.992 4.992 0 0 0 2-4 4.992 4.992 0 0 0-2-4h3zm-6 8a4 4 0 1 1 0-8 4 4 0 0 1 0 8zM0 8a5 5 0 0 0 5 5h6a5 5 0 0 0 0-10H5a5 5 0 0 0-5 5z"/>'),Fj=Xu("ToggleOn",'<path d="M5 3a5 5 0 0 0 0 10h6a5 5 0 0 0 0-10H5zm6 9a4 4 0 1 1 0-8 4 4 0 0 1 0 8z"/>'),Ej=Xu("Toggles",'<path d="M4.5 9a3.5 3.5 0 1 0 0 7h7a3.5 3.5 0 1 0 0-7h-7zm7 6a2.5 2.5 0 1 1 0-5 2.5 2.5 0 0 1 0 5zm-7-14a2.5 2.5 0 1 0 0 5 2.5 2.5 0 0 0 0-5zm2.45 0A3.49 3.49 0 0 1 8 3.5 3.49 3.49 0 0 1 6.95 6h4.55a2.5 2.5 0 0 0 0-5H6.95zM4.5 0h7a3.5 3.5 0 1 1 0 7h-7a3.5 3.5 0 1 1 0-7z"/>'),$j=Xu("Toggles2",'<path d="M9.465 10H12a2 2 0 1 1 0 4H9.465c.34-.588.535-1.271.535-2 0-.729-.195-1.412-.535-2z"/><path d="M6 15a3 3 0 1 0 0-6 3 3 0 0 0 0 6zm0 1a4 4 0 1 1 0-8 4 4 0 0 1 0 8zm.535-10a3.975 3.975 0 0 1-.409-1H4a1 1 0 0 1 0-2h2.126c.091-.355.23-.69.41-1H4a2 2 0 1 0 0 4h2.535z"/><path d="M14 4a4 4 0 1 1-8 0 4 4 0 0 1 8 0z"/>'),Rj=Xu("Tools",'<path d="M1 0 0 1l2.2 3.081a1 1 0 0 0 .815.419h.07a1 1 0 0 1 .708.293l2.675 2.675-2.617 2.654A3.003 3.003 0 0 0 0 13a3 3 0 1 0 5.878-.851l2.654-2.617.968.968-.305.914a1 1 0 0 0 .242 1.023l3.356 3.356a1 1 0 0 0 1.414 0l1.586-1.586a1 1 0 0 0 0-1.414l-3.356-3.356a1 1 0 0 0-1.023-.242L10.5 9.5l-.96-.96 2.68-2.643A3.005 3.005 0 0 0 16 3c0-.269-.035-.53-.102-.777l-2.14 2.141L12 4l-.364-1.757L13.777.102a3 3 0 0 0-3.675 3.68L7.462 6.46 4.793 3.793a1 1 0 0 1-.293-.707v-.071a1 1 0 0 0-.419-.814L1 0zm9.646 10.646a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708l-3-3a.5.5 0 0 1 0-.708zM3 11l.471.242.529.026.287.445.445.287.026.529L5 13l-.242.471-.026.529-.445.287-.287.445-.529.026L3 15l-.471-.242L2 14.732l-.287-.445L1.268 14l-.026-.529L1 13l.242-.471.026-.529.445-.287.287-.445.529-.026L3 11z"/>'),Nj=Xu("Tornado",'<path d="M1.125 2.45A.892.892 0 0 1 1 2c0-.26.116-.474.258-.634a1.9 1.9 0 0 1 .513-.389c.387-.21.913-.385 1.52-.525C4.514.17 6.18 0 8 0c1.821 0 3.486.17 4.709.452.607.14 1.133.314 1.52.525.193.106.374.233.513.389.141.16.258.374.258.634 0 1.011-.35 1.612-.634 2.102-.04.07-.08.137-.116.203a2.55 2.55 0 0 0-.313.809 2.938 2.938 0 0 0-.011.891.5.5 0 0 1 .428.849c-.06.06-.133.126-.215.195.204 1.116.088 1.99-.3 2.711-.453.84-1.231 1.383-2.02 1.856-.204.123-.412.243-.62.364-1.444.832-2.928 1.689-3.735 3.706a.5.5 0 0 1-.748.226l-.001-.001-.002-.001-.004-.003-.01-.008a2.142 2.142 0 0 1-.147-.115 4.095 4.095 0 0 1-1.179-1.656 3.786 3.786 0 0 1-.247-1.296A.498.498 0 0 1 5 12.5v-.018a.62.62 0 0 1 .008-.079.728.728 0 0 1 .188-.386c.09-.489.272-1.014.573-1.574a.5.5 0 0 1 .073-.918 3.29 3.29 0 0 1 .617-.144l.15-.193c.285-.356.404-.639.437-.861a.948.948 0 0 0-.122-.619c-.249-.455-.815-.903-1.613-1.43-.193-.127-.398-.258-.609-.394l-.119-.076a12.307 12.307 0 0 1-1.241-.334.5.5 0 0 1-.285-.707l-.23-.18C2.117 4.01 1.463 3.32 1.125 2.45zm1.973 1.051c.113.104.233.207.358.308.472.381.99.722 1.515 1.06 1.54.317 3.632.5 5.43.14a.5.5 0 0 1 .197.981c-1.216.244-2.537.26-3.759.157.399.326.744.682.963 1.081.203.373.302.79.233 1.247-.05.33-.182.657-.39.985.075.017.148.035.22.053l.006.002c.481.12.863.213 1.47.01a.5.5 0 1 1 .317.95c-.888.295-1.505.141-2.023.012l-.006-.002a3.894 3.894 0 0 0-.644-.123c-.37.55-.598 1.05-.726 1.497.142.045.296.11.465.194a.5.5 0 1 1-.448.894 3.11 3.11 0 0 0-.148-.07c.012.345.084.643.18.895.14.369.342.666.528.886.992-1.903 2.583-2.814 3.885-3.56.203-.116.399-.228.584-.34.775-.464 1.34-.89 1.653-1.472.212-.393.33-.9.26-1.617A6.74 6.74 0 0 1 10 8.5a.5.5 0 0 1 0-1 5.76 5.76 0 0 0 3.017-.872.515.515 0 0 1-.007-.03c-.135-.673-.14-1.207-.056-1.665.084-.46.253-.81.421-1.113l.131-.23c.065-.112.126-.22.182-.327-.29.107-.62.202-.98.285C11.487 3.83 9.822 4 8 4c-1.821 0-3.486-.17-4.709-.452-.065-.015-.13-.03-.193-.047zM13.964 2a1.12 1.12 0 0 0-.214-.145c-.272-.148-.697-.297-1.266-.428C11.354 1.166 9.769 1 8 1c-1.769 0-3.354.166-4.484.427-.569.13-.994.28-1.266.428A1.12 1.12 0 0 0 2.036 2c.04.038.109.087.214.145.272.148.697.297 1.266.428C4.646 2.834 6.231 3 8 3c1.769 0 3.354-.166 4.484-.427.569-.13.994-.28 1.266-.428A1.12 1.12 0 0 0 13.964 2z"/>'),Wj=Xu("Translate",'<path d="M4.545 6.714 4.11 8H3l1.862-5h1.284L8 8H6.833l-.435-1.286H4.545zm1.634-.736L5.5 3.956h-.049l-.679 2.022H6.18z"/><path d="M0 2a2 2 0 0 1 2-2h7a2 2 0 0 1 2 2v3h3a2 2 0 0 1 2 2v7a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2v-3H2a2 2 0 0 1-2-2V2zm2-1a1 1 0 0 0-1 1v7a1 1 0 0 0 1 1h7a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H2zm7.138 9.995c.193.301.402.583.63.846-.748.575-1.673 1.001-2.768 1.292.178.217.451.635.555.867 1.125-.359 2.08-.844 2.886-1.494.777.665 1.739 1.165 2.93 1.472.133-.254.414-.673.629-.89-1.125-.253-2.057-.694-2.82-1.284.681-.747 1.222-1.651 1.621-2.757H14V8h-3v1.047h.765c-.318.844-.74 1.546-1.272 2.13a6.066 6.066 0 0 1-.415-.492 1.988 1.988 0 0 1-.94.31z"/>'),Uj=Xu("Trash",'<path d="M5.5 5.5A.5.5 0 0 1 6 6v6a.5.5 0 0 1-1 0V6a.5.5 0 0 1 .5-.5zm2.5 0a.5.5 0 0 1 .5.5v6a.5.5 0 0 1-1 0V6a.5.5 0 0 1 .5-.5zm3 .5a.5.5 0 0 0-1 0v6a.5.5 0 0 0 1 0V6z"/><path fill-rule="evenodd" d="M14.5 3a1 1 0 0 1-1 1H13v9a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2V4h-.5a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1H6a1 1 0 0 1 1-1h2a1 1 0 0 1 1 1h3.5a1 1 0 0 1 1 1v1zM4.118 4 4 4.059V13a1 1 0 0 0 1 1h6a1 1 0 0 0 1-1V4.059L11.882 4H4.118zM2.5 3V2h11v1h-11z"/>'),Gj=Xu("Trash2",'<path d="M14 3a.702.702 0 0 1-.037.225l-1.684 10.104A2 2 0 0 1 10.305 15H5.694a2 2 0 0 1-1.973-1.671L2.037 3.225A.703.703 0 0 1 2 3c0-1.105 2.686-2 6-2s6 .895 6 2zM3.215 4.207l1.493 8.957a1 1 0 0 0 .986.836h4.612a1 1 0 0 0 .986-.836l1.493-8.957C11.69 4.689 9.954 5 8 5c-1.954 0-3.69-.311-4.785-.793z"/>'),qj=Xu("Trash2Fill",'<path d="M2.037 3.225A.703.703 0 0 1 2 3c0-1.105 2.686-2 6-2s6 .895 6 2a.702.702 0 0 1-.037.225l-1.684 10.104A2 2 0 0 1 10.305 15H5.694a2 2 0 0 1-1.973-1.671L2.037 3.225zm9.89-.69C10.966 2.214 9.578 2 8 2c-1.58 0-2.968.215-3.926.534-.477.16-.795.327-.975.466.18.14.498.307.975.466C5.032 3.786 6.42 4 8 4s2.967-.215 3.926-.534c.477-.16.795-.327.975-.466-.18-.14-.498-.307-.975-.466z"/>'),Jj=Xu("TrashFill",'<path d="M2.5 1a1 1 0 0 0-1 1v1a1 1 0 0 0 1 1H3v9a2 2 0 0 0 2 2h6a2 2 0 0 0 2-2V4h.5a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H10a1 1 0 0 0-1-1H7a1 1 0 0 0-1 1H2.5zm3 4a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-1 0v-7a.5.5 0 0 1 .5-.5zM8 5a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-1 0v-7A.5.5 0 0 1 8 5zm3 .5v7a.5.5 0 0 1-1 0v-7a.5.5 0 0 1 1 0z"/>'),Kj=Xu("Tree",'<path d="M8.416.223a.5.5 0 0 0-.832 0l-3 4.5A.5.5 0 0 0 5 5.5h.098L3.076 8.735A.5.5 0 0 0 3.5 9.5h.191l-1.638 3.276a.5.5 0 0 0 .447.724H7V16h2v-2.5h4.5a.5.5 0 0 0 .447-.724L12.31 9.5h.191a.5.5 0 0 0 .424-.765L10.902 5.5H11a.5.5 0 0 0 .416-.777l-3-4.5zM6.437 4.758A.5.5 0 0 0 6 4.5h-.066L8 1.401 10.066 4.5H10a.5.5 0 0 0-.424.765L11.598 8.5H11.5a.5.5 0 0 0-.447.724L12.69 12.5H3.309l1.638-3.276A.5.5 0 0 0 4.5 8.5h-.098l2.022-3.235a.5.5 0 0 0 .013-.507z"/>'),Xj=Xu("TreeFill",'<path d="M8.416.223a.5.5 0 0 0-.832 0l-3 4.5A.5.5 0 0 0 5 5.5h.098L3.076 8.735A.5.5 0 0 0 3.5 9.5h.191l-1.638 3.276a.5.5 0 0 0 .447.724H7V16h2v-2.5h4.5a.5.5 0 0 0 .447-.724L12.31 9.5h.191a.5.5 0 0 0 .424-.765L10.902 5.5H11a.5.5 0 0 0 .416-.777l-3-4.5z"/>'),Zj=Xu("Triangle",'<path d="M7.938 2.016A.13.13 0 0 1 8.002 2a.13.13 0 0 1 .063.016.146.146 0 0 1 .054.057l6.857 11.667c.036.06.035.124.002.183a.163.163 0 0 1-.054.06.116.116 0 0 1-.066.017H1.146a.115.115 0 0 1-.066-.017.163.163 0 0 1-.054-.06.176.176 0 0 1 .002-.183L7.884 2.073a.147.147 0 0 1 .054-.057zm1.044-.45a1.13 1.13 0 0 0-1.96 0L.165 13.233c-.457.778.091 1.767.98 1.767h13.713c.889 0 1.438-.99.98-1.767L8.982 1.566z"/>'),Qj=Xu("TriangleFill",'<path fill-rule="evenodd" d="M7.022 1.566a1.13 1.13 0 0 1 1.96 0l6.857 11.667c.457.778-.092 1.767-.98 1.767H1.144c-.889 0-1.437-.99-.98-1.767L7.022 1.566z"/>'),ex=Xu("TriangleHalf",'<path d="M8.065 2.016A.13.13 0 0 0 8.002 2v11.983l6.856.017a.12.12 0 0 0 .066-.017.162.162 0 0 0 .054-.06.176.176 0 0 0-.002-.183L8.12 2.073a.146.146 0 0 0-.054-.057zm-1.043-.45a1.13 1.13 0 0 1 1.96 0l6.856 11.667c.458.778-.091 1.767-.98 1.767H1.146c-.889 0-1.437-.99-.98-1.767L7.022 1.566z"/>'),tx=Xu("Trophy",'<path d="M2.5.5A.5.5 0 0 1 3 0h10a.5.5 0 0 1 .5.5c0 .538-.012 1.05-.034 1.536a3 3 0 1 1-1.133 5.89c-.79 1.865-1.878 2.777-2.833 3.011v2.173l1.425.356c.194.048.377.135.537.255L13.3 15.1a.5.5 0 0 1-.3.9H3a.5.5 0 0 1-.3-.9l1.838-1.379c.16-.12.343-.207.537-.255L6.5 13.11v-2.173c-.955-.234-2.043-1.146-2.833-3.012a3 3 0 1 1-1.132-5.89A33.076 33.076 0 0 1 2.5.5zm.099 2.54a2 2 0 0 0 .72 3.935c-.333-1.05-.588-2.346-.72-3.935zm10.083 3.935a2 2 0 0 0 .72-3.935c-.133 1.59-.388 2.885-.72 3.935zM3.504 1c.007.517.026 1.006.056 1.469.13 2.028.457 3.546.87 4.667C5.294 9.48 6.484 10 7 10a.5.5 0 0 1 .5.5v2.61a1 1 0 0 1-.757.97l-1.426.356a.5.5 0 0 0-.179.085L4.5 15h7l-.638-.479a.501.501 0 0 0-.18-.085l-1.425-.356a1 1 0 0 1-.757-.97V10.5A.5.5 0 0 1 9 10c.516 0 1.706-.52 2.57-2.864.413-1.12.74-2.64.87-4.667.03-.463.049-.952.056-1.469H3.504z"/>'),ax=Xu("TrophyFill",'<path d="M2.5.5A.5.5 0 0 1 3 0h10a.5.5 0 0 1 .5.5c0 .538-.012 1.05-.034 1.536a3 3 0 1 1-1.133 5.89c-.79 1.865-1.878 2.777-2.833 3.011v2.173l1.425.356c.194.048.377.135.537.255L13.3 15.1a.5.5 0 0 1-.3.9H3a.5.5 0 0 1-.3-.9l1.838-1.379c.16-.12.343-.207.537-.255L6.5 13.11v-2.173c-.955-.234-2.043-1.146-2.833-3.012a3 3 0 1 1-1.132-5.89A33.076 33.076 0 0 1 2.5.5zm.099 2.54a2 2 0 0 0 .72 3.935c-.333-1.05-.588-2.346-.72-3.935zm10.083 3.935a2 2 0 0 0 .72-3.935c-.133 1.59-.388 2.885-.72 3.935z"/>'),nx=Xu("TropicalStorm",'<path d="M8 9.5a2 2 0 1 0 0-4 2 2 0 0 0 0 4z"/><path d="M9.5 2c-.9 0-1.75.216-2.501.6A5 5 0 0 1 13 7.5a6.5 6.5 0 1 1-13 0 .5.5 0 0 1 1 0 5.5 5.5 0 0 0 8.001 4.9A5 5 0 0 1 3 7.5a6.5 6.5 0 0 1 13 0 .5.5 0 0 1-1 0A5.5 5.5 0 0 0 9.5 2zM8 3.5a4 4 0 1 0 0 8 4 4 0 0 0 0-8z"/>'),rx=Xu("Truck",'<path d="M0 3.5A1.5 1.5 0 0 1 1.5 2h9A1.5 1.5 0 0 1 12 3.5V5h1.02a1.5 1.5 0 0 1 1.17.563l1.481 1.85a1.5 1.5 0 0 1 .329.938V10.5a1.5 1.5 0 0 1-1.5 1.5H14a2 2 0 1 1-4 0H5a2 2 0 1 1-3.998-.085A1.5 1.5 0 0 1 0 10.5v-7zm1.294 7.456A1.999 1.999 0 0 1 4.732 11h5.536a2.01 2.01 0 0 1 .732-.732V3.5a.5.5 0 0 0-.5-.5h-9a.5.5 0 0 0-.5.5v7a.5.5 0 0 0 .294.456zM12 10a2 2 0 0 1 1.732 1h.768a.5.5 0 0 0 .5-.5V8.35a.5.5 0 0 0-.11-.312l-1.48-1.85A.5.5 0 0 0 13.02 6H12v4zm-9 1a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm9 0a1 1 0 1 0 0 2 1 1 0 0 0 0-2z"/>'),ix=Xu("TruckFlatbed",'<path d="M11.5 4a.5.5 0 0 1 .5.5V5h1.02a1.5 1.5 0 0 1 1.17.563l1.481 1.85a1.5 1.5 0 0 1 .329.938V10.5a1.5 1.5 0 0 1-1.5 1.5H14a2 2 0 1 1-4 0H5a2 2 0 1 1-4 0 1 1 0 0 1-1-1v-1h11V4.5a.5.5 0 0 1 .5-.5zM3 11a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm9 0a1 1 0 1 0 0 2 1 1 0 0 0 0-2zm1.732 0h.768a.5.5 0 0 0 .5-.5V8.35a.5.5 0 0 0-.11-.312l-1.48-1.85A.5.5 0 0 0 13.02 6H12v4a2 2 0 0 1 1.732 1z"/>'),ox=Xu("Tsunami",'<path d="M.036 12.314a.5.5 0 0 1 .65-.278l1.757.703a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.757-.703a.5.5 0 1 1 .372.928l-1.758.703a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.757-.703a.5.5 0 0 1-.278-.65zm0 2a.5.5 0 0 1 .65-.278l1.757.703a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.757-.703a.5.5 0 1 1 .372.928l-1.758.703a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.757-.703a.5.5 0 0 1-.278-.65zM2.662 8.08c-.456 1.063-.994 2.098-1.842 2.804a.5.5 0 0 1-.64-.768c.652-.544 1.114-1.384 1.564-2.43.14-.328.281-.68.427-1.044.302-.754.624-1.559 1.01-2.308C3.763 3.2 4.528 2.105 5.7 1.299 6.877.49 8.418 0 10.5 0c1.463 0 2.511.4 3.179 1.058.67.66.893 1.518.819 2.302-.074.771-.441 1.516-1.02 1.965a1.878 1.878 0 0 1-1.904.27c-.65.642-.907 1.679-.71 2.614C11.076 9.215 11.784 10 13 10h2.5a.5.5 0 0 1 0 1H13c-1.784 0-2.826-1.215-3.114-2.585-.232-1.1.005-2.373.758-3.284L10.5 5.06l-.777.388a.5.5 0 0 1-.447 0l-1-.5a.5.5 0 0 1 .447-.894l.777.388.776-.388a.5.5 0 0 1 .447 0l1 .5a.493.493 0 0 1 .034.018c.44.264.81.195 1.108-.036.328-.255.586-.729.637-1.27.05-.529-.1-1.076-.525-1.495-.426-.42-1.19-.77-2.477-.77-1.918 0-3.252.448-4.232 1.123C5.283 2.8 4.61 3.738 4.07 4.79c-.365.71-.655 1.433-.945 2.16-.15.376-.301.753-.463 1.13z"/>'),sx=Xu("Tv",'<path d="M2.5 13.5A.5.5 0 0 1 3 13h10a.5.5 0 0 1 0 1H3a.5.5 0 0 1-.5-.5zM13.991 3l.024.001a1.46 1.46 0 0 1 .538.143.757.757 0 0 1 .302.254c.067.1.145.277.145.602v5.991l-.001.024a1.464 1.464 0 0 1-.143.538.758.758 0 0 1-.254.302c-.1.067-.277.145-.602.145H2.009l-.024-.001a1.464 1.464 0 0 1-.538-.143.758.758 0 0 1-.302-.254C1.078 10.502 1 10.325 1 10V4.009l.001-.024a1.46 1.46 0 0 1 .143-.538.758.758 0 0 1 .254-.302C1.498 3.078 1.675 3 2 3h11.991zM14 2H2C0 2 0 4 0 4v6c0 2 2 2 2 2h12c2 0 2-2 2-2V4c0-2-2-2-2-2z"/>'),lx=Xu("TvFill",'<path d="M2.5 13.5A.5.5 0 0 1 3 13h10a.5.5 0 0 1 0 1H3a.5.5 0 0 1-.5-.5zM2 2h12s2 0 2 2v6s0 2-2 2H2s-2 0-2-2V4s0-2 2-2z"/>'),cx=Xu("Twitch",'<path d="M3.857 0 1 2.857v10.286h3.429V16l2.857-2.857H9.57L14.714 8V0H3.857zm9.714 7.429-2.285 2.285H9l-2 2v-2H4.429V1.143h9.142v6.286z"/><path d="M11.857 3.143h-1.143V6.57h1.143V3.143zm-3.143 0H7.571V6.57h1.143V3.143z"/>'),ux=Xu("Twitter",'<path d="M5.026 15c6.038 0 9.341-5.003 9.341-9.334 0-.14 0-.282-.006-.422A6.685 6.685 0 0 0 16 3.542a6.658 6.658 0 0 1-1.889.518 3.301 3.301 0 0 0 1.447-1.817 6.533 6.533 0 0 1-2.087.793A3.286 3.286 0 0 0 7.875 6.03a9.325 9.325 0 0 1-6.767-3.429 3.289 3.289 0 0 0 1.018 4.382A3.323 3.323 0 0 1 .64 6.575v.045a3.288 3.288 0 0 0 2.632 3.218 3.203 3.203 0 0 1-.865.115 3.23 3.23 0 0 1-.614-.057 3.283 3.283 0 0 0 3.067 2.277A6.588 6.588 0 0 1 .78 13.58a6.32 6.32 0 0 1-.78-.045A9.344 9.344 0 0 0 5.026 15z"/>'),dx=Xu("Type",'<path d="m2.244 13.081.943-2.803H6.66l.944 2.803H8.86L5.54 3.75H4.322L1 13.081h1.244zm2.7-7.923L6.34 9.314H3.51l1.4-4.156h.034zm9.146 7.027h.035v.896h1.128V8.125c0-1.51-1.114-2.345-2.646-2.345-1.736 0-2.59.916-2.666 2.174h1.108c.068-.718.595-1.19 1.517-1.19.971 0 1.518.52 1.518 1.464v.731H12.19c-1.647.007-2.522.8-2.522 2.058 0 1.319.957 2.18 2.345 2.18 1.06 0 1.716-.43 2.078-1.011zm-1.763.035c-.752 0-1.456-.397-1.456-1.244 0-.65.424-1.115 1.408-1.115h1.805v.834c0 .896-.752 1.525-1.757 1.525z"/>'),hx=Xu("TypeBold",'<path d="M8.21 13c2.106 0 3.412-1.087 3.412-2.823 0-1.306-.984-2.283-2.324-2.386v-.055a2.176 2.176 0 0 0 1.852-2.14c0-1.51-1.162-2.46-3.014-2.46H3.843V13H8.21zM5.908 4.674h1.696c.963 0 1.517.451 1.517 1.244 0 .834-.629 1.32-1.73 1.32H5.908V4.673zm0 6.788V8.598h1.73c1.217 0 1.88.492 1.88 1.415 0 .943-.643 1.449-1.832 1.449H5.907z"/>'),fx=Xu("TypeH1",'<path d="M8.637 13V3.669H7.379V7.62H2.758V3.67H1.5V13h1.258V8.728h4.62V13h1.259zm5.329 0V3.669h-1.244L10.5 5.316v1.265l2.16-1.565h.062V13h1.244z"/>'),px=Xu("TypeH2",'<path d="M7.638 13V3.669H6.38V7.62H1.759V3.67H.5V13h1.258V8.728h4.62V13h1.259zm3.022-6.733v-.048c0-.889.63-1.668 1.716-1.668.957 0 1.675.608 1.675 1.572 0 .855-.554 1.504-1.067 2.085l-3.513 3.999V13H15.5v-1.094h-4.245v-.075l2.481-2.844c.875-.998 1.586-1.784 1.586-2.953 0-1.463-1.155-2.556-2.919-2.556-1.941 0-2.966 1.326-2.966 2.74v.049h1.223z"/>'),mx=Xu("TypeH3",'<path d="M7.637 13V3.669H6.379V7.62H1.758V3.67H.5V13h1.258V8.728h4.62V13h1.259zm3.625-4.272h1.018c1.142 0 1.935.67 1.949 1.674.013 1.005-.78 1.737-2.01 1.73-1.08-.007-1.853-.588-1.935-1.32H9.108c.069 1.327 1.224 2.386 3.083 2.386 1.935 0 3.343-1.155 3.309-2.789-.027-1.51-1.251-2.16-2.037-2.249v-.068c.704-.123 1.764-.91 1.723-2.229-.035-1.353-1.176-2.4-2.954-2.385-1.873.006-2.857 1.162-2.898 2.358h1.196c.062-.69.711-1.299 1.696-1.299.998 0 1.695.622 1.695 1.525.007.922-.718 1.592-1.695 1.592h-.964v1.074z"/>'),vx=Xu("TypeItalic",'<path d="M7.991 11.674 9.53 4.455c.123-.595.246-.71 1.347-.807l.11-.52H7.211l-.11.52c1.06.096 1.128.212 1.005.807L6.57 11.674c-.123.595-.246.71-1.346.806l-.11.52h3.774l.11-.52c-1.06-.095-1.129-.211-1.006-.806z"/>'),_x=Xu("TypeStrikethrough",'<path d="M6.333 5.686c0 .31.083.581.27.814H5.166a2.776 2.776 0 0 1-.099-.76c0-1.627 1.436-2.768 3.48-2.768 1.969 0 3.39 1.175 3.445 2.85h-1.23c-.11-1.08-.964-1.743-2.25-1.743-1.23 0-2.18.602-2.18 1.607zm2.194 7.478c-2.153 0-3.589-1.107-3.705-2.81h1.23c.144 1.06 1.129 1.703 2.544 1.703 1.34 0 2.31-.705 2.31-1.675 0-.827-.547-1.374-1.914-1.675L8.046 8.5H1v-1h14v1h-3.504c.468.437.675.994.675 1.697 0 1.826-1.436 2.967-3.644 2.967z"/>'),gx=Xu("TypeUnderline",'<path d="M5.313 3.136h-1.23V9.54c0 2.105 1.47 3.623 3.917 3.623s3.917-1.518 3.917-3.623V3.136h-1.23v6.323c0 1.49-.978 2.57-2.687 2.57-1.709 0-2.687-1.08-2.687-2.57V3.136zM12.5 15h-9v-1h9v1z"/>'),yx=Xu("UiChecks",'<path d="M7 2.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-1zM2 1a2 2 0 0 0-2 2v2a2 2 0 0 0 2 2h2a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2H2zm0 8a2 2 0 0 0-2 2v2a2 2 0 0 0 2 2h2a2 2 0 0 0 2-2v-2a2 2 0 0 0-2-2H2zm.854-3.646a.5.5 0 0 1-.708 0l-1-1a.5.5 0 1 1 .708-.708l.646.647 1.646-1.647a.5.5 0 1 1 .708.708l-2 2zm0 8a.5.5 0 0 1-.708 0l-1-1a.5.5 0 0 1 .708-.708l.646.647 1.646-1.647a.5.5 0 0 1 .708.708l-2 2zM7 10.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-1zm0-5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 8a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5z"/>'),bx=Xu("UiChecksGrid",'<path d="M2 10h3a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1v-3a1 1 0 0 1 1-1zm9-9h3a1 1 0 0 1 1 1v3a1 1 0 0 1-1 1h-3a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1zm0 9a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h3a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1h-3zm0-10a2 2 0 0 0-2 2v3a2 2 0 0 0 2 2h3a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2h-3zM2 9a2 2 0 0 0-2 2v3a2 2 0 0 0 2 2h3a2 2 0 0 0 2-2v-3a2 2 0 0 0-2-2H2zm7 2a2 2 0 0 1 2-2h3a2 2 0 0 1 2 2v3a2 2 0 0 1-2 2h-3a2 2 0 0 1-2-2v-3zM0 2a2 2 0 0 1 2-2h3a2 2 0 0 1 2 2v3a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2V2zm5.354.854a.5.5 0 1 0-.708-.708L3 3.793l-.646-.647a.5.5 0 1 0-.708.708l1 1a.5.5 0 0 0 .708 0l2-2z"/>'),Mx=Xu("UiRadios",'<path d="M7 2.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-1zM0 12a3 3 0 1 1 6 0 3 3 0 0 1-6 0zm7-1.5a.5.5 0 0 1 .5-.5h7a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-7a.5.5 0 0 1-.5-.5v-1zm0-5a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zm0 8a.5.5 0 0 1 .5-.5h5a.5.5 0 0 1 0 1h-5a.5.5 0 0 1-.5-.5zM3 1a3 3 0 1 0 0 6 3 3 0 0 0 0-6zm0 4.5a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3z"/>'),zx=Xu("UiRadiosGrid",'<path d="M3.5 15a2.5 2.5 0 1 0 0-5 2.5 2.5 0 0 0 0 5zm9-9a2.5 2.5 0 1 0 0-5 2.5 2.5 0 0 0 0 5zm0 9a2.5 2.5 0 1 1 0-5 2.5 2.5 0 0 1 0 5zM16 3.5a3.5 3.5 0 1 1-7 0 3.5 3.5 0 0 1 7 0zm-9 9a3.5 3.5 0 1 1-7 0 3.5 3.5 0 0 1 7 0zm5.5 3.5a3.5 3.5 0 1 0 0-7 3.5 3.5 0 0 0 0 7zm-9-11a1.5 1.5 0 1 1 0-3 1.5 1.5 0 0 1 0 3zm0 2a3.5 3.5 0 1 0 0-7 3.5 3.5 0 0 0 0 7z"/>'),Lx=Xu("Umbrella",'<path d="M8 0a.5.5 0 0 1 .5.5v.514C12.625 1.238 16 4.22 16 8c0 0 0 .5-.5.5-.149 0-.352-.145-.352-.145l-.004-.004-.025-.023a3.484 3.484 0 0 0-.555-.394A3.166 3.166 0 0 0 13 7.5c-.638 0-1.178.213-1.564.434a3.484 3.484 0 0 0-.555.394l-.025.023-.003.003s-.204.146-.353.146-.352-.145-.352-.145l-.004-.004-.025-.023a3.484 3.484 0 0 0-.555-.394 3.3 3.3 0 0 0-1.064-.39V13.5H8h.5v.039l-.005.083a2.958 2.958 0 0 1-.298 1.102 2.257 2.257 0 0 1-.763.88C7.06 15.851 6.587 16 6 16s-1.061-.148-1.434-.396a2.255 2.255 0 0 1-.763-.88 2.958 2.958 0 0 1-.302-1.185v-.025l-.001-.009v-.003s0-.002.5-.002h-.5V13a.5.5 0 0 1 1 0v.506l.003.044a1.958 1.958 0 0 0 .195.726c.095.191.23.367.423.495.19.127.466.229.879.229s.689-.102.879-.229c.193-.128.328-.304.424-.495a1.958 1.958 0 0 0 .197-.77V7.544a3.3 3.3 0 0 0-1.064.39 3.482 3.482 0 0 0-.58.417l-.004.004S5.65 8.5 5.5 8.5c-.149 0-.352-.145-.352-.145l-.004-.004a3.482 3.482 0 0 0-.58-.417A3.166 3.166 0 0 0 3 7.5c-.638 0-1.177.213-1.564.434a3.482 3.482 0 0 0-.58.417l-.004.004S.65 8.5.5 8.5C0 8.5 0 8 0 8c0-3.78 3.375-6.762 7.5-6.986V.5A.5.5 0 0 1 8 0zM6.577 2.123c-2.833.5-4.99 2.458-5.474 4.854A4.124 4.124 0 0 1 3 6.5c.806 0 1.48.25 1.962.511a9.706 9.706 0 0 1 .344-2.358c.242-.868.64-1.765 1.271-2.53zm-.615 4.93A4.16 4.16 0 0 1 8 6.5a4.16 4.16 0 0 1 2.038.553 8.688 8.688 0 0 0-.307-2.13C9.434 3.858 8.898 2.83 8 2.117c-.898.712-1.434 1.74-1.731 2.804a8.687 8.687 0 0 0-.307 2.131zm3.46-4.93c.631.765 1.03 1.662 1.272 2.53.233.833.328 1.66.344 2.358A4.14 4.14 0 0 1 13 6.5c.77 0 1.42.23 1.897.477-.484-2.396-2.641-4.355-5.474-4.854z"/>'),wx=Xu("UmbrellaFill",'<path fill-rule="evenodd" d="M8 0a.5.5 0 0 1 .5.5v.514C12.625 1.238 16 4.22 16 8c0 0 0 .5-.5.5-.149 0-.352-.145-.352-.145l-.004-.004-.025-.023a3.484 3.484 0 0 0-.555-.394A3.166 3.166 0 0 0 13 7.5c-.638 0-1.178.213-1.564.434a3.484 3.484 0 0 0-.555.394l-.025.023-.003.003s-.204.146-.353.146-.352-.145-.352-.145l-.004-.004-.025-.023a3.484 3.484 0 0 0-.555-.394 3.3 3.3 0 0 0-1.064-.39V13.5H8h.5v.039l-.005.083a2.958 2.958 0 0 1-.298 1.102 2.257 2.257 0 0 1-.763.88C7.06 15.851 6.587 16 6 16s-1.061-.148-1.434-.396a2.255 2.255 0 0 1-.763-.88 2.958 2.958 0 0 1-.302-1.185v-.025l-.001-.009v-.003s0-.002.5-.002h-.5V13a.5.5 0 0 1 1 0v.506l.003.044a1.958 1.958 0 0 0 .195.726c.095.191.23.367.423.495.19.127.466.229.879.229s.689-.102.879-.229c.193-.128.328-.304.424-.495a1.958 1.958 0 0 0 .197-.77V7.544a3.3 3.3 0 0 0-1.064.39 3.482 3.482 0 0 0-.58.417l-.004.004S5.65 8.5 5.5 8.5c-.149 0-.352-.145-.352-.145l-.004-.004a3.482 3.482 0 0 0-.58-.417A3.166 3.166 0 0 0 3 7.5c-.638 0-1.177.213-1.564.434a3.482 3.482 0 0 0-.58.417l-.004.004S.65 8.5.5 8.5C0 8.5 0 8 0 8c0-3.78 3.375-6.762 7.5-6.986V.5A.5.5 0 0 1 8 0z"/>'),Hx=Xu("Union",'<path d="M0 2a2 2 0 0 1 2-2h8a2 2 0 0 1 2 2v2h2a2 2 0 0 1 2 2v8a2 2 0 0 1-2 2H6a2 2 0 0 1-2-2v-2H2a2 2 0 0 1-2-2V2z"/>'),kx=Xu("Unlock",'<path d="M11 1a2 2 0 0 0-2 2v4a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V9a2 2 0 0 1 2-2h5V3a3 3 0 0 1 6 0v4a.5.5 0 0 1-1 0V3a2 2 0 0 0-2-2zM3 8a1 1 0 0 0-1 1v5a1 1 0 0 0 1 1h6a1 1 0 0 0 1-1V9a1 1 0 0 0-1-1H3z"/>'),Ax=Xu("UnlockFill",'<path d="M11 1a2 2 0 0 0-2 2v4a2 2 0 0 1 2 2v5a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V9a2 2 0 0 1 2-2h5V3a3 3 0 0 1 6 0v4a.5.5 0 0 1-1 0V3a2 2 0 0 0-2-2z"/>'),Sx=Xu("Upc",'<path d="M3 4.5a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7zm2 0a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7zm2 0a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7zm2 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-7zm3 0a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7z"/>'),Vx=Xu("UpcScan",'<path d="M1.5 1a.5.5 0 0 0-.5.5v3a.5.5 0 0 1-1 0v-3A1.5 1.5 0 0 1 1.5 0h3a.5.5 0 0 1 0 1h-3zM11 .5a.5.5 0 0 1 .5-.5h3A1.5 1.5 0 0 1 16 1.5v3a.5.5 0 0 1-1 0v-3a.5.5 0 0 0-.5-.5h-3a.5.5 0 0 1-.5-.5zM.5 11a.5.5 0 0 1 .5.5v3a.5.5 0 0 0 .5.5h3a.5.5 0 0 1 0 1h-3A1.5 1.5 0 0 1 0 14.5v-3a.5.5 0 0 1 .5-.5zm15 0a.5.5 0 0 1 .5.5v3a1.5 1.5 0 0 1-1.5 1.5h-3a.5.5 0 0 1 0-1h3a.5.5 0 0 0 .5-.5v-3a.5.5 0 0 1 .5-.5zM3 4.5a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7zm2 0a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7zm2 0a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7zm2 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v7a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-7zm3 0a.5.5 0 0 1 1 0v7a.5.5 0 0 1-1 0v-7z"/>'),Ox=Xu("Upload",'<path d="M.5 9.9a.5.5 0 0 1 .5.5v2.5a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-2.5a.5.5 0 0 1 1 0v2.5a2 2 0 0 1-2 2H2a2 2 0 0 1-2-2v-2.5a.5.5 0 0 1 .5-.5z"/><path d="M7.646 1.146a.5.5 0 0 1 .708 0l3 3a.5.5 0 0 1-.708.708L8.5 2.707V11.5a.5.5 0 0 1-1 0V2.707L5.354 4.854a.5.5 0 1 1-.708-.708l3-3z"/>'),Tx=Xu("VectorPen",'<path fill-rule="evenodd" d="M10.646.646a.5.5 0 0 1 .708 0l4 4a.5.5 0 0 1 0 .708l-1.902 1.902-.829 3.313a1.5 1.5 0 0 1-1.024 1.073L1.254 14.746 4.358 4.4A1.5 1.5 0 0 1 5.43 3.377l3.313-.828L10.646.646zm-1.8 2.908-3.173.793a.5.5 0 0 0-.358.342l-2.57 8.565 8.567-2.57a.5.5 0 0 0 .34-.357l.794-3.174-3.6-3.6z"/><path fill-rule="evenodd" d="M2.832 13.228 8 9a1 1 0 1 0-1-1l-4.228 5.168-.026.086.086-.026z"/>'),Dx=Xu("ViewList",'<path d="M3 4.5h10a2 2 0 0 1 2 2v3a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-3a2 2 0 0 1 2-2zm0 1a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1H3zM1 2a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13A.5.5 0 0 1 1 2zm0 12a.5.5 0 0 1 .5-.5h13a.5.5 0 0 1 0 1h-13A.5.5 0 0 1 1 14z"/>'),Cx=Xu("ViewStacked",'<path d="M3 0h10a2 2 0 0 1 2 2v3a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2V2a2 2 0 0 1 2-2zm0 1a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1V2a1 1 0 0 0-1-1H3zm0 8h10a2 2 0 0 1 2 2v3a2 2 0 0 1-2 2H3a2 2 0 0 1-2-2v-3a2 2 0 0 1 2-2zm0 1a1 1 0 0 0-1 1v3a1 1 0 0 0 1 1h10a1 1 0 0 0 1-1v-3a1 1 0 0 0-1-1H3z"/>'),jx=Xu("Vinyl",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M8 6a2 2 0 1 0 0 4 2 2 0 0 0 0-4zM4 8a4 4 0 1 1 8 0 4 4 0 0 1-8 0z"/><path d="M9 8a1 1 0 1 1-2 0 1 1 0 0 1 2 0z"/>'),xx=Xu("VinylFill",'<path d="M8 6a2 2 0 1 0 0 4 2 2 0 0 0 0-4zm0 3a1 1 0 1 1 0-2 1 1 0 0 1 0 2z"/><path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM4 8a4 4 0 1 0 8 0 4 4 0 0 0-8 0z"/>'),Bx=Xu("Voicemail",'<path d="M7 8.5A3.49 3.49 0 0 1 5.95 11h4.1a3.5 3.5 0 1 1 2.45 1h-9A3.5 3.5 0 1 1 7 8.5zm-6 0a2.5 2.5 0 1 0 5 0 2.5 2.5 0 0 0-5 0zm14 0a2.5 2.5 0 1 0-5 0 2.5 2.5 0 0 0 5 0z"/>'),Yx=Xu("VolumeDown",'<path d="M9 4a.5.5 0 0 0-.812-.39L5.825 5.5H3.5A.5.5 0 0 0 3 6v4a.5.5 0 0 0 .5.5h2.325l2.363 1.89A.5.5 0 0 0 9 12V4zM6.312 6.39 8 5.04v5.92L6.312 9.61A.5.5 0 0 0 6 9.5H4v-3h2a.5.5 0 0 0 .312-.11zM12.025 8a4.486 4.486 0 0 1-1.318 3.182L10 10.475A3.489 3.489 0 0 0 11.025 8 3.49 3.49 0 0 0 10 5.525l.707-.707A4.486 4.486 0 0 1 12.025 8z"/>'),Px=Xu("VolumeDownFill",'<path d="M9 4a.5.5 0 0 0-.812-.39L5.825 5.5H3.5A.5.5 0 0 0 3 6v4a.5.5 0 0 0 .5.5h2.325l2.363 1.89A.5.5 0 0 0 9 12V4zm3.025 4a4.486 4.486 0 0 1-1.318 3.182L10 10.475A3.489 3.489 0 0 0 11.025 8 3.49 3.49 0 0 0 10 5.525l.707-.707A4.486 4.486 0 0 1 12.025 8z"/>'),Ix=Xu("VolumeMute",'<path d="M6.717 3.55A.5.5 0 0 1 7 4v8a.5.5 0 0 1-.812.39L3.825 10.5H1.5A.5.5 0 0 1 1 10V6a.5.5 0 0 1 .5-.5h2.325l2.363-1.89a.5.5 0 0 1 .529-.06zM6 5.04 4.312 6.39A.5.5 0 0 1 4 6.5H2v3h2a.5.5 0 0 1 .312.11L6 10.96V5.04zm7.854.606a.5.5 0 0 1 0 .708L12.207 8l1.647 1.646a.5.5 0 0 1-.708.708L11.5 8.707l-1.646 1.647a.5.5 0 0 1-.708-.708L10.793 8 9.146 6.354a.5.5 0 1 1 .708-.708L11.5 7.293l1.646-1.647a.5.5 0 0 1 .708 0z"/>'),Fx=Xu("VolumeMuteFill",'<path d="M6.717 3.55A.5.5 0 0 1 7 4v8a.5.5 0 0 1-.812.39L3.825 10.5H1.5A.5.5 0 0 1 1 10V6a.5.5 0 0 1 .5-.5h2.325l2.363-1.89a.5.5 0 0 1 .529-.06zm7.137 2.096a.5.5 0 0 1 0 .708L12.207 8l1.647 1.646a.5.5 0 0 1-.708.708L11.5 8.707l-1.646 1.647a.5.5 0 0 1-.708-.708L10.793 8 9.146 6.354a.5.5 0 1 1 .708-.708L11.5 7.293l1.646-1.647a.5.5 0 0 1 .708 0z"/>'),Ex=Xu("VolumeOff",'<path d="M10.717 3.55A.5.5 0 0 1 11 4v8a.5.5 0 0 1-.812.39L7.825 10.5H5.5A.5.5 0 0 1 5 10V6a.5.5 0 0 1 .5-.5h2.325l2.363-1.89a.5.5 0 0 1 .529-.06zM10 5.04 8.312 6.39A.5.5 0 0 1 8 6.5H6v3h2a.5.5 0 0 1 .312.11L10 10.96V5.04z"/>'),$x=Xu("VolumeOffFill",'<path d="M10.717 3.55A.5.5 0 0 1 11 4v8a.5.5 0 0 1-.812.39L7.825 10.5H5.5A.5.5 0 0 1 5 10V6a.5.5 0 0 1 .5-.5h2.325l2.363-1.89a.5.5 0 0 1 .529-.06z"/>'),Rx=Xu("VolumeUp",'<path d="M11.536 14.01A8.473 8.473 0 0 0 14.026 8a8.473 8.473 0 0 0-2.49-6.01l-.708.707A7.476 7.476 0 0 1 13.025 8c0 2.071-.84 3.946-2.197 5.303l.708.707z"/><path d="M10.121 12.596A6.48 6.48 0 0 0 12.025 8a6.48 6.48 0 0 0-1.904-4.596l-.707.707A5.483 5.483 0 0 1 11.025 8a5.483 5.483 0 0 1-1.61 3.89l.706.706z"/><path d="M10.025 8a4.486 4.486 0 0 1-1.318 3.182L8 10.475A3.489 3.489 0 0 0 9.025 8c0-.966-.392-1.841-1.025-2.475l.707-.707A4.486 4.486 0 0 1 10.025 8zM7 4a.5.5 0 0 0-.812-.39L3.825 5.5H1.5A.5.5 0 0 0 1 6v4a.5.5 0 0 0 .5.5h2.325l2.363 1.89A.5.5 0 0 0 7 12V4zM4.312 6.39 6 5.04v5.92L4.312 9.61A.5.5 0 0 0 4 9.5H2v-3h2a.5.5 0 0 0 .312-.11z"/>'),Nx=Xu("VolumeUpFill",'<path d="M11.536 14.01A8.473 8.473 0 0 0 14.026 8a8.473 8.473 0 0 0-2.49-6.01l-.708.707A7.476 7.476 0 0 1 13.025 8c0 2.071-.84 3.946-2.197 5.303l.708.707z"/><path d="M10.121 12.596A6.48 6.48 0 0 0 12.025 8a6.48 6.48 0 0 0-1.904-4.596l-.707.707A5.483 5.483 0 0 1 11.025 8a5.483 5.483 0 0 1-1.61 3.89l.706.706z"/><path d="M8.707 11.182A4.486 4.486 0 0 0 10.025 8a4.486 4.486 0 0 0-1.318-3.182L8 5.525A3.489 3.489 0 0 1 9.025 8 3.49 3.49 0 0 1 8 10.475l.707.707zM6.717 3.55A.5.5 0 0 1 7 4v8a.5.5 0 0 1-.812.39L3.825 10.5H1.5A.5.5 0 0 1 1 10V6a.5.5 0 0 1 .5-.5h2.325l2.363-1.89a.5.5 0 0 1 .529-.06z"/>'),Wx=Xu("Vr",'<path d="M3 12V4a1 1 0 0 1 1-1h2.5V2H4a2 2 0 0 0-2 2v8a2 2 0 0 0 2 2h2.5v-1H4a1 1 0 0 1-1-1zm6.5 1v1H12a2 2 0 0 0 2-2V4a2 2 0 0 0-2-2H9.5v1H12a1 1 0 0 1 1 1v8a1 1 0 0 1-1 1H9.5zM8 16a.5.5 0 0 1-.5-.5V.5a.5.5 0 0 1 1 0v15a.5.5 0 0 1-.5.5z"/>'),Ux=Xu("Wallet",'<path d="M0 3a2 2 0 0 1 2-2h13.5a.5.5 0 0 1 0 1H15v2a1 1 0 0 1 1 1v8.5a1.5 1.5 0 0 1-1.5 1.5h-12A2.5 2.5 0 0 1 0 12.5V3zm1 1.732V12.5A1.5 1.5 0 0 0 2.5 14h12a.5.5 0 0 0 .5-.5V5H2a1.99 1.99 0 0 1-1-.268zM1 3a1 1 0 0 0 1 1h12V2H2a1 1 0 0 0-1 1z"/>'),Gx=Xu("Wallet2",'<path d="M12.136.326A1.5 1.5 0 0 1 14 1.78V3h.5A1.5 1.5 0 0 1 16 4.5v9a1.5 1.5 0 0 1-1.5 1.5h-13A1.5 1.5 0 0 1 0 13.5v-9a1.5 1.5 0 0 1 1.432-1.499L12.136.326zM5.562 3H13V1.78a.5.5 0 0 0-.621-.484L5.562 3zM1.5 4a.5.5 0 0 0-.5.5v9a.5.5 0 0 0 .5.5h13a.5.5 0 0 0 .5-.5v-9a.5.5 0 0 0-.5-.5h-13z"/>'),qx=Xu("WalletFill",'<path d="M1.5 2A1.5 1.5 0 0 0 0 3.5v2h6a.5.5 0 0 1 .5.5c0 .253.08.644.306.958.207.288.557.542 1.194.542.637 0 .987-.254 1.194-.542.226-.314.306-.705.306-.958a.5.5 0 0 1 .5-.5h6v-2A1.5 1.5 0 0 0 14.5 2h-13z"/><path d="M16 6.5h-5.551a2.678 2.678 0 0 1-.443 1.042C9.613 8.088 8.963 8.5 8 8.5c-.963 0-1.613-.412-2.006-.958A2.679 2.679 0 0 1 5.551 6.5H0v6A1.5 1.5 0 0 0 1.5 14h13a1.5 1.5 0 0 0 1.5-1.5v-6z"/>'),Jx=Xu("Watch",'<path d="M8.5 5a.5.5 0 0 0-1 0v2.5H6a.5.5 0 0 0 0 1h2a.5.5 0 0 0 .5-.5V5z"/><path d="M5.667 16C4.747 16 4 15.254 4 14.333v-1.86A5.985 5.985 0 0 1 2 8c0-1.777.772-3.374 2-4.472V1.667C4 .747 4.746 0 5.667 0h4.666C11.253 0 12 .746 12 1.667v1.86a5.99 5.99 0 0 1 1.918 3.48.502.502 0 0 1 .582.493v1a.5.5 0 0 1-.582.493A5.99 5.99 0 0 1 12 12.473v1.86c0 .92-.746 1.667-1.667 1.667H5.667zM13 8A5 5 0 1 0 3 8a5 5 0 0 0 10 0z"/>'),Kx=Xu("Water",'<path d="M.036 3.314a.5.5 0 0 1 .65-.278l1.757.703a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.757-.703a.5.5 0 1 1 .372.928l-1.758.703a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0L.314 3.964a.5.5 0 0 1-.278-.65zm0 3a.5.5 0 0 1 .65-.278l1.757.703a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.757-.703a.5.5 0 1 1 .372.928l-1.758.703a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0L.314 6.964a.5.5 0 0 1-.278-.65zm0 3a.5.5 0 0 1 .65-.278l1.757.703a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.757-.703a.5.5 0 1 1 .372.928l-1.758.703a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0L.314 9.964a.5.5 0 0 1-.278-.65zm0 3a.5.5 0 0 1 .65-.278l1.757.703a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.014-.406a2.5 2.5 0 0 1 1.857 0l1.015.406a1.5 1.5 0 0 0 1.114 0l1.757-.703a.5.5 0 1 1 .372.928l-1.758.703a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.014-.406a1.5 1.5 0 0 0-1.114 0l-1.015.406a2.5 2.5 0 0 1-1.857 0l-1.757-.703a.5.5 0 0 1-.278-.65z"/>'),Xx=Xu("Whatsapp",'<path d="M13.601 2.326A7.854 7.854 0 0 0 7.994 0C3.627 0 .068 3.558.064 7.926c0 1.399.366 2.76 1.057 3.965L0 16l4.204-1.102a7.933 7.933 0 0 0 3.79.965h.004c4.368 0 7.926-3.558 7.93-7.93A7.898 7.898 0 0 0 13.6 2.326zM7.994 14.521a6.573 6.573 0 0 1-3.356-.92l-.24-.144-2.494.654.666-2.433-.156-.251a6.56 6.56 0 0 1-1.007-3.505c0-3.626 2.957-6.584 6.591-6.584a6.56 6.56 0 0 1 4.66 1.931 6.557 6.557 0 0 1 1.928 4.66c-.004 3.639-2.961 6.592-6.592 6.592zm3.615-4.934c-.197-.099-1.17-.578-1.353-.646-.182-.065-.315-.099-.445.099-.133.197-.513.646-.627.775-.114.133-.232.148-.43.05-.197-.1-.836-.308-1.592-.985-.59-.525-.985-1.175-1.103-1.372-.114-.198-.011-.304.088-.403.087-.088.197-.232.296-.346.1-.114.133-.198.198-.33.065-.134.034-.248-.015-.347-.05-.099-.445-1.076-.612-1.47-.16-.389-.323-.335-.445-.34-.114-.007-.247-.007-.38-.007a.729.729 0 0 0-.529.247c-.182.198-.691.677-.691 1.654 0 .977.71 1.916.81 2.049.098.133 1.394 2.132 3.383 2.992.47.205.84.326 1.129.418.475.152.904.129 1.246.08.38-.058 1.171-.48 1.338-.943.164-.464.164-.86.114-.943-.049-.084-.182-.133-.38-.232z"/>'),Zx=Xu("Wifi",'<path d="M15.384 6.115a.485.485 0 0 0-.047-.736A12.444 12.444 0 0 0 8 3C5.259 3 2.723 3.882.663 5.379a.485.485 0 0 0-.048.736.518.518 0 0 0 .668.05A11.448 11.448 0 0 1 8 4c2.507 0 4.827.802 6.716 2.164.205.148.49.13.668-.049z"/><path d="M13.229 8.271a.482.482 0 0 0-.063-.745A9.455 9.455 0 0 0 8 6c-1.905 0-3.68.56-5.166 1.526a.48.48 0 0 0-.063.745.525.525 0 0 0 .652.065A8.46 8.46 0 0 1 8 7a8.46 8.46 0 0 1 4.576 1.336c.206.132.48.108.653-.065zm-2.183 2.183c.226-.226.185-.605-.1-.75A6.473 6.473 0 0 0 8 9c-1.06 0-2.062.254-2.946.704-.285.145-.326.524-.1.75l.015.015c.16.16.407.19.611.09A5.478 5.478 0 0 1 8 10c.868 0 1.69.201 2.42.56.203.1.45.07.61-.091l.016-.015zM9.06 12.44c.196-.196.198-.52-.04-.66A1.99 1.99 0 0 0 8 11.5a1.99 1.99 0 0 0-1.02.28c-.238.14-.236.464-.04.66l.706.706a.5.5 0 0 0 .707 0l.707-.707z"/>'),Qx=Xu("Wifi1",'<path d="M11.046 10.454c.226-.226.185-.605-.1-.75A6.473 6.473 0 0 0 8 9c-1.06 0-2.062.254-2.946.704-.285.145-.326.524-.1.75l.015.015c.16.16.407.19.611.09A5.478 5.478 0 0 1 8 10c.868 0 1.69.201 2.42.56.203.1.45.07.611-.091l.015-.015zM9.06 12.44c.196-.196.198-.52-.04-.66A1.99 1.99 0 0 0 8 11.5a1.99 1.99 0 0 0-1.02.28c-.238.14-.236.464-.04.66l.706.706a.5.5 0 0 0 .707 0l.708-.707z"/>'),eB=Xu("Wifi2",'<path d="M13.229 8.271c.216-.216.194-.578-.063-.745A9.456 9.456 0 0 0 8 6c-1.905 0-3.68.56-5.166 1.526a.48.48 0 0 0-.063.745.525.525 0 0 0 .652.065A8.46 8.46 0 0 1 8 7a8.46 8.46 0 0 1 4.577 1.336c.205.132.48.108.652-.065zm-2.183 2.183c.226-.226.185-.605-.1-.75A6.473 6.473 0 0 0 8 9c-1.06 0-2.062.254-2.946.704-.285.145-.326.524-.1.75l.015.015c.16.16.408.19.611.09A5.478 5.478 0 0 1 8 10c.868 0 1.69.201 2.42.56.203.1.45.07.611-.091l.015-.015zM9.06 12.44c.196-.196.198-.52-.04-.66A1.99 1.99 0 0 0 8 11.5a1.99 1.99 0 0 0-1.02.28c-.238.14-.236.464-.04.66l.706.706a.5.5 0 0 0 .708 0l.707-.707z"/>'),tB=Xu("WifiOff",'<path d="M10.706 3.294A12.545 12.545 0 0 0 8 3C5.259 3 2.723 3.882.663 5.379a.485.485 0 0 0-.048.736.518.518 0 0 0 .668.05A11.448 11.448 0 0 1 8 4c.63 0 1.249.05 1.852.148l.854-.854zM8 6c-1.905 0-3.68.56-5.166 1.526a.48.48 0 0 0-.063.745.525.525 0 0 0 .652.065 8.448 8.448 0 0 1 3.51-1.27L8 6zm2.596 1.404.785-.785c.63.24 1.227.545 1.785.907a.482.482 0 0 1 .063.745.525.525 0 0 1-.652.065 8.462 8.462 0 0 0-1.98-.932zM8 10l.933-.933a6.455 6.455 0 0 1 2.013.637c.285.145.326.524.1.75l-.015.015a.532.532 0 0 1-.611.09A5.478 5.478 0 0 0 8 10zm4.905-4.905.747-.747c.59.3 1.153.645 1.685 1.03a.485.485 0 0 1 .047.737.518.518 0 0 1-.668.05 11.493 11.493 0 0 0-1.811-1.07zM9.02 11.78c.238.14.236.464.04.66l-.707.706a.5.5 0 0 1-.707 0l-.707-.707c-.195-.195-.197-.518.04-.66A1.99 1.99 0 0 1 8 11.5c.374 0 .723.102 1.021.28zm4.355-9.905a.53.53 0 0 1 .75.75l-10.75 10.75a.53.53 0 0 1-.75-.75l10.75-10.75z"/>'),aB=Xu("Wind",'<path d="M12.5 2A2.5 2.5 0 0 0 10 4.5a.5.5 0 0 1-1 0A3.5 3.5 0 1 1 12.5 8H.5a.5.5 0 0 1 0-1h12a2.5 2.5 0 0 0 0-5zm-7 1a1 1 0 0 0-1 1 .5.5 0 0 1-1 0 2 2 0 1 1 2 2h-5a.5.5 0 0 1 0-1h5a1 1 0 0 0 0-2zM0 9.5A.5.5 0 0 1 .5 9h10.042a3 3 0 1 1-3 3 .5.5 0 0 1 1 0 2 2 0 1 0 2-2H.5a.5.5 0 0 1-.5-.5z"/>'),nB=Xu("Window",'<path d="M2.5 4a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zm2-.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm1 .5a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1z"/><path d="M2 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2H2zm13 2v2H1V3a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1zM2 14a1 1 0 0 1-1-1V6h14v7a1 1 0 0 1-1 1H2z"/>'),rB=Xu("WindowDock",'<path fill-rule="evenodd" d="M15 5H1v8a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1V5zm0-1H1V3a1 1 0 0 1 1-1h12a1 1 0 0 1 1 1v1zm1-1a2 2 0 0 0-2-2H2a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3z"/><path d="M3 11.5a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm4 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1zm4 0a.5.5 0 0 1 .5-.5h1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-1a.5.5 0 0 1-.5-.5v-1z"/>'),iB=Xu("WindowSidebar",'<path d="M2.5 4a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1zm2-.5a.5.5 0 1 1-1 0 .5.5 0 0 1 1 0zm1 .5a.5.5 0 1 0 0-1 .5.5 0 0 0 0 1z"/><path d="M2 1a2 2 0 0 0-2 2v10a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V3a2 2 0 0 0-2-2H2zm12 1a1 1 0 0 1 1 1v2H1V3a1 1 0 0 1 1-1h12zM1 13V6h4v8H2a1 1 0 0 1-1-1zm5 1V6h9v7a1 1 0 0 1-1 1H6z"/>'),oB=Xu("Wrench",'<path d="M.102 2.223A3.004 3.004 0 0 0 3.78 5.897l6.341 6.252A3.003 3.003 0 0 0 13 16a3 3 0 1 0-.851-5.878L5.897 3.781A3.004 3.004 0 0 0 2.223.1l2.141 2.142L4 4l-1.757.364L.102 2.223zm13.37 9.019.528.026.287.445.445.287.026.529L15 13l-.242.471-.026.529-.445.287-.287.445-.529.026L13 15l-.471-.242-.529-.026-.287-.445-.445-.287-.026-.529L11 13l.242-.471.026-.529.445-.287.287-.445.529-.026L13 11l.471.242z"/>'),sB=Xu("X",'<path d="M4.646 4.646a.5.5 0 0 1 .708 0L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 0 1 0-.708z"/>'),lB=Xu("XCircle",'<path d="M8 15A7 7 0 1 1 8 1a7 7 0 0 1 0 14zm0 1A8 8 0 1 0 8 0a8 8 0 0 0 0 16z"/><path d="M4.646 4.646a.5.5 0 0 1 .708 0L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 0 1 0-.708z"/>'),cB=Xu("XCircleFill",'<path d="M16 8A8 8 0 1 1 0 8a8 8 0 0 1 16 0zM5.354 4.646a.5.5 0 1 0-.708.708L7.293 8l-2.647 2.646a.5.5 0 0 0 .708.708L8 8.707l2.646 2.647a.5.5 0 0 0 .708-.708L8.707 8l2.647-2.646a.5.5 0 0 0-.708-.708L8 7.293 5.354 4.646z"/>'),uB=Xu("XDiamond",'<path d="M7.987 16a1.526 1.526 0 0 1-1.07-.448L.45 9.082a1.531 1.531 0 0 1 0-2.165L6.917.45a1.531 1.531 0 0 1 2.166 0l6.469 6.468A1.526 1.526 0 0 1 16 8.013a1.526 1.526 0 0 1-.448 1.07l-6.47 6.469A1.526 1.526 0 0 1 7.988 16zM7.639 1.17 4.766 4.044 8 7.278l3.234-3.234L8.361 1.17a.51.51 0 0 0-.722 0zM8.722 8l3.234 3.234 2.873-2.873c.2-.2.2-.523 0-.722l-2.873-2.873L8.722 8zM8 8.722l-3.234 3.234 2.873 2.873c.2.2.523.2.722 0l2.873-2.873L8 8.722zM7.278 8 4.044 4.766 1.17 7.639a.511.511 0 0 0 0 .722l2.874 2.873L7.278 8z"/>'),dB=Xu("XDiamondFill",'<path d="M9.05.435c-.58-.58-1.52-.58-2.1 0L4.047 3.339 8 7.293l3.954-3.954L9.049.435zm3.61 3.611L8.708 8l3.954 3.954 2.904-2.905c.58-.58.58-1.519 0-2.098l-2.904-2.905zm-.706 8.614L8 8.708l-3.954 3.954 2.905 2.904c.58.58 1.519.58 2.098 0l2.905-2.904zm-8.614-.706L7.292 8 3.339 4.046.435 6.951c-.58.58-.58 1.519 0 2.098l2.904 2.905z"/>'),hB=Xu("XLg",'<path d="M1.293 1.293a1 1 0 0 1 1.414 0L8 6.586l5.293-5.293a1 1 0 1 1 1.414 1.414L9.414 8l5.293 5.293a1 1 0 0 1-1.414 1.414L8 9.414l-5.293 5.293a1 1 0 0 1-1.414-1.414L6.586 8 1.293 2.707a1 1 0 0 1 0-1.414z"/>'),fB=Xu("XOctagon",'<path d="M4.54.146A.5.5 0 0 1 4.893 0h6.214a.5.5 0 0 1 .353.146l4.394 4.394a.5.5 0 0 1 .146.353v6.214a.5.5 0 0 1-.146.353l-4.394 4.394a.5.5 0 0 1-.353.146H4.893a.5.5 0 0 1-.353-.146L.146 11.46A.5.5 0 0 1 0 11.107V4.893a.5.5 0 0 1 .146-.353L4.54.146zM5.1 1 1 5.1v5.8L5.1 15h5.8l4.1-4.1V5.1L10.9 1H5.1z"/><path d="M4.646 4.646a.5.5 0 0 1 .708 0L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 0 1 0-.708z"/>'),pB=Xu("XOctagonFill",'<path d="M11.46.146A.5.5 0 0 0 11.107 0H4.893a.5.5 0 0 0-.353.146L.146 4.54A.5.5 0 0 0 0 4.893v6.214a.5.5 0 0 0 .146.353l4.394 4.394a.5.5 0 0 0 .353.146h6.214a.5.5 0 0 0 .353-.146l4.394-4.394a.5.5 0 0 0 .146-.353V4.893a.5.5 0 0 0-.146-.353L11.46.146zm-6.106 4.5L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 1 1 .708-.708z"/>'),mB=Xu("XSquare",'<path d="M14 1a1 1 0 0 1 1 1v12a1 1 0 0 1-1 1H2a1 1 0 0 1-1-1V2a1 1 0 0 1 1-1h12zM2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2z"/><path d="M4.646 4.646a.5.5 0 0 1 .708 0L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 0 1 0-.708z"/>'),vB=Xu("XSquareFill",'<path d="M2 0a2 2 0 0 0-2 2v12a2 2 0 0 0 2 2h12a2 2 0 0 0 2-2V2a2 2 0 0 0-2-2H2zm3.354 4.646L8 7.293l2.646-2.647a.5.5 0 0 1 .708.708L8.707 8l2.647 2.646a.5.5 0 0 1-.708.708L8 8.707l-2.646 2.647a.5.5 0 0 1-.708-.708L7.293 8 4.646 5.354a.5.5 0 1 1 .708-.708z"/>'),_B=Xu("Youtube",'<path d="M8.051 1.999h.089c.822.003 4.987.033 6.11.335a2.01 2.01 0 0 1 1.415 1.42c.101.38.172.883.22 1.402l.01.104.022.26.008.104c.065.914.073 1.77.074 1.957v.075c-.001.194-.01 1.108-.082 2.06l-.008.105-.009.104c-.05.572-.124 1.14-.235 1.558a2.007 2.007 0 0 1-1.415 1.42c-1.16.312-5.569.334-6.18.335h-.142c-.309 0-1.587-.006-2.927-.052l-.17-.006-.087-.004-.171-.007-.171-.007c-1.11-.049-2.167-.128-2.654-.26a2.007 2.007 0 0 1-1.415-1.419c-.111-.417-.185-.986-.235-1.558L.09 9.82l-.008-.104A31.4 31.4 0 0 1 0 7.68v-.123c.002-.215.01-.958.064-1.778l.007-.103.003-.052.008-.104.022-.26.01-.104c.048-.519.119-1.023.22-1.402a2.007 2.007 0 0 1 1.415-1.42c.487-.13 1.544-.21 2.654-.26l.17-.007.172-.006.086-.003.171-.007A99.788 99.788 0 0 1 7.858 2h.193zM6.4 5.209v4.818l4.157-2.408L6.4 5.209z"/>'),gB=Xu("ZoomIn",'<path fill-rule="evenodd" d="M6.5 12a5.5 5.5 0 1 0 0-11 5.5 5.5 0 0 0 0 11zM13 6.5a6.5 6.5 0 1 1-13 0 6.5 6.5 0 0 1 13 0z"/><path d="M10.344 11.742c.03.04.062.078.098.115l3.85 3.85a1 1 0 0 0 1.415-1.414l-3.85-3.85a1.007 1.007 0 0 0-.115-.1 6.538 6.538 0 0 1-1.398 1.4z"/><path fill-rule="evenodd" d="M6.5 3a.5.5 0 0 1 .5.5V6h2.5a.5.5 0 0 1 0 1H7v2.5a.5.5 0 0 1-1 0V7H3.5a.5.5 0 0 1 0-1H6V3.5a.5.5 0 0 1 .5-.5z"/>'),yB=Xu("ZoomOut",'<path fill-rule="evenodd" d="M6.5 12a5.5 5.5 0 1 0 0-11 5.5 5.5 0 0 0 0 11zM13 6.5a6.5 6.5 0 1 1-13 0 6.5 6.5 0 0 1 13 0z"/><path d="M10.344 11.742c.03.04.062.078.098.115l3.85 3.85a1 1 0 0 0 1.415-1.414l-3.85-3.85a1.007 1.007 0 0 0-.115-.1 6.538 6.538 0 0 1-1.398 1.4z"/><path fill-rule="evenodd" d="M3 6.5a.5.5 0 0 1 .5-.5h6a.5.5 0 0 1 0 1h-6a.5.5 0 0 1-.5-.5z"/>');function bB(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function MB(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?bB(Object(a),!0).forEach((function(t){zB(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):bB(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function zB(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var LB=function e(t,a){if(!t)return i["default"].component(a);var n=(t.$options||{}).components,r=n&&n[a];return r||e(t.$parent,a)},wB=ia(Wu,["content"]),HB=Sc(sa(MB(MB({},wB),{},{icon:wc(Yo)})),rr),kB=i["default"].extend({name:rr,functional:!0,props:HB,render:function(e,t){var a=t.data,n=t.props,r=t.parent,i=_l(wl(n.icon||"")).replace(ct,"");return e(i&&LB(r,"BIcon".concat(i))||Zu,Ja(a,{props:kc(wB,n)}))}}),AB=8,SB=46,VB=40,OB=35,TB=13,DB=27,CB=36,jB=37,xB=34,BB=33,YB=39,PB=32,IB=38,FB=function(e,t){if(e.length!==t.length)return!1;for(var a=!0,n=0;a&&n<e.length;n++)a=EB(e[n],t[n]);return a},EB=function e(t,a){if(t===a)return!0;var n=Et(t),r=Et(a);if(n||r)return!(!n||!r)&&t.getTime()===a.getTime();if(n=Pt(t),r=Pt(a),n||r)return!(!n||!r)&&FB(t,a);if(n=It(t),r=It(a),n||r){if(!n||!r)return!1;var i=ea(t).length,o=ea(a).length;if(i!==o)return!1;for(var s in t){var l=ta(t,s),c=ta(a,s);if(l&&!c||!l&&c||!e(t[s],a[s]))return!1}}return String(t)===String(a)};function $B(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var RB=function(e){return!e||0===ea(e).length},NB=function(e){return{handler:function(t,a){if(!EB(t,a))if(RB(t)||RB(a))this[e]=ga(t);else{for(var n in a)ta(t,n)||this.$delete(this.$data[e],n);for(var r in t)this.$set(this.$data[e],r,t[r])}}}},WB=function(e,t){return i["default"].extend({data:function(){return $B({},t,ga(this[e]))},watch:$B({},e,NB(t))})},UB=WB("$attrs","bvAttrs"),GB="$_rootListeners",qB=i["default"].extend({created:function(){this[GB]={}},beforeDestroy:function(){var e=this;ea(this[GB]||{}).forEach((function(t){e[GB][t].forEach((function(a){e.listenOffRoot(t,a)}))})),this[GB]=null},methods:{registerRootListener:function(e,t){this[GB]&&(this[GB][e]=this[GB][e]||[],ll(this[GB][e],t)||this[GB][e].push(t))},unregisterRootListener:function(e,t){this[GB]&&this[GB][e]&&(this[GB][e]=this[GB][e].filter((function(e){return e!==t})))},listenOnRoot:function(e,t){this.$root&&(this.$root.$on(e,t),this.registerRootListener(e,t))},listenOnRootOnce:function(e,t){var a=this;if(this.$root){var n=function e(){a.unregisterRootListener(e),t.apply(void 0,arguments)};this.$root.$once(e,n),this.registerRootListener(e,n)}},listenOffRoot:function(e,t){this.unregisterRootListener(e,t),this.$root&&this.$root.$off(e,t)},emitOnRoot:function(e){if(this.$root){for(var t,a=arguments.length,n=new Array(a>1?a-1:0),r=1;r<a;r++)n[r-1]=arguments[r];(t=this.$root).$emit.apply(t,[e].concat(n))}}}}),JB=WB("$listeners","bvListeners");function KB(e){return eY(e)||QB(e)||ZB(e)||XB()}function XB(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function ZB(e,t){if(e){if("string"===typeof e)return tY(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?tY(e,t):void 0}}function QB(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function eY(e){if(Array.isArray(e))return tY(e)}function tY(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}function aY(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function nY(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?aY(Object(a),!0).forEach((function(t){rY(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):aY(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function rY(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var iY=$c(mr,"clicked"),oY={activeClass:wc(Yo),append:wc(To,!1),event:wc(Eo),exact:wc(To,!1),exactActiveClass:wc(Yo),exactPath:wc(To,!1),exactPathActiveClass:wc(Yo),replace:wc(To,!1),routerTag:wc(Yo),to:wc(Ko)},sY={noPrefetch:wc(To,!1),prefetch:wc(To,null)},lY=Sc(sa(nY(nY(nY({},sY),oY),{},{active:wc(To,!1),disabled:wc(To,!1),href:wc(Yo),rel:wc(Yo,null),routerComponentName:wc(Yo),target:wc(Yo,"_self")})),mr),cY=i["default"].extend({name:mr,mixins:[UB,JB,qB,xc],inheritAttrs:!1,props:lY,computed:{computedTag:function(){var e=this.to,t=this.disabled,a=this.routerComponentName;return Iu({to:e,disabled:t,routerComponentName:a},this)},isRouterLink:function(){return Pu(this.computedTag)},computedRel:function(){var e=this.target,t=this.rel;return Fu({target:e,rel:t})},computedHref:function(){var e=this.to,t=this.href;return Eu({to:e,href:t},this.computedTag)},computedProps:function(){var e=this.event,t=this.prefetch,a=this.routerTag;return this.isRouterLink?nY(nY(nY(nY({},kc(ia(nY(nY({},oY),sY),["event","prefetch","routerTag"]),this)),e?{event:e}:{}),jt(t)?{prefetch:t}:{}),a?{tag:a}:{}):{}},computedAttrs:function(){var e=this.bvAttrs,t=this.computedHref,a=this.computedRel,n=this.disabled,r=this.target,i=this.routerTag,o=this.isRouterLink;return nY(nY(nY(nY({},e),t?{href:t}:{}),o&&i&&!xl(i,"a")?{}:{rel:a,target:r}),{},{tabindex:n?"-1":Ot(e.tabindex)?null:e.tabindex,"aria-disabled":n?"true":null})},computedListeners:function(){return nY(nY({},this.bvListeners),{},{click:this.onClick})}},methods:{onClick:function(e){var t=arguments,a=$t(e),n=this.isRouterLink,r=this.bvListeners.click;a&&this.disabled?Fc(e,{immediatePropagation:!0}):(n&&e.currentTarget.__vue__&&e.currentTarget.__vue__.$emit(Hi,e),cl(r).filter((function(e){return Ct(e)})).forEach((function(e){e.apply(void 0,KB(t))})),this.emitOnRoot(iY,e),this.emitOnRoot("clicked::link",e)),a&&!n&&"#"===this.computedHref&&Fc(e,{propagation:!1})},focus:function(){lc(this.$el)},blur:function(){cc(this.$el)}},render:function(e){var t=this.active,a=this.disabled;return e(this.computedTag,rY({class:{active:t,disabled:a},attrs:this.computedAttrs,props:this.computedProps},this.isRouterLink?"nativeOn":"on",this.computedListeners),this.normalizeSlot())}});function uY(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function dY(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?uY(Object(a),!0).forEach((function(t){hY(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):uY(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function hY(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var fY=ia(lY,["event","routerTag"]);delete fY.href.default,delete fY.to.default;var pY=Sc(sa(dY(dY({},fY),{},{block:wc(To,!1),disabled:wc(To,!1),pill:wc(To,!1),pressed:wc(To,null),size:wc(Yo),squared:wc(To,!1),tag:wc(Yo,"button"),type:wc(Yo,"button"),variant:wc(Yo,"secondary")})),on),mY=function(e){"focusin"===e.type?Ul(e.target,"focus"):"focusout"===e.type&&Gl(e.target,"focus")},vY=function(e){return Yu(e)||xl(e.tag,"a")},_Y=function(e){return jt(e.pressed)},gY=function(e){return!(vY(e)||e.tag&&!xl(e.tag,"button"))},yY=function(e){return!vY(e)&&!gY(e)},bY=function(e){var t;return["btn-".concat(e.variant||"secondary"),(t={},hY(t,"btn-".concat(e.size),e.size),hY(t,"btn-block",e.block),hY(t,"rounded-pill",e.pill),hY(t,"rounded-0",e.squared&&!e.pill),hY(t,"disabled",e.disabled),hY(t,"active",e.pressed),t)]},MY=function(e){return vY(e)?kc(fY,e):{}},zY=function(e,t){var a=gY(e),n=vY(e),r=_Y(e),i=yY(e),o=n&&"#"===e.href,s=t.attrs&&t.attrs.role?t.attrs.role:null,l=t.attrs?t.attrs.tabindex:null;return(i||o)&&(l="0"),{type:a&&!n?e.type:null,disabled:a?e.disabled:null,role:i||o?"button":s,"aria-disabled":i?String(e.disabled):null,"aria-pressed":r?String(e.pressed):null,autocomplete:r?"off":null,tabindex:e.disabled&&!a?"-1":l}},LY=i["default"].extend({name:on,functional:!0,props:pY,render:function(e,t){var a=t.props,n=t.data,r=t.listeners,i=t.children,o=_Y(a),s=vY(a),l=yY(a),c=s&&"#"===a.href,u={keydown:function(e){if(!a.disabled&&(l||c)){var t=e.keyCode;if(t===PB||t===TB&&l){var n=e.currentTarget||e.target;Fc(e,{propagation:!1}),n.click()}}},click:function(e){a.disabled&&$t(e)?Fc(e):o&&r&&r["update:pressed"]&&cl(r["update:pressed"]).forEach((function(e){Ct(e)&&e(!a.pressed)}))}};o&&(u.focusin=mY,u.focusout=mY);var d={staticClass:"btn",class:bY(a),props:MY(a),attrs:zY(a,n),on:u};return e(s?cY:a.tag,Ja(n,d),i)}});function wY(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function HY(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?wY(Object(a),!0).forEach((function(t){kY(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):wY(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function kY(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var AY="b-avatar",SY=["sm",null,"lg"],VY=.4,OY=.7*VY,TY=function(e){return e=xt(e)&&Yt(e)?pl(e,0):e,Bt(e)?"".concat(e,"px"):e||null},DY=ia(lY,["active","event","routerTag"]),CY=Sc(sa(HY(HY({},DY),{},{alt:wc(Yo,"avatar"),ariaLabel:wc(Yo),badge:wc(No,!1),badgeLeft:wc(To,!1),badgeOffset:wc(Yo),badgeTop:wc(To,!1),badgeVariant:wc(Yo,"primary"),button:wc(To,!1),buttonType:wc(Yo,"button"),icon:wc(Yo),rounded:wc(No,!1),size:wc(Go),square:wc(To,!1),src:wc(Yo),text:wc(Yo),variant:wc(Yo,"secondary")})),Qa),jY=i["default"].extend({name:Qa,mixins:[xc],inject:{bvAvatarGroup:{default:null}},props:CY,data:function(){return{localSrc:this.src||null}},computed:{computedSize:function(){var e=this.bvAvatarGroup;return TY(e?e.size:this.size)},computedVariant:function(){var e=this.bvAvatarGroup;return e&&e.variant?e.variant:this.variant},computedRounded:function(){var e=this.bvAvatarGroup,t=!(!e||!e.square)||this.square,a=e&&e.rounded?e.rounded:this.rounded;return t?"0":""===a||(a||"circle")},fontStyle:function(){var e=this.computedSize,t=-1===SY.indexOf(e)?"calc(".concat(e," * ").concat(VY,")"):null;return t?{fontSize:t}:{}},marginStyle:function(){var e=this.computedSize,t=this.bvAvatarGroup,a=t?t.overlapScale:0,n=e&&a?"calc(".concat(e," * -").concat(a,")"):null;return n?{marginLeft:n,marginRight:n}:{}},badgeStyle:function(){var e=this.computedSize,t=this.badgeTop,a=this.badgeLeft,n=this.badgeOffset,r=n||"0px";return{fontSize:-1===SY.indexOf(e)?"calc(".concat(e," * ").concat(OY," )"):null,top:t?r:null,bottom:t?null:r,left:a?r:null,right:a?null:r}}},watch:{src:function(e,t){e!==t&&(this.localSrc=e||null)}},methods:{onImgError:function(e){this.localSrc=null,this.$emit(Ni,e)},onClick:function(e){this.$emit(Hi,e)}},render:function(e){var t,a=this.computedVariant,n=this.disabled,r=this.computedRounded,i=this.icon,o=this.localSrc,s=this.text,l=this.fontStyle,c=this.marginStyle,u=this.computedSize,d=this.button,h=this.buttonType,f=this.badge,p=this.badgeVariant,m=this.badgeStyle,v=!d&&Yu(this),_=d?LY:v?cY:"span",g=this.alt,y=this.ariaLabel||null,b=null;this.hasNormalizedSlot()?b=e("span",{staticClass:"b-avatar-custom"},[this.normalizeSlot()]):o?(b=e("img",{style:a?{}:{width:"100%",height:"100%"},attrs:{src:o,alt:g},on:{error:this.onImgError}}),b=e("span",{staticClass:"b-avatar-img"},[b])):b=i?e(kB,{props:{icon:i},attrs:{"aria-hidden":"true",alt:g}}):s?e("span",{staticClass:"b-avatar-text",style:l},[e("span",s)]):e(EV,{attrs:{"aria-hidden":"true",alt:g}});var M=e(),z=this.hasNormalizedSlot(es);if(f||""===f||z){var L=!0===f?"":f;M=e("span",{staticClass:"b-avatar-badge",class:kY({},"badge-".concat(p),p),style:m},[z?this.normalizeSlot(es):L])}var w={staticClass:AY,class:(t={},kY(t,"".concat(AY,"-").concat(u),u&&-1!==SY.indexOf(u)),kY(t,"badge-".concat(a),!d&&a),kY(t,"rounded",!0===r),kY(t,"rounded-".concat(r),r&&!0!==r),kY(t,"disabled",n),t),style:HY(HY({},c),{},{width:u,height:u}),attrs:{"aria-label":y||null},props:d?{variant:a,disabled:n,type:h}:v?kc(DY,this):{},on:d||v?{click:this.onClick}:{}};return e(_,w,[b,M])}}),xY=Sc({overlap:wc(Go,.3),rounded:wc(No,!1),size:wc(Yo),square:wc(To,!1),tag:wc(Yo,"div"),variant:wc(Yo)},en),BY=i["default"].extend({name:en,mixins:[xc],provide:function(){return{bvAvatarGroup:this}},props:xY,computed:{computedSize:function(){return TY(this.size)},overlapScale:function(){return pu(mu(pl(this.overlap,0),0),1)/2},paddingStyle:function(){var e=this.computedSize;return e=e?"calc(".concat(e," * ").concat(this.overlapScale,")"):null,e?{paddingLeft:e,paddingRight:e}:{}}},render:function(e){var t=e("div",{staticClass:"b-avatar-group-inner",style:this.paddingStyle},this.normalizeSlot());return e(this.tag,{staticClass:"b-avatar-group",attrs:{role:"group"}},[t])}}),YY=Pa({components:{BAvatar:jY,BAvatarGroup:BY}});function PY(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function IY(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?PY(Object(a),!0).forEach((function(t){FY(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):PY(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function FY(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var EY=ia(lY,["event","routerTag"]);delete EY.href.default,delete EY.to.default;var $Y=Sc(sa(IY(IY({},EY),{},{pill:wc(To,!1),tag:wc(Yo,"span"),variant:wc(Yo,"secondary")})),tn),RY=i["default"].extend({name:tn,functional:!0,props:$Y,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.active,o=a.disabled,s=Yu(a),l=s?cY:a.tag,c=a.variant||"secondary";return e(l,Ja(n,{staticClass:"badge",class:["badge-".concat(c),{"badge-pill":a.pill,active:i,disabled:o}],props:s?kc(EY,a):{}}),r)}}),NY=Pa({components:{BBadge:RY}}),WY=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"";return String(e).replace(Fe,"")},UY=function(e,t){return e?{innerHTML:e}:t?{textContent:t}:{}};function GY(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function qY(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?GY(Object(a),!0).forEach((function(t){JY(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):GY(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function JY(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var KY=Sc(sa(qY(qY({},ia(lY,["event","routerTag"])),{},{ariaCurrent:wc(Yo,"location"),html:wc(Yo),text:wc(Yo)})),rn),XY=i["default"].extend({name:rn,functional:!0,props:KY,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.active,o=i?"span":cY,s={attrs:{"aria-current":i?a.ariaCurrent:null},props:kc(KY,a)};return r||(s.domProps=UY(a.html,a.text)),e(o,Ja(n,s),r)}}),ZY=Sc(KY,nn),QY=i["default"].extend({name:nn,functional:!0,props:ZY,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e("li",Ja(n,{staticClass:"breadcrumb-item",class:{active:a.active}}),[e(XY,{props:a},r)])}});function eP(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function tP(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?eP(Object(a),!0).forEach((function(t){aP(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):eP(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function aP(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var nP=Sc({items:wc(Oo)},an),rP=i["default"].extend({name:an,functional:!0,props:nP,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.items,o=r;if(Pt(i)){var s=!1;o=i.map((function(t,a){It(t)||(t={text:zl(t)});var n=t,r=n.active;return r&&(s=!0),r||s||(r=a+1===i.length),e(QY,{props:tP(tP({},t),{},{active:r})})}))}return e("ol",Ja(n,{staticClass:"breadcrumb"}),o)}}),iP=Pa({components:{BBreadcrumb:rP,BBreadcrumbItem:QY,BBreadcrumbLink:XY}}),oP=Pa({components:{BButton:LY,BBtn:LY,BButtonClose:Uc,BBtnClose:Uc}});function sP(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function lP(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?sP(Object(a),!0).forEach((function(t){cP(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):sP(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function cP(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var uP=Sc(sa(lP(lP({},ra(pY,["size"])),{},{ariaRole:wc(Yo,"group"),size:wc(Yo),tag:wc(Yo,"div"),vertical:wc(To,!1)})),ln),dP=i["default"].extend({name:ln,functional:!0,props:uP,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.tag,Ja(n,{class:cP({"btn-group":!a.vertical,"btn-group-vertical":a.vertical},"btn-group-".concat(a.size),a.size),attrs:{role:a.ariaRole}}),r)}}),hP=Pa({components:{BButtonGroup:dP,BBtnGroup:dP}}),fP=[".btn:not(.disabled):not([disabled]):not(.dropdown-item)",".form-control:not(.disabled):not([disabled])","select:not(.disabled):not([disabled])",'input[type="checkbox"]:not(.disabled)','input[type="radio"]:not(.disabled)'].join(","),pP=Sc({justify:wc(To,!1),keyNav:wc(To,!1)},cn),mP=i["default"].extend({name:cn,mixins:[xc],props:pP,mounted:function(){this.keyNav&&this.getItems()},methods:{getItems:function(){var e=Fl(fP,this.$el);return e.forEach((function(e){e.tabIndex=-1})),e.filter((function(e){return Yl(e)}))},focusFirst:function(){var e=this.getItems();lc(e[0])},focusPrev:function(e){var t=this.getItems(),a=t.indexOf(e.target);a>-1&&(t=t.slice(0,a).reverse(),lc(t[0]))},focusNext:function(e){var t=this.getItems(),a=t.indexOf(e.target);a>-1&&(t=t.slice(a+1),lc(t[0]))},focusLast:function(){var e=this.getItems().reverse();lc(e[0])},onFocusin:function(e){var t=this.$el;e.target!==t||Nl(t,e.relatedTarget)||(Fc(e),this.focusFirst(e))},onKeydown:function(e){var t=e.keyCode,a=e.shiftKey;t===IB||t===jB?(Fc(e),a?this.focusFirst(e):this.focusPrev(e)):t!==VB&&t!==YB||(Fc(e),a?this.focusLast(e):this.focusNext(e))}},render:function(e){var t=this.keyNav;return e("div",{staticClass:"btn-toolbar",class:{"justify-content-between":this.justify},attrs:{role:"toolbar",tabindex:t?"0":null},on:t?{focusin:this.onFocusin,keydown:this.onKeydown}:{}},[this.normalizeSlot()])}}),vP=Pa({components:{BButtonToolbar:mP,BBtnToolbar:mP}}),_P="gregory",gP="long",yP="narrow",bP="short",MP="2-digit",zP="numeric";function LP(e,t){return SP(e)||AP(e,t)||HP(e,t)||wP()}function wP(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function HP(e,t){if(e){if("string"===typeof e)return kP(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?kP(e,t):void 0}}function kP(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}function AP(e,t){var a=null==e?null:"undefined"!==typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=a){var n,r,i=[],o=!0,s=!1;try{for(a=a.call(e);!(o=(n=a.next()).done);o=!0)if(i.push(n.value),t&&i.length===t)break}catch(l){s=!0,r=l}finally{try{o||null==a["return"]||a["return"]()}finally{if(s)throw r}}return i}}function SP(e){if(Array.isArray(e))return e}function VP(e,t,a){return VP=OP()?Reflect.construct:function(e,t,a){var n=[null];n.push.apply(n,t);var r=Function.bind.apply(e,n),i=new r;return a&&TP(i,a.prototype),i},VP.apply(null,arguments)}function OP(){if("undefined"===typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"===typeof Proxy)return!0;try{return Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],(function(){}))),!0}catch(e){return!1}}function TP(e,t){return TP=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e},TP(e,t)}var DP,CP=function(){for(var e=arguments.length,t=new Array(e),a=0;a<e;a++)t[a]=arguments[a];return VP(Date,t)},jP=function(e){if(xt(e)&&Qe.test(e.trim())){var t=e.split(et).map((function(e){return fl(e,1)})),a=LP(t,3),n=a[0],r=a[1],i=a[2];return CP(n,r-1,i)}return Et(e)?CP(e.getFullYear(),e.getMonth(),e.getDate()):null},xP=function(e){if(e=jP(e),!e)return null;var t=e.getFullYear(),a="0".concat(e.getMonth()+1).slice(-2),n="0".concat(e.getDate()).slice(-2);return"".concat(t,"-").concat(a,"-").concat(n)},BP=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:_P;e=cl(e).filter(ya);var a=new Intl.DateTimeFormat(e,{calendar:t});return a.resolvedOptions().locale},YP=function(e,t){var a=new Intl.DateTimeFormat(e,t);return a.format},PP=function(e,t){return xP(e)===xP(t)},IP=function(e){return e=CP(e),e.setDate(1),e},FP=function(e){return e=CP(e),e.setMonth(e.getMonth()+1),e.setDate(0),e},EP=function(e,t){e=CP(e);var a=e.getMonth();return e.setFullYear(e.getFullYear()+t),e.getMonth()!==a&&e.setDate(0),e},$P=function(e){e=CP(e);var t=e.getMonth();return e.setMonth(t-1),e.getMonth()===t&&e.setDate(0),e},RP=function(e){e=CP(e);var t=e.getMonth();return e.setMonth(t+1),e.getMonth()===(t+2)%12&&e.setDate(0),e},NP=function(e){return EP(e,-1)},WP=function(e){return EP(e,1)},UP=function(e){return EP(e,-10)},GP=function(e){return EP(e,10)},qP=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:null;return e=jP(e),t=jP(t)||e,a=jP(a)||e,e?e<t?t:e>a?a:e:null},JP=["ar","az","ckb","fa","he","ks","lrc","mzn","ps","sd","te","ug","ur","yi"].map((function(e){return e.toLowerCase()})),KP=function(e){var t=zl(e).toLowerCase().replace(ut,"").split("-"),a=t.slice(0,2).join("-"),n=t[0];return ll(JP,a)||ll(JP,n)},XP={id:wc(Yo)},ZP=i["default"].extend({props:XP,data:function(){return{localId_:null}},computed:{safeId:function(){var e=this.id||this.localId_,t=function(t){return e?(t=String(t||"").replace(/\s+/g,"_"),t?e+"_"+t:e):null};return t}},mounted:function(){var e=this;this.$nextTick((function(){e.localId_="__BVID__".concat(e[Ka])}))}});function QP(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function eI(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?QP(Object(a),!0).forEach((function(t){tI(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):QP(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function tI(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var aI=Dc("value",{type:Wo}),nI=aI.mixin,rI=aI.props,iI=aI.prop,oI=aI.event,sI=Sc(sa(eI(eI(eI({},XP),rI),{},{ariaControls:wc(Yo),block:wc(To,!1),dateDisabledFn:wc(Co),dateFormatOptions:wc(xo,{year:zP,month:gP,day:zP,weekday:gP}),dateInfoFn:wc(Co),direction:wc(Yo),disabled:wc(To,!1),headerTag:wc(Yo,"header"),hidden:wc(To,!1),hideHeader:wc(To,!1),initialDate:wc(Wo),labelCalendar:wc(Yo,"Calendar"),labelCurrentMonth:wc(Yo,"Current month"),labelHelp:wc(Yo,"Use cursor keys to navigate calendar dates"),labelNav:wc(Yo,"Calendar navigation"),labelNextDecade:wc(Yo,"Next decade"),labelNextMonth:wc(Yo,"Next month"),labelNextYear:wc(Yo,"Next year"),labelNoDateSelected:wc(Yo,"No date selected"),labelPrevDecade:wc(Yo,"Previous decade"),labelPrevMonth:wc(Yo,"Previous month"),labelPrevYear:wc(Yo,"Previous year"),labelSelected:wc(Yo,"Selected date"),labelToday:wc(Yo,"Today"),locale:wc(Eo),max:wc(Wo),min:wc(Wo),navButtonVariant:wc(Yo,"secondary"),noHighlightToday:wc(To,!1),noKeyNav:wc(To,!1),readonly:wc(To,!1),roleDescription:wc(Yo),selectedVariant:wc(Yo,"primary"),showDecadeNav:wc(To,!1),startWeekday:wc(Go,0),todayVariant:wc(Yo),valueAsDate:wc(To,!1),weekdayHeaderFormat:wc(Yo,bP,(function(e){return ll([gP,bP,yP],e)})),width:wc(Yo,"270px")})),un),lI=i["default"].extend({name:un,mixins:[UB,ZP,nI,xc],props:sI,data:function(){var e=xP(this[iI])||"";return{selectedYMD:e,activeYMD:e||xP(qP(this.initialDate||this.getToday()),this.min,this.max),gridHasFocus:!1,isLive:!1}},computed:{valueId:function(){return this.safeId()},widgetId:function(){return this.safeId("_calendar-wrapper_")},navId:function(){return this.safeId("_calendar-nav_")},gridId:function(){return this.safeId("_calendar-grid_")},gridCaptionId:function(){return this.safeId("_calendar-grid-caption_")},gridHelpId:function(){return this.safeId("_calendar-grid-help_")},activeId:function(){return this.activeYMD?this.safeId("_cell-".concat(this.activeYMD,"_")):null},selectedDate:function(){return jP(this.selectedYMD)},activeDate:function(){return jP(this.activeYMD)},computedMin:function(){return jP(this.min)},computedMax:function(){return jP(this.max)},computedWeekStarts:function(){return mu(fl(this.startWeekday,0),0)%7},computedLocale:function(){return BP(cl(this.locale).filter(ya),_P)},computedDateDisabledFn:function(){var e=this.dateDisabledFn;return Oc(e)?e:function(){return!1}},computedDateInfoFn:function(){var e=this.dateInfoFn;return Oc(e)?e:function(){return{}}},calendarLocale:function(){var e=new Intl.DateTimeFormat(this.computedLocale,{calendar:_P}),t=e.resolvedOptions().calendar,a=e.resolvedOptions().locale;return t!==_P&&(a=a.replace(/-u-.+$/i,"").concat("-u-ca-gregory")),a},calendarYear:function(){return this.activeDate.getFullYear()},calendarMonth:function(){return this.activeDate.getMonth()},calendarFirstDay:function(){return CP(this.calendarYear,this.calendarMonth,1,12)},calendarDaysInMonth:function(){var e=CP(this.calendarFirstDay);return e.setMonth(e.getMonth()+1,0),e.getDate()},computedVariant:function(){return"btn-".concat(this.selectedVariant||"primary")},computedTodayVariant:function(){return"btn-outline-".concat(this.todayVariant||this.selectedVariant||"primary")},computedNavButtonVariant:function(){return"btn-outline-".concat(this.navButtonVariant||"primary")},isRTL:function(){var e=zl(this.direction).toLowerCase();return"rtl"===e||"ltr"!==e&&KP(this.computedLocale)},context:function(){var e=this.selectedYMD,t=this.activeYMD,a=jP(e),n=jP(t);return{selectedYMD:e,selectedDate:a,selectedFormatted:a?this.formatDateString(a):this.labelNoDateSelected,activeYMD:t,activeDate:n,activeFormatted:n?this.formatDateString(n):"",disabled:this.dateDisabled(n),locale:this.computedLocale,calendarLocale:this.calendarLocale,rtl:this.isRTL}},dateOutOfRange:function(){var e=this.computedMin,t=this.computedMax;return function(a){return a=jP(a),e&&a<e||t&&a>t}},dateDisabled:function(){var e=this,t=this.dateOutOfRange;return function(a){a=jP(a);var n=xP(a);return!(!t(a)&&!e.computedDateDisabledFn(n,a))}},formatDateString:function(){return YP(this.calendarLocale,eI(eI({year:zP,month:MP,day:MP},this.dateFormatOptions),{},{hour:void 0,minute:void 0,second:void 0,calendar:_P}))},formatYearMonth:function(){return YP(this.calendarLocale,{year:zP,month:gP,calendar:_P})},formatWeekdayName:function(){return YP(this.calendarLocale,{weekday:gP,calendar:_P})},formatWeekdayNameShort:function(){return YP(this.calendarLocale,{weekday:this.weekdayHeaderFormat||bP,calendar:_P})},formatDay:function(){var e=new Intl.NumberFormat([this.computedLocale],{style:"decimal",minimumIntegerDigits:1,minimumFractionDigits:0,maximumFractionDigits:0,notation:"standard"});return function(t){return e.format(t.getDate())}},prevDecadeDisabled:function(){var e=this.computedMin;return this.disabled||e&&FP(UP(this.activeDate))<e},prevYearDisabled:function(){var e=this.computedMin;return this.disabled||e&&FP(NP(this.activeDate))<e},prevMonthDisabled:function(){var e=this.computedMin;return this.disabled||e&&FP($P(this.activeDate))<e},thisMonthDisabled:function(){return this.disabled},nextMonthDisabled:function(){var e=this.computedMax;return this.disabled||e&&IP(RP(this.activeDate))>e},nextYearDisabled:function(){var e=this.computedMax;return this.disabled||e&&IP(WP(this.activeDate))>e},nextDecadeDisabled:function(){var e=this.computedMax;return this.disabled||e&&IP(GP(this.activeDate))>e},calendar:function(){for(var e=[],t=this.calendarFirstDay,a=t.getFullYear(),n=t.getMonth(),r=this.calendarDaysInMonth,i=t.getDay(),o=(this.computedWeekStarts>i?7:0)-this.computedWeekStarts,s=0-o-i,l=0;l<6&&s<r;l++){e[l]=[];for(var c=0;c<7;c++){s++;var u=CP(a,n,s),d=u.getMonth(),h=xP(u),f=this.dateDisabled(u),p=this.computedDateInfoFn(h,jP(h));p=xt(p)||Pt(p)?{class:p}:Ft(p)?eI({class:""},p):{class:""},e[l].push({ymd:h,day:this.formatDay(u),label:this.formatDateString(u),isThisMonth:d===n,isDisabled:f,info:p})}}return e},calendarHeadings:function(){var e=this;return this.calendar[0].map((function(t){return{text:e.formatWeekdayNameShort(jP(t.ymd)),label:e.formatWeekdayName(jP(t.ymd))}}))}},watch:(DP={},tI(DP,iI,(function(e,t){var a=xP(e)||"",n=xP(t)||"";PP(a,n)||(this.activeYMD=a||this.activeYMD,this.selectedYMD=a)})),tI(DP,"selectedYMD",(function(e,t){e!==t&&this.$emit(oI,this.valueAsDate?jP(e)||null:e||"")})),tI(DP,"context",(function(e,t){EB(e,t)||this.$emit(Ai,e)})),tI(DP,"hidden",(function(e){this.activeYMD=this.selectedYMD||xP(this[iI]||this.constrainDate(this.initialDate||this.getToday())),this.setLive(!e)})),DP),created:function(){var e=this;this.$nextTick((function(){e.$emit(Ai,e.context)}))},mounted:function(){this.setLive(!0)},activated:function(){this.setLive(!0)},deactivated:function(){this.setLive(!1)},beforeDestroy:function(){this.setLive(!1)},methods:{focus:function(){this.disabled||lc(this.$refs.grid)},blur:function(){this.disabled||cc(this.$refs.grid)},setLive:function(e){var t=this;e?this.$nextTick((function(){Ol((function(){t.isLive=!0}))})):this.isLive=!1},getToday:function(){return jP(CP())},constrainDate:function(e){return qP(e,this.computedMin,this.computedMax)},emitSelected:function(e){var t=this;this.$nextTick((function(){t.$emit(ho,xP(e)||"",jP(e)||null)}))},setGridFocusFlag:function(e){this.gridHasFocus=!this.disabled&&"focus"===e.type},onKeydownWrapper:function(e){if(!this.noKeyNav){var t=e.altKey,a=e.ctrlKey,n=e.keyCode;if(ll([BB,xB,OB,CB,jB,IB,YB,VB],n)){Fc(e);var r=CP(this.activeDate),i=CP(this.activeDate),o=r.getDate(),s=this.constrainDate(this.getToday()),l=this.isRTL;n===BB?(r=(t?a?UP:NP:$P)(r),i=CP(r),i.setDate(1)):n===xB?(r=(t?a?GP:WP:RP)(r),i=CP(r),i.setMonth(i.getMonth()+1),i.setDate(0)):n===jB?(r.setDate(o+(l?1:-1)),r=this.constrainDate(r),i=r):n===YB?(r.setDate(o+(l?-1:1)),r=this.constrainDate(r),i=r):n===IB?(r.setDate(o-7),r=this.constrainDate(r),i=r):n===VB?(r.setDate(o+7),r=this.constrainDate(r),i=r):n===CB?(r=s,i=r):n===OB&&(r=jP(this.selectedDate)||s,i=r),this.dateOutOfRange(i)||PP(r,this.activeDate)||(this.activeYMD=xP(r)),this.focus()}}},onKeydownGrid:function(e){var t=e.keyCode,a=this.activeDate;t!==TB&&t!==PB||(Fc(e),this.disabled||this.readonly||this.dateDisabled(a)||(this.selectedYMD=xP(a),this.emitSelected(a)),this.focus())},onClickDay:function(e){var t=this.selectedDate,a=this.activeDate,n=jP(e.ymd);this.disabled||e.isDisabled||this.dateDisabled(n)||(this.readonly||(this.selectedYMD=xP(PP(n,t)?t:n),this.emitSelected(n)),this.activeYMD=xP(PP(n,a)?a:CP(n)),this.focus())},gotoPrevDecade:function(){this.activeYMD=xP(this.constrainDate(UP(this.activeDate)))},gotoPrevYear:function(){this.activeYMD=xP(this.constrainDate(NP(this.activeDate)))},gotoPrevMonth:function(){this.activeYMD=xP(this.constrainDate($P(this.activeDate)))},gotoCurrentMonth:function(){this.activeYMD=xP(this.constrainDate(this.getToday()))},gotoNextMonth:function(){this.activeYMD=xP(this.constrainDate(RP(this.activeDate)))},gotoNextYear:function(){this.activeYMD=xP(this.constrainDate(WP(this.activeDate)))},gotoNextDecade:function(){this.activeYMD=xP(this.constrainDate(GP(this.activeDate)))},onHeaderClick:function(){this.disabled||(this.activeYMD=this.selectedYMD||xP(this.getToday()),this.focus())}},render:function(e){var t=this;if(this.hidden)return e();var a=this.valueId,n=this.widgetId,r=this.navId,i=this.gridId,o=this.gridCaptionId,s=this.gridHelpId,l=this.activeId,c=this.disabled,u=this.noKeyNav,d=this.isLive,h=this.isRTL,f=this.activeYMD,p=this.selectedYMD,m=this.safeId,v=!this.showDecadeNav,_=xP(this.getToday()),g=!this.noHighlightToday,y=e("output",{staticClass:"form-control form-control-sm text-center",class:{"text-muted":c,readonly:this.readonly||c},attrs:{id:a,for:i,role:"status",tabindex:c?null:"-1","data-selected":zl(p),"aria-live":d?"polite":"off","aria-atomic":d?"true":null},on:{click:this.onHeaderClick,focus:this.onHeaderClick}},this.selectedDate?[e("bdi",{staticClass:"sr-only"}," (".concat(zl(this.labelSelected),") ")),e("bdi",this.formatDateString(this.selectedDate))]:this.labelNoDateSelected||" ");y=e(this.headerTag,{staticClass:"b-calendar-header",class:{"sr-only":this.hideHeader},attrs:{title:this.selectedDate&&this.labelSelectedDate||null}},[y]);var b={isRTL:h},M={shiftV:.5},z=eI(eI({},M),{},{flipH:h}),L=eI(eI({},M),{},{flipH:!h}),w=this.normalizeSlot(Is,b)||e(tg,{props:z}),H=this.normalizeSlot(Es,b)||e(ug,{props:z}),k=this.normalizeSlot(Fs,b)||e(mg,{props:z}),A=this.normalizeSlot($s,b)||e(yg,{props:M}),S=this.normalizeSlot(Ys,b)||e(mg,{props:L}),V=this.normalizeSlot(Ps,b)||e(ug,{props:L}),O=this.normalizeSlot(Bs,b)||e(tg,{props:L}),T=function(a,n,r,i,o){return e("button",{staticClass:"btn btn-sm border-0 flex-fill",class:[t.computedNavButtonVariant,{disabled:i}],attrs:{title:n||null,type:"button",tabindex:u?"-1":null,"aria-label":n||null,"aria-disabled":i?"true":null,"aria-keyshortcuts":o||null},on:i?{}:{click:r}},[e("div",{attrs:{"aria-hidden":"true"}},[a])])},D=e("div",{staticClass:"b-calendar-nav d-flex",attrs:{id:r,role:"group",tabindex:u?"-1":null,"aria-hidden":c?"true":null,"aria-label":this.labelNav||null,"aria-controls":i}},[v?e():T(w,this.labelPrevDecade,this.gotoPrevDecade,this.prevDecadeDisabled,"Ctrl+Alt+PageDown"),T(H,this.labelPrevYear,this.gotoPrevYear,this.prevYearDisabled,"Alt+PageDown"),T(k,this.labelPrevMonth,this.gotoPrevMonth,this.prevMonthDisabled,"PageDown"),T(A,this.labelCurrentMonth,this.gotoCurrentMonth,this.thisMonthDisabled,"Home"),T(S,this.labelNextMonth,this.gotoNextMonth,this.nextMonthDisabled,"PageUp"),T(V,this.labelNextYear,this.gotoNextYear,this.nextYearDisabled,"Alt+PageUp"),v?e():T(O,this.labelNextDecade,this.gotoNextDecade,this.nextDecadeDisabled,"Ctrl+Alt+PageUp")]),C=e("div",{staticClass:"b-calendar-grid-caption text-center font-weight-bold",class:{"text-muted":c},attrs:{id:o,"aria-live":d?"polite":null,"aria-atomic":d?"true":null},key:"grid-caption"},this.formatYearMonth(this.calendarFirstDay)),j=e("div",{staticClass:"b-calendar-grid-weekdays row no-gutters border-bottom",attrs:{"aria-hidden":"true"}},this.calendarHeadings.map((function(t,a){return e("small",{staticClass:"col text-truncate",class:{"text-muted":c},attrs:{title:t.label===t.text?null:t.label,"aria-label":t.label},key:a},t.text)}))),x=this.calendar.map((function(a){var n=a.map((function(a,n){var r,i=a.ymd===p,o=a.ymd===f,s=a.ymd===_,l=m("_cell-".concat(a.ymd,"_")),u=e("span",{staticClass:"btn border-0 rounded-circle text-nowrap",class:(r={focus:o&&t.gridHasFocus,disabled:a.isDisabled||c,active:i},tI(r,t.computedVariant,i),tI(r,t.computedTodayVariant,s&&g&&!i&&a.isThisMonth),tI(r,"btn-outline-light",!(s&&g)&&!i&&!o),tI(r,"btn-light",!(s&&g)&&!i&&o),tI(r,"text-muted",!a.isThisMonth&&!i),tI(r,"text-dark",!(s&&g)&&!i&&!o&&a.isThisMonth),tI(r,"font-weight-bold",(i||a.isThisMonth)&&!a.isDisabled),r),on:{click:function(){return t.onClickDay(a)}}},a.day);return e("div",{staticClass:"col p-0",class:a.isDisabled?"bg-light":a.info.class||"",attrs:{id:l,role:"button","data-date":a.ymd,"aria-hidden":a.isThisMonth?null:"true","aria-disabled":a.isDisabled||c?"true":null,"aria-label":[a.label,i?"(".concat(t.labelSelected,")"):null,s?"(".concat(t.labelToday,")"):null].filter(ya).join(" "),"aria-selected":i?"true":null,"aria-current":i?"date":null},key:n},[u])}));return e("div",{staticClass:"row no-gutters",key:a[0].ymd},n)}));x=e("div",{staticClass:"b-calendar-grid-body",style:c?{pointerEvents:"none"}:{}},x);var B=e("div",{staticClass:"b-calendar-grid-help border-top small text-muted text-center bg-light",attrs:{id:s}},[e("div",{staticClass:"small"},this.labelHelp)]),Y=e("div",{staticClass:"b-calendar-grid form-control h-auto text-center",attrs:{id:i,role:"application",tabindex:u?"-1":c?null:"0","data-month":f.slice(0,-3),"aria-roledescription":this.labelCalendar||null,"aria-labelledby":o,"aria-describedby":s,"aria-disabled":c?"true":null,"aria-activedescendant":l},on:{keydown:this.onKeydownGrid,focus:this.setGridFocusFlag,blur:this.setGridFocusFlag},ref:"grid"},[C,j,x,B]),P=this.normalizeSlot();P=P?e("footer",{staticClass:"b-calendar-footer"},P):e();var I=e("div",{staticClass:"b-calendar-inner",style:this.block?{}:{width:this.width},attrs:{id:n,dir:h?"rtl":"ltr",lang:this.computedLocale||null,role:"group","aria-disabled":c?"true":null,"aria-controls":this.ariaControls||null,"aria-roledescription":this.roleDescription||null,"aria-describedby":[this.bvAttrs["aria-describedby"],a,s].filter(ya).join(" ")},on:{keydown:this.onKeydownWrapper}},[y,D,Y,P]);return e("div",{staticClass:"b-calendar",class:{"d-block":this.block}},[I])}}),cI=Pa({components:{BCalendar:lI}}),uI=Sc({bgVariant:wc(Yo),borderVariant:wc(Yo),tag:wc(Yo,"div"),textVariant:wc(Yo)},dn),dI=(i["default"].extend({props:uI}),Sc({title:wc(Yo),titleTag:wc(Yo,"h4")},bn)),hI=i["default"].extend({name:bn,functional:!0,props:dI,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.titleTag,Ja(n,{staticClass:"card-title"}),r||zl(a.title))}}),fI=Sc({subTitle:wc(Yo),subTitleTag:wc(Yo,"h6"),subTitleTextVariant:wc(Yo,"muted")},gn),pI=i["default"].extend({name:gn,functional:!0,props:fI,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.subTitleTag,Ja(n,{staticClass:"card-subtitle",class:[a.subTitleTextVariant?"text-".concat(a.subTitleTextVariant):null]}),r||zl(a.subTitle))}});function mI(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function vI(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?mI(Object(a),!0).forEach((function(t){_I(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):mI(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function _I(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var gI=Sc(sa(vI(vI(vI(vI({},dI),fI),Hc(uI,Mc.bind(null,"body"))),{},{bodyClass:wc(Fo),overlay:wc(To,!1)})),hn),yI=i["default"].extend({name:hn,functional:!0,props:gI,render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.bodyBgVariant,s=n.bodyBorderVariant,l=n.bodyTextVariant,c=e();n.title&&(c=e(hI,{props:kc(dI,n)}));var u=e();return n.subTitle&&(u=e(pI,{props:kc(fI,n),class:["mb-2"]})),e(n.bodyTag,Ja(r,{staticClass:"card-body",class:[(a={"card-img-overlay":n.overlay},_I(a,"bg-".concat(o),o),_I(a,"border-".concat(s),s),_I(a,"text-".concat(l),l),a),n.bodyClass]}),[c,u,i])}});function bI(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function MI(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?bI(Object(a),!0).forEach((function(t){zI(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):bI(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function zI(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var LI=Sc(sa(MI(MI({},Hc(uI,Mc.bind(null,"header"))),{},{header:wc(Yo),headerClass:wc(Fo),headerHtml:wc(Yo)})),mn),wI=i["default"].extend({name:mn,functional:!0,props:LI,render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.headerBgVariant,s=n.headerBorderVariant,l=n.headerTextVariant;return e(n.headerTag,Ja(r,{staticClass:"card-header",class:[n.headerClass,(a={},zI(a,"bg-".concat(o),o),zI(a,"border-".concat(s),s),zI(a,"text-".concat(l),l),a)],domProps:i?{}:UY(n.headerHtml,n.header)}),i)}});function HI(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function kI(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?HI(Object(a),!0).forEach((function(t){AI(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):HI(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function AI(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var SI=Sc(sa(kI(kI({},Hc(uI,Mc.bind(null,"footer"))),{},{footer:wc(Yo),footerClass:wc(Fo),footerHtml:wc(Yo)})),fn),VI=i["default"].extend({name:fn,functional:!0,props:SI,render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.footerBgVariant,s=n.footerBorderVariant,l=n.footerTextVariant;return e(n.footerTag,Ja(r,{staticClass:"card-footer",class:[n.footerClass,(a={},AI(a,"bg-".concat(o),o),AI(a,"border-".concat(s),s),AI(a,"text-".concat(l),l),a)],domProps:i?{}:UY(n.footerHtml,n.footer)}),i)}});function OI(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var TI='<svg width="%{w}" height="%{h}" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 %{w} %{h}" preserveAspectRatio="none"><rect width="100%" height="100%" style="fill:%{f};"></rect></svg>',DI=function(e,t,a){var n=encodeURIComponent(TI.replace("%{w}",zl(e)).replace("%{h}",zl(t)).replace("%{f}",a));return"data:image/svg+xml;charset=UTF-8,".concat(n)},CI=Sc({alt:wc(Yo),blank:wc(To,!1),blankColor:wc(Yo,"transparent"),block:wc(To,!1),center:wc(To,!1),fluid:wc(To,!1),fluidGrow:wc(To,!1),height:wc(Go),left:wc(To,!1),right:wc(To,!1),rounded:wc(No,!1),sizes:wc(Eo),src:wc(Yo),srcset:wc(Eo),thumbnail:wc(To,!1),width:wc(Go)},sr),jI=i["default"].extend({name:sr,functional:!0,props:CI,render:function(e,t){var a,n=t.props,r=t.data,i=n.alt,o=n.src,s=n.block,l=n.fluidGrow,c=n.rounded,u=fl(n.width)||null,d=fl(n.height)||null,h=null,f=cl(n.srcset).filter(ya).join(","),p=cl(n.sizes).filter(ya).join(",");return n.blank&&(!d&&u?d=u:!u&&d&&(u=d),u||d||(u=1,d=1),o=DI(u,d,n.blankColor||"transparent"),f=null,p=null),n.left?h="float-left":n.right?h="float-right":n.center&&(h="mx-auto",s=!0),e("img",Ja(r,{attrs:{src:o,alt:i,width:u?zl(u):null,height:d?zl(d):null,srcset:f||null,sizes:p||null},class:(a={"img-thumbnail":n.thumbnail,"img-fluid":n.fluid||l,"w-100":l,rounded:""===c||!0===c},OI(a,"rounded-".concat(c),xt(c)&&""!==c),OI(a,h,h),OI(a,"d-block",s),a)}))}});function xI(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function BI(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?xI(Object(a),!0).forEach((function(t){YI(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):xI(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function YI(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var PI=Sc(sa(BI(BI({},ra(CI,["src","alt","width","height","left","right"])),{},{bottom:wc(To,!1),end:wc(To,!1),start:wc(To,!1),top:wc(To,!1)})),vn),II=i["default"].extend({name:vn,functional:!0,props:PI,render:function(e,t){var a=t.props,n=t.data,r=a.src,i=a.alt,o=a.width,s=a.height,l="card-img";return a.top?l+="-top":a.right||a.end?l+="-right":a.bottom?l+="-bottom":(a.left||a.start)&&(l+="-left"),e("img",Ja(n,{class:l,attrs:{src:r,alt:i,width:o,height:s}}))}});function FI(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function EI(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?FI(Object(a),!0).forEach((function(t){$I(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):FI(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function $I(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var RI=Hc(PI,Mc.bind(null,"img"));RI.imgSrc.required=!1;var NI=Sc(sa(EI(EI(EI(EI(EI(EI({},gI),LI),SI),RI),uI),{},{align:wc(Yo),noBody:wc(To,!1)})),dn),WI=i["default"].extend({name:dn,functional:!0,props:NI,render:function(e,t){var a,n=t.props,r=t.data,i=t.slots,o=t.scopedSlots,s=n.imgSrc,l=n.imgLeft,c=n.imgRight,u=n.imgStart,d=n.imgEnd,h=n.imgBottom,f=n.header,p=n.headerHtml,m=n.footer,v=n.footerHtml,_=n.align,g=n.textVariant,y=n.bgVariant,b=n.borderVariant,M=o||{},z=i(),L={},w=e(),H=e();if(s){var k=e(II,{props:kc(RI,n,zc.bind(null,"img"))});h?H=k:w=k}var A=e(),S=Cc(vs,M,z);(S||f||p)&&(A=e(wI,{props:kc(LI,n),domProps:S?{}:UY(p,f)},jc(vs,L,M,z)));var V=jc(is,L,M,z);n.noBody||(V=e(yI,{props:kc(gI,n)},V),n.overlay&&s&&(V=e("div",{staticClass:"position-relative"},[w,V,H]),w=e(),H=e()));var O=e(),T=Cc(ms,M,z);return(T||m||v)&&(O=e(VI,{props:kc(SI,n),domProps:S?{}:UY(v,m)},jc(ms,L,M,z))),e(n.tag,Ja(r,{staticClass:"card",class:(a={"flex-row":l||u,"flex-row-reverse":(c||d)&&!(l||u)},$I(a,"text-".concat(_),_),$I(a,"bg-".concat(y),y),$I(a,"border-".concat(b),b),$I(a,"text-".concat(g),g),a)}),[w,A,V,O,H])}});function UI(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function GI(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function qI(e,t,a){return t&&GI(e.prototype,t),a&&GI(e,a),Object.defineProperty(e,"prototype",{writable:!1}),e}var JI,KI="__bv__visibility_observer",XI=function(){function e(t,a,n){UI(this,e),this.el=t,this.callback=a.callback,this.margin=a.margin||0,this.once=a.once||!1,this.observer=null,this.visible=void 0,this.doneOnce=!1,this.createObserver(n)}return qI(e,[{key:"createObserver",value:function(e){var t=this;if(this.observer&&this.stop(),!this.doneOnce&&Ct(this.callback)){try{this.observer=new IntersectionObserver(this.handler.bind(this),{root:null,rootMargin:this.margin,threshold:0})}catch(a){return this.doneOnce=!0,this.observer=void 0,void this.callback(null)}e.context.$nextTick((function(){Ol((function(){t.observer&&t.observer.observe(t.el)}))}))}}},{key:"handler",value:function(e){var t=e?e[0]:{},a=Boolean(t.isIntersecting||t.intersectionRatio>0);a!==this.visible&&(this.visible=a,this.callback(a),this.once&&this.visible&&(this.doneOnce=!0,this.stop()))}},{key:"stop",value:function(){this.observer&&this.observer.disconnect(),this.observer=null}}]),e}(),ZI=function(e){var t=e[KI];t&&t.stop&&t.stop(),delete e[KI]},QI=function(e,t,a){var n=t.value,r=t.modifiers,i={margin:"0px",once:!1,callback:n};ea(r).forEach((function(e){Be.test(e)?i.margin="".concat(e,"px"):"once"===e.toLowerCase()&&(i.once=!0)})),ZI(e),e[KI]=new XI(e,i,a),e[KI]._prevModifiers=na(r)},eF=function(e,t,a){var n=t.value,r=t.oldValue,i=t.modifiers;i=na(i),!e||n===r&&e[KI]&&EB(i,e[KI]._prevModifiers)||QI(e,{value:n,modifiers:i},a)},tF=function(e){ZI(e)},aF={bind:QI,componentUpdated:eF,unbind:tF};function nF(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function rF(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?nF(Object(a),!0).forEach((function(t){iF(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):nF(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function iF(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var oF="show",sF=wo+oF,lF=ia(CI,["blank"]),cF=Sc(rF(rF({},lF),{},iF({blankHeight:wc(Go),blankSrc:wc(Yo,null),blankWidth:wc(Go),offset:wc(Go,360)},oF,wc(To,!1))),lr),uF=i["default"].extend({name:lr,directives:{"b-visible":aF},props:cF,data:function(){return{isShown:this[oF]}},computed:{computedSrc:function(){var e=this.blankSrc;return!e||this.isShown?this.src:e},computedBlank:function(){return!(this.isShown||this.blankSrc)},computedWidth:function(){var e=this.width;return this.isShown?e:this.blankWidth||e},computedHeight:function(){var e=this.height;return this.isShown?e:this.blankHeight||e},computedSrcset:function(){var e=cl(this.srcset).filter(ya).join(",");return!e||this.blankSrc&&!this.isShown?null:e},computedSizes:function(){var e=cl(this.sizes).filter(ya).join(",");return!e||this.blankSrc&&!this.isShown?null:e}},watch:(JI={},iF(JI,oF,(function(e,t){if(e!==t){var a=!Oe||e;this.isShown=a,e!==a&&this.$nextTick(this.updateShowProp)}})),iF(JI,"isShown",(function(e,t){e!==t&&this.updateShowProp()})),JI),mounted:function(){this.isShown=!Oe||this[oF]},methods:{updateShowProp:function(){this.$emit(sF,this.isShown)},doShow:function(e){var t=this;!e&&null!==e||this.isShown||Ol((function(){t.isShown=!0}))}},render:function(e){var t,a=[];this.isShown||a.push({name:"b-visible",value:this.doShow,modifiers:(t={},iF(t,"".concat(fl(this.offset,0)),!0),iF(t,"once",!0),t)});return e(jI,{directives:a,props:rF(rF({},kc(lF,this.$props)),{},{src:this.computedSrc,blank:this.computedBlank,width:this.computedWidth,height:this.computedHeight,srcset:this.computedSrcset,sizes:this.computedSizes})})}});function dF(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function hF(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?dF(Object(a),!0).forEach((function(t){fF(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):dF(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function fF(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var pF=Sc(sa(hF(hF({},ia(cF,ea(CI))),ia(PI,["src","alt","width","height"]))),_n),mF=i["default"].extend({name:_n,functional:!0,props:pF,render:function(e,t){var a=t.props,n=t.data,r="card-img";return a.top?r+="-top":a.right||a.end?r+="-right":a.bottom?r+="-bottom":(a.left||a.start)&&(r+="-left"),e(uF,Ja(n,{class:[r],props:ia(a,["left","right"])}))}}),vF=Sc({textTag:wc(Yo,"p")},yn),_F=i["default"].extend({name:yn,functional:!0,props:vF,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.textTag,Ja(n,{staticClass:"card-text"}),r)}}),gF=Sc({columns:wc(To,!1),deck:wc(To,!1),tag:wc(Yo,"div")},pn),yF=i["default"].extend({name:pn,functional:!0,props:gF,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.tag,Ja(n,{class:a.deck?"card-deck":a.columns?"card-columns":"card-group"}),r)}}),bF=Pa({components:{BCard:WI,BCardHeader:wI,BCardBody:yI,BCardTitle:hI,BCardSubTitle:pI,BCardFooter:VI,BCardImg:II,BCardImgLazy:mF,BCardText:_F,BCardGroup:yF}}),MF=function(){};function zF(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function LF(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?zF(Object(a),!0).forEach((function(t){wF(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):zF(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function wF(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var HF,kF=function(e,t,a){if(e=e?e.$el||e:null,!Cl(e))return null;if(ka("observeDom"))return null;var n=new Tl((function(e){for(var a=!1,n=0;n<e.length&&!a;n++){var r=e[n],i=r.type,o=r.target;("characterData"===i&&o.nodeType===Node.TEXT_NODE||"attributes"===i||"childList"===i&&(r.addedNodes.length>0||r.removedNodes.length>0))&&(a=!0)}a&&t()}));return n.observe(e,LF({childList:!0,subtree:!0},a)),n};function AF(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function SF(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?AF(Object(a),!0).forEach((function(t){VF(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):AF(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function VF(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var OF=Dc("value",{type:jo,defaultValue:0}),TF=OF.mixin,DF=OF.props,CF=OF.prop,jF=OF.event,xF={next:{dirClass:"carousel-item-left",overlayClass:"carousel-item-next"},prev:{dirClass:"carousel-item-right",overlayClass:"carousel-item-prev"}},BF=650,YF=500,PF=40,IF={TOUCH:"touch",PEN:"pen"},FF={WebkitTransition:"webkitTransitionEnd",MozTransition:"transitionend",OTransition:"otransitionend oTransitionEnd",transition:"transitionend"},EF=function(e){for(var t in FF)if(!Ot(e.style[t]))return FF[t];return null},$F=Sc(sa(SF(SF(SF({},XP),DF),{},{background:wc(Yo),controls:wc(To,!1),fade:wc(To,!1),imgHeight:wc(Go),imgWidth:wc(Go),indicators:wc(To,!1),interval:wc(jo,5e3),labelGotoSlide:wc(Yo,"Goto slide"),labelIndicators:wc(Yo,"Select a slide to display"),labelNext:wc(Yo,"Next slide"),labelPrev:wc(Yo,"Previous slide"),noAnimation:wc(To,!1),noHoverPause:wc(To,!1),noTouch:wc(To,!1),noWrap:wc(To,!1)})),Mn),RF=i["default"].extend({name:Mn,mixins:[ZP,TF,xc],provide:function(){return{bvCarousel:this}},props:$F,data:function(){return{index:this[CF]||0,isSliding:!1,transitionEndEvent:null,slides:[],direction:null,isPaused:!(fl(this.interval,0)>0),touchStartX:0,touchDeltaX:0}},computed:{numSlides:function(){return this.slides.length}},watch:(HF={},VF(HF,CF,(function(e,t){e!==t&&this.setSlide(fl(e,0))})),VF(HF,"interval",(function(e,t){e!==t&&(e?(this.pause(!0),this.start(!1)):this.pause(!1))})),VF(HF,"isPaused",(function(e,t){e!==t&&this.$emit(e?Qi:bo)})),VF(HF,"index",(function(e,t){e===t||this.isSliding||this.doSlide(e,t)})),HF),created:function(){this.$_interval=null,this.$_animationTimeout=null,this.$_touchTimeout=null,this.$_observer=null,this.isPaused=!(fl(this.interval,0)>0)},mounted:function(){this.transitionEndEvent=EF(this.$el)||null,this.updateSlides(),this.setObserver(!0)},beforeDestroy:function(){this.clearInterval(),this.clearAnimationTimeout(),this.clearTouchTimeout(),this.setObserver(!1)},methods:{clearInterval:function(e){function t(){return e.apply(this,arguments)}return t.toString=function(){return e.toString()},t}((function(){clearInterval(this.$_interval),this.$_interval=null})),clearAnimationTimeout:function(){clearTimeout(this.$_animationTimeout),this.$_animationTimeout=null},clearTouchTimeout:function(){clearTimeout(this.$_touchTimeout),this.$_touchTimeout=null},setObserver:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.$_observer&&this.$_observer.disconnect(),this.$_observer=null,e&&(this.$_observer=kF(this.$refs.inner,this.updateSlides.bind(this),{subtree:!1,childList:!0,attributes:!0,attributeFilter:["id"]}))},setSlide:function(e){var t=this,a=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null;if(!(Me&&document.visibilityState&&document.hidden)){var n=this.noWrap,r=this.numSlides;e=gu(e),0!==r&&(this.isSliding?this.$once(mo,(function(){Ol((function(){return t.setSlide(e,a)}))})):(this.direction=a,this.index=e>=r?n?r-1:0:e<0?n?0:r-1:e,n&&this.index!==e&&this.index!==this[CF]&&this.$emit(jF,this.index)))}},prev:function(){this.setSlide(this.index-1,"prev")},next:function(){this.setSlide(this.index+1,"next")},pause:function(e){e||(this.isPaused=!0),this.clearInterval()},start:function(e){e||(this.isPaused=!1),this.clearInterval(),this.interval&&this.numSlides>1&&(this.$_interval=setInterval(this.next,mu(1e3,this.interval)))},restart:function(){this.$el.contains(jl())||this.start()},doSlide:function(e,t){var a=this,n=Boolean(this.interval),r=this.calcDirection(this.direction,t,e),i=r.overlayClass,o=r.dirClass,s=this.slides[t],l=this.slides[e];if(s&&l){if(this.isSliding=!0,n&&this.pause(!1),this.$emit(vo,e),this.$emit(jF,this.index),this.noAnimation)Ul(l,"active"),Gl(s,"active"),this.isSliding=!1,this.$nextTick((function(){return a.$emit(mo,e)}));else{Ul(l,i),Il(l),Ul(s,o),Ul(l,o);var c=!1,u=function t(){if(!c){if(c=!0,a.transitionEndEvent){var n=a.transitionEndEvent.split(/\s+/);n.forEach((function(e){return Pc(l,e,t,So)}))}a.clearAnimationTimeout(),Gl(l,o),Gl(l,i),Ul(l,"active"),Gl(s,"active"),Gl(s,o),Gl(s,i),Jl(s,"aria-current","false"),Jl(l,"aria-current","true"),Jl(s,"aria-hidden","true"),Jl(l,"aria-hidden","false"),a.isSliding=!1,a.direction=null,a.$nextTick((function(){return a.$emit(mo,e)}))}};if(this.transitionEndEvent){var d=this.transitionEndEvent.split(/\s+/);d.forEach((function(e){return Yc(l,e,u,So)}))}this.$_animationTimeout=setTimeout(u,BF)}n&&this.start(!1)}},updateSlides:function(){this.pause(!0),this.slides=Fl(".carousel-item",this.$refs.inner);var e=this.slides.length,t=mu(0,pu(gu(this.index),e-1));this.slides.forEach((function(a,n){var r=n+1;n===t?(Ul(a,"active"),Jl(a,"aria-current","true")):(Gl(a,"active"),Jl(a,"aria-current","false")),Jl(a,"aria-posinset",String(r)),Jl(a,"aria-setsize",String(e))})),this.setSlide(t),this.start(this.isPaused)},calcDirection:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0,a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0;return e?xF[e]:a>t?xF.next:xF.prev},handleClick:function(e,t){var a=e.keyCode;"click"!==e.type&&a!==PB&&a!==TB||(Fc(e),t())},handleSwipe:function(){var e=vu(this.touchDeltaX);if(!(e<=PF)){var t=e/this.touchDeltaX;this.touchDeltaX=0,t>0?this.prev():t<0&&this.next()}},touchStart:function(e){Ve&&IF[e.pointerType.toUpperCase()]?this.touchStartX=e.clientX:Ve||(this.touchStartX=e.touches[0].clientX)},touchMove:function(e){e.touches&&e.touches.length>1?this.touchDeltaX=0:this.touchDeltaX=e.touches[0].clientX-this.touchStartX},touchEnd:function(e){Ve&&IF[e.pointerType.toUpperCase()]&&(this.touchDeltaX=e.clientX-this.touchStartX),this.handleSwipe(),this.pause(!1),this.clearTouchTimeout(),this.$_touchTimeout=setTimeout(this.start,YF+mu(1e3,this.interval))}},render:function(e){var t=this,a=this.indicators,n=this.background,r=this.noAnimation,i=this.noHoverPause,o=this.noTouch,s=this.index,l=this.isSliding,c=this.pause,u=this.restart,d=this.touchStart,h=this.touchEnd,f=this.safeId("__BV_inner_"),p=e("div",{staticClass:"carousel-inner",attrs:{id:f,role:"list"},ref:"inner"},[this.normalizeSlot()]),m=e();if(this.controls){var v=function(a,n,r){var i=function(e){l?Fc(e,{propagation:!1}):t.handleClick(e,r)};return e("a",{staticClass:"carousel-control-".concat(a),attrs:{href:"#",role:"button","aria-controls":f,"aria-disabled":l?"true":null},on:{click:i,keydown:i}},[e("span",{staticClass:"carousel-control-".concat(a,"-icon"),attrs:{"aria-hidden":"true"}}),e("span",{class:"sr-only"},[n])])};m=[v("prev",this.labelPrev,this.prev),v("next",this.labelNext,this.next)]}var _=e("ol",{staticClass:"carousel-indicators",directives:[{name:"show",value:a}],attrs:{id:this.safeId("__BV_indicators_"),"aria-hidden":a?"false":"true","aria-label":this.labelIndicators,"aria-owns":f}},this.slides.map((function(n,r){var i=function(e){t.handleClick(e,(function(){t.setSlide(r)}))};return e("li",{class:{active:r===s},attrs:{role:"button",id:t.safeId("__BV_indicator_".concat(r+1,"_")),tabindex:a?"0":"-1","aria-current":r===s?"true":"false","aria-label":"".concat(t.labelGotoSlide," ").concat(r+1),"aria-describedby":n.id||null,"aria-controls":f},on:{click:i,keydown:i},key:"slide_".concat(r)})}))),g={mouseenter:i?MF:c,mouseleave:i?MF:u,focusin:c,focusout:u,keydown:function(e){if(!/input|textarea/i.test(e.target.tagName)){var a=e.keyCode;a!==jB&&a!==YB||(Fc(e),t[a===jB?"prev":"next"]())}}};return Se&&!o&&(Ve?(g["&pointerdown"]=d,g["&pointerup"]=h):(g["&touchstart"]=d,g["&touchmove"]=this.touchMove,g["&touchend"]=h)),e("div",{staticClass:"carousel",class:{slide:!r,"carousel-fade":!r&&this.fade,"pointer-event":Se&&Ve&&!o},style:{background:n},attrs:{role:"region",id:this.safeId(),"aria-busy":l?"true":"false"},on:g},[p,m,_])}});function NF(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function WF(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?NF(Object(a),!0).forEach((function(t){UF(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):NF(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function UF(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var GF,qF={imgAlt:wc(Yo),imgBlank:wc(To,!1),imgBlankColor:wc(Yo,"transparent"),imgHeight:wc(Go),imgSrc:wc(Yo),imgWidth:wc(Go)},JF=Sc(sa(WF(WF(WF({},XP),qF),{},{background:wc(Yo),caption:wc(Yo),captionHtml:wc(Yo),captionTag:wc(Yo,"h3"),contentTag:wc(Yo,"div"),contentVisibleUp:wc(Yo),text:wc(Yo),textHtml:wc(Yo),textTag:wc(Yo,"p")})),zn),KF=i["default"].extend({name:zn,mixins:[ZP,xc],inject:{bvCarousel:{default:function(){return{noTouch:!0}}}},props:JF,computed:{contentClasses:function(){return[this.contentVisibleUp?"d-none":"",this.contentVisibleUp?"d-".concat(this.contentVisibleUp,"-block"):""]},computedWidth:function(){return this.imgWidth||this.bvCarousel.imgWidth||null},computedHeight:function(){return this.imgHeight||this.bvCarousel.imgHeight||null}},render:function(e){var t=this.normalizeSlot(zs);if(!t&&(this.imgSrc||this.imgBlank)){var a={};!this.bvCarousel.noTouch&&Se&&(a.dragstart=function(e){return Fc(e,{propagation:!1})}),t=e(jI,{props:WF(WF({},kc(qF,this.$props,zc.bind(null,"img"))),{},{width:this.computedWidth,height:this.computedHeight,fluidGrow:!0,block:!0}),on:a})}var n=[!(!this.caption&&!this.captionHtml)&&e(this.captionTag,{domProps:UY(this.captionHtml,this.caption)}),!(!this.text&&!this.textHtml)&&e(this.textTag,{domProps:UY(this.textHtml,this.text)}),this.normalizeSlot()||!1],r=e();return n.some(ya)&&(r=e(this.contentTag,{staticClass:"carousel-caption",class:this.contentClasses},n.map((function(t){return t||e()})))),e("div",{staticClass:"carousel-item",style:{background:this.background||this.bvCarousel.background||null},attrs:{id:this.safeId(),role:"listitem"}},[t,r])}}),XF=Pa({components:{BCarousel:RF,BCarouselSlide:KF}}),ZF="show",QF=function(e){Ql(e,"height",0),Ol((function(){Il(e),Ql(e,"height","".concat(e.scrollHeight,"px"))}))},eE=function(e){ec(e,"height")},tE=function(e){Ql(e,"height","auto"),Ql(e,"display","block"),Ql(e,"height","".concat(ac(e).height,"px")),Il(e),Ql(e,"height",0)},aE=function(e){ec(e,"height")},nE={css:!0,enterClass:"",enterActiveClass:"collapsing",enterToClass:"collapse show",leaveClass:"collapse show",leaveActiveClass:"collapsing",leaveToClass:"collapse"},rE={enter:QF,afterEnter:eE,leave:tE,afterLeave:aE},iE={appear:wc(To,!1)},oE=i["default"].extend({name:si,functional:!0,props:iE,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e("transition",Ja(n,{props:nE,on:rE},{props:a}),r)}});function sE(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function lE(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?sE(Object(a),!0).forEach((function(t){cE(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):sE(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function cE(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var uE=Rc(wn,"toggle"),dE=Rc(wn,"request-state"),hE=$c(wn,"accordion"),fE=$c(wn,"state"),pE=$c(wn,"sync-state"),mE=Dc("visible",{type:To,defaultValue:!1}),vE=mE.mixin,_E=mE.props,gE=mE.prop,yE=mE.event,bE=Sc(sa(lE(lE(lE({},XP),_E),{},{accordion:wc(Yo),appear:wc(To,!1),isNav:wc(To,!1),tag:wc(Yo,"div")})),wn),ME=i["default"].extend({name:wn,mixins:[ZP,vE,xc,qB],props:bE,data:function(){return{show:this[gE],transitioning:!1}},computed:{classObject:function(){var e=this.transitioning;return{"navbar-collapse":this.isNav,collapse:!e,show:this.show&&!e}},slotScope:function(){var e=this;return{visible:this.show,close:function(){e.show=!1}}}},watch:(GF={},cE(GF,gE,(function(e){e!==this.show&&(this.show=e)})),cE(GF,"show",(function(e,t){e!==t&&this.emitState()})),GF),created:function(){this.show=this[gE]},mounted:function(){var e=this;this.show=this[gE],this.listenOnRoot(uE,this.handleToggleEvent),this.listenOnRoot(hE,this.handleAccordionEvent),this.isNav&&(this.setWindowEvents(!0),this.handleResize()),this.$nextTick((function(){e.emitState()})),this.listenOnRoot(dE,(function(t){t===e.safeId()&&e.$nextTick(e.emitSync)}))},updated:function(){this.emitSync()},deactivated:function(){this.isNav&&this.setWindowEvents(!1)},activated:function(){this.isNav&&this.setWindowEvents(!0),this.emitSync()},beforeDestroy:function(){this.show=!1,this.isNav&&Me&&this.setWindowEvents(!1)},methods:{setWindowEvents:function(e){Ic(e,window,"resize",this.handleResize,So),Ic(e,window,"orientationchange",this.handleResize,So)},toggle:function(){this.show=!this.show},onEnter:function(){this.transitioning=!0,this.$emit(fo)},onAfterEnter:function(){this.transitioning=!1,this.$emit(po)},onLeave:function(){this.transitioning=!0,this.$emit(Ri)},onAfterLeave:function(){this.transitioning=!1,this.$emit($i)},emitState:function(){var e=this.show,t=this.accordion,a=this.safeId();this.$emit(yE,e),this.emitOnRoot(fE,a,e),t&&e&&this.emitOnRoot(hE,a,t)},emitSync:function(){this.emitOnRoot(pE,this.safeId(),this.show)},checkDisplayBlock:function(){var e=this.$el,t=ql(e,ZF);Gl(e,ZF);var a="block"===nc(e).display;return t&&Ul(e,ZF),a},clickHandler:function(e){var t=e.target;this.isNav&&t&&"block"===nc(this.$el).display&&(!$l(t,".nav-link,.dropdown-item")&&!Rl(".nav-link,.dropdown-item",t)||this.checkDisplayBlock()||(this.show=!1))},handleToggleEvent:function(e){e===this.safeId()&&this.toggle()},handleAccordionEvent:function(e,t){var a=this.accordion,n=this.show;if(a&&a===t){var r=e===this.safeId();(r&&!n||!r&&n)&&this.toggle()}},handleResize:function(){this.show="block"===nc(this.$el).display}},render:function(e){var t=this.appear,a=e(this.tag,{class:this.classObject,directives:[{name:"show",value:this.show}],attrs:{id:this.safeId()},on:{click:this.clickHandler}},this.normalizeSlot(is,this.slotScope));return e(oE,{props:{appear:t},on:{enter:this.onEnter,afterEnter:this.onAfterEnter,leave:this.onLeave,afterLeave:this.onAfterLeave}},[a])}}),zE="collapsed",LE="not-collapsed",wE="__BV_toggle",HE="".concat(wE,"_HANDLER__"),kE="".concat(wE,"_CLICK__"),AE="".concat(wE,"_STATE__"),SE="".concat(wE,"_TARGETS__"),VE="false",OE="true",TE="aria-controls",DE="aria-expanded",CE="role",jE="tabindex",xE="overflow-anchor",BE=Rc(wn,"toggle"),YE=$c(wn,"state"),PE=$c(wn,"sync-state"),IE=Rc(wn,"request-state"),FE=[TB,PB],EE=function(e){return!ll(["button","a"],e.tagName.toLowerCase())},$E=function(e,t){var a=e.modifiers,n=e.arg,r=e.value,i=ea(a||{});if(r=xt(r)?r.split(Ge):r,xl(t.tagName,"a")){var o=Xl(t,"href")||"";Ie.test(o)&&i.push(o.replace(Pe,""))}return cl(n,r).forEach((function(e){return xt(e)&&i.push(e)})),i.filter((function(e,t,a){return e&&a.indexOf(e)===t}))},RE=function(e){var t=e[kE];t&&(Pc(e,"click",t,Ao),Pc(e,"keydown",t,Ao)),e[kE]=null},NE=function(e,t){if(RE(e),t.context){var a=function(a){if(("keydown"!==a.type||ll(FE,a.keyCode))&&!Pl(e)){var n=e[SE]||[];n.forEach((function(e){t.context.$root.$emit(BE,e)}))}};e[kE]=a,Yc(e,"click",a,Ao),EE(e)&&Yc(e,"keydown",a,Ao)}},WE=function(e,t){e[HE]&&t.context&&t.context.$root.$off([YE,PE],e[HE]),e[HE]=null},UE=function(e,t){if(WE(e,t),t.context){var a=function(t,a){ll(e[SE]||[],t)&&(e[AE]=a,GE(e,a))};e[HE]=a,t.context.$root.$on([YE,PE],a)}},GE=function(e,t){t?(Gl(e,zE),Ul(e,LE),Jl(e,DE,OE)):(Gl(e,LE),Ul(e,zE),Jl(e,DE,VE))},qE=function(e,t){e[t]=null,delete e[t]},JE=function(e,t,a){if(Me&&a.context){EE(e)&&(Zl(e,CE)||Jl(e,CE,"button"),Zl(e,jE)||Jl(e,jE,"0")),GE(e,e[AE]);var n=$E(t,e);n.length>0?(Jl(e,TE,n.join(" ")),Ql(e,xE,"none")):(Kl(e,TE),ec(e,xE)),Ol((function(){NE(e,a)})),EB(n,e[SE])||(e[SE]=n,n.forEach((function(e){a.context.$root.$emit(IE,e)})))}},KE={bind:function(e,t,a){e[AE]=!1,e[SE]=[],UE(e,a),JE(e,t,a)},componentUpdated:JE,updated:JE,unbind:function(e,t,a){RE(e),WE(e,a),qE(e,HE),qE(e,kE),qE(e,AE),qE(e,SE),Gl(e,zE),Gl(e,LE),Kl(e,DE),Kl(e,TE),Kl(e,CE),ec(e,xE)}},XE=Pa({directives:{VBToggle:KE}}),ZE=Pa({components:{BCollapse:ME},plugins:{VBTogglePlugin:XE}}),QE=a("f0bd"),e$="top-start",t$="top-end",a$="bottom-start",n$="bottom-end",r$="right-start",i$="left-start";function o$(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function s$(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function l$(e,t,a){return t&&s$(e.prototype,t),a&&s$(e,a),Object.defineProperty(e,"prototype",{writable:!1}),e}var c$=function(){function e(t){var a=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(o$(this,e),!t)throw new TypeError("Failed to construct '".concat(this.constructor.name,"'. 1 argument required, ").concat(arguments.length," given."));Jt(this,e.Defaults,this.constructor.Defaults,a,{type:t}),Xt(this,{type:la(),cancelable:la(),nativeEvent:la(),target:la(),relatedTarget:la(),vueTarget:la(),componentId:la()});var n=!1;this.preventDefault=function(){this.cancelable&&(n=!0)},Zt(this,"defaultPrevented",{enumerable:!0,get:function(){return n}})}return l$(e,null,[{key:"Defaults",get:function(){return{type:"",cancelable:!0,nativeEvent:null,target:null,relatedTarget:null,vueTarget:null,componentId:null}}}]),e}(),u$=i["default"].extend({data:function(){return{listenForClickOut:!1}},watch:{listenForClickOut:function(e,t){e!==t&&(Pc(this.clickOutElement,this.clickOutEventName,this._clickOutHandler,So),e&&Yc(this.clickOutElement,this.clickOutEventName,this._clickOutHandler,So))}},beforeCreate:function(){this.clickOutElement=null,this.clickOutEventName=null},mounted:function(){this.clickOutElement||(this.clickOutElement=document),this.clickOutEventName||(this.clickOutEventName="click"),this.listenForClickOut&&Yc(this.clickOutElement,this.clickOutEventName,this._clickOutHandler,So)},beforeDestroy:function(){Pc(this.clickOutElement,this.clickOutEventName,this._clickOutHandler,So)},methods:{isClickOut:function(e){return!Nl(this.$el,e.target)},_clickOutHandler:function(e){this.clickOutHandler&&this.isClickOut(e)&&this.clickOutHandler(e)}}}),d$=i["default"].extend({data:function(){return{listenForFocusIn:!1}},watch:{listenForFocusIn:function(e,t){e!==t&&(Pc(this.focusInElement,"focusin",this._focusInHandler,So),e&&Yc(this.focusInElement,"focusin",this._focusInHandler,So))}},beforeCreate:function(){this.focusInElement=null},mounted:function(){this.focusInElement||(this.focusInElement=document),this.listenForFocusIn&&Yc(this.focusInElement,"focusin",this._focusInHandler,So)},beforeDestroy:function(){Pc(this.focusInElement,"focusin",this._focusInHandler,So)},methods:{_focusInHandler:function(e){this.focusInHandler&&this.focusInHandler(e)}}});function h$(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function f$(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?h$(Object(a),!0).forEach((function(t){p$(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):h$(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function p$(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var m$=$c(kn,po),v$=$c(kn,$i),_$=".dropdown form",g$=[".dropdown-item",".b-dropdown-form"].map((function(e){return"".concat(e,":not(.disabled):not([disabled])")})).join(", "),y$=function(e){return(e||[]).filter(Yl)},b$=Sc(sa(f$(f$({},XP),{},{boundary:wc([wt,Yo],"scrollParent"),disabled:wc(To,!1),dropleft:wc(To,!1),dropright:wc(To,!1),dropup:wc(To,!1),noFlip:wc(To,!1),offset:wc(Go,0),popperOpts:wc(xo,{}),right:wc(To,!1)})),kn),M$=i["default"].extend({mixins:[ZP,qB,u$,d$],provide:function(){return{bvDropdown:this}},inject:{bvNavbar:{default:null}},props:b$,data:function(){return{visible:!1,visibleChangePrevented:!1}},computed:{inNavbar:function(){return!Tt(this.bvNavbar)},toggler:function(){var e=this.$refs.toggle;return e?e.$el||e:null},directionClass:function(){return this.dropup?"dropup":this.dropright?"dropright":this.dropleft?"dropleft":""},boundaryClass:function(){return"scrollParent"===this.boundary||this.inNavbar?"":"position-static"},hideDelay:function(){return this.inNavbar?Se?300:50:0}},watch:{visible:function(e,t){if(this.visibleChangePrevented)this.visibleChangePrevented=!1;else if(e!==t){var a=e?fo:Ri,n=new c$(a,{cancelable:!0,vueTarget:this,target:this.$refs.menu,relatedTarget:null,componentId:this.safeId?this.safeId():this.id||null});if(this.emitEvent(n),n.defaultPrevented)return this.visibleChangePrevented=!0,this.visible=t,void this.$off($i,this.focusToggler);e?this.showMenu():this.hideMenu()}},disabled:function(e,t){e!==t&&e&&this.visible&&(this.visible=!1)}},created:function(){this.$_popper=null,this.$_hideTimeout=null},deactivated:function(){this.visible=!1,this.whileOpenListen(!1),this.destroyPopper()},beforeDestroy:function(){this.visible=!1,this.whileOpenListen(!1),this.destroyPopper(),this.clearHideTimeout()},methods:{emitEvent:function(e){var t=e.type;this.emitOnRoot($c(kn,t),e),this.$emit(t,e)},showMenu:function(){var e=this;if(!this.disabled){if(!this.inNavbar)if("undefined"===typeof QE["a"])La("Popper.js not found. Falling back to CSS positioning",kn);else{var t=this.dropup&&this.right||this.split?this.$el:this.$refs.toggle;t=t.$el||t,this.createPopper(t)}this.emitOnRoot(m$,this),this.whileOpenListen(!0),this.$nextTick((function(){e.focusMenu(),e.$emit(po)}))}},hideMenu:function(){this.whileOpenListen(!1),this.emitOnRoot(v$,this),this.$emit($i),this.destroyPopper()},createPopper:function(e){this.destroyPopper(),this.$_popper=new QE["a"](e,this.$refs.menu,this.getPopperConfig())},destroyPopper:function(){this.$_popper&&this.$_popper.destroy(),this.$_popper=null},updatePopper:function(){try{this.$_popper.scheduleUpdate()}catch(e){}},clearHideTimeout:function(){clearTimeout(this.$_hideTimeout),this.$_hideTimeout=null},getPopperConfig:function(){var e=a$;this.dropup?e=this.right?t$:e$:this.dropright?e=r$:this.dropleft?e=i$:this.right&&(e=n$);var t={placement:e,modifiers:{offset:{offset:this.offset||0},flip:{enabled:!this.noFlip}}},a=this.boundary;return a&&(t.modifiers.preventOverflow={boundariesElement:a}),oa(t,this.popperOpts||{})},whileOpenListen:function(e){this.listenForClickOut=e,this.listenForFocusIn=e;var t=e?"$on":"$off";this.$root[t](m$,this.rootCloseListener)},rootCloseListener:function(e){e!==this&&(this.visible=!1)},show:function(){var e=this;this.disabled||Ol((function(){e.visible=!0}))},hide:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.disabled||(this.visible=!1,e&&this.$once($i,this.focusToggler))},toggle:function(e){e=e||{};var t=e,a=t.type,n=t.keyCode;("click"===a||"keydown"===a&&-1!==[TB,PB,VB].indexOf(n))&&(this.disabled?this.visible=!1:(this.$emit(yo,e),Fc(e),this.visible?this.hide(!0):this.show()))},onMousedown:function(e){Fc(e,{propagation:!1})},onKeydown:function(e){var t=e.keyCode;t===DB?this.onEsc(e):t===VB?this.focusNext(e,!1):t===IB&&this.focusNext(e,!0)},onEsc:function(e){this.visible&&(this.visible=!1,Fc(e),this.$once($i,this.focusToggler))},onSplitClick:function(e){this.disabled?this.visible=!1:this.$emit(Hi,e)},hideHandler:function(e){var t=this,a=e.target;!this.visible||Nl(this.$refs.menu,a)||Nl(this.toggler,a)||(this.clearHideTimeout(),this.$_hideTimeout=setTimeout((function(){return t.hide()}),this.hideDelay))},clickOutHandler:function(e){this.hideHandler(e)},focusInHandler:function(e){this.hideHandler(e)},focusNext:function(e,t){var a=this,n=e.target;!this.visible||e&&Rl(_$,n)||(Fc(e),this.$nextTick((function(){var e=a.getItems();if(!(e.length<1)){var r=e.indexOf(n);t&&r>0?r--:!t&&r<e.length-1&&r++,r<0&&(r=0),a.focusItem(r,e)}})))},focusItem:function(e,t){var a=t.find((function(t,a){return a===e}));lc(a)},getItems:function(){return y$(Fl(g$,this.$refs.menu))},focusMenu:function(){lc(this.$refs.menu)},focusToggler:function(){var e=this;this.$nextTick((function(){lc(e.toggler)}))}}});function z$(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function L$(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?z$(Object(a),!0).forEach((function(t){w$(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):z$(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function w$(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var H$=Sc(sa(L$(L$(L$({},XP),b$),{},{block:wc(To,!1),html:wc(Yo),lazy:wc(To,!1),menuClass:wc(Fo),noCaret:wc(To,!1),role:wc(Yo,"menu"),size:wc(Yo),split:wc(To,!1),splitButtonType:wc(Yo,"button",(function(e){return ll(["button","submit","reset"],e)})),splitClass:wc(Fo),splitHref:wc(Yo),splitTo:wc(Ko),splitVariant:wc(Yo),text:wc(Yo),toggleAttrs:wc(xo,{}),toggleClass:wc(Fo),toggleTag:wc(Yo,"button"),toggleText:wc(Yo,"Toggle dropdown"),variant:wc(Yo,"secondary")})),kn),k$=i["default"].extend({name:kn,mixins:[ZP,M$,xc],props:H$,computed:{dropdownClasses:function(){var e=this.block,t=this.split;return[this.directionClass,this.boundaryClass,{show:this.visible,"btn-group":t||!e,"d-flex":e&&t}]},menuClasses:function(){return[this.menuClass,{"dropdown-menu-right":this.right,show:this.visible}]},toggleClasses:function(){var e=this.split;return[this.toggleClass,{"dropdown-toggle-split":e,"dropdown-toggle-no-caret":this.noCaret&&!e}]}},render:function(e){var t=this.visible,a=this.variant,n=this.size,r=this.block,i=this.disabled,o=this.split,s=this.role,l=this.hide,c=this.toggle,u={variant:a,size:n,block:r,disabled:i},d=this.normalizeSlot(as),h=this.hasNormalizedSlot(as)?{}:UY(this.html,this.text),f=e();if(o){var p=this.splitTo,m=this.splitHref,v=this.splitButtonType,_=L$(L$({},u),{},{variant:this.splitVariant||a});p?_.to=p:m?_.href=m:v&&(_.type=v),f=e(LY,{class:this.splitClass,attrs:{id:this.safeId("_BV_button_")},props:_,domProps:h,on:{click:this.onSplitClick},ref:"button"},d),d=[e("span",{class:["sr-only"]},[this.toggleText])],h={}}var g=["menu","listbox","tree","grid","dialog"],y=e(LY,{staticClass:"dropdown-toggle",class:this.toggleClasses,attrs:L$(L$({},this.toggleAttrs),{},{id:this.safeId("_BV_toggle_"),"aria-haspopup":g.includes(s)?s:"false","aria-expanded":zl(t)}),props:L$(L$({},u),{},{tag:this.toggleTag,block:r&&!o}),domProps:h,on:{mousedown:this.onMousedown,click:c,keydown:c},ref:"toggle"},d),b=e("ul",{staticClass:"dropdown-menu",class:this.menuClasses,attrs:{role:s,tabindex:"-1","aria-labelledby":this.safeId(o?"_BV_button_":"_BV_toggle_")},on:{keydown:this.onKeydown},ref:"menu"},[!this.lazy||t?this.normalizeSlot(is,{hide:l}):e()]);return e("div",{staticClass:"dropdown b-dropdown",class:this.dropdownClasses,attrs:{id:this.safeId()}},[f,y,b])}});function A$(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function S$(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?A$(Object(a),!0).forEach((function(t){V$(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):A$(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function V$(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var O$=ia(lY,["event","routerTag"]),T$=Sc(sa(S$(S$({},O$),{},{linkClass:wc(Fo),variant:wc(Yo)})),Tn),D$=i["default"].extend({name:Tn,mixins:[UB,xc],inject:{bvDropdown:{default:null}},inheritAttrs:!1,props:T$,computed:{computedAttrs:function(){return S$(S$({},this.bvAttrs),{},{role:"menuitem"})}},methods:{closeDropdown:function(){var e=this;Ol((function(){e.bvDropdown&&e.bvDropdown.hide(!0)}))},onClick:function(e){this.$emit(Hi,e),this.closeDropdown()}},render:function(e){var t=this.linkClass,a=this.variant,n=this.active,r=this.disabled,i=this.onClick,o=this.bvAttrs;return e("li",{class:o.class,style:o.style,attrs:{role:"presentation"}},[e(cY,{staticClass:"dropdown-item",class:[t,V$({},"text-".concat(a),a&&!(n||r))],props:kc(O$,this.$props),attrs:this.computedAttrs,on:{click:i},ref:"item"},this.normalizeSlot())])}});function C$(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function j$(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?C$(Object(a),!0).forEach((function(t){x$(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):C$(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function x$(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var B$=Sc({active:wc(To,!1),activeClass:wc(Yo,"active"),buttonClass:wc(Fo),disabled:wc(To,!1),variant:wc(Yo)},Dn),Y$=i["default"].extend({name:Dn,mixins:[UB,xc],inject:{bvDropdown:{default:null}},inheritAttrs:!1,props:B$,computed:{computedAttrs:function(){return j$(j$({},this.bvAttrs),{},{role:"menuitem",type:"button",disabled:this.disabled})}},methods:{closeDropdown:function(){this.bvDropdown&&this.bvDropdown.hide(!0)},onClick:function(e){this.$emit(Hi,e),this.closeDropdown()}},render:function(e){var t,a=this.active,n=this.variant,r=this.bvAttrs;return e("li",{class:r.class,style:r.style,attrs:{role:"presentation"}},[e("button",{staticClass:"dropdown-item",class:[this.buttonClass,(t={},x$(t,this.activeClass,a),x$(t,"text-".concat(n),n&&!(a||this.disabled)),t)],attrs:this.computedAttrs,on:{click:this.onClick},ref:"button"},this.normalizeSlot())])}});function P$(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function I$(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?P$(Object(a),!0).forEach((function(t){F$(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):P$(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function F$(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var E$=Sc({id:wc(Yo),tag:wc(Yo,"header"),variant:wc(Yo)},On),$$=i["default"].extend({name:On,functional:!0,props:E$,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.tag,o=a.variant;return e("li",Ja(ia(n,["attrs"]),{attrs:{role:"presentation"}}),[e(i,{staticClass:"dropdown-header",class:F$({},"text-".concat(o),o),attrs:I$(I$({},n.attrs||{}),{},{id:a.id||null,role:xl(i,"header")?null:"heading"}),ref:"header"},r)])}});function R$(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function N$(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?R$(Object(a),!0).forEach((function(t){W$(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):R$(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function W$(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var U$=Sc({tag:wc(Yo,"hr")},An),G$=i["default"].extend({name:An,functional:!0,props:U$,render:function(e,t){var a=t.props,n=t.data;return e("li",Ja(ia(n,["attrs"]),{attrs:{role:"presentation"}}),[e(a.tag,{staticClass:"dropdown-divider",attrs:N$(N$({},n.attrs||{}),{},{role:"separator","aria-orientation":"horizontal"}),ref:"divider"})])}}),q$=Sc({id:wc(Yo),inline:wc(To,!1),novalidate:wc(To,!1),validated:wc(To,!1)},xn),J$=i["default"].extend({name:xn,functional:!0,props:q$,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e("form",Ja(n,{class:{"form-inline":a.inline,"was-validated":a.validated},attrs:{id:a.id,novalidate:a.novalidate}}),r)}});function K$(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function X$(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?K$(Object(a),!0).forEach((function(t){Z$(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):K$(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function Z$(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Q$=Sc(sa(X$(X$({},q$),{},{disabled:wc(To,!1),formClass:wc(Fo)})),Sn),eR=i["default"].extend({name:Sn,functional:!0,props:Q$,render:function(e,t){var a=t.props,n=t.data,r=t.listeners,i=t.children;return e("li",Ja(ia(n,["attrs","on"]),{attrs:{role:"presentation"}}),[e(J$,{staticClass:"b-dropdown-form",class:[a.formClass,{disabled:a.disabled}],props:a,attrs:X$(X$({},n.attrs||{}),{},{disabled:a.disabled,tabindex:a.disabled?null:"-1"}),on:r,ref:"form"},i)])}});function tR(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var aR=Sc({tag:wc(Yo,"p"),textClass:wc(Fo),variant:wc(Yo)},Cn),nR=i["default"].extend({name:Cn,functional:!0,props:aR,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.tag,o=a.textClass,s=a.variant;return e("li",Ja(ia(n,["attrs"]),{attrs:{role:"presentation"}}),[e(i,{staticClass:"b-dropdown-text",class:[o,tR({},"text-".concat(s),s)],props:a,attrs:n.attrs||{},ref:"text"},r)])}});function rR(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function iR(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?rR(Object(a),!0).forEach((function(t){oR(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):rR(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function oR(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var sR=Sc({ariaDescribedby:wc(Yo),header:wc(Yo),headerClasses:wc(Fo),headerTag:wc(Yo,"header"),headerVariant:wc(Yo),id:wc(Yo)},Vn),lR=i["default"].extend({name:Vn,functional:!0,props:sR,render:function(e,t){var a=t.props,n=t.data,r=t.slots,i=t.scopedSlots,o=a.id,s=a.variant,l=a.header,c=a.headerTag,u=r(),d=i||{},h={},f=o?"_bv_".concat(o,"_group_dd_header"):null,p=e();return(Cc(vs,d,u)||l)&&(p=e(c,{staticClass:"dropdown-header",class:[a.headerClasses,oR({},"text-".concat(s),s)],attrs:{id:f,role:xl(c,"header")?null:"heading"}},jc(vs,h,d,u)||l)),e("li",Ja(ia(n,["attrs"]),{attrs:{role:"presentation"}}),[p,e("ul",{staticClass:"list-unstyled",attrs:iR(iR({},n.attrs||{}),{},{id:o,role:"group","aria-describedby":[f,a.ariaDescribedBy].filter(ya).join(" ").trim()||null})},jc(is,h,d,u))])}}),cR=Pa({components:{BDropdown:k$,BDd:k$,BDropdownItem:D$,BDdItem:D$,BDropdownItemButton:Y$,BDropdownItemBtn:Y$,BDdItemButton:Y$,BDdItemBtn:Y$,BDropdownHeader:$$,BDdHeader:$$,BDropdownDivider:G$,BDdDivider:G$,BDropdownForm:eR,BDdForm:eR,BDropdownText:nR,BDdText:nR,BDropdownGroup:lR,BDdGroup:lR}});function uR(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var dR=["iframe","embed","video","object","img","b-img","b-img-lazy"],hR=Sc({aspect:wc(Yo,"16by9"),tag:wc(Yo,"div"),type:wc(Yo,"iframe",(function(e){return ll(dR,e)}))},jn),fR=i["default"].extend({name:jn,functional:!0,props:hR,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.aspect;return e(a.tag,{staticClass:"embed-responsive",class:uR({},"embed-responsive-".concat(i),i),ref:n.ref},[e(a.type,Ja(ia(n,["ref"]),{staticClass:"embed-responsive-item"}),r)])}}),pR=Pa({components:{BEmbed:fR}}),mR='Setting prop "options" to an object is deprecated. Use the array format instead.',vR=Sc({disabledField:wc(Yo,"disabled"),htmlField:wc(Yo,"html"),options:wc(Io,[]),textField:wc(Yo,"text"),valueField:wc(Yo,"value")},"formOptionControls"),_R=i["default"].extend({props:vR,computed:{formOptions:function(){return this.normalizeOptions(this.options)}},methods:{normalizeOption:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null;if(Ft(e)){var a=Ma(e,this.valueField),n=Ma(e,this.textField);return{value:Ot(a)?t||n:a,text:WY(String(Ot(n)?t:n)),html:Ma(e,this.htmlField),disabled:Boolean(Ma(e,this.disabledField))}}return{value:t||e,text:WY(String(e)),disabled:!1}},normalizeOptions:function(e){var t=this;return Pt(e)?e.map((function(e){return t.normalizeOption(e)})):Ft(e)?(La(mR,this.$options.name),ea(e).map((function(a){return t.normalizeOption(e[a]||{},a)}))):[]}}});function gR(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function yR(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?gR(Object(a),!0).forEach((function(t){bR(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):gR(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function bR(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var MR=Sc(sa(yR(yR({},vR),{},{id:wc(Yo,void 0,!0)})),Pn),zR=i["default"].extend({name:Pn,mixins:[_R,xc],props:MR,render:function(e){var t=this.id,a=this.formOptions.map((function(t,a){var n=t.value,r=t.text,i=t.html,o=t.disabled;return e("option",{attrs:{value:n,disabled:o},domProps:UY(i,r),key:"option_".concat(a)})}));return e("datalist",{attrs:{id:t}},[a,this.normalizeSlot()])}});function LR(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var wR,HR,kR=Sc({id:wc(Yo),inline:wc(To,!1),tag:wc(Yo,"small"),textVariant:wc(Yo,"muted")},er),AR=i["default"].extend({name:er,functional:!0,props:kR,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.tag,Ja(n,{class:LR({"form-text":!a.inline},"text-".concat(a.textVariant),a.textVariant),attrs:{id:a.id}}),r)}}),SR=Sc({ariaLive:wc(Yo),forceShow:wc(To,!1),id:wc(Yo),role:wc(Yo),state:wc(To,null),tag:wc(Yo,"div"),tooltip:wc(To,!1)},Rn),VR=i["default"].extend({name:Rn,functional:!0,props:SR,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.tooltip,o=a.ariaLive,s=!0===a.forceShow||!1===a.state;return e(a.tag,Ja(n,{class:{"d-block":s,"invalid-feedback":!i,"invalid-tooltip":i},attrs:{id:a.id||null,role:a.role||null,"aria-live":o||null,"aria-atomic":o?"true":null}}),r)}}),OR=Sc({ariaLive:wc(Yo),forceShow:wc(To,!1),id:wc(Yo),role:wc(Yo),state:wc(To,null),tag:wc(Yo,"div"),tooltip:wc(To,!1)},nr),TR=i["default"].extend({name:nr,functional:!0,props:OR,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.tooltip,o=a.ariaLive,s=!0===a.forceShow||!0===a.state;return e(a.tag,Ja(n,{class:{"d-block":s,"valid-feedback":!i,"valid-tooltip":i},attrs:{id:a.id||null,role:a.role||null,"aria-live":o||null,"aria-atomic":o?"true":null}}),r)}}),DR=Sc({tag:wc(Yo,"div")},Gn),CR=i["default"].extend({name:Gn,functional:!0,props:DR,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.tag,Ja(n,{staticClass:"form-row"}),r)}}),jR=Pa({components:{BForm:J$,BFormDatalist:zR,BDatalist:zR,BFormText:AR,BFormInvalidFeedback:VR,BFormFeedback:VR,BFormValidFeedback:TR,BFormRow:CR}}),xR=function(e,t){for(var a=0;a<e.length;a++)if(EB(e[a],t))return a;return-1},BR="input, textarea, select",YR=Sc({autofocus:wc(To,!1),disabled:wc(To,!1),form:wc(Yo),id:wc(Yo),name:wc(Yo),required:wc(To,!1)},"formControls"),PR=i["default"].extend({props:YR,mounted:function(){this.handleAutofocus()},activated:function(){this.handleAutofocus()},methods:{handleAutofocus:function(){var e=this;this.$nextTick((function(){Ol((function(){var t=e.$el;e.autofocus&&Yl(t)&&($l(t,BR)||(t=El(BR,t)),lc(t))}))}))}}}),IR=Sc({plain:wc(To,!1)},"formControls"),FR=i["default"].extend({props:IR,computed:{custom:function(){return!this.plain}}}),ER=Sc({size:wc(Yo)},"formControls"),$R=i["default"].extend({props:ER,computed:{sizeFormClass:function(){return[this.size?"form-control-".concat(this.size):null]}}}),RR=Sc({state:wc(To,null)},"formState"),NR=i["default"].extend({props:RR,computed:{computedState:function(){return jt(this.state)?this.state:null},stateClass:function(){var e=this.computedState;return!0===e?"is-valid":!1===e?"is-invalid":null},computedAriaInvalid:function(){var e=this.ariaInvalid;return!0===e||"true"===e||""===e||!1===this.computedState?"true":e}}});function WR(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function UR(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?WR(Object(a),!0).forEach((function(t){GR(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):WR(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function GR(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var qR,JR=Dc("checked",{defaultValue:null}),KR=JR.mixin,XR=JR.props,ZR=JR.prop,QR=JR.event,eN=Sc(sa(UR(UR(UR(UR(UR(UR(UR({},XP),XR),YR),ER),RR),IR),{},{ariaLabel:wc(Yo),ariaLabelledby:wc(Yo),button:wc(To,!1),buttonVariant:wc(Yo),inline:wc(To,!1),value:wc(Vo)})),"formRadioCheckControls"),tN=i["default"].extend({mixins:[UB,ZP,KR,xc,PR,$R,NR,FR],inheritAttrs:!1,props:eN,data:function(){return{localChecked:this.isGroup?this.bvGroup[ZR]:this[ZR],hasFocus:!1}},computed:{computedLocalChecked:{get:function(){return this.isGroup?this.bvGroup.localChecked:this.localChecked},set:function(e){this.isGroup?this.bvGroup.localChecked=e:this.localChecked=e}},isChecked:function(){return EB(this.value,this.computedLocalChecked)},isRadio:function(){return!0},isGroup:function(){return!!this.bvGroup},isBtnMode:function(){return this.isGroup?this.bvGroup.buttons:this.button},isPlain:function(){return!this.isBtnMode&&(this.isGroup?this.bvGroup.plain:this.plain)},isCustom:function(){return!this.isBtnMode&&!this.isPlain},isSwitch:function(){return!(this.isBtnMode||this.isRadio||this.isPlain)&&(this.isGroup?this.bvGroup.switches:this.switch)},isInline:function(){return this.isGroup?this.bvGroup.inline:this.inline},isDisabled:function(){return this.isGroup&&this.bvGroup.disabled||this.disabled},isRequired:function(){return this.computedName&&(this.isGroup?this.bvGroup.required:this.required)},computedName:function(){return(this.isGroup?this.bvGroup.groupName:this.name)||null},computedForm:function(){return(this.isGroup?this.bvGroup.form:this.form)||null},computedSize:function(){return(this.isGroup?this.bvGroup.size:this.size)||""},computedState:function(){return this.isGroup?this.bvGroup.computedState:jt(this.state)?this.state:null},computedButtonVariant:function(){var e=this.buttonVariant;return e||(this.isGroup&&this.bvGroup.buttonVariant?this.bvGroup.buttonVariant:"secondary")},buttonClasses:function(){var e,t=this.computedSize;return["btn","btn-".concat(this.computedButtonVariant),(e={},GR(e,"btn-".concat(t),t),GR(e,"disabled",this.isDisabled),GR(e,"active",this.isChecked),GR(e,"focus",this.hasFocus),e)]},computedAttrs:function(){var e=this.isDisabled,t=this.isRequired;return UR(UR({},this.bvAttrs),{},{id:this.safeId(),type:this.isRadio?"radio":"checkbox",name:this.computedName,form:this.computedForm,disabled:e,required:t,"aria-required":t||null,"aria-label":this.ariaLabel||null,"aria-labelledby":this.ariaLabelledby||null})}},watch:(wR={},GR(wR,ZR,(function(){this["".concat(ZR,"Watcher")].apply(this,arguments)})),GR(wR,"computedLocalChecked",(function(){this.computedLocalCheckedWatcher.apply(this,arguments)})),wR),methods:(HR={},GR(HR,"".concat(ZR,"Watcher"),(function(e){EB(e,this.computedLocalChecked)||(this.computedLocalChecked=e)})),GR(HR,"computedLocalCheckedWatcher",(function(e,t){EB(e,t)||this.$emit(QR,e)})),GR(HR,"handleChange",(function(e){var t=this,a=e.target.checked,n=this.value,r=a?n:null;this.computedLocalChecked=n,this.$nextTick((function(){t.$emit(Li,r),t.isGroup&&t.bvGroup.$emit(Li,r)}))})),GR(HR,"handleFocus",(function(e){e.target&&("focus"===e.type?this.hasFocus=!0:"blur"===e.type&&(this.hasFocus=!1))})),GR(HR,"focus",(function(){this.isDisabled||lc(this.$refs.input)})),GR(HR,"blur",(function(){this.isDisabled||cc(this.$refs.input)})),HR),render:function(e){var t=this.isRadio,a=this.isBtnMode,n=this.isPlain,r=this.isCustom,i=this.isInline,o=this.isSwitch,s=this.computedSize,l=this.bvAttrs,c=this.normalizeSlot(),u=e("input",{class:[{"form-check-input":n,"custom-control-input":r,"position-static":n&&!c},a?"":this.stateClass],directives:[{name:"model",value:this.computedLocalChecked}],attrs:this.computedAttrs,domProps:{value:this.value,checked:this.isChecked},on:UR({change:this.handleChange},a?{focus:this.handleFocus,blur:this.handleFocus}:{}),key:"input",ref:"input"});if(a){var d=e("label",{class:this.buttonClasses},[u,c]);return this.isGroup||(d=e("div",{class:["btn-group-toggle","d-inline-block"]},[d])),d}var h=e();return n&&!c||(h=e("label",{class:{"form-check-label":n,"custom-control-label":r},attrs:{for:this.safeId()}},c)),e("div",{class:[GR({"form-check":n,"form-check-inline":n&&i,"custom-control":r,"custom-control-inline":r&&i,"custom-checkbox":r&&!t&&!o,"custom-switch":o,"custom-radio":r&&t},"b-custom-control-".concat(s),s&&!a),l.class],style:l.style},[u,h])}});function aN(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function nN(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?aN(Object(a),!0).forEach((function(t){rN(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):aN(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function rN(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var iN,oN="indeterminate",sN=wo+oN,lN=Sc(sa(nN(nN({},eN),{},(qR={},rN(qR,oN,wc(To,!1)),rN(qR,"switch",wc(To,!1)),rN(qR,"uncheckedValue",wc(Vo,!1)),rN(qR,"value",wc(Vo,!0)),qR))),Bn),cN=i["default"].extend({name:Bn,mixins:[tN],inject:{bvGroup:{from:"bvCheckGroup",default:null}},props:lN,computed:{isChecked:function(){var e=this.value,t=this.computedLocalChecked;return Pt(t)?xR(t,e)>-1:EB(t,e)},isRadio:function(){return!1}},watch:rN({},oN,(function(e,t){EB(e,t)||this.setIndeterminate(e)})),mounted:function(){this.setIndeterminate(this[oN])},methods:{computedLocalCheckedWatcher:function(e,t){if(!EB(e,t)){this.$emit(QR,e);var a=this.$refs.input;a&&this.$emit(sN,a.indeterminate)}},handleChange:function(e){var t=this,a=e.target,n=a.checked,r=a.indeterminate,i=this.value,o=this.uncheckedValue,s=this.computedLocalChecked;if(Pt(s)){var l=xR(s,i);n&&l<0?s=s.concat(i):!n&&l>-1&&(s=s.slice(0,l).concat(s.slice(l+1)))}else s=n?i:o;this.computedLocalChecked=s,this.$nextTick((function(){t.$emit(Li,s),t.isGroup&&t.bvGroup.$emit(Li,s),t.$emit(sN,r)}))},setIndeterminate:function(e){Pt(this.computedLocalChecked)&&(e=!1);var t=this.$refs.input;t&&(t.indeterminate=e,this.$emit(sN,e))}}}),uN=Sc(eN,Nn),dN=i["default"].extend({name:Nn,mixins:[tN],inject:{bvGroup:{from:"bvRadioGroup",default:!1}},props:uN,watch:{computedLocalChecked:function(e,t){EB(e,t)||this.$emit(QR,e)}}});function hN(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function fN(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?hN(Object(a),!0).forEach((function(t){pN(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):hN(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function pN(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var mN,vN=["aria-describedby","aria-labelledby"],_N=Dc("checked"),gN=_N.mixin,yN=_N.props,bN=_N.prop,MN=_N.event,zN=Sc(sa(fN(fN(fN(fN(fN(fN(fN(fN({},XP),yN),YR),vR),ER),RR),IR),{},{ariaInvalid:wc(No,!1),buttonVariant:wc(Yo),buttons:wc(To,!1),stacked:wc(To,!1),validated:wc(To,!1)})),"formRadioCheckGroups"),LN=i["default"].extend({mixins:[ZP,gN,xc,PR,_R,$R,NR,FR],inheritAttrs:!1,props:zN,data:function(){return{localChecked:this[bN]}},computed:{inline:function(){return!this.stacked},groupName:function(){return this.name||this.safeId()},groupClasses:function(){var e=this.inline,t=this.size,a=this.validated,n={"was-validated":a};return this.buttons&&(n=[n,"btn-group-toggle",pN({"btn-group":e,"btn-group-vertical":!e},"btn-group-".concat(t),t)]),n}},watch:(iN={},pN(iN,bN,(function(e){EB(e,this.localChecked)||(this.localChecked=e)})),pN(iN,"localChecked",(function(e,t){EB(e,t)||this.$emit(MN,e)})),iN),render:function(e){var t=this,a=this.isRadioGroup,n=ra(this.$attrs,vN),r=a?dN:cN,i=this.formOptions.map((function(a,i){var o="BV_option_".concat(i);return e(r,{props:{disabled:a.disabled||!1,id:t.safeId(o),value:a.value},attrs:n,key:o},[e("span",{domProps:UY(a.html,a.text)})])}));return e("div",{class:[this.groupClasses,"bv-no-focus-ring"],attrs:fN(fN({},ia(this.$attrs,vN)),{},{"aria-invalid":this.computedAriaInvalid,"aria-required":this.required?"true":null,id:this.safeId(),role:a?"radiogroup":"group",tabindex:"-1"})},[this.normalizeSlot(fs),i,this.normalizeSlot()])}});function wN(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function HN(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?wN(Object(a),!0).forEach((function(t){kN(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):wN(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function kN(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var AN=Sc(sa(HN(HN({},zN),{},(mN={},kN(mN,bN,wc(Oo,[])),kN(mN,"switches",wc(To,!1)),mN))),Yn),SN=i["default"].extend({name:Yn,mixins:[LN],provide:function(){return{bvCheckGroup:this}},props:AN,computed:{isRadioGroup:function(){return!1}}}),VN=Pa({components:{BFormCheckbox:cN,BCheckbox:cN,BCheck:cN,BFormCheckboxGroup:SN,BCheckboxGroup:SN,BCheckGroup:SN}}),ON="__BV_hover_handler__",TN="mouseenter",DN="mouseleave",CN=function(e){var t=function(t){e(t.type===TN,t)};return t.fn=e,t},jN=function(e,t,a){Ic(e,t,TN,a,So),Ic(e,t,DN,a,So)},xN=function(e,t){var a=t.value,n=void 0===a?null:a;if(Me){var r=e[ON],i=Ct(r),o=!(i&&r.fn===n);i&&o&&(jN(!1,e,r),delete e[ON]),Ct(n)&&o&&(e[ON]=CN(n),jN(!0,e,e[ON]))}},BN={bind:xN,componentUpdated:xN,unbind:function(e){xN(e,{value:null})}};function YN(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function PN(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?YN(Object(a),!0).forEach((function(t){IN(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):YN(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function IN(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var FN,EN=sa(PN(PN(PN(PN(PN(PN({},XP),ER),RR),ia(b$,["disabled"])),ia(YR,["autofocus"])),{},{buttonOnly:wc(To,!1),buttonVariant:wc(Yo,"secondary"),formattedValue:wc(Yo),labelSelected:wc(Yo),lang:wc(Yo),menuClass:wc(Fo),placeholder:wc(Yo),readonly:wc(To,!1),rtl:wc(To,null),value:wc(Yo,"")})),$N=i["default"].extend({name:li,directives:{"b-hover":BN},mixins:[ZP,$R,NR,M$,xc],props:EN,data:function(){return{isHovered:!1,hasFocus:!1}},computed:{idButton:function(){return this.safeId()},idLabel:function(){return this.safeId("_value_")},idMenu:function(){return this.safeId("_dialog_")},idWrapper:function(){return this.safeId("_outer_")},computedDir:function(){return!0===this.rtl?"rtl":!1===this.rtl?"ltr":null}},methods:{focus:function(){this.disabled||lc(this.$refs.toggle)},blur:function(){this.disabled||cc(this.$refs.toggle)},setFocus:function(e){this.hasFocus="focus"===e.type},handleHover:function(e){this.isHovered=e}},render:function(e){var t,a=this.idButton,n=this.idLabel,r=this.idMenu,i=this.idWrapper,o=this.disabled,s=this.readonly,l=this.required,c=this.name,u=this.state,d=this.visible,h=this.size,f=this.isHovered,p=this.hasFocus,m=this.labelSelected,v=this.buttonVariant,_=this.buttonOnly,g=zl(this.value)||"",y=!1===u||l&&!g,b={isHovered:f,hasFocus:p,state:u,opened:d},M=e("button",{staticClass:"btn",class:(t={},IN(t,"btn-".concat(v),_),IN(t,"btn-".concat(h),h),IN(t,"h-auto",!_),IN(t,"dropdown-toggle",_),IN(t,"dropdown-toggle-no-caret",_),t),attrs:{id:a,type:"button",disabled:o,"aria-haspopup":"dialog","aria-expanded":d?"true":"false","aria-invalid":y?"true":null,"aria-required":l?"true":null},directives:[{name:"b-hover",value:this.handleHover}],on:{mousedown:this.onMousedown,click:this.toggle,keydown:this.toggle,"!focus":this.setFocus,"!blur":this.setFocus},ref:"toggle"},[this.hasNormalizedSlot(as)?this.normalizeSlot(as,b):e(fg,{props:{scale:1.25}})]),z=e();c&&!o&&(z=e("input",{attrs:{type:"hidden",name:c||null,form:this.form||null,value:g}}));var L=e("div",{staticClass:"dropdown-menu",class:[this.menuClass,{show:d,"dropdown-menu-right":this.right}],attrs:{id:r,role:"dialog",tabindex:"-1","aria-modal":"false","aria-labelledby":n},on:{keydown:this.onKeydown},ref:"menu"},[this.normalizeSlot(is,{opened:d})]),w=e("label",{class:_?"sr-only":["form-control",{"text-muted":!g},this.stateClass,this.sizeFormClass],attrs:{id:n,for:a,"aria-invalid":y?"true":null,"aria-required":l?"true":null},directives:[{name:"b-hover",value:this.handleHover}],on:{"!click":function(e){Fc(e,{preventDefault:!1})}}},[g?this.formattedValue||g:this.placeholder||"",g&&m?e("bdi",{staticClass:"sr-only"},m):""]);return e("div",{staticClass:"b-form-btn-label-control dropdown",class:[this.directionClass,this.boundaryClass,[{"btn-group":_,"form-control":!_,focus:p&&!_,show:d,"is-valid":!0===u,"is-invalid":!1===u},_?null:this.sizeFormClass]],attrs:{id:i,role:_?null:"group",lang:this.lang||null,dir:this.computedDir,"aria-disabled":o,"aria-readonly":s&&!o,"aria-labelledby":n,"aria-invalid":!1===u||l&&!g?"true":null,"aria-required":l?"true":null}},[M,z,L,w])}});function RN(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function NN(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?RN(Object(a),!0).forEach((function(t){WN(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):RN(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function WN(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var UN,GN=Dc("value",{type:Wo}),qN=GN.mixin,JN=GN.props,KN=GN.prop,XN=GN.event,ZN=ia(sI,["block","hidden","id","noKeyNav","roleDescription","value","width"]),QN=ia(EN,["formattedValue","id","lang","rtl","value"]),eW=Sc(sa(NN(NN(NN(NN(NN({},XP),JN),ZN),QN),{},{calendarWidth:wc(Yo,"270px"),closeButton:wc(To,!1),closeButtonVariant:wc(Yo,"outline-secondary"),dark:wc(To,!1),labelCloseButton:wc(Yo,"Close"),labelResetButton:wc(Yo,"Reset"),labelTodayButton:wc(Yo,"Select today"),noCloseOnSelect:wc(To,!1),resetButton:wc(To,!1),resetButtonVariant:wc(Yo,"outline-danger"),resetValue:wc(Wo),todayButton:wc(To,!1),todayButtonVariant:wc(Yo,"outline-primary")})),In),tW=i["default"].extend({name:In,mixins:[ZP,qN],props:eW,data:function(){return{localYMD:xP(this[KN])||"",isVisible:!1,localLocale:null,isRTL:!1,formattedValue:"",activeYMD:""}},computed:{calendarYM:function(){return this.activeYMD.slice(0,-3)},computedLang:function(){return(this.localLocale||"").replace(/-u-.*$/i,"")||null},computedResetValue:function(){return xP(qP(this.resetValue))||""}},watch:(FN={},WN(FN,KN,(function(e){this.localYMD=xP(e)||""})),WN(FN,"localYMD",(function(e){this.isVisible&&this.$emit(XN,this.valueAsDate?jP(e)||null:e||"")})),WN(FN,"calendarYM",(function(e,t){if(e!==t&&t)try{this.$refs.control.updatePopper()}catch(a){}})),FN),methods:{focus:function(){this.disabled||lc(this.$refs.control)},blur:function(){this.disabled||cc(this.$refs.control)},setAndClose:function(e){var t=this;this.localYMD=e,this.noCloseOnSelect||this.$nextTick((function(){t.$refs.control.hide(!0)}))},onSelected:function(e){var t=this;this.$nextTick((function(){t.setAndClose(e)}))},onInput:function(e){this.localYMD!==e&&(this.localYMD=e)},onContext:function(e){var t=e.activeYMD,a=e.isRTL,n=e.locale,r=e.selectedYMD,i=e.selectedFormatted;this.isRTL=a,this.localLocale=n,this.formattedValue=i,this.localYMD=r,this.activeYMD=t,this.$emit(Ai,e)},onTodayButton:function(){this.setAndClose(xP(qP(CP(),this.min,this.max)))},onResetButton:function(){this.setAndClose(this.computedResetValue)},onCloseButton:function(){this.$refs.control.hide(!0)},onShow:function(){this.isVisible=!0},onShown:function(){var e=this;this.$nextTick((function(){lc(e.$refs.calendar),e.$emit(po)}))},onHidden:function(){this.isVisible=!1,this.$emit($i)},defaultButtonFn:function(e){var t=e.isHovered,a=e.hasFocus;return this.$createElement(t||a?tv:hm,{attrs:{"aria-hidden":"true"}})}},render:function(e){var t=this.localYMD,a=this.disabled,n=this.readonly,r=this.dark,i=this.$props,o=this.$scopedSlots,s=Dt(this.placeholder)?this.labelNoDateSelected:this.placeholder,l=[];if(this.todayButton){var c=this.labelTodayButton;l.push(e(LY,{props:{disabled:a||n,size:"sm",variant:this.todayButtonVariant},attrs:{"aria-label":c||null},on:{click:this.onTodayButton}},c))}if(this.resetButton){var u=this.labelResetButton;l.push(e(LY,{props:{disabled:a||n,size:"sm",variant:this.resetButtonVariant},attrs:{"aria-label":u||null},on:{click:this.onResetButton}},u))}if(this.closeButton){var d=this.labelCloseButton;l.push(e(LY,{props:{disabled:a,size:"sm",variant:this.closeButtonVariant},attrs:{"aria-label":d||null},on:{click:this.onCloseButton}},d))}l.length>0&&(l=[e("div",{staticClass:"b-form-date-controls d-flex flex-wrap",class:{"justify-content-between":l.length>1,"justify-content-end":l.length<2}},l)]);var h=e(lI,{staticClass:"b-form-date-calendar w-100",props:NN(NN({},kc(ZN,i)),{},{hidden:!this.isVisible,value:t,valueAsDate:!1,width:this.calendarWidth}),on:{selected:this.onSelected,input:this.onInput,context:this.onContext},scopedSlots:ra(o,["nav-prev-decade","nav-prev-year","nav-prev-month","nav-this-month","nav-next-month","nav-next-year","nav-next-decade"]),key:"calendar",ref:"calendar"},l);return e($N,{staticClass:"b-form-datepicker",props:NN(NN({},kc(QN,i)),{},{formattedValue:t?this.formattedValue:"",id:this.safeId(),lang:this.computedLang,menuClass:[{"bg-dark":r,"text-light":r},this.menuClass],placeholder:s,rtl:this.isRTL,value:t}),on:{show:this.onShow,shown:this.onShown,hidden:this.onHidden},scopedSlots:WN({},as,o[as]||this.defaultButtonFn),ref:"control"},[h])}}),aW=Pa({components:{BFormDatepicker:tW,BDatepicker:tW}});function nW(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function rW(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?nW(Object(a),!0).forEach((function(t){iW(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):nW(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function iW(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var oW=Dc("value",{type:[Oo,kt],defaultValue:null,validator:function(e){return""===e?(La(dW,Fn),!0):Dt(e)||hW(e)}}),sW=oW.mixin,lW=oW.props,cW=oW.prop,uW=oW.event,dW='Setting "value"/"v-model" to an empty string for reset is deprecated. Set to "null" instead.',hW=function e(t){return Rt(t)||Pt(t)&&t.every((function(t){return e(t)}))},fW=function(e){return Ct(e.getAsEntry)?e.getAsEntry():Ct(e.webkitGetAsEntry)?e.webkitGetAsEntry():null},pW=function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1];return Promise.all(sl(e).filter((function(e){return"file"===e.kind})).map((function(e){var a=fW(e);if(a){if(a.isDirectory&&t)return mW(a.createReader(),"".concat(a.name,"/"));if(a.isFile)return new Promise((function(e){a.file((function(t){t.$path="",e(t)}))}))}return null})).filter(ya))},mW=function e(t){var a=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"";return new Promise((function(n){var r=[],i=function i(){t.readEntries((function(t){0===t.length?n(Promise.all(r).then((function(e){return dl(e)}))):(r.push(Promise.all(t.map((function(t){if(t){if(t.isDirectory)return e(t.createReader(),"".concat(a).concat(t.name,"/"));if(t.isFile)return new Promise((function(e){t.file((function(t){t.$path="".concat(a).concat(t.name),e(t)}))}))}return null})).filter(ya))),i())}))};i()}))},vW=Sc(sa(rW(rW(rW(rW(rW(rW(rW({},XP),lW),YR),IR),RR),ER),{},{accept:wc(Yo,""),browseText:wc(Yo,"Browse"),capture:wc(To,!1),directory:wc(To,!1),dropPlaceholder:wc(Yo,"Drop files here"),fileNameFormatter:wc(Co),multiple:wc(To,!1),noDrop:wc(To,!1),noDropPlaceholder:wc(Yo,"Not allowed"),noTraverse:wc(To,!1),placeholder:wc(Yo,"No file chosen")})),Fn),_W=i["default"].extend({name:Fn,mixins:[UB,ZP,sW,xc,PR,NR,FR,xc],inheritAttrs:!1,props:vW,data:function(){return{files:[],dragging:!1,dropAllowed:!this.noDrop,hasFocus:!1}},computed:{computedAccept:function(){var e=this.accept;return e=(e||"").trim().split(/[,\s]+/).filter(ya),0===e.length?null:e.map((function(e){var t="name",a="^",n="$";Ye.test(e)?a="":(t="type",qe.test(e)&&(n=".+$",e=e.slice(0,-1))),e=Ml(e);var r=new RegExp("".concat(a).concat(e).concat(n));return{rx:r,prop:t}}))},computedCapture:function(){var e=this.capture;return!0===e||""===e||(e||null)},computedAttrs:function(){var e=this.name,t=this.disabled,a=this.required,n=this.form,r=this.computedCapture,i=this.accept,o=this.multiple,s=this.directory;return rW(rW({},this.bvAttrs),{},{type:"file",id:this.safeId(),name:e,disabled:t,required:a,form:n||null,capture:r,accept:i||null,multiple:o,directory:s,webkitdirectory:s,"aria-required":a?"true":null})},computedFileNameFormatter:function(){var e=this.fileNameFormatter;return Oc(e)?e:this.defaultFileNameFormatter},clonedFiles:function(){return ga(this.files)},flattenedFiles:function(){return hl(this.files)},fileNames:function(){return this.flattenedFiles.map((function(e){return e.name}))},labelContent:function(){if(this.dragging&&!this.noDrop)return this.normalizeSlot(ls,{allowed:this.dropAllowed})||(this.dropAllowed?this.dropPlaceholder:this.$createElement("span",{staticClass:"text-danger"},this.noDropPlaceholder));if(0===this.files.length)return this.normalizeSlot(Us)||this.placeholder;var e=this.flattenedFiles,t=this.clonedFiles,a=this.fileNames,n=this.computedFileNameFormatter;return this.hasNormalizedSlot(hs)?this.normalizeSlot(hs,{files:e,filesTraversed:t,names:a}):n(e,t,a)}},watch:(UN={},iW(UN,cW,(function(e){(!e||Pt(e)&&0===e.length)&&this.reset()})),iW(UN,"files",(function(e,t){if(!EB(e,t)){var a=this.multiple,n=this.noTraverse,r=!a||n?hl(e):e;this.$emit(uW,a?r:r[0]||null)}})),UN),created:function(){this.$_form=null},mounted:function(){var e=Rl("form",this.$el);e&&(Yc(e,"reset",this.reset,Ao),this.$_form=e)},beforeDestroy:function(){var e=this.$_form;e&&Pc(e,"reset",this.reset,Ao)},methods:{isFileValid:function(e){if(!e)return!1;var t=this.computedAccept;return!t||t.some((function(t){return t.rx.test(e[t.prop])}))},isFilesArrayValid:function(e){var t=this;return Pt(e)?e.every((function(e){return t.isFileValid(e)})):this.isFileValid(e)},defaultFileNameFormatter:function(e,t,a){return a.join(", ")},setFiles:function(e){this.dropAllowed=!this.noDrop,this.dragging=!1,this.files=this.multiple?this.directory?e:hl(e):hl(e).slice(0,1)},setInputFiles:function(e){try{var t=new ClipboardEvent("").clipboardData||new DataTransfer;hl(ga(e)).forEach((function(e){delete e.$path,t.items.add(e)})),this.$refs.input.files=t.files}catch(a){}},reset:function(){try{var e=this.$refs.input;e.value="",e.type="",e.type="file"}catch(t){}this.files=[]},handleFiles:function(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(t){var a=e.filter(this.isFilesArrayValid);a.length>0&&(this.setFiles(a),this.setInputFiles(a))}else this.setFiles(e)},focusHandler:function(e){this.plain||"focusout"===e.type?this.hasFocus=!1:this.hasFocus=!0},onChange:function(e){var t=this,a=e.type,n=e.target,r=e.dataTransfer,i=void 0===r?{}:r,o="drop"===a;this.$emit(Li,e);var s=sl(i.items||[]);if(ye&&s.length>0&&!Tt(fW(s[0])))pW(s,this.directory).then((function(e){return t.handleFiles(e,o)}));else{var l=sl(n.files||i.files||[]).map((function(e){return e.$path=e.webkitRelativePath||"",e}));this.handleFiles(l,o)}},onDragenter:function(e){Fc(e),this.dragging=!0;var t=e.dataTransfer,a=void 0===t?{}:t;if(this.noDrop||this.disabled||!this.dropAllowed)return a.dropEffect="none",void(this.dropAllowed=!1);a.dropEffect="copy"},onDragover:function(e){Fc(e),this.dragging=!0;var t=e.dataTransfer,a=void 0===t?{}:t;if(this.noDrop||this.disabled||!this.dropAllowed)return a.dropEffect="none",void(this.dropAllowed=!1);a.dropEffect="copy"},onDragleave:function(e){var t=this;Fc(e),this.$nextTick((function(){t.dragging=!1,t.dropAllowed=!t.noDrop}))},onDrop:function(e){var t=this;Fc(e),this.dragging=!1,this.noDrop||this.disabled||!this.dropAllowed?this.$nextTick((function(){t.dropAllowed=!t.noDrop})):this.onChange(e)}},render:function(e){var t=this.custom,a=this.plain,n=this.size,r=this.dragging,i=this.stateClass,o=this.bvAttrs,s=e("input",{class:[{"form-control-file":a,"custom-file-input":t,focus:t&&this.hasFocus},i],style:t?{zIndex:-5}:{},attrs:this.computedAttrs,on:{change:this.onChange,focusin:this.focusHandler,focusout:this.focusHandler,reset:this.reset},ref:"input"});if(a)return s;var l=e("label",{staticClass:"custom-file-label",class:{dragging:r},attrs:{for:this.safeId(),"data-browse":this.browseText||null}},[e("span",{staticClass:"d-block form-file-text",style:{pointerEvents:"none"}},[this.labelContent])]);return e("div",{staticClass:"custom-file b-form-file",class:[iW({},"b-custom-control-".concat(n),n),i,o.class],style:o.style,attrs:{id:this.safeId("_BV_file_outer_")},on:{dragenter:this.onDragenter,dragover:this.onDragover,dragleave:this.onDragleave,drop:this.onDrop}},[s,l])}}),gW=Pa({components:{BFormFile:_W,BFile:_W}}),yW=function(e){return"\\"+e},bW=function(e){e=zl(e);var t=e.length,a=e.charCodeAt(0);return e.split("").reduce((function(n,r,i){var o=e.charCodeAt(i);return 0===o?n+"�":127===o||o>=1&&o<=31||0===i&&o>=48&&o<=57||1===i&&o>=48&&o<=57&&45===a?n+yW("".concat(o.toString(16)," ")):0===i&&45===o&&1===t?n+yW(r):o>=128||45===o||95===o||o>=48&&o<=57||o>=65&&o<=90||o>=97&&o<=122?n+r:n+yW(r)}),"")};function MW(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function zW(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?MW(Object(a),!0).forEach((function(t){LW(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):MW(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function LW(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var wW=["auto","start","end","center","baseline","stretch"],HW=function(e,t,a){var n=e;if(!Dt(a)&&!1!==a)return t&&(n+="-".concat(t)),"col"!==e||""!==a&&!0!==a?(n+="-".concat(a),Hl(n)):Hl(n)},kW=uc(HW),AW=Kt(null),SW=function(){var e=_c().filter(ya),t=e.reduce((function(e,t){return e[t]=wc(Ro),e}),Kt(null)),a=e.reduce((function(e,t){return e[Lc(t,"offset")]=wc(Go),e}),Kt(null)),n=e.reduce((function(e,t){return e[Lc(t,"order")]=wc(Go),e}),Kt(null));return AW=Jt(Kt(null),{col:ea(t),offset:ea(a),order:ea(n)}),Sc(sa(zW(zW(zW(zW({},t),a),n),{},{alignSelf:wc(Yo,null,(function(e){return ll(wW,e)})),col:wc(To,!1),cols:wc(Go),offset:wc(Go),order:wc(Go),tag:wc(Yo,"div")})),Ln)},VW={name:Ln,functional:!0,get props(){return delete this.props,this.props=SW()},render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.cols,s=n.offset,l=n.order,c=n.alignSelf,u=[];for(var d in AW)for(var h=AW[d],f=0;f<h.length;f++){var p=kW(d,h[f].replace(d,""),n[h[f]]);p&&u.push(p)}var m=u.some((function(e){return lt.test(e)}));return u.push((a={col:n.col||!m&&!o},LW(a,"col-".concat(o),o),LW(a,"offset-".concat(s),s),LW(a,"order-".concat(l),l),LW(a,"align-self-".concat(c),c),a)),e(n.tag,Ja(r,{class:u}),i)}};function OW(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function TW(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?OW(Object(a),!0).forEach((function(t){DW(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):OW(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function DW(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var CW=["input","select","textarea"],jW=CW.map((function(e){return"".concat(e,":not([disabled])")})).join(),xW=[].concat(CW,["a","button","label"]),BW=function(){return Sc(sa(TW(TW(TW(TW({},XP),RR),_c().reduce((function(e,t){return e[Lc(t,"contentCols")]=wc(Ro),e[Lc(t,"labelAlign")]=wc(Yo),e[Lc(t,"labelCols")]=wc(Ro),e}),Kt(null))),{},{description:wc(Yo),disabled:wc(To,!1),feedbackAriaLive:wc(Yo,"assertive"),invalidFeedback:wc(Yo),label:wc(Yo),labelClass:wc(Fo),labelFor:wc(Yo),labelSize:wc(Yo),labelSrOnly:wc(To,!1),tooltip:wc(To,!1),validFeedback:wc(Yo),validated:wc(To,!1)})),En)},YW={name:En,mixins:[ZP,NR,xc],get props(){return delete this.props,this.props=BW()},data:function(){return{ariaDescribedby:null}},computed:{contentColProps:function(){return this.getColProps(this.$props,"content")},labelAlignClasses:function(){return this.getAlignClasses(this.$props,"label")},labelColProps:function(){return this.getColProps(this.$props,"label")},isHorizontal:function(){return ea(this.contentColProps).length>0||ea(this.labelColProps).length>0}},watch:{ariaDescribedby:function(e,t){e!==t&&this.updateAriaDescribedby(e,t)}},mounted:function(){var e=this;this.$nextTick((function(){e.updateAriaDescribedby(e.ariaDescribedby)}))},methods:{getAlignClasses:function(e,t){return _c().reduce((function(a,n){var r=e[Lc(n,"".concat(t,"Align"))]||null;return r&&a.push(["text",n,r].filter(ya).join("-")),a}),[])},getColProps:function(e,t){return _c().reduce((function(a,n){var r=e[Lc(n,"".concat(t,"Cols"))];return r=""===r||(r||!1),jt(r)||"auto"===r||(r=fl(r,0),r=r>0&&r),r&&(a[n||(jt(r)?"col":"cols")]=r),a}),{})},updateAriaDescribedby:function(e,t){var a=this.labelFor;if(Me&&a){var n=El("#".concat(bW(a)),this.$refs.content);if(n){var r="aria-describedby",i=(e||"").split(Ge),o=(t||"").split(Ge),s=(Xl(n,r)||"").split(Ge).filter((function(e){return!ll(o,e)})).concat(i).filter((function(e,t,a){return a.indexOf(e)===t})).filter(ya).join(" ").trim();s?Jl(n,r,s):Kl(n,r)}}},onLegendClick:function(e){if(!this.labelFor){var t=e.target,a=t?t.tagName:"";if(-1===xW.indexOf(a)){var n=Fl(jW,this.$refs.content).filter(Yl);1===n.length&&lc(n[0])}}}},render:function(e){var t=this.computedState,a=this.feedbackAriaLive,n=this.isHorizontal,r=this.labelFor,i=this.normalizeSlot,o=this.safeId,s=this.tooltip,l=o(),c=!r,u=e(),d=i(Hs)||this.label,h=d?o("_BV_label_"):null;if(d||n){var f=this.labelSize,p=this.labelColProps,m=c?"legend":"label";this.labelSrOnly?(d&&(u=e(m,{class:"sr-only",attrs:{id:h,for:r||null}},[d])),u=e(n?VW:"div",{props:n?p:{}},[u])):u=e(n?VW:m,{on:c?{click:this.onLegendClick}:{},props:n?TW(TW({},p),{},{tag:m}):{},attrs:{id:h,for:r||null,tabindex:c?"-1":null},class:[c?"bv-no-focus-ring":"",n||c?"col-form-label":"",!n&&c?"pt-0":"",n||c?"":"d-block",f?"col-form-label-".concat(f):"",this.labelAlignClasses,this.labelClass]},[d])}var v=e(),_=i(ws)||this.invalidFeedback,g=_?o("_BV_feedback_invalid_"):null;_&&(v=e(VR,{props:{ariaLive:a,id:g,state:t,tooltip:s},attrs:{tabindex:_?"-1":null}},[_]));var y=e(),b=i(ol)||this.validFeedback,M=b?o("_BV_feedback_valid_"):null;b&&(y=e(TR,{props:{ariaLive:a,id:M,state:t,tooltip:s},attrs:{tabindex:b?"-1":null}},[b]));var z=e(),L=i(os)||this.description,w=L?o("_BV_description_"):null;L&&(z=e(AR,{attrs:{id:w,tabindex:"-1"}},[L]));var H=this.ariaDescribedby=[w,!1===t?g:null,!0===t?M:null].filter(ya).join(" ")||null,k=e(n?VW:"div",{props:n?this.contentColProps:{},ref:"content"},[i(is,{ariaDescribedby:H,descriptionId:w,id:l,labelId:h})||e(),v,y,z]);return e(c?"fieldset":n?CR:"div",{staticClass:"form-group",class:[{"was-validated":this.validated},this.stateClass],attrs:{id:l,disabled:c?this.disabled:null,role:c?null:"group","aria-invalid":this.computedAriaInvalid,"aria-labelledby":c&&n?h:null}},n&&c?[e(CR,[u,k])]:[u,k])}},PW=Pa({components:{BFormGroup:YW,BFormFieldset:YW}}),IW=i["default"].extend({computed:{selectionStart:{cache:!1,get:function(){return this.$refs.input.selectionStart},set:function(e){this.$refs.input.selectionStart=e}},selectionEnd:{cache:!1,get:function(){return this.$refs.input.selectionEnd},set:function(e){this.$refs.input.selectionEnd=e}},selectionDirection:{cache:!1,get:function(){return this.$refs.input.selectionDirection},set:function(e){this.$refs.input.selectionDirection=e}}},methods:{select:function(){var e;(e=this.$refs.input).select.apply(e,arguments)},setSelectionRange:function(){var e;(e=this.$refs.input).setSelectionRange.apply(e,arguments)},setRangeText:function(){var e;(e=this.$refs.input).setRangeText.apply(e,arguments)}}});function FW(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function EW(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?FW(Object(a),!0).forEach((function(t){$W(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):FW(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function $W(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var RW=Dc("value",{type:Go,defaultValue:"",event:Mo}),NW=RW.mixin,WW=RW.props,UW=RW.prop,GW=RW.event,qW=Sc(sa(EW(EW({},WW),{},{ariaInvalid:wc(No,!1),autocomplete:wc(Yo),debounce:wc(Go,0),formatter:wc(Co),lazy:wc(To,!1),lazyFormatter:wc(To,!1),number:wc(To,!1),placeholder:wc(Yo),plaintext:wc(To,!1),readonly:wc(To,!1),trim:wc(To,!1)})),"formTextControls"),JW=i["default"].extend({mixins:[NW],props:qW,data:function(){var e=this[UW];return{localValue:zl(e),vModelValue:this.modifyValue(e)}},computed:{computedClass:function(){var e=this.plaintext,t=this.type,a="range"===t,n="color"===t;return[{"custom-range":a,"form-control-plaintext":e&&!a&&!n,"form-control":n||!e&&!a},this.sizeFormClass,this.stateClass]},computedDebounce:function(){return mu(fl(this.debounce,0),0)},hasFormatter:function(){return Oc(this.formatter)}},watch:$W({},UW,(function(e){var t=zl(e),a=this.modifyValue(e);t===this.localValue&&a===this.vModelValue||(this.clearDebounce(),this.localValue=t,this.vModelValue=a)})),created:function(){this.$_inputDebounceTimer=null},beforeDestroy:function(){this.clearDebounce()},methods:{clearDebounce:function(){clearTimeout(this.$_inputDebounceTimer),this.$_inputDebounceTimer=null},formatValue:function(e,t){var a=arguments.length>2&&void 0!==arguments[2]&&arguments[2];return e=zl(e),!this.hasFormatter||this.lazyFormatter&&!a||(e=this.formatter(e,t)),e},modifyValue:function(e){return e=zl(e),this.trim&&(e=e.trim()),this.number&&(e=pl(e,e)),e},updateValue:function(e){var t=this,a=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=this.lazy;if(!n||a){this.clearDebounce();var r=function(){if(e=t.modifyValue(e),e!==t.vModelValue)t.vModelValue=e,t.$emit(GW,e);else if(t.hasFormatter){var a=t.$refs.input;a&&e!==a.value&&(a.value=e)}},i=this.computedDebounce;i>0&&!n&&!a?this.$_inputDebounceTimer=setTimeout(r,i):r()}},onInput:function(e){if(!e.target.composing){var t=e.target.value,a=this.formatValue(t,e);!1===a||e.defaultPrevented?Fc(e,{propagation:!1}):(this.localValue=a,this.updateValue(a),this.$emit(Wi,a))}},onChange:function(e){var t=e.target.value,a=this.formatValue(t,e);!1===a||e.defaultPrevented?Fc(e,{propagation:!1}):(this.localValue=a,this.updateValue(a,!0),this.$emit(Li,a))},onBlur:function(e){var t=e.target.value,a=this.formatValue(t,e,!0);!1!==a&&(this.localValue=zl(this.modifyValue(a)),this.updateValue(a,!0)),this.$emit(Mi,e)},focus:function(){this.disabled||lc(this.$el)},blur:function(){this.disabled||cc(this.$el)}}}),KW=i["default"].extend({computed:{validity:{cache:!1,get:function(){return this.$refs.input.validity}},validationMessage:{cache:!1,get:function(){return this.$refs.input.validationMessage}},willValidate:{cache:!1,get:function(){return this.$refs.input.willValidate}}},methods:{setCustomValidity:function(){var e;return(e=this.$refs.input).setCustomValidity.apply(e,arguments)},checkValidity:function(){var e;return(e=this.$refs.input).checkValidity.apply(e,arguments)},reportValidity:function(){var e;return(e=this.$refs.input).reportValidity.apply(e,arguments)}}});function XW(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function ZW(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?XW(Object(a),!0).forEach((function(t){QW(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):XW(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function QW(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var eU,tU=["text","password","email","number","url","tel","search","range","color","date","time","datetime","datetime-local","month","week"],aU=Sc(sa(ZW(ZW(ZW(ZW(ZW(ZW({},XP),YR),ER),RR),qW),{},{list:wc(Yo),max:wc(Go),min:wc(Go),noWheel:wc(To,!1),step:wc(Go),type:wc(Yo,"text",(function(e){return ll(tU,e)}))})),$n),nU=i["default"].extend({name:$n,mixins:[JB,ZP,PR,$R,NR,JW,IW,KW],props:aU,computed:{localType:function(){var e=this.type;return ll(tU,e)?e:"text"},computedAttrs:function(){var e=this.localType,t=this.name,a=this.form,n=this.disabled,r=this.placeholder,i=this.required,o=this.min,s=this.max,l=this.step;return{id:this.safeId(),name:t,form:a,type:e,disabled:n,placeholder:r,required:i,autocomplete:this.autocomplete||null,readonly:this.readonly||this.plaintext,min:o,max:s,step:l,list:"password"!==e?this.list:null,"aria-required":i?"true":null,"aria-invalid":this.computedAriaInvalid}},computedListeners:function(){return ZW(ZW({},this.bvListeners),{},{input:this.onInput,change:this.onChange,blur:this.onBlur})}},watch:{noWheel:function(e){this.setWheelStopper(e)}},mounted:function(){this.setWheelStopper(this.noWheel)},deactivated:function(){this.setWheelStopper(!1)},activated:function(){this.setWheelStopper(this.noWheel)},beforeDestroy:function(){this.setWheelStopper(!1)},methods:{setWheelStopper:function(e){var t=this.$el;Ic(e,t,"focus",this.onWheelFocus),Ic(e,t,"blur",this.onWheelBlur),e||Pc(document,"wheel",this.stopWheel)},onWheelFocus:function(){Yc(document,"wheel",this.stopWheel)},onWheelBlur:function(){Pc(document,"wheel",this.stopWheel)},stopWheel:function(e){Fc(e,{propagation:!1}),cc(this.$el)}},render:function(e){return e("input",{class:this.computedClass,attrs:this.computedAttrs,domProps:{value:this.localValue},on:this.computedListeners,ref:"input"})}}),rU=Pa({components:{BFormInput:nU,BInput:nU}}),iU=Sc(zN,Wn),oU=i["default"].extend({name:Wn,mixins:[LN],provide:function(){return{bvRadioGroup:this}},props:iU,computed:{isRadioGroup:function(){return!0}}}),sU=Pa({components:{BFormRadio:dN,BRadio:dN,BFormRadioGroup:oU,BRadioGroup:oU}});function lU(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function cU(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?lU(Object(a),!0).forEach((function(t){uU(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):lU(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function uU(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var dU=Dc("value",{type:Go,event:Li}),hU=dU.mixin,fU=dU.props,pU=dU.prop,mU=dU.event,vU=3,_U=5,gU=function(e){return mu(vU,fl(e,_U))},yU=function(e,t,a){return mu(pu(e,a),t)},bU=i["default"].extend({name:ci,mixins:[xc],props:{disabled:wc(To,!1),focused:wc(To,!1),hasClear:wc(To,!1),rating:wc(jo,0),readonly:wc(To,!1),star:wc(jo,0),variant:wc(Yo)},methods:{onClick:function(e){this.disabled||this.readonly||(Fc(e,{propagation:!1}),this.$emit(ho,this.star))}},render:function(e){var t=this.rating,a=this.star,n=this.focused,r=this.hasClear,i=this.variant,o=this.disabled,s=this.readonly,l=r?0:1,c=t>=a?"full":t>=a-.5?"half":"empty",u={variant:i,disabled:o,readonly:s};return e("span",{staticClass:"b-rating-star",class:{focused:n&&t===a||!fl(t)&&a===l,"b-rating-star-empty":"empty"===c,"b-rating-star-half":"half"===c,"b-rating-star-full":"full"===c},attrs:{tabindex:o||s?null:"-1"},on:{click:this.onClick}},[e("span",{staticClass:"b-rating-icon"},[this.normalizeSlot(c,u)])])}}),MU=Sc(sa(cU(cU(cU(cU(cU({},XP),fU),ia(YR,["required","autofocus"])),ER),{},{color:wc(Yo),iconClear:wc(Yo,"x"),iconEmpty:wc(Yo,"star"),iconFull:wc(Yo,"star-fill"),iconHalf:wc(Yo,"star-half"),inline:wc(To,!1),locale:wc(Eo),noBorder:wc(To,!1),precision:wc(Go),readonly:wc(To,!1),showClear:wc(To,!1),showValue:wc(To,!1),showValueMax:wc(To,!1),stars:wc(Go,_U,(function(e){return fl(e)>=vU})),variant:wc(Yo)})),Un),zU=i["default"].extend({name:Un,components:{BIconStar:hC,BIconStarHalf:pC,BIconStarFill:fC,BIconX:sB},mixins:[ZP,hU,$R],props:MU,data:function(){var e=pl(this[pU],null),t=gU(this.stars);return{localValue:Tt(e)?null:yU(e,0,t),hasFocus:!1}},computed:{computedStars:function(){return gU(this.stars)},computedRating:function(){var e=pl(this.localValue,0),t=fl(this.precision,3);return yU(pl(e.toFixed(t)),0,this.computedStars)},computedLocale:function(){var e=cl(this.locale).filter(ya),t=new Intl.NumberFormat(e);return t.resolvedOptions().locale},isInteractive:function(){return!this.disabled&&!this.readonly},isRTL:function(){return KP(this.computedLocale)},formattedRating:function(){var e=fl(this.precision),t=this.showValueMax,a=this.computedLocale,n={notation:"standard",minimumFractionDigits:isNaN(e)?0:e,maximumFractionDigits:isNaN(e)?3:e},r=this.computedStars.toLocaleString(a),i=this.localValue;return i=Tt(i)?t?"-":"":i.toLocaleString(a,n),t?"".concat(i,"/").concat(r):i}},watch:(eU={},uU(eU,pU,(function(e,t){if(e!==t){var a=pl(e,null);this.localValue=Tt(a)?null:yU(a,0,this.computedStars)}})),uU(eU,"localValue",(function(e,t){e!==t&&e!==(this.value||0)&&this.$emit(mU,e||null)})),uU(eU,"disabled",(function(e){e&&(this.hasFocus=!1,this.blur())})),eU),methods:{focus:function(){this.disabled||lc(this.$el)},blur:function(){this.disabled||cc(this.$el)},onKeydown:function(e){var t=e.keyCode;if(this.isInteractive&&ll([jB,VB,YB,IB],t)){Fc(e,{propagation:!1});var a=fl(this.localValue,0),n=this.showClear?0:1,r=this.computedStars,i=this.isRTL?-1:1;t===jB?this.localValue=yU(a-i,n,r)||null:t===YB?this.localValue=yU(a+i,n,r):t===VB?this.localValue=yU(a-1,n,r)||null:t===IB&&(this.localValue=yU(a+1,n,r))}},onSelected:function(e){this.isInteractive&&(this.localValue=e)},onFocus:function(e){this.hasFocus=!!this.isInteractive&&"focus"===e.type},renderIcon:function(e){return this.$createElement(kB,{props:{icon:e,variant:this.disabled||this.color?null:this.variant||null}})},iconEmptyFn:function(){return this.renderIcon(this.iconEmpty)},iconHalfFn:function(){return this.renderIcon(this.iconHalf)},iconFullFn:function(){return this.renderIcon(this.iconFull)},iconClearFn:function(){return this.$createElement(kB,{props:{icon:this.iconClear}})}},render:function(e){var t=this,a=this.disabled,n=this.readonly,r=this.name,i=this.form,o=this.inline,s=this.variant,l=this.color,c=this.noBorder,u=this.hasFocus,d=this.computedRating,h=this.computedStars,f=this.formattedRating,p=this.showClear,m=this.isRTL,v=this.isInteractive,_=this.$scopedSlots,g=[];if(p&&!a&&!n){var y=e("span",{staticClass:"b-rating-icon"},[(_[gs]||this.iconClearFn)()]);g.push(e("span",{staticClass:"b-rating-star b-rating-star-clear flex-grow-1",class:{focused:u&&0===d},attrs:{tabindex:v?"-1":null},on:{click:function(){return t.onSelected(null)}},key:"clear"},[y]))}for(var b=0;b<h;b++){var M=b+1;g.push(e(bU,{staticClass:"flex-grow-1",style:l&&!a?{color:l}:{},props:{rating:d,star:M,variant:a?null:s||null,disabled:a,readonly:n,focused:u,hasClear:p},on:{selected:this.onSelected},scopedSlots:{empty:_[ys]||this.iconEmptyFn,half:_[Ms]||this.iconHalfFn,full:_[bs]||this.iconFullFn},key:b}))}return r&&g.push(e("input",{attrs:{type:"hidden",value:Tt(this.localValue)?"":d,name:r,form:i||null},key:"hidden"})),this.showValue&&g.push(e("b",{staticClass:"b-rating-value flex-grow-1",attrs:{"aria-hidden":"true"},key:"value"},zl(f))),e("output",{staticClass:"b-rating form-control align-items-center",class:[{"d-inline-flex":o,"d-flex":!o,"border-0":c,disabled:a,readonly:!a&&n},this.sizeFormClass],attrs:{id:this.safeId(),dir:m?"rtl":"ltr",tabindex:a?null:"0",disabled:a,role:"slider","aria-disabled":a?"true":null,"aria-readonly":!a&&n?"true":null,"aria-live":"off","aria-valuemin":p?"0":"1","aria-valuemax":zl(h),"aria-valuenow":d?zl(d):null},on:{keydown:this.onKeydown,focus:this.onFocus,blur:this.onFocus}},g)}}),LU=Pa({components:{BFormRating:zU,BRating:zU}}),wU=Dc("value"),HU=wU.mixin,kU=wU.props,AU=wU.prop,SU=wU.event;function VU(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function OU(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?VU(Object(a),!0).forEach((function(t){TU(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):VU(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function TU(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var DU=Sc(sa(OU(OU({},vR),{},{labelField:wc(Yo,"label"),optionsField:wc(Yo,"options")})),"formOptions"),CU=i["default"].extend({mixins:[_R],props:DU,methods:{normalizeOption:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null;if(Ft(e)){var a=Ma(e,this.valueField),n=Ma(e,this.textField),r=Ma(e,this.optionsField,null);return Tt(r)?{value:Ot(a)?t||n:a,text:String(Ot(n)?t:n),html:Ma(e,this.htmlField),disabled:Boolean(Ma(e,this.disabledField))}:{label:String(Ma(e,this.labelField)||n),options:this.normalizeOptions(r)}}return{value:t||e,text:String(e),disabled:!1}}}}),jU=Sc({disabled:wc(To,!1),value:wc(Vo,void 0,!0)},Jn),xU=i["default"].extend({name:Jn,functional:!0,props:jU,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.value,o=a.disabled;return e("option",Ja(n,{attrs:{disabled:o},domProps:{value:i}}),r)}});function BU(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function YU(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?BU(Object(a),!0).forEach((function(t){PU(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):BU(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function PU(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var IU=Sc(sa(YU(YU({},vR),{},{label:wc(Yo,void 0,!0)})),Kn),FU=i["default"].extend({name:Kn,mixins:[xc,_R],props:IU,render:function(e){var t=this.label,a=this.formOptions.map((function(t,a){var n=t.value,r=t.text,i=t.html,o=t.disabled;return e(xU,{attrs:{value:n,disabled:o},domProps:UY(i,r),key:"option_".concat(a)})}));return e("optgroup",{attrs:{label:t}},[this.normalizeSlot(fs),a,this.normalizeSlot()])}});function EU(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function $U(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?EU(Object(a),!0).forEach((function(t){RU(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):EU(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function RU(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var NU,WU=Sc(sa($U($U($U($U($U($U($U({},XP),kU),YR),IR),ER),RR),{},{ariaInvalid:wc(No,!1),multiple:wc(To,!1),selectSize:wc(jo,0)})),qn),UU=i["default"].extend({name:qn,mixins:[ZP,HU,PR,$R,NR,FR,CU,xc],props:WU,data:function(){return{localValue:this[AU]}},computed:{computedSelectSize:function(){return this.plain||0!==this.selectSize?this.selectSize:null},inputClass:function(){return[this.plain?"form-control":"custom-select",this.size&&this.plain?"form-control-".concat(this.size):null,this.size&&!this.plain?"custom-select-".concat(this.size):null,this.stateClass]}},watch:{value:function(e){this.localValue=e},localValue:function(){this.$emit(SU,this.localValue)}},methods:{focus:function(){lc(this.$refs.input)},blur:function(){cc(this.$refs.input)},onChange:function(e){var t=this,a=e.target,n=sl(a.options).filter((function(e){return e.selected})).map((function(e){return"_value"in e?e._value:e.value}));this.localValue=a.multiple?n:n[0],this.$nextTick((function(){t.$emit(Li,t.localValue)}))}},render:function(e){var t=this.name,a=this.disabled,n=this.required,r=this.computedSelectSize,i=this.localValue,o=this.formOptions.map((function(t,a){var n=t.value,r=t.label,i=t.options,o=t.disabled,s="option_".concat(a);return Pt(i)?e(FU,{props:{label:r,options:i},key:s}):e(xU,{props:{value:n,disabled:o},domProps:UY(t.html,t.text),key:s})}));return e("select",{class:this.inputClass,attrs:{id:this.safeId(),name:t,form:this.form||null,multiple:this.multiple||null,size:r,disabled:a,required:n,"aria-required":n?"true":null,"aria-invalid":this.computedAriaInvalid},on:{change:this.onChange},directives:[{name:"model",value:i}],ref:"input"},[this.normalizeSlot(fs),o,this.normalizeSlot()])}}),GU=Pa({components:{BFormSelect:UU,BFormSelectOption:xU,BFormSelectOptionGroup:FU,BSelect:UU,BSelectOption:xU,BSelectOptionGroup:FU}});function qU(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function JU(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?qU(Object(a),!0).forEach((function(t){KU(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):qU(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function KU(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var XU=Dc("value",{type:$o}),ZU=XU.mixin,QU=XU.props,eG=XU.prop,tG=XU.event,aG=1,nG=100,rG=1,iG=500,oG=100,sG=10,lG=4,cG=[IB,VB,CB,OB,BB,xB],uG=Sc(sa(JU(JU(JU(JU(JU(JU({},XP),QU),ia(YR,["required","autofocus"])),ER),RR),{},{ariaControls:wc(Yo),ariaLabel:wc(Yo),formatterFn:wc(Co),inline:wc(To,!1),labelDecrement:wc(Yo,"Decrement"),labelIncrement:wc(Yo,"Increment"),locale:wc(Eo),max:wc(Go,nG),min:wc(Go,aG),placeholder:wc(Yo),readonly:wc(To,!1),repeatDelay:wc(Go,iG),repeatInterval:wc(Go,oG),repeatStepMultiplier:wc(Go,lG),repeatThreshold:wc(Go,sG),step:wc(Go,rG),vertical:wc(To,!1),wrap:wc(To,!1)})),Xn),dG=i["default"].extend({name:Xn,mixins:[UB,ZP,ZU,$R,NR,xc],inheritAttrs:!1,props:uG,data:function(){return{localValue:pl(this[eG],null),hasFocus:!1}},computed:{spinId:function(){return this.safeId()},computedInline:function(){return this.inline&&!this.vertical},computedReadonly:function(){return this.readonly&&!this.disabled},computedRequired:function(){return this.required&&!this.computedReadonly&&!this.disabled},computedStep:function(){return pl(this.step,rG)},computedMin:function(){return pl(this.min,aG)},computedMax:function(){var e=pl(this.max,nG),t=this.computedStep,a=this.computedMin;return gu((e-a)/t)*t+a},computedDelay:function(){var e=fl(this.repeatDelay,0);return e>0?e:iG},computedInterval:function(){var e=fl(this.repeatInterval,0);return e>0?e:oG},computedThreshold:function(){return mu(fl(this.repeatThreshold,sG),1)},computedStepMultiplier:function(){return mu(fl(this.repeatStepMultiplier,lG),1)},computedPrecision:function(){var e=this.computedStep;return gu(e)===e?0:(e.toString().split(".")[1]||"").length},computedMultiplier:function(){return yu(10,this.computedPrecision||0)},valueAsFixed:function(){var e=this.localValue;return Tt(e)?"":e.toFixed(this.computedPrecision)},computedLocale:function(){var e=cl(this.locale).filter(ya),t=new Intl.NumberFormat(e);return t.resolvedOptions().locale},computedRTL:function(){return KP(this.computedLocale)},defaultFormatter:function(){var e=this.computedPrecision,t=new Intl.NumberFormat(this.computedLocale,{style:"decimal",useGrouping:!1,minimumIntegerDigits:1,minimumFractionDigits:e,maximumFractionDigits:e,notation:"standard"});return t.format},computedFormatter:function(){var e=this.formatterFn;return Oc(e)?e:this.defaultFormatter},computedAttrs:function(){return JU(JU({},this.bvAttrs),{},{role:"group",lang:this.computedLocale,tabindex:this.disabled?null:"-1",title:this.ariaLabel})},computedSpinAttrs:function(){var e=this.spinId,t=this.localValue,a=this.computedRequired,n=this.disabled,r=this.state,i=this.computedFormatter,o=!Tt(t);return JU(JU({dir:this.computedRTL?"rtl":"ltr"},this.bvAttrs),{},{id:e,role:"spinbutton",tabindex:n?null:"0","aria-live":"off","aria-label":this.ariaLabel||null,"aria-controls":this.ariaControls||null,"aria-invalid":!1===r||!o&&a?"true":null,"aria-required":a?"true":null,"aria-valuemin":zl(this.computedMin),"aria-valuemax":zl(this.computedMax),"aria-valuenow":o?t:null,"aria-valuetext":o?i(t):null})}},watch:(NU={},KU(NU,eG,(function(e){this.localValue=pl(e,null)})),KU(NU,"localValue",(function(e){this.$emit(tG,e)})),KU(NU,"disabled",(function(e){e&&this.clearRepeat()})),KU(NU,"readonly",(function(e){e&&this.clearRepeat()})),NU),created:function(){this.$_autoDelayTimer=null,this.$_autoRepeatTimer=null,this.$_keyIsDown=!1},beforeDestroy:function(){this.clearRepeat()},deactivated:function(){this.clearRepeat()},methods:{focus:function(){this.disabled||lc(this.$refs.spinner)},blur:function(){this.disabled||cc(this.$refs.spinner)},emitChange:function(){this.$emit(Li,this.localValue)},stepValue:function(e){var t=this.localValue;if(!this.disabled&&!Tt(t)){var a=this.computedStep*e,n=this.computedMin,r=this.computedMax,i=this.computedMultiplier,o=this.wrap;t=bu((t-n)/a)*a+n+a,t=bu(t*i)/i,this.localValue=t>r?o?n:r:t<n?o?r:n:t}},onFocusBlur:function(e){this.hasFocus=!this.disabled&&"focus"===e.type},stepUp:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:1,t=this.localValue;Tt(t)?this.localValue=this.computedMin:this.stepValue(1*e)},stepDown:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:1,t=this.localValue;Tt(t)?this.localValue=this.wrap?this.computedMax:this.computedMin:this.stepValue(-1*e)},onKeydown:function(e){var t=e.keyCode,a=e.altKey,n=e.ctrlKey,r=e.metaKey;if(!(this.disabled||this.readonly||a||n||r)&&ll(cG,t)){if(Fc(e,{propagation:!1}),this.$_keyIsDown)return;this.resetTimers(),ll([IB,VB],t)?(this.$_keyIsDown=!0,t===IB?this.handleStepRepeat(e,this.stepUp):t===VB&&this.handleStepRepeat(e,this.stepDown)):t===BB?this.stepUp(this.computedStepMultiplier):t===xB?this.stepDown(this.computedStepMultiplier):t===CB?this.localValue=this.computedMin:t===OB&&(this.localValue=this.computedMax)}},onKeyup:function(e){var t=e.keyCode,a=e.altKey,n=e.ctrlKey,r=e.metaKey;this.disabled||this.readonly||a||n||r||ll(cG,t)&&(Fc(e,{propagation:!1}),this.resetTimers(),this.$_keyIsDown=!1,this.emitChange())},handleStepRepeat:function(e,t){var a=this,n=e||{},r=n.type,i=n.button;if(!this.disabled&&!this.readonly){if("mousedown"===r&&i)return;this.resetTimers(),t(1);var o=this.computedThreshold,s=this.computedStepMultiplier,l=this.computedDelay,c=this.computedInterval;this.$_autoDelayTimer=setTimeout((function(){var e=0;a.$_autoRepeatTimer=setInterval((function(){t(e<o?1:s),e++}),c)}),l)}},onMouseup:function(e){var t=e||{},a=t.type,n=t.button;"mouseup"===a&&n||(Fc(e,{propagation:!1}),this.resetTimers(),this.setMouseup(!1),this.emitChange())},setMouseup:function(e){try{Ic(e,document.body,"mouseup",this.onMouseup,!1),Ic(e,document.body,"touchend",this.onMouseup,!1)}catch(t){}},resetTimers:function(){clearTimeout(this.$_autoDelayTimer),clearInterval(this.$_autoRepeatTimer),this.$_autoDelayTimer=null,this.$_autoRepeatTimer=null},clearRepeat:function(){this.resetTimers(),this.setMouseup(!1),this.$_keyIsDown=!1}},render:function(e){var t=this,a=this.spinId,n=this.localValue,r=this.computedInline,i=this.computedReadonly,o=this.vertical,s=this.disabled,l=this.computedFormatter,c=!Tt(n),u=function(n,r,l,c,u,d,h){var f=e(l,{props:{scale:t.hasFocus?1.5:1.25},attrs:{"aria-hidden":"true"}}),p={hasFocus:t.hasFocus},m=function(e){s||i||(Fc(e,{propagation:!1}),t.setMouseup(!0),lc(e.currentTarget),t.handleStepRepeat(e,n))};return e("button",{staticClass:"btn btn-sm border-0 rounded-0",class:{"py-0":!o},attrs:{tabindex:"-1",type:"button",disabled:s||i||d,"aria-disabled":s||i||d?"true":null,"aria-controls":a,"aria-label":r||null,"aria-keyshortcuts":u||null},on:{mousedown:m,touchstart:m},key:c||null,ref:c},[t.normalizeSlot(h,p)||f])},d=u(this.stepUp,this.labelIncrement,bO,"inc","ArrowUp",!1,Ls),h=u(this.stepDown,this.labelDecrement,ub,"dec","ArrowDown",!1,rs),f=e();this.name&&!s&&(f=e("input",{attrs:{type:"hidden",name:this.name,form:this.form||null,value:this.valueAsFixed},key:"hidden"}));var p=e("output",{staticClass:"flex-grow-1",class:{"d-flex":o,"align-self-center":!o,"align-items-center":o,"border-top":o,"border-bottom":o,"border-left":!o,"border-right":!o},attrs:this.computedSpinAttrs,key:"output",ref:"spinner"},[e("bdi",c?l(n):this.placeholder||"")]);return e("div",{staticClass:"b-form-spinbutton form-control",class:[{disabled:s,readonly:i,focus:this.hasFocus,"d-inline-flex":r||o,"d-flex":!r&&!o,"align-items-stretch":!o,"flex-column":o},this.sizeFormClass,this.stateClass],attrs:this.computedAttrs,on:{keydown:this.onKeydown,keyup:this.onKeyup,"!focus":this.onFocusBlur,"!blur":this.onFocusBlur}},o?[d,f,p,h]:[h,f,p,d])}}),hG=Pa({components:{BFormSpinbutton:dG,BSpinbutton:dG}});function fG(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function pG(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?fG(Object(a),!0).forEach((function(t){mG(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):fG(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function mG(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var vG,_G=Sc(sa(pG(pG({},XP),{},{disabled:wc(To,!1),noRemove:wc(To,!1),pill:wc(To,!1),removeLabel:wc(Yo,"Remove tag"),tag:wc(Yo,"span"),title:wc(Yo),variant:wc(Yo,"secondary")})),Zn),gG=i["default"].extend({name:Zn,mixins:[ZP,xc],props:_G,methods:{onRemove:function(e){var t=e.type,a=e.keyCode;this.disabled||"click"!==t&&("keydown"!==t||a!==SB)||this.$emit(no)}},render:function(e){var t=this.title,a=this.tag,n=this.variant,r=this.pill,i=this.disabled,o=this.safeId(),s=this.safeId("_taglabel_"),l=e();this.noRemove||i||(l=e(Uc,{staticClass:"b-form-tag-remove",props:{ariaLabel:this.removeLabel},attrs:{"aria-controls":o,"aria-describedby":s,"aria-keyshortcuts":"Delete"},on:{click:this.onRemove,keydown:this.onRemove}}));var c=e("span",{staticClass:"b-form-tag-content flex-grow-1 text-truncate",attrs:{id:s}},this.normalizeSlot()||t);return e(RY,{staticClass:"b-form-tag d-inline-flex align-items-baseline mw-100",class:{disabled:i},props:{tag:a,variant:n,pill:r},attrs:{id:o,title:t||null,"aria-labelledby":s}},[c,l])}});function yG(e){return LG(e)||zG(e)||MG(e)||bG()}function bG(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function MG(e,t){if(e){if("string"===typeof e)return wG(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?wG(e,t):void 0}}function zG(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function LG(e){if(Array.isArray(e))return wG(e)}function wG(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}function HG(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function kG(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?HG(Object(a),!0).forEach((function(t){AG(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):HG(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function AG(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var SG=Dc("value",{type:Oo,defaultValue:[]}),VG=SG.mixin,OG=SG.props,TG=SG.prop,DG=SG.event,CG=["text","email","tel","url","number"],jG=[".b-form-tag","button","input","select"].join(" "),xG=function(e){return Ml(e).replace(Ue,"\\s")},BG=function(e){return cl(e).map((function(e){return wl(zl(e))})).filter((function(e,t,a){return e.length>0&&a.indexOf(e)===t}))},YG=function(e){return xt(e)?e:$t(e)&&e.target.value||""},PG=function(){return{all:[],valid:[],invalid:[],duplicate:[]}},IG=Sc(sa(kG(kG(kG(kG(kG(kG({},XP),OG),YR),ER),RR),{},{addButtonText:wc(Yo,"Add"),addButtonVariant:wc(Yo,"outline-secondary"),addOnChange:wc(To,!1),duplicateTagText:wc(Yo,"Duplicate tag(s)"),feedbackAriaLive:wc(Yo,"assertive"),ignoreInputFocusSelector:wc(Eo,jG),inputAttrs:wc(xo,{}),inputClass:wc(Fo),inputId:wc(Yo),inputType:wc(Yo,"text",(function(e){return ll(CG,e)})),invalidTagText:wc(Yo,"Invalid tag(s)"),limit:wc(jo),limitTagsText:wc(Yo,"Tag limit reached"),noAddOnEnter:wc(To,!1),noOuterFocus:wc(To,!1),noTagRemove:wc(To,!1),placeholder:wc(Yo,"Add tag..."),removeOnDelete:wc(To,!1),separator:wc(Eo),tagClass:wc(Fo),tagPills:wc(To,!1),tagRemoveLabel:wc(Yo,"Remove tag"),tagRemovedLabel:wc(Yo,"Tag removed"),tagValidator:wc(Co),tagVariant:wc(Yo,"secondary")})),Qn),FG=i["default"].extend({name:Qn,mixins:[JB,ZP,VG,PR,$R,NR,xc],props:IG,data:function(){return{hasFocus:!1,newTag:"",tags:[],removedTags:[],tagsState:PG(),focusState:null}},computed:{computedInputId:function(){return this.inputId||this.safeId("__input__")},computedInputType:function(){return ll(CG,this.inputType)?this.inputType:"text"},computedInputAttrs:function(){var e=this.disabled,t=this.form;return kG(kG({},this.inputAttrs),{},{id:this.computedInputId,value:this.newTag,disabled:e,form:t})},computedInputHandlers:function(){return kG(kG({},ia(this.bvListeners,[Ii,Fi])),{},{blur:this.onInputBlur,change:this.onInputChange,focus:this.onInputFocus,input:this.onInputInput,keydown:this.onInputKeydown,reset:this.reset})},computedSeparator:function(){return cl(this.separator).filter(xt).filter(ya).join("")},computedSeparatorRegExp:function(){var e=this.computedSeparator;return e?new RegExp("[".concat(xG(e),"]+")):null},computedJoiner:function(){var e=this.computedSeparator.charAt(0);return" "!==e?"".concat(e," "):e},computeIgnoreInputFocusSelector:function(){return cl(this.ignoreInputFocusSelector).filter(ya).join(",").trim()},disableAddButton:function(){var e=this,t=wl(this.newTag);return""===t||!this.splitTags(t).some((function(t){return!ll(e.tags,t)&&e.validateTag(t)}))},duplicateTags:function(){return this.tagsState.duplicate},hasDuplicateTags:function(){return this.duplicateTags.length>0},invalidTags:function(){return this.tagsState.invalid},hasInvalidTags:function(){return this.invalidTags.length>0},isLimitReached:function(){var e=this.limit;return Bt(e)&&e>=0&&this.tags.length>=e}},watch:(vG={},AG(vG,TG,(function(e){this.tags=BG(e)})),AG(vG,"tags",(function(e,t){EB(e,this[TG])||this.$emit(DG,e),EB(e,t)||(e=cl(e).filter(ya),t=cl(t).filter(ya),this.removedTags=t.filter((function(t){return!ll(e,t)})))})),AG(vG,"tagsState",(function(e,t){EB(e,t)||this.$emit(go,e.valid,e.invalid,e.duplicate)})),vG),created:function(){this.tags=BG(this[TG])},mounted:function(){var e=Rl("form",this.$el);e&&Yc(e,"reset",this.reset,Ao)},beforeDestroy:function(){var e=Rl("form",this.$el);e&&Pc(e,"reset",this.reset,Ao)},methods:{addTag:function(e){if(e=xt(e)?e:this.newTag,!this.disabled&&""!==wl(e)&&!this.isLimitReached){var t=this.parseTags(e);if(t.valid.length>0||0===t.all.length)if($l(this.getInput(),"select"))this.newTag="";else{var a=[].concat(yG(t.invalid),yG(t.duplicate));this.newTag=t.all.filter((function(e){return ll(a,e)})).join(this.computedJoiner).concat(a.length>0?this.computedJoiner.charAt(0):"")}t.valid.length>0&&(this.tags=cl(this.tags,t.valid)),this.tagsState=t,this.focus()}},removeTag:function(e){this.disabled||(this.tags=this.tags.filter((function(t){return t!==e})))},reset:function(){var e=this;this.newTag="",this.tags=[],this.$nextTick((function(){e.removedTags=[],e.tagsState=PG()}))},onInputInput:function(e){if(!(this.disabled||$t(e)&&e.target.composing)){var t=YG(e),a=this.computedSeparatorRegExp;this.newTag!==t&&(this.newTag=t),t=Ll(t),a&&a.test(t.slice(-1))?this.addTag():this.tagsState=""===t?PG():this.parseTags(t)}},onInputChange:function(e){if(!this.disabled&&this.addOnChange){var t=YG(e);this.newTag!==t&&(this.newTag=t),this.addTag()}},onInputKeydown:function(e){if(!this.disabled&&$t(e)){var t=e.keyCode,a=e.target.value||"";this.noAddOnEnter||t!==TB?!this.removeOnDelete||t!==AB&&t!==SB||""!==a||(Fc(e,{propagation:!1}),this.tags=this.tags.slice(0,-1)):(Fc(e,{propagation:!1}),this.addTag())}},onClick:function(e){var t=this,a=this.computeIgnoreInputFocusSelector;a&&Rl(a,e.target,!0)||this.$nextTick((function(){t.focus()}))},onInputFocus:function(e){var t=this;"out"!==this.focusState&&(this.focusState="in",this.$nextTick((function(){Ol((function(){t.hasFocus&&(t.$emit(Pi,e),t.focusState=null)}))})))},onInputBlur:function(e){var t=this;"in"!==this.focusState&&(this.focusState="out",this.$nextTick((function(){Ol((function(){t.hasFocus||(t.$emit(Mi,e),t.focusState=null)}))})))},onFocusin:function(e){this.hasFocus=!0,this.$emit(Ii,e)},onFocusout:function(e){this.hasFocus=!1,this.$emit(Fi,e)},handleAutofocus:function(){var e=this;this.$nextTick((function(){Ol((function(){e.autofocus&&e.focus()}))}))},focus:function(){this.disabled||lc(this.getInput())},blur:function(){this.disabled||cc(this.getInput())},splitTags:function(e){e=zl(e);var t=this.computedSeparatorRegExp;return(t?e.split(t):[e]).map(wl).filter(ya)},parseTags:function(e){var t=this,a=this.splitTags(e),n={all:a,valid:[],invalid:[],duplicate:[]};return a.forEach((function(e){ll(t.tags,e)||ll(n.valid,e)?ll(n.duplicate,e)||n.duplicate.push(e):t.validateTag(e)?n.valid.push(e):ll(n.invalid,e)||n.invalid.push(e)})),n},validateTag:function(e){var t=this.tagValidator;return!Oc(t)||t(e)},getInput:function(){return El("#".concat(bW(this.computedInputId)),this.$el)},defaultRender:function(e){var t=e.addButtonText,a=e.addButtonVariant,n=e.addTag,r=e.disableAddButton,i=e.disabled,o=e.duplicateTagText,s=e.inputAttrs,l=e.inputClass,c=e.inputHandlers,u=e.inputType,d=e.invalidTagText,h=e.isDuplicate,f=e.isInvalid,p=e.isLimitReached,m=e.limitTagsText,v=e.noTagRemove,_=e.placeholder,g=e.removeTag,y=e.tagClass,b=e.tagPills,M=e.tagRemoveLabel,z=e.tagVariant,L=e.tags,w=this.$createElement,H=L.map((function(e){return e=zl(e),w(gG,{class:y,props:{disabled:i,noRemove:v,pill:b,removeLabel:M,tag:"li",title:e,variant:z},on:{remove:function(){return g(e)}},key:"tags_".concat(e)},e)})),k=d&&f?this.safeId("__invalid_feedback__"):null,A=o&&h?this.safeId("__duplicate_feedback__"):null,S=m&&p?this.safeId("__limit_feedback__"):null,V=[s["aria-describedby"],k,A,S].filter(ya).join(" "),O=w("input",{staticClass:"b-form-tags-input w-100 flex-grow-1 p-0 m-0 bg-transparent border-0",class:l,style:{outline:0,minWidth:"5rem"},attrs:kG(kG({},s),{},{"aria-describedby":V||null,type:u,placeholder:_||null}),domProps:{value:s.value},on:c,directives:[{name:"model",value:s.value}],ref:"input"}),T=w(LY,{staticClass:"b-form-tags-button py-0",class:{invisible:r},style:{fontSize:"90%"},props:{disabled:r||p,variant:a},on:{click:function(){return n()}},ref:"button"},[this.normalizeSlot(Xo)||t]),D=this.safeId("__tag_list__"),C=w("li",{staticClass:"b-form-tags-field flex-grow-1",attrs:{role:"none","aria-live":"off","aria-controls":D},key:"tags_field"},[w("div",{staticClass:"d-flex",attrs:{role:"group"}},[O,T])]),j=w("ul",{staticClass:"b-form-tags-list list-unstyled mb-0 d-flex flex-wrap align-items-center",attrs:{id:D},key:"tags_list"},[H,C]),x=w();if(d||o||m){var B=this.feedbackAriaLive,Y=this.computedJoiner,P=w();k&&(P=w(VR,{props:{id:k,ariaLive:B,forceShow:!0},key:"tags_invalid_feedback"},[this.invalidTagText,": ",this.invalidTags.join(Y)]));var I=w();A&&(I=w(AR,{props:{id:A,ariaLive:B},key:"tags_duplicate_feedback"},[this.duplicateTagText,": ",this.duplicateTags.join(Y)]));var F=w();S&&(F=w(AR,{props:{id:S,ariaLive:B},key:"tags_limit_feedback"},[m])),x=w("div",{attrs:{"aria-live":"polite","aria-atomic":"true"},key:"tags_feedback"},[P,I,F])}return[j,x]}},render:function(e){var t=this.name,a=this.disabled,n=this.required,r=this.form,i=this.tags,o=this.computedInputId,s=this.hasFocus,l=this.noOuterFocus,c=kG({tags:i.slice(),inputAttrs:this.computedInputAttrs,inputType:this.computedInputType,inputHandlers:this.computedInputHandlers,removeTag:this.removeTag,addTag:this.addTag,reset:this.reset,inputId:o,isInvalid:this.hasInvalidTags,invalidTags:this.invalidTags.slice(),isDuplicate:this.hasDuplicateTags,duplicateTags:this.duplicateTags.slice(),isLimitReached:this.isLimitReached,disableAddButton:this.disableAddButton},ra(this.$props,["addButtonText","addButtonVariant","disabled","duplicateTagText","form","inputClass","invalidTagText","limit","limitTagsText","noTagRemove","placeholder","required","separator","size","state","tagClass","tagPills","tagRemoveLabel","tagVariant"])),u=this.normalizeSlot(is,c)||this.defaultRender(c),d=e("output",{staticClass:"sr-only",attrs:{id:this.safeId("__selected_tags__"),role:"status",for:o,"aria-live":s?"polite":"off","aria-atomic":"true","aria-relevant":"additions text"}},this.tags.join(", ")),h=e("div",{staticClass:"sr-only",attrs:{id:this.safeId("__removed_tags__"),role:"status","aria-live":s?"assertive":"off","aria-atomic":"true"}},this.removedTags.length>0?"(".concat(this.tagRemovedLabel,") ").concat(this.removedTags.join(", ")):""),f=e();if(t&&!a){var p=i.length>0;f=(p?i:[""]).map((function(a){return e("input",{class:{"sr-only":!p},attrs:{type:p?"hidden":"text",value:a,required:n,name:t,form:r},key:"tag_input_".concat(a)})}))}return e("div",{staticClass:"b-form-tags form-control h-auto",class:[{focus:s&&!l&&!a,disabled:a},this.sizeFormClass,this.stateClass],attrs:{id:this.safeId(),role:"group",tabindex:a||l?null:"-1","aria-describedby":this.safeId("__selected_tags__")},on:{click:this.onClick,focusin:this.onFocusin,focusout:this.onFocusout}},[d,h,u,f])}}),EG=Pa({components:{BFormTags:FG,BTags:FG,BFormTag:gG,BTag:gG}});function $G(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function RG(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?$G(Object(a),!0).forEach((function(t){NG(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):$G(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function NG(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var WG,UG=Sc(sa(RG(RG(RG(RG(RG(RG({},XP),YR),ER),RR),qW),{},{maxRows:wc(Go),noAutoShrink:wc(To,!1),noResize:wc(To,!1),rows:wc(Go,2),wrap:wc(Yo,"soft")})),tr),GG=i["default"].extend({name:tr,directives:{"b-visible":aF},mixins:[JB,ZP,qB,PR,$R,NR,JW,IW,KW],props:UG,data:function(){return{heightInPx:null}},computed:{computedStyle:function(){var e={resize:!this.computedRows||this.noResize?"none":null};return this.computedRows||(e.height=this.heightInPx,e.overflowY="scroll"),e},computedMinRows:function(){return mu(fl(this.rows,2),2)},computedMaxRows:function(){return mu(this.computedMinRows,fl(this.maxRows,0))},computedRows:function(){return this.computedMinRows===this.computedMaxRows?this.computedMinRows:null},computedAttrs:function(){var e=this.disabled,t=this.required;return{id:this.safeId(),name:this.name||null,form:this.form||null,disabled:e,placeholder:this.placeholder||null,required:t,autocomplete:this.autocomplete||null,readonly:this.readonly||this.plaintext,rows:this.computedRows,wrap:this.wrap||null,"aria-required":this.required?"true":null,"aria-invalid":this.computedAriaInvalid}},computedListeners:function(){return RG(RG({},this.bvListeners),{},{input:this.onInput,change:this.onChange,blur:this.onBlur})}},watch:{localValue:function(){this.setHeight()}},mounted:function(){this.setHeight()},methods:{visibleCallback:function(e){e&&this.$nextTick(this.setHeight)},setHeight:function(){var e=this;this.$nextTick((function(){Ol((function(){e.heightInPx=e.computeHeight()}))}))},computeHeight:function(){if(this.$isServer||!Tt(this.computedRows))return null;var e=this.$el;if(!Yl(e))return null;var t=nc(e),a=pl(t.lineHeight,1),n=pl(t.borderTopWidth,0)+pl(t.borderBottomWidth,0),r=pl(t.paddingTop,0)+pl(t.paddingBottom,0),i=n+r,o=a*this.computedMinRows+i,s=tc(e,"height")||t.height;Ql(e,"height","auto");var l=e.scrollHeight;Ql(e,"height",s);var c=mu((l-r)/a,2),u=pu(mu(c,this.computedMinRows),this.computedMaxRows),d=mu(_u(u*a+i),o);return this.noAutoShrink&&pl(s,0)>d?s:"".concat(d,"px")}},render:function(e){return e("textarea",{class:this.computedClass,style:this.computedStyle,directives:[{name:"b-visible",value:this.visibleCallback,modifiers:{640:!0}}],attrs:this.computedAttrs,domProps:{value:this.localValue},on:this.computedListeners,ref:"input"})}}),qG=Pa({components:{BFormTextarea:GG,BTextarea:GG}});function JG(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function KG(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?JG(Object(a),!0).forEach((function(t){XG(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):JG(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function XG(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}function ZG(e,t){return nq(e)||aq(e,t)||eq(e,t)||QG()}function QG(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function eq(e,t){if(e){if("string"===typeof e)return tq(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?tq(e,t):void 0}}function tq(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}function aq(e,t){var a=null==e?null:"undefined"!==typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=a){var n,r,i=[],o=!0,s=!1;try{for(a=a.call(e);!(o=(n=a.next()).done);o=!0)if(i.push(n.value),t&&i.length===t)break}catch(l){s=!0,r=l}finally{try{o||null==a["return"]||a["return"]()}finally{if(s)throw r}}return i}}function nq(e){if(Array.isArray(e))return e}var rq,iq=Dc("value",{type:Yo,defaultValue:""}),oq=iq.mixin,sq=iq.props,lq=iq.prop,cq=iq.event,uq="numeric",dq=function(e){return"00".concat(e||"").slice(-2)},hq=function(e){e=zl(e);var t=null,a=null,n=null;if(tt.test(e)){var r=e.split(":").map((function(e){return fl(e,null)})),i=ZG(r,3);t=i[0],a=i[1],n=i[2]}return{hours:Dt(t)?null:t,minutes:Dt(a)?null:a,seconds:Dt(n)?null:n,ampm:Dt(t)||t<12?0:1}},fq=function(e){var t=e.hours,a=e.minutes,n=e.seconds,r=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(Tt(t)||Tt(a)||r&&Tt(n))return"";var i=[t,a,r?n:0];return i.map(dq).join(":")},pq=Sc(sa(KG(KG(KG(KG({},XP),sq),ra(uG,["labelIncrement","labelDecrement"])),{},{ariaLabelledby:wc(Yo),disabled:wc(To,!1),footerTag:wc(Yo,"footer"),headerTag:wc(Yo,"header"),hidden:wc(To,!1),hideHeader:wc(To,!1),hour12:wc(To,null),labelAm:wc(Yo,"AM"),labelAmpm:wc(Yo,"AM/PM"),labelHours:wc(Yo,"Hours"),labelMinutes:wc(Yo,"Minutes"),labelNoTimeSelected:wc(Yo,"No time selected"),labelPm:wc(Yo,"PM"),labelSeconds:wc(Yo,"Seconds"),labelSelected:wc(Yo,"Selected time"),locale:wc(Eo),minutesStep:wc(Go,1),readonly:wc(To,!1),secondsStep:wc(Go,1),showSeconds:wc(To,!1)})),ai),mq=i["default"].extend({name:ai,mixins:[ZP,oq,xc],props:pq,data:function(){var e=hq(this[lq]||"");return{modelHours:e.hours,modelMinutes:e.minutes,modelSeconds:e.seconds,modelAmpm:e.ampm,isLive:!1}},computed:{computedHMS:function(){var e=this.modelHours,t=this.modelMinutes,a=this.modelSeconds;return fq({hours:e,minutes:t,seconds:a},this.showSeconds)},resolvedOptions:function(){var e=cl(this.locale).filter(ya),t={hour:uq,minute:uq,second:uq};Dt(this.hour12)||(t.hour12=!!this.hour12);var a=new Intl.DateTimeFormat(e,t),n=a.resolvedOptions(),r=n.hour12||!1,i=n.hourCycle||(r?"h12":"h23");return{locale:n.locale,hour12:r,hourCycle:i}},computedLocale:function(){return this.resolvedOptions.locale},computedLang:function(){return(this.computedLocale||"").replace(/-u-.*$/,"")},computedRTL:function(){return KP(this.computedLang)},computedHourCycle:function(){return this.resolvedOptions.hourCycle},is12Hour:function(){return!!this.resolvedOptions.hour12},context:function(){return{locale:this.computedLocale,isRTL:this.computedRTL,hourCycle:this.computedHourCycle,hour12:this.is12Hour,hours:this.modelHours,minutes:this.modelMinutes,seconds:this.showSeconds?this.modelSeconds:0,value:this.computedHMS,formatted:this.formattedTimeString}},valueId:function(){return this.safeId()||null},computedAriaLabelledby:function(){return[this.ariaLabelledby,this.valueId].filter(ya).join(" ")||null},timeFormatter:function(){var e={hour12:this.is12Hour,hourCycle:this.computedHourCycle,hour:uq,minute:uq,timeZone:"UTC"};return this.showSeconds&&(e.second=uq),YP(this.computedLocale,e)},numberFormatter:function(){var e=new Intl.NumberFormat(this.computedLocale,{style:"decimal",minimumIntegerDigits:2,minimumFractionDigits:0,maximumFractionDigits:0,notation:"standard"});return e.format},formattedTimeString:function(){var e=this.modelHours,t=this.modelMinutes,a=this.showSeconds&&this.modelSeconds||0;return this.computedHMS?this.timeFormatter(CP(Date.UTC(0,0,1,e,t,a))):this.labelNoTimeSelected||" "},spinScopedSlots:function(){var e=this.$createElement;return{increment:function(t){var a=t.hasFocus;return e(_g,{props:{scale:a?1.5:1.25},attrs:{"aria-hidden":"true"}})},decrement:function(t){var a=t.hasFocus;return e(_g,{props:{flipV:!0,scale:a?1.5:1.25},attrs:{"aria-hidden":"true"}})}}}},watch:(WG={},XG(WG,lq,(function(e,t){if(e!==t&&!EB(hq(e),hq(this.computedHMS))){var a=hq(e),n=a.hours,r=a.minutes,i=a.seconds,o=a.ampm;this.modelHours=n,this.modelMinutes=r,this.modelSeconds=i,this.modelAmpm=o}})),XG(WG,"computedHMS",(function(e,t){e!==t&&this.$emit(cq,e)})),XG(WG,"context",(function(e,t){EB(e,t)||this.$emit(Ai,e)})),XG(WG,"modelAmpm",(function(e,t){var a=this;if(e!==t){var n=Tt(this.modelHours)?0:this.modelHours;this.$nextTick((function(){0===e&&n>11?a.modelHours=n-12:1===e&&n<12&&(a.modelHours=n+12)}))}})),XG(WG,"modelHours",(function(e,t){e!==t&&(this.modelAmpm=e>11?1:0)})),WG),created:function(){var e=this;this.$nextTick((function(){e.$emit(Ai,e.context)}))},mounted:function(){this.setLive(!0)},activated:function(){this.setLive(!0)},deactivated:function(){this.setLive(!1)},beforeDestroy:function(){this.setLive(!1)},methods:{focus:function(){this.disabled||lc(this.$refs.spinners[0])},blur:function(){if(!this.disabled){var e=jl();Nl(this.$el,e)&&cc(e)}},formatHours:function(e){var t=this.computedHourCycle;return e=this.is12Hour&&e>12?e-12:e,e=0===e&&"h12"===t?12:0===e&&"h24"===t?24:12===e&&"h11"===t?0:e,this.numberFormatter(e)},formatMinutes:function(e){return this.numberFormatter(e)},formatSeconds:function(e){return this.numberFormatter(e)},formatAmpm:function(e){return 0===e?this.labelAm:1===e?this.labelPm:""},setHours:function(e){this.modelHours=e},setMinutes:function(e){this.modelMinutes=e},setSeconds:function(e){this.modelSeconds=e},setAmpm:function(e){this.modelAmpm=e},onSpinLeftRight:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.type,a=e.keyCode;if(!this.disabled&&"keydown"===t&&(a===jB||a===YB)){Fc(e);var n=this.$refs.spinners||[],r=n.map((function(e){return!!e.hasFocus})).indexOf(!0);r+=a===jB?-1:1,r=r>=n.length?0:r<0?n.length-1:r,lc(n[r])}},setLive:function(e){var t=this;e?this.$nextTick((function(){Ol((function(){t.isLive=!0}))})):this.isLive=!1}},render:function(e){var t=this;if(this.hidden)return e();var a=this.disabled,n=this.readonly,r=this.computedLocale,i=this.computedAriaLabelledby,o=this.labelIncrement,s=this.labelDecrement,l=this.valueId,c=this.focus,u=[],d=function(i,c,d){var h=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},f=t.safeId("_spinbutton_".concat(c,"_"))||null;return u.push(f),e(dG,{class:d,props:KG({id:f,placeholder:"--",vertical:!0,required:!0,disabled:a,readonly:n,locale:r,labelIncrement:o,labelDecrement:s,wrap:!0,ariaControls:l,min:0},h),scopedSlots:t.spinScopedSlots,on:{change:i},key:c,ref:"spinners",refInFor:!0})},h=function(){return e("div",{staticClass:"d-flex flex-column",class:{"text-muted":a||n},attrs:{"aria-hidden":"true"}},[e(yg,{props:{shiftV:4,scale:.5}}),e(yg,{props:{shiftV:-4,scale:.5}})])},f=[];f.push(d(this.setHours,"hours","b-time-hours",{value:this.modelHours,max:23,step:1,formatterFn:this.formatHours,ariaLabel:this.labelHours})),f.push(h()),f.push(d(this.setMinutes,"minutes","b-time-minutes",{value:this.modelMinutes,max:59,step:this.minutesStep||1,formatterFn:this.formatMinutes,ariaLabel:this.labelMinutes})),this.showSeconds&&(f.push(h()),f.push(d(this.setSeconds,"seconds","b-time-seconds",{value:this.modelSeconds,max:59,step:this.secondsStep||1,formatterFn:this.formatSeconds,ariaLabel:this.labelSeconds}))),this.is12Hour&&f.push(d(this.setAmpm,"ampm","b-time-ampm",{value:this.modelAmpm,max:1,formatterFn:this.formatAmpm,ariaLabel:this.labelAmpm,required:!1})),f=e("div",{staticClass:"d-flex align-items-center justify-content-center mx-auto",attrs:{role:"group",tabindex:a||n?null:"-1","aria-labelledby":i},on:{keydown:this.onSpinLeftRight,click:function(e){e.target===e.currentTarget&&c()}}},f);var p=e("output",{staticClass:"form-control form-control-sm text-center",class:{disabled:a||n},attrs:{id:l,role:"status",for:u.filter(ya).join(" ")||null,tabindex:a?null:"-1","aria-live":this.isLive?"polite":"off","aria-atomic":"true"},on:{click:c,focus:c}},[e("bdi",this.formattedTimeString),this.computedHMS?e("span",{staticClass:"sr-only"}," (".concat(this.labelSelected,") ")):""]),m=e(this.headerTag,{staticClass:"b-time-header",class:{"sr-only":this.hideHeader}},[p]),v=this.normalizeSlot(),_=v?e(this.footerTag,{staticClass:"b-time-footer"},v):e();return e("div",{staticClass:"b-time d-inline-flex flex-column text-center",attrs:{role:"group",lang:this.computedLang||null,"aria-labelledby":i||null,"aria-disabled":a?"true":null,"aria-readonly":n&&!a?"true":null}},[m,f,_])}});function vq(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function _q(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?vq(Object(a),!0).forEach((function(t){gq(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):vq(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function gq(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var yq=Dc("value",{type:Yo,defaultValue:""}),bq=yq.mixin,Mq=yq.props,zq=yq.prop,Lq=yq.event,wq=ia(pq,["hidden","id","value"]),Hq=ia(EN,["formattedValue","id","lang","rtl","value"]),kq=Sc(sa(_q(_q(_q(_q(_q({},XP),Mq),wq),Hq),{},{closeButtonVariant:wc(Yo,"outline-secondary"),labelCloseButton:wc(Yo,"Close"),labelNowButton:wc(Yo,"Select now"),labelResetButton:wc(Yo,"Reset"),noCloseButton:wc(To,!1),nowButton:wc(To,!1),nowButtonVariant:wc(Yo,"outline-primary"),resetButton:wc(To,!1),resetButtonVariant:wc(Yo,"outline-danger"),resetValue:wc(Wo)})),ar),Aq=i["default"].extend({name:ar,mixins:[ZP,bq],props:kq,data:function(){return{localHMS:this[zq]||"",localLocale:null,isRTL:!1,formattedValue:"",isVisible:!1}},computed:{computedLang:function(){return(this.localLocale||"").replace(/-u-.*$/i,"")||null}},watch:(rq={},gq(rq,zq,(function(e){this.localHMS=e||""})),gq(rq,"localHMS",(function(e){this.isVisible&&this.$emit(Lq,e||"")})),rq),methods:{focus:function(){this.disabled||lc(this.$refs.control)},blur:function(){this.disabled||cc(this.$refs.control)},setAndClose:function(e){var t=this;this.localHMS=e,this.$nextTick((function(){t.$refs.control.hide(!0)}))},onInput:function(e){this.localHMS!==e&&(this.localHMS=e)},onContext:function(e){var t=e.isRTL,a=e.locale,n=e.value,r=e.formatted;this.isRTL=t,this.localLocale=a,this.formattedValue=r,this.localHMS=n||"",this.$emit(Ai,e)},onNowButton:function(){var e=new Date,t=e.getHours(),a=e.getMinutes(),n=this.showSeconds?e.getSeconds():0,r=[t,a,n].map((function(e){return"00".concat(e||"").slice(-2)})).join(":");this.setAndClose(r)},onResetButton:function(){this.setAndClose(this.resetValue)},onCloseButton:function(){this.$refs.control.hide(!0)},onShow:function(){this.isVisible=!0},onShown:function(){var e=this;this.$nextTick((function(){lc(e.$refs.time),e.$emit(po)}))},onHidden:function(){this.isVisible=!1,this.$emit($i)},defaultButtonFn:function(e){var t=e.isHovered,a=e.hasFocus;return this.$createElement(t||a?Vg:Sg,{attrs:{"aria-hidden":"true"}})}},render:function(e){var t=this.localHMS,a=this.disabled,n=this.readonly,r=this.$props,i=Dt(this.placeholder)?this.labelNoTimeSelected:this.placeholder,o=[];if(this.nowButton){var s=this.labelNowButton;o.push(e(LY,{props:{size:"sm",disabled:a||n,variant:this.nowButtonVariant},attrs:{"aria-label":s||null},on:{click:this.onNowButton},key:"now-btn"},s))}if(this.resetButton){o.length>0&&o.push(e("span"," "));var l=this.labelResetButton;o.push(e(LY,{props:{size:"sm",disabled:a||n,variant:this.resetButtonVariant},attrs:{"aria-label":l||null},on:{click:this.onResetButton},key:"reset-btn"},l))}if(!this.noCloseButton){o.length>0&&o.push(e("span"," "));var c=this.labelCloseButton;o.push(e(LY,{props:{size:"sm",disabled:a,variant:this.closeButtonVariant},attrs:{"aria-label":c||null},on:{click:this.onCloseButton},key:"close-btn"},c))}o.length>0&&(o=[e("div",{staticClass:"b-form-date-controls d-flex flex-wrap",class:{"justify-content-between":o.length>1,"justify-content-end":o.length<2}},o)]);var u=e(mq,{staticClass:"b-form-time-control",props:_q(_q({},kc(wq,r)),{},{value:t,hidden:!this.isVisible}),on:{input:this.onInput,context:this.onContext},ref:"time"},o);return e($N,{staticClass:"b-form-timepicker",props:_q(_q({},kc(Hq,r)),{},{id:this.safeId(),value:t,formattedValue:t?this.formattedValue:"",placeholder:i,rtl:this.isRTL,lang:this.computedLang}),on:{show:this.onShow,shown:this.onShown,hidden:this.onHidden},scopedSlots:gq({},as,this.$scopedSlots[as]||this.defaultButtonFn),ref:"control"},[u])}}),Sq=Pa({components:{BFormTimepicker:Aq,BTimepicker:Aq}}),Vq=Pa({components:{BImg:jI,BImgLazy:uF}}),Oq=Sc({tag:wc(Yo,"div")},fr),Tq=i["default"].extend({name:fr,functional:!0,props:Oq,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.tag,Ja(n,{staticClass:"input-group-text"}),r)}}),Dq=Sc({append:wc(To,!1),id:wc(Yo),isText:wc(To,!1),tag:wc(Yo,"div")},ur),Cq=i["default"].extend({name:ur,functional:!0,props:Dq,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.append;return e(a.tag,Ja(n,{class:{"input-group-append":i,"input-group-prepend":!i},attrs:{id:a.id}}),a.isText?[e(Tq,r)]:r)}});function jq(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function xq(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?jq(Object(a),!0).forEach((function(t){Bq(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):jq(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function Bq(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Yq=Sc(ia(Dq,["append"]),dr),Pq=i["default"].extend({name:dr,functional:!0,props:Yq,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(Cq,Ja(n,{props:xq(xq({},a),{},{append:!0})}),r)}});function Iq(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function Fq(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?Iq(Object(a),!0).forEach((function(t){Eq(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):Iq(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function Eq(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var $q=Sc(ia(Dq,["append"]),hr),Rq=i["default"].extend({name:hr,functional:!0,props:$q,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(Cq,Ja(n,{props:Fq(Fq({},a),{},{append:!1})}),r)}});function Nq(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Wq=Sc({append:wc(Yo),appendHtml:wc(Yo),id:wc(Yo),prepend:wc(Yo),prependHtml:wc(Yo),size:wc(Yo),tag:wc(Yo,"div")},cr),Uq=i["default"].extend({name:cr,functional:!0,props:Wq,render:function(e,t){var a=t.props,n=t.data,r=t.slots,i=t.scopedSlots,o=a.prepend,s=a.prependHtml,l=a.append,c=a.appendHtml,u=a.size,d=i||{},h=r(),f={},p=e(),m=Cc(Gs,d,h);(m||o||s)&&(p=e(Rq,[m?jc(Gs,f,d,h):e(Tq,{domProps:UY(s,o)})]));var v=e(),_=Cc(Zo,d,h);return(_||l||c)&&(v=e(Pq,[_?jc(Zo,f,d,h):e(Tq,{domProps:UY(c,l)})])),e(a.tag,Ja(n,{staticClass:"input-group",class:Nq({},"input-group-".concat(u),u),attrs:{id:a.id||null,role:"group"}}),[p,jc(is,f,d,h),v])}}),Gq=Pa({components:{BInputGroup:Uq,BInputGroupAddon:Cq,BInputGroupPrepend:Rq,BInputGroupAppend:Pq,BInputGroupText:Tq}});function qq(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Jq=Sc({fluid:wc(No,!1),tag:wc(Yo,"div")},Hn),Kq=i["default"].extend({name:Hn,functional:!0,props:Jq,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.fluid;return e(a.tag,Ja(n,{class:qq({container:!(i||""===i),"container-fluid":!0===i||""===i},"container-".concat(i),i&&!0!==i)}),r)}});function Xq(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Zq=Sc({bgVariant:wc(Yo),borderVariant:wc(Yo),containerFluid:wc(No,!1),fluid:wc(To,!1),header:wc(Yo),headerHtml:wc(Yo),headerLevel:wc(Go,3),headerTag:wc(Yo,"h1"),lead:wc(Yo),leadHtml:wc(Yo),leadTag:wc(Yo,"p"),tag:wc(Yo,"div"),textVariant:wc(Yo)},pr),Qq=i["default"].extend({name:pr,functional:!0,props:Zq,render:function(e,t){var a,n=t.props,r=t.data,i=t.slots,o=t.scopedSlots,s=n.header,l=n.headerHtml,c=n.lead,u=n.leadHtml,d=n.textVariant,h=n.bgVariant,f=n.borderVariant,p=o||{},m=i(),v={},_=e(),g=Cc(vs,p,m);if(g||s||l){var y=n.headerLevel;_=e(n.headerTag,{class:Xq({},"display-".concat(y),y),domProps:g?{}:UY(l,s)},jc(vs,v,p,m))}var b=e(),M=Cc(As,p,m);(M||c||u)&&(b=e(n.leadTag,{staticClass:"lead",domProps:M?{}:UY(u,c)},jc(As,v,p,m)));var z=[_,b,jc(is,v,p,m)];return n.fluid&&(z=[e(Kq,{props:{fluid:n.containerFluid}},z)]),e(n.tag,Ja(r,{staticClass:"jumbotron",class:(a={"jumbotron-fluid":n.fluid},Xq(a,"text-".concat(d),d),Xq(a,"bg-".concat(h),h),Xq(a,"border-".concat(f),f),Xq(a,"border",f),a)}),z)}}),eJ=Pa({components:{BJumbotron:Qq}});function tJ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function aJ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?tJ(Object(a),!0).forEach((function(t){nJ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):tJ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function nJ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var rJ=["start","end","center"],iJ=uc((function(e,t){return t=wl(zl(t)),t?Hl(["row-cols",e,t].filter(ya).join("-")):null})),oJ=uc((function(e){return Hl(e.replace("cols",""))})),sJ=[],lJ=function(){var e=_c().reduce((function(e,t){return e[Lc(t,"cols")]=wc(Go),e}),Kt(null));return sJ=ea(e),Sc(sa(aJ(aJ({},e),{},{alignContent:wc(Yo,null,(function(e){return ll(cl(rJ,"between","around","stretch"),e)})),alignH:wc(Yo,null,(function(e){return ll(cl(rJ,"between","around"),e)})),alignV:wc(Yo,null,(function(e){return ll(cl(rJ,"baseline","stretch"),e)})),noGutters:wc(To,!1),tag:wc(Yo,"div")})),Pr)},cJ={name:Pr,functional:!0,get props(){return delete this.props,this.props=lJ(),this.props},render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.alignV,s=n.alignH,l=n.alignContent,c=[];return sJ.forEach((function(e){var t=iJ(oJ(e),n[e]);t&&c.push(t)})),c.push((a={"no-gutters":n.noGutters},nJ(a,"align-items-".concat(o),o),nJ(a,"justify-content-".concat(s),s),nJ(a,"align-content-".concat(l),l),a)),e(n.tag,Ja(r,{staticClass:"row",class:c}),i)}},uJ=Pa({components:{BContainer:Kq,BRow:cJ,BCol:VW,BFormRow:CR}}),dJ=Pa({components:{BLink:cY}});function hJ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var fJ=Sc({flush:wc(To,!1),horizontal:wc(No,!1),tag:wc(Yo,"div")},vr),pJ=i["default"].extend({name:vr,functional:!0,props:fJ,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=""===a.horizontal||a.horizontal;i=!a.flush&&i;var o={staticClass:"list-group",class:hJ({"list-group-flush":a.flush,"list-group-horizontal":!0===i},"list-group-horizontal-".concat(i),xt(i))};return e(a.tag,Ja(n,o),r)}});function mJ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function vJ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?mJ(Object(a),!0).forEach((function(t){_J(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):mJ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function _J(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var gJ=["a","router-link","button","b-link"],yJ=ia(lY,["event","routerTag"]);delete yJ.href.default,delete yJ.to.default;var bJ=Sc(sa(vJ(vJ({},yJ),{},{action:wc(To,!1),button:wc(To,!1),tag:wc(Yo,"div"),variant:wc(Yo)})),_r),MJ=i["default"].extend({name:_r,functional:!0,props:bJ,render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.button,s=n.variant,l=n.active,c=n.disabled,u=Yu(n),d=o?"button":u?cY:n.tag,h=!!(n.action||u||o||ll(gJ,n.tag)),f={},p={};return xl(d,"button")?(r.attrs&&r.attrs.type||(f.type="button"),n.disabled&&(f.disabled=!0)):p=kc(yJ,n),e(d,Ja(r,{attrs:f,props:p,staticClass:"list-group-item",class:(a={},_J(a,"list-group-item-".concat(s),s),_J(a,"list-group-item-action",h),_J(a,"active",l),_J(a,"disabled",c),a)}),i)}}),zJ=Pa({components:{BListGroup:pJ,BListGroupItem:MJ}});function LJ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var wJ=Sc({right:wc(To,!1),tag:wc(Yo,"div"),verticalAlign:wc(Yo,"top")},yr),HJ=i["default"].extend({name:yr,functional:!0,props:wJ,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.verticalAlign,o="top"===i?"start":"bottom"===i?"end":i;return e(a.tag,Ja(n,{staticClass:"media-aside",class:LJ({"media-aside-right":a.right},"align-self-".concat(o),o)}),r)}}),kJ=Sc({tag:wc(Yo,"div")},br),AJ=i["default"].extend({name:br,functional:!0,props:kJ,render:function(e,t){var a=t.props,n=t.data,r=t.children;return e(a.tag,Ja(n,{staticClass:"media-body"}),r)}}),SJ=Sc({noBody:wc(To,!1),rightAlign:wc(To,!1),tag:wc(Yo,"div"),verticalAlign:wc(Yo,"top")},gr),VJ=i["default"].extend({name:gr,functional:!0,props:SJ,render:function(e,t){var a=t.props,n=t.data,r=t.slots,i=t.scopedSlots,o=t.children,s=a.noBody,l=a.rightAlign,c=a.verticalAlign,u=s?o:[];if(!s){var d={},h=r(),f=i||{};u.push(e(AJ,jc(is,d,f,h)));var p=jc(Qo,d,f,h);p&&u[l?"push":"unshift"](e(HJ,{props:{right:l,verticalAlign:c}},p))}return e(a.tag,Ja(n,{staticClass:"media"}),u)}}),OJ=Pa({components:{BMedia:VJ,BMediaAside:HJ,BMediaBody:AJ}}),TJ="$_documentListeners",DJ=i["default"].extend({created:function(){this[TJ]={}},beforeDestroy:function(){var e=this;ea(this[TJ]||{}).forEach((function(t){e[TJ][t].forEach((function(a){e.listenOffDocument(t,a)}))})),this[TJ]=null},methods:{registerDocumentListener:function(e,t){this[TJ]&&(this[TJ][e]=this[TJ][e]||[],ll(this[TJ][e],t)||this[TJ][e].push(t))},unregisterDocumentListener:function(e,t){this[TJ]&&this[TJ][e]&&(this[TJ][e]=this[TJ][e].filter((function(e){return e!==t})))},listenDocument:function(e,t,a){e?this.listenOnDocument(t,a):this.listenOffDocument(t,a)},listenOnDocument:function(e,t){Me&&(Yc(document,e,t,So),this.registerDocumentListener(e,t))},listenOffDocument:function(e,t){Me&&Pc(document,e,t,So),this.unregisterDocumentListener(e,t)}}}),CJ="$_windowListeners",jJ=i["default"].extend({created:function(){this[CJ]={}},beforeDestroy:function(){var e=this;ea(this[CJ]||{}).forEach((function(t){e[CJ][t].forEach((function(a){e.listenOffWindow(t,a)}))})),this[CJ]=null},methods:{registerWindowListener:function(e,t){this[CJ]&&(this[CJ][e]=this[CJ][e]||[],ll(this[CJ][e],t)||this[CJ][e].push(t))},unregisterWindowListener:function(e,t){this[CJ]&&this[CJ][e]&&(this[CJ][e]=this[CJ][e].filter((function(e){return e!==t})))},listenWindow:function(e,t,a){e?this.listenOnWindow(t,a):this.listenOffWindow(t,a)},listenOnWindow:function(e,t){Me&&(Yc(window,e,t,So),this.registerWindowListener(e,t))},listenOffWindow:function(e,t){Me&&Pc(window,e,t,So),this.unregisterWindowListener(e,t)}}}),xJ=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null;return e&&e.$options._scopeId||t};function BJ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var YJ=i["default"].extend({computed:{scopedStyleAttrs:function(){var e=xJ(this.$parent);return e?BJ({},e,""):{}}}}),PJ=i["default"].extend({abstract:!0,name:yi,props:{nodes:wc(Po)},data:function(e){return{updatedNodes:e.nodes}},destroyed:function(){Dl(this.$el)},render:function(e){var t=this.updatedNodes,a=Ct(t)?t({}):t;return a=cl(a).filter(ya),a&&a.length>0&&!a[0].text?a[0]:e()}}),IJ={container:wc([wt,Yo],"body"),disabled:wc(To,!1),tag:wc(Yo,"div")},FJ=i["default"].extend({name:gi,mixins:[xc],props:IJ,watch:{disabled:{immediate:!0,handler:function(e){e?this.unmountTarget():this.$nextTick(this.mountTarget)}}},created:function(){this.$_defaultFn=null,this.$_target=null},beforeMount:function(){this.mountTarget()},updated:function(){this.updateTarget()},beforeDestroy:function(){this.unmountTarget(),this.$_defaultFn=null},methods:{getContainer:function(){if(Me){var e=this.container;return xt(e)?El(e):e}return null},mountTarget:function(){if(!this.$_target){var e=this.getContainer();if(e){var t=document.createElement("div");e.appendChild(t),this.$_target=new PJ({el:t,parent:this,propsData:{nodes:cl(this.normalizeSlot())}})}}},updateTarget:function(){if(Me&&this.$_target){var e=this.$scopedSlots.default;this.disabled||(e&&this.$_defaultFn!==e?this.$_target.updatedNodes=e:e||(this.$_target.updatedNodes=this.$slots.default)),this.$_defaultFn=e}},unmountTarget:function(){this.$_target&&this.$_target.$destroy(),this.$_target=null}},render:function(e){if(this.disabled){var t=cl(this.normalizeSlot()).filter(ya);if(t.length>0&&!t[0].text)return t[0]}return e()}});function EJ(e){return EJ="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},EJ(e)}function $J(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function RJ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?$J(Object(a),!0).forEach((function(t){NJ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):$J(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function NJ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}function WJ(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function UJ(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function GJ(e,t,a){return t&&UJ(e.prototype,t),a&&UJ(e,a),Object.defineProperty(e,"prototype",{writable:!1}),e}function qJ(){return qJ="undefined"!==typeof Reflect&&Reflect.get?Reflect.get:function(e,t,a){var n=JJ(e,t);if(n){var r=Object.getOwnPropertyDescriptor(n,t);return r.get?r.get.call(arguments.length<3?e:a):r.value}},qJ.apply(this,arguments)}function JJ(e,t){while(!Object.prototype.hasOwnProperty.call(e,t))if(e=aK(e),null===e)break;return e}function KJ(e,t){if("function"!==typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function");Object.defineProperty(e,"prototype",{value:Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),writable:!1}),t&&XJ(e,t)}function XJ(e,t){return XJ=Object.setPrototypeOf||function(e,t){return e.__proto__=t,e},XJ(e,t)}function ZJ(e){var t=tK();return function(){var a,n=aK(e);if(t){var r=aK(this).constructor;a=Reflect.construct(n,arguments,r)}else a=n.apply(this,arguments);return QJ(this,a)}}function QJ(e,t){if(t&&("object"===EJ(t)||"function"===typeof t))return t;if(void 0!==t)throw new TypeError("Derived constructors may only return object or undefined");return eK(e)}function eK(e){if(void 0===e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function tK(){if("undefined"===typeof Reflect||!Reflect.construct)return!1;if(Reflect.construct.sham)return!1;if("function"===typeof Proxy)return!0;try{return Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],(function(){}))),!0}catch(e){return!1}}function aK(e){return aK=Object.setPrototypeOf?Object.getPrototypeOf:function(e){return e.__proto__||Object.getPrototypeOf(e)},aK(e)}var nK=function(e){KJ(a,e);var t=ZJ(a);function a(e){var n,r=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return WJ(this,a),n=t.call(this,e,r),Xt(eK(n),{trigger:la()}),n}return GJ(a,null,[{key:"Defaults",get:function(){return RJ(RJ({},qJ(aK(a),"Defaults",this)),{},{trigger:null})}}]),a}(c$),rK=1040,iK=".fixed-top, .fixed-bottom, .is-fixed, .sticky-top",oK=".sticky-top",sK=".navbar-toggler",lK=i["default"].extend({data:function(){return{modals:[],baseZIndex:null,scrollbarWidth:null,isBodyOverflowing:!1}},computed:{modalCount:function(){return this.modals.length},modalsAreOpen:function(){return this.modalCount>0}},watch:{modalCount:function(e,t){Me&&(this.getScrollbarWidth(),e>0&&0===t?(this.checkScrollbar(),this.setScrollbar(),Ul(document.body,"modal-open")):0===e&&t>0&&(this.resetScrollbar(),Gl(document.body,"modal-open")),Jl(document.body,"data-modal-open-count",String(e)))},modals:function(e){var t=this;this.checkScrollbar(),Ol((function(){t.updateModals(e||[])}))}},methods:{registerModal:function(e){e&&-1===this.modals.indexOf(e)&&this.modals.push(e)},unregisterModal:function(e){var t=this.modals.indexOf(e);t>-1&&(this.modals.splice(t,1),e._isBeingDestroyed||e._isDestroyed||this.resetModal(e))},getBaseZIndex:function(){if(Me&&Tt(this.baseZIndex)){var e=document.createElement("div");Ul(e,"modal-backdrop"),Ul(e,"d-none"),Ql(e,"display","none"),document.body.appendChild(e),this.baseZIndex=fl(nc(e).zIndex,rK),document.body.removeChild(e)}return this.baseZIndex||rK},getScrollbarWidth:function(){if(Me&&Tt(this.scrollbarWidth)){var e=document.createElement("div");Ul(e,"modal-scrollbar-measure"),document.body.appendChild(e),this.scrollbarWidth=ac(e).width-e.clientWidth,document.body.removeChild(e)}return this.scrollbarWidth||0},updateModals:function(e){var t=this,a=this.getBaseZIndex(),n=this.getScrollbarWidth();e.forEach((function(e,r){e.zIndex=a+r,e.scrollbarWidth=n,e.isTop=r===t.modals.length-1,e.isBodyOverflowing=t.isBodyOverflowing}))},resetModal:function(e){e&&(e.zIndex=this.getBaseZIndex(),e.isTop=!0,e.isBodyOverflowing=!1)},checkScrollbar:function(){var e=ac(document.body),t=e.left,a=e.right;this.isBodyOverflowing=t+a<window.innerWidth},setScrollbar:function(){var e=document.body;if(e._paddingChangedForModal=e._paddingChangedForModal||[],e._marginChangedForModal=e._marginChangedForModal||[],this.isBodyOverflowing){var t=this.scrollbarWidth;Fl(iK).forEach((function(a){var n=tc(a,"paddingRight")||"";Jl(a,"data-padding-right",n),Ql(a,"paddingRight","".concat(pl(nc(a).paddingRight,0)+t,"px")),e._paddingChangedForModal.push(a)})),Fl(oK).forEach((function(a){var n=tc(a,"marginRight")||"";Jl(a,"data-margin-right",n),Ql(a,"marginRight","".concat(pl(nc(a).marginRight,0)-t,"px")),e._marginChangedForModal.push(a)})),Fl(sK).forEach((function(a){var n=tc(a,"marginRight")||"";Jl(a,"data-margin-right",n),Ql(a,"marginRight","".concat(pl(nc(a).marginRight,0)+t,"px")),e._marginChangedForModal.push(a)}));var a=tc(e,"paddingRight")||"";Jl(e,"data-padding-right",a),Ql(e,"paddingRight","".concat(pl(nc(e).paddingRight,0)+t,"px"))}},resetScrollbar:function(){var e=document.body;e._paddingChangedForModal&&e._paddingChangedForModal.forEach((function(e){Zl(e,"data-padding-right")&&(Ql(e,"paddingRight",Xl(e,"data-padding-right")||""),Kl(e,"data-padding-right"))})),e._marginChangedForModal&&e._marginChangedForModal.forEach((function(e){Zl(e,"data-margin-right")&&(Ql(e,"marginRight",Xl(e,"data-margin-right")||""),Kl(e,"data-margin-right"))})),e._paddingChangedForModal=null,e._marginChangedForModal=null,Zl(e,"data-padding-right")&&(Ql(e,"paddingRight",Xl(e,"data-padding-right")||""),Kl(e,"data-padding-right"))}}}),cK=new lK;function uK(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function dK(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?uK(Object(a),!0).forEach((function(t){hK(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):uK(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function hK(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var fK=Dc("visible",{type:To,defaultValue:!1,event:Li}),pK=fK.mixin,mK=fK.props,vK=fK.prop,_K=fK.event,gK="backdrop",yK="esc",bK="FORCE",MK="toggle",zK="cancel",LK="headerclose",wK="ok",HK=[zK,LK,wK],kK={subtree:!0,childList:!0,characterData:!0,attributes:!0,attributeFilter:["style","class"]},AK=Sc(sa(dK(dK(dK({},XP),mK),{},{ariaLabel:wc(Yo),autoFocusButton:wc(Yo,null,(function(e){return Dt(e)||ll(HK,e)})),bodyBgVariant:wc(Yo),bodyClass:wc(Fo),bodyTextVariant:wc(Yo),busy:wc(To,!1),buttonSize:wc(Yo),cancelDisabled:wc(To,!1),cancelTitle:wc(Yo,"Cancel"),cancelTitleHtml:wc(Yo),cancelVariant:wc(Yo,"secondary"),centered:wc(To,!1),contentClass:wc(Fo),dialogClass:wc(Fo),footerBgVariant:wc(Yo),footerBorderVariant:wc(Yo),footerClass:wc(Fo),footerTag:wc(Yo,"footer"),footerTextVariant:wc(Yo),headerBgVariant:wc(Yo),headerBorderVariant:wc(Yo),headerClass:wc(Fo),headerCloseContent:wc(Yo,"&times;"),headerCloseLabel:wc(Yo,"Close"),headerCloseVariant:wc(Yo),headerTag:wc(Yo,"header"),headerTextVariant:wc(Yo),hideBackdrop:wc(To,!1),hideFooter:wc(To,!1),hideHeader:wc(To,!1),hideHeaderClose:wc(To,!1),ignoreEnforceFocusSelector:wc(Eo),lazy:wc(To,!1),modalClass:wc(Fo),noCloseOnBackdrop:wc(To,!1),noCloseOnEsc:wc(To,!1),noEnforceFocus:wc(To,!1),noFade:wc(To,!1),noStacking:wc(To,!1),okDisabled:wc(To,!1),okOnly:wc(To,!1),okTitle:wc(Yo,"OK"),okTitleHtml:wc(Yo),okVariant:wc(Yo,"primary"),returnFocus:wc([wt,xo,Yo]),scrollable:wc(To,!1),size:wc(Yo,"md"),static:wc(To,!1),title:wc(Yo),titleClass:wc(Fo),titleHtml:wc(Yo),titleSrOnly:wc(To,!1),titleTag:wc(Yo,"h5")})),Mr),SK=i["default"].extend({name:Mr,mixins:[UB,ZP,pK,DJ,qB,jJ,xc,YJ],inheritAttrs:!1,props:AK,data:function(){return{isHidden:!0,isVisible:!1,isTransitioning:!1,isShow:!1,isBlock:!1,isOpening:!1,isClosing:!1,ignoreBackdropClick:!1,isModalOverflowing:!1,scrollbarWidth:0,zIndex:cK.getBaseZIndex(),isTop:!0,isBodyOverflowing:!1}},computed:{modalId:function(){return this.safeId()},modalOuterId:function(){return this.safeId("__BV_modal_outer_")},modalHeaderId:function(){return this.safeId("__BV_modal_header_")},modalBodyId:function(){return this.safeId("__BV_modal_body_")},modalTitleId:function(){return this.safeId("__BV_modal_title_")},modalContentId:function(){return this.safeId("__BV_modal_content_")},modalFooterId:function(){return this.safeId("__BV_modal_footer_")},modalBackdropId:function(){return this.safeId("__BV_modal_backdrop_")},modalClasses:function(){return[{fade:!this.noFade,show:this.isShow},this.modalClass]},modalStyles:function(){var e="".concat(this.scrollbarWidth,"px");return{paddingLeft:!this.isBodyOverflowing&&this.isModalOverflowing?e:"",paddingRight:this.isBodyOverflowing&&!this.isModalOverflowing?e:"",display:this.isBlock?"block":"none"}},dialogClasses:function(){var e;return[(e={},hK(e,"modal-".concat(this.size),this.size),hK(e,"modal-dialog-centered",this.centered),hK(e,"modal-dialog-scrollable",this.scrollable),e),this.dialogClass]},headerClasses:function(){var e;return[(e={},hK(e,"bg-".concat(this.headerBgVariant),this.headerBgVariant),hK(e,"text-".concat(this.headerTextVariant),this.headerTextVariant),hK(e,"border-".concat(this.headerBorderVariant),this.headerBorderVariant),e),this.headerClass]},titleClasses:function(){return[{"sr-only":this.titleSrOnly},this.titleClass]},bodyClasses:function(){var e;return[(e={},hK(e,"bg-".concat(this.bodyBgVariant),this.bodyBgVariant),hK(e,"text-".concat(this.bodyTextVariant),this.bodyTextVariant),e),this.bodyClass]},footerClasses:function(){var e;return[(e={},hK(e,"bg-".concat(this.footerBgVariant),this.footerBgVariant),hK(e,"text-".concat(this.footerTextVariant),this.footerTextVariant),hK(e,"border-".concat(this.footerBorderVariant),this.footerBorderVariant),e),this.footerClass]},modalOuterStyle:function(){return{position:"absolute",zIndex:this.zIndex}},slotScope:function(){return{cancel:this.onCancel,close:this.onClose,hide:this.hide,ok:this.onOk,visible:this.isVisible}},computeIgnoreEnforceFocusSelector:function(){return cl(this.ignoreEnforceFocusSelector).filter(ya).join(",").trim()},computedAttrs:function(){var e=this.static?{}:this.scopedStyleAttrs;return dK(dK(dK({},e),this.bvAttrs),{},{id:this.modalOuterId})},computedModalAttrs:function(){var e=this.isVisible,t=this.ariaLabel;return{id:this.modalId,role:"dialog","aria-hidden":e?null:"true","aria-modal":e?"true":null,"aria-label":t,"aria-labelledby":this.hideHeader||t||!(this.hasNormalizedSlot(xs)||this.titleHtml||this.title)?null:this.modalTitleId,"aria-describedby":this.modalBodyId}}},watch:hK({},vK,(function(e,t){e!==t&&this[e?"show":"hide"]()})),created:function(){this.$_observer=null,this.$_returnFocus=this.returnFocus||null},mounted:function(){this.zIndex=cK.getBaseZIndex(),this.listenOnRoot(Rc(Mr,fo),this.showHandler),this.listenOnRoot(Rc(Mr,Ri),this.hideHandler),this.listenOnRoot(Rc(Mr,yo),this.toggleHandler),this.listenOnRoot($c(Mr,fo),this.modalListener),!0===this[vK]&&this.$nextTick(this.show)},beforeDestroy:function(){cK.unregisterModal(this),this.setObserver(!1),this.isVisible&&(this.isVisible=!1,this.isShow=!1,this.isTransitioning=!1)},methods:{setObserver:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0];this.$_observer&&this.$_observer.disconnect(),this.$_observer=null,e&&(this.$_observer=kF(this.$refs.content,this.checkModalOverflow.bind(this),kK))},updateModel:function(e){e!==this[vK]&&this.$emit(_K,e)},buildEvent:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return new nK(e,dK(dK({cancelable:!1,target:this.$refs.modal||this.$el||null,relatedTarget:null,trigger:null},t),{},{vueTarget:this,componentId:this.modalId}))},show:function(){if(!this.isVisible&&!this.isOpening)if(this.isClosing)this.$once($i,this.show);else{this.isOpening=!0,this.$_returnFocus=this.$_returnFocus||this.getActiveElement();var e=this.buildEvent(fo,{cancelable:!0});if(this.emitEvent(e),e.defaultPrevented||this.isVisible)return this.isOpening=!1,void this.updateModel(!1);this.doShow()}},hide:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"";if(this.isVisible&&!this.isClosing){this.isClosing=!0;var t=this.buildEvent(Ri,{cancelable:e!==bK,trigger:e||null});if(e===wK?this.$emit(Ki,t):e===zK?this.$emit(zi,t):e===LK&&this.$emit(ki,t),this.emitEvent(t),t.defaultPrevented||!this.isVisible)return this.isClosing=!1,void this.updateModel(!0);this.setObserver(!1),this.isVisible=!1,this.updateModel(!1)}},toggle:function(e){e&&(this.$_returnFocus=e),this.isVisible?this.hide(MK):this.show()},getActiveElement:function(){var e=jl(Me?[document.body]:[]);return e&&e.focus?e:null},doShow:function(){var e=this;cK.modalsAreOpen&&this.noStacking?this.listenOnRootOnce($c(Mr,$i),this.doShow):(cK.registerModal(this),this.isHidden=!1,this.$nextTick((function(){e.isVisible=!0,e.isOpening=!1,e.updateModel(!0),e.$nextTick((function(){e.setObserver(!0)}))})))},onBeforeEnter:function(){this.isTransitioning=!0,this.setResizeEvent(!0)},onEnter:function(){var e=this;this.isBlock=!0,Ol((function(){Ol((function(){e.isShow=!0}))}))},onAfterEnter:function(){var e=this;this.checkModalOverflow(),this.isTransitioning=!1,Ol((function(){e.emitEvent(e.buildEvent(po)),e.setEnforceFocus(!0),e.$nextTick((function(){e.focusFirst()}))}))},onBeforeLeave:function(){this.isTransitioning=!0,this.setResizeEvent(!1),this.setEnforceFocus(!1)},onLeave:function(){this.isShow=!1},onAfterLeave:function(){var e=this;this.isBlock=!1,this.isTransitioning=!1,this.isModalOverflowing=!1,this.isHidden=!0,this.$nextTick((function(){e.isClosing=!1,cK.unregisterModal(e),e.returnFocusTo(),e.emitEvent(e.buildEvent($i))}))},emitEvent:function(e){var t=e.type;this.emitOnRoot($c(Mr,t),e,e.componentId),this.$emit(t,e)},onDialogMousedown:function(){var e=this,t=this.$refs.modal,a=function a(n){Pc(t,"mouseup",a,So),n.target===t&&(e.ignoreBackdropClick=!0)};Yc(t,"mouseup",a,So)},onClickOut:function(e){this.ignoreBackdropClick?this.ignoreBackdropClick=!1:this.isVisible&&!this.noCloseOnBackdrop&&Nl(document.body,e.target)&&(Nl(this.$refs.content,e.target)||this.hide(gK))},onOk:function(){this.hide(wK)},onCancel:function(){this.hide(zK)},onClose:function(){this.hide(LK)},onEsc:function(e){e.keyCode===DB&&this.isVisible&&!this.noCloseOnEsc&&this.hide(yK)},focusHandler:function(e){var t=this.$refs.content,a=e.target;if(!(this.noEnforceFocus||!this.isTop||!this.isVisible||!t||document===a||Nl(t,a)||this.computeIgnoreEnforceFocusSelector&&Rl(this.computeIgnoreEnforceFocusSelector,a,!0))){var n=sc(this.$refs.content),r=this.$refs["bottom-trap"],i=this.$refs["top-trap"];if(r&&a===r){if(lc(n[0]))return}else if(i&&a===i&&lc(n[n.length-1]))return;lc(t,{preventScroll:!0})}},setEnforceFocus:function(e){this.listenDocument(e,"focusin",this.focusHandler)},setResizeEvent:function(e){this.listenWindow(e,"resize",this.checkModalOverflow),this.listenWindow(e,"orientationchange",this.checkModalOverflow)},showHandler:function(e,t){e===this.modalId&&(this.$_returnFocus=t||this.getActiveElement(),this.show())},hideHandler:function(e){e===this.modalId&&this.hide("event")},toggleHandler:function(e,t){e===this.modalId&&this.toggle(t)},modalListener:function(e){this.noStacking&&e.vueTarget!==this&&this.hide()},focusFirst:function(){var e=this;Me&&Ol((function(){var t=e.$refs.modal,a=e.$refs.content,n=e.getActiveElement();if(t&&a&&(!n||!Nl(a,n))){var r=e.$refs["ok-button"],i=e.$refs["cancel-button"],o=e.$refs["close-button"],s=e.autoFocusButton,l=s===wK&&r?r.$el||r:s===zK&&i?i.$el||i:s===LK&&o?o.$el||o:a;lc(l),l===a&&e.$nextTick((function(){t.scrollTop=0}))}}))},returnFocusTo:function(){var e=this.returnFocus||this.$_returnFocus||null;this.$_returnFocus=null,this.$nextTick((function(){e=xt(e)?El(e):e,e&&(e=e.$el||e,lc(e))}))},checkModalOverflow:function(){if(this.isVisible){var e=this.$refs.modal;this.isModalOverflowing=e.scrollHeight>document.documentElement.clientHeight}},makeModal:function(e){var t=e();if(!this.hideHeader){var a=this.normalizeSlot(Ds,this.slotScope);if(!a){var n=e();this.hideHeaderClose||(n=e(Uc,{props:{content:this.headerCloseContent,disabled:this.isTransitioning,ariaLabel:this.headerCloseLabel,textVariant:this.headerCloseVariant||this.headerTextVariant},on:{click:this.onClose},ref:"close-button"},[this.normalizeSlot(Cs)])),a=[e(this.titleTag,{staticClass:"modal-title",class:this.titleClasses,attrs:{id:this.modalTitleId},domProps:this.hasNormalizedSlot(xs)?{}:UY(this.titleHtml,this.title)},this.normalizeSlot(xs,this.slotScope)),n]}t=e(this.headerTag,{staticClass:"modal-header",class:this.headerClasses,attrs:{id:this.modalHeaderId},ref:"header"},[a])}var r=e("div",{staticClass:"modal-body",class:this.bodyClasses,attrs:{id:this.modalBodyId},ref:"body"},this.normalizeSlot(is,this.slotScope)),i=e();if(!this.hideFooter){var o=this.normalizeSlot(Ts,this.slotScope);if(!o){var s=e();this.okOnly||(s=e(LY,{props:{variant:this.cancelVariant,size:this.buttonSize,disabled:this.cancelDisabled||this.busy||this.isTransitioning},domProps:this.hasNormalizedSlot(Os)?{}:UY(this.cancelTitleHtml,this.cancelTitle),on:{click:this.onCancel},ref:"cancel-button"},this.normalizeSlot(Os)));var l=e(LY,{props:{variant:this.okVariant,size:this.buttonSize,disabled:this.okDisabled||this.busy||this.isTransitioning},domProps:this.hasNormalizedSlot(js)?{}:UY(this.okTitleHtml,this.okTitle),on:{click:this.onOk},ref:"ok-button"},this.normalizeSlot(js));o=[s,l]}i=e(this.footerTag,{staticClass:"modal-footer",class:this.footerClasses,attrs:{id:this.modalFooterId},ref:"footer"},[o])}var c=e("div",{staticClass:"modal-content",class:this.contentClass,attrs:{id:this.modalContentId,tabindex:"-1"},ref:"content"},[t,r,i]),u=e(),d=e();this.isVisible&&!this.noEnforceFocus&&(u=e("span",{attrs:{tabindex:"0"},ref:"top-trap"}),d=e("span",{attrs:{tabindex:"0"},ref:"bottom-trap"}));var h=e("div",{staticClass:"modal-dialog",class:this.dialogClasses,on:{mousedown:this.onDialogMousedown},ref:"dialog"},[u,c,d]),f=e("div",{staticClass:"modal",class:this.modalClasses,style:this.modalStyles,attrs:this.computedModalAttrs,on:{keydown:this.onEsc,click:this.onClickOut},directives:[{name:"show",value:this.isVisible}],ref:"modal"},[h]);f=e("transition",{props:{enterClass:"",enterToClass:"",enterActiveClass:"",leaveClass:"",leaveActiveClass:"",leaveToClass:""},on:{beforeEnter:this.onBeforeEnter,enter:this.onEnter,afterEnter:this.onAfterEnter,beforeLeave:this.onBeforeLeave,leave:this.onLeave,afterLeave:this.onAfterLeave}},[f]);var p=e();return!this.hideBackdrop&&this.isVisible&&(p=e("div",{staticClass:"modal-backdrop",attrs:{id:this.modalBackdropId}},this.normalizeSlot(Vs))),p=e(eu,{props:{noFade:this.noFade}},[p]),e("div",{style:this.modalOuterStyle,attrs:this.computedAttrs,key:"modal-outer-".concat(this[Ka])},[f,p])}},render:function(e){return this.static?this.lazy&&this.isHidden?e():this.makeModal(e):this.isHidden?e():e(FJ,[this.makeModal(e)])}}),VK=Rc(Mr,fo),OK="__bv_modal_directive__",TK=function(e){var t=e.modifiers,a=void 0===t?{}:t,n=e.arg,r=e.value;return xt(r)?r:xt(n)?n:ea(a).reverse()[0]},DK=function(e){return e&&$l(e,".dropdown-menu > li, li.nav-item")&&El("a, button",e)||e},CK=function(e){e&&"BUTTON"!==e.tagName&&(Zl(e,"role")||Jl(e,"role","button"),"A"===e.tagName||Zl(e,"tabindex")||Jl(e,"tabindex","0"))},jK=function(e,t,a){var n=TK(t),r=DK(e);if(n&&r){var i=function(e){var t=e.currentTarget;if(!Pl(t)){var r=e.type,i=e.keyCode;"click"!==r&&("keydown"!==r||i!==TB&&i!==PB)||a.context.$root.$emit(VK,n,t)}};e[OK]={handler:i,target:n,trigger:r},CK(r),Yc(r,"click",i,Ao),"BUTTON"!==r.tagName&&"button"===Xl(r,"role")&&Yc(r,"keydown",i,Ao)}},xK=function(e){var t=e[OK]||{},a=t.trigger,n=t.handler;a&&n&&(Pc(a,"click",n,Ao),Pc(a,"keydown",n,Ao),Pc(e,"click",n,Ao),Pc(e,"keydown",n,Ao)),delete e[OK]},BK=function(e,t,a){var n=e[OK]||{},r=TK(t),i=DK(e);r===n.target&&i===n.trigger||(xK(e,t,a),jK(e,t,a)),CK(i)},YK=function(){},PK={inserted:BK,updated:YK,componentUpdated:BK,unbind:xK};function IK(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function FK(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function EK(e,t,a){return t&&FK(e.prototype,t),a&&FK(e,a),Object.defineProperty(e,"prototype",{writable:!1}),e}function $K(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function RK(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?$K(Object(a),!0).forEach((function(t){NK(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):$K(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function NK(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}function WK(e){return JK(e)||qK(e)||GK(e)||UK()}function UK(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function GK(e,t){if(e){if("string"===typeof e)return KK(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?KK(e,t):void 0}}function qK(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function JK(e){if(Array.isArray(e))return KK(e)}function KK(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}var XK="$bvModal",ZK="_bv__modal",QK=["id"].concat(WK(ea(ia(AK,["busy","lazy","noStacking","static","visible"])))),eX=function(){},tX={msgBoxContent:"default",title:"modal-title",okTitle:"modal-ok",cancelTitle:"modal-cancel"},aX=function(e){return QK.reduce((function(t,a){return Ot(e[a])||(t[a]=e[a]),t}),{})},nX=function(e){var t=e.extend({name:zr,extends:SK,destroyed:function(){this.$el&&this.$el.parentNode&&this.$el.parentNode.removeChild(this.$el)},mounted:function(){var e=this,t=function(){e.$nextTick((function(){Ol((function(){e.$destroy()}))}))};this.$parent.$once(Lo,t),this.$once($i,t),this.$router&&this.$route&&this.$once(zo,this.$watch("$router",t)),this.show()}}),a=function(e,a){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:eX;if(!wa(XK)&&!Ha(XK)){var r=new t({parent:e,propsData:RK(RK(RK({},aX(fc(Mr))),{},{hideHeaderClose:!0,hideHeader:!(a.title||a.titleHtml)},ia(a,ea(tX))),{},{lazy:!1,busy:!1,visible:!1,noStacking:!1,noEnforceFocus:!1})});return ea(tX).forEach((function(e){Ot(a[e])||(r.$slots[tX[e]]=cl(a[e]))})),new Promise((function(e,t){var a=!1;r.$once(Lo,(function(){a||t(new Error("BootstrapVue MsgBox destroyed before resolve"))})),r.$on(Ri,(function(t){if(!t.defaultPrevented){var r=n(t);t.defaultPrevented||(a=!0,e(r))}}));var i=document.createElement("div");document.body.appendChild(i),r.$mount(i)}))}},n=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null;if(t&&!Ha(XK)&&!wa(XK)&&Ct(r))return a(e,RK(RK({},aX(n)),{},{msgBoxContent:t}),r)},r=function(){function e(t){IK(this,e),Jt(this,{_vm:t,_root:t.$root}),Xt(this,{_vm:la(),_root:la()})}return EK(e,[{key:"show",value:function(e){if(e&&this._root){for(var t,a=arguments.length,n=new Array(a>1?a-1:0),r=1;r<a;r++)n[r-1]=arguments[r];(t=this._root).$emit.apply(t,[Rc(Mr,"show"),e].concat(n))}}},{key:"hide",value:function(e){if(e&&this._root){for(var t,a=arguments.length,n=new Array(a>1?a-1:0),r=1;r<a;r++)n[r-1]=arguments[r];(t=this._root).$emit.apply(t,[Rc(Mr,"hide"),e].concat(n))}}},{key:"msgBoxOk",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},a=RK(RK({},t),{},{okOnly:!0,okDisabled:!1,hideFooter:!1,msgBoxContent:e});return n(this._vm,e,a,(function(){return!0}))}},{key:"msgBoxConfirm",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},a=RK(RK({},t),{},{okOnly:!1,okDisabled:!1,cancelDisabled:!1,hideFooter:!1});return n(this._vm,e,a,(function(e){var t=e.trigger;return"ok"===t||"cancel"!==t&&null}))}}]),e}();e.mixin({beforeCreate:function(){this[ZK]=new r(this)}}),ta(e.prototype,XK)||Zt(e.prototype,XK,{get:function(){return this&&this[ZK]||La('"'.concat(XK,'" must be accessed from a Vue instance "this" context.'),Mr),this[ZK]}})},rX=Pa({plugins:{plugin:nX}}),iX=Pa({components:{BModal:SK},directives:{VBModal:PK},plugins:{BVModalPlugin:rX}});function oX(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var sX=function(e){return e="left"===e?"start":"right"===e?"end":e,"justify-content-".concat(e)},lX=Sc({align:wc(Yo),cardHeader:wc(To,!1),fill:wc(To,!1),justified:wc(To,!1),pills:wc(To,!1),small:wc(To,!1),tabs:wc(To,!1),tag:wc(Yo,"ul"),vertical:wc(To,!1)},Lr),cX=i["default"].extend({name:Lr,functional:!0,props:lX,render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.tabs,s=n.pills,l=n.vertical,c=n.align,u=n.cardHeader;return e(n.tag,Ja(r,{staticClass:"nav",class:(a={"nav-tabs":o,"nav-pills":s&&!o,"card-header-tabs":!l&&u&&o,"card-header-pills":!l&&u&&s&&!o,"flex-column":l,"nav-fill":!l&&n.fill,"nav-justified":!l&&n.justified},oX(a,sX(c),!l&&c),oX(a,"small",n.small),a)}),i)}});function uX(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function dX(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?uX(Object(a),!0).forEach((function(t){hX(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):uX(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function hX(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var fX=ia(lY,["event","routerTag"]),pX=Sc(sa(dX(dX({},fX),{},{linkAttrs:wc(xo,{}),linkClasses:wc(Fo)})),Vr),mX=i["default"].extend({name:Vr,functional:!0,props:pX,render:function(e,t){var a=t.props,n=t.data,r=t.listeners,i=t.children;return e("li",Ja(ia(n,["on"]),{staticClass:"nav-item"}),[e(cY,{staticClass:"nav-link",class:a.linkClasses,attrs:a.linkAttrs,props:kc(fX,a),on:r},i)])}}),vX={},_X=i["default"].extend({name:Tr,functional:!0,props:vX,render:function(e,t){var a=t.data,n=t.children;return e("li",Ja(a,{staticClass:"navbar-text"}),n)}});function gX(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function yX(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?gX(Object(a),!0).forEach((function(t){bX(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):gX(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function bX(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var MX=ia(q$,["inline"]),zX=Sc(sa(yX(yX({},MX),{},{formClass:wc(Fo)})),Sr),LX=i["default"].extend({name:Sr,functional:!0,props:zX,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=t.listeners,o=e(J$,{class:a.formClass,props:yX(yX({},kc(MX,a)),{},{inline:!0}),attrs:n.attrs,on:i},r);return e("li",Ja(ia(n,["attrs","on"]),{staticClass:"form-inline"}),[o])}});function wX(e){return SX(e)||AX(e)||kX(e)||HX()}function HX(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function kX(e,t){if(e){if("string"===typeof e)return VX(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?VX(e,t):void 0}}function AX(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function SX(e){if(Array.isArray(e))return VX(e)}function VX(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}function OX(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function TX(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?OX(Object(a),!0).forEach((function(t){DX(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):OX(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function DX(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var CX=Sc(sa(TX(TX({},XP),ra(H$,[].concat(wX(ea(b$)),["html","lazy","menuClass","noCaret","role","text","toggleClass"])))),Or),jX=i["default"].extend({name:Or,mixins:[ZP,M$,xc],props:CX,computed:{toggleId:function(){return this.safeId("_BV_toggle_")},menuId:function(){return this.safeId("_BV_toggle_menu_")},dropdownClasses:function(){return[this.directionClass,this.boundaryClass,{show:this.visible}]},menuClasses:function(){return[this.menuClass,{"dropdown-menu-right":this.right,show:this.visible}]},toggleClasses:function(){return[this.toggleClass,{"dropdown-toggle-no-caret":this.noCaret}]}},render:function(e){var t=this.toggleId,a=this.menuId,n=this.visible,r=this.hide,i=e(cY,{staticClass:"nav-link dropdown-toggle",class:this.toggleClasses,props:{href:"#".concat(this.id||""),disabled:this.disabled},attrs:{id:t,role:"button","aria-haspopup":"true","aria-expanded":n?"true":"false","aria-controls":a},on:{mousedown:this.onMousedown,click:this.toggle,keydown:this.toggle},ref:"toggle"},[this.normalizeSlot([as,tl])||e("span",{domProps:UY(this.html,this.text)})]),o=e("ul",{staticClass:"dropdown-menu",class:this.menuClasses,attrs:{tabindex:"-1","aria-labelledby":t,id:a},on:{keydown:this.onKeydown},ref:"menu"},!this.lazy||n?this.normalizeSlot(is,{hide:r}):[e()]);return e("li",{staticClass:"nav-item b-nav-dropdown dropdown",class:this.dropdownClasses,attrs:{id:this.safeId()}},[i,o])}}),xX=Pa({components:{BNav:cX,BNavItem:mX,BNavText:_X,BNavForm:LX,BNavItemDropdown:jX,BNavItemDd:jX,BNavDropdown:jX,BNavDd:jX},plugins:{DropdownPlugin:cR}});function BX(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var YX=Sc({fixed:wc(Yo),print:wc(To,!1),sticky:wc(To,!1),tag:wc(Yo,"nav"),toggleable:wc(No,!1),type:wc(Yo,"light"),variant:wc(Yo)},wr),PX=i["default"].extend({name:wr,mixins:[xc],provide:function(){return{bvNavbar:this}},props:YX,computed:{breakpointClass:function(){var e=this.toggleable,t=pc()[0],a=null;return e&&xt(e)&&e!==t?a="navbar-expand-".concat(e):!1===e&&(a="navbar-expand"),a}},render:function(e){var t,a=this.tag,n=this.type,r=this.variant,i=this.fixed;return e(a,{staticClass:"navbar",class:[(t={"d-print":this.print,"sticky-top":this.sticky},BX(t,"navbar-".concat(n),n),BX(t,"bg-".concat(r),r),BX(t,"fixed-".concat(i),i),t),this.breakpointClass],attrs:{role:xl(a,"nav")?null:"navigation"}},[this.normalizeSlot()])}});function IX(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var FX=function(e){return e="left"===e?"start":"right"===e?"end":e,"justify-content-".concat(e)},EX=Sc(ra(lX,["tag","fill","justified","align","small"]),kr),$X=i["default"].extend({name:kr,functional:!0,props:EX,render:function(e,t){var a,n=t.props,r=t.data,i=t.children,o=n.align;return e(n.tag,Ja(r,{staticClass:"navbar-nav",class:(a={"nav-fill":n.fill,"nav-justified":n.justified},IX(a,FX(o),o),IX(a,"small",n.small),a)}),i)}});function RX(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function NX(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?RX(Object(a),!0).forEach((function(t){WX(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):RX(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function WX(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var UX=ia(lY,["event","routerTag"]);UX.href.default=void 0,UX.to.default=void 0;var GX=Sc(sa(NX(NX({},UX),{},{tag:wc(Yo,"div")})),Hr),qX=i["default"].extend({name:Hr,functional:!0,props:GX,render:function(e,t){var a=t.props,n=t.data,r=t.children,i=a.to||a.href,o=i?cY:a.tag;return e(o,Ja(n,{staticClass:"navbar-brand",props:i?kc(UX,a):{}}),r)}}),JX="navbar-toggler",KX=$c(wn,"state"),XX=$c(wn,"sync-state"),ZX=Sc({disabled:wc(To,!1),label:wc(Yo,"Toggle navigation"),target:wc(Eo,void 0,!0)},Ar),QX=i["default"].extend({name:Ar,directives:{VBToggle:KE},mixins:[qB,xc],props:ZX,data:function(){return{toggleState:!1}},created:function(){this.listenOnRoot(KX,this.handleStateEvent),this.listenOnRoot(XX,this.handleStateEvent)},methods:{onClick:function(e){this.disabled||this.$emit(Hi,e)},handleStateEvent:function(e,t){e===this.target&&(this.toggleState=t)}},render:function(e){var t=this.disabled;return e("button",{staticClass:JX,class:{disabled:t},directives:[{name:"VBToggle",value:this.target}],attrs:{type:"button",disabled:t,"aria-label":this.label},on:{click:this.onClick}},[this.normalizeSlot(is,{expanded:this.toggleState})||e("span",{staticClass:"".concat(JX,"-icon")})])}}),eZ=Pa({components:{BNavbar:PX,BNavbarNav:$X,BNavbarBrand:qX,BNavbarToggle:QX,BNavToggle:QX},plugins:{NavPlugin:xX,CollapsePlugin:ZE,DropdownPlugin:cR}});function tZ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var aZ=Sc({label:wc(Yo),role:wc(Yo,"status"),small:wc(To,!1),tag:wc(Yo,"span"),type:wc(Yo,"border"),variant:wc(Yo)},Wr),nZ=i["default"].extend({name:Wr,functional:!0,props:aZ,render:function(e,t){var a,n=t.props,r=t.data,i=t.slots,o=t.scopedSlots,s=i(),l=o||{},c=jc(Hs,{},l,s)||n.label;return c&&(c=e("span",{staticClass:"sr-only"},c)),e(n.tag,Ja(r,{attrs:{role:c?n.role||"status":null,"aria-hidden":c?null:"true"},class:(a={},tZ(a,"spinner-".concat(n.type),n.type),tZ(a,"spinner-".concat(n.type,"-sm"),n.small),tZ(a,"text-".concat(n.variant),n.variant),a)}),[c||e()])}});function rZ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function iZ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?rZ(Object(a),!0).forEach((function(t){oZ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):rZ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function oZ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var sZ,lZ={top:0,left:0,bottom:0,right:0},cZ=Sc({bgColor:wc(Yo),blur:wc(Yo,"2px"),fixed:wc(To,!1),noCenter:wc(To,!1),noFade:wc(To,!1),noWrap:wc(To,!1),opacity:wc(Go,.85,(function(e){var t=pl(e,0);return t>=0&&t<=1})),overlayTag:wc(Yo,"div"),rounded:wc(No,!1),show:wc(To,!1),spinnerSmall:wc(To,!1),spinnerType:wc(Yo,"border"),spinnerVariant:wc(Yo),variant:wc(Yo,"light"),wrapTag:wc(Yo,"div"),zIndex:wc(Go,10)},Dr),uZ=i["default"].extend({name:Dr,mixins:[xc],props:cZ,computed:{computedRounded:function(){var e=this.rounded;return!0===e||""===e?"rounded":e?"rounded-".concat(e):""},computedVariant:function(){var e=this.variant;return e&&!this.bgColor?"bg-".concat(e):""},slotScope:function(){return{spinnerType:this.spinnerType||null,spinnerVariant:this.spinnerVariant||null,spinnerSmall:this.spinnerSmall}}},methods:{defaultOverlayFn:function(e){var t=e.spinnerType,a=e.spinnerVariant,n=e.spinnerSmall;return this.$createElement(nZ,{props:{type:t,variant:a,small:n}})}},render:function(e){var t=this,a=this.show,n=this.fixed,r=this.noFade,i=this.noWrap,o=this.slotScope,s=e();if(a){var l=e("div",{staticClass:"position-absolute",class:[this.computedVariant,this.computedRounded],style:iZ(iZ({},lZ),{},{opacity:this.opacity,backgroundColor:this.bgColor||null,backdropFilter:this.blur?"blur(".concat(this.blur,")"):null})}),c=e("div",{staticClass:"position-absolute",style:this.noCenter?iZ({},lZ):{top:"50%",left:"50%",transform:"translateX(-50%) translateY(-50%)"}},[this.normalizeSlot(Ns,o)||this.defaultOverlayFn(o)]);s=e(this.overlayTag,{staticClass:"b-overlay",class:{"position-absolute":!i||i&&!n,"position-fixed":i&&n},style:iZ(iZ({},lZ),{},{zIndex:this.zIndex||10}),on:{click:function(e){return t.$emit(Hi,e)}},key:"overlay"},[l,c])}return s=e(eu,{props:{noFade:r,appear:!0},on:{"after-enter":function(){return t.$emit(po)},"after-leave":function(){return t.$emit($i)}}},[s]),i?s:e(this.wrapTag,{staticClass:"b-overlay-wrap position-relative",attrs:{"aria-busy":a?"true":null}},i?[s]:[this.normalizeSlot(),s])}}),dZ=Pa({components:{BOverlay:uZ}});function hZ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function fZ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?hZ(Object(a),!0).forEach((function(t){pZ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):hZ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function pZ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var mZ=Dc("value",{type:Ro,defaultValue:null,validator:function(e){return!(!Tt(e)&&fl(e,0)<1)||(La('"v-model" value must be a number greater than "0"',Cr),!1)}}),vZ=mZ.mixin,_Z=mZ.props,gZ=mZ.prop,yZ=mZ.event,bZ=3,MZ=5,zZ=function(e,t){return ul(t,(function(t,a){return{number:e+a,classes:null}}))},LZ=function(e){var t=fl(e)||1;return t<1?MZ:t},wZ=function(e,t){var a=fl(e)||1;return a>t?t:a<1?1:a},HZ=function(e){if(e.keyCode===PB)return Fc(e,{immediatePropagation:!0}),e.currentTarget.click(),!1},kZ=Sc(sa(fZ(fZ({},_Z),{},{align:wc(Yo,"left"),ariaLabel:wc(Yo,"Pagination"),disabled:wc(To,!1),ellipsisClass:wc(Fo),ellipsisText:wc(Yo,"…"),firstClass:wc(Fo),firstNumber:wc(To,!1),firstText:wc(Yo,"«"),hideEllipsis:wc(To,!1),hideGotoEndButtons:wc(To,!1),labelFirstPage:wc(Yo,"Go to first page"),labelLastPage:wc(Yo,"Go to last page"),labelNextPage:wc(Yo,"Go to next page"),labelPage:wc(Uo,"Go to page"),labelPrevPage:wc(Yo,"Go to previous page"),lastClass:wc(Fo),lastNumber:wc(To,!1),lastText:wc(Yo,"»"),limit:wc(Go,MZ,(function(e){return!(fl(e,0)<1)||(La('Prop "limit" must be a number greater than "0"',Cr),!1)})),nextClass:wc(Fo),nextText:wc(Yo,"›"),pageClass:wc(Fo),pills:wc(To,!1),prevClass:wc(Fo),prevText:wc(Yo,"‹"),size:wc(Yo)})),"pagination"),AZ=i["default"].extend({mixins:[vZ,xc],props:kZ,data:function(){var e=fl(this[gZ],0);return e=e>0?e:-1,{currentPage:e,localNumberOfPages:1,localLimit:MZ}},computed:{btnSize:function(){var e=this.size;return e?"pagination-".concat(e):""},alignment:function(){var e=this.align;return"center"===e?"justify-content-center":"end"===e||"right"===e?"justify-content-end":"fill"===e?"text-center":""},styleClass:function(){return this.pills?"b-pagination-pills":""},computedCurrentPage:function(){return wZ(this.currentPage,this.localNumberOfPages)},paginationParams:function(){var e=this.localLimit,t=this.localNumberOfPages,a=this.computedCurrentPage,n=this.hideEllipsis,r=this.firstNumber,i=this.lastNumber,o=!1,s=!1,l=e,c=1;t<=e?l=t:a<e-1&&e>bZ?(n&&!i||(s=!0,l=e-(r?0:1)),l=pu(l,e)):t-a+2<e&&e>bZ?(n&&!r||(o=!0,l=e-(i?0:1)),c=t-l+1):(e>bZ&&(l=e-(n?0:2),o=!(n&&!r),s=!(n&&!i)),c=a-gu(l/2)),c<1?(c=1,o=!1):c>t-l&&(c=t-l+1,s=!1),o&&r&&c<4&&(l+=2,c=1,o=!1);var u=c+l-1;return s&&i&&u>t-3&&(l+=u===t-2?2:3,s=!1),e<=bZ&&(r&&1===c?l=pu(l+1,t,e+1):i&&t===c+l-1&&(c=mu(c-1,1),l=pu(t-c+1,t,e+1))),l=pu(l,t-c+1),{showFirstDots:o,showLastDots:s,numberOfLinks:l,startNumber:c}},pageList:function(){var e=this.paginationParams,t=e.numberOfLinks,a=e.startNumber,n=this.computedCurrentPage,r=zZ(a,t);if(r.length>3){var i=n-a,o="bv-d-xs-down-none";if(0===i)for(var s=3;s<r.length;s++)r[s].classes=o;else if(i===r.length-1)for(var l=0;l<r.length-3;l++)r[l].classes=o;else{for(var c=0;c<i-1;c++)r[c].classes=o;for(var u=r.length-1;u>i+1;u--)r[u].classes=o}}return r}},watch:(sZ={},pZ(sZ,gZ,(function(e,t){e!==t&&(this.currentPage=wZ(e,this.localNumberOfPages))})),pZ(sZ,"currentPage",(function(e,t){e!==t&&this.$emit(yZ,e>0?e:null)})),pZ(sZ,"limit",(function(e,t){e!==t&&(this.localLimit=LZ(e))})),sZ),created:function(){var e=this;this.localLimit=LZ(this.limit),this.$nextTick((function(){e.currentPage=e.currentPage>e.localNumberOfPages?e.localNumberOfPages:e.currentPage}))},methods:{handleKeyNav:function(e){var t=e.keyCode,a=e.shiftKey;this.isNav||(t===jB||t===IB?(Fc(e,{propagation:!1}),a?this.focusFirst():this.focusPrev()):t!==YB&&t!==VB||(Fc(e,{propagation:!1}),a?this.focusLast():this.focusNext()))},getButtons:function(){return Fl("button.page-link, a.page-link",this.$el).filter((function(e){return Yl(e)}))},focusCurrent:function(){var e=this;this.$nextTick((function(){var t=e.getButtons().find((function(t){return fl(Xl(t,"aria-posinset"),0)===e.computedCurrentPage}));lc(t)||e.focusFirst()}))},focusFirst:function(){var e=this;this.$nextTick((function(){var t=e.getButtons().find((function(e){return!Pl(e)}));lc(t)}))},focusLast:function(){var e=this;this.$nextTick((function(){var t=e.getButtons().reverse().find((function(e){return!Pl(e)}));lc(t)}))},focusPrev:function(){var e=this;this.$nextTick((function(){var t=e.getButtons(),a=t.indexOf(jl());a>0&&!Pl(t[a-1])&&lc(t[a-1])}))},focusNext:function(){var e=this;this.$nextTick((function(){var t=e.getButtons(),a=t.indexOf(jl());a<t.length-1&&!Pl(t[a+1])&&lc(t[a+1])}))}},render:function(e){var t=this,a=this.disabled,n=this.labelPage,r=this.ariaLabel,i=this.isNav,o=this.localNumberOfPages,s=this.computedCurrentPage,l=this.pageList.map((function(e){return e.number})),c=this.paginationParams,u=c.showFirstDots,d=c.showLastDots,h="fill"===this.align,f=[],p=function(e){return e===s},m=this.currentPage<1,v=function(n,r,s,l,c,u,d){var f=a||p(u)||m||n<1||n>o,v=n<1?1:n>o?o:n,_={disabled:f,page:v,index:v-1},g=t.normalizeSlot(s,_)||zl(l)||e(),y=e(f?"span":i?cY:"button",{staticClass:"page-link",class:{"flex-grow-1":!i&&!f&&h},props:f||!i?{}:t.linkProps(n),attrs:{role:i?null:"menuitem",type:i||f?null:"button",tabindex:f||i?null:"-1","aria-label":r,"aria-controls":t.ariaControls||null,"aria-disabled":f?"true":null},on:f?{}:{"!click":function(e){t.onClick(e,n)},keydown:HZ}},[g]);return e("li",{key:d,staticClass:"page-item",class:[{disabled:f,"flex-fill":h,"d-flex":h&&!i&&!f},c],attrs:{role:i?null:"presentation","aria-hidden":f?"true":null}},[y])},_=function(a){return e("li",{staticClass:"page-item",class:["disabled","bv-d-xs-down-none",h?"flex-fill":"",t.ellipsisClass],attrs:{role:"separator"},key:"ellipsis-".concat(a?"last":"first")},[e("span",{staticClass:"page-link"},[t.normalizeSlot(cs)||zl(t.ellipsisText)||e()])])},g=function(r,s){var l=r.number,c=p(l)&&!m,u=a?null:c||m&&0===s?"0":"-1",d={role:i?null:"menuitemradio",type:i||a?null:"button","aria-disabled":a?"true":null,"aria-controls":t.ariaControls||null,"aria-label":Oc(n)?n(l):"".concat(Ct(n)?n():n," ").concat(l),"aria-checked":i?null:c?"true":"false","aria-current":i&&c?"page":null,"aria-posinset":i?null:l,"aria-setsize":i?null:o,tabindex:i?null:u},f=zl(t.makePage(l)),v={page:l,index:l-1,content:f,active:c,disabled:a},_=e(a?"span":i?cY:"button",{props:a||!i?{}:t.linkProps(l),staticClass:"page-link",class:{"flex-grow-1":!i&&!a&&h},attrs:d,on:a?{}:{"!click":function(e){t.onClick(e,l)},keydown:HZ}},[t.normalizeSlot(Ws,v)||f]);return e("li",{staticClass:"page-item",class:[{disabled:a,active:c,"flex-fill":h,"d-flex":h&&!i&&!a},r.classes,t.pageClass],attrs:{role:i?null:"presentation"},key:"page-".concat(l)},[_])},y=e();this.firstNumber||this.hideGotoEndButtons||(y=v(1,this.labelFirstPage,ps,this.firstText,this.firstClass,1,"pagination-goto-first")),f.push(y),f.push(v(s-1,this.labelPrevPage,qs,this.prevText,this.prevClass,1,"pagination-goto-prev")),f.push(this.firstNumber&&1!==l[0]?g({number:1},0):e()),f.push(u?_(!1):e()),this.pageList.forEach((function(e,a){var n=u&&t.firstNumber&&1!==l[0]?1:0;f.push(g(e,a+n))})),f.push(d?_(!0):e()),f.push(this.lastNumber&&l[l.length-1]!==o?g({number:o},-1):e()),f.push(v(s+1,this.labelNextPage,Rs,this.nextText,this.nextClass,o,"pagination-goto-next"));var b=e();this.lastNumber||this.hideGotoEndButtons||(b=v(o,this.labelLastPage,ks,this.lastText,this.lastClass,o,"pagination-goto-last")),f.push(b);var M=e("ul",{staticClass:"pagination",class:["b-pagination",this.btnSize,this.alignment,this.styleClass],attrs:{role:i?null:"menubar","aria-disabled":a?"true":"false","aria-label":i?null:r||null},on:i?{}:{keydown:this.handleKeyNav},ref:"ul"},f);return i?e("nav",{attrs:{"aria-disabled":a?"true":null,"aria-hidden":a?"true":"false","aria-label":i&&r||null}},[M]):M}});function SZ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function VZ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?SZ(Object(a),!0).forEach((function(t){OZ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):SZ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function OZ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var TZ=20,DZ=0,CZ=function(e){return mu(fl(e)||TZ,1)},jZ=function(e){return mu(fl(e)||DZ,0)},xZ=Sc(sa(VZ(VZ({},kZ),{},{ariaControls:wc(Yo),perPage:wc(Go,TZ),totalRows:wc(Go,DZ)})),Cr),BZ=i["default"].extend({name:Cr,mixins:[AZ],props:xZ,computed:{numberOfPages:function(){var e=_u(jZ(this.totalRows)/CZ(this.perPage));return e<1?1:e},pageSizeNumberOfPages:function(){return{perPage:CZ(this.perPage),totalRows:jZ(this.totalRows),numberOfPages:this.numberOfPages}}},watch:{pageSizeNumberOfPages:function(e,t){Dt(t)||(e.perPage!==t.perPage&&e.totalRows===t.totalRows||e.numberOfPages!==t.numberOfPages&&this.currentPage>e.numberOfPages)&&(this.currentPage=1),this.localNumberOfPages=e.numberOfPages}},created:function(){var e=this;this.localNumberOfPages=this.numberOfPages;var t=fl(this[gZ],0);t>0?this.currentPage=t:this.$nextTick((function(){e.currentPage=0}))},methods:{onClick:function(e,t){var a=this;if(t!==this.currentPage){var n=e.target,r=new c$(Zi,{cancelable:!0,vueTarget:this,target:n});this.$emit(r.type,r,t),r.defaultPrevented||(this.currentPage=t,this.$emit(Li,this.currentPage),this.$nextTick((function(){Yl(n)&&a.$el.contains(n)?lc(n):a.focusCurrent()})))}},makePage:function(e){return e},linkProps:function(){return{}}}}),YZ=Pa({components:{BPagination:BZ}});function PZ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function IZ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?PZ(Object(a),!0).forEach((function(t){FZ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):PZ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function FZ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var EZ=function(e){return mu(fl(e,0),1)},$Z=ia(lY,["event","routerTag"]),RZ=Sc(sa(IZ(IZ(IZ({},kZ),$Z),{},{baseUrl:wc(Yo,"/"),linkGen:wc(Co),noPageDetect:wc(To,!1),numberOfPages:wc(Go,1,(function(e){var t=fl(e,0);return!(t<1)||(La('Prop "number-of-pages" must be a number greater than "0"',jr),!1)})),pageGen:wc(Co),pages:wc(Oo),useRouter:wc(To,!1)})),jr),NZ=i["default"].extend({name:jr,mixins:[AZ],props:RZ,computed:{isNav:function(){return!0},computedValue:function(){var e=fl(this.value,0);return e<1?null:e}},watch:{numberOfPages:function(){var e=this;this.$nextTick((function(){e.setNumberOfPages()}))},pages:function(){var e=this;this.$nextTick((function(){e.setNumberOfPages()}))}},created:function(){this.setNumberOfPages()},mounted:function(){var e=this;this.$router&&this.$watch("$route",(function(){e.$nextTick((function(){Ol((function(){e.guessCurrentPage()}))}))}))},methods:{setNumberOfPages:function(){var e=this;Pt(this.pages)&&this.pages.length>0?this.localNumberOfPages=this.pages.length:this.localNumberOfPages=EZ(this.numberOfPages),this.$nextTick((function(){e.guessCurrentPage()}))},onClick:function(e,t){var a=this;if(t!==this.currentPage){var n=e.currentTarget||e.target,r=new c$(Zi,{cancelable:!0,vueTarget:this,target:n});this.$emit(r.type,r,t),r.defaultPrevented||(Ol((function(){a.currentPage=t,a.$emit(Li,t)})),this.$nextTick((function(){cc(n)})))}},getPageInfo:function(e){if(!Pt(this.pages)||0===this.pages.length||Ot(this.pages[e-1])){var t="".concat(this.baseUrl).concat(e);return{link:this.useRouter?{path:t}:t,text:zl(e)}}var a=this.pages[e-1];if(It(a)){var n=a.link;return{link:It(n)?n:this.useRouter?{path:n}:n,text:zl(a.text||e)}}return{link:zl(a),text:zl(e)}},makePage:function(e){var t=this.pageGen,a=this.getPageInfo(e);return Oc(t)?t(e,a):a.text},makeLink:function(e){var t=this.linkGen,a=this.getPageInfo(e);return Oc(t)?t(e,a):a.link},linkProps:function(e){var t=kc($Z,this),a=this.makeLink(e);return this.useRouter||It(a)?t.to=a:t.href=a,t},resolveLink:function(){var e,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"";try{e=document.createElement("a"),e.href=Eu({to:t},"a","/","/"),document.body.appendChild(e);var a=e,n=a.pathname,r=a.hash,i=a.search;return document.body.removeChild(e),{path:n,hash:r,query:Bu(i)}}catch(o){try{e&&e.parentNode&&e.parentNode.removeChild(e)}catch(s){}return{}}},resolveRoute:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:"";try{var t=this.$router.resolve(e,this.$route).route;return{path:t.path,hash:t.hash,query:t.query}}catch(a){return{}}},guessCurrentPage:function(){var e=this.$router,t=this.$route,a=this.computedValue;if(!this.noPageDetect&&!a&&(Me||!Me&&e))for(var n=e&&t?{path:t.path,hash:t.hash,query:t.query}:{},r=Me?window.location||document.location:null,i=r?{path:r.pathname,hash:r.hash,query:Bu(r.search)}:{},o=1;!a&&o<=this.localNumberOfPages;o++){var s=this.makeLink(o);a=e&&(It(s)||this.useRouter)?EB(this.resolveRoute(s),n)?o:null:Me?EB(this.resolveLink(s),i)?o:null:-1}this.currentPage=a>0?a:0}}}),WZ=Pa({components:{BPaginationNav:NZ}}),UZ={AUTO:"auto",TOP:"top",RIGHT:"right",BOTTOM:"bottom",LEFT:"left",TOPLEFT:"top",TOPRIGHT:"top",RIGHTTOP:"right",RIGHTBOTTOM:"right",BOTTOMLEFT:"bottom",BOTTOMRIGHT:"bottom",LEFTTOP:"left",LEFTBOTTOM:"left"},GZ={AUTO:0,TOPLEFT:-1,TOP:0,TOPRIGHT:1,RIGHTTOP:-1,RIGHT:0,RIGHTBOTTOM:1,BOTTOMLEFT:-1,BOTTOM:0,BOTTOMRIGHT:1,LEFTTOP:-1,LEFT:0,LEFTBOTTOM:1},qZ={arrowPadding:wc(Go,6),boundary:wc([wt,Yo],"scrollParent"),boundaryPadding:wc(Go,5),fallbackPlacement:wc(Eo,"flip"),offset:wc(Go,0),placement:wc(Yo,"top"),target:wc([wt,Ht])},JZ=i["default"].extend({name:hi,props:qZ,data:function(){return{noFade:!1,localShow:!0,attachment:this.getAttachment(this.placement)}},computed:{templateType:function(){return"unknown"},popperConfig:function(){var e=this,t=this.placement;return{placement:this.getAttachment(t),modifiers:{offset:{offset:this.getOffset(t)},flip:{behavior:this.fallbackPlacement},arrow:{element:".arrow"},preventOverflow:{padding:this.boundaryPadding,boundariesElement:this.boundary}},onCreate:function(t){t.originalPlacement!==t.placement&&e.popperPlacementChange(t)},onUpdate:function(t){e.popperPlacementChange(t)}}}},created:function(){var e=this;this.$_popper=null,this.localShow=!0,this.$on(fo,(function(t){e.popperCreate(t)}));var t=function(){e.$nextTick((function(){Ol((function(){e.$destroy()}))}))};this.$parent.$once(Lo,t),this.$once($i,t)},beforeMount:function(){this.attachment=this.getAttachment(this.placement)},updated:function(){this.updatePopper()},beforeDestroy:function(){this.destroyPopper()},destroyed:function(){var e=this.$el;e&&e.parentNode&&e.parentNode.removeChild(e)},methods:{hide:function(){this.localShow=!1},getAttachment:function(e){return UZ[String(e).toUpperCase()]||"auto"},getOffset:function(e){if(!this.offset){var t=this.$refs.arrow||El(".arrow",this.$el),a=pl(nc(t).width,0)+pl(this.arrowPadding,0);switch(GZ[String(e).toUpperCase()]||0){case 1:return"+50%p - ".concat(a,"px");case-1:return"-50%p + ".concat(a,"px");default:return 0}}return this.offset},popperCreate:function(e){this.destroyPopper(),this.$_popper=new QE["a"](this.target,e,this.popperConfig)},destroyPopper:function(){this.$_popper&&this.$_popper.destroy(),this.$_popper=null},updatePopper:function(){this.$_popper&&this.$_popper.scheduleUpdate()},popperPlacementChange:function(e){this.attachment=this.getAttachment(e.placement)},renderTemplate:function(e){return e("div")}},render:function(e){var t=this,a=this.noFade;return e(eu,{props:{appear:!0,noFade:a},on:{beforeEnter:function(e){return t.$emit(fo,e)},afterEnter:function(e){return t.$emit(po,e)},beforeLeave:function(e){return t.$emit(Ri,e)},afterLeave:function(e){return t.$emit($i,e)}}},[this.localShow?this.renderTemplate(e):e()])}});function KZ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function XZ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?KZ(Object(a),!0).forEach((function(t){ZZ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):KZ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function ZZ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var QZ={html:wc(To,!1),id:wc(Yo)},eQ=i["default"].extend({name:vi,extends:JZ,mixins:[YJ],props:QZ,data:function(){return{title:"",content:"",variant:null,customClass:null,interactive:!0}},computed:{templateType:function(){return"tooltip"},templateClasses:function(){var e,t=this.variant,a=this.attachment,n=this.templateType;return[(e={noninteractive:!this.interactive},ZZ(e,"b-".concat(n,"-").concat(t),t),ZZ(e,"bs-".concat(n,"-").concat(a),a),e),this.customClass]},templateAttributes:function(){var e=this.id;return XZ(XZ({},this.$parent.$parent.$attrs),{},{id:e,role:"tooltip",tabindex:"-1"},this.scopedStyleAttrs)},templateListeners:function(){var e=this;return{mouseenter:function(t){e.$emit(Gi,t)},mouseleave:function(t){e.$emit(qi,t)},focusin:function(t){e.$emit(Ii,t)},focusout:function(t){e.$emit(Fi,t)}}}},methods:{renderTemplate:function(e){var t=this.title,a=Ct(t)?t({}):t,n=this.html&&!Ct(t)?{innerHTML:t}:{};return e("div",{staticClass:"tooltip b-tooltip",class:this.templateClasses,attrs:this.templateAttributes,on:this.templateListeners},[e("div",{staticClass:"arrow",ref:"arrow"}),e("div",{staticClass:"tooltip-inner",domProps:n},[a])])}}});function tQ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function aQ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?tQ(Object(a),!0).forEach((function(t){nQ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):tQ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function nQ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var rQ,iQ,oQ=".modal-content",sQ=$c(Mr,$i),lQ=".b-sidebar",cQ=[oQ,lQ].join(", "),uQ="dropdown",dQ=".dropdown-menu.show",hQ="data-original-title",fQ={title:"",content:"",variant:null,customClass:null,triggers:"",placement:"auto",fallbackPlacement:"flip",target:null,container:null,noFade:!1,boundary:"scrollParent",boundaryPadding:5,offset:0,delay:0,arrowPadding:6,interactive:!0,disabled:!1,id:null,html:!1},pQ=i["default"].extend({name:mi,mixins:[qB],data:function(){return aQ(aQ({},fQ),{},{activeTrigger:{hover:!1,click:!1,focus:!1},localShow:!1})},computed:{templateType:function(){return"tooltip"},computedId:function(){return this.id||"__bv_".concat(this.templateType,"_").concat(this[Ka],"__")},computedDelay:function(){var e={show:0,hide:0};return Ft(this.delay)?(e.show=mu(fl(this.delay.show,0),0),e.hide=mu(fl(this.delay.hide,0),0)):(Bt(this.delay)||xt(this.delay))&&(e.show=e.hide=mu(fl(this.delay,0),0)),e},computedTriggers:function(){return cl(this.triggers).filter(ya).join(" ").trim().toLowerCase().split(/\s+/).sort()},isWithActiveTrigger:function(){for(var e in this.activeTrigger)if(this.activeTrigger[e])return!0;return!1},computedTemplateData:function(){var e=this.title,t=this.content,a=this.variant,n=this.customClass,r=this.noFade,i=this.interactive;return{title:e,content:t,variant:a,customClass:n,noFade:r,interactive:i}}},watch:{computedTriggers:function(e,t){var a=this;EB(e,t)||this.$nextTick((function(){a.unListen(),t.forEach((function(t){ll(e,t)||a.activeTrigger[t]&&(a.activeTrigger[t]=!1)})),a.listen()}))},computedTemplateData:function(){this.handleTemplateUpdate()},title:function(e,t){e===t||e||this.hide()},disabled:function(e){e?this.disable():this.enable()}},created:function(){var e=this;this.$_tip=null,this.$_hoverTimeout=null,this.$_hoverState="",this.$_visibleInterval=null,this.$_enabled=!this.disabled,this.$_noop=MF.bind(this),this.$parent&&this.$parent.$once(zo,(function(){e.$nextTick((function(){Ol((function(){e.$destroy()}))}))})),this.$nextTick((function(){var t=e.getTarget();t&&Nl(document.body,t)?(e.scopeId=xJ(e.$parent),e.listen()):La(xt(e.target)?'Unable to find target element by ID "#'.concat(e.target,'" in document.'):"The provided target is no valid HTML element.",e.templateType)}))},updated:function(){this.$nextTick(this.handleTemplateUpdate)},deactivated:function(){this.forceHide()},beforeDestroy:function(){this.unListen(),this.setWhileOpenListeners(!1),this.clearHoverTimeout(),this.clearVisibilityInterval(),this.destroyTemplate(),this.$_noop=null},methods:{getTemplate:function(){return eQ},updateData:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},a=!1;ea(fQ).forEach((function(n){Ot(t[n])||e[n]===t[n]||(e[n]=t[n],"title"===n&&(a=!0))})),a&&this.localShow&&this.fixTitle()},createTemplateAndShow:function(){var e=this.getContainer(),t=this.getTemplate(),a=this.$_tip=new t({parent:this,propsData:{id:this.computedId,html:this.html,placement:this.placement,fallbackPlacement:this.fallbackPlacement,target:this.getPlacementTarget(),boundary:this.getBoundary(),offset:fl(this.offset,0),arrowPadding:fl(this.arrowPadding,0),boundaryPadding:fl(this.boundaryPadding,0)}});this.handleTemplateUpdate(),a.$once(fo,this.onTemplateShow),a.$once(po,this.onTemplateShown),a.$once(Ri,this.onTemplateHide),a.$once($i,this.onTemplateHidden),a.$once(Lo,this.destroyTemplate),a.$on(Ii,this.handleEvent),a.$on(Fi,this.handleEvent),a.$on(Gi,this.handleEvent),a.$on(qi,this.handleEvent),a.$mount(e.appendChild(document.createElement("div")))},hideTemplate:function(){this.$_tip&&this.$_tip.hide(),this.clearActiveTriggers(),this.$_hoverState=""},destroyTemplate:function(){this.setWhileOpenListeners(!1),this.clearHoverTimeout(),this.$_hoverState="",this.clearActiveTriggers(),this.localPlacementTarget=null;try{this.$_tip.$destroy()}catch(e){}this.$_tip=null,this.removeAriaDescribedby(),this.restoreTitle(),this.localShow=!1},getTemplateElement:function(){return this.$_tip?this.$_tip.$el:null},handleTemplateUpdate:function(){var e=this,t=this.$_tip;if(t){var a=["title","content","variant","customClass","noFade","interactive"];a.forEach((function(a){t[a]!==e[a]&&(t[a]=e[a])}))}},show:function(){var e=this.getTarget();if(e&&Nl(document.body,e)&&Yl(e)&&!this.dropdownOpen()&&(!Dt(this.title)&&""!==this.title||!Dt(this.content)&&""!==this.content)&&!this.$_tip&&!this.localShow){this.localShow=!0;var t=this.buildEvent(fo,{cancelable:!0});this.emitEvent(t),t.defaultPrevented?this.destroyTemplate():(this.fixTitle(),this.addAriaDescribedby(),this.createTemplateAndShow())}},hide:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0],t=this.getTemplateElement();if(t&&this.localShow){var a=this.buildEvent(Ri,{cancelable:!e});this.emitEvent(a),a.defaultPrevented||this.hideTemplate()}else this.restoreTitle()},forceHide:function(){var e=this.getTemplateElement();e&&this.localShow&&(this.setWhileOpenListeners(!1),this.clearHoverTimeout(),this.$_hoverState="",this.clearActiveTriggers(),this.$_tip&&(this.$_tip.noFade=!0),this.hide(!0))},enable:function(){this.$_enabled=!0,this.emitEvent(this.buildEvent(xi))},disable:function(){this.$_enabled=!1,this.emitEvent(this.buildEvent(Ti))},onTemplateShow:function(){this.setWhileOpenListeners(!0)},onTemplateShown:function(){var e=this.$_hoverState;this.$_hoverState="","out"===e&&this.leave(null),this.emitEvent(this.buildEvent(po))},onTemplateHide:function(){this.setWhileOpenListeners(!1)},onTemplateHidden:function(){this.destroyTemplate(),this.emitEvent(this.buildEvent($i))},getTarget:function(){var e=this.target;return xt(e)?e=Wl(e.replace(/^#/,"")):Ct(e)?e=e():e&&(e=e.$el||e),Cl(e)?e:null},getPlacementTarget:function(){return this.getTarget()},getTargetId:function(){var e=this.getTarget();return e&&e.id?e.id:null},getContainer:function(){var e=!!this.container&&(this.container.$el||this.container),t=document.body,a=this.getTarget();return!1===e?Rl(cQ,a)||t:xt(e)&&Wl(e.replace(/^#/,""))||t},getBoundary:function(){return this.boundary?this.boundary.$el||this.boundary:"scrollParent"},isInModal:function(){var e=this.getTarget();return e&&Rl(oQ,e)},isDropdown:function(){var e=this.getTarget();return e&&ql(e,uQ)},dropdownOpen:function(){var e=this.getTarget();return this.isDropdown()&&e&&El(dQ,e)},clearHoverTimeout:function(){clearTimeout(this.$_hoverTimeout),this.$_hoverTimeout=null},clearVisibilityInterval:function(){clearInterval(this.$_visibleInterval),this.$_visibleInterval=null},clearActiveTriggers:function(){for(var e in this.activeTrigger)this.activeTrigger[e]=!1},addAriaDescribedby:function(){var e=this.getTarget(),t=Xl(e,"aria-describedby")||"";t=t.split(/\s+/).concat(this.computedId).join(" ").trim(),Jl(e,"aria-describedby",t)},removeAriaDescribedby:function(){var e=this,t=this.getTarget(),a=Xl(t,"aria-describedby")||"";a=a.split(/\s+/).filter((function(t){return t!==e.computedId})).join(" ").trim(),a?Jl(t,"aria-describedby",a):Kl(t,"aria-describedby")},fixTitle:function(){var e=this.getTarget();if(Zl(e,"title")){var t=Xl(e,"title");Jl(e,"title",""),t&&Jl(e,hQ,t)}},restoreTitle:function(){var e=this.getTarget();if(Zl(e,hQ)){var t=Xl(e,hQ);Kl(e,hQ),t&&Jl(e,"title",t)}},buildEvent:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return new c$(e,aQ({cancelable:!1,target:this.getTarget(),relatedTarget:this.getTemplateElement()||null,componentId:this.computedId,vueTarget:this},t))},emitEvent:function(e){var t=e.type;this.emitOnRoot($c(this.templateType,t),e),this.$emit(t,e)},listen:function(){var e=this,t=this.getTarget();t&&(this.setRootListener(!0),this.computedTriggers.forEach((function(a){"click"===a?Yc(t,"click",e.handleEvent,So):"focus"===a?(Yc(t,"focusin",e.handleEvent,So),Yc(t,"focusout",e.handleEvent,So)):"blur"===a?Yc(t,"focusout",e.handleEvent,So):"hover"===a&&(Yc(t,"mouseenter",e.handleEvent,So),Yc(t,"mouseleave",e.handleEvent,So))}),this))},unListen:function(){var e=this,t=["click","focusin","focusout","mouseenter","mouseleave"],a=this.getTarget();this.setRootListener(!1),t.forEach((function(t){a&&Pc(a,t,e.handleEvent,So)}),this)},setRootListener:function(e){var t=this.$root;if(t){var a=e?"$on":"$off",n=this.templateType;t[a](Rc(n,Ri),this.doHide),t[a](Rc(n,fo),this.doShow),t[a](Rc(n,Oi),this.doDisable),t[a](Rc(n,ji),this.doEnable)}},setWhileOpenListeners:function(e){this.setModalListener(e),this.setDropdownListener(e),this.visibleCheck(e),this.setOnTouchStartListener(e)},visibleCheck:function(e){var t=this;this.clearVisibilityInterval();var a=this.getTarget();e&&(this.$_visibleInterval=setInterval((function(){var e=t.getTemplateElement();!e||!t.localShow||a.parentNode&&Yl(a)||t.forceHide()}),100))},setModalListener:function(e){this.isInModal()&&this.$root[e?"$on":"$off"](sQ,this.forceHide)},setOnTouchStartListener:function(e){var t=this;"ontouchstart"in document.documentElement&&sl(document.body.children).forEach((function(a){Ic(e,a,"mouseover",t.$_noop)}))},setDropdownListener:function(e){var t=this.getTarget();t&&this.$root&&this.isDropdown&&t.__vue__&&t.__vue__[e?"$on":"$off"](po,this.forceHide)},handleEvent:function(e){var t=this.getTarget();if(t&&!Pl(t)&&this.$_enabled&&!this.dropdownOpen()){var a=e.type,n=this.computedTriggers;if("click"===a&&ll(n,"click"))this.click(e);else if("mouseenter"===a&&ll(n,"hover"))this.enter(e);else if("focusin"===a&&ll(n,"focus"))this.enter(e);else if("focusout"===a&&(ll(n,"focus")||ll(n,"blur"))||"mouseleave"===a&&ll(n,"hover")){var r=this.getTemplateElement(),i=e.target,o=e.relatedTarget;if(r&&Nl(r,i)&&Nl(t,o)||r&&Nl(t,i)&&Nl(r,o)||r&&Nl(r,i)&&Nl(r,o)||Nl(t,i)&&Nl(t,o))return;this.leave(e)}}},doHide:function(e){e&&this.getTargetId()!==e&&this.computedId!==e||this.forceHide()},doShow:function(e){e&&this.getTargetId()!==e&&this.computedId!==e||this.show()},doDisable:function(e){e&&this.getTargetId()!==e&&this.computedId!==e||this.disable()},doEnable:function(e){e&&this.getTargetId()!==e&&this.computedId!==e||this.enable()},click:function(e){this.$_enabled&&!this.dropdownOpen()&&(lc(e.currentTarget),this.activeTrigger.click=!this.activeTrigger.click,this.isWithActiveTrigger?this.enter(null):this.leave(null))},toggle:function(){this.$_enabled&&!this.dropdownOpen()&&(this.localShow?this.leave(null):this.enter(null))},enter:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null;t&&(this.activeTrigger["focusin"===t.type?"focus":"hover"]=!0),this.localShow||"in"===this.$_hoverState?this.$_hoverState="in":(this.clearHoverTimeout(),this.$_hoverState="in",this.computedDelay.show?(this.fixTitle(),this.$_hoverTimeout=setTimeout((function(){"in"===e.$_hoverState?e.show():e.localShow||e.restoreTitle()}),this.computedDelay.show)):this.show())},leave:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null;t&&(this.activeTrigger["focusout"===t.type?"focus":"hover"]=!1,"focusout"===t.type&&ll(this.computedTriggers,"blur")&&(this.activeTrigger.click=!1,this.activeTrigger.hover=!1)),this.isWithActiveTrigger||(this.clearHoverTimeout(),this.$_hoverState="out",this.computedDelay.hide?this.$_hoverTimeout=setTimeout((function(){"out"===e.$_hoverState&&e.hide()}),this.computedDelay.hide):this.hide())}}});function mQ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function vQ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?mQ(Object(a),!0).forEach((function(t){_Q(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):mQ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function _Q(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var gQ="disabled",yQ=wo+gQ,bQ="show",MQ=wo+bQ,zQ=Sc((rQ={boundary:wc([wt,xo,Yo],"scrollParent"),boundaryPadding:wc(Go,50),container:wc([wt,xo,Yo]),customClass:wc(Yo),delay:wc(qo,50)},_Q(rQ,gQ,wc(To,!1)),_Q(rQ,"fallbackPlacement",wc(Eo,"flip")),_Q(rQ,"id",wc(Yo)),_Q(rQ,"noFade",wc(To,!1)),_Q(rQ,"noninteractive",wc(To,!1)),_Q(rQ,"offset",wc(Go,0)),_Q(rQ,"placement",wc(Yo,"top")),_Q(rQ,bQ,wc(To,!1)),_Q(rQ,"target",wc([wt,Ht,Co,xo,Yo],void 0,!0)),_Q(rQ,"title",wc(Yo)),_Q(rQ,"triggers",wc(Eo,"hover focus")),_Q(rQ,"variant",wc(Yo)),rQ),ii),LQ=i["default"].extend({name:ii,mixins:[xc],inheritAttrs:!1,props:zQ,data:function(){return{localShow:this[bQ],localTitle:"",localContent:""}},computed:{templateData:function(){return vQ({title:this.localTitle,content:this.localContent,interactive:!this.noninteractive},ra(this.$props,["boundary","boundaryPadding","container","customClass","delay","fallbackPlacement","id","noFade","offset","placement","target","target","triggers","variant",gQ]))},templateTitleContent:function(){var e=this.title,t=this.content;return{title:e,content:t}}},watch:(iQ={},_Q(iQ,bQ,(function(e,t){e!==t&&e!==this.localShow&&this.$_toolpop&&(e?this.$_toolpop.show():this.$_toolpop.forceHide())})),_Q(iQ,gQ,(function(e){e?this.doDisable():this.doEnable()})),_Q(iQ,"localShow",(function(e){this.$emit(MQ,e)})),_Q(iQ,"templateData",(function(){var e=this;this.$nextTick((function(){e.$_toolpop&&e.$_toolpop.updateData(e.templateData)}))})),_Q(iQ,"templateTitleContent",(function(){this.$nextTick(this.updateContent)})),iQ),created:function(){this.$_toolpop=null},updated:function(){this.$nextTick(this.updateContent)},beforeDestroy:function(){this.$off(Xi,this.doOpen),this.$off(ki,this.doClose),this.$off(Oi,this.doDisable),this.$off(ji,this.doEnable),this.$_toolpop&&(this.$_toolpop.$destroy(),this.$_toolpop=null)},mounted:function(){var e=this;this.$nextTick((function(){var t=e.getComponent();e.updateContent();var a=xJ(e)||xJ(e.$parent),n=e.$_toolpop=new t({parent:e,_scopeId:a||void 0});n.updateData(e.templateData),n.$on(fo,e.onShow),n.$on(po,e.onShown),n.$on(Ri,e.onHide),n.$on($i,e.onHidden),n.$on(Ti,e.onDisabled),n.$on(xi,e.onEnabled),e[gQ]&&e.doDisable(),e.$on(Xi,e.doOpen),e.$on(ki,e.doClose),e.$on(Oi,e.doDisable),e.$on(ji,e.doEnable),e.localShow&&n.show()}))},methods:{getComponent:function(){return pQ},updateContent:function(){this.setTitle(this.normalizeSlot()||this.title)},setTitle:function(e){e=Dt(e)?"":e,this.localTitle!==e&&(this.localTitle=e)},setContent:function(e){e=Dt(e)?"":e,this.localContent!==e&&(this.localContent=e)},onShow:function(e){this.$emit(fo,e),e&&(this.localShow=!e.defaultPrevented)},onShown:function(e){this.localShow=!0,this.$emit(po,e)},onHide:function(e){this.$emit(Ri,e)},onHidden:function(e){this.$emit($i,e),this.localShow=!1},onDisabled:function(e){e&&e.type===Ti&&(this.$emit(yQ,!0),this.$emit(Ti,e))},onEnabled:function(e){e&&e.type===xi&&(this.$emit(yQ,!1),this.$emit(xi,e))},doOpen:function(){!this.localShow&&this.$_toolpop&&this.$_toolpop.show()},doClose:function(){this.localShow&&this.$_toolpop&&this.$_toolpop.hide()},doDisable:function(){this.$_toolpop&&this.$_toolpop.disable()},doEnable:function(){this.$_toolpop&&this.$_toolpop.enable()}},render:function(e){return e()}}),wQ=i["default"].extend({name:di,extends:eQ,computed:{templateType:function(){return"popover"}},methods:{renderTemplate:function(e){var t=this.title,a=this.content,n=Ct(t)?t({}):t,r=Ct(a)?a({}):a,i=this.html&&!Ct(t)?{innerHTML:t}:{},o=this.html&&!Ct(a)?{innerHTML:a}:{};return e("div",{staticClass:"popover b-popover",class:this.templateClasses,attrs:this.templateAttributes,on:this.templateListeners},[e("div",{staticClass:"arrow",ref:"arrow"}),Dt(n)||""===n?e():e("h3",{staticClass:"popover-header",domProps:i},[n]),Dt(r)||""===r?e():e("div",{staticClass:"popover-body",domProps:o},[r])])}}}),HQ=i["default"].extend({name:ui,extends:pQ,computed:{templateType:function(){return"popover"}},methods:{getTemplate:function(){return wQ}}});function kQ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function AQ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?kQ(Object(a),!0).forEach((function(t){SQ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):kQ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function SQ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var VQ=Sc(sa(AQ(AQ({},zQ),{},{content:wc(Yo),placement:wc(Yo,"right"),triggers:wc(Eo,Hi)})),xr),OQ=i["default"].extend({name:xr,extends:LQ,inheritAttrs:!1,props:VQ,methods:{getComponent:function(){return HQ},updateContent:function(){this.setContent(this.normalizeSlot()||this.content),this.setTitle(this.normalizeSlot(nl)||this.title)}}});function TQ(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function DQ(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?TQ(Object(a),!0).forEach((function(t){CQ(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):TQ(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function CQ(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var jQ="__BV_Popover__",xQ="click",BQ={focus:!0,hover:!0,click:!0,blur:!0,manual:!0},YQ=/^html$/i,PQ=/^nofade$/i,IQ=/^(auto|top(left|right)?|bottom(left|right)?|left(top|bottom)?|right(top|bottom)?)$/i,FQ=/^(window|viewport|scrollParent)$/i,EQ=/^d\d+$/i,$Q=/^ds\d+$/i,RQ=/^dh\d+$/i,NQ=/^o-?\d+$/i,WQ=/^v-.+$/i,UQ=/\s+/,GQ=function(e,t){var a={title:void 0,content:void 0,trigger:"",placement:"right",fallbackPlacement:"flip",container:!1,animation:!0,offset:0,disabled:!1,id:null,html:!1,delay:fc(xr,"delay",50),boundary:String(fc(xr,"boundary","scrollParent")),boundaryPadding:fl(fc(xr,"boundaryPadding",5),0),variant:fc(xr,"variant"),customClass:fc(xr,"customClass")};if(xt(e.value)||Bt(e.value)||Ct(e.value)?a.content=e.value:Ft(e.value)&&(a=DQ(DQ({},a),e.value)),e.arg&&(a.container="#".concat(e.arg)),Ot(a.title)){var n=t.data||{};a.title=n.attrs&&!Dt(n.attrs.title)?n.attrs.title:void 0}Ft(a.delay)||(a.delay={show:fl(a.delay,0),hide:fl(a.delay,0)}),ea(e.modifiers).forEach((function(e){if(YQ.test(e))a.html=!0;else if(PQ.test(e))a.animation=!1;else if(IQ.test(e))a.placement=e;else if(FQ.test(e))e="scrollparent"===e?"scrollParent":e,a.boundary=e;else if(EQ.test(e)){var t=fl(e.slice(1),0);a.delay.show=t,a.delay.hide=t}else $Q.test(e)?a.delay.show=fl(e.slice(2),0):RQ.test(e)?a.delay.hide=fl(e.slice(2),0):NQ.test(e)?a.offset=fl(e.slice(1),0):WQ.test(e)&&(a.variant=e.slice(2)||null)}));var r={};return cl(a.trigger||"").filter(ya).join(" ").trim().toLowerCase().split(UQ).forEach((function(e){BQ[e]&&(r[e]=!0)})),ea(e.modifiers).forEach((function(e){e=e.toLowerCase(),BQ[e]&&(r[e]=!0)})),a.trigger=ea(r).join(" "),"blur"===a.trigger&&(a.trigger="focus"),a.trigger||(a.trigger=xQ),a},qQ=function(e,t,a){if(Me){var n=GQ(t,a);if(!e[jQ]){var r=a.context;e[jQ]=new HQ({parent:r,_scopeId:xJ(r,void 0)}),e[jQ].__bv_prev_data__={},e[jQ].$on(fo,(function(){var t={};Ct(n.title)&&(t.title=n.title(e)),Ct(n.content)&&(t.content=n.content(e)),ea(t).length>0&&e[jQ].updateData(t)}))}var i={title:n.title,content:n.content,triggers:n.trigger,placement:n.placement,fallbackPlacement:n.fallbackPlacement,variant:n.variant,customClass:n.customClass,container:n.container,boundary:n.boundary,delay:n.delay,offset:n.offset,noFade:!n.animation,id:n.id,disabled:n.disabled,html:n.html},o=e[jQ].__bv_prev_data__;if(e[jQ].__bv_prev_data__=i,!EB(i,o)){var s={target:e};ea(i).forEach((function(t){i[t]!==o[t]&&(s[t]="title"!==t&&"content"!==t||!Ct(i[t])?i[t]:i[t](e))})),e[jQ].updateData(s)}}},JQ=function(e){e[jQ]&&(e[jQ].$destroy(),e[jQ]=null),delete e[jQ]},KQ={bind:function(e,t,a){qQ(e,t,a)},componentUpdated:function(e,t,a){a.context.$nextTick((function(){qQ(e,t,a)}))},unbind:function(e){JQ(e)}},XQ=Pa({directives:{VBPopover:KQ}}),ZQ=Pa({components:{BPopover:OQ},plugins:{VBPopoverPlugin:XQ}}),QQ=Sc({animated:wc(To,null),label:wc(Yo),labelHtml:wc(Yo),max:wc(Go,null),precision:wc(Go,null),showProgress:wc(To,null),showValue:wc(To,null),striped:wc(To,null),value:wc(Go,0),variant:wc(Yo)},Yr),e0=i["default"].extend({name:Yr,mixins:[xc],inject:{bvProgress:{default:function(){return{}}}},props:QQ,computed:{progressBarClasses:function(){var e=this.computedAnimated,t=this.computedVariant;return[t?"bg-".concat(t):"",this.computedStriped||e?"progress-bar-striped":"",e?"progress-bar-animated":""]},progressBarStyles:function(){return{width:this.computedValue/this.computedMax*100+"%"}},computedValue:function(){return pl(this.value,0)},computedMax:function(){var e=pl(this.max)||pl(this.bvProgress.max,0);return e>0?e:100},computedPrecision:function(){return mu(fl(this.precision,fl(this.bvProgress.precision,0)),0)},computedProgress:function(){var e=this.computedPrecision,t=yu(10,e);return ml(100*t*this.computedValue/this.computedMax/t,e)},computedVariant:function(){return this.variant||this.bvProgress.variant},computedStriped:function(){return jt(this.striped)?this.striped:this.bvProgress.striped||!1},computedAnimated:function(){return jt(this.animated)?this.animated:this.bvProgress.animated||!1},computedShowProgress:function(){return jt(this.showProgress)?this.showProgress:this.bvProgress.showProgress||!1},computedShowValue:function(){return jt(this.showValue)?this.showValue:this.bvProgress.showValue||!1}},render:function(e){var t,a=this.label,n=this.labelHtml,r=this.computedValue,i=this.computedPrecision,o={};return this.hasNormalizedSlot()?t=this.normalizeSlot():a||n?o=UY(n,a):this.computedShowProgress?t=this.computedProgress:this.computedShowValue&&(t=ml(r,i)),e("div",{staticClass:"progress-bar",class:this.progressBarClasses,style:this.progressBarStyles,attrs:{role:"progressbar","aria-valuemin":"0","aria-valuemax":zl(this.computedMax),"aria-valuenow":ml(r,i)},domProps:o},t)}});function t0(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function a0(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?t0(Object(a),!0).forEach((function(t){n0(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):t0(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function n0(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var r0,i0=ia(QQ,["label","labelHtml"]),o0=Sc(sa(a0(a0({},i0),{},{animated:wc(To,!1),height:wc(Yo),max:wc(Go,100),precision:wc(Go,0),showProgress:wc(To,!1),showValue:wc(To,!1),striped:wc(To,!1)})),Br),s0=i["default"].extend({name:Br,mixins:[xc],provide:function(){return{bvProgress:this}},props:o0,computed:{progressHeight:function(){return{height:this.height||null}}},render:function(e){var t=this.normalizeSlot();return t||(t=e(e0,{props:kc(i0,this.$props)})),e("div",{staticClass:"progress",style:this.progressHeight},[t])}}),l0=Pa({components:{BProgress:s0,BProgressBar:e0}});function c0(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function u0(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?c0(Object(a),!0).forEach((function(t){d0(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):c0(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function d0(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var h0="b-sidebar",f0=Rc(wn,"request-state"),p0=Rc(wn,"toggle"),m0=$c(wn,"state"),v0=$c(wn,"sync-state"),_0=Dc("visible",{type:To,defaultValue:!1,event:Li}),g0=_0.mixin,y0=_0.props,b0=_0.prop,M0=_0.event,z0=Sc(sa(u0(u0(u0({},XP),y0),{},{ariaLabel:wc(Yo),ariaLabelledby:wc(Yo),backdrop:wc(To,!1),backdropVariant:wc(Yo,"dark"),bgVariant:wc(Yo,"light"),bodyClass:wc(Fo),closeLabel:wc(Yo),footerClass:wc(Fo),footerTag:wc(Yo,"footer"),headerClass:wc(Fo),headerTag:wc(Yo,"header"),lazy:wc(To,!1),noCloseOnBackdrop:wc(To,!1),noCloseOnEsc:wc(To,!1),noCloseOnRouteChange:wc(To,!1),noEnforceFocus:wc(To,!1),noHeader:wc(To,!1),noHeaderClose:wc(To,!1),noSlide:wc(To,!1),right:wc(To,!1),shadow:wc(No,!1),sidebarClass:wc(Fo),tag:wc(Yo,"div"),textVariant:wc(Yo,"dark"),title:wc(Yo),width:wc(Yo),zIndex:wc(Go)})),Ir),L0=function(e,t){var a=t.normalizeSlot(nl,t.slotScope)||t.title;return a?e("strong",{attrs:{id:t.safeId("__title__")}},[a]):e("span")},w0=function(e,t){if(t.noHeaderClose)return e();var a=t.closeLabel,n=t.textVariant,r=t.hide;return e(Uc,{props:{ariaLabel:a,textVariant:n},on:{click:r},ref:"close-button"},[t.normalizeSlot(_s)||e(sB)])},H0=function(e,t){if(t.noHeader)return e();var a=t.normalizeSlot(vs,t.slotScope);if(!a){var n=L0(e,t),r=w0(e,t);a=t.right?[r,n]:[n,r]}return e(t.headerTag,{staticClass:"".concat(h0,"-header"),class:t.headerClass,key:"header"},a)},k0=function(e,t){return e("div",{staticClass:"".concat(h0,"-body"),class:t.bodyClass,key:"body"},[t.normalizeSlot(is,t.slotScope)])},A0=function(e,t){var a=t.normalizeSlot(ms,t.slotScope);return a?e(t.footerTag,{staticClass:"".concat(h0,"-footer"),class:t.footerClass,key:"footer"},[a]):e()},S0=function(e,t){var a=H0(e,t);return t.lazy&&!t.isOpen?a:[a,k0(e,t),A0(e,t)]},V0=function(e,t){if(!t.backdrop)return e();var a=t.backdropVariant;return e("div",{directives:[{name:"show",value:t.localShow}],staticClass:"b-sidebar-backdrop",class:d0({},"bg-".concat(a),a),on:{click:t.onBackdropClick}})},O0=i["default"].extend({name:Ir,mixins:[UB,ZP,g0,qB,xc],inheritAttrs:!1,props:z0,data:function(){var e=!!this[b0];return{localShow:e,isOpen:e}},computed:{transitionProps:function(){return this.noSlide?{css:!0}:{css:!0,enterClass:"",enterActiveClass:"slide",enterToClass:"show",leaveClass:"show",leaveActiveClass:"slide",leaveToClass:""}},slotScope:function(){var e=this.hide,t=this.right,a=this.localShow;return{hide:e,right:t,visible:a}},hasTitle:function(){var e=this.$scopedSlots,t=this.$slots;return!this.noHeader&&!this.hasNormalizedSlot(vs)&&!(!this.normalizeSlot(nl,this.slotScope,e,t)&&!this.title)},titleId:function(){return this.hasTitle?this.safeId("__title__"):null},computedAttrs:function(){return u0(u0({},this.bvAttrs),{},{id:this.safeId(),tabindex:"-1",role:"dialog","aria-modal":this.backdrop?"true":"false","aria-hidden":this.localShow?null:"true","aria-label":this.ariaLabel||null,"aria-labelledby":this.ariaLabelledby||this.titleId||null})}},watch:(r0={},d0(r0,b0,(function(e,t){e!==t&&(this.localShow=e)})),d0(r0,"localShow",(function(e,t){e!==t&&(this.emitState(e),this.$emit(M0,e))})),d0(r0,"$route",(function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};this.noCloseOnRouteChange||e.fullPath===t.fullPath||this.hide()})),r0),created:function(){this.$_returnFocusEl=null},mounted:function(){var e=this;this.listenOnRoot(p0,this.handleToggle),this.listenOnRoot(f0,this.handleSync),this.$nextTick((function(){e.emitState(e.localShow)}))},activated:function(){this.emitSync()},beforeDestroy:function(){this.localShow=!1,this.$_returnFocusEl=null},methods:{hide:function(){this.localShow=!1},emitState:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.localShow;this.emitOnRoot(m0,this.safeId(),e)},emitSync:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:this.localShow;this.emitOnRoot(v0,this.safeId(),e)},handleToggle:function(e){e&&e===this.safeId()&&(this.localShow=!this.localShow)},handleSync:function(e){var t=this;e&&e===this.safeId()&&this.$nextTick((function(){t.emitSync(t.localShow)}))},onKeydown:function(e){var t=e.keyCode;!this.noCloseOnEsc&&t===DB&&this.localShow&&this.hide()},onBackdropClick:function(){this.localShow&&!this.noCloseOnBackdrop&&this.hide()},onTopTrapFocus:function(){var e=sc(this.$refs.content);this.enforceFocus(e.reverse()[0])},onBottomTrapFocus:function(){var e=sc(this.$refs.content);this.enforceFocus(e[0])},onBeforeEnter:function(){this.$_returnFocusEl=jl(Me?[document.body]:[]),this.isOpen=!0},onAfterEnter:function(e){Nl(e,jl())||this.enforceFocus(e),this.$emit(po)},onAfterLeave:function(){this.enforceFocus(this.$_returnFocusEl),this.$_returnFocusEl=null,this.isOpen=!1,this.$emit($i)},enforceFocus:function(e){this.noEnforceFocus||lc(e)}},render:function(e){var t,a=this.bgVariant,n=this.width,r=this.textVariant,i=this.localShow,o=""===this.shadow||this.shadow,s=e(this.tag,{staticClass:h0,class:[(t={shadow:!0===o},d0(t,"shadow-".concat(o),o&&!0!==o),d0(t,"".concat(h0,"-right"),this.right),d0(t,"bg-".concat(a),a),d0(t,"text-".concat(r),r),t),this.sidebarClass],style:{width:n},attrs:this.computedAttrs,directives:[{name:"show",value:i}],ref:"content"},[S0(e,this)]);s=e("transition",{props:this.transitionProps,on:{beforeEnter:this.onBeforeEnter,afterEnter:this.onAfterEnter,afterLeave:this.onAfterLeave}},[s]);var l=e(eu,{props:{noFade:this.noSlide}},[V0(e,this)]),c=e(),u=e();return this.backdrop&&i&&(c=e("div",{attrs:{tabindex:"0"},on:{focus:this.onTopTrapFocus}}),u=e("div",{attrs:{tabindex:"0"},on:{focus:this.onBottomTrapFocus}})),e("div",{staticClass:"b-sidebar-outer",style:{zIndex:this.zIndex},attrs:{tabindex:"-1"},on:{keydown:this.onKeydown}},[c,s,u,l])}}),T0=Pa({components:{BSidebar:O0},plugins:{VBTogglePlugin:XE}});function D0(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var C0=Sc({animation:wc(Yo,"wave"),height:wc(Yo),size:wc(Yo),type:wc(Yo,"text"),variant:wc(Yo),width:wc(Yo)},Fr),j0=i["default"].extend({name:Fr,functional:!0,props:C0,render:function(e,t){var a,n=t.data,r=t.props,i=r.size,o=r.animation,s=r.variant;return e("div",Ja(n,{staticClass:"b-skeleton",style:{width:i||r.width,height:i||r.height},class:(a={},D0(a,"b-skeleton-".concat(r.type),!0),D0(a,"b-skeleton-animate-".concat(o),o),D0(a,"bg-".concat(s),s),a)}))}});function x0(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function B0(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?x0(Object(a),!0).forEach((function(t){Y0(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):x0(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function Y0(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var P0=Sc({animation:wc(Yo,"wave"),icon:wc(Yo),iconProps:wc(xo,{})},Er),I0=i["default"].extend({name:Er,functional:!0,props:P0,render:function(e,t){var a=t.data,n=t.props,r=n.icon,i=n.animation,o=e(kB,{staticClass:"b-skeleton-icon",props:B0(B0({},n.iconProps),{},{icon:r})});return e("div",Ja(a,{staticClass:"b-skeleton-icon-wrapper position-relative d-inline-block overflow-hidden",class:Y0({},"b-skeleton-animate-".concat(i),i)}),[o])}});function F0(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var E0=Sc({animation:wc(Yo),aspect:wc(Yo,"16:9"),cardImg:wc(Yo),height:wc(Yo),noAspect:wc(To,!1),variant:wc(Yo),width:wc(Yo)},$r),$0=i["default"].extend({name:$r,functional:!0,props:E0,render:function(e,t){var a=t.data,n=t.props,r=n.aspect,i=n.width,o=n.height,s=n.animation,l=n.variant,c=n.cardImg,u=e(j0,Ja(a,{props:{type:"img",width:i,height:o,animation:s,variant:l},class:F0({},"card-img-".concat(c),c)}));return n.noAspect?u:e(Vu,{props:{aspect:r}},[u])}}),R0=i["default"].extend({methods:{hasListener:function(e){var t=this.$listeners||{},a=this._events||{};return!Ot(t[e])||Pt(a[e])&&a[e].length>0}}});function N0(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var W0={stacked:wc(No,!1)},U0=i["default"].extend({props:W0,computed:{isStacked:function(){var e=this.stacked;return""===e||e},isStackedAlways:function(){return!0===this.isStacked},stackedTableClasses:function(){var e=this.isStackedAlways;return N0({"b-table-stacked":e},"b-table-stacked-".concat(this.stacked),!e&&this.isStacked)}}});function G0(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function q0(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?G0(Object(a),!0).forEach((function(t){J0(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):G0(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function J0(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var K0={bordered:wc(To,!1),borderless:wc(To,!1),captionTop:wc(To,!1),dark:wc(To,!1),fixed:wc(To,!1),hover:wc(To,!1),noBorderCollapse:wc(To,!1),outlined:wc(To,!1),responsive:wc(No,!1),small:wc(To,!1),stickyHeader:wc(No,!1),striped:wc(To,!1),tableClass:wc(Fo),tableVariant:wc(Yo)},X0=i["default"].extend({mixins:[UB],provide:function(){return{bvTable:this}},inheritAttrs:!1,props:K0,computed:{isResponsive:function(){var e=this.responsive;return""===e||e},isStickyHeader:function(){var e=this.stickyHeader;return e=""===e||e,!this.isStacked&&e},wrapperClasses:function(){var e=this.isResponsive;return[this.isStickyHeader?"b-table-sticky-header":"",!0===e?"table-responsive":e?"table-responsive-".concat(this.responsive):""].filter(ya)},wrapperStyles:function(){var e=this.isStickyHeader;return e&&!jt(e)?{maxHeight:e}:{}},tableClasses:function(){var e=this.hover,t=this.tableVariant;return e=this.isTableSimple?e:e&&this.computedItems.length>0&&!this.computedBusy,[this.tableClass,{"table-striped":this.striped,"table-hover":e,"table-dark":this.dark,"table-bordered":this.bordered,"table-borderless":this.borderless,"table-sm":this.small,border:this.outlined,"b-table-fixed":this.fixed,"b-table-caption-top":this.captionTop,"b-table-no-border-collapse":this.noBorderCollapse},t?"".concat(this.dark?"bg":"table","-").concat(t):"",this.stackedTableClasses,this.selectableTableClasses]},tableAttrs:function(){var e=this.computedItems,t=this.filteredItems,a=this.computedFields,n=this.selectableTableAttrs,r=this.isTableSimple?{}:{"aria-busy":zl(this.computedBusy),"aria-colcount":zl(a.length),"aria-describedby":this.bvAttrs["aria-describedby"]||this.$refs.caption?this.captionId:null},i=e&&t&&t.length>e.length?zl(t.length):null;return q0(q0(q0({"aria-rowcount":i},this.bvAttrs),{},{id:this.safeId(),role:this.bvAttrs.role||"table"},r),n)}},render:function(e){var t=this.wrapperClasses,a=this.renderCaption,n=this.renderColgroup,r=this.renderThead,i=this.renderTbody,o=this.renderTfoot,s=[];this.isTableSimple?s.push(this.normalizeSlot()):(s.push(a?a():null),s.push(n?n():null),s.push(r?r():null),s.push(i?i():null),s.push(o?o():null));var l=e("table",{staticClass:"table b-table",class:this.tableClasses,attrs:this.tableAttrs,key:"b-table"},s.filter(ya));return t.length>0?e("div",{class:t,style:this.wrapperStyles,key:"wrap"},[l]):l}});function Z0(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function Q0(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?Z0(Object(a),!0).forEach((function(t){e1(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):Z0(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function e1(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var t1=Sc(sa(Q0(Q0(Q0({},XP),W0),K0)),Kr),a1=i["default"].extend({name:Kr,mixins:[UB,R0,ZP,xc,X0,U0],props:t1,computed:{isTableSimple:function(){return!0}}});function n1(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function r1(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?n1(Object(a),!0).forEach((function(t){i1(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):n1(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function i1(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var o1=function(e){return e>0},s1=Sc({animation:wc(Yo),columns:wc(jo,5,o1),hideHeader:wc(To,!1),rows:wc(jo,3,o1),showFooter:wc(To,!1),tableProps:wc(xo,{})},Rr),l1=i["default"].extend({name:Rr,functional:!0,props:s1,render:function(e,t){var a=t.data,n=t.props,r=n.animation,i=n.columns,o=e("th",[e(j0,{props:{animation:r}})]),s=e("tr",ul(i,o)),l=e("td",[e(j0,{props:{width:"75%",animation:r}})]),c=e("tr",ul(i,l)),u=e("tbody",ul(n.rows,c)),d=n.hideHeader?e():e("thead",[s]),h=n.showFooter?e("tfoot",[s]):e();return e(a1,Ja(a,{props:r1({},n.tableProps)}),[d,u,h])}}),c1=Sc({loading:wc(To,!1)},Nr),u1=i["default"].extend({name:Nr,functional:!0,props:c1,render:function(e,t){var a=t.data,n=t.props,r=t.slots,i=t.scopedSlots,o=r(),s=i||{},l={};return n.loading?e("div",Ja(a,{attrs:{role:"alert","aria-live":"polite","aria-busy":!0},staticClass:"b-skeleton-wrapper",key:"loading"}),jc(Ss,l,s,o)):jc(is,l,s,o)}}),d1=Pa({components:{BSkeleton:j0,BSkeletonIcon:I0,BSkeletonImg:$0,BSkeletonTable:l1,BSkeletonWrapper:u1}}),h1=Pa({components:{BSpinner:nZ}});function f1(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function p1(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?f1(Object(a),!0).forEach((function(t){m1(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):f1(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function m1(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var v1="light",_1="dark",g1=Sc({variant:wc(Yo)},oi),y1=i["default"].extend({name:oi,mixins:[UB,JB,xc],provide:function(){return{bvTableTr:this}},inject:{bvTableRowGroup:{default:function(){return{}}}},inheritAttrs:!1,props:g1,computed:{inTbody:function(){return this.bvTableRowGroup.isTbody},inThead:function(){return this.bvTableRowGroup.isThead},inTfoot:function(){return this.bvTableRowGroup.isTfoot},isDark:function(){return this.bvTableRowGroup.isDark},isStacked:function(){return this.bvTableRowGroup.isStacked},isResponsive:function(){return this.bvTableRowGroup.isResponsive},isStickyHeader:function(){return this.bvTableRowGroup.isStickyHeader},hasStickyHeader:function(){return!this.isStacked&&this.bvTableRowGroup.hasStickyHeader},tableVariant:function(){return this.bvTableRowGroup.tableVariant},headVariant:function(){return this.inThead?this.bvTableRowGroup.headVariant:null},footVariant:function(){return this.inTfoot?this.bvTableRowGroup.footVariant:null},isRowDark:function(){return this.headVariant!==v1&&this.footVariant!==v1&&(this.headVariant===_1||this.footVariant===_1||this.isDark)},trClasses:function(){var e=this.variant;return[e?"".concat(this.isRowDark?"bg":"table","-").concat(e):null]},trAttrs:function(){return p1({role:"row"},this.bvAttrs)}},render:function(e){return e("tr",{class:this.trClasses,attrs:this.trAttrs,on:this.bvListeners},this.normalizeSlot())}}),b1={},M1=i["default"].extend({props:b1,methods:{renderBottomRow:function(){var e=this.computedFields,t=this.stacked,a=this.tbodyTrClass,n=this.tbodyTrAttr,r=this.$createElement;return this.hasNormalizedSlot(ts)&&!0!==t&&""!==t?r(y1,{staticClass:"b-table-bottom-row",class:[Ct(a)?a(null,"row-bottom"):a],attrs:Ct(n)?n(null,"row-bottom"):n,key:"b-bottom-row"},this.normalizeSlot(ts,{columns:e.length,fields:e})):r()}}});function z1(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function L1(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?z1(Object(a),!0).forEach((function(t){w1(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):z1(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function w1(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var H1=function(e){return e=fl(e,0),e>0?e:null},k1=function(e){return Dt(e)||H1(e)>0},A1=Sc({colspan:wc(Go,null,k1),rowspan:wc(Go,null,k1),stackedHeading:wc(Yo),stickyColumn:wc(To,!1),variant:wc(Yo)},qr),S1=i["default"].extend({name:qr,mixins:[UB,JB,xc],inject:{bvTableTr:{default:function(){return{}}}},inheritAttrs:!1,props:A1,computed:{tag:function(){return"td"},inTbody:function(){return this.bvTableTr.inTbody},inThead:function(){return this.bvTableTr.inThead},inTfoot:function(){return this.bvTableTr.inTfoot},isDark:function(){return this.bvTableTr.isDark},isStacked:function(){return this.bvTableTr.isStacked},isStackedCell:function(){return this.inTbody&&this.isStacked},isResponsive:function(){return this.bvTableTr.isResponsive},isStickyHeader:function(){return this.bvTableTr.isStickyHeader},hasStickyHeader:function(){return this.bvTableTr.hasStickyHeader},isStickyColumn:function(){return!this.isStacked&&(this.isResponsive||this.hasStickyHeader)&&this.stickyColumn},rowVariant:function(){return this.bvTableTr.variant},headVariant:function(){return this.bvTableTr.headVariant},footVariant:function(){return this.bvTableTr.footVariant},tableVariant:function(){return this.bvTableTr.tableVariant},computedColspan:function(){return H1(this.colspan)},computedRowspan:function(){return H1(this.rowspan)},cellClasses:function(){var e=this.variant,t=this.headVariant,a=this.isStickyColumn;return(!e&&this.isStickyHeader&&!t||!e&&a&&this.inTfoot&&!this.footVariant||!e&&a&&this.inThead&&!t||!e&&a&&this.inTbody)&&(e=this.rowVariant||this.tableVariant||"b-table-default"),[e?"".concat(this.isDark?"bg":"table","-").concat(e):null,a?"b-table-sticky-column":null]},cellAttrs:function(){var e=this.stackedHeading,t=this.inThead||this.inTfoot,a=this.computedColspan,n=this.computedRowspan,r="cell",i=null;return t?(r="columnheader",i=a>0?"colspan":"col"):xl(this.tag,"th")&&(r="rowheader",i=n>0?"rowgroup":"row"),L1(L1({colspan:a,rowspan:n,role:r,scope:i},this.bvAttrs),{},{"data-label":this.isStackedCell&&!Dt(e)?zl(e):null})}},render:function(e){var t=[this.normalizeSlot()];return e(this.tag,{class:this.cellClasses,attrs:this.cellAttrs,on:this.bvListeners},[this.isStackedCell?e("div",[t]):t])}});function V1(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var O1="busy",T1=wo+O1,D1=V1({},O1,wc(To,!1)),C1=i["default"].extend({props:D1,data:function(){return{localBusy:!1}},computed:{computedBusy:function(){return this[O1]||this.localBusy}},watch:{localBusy:function(e,t){e!==t&&this.$emit(T1,e)}},methods:{stopIfBusy:function(e){return!!this.computedBusy&&(Fc(e),!0)},renderBusy:function(){var e=this.tbodyTrClass,t=this.tbodyTrAttr,a=this.$createElement;return this.computedBusy&&this.hasNormalizedSlot(Ks)?a(y1,{staticClass:"b-table-busy-slot",class:[Ct(e)?e(null,Ks):e],attrs:Ct(t)?t(null,Ks):t,key:"table-busy-slot"},[a(S1,{props:{colspan:this.computedFields.length||null}},[this.normalizeSlot(Ks)])]):null}}}),j1={caption:wc(Yo),captionHtml:wc(Yo)},x1=i["default"].extend({props:j1,computed:{captionId:function(){return this.isStacked?this.safeId("_caption_"):null}},methods:{renderCaption:function(){var e=this.caption,t=this.captionHtml,a=this.$createElement,n=a(),r=this.hasNormalizedSlot(Xs);return(r||e||t)&&(n=a("caption",{attrs:{id:this.captionId},domProps:r?{}:UY(t,e),key:"caption",ref:"caption"},this.normalizeSlot(Xs))),n}}}),B1={},Y1=i["default"].extend({methods:{renderColgroup:function(){var e=this.computedFields,t=this.$createElement,a=t();return this.hasNormalizedSlot(Zs)&&(a=t("colgroup",{key:"colgroup"},[this.normalizeSlot(Zs,{columns:e.length,fields:e})])),a}}}),P1={emptyFilteredHtml:wc(Yo),emptyFilteredText:wc(Yo,"There are no records matching your request"),emptyHtml:wc(Yo),emptyText:wc(Yo,"There are no records to show"),showEmpty:wc(To,!1)},I1=i["default"].extend({props:P1,methods:{renderEmpty:function(){var e=this.computedItems,t=this.$createElement,a=t();if(this.showEmpty&&(!e||0===e.length)&&(!this.computedBusy||!this.hasNormalizedSlot(Ks))){var n=this.computedFields,r=this.isFiltered,i=this.emptyText,o=this.emptyHtml,s=this.emptyFilteredText,l=this.emptyFilteredHtml,c=this.tbodyTrClass,u=this.tbodyTrAttr;a=this.normalizeSlot(r?ds:us,{emptyFilteredHtml:l,emptyFilteredText:s,emptyHtml:o,emptyText:i,fields:n,items:e}),a||(a=t("div",{class:["text-center","my-2"],domProps:r?UY(l,s):UY(o,i)})),a=t(S1,{props:{colspan:n.length||null}},[t("div",{attrs:{role:"alert","aria-live":"polite"}},[a])]),a=t(y1,{staticClass:"b-table-empty-row",class:[Ct(c)?c(null,"row-empty"):c],attrs:Ct(u)?u(null,"row-empty"):u,key:r?"b-empty-filtered-row":"b-empty-row"},[a])}return a}}}),F1=function e(t){return Dt(t)?"":It(t)&&!Et(t)?ea(t).sort().map((function(a){return e(t[a])})).filter((function(e){return!!e})).join(" "):zl(t)};function E1(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function $1(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?E1(Object(a),!0).forEach((function(t){R1(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):E1(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function R1(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var N1="_cellVariants",W1="_rowVariant",U1="_showDetails",G1=[N1,W1,U1].reduce((function(e,t){return $1($1({},e),{},R1({},t,!0))}),{}),q1=["a","a *","button","button *","input:not(.disabled):not([disabled])","select:not(.disabled):not([disabled])","textarea:not(.disabled):not([disabled])",'[role="link"]','[role="link"] *','[role="button"]','[role="button"] *',"[tabindex]:not(.disabled):not([disabled])"].join(","),J1=function(e,t,a){var n=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{},r=ea(n).reduce((function(t,a){var r=n[a],i=r.filterByFormatted,o=Ct(i)?i:i?r.formatter:null;return Ct(o)&&(t[a]=o(e[a],a,e)),t}),na(e)),i=ea(r).filter((function(e){return!G1[e]&&!(Pt(t)&&t.length>0&&ll(t,e))&&!(Pt(a)&&a.length>0&&!ll(a,e))}));return ra(r,i)},K1=function(e,t,a,n){return It(e)?F1(J1(e,t,a,n)):""};function X1(e){return t5(e)||e5(e)||Q1(e)||Z1()}function Z1(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function Q1(e,t){if(e){if("string"===typeof e)return a5(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?a5(e,t):void 0}}function e5(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function t5(e){if(Array.isArray(e))return a5(e)}function a5(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}var n5='Prop "filter-debounce" is deprecated. Use the debounce feature of "<b-form-input>" instead.',r5={filter:wc([].concat(X1(Fo),[Bo])),filterDebounce:wc(Go,0,(function(e){return Be.test(String(e))})),filterFunction:wc(Co),filterIgnoredFields:wc(Oo,[]),filterIncludedFields:wc(Oo,[])},i5=i["default"].extend({props:r5,data:function(){return{isFiltered:!1,localFilter:this.filterSanitize(this.filter)}},computed:{computedFilterIgnored:function(){return cl(this.filterIgnoredFields||[]).filter(ya)},computedFilterIncluded:function(){return cl(this.filterIncludedFields||[]).filter(ya)},computedFilterDebounce:function(){var e=fl(this.filterDebounce,0);return e>0&&La(n5,Gr),e},localFiltering:function(){return!this.hasProvider||!!this.noProviderFiltering},filteredCheck:function(){var e=this.filteredItems,t=this.localItems,a=this.localFilter;return{filteredItems:e,localItems:t,localFilter:a}},localFilterFn:function(){var e=this.filterFunction;return Oc(e)?e:null},filteredItems:function(){var e=this.localItems,t=this.localFilter,a=this.localFiltering?this.filterFnFactory(this.localFilterFn,t)||this.defaultFilterFnFactory(t):null;return a&&e.length>0?e.filter(a):e}},watch:{computedFilterDebounce:function(e){!e&&this.$_filterTimer&&(this.clearFilterTimer(),this.localFilter=this.filterSanitize(this.filter))},filter:{deep:!0,handler:function(e){var t=this,a=this.computedFilterDebounce;this.clearFilterTimer(),a&&a>0?this.$_filterTimer=setTimeout((function(){t.localFilter=t.filterSanitize(e)}),a):this.localFilter=this.filterSanitize(e)}},filteredCheck:function(e){var t=e.filteredItems,a=e.localFilter,n=!1;a?EB(a,[])||EB(a,{})?n=!1:a&&(n=!0):n=!1,n&&this.$emit(Bi,t,t.length),this.isFiltered=n},isFiltered:function(e,t){if(!1===e&&!0===t){var a=this.localItems;this.$emit(Bi,a,a.length)}}},created:function(){var e=this;this.$_filterTimer=null,this.$nextTick((function(){e.isFiltered=Boolean(e.localFilter)}))},beforeDestroy:function(){this.clearFilterTimer()},methods:{clearFilterTimer:function(){clearTimeout(this.$_filterTimer),this.$_filterTimer=null},filterSanitize:function(e){return!this.localFiltering||this.localFilterFn||xt(e)||Nt(e)?ga(e):""},filterFnFactory:function(e,t){if(!e||!Ct(e)||!t||EB(t,[])||EB(t,{}))return null;var a=function(a){return e(a,t)};return a},defaultFilterFnFactory:function(e){var t=this;if(!e||!xt(e)&&!Nt(e))return null;var a=e;if(xt(a)){var n=Ml(e).replace(Ue,"\\s+");a=new RegExp(".*".concat(n,".*"),"i")}var r=function(e){return a.lastIndex=0,a.test(K1(e,t.computedFilterIgnored,t.computedFilterIncluded,t.computedFieldsObj))};return r}}}),o5=function(e,t){var a=null;return xt(t)?a={key:e,label:t}:Ct(t)?a={key:e,formatter:t}:It(t)?(a=na(t),a.key=a.key||e):!1!==t&&(a={key:e}),a},s5=function(e,t){var a=[];if(Pt(e)&&e.filter(ya).forEach((function(e){if(xt(e))a.push({key:e,label:gl(e)});else if(It(e)&&e.key&&xt(e.key))a.push(na(e));else if(It(e)&&1===ea(e).length){var t=ea(e)[0],n=o5(t,e[t]);n&&a.push(n)}})),0===a.length&&Pt(t)&&t.length>0){var n=t[0];ea(n).forEach((function(e){G1[e]||a.push({key:e,label:gl(e)})}))}var r={};return a.filter((function(e){return!r[e.key]&&(r[e.key]=!0,e.label=xt(e.label)?e.label:gl(e.key),!0)}))};function l5(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function c5(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?l5(Object(a),!0).forEach((function(t){u5(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):l5(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function u5(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var d5=Dc("value",{type:Oo,defaultValue:[]}),h5=d5.mixin,f5=d5.props,p5=d5.prop,m5=d5.event,v5=sa(c5(c5({},f5),{},u5({fields:wc(Oo,null),items:wc(Oo,[]),primaryKey:wc(Yo)},p5,wc(Oo,[])))),_5=i["default"].extend({mixins:[h5],props:v5,data:function(){var e=this.items;return{localItems:Pt(e)?e.slice():[]}},computed:{computedFields:function(){return s5(this.fields,this.localItems)},computedFieldsObj:function(){var e=this.$parent;return this.computedFields.reduce((function(t,a){if(t[a.key]=na(a),a.formatter){var n=a.formatter;xt(n)&&Ct(e[n])?n=e[n]:Ct(n)||(n=void 0),t[a.key].formatter=n}return t}),{})},computedItems:function(){return(this.paginatedItems||this.sortedItems||this.filteredItems||this.localItems||[]).slice()},context:function(){return{filter:this.localFilter,sortBy:this.localSortBy,sortDesc:this.localSortDesc,perPage:mu(fl(this.perPage,0),0),currentPage:mu(fl(this.currentPage,0),1),apiUrl:this.apiUrl}}},watch:{items:function(e){this.localItems=Pt(e)?e.slice():[]},computedItems:function(e,t){EB(e,t)||this.$emit(m5,e)},context:function(e,t){EB(e,t)||this.$emit(Si,e)}},mounted:function(){this.$emit(m5,this.computedItems)},methods:{getFieldFormatter:function(e){var t=this.computedFieldsObj[e];return t?t.formatter:void 0}}}),g5={currentPage:wc(Go,1),perPage:wc(Go,0)},y5=i["default"].extend({props:g5,computed:{localPaging:function(){return!this.hasProvider||!!this.noProviderPaging},paginatedItems:function(){var e=this.sortedItems||this.filteredItems||this.localItems||[],t=mu(fl(this.currentPage,1),1),a=mu(fl(this.perPage,0),0);return this.localPaging&&a&&(e=e.slice((t-1)*a,t*a)),e}}}),b5=$c(Gr,ao),M5=Rc(Gr,to),z5={apiUrl:wc(Yo),items:wc(Po,[]),noProviderFiltering:wc(To,!1),noProviderPaging:wc(To,!1),noProviderSorting:wc(To,!1)},L5=i["default"].extend({mixins:[qB],props:z5,computed:{hasProvider:function(){return Ct(this.items)},providerTriggerContext:function(){var e={apiUrl:this.apiUrl,filter:null,sortBy:null,sortDesc:null,perPage:null,currentPage:null};return this.noProviderFiltering||(e.filter=this.localFilter),this.noProviderSorting||(e.sortBy=this.localSortBy,e.sortDesc=this.localSortDesc),this.noProviderPaging||(e.perPage=this.perPage,e.currentPage=this.currentPage),na(e)}},watch:{items:function(e){(this.hasProvider||Ct(e))&&this.$nextTick(this._providerUpdate)},providerTriggerContext:function(e,t){EB(e,t)||this.$nextTick(this._providerUpdate)}},mounted:function(){var e=this;!this.hasProvider||this.localItems&&0!==this.localItems.length||this._providerUpdate(),this.listenOnRoot(M5,(function(t){t!==e.id&&t!==e||e.refresh()}))},methods:{refresh:function(){var e=this.items,t=this.refresh;this.$off(ao,t),this.computedBusy?this.localBusy&&this.hasProvider&&this.$on(ao,t):(this.clearSelected(),this.hasProvider?this.$nextTick(this._providerUpdate):this.localItems=Pt(e)?e.slice():[])},_providerSetLocal:function(e){this.localItems=Pt(e)?e.slice():[],this.localBusy=!1,this.$emit(ao),this.id&&this.emitOnRoot(b5,this.id)},_providerUpdate:function(){var e=this;this.hasProvider&&(this.computedBusy?this.$nextTick(this.refresh):(this.localBusy=!0,this.$nextTick((function(){try{var t=e.items(e.context,e._providerSetLocal);Wt(t)?t.then((function(t){e._providerSetLocal(t)})):Pt(t)?e._providerSetLocal(t):2!==e.items.length&&(La("Provider function didn't request callback and did not return a promise or data.",Gr),e.localBusy=!1)}catch(a){La("Provider function error [".concat(a.name,"] ").concat(a.message,"."),Gr),e.localBusy=!1,e.$off(ao,e.refresh)}}))))}}});function w5(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var H5,k5,A5=["range","multi","single"],S5="grid",V5={noSelectOnClick:wc(To,!1),selectMode:wc(Yo,"multi",(function(e){return ll(A5,e)})),selectable:wc(To,!1),selectedVariant:wc(Yo,"active")},O5=i["default"].extend({props:V5,data:function(){return{selectedRows:[],selectedLastRow:-1}},computed:{isSelectable:function(){return this.selectable&&this.selectMode},hasSelectableRowClick:function(){return this.isSelectable&&!this.noSelectOnClick},supportsSelectableRows:function(){return!0},selectableHasSelection:function(){var e=this.selectedRows;return this.isSelectable&&e&&e.length>0&&e.some(ya)},selectableIsMultiSelect:function(){return this.isSelectable&&ll(["range","multi"],this.selectMode)},selectableTableClasses:function(){var e,t=this.isSelectable;return e={"b-table-selectable":t},w5(e,"b-table-select-".concat(this.selectMode),t),w5(e,"b-table-selecting",this.selectableHasSelection),w5(e,"b-table-selectable-no-click",t&&!this.hasSelectableRowClick),e},selectableTableAttrs:function(){if(!this.isSelectable)return{};var e=this.bvAttrs.role||S5;return{role:e,"aria-multiselectable":e===S5?zl(this.selectableIsMultiSelect):null}}},watch:{computedItems:function(e,t){var a=!1;if(this.isSelectable&&this.selectedRows.length>0){a=Pt(e)&&Pt(t)&&e.length===t.length;for(var n=0;a&&n<e.length;n++)a=EB(J1(e[n]),J1(t[n]))}a||this.clearSelected()},selectable:function(e){this.clearSelected(),this.setSelectionHandlers(e)},selectMode:function(){this.clearSelected()},hasSelectableRowClick:function(e){this.clearSelected(),this.setSelectionHandlers(!e)},selectedRows:function(e,t){var a=this;if(this.isSelectable&&!EB(e,t)){var n=[];e.forEach((function(e,t){e&&n.push(a.computedItems[t])})),this.$emit(co,n)}}},beforeMount:function(){this.isSelectable&&this.setSelectionHandlers(!0)},methods:{selectRow:function(e){if(this.isSelectable&&Bt(e)&&e>=0&&e<this.computedItems.length&&!this.isRowSelected(e)){var t=this.selectableIsMultiSelect?this.selectedRows.slice():[];t[e]=!0,this.selectedLastClicked=-1,this.selectedRows=t}},unselectRow:function(e){if(this.isSelectable&&Bt(e)&&this.isRowSelected(e)){var t=this.selectedRows.slice();t[e]=!1,this.selectedLastClicked=-1,this.selectedRows=t}},selectAllRows:function(){var e=this.computedItems.length;this.isSelectable&&e>0&&(this.selectedLastClicked=-1,this.selectedRows=this.selectableIsMultiSelect?ul(e,!0):[!0])},isRowSelected:function(e){return!(!Bt(e)||!this.selectedRows[e])},clearSelected:function(){this.selectedLastClicked=-1,this.selectedRows=[]},selectableRowClasses:function(e){if(this.isSelectable&&this.isRowSelected(e)){var t=this.selectedVariant;return w5({"b-table-row-selected":!0},"".concat(this.dark?"bg":"table","-").concat(t),t)}return{}},selectableRowAttrs:function(e){return{"aria-selected":this.isSelectable?this.isRowSelected(e)?"true":"false":null}},setSelectionHandlers:function(e){var t=e&&!this.noSelectOnClick?"$on":"$off";this[t](ro,this.selectionHandler),this[t](Bi,this.clearSelected),this[t](Si,this.clearSelected)},selectionHandler:function(e,t,a){if(this.isSelectable&&!this.noSelectOnClick){var n=this.selectMode,r=this.selectedLastRow,i=this.selectedRows.slice(),o=!i[t];if("single"===n)i=[];else if("range"===n)if(r>-1&&a.shiftKey){for(var s=pu(r,t);s<=mu(r,t);s++)i[s]=!0;o=!0}else a.ctrlKey||a.metaKey||(i=[],o=!0),o&&(this.selectedLastRow=t);i[t]=o,this.selectedRows=i}else this.clearSelected()}}}),T5=function(e,t){return e.map((function(e,t){return[t,e]})).sort(function(e,t){return this(e[1],t[1])||e[0]-t[0]}.bind(t)).map((function(e){return e[1]}))},D5=function(e){return Dt(e)?"":Yt(e)?pl(e,e):e},C5=function(e,t){var a=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},n=a.sortBy,r=void 0===n?null:n,i=a.formatter,o=void 0===i?null:i,s=a.locale,l=void 0===s?void 0:s,c=a.localeOptions,u=void 0===c?{}:c,d=a.nullLast,h=void 0!==d&&d,f=Ma(e,r,null),p=Ma(t,r,null);return Ct(o)&&(f=o(f,r,e),p=o(p,r,t)),f=D5(f),p=D5(p),Et(f)&&Et(p)||Bt(f)&&Bt(p)?f<p?-1:f>p?1:0:h&&""===f&&""!==p?1:h&&""!==f&&""===p?-1:F1(f).localeCompare(F1(p),l,u)};function j5(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function x5(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?j5(Object(a),!0).forEach((function(t){B5(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):j5(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function B5(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var Y5="sortBy",P5=wo+Y5,I5="sortDesc",F5=wo+I5,E5="asc",$5="desc",R5="last",N5=[E5,$5,R5],W5=(H5={labelSortAsc:wc(Yo,"Click to sort ascending"),labelSortClear:wc(Yo,"Click to clear sorting"),labelSortDesc:wc(Yo,"Click to sort descending"),noFooterSorting:wc(To,!1),noLocalSorting:wc(To,!1),noSortReset:wc(To,!1)},B5(H5,Y5,wc(Yo)),B5(H5,"sortCompare",wc(Co)),B5(H5,"sortCompareLocale",wc(Eo)),B5(H5,"sortCompareOptions",wc(xo,{numeric:!0})),B5(H5,I5,wc(To,!1)),B5(H5,"sortDirection",wc(Yo,E5,(function(e){return ll(N5,e)}))),B5(H5,"sortIconLeft",wc(To,!1)),B5(H5,"sortNullLast",wc(To,!1)),H5),U5=i["default"].extend({props:W5,data:function(){return{localSortBy:this[Y5]||"",localSortDesc:this[I5]||!1}},computed:{localSorting:function(){return this.hasProvider?!!this.noProviderSorting:!this.noLocalSorting},isSortable:function(){return this.computedFields.some((function(e){return e.sortable}))},sortedItems:function(){var e=this.localSortBy,t=this.localSortDesc,a=this.sortCompareLocale,n=this.sortNullLast,r=this.sortCompare,i=this.localSorting,o=(this.filteredItems||this.localItems||[]).slice(),s=x5(x5({},this.sortCompareOptions),{},{usage:"sort"});if(e&&i){var l=this.computedFieldsObj[e]||{},c=l.sortByFormatted,u=Ct(c)?c:c?this.getFieldFormatter(e):void 0;return T5(o,(function(i,o){var l=null;return Ct(r)&&(l=r(i,o,e,t,u,s,a)),(Dt(l)||!1===l)&&(l=C5(i,o,{sortBy:e,formatter:u,locale:a,localeOptions:s,nullLast:n})),(l||0)*(t?-1:1)}))}return o}},watch:(k5={isSortable:function(e){e?this.isSortable&&this.$on(Ei,this.handleSort):this.$off(Ei,this.handleSort)}},B5(k5,I5,(function(e){e!==this.localSortDesc&&(this.localSortDesc=e||!1)})),B5(k5,Y5,(function(e){e!==this.localSortBy&&(this.localSortBy=e||"")})),B5(k5,"localSortDesc",(function(e,t){e!==t&&this.$emit(F5,e)})),B5(k5,"localSortBy",(function(e,t){e!==t&&this.$emit(P5,e)})),k5),created:function(){this.isSortable&&this.$on(Ei,this.handleSort)},methods:{handleSort:function(e,t,a,n){var r=this;if(this.isSortable&&(!n||!this.noFooterSorting)){var i=!1,o=function(){var e=t.sortDirection||r.sortDirection;e===E5?r.localSortDesc=!1:e===$5&&(r.localSortDesc=!0)};if(t.sortable){var s=!this.localSorting&&t.sortKey?t.sortKey:e;this.localSortBy===s?this.localSortDesc=!this.localSortDesc:(this.localSortBy=s,o()),i=!0}else this.localSortBy&&!this.noSortReset&&(this.localSortBy="",o(),i=!0);i&&this.$emit(_o,this.context)}},sortTheadThClasses:function(e,t,a){return{"b-table-sort-icon-left":t.sortable&&this.sortIconLeft&&!(a&&this.noFooterSorting)}},sortTheadThAttrs:function(e,t,a){var n,r=this.isSortable,i=this.noFooterSorting,o=this.localSortDesc,s=this.localSortBy,l=this.localSorting;if(!r||a&&i)return{};var c=t.sortable,u=l?e:null!==(n=t.sortKey)&&void 0!==n?n:e,d=c&&s===u?o?"descending":"ascending":c?"none":null;return{"aria-sort":d}},sortTheadThLabel:function(e,t,a){if(!this.isSortable||a&&this.noFooterSorting)return null;var n=this.localSortBy,r=this.localSortDesc,i=this.labelSortAsc,o=this.labelSortDesc,s=t.sortable,l="";if(s)if(n===e)l=r?i:o;else{l=r?o:i;var c=this.sortDirection||t.sortDirection;c===E5?l=i:c===$5&&(l=o)}else this.noSortReset||(l=n?this.labelSortClear:"");return wl(l)||null}}});function G5(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function q5(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?G5(Object(a),!0).forEach((function(t){J5(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):G5(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function J5(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var K5=Sc({tbodyTransitionHandlers:wc(xo),tbodyTransitionProps:wc(xo)},Zr),X5=i["default"].extend({name:Zr,mixins:[UB,JB,xc],provide:function(){return{bvTableRowGroup:this}},inject:{bvTable:{default:function(){return{}}}},inheritAttrs:!1,props:K5,computed:{isTbody:function(){return!0},isDark:function(){return this.bvTable.dark},isStacked:function(){return this.bvTable.isStacked},isResponsive:function(){return this.bvTable.isResponsive},isStickyHeader:function(){return!1},hasStickyHeader:function(){return!this.isStacked&&this.bvTable.stickyHeader},tableVariant:function(){return this.bvTable.tableVariant},isTransitionGroup:function(){return this.tbodyTransitionProps||this.tbodyTransitionHandlers},tbodyAttrs:function(){return q5({role:"rowgroup"},this.bvAttrs)},tbodyProps:function(){var e=this.tbodyTransitionProps;return e?q5(q5({},e),{},{tag:"tbody"}):{}}},render:function(e){var t={props:this.tbodyProps,attrs:this.tbodyAttrs};return this.isTransitionGroup?(t.on=this.tbodyTransitionHandlers||{},t.nativeOn=this.bvListeners):t.on=this.bvListeners,e(this.isTransitionGroup?"transition-group":"tbody",t,this.normalizeSlot())}}),Z5=["TD","TH","TR"],Q5=function(e){if(!e||!e.target)return!1;var t=e.target;if(t.disabled||-1!==Z5.indexOf(t.tagName))return!1;if(Rl(".dropdown-menu",t))return!0;var a="LABEL"===t.tagName?t:Rl("label",t);if(a){var n=Xl(a,"for"),r=n?Wl(n):El("input, select, textarea",a);if(r&&!r.disabled)return!0}return $l(t,q1)},e2=function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:document,t=rc();return!!(t&&""!==t.toString().trim()&&t.containsNode&&Cl(e))&&t.containsNode(e,!0)},t2=Sc(A1,ei),a2=i["default"].extend({name:ei,extends:S1,props:t2,computed:{tag:function(){return"th"}}});function n2(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function r2(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?n2(Object(a),!0).forEach((function(t){i2(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):n2(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function i2(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}function o2(e){return u2(e)||c2(e)||l2(e)||s2()}function s2(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function l2(e,t){if(e){if("string"===typeof e)return d2(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?d2(e,t):void 0}}function c2(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function u2(e){if(Array.isArray(e))return d2(e)}function d2(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}var h2={detailsTdClass:wc(Fo),tbodyTrAttr:wc(Jo),tbodyTrClass:wc([].concat(o2(Fo),[Co]))},f2=i["default"].extend({props:h2,methods:{getTdValues:function(e,t,a,n){var r=this.$parent;if(a){var i=Ma(e,t,"");return Ct(a)?a(i,t,e):xt(a)&&Ct(r[a])?r[a](i,t,e):a}return n},getThValues:function(e,t,a,n,r){var i=this.$parent;if(a){var o=Ma(e,t,"");return Ct(a)?a(o,t,e,n):xt(a)&&Ct(i[a])?i[a](o,t,e,n):a}return r},getFormattedValue:function(e,t){var a=t.key,n=this.getFieldFormatter(a),r=Ma(e,a,null);return Ct(n)&&(r=n(r,a,e)),Dt(r)?"":r},toggleDetailsFactory:function(e,t){var a=this;return function(){e&&a.$set(t,U1,!t[U1])}},rowHovered:function(e){this.tbodyRowEventStopped(e)||this.emitTbodyRowEvent(so,e)},rowUnhovered:function(e){this.tbodyRowEventStopped(e)||this.emitTbodyRowEvent(uo,e)},renderTbodyRowCell:function(e,t,a,n){var r=this,i=this.isStacked,o=e.key,s=e.label,l=e.isRowHeader,c=this.$createElement,u=this.hasNormalizedSlot(Js),d=this.getFormattedValue(a,e),h=!i&&(this.isResponsive||this.stickyHeader)&&e.stickyColumn,f=h?l?a2:S1:l?"th":"td",p=a[N1]&&a[N1][o]?a[N1][o]:e.variant||null,m={class:[e.class?e.class:"",this.getTdValues(a,o,e.tdClass,"")],props:{},attrs:r2({"aria-colindex":String(t+1)},l?this.getThValues(a,o,e.thAttr,"row",{}):this.getTdValues(a,o,e.tdAttr,{})),key:"row-".concat(n,"-cell-").concat(t,"-").concat(o)};h?m.props={stackedHeading:i?s:null,stickyColumn:!0,variant:p}:(m.attrs["data-label"]=i&&!Dt(s)?zl(s):null,m.attrs.role=l?"rowheader":"cell",m.attrs.scope=l?"row":null,p&&m.class.push("".concat(this.dark?"bg":"table","-").concat(p)));var v={item:a,index:n,field:e,unformatted:Ma(a,o,""),value:d,toggleDetails:this.toggleDetailsFactory(u,a),detailsShowing:Boolean(a[U1])};this.supportsSelectableRows&&(v.rowSelected=this.isRowSelected(n),v.selectRow=function(){return r.selectRow(n)},v.unselectRow=function(){return r.unselectRow(n)});var _=this.$_bodyFieldSlotNameCache[o],g=_?this.normalizeSlot(_,v):zl(d);return this.isStacked&&(g=[c("div",[g])]),c(f,m,[g])},renderTbodyRow:function(e,t){var a=this,n=this.computedFields,r=this.striped,i=this.primaryKey,o=this.currentPage,s=this.perPage,l=this.tbodyTrClass,c=this.tbodyTrAttr,u=this.$createElement,d=this.hasNormalizedSlot(Js),h=e[U1]&&d,f=this.$listeners[ro]||this.hasSelectableRowClick,p=[],m=h?this.safeId("_details_".concat(t,"_")):null,v=n.map((function(n,r){return a.renderTbodyRowCell(n,r,e,t)})),_=null;o&&s&&s>0&&(_=String((o-1)*s+t+1));var g=zl(Ma(e,i))||null,y=g||zl(t),b=g?this.safeId("_row_".concat(g)):null,M=this.selectableRowClasses?this.selectableRowClasses(t):{},z=this.selectableRowAttrs?this.selectableRowAttrs(t):{},L=Ct(l)?l(e,"row"):l,w=Ct(c)?c(e,"row"):c;if(p.push(u(y1,{class:[L,M,h?"b-table-has-details":""],props:{variant:e[W1]||null},attrs:r2(r2({id:b},w),{},{tabindex:f?"0":null,"data-pk":g||null,"aria-details":m,"aria-owns":m,"aria-rowindex":_},z),on:{mouseenter:this.rowHovered,mouseleave:this.rowUnhovered},key:"__b-table-row-".concat(y,"__"),ref:"item-rows",refInFor:!0},v)),h){var H={item:e,index:t,fields:n,toggleDetails:this.toggleDetailsFactory(d,e)};this.supportsSelectableRows&&(H.rowSelected=this.isRowSelected(t),H.selectRow=function(){return a.selectRow(t)},H.unselectRow=function(){return a.unselectRow(t)});var k=u(S1,{props:{colspan:n.length},class:this.detailsTdClass},[this.normalizeSlot(Js,H)]);r&&p.push(u("tr",{staticClass:"d-none",attrs:{"aria-hidden":"true",role:"presentation"},key:"__b-table-details-stripe__".concat(y)}));var A=Ct(this.tbodyTrClass)?this.tbodyTrClass(e,Js):this.tbodyTrClass,S=Ct(this.tbodyTrAttr)?this.tbodyTrAttr(e,Js):this.tbodyTrAttr;p.push(u(y1,{staticClass:"b-table-details",class:[A],props:{variant:e[W1]||null},attrs:r2(r2({},S),{},{id:m,tabindex:"-1"}),key:"__b-table-details__".concat(y)},[k]))}else d&&(p.push(u()),r&&p.push(u()));return p}}});function p2(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function m2(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?p2(Object(a),!0).forEach((function(t){v2(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):p2(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function v2(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var _2=function(e){return"cell(".concat(e||"",")")},g2=sa(m2(m2(m2({},K5),h2),{},{tbodyClass:wc(Fo)})),y2=i["default"].extend({mixins:[f2],props:g2,beforeDestroy:function(){this.$_bodyFieldSlotNameCache=null},methods:{getTbodyTrs:function(){var e=this.$refs,t=e.tbody?e.tbody.$el||e.tbody:null,a=(e["item-rows"]||[]).map((function(e){return e.$el||e}));return t&&t.children&&t.children.length>0&&a&&a.length>0?sl(t.children).filter((function(e){return ll(a,e)})):[]},getTbodyTrIndex:function(e){if(!Cl(e))return-1;var t="TR"===e.tagName?e:Rl("tr",e,!0);return t?this.getTbodyTrs().indexOf(t):-1},emitTbodyRowEvent:function(e,t){if(e&&this.hasListener(e)&&t&&t.target){var a=this.getTbodyTrIndex(t.target);if(a>-1){var n=this.computedItems[a];this.$emit(e,n,a,t)}}},tbodyRowEventStopped:function(e){return this.stopIfBusy&&this.stopIfBusy(e)},onTbodyRowKeydown:function(e){var t=e.target,a=e.keyCode;if(!this.tbodyRowEventStopped(e)&&"TR"===t.tagName&&Bl(t)&&0===t.tabIndex)if(ll([TB,PB],a))Fc(e),this.onTBodyRowClicked(e);else if(ll([IB,VB,CB,OB],a)){var n=this.getTbodyTrIndex(t);if(n>-1){Fc(e);var r=this.getTbodyTrs(),i=e.shiftKey;a===CB||i&&a===IB?lc(r[0]):a===OB||i&&a===VB?lc(r[r.length-1]):a===IB&&n>0?lc(r[n-1]):a===VB&&n<r.length-1&&lc(r[n+1])}}},onTBodyRowClicked:function(e){var t=this.$refs,a=t.tbody?t.tbody.$el||t.tbody:null;this.tbodyRowEventStopped(e)||Q5(e)||e2(a||this.$el)||this.emitTbodyRowEvent(ro,e)},onTbodyRowMiddleMouseRowClicked:function(e){this.tbodyRowEventStopped(e)||2!==e.which||this.emitTbodyRowEvent(lo,e)},onTbodyRowContextmenu:function(e){this.tbodyRowEventStopped(e)||this.emitTbodyRowEvent(io,e)},onTbodyRowDblClicked:function(e){this.tbodyRowEventStopped(e)||Q5(e)||this.emitTbodyRowEvent(oo,e)},renderTbody:function(){var e=this,t=this.computedItems,a=this.renderBusy,n=this.renderTopRow,r=this.renderEmpty,i=this.renderBottomRow,o=this.$createElement,s=this.hasListener(ro)||this.hasSelectableRowClick,l=[],c=a?a():null;if(c)l.push(c);else{var u={},d=_2();d=this.hasNormalizedSlot(d)?d:null,this.computedFields.forEach((function(t){var a=t.key,n=_2(a),r=_2(a.toLowerCase());u[a]=e.hasNormalizedSlot(n)?n:e.hasNormalizedSlot(r)?r:d})),this.$_bodyFieldSlotNameCache=u,l.push(n?n():o()),t.forEach((function(t,a){l.push(e.renderTbodyRow(t,a))})),l.push(r?r():o()),l.push(i?i():o())}var h={auxclick:this.onTbodyRowMiddleMouseRowClicked,contextmenu:this.onTbodyRowContextmenu,dblclick:this.onTbodyRowDblClicked};s&&(h.click=this.onTBodyRowClicked,h.keydown=this.onTbodyRowKeydown);var f=o(X5,{class:this.tbodyClass||null,props:kc(K5,this.$props),on:h,ref:"tbody"},l);return f}}});function b2(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function M2(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?b2(Object(a),!0).forEach((function(t){z2(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):b2(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function z2(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var L2=Sc({footVariant:wc(Yo)},Qr),w2=i["default"].extend({name:Qr,mixins:[UB,JB,xc],provide:function(){return{bvTableRowGroup:this}},inject:{bvTable:{default:function(){return{}}}},inheritAttrs:!1,props:L2,computed:{isTfoot:function(){return!0},isDark:function(){return this.bvTable.dark},isStacked:function(){return this.bvTable.isStacked},isResponsive:function(){return this.bvTable.isResponsive},isStickyHeader:function(){return!1},hasStickyHeader:function(){return!this.isStacked&&this.bvTable.stickyHeader},tableVariant:function(){return this.bvTable.tableVariant},tfootClasses:function(){return[this.footVariant?"thead-".concat(this.footVariant):null]},tfootAttrs:function(){return M2(M2({},this.bvAttrs),{},{role:"rowgroup"})}},render:function(e){return e("tfoot",{class:this.tfootClasses,attrs:this.tfootAttrs,on:this.bvListeners},this.normalizeSlot())}}),H2={footClone:wc(To,!1),footRowVariant:wc(Yo),footVariant:wc(Yo),tfootClass:wc(Fo),tfootTrClass:wc(Fo)},k2=i["default"].extend({props:H2,methods:{renderTFootCustom:function(){var e=this.$createElement;return this.hasNormalizedSlot(ns)?e(w2,{class:this.tfootClass||null,props:{footVariant:this.footVariant||this.headVariant||null},key:"bv-tfoot-custom"},this.normalizeSlot(ns,{items:this.computedItems.slice(),fields:this.computedFields.slice(),columns:this.computedFields.length})):e()},renderTfoot:function(){return this.footClone?this.renderThead(!0):this.renderTFootCustom()}}});function A2(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function S2(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?A2(Object(a),!0).forEach((function(t){V2(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):A2(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function V2(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var O2=Sc({headVariant:wc(Yo)},ti),T2=i["default"].extend({name:ti,mixins:[UB,JB,xc],provide:function(){return{bvTableRowGroup:this}},inject:{bvTable:{default:function(){return{}}}},inheritAttrs:!1,props:O2,computed:{isThead:function(){return!0},isDark:function(){return this.bvTable.dark},isStacked:function(){return this.bvTable.isStacked},isResponsive:function(){return this.bvTable.isResponsive},isStickyHeader:function(){return!this.isStacked&&this.bvTable.stickyHeader},hasStickyHeader:function(){return!this.isStacked&&this.bvTable.stickyHeader},tableVariant:function(){return this.bvTable.tableVariant},theadClasses:function(){return[this.headVariant?"thead-".concat(this.headVariant):null]},theadAttrs:function(){return S2({role:"rowgroup"},this.bvAttrs)}},render:function(e){return e("thead",{class:this.theadClasses,attrs:this.theadAttrs,on:this.bvListeners},this.normalizeSlot())}});function D2(e){return B2(e)||x2(e)||j2(e)||C2()}function C2(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function j2(e,t){if(e){if("string"===typeof e)return Y2(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?Y2(e,t):void 0}}function x2(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function B2(e){if(Array.isArray(e))return Y2(e)}function Y2(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}function P2(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function I2(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?P2(Object(a),!0).forEach((function(t){F2(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):P2(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function F2(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var E2=function(e){return"head(".concat(e||"",")")},$2=function(e){return"foot(".concat(e||"",")")},R2={headRowVariant:wc(Yo),headVariant:wc(Yo),theadClass:wc(Fo),theadTrClass:wc(Fo)},N2=i["default"].extend({props:R2,methods:{fieldClasses:function(e){return[e.class?e.class:"",e.thClass?e.thClass:""]},headClicked:function(e,t,a){this.stopIfBusy&&this.stopIfBusy(e)||Q5(e)||e2(this.$el)||(Fc(e),this.$emit(Ei,t.key,t,e,a))},renderThead:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]&&arguments[0],a=this.computedFields,n=this.isSortable,r=this.isSelectable,i=this.headVariant,o=this.footVariant,s=this.headRowVariant,l=this.footRowVariant,c=this.$createElement;if(this.isStackedAlways||0===a.length)return c();var u=n||this.hasListener(Ei),d=r?this.selectAllRows:MF,h=r?this.clearSelected:MF,f=function(a,r){var i=a.label,o=a.labelHtml,s=a.variant,l=a.stickyColumn,f=a.key,p=null;a.label.trim()||a.headerTitle||(p=gl(a.key));var m={};u&&(m.click=function(n){e.headClicked(n,a,t)},m.keydown=function(n){var r=n.keyCode;r!==TB&&r!==PB||e.headClicked(n,a,t)});var v=n?e.sortTheadThAttrs(f,a,t):{},_=n?e.sortTheadThClasses(f,a,t):null,g=n?e.sortTheadThLabel(f,a,t):null,y={class:[{"position-relative":g},e.fieldClasses(a),_],props:{variant:s,stickyColumn:l},style:a.thStyle||{},attrs:I2(I2({tabindex:u&&a.sortable?"0":null,abbr:a.headerAbbr||null,title:a.headerTitle||null,"aria-colindex":r+1,"aria-label":p},e.getThValues(null,f,a.thAttr,t?"foot":"head",{})),v),on:m,key:f},b=[E2(f),E2(f.toLowerCase()),E2()];t&&(b=[$2(f),$2(f.toLowerCase()),$2()].concat(D2(b)));var M={label:i,column:f,field:a,isFoot:t,selectAllRows:d,clearSelected:h},z=e.normalizeSlot(b,M)||c("div",{domProps:UY(o,i)}),L=g?c("span",{staticClass:"sr-only"}," (".concat(g,")")):null;return c(a2,y,[z,L].filter(ya))},p=a.map(f).filter(ya),m=[];if(t)m.push(c(y1,{class:this.tfootTrClass,props:{variant:Dt(l)?s:l}},p));else{var v={columns:a.length,fields:a,selectAllRows:d,clearSelected:h};m.push(this.normalizeSlot(al,v)||c()),m.push(c(y1,{class:this.theadTrClass,props:{variant:s}},p))}return c(t?w2:T2,{class:(t?this.tfootClass:this.theadClass)||null,props:t?{footVariant:o||i||null}:{headVariant:i||null},key:t?"bv-tfoot":"bv-thead"},m)}}}),W2={},U2=i["default"].extend({methods:{renderTopRow:function(){var e=this.computedFields,t=this.stacked,a=this.tbodyTrClass,n=this.tbodyTrAttr,r=this.$createElement;return this.hasNormalizedSlot(il)&&!0!==t&&""!==t?r(y1,{staticClass:"b-table-top-row",class:[Ct(a)?a(null,"row-top"):a],attrs:Ct(n)?n(null,"row-top"):n,key:"b-top-row"},[this.normalizeSlot(il,{columns:e.length,fields:e})]):r()}}});function G2(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function q2(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?G2(Object(a),!0).forEach((function(t){J2(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):G2(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function J2(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var K2=Sc(sa(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2(q2({},XP),b1),D1),j1),B1),P1),r5),v5),g5),z5),V5),W5),W0),K0),g2),H2),R2),W2)),Gr),X2=i["default"].extend({name:Gr,mixins:[UB,R0,ZP,xc,_5,X0,U0,N2,k2,y2,U0,i5,U5,y5,x1,Y1,O5,I1,U2,M1,C1,L5],props:K2});function Z2(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function Q2(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?Z2(Object(a),!0).forEach((function(t){e4(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):Z2(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function e4(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var t4,a4=Sc(sa(Q2(Q2(Q2(Q2(Q2(Q2(Q2(Q2(Q2({},XP),j1),B1),v5),W0),K0),g2),H2),R2)),Jr),n4=i["default"].extend({name:Jr,mixins:[UB,R0,ZP,xc,_5,X0,U0,N2,k2,y2,x1,Y1],props:a4}),r4=Pa({components:{BTableLite:n4}}),i4=Pa({components:{BTableSimple:a1,BTbody:X5,BThead:T2,BTfoot:w2,BTr:y1,BTd:S1,BTh:a2}}),o4=Pa({components:{BTable:X2},plugins:{TableLitePlugin:r4,TableSimplePlugin:i4}});function s4(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function l4(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?s4(Object(a),!0).forEach((function(t){c4(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):s4(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function c4(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var u4,d4,h4=Dc("value",{type:jo}),f4=h4.mixin,p4=h4.props,m4=h4.prop,v4=h4.event,_4=function(e){return!e.disabled},g4=i["default"].extend({name:fi,inject:{bvTabs:{default:function(){return{}}}},props:{controls:wc(Yo),id:wc(Yo),noKeyNav:wc(To,!1),posInSet:wc(jo),setSize:wc(jo),tab:wc(),tabIndex:wc(jo)},methods:{focus:function(){lc(this.$refs.link)},handleEvent:function(e){if(!this.tab.disabled){var t=e.type,a=e.keyCode,n=e.shiftKey;"click"===t||"keydown"===t&&a===PB?(Fc(e),this.$emit(Hi,e)):"keydown"!==t||this.noKeyNav||(-1!==[IB,jB,CB].indexOf(a)?(Fc(e),n||a===CB?this.$emit(Yi,e):this.$emit(eo,e)):-1!==[VB,YB,OB].indexOf(a)&&(Fc(e),n||a===OB?this.$emit(Ui,e):this.$emit(Ji,e)))}}},render:function(e){var t=this.id,a=this.tabIndex,n=this.setSize,r=this.posInSet,i=this.controls,o=this.handleEvent,s=this.tab,l=s.title,c=s.localActive,u=s.disabled,d=s.titleItemClass,h=s.titleLinkClass,f=s.titleLinkAttributes,p=e(cY,{staticClass:"nav-link",class:[{active:c&&!u,disabled:u},h,c?this.bvTabs.activeNavItemClass:null],props:{disabled:u},attrs:l4(l4({},f),{},{id:t,role:"tab",tabindex:a,"aria-selected":c&&!u?"true":"false","aria-setsize":n,"aria-posinset":r,"aria-controls":i}),on:{click:o,keydown:o},ref:"link"},[this.tab.normalizeSlot(nl)||l]);return e("li",{staticClass:"nav-item",class:[d],attrs:{role:"presentation"}},[p])}}),y4=ia(lX,["tabs","isNavBar","cardHeader"]),b4=Sc(sa(l4(l4(l4(l4({},XP),p4),y4),{},{activeNavItemClass:wc(Fo),activeTabClass:wc(Fo),card:wc(To,!1),contentClass:wc(Fo),end:wc(To,!1),lazy:wc(To,!1),navClass:wc(Fo),navWrapperClass:wc(Fo),noFade:wc(To,!1),noKeyNav:wc(To,!1),noNavStyle:wc(To,!1),tag:wc(Yo,"div")})),Xr),M4=i["default"].extend({name:Xr,mixins:[ZP,f4,xc],provide:function(){return{bvTabs:this}},props:b4,data:function(){return{currentTab:fl(this[m4],-1),tabs:[],registeredTabs:[]}},computed:{fade:function(){return!this.noFade},localNavClass:function(){var e=[];return this.card&&this.vertical&&e.push("card-header","h-100","border-bottom-0","rounded-0"),[].concat(e,[this.navClass])}},watch:(t4={},c4(t4,m4,(function(e,t){if(e!==t){e=fl(e,-1),t=fl(t,0);var a=this.tabs[e];a&&!a.disabled?this.activateTab(a):e<t?this.previousTab():this.nextTab()}})),c4(t4,"currentTab",(function(e){var t=-1;this.tabs.forEach((function(a,n){n!==e||a.disabled?a.localActive=!1:(a.localActive=!0,t=n)})),this.$emit(v4,t)})),c4(t4,"tabs",(function(e,t){var a=this;EB(e.map((function(e){return e[Ka]})),t.map((function(e){return e[Ka]})))||this.$nextTick((function(){a.$emit(wi,e.slice(),t.slice())}))})),c4(t4,"registeredTabs",(function(){this.updateTabs()})),t4),created:function(){this.$_observer=null},mounted:function(){this.setObserver(!0)},beforeDestroy:function(){this.setObserver(!1),this.tabs=[]},methods:{registerTab:function(e){ll(this.registeredTabs,e)||this.registeredTabs.push(e)},unregisterTab:function(e){this.registeredTabs=this.registeredTabs.slice().filter((function(t){return t!==e}))},setObserver:function(){var e=this,t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0];if(this.$_observer&&this.$_observer.disconnect(),this.$_observer=null,t){var a=function(){e.$nextTick((function(){Ol((function(){e.updateTabs()}))}))};this.$_observer=kF(this.$refs.content,a,{childList:!0,subtree:!1,attributes:!0,attributeFilter:["id"]})}},getTabs:function(){var e=this.registeredTabs.filter((function(e){return 0===e.$children.filter((function(e){return e._isTab})).length})),t=[];if(Me&&e.length>0){var a=e.map((function(e){return"#".concat(e.safeId())})).join(", ");t=Fl(a,this.$el).map((function(e){return e.id})).filter(ya)}return T5(e,(function(e,a){return t.indexOf(e.safeId())-t.indexOf(a.safeId())}))},updateTabs:function(){var e=this.getTabs(),t=e.indexOf(e.slice().reverse().find((function(e){return e.localActive&&!e.disabled})));if(t<0){var a=this.currentTab;a>=e.length?t=e.indexOf(e.slice().reverse().find(_4)):e[a]&&!e[a].disabled&&(t=a)}t<0&&(t=e.indexOf(e.find(_4))),e.forEach((function(e,a){e.localActive=a===t})),this.tabs=e,this.currentTab=t},getButtonForTab:function(e){return(this.$refs.buttons||[]).find((function(t){return t.tab===e}))},updateButton:function(e){var t=this.getButtonForTab(e);t&&t.$forceUpdate&&t.$forceUpdate()},activateTab:function(e){var t=this.currentTab,a=this.tabs,n=!1;if(e){var r=a.indexOf(e);if(r!==t&&r>-1&&!e.disabled){var i=new c$(bi,{cancelable:!0,vueTarget:this,componentId:this.safeId()});this.$emit(i.type,r,t,i),i.defaultPrevented||(this.currentTab=r,n=!0)}}return n||this[m4]===t||this.$emit(v4,t),n},deactivateTab:function(e){return!!e&&this.activateTab(this.tabs.filter((function(t){return t!==e})).find(_4))},focusButton:function(e){var t=this;this.$nextTick((function(){lc(t.getButtonForTab(e))}))},emitTabClick:function(e,t){$t(t)&&e&&e.$emit&&!e.disabled&&e.$emit(Hi,t)},clickTab:function(e,t){this.activateTab(e),this.emitTabClick(e,t)},firstTab:function(e){var t=this.tabs.find(_4);this.activateTab(t)&&e&&(this.focusButton(t),this.emitTabClick(t,e))},previousTab:function(e){var t=mu(this.currentTab,0),a=this.tabs.slice(0,t).reverse().find(_4);this.activateTab(a)&&e&&(this.focusButton(a),this.emitTabClick(a,e))},nextTab:function(e){var t=mu(this.currentTab,-1),a=this.tabs.slice(t+1).find(_4);this.activateTab(a)&&e&&(this.focusButton(a),this.emitTabClick(a,e))},lastTab:function(e){var t=this.tabs.slice().reverse().find(_4);this.activateTab(t)&&e&&(this.focusButton(t),this.emitTabClick(t,e))}},render:function(e){var t=this,a=this.align,n=this.card,r=this.end,i=this.fill,o=this.firstTab,s=this.justified,l=this.lastTab,c=this.nextTab,u=this.noKeyNav,d=this.noNavStyle,h=this.pills,f=this.previousTab,p=this.small,m=this.tabs,v=this.vertical,_=m.find((function(e){return e.localActive&&!e.disabled})),g=m.find((function(e){return!e.disabled})),y=m.map((function(a,n){var r,i=a.safeId,s=null;return u||(s=-1,(a===_||!_&&a===g)&&(s=null)),e(g4,{props:{controls:i?i():null,id:a.controlledBy||(i?i("_BV_tab_button_"):null),noKeyNav:u,posInSet:n+1,setSize:m.length,tab:a,tabIndex:s},on:(r={},c4(r,Hi,(function(e){t.clickTab(a,e)})),c4(r,Yi,o),c4(r,eo,f),c4(r,Ji,c),c4(r,Ui,l),r),key:a[Ka]||n,ref:"buttons",refInFor:!0})})),b=e(cX,{class:this.localNavClass,attrs:{role:"tablist",id:this.safeId("_BV_tab_controls_")},props:{fill:i,justified:s,align:a,tabs:!d&&!h,pills:!d&&h,vertical:v,small:p,cardHeader:n&&!v},ref:"nav"},[this.normalizeSlot(el)||e(),y,this.normalizeSlot(Qs)||e()]);b=e("div",{class:[{"card-header":n&&!v&&!r,"card-footer":n&&!v&&r,"col-auto":v},this.navWrapperClass],key:"bv-tabs-nav"},[b]);var M=this.normalizeSlot()||[],z=e();0===M.length&&(z=e("div",{class:["tab-pane","active",{"card-body":n}],key:"bv-empty-tab"},this.normalizeSlot(us)));var L=e("div",{staticClass:"tab-content",class:[{col:v},this.contentClass],attrs:{id:this.safeId("_BV_tab_container_")},key:"bv-content",ref:"content"},[M,z]);return e(this.tag,{staticClass:"tabs",class:{row:v,"no-gutters":v&&n},attrs:{id:this.safeId()}},[r?L:e(),b,r?e():L])}});function z4(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function L4(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?z4(Object(a),!0).forEach((function(t){w4(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):z4(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function w4(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var H4,k4="active",A4=wo+k4,S4=Sc(sa(L4(L4({},XP),{},(u4={},w4(u4,k4,wc(To,!1)),w4(u4,"buttonId",wc(Yo)),w4(u4,"disabled",wc(To,!1)),w4(u4,"lazy",wc(To,!1)),w4(u4,"noBody",wc(To,!1)),w4(u4,"tag",wc(Yo,"div")),w4(u4,"title",wc(Yo)),w4(u4,"titleItemClass",wc(Fo)),w4(u4,"titleLinkAttributes",wc(xo)),w4(u4,"titleLinkClass",wc(Fo)),u4))),Ur),V4=i["default"].extend({name:Ur,mixins:[ZP,xc],inject:{bvTabs:{default:function(){return{}}}},props:S4,data:function(){return{localActive:this[k4]&&!this.disabled}},computed:{_isTab:function(){return!0},tabClasses:function(){var e=this.localActive,t=this.disabled;return[{active:e,disabled:t,"card-body":this.bvTabs.card&&!this.noBody},e?this.bvTabs.activeTabClass:null]},controlledBy:function(){return this.buttonId||this.safeId("__BV_tab_button__")},computedNoFade:function(){return!this.bvTabs.fade},computedLazy:function(){return this.bvTabs.lazy||this.lazy}},watch:(d4={},w4(d4,k4,(function(e,t){e!==t&&(e?this.activate():this.deactivate()||this.$emit(A4,this.localActive))})),w4(d4,"disabled",(function(e,t){if(e!==t){var a=this.bvTabs.firstTab;e&&this.localActive&&a&&(this.localActive=!1,a())}})),w4(d4,"localActive",(function(e){this.$emit(A4,e)})),d4),mounted:function(){this.registerTab()},updated:function(){var e=this.bvTabs.updateButton;e&&this.hasNormalizedSlot(nl)&&e(this)},beforeDestroy:function(){this.unregisterTab()},methods:{registerTab:function(){var e=this.bvTabs.registerTab;e&&e(this)},unregisterTab:function(){var e=this.bvTabs.unregisterTab;e&&e(this)},activate:function(){var e=this.bvTabs.activateTab;return!(!e||this.disabled)&&e(this)},deactivate:function(){var e=this.bvTabs.deactivateTab;return!(!e||!this.localActive)&&e(this)}},render:function(e){var t=this.localActive,a=e(this.tag,{staticClass:"tab-pane",class:this.tabClasses,directives:[{name:"show",value:t}],attrs:{role:"tabpanel",id:this.safeId(),"aria-hidden":t?"false":"true","aria-labelledby":this.controlledBy||null},ref:"panel"},[t||!this.computedLazy?this.normalizeSlot():e()]);return e(eu,{props:{mode:"out-in",noFade:this.computedNoFade}},[a])}}),O4=Pa({components:{BTabs:M4,BTab:V4}}),T4=Pa({components:{BTime:mq}}),D4=a("2b88"),C4=i["default"].extend({mixins:[xc],data:function(){return{name:"b-toaster"}},methods:{onAfterEnter:function(e){var t=this;Ol((function(){Gl(e,"".concat(t.name,"-enter-to"))}))}},render:function(e){return e("transition-group",{props:{tag:"div",name:this.name},on:{afterEnter:this.onAfterEnter}},this.normalizeSlot())}}),j4=Sc({ariaAtomic:wc(Yo),ariaLive:wc(Yo),name:wc(Yo,void 0,!0),role:wc(Yo)},ri),x4=i["default"].extend({name:ri,mixins:[qB],props:j4,data:function(){return{doRender:!1,dead:!1,staticName:this.name}},beforeMount:function(){var e=this.name;this.staticName=e,D4["Wormhole"].hasTarget(e)?(La('A "<portal-target>" with name "'.concat(e,'" already exists in the document.'),ri),this.dead=!0):this.doRender=!0},beforeDestroy:function(){this.doRender&&this.emitOnRoot($c(ri,Vi),this.name)},destroyed:function(){var e=this.$el;e&&e.parentNode&&e.parentNode.removeChild(e)},render:function(e){var t=e("div",{class:["d-none",{"b-dead-toaster":this.dead}]});if(this.doRender){var a=e(D4["PortalTarget"],{staticClass:"b-toaster-slot",props:{name:this.staticName,multiple:!0,tag:"div",slim:!1,transition:C4}});t=e("div",{staticClass:"b-toaster",class:[this.staticName],attrs:{id:this.staticName,role:this.role||null,"aria-live":this.ariaLive,"aria-atomic":this.ariaAtomic}},[a])}return t}});function B4(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function Y4(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?B4(Object(a),!0).forEach((function(t){P4(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):B4(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function P4(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var I4=Dc("visible",{type:To,defaultValue:!1,event:Li}),F4=I4.mixin,E4=I4.props,$4=I4.prop,R4=I4.event,N4=1e3,W4=ra(lY,["href","to"]),U4=Sc(sa(Y4(Y4(Y4(Y4({},XP),E4),W4),{},{appendToast:wc(To,!1),autoHideDelay:wc(Go,5e3),bodyClass:wc(Fo),headerClass:wc(Fo),headerTag:wc(Yo,"header"),isStatus:wc(To,!1),noAutoHide:wc(To,!1),noCloseButton:wc(To,!1),noFade:wc(To,!1),noHoverPause:wc(To,!1),solid:wc(To,!1),static:wc(To,!1),title:wc(Yo),toastClass:wc(Fo),toaster:wc(Yo,"b-toaster-top-right"),variant:wc(Yo)})),ni),G4=i["default"].extend({name:ni,mixins:[UB,ZP,F4,qB,xc,YJ],inheritAttrs:!1,props:U4,data:function(){return{isMounted:!1,doRender:!1,localShow:!1,isTransitioning:!1,isHiding:!1,order:0,dismissStarted:0,resumeDismiss:0}},computed:{toastClasses:function(){var e=this.appendToast,t=this.variant;return P4({"b-toast-solid":this.solid,"b-toast-append":e,"b-toast-prepend":!e},"b-toast-".concat(t),t)},slotScope:function(){var e=this.hide;return{hide:e}},computedDuration:function(){return mu(fl(this.autoHideDelay,0),N4)},computedToaster:function(){return String(this.toaster)},transitionHandlers:function(){return{beforeEnter:this.onBeforeEnter,afterEnter:this.onAfterEnter,beforeLeave:this.onBeforeLeave,afterLeave:this.onAfterLeave}},computedAttrs:function(){return Y4(Y4({},this.bvAttrs),{},{id:this.safeId(),tabindex:"0"})}},watch:(H4={},P4(H4,$4,(function(e){this[e?"show":"hide"]()})),P4(H4,"localShow",(function(e){e!==this[$4]&&this.$emit(R4,e)})),P4(H4,"toaster",(function(){this.$nextTick(this.ensureToaster)})),P4(H4,"static",(function(e){e&&this.localShow&&this.ensureToaster()})),H4),created:function(){this.$_dismissTimer=null},mounted:function(){var e=this;this.isMounted=!0,this.$nextTick((function(){e[$4]&&Ol((function(){e.show()}))})),this.listenOnRoot(Rc(ni,fo),(function(t){t===e.safeId()&&e.show()})),this.listenOnRoot(Rc(ni,Ri),(function(t){t&&t!==e.safeId()||e.hide()})),this.listenOnRoot($c(ri,Vi),(function(t){t===e.computedToaster&&e.hide()}))},beforeDestroy:function(){this.clearDismissTimer()},methods:{show:function(){var e=this;if(!this.localShow){this.ensureToaster();var t=this.buildEvent(fo);this.emitEvent(t),this.dismissStarted=this.resumeDismiss=0,this.order=Date.now()*(this.appendToast?1:-1),this.isHiding=!1,this.doRender=!0,this.$nextTick((function(){Ol((function(){e.localShow=!0}))}))}},hide:function(){var e=this;if(this.localShow){var t=this.buildEvent(Ri);this.emitEvent(t),this.setHoverHandler(!1),this.dismissStarted=this.resumeDismiss=0,this.clearDismissTimer(),this.isHiding=!0,Ol((function(){e.localShow=!1}))}},buildEvent:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};return new c$(e,Y4(Y4({cancelable:!1,target:this.$el||null,relatedTarget:null},t),{},{vueTarget:this,componentId:this.safeId()}))},emitEvent:function(e){var t=e.type;this.emitOnRoot($c(ni,t),e),this.$emit(t,e)},ensureToaster:function(){if(!this.static){var e=this.computedToaster;if(!D4["Wormhole"].hasTarget(e)){var t=document.createElement("div");document.body.appendChild(t);var a=new x4({parent:this.$root,propsData:{name:e}});a.$mount(t)}}},startDismissTimer:function(){this.clearDismissTimer(),this.noAutoHide||(this.$_dismissTimer=setTimeout(this.hide,this.resumeDismiss||this.computedDuration),this.dismissStarted=Date.now(),this.resumeDismiss=0)},clearDismissTimer:function(){clearTimeout(this.$_dismissTimer),this.$_dismissTimer=null},setHoverHandler:function(e){var t=this.$refs["b-toast"];Ic(e,t,"mouseenter",this.onPause,So),Ic(e,t,"mouseleave",this.onUnPause,So)},onPause:function(){if(!this.noAutoHide&&!this.noHoverPause&&this.$_dismissTimer&&!this.resumeDismiss){var e=Date.now()-this.dismissStarted;e>0&&(this.clearDismissTimer(),this.resumeDismiss=mu(this.computedDuration-e,N4))}},onUnPause:function(){this.noAutoHide||this.noHoverPause||!this.resumeDismiss?this.resumeDismiss=this.dismissStarted=0:this.startDismissTimer()},onLinkClick:function(){var e=this;this.$nextTick((function(){Ol((function(){e.hide()}))}))},onBeforeEnter:function(){this.isTransitioning=!0},onAfterEnter:function(){this.isTransitioning=!1;var e=this.buildEvent(po);this.emitEvent(e),this.startDismissTimer(),this.setHoverHandler(!0)},onBeforeLeave:function(){this.isTransitioning=!0},onAfterLeave:function(){this.isTransitioning=!1,this.order=0,this.resumeDismiss=this.dismissStarted=0;var e=this.buildEvent($i);this.emitEvent(e),this.doRender=!1},makeToast:function(e){var t=this,a=this.title,n=this.slotScope,r=Yu(this),i=[],o=this.normalizeSlot(rl,n);o?i.push(o):a&&i.push(e("strong",{staticClass:"mr-2"},a)),this.noCloseButton||i.push(e(Uc,{staticClass:"ml-auto mb-1",on:{click:function(){t.hide()}}}));var s=e();i.length>0&&(s=e(this.headerTag,{staticClass:"toast-header",class:this.headerClass},i));var l=e(r?cY:"div",{staticClass:"toast-body",class:this.bodyClass,props:r?kc(W4,this):{},on:r?{click:this.onLinkClick}:{}},this.normalizeSlot(is,n));return e("div",{staticClass:"toast",class:this.toastClass,attrs:this.computedAttrs,key:"toast-".concat(this[Ka]),ref:"toast"},[s,l])}},render:function(e){if(!this.doRender||!this.isMounted)return e();var t=this.order,a=this.static,n=this.isHiding,r=this.isStatus,i="b-toast-".concat(this[Ka]),o=e("div",{staticClass:"b-toast",class:this.toastClasses,attrs:Y4(Y4({},a?{}:this.scopedStyleAttrs),{},{id:this.safeId("_toast_outer"),role:n?null:r?"status":"alert","aria-live":n?null:r?"polite":"assertive","aria-atomic":n?null:"true"}),key:i,ref:"b-toast"},[e(eu,{props:{noFade:this.noFade},on:this.transitionHandlers},[this.localShow?this.makeToast(e):e()])]);return e(D4["Portal"],{props:{name:i,to:this.computedToaster,order:t,slim:!0,disabled:a}},[o])}});function q4(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function J4(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function K4(e,t,a){return t&&J4(e.prototype,t),a&&J4(e,a),Object.defineProperty(e,"prototype",{writable:!1}),e}function X4(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function Z4(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?X4(Object(a),!0).forEach((function(t){Q4(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):X4(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function Q4(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}function e3(e){return r3(e)||n3(e)||a3(e)||t3()}function t3(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function a3(e,t){if(e){if("string"===typeof e)return i3(e,t);var a=Object.prototype.toString.call(e).slice(8,-1);return"Object"===a&&e.constructor&&(a=e.constructor.name),"Map"===a||"Set"===a?Array.from(e):"Arguments"===a||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(a)?i3(e,t):void 0}}function n3(e){if("undefined"!==typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function r3(e){if(Array.isArray(e))return i3(e)}function i3(e,t){(null==t||t>e.length)&&(t=e.length);for(var a=0,n=new Array(t);a<t;a++)n[a]=e[a];return n}var o3="$bvToast",s3="_bv__toast",l3=["id"].concat(e3(ea(ia(U4,["static","visible"])))),c3={toastContent:"default",title:"toast-title"},u3=function(e){return l3.reduce((function(t,a){return Ot(e[a])||(t[a]=e[a]),t}),{})},d3=function(e){var t=e.extend({name:pi,extends:G4,destroyed:function(){var e=this.$el;e&&e.parentNode&&e.parentNode.removeChild(e)},mounted:function(){var e=this,t=function(){e.localShow=!1,e.doRender=!1,e.$nextTick((function(){e.$nextTick((function(){Ol((function(){e.$destroy()}))}))}))};this.$parent.$once(Lo,t),this.$once($i,t),this.listenOnRoot($c(ri,Vi),(function(a){a===e.toaster&&t()}))}}),a=function(e,a){if(!wa(o3)){var n=new t({parent:a,propsData:Z4(Z4(Z4({},u3(fc(ni))),ia(e,ea(c3))),{},{static:!1,visible:!0})});ea(c3).forEach((function(t){var r=e[t];Ot(r)||("title"===t&&xt(r)&&(r=[a.$createElement("strong",{class:"mr-2"},r)]),n.$slots[c3[t]]=cl(r))}));var r=document.createElement("div");document.body.appendChild(r),n.$mount(r)}},n=function(){function e(t){q4(this,e),Jt(this,{_vm:t,_root:t.$root}),Xt(this,{_vm:la(),_root:la()})}return K4(e,[{key:"toast",value:function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};e&&!wa(o3)&&a(Z4(Z4({},u3(t)),{},{toastContent:e}),this._vm)}},{key:"show",value:function(e){e&&this._root.$emit(Rc(ni,fo),e)}},{key:"hide",value:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null;this._root.$emit(Rc(ni,Ri),e)}}]),e}();e.mixin({beforeCreate:function(){this[s3]=new n(this)}}),ta(e.prototype,o3)||Zt(e.prototype,o3,{get:function(){return this&&this[s3]||La('"'.concat(o3,'" must be accessed from a Vue instance "this" context.'),ni),this[s3]}})},h3=Pa({plugins:{plugin:d3}}),f3=Pa({components:{BToast:G4,BToaster:x4},plugins:{BVToastPlugin:h3}});function p3(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function m3(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?p3(Object(a),!0).forEach((function(t){v3(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):p3(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function v3(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}var _3="__BV_Tooltip__",g3="hover focus",y3={focus:!0,hover:!0,click:!0,blur:!0,manual:!0},b3=/^html$/i,M3=/^noninteractive$/i,z3=/^nofade$/i,L3=/^(auto|top(left|right)?|bottom(left|right)?|left(top|bottom)?|right(top|bottom)?)$/i,w3=/^(window|viewport|scrollParent)$/i,H3=/^d\d+$/i,k3=/^ds\d+$/i,A3=/^dh\d+$/i,S3=/^o-?\d+$/i,V3=/^v-.+$/i,O3=/\s+/,T3=function(e,t){var a={title:void 0,trigger:"",placement:"top",fallbackPlacement:"flip",container:!1,animation:!0,offset:0,id:null,html:!1,interactive:!0,disabled:!1,delay:fc(ii,"delay",50),boundary:String(fc(ii,"boundary","scrollParent")),boundaryPadding:fl(fc(ii,"boundaryPadding",5),0),variant:fc(ii,"variant"),customClass:fc(ii,"customClass")};if(xt(e.value)||Bt(e.value)||Ct(e.value)?a.title=e.value:Ft(e.value)&&(a=m3(m3({},a),e.value)),Ot(a.title)){var n=t.data||{};a.title=n.attrs&&!Dt(n.attrs.title)?n.attrs.title:void 0}Ft(a.delay)||(a.delay={show:fl(a.delay,0),hide:fl(a.delay,0)}),e.arg&&(a.container="#".concat(e.arg)),ea(e.modifiers).forEach((function(e){if(b3.test(e))a.html=!0;else if(M3.test(e))a.interactive=!1;else if(z3.test(e))a.animation=!1;else if(L3.test(e))a.placement=e;else if(w3.test(e))e="scrollparent"===e?"scrollParent":e,a.boundary=e;else if(H3.test(e)){var t=fl(e.slice(1),0);a.delay.show=t,a.delay.hide=t}else k3.test(e)?a.delay.show=fl(e.slice(2),0):A3.test(e)?a.delay.hide=fl(e.slice(2),0):S3.test(e)?a.offset=fl(e.slice(1),0):V3.test(e)&&(a.variant=e.slice(2)||null)}));var r={};return cl(a.trigger||"").filter(ya).join(" ").trim().toLowerCase().split(O3).forEach((function(e){y3[e]&&(r[e]=!0)})),ea(e.modifiers).forEach((function(e){e=e.toLowerCase(),y3[e]&&(r[e]=!0)})),a.trigger=ea(r).join(" "),"blur"===a.trigger&&(a.trigger="focus"),a.trigger||(a.trigger=g3),a},D3=function(e,t,a){if(Me){var n=T3(t,a);if(!e[_3]){var r=a.context;e[_3]=new pQ({parent:r,_scopeId:xJ(r,void 0)}),e[_3].__bv_prev_data__={},e[_3].$on(fo,(function(){Ct(n.title)&&e[_3].updateData({title:n.title(e)})}))}var i={title:n.title,triggers:n.trigger,placement:n.placement,fallbackPlacement:n.fallbackPlacement,variant:n.variant,customClass:n.customClass,container:n.container,boundary:n.boundary,delay:n.delay,offset:n.offset,noFade:!n.animation,id:n.id,interactive:n.interactive,disabled:n.disabled,html:n.html},o=e[_3].__bv_prev_data__;if(e[_3].__bv_prev_data__=i,!EB(i,o)){var s={target:e};ea(i).forEach((function(t){i[t]!==o[t]&&(s[t]="title"===t&&Ct(i[t])?i[t](e):i[t])})),e[_3].updateData(s)}}},C3=function(e){e[_3]&&(e[_3].$destroy(),e[_3]=null),delete e[_3]},j3={bind:function(e,t,a){D3(e,t,a)},componentUpdated:function(e,t,a){a.context.$nextTick((function(){D3(e,t,a)}))},unbind:function(e){C3(e)}},x3=Pa({directives:{VBTooltip:j3}}),B3=Pa({components:{BTooltip:LQ},plugins:{VBTooltipPlugin:x3}}),Y3=Pa({plugins:{AlertPlugin:fu,AspectPlugin:Ou,AvatarPlugin:YY,BadgePlugin:NY,BreadcrumbPlugin:iP,ButtonPlugin:oP,ButtonGroupPlugin:hP,ButtonToolbarPlugin:vP,CalendarPlugin:cI,CardPlugin:bF,CarouselPlugin:XF,CollapsePlugin:ZE,DropdownPlugin:cR,EmbedPlugin:pR,FormPlugin:jR,FormCheckboxPlugin:VN,FormDatepickerPlugin:aW,FormFilePlugin:gW,FormGroupPlugin:PW,FormInputPlugin:rU,FormRadioPlugin:sU,FormRatingPlugin:LU,FormSelectPlugin:GU,FormSpinbuttonPlugin:hG,FormTagsPlugin:EG,FormTextareaPlugin:qG,FormTimepickerPlugin:Sq,ImagePlugin:Vq,InputGroupPlugin:Gq,JumbotronPlugin:eJ,LayoutPlugin:uJ,LinkPlugin:dJ,ListGroupPlugin:zJ,MediaPlugin:OJ,ModalPlugin:iX,NavPlugin:xX,NavbarPlugin:eZ,OverlayPlugin:dZ,PaginationPlugin:YZ,PaginationNavPlugin:WZ,PopoverPlugin:ZQ,ProgressPlugin:l0,SidebarPlugin:T0,SkeletonPlugin:d1,SpinnerPlugin:h1,TablePlugin:o4,TabsPlugin:O4,TimePlugin:T4,ToastPlugin:f3,TooltipPlugin:B3}}),P3=Pa({directives:{VBHover:BN}}),I3=Pa({directives:{VBModal:PK}});function F3(e,t){var a=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),a.push.apply(a,n)}return a}function E3(e){for(var t=1;t<arguments.length;t++){var a=null!=arguments[t]?arguments[t]:{};t%2?F3(Object(a),!0).forEach((function(t){$3(e,t,a[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(a)):F3(Object(a)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(a,t))}))}return e}function $3(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e}function R3(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function N3(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}function W3(e,t,a){return t&&N3(e.prototype,t),a&&N3(e,a),Object.defineProperty(e,"prototype",{writable:!1}),e}var U3="v-b-scrollspy",G3="dropdown-item",q3="active",J3=".nav, .list-group",K3=".nav-link",X3=".nav-item",Z3=".list-group-item",Q3=".dropdown, .dropup",e8=".dropdown-item",t8=".dropdown-toggle",a8=$c("BVScrollspy","activate"),n8="offset",r8="position",i8={element:"body",offset:10,method:"auto",throttle:75},o8={element:"(string|element|component)",offset:"number",method:"string",throttle:"number"},s8=["webkitTransitionEnd","transitionend","otransitionend","oTransitionEnd"],l8=function(e){return aa(e).match(/\s([a-zA-Z]+)/)[1].toLowerCase()},c8=function(e,t,a){for(var n in a)if(ta(a,n)){var r=a[n],i=t[n],o=i&&Cl(i)?"element":l8(i);o=i&&i._isVue?"component":o,new RegExp(r).test(o)||La("".concat(e,': Option "').concat(n,'" provided type "').concat(o,'" but expected type "').concat(r,'"'))}},u8=function(){function e(t,a,n){R3(this,e),this.$el=t,this.$scroller=null,this.$selector=[K3,Z3,e8].join(","),this.$offsets=[],this.$targets=[],this.$activeTarget=null,this.$scrollHeight=0,this.$resizeTimeout=null,this.$scrollerObserver=null,this.$targetsObserver=null,this.$root=n||null,this.$config=null,this.updateConfig(a)}return W3(e,[{key:"updateConfig",value:function(e,t){this.$scroller&&(this.unlisten(),this.$scroller=null);var a=E3(E3({},this.constructor.Default),e);if(t&&(this.$root=t),c8(this.constructor.Name,a,this.constructor.DefaultType),this.$config=a,this.$root){var n=this;this.$root.$nextTick((function(){n.listen()}))}else this.listen()}},{key:"dispose",value:function(){this.unlisten(),clearTimeout(this.$resizeTimeout),this.$resizeTimeout=null,this.$el=null,this.$config=null,this.$scroller=null,this.$selector=null,this.$offsets=null,this.$targets=null,this.$activeTarget=null,this.$scrollHeight=null}},{key:"listen",value:function(){var e=this,t=this.getScroller();t&&"BODY"!==t.tagName&&Yc(t,"scroll",this,So),Yc(window,"scroll",this,So),Yc(window,"resize",this,So),Yc(window,"orientationchange",this,So),s8.forEach((function(t){Yc(window,t,e,So)})),this.setObservers(!0),this.handleEvent("refresh")}},{key:"unlisten",value:function(){var e=this,t=this.getScroller();this.setObservers(!1),t&&"BODY"!==t.tagName&&Pc(t,"scroll",this,So),Pc(window,"scroll",this,So),Pc(window,"resize",this,So),Pc(window,"orientationchange",this,So),s8.forEach((function(t){Pc(window,t,e,So)}))}},{key:"setObservers",value:function(e){var t=this;this.$scrollerObserver&&this.$scrollerObserver.disconnect(),this.$targetsObserver&&this.$targetsObserver.disconnect(),this.$scrollerObserver=null,this.$targetsObserver=null,e&&(this.$targetsObserver=kF(this.$el,(function(){t.handleEvent("mutation")}),{subtree:!0,childList:!0,attributes:!0,attributeFilter:["href"]}),this.$scrollerObserver=kF(this.getScroller(),(function(){t.handleEvent("mutation")}),{subtree:!0,childList:!0,characterData:!0,attributes:!0,attributeFilter:["id","style","class"]}))}},{key:"handleEvent",value:function(e){var t=xt(e)?e:e.type,a=this,n=function(){a.$resizeTimeout||(a.$resizeTimeout=setTimeout((function(){a.refresh(),a.process(),a.$resizeTimeout=null}),a.$config.throttle))};"scroll"===t?(this.$scrollerObserver||this.listen(),this.process()):/(resize|orientationchange|mutation|refresh)/.test(t)&&n()}},{key:"refresh",value:function(){var e=this,t=this.getScroller();if(t){var a=t!==t.window?r8:n8,n="auto"===this.$config.method?a:this.$config.method,r=n===r8?oc:ic,i=n===r8?this.getScrollTop():0;return this.$offsets=[],this.$targets=[],this.$scrollHeight=this.getScrollHeight(),Fl(this.$selector,this.$el).map((function(e){return Xl(e,"href")})).filter((function(e){return e&&at.test(e||"")})).map((function(e){var a=e.replace(at,"$1").trim();if(!a)return null;var n=El(a,t);return n&&Yl(n)?{offset:fl(r(n).top,0)+i,target:a}:null})).filter(ya).sort((function(e,t){return e.offset-t.offset})).reduce((function(t,a){return t[a.target]||(e.$offsets.push(a.offset),e.$targets.push(a.target),t[a.target]=!0),t}),{}),this}}},{key:"process",value:function(){var e=this.getScrollTop()+this.$config.offset,t=this.getScrollHeight(),a=this.$config.offset+t-this.getOffsetHeight();if(this.$scrollHeight!==t&&this.refresh(),e>=a){var n=this.$targets[this.$targets.length-1];this.$activeTarget!==n&&this.activate(n)}else{if(this.$activeTarget&&e<this.$offsets[0]&&this.$offsets[0]>0)return this.$activeTarget=null,void this.clear();for(var r=this.$offsets.length;r--;){var i=this.$activeTarget!==this.$targets[r]&&e>=this.$offsets[r]&&(Ot(this.$offsets[r+1])||e<this.$offsets[r+1]);i&&this.activate(this.$targets[r])}}}},{key:"getScroller",value:function(){if(this.$scroller)return this.$scroller;var e=this.$config.element;return e?(Cl(e.$el)?e=e.$el:xt(e)&&(e=El(e)),e?(this.$scroller="BODY"===e.tagName?window:e,this.$scroller):null):null}},{key:"getScrollTop",value:function(){var e=this.getScroller();return e===window?e.pageYOffset:e.scrollTop}},{key:"getScrollHeight",value:function(){return this.getScroller().scrollHeight||mu(document.body.scrollHeight,document.documentElement.scrollHeight)}},{key:"getOffsetHeight",value:function(){var e=this.getScroller();return e===window?window.innerHeight:ac(e).height}},{key:"activate",value:function(e){var t=this;this.$activeTarget=e,this.clear();var a=Fl(this.$selector.split(",").map((function(t){return"".concat(t,'[href$="').concat(e,'"]')})).join(","),this.$el);a.forEach((function(e){if(ql(e,G3)){var a=Rl(Q3,e);a&&t.setActiveState(El(t8,a),!0),t.setActiveState(e,!0)}else{t.setActiveState(e,!0),$l(e.parentElement,X3)&&t.setActiveState(e.parentElement,!0);var n=e;while(n){n=Rl(J3,n);var r=n?n.previousElementSibling:null;r&&$l(r,"".concat(K3,", ").concat(Z3))&&t.setActiveState(r,!0),r&&$l(r,X3)&&(t.setActiveState(El(K3,r),!0),t.setActiveState(r,!0))}}})),a&&a.length>0&&this.$root&&this.$root.$emit(a8,e,a)}},{key:"clear",value:function(){var e=this;Fl("".concat(this.$selector,", ").concat(X3),this.$el).filter((function(e){return ql(e,q3)})).forEach((function(t){return e.setActiveState(t,!1)}))}},{key:"setActiveState",value:function(e,t){e&&(t?Ul(e,q3):Gl(e,q3))}}],[{key:"Name",get:function(){return U3}},{key:"Default",get:function(){return i8}},{key:"DefaultType",get:function(){return o8}}]),e}(),d8="__BV_Scrollspy__",h8=/^\d+$/,f8=/^(auto|position|offset)$/,p8=function(e){var t={};return e.arg&&(t.element="#".concat(e.arg)),ea(e.modifiers).forEach((function(e){h8.test(e)?t.offset=fl(e,0):f8.test(e)&&(t.method=e)})),xt(e.value)?t.element=e.value:Bt(e.value)?t.offset=bu(e.value):It(e.value)&&ea(e.value).filter((function(e){return!!u8.DefaultType[e]})).forEach((function(a){t[a]=e.value[a]})),t},m8=function(e,t,a){if(Me){var n=p8(t);e[d8]?e[d8].updateConfig(n,a.context.$root):e[d8]=new u8(e,n,a.context.$root)}},v8=function(e){e[d8]&&(e[d8].dispose(),e[d8]=null,delete e[d8])},_8={bind:function(e,t,a){m8(e,t,a)},inserted:function(e,t,a){m8(e,t,a)},update:function(e,t,a){t.value!==t.oldValue&&m8(e,t,a)},componentUpdated:function(e,t,a){t.value!==t.oldValue&&m8(e,t,a)},unbind:function(e){v8(e)}},g8=Pa({directives:{VBScrollspy:_8}}),y8=Pa({directives:{VBVisible:aF}}),b8=Pa({plugins:{VBHoverPlugin:P3,VBModalPlugin:I3,VBPopoverPlugin:XQ,VBScrollspyPlugin:g8,VBTogglePlugin:XE,VBTooltipPlugin:x3,VBVisiblePlugin:y8}}),M8="BootstrapVue",z8=Ba({plugins:{componentsPlugin:Y3,directivesPlugin:b8}}),L8={install:z8,NAME:M8},w8=Sc(ia(Wu,["content","stacked"]),ir),H8=i["default"].extend({name:ir,functional:!0,props:w8,render:function(e,t){var a=t.data,n=t.props,r=t.children;return e(Uu,Ja(a,{staticClass:"b-iconstack",props:n}),r)}}),k8=Ia({components:{BIcon:kB,BIconstack:H8,BIconBlank:Zu,BIconAlarm:Qu,BIconAlarmFill:ed,BIconAlignBottom:td,BIconAlignCenter:ad,BIconAlignEnd:nd,BIconAlignMiddle:rd,BIconAlignStart:id,BIconAlignTop:od,BIconAlt:sd,BIconApp:ld,BIconAppIndicator:cd,BIconArchive:ud,BIconArchiveFill:dd,BIconArrow90degDown:hd,BIconArrow90degLeft:fd,BIconArrow90degRight:pd,BIconArrow90degUp:md,BIconArrowBarDown:vd,BIconArrowBarLeft:_d,BIconArrowBarRight:gd,BIconArrowBarUp:yd,BIconArrowClockwise:bd,BIconArrowCounterclockwise:Md,BIconArrowDown:zd,BIconArrowDownCircle:Ld,BIconArrowDownCircleFill:wd,BIconArrowDownLeft:Hd,BIconArrowDownLeftCircle:kd,BIconArrowDownLeftCircleFill:Ad,BIconArrowDownLeftSquare:Sd,BIconArrowDownLeftSquareFill:Vd,BIconArrowDownRight:Od,BIconArrowDownRightCircle:Td,BIconArrowDownRightCircleFill:Dd,BIconArrowDownRightSquare:Cd,BIconArrowDownRightSquareFill:jd,BIconArrowDownShort:xd,BIconArrowDownSquare:Bd,BIconArrowDownSquareFill:Yd,BIconArrowDownUp:Pd,BIconArrowLeft:Id,BIconArrowLeftCircle:Fd,BIconArrowLeftCircleFill:Ed,BIconArrowLeftRight:$d,BIconArrowLeftShort:Rd,BIconArrowLeftSquare:Nd,BIconArrowLeftSquareFill:Wd,BIconArrowRepeat:Ud,BIconArrowReturnLeft:Gd,BIconArrowReturnRight:qd,BIconArrowRight:Jd,BIconArrowRightCircle:Kd,BIconArrowRightCircleFill:Xd,BIconArrowRightShort:Zd,BIconArrowRightSquare:Qd,BIconArrowRightSquareFill:eh,BIconArrowUp:th,BIconArrowUpCircle:ah,BIconArrowUpCircleFill:nh,BIconArrowUpLeft:rh,BIconArrowUpLeftCircle:ih,BIconArrowUpLeftCircleFill:oh,BIconArrowUpLeftSquare:sh,BIconArrowUpLeftSquareFill:lh,BIconArrowUpRight:ch,BIconArrowUpRightCircle:uh,BIconArrowUpRightCircleFill:dh,BIconArrowUpRightSquare:hh,BIconArrowUpRightSquareFill:fh,BIconArrowUpShort:ph,BIconArrowUpSquare:mh,BIconArrowUpSquareFill:vh,BIconArrowsAngleContract:_h,BIconArrowsAngleExpand:gh,BIconArrowsCollapse:yh,BIconArrowsExpand:bh,BIconArrowsFullscreen:Mh,BIconArrowsMove:zh,BIconAspectRatio:Lh,BIconAspectRatioFill:wh,BIconAsterisk:Hh,BIconAt:kh,BIconAward:Ah,BIconAwardFill:Sh,BIconBack:Vh,BIconBackspace:Oh,BIconBackspaceFill:Th,BIconBackspaceReverse:Dh,BIconBackspaceReverseFill:Ch,BIconBadge3d:jh,BIconBadge3dFill:xh,BIconBadge4k:Bh,BIconBadge4kFill:Yh,BIconBadge8k:Ph,BIconBadge8kFill:Ih,BIconBadgeAd:Fh,BIconBadgeAdFill:Eh,BIconBadgeAr:$h,BIconBadgeArFill:Rh,BIconBadgeCc:Nh,BIconBadgeCcFill:Wh,BIconBadgeHd:Uh,BIconBadgeHdFill:Gh,BIconBadgeTm:qh,BIconBadgeTmFill:Jh,BIconBadgeVo:Kh,BIconBadgeVoFill:Xh,BIconBadgeVr:Zh,BIconBadgeVrFill:Qh,BIconBadgeWc:ef,BIconBadgeWcFill:tf,BIconBag:af,BIconBagCheck:nf,BIconBagCheckFill:rf,BIconBagDash:of,BIconBagDashFill:sf,BIconBagFill:lf,BIconBagPlus:cf,BIconBagPlusFill:uf,BIconBagX:df,BIconBagXFill:hf,BIconBank:ff,BIconBank2:pf,BIconBarChart:mf,BIconBarChartFill:vf,BIconBarChartLine:_f,BIconBarChartLineFill:gf,BIconBarChartSteps:yf,BIconBasket:bf,BIconBasket2:Mf,BIconBasket2Fill:zf,BIconBasket3:Lf,BIconBasket3Fill:wf,BIconBasketFill:Hf,BIconBattery:kf,BIconBatteryCharging:Af,BIconBatteryFull:Sf,BIconBatteryHalf:Vf,BIconBell:Of,BIconBellFill:Tf,BIconBellSlash:Df,BIconBellSlashFill:Cf,BIconBezier:jf,BIconBezier2:xf,BIconBicycle:Bf,BIconBinoculars:Yf,BIconBinocularsFill:Pf,BIconBlockquoteLeft:If,BIconBlockquoteRight:Ff,BIconBook:Ef,BIconBookFill:$f,BIconBookHalf:Rf,BIconBookmark:Nf,BIconBookmarkCheck:Wf,BIconBookmarkCheckFill:Uf,BIconBookmarkDash:Gf,BIconBookmarkDashFill:qf,BIconBookmarkFill:Jf,BIconBookmarkHeart:Kf,BIconBookmarkHeartFill:Xf,BIconBookmarkPlus:Zf,BIconBookmarkPlusFill:Qf,BIconBookmarkStar:ep,BIconBookmarkStarFill:tp,BIconBookmarkX:ap,BIconBookmarkXFill:np,BIconBookmarks:rp,BIconBookmarksFill:ip,BIconBookshelf:op,BIconBootstrap:sp,BIconBootstrapFill:lp,BIconBootstrapReboot:cp,BIconBorder:up,BIconBorderAll:dp,BIconBorderBottom:hp,BIconBorderCenter:fp,BIconBorderInner:pp,BIconBorderLeft:mp,BIconBorderMiddle:vp,BIconBorderOuter:_p,BIconBorderRight:gp,BIconBorderStyle:yp,BIconBorderTop:bp,BIconBorderWidth:Mp,BIconBoundingBox:zp,BIconBoundingBoxCircles:Lp,BIconBox:wp,BIconBoxArrowDown:Hp,BIconBoxArrowDownLeft:kp,BIconBoxArrowDownRight:Ap,BIconBoxArrowInDown:Sp,BIconBoxArrowInDownLeft:Vp,BIconBoxArrowInDownRight:Op,BIconBoxArrowInLeft:Tp,BIconBoxArrowInRight:Dp,BIconBoxArrowInUp:Cp,BIconBoxArrowInUpLeft:jp,BIconBoxArrowInUpRight:xp,BIconBoxArrowLeft:Bp,BIconBoxArrowRight:Yp,BIconBoxArrowUp:Pp,BIconBoxArrowUpLeft:Ip,BIconBoxArrowUpRight:Fp,BIconBoxSeam:Ep,BIconBraces:$p,BIconBricks:Rp,BIconBriefcase:Np,BIconBriefcaseFill:Wp,BIconBrightnessAltHigh:Up,BIconBrightnessAltHighFill:Gp,BIconBrightnessAltLow:qp,BIconBrightnessAltLowFill:Jp,BIconBrightnessHigh:Kp,BIconBrightnessHighFill:Xp,BIconBrightnessLow:Zp,BIconBrightnessLowFill:Qp,BIconBroadcast:em,BIconBroadcastPin:tm,BIconBrush:am,BIconBrushFill:nm,BIconBucket:rm,BIconBucketFill:im,BIconBug:om,BIconBugFill:sm,BIconBuilding:lm,BIconBullseye:cm,BIconCalculator:um,BIconCalculatorFill:dm,BIconCalendar:hm,BIconCalendar2:fm,BIconCalendar2Check:pm,BIconCalendar2CheckFill:mm,BIconCalendar2Date:vm,BIconCalendar2DateFill:_m,BIconCalendar2Day:gm,BIconCalendar2DayFill:ym,BIconCalendar2Event:bm,BIconCalendar2EventFill:Mm,BIconCalendar2Fill:zm,BIconCalendar2Minus:Lm,BIconCalendar2MinusFill:wm,BIconCalendar2Month:Hm,BIconCalendar2MonthFill:km,BIconCalendar2Plus:Am,BIconCalendar2PlusFill:Sm,BIconCalendar2Range:Vm,BIconCalendar2RangeFill:Om,BIconCalendar2Week:Tm,BIconCalendar2WeekFill:Dm,BIconCalendar2X:Cm,BIconCalendar2XFill:jm,BIconCalendar3:xm,BIconCalendar3Event:Bm,BIconCalendar3EventFill:Ym,BIconCalendar3Fill:Pm,BIconCalendar3Range:Im,BIconCalendar3RangeFill:Fm,BIconCalendar3Week:Em,BIconCalendar3WeekFill:$m,BIconCalendar4:Rm,BIconCalendar4Event:Nm,BIconCalendar4Range:Wm,BIconCalendar4Week:Um,BIconCalendarCheck:Gm,BIconCalendarCheckFill:qm,BIconCalendarDate:Jm,BIconCalendarDateFill:Km,BIconCalendarDay:Xm,BIconCalendarDayFill:Zm,BIconCalendarEvent:Qm,BIconCalendarEventFill:ev,BIconCalendarFill:tv,BIconCalendarMinus:av,BIconCalendarMinusFill:nv,BIconCalendarMonth:rv,BIconCalendarMonthFill:iv,BIconCalendarPlus:ov,BIconCalendarPlusFill:sv,BIconCalendarRange:lv,BIconCalendarRangeFill:cv,BIconCalendarWeek:uv,BIconCalendarWeekFill:dv,BIconCalendarX:hv,BIconCalendarXFill:fv,BIconCamera:pv,BIconCamera2:mv,BIconCameraFill:vv,BIconCameraReels:_v,BIconCameraReelsFill:gv,BIconCameraVideo:yv,BIconCameraVideoFill:bv,BIconCameraVideoOff:Mv,BIconCameraVideoOffFill:zv,BIconCapslock:Lv,BIconCapslockFill:wv,BIconCardChecklist:Hv,BIconCardHeading:kv,BIconCardImage:Av,BIconCardList:Sv,BIconCardText:Vv,BIconCaretDown:Ov,BIconCaretDownFill:Tv,BIconCaretDownSquare:Dv,BIconCaretDownSquareFill:Cv,BIconCaretLeft:jv,BIconCaretLeftFill:xv,BIconCaretLeftSquare:Bv,BIconCaretLeftSquareFill:Yv,BIconCaretRight:Pv,BIconCaretRightFill:Iv,BIconCaretRightSquare:Fv,BIconCaretRightSquareFill:Ev,BIconCaretUp:$v,BIconCaretUpFill:Rv,BIconCaretUpSquare:Nv,BIconCaretUpSquareFill:Wv,BIconCart:Uv,BIconCart2:Gv,BIconCart3:qv,BIconCart4:Jv,BIconCartCheck:Kv,BIconCartCheckFill:Xv,BIconCartDash:Zv,BIconCartDashFill:Qv,BIconCartFill:e_,BIconCartPlus:t_,BIconCartPlusFill:a_,BIconCartX:n_,BIconCartXFill:r_,BIconCash:i_,BIconCashCoin:o_,BIconCashStack:s_,BIconCast:l_,BIconChat:c_,BIconChatDots:u_,BIconChatDotsFill:d_,BIconChatFill:h_,BIconChatLeft:f_,BIconChatLeftDots:p_,BIconChatLeftDotsFill:m_,BIconChatLeftFill:v_,BIconChatLeftQuote:__,BIconChatLeftQuoteFill:g_,BIconChatLeftText:y_,BIconChatLeftTextFill:b_,BIconChatQuote:M_,BIconChatQuoteFill:z_,BIconChatRight:L_,BIconChatRightDots:w_,BIconChatRightDotsFill:H_,BIconChatRightFill:k_,BIconChatRightQuote:A_,BIconChatRightQuoteFill:S_,BIconChatRightText:V_,BIconChatRightTextFill:O_,BIconChatSquare:T_,BIconChatSquareDots:D_,BIconChatSquareDotsFill:C_,BIconChatSquareFill:j_,BIconChatSquareQuote:x_,BIconChatSquareQuoteFill:B_,BIconChatSquareText:Y_,BIconChatSquareTextFill:P_,BIconChatText:I_,BIconChatTextFill:F_,BIconCheck:E_,BIconCheck2:$_,BIconCheck2All:R_,BIconCheck2Circle:N_,BIconCheck2Square:W_,BIconCheckAll:U_,BIconCheckCircle:G_,BIconCheckCircleFill:q_,BIconCheckLg:J_,BIconCheckSquare:K_,BIconCheckSquareFill:X_,BIconChevronBarContract:Z_,BIconChevronBarDown:Q_,BIconChevronBarExpand:eg,BIconChevronBarLeft:tg,BIconChevronBarRight:ag,BIconChevronBarUp:ng,BIconChevronCompactDown:rg,BIconChevronCompactLeft:ig,BIconChevronCompactRight:og,BIconChevronCompactUp:sg,BIconChevronContract:lg,BIconChevronDoubleDown:cg,BIconChevronDoubleLeft:ug,BIconChevronDoubleRight:dg,BIconChevronDoubleUp:hg,BIconChevronDown:fg,BIconChevronExpand:pg,BIconChevronLeft:mg,BIconChevronRight:vg,BIconChevronUp:_g,BIconCircle:gg,BIconCircleFill:yg,BIconCircleHalf:bg,BIconCircleSquare:Mg,BIconClipboard:zg,BIconClipboardCheck:Lg,BIconClipboardData:wg,BIconClipboardMinus:Hg,BIconClipboardPlus:kg,BIconClipboardX:Ag,BIconClock:Sg,BIconClockFill:Vg,BIconClockHistory:Og,BIconCloud:Tg,BIconCloudArrowDown:Dg,BIconCloudArrowDownFill:Cg,BIconCloudArrowUp:jg,BIconCloudArrowUpFill:xg,BIconCloudCheck:Bg,BIconCloudCheckFill:Yg,BIconCloudDownload:Pg,BIconCloudDownloadFill:Ig,BIconCloudDrizzle:Fg,BIconCloudDrizzleFill:Eg,BIconCloudFill:$g,BIconCloudFog:Rg,BIconCloudFog2:Ng,BIconCloudFog2Fill:Wg,BIconCloudFogFill:Ug,BIconCloudHail:Gg,BIconCloudHailFill:qg,BIconCloudHaze:Jg,BIconCloudHaze1:Kg,BIconCloudHaze2Fill:Xg,BIconCloudHazeFill:Zg,BIconCloudLightning:Qg,BIconCloudLightningFill:ey,BIconCloudLightningRain:ty,BIconCloudLightningRainFill:ay,BIconCloudMinus:ny,BIconCloudMinusFill:ry,BIconCloudMoon:iy,BIconCloudMoonFill:oy,BIconCloudPlus:sy,BIconCloudPlusFill:ly,BIconCloudRain:cy,BIconCloudRainFill:uy,BIconCloudRainHeavy:dy,BIconCloudRainHeavyFill:hy,BIconCloudSlash:fy,BIconCloudSlashFill:py,BIconCloudSleet:my,BIconCloudSleetFill:vy,BIconCloudSnow:_y,BIconCloudSnowFill:gy,BIconCloudSun:yy,BIconCloudSunFill:by,BIconCloudUpload:My,BIconCloudUploadFill:zy,BIconClouds:Ly,BIconCloudsFill:wy,BIconCloudy:Hy,BIconCloudyFill:ky,BIconCode:Ay,BIconCodeSlash:Sy,BIconCodeSquare:Vy,BIconCoin:Oy,BIconCollection:Ty,BIconCollectionFill:Dy,BIconCollectionPlay:Cy,BIconCollectionPlayFill:jy,BIconColumns:xy,BIconColumnsGap:By,BIconCommand:Yy,BIconCompass:Py,BIconCompassFill:Iy,BIconCone:Fy,BIconConeStriped:Ey,BIconController:$y,BIconCpu:Ry,BIconCpuFill:Ny,BIconCreditCard:Wy,BIconCreditCard2Back:Uy,BIconCreditCard2BackFill:Gy,BIconCreditCard2Front:qy,BIconCreditCard2FrontFill:Jy,BIconCreditCardFill:Ky,BIconCrop:Xy,BIconCup:Zy,BIconCupFill:Qy,BIconCupStraw:eb,BIconCurrencyBitcoin:tb,BIconCurrencyDollar:ab,BIconCurrencyEuro:nb,BIconCurrencyExchange:rb,BIconCurrencyPound:ib,BIconCurrencyYen:ob,BIconCursor:sb,BIconCursorFill:lb,BIconCursorText:cb,BIconDash:ub,BIconDashCircle:db,BIconDashCircleDotted:hb,BIconDashCircleFill:fb,BIconDashLg:pb,BIconDashSquare:mb,BIconDashSquareDotted:vb,BIconDashSquareFill:_b,BIconDiagram2:gb,BIconDiagram2Fill:yb,BIconDiagram3:bb,BIconDiagram3Fill:Mb,BIconDiamond:zb,BIconDiamondFill:Lb,BIconDiamondHalf:wb,BIconDice1:Hb,BIconDice1Fill:kb,BIconDice2:Ab,BIconDice2Fill:Sb,BIconDice3:Vb,BIconDice3Fill:Ob,BIconDice4:Tb,BIconDice4Fill:Db,BIconDice5:Cb,BIconDice5Fill:jb,BIconDice6:xb,BIconDice6Fill:Bb,BIconDisc:Yb,BIconDiscFill:Pb,BIconDiscord:Ib,BIconDisplay:Fb,BIconDisplayFill:Eb,BIconDistributeHorizontal:$b,BIconDistributeVertical:Rb,BIconDoorClosed:Nb,BIconDoorClosedFill:Wb,BIconDoorOpen:Ub,BIconDoorOpenFill:Gb,BIconDot:qb,BIconDownload:Jb,BIconDroplet:Kb,BIconDropletFill:Xb,BIconDropletHalf:Zb,BIconEarbuds:Qb,BIconEasel:eM,BIconEaselFill:tM,BIconEgg:aM,BIconEggFill:nM,BIconEggFried:rM,BIconEject:iM,BIconEjectFill:oM,BIconEmojiAngry:sM,BIconEmojiAngryFill:lM,BIconEmojiDizzy:cM,BIconEmojiDizzyFill:uM,BIconEmojiExpressionless:dM,BIconEmojiExpressionlessFill:hM,BIconEmojiFrown:fM,BIconEmojiFrownFill:pM,BIconEmojiHeartEyes:mM,BIconEmojiHeartEyesFill:vM,BIconEmojiLaughing:_M,BIconEmojiLaughingFill:gM,BIconEmojiNeutral:yM,BIconEmojiNeutralFill:bM,BIconEmojiSmile:MM,BIconEmojiSmileFill:zM,BIconEmojiSmileUpsideDown:LM,BIconEmojiSmileUpsideDownFill:wM,BIconEmojiSunglasses:HM,BIconEmojiSunglassesFill:kM,BIconEmojiWink:AM,BIconEmojiWinkFill:SM,BIconEnvelope:VM,BIconEnvelopeFill:OM,BIconEnvelopeOpen:TM,BIconEnvelopeOpenFill:DM,BIconEraser:CM,BIconEraserFill:jM,BIconExclamation:xM,BIconExclamationCircle:BM,BIconExclamationCircleFill:YM,BIconExclamationDiamond:PM,BIconExclamationDiamondFill:IM,BIconExclamationLg:FM,BIconExclamationOctagon:EM,BIconExclamationOctagonFill:$M,BIconExclamationSquare:RM,BIconExclamationSquareFill:NM,BIconExclamationTriangle:WM,BIconExclamationTriangleFill:UM,BIconExclude:GM,BIconEye:qM,BIconEyeFill:JM,BIconEyeSlash:KM,BIconEyeSlashFill:XM,BIconEyedropper:ZM,BIconEyeglasses:QM,BIconFacebook:ez,BIconFile:tz,BIconFileArrowDown:az,BIconFileArrowDownFill:nz,BIconFileArrowUp:rz,BIconFileArrowUpFill:iz,BIconFileBarGraph:oz,BIconFileBarGraphFill:sz,BIconFileBinary:lz,BIconFileBinaryFill:cz,BIconFileBreak:uz,BIconFileBreakFill:dz,BIconFileCheck:hz,BIconFileCheckFill:fz,BIconFileCode:pz,BIconFileCodeFill:mz,BIconFileDiff:vz,BIconFileDiffFill:_z,BIconFileEarmark:gz,BIconFileEarmarkArrowDown:yz,BIconFileEarmarkArrowDownFill:bz,BIconFileEarmarkArrowUp:Mz,BIconFileEarmarkArrowUpFill:zz,BIconFileEarmarkBarGraph:Lz,BIconFileEarmarkBarGraphFill:wz,BIconFileEarmarkBinary:Hz,BIconFileEarmarkBinaryFill:kz,BIconFileEarmarkBreak:Az,BIconFileEarmarkBreakFill:Sz,BIconFileEarmarkCheck:Vz,BIconFileEarmarkCheckFill:Oz,BIconFileEarmarkCode:Tz,BIconFileEarmarkCodeFill:Dz,BIconFileEarmarkDiff:Cz,BIconFileEarmarkDiffFill:jz,BIconFileEarmarkEasel:xz,BIconFileEarmarkEaselFill:Bz,BIconFileEarmarkExcel:Yz,BIconFileEarmarkExcelFill:Pz,BIconFileEarmarkFill:Iz,BIconFileEarmarkFont:Fz,BIconFileEarmarkFontFill:Ez,BIconFileEarmarkImage:$z,BIconFileEarmarkImageFill:Rz,BIconFileEarmarkLock:Nz,BIconFileEarmarkLock2:Wz,BIconFileEarmarkLock2Fill:Uz,BIconFileEarmarkLockFill:Gz,BIconFileEarmarkMedical:qz,BIconFileEarmarkMedicalFill:Jz,BIconFileEarmarkMinus:Kz,BIconFileEarmarkMinusFill:Xz,BIconFileEarmarkMusic:Zz,BIconFileEarmarkMusicFill:Qz,BIconFileEarmarkPdf:eL,BIconFileEarmarkPdfFill:tL,BIconFileEarmarkPerson:aL,BIconFileEarmarkPersonFill:nL,BIconFileEarmarkPlay:rL,BIconFileEarmarkPlayFill:iL,BIconFileEarmarkPlus:oL,BIconFileEarmarkPlusFill:sL,BIconFileEarmarkPost:lL,BIconFileEarmarkPostFill:cL,BIconFileEarmarkPpt:uL,BIconFileEarmarkPptFill:dL,BIconFileEarmarkRichtext:hL,BIconFileEarmarkRichtextFill:fL,BIconFileEarmarkRuled:pL,BIconFileEarmarkRuledFill:mL,BIconFileEarmarkSlides:vL,BIconFileEarmarkSlidesFill:_L,BIconFileEarmarkSpreadsheet:gL,BIconFileEarmarkSpreadsheetFill:yL,BIconFileEarmarkText:bL,BIconFileEarmarkTextFill:ML,BIconFileEarmarkWord:zL,BIconFileEarmarkWordFill:LL,BIconFileEarmarkX:wL,BIconFileEarmarkXFill:HL,BIconFileEarmarkZip:kL,BIconFileEarmarkZipFill:AL,BIconFileEasel:SL,BIconFileEaselFill:VL,BIconFileExcel:OL,BIconFileExcelFill:TL,BIconFileFill:DL,BIconFileFont:CL,BIconFileFontFill:jL,BIconFileImage:xL,BIconFileImageFill:BL,BIconFileLock:YL,BIconFileLock2:PL,BIconFileLock2Fill:IL,BIconFileLockFill:FL,BIconFileMedical:EL,BIconFileMedicalFill:$L,BIconFileMinus:RL,BIconFileMinusFill:NL,BIconFileMusic:WL,BIconFileMusicFill:UL,BIconFilePdf:GL,BIconFilePdfFill:qL,BIconFilePerson:JL,BIconFilePersonFill:KL,BIconFilePlay:XL,BIconFilePlayFill:ZL,BIconFilePlus:QL,BIconFilePlusFill:ew,BIconFilePost:tw,BIconFilePostFill:aw,BIconFilePpt:nw,BIconFilePptFill:rw,BIconFileRichtext:iw,BIconFileRichtextFill:ow,BIconFileRuled:sw,BIconFileRuledFill:lw,BIconFileSlides:cw,BIconFileSlidesFill:uw,BIconFileSpreadsheet:dw,BIconFileSpreadsheetFill:hw,BIconFileText:fw,BIconFileTextFill:pw,BIconFileWord:mw,BIconFileWordFill:vw,BIconFileX:_w,BIconFileXFill:gw,BIconFileZip:yw,BIconFileZipFill:bw,BIconFiles:Mw,BIconFilesAlt:zw,BIconFilm:Lw,BIconFilter:ww,BIconFilterCircle:Hw,BIconFilterCircleFill:kw,BIconFilterLeft:Aw,BIconFilterRight:Sw,BIconFilterSquare:Vw,BIconFilterSquareFill:Ow,BIconFlag:Tw,BIconFlagFill:Dw,BIconFlower1:Cw,BIconFlower2:jw,BIconFlower3:xw,BIconFolder:Bw,BIconFolder2:Yw,BIconFolder2Open:Pw,BIconFolderCheck:Iw,BIconFolderFill:Fw,BIconFolderMinus:Ew,BIconFolderPlus:$w,BIconFolderSymlink:Rw,BIconFolderSymlinkFill:Nw,BIconFolderX:Ww,BIconFonts:Uw,BIconForward:Gw,BIconForwardFill:qw,BIconFront:Jw,BIconFullscreen:Kw,BIconFullscreenExit:Xw,BIconFunnel:Zw,BIconFunnelFill:Qw,BIconGear:eH,BIconGearFill:tH,BIconGearWide:aH,BIconGearWideConnected:nH,BIconGem:rH,BIconGenderAmbiguous:iH,BIconGenderFemale:oH,BIconGenderMale:sH,BIconGenderTrans:lH,BIconGeo:cH,BIconGeoAlt:uH,BIconGeoAltFill:dH,BIconGeoFill:hH,BIconGift:fH,BIconGiftFill:pH,BIconGithub:mH,BIconGlobe:vH,BIconGlobe2:_H,BIconGoogle:gH,BIconGraphDown:yH,BIconGraphUp:bH,BIconGrid:MH,BIconGrid1x2:zH,BIconGrid1x2Fill:LH,BIconGrid3x2:wH,BIconGrid3x2Gap:HH,BIconGrid3x2GapFill:kH,BIconGrid3x3:AH,BIconGrid3x3Gap:SH,BIconGrid3x3GapFill:VH,BIconGridFill:OH,BIconGripHorizontal:TH,BIconGripVertical:DH,BIconHammer:CH,BIconHandIndex:jH,BIconHandIndexFill:xH,BIconHandIndexThumb:BH,BIconHandIndexThumbFill:YH,BIconHandThumbsDown:PH,BIconHandThumbsDownFill:IH,BIconHandThumbsUp:FH,BIconHandThumbsUpFill:EH,BIconHandbag:$H,BIconHandbagFill:RH,BIconHash:NH,BIconHdd:WH,BIconHddFill:UH,BIconHddNetwork:GH,BIconHddNetworkFill:qH,BIconHddRack:JH,BIconHddRackFill:KH,BIconHddStack:XH,BIconHddStackFill:ZH,BIconHeadphones:QH,BIconHeadset:ek,BIconHeadsetVr:tk,BIconHeart:ak,BIconHeartFill:nk,BIconHeartHalf:rk,BIconHeptagon:ik,BIconHeptagonFill:ok,BIconHeptagonHalf:sk,BIconHexagon:lk,BIconHexagonFill:ck,BIconHexagonHalf:uk,BIconHourglass:dk,BIconHourglassBottom:hk,BIconHourglassSplit:fk,BIconHourglassTop:pk,BIconHouse:mk,BIconHouseDoor:vk,BIconHouseDoorFill:_k,BIconHouseFill:gk,BIconHr:yk,BIconHurricane:bk,BIconImage:Mk,BIconImageAlt:zk,BIconImageFill:Lk,BIconImages:wk,BIconInbox:Hk,BIconInboxFill:kk,BIconInboxes:Ak,BIconInboxesFill:Sk,BIconInfo:Vk,BIconInfoCircle:Ok,BIconInfoCircleFill:Tk,BIconInfoLg:Dk,BIconInfoSquare:Ck,BIconInfoSquareFill:jk,BIconInputCursor:xk,BIconInputCursorText:Bk,BIconInstagram:Yk,BIconIntersect:Pk,BIconJournal:Ik,BIconJournalAlbum:Fk,BIconJournalArrowDown:Ek,BIconJournalArrowUp:$k,BIconJournalBookmark:Rk,BIconJournalBookmarkFill:Nk,BIconJournalCheck:Wk,BIconJournalCode:Uk,BIconJournalMedical:Gk,BIconJournalMinus:qk,BIconJournalPlus:Jk,BIconJournalRichtext:Kk,BIconJournalText:Xk,BIconJournalX:Zk,BIconJournals:Qk,BIconJoystick:eA,BIconJustify:tA,BIconJustifyLeft:aA,BIconJustifyRight:nA,BIconKanban:rA,BIconKanbanFill:iA,BIconKey:oA,BIconKeyFill:sA,BIconKeyboard:lA,BIconKeyboardFill:cA,BIconLadder:uA,BIconLamp:dA,BIconLampFill:hA,BIconLaptop:fA,BIconLaptopFill:pA,BIconLayerBackward:mA,BIconLayerForward:vA,BIconLayers:_A,BIconLayersFill:gA,BIconLayersHalf:yA,BIconLayoutSidebar:bA,BIconLayoutSidebarInset:MA,BIconLayoutSidebarInsetReverse:zA,BIconLayoutSidebarReverse:LA,BIconLayoutSplit:wA,BIconLayoutTextSidebar:HA,BIconLayoutTextSidebarReverse:kA,BIconLayoutTextWindow:AA,BIconLayoutTextWindowReverse:SA,BIconLayoutThreeColumns:VA,BIconLayoutWtf:OA,BIconLifePreserver:TA,BIconLightbulb:DA,BIconLightbulbFill:CA,BIconLightbulbOff:jA,BIconLightbulbOffFill:xA,BIconLightning:BA,BIconLightningCharge:YA,BIconLightningChargeFill:PA,BIconLightningFill:IA,BIconLink:FA,BIconLink45deg:EA,BIconLinkedin:$A,BIconList:RA,BIconListCheck:NA,BIconListNested:WA,BIconListOl:UA,BIconListStars:GA,BIconListTask:qA,BIconListUl:JA,BIconLock:KA,BIconLockFill:XA,BIconMailbox:ZA,BIconMailbox2:QA,BIconMap:eS,BIconMapFill:tS,BIconMarkdown:aS,BIconMarkdownFill:nS,BIconMask:rS,BIconMastodon:iS,BIconMegaphone:oS,BIconMegaphoneFill:sS,BIconMenuApp:lS,BIconMenuAppFill:cS,BIconMenuButton:uS,BIconMenuButtonFill:dS,BIconMenuButtonWide:hS,BIconMenuButtonWideFill:fS,BIconMenuDown:pS,BIconMenuUp:mS,BIconMessenger:vS,BIconMic:_S,BIconMicFill:gS,BIconMicMute:yS,BIconMicMuteFill:bS,BIconMinecart:MS,BIconMinecartLoaded:zS,BIconMoisture:LS,BIconMoon:wS,BIconMoonFill:HS,BIconMoonStars:kS,BIconMoonStarsFill:AS,BIconMouse:SS,BIconMouse2:VS,BIconMouse2Fill:OS,BIconMouse3:TS,BIconMouse3Fill:DS,BIconMouseFill:CS,BIconMusicNote:jS,BIconMusicNoteBeamed:xS,BIconMusicNoteList:BS,BIconMusicPlayer:YS,BIconMusicPlayerFill:PS,BIconNewspaper:IS,BIconNodeMinus:FS,BIconNodeMinusFill:ES,BIconNodePlus:$S,BIconNodePlusFill:RS,BIconNut:NS,BIconNutFill:WS,BIconOctagon:US,BIconOctagonFill:GS,BIconOctagonHalf:qS,BIconOption:JS,BIconOutlet:KS,BIconPaintBucket:XS,BIconPalette:ZS,BIconPalette2:QS,BIconPaletteFill:eV,BIconPaperclip:tV,BIconParagraph:aV,BIconPatchCheck:nV,BIconPatchCheckFill:rV,BIconPatchExclamation:iV,BIconPatchExclamationFill:oV,BIconPatchMinus:sV,BIconPatchMinusFill:lV,BIconPatchPlus:cV,BIconPatchPlusFill:uV,BIconPatchQuestion:dV,BIconPatchQuestionFill:hV,BIconPause:fV,BIconPauseBtn:pV,BIconPauseBtnFill:mV,BIconPauseCircle:vV,BIconPauseCircleFill:_V,BIconPauseFill:gV,BIconPeace:yV,BIconPeaceFill:bV,BIconPen:MV,BIconPenFill:zV,BIconPencil:LV,BIconPencilFill:wV,BIconPencilSquare:HV,BIconPentagon:kV,BIconPentagonFill:AV,BIconPentagonHalf:SV,BIconPeople:VV,BIconPeopleFill:OV,BIconPercent:TV,BIconPerson:DV,BIconPersonBadge:CV,BIconPersonBadgeFill:jV,BIconPersonBoundingBox:xV,BIconPersonCheck:BV,BIconPersonCheckFill:YV,BIconPersonCircle:PV,BIconPersonDash:IV,BIconPersonDashFill:FV,BIconPersonFill:EV,BIconPersonLinesFill:$V,BIconPersonPlus:RV,BIconPersonPlusFill:NV,BIconPersonSquare:WV,BIconPersonX:UV,BIconPersonXFill:GV,BIconPhone:qV,BIconPhoneFill:JV,BIconPhoneLandscape:KV,BIconPhoneLandscapeFill:XV,BIconPhoneVibrate:ZV,BIconPhoneVibrateFill:QV,BIconPieChart:eO,BIconPieChartFill:tO,BIconPiggyBank:aO,BIconPiggyBankFill:nO,BIconPin:rO,BIconPinAngle:iO,BIconPinAngleFill:oO,BIconPinFill:sO,BIconPinMap:lO,BIconPinMapFill:cO,BIconPip:uO,BIconPipFill:dO,BIconPlay:hO,BIconPlayBtn:fO,BIconPlayBtnFill:pO,BIconPlayCircle:mO,BIconPlayCircleFill:vO,BIconPlayFill:_O,BIconPlug:gO,BIconPlugFill:yO,BIconPlus:bO,BIconPlusCircle:MO,BIconPlusCircleDotted:zO,BIconPlusCircleFill:LO,BIconPlusLg:wO,BIconPlusSquare:HO,BIconPlusSquareDotted:kO,BIconPlusSquareFill:AO,BIconPower:SO,BIconPrinter:VO,BIconPrinterFill:OO,BIconPuzzle:TO,BIconPuzzleFill:DO,BIconQuestion:CO,BIconQuestionCircle:jO,BIconQuestionCircleFill:xO,BIconQuestionDiamond:BO,BIconQuestionDiamondFill:YO,BIconQuestionLg:PO,BIconQuestionOctagon:IO,BIconQuestionOctagonFill:FO,BIconQuestionSquare:EO,BIconQuestionSquareFill:$O,BIconRainbow:RO,BIconReceipt:NO,BIconReceiptCutoff:WO,BIconReception0:UO,BIconReception1:GO,BIconReception2:qO,BIconReception3:JO,BIconReception4:KO,BIconRecord:XO,BIconRecord2:ZO,BIconRecord2Fill:QO,BIconRecordBtn:eT,BIconRecordBtnFill:tT,BIconRecordCircle:aT,BIconRecordCircleFill:nT,BIconRecordFill:rT,BIconRecycle:iT,BIconReddit:oT,BIconReply:sT,BIconReplyAll:lT,BIconReplyAllFill:cT,BIconReplyFill:uT,BIconRss:dT,BIconRssFill:hT,BIconRulers:fT,BIconSafe:pT,BIconSafe2:mT,BIconSafe2Fill:vT,BIconSafeFill:_T,BIconSave:gT,BIconSave2:yT,BIconSave2Fill:bT,BIconSaveFill:MT,BIconScissors:zT,BIconScrewdriver:LT,BIconSdCard:wT,BIconSdCardFill:HT,BIconSearch:kT,BIconSegmentedNav:AT,BIconServer:ST,BIconShare:VT,BIconShareFill:OT,BIconShield:TT,BIconShieldCheck:DT,BIconShieldExclamation:CT,BIconShieldFill:jT,BIconShieldFillCheck:xT,BIconShieldFillExclamation:BT,BIconShieldFillMinus:YT,BIconShieldFillPlus:PT,BIconShieldFillX:IT,BIconShieldLock:FT,BIconShieldLockFill:ET,BIconShieldMinus:$T,BIconShieldPlus:RT,BIconShieldShaded:NT,BIconShieldSlash:WT,BIconShieldSlashFill:UT,BIconShieldX:GT,BIconShift:qT,BIconShiftFill:JT,BIconShop:KT,BIconShopWindow:XT,BIconShuffle:ZT,BIconSignpost:QT,BIconSignpost2:eD,BIconSignpost2Fill:tD,BIconSignpostFill:aD,BIconSignpostSplit:nD,BIconSignpostSplitFill:rD,BIconSim:iD,BIconSimFill:oD,BIconSkipBackward:sD,BIconSkipBackwardBtn:lD,BIconSkipBackwardBtnFill:cD,BIconSkipBackwardCircle:uD,BIconSkipBackwardCircleFill:dD,BIconSkipBackwardFill:hD,BIconSkipEnd:fD,BIconSkipEndBtn:pD,BIconSkipEndBtnFill:mD,BIconSkipEndCircle:vD,BIconSkipEndCircleFill:_D,BIconSkipEndFill:gD,BIconSkipForward:yD,BIconSkipForwardBtn:bD,BIconSkipForwardBtnFill:MD,BIconSkipForwardCircle:zD,BIconSkipForwardCircleFill:LD,BIconSkipForwardFill:wD,BIconSkipStart:HD,BIconSkipStartBtn:kD,BIconSkipStartBtnFill:AD,BIconSkipStartCircle:SD,BIconSkipStartCircleFill:VD,BIconSkipStartFill:OD,BIconSkype:TD,BIconSlack:DD,BIconSlash:CD,BIconSlashCircle:jD,BIconSlashCircleFill:xD,BIconSlashLg:BD,BIconSlashSquare:YD,BIconSlashSquareFill:PD,BIconSliders:ID,BIconSmartwatch:FD,BIconSnow:ED,BIconSnow2:$D,BIconSnow3:RD,BIconSortAlphaDown:ND,BIconSortAlphaDownAlt:WD,BIconSortAlphaUp:UD,BIconSortAlphaUpAlt:GD,BIconSortDown:qD,BIconSortDownAlt:JD,BIconSortNumericDown:KD,BIconSortNumericDownAlt:XD,BIconSortNumericUp:ZD,BIconSortNumericUpAlt:QD,BIconSortUp:eC,BIconSortUpAlt:tC,BIconSoundwave:aC,BIconSpeaker:nC,BIconSpeakerFill:rC,BIconSpeedometer:iC,BIconSpeedometer2:oC,BIconSpellcheck:sC,BIconSquare:lC,BIconSquareFill:cC,BIconSquareHalf:uC,BIconStack:dC,BIconStar:hC,BIconStarFill:fC,BIconStarHalf:pC,BIconStars:mC,BIconStickies:vC,BIconStickiesFill:_C,BIconSticky:gC,BIconStickyFill:yC,BIconStop:bC,BIconStopBtn:MC,BIconStopBtnFill:zC,BIconStopCircle:LC,BIconStopCircleFill:wC,BIconStopFill:HC,BIconStoplights:kC,BIconStoplightsFill:AC,BIconStopwatch:SC,BIconStopwatchFill:VC,BIconSubtract:OC,BIconSuitClub:TC,BIconSuitClubFill:DC,BIconSuitDiamond:CC,BIconSuitDiamondFill:jC,BIconSuitHeart:xC,BIconSuitHeartFill:BC,BIconSuitSpade:YC,BIconSuitSpadeFill:PC,BIconSun:IC,BIconSunFill:FC,BIconSunglasses:EC,BIconSunrise:$C,BIconSunriseFill:RC,BIconSunset:NC,BIconSunsetFill:WC,BIconSymmetryHorizontal:UC,BIconSymmetryVertical:GC,BIconTable:qC,BIconTablet:JC,BIconTabletFill:KC,BIconTabletLandscape:XC,BIconTabletLandscapeFill:ZC,BIconTag:QC,BIconTagFill:ej,BIconTags:tj,BIconTagsFill:aj,BIconTelegram:nj,BIconTelephone:rj,BIconTelephoneFill:ij,BIconTelephoneForward:oj,BIconTelephoneForwardFill:sj,BIconTelephoneInbound:lj,BIconTelephoneInboundFill:cj,BIconTelephoneMinus:uj,BIconTelephoneMinusFill:dj,BIconTelephoneOutbound:hj,BIconTelephoneOutboundFill:fj,BIconTelephonePlus:pj,BIconTelephonePlusFill:mj,BIconTelephoneX:vj,BIconTelephoneXFill:_j,BIconTerminal:gj,BIconTerminalFill:yj,BIconTextCenter:bj,BIconTextIndentLeft:Mj,BIconTextIndentRight:zj,BIconTextLeft:Lj,BIconTextParagraph:wj,BIconTextRight:Hj,BIconTextarea:kj,BIconTextareaResize:Aj,BIconTextareaT:Sj,BIconThermometer:Vj,BIconThermometerHalf:Oj,BIconThermometerHigh:Tj,BIconThermometerLow:Dj,BIconThermometerSnow:Cj,BIconThermometerSun:jj,BIconThreeDots:xj,BIconThreeDotsVertical:Bj,BIconToggle2Off:Yj,BIconToggle2On:Pj,BIconToggleOff:Ij,BIconToggleOn:Fj,BIconToggles:Ej,BIconToggles2:$j,BIconTools:Rj,BIconTornado:Nj,BIconTranslate:Wj,BIconTrash:Uj,BIconTrash2:Gj,BIconTrash2Fill:qj,BIconTrashFill:Jj,BIconTree:Kj,BIconTreeFill:Xj,BIconTriangle:Zj,BIconTriangleFill:Qj,BIconTriangleHalf:ex,BIconTrophy:tx,BIconTrophyFill:ax,BIconTropicalStorm:nx,BIconTruck:rx,BIconTruckFlatbed:ix,BIconTsunami:ox,BIconTv:sx,BIconTvFill:lx,BIconTwitch:cx,BIconTwitter:ux,BIconType:dx,BIconTypeBold:hx,BIconTypeH1:fx,BIconTypeH2:px,BIconTypeH3:mx,BIconTypeItalic:vx,BIconTypeStrikethrough:_x,BIconTypeUnderline:gx,BIconUiChecks:yx,BIconUiChecksGrid:bx,BIconUiRadios:Mx,BIconUiRadiosGrid:zx,BIconUmbrella:Lx,BIconUmbrellaFill:wx,BIconUnion:Hx,BIconUnlock:kx,BIconUnlockFill:Ax,BIconUpc:Sx,BIconUpcScan:Vx,BIconUpload:Ox,BIconVectorPen:Tx,BIconViewList:Dx,BIconViewStacked:Cx,BIconVinyl:jx,BIconVinylFill:xx,BIconVoicemail:Bx,BIconVolumeDown:Yx,BIconVolumeDownFill:Px,BIconVolumeMute:Ix,BIconVolumeMuteFill:Fx,BIconVolumeOff:Ex,BIconVolumeOffFill:$x,BIconVolumeUp:Rx,BIconVolumeUpFill:Nx,BIconVr:Wx,BIconWallet:Ux,BIconWallet2:Gx,BIconWalletFill:qx,BIconWatch:Jx,BIconWater:Kx,BIconWhatsapp:Xx,BIconWifi:Zx,BIconWifi1:Qx,BIconWifi2:eB,BIconWifiOff:tB,BIconWind:aB,BIconWindow:nB,BIconWindowDock:rB,BIconWindowSidebar:iB,BIconWrench:oB,BIconX:sB,BIconXCircle:lB,BIconXCircleFill:cB,BIconXDiamond:uB,BIconXDiamondFill:dB,BIconXLg:hB,BIconXOctagon:fB,BIconXOctagonFill:pB,BIconXSquare:mB,BIconXSquareFill:vB,BIconYoutube:_B,BIconZoomIn:gB,BIconZoomOut:yB}}),A8=Ia({plugins:{IconsPlugin:k8}},{NAME:"BootstrapVueIcons"}),S8=a("d099");a("f9e3"),a("2dd8");i["default"].use(L8),i["default"].use(A8),i["default"].use(S8["TOMToolkitComponentLib"]),i["default"].use(o["a"]),i["default"].use(z),i["default"].config.productionTip=!1,r.a.defaults.xsrfHeaderName="X-CSRFToken",r.a.defaults.xsrfCookieName="csrftoken",r.a.defaults.withCredentials=!0,i["default"].prototype.$store.commit("setTomAxiosConfig",{xsrfHeaderName:"x-csrftoken",xsrfCookieName:"csrftoken",withCredentials:!0}),i["default"].prototype.$store.commit("setHermesAxiosConfig",{withCredentials:!1}),new i["default"]({el:"#superevent-sequences",components:{SupereventSequences:me}})},"342f":function(e,t,a){var n=a("d066");e.exports=n("navigator","userAgent")||""},3511:function(e,t){var a=TypeError,n=9007199254740991;e.exports=function(e){if(e>n)throw a("Maximum allowed index exceeded");return e}},"387f":function(e,t,a){"use strict";e.exports=function(e,t,a,n,r){return e.config=t,a&&(e.code=a),e.request=n,e.response=r,e.isAxiosError=!0,e.toJSON=function(){return{message:this.message,name:this.name,description:this.description,number:this.number,fileName:this.fileName,lineNumber:this.lineNumber,columnNumber:this.columnNumber,stack:this.stack,config:this.config,code:this.code}},e}},3886:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-ca",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"YYYY-MM-DD",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a}});return t}))},3934:function(e,t,a){"use strict";var n=a("c532");e.exports=n.isStandardBrowserEnv()?function(){var e,t=/(msie|trident)/i.test(navigator.userAgent),a=document.createElement("a");function r(e){var n=e;return t&&(a.setAttribute("href",n),n=a.href),a.setAttribute("href",n),{href:a.href,protocol:a.protocol?a.protocol.replace(/:$/,""):"",host:a.host,search:a.search?a.search.replace(/^\?/,""):"",hash:a.hash?a.hash.replace(/^#/,""):"",hostname:a.hostname,port:a.port,pathname:"/"===a.pathname.charAt(0)?a.pathname:"/"+a.pathname}}return e=r(window.location.href),function(t){var a=n.isString(t)?r(t):t;return a.protocol===e.protocol&&a.host===e.host}}():function(){return function(){return!0}}()},"39a6":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-gb",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:1,doy:4}});return t}))},"39bd":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},a={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"};function n(e,t,a,n){var r="";if(t)switch(a){case"s":r="काही सेकंद";break;case"ss":r="%d सेकंद";break;case"m":r="एक मिनिट";break;case"mm":r="%d मिनिटे";break;case"h":r="एक तास";break;case"hh":r="%d तास";break;case"d":r="एक दिवस";break;case"dd":r="%d दिवस";break;case"M":r="एक महिना";break;case"MM":r="%d महिने";break;case"y":r="एक वर्ष";break;case"yy":r="%d वर्षे";break}else switch(a){case"s":r="काही सेकंदां";break;case"ss":r="%d सेकंदां";break;case"m":r="एका मिनिटा";break;case"mm":r="%d मिनिटां";break;case"h":r="एका तासा";break;case"hh":r="%d तासां";break;case"d":r="एका दिवसा";break;case"dd":r="%d दिवसां";break;case"M":r="एका महिन्या";break;case"MM":r="%d महिन्यां";break;case"y":r="एका वर्षा";break;case"yy":r="%d वर्षां";break}return r.replace(/%d/i,e)}var r=e.defineLocale("mr",{months:"जानेवारी_फेब्रुवारी_मार्च_एप्रिल_मे_जून_जुलै_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),monthsShort:"जाने._फेब्रु._मार्च._एप्रि._मे._जून._जुलै._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"रविवार_सोमवार_मंगळवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगळ_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm वाजता",LTS:"A h:mm:ss वाजता",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm वाजता",LLLL:"dddd, D MMMM YYYY, A h:mm वाजता"},calendar:{sameDay:"[आज] LT",nextDay:"[उद्या] LT",nextWeek:"dddd, LT",lastDay:"[काल] LT",lastWeek:"[मागील] dddd, LT",sameElse:"L"},relativeTime:{future:"%sमध्ये",past:"%sपूर्वी",s:n,ss:n,m:n,mm:n,h:n,hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/पहाटे|सकाळी|दुपारी|सायंकाळी|रात्री/,meridiemHour:function(e,t){return 12===e&&(e=0),"पहाटे"===t||"सकाळी"===t?e:"दुपारी"===t||"सायंकाळी"===t||"रात्री"===t?e>=12?e:e+12:void 0},meridiem:function(e,t,a){return e>=0&&e<6?"पहाटे":e<12?"सकाळी":e<17?"दुपारी":e<20?"सायंकाळी":"रात्री"},week:{dow:0,doy:6}});return r}))},"3a34":function(e,t,a){"use strict";var n=a("83ab"),r=a("e8b5"),i=TypeError,o=Object.getOwnPropertyDescriptor,s=n&&!function(){if(void 0!==this)return!0;try{Object.defineProperty([],"length",{writable:!1}).length=1}catch(e){return e instanceof TypeError}}();e.exports=s?function(e,t){if(r(e)&&!o(e,"length").writable)throw i("Cannot set read only .length");return e.length=t}:function(e,t){return e.length=t}},"3a39":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},a={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},n=e.defineLocale("ne",{months:"जनवरी_फेब्रुवरी_मार्च_अप्रिल_मई_जुन_जुलाई_अगष्ट_सेप्टेम्बर_अक्टोबर_नोभेम्बर_डिसेम्बर".split("_"),monthsShort:"जन._फेब्रु._मार्च_अप्रि._मई_जुन_जुलाई._अग._सेप्ट._अक्टो._नोभे._डिसे.".split("_"),monthsParseExact:!0,weekdays:"आइतबार_सोमबार_मङ्गलबार_बुधबार_बिहिबार_शुक्रबार_शनिबार".split("_"),weekdaysShort:"आइत._सोम._मङ्गल._बुध._बिहि._शुक्र._शनि.".split("_"),weekdaysMin:"आ._सो._मं._बु._बि._शु._श.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"Aको h:mm बजे",LTS:"Aको h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, Aको h:mm बजे",LLLL:"dddd, D MMMM YYYY, Aको h:mm बजे"},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/राति|बिहान|दिउँसो|साँझ/,meridiemHour:function(e,t){return 12===e&&(e=0),"राति"===t?e<4?e:e+12:"बिहान"===t?e:"दिउँसो"===t?e>=10?e:e+12:"साँझ"===t?e+12:void 0},meridiem:function(e,t,a){return e<3?"राति":e<12?"बिहान":e<16?"दिउँसो":e<20?"साँझ":"राति"},calendar:{sameDay:"[आज] LT",nextDay:"[भोलि] LT",nextWeek:"[आउँदो] dddd[,] LT",lastDay:"[हिजो] LT",lastWeek:"[गएको] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%sमा",past:"%s अगाडि",s:"केही क्षण",ss:"%d सेकेण्ड",m:"एक मिनेट",mm:"%d मिनेट",h:"एक घण्टा",hh:"%d घण्टा",d:"एक दिन",dd:"%d दिन",M:"एक महिना",MM:"%d महिना",y:"एक बर्ष",yy:"%d बर्ष"},week:{dow:0,doy:6}});return n}))},"3a6c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-mo",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"D/M/YYYY",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,a){var n=100*e+t;return n<600?"凌晨":n<900?"早上":n<1130?"上午":n<1230?"中午":n<1800?"下午":"晚上"},calendar:{sameDay:"[今天] LT",nextDay:"[明天] LT",nextWeek:"[下]dddd LT",lastDay:"[昨天] LT",lastWeek:"[上]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s內",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}});return t}))},"3a9b":function(e,t,a){var n=a("e330");e.exports=n({}.isPrototypeOf)},"3b1b":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={0:"-ум",1:"-ум",2:"-юм",3:"-юм",4:"-ум",5:"-ум",6:"-ум",7:"-ум",8:"-ум",9:"-ум",10:"-ум",12:"-ум",13:"-ум",20:"-ум",30:"-юм",40:"-ум",50:"-ум",60:"-ум",70:"-ум",80:"-ум",90:"-ум",100:"-ум"},a=e.defineLocale("tg",{months:{format:"январи_феврали_марти_апрели_майи_июни_июли_августи_сентябри_октябри_ноябри_декабри".split("_"),standalone:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_")},monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"якшанбе_душанбе_сешанбе_чоршанбе_панҷшанбе_ҷумъа_шанбе".split("_"),weekdaysShort:"яшб_дшб_сшб_чшб_пшб_ҷум_шнб".split("_"),weekdaysMin:"яш_дш_сш_чш_пш_ҷм_шб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Имрӯз соати] LT",nextDay:"[Фардо соати] LT",lastDay:"[Дирӯз соати] LT",nextWeek:"dddd[и] [ҳафтаи оянда соати] LT",lastWeek:"dddd[и] [ҳафтаи гузашта соати] LT",sameElse:"L"},relativeTime:{future:"баъди %s",past:"%s пеш",s:"якчанд сония",m:"як дақиқа",mm:"%d дақиқа",h:"як соат",hh:"%d соат",d:"як рӯз",dd:"%d рӯз",M:"як моҳ",MM:"%d моҳ",y:"як сол",yy:"%d сол"},meridiemParse:/шаб|субҳ|рӯз|бегоҳ/,meridiemHour:function(e,t){return 12===e&&(e=0),"шаб"===t?e<4?e:e+12:"субҳ"===t?e:"рӯз"===t?e>=11?e:e+12:"бегоҳ"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"шаб":e<11?"субҳ":e<16?"рӯз":e<19?"бегоҳ":"шаб"},dayOfMonthOrdinalParse:/\d{1,2}-(ум|юм)/,ordinal:function(e){var a=e%10,n=e>=100?100:null;return e+(t[e]||t[a]||t[n])},week:{dow:1,doy:7}});return a}))},"3c0d":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={format:"leden_únor_březen_duben_květen_červen_červenec_srpen_září_říjen_listopad_prosinec".split("_"),standalone:"ledna_února_března_dubna_května_června_července_srpna_září_října_listopadu_prosince".split("_")},a="led_úno_bře_dub_kvě_čvn_čvc_srp_zář_říj_lis_pro".split("_"),n=[/^led/i,/^úno/i,/^bře/i,/^dub/i,/^kvě/i,/^(čvn|červen$|června)/i,/^(čvc|červenec|července)/i,/^srp/i,/^zář/i,/^říj/i,/^lis/i,/^pro/i],r=/^(leden|únor|březen|duben|květen|červenec|července|červen|června|srpen|září|říjen|listopad|prosinec|led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i;function i(e){return e>1&&e<5&&1!==~~(e/10)}function o(e,t,a,n){var r=e+" ";switch(a){case"s":return t||n?"pár sekund":"pár sekundami";case"ss":return t||n?r+(i(e)?"sekundy":"sekund"):r+"sekundami";case"m":return t?"minuta":n?"minutu":"minutou";case"mm":return t||n?r+(i(e)?"minuty":"minut"):r+"minutami";case"h":return t?"hodina":n?"hodinu":"hodinou";case"hh":return t||n?r+(i(e)?"hodiny":"hodin"):r+"hodinami";case"d":return t||n?"den":"dnem";case"dd":return t||n?r+(i(e)?"dny":"dní"):r+"dny";case"M":return t||n?"měsíc":"měsícem";case"MM":return t||n?r+(i(e)?"měsíce":"měsíců"):r+"měsíci";case"y":return t||n?"rok":"rokem";case"yy":return t||n?r+(i(e)?"roky":"let"):r+"lety"}}var s=e.defineLocale("cs",{months:t,monthsShort:a,monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(leden|ledna|února|únor|březen|března|duben|dubna|květen|května|červenec|července|červen|června|srpen|srpna|září|říjen|října|listopadu|listopad|prosinec|prosince)/i,monthsShortStrictRegex:/^(led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"neděle_pondělí_úterý_středa_čtvrtek_pátek_sobota".split("_"),weekdaysShort:"ne_po_út_st_čt_pá_so".split("_"),weekdaysMin:"ne_po_út_st_čt_pá_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm",l:"D. M. YYYY"},calendar:{sameDay:"[dnes v] LT",nextDay:"[zítra v] LT",nextWeek:function(){switch(this.day()){case 0:return"[v neděli v] LT";case 1:case 2:return"[v] dddd [v] LT";case 3:return"[ve středu v] LT";case 4:return"[ve čtvrtek v] LT";case 5:return"[v pátek v] LT";case 6:return"[v sobotu v] LT"}},lastDay:"[včera v] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulou neděli v] LT";case 1:case 2:return"[minulé] dddd [v] LT";case 3:return"[minulou středu v] LT";case 4:case 5:return"[minulý] dddd [v] LT";case 6:return"[minulou sobotu v] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"před %s",s:o,ss:o,m:o,mm:o,h:o,hh:o,d:o,dd:o,M:o,MM:o,y:o,yy:o},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return s}))},"3de5":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"௧",2:"௨",3:"௩",4:"௪",5:"௫",6:"௬",7:"௭",8:"௮",9:"௯",0:"௦"},a={"௧":"1","௨":"2","௩":"3","௪":"4","௫":"5","௬":"6","௭":"7","௮":"8","௯":"9","௦":"0"},n=e.defineLocale("ta",{months:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),monthsShort:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),weekdays:"ஞாயிற்றுக்கிழமை_திங்கட்கிழமை_செவ்வாய்கிழமை_புதன்கிழமை_வியாழக்கிழமை_வெள்ளிக்கிழமை_சனிக்கிழமை".split("_"),weekdaysShort:"ஞாயிறு_திங்கள்_செவ்வாய்_புதன்_வியாழன்_வெள்ளி_சனி".split("_"),weekdaysMin:"ஞா_தி_செ_பு_வி_வெ_ச".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, HH:mm",LLLL:"dddd, D MMMM YYYY, HH:mm"},calendar:{sameDay:"[இன்று] LT",nextDay:"[நாளை] LT",nextWeek:"dddd, LT",lastDay:"[நேற்று] LT",lastWeek:"[கடந்த வாரம்] dddd, LT",sameElse:"L"},relativeTime:{future:"%s இல்",past:"%s முன்",s:"ஒரு சில விநாடிகள்",ss:"%d விநாடிகள்",m:"ஒரு நிமிடம்",mm:"%d நிமிடங்கள்",h:"ஒரு மணி நேரம்",hh:"%d மணி நேரம்",d:"ஒரு நாள்",dd:"%d நாட்கள்",M:"ஒரு மாதம்",MM:"%d மாதங்கள்",y:"ஒரு வருடம்",yy:"%d ஆண்டுகள்"},dayOfMonthOrdinalParse:/\d{1,2}வது/,ordinal:function(e){return e+"வது"},preparse:function(e){return e.replace(/[௧௨௩௪௫௬௭௮௯௦]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/யாமம்|வைகறை|காலை|நண்பகல்|எற்பாடு|மாலை/,meridiem:function(e,t,a){return e<2?" யாமம்":e<6?" வைகறை":e<10?" காலை":e<14?" நண்பகல்":e<18?" எற்பாடு":e<22?" மாலை":" யாமம்"},meridiemHour:function(e,t){return 12===e&&(e=0),"யாமம்"===t?e<2?e:e+12:"வைகறை"===t||"காலை"===t||"நண்பகல்"===t&&e>=10?e:e+12},week:{dow:0,doy:6}});return n}))},"3e92":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"೧",2:"೨",3:"೩",4:"೪",5:"೫",6:"೬",7:"೭",8:"೮",9:"೯",0:"೦"},a={"೧":"1","೨":"2","೩":"3","೪":"4","೫":"5","೬":"6","೭":"7","೮":"8","೯":"9","೦":"0"},n=e.defineLocale("kn",{months:"ಜನವರಿ_ಫೆಬ್ರವರಿ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂಬರ್_ಅಕ್ಟೋಬರ್_ನವೆಂಬರ್_ಡಿಸೆಂಬರ್".split("_"),monthsShort:"ಜನ_ಫೆಬ್ರ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂ_ಅಕ್ಟೋ_ನವೆಂ_ಡಿಸೆಂ".split("_"),monthsParseExact:!0,weekdays:"ಭಾನುವಾರ_ಸೋಮವಾರ_ಮಂಗಳವಾರ_ಬುಧವಾರ_ಗುರುವಾರ_ಶುಕ್ರವಾರ_ಶನಿವಾರ".split("_"),weekdaysShort:"ಭಾನು_ಸೋಮ_ಮಂಗಳ_ಬುಧ_ಗುರು_ಶುಕ್ರ_ಶನಿ".split("_"),weekdaysMin:"ಭಾ_ಸೋ_ಮಂ_ಬು_ಗು_ಶು_ಶ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[ಇಂದು] LT",nextDay:"[ನಾಳೆ] LT",nextWeek:"dddd, LT",lastDay:"[ನಿನ್ನೆ] LT",lastWeek:"[ಕೊನೆಯ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ನಂತರ",past:"%s ಹಿಂದೆ",s:"ಕೆಲವು ಕ್ಷಣಗಳು",ss:"%d ಸೆಕೆಂಡುಗಳು",m:"ಒಂದು ನಿಮಿಷ",mm:"%d ನಿಮಿಷ",h:"ಒಂದು ಗಂಟೆ",hh:"%d ಗಂಟೆ",d:"ಒಂದು ದಿನ",dd:"%d ದಿನ",M:"ಒಂದು ತಿಂಗಳು",MM:"%d ತಿಂಗಳು",y:"ಒಂದು ವರ್ಷ",yy:"%d ವರ್ಷ"},preparse:function(e){return e.replace(/[೧೨೩೪೫೬೭೮೯೦]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/ರಾತ್ರಿ|ಬೆಳಿಗ್ಗೆ|ಮಧ್ಯಾಹ್ನ|ಸಂಜೆ/,meridiemHour:function(e,t){return 12===e&&(e=0),"ರಾತ್ರಿ"===t?e<4?e:e+12:"ಬೆಳಿಗ್ಗೆ"===t?e:"ಮಧ್ಯಾಹ್ನ"===t?e>=10?e:e+12:"ಸಂಜೆ"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"ರಾತ್ರಿ":e<10?"ಬೆಳಿಗ್ಗೆ":e<17?"ಮಧ್ಯಾಹ್ನ":e<20?"ಸಂಜೆ":"ರಾತ್ರಿ"},dayOfMonthOrdinalParse:/\d{1,2}(ನೇ)/,ordinal:function(e){return e+"ನೇ"},week:{dow:0,doy:6}});return n}))},"40d5":function(e,t,a){var n=a("d039");e.exports=!n((function(){var e=function(){}.bind();return"function"!=typeof e||e.hasOwnProperty("prototype")}))},"423e":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ar-kw",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإتنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اتنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:0,doy:12}});return t}))},4362:function(e,t,a){t.nextTick=function(e){var t=Array.prototype.slice.call(arguments);t.shift(),setTimeout((function(){e.apply(null,t)}),0)},t.platform=t.arch=t.execPath=t.title="browser",t.pid=1,t.browser=!0,t.env={},t.argv=[],t.binding=function(e){throw new Error("No such module. (Possibly not yet loaded)")},function(){var e,n="/";t.cwd=function(){return n},t.chdir=function(t){e||(e=a("df7c")),n=e.resolve(t,n)}}(),t.exit=t.kill=t.umask=t.dlopen=t.uptime=t.memoryUsage=t.uvCounters=function(){},t.features={}},"440c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r={m:["eng Minutt","enger Minutt"],h:["eng Stonn","enger Stonn"],d:["een Dag","engem Dag"],M:["ee Mount","engem Mount"],y:["ee Joer","engem Joer"]};return t?r[a][0]:r[a][1]}function a(e){var t=e.substr(0,e.indexOf(" "));return r(t)?"a "+e:"an "+e}function n(e){var t=e.substr(0,e.indexOf(" "));return r(t)?"viru "+e:"virun "+e}function r(e){if(e=parseInt(e,10),isNaN(e))return!1;if(e<0)return!0;if(e<10)return 4<=e&&e<=7;if(e<100){var t=e%10,a=e/10;return r(0===t?a:t)}if(e<1e4){while(e>=10)e/=10;return r(e)}return e/=1e3,r(e)}var i=e.defineLocale("lb",{months:"Januar_Februar_Mäerz_Abrëll_Mee_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Febr._Mrz._Abr._Mee_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonndeg_Méindeg_Dënschdeg_Mëttwoch_Donneschdeg_Freideg_Samschdeg".split("_"),weekdaysShort:"So._Mé._Dë._Më._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mé_Dë_Më_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm [Auer]",LTS:"H:mm:ss [Auer]",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm [Auer]",LLLL:"dddd, D. MMMM YYYY H:mm [Auer]"},calendar:{sameDay:"[Haut um] LT",sameElse:"L",nextDay:"[Muer um] LT",nextWeek:"dddd [um] LT",lastDay:"[Gëschter um] LT",lastWeek:function(){switch(this.day()){case 2:case 4:return"[Leschten] dddd [um] LT";default:return"[Leschte] dddd [um] LT"}}},relativeTime:{future:a,past:n,s:"e puer Sekonnen",ss:"%d Sekonnen",m:t,mm:"%d Minutten",h:t,hh:"%d Stonnen",d:t,dd:"%d Deeg",M:t,MM:"%d Méint",y:t,yy:"%d Joer"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return i}))},"44ad":function(e,t,a){var n=a("e330"),r=a("d039"),i=a("c6b6"),o=Object,s=n("".split);e.exports=r((function(){return!o("z").propertyIsEnumerable(0)}))?function(e){return"String"==i(e)?s(e,""):o(e)}:o},4678:function(e,t,a){var n={"./af":"2bfb","./af.js":"2bfb","./ar":"8e73","./ar-dz":"a356","./ar-dz.js":"a356","./ar-kw":"423e","./ar-kw.js":"423e","./ar-ly":"1cfd","./ar-ly.js":"1cfd","./ar-ma":"0a84","./ar-ma.js":"0a84","./ar-sa":"8230","./ar-sa.js":"8230","./ar-tn":"6d83","./ar-tn.js":"6d83","./ar.js":"8e73","./az":"485c","./az.js":"485c","./be":"1fc1","./be.js":"1fc1","./bg":"84aa","./bg.js":"84aa","./bm":"a7fa","./bm.js":"a7fa","./bn":"9043","./bn-bd":"9686","./bn-bd.js":"9686","./bn.js":"9043","./bo":"d26a","./bo.js":"d26a","./br":"6887","./br.js":"6887","./bs":"2554","./bs.js":"2554","./ca":"d716","./ca.js":"d716","./cs":"3c0d","./cs.js":"3c0d","./cv":"03ec","./cv.js":"03ec","./cy":"9797","./cy.js":"9797","./da":"0f14","./da.js":"0f14","./de":"b469","./de-at":"b3eb","./de-at.js":"b3eb","./de-ch":"bb71","./de-ch.js":"bb71","./de.js":"b469","./dv":"598a","./dv.js":"598a","./el":"8d47","./el.js":"8d47","./en-au":"0e6b","./en-au.js":"0e6b","./en-ca":"3886","./en-ca.js":"3886","./en-gb":"39a6","./en-gb.js":"39a6","./en-ie":"e1d3","./en-ie.js":"e1d3","./en-il":"7333","./en-il.js":"7333","./en-in":"ec2e","./en-in.js":"ec2e","./en-nz":"6f50","./en-nz.js":"6f50","./en-sg":"b7e9","./en-sg.js":"b7e9","./eo":"65db","./eo.js":"65db","./es":"898b","./es-do":"0a3c","./es-do.js":"0a3c","./es-mx":"b5b7","./es-mx.js":"b5b7","./es-us":"55c9","./es-us.js":"55c9","./es.js":"898b","./et":"ec18","./et.js":"ec18","./eu":"0ff2","./eu.js":"0ff2","./fa":"8df4","./fa.js":"8df4","./fi":"81e9","./fi.js":"81e9","./fil":"d69a","./fil.js":"d69a","./fo":"0721","./fo.js":"0721","./fr":"9f26","./fr-ca":"d9f8","./fr-ca.js":"d9f8","./fr-ch":"0e49","./fr-ch.js":"0e49","./fr.js":"9f26","./fy":"7118","./fy.js":"7118","./ga":"5120","./ga.js":"5120","./gd":"f6b4","./gd.js":"f6b4","./gl":"8840","./gl.js":"8840","./gom-deva":"aaf2","./gom-deva.js":"aaf2","./gom-latn":"0caa","./gom-latn.js":"0caa","./gu":"e0c5","./gu.js":"e0c5","./he":"c7aa","./he.js":"c7aa","./hi":"dc4d","./hi.js":"dc4d","./hr":"4ba9","./hr.js":"4ba9","./hu":"5b14","./hu.js":"5b14","./hy-am":"d6b6","./hy-am.js":"d6b6","./id":"5038","./id.js":"5038","./is":"0558","./is.js":"0558","./it":"6e98","./it-ch":"6f12","./it-ch.js":"6f12","./it.js":"6e98","./ja":"079e","./ja.js":"079e","./jv":"b540","./jv.js":"b540","./ka":"201b","./ka.js":"201b","./kk":"6d79","./kk.js":"6d79","./km":"e81d","./km.js":"e81d","./kn":"3e92","./kn.js":"3e92","./ko":"22f8","./ko.js":"22f8","./ku":"2421","./ku.js":"2421","./ky":"9609","./ky.js":"9609","./lb":"440c","./lb.js":"440c","./lo":"b29d","./lo.js":"b29d","./lt":"26f9","./lt.js":"26f9","./lv":"b97c","./lv.js":"b97c","./me":"293c","./me.js":"293c","./mi":"688b","./mi.js":"688b","./mk":"6909","./mk.js":"6909","./ml":"02fb","./ml.js":"02fb","./mn":"958b","./mn.js":"958b","./mr":"39bd","./mr.js":"39bd","./ms":"ebe4","./ms-my":"6403","./ms-my.js":"6403","./ms.js":"ebe4","./mt":"1b45","./mt.js":"1b45","./my":"8689","./my.js":"8689","./nb":"6ce3","./nb.js":"6ce3","./ne":"3a39","./ne.js":"3a39","./nl":"facd","./nl-be":"db29","./nl-be.js":"db29","./nl.js":"facd","./nn":"b84c","./nn.js":"b84c","./oc-lnc":"167b","./oc-lnc.js":"167b","./pa-in":"f3ff","./pa-in.js":"f3ff","./pl":"8d57","./pl.js":"8d57","./pt":"f260","./pt-br":"d2d4","./pt-br.js":"d2d4","./pt.js":"f260","./ro":"972c","./ro.js":"972c","./ru":"957c","./ru.js":"957c","./sd":"6784","./sd.js":"6784","./se":"ffff","./se.js":"ffff","./si":"eda5","./si.js":"eda5","./sk":"7be6","./sk.js":"7be6","./sl":"8155","./sl.js":"8155","./sq":"c8f3","./sq.js":"c8f3","./sr":"cf1e","./sr-cyrl":"13e9","./sr-cyrl.js":"13e9","./sr.js":"cf1e","./ss":"52bd","./ss.js":"52bd","./sv":"5fbd","./sv.js":"5fbd","./sw":"74dc","./sw.js":"74dc","./ta":"3de5","./ta.js":"3de5","./te":"5cbb","./te.js":"5cbb","./tet":"576c","./tet.js":"576c","./tg":"3b1b","./tg.js":"3b1b","./th":"10e8","./th.js":"10e8","./tk":"5aff","./tk.js":"5aff","./tl-ph":"0f38","./tl-ph.js":"0f38","./tlh":"cf75","./tlh.js":"cf75","./tr":"0e81","./tr.js":"0e81","./tzl":"cf51","./tzl.js":"cf51","./tzm":"c109","./tzm-latn":"b53d","./tzm-latn.js":"b53d","./tzm.js":"c109","./ug-cn":"6117","./ug-cn.js":"6117","./uk":"ada2","./uk.js":"ada2","./ur":"5294","./ur.js":"5294","./uz":"2e8c","./uz-latn":"010e","./uz-latn.js":"010e","./uz.js":"2e8c","./vi":"2921","./vi.js":"2921","./x-pseudo":"fd7e","./x-pseudo.js":"fd7e","./yo":"7f33","./yo.js":"7f33","./zh-cn":"5c3a","./zh-cn.js":"5c3a","./zh-hk":"49ab","./zh-hk.js":"49ab","./zh-mo":"3a6c","./zh-mo.js":"3a6c","./zh-tw":"90ea","./zh-tw.js":"90ea"};function r(e){var t=i(e);return a(t)}function i(e){if(!a.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}r.keys=function(){return Object.keys(n)},r.resolve=i,e.exports=r,r.id="4678"},"467f":function(e,t,a){"use strict";var n=a("2d83");e.exports=function(e,t,a){var r=a.config.validateStatus;a.status&&r&&!r(a.status)?t(n("Request failed with status code "+a.status,a.config,null,a.request,a)):e(a)}},"485a":function(e,t,a){var n=a("c65b"),r=a("1626"),i=a("861d"),o=TypeError;e.exports=function(e,t){var a,s;if("string"===t&&r(a=e.toString)&&!i(s=n(a,e)))return s;if(r(a=e.valueOf)&&!i(s=n(a,e)))return s;if("string"!==t&&r(a=e.toString)&&!i(s=n(a,e)))return s;throw o("Can't convert object to primitive value")}},"485c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"-inci",5:"-inci",8:"-inci",70:"-inci",80:"-inci",2:"-nci",7:"-nci",20:"-nci",50:"-nci",3:"-üncü",4:"-üncü",100:"-üncü",6:"-ncı",9:"-uncu",10:"-uncu",30:"-uncu",60:"-ıncı",90:"-ıncı"},a=e.defineLocale("az",{months:"yanvar_fevral_mart_aprel_may_iyun_iyul_avqust_sentyabr_oktyabr_noyabr_dekabr".split("_"),monthsShort:"yan_fev_mar_apr_may_iyn_iyl_avq_sen_okt_noy_dek".split("_"),weekdays:"Bazar_Bazar ertəsi_Çərşənbə axşamı_Çərşənbə_Cümə axşamı_Cümə_Şənbə".split("_"),weekdaysShort:"Baz_BzE_ÇAx_Çər_CAx_Cüm_Şən".split("_"),weekdaysMin:"Bz_BE_ÇA_Çə_CA_Cü_Şə".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[sabah saat] LT",nextWeek:"[gələn həftə] dddd [saat] LT",lastDay:"[dünən] LT",lastWeek:"[keçən həftə] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s əvvəl",s:"bir neçə saniyə",ss:"%d saniyə",m:"bir dəqiqə",mm:"%d dəqiqə",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",M:"bir ay",MM:"%d ay",y:"bir il",yy:"%d il"},meridiemParse:/gecə|səhər|gündüz|axşam/,isPM:function(e){return/^(gündüz|axşam)$/.test(e)},meridiem:function(e,t,a){return e<4?"gecə":e<12?"səhər":e<17?"gündüz":"axşam"},dayOfMonthOrdinalParse:/\d{1,2}-(ıncı|inci|nci|üncü|ncı|uncu)/,ordinal:function(e){if(0===e)return e+"-ıncı";var a=e%10,n=e%100-a,r=e>=100?100:null;return e+(t[a]||t[n]||t[r])},week:{dow:1,doy:7}});return a}))},"49ab":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-hk",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,a){var n=100*e+t;return n<600?"凌晨":n<900?"早上":n<1200?"上午":1200===n?"中午":n<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:"[下]ddddLT",lastDay:"[昨天]LT",lastWeek:"[上]ddddLT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}});return t}))},"4a0c":function(e){e.exports=JSON.parse('{"name":"axios","version":"0.21.4","description":"Promise based HTTP client for the browser and node.js","main":"index.js","scripts":{"test":"grunt test","start":"node ./sandbox/server.js","build":"NODE_ENV=production grunt build","preversion":"npm test","version":"npm run build && grunt version && git add -A dist && git add CHANGELOG.md bower.json package.json","postversion":"git push && git push --tags","examples":"node ./examples/server.js","coveralls":"cat coverage/lcov.info | ./node_modules/coveralls/bin/coveralls.js","fix":"eslint --fix lib/**/*.js"},"repository":{"type":"git","url":"https://github.com/axios/axios.git"},"keywords":["xhr","http","ajax","promise","node"],"author":"Matt Zabriskie","license":"MIT","bugs":{"url":"https://github.com/axios/axios/issues"},"homepage":"https://axios-http.com","devDependencies":{"coveralls":"^3.0.0","es6-promise":"^4.2.4","grunt":"^1.3.0","grunt-banner":"^0.6.0","grunt-cli":"^1.2.0","grunt-contrib-clean":"^1.1.0","grunt-contrib-watch":"^1.0.0","grunt-eslint":"^23.0.0","grunt-karma":"^4.0.0","grunt-mocha-test":"^0.13.3","grunt-ts":"^6.0.0-beta.19","grunt-webpack":"^4.0.2","istanbul-instrumenter-loader":"^1.0.0","jasmine-core":"^2.4.1","karma":"^6.3.2","karma-chrome-launcher":"^3.1.0","karma-firefox-launcher":"^2.1.0","karma-jasmine":"^1.1.1","karma-jasmine-ajax":"^0.1.13","karma-safari-launcher":"^1.0.0","karma-sauce-launcher":"^4.3.6","karma-sinon":"^1.0.5","karma-sourcemap-loader":"^0.3.8","karma-webpack":"^4.0.2","load-grunt-tasks":"^3.5.2","minimist":"^1.2.0","mocha":"^8.2.1","sinon":"^4.5.0","terser-webpack-plugin":"^4.2.3","typescript":"^4.0.5","url-search-params":"^0.10.0","webpack":"^4.44.2","webpack-dev-server":"^3.11.0"},"browser":{"./lib/adapters/http.js":"./lib/adapters/xhr.js"},"jsdelivr":"dist/axios.min.js","unpkg":"dist/axios.min.js","typings":"./index.d.ts","dependencies":{"follow-redirects":"^1.14.0"},"bundlesize":[{"path":"./dist/axios.min.js","threshold":"5kB"}]}')},"4a7b":function(e,t,a){"use strict";var n=a("c532");e.exports=function(e,t){t=t||{};var a={},r=["url","method","data"],i=["headers","auth","proxy","params"],o=["baseURL","transformRequest","transformResponse","paramsSerializer","timeout","timeoutMessage","withCredentials","adapter","responseType","xsrfCookieName","xsrfHeaderName","onUploadProgress","onDownloadProgress","decompress","maxContentLength","maxBodyLength","maxRedirects","transport","httpAgent","httpsAgent","cancelToken","socketPath","responseEncoding"],s=["validateStatus"];function l(e,t){return n.isPlainObject(e)&&n.isPlainObject(t)?n.merge(e,t):n.isPlainObject(t)?n.merge({},t):n.isArray(t)?t.slice():t}function c(r){n.isUndefined(t[r])?n.isUndefined(e[r])||(a[r]=l(void 0,e[r])):a[r]=l(e[r],t[r])}n.forEach(r,(function(e){n.isUndefined(t[e])||(a[e]=l(void 0,t[e]))})),n.forEach(i,c),n.forEach(o,(function(r){n.isUndefined(t[r])?n.isUndefined(e[r])||(a[r]=l(void 0,e[r])):a[r]=l(void 0,t[r])})),n.forEach(s,(function(n){n in t?a[n]=l(e[n],t[n]):n in e&&(a[n]=l(void 0,e[n]))}));var u=r.concat(i).concat(o).concat(s),d=Object.keys(e).concat(Object.keys(t)).filter((function(e){return-1===u.indexOf(e)}));return n.forEach(d,c),a}},"4ba9":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a){var n=e+" ";switch(a){case"ss":return n+=1===e?"sekunda":2===e||3===e||4===e?"sekunde":"sekundi",n;case"m":return t?"jedna minuta":"jedne minute";case"mm":return n+=1===e?"minuta":2===e||3===e||4===e?"minute":"minuta",n;case"h":return t?"jedan sat":"jednog sata";case"hh":return n+=1===e?"sat":2===e||3===e||4===e?"sata":"sati",n;case"dd":return n+=1===e?"dan":"dana",n;case"MM":return n+=1===e?"mjesec":2===e||3===e||4===e?"mjeseca":"mjeseci",n;case"yy":return n+=1===e?"godina":2===e||3===e||4===e?"godine":"godina",n}}var a=e.defineLocale("hr",{months:{format:"siječnja_veljače_ožujka_travnja_svibnja_lipnja_srpnja_kolovoza_rujna_listopada_studenoga_prosinca".split("_"),standalone:"siječanj_veljača_ožujak_travanj_svibanj_lipanj_srpanj_kolovoz_rujan_listopad_studeni_prosinac".split("_")},monthsShort:"sij._velj._ožu._tra._svi._lip._srp._kol._ruj._lis._stu._pro.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"Do MMMM YYYY",LLL:"Do MMMM YYYY H:mm",LLLL:"dddd, Do MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:return"[prošlu] [nedjelju] [u] LT";case 3:return"[prošlu] [srijedu] [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:t,m:t,mm:t,h:t,hh:t,d:"dan",dd:t,M:"mjesec",MM:t,y:"godinu",yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return a}))},"4d64":function(e,t,a){var n=a("fc6a"),r=a("23cb"),i=a("07fa"),o=function(e){return function(t,a,o){var s,l=n(t),c=i(l),u=r(o,c);if(e&&a!=a){while(c>u)if(s=l[u++],s!=s)return!0}else for(;c>u;u++)if((e||u in l)&&l[u]===a)return e||u||0;return!e&&-1}};e.exports={includes:o(!0),indexOf:o(!1)}},5038:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("id",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Agt_Sep_Okt_Nov_Des".split("_"),weekdays:"Minggu_Senin_Selasa_Rabu_Kamis_Jumat_Sabtu".split("_"),weekdaysShort:"Min_Sen_Sel_Rab_Kam_Jum_Sab".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|siang|sore|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"siang"===t?e>=11?e:e+12:"sore"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,a){return e<11?"pagi":e<15?"siang":e<19?"sore":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Besok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kemarin pukul] LT",lastWeek:"dddd [lalu pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lalu",s:"beberapa detik",ss:"%d detik",m:"semenit",mm:"%d menit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:0,doy:6}});return t}))},"50c4":function(e,t,a){var n=a("5926"),r=Math.min;e.exports=function(e){return e>0?r(n(e),9007199254740991):0}},5120:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["Eanáir","Feabhra","Márta","Aibreán","Bealtaine","Meitheamh","Iúil","Lúnasa","Meán Fómhair","Deireadh Fómhair","Samhain","Nollaig"],a=["Ean","Feabh","Márt","Aib","Beal","Meith","Iúil","Lún","M.F.","D.F.","Samh","Noll"],n=["Dé Domhnaigh","Dé Luain","Dé Máirt","Dé Céadaoin","Déardaoin","Dé hAoine","Dé Sathairn"],r=["Domh","Luan","Máirt","Céad","Déar","Aoine","Sath"],i=["Do","Lu","Má","Cé","Dé","A","Sa"],o=e.defineLocale("ga",{months:t,monthsShort:a,monthsParseExact:!0,weekdays:n,weekdaysShort:r,weekdaysMin:i,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Inniu ag] LT",nextDay:"[Amárach ag] LT",nextWeek:"dddd [ag] LT",lastDay:"[Inné ag] LT",lastWeek:"dddd [seo caite] [ag] LT",sameElse:"L"},relativeTime:{future:"i %s",past:"%s ó shin",s:"cúpla soicind",ss:"%d soicind",m:"nóiméad",mm:"%d nóiméad",h:"uair an chloig",hh:"%d uair an chloig",d:"lá",dd:"%d lá",M:"mí",MM:"%d míonna",y:"bliain",yy:"%d bliain"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(e){var t=1===e?"d":e%10===2?"na":"mh";return e+t},week:{dow:1,doy:4}});return o}))},5270:function(e,t,a){"use strict";var n=a("c532"),r=a("c401"),i=a("2e67"),o=a("2444");function s(e){e.cancelToken&&e.cancelToken.throwIfRequested()}e.exports=function(e){s(e),e.headers=e.headers||{},e.data=r.call(e,e.data,e.headers,e.transformRequest),e.headers=n.merge(e.headers.common||{},e.headers[e.method]||{},e.headers),n.forEach(["delete","get","head","post","put","patch","common"],(function(t){delete e.headers[t]}));var t=e.adapter||o.adapter;return t(e).then((function(t){return s(e),t.data=r.call(e,t.data,t.headers,e.transformResponse),t}),(function(t){return i(t)||(s(e),t&&t.response&&(t.response.data=r.call(e,t.response.data,t.response.headers,e.transformResponse))),Promise.reject(t)}))}},5294:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["جنوری","فروری","مارچ","اپریل","مئی","جون","جولائی","اگست","ستمبر","اکتوبر","نومبر","دسمبر"],a=["اتوار","پیر","منگل","بدھ","جمعرات","جمعہ","ہفتہ"],n=e.defineLocale("ur",{months:t,monthsShort:t,weekdays:a,weekdaysShort:a,weekdaysMin:a,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(e){return"شام"===e},meridiem:function(e,t,a){return e<12?"صبح":"شام"},calendar:{sameDay:"[آج بوقت] LT",nextDay:"[کل بوقت] LT",nextWeek:"dddd [بوقت] LT",lastDay:"[گذشتہ روز بوقت] LT",lastWeek:"[گذشتہ] dddd [بوقت] LT",sameElse:"L"},relativeTime:{future:"%s بعد",past:"%s قبل",s:"چند سیکنڈ",ss:"%d سیکنڈ",m:"ایک منٹ",mm:"%d منٹ",h:"ایک گھنٹہ",hh:"%d گھنٹے",d:"ایک دن",dd:"%d دن",M:"ایک ماہ",MM:"%d ماہ",y:"ایک سال",yy:"%d سال"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:4}});return n}))},"52bd":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ss",{months:"Bhimbidvwane_Indlovana_Indlov'lenkhulu_Mabasa_Inkhwekhweti_Inhlaba_Kholwane_Ingci_Inyoni_Imphala_Lweti_Ingongoni".split("_"),monthsShort:"Bhi_Ina_Inu_Mab_Ink_Inh_Kho_Igc_Iny_Imp_Lwe_Igo".split("_"),weekdays:"Lisontfo_Umsombuluko_Lesibili_Lesitsatfu_Lesine_Lesihlanu_Umgcibelo".split("_"),weekdaysShort:"Lis_Umb_Lsb_Les_Lsi_Lsh_Umg".split("_"),weekdaysMin:"Li_Us_Lb_Lt_Ls_Lh_Ug".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Namuhla nga] LT",nextDay:"[Kusasa nga] LT",nextWeek:"dddd [nga] LT",lastDay:"[Itolo nga] LT",lastWeek:"dddd [leliphelile] [nga] LT",sameElse:"L"},relativeTime:{future:"nga %s",past:"wenteka nga %s",s:"emizuzwana lomcane",ss:"%d mzuzwana",m:"umzuzu",mm:"%d emizuzu",h:"lihora",hh:"%d emahora",d:"lilanga",dd:"%d emalanga",M:"inyanga",MM:"%d tinyanga",y:"umnyaka",yy:"%d iminyaka"},meridiemParse:/ekuseni|emini|entsambama|ebusuku/,meridiem:function(e,t,a){return e<11?"ekuseni":e<15?"emini":e<19?"entsambama":"ebusuku"},meridiemHour:function(e,t){return 12===e&&(e=0),"ekuseni"===t?e:"emini"===t?e>=11?e:e+12:"entsambama"===t||"ebusuku"===t?0===e?0:e+12:void 0},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:"%d",week:{dow:1,doy:4}});return t}))},"55c9":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),a="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),n=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],r=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,i=e.defineLocale("es-us",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,n){return e?/-MMM-/.test(n)?a[e.month()]:t[e.month()]:t},monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"MM/DD/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:0,doy:6}});return i}))},5692:function(e,t,a){var n=a("c430"),r=a("c6cd");(e.exports=function(e,t){return r[e]||(r[e]=void 0!==t?t:{})})("versions",[]).push({version:"3.25.5",mode:n?"pure":"global",copyright:"© 2014-2022 Denis Pushkarev (zloirock.ru)",license:"https://github.com/zloirock/core-js/blob/v3.25.5/LICENSE",source:"https://github.com/zloirock/core-js"})},"56ef":function(e,t,a){var n=a("d066"),r=a("e330"),i=a("241c"),o=a("7418"),s=a("825a"),l=r([].concat);e.exports=n("Reflect","ownKeys")||function(e){var t=i.f(s(e)),a=o.f;return a?l(t,a(e)):t}},"576c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tet",{months:"Janeiru_Fevereiru_Marsu_Abril_Maiu_Juñu_Jullu_Agustu_Setembru_Outubru_Novembru_Dezembru".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingu_Segunda_Tersa_Kuarta_Kinta_Sesta_Sabadu".split("_"),weekdaysShort:"Dom_Seg_Ters_Kua_Kint_Sest_Sab".split("_"),weekdaysMin:"Do_Seg_Te_Ku_Ki_Ses_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Ohin iha] LT",nextDay:"[Aban iha] LT",nextWeek:"dddd [iha] LT",lastDay:"[Horiseik iha] LT",lastWeek:"dddd [semana kotuk] [iha] LT",sameElse:"L"},relativeTime:{future:"iha %s",past:"%s liuba",s:"segundu balun",ss:"segundu %d",m:"minutu ida",mm:"minutu %d",h:"oras ida",hh:"oras %d",d:"loron ida",dd:"loron %d",M:"fulan ida",MM:"fulan %d",y:"tinan ida",yy:"tinan %d"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:1,doy:4}});return t}))},5926:function(e,t,a){var n=a("b42e");e.exports=function(e){var t=+e;return t!==t||0===t?0:n(t)}},"598a":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["ޖެނުއަރީ","ފެބްރުއަރީ","މާރިޗު","އޭޕްރީލު","މޭ","ޖޫން","ޖުލައި","އޯގަސްޓު","ސެޕްޓެމްބަރު","އޮކްޓޯބަރު","ނޮވެމްބަރު","ޑިސެމްބަރު"],a=["އާދިއްތަ","ހޯމަ","އަންގާރަ","ބުދަ","ބުރާސްފަތި","ހުކުރު","ހޮނިހިރު"],n=e.defineLocale("dv",{months:t,monthsShort:t,weekdays:a,weekdaysShort:a,weekdaysMin:"އާދި_ހޯމަ_އަން_ބުދަ_ބުރާ_ހުކު_ހޮނި".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/M/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/މކ|މފ/,isPM:function(e){return"މފ"===e},meridiem:function(e,t,a){return e<12?"މކ":"މފ"},calendar:{sameDay:"[މިއަދު] LT",nextDay:"[މާދަމާ] LT",nextWeek:"dddd LT",lastDay:"[އިއްޔެ] LT",lastWeek:"[ފާއިތުވި] dddd LT",sameElse:"L"},relativeTime:{future:"ތެރޭގައި %s",past:"ކުރިން %s",s:"ސިކުންތުކޮޅެއް",ss:"d% ސިކުންތު",m:"މިނިޓެއް",mm:"މިނިޓު %d",h:"ގަޑިއިރެއް",hh:"ގަޑިއިރު %d",d:"ދުވަހެއް",dd:"ދުވަސް %d",M:"މަހެއް",MM:"މަސް %d",y:"އަހަރެއް",yy:"އަހަރު %d"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:7,doy:12}});return n}))},"59ed":function(e,t,a){var n=a("1626"),r=a("0d51"),i=TypeError;e.exports=function(e){if(n(e))return e;throw i(r(e)+" is not a function")}},"5aff":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"'inji",5:"'inji",8:"'inji",70:"'inji",80:"'inji",2:"'nji",7:"'nji",20:"'nji",50:"'nji",3:"'ünji",4:"'ünji",100:"'ünji",6:"'njy",9:"'unjy",10:"'unjy",30:"'unjy",60:"'ynjy",90:"'ynjy"},a=e.defineLocale("tk",{months:"Ýanwar_Fewral_Mart_Aprel_Maý_Iýun_Iýul_Awgust_Sentýabr_Oktýabr_Noýabr_Dekabr".split("_"),monthsShort:"Ýan_Few_Mar_Apr_Maý_Iýn_Iýl_Awg_Sen_Okt_Noý_Dek".split("_"),weekdays:"Ýekşenbe_Duşenbe_Sişenbe_Çarşenbe_Penşenbe_Anna_Şenbe".split("_"),weekdaysShort:"Ýek_Duş_Siş_Çar_Pen_Ann_Şen".split("_"),weekdaysMin:"Ýk_Dş_Sş_Çr_Pn_An_Şn".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün sagat] LT",nextDay:"[ertir sagat] LT",nextWeek:"[indiki] dddd [sagat] LT",lastDay:"[düýn] LT",lastWeek:"[geçen] dddd [sagat] LT",sameElse:"L"},relativeTime:{future:"%s soň",past:"%s öň",s:"birnäçe sekunt",m:"bir minut",mm:"%d minut",h:"bir sagat",hh:"%d sagat",d:"bir gün",dd:"%d gün",M:"bir aý",MM:"%d aý",y:"bir ýyl",yy:"%d ýyl"},ordinal:function(e,a){switch(a){case"d":case"D":case"Do":case"DD":return e;default:if(0===e)return e+"'unjy";var n=e%10,r=e%100-n,i=e>=100?100:null;return e+(t[n]||t[r]||t[i])}},week:{dow:1,doy:7}});return a}))},"5b14":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="vasárnap hétfőn kedden szerdán csütörtökön pénteken szombaton".split(" ");function a(e,t,a,n){var r=e;switch(a){case"s":return n||t?"néhány másodperc":"néhány másodperce";case"ss":return r+(n||t)?" másodperc":" másodperce";case"m":return"egy"+(n||t?" perc":" perce");case"mm":return r+(n||t?" perc":" perce");case"h":return"egy"+(n||t?" óra":" órája");case"hh":return r+(n||t?" óra":" órája");case"d":return"egy"+(n||t?" nap":" napja");case"dd":return r+(n||t?" nap":" napja");case"M":return"egy"+(n||t?" hónap":" hónapja");case"MM":return r+(n||t?" hónap":" hónapja");case"y":return"egy"+(n||t?" év":" éve");case"yy":return r+(n||t?" év":" éve")}return""}function n(e){return(e?"":"[múlt] ")+"["+t[this.day()]+"] LT[-kor]"}var r=e.defineLocale("hu",{months:"január_február_március_április_május_június_július_augusztus_szeptember_október_november_december".split("_"),monthsShort:"jan._feb._márc._ápr._máj._jún._júl._aug._szept._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"vasárnap_hétfő_kedd_szerda_csütörtök_péntek_szombat".split("_"),weekdaysShort:"vas_hét_kedd_sze_csüt_pén_szo".split("_"),weekdaysMin:"v_h_k_sze_cs_p_szo".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY. MMMM D.",LLL:"YYYY. MMMM D. H:mm",LLLL:"YYYY. MMMM D., dddd H:mm"},meridiemParse:/de|du/i,isPM:function(e){return"u"===e.charAt(1).toLowerCase()},meridiem:function(e,t,a){return e<12?!0===a?"de":"DE":!0===a?"du":"DU"},calendar:{sameDay:"[ma] LT[-kor]",nextDay:"[holnap] LT[-kor]",nextWeek:function(){return n.call(this,!0)},lastDay:"[tegnap] LT[-kor]",lastWeek:function(){return n.call(this,!1)},sameElse:"L"},relativeTime:{future:"%s múlva",past:"%s",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return r}))},"5c3a":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-cn",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"周日_周一_周二_周三_周四_周五_周六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日Ah点mm分",LLLL:"YYYY年M月D日ddddAh点mm分",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"下午"===t||"晚上"===t?e+12:e>=11?e:e+12},meridiem:function(e,t,a){var n=100*e+t;return n<600?"凌晨":n<900?"早上":n<1130?"上午":n<1230?"中午":n<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:function(e){return e.week()!==this.week()?"[下]dddLT":"[本]dddLT"},lastDay:"[昨天]LT",lastWeek:function(e){return this.week()!==e.week()?"[上]dddLT":"[本]dddLT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|周)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"周";default:return e}},relativeTime:{future:"%s后",past:"%s前",s:"几秒",ss:"%d 秒",m:"1 分钟",mm:"%d 分钟",h:"1 小时",hh:"%d 小时",d:"1 天",dd:"%d 天",w:"1 周",ww:"%d 周",M:"1 个月",MM:"%d 个月",y:"1 年",yy:"%d 年"},week:{dow:1,doy:4}});return t}))},"5c6c":function(e,t){e.exports=function(e,t){return{enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}}},"5cbb":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("te",{months:"జనవరి_ఫిబ్రవరి_మార్చి_ఏప్రిల్_మే_జూన్_జులై_ఆగస్టు_సెప్టెంబర్_అక్టోబర్_నవంబర్_డిసెంబర్".split("_"),monthsShort:"జన._ఫిబ్ర._మార్చి_ఏప్రి._మే_జూన్_జులై_ఆగ._సెప్._అక్టో._నవ._డిసె.".split("_"),monthsParseExact:!0,weekdays:"ఆదివారం_సోమవారం_మంగళవారం_బుధవారం_గురువారం_శుక్రవారం_శనివారం".split("_"),weekdaysShort:"ఆది_సోమ_మంగళ_బుధ_గురు_శుక్ర_శని".split("_"),weekdaysMin:"ఆ_సో_మం_బు_గు_శు_శ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[నేడు] LT",nextDay:"[రేపు] LT",nextWeek:"dddd, LT",lastDay:"[నిన్న] LT",lastWeek:"[గత] dddd, LT",sameElse:"L"},relativeTime:{future:"%s లో",past:"%s క్రితం",s:"కొన్ని క్షణాలు",ss:"%d సెకన్లు",m:"ఒక నిమిషం",mm:"%d నిమిషాలు",h:"ఒక గంట",hh:"%d గంటలు",d:"ఒక రోజు",dd:"%d రోజులు",M:"ఒక నెల",MM:"%d నెలలు",y:"ఒక సంవత్సరం",yy:"%d సంవత్సరాలు"},dayOfMonthOrdinalParse:/\d{1,2}వ/,ordinal:"%dవ",meridiemParse:/రాత్రి|ఉదయం|మధ్యాహ్నం|సాయంత్రం/,meridiemHour:function(e,t){return 12===e&&(e=0),"రాత్రి"===t?e<4?e:e+12:"ఉదయం"===t?e:"మధ్యాహ్నం"===t?e>=10?e:e+12:"సాయంత్రం"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"రాత్రి":e<10?"ఉదయం":e<17?"మధ్యాహ్నం":e<20?"సాయంత్రం":"రాత్రి"},week:{dow:0,doy:6}});return t}))},"5e77":function(e,t,a){var n=a("83ab"),r=a("1a2d"),i=Function.prototype,o=n&&Object.getOwnPropertyDescriptor,s=r(i,"name"),l=s&&"something"===function(){}.name,c=s&&(!n||n&&o(i,"name").configurable);e.exports={EXISTS:s,PROPER:l,CONFIGURABLE:c}},"5f02":function(e,t,a){"use strict";e.exports=function(e){return"object"===typeof e&&!0===e.isAxiosError}},"5fbd":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("sv",{months:"januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"söndag_måndag_tisdag_onsdag_torsdag_fredag_lördag".split("_"),weekdaysShort:"sön_mån_tis_ons_tor_fre_lör".split("_"),weekdaysMin:"sö_må_ti_on_to_fr_lö".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [kl.] HH:mm",LLLL:"dddd D MMMM YYYY [kl.] HH:mm",lll:"D MMM YYYY HH:mm",llll:"ddd D MMM YYYY HH:mm"},calendar:{sameDay:"[Idag] LT",nextDay:"[Imorgon] LT",lastDay:"[Igår] LT",nextWeek:"[På] dddd LT",lastWeek:"[I] dddd[s] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"för %s sedan",s:"några sekunder",ss:"%d sekunder",m:"en minut",mm:"%d minuter",h:"en timme",hh:"%d timmar",d:"en dag",dd:"%d dagar",M:"en månad",MM:"%d månader",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}(\:e|\:a)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?":e":1===t||2===t?":a":":e";return e+a},week:{dow:1,doy:4}});return t}))},6117:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ug-cn",{months:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),monthsShort:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),weekdays:"يەكشەنبە_دۈشەنبە_سەيشەنبە_چارشەنبە_پەيشەنبە_جۈمە_شەنبە".split("_"),weekdaysShort:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),weekdaysMin:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY-يىلىM-ئاينىڭD-كۈنى",LLL:"YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm",LLLL:"dddd، YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm"},meridiemParse:/يېرىم كېچە|سەھەر|چۈشتىن بۇرۇن|چۈش|چۈشتىن كېيىن|كەچ/,meridiemHour:function(e,t){return 12===e&&(e=0),"يېرىم كېچە"===t||"سەھەر"===t||"چۈشتىن بۇرۇن"===t?e:"چۈشتىن كېيىن"===t||"كەچ"===t?e+12:e>=11?e:e+12},meridiem:function(e,t,a){var n=100*e+t;return n<600?"يېرىم كېچە":n<900?"سەھەر":n<1130?"چۈشتىن بۇرۇن":n<1230?"چۈش":n<1800?"چۈشتىن كېيىن":"كەچ"},calendar:{sameDay:"[بۈگۈن سائەت] LT",nextDay:"[ئەتە سائەت] LT",nextWeek:"[كېلەركى] dddd [سائەت] LT",lastDay:"[تۆنۈگۈن] LT",lastWeek:"[ئالدىنقى] dddd [سائەت] LT",sameElse:"L"},relativeTime:{future:"%s كېيىن",past:"%s بۇرۇن",s:"نەچچە سېكونت",ss:"%d سېكونت",m:"بىر مىنۇت",mm:"%d مىنۇت",h:"بىر سائەت",hh:"%d سائەت",d:"بىر كۈن",dd:"%d كۈن",M:"بىر ئاي",MM:"%d ئاي",y:"بىر يىل",yy:"%d يىل"},dayOfMonthOrdinalParse:/\d{1,2}(-كۈنى|-ئاي|-ھەپتە)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"-كۈنى";case"w":case"W":return e+"-ھەپتە";default:return e}},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:7}});return t}))},"620a":function(e,t,a){"use strict";a("16d2")},"62e4":function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},6374:function(e,t,a){var n=a("da84"),r=Object.defineProperty;e.exports=function(e,t){try{r(n,e,{value:t,configurable:!0,writable:!0})}catch(a){n[e]=t}return t}},6403:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ms-my",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"tengahari"===t?e>=11?e:e+12:"petang"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,a){return e<11?"pagi":e<15?"tengahari":e<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}});return t}))},"65db":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("eo",{months:"januaro_februaro_marto_aprilo_majo_junio_julio_aŭgusto_septembro_oktobro_novembro_decembro".split("_"),monthsShort:"jan_feb_mart_apr_maj_jun_jul_aŭg_sept_okt_nov_dec".split("_"),weekdays:"dimanĉo_lundo_mardo_merkredo_ĵaŭdo_vendredo_sabato".split("_"),weekdaysShort:"dim_lun_mard_merk_ĵaŭ_ven_sab".split("_"),weekdaysMin:"di_lu_ma_me_ĵa_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"[la] D[-an de] MMMM, YYYY",LLL:"[la] D[-an de] MMMM, YYYY HH:mm",LLLL:"dddd[n], [la] D[-an de] MMMM, YYYY HH:mm",llll:"ddd, [la] D[-an de] MMM, YYYY HH:mm"},meridiemParse:/[ap]\.t\.m/i,isPM:function(e){return"p"===e.charAt(0).toLowerCase()},meridiem:function(e,t,a){return e>11?a?"p.t.m.":"P.T.M.":a?"a.t.m.":"A.T.M."},calendar:{sameDay:"[Hodiaŭ je] LT",nextDay:"[Morgaŭ je] LT",nextWeek:"dddd[n je] LT",lastDay:"[Hieraŭ je] LT",lastWeek:"[pasintan] dddd[n je] LT",sameElse:"L"},relativeTime:{future:"post %s",past:"antaŭ %s",s:"kelkaj sekundoj",ss:"%d sekundoj",m:"unu minuto",mm:"%d minutoj",h:"unu horo",hh:"%d horoj",d:"unu tago",dd:"%d tagoj",M:"unu monato",MM:"%d monatoj",y:"unu jaro",yy:"%d jaroj"},dayOfMonthOrdinalParse:/\d{1,2}a/,ordinal:"%da",week:{dow:1,doy:7}});return t}))},6784:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["جنوري","فيبروري","مارچ","اپريل","مئي","جون","جولاءِ","آگسٽ","سيپٽمبر","آڪٽوبر","نومبر","ڊسمبر"],a=["آچر","سومر","اڱارو","اربع","خميس","جمع","ڇنڇر"],n=e.defineLocale("sd",{months:t,monthsShort:t,weekdays:a,weekdaysShort:a,weekdaysMin:a,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(e){return"شام"===e},meridiem:function(e,t,a){return e<12?"صبح":"شام"},calendar:{sameDay:"[اڄ] LT",nextDay:"[سڀاڻي] LT",nextWeek:"dddd [اڳين هفتي تي] LT",lastDay:"[ڪالهه] LT",lastWeek:"[گزريل هفتي] dddd [تي] LT",sameElse:"L"},relativeTime:{future:"%s پوء",past:"%s اڳ",s:"چند سيڪنڊ",ss:"%d سيڪنڊ",m:"هڪ منٽ",mm:"%d منٽ",h:"هڪ ڪلاڪ",hh:"%d ڪلاڪ",d:"هڪ ڏينهن",dd:"%d ڏينهن",M:"هڪ مهينو",MM:"%d مهينا",y:"هڪ سال",yy:"%d سال"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:4}});return n}))},6887:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a){var n={mm:"munutenn",MM:"miz",dd:"devezh"};return e+" "+r(n[a],e)}function a(e){switch(n(e)){case 1:case 3:case 4:case 5:case 9:return e+" bloaz";default:return e+" vloaz"}}function n(e){return e>9?n(e%10):e}function r(e,t){return 2===t?i(e):e}function i(e){var t={m:"v",b:"v",d:"z"};return void 0===t[e.charAt(0)]?e:t[e.charAt(0)]+e.substring(1)}var o=[/^gen/i,/^c[ʼ\']hwe/i,/^meu/i,/^ebr/i,/^mae/i,/^(mez|eve)/i,/^gou/i,/^eos/i,/^gwe/i,/^her/i,/^du/i,/^ker/i],s=/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu|gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,l=/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu)/i,c=/^(gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,u=[/^sul/i,/^lun/i,/^meurzh/i,/^merc[ʼ\']her/i,/^yaou/i,/^gwener/i,/^sadorn/i],d=[/^Sul/i,/^Lun/i,/^Meu/i,/^Mer/i,/^Yao/i,/^Gwe/i,/^Sad/i],h=[/^Su/i,/^Lu/i,/^Me([^r]|$)/i,/^Mer/i,/^Ya/i,/^Gw/i,/^Sa/i],f=e.defineLocale("br",{months:"Genver_Cʼhwevrer_Meurzh_Ebrel_Mae_Mezheven_Gouere_Eost_Gwengolo_Here_Du_Kerzu".split("_"),monthsShort:"Gen_Cʼhwe_Meu_Ebr_Mae_Eve_Gou_Eos_Gwe_Her_Du_Ker".split("_"),weekdays:"Sul_Lun_Meurzh_Mercʼher_Yaou_Gwener_Sadorn".split("_"),weekdaysShort:"Sul_Lun_Meu_Mer_Yao_Gwe_Sad".split("_"),weekdaysMin:"Su_Lu_Me_Mer_Ya_Gw_Sa".split("_"),weekdaysParse:h,fullWeekdaysParse:u,shortWeekdaysParse:d,minWeekdaysParse:h,monthsRegex:s,monthsShortRegex:s,monthsStrictRegex:l,monthsShortStrictRegex:c,monthsParse:o,longMonthsParse:o,shortMonthsParse:o,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [a viz] MMMM YYYY",LLL:"D [a viz] MMMM YYYY HH:mm",LLLL:"dddd, D [a viz] MMMM YYYY HH:mm"},calendar:{sameDay:"[Hiziv da] LT",nextDay:"[Warcʼhoazh da] LT",nextWeek:"dddd [da] LT",lastDay:"[Decʼh da] LT",lastWeek:"dddd [paset da] LT",sameElse:"L"},relativeTime:{future:"a-benn %s",past:"%s ʼzo",s:"un nebeud segondennoù",ss:"%d eilenn",m:"ur vunutenn",mm:t,h:"un eur",hh:"%d eur",d:"un devezh",dd:t,M:"ur miz",MM:t,y:"ur bloaz",yy:a},dayOfMonthOrdinalParse:/\d{1,2}(añ|vet)/,ordinal:function(e){var t=1===e?"añ":"vet";return e+t},week:{dow:1,doy:4},meridiemParse:/a.m.|g.m./,isPM:function(e){return"g.m."===e},meridiem:function(e,t,a){return e<12?"a.m.":"g.m."}});return f}))},"688b":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("mi",{months:"Kohi-tāte_Hui-tanguru_Poutū-te-rangi_Paenga-whāwhā_Haratua_Pipiri_Hōngoingoi_Here-turi-kōkā_Mahuru_Whiringa-ā-nuku_Whiringa-ā-rangi_Hakihea".split("_"),monthsShort:"Kohi_Hui_Pou_Pae_Hara_Pipi_Hōngoi_Here_Mahu_Whi-nu_Whi-ra_Haki".split("_"),monthsRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,2}/i,weekdays:"Rātapu_Mane_Tūrei_Wenerei_Tāite_Paraire_Hātarei".split("_"),weekdaysShort:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),weekdaysMin:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [i] HH:mm",LLLL:"dddd, D MMMM YYYY [i] HH:mm"},calendar:{sameDay:"[i teie mahana, i] LT",nextDay:"[apopo i] LT",nextWeek:"dddd [i] LT",lastDay:"[inanahi i] LT",lastWeek:"dddd [whakamutunga i] LT",sameElse:"L"},relativeTime:{future:"i roto i %s",past:"%s i mua",s:"te hēkona ruarua",ss:"%d hēkona",m:"he meneti",mm:"%d meneti",h:"te haora",hh:"%d haora",d:"he ra",dd:"%d ra",M:"he marama",MM:"%d marama",y:"he tau",yy:"%d tau"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},6909:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("mk",{months:"јануари_февруари_март_април_мај_јуни_јули_август_септември_октомври_ноември_декември".split("_"),monthsShort:"јан_фев_мар_апр_мај_јун_јул_авг_сеп_окт_ное_дек".split("_"),weekdays:"недела_понеделник_вторник_среда_четврток_петок_сабота".split("_"),weekdaysShort:"нед_пон_вто_сре_чет_пет_саб".split("_"),weekdaysMin:"нe_пo_вт_ср_че_пе_сa".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Денес во] LT",nextDay:"[Утре во] LT",nextWeek:"[Во] dddd [во] LT",lastDay:"[Вчера во] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Изминатата] dddd [во] LT";case 1:case 2:case 4:case 5:return"[Изминатиот] dddd [во] LT"}},sameElse:"L"},relativeTime:{future:"за %s",past:"пред %s",s:"неколку секунди",ss:"%d секунди",m:"една минута",mm:"%d минути",h:"еден час",hh:"%d часа",d:"еден ден",dd:"%d дена",M:"еден месец",MM:"%d месеци",y:"една година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(e){var t=e%10,a=e%100;return 0===e?e+"-ев":0===a?e+"-ен":a>10&&a<20?e+"-ти":1===t?e+"-ви":2===t?e+"-ри":7===t||8===t?e+"-ми":e+"-ти"},week:{dow:1,doy:7}});return t}))},"69f3":function(e,t,a){var n,r,i,o=a("cdce"),s=a("da84"),l=a("861d"),c=a("9112"),u=a("1a2d"),d=a("c6cd"),h=a("f772"),f=a("d012"),p="Object already initialized",m=s.TypeError,v=s.WeakMap,_=function(e){return i(e)?r(e):n(e,{})},g=function(e){return function(t){var a;if(!l(t)||(a=r(t)).type!==e)throw m("Incompatible receiver, "+e+" required");return a}};if(o||d.state){var y=d.state||(d.state=new v);y.get=y.get,y.has=y.has,y.set=y.set,n=function(e,t){if(y.has(e))throw m(p);return t.facade=e,y.set(e,t),t},r=function(e){return y.get(e)||{}},i=function(e){return y.has(e)}}else{var b=h("state");f[b]=!0,n=function(e,t){if(u(e,b))throw m(p);return t.facade=e,c(e,b,t),t},r=function(e){return u(e,b)?e[b]:{}},i=function(e){return u(e,b)}}e.exports={set:n,get:r,has:i,enforce:_,getterFor:g}},"6ce3":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("nb",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"sø._ma._ti._on._to._fr._lø.".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] HH:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[forrige] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"noen sekunder",ss:"%d sekunder",m:"ett minutt",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dager",w:"en uke",ww:"%d uker",M:"en måned",MM:"%d måneder",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},"6d79":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={0:"-ші",1:"-ші",2:"-ші",3:"-ші",4:"-ші",5:"-ші",6:"-шы",7:"-ші",8:"-ші",9:"-шы",10:"-шы",20:"-шы",30:"-шы",40:"-шы",50:"-ші",60:"-шы",70:"-ші",80:"-ші",90:"-шы",100:"-ші"},a=e.defineLocale("kk",{months:"қаңтар_ақпан_наурыз_сәуір_мамыр_маусым_шілде_тамыз_қыркүйек_қазан_қараша_желтоқсан".split("_"),monthsShort:"қаң_ақп_нау_сәу_мам_мау_шіл_там_қыр_қаз_қар_жел".split("_"),weekdays:"жексенбі_дүйсенбі_сейсенбі_сәрсенбі_бейсенбі_жұма_сенбі".split("_"),weekdaysShort:"жек_дүй_сей_сәр_бей_жұм_сен".split("_"),weekdaysMin:"жк_дй_сй_ср_бй_жм_сн".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгін сағат] LT",nextDay:"[Ертең сағат] LT",nextWeek:"dddd [сағат] LT",lastDay:"[Кеше сағат] LT",lastWeek:"[Өткен аптаның] dddd [сағат] LT",sameElse:"L"},relativeTime:{future:"%s ішінде",past:"%s бұрын",s:"бірнеше секунд",ss:"%d секунд",m:"бір минут",mm:"%d минут",h:"бір сағат",hh:"%d сағат",d:"бір күн",dd:"%d күн",M:"бір ай",MM:"%d ай",y:"бір жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(ші|шы)/,ordinal:function(e){var a=e%10,n=e>=100?100:null;return e+(t[e]||t[a]||t[n])},week:{dow:1,doy:7}});return a}))},"6d83":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ar-tn",{months:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,doy:4}});return t}))},"6e98":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("it",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:function(){return"[Oggi a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},nextDay:function(){return"[Domani a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},nextWeek:function(){return"dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},lastDay:function(){return"[Ieri a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},lastWeek:function(){switch(this.day()){case 0:return"[La scorsa] dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT";default:return"[Lo scorso] dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"}},sameElse:"L"},relativeTime:{future:"tra %s",past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",w:"una settimana",ww:"%d settimane",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},"6f12":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("it-ch",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Oggi alle] LT",nextDay:"[Domani alle] LT",nextWeek:"dddd [alle] LT",lastDay:"[Ieri alle] LT",lastWeek:function(){switch(this.day()){case 0:return"[la scorsa] dddd [alle] LT";default:return"[lo scorso] dddd [alle] LT"}},sameElse:"L"},relativeTime:{future:function(e){return(/^[0-9].+$/.test(e)?"tra":"in")+" "+e},past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},"6f50":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-nz",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:1,doy:4}});return t}))},7118:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mai_jun._jul._aug._sep._okt._nov._des.".split("_"),a="jan_feb_mrt_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),n=e.defineLocale("fy",{months:"jannewaris_febrewaris_maart_april_maaie_juny_july_augustus_septimber_oktober_novimber_desimber".split("_"),monthsShort:function(e,n){return e?/-MMM-/.test(n)?a[e.month()]:t[e.month()]:t},monthsParseExact:!0,weekdays:"snein_moandei_tiisdei_woansdei_tongersdei_freed_sneon".split("_"),weekdaysShort:"si._mo._ti._wo._to._fr._so.".split("_"),weekdaysMin:"Si_Mo_Ti_Wo_To_Fr_So".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[hjoed om] LT",nextDay:"[moarn om] LT",nextWeek:"dddd [om] LT",lastDay:"[juster om] LT",lastWeek:"[ôfrûne] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oer %s",past:"%s lyn",s:"in pear sekonden",ss:"%d sekonden",m:"ien minút",mm:"%d minuten",h:"ien oere",hh:"%d oeren",d:"ien dei",dd:"%d dagen",M:"ien moanne",MM:"%d moannen",y:"ien jier",yy:"%d jierren"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return n}))},7234:function(e,t){e.exports=function(e){return null===e||void 0===e}},7333:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-il",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a}});return t}))},7418:function(e,t){t.f=Object.getOwnPropertySymbols},"74dc":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("sw",{months:"Januari_Februari_Machi_Aprili_Mei_Juni_Julai_Agosti_Septemba_Oktoba_Novemba_Desemba".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ago_Sep_Okt_Nov_Des".split("_"),weekdays:"Jumapili_Jumatatu_Jumanne_Jumatano_Alhamisi_Ijumaa_Jumamosi".split("_"),weekdaysShort:"Jpl_Jtat_Jnne_Jtan_Alh_Ijm_Jmos".split("_"),weekdaysMin:"J2_J3_J4_J5_Al_Ij_J1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"hh:mm A",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[leo saa] LT",nextDay:"[kesho saa] LT",nextWeek:"[wiki ijayo] dddd [saat] LT",lastDay:"[jana] LT",lastWeek:"[wiki iliyopita] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s baadaye",past:"tokea %s",s:"hivi punde",ss:"sekunde %d",m:"dakika moja",mm:"dakika %d",h:"saa limoja",hh:"masaa %d",d:"siku moja",dd:"siku %d",M:"mwezi mmoja",MM:"miezi %d",y:"mwaka mmoja",yy:"miaka %d"},week:{dow:1,doy:7}});return t}))},7839:function(e,t){e.exports=["constructor","hasOwnProperty","isPrototypeOf","propertyIsEnumerable","toLocaleString","toString","valueOf"]},"7a77":function(e,t,a){"use strict";function n(e){this.message=e}n.prototype.toString=function(){return"Cancel"+(this.message?": "+this.message:"")},n.prototype.__CANCEL__=!0,e.exports=n},"7aac":function(e,t,a){"use strict";var n=a("c532");e.exports=n.isStandardBrowserEnv()?function(){return{write:function(e,t,a,r,i,o){var s=[];s.push(e+"="+encodeURIComponent(t)),n.isNumber(a)&&s.push("expires="+new Date(a).toGMTString()),n.isString(r)&&s.push("path="+r),n.isString(i)&&s.push("domain="+i),!0===o&&s.push("secure"),document.cookie=s.join("; ")},read:function(e){var t=document.cookie.match(new RegExp("(^|;\\s*)("+e+")=([^;]*)"));return t?decodeURIComponent(t[3]):null},remove:function(e){this.write(e,"",Date.now()-864e5)}}}():function(){return{write:function(){},read:function(){return null},remove:function(){}}}()},"7b0b":function(e,t,a){var n=a("1d80"),r=Object;e.exports=function(e){return r(n(e))}},"7be6":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="január_február_marec_apríl_máj_jún_júl_august_september_október_november_december".split("_"),a="jan_feb_mar_apr_máj_jún_júl_aug_sep_okt_nov_dec".split("_");function n(e){return e>1&&e<5}function r(e,t,a,r){var i=e+" ";switch(a){case"s":return t||r?"pár sekúnd":"pár sekundami";case"ss":return t||r?i+(n(e)?"sekundy":"sekúnd"):i+"sekundami";case"m":return t?"minúta":r?"minútu":"minútou";case"mm":return t||r?i+(n(e)?"minúty":"minút"):i+"minútami";case"h":return t?"hodina":r?"hodinu":"hodinou";case"hh":return t||r?i+(n(e)?"hodiny":"hodín"):i+"hodinami";case"d":return t||r?"deň":"dňom";case"dd":return t||r?i+(n(e)?"dni":"dní"):i+"dňami";case"M":return t||r?"mesiac":"mesiacom";case"MM":return t||r?i+(n(e)?"mesiace":"mesiacov"):i+"mesiacmi";case"y":return t||r?"rok":"rokom";case"yy":return t||r?i+(n(e)?"roky":"rokov"):i+"rokmi"}}var i=e.defineLocale("sk",{months:t,monthsShort:a,weekdays:"nedeľa_pondelok_utorok_streda_štvrtok_piatok_sobota".split("_"),weekdaysShort:"ne_po_ut_st_št_pi_so".split("_"),weekdaysMin:"ne_po_ut_st_št_pi_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm"},calendar:{sameDay:"[dnes o] LT",nextDay:"[zajtra o] LT",nextWeek:function(){switch(this.day()){case 0:return"[v nedeľu o] LT";case 1:case 2:return"[v] dddd [o] LT";case 3:return"[v stredu o] LT";case 4:return"[vo štvrtok o] LT";case 5:return"[v piatok o] LT";case 6:return"[v sobotu o] LT"}},lastDay:"[včera o] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulú nedeľu o] LT";case 1:case 2:return"[minulý] dddd [o] LT";case 3:return"[minulú stredu o] LT";case 4:case 5:return"[minulý] dddd [o] LT";case 6:return"[minulú sobotu o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"pred %s",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return i}))},"7d7e":function(e,t,a){var n=a("40d5"),r=Function.prototype,i=r.call,o=n&&r.bind.bind(i,i);e.exports=function(e){return n?o(e):function(){return i.apply(e,arguments)}}},"7f33":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("yo",{months:"Sẹ́rẹ́_Èrèlè_Ẹrẹ̀nà_Ìgbé_Èbibi_Òkùdu_Agẹmo_Ògún_Owewe_Ọ̀wàrà_Bélú_Ọ̀pẹ̀̀".split("_"),monthsShort:"Sẹ́r_Èrl_Ẹrn_Ìgb_Èbi_Òkù_Agẹ_Ògú_Owe_Ọ̀wà_Bél_Ọ̀pẹ̀̀".split("_"),weekdays:"Àìkú_Ajé_Ìsẹ́gun_Ọjọ́rú_Ọjọ́bọ_Ẹtì_Àbámẹ́ta".split("_"),weekdaysShort:"Àìk_Ajé_Ìsẹ́_Ọjr_Ọjb_Ẹtì_Àbá".split("_"),weekdaysMin:"Àì_Aj_Ìs_Ọr_Ọb_Ẹt_Àb".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Ònì ni] LT",nextDay:"[Ọ̀la ni] LT",nextWeek:"dddd [Ọsẹ̀ tón'bọ] [ni] LT",lastDay:"[Àna ni] LT",lastWeek:"dddd [Ọsẹ̀ tólọ́] [ni] LT",sameElse:"L"},relativeTime:{future:"ní %s",past:"%s kọjá",s:"ìsẹjú aayá die",ss:"aayá %d",m:"ìsẹjú kan",mm:"ìsẹjú %d",h:"wákati kan",hh:"wákati %d",d:"ọjọ́ kan",dd:"ọjọ́ %d",M:"osù kan",MM:"osù %d",y:"ọdún kan",yy:"ọdún %d"},dayOfMonthOrdinalParse:/ọjọ́\s\d{1,2}/,ordinal:"ọjọ́ %d",week:{dow:1,doy:4}});return t}))},8155:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r=e+" ";switch(a){case"s":return t||n?"nekaj sekund":"nekaj sekundami";case"ss":return r+=1===e?t?"sekundo":"sekundi":2===e?t||n?"sekundi":"sekundah":e<5?t||n?"sekunde":"sekundah":"sekund",r;case"m":return t?"ena minuta":"eno minuto";case"mm":return r+=1===e?t?"minuta":"minuto":2===e?t||n?"minuti":"minutama":e<5?t||n?"minute":"minutami":t||n?"minut":"minutami",r;case"h":return t?"ena ura":"eno uro";case"hh":return r+=1===e?t?"ura":"uro":2===e?t||n?"uri":"urama":e<5?t||n?"ure":"urami":t||n?"ur":"urami",r;case"d":return t||n?"en dan":"enim dnem";case"dd":return r+=1===e?t||n?"dan":"dnem":2===e?t||n?"dni":"dnevoma":t||n?"dni":"dnevi",r;case"M":return t||n?"en mesec":"enim mesecem";case"MM":return r+=1===e?t||n?"mesec":"mesecem":2===e?t||n?"meseca":"mesecema":e<5?t||n?"mesece":"meseci":t||n?"mesecev":"meseci",r;case"y":return t||n?"eno leto":"enim letom";case"yy":return r+=1===e?t||n?"leto":"letom":2===e?t||n?"leti":"letoma":e<5?t||n?"leta":"leti":t||n?"let":"leti",r}}var a=e.defineLocale("sl",{months:"januar_februar_marec_april_maj_junij_julij_avgust_september_oktober_november_december".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljek_torek_sreda_četrtek_petek_sobota".split("_"),weekdaysShort:"ned._pon._tor._sre._čet._pet._sob.".split("_"),weekdaysMin:"ne_po_to_sr_če_pe_so".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD. MM. YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danes ob] LT",nextDay:"[jutri ob] LT",nextWeek:function(){switch(this.day()){case 0:return"[v] [nedeljo] [ob] LT";case 3:return"[v] [sredo] [ob] LT";case 6:return"[v] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[v] dddd [ob] LT"}},lastDay:"[včeraj ob] LT",lastWeek:function(){switch(this.day()){case 0:return"[prejšnjo] [nedeljo] [ob] LT";case 3:return"[prejšnjo] [sredo] [ob] LT";case 6:return"[prejšnjo] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[prejšnji] dddd [ob] LT"}},sameElse:"L"},relativeTime:{future:"čez %s",past:"pred %s",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return a}))},"81e9":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="nolla yksi kaksi kolme neljä viisi kuusi seitsemän kahdeksan yhdeksän".split(" "),a=["nolla","yhden","kahden","kolmen","neljän","viiden","kuuden",t[7],t[8],t[9]];function n(e,t,a,n){var i="";switch(a){case"s":return n?"muutaman sekunnin":"muutama sekunti";case"ss":i=n?"sekunnin":"sekuntia";break;case"m":return n?"minuutin":"minuutti";case"mm":i=n?"minuutin":"minuuttia";break;case"h":return n?"tunnin":"tunti";case"hh":i=n?"tunnin":"tuntia";break;case"d":return n?"päivän":"päivä";case"dd":i=n?"päivän":"päivää";break;case"M":return n?"kuukauden":"kuukausi";case"MM":i=n?"kuukauden":"kuukautta";break;case"y":return n?"vuoden":"vuosi";case"yy":i=n?"vuoden":"vuotta";break}return i=r(e,n)+" "+i,i}function r(e,n){return e<10?n?a[e]:t[e]:e}var i=e.defineLocale("fi",{months:"tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kesäkuu_heinäkuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu".split("_"),monthsShort:"tammi_helmi_maalis_huhti_touko_kesä_heinä_elo_syys_loka_marras_joulu".split("_"),weekdays:"sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai".split("_"),weekdaysShort:"su_ma_ti_ke_to_pe_la".split("_"),weekdaysMin:"su_ma_ti_ke_to_pe_la".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"Do MMMM[ta] YYYY",LLL:"Do MMMM[ta] YYYY, [klo] HH.mm",LLLL:"dddd, Do MMMM[ta] YYYY, [klo] HH.mm",l:"D.M.YYYY",ll:"Do MMM YYYY",lll:"Do MMM YYYY, [klo] HH.mm",llll:"ddd, Do MMM YYYY, [klo] HH.mm"},calendar:{sameDay:"[tänään] [klo] LT",nextDay:"[huomenna] [klo] LT",nextWeek:"dddd [klo] LT",lastDay:"[eilen] [klo] LT",lastWeek:"[viime] dddd[na] [klo] LT",sameElse:"L"},relativeTime:{future:"%s päästä",past:"%s sitten",s:n,ss:n,m:n,mm:n,h:n,hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return i}))},8230:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},a={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},n=e.defineLocale("ar-sa",{months:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,a){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return a[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:0,doy:6}});return n}))},"825a":function(e,t,a){var n=a("861d"),r=String,i=TypeError;e.exports=function(e){if(n(e))return e;throw i(r(e)+" is not an object")}},"83ab":function(e,t,a){var n=a("d039");e.exports=!n((function(){return 7!=Object.defineProperty({},1,{get:function(){return 7}})[1]}))},"83b9":function(e,t,a){"use strict";var n=a("d925"),r=a("e683");e.exports=function(e,t){return e&&!n(t)?r(e,t):t}},"848b":function(e,t,a){"use strict";var n=a("4a0c"),r={};["object","boolean","number","function","string","symbol"].forEach((function(e,t){r[e]=function(a){return typeof a===e||"a"+(t<1?"n ":" ")+e}}));var i={},o=n.version.split(".");function s(e,t){for(var a=t?t.split("."):o,n=e.split("."),r=0;r<3;r++){if(a[r]>n[r])return!0;if(a[r]<n[r])return!1}return!1}function l(e,t,a){if("object"!==typeof e)throw new TypeError("options must be an object");var n=Object.keys(e),r=n.length;while(r-- >0){var i=n[r],o=t[i];if(o){var s=e[i],l=void 0===s||o(s,i,e);if(!0!==l)throw new TypeError("option "+i+" must be "+l)}else if(!0!==a)throw Error("Unknown option "+i)}}r.transitional=function(e,t,a){var r=t&&s(t);function o(e,t){return"[Axios v"+n.version+"] Transitional option '"+e+"'"+t+(a?". "+a:"")}return function(a,n,s){if(!1===e)throw new Error(o(n," has been removed in "+t));return r&&!i[n]&&(i[n]=!0,console.warn(o(n," has been deprecated since v"+t+" and will be removed in the near future"))),!e||e(a,n,s)}},e.exports={isOlderVersion:s,assertOptions:l,validators:r}},"84aa":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("bg",{months:"януари_февруари_март_април_май_юни_юли_август_септември_октомври_ноември_декември".split("_"),monthsShort:"яну_фев_мар_апр_май_юни_юли_авг_сеп_окт_ное_дек".split("_"),weekdays:"неделя_понеделник_вторник_сряда_четвъртък_петък_събота".split("_"),weekdaysShort:"нед_пон_вто_сря_чет_пет_съб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Днес в] LT",nextDay:"[Утре в] LT",nextWeek:"dddd [в] LT",lastDay:"[Вчера в] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Миналата] dddd [в] LT";case 1:case 2:case 4:case 5:return"[Миналия] dddd [в] LT"}},sameElse:"L"},relativeTime:{future:"след %s",past:"преди %s",s:"няколко секунди",ss:"%d секунди",m:"минута",mm:"%d минути",h:"час",hh:"%d часа",d:"ден",dd:"%d дена",w:"седмица",ww:"%d седмици",M:"месец",MM:"%d месеца",y:"година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(e){var t=e%10,a=e%100;return 0===e?e+"-ев":0===a?e+"-ен":a>10&&a<20?e+"-ти":1===t?e+"-ви":2===t?e+"-ри":7===t||8===t?e+"-ми":e+"-ти"},week:{dow:1,doy:7}});return t}))},"861d":function(e,t,a){var n=a("1626"),r=a("8ea1"),i=r.all;e.exports=r.IS_HTMLDDA?function(e){return"object"==typeof e?null!==e:n(e)||e===i}:function(e){return"object"==typeof e?null!==e:n(e)}},8689:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"၁",2:"၂",3:"၃",4:"၄",5:"၅",6:"၆",7:"၇",8:"၈",9:"၉",0:"၀"},a={"၁":"1","၂":"2","၃":"3","၄":"4","၅":"5","၆":"6","၇":"7","၈":"8","၉":"9","၀":"0"},n=e.defineLocale("my",{months:"ဇန်နဝါရီ_ဖေဖော်ဝါရီ_မတ်_ဧပြီ_မေ_ဇွန်_ဇူလိုင်_သြဂုတ်_စက်တင်ဘာ_အောက်တိုဘာ_နိုဝင်ဘာ_ဒီဇင်ဘာ".split("_"),monthsShort:"ဇန်_ဖေ_မတ်_ပြီ_မေ_ဇွန်_လိုင်_သြ_စက်_အောက်_နို_ဒီ".split("_"),weekdays:"တနင်္ဂနွေ_တနင်္လာ_အင်္ဂါ_ဗုဒ္ဓဟူး_ကြာသပတေး_သောကြာ_စနေ".split("_"),weekdaysShort:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),weekdaysMin:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ယနေ.] LT [မှာ]",nextDay:"[မနက်ဖြန်] LT [မှာ]",nextWeek:"dddd LT [မှာ]",lastDay:"[မနေ.က] LT [မှာ]",lastWeek:"[ပြီးခဲ့သော] dddd LT [မှာ]",sameElse:"L"},relativeTime:{future:"လာမည့် %s မှာ",past:"လွန်ခဲ့သော %s က",s:"စက္ကန်.အနည်းငယ်",ss:"%d စက္ကန့်",m:"တစ်မိနစ်",mm:"%d မိနစ်",h:"တစ်နာရီ",hh:"%d နာရီ",d:"တစ်ရက်",dd:"%d ရက်",M:"တစ်လ",MM:"%d လ",y:"တစ်နှစ်",yy:"%d နှစ်"},preparse:function(e){return e.replace(/[၁၂၃၄၅၆၇၈၉၀]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},week:{dow:1,doy:4}});return n}))},8840:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("gl",{months:"xaneiro_febreiro_marzo_abril_maio_xuño_xullo_agosto_setembro_outubro_novembro_decembro".split("_"),monthsShort:"xan._feb._mar._abr._mai._xuñ._xul._ago._set._out._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"domingo_luns_martes_mércores_xoves_venres_sábado".split("_"),weekdaysShort:"dom._lun._mar._mér._xov._ven._sáb.".split("_"),weekdaysMin:"do_lu_ma_mé_xo_ve_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoxe "+(1!==this.hours()?"ás":"á")+"] LT"},nextDay:function(){return"[mañá "+(1!==this.hours()?"ás":"á")+"] LT"},nextWeek:function(){return"dddd ["+(1!==this.hours()?"ás":"a")+"] LT"},lastDay:function(){return"[onte "+(1!==this.hours()?"á":"a")+"] LT"},lastWeek:function(){return"[o] dddd [pasado "+(1!==this.hours()?"ás":"a")+"] LT"},sameElse:"L"},relativeTime:{future:function(e){return 0===e.indexOf("un")?"n"+e:"en "+e},past:"hai %s",s:"uns segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"unha hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},8925:function(e,t,a){var n=a("e330"),r=a("1626"),i=a("c6cd"),o=n(Function.toString);r(i.inspectSource)||(i.inspectSource=function(e){return o(e)}),e.exports=i.inspectSource},"898b":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),a="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),n=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],r=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,i=e.defineLocale("es",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,n){return e?/-MMM-/.test(n)?a[e.month()]:t[e.month()]:t},monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4},invalidDate:"Fecha inválida"});return i}))},"8d47":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e){return"undefined"!==typeof Function&&e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}var a=e.defineLocale("el",{monthsNominativeEl:"Ιανουάριος_Φεβρουάριος_Μάρτιος_Απρίλιος_Μάιος_Ιούνιος_Ιούλιος_Αύγουστος_Σεπτέμβριος_Οκτώβριος_Νοέμβριος_Δεκέμβριος".split("_"),monthsGenitiveEl:"Ιανουαρίου_Φεβρουαρίου_Μαρτίου_Απριλίου_Μαΐου_Ιουνίου_Ιουλίου_Αυγούστου_Σεπτεμβρίου_Οκτωβρίου_Νοεμβρίου_Δεκεμβρίου".split("_"),months:function(e,t){return e?"string"===typeof t&&/D/.test(t.substring(0,t.indexOf("MMMM")))?this._monthsGenitiveEl[e.month()]:this._monthsNominativeEl[e.month()]:this._monthsNominativeEl},monthsShort:"Ιαν_Φεβ_Μαρ_Απρ_Μαϊ_Ιουν_Ιουλ_Αυγ_Σεπ_Οκτ_Νοε_Δεκ".split("_"),weekdays:"Κυριακή_Δευτέρα_Τρίτη_Τετάρτη_Πέμπτη_Παρασκευή_Σάββατο".split("_"),weekdaysShort:"Κυρ_Δευ_Τρι_Τετ_Πεμ_Παρ_Σαβ".split("_"),weekdaysMin:"Κυ_Δε_Τρ_Τε_Πε_Πα_Σα".split("_"),meridiem:function(e,t,a){return e>11?a?"μμ":"ΜΜ":a?"πμ":"ΠΜ"},isPM:function(e){return"μ"===(e+"").toLowerCase()[0]},meridiemParse:/[ΠΜ]\.?Μ?\.?/i,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendarEl:{sameDay:"[Σήμερα {}] LT",nextDay:"[Αύριο {}] LT",nextWeek:"dddd [{}] LT",lastDay:"[Χθες {}] LT",lastWeek:function(){switch(this.day()){case 6:return"[το προηγούμενο] dddd [{}] LT";default:return"[την προηγούμενη] dddd [{}] LT"}},sameElse:"L"},calendar:function(e,a){var n=this._calendarEl[e],r=a&&a.hours();return t(n)&&(n=n.apply(a)),n.replace("{}",r%12===1?"στη":"στις")},relativeTime:{future:"σε %s",past:"%s πριν",s:"λίγα δευτερόλεπτα",ss:"%d δευτερόλεπτα",m:"ένα λεπτό",mm:"%d λεπτά",h:"μία ώρα",hh:"%d ώρες",d:"μία μέρα",dd:"%d μέρες",M:"ένας μήνας",MM:"%d μήνες",y:"ένας χρόνος",yy:"%d χρόνια"},dayOfMonthOrdinalParse:/\d{1,2}η/,ordinal:"%dη",week:{dow:1,doy:4}});return a}))},"8d57":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="styczeń_luty_marzec_kwiecień_maj_czerwiec_lipiec_sierpień_wrzesień_październik_listopad_grudzień".split("_"),a="stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_września_października_listopada_grudnia".split("_"),n=[/^sty/i,/^lut/i,/^mar/i,/^kwi/i,/^maj/i,/^cze/i,/^lip/i,/^sie/i,/^wrz/i,/^paź/i,/^lis/i,/^gru/i];function r(e){return e%10<5&&e%10>1&&~~(e/10)%10!==1}function i(e,t,a){var n=e+" ";switch(a){case"ss":return n+(r(e)?"sekundy":"sekund");case"m":return t?"minuta":"minutę";case"mm":return n+(r(e)?"minuty":"minut");case"h":return t?"godzina":"godzinę";case"hh":return n+(r(e)?"godziny":"godzin");case"ww":return n+(r(e)?"tygodnie":"tygodni");case"MM":return n+(r(e)?"miesiące":"miesięcy");case"yy":return n+(r(e)?"lata":"lat")}}var o=e.defineLocale("pl",{months:function(e,n){return e?/D MMMM/.test(n)?a[e.month()]:t[e.month()]:t},monthsShort:"sty_lut_mar_kwi_maj_cze_lip_sie_wrz_paź_lis_gru".split("_"),monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"niedziela_poniedziałek_wtorek_środa_czwartek_piątek_sobota".split("_"),weekdaysShort:"ndz_pon_wt_śr_czw_pt_sob".split("_"),weekdaysMin:"Nd_Pn_Wt_Śr_Cz_Pt_So".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Dziś o] LT",nextDay:"[Jutro o] LT",nextWeek:function(){switch(this.day()){case 0:return"[W niedzielę o] LT";case 2:return"[We wtorek o] LT";case 3:return"[W środę o] LT";case 6:return"[W sobotę o] LT";default:return"[W] dddd [o] LT"}},lastDay:"[Wczoraj o] LT",lastWeek:function(){switch(this.day()){case 0:return"[W zeszłą niedzielę o] LT";case 3:return"[W zeszłą środę o] LT";case 6:return"[W zeszłą sobotę o] LT";default:return"[W zeszły] dddd [o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"%s temu",s:"kilka sekund",ss:i,m:i,mm:i,h:i,hh:i,d:"1 dzień",dd:"%d dni",w:"tydzień",ww:i,M:"miesiąc",MM:i,y:"rok",yy:i},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return o}))},"8df4":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"۱",2:"۲",3:"۳",4:"۴",5:"۵",6:"۶",7:"۷",8:"۸",9:"۹",0:"۰"},a={"۱":"1","۲":"2","۳":"3","۴":"4","۵":"5","۶":"6","۷":"7","۸":"8","۹":"9","۰":"0"},n=e.defineLocale("fa",{months:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),monthsShort:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),weekdays:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysShort:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysMin:"ی_د_س_چ_پ_ج_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/قبل از ظهر|بعد از ظهر/,isPM:function(e){return/بعد از ظهر/.test(e)},meridiem:function(e,t,a){return e<12?"قبل از ظهر":"بعد از ظهر"},calendar:{sameDay:"[امروز ساعت] LT",nextDay:"[فردا ساعت] LT",nextWeek:"dddd [ساعت] LT",lastDay:"[دیروز ساعت] LT",lastWeek:"dddd [پیش] [ساعت] LT",sameElse:"L"},relativeTime:{future:"در %s",past:"%s پیش",s:"چند ثانیه",ss:"%d ثانیه",m:"یک دقیقه",mm:"%d دقیقه",h:"یک ساعت",hh:"%d ساعت",d:"یک روز",dd:"%d روز",M:"یک ماه",MM:"%d ماه",y:"یک سال",yy:"%d سال"},preparse:function(e){return e.replace(/[۰-۹]/g,(function(e){return a[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},dayOfMonthOrdinalParse:/\d{1,2}م/,ordinal:"%dم",week:{dow:6,doy:12}});return n}))},"8df4b":function(e,t,a){"use strict";var n=a("7a77");function r(e){if("function"!==typeof e)throw new TypeError("executor must be a function.");var t;this.promise=new Promise((function(e){t=e}));var a=this;e((function(e){a.reason||(a.reason=new n(e),t(a.reason))}))}r.prototype.throwIfRequested=function(){if(this.reason)throw this.reason},r.source=function(){var e,t=new r((function(t){e=t}));return{token:t,cancel:e}},e.exports=r},"8e73":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},a={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},n=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},r={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},i=function(e){return function(t,a,i,o){var s=n(t),l=r[e][n(t)];return 2===s&&(l=l[a?0:1]),l.replace(/%d/i,t)}},o=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"],s=e.defineLocale("ar",{months:o,monthsShort:o,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,a){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:i("s"),ss:i("s"),m:i("m"),mm:i("m"),h:i("h"),hh:i("h"),d:i("d"),dd:i("d"),M:i("M"),MM:i("M"),y:i("y"),yy:i("y")},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return a[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}});return s}))},"8ea1":function(e,t){var a="object"==typeof document&&document.all,n="undefined"==typeof a&&void 0!==a;e.exports={all:a,IS_HTMLDDA:n}},9043:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"},a={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"},n=e.defineLocale("bn",{months:"জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",ss:"%d সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(e){return e.replace(/[১২৩৪৫৬৭৮৯০]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/রাত|সকাল|দুপুর|বিকাল|রাত/,meridiemHour:function(e,t){return 12===e&&(e=0),"রাত"===t&&e>=4||"দুপুর"===t&&e<5||"বিকাল"===t?e+12:e},meridiem:function(e,t,a){return e<4?"রাত":e<10?"সকাল":e<17?"দুপুর":e<20?"বিকাল":"রাত"},week:{dow:0,doy:6}});return n}))},"90e3":function(e,t,a){var n=a("e330"),r=0,i=Math.random(),o=n(1..toString);e.exports=function(e){return"Symbol("+(void 0===e?"":e)+")_"+o(++r+i,36)}},"90ea":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-tw",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,a){var n=100*e+t;return n<600?"凌晨":n<900?"早上":n<1130?"上午":n<1230?"中午":n<1800?"下午":"晚上"},calendar:{sameDay:"[今天] LT",nextDay:"[明天] LT",nextWeek:"[下]dddd LT",lastDay:"[昨天] LT",lastWeek:"[上]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}});return t}))},9112:function(e,t,a){var n=a("83ab"),r=a("9bf2"),i=a("5c6c");e.exports=n?function(e,t,a){return r.f(e,t,i(1,a))}:function(e,t,a){return e[t]=a,e}},"938d":function(e,t,a){"use strict";(function(e){a.d(t,"a",(function(){return r}));var n=function(t){var a=arguments.length>1&&void 0!==arguments[1]?arguments[1]:null,n="undefined"!==typeof e&&e?Object({NODE_ENV:"production",BASE_URL:""})||!1:{};return t?n[t]||a:n},r=function(){return n("BOOTSTRAP_VUE_NO_WARN")||"production"===n("NODE_ENV")}}).call(this,a("4362"))},"94ca":function(e,t,a){var n=a("d039"),r=a("1626"),i=/#|\.prototype\./,o=function(e,t){var a=l[s(e)];return a==u||a!=c&&(r(t)?n(t):!!t)},s=o.normalize=function(e){return String(e).replace(i,".").toLowerCase()},l=o.data={},c=o.NATIVE="N",u=o.POLYFILL="P";e.exports=o},"957c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t){var a=e.split("_");return t%10===1&&t%100!==11?a[0]:t%10>=2&&t%10<=4&&(t%100<10||t%100>=20)?a[1]:a[2]}function a(e,a,n){var r={ss:a?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:a?"минута_минуты_минут":"минуту_минуты_минут",hh:"час_часа_часов",dd:"день_дня_дней",ww:"неделя_недели_недель",MM:"месяц_месяца_месяцев",yy:"год_года_лет"};return"m"===n?a?"минута":"минуту":e+" "+t(r[n],+e)}var n=[/^янв/i,/^фев/i,/^мар/i,/^апр/i,/^ма[йя]/i,/^июн/i,/^июл/i,/^авг/i,/^сен/i,/^окт/i,/^ноя/i,/^дек/i],r=e.defineLocale("ru",{months:{format:"января_февраля_марта_апреля_мая_июня_июля_августа_сентября_октября_ноября_декабря".split("_"),standalone:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_")},monthsShort:{format:"янв._февр._мар._апр._мая_июня_июля_авг._сент._окт._нояб._дек.".split("_"),standalone:"янв._февр._март_апр._май_июнь_июль_авг._сент._окт._нояб._дек.".split("_")},weekdays:{standalone:"воскресенье_понедельник_вторник_среда_четверг_пятница_суббота".split("_"),format:"воскресенье_понедельник_вторник_среду_четверг_пятницу_субботу".split("_"),isFormat:/\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?] ?dddd/},weekdaysShort:"вс_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"вс_пн_вт_ср_чт_пт_сб".split("_"),monthsParse:n,longMonthsParse:n,shortMonthsParse:n,monthsRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsShortRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsStrictRegex:/^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,monthsShortStrictRegex:/^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., H:mm",LLLL:"dddd, D MMMM YYYY г., H:mm"},calendar:{sameDay:"[Сегодня, в] LT",nextDay:"[Завтра, в] LT",lastDay:"[Вчера, в] LT",nextWeek:function(e){if(e.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В следующее] dddd, [в] LT";case 1:case 2:case 4:return"[В следующий] dddd, [в] LT";case 3:case 5:case 6:return"[В следующую] dddd, [в] LT"}},lastWeek:function(e){if(e.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В прошлое] dddd, [в] LT";case 1:case 2:case 4:return"[В прошлый] dddd, [в] LT";case 3:case 5:case 6:return"[В прошлую] dddd, [в] LT"}},sameElse:"L"},relativeTime:{future:"через %s",past:"%s назад",s:"несколько секунд",ss:a,m:a,mm:a,h:"час",hh:a,d:"день",dd:a,w:"неделя",ww:a,M:"месяц",MM:a,y:"год",yy:a},meridiemParse:/ночи|утра|дня|вечера/i,isPM:function(e){return/^(дня|вечера)$/.test(e)},meridiem:function(e,t,a){return e<4?"ночи":e<12?"утра":e<17?"дня":"вечера"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го|я)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":return e+"-й";case"D":return e+"-го";case"w":case"W":return e+"-я";default:return e}},week:{dow:1,doy:4}});return r}))},"958b":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){switch(a){case"s":return t?"хэдхэн секунд":"хэдхэн секундын";case"ss":return e+(t?" секунд":" секундын");case"m":case"mm":return e+(t?" минут":" минутын");case"h":case"hh":return e+(t?" цаг":" цагийн");case"d":case"dd":return e+(t?" өдөр":" өдрийн");case"M":case"MM":return e+(t?" сар":" сарын");case"y":case"yy":return e+(t?" жил":" жилийн");default:return e}}var a=e.defineLocale("mn",{months:"Нэгдүгээр сар_Хоёрдугаар сар_Гуравдугаар сар_Дөрөвдүгээр сар_Тавдугаар сар_Зургадугаар сар_Долдугаар сар_Наймдугаар сар_Есдүгээр сар_Аравдугаар сар_Арван нэгдүгээр сар_Арван хоёрдугаар сар".split("_"),monthsShort:"1 сар_2 сар_3 сар_4 сар_5 сар_6 сар_7 сар_8 сар_9 сар_10 сар_11 сар_12 сар".split("_"),monthsParseExact:!0,weekdays:"Ням_Даваа_Мягмар_Лхагва_Пүрэв_Баасан_Бямба".split("_"),weekdaysShort:"Ням_Дав_Мяг_Лха_Пүр_Баа_Бям".split("_"),weekdaysMin:"Ня_Да_Мя_Лх_Пү_Ба_Бя".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY оны MMMMын D",LLL:"YYYY оны MMMMын D HH:mm",LLLL:"dddd, YYYY оны MMMMын D HH:mm"},meridiemParse:/ҮӨ|ҮХ/i,isPM:function(e){return"ҮХ"===e},meridiem:function(e,t,a){return e<12?"ҮӨ":"ҮХ"},calendar:{sameDay:"[Өнөөдөр] LT",nextDay:"[Маргааш] LT",nextWeek:"[Ирэх] dddd LT",lastDay:"[Өчигдөр] LT",lastWeek:"[Өнгөрсөн] dddd LT",sameElse:"L"},relativeTime:{future:"%s дараа",past:"%s өмнө",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2} өдөр/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+" өдөр";default:return e}}});return a}))},9609:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={0:"-чү",1:"-чи",2:"-чи",3:"-чү",4:"-чү",5:"-чи",6:"-чы",7:"-чи",8:"-чи",9:"-чу",10:"-чу",20:"-чы",30:"-чу",40:"-чы",50:"-чү",60:"-чы",70:"-чи",80:"-чи",90:"-чу",100:"-чү"},a=e.defineLocale("ky",{months:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_"),monthsShort:"янв_фев_март_апр_май_июнь_июль_авг_сен_окт_ноя_дек".split("_"),weekdays:"Жекшемби_Дүйшөмбү_Шейшемби_Шаршемби_Бейшемби_Жума_Ишемби".split("_"),weekdaysShort:"Жек_Дүй_Шей_Шар_Бей_Жум_Ише".split("_"),weekdaysMin:"Жк_Дй_Шй_Шр_Бй_Жм_Иш".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгүн саат] LT",nextDay:"[Эртең саат] LT",nextWeek:"dddd [саат] LT",lastDay:"[Кечээ саат] LT",lastWeek:"[Өткөн аптанын] dddd [күнү] [саат] LT",sameElse:"L"},relativeTime:{future:"%s ичинде",past:"%s мурун",s:"бирнече секунд",ss:"%d секунд",m:"бир мүнөт",mm:"%d мүнөт",h:"бир саат",hh:"%d саат",d:"бир күн",dd:"%d күн",M:"бир ай",MM:"%d ай",y:"бир жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(чи|чы|чү|чу)/,ordinal:function(e){var a=e%10,n=e>=100?100:null;return e+(t[e]||t[a]||t[n])},week:{dow:1,doy:7}});return a}))},9686:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"},a={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"},n=e.defineLocale("bn-bd",{months:"জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",ss:"%d সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(e){return e.replace(/[১২৩৪৫৬৭৮৯০]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/রাত|ভোর|সকাল|দুপুর|বিকাল|সন্ধ্যা|রাত/,meridiemHour:function(e,t){return 12===e&&(e=0),"রাত"===t?e<4?e:e+12:"ভোর"===t||"সকাল"===t?e:"দুপুর"===t?e>=3?e:e+12:"বিকাল"===t||"সন্ধ্যা"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"রাত":e<6?"ভোর":e<12?"সকাল":e<15?"দুপুর":e<18?"বিকাল":e<20?"সন্ধ্যা":"রাত"},week:{dow:0,doy:6}});return n}))},"972c":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a){var n={ss:"secunde",mm:"minute",hh:"ore",dd:"zile",ww:"săptămâni",MM:"luni",yy:"ani"},r=" ";return(e%100>=20||e>=100&&e%100===0)&&(r=" de "),e+r+n[a]}var a=e.defineLocale("ro",{months:"ianuarie_februarie_martie_aprilie_mai_iunie_iulie_august_septembrie_octombrie_noiembrie_decembrie".split("_"),monthsShort:"ian._feb._mart._apr._mai_iun._iul._aug._sept._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"duminică_luni_marți_miercuri_joi_vineri_sâmbătă".split("_"),weekdaysShort:"Dum_Lun_Mar_Mie_Joi_Vin_Sâm".split("_"),weekdaysMin:"Du_Lu_Ma_Mi_Jo_Vi_Sâ".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[azi la] LT",nextDay:"[mâine la] LT",nextWeek:"dddd [la] LT",lastDay:"[ieri la] LT",lastWeek:"[fosta] dddd [la] LT",sameElse:"L"},relativeTime:{future:"peste %s",past:"%s în urmă",s:"câteva secunde",ss:t,m:"un minut",mm:t,h:"o oră",hh:t,d:"o zi",dd:t,w:"o săptămână",ww:t,M:"o lună",MM:t,y:"un an",yy:t},week:{dow:1,doy:7}});return a}))},9797:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("cy",{months:"Ionawr_Chwefror_Mawrth_Ebrill_Mai_Mehefin_Gorffennaf_Awst_Medi_Hydref_Tachwedd_Rhagfyr".split("_"),monthsShort:"Ion_Chwe_Maw_Ebr_Mai_Meh_Gor_Aws_Med_Hyd_Tach_Rhag".split("_"),weekdays:"Dydd Sul_Dydd Llun_Dydd Mawrth_Dydd Mercher_Dydd Iau_Dydd Gwener_Dydd Sadwrn".split("_"),weekdaysShort:"Sul_Llun_Maw_Mer_Iau_Gwe_Sad".split("_"),weekdaysMin:"Su_Ll_Ma_Me_Ia_Gw_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Heddiw am] LT",nextDay:"[Yfory am] LT",nextWeek:"dddd [am] LT",lastDay:"[Ddoe am] LT",lastWeek:"dddd [diwethaf am] LT",sameElse:"L"},relativeTime:{future:"mewn %s",past:"%s yn ôl",s:"ychydig eiliadau",ss:"%d eiliad",m:"munud",mm:"%d munud",h:"awr",hh:"%d awr",d:"diwrnod",dd:"%d diwrnod",M:"mis",MM:"%d mis",y:"blwyddyn",yy:"%d flynedd"},dayOfMonthOrdinalParse:/\d{1,2}(fed|ain|af|il|ydd|ed|eg)/,ordinal:function(e){var t=e,a="",n=["","af","il","ydd","ydd","ed","ed","ed","fed","fed","fed","eg","fed","eg","eg","fed","eg","eg","fed","eg","fed"];return t>20?a=40===t||50===t||60===t||80===t||100===t?"fed":"ain":t>0&&(a=n[t]),e+a},week:{dow:1,doy:4}});return t}))},"9bf2":function(e,t,a){var n=a("83ab"),r=a("0cfb"),i=a("aed9"),o=a("825a"),s=a("a04b"),l=TypeError,c=Object.defineProperty,u=Object.getOwnPropertyDescriptor,d="enumerable",h="configurable",f="writable";t.f=n?i?function(e,t,a){if(o(e),t=s(t),o(a),"function"===typeof e&&"prototype"===t&&"value"in a&&f in a&&!a[f]){var n=u(e,t);n&&n[f]&&(e[t]=a.value,a={configurable:h in a?a[h]:n[h],enumerable:d in a?a[d]:n[d],writable:!1})}return c(e,t,a)}:c:function(e,t,a){if(o(e),t=s(t),o(a),r)try{return c(e,t,a)}catch(n){}if("get"in a||"set"in a)throw l("Accessors not supported");return"value"in a&&(e[t]=a.value),e}},"9f26":function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=/^(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,a=/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?)/i,n=/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?|janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,r=[/^janv/i,/^févr/i,/^mars/i,/^avr/i,/^mai/i,/^juin/i,/^juil/i,/^août/i,/^sept/i,/^oct/i,/^nov/i,/^déc/i],i=e.defineLocale("fr",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:t,monthsShortStrictRegex:a,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",w:"une semaine",ww:"%d semaines",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|)/,ordinal:function(e,t){switch(t){case"D":return e+(1===e?"er":"");default:case"M":case"Q":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}},week:{dow:1,doy:4}});return i}))},a026:function(e,t,a){"use strict";a.r(t),function(e){
/*!
 * Vue.js v2.6.14
 * (c) 2014-2021 Evan You
 * Released under the MIT License.
 */
var a=Object.freeze({});function n(e){return void 0===e||null===e}function r(e){return void 0!==e&&null!==e}function i(e){return!0===e}function o(e){return!1===e}function s(e){return"string"===typeof e||"number"===typeof e||"symbol"===typeof e||"boolean"===typeof e}function l(e){return null!==e&&"object"===typeof e}var c=Object.prototype.toString;function u(e){return"[object Object]"===c.call(e)}function d(e){return"[object RegExp]"===c.call(e)}function h(e){var t=parseFloat(String(e));return t>=0&&Math.floor(t)===t&&isFinite(e)}function f(e){return r(e)&&"function"===typeof e.then&&"function"===typeof e.catch}function p(e){return null==e?"":Array.isArray(e)||u(e)&&e.toString===c?JSON.stringify(e,null,2):String(e)}function m(e){var t=parseFloat(e);return isNaN(t)?e:t}function v(e,t){for(var a=Object.create(null),n=e.split(","),r=0;r<n.length;r++)a[n[r]]=!0;return t?function(e){return a[e.toLowerCase()]}:function(e){return a[e]}}var _=v("slot,component",!0),g=v("key,ref,slot,slot-scope,is");function y(e,t){if(e.length){var a=e.indexOf(t);if(a>-1)return e.splice(a,1)}}var b=Object.prototype.hasOwnProperty;function M(e,t){return b.call(e,t)}function z(e){var t=Object.create(null);return function(a){var n=t[a];return n||(t[a]=e(a))}}var L=/-(\w)/g,w=z((function(e){return e.replace(L,(function(e,t){return t?t.toUpperCase():""}))})),H=z((function(e){return e.charAt(0).toUpperCase()+e.slice(1)})),k=/\B([A-Z])/g,A=z((function(e){return e.replace(k,"-$1").toLowerCase()}));function S(e,t){function a(a){var n=arguments.length;return n?n>1?e.apply(t,arguments):e.call(t,a):e.call(t)}return a._length=e.length,a}function V(e,t){return e.bind(t)}var O=Function.prototype.bind?V:S;function T(e,t){t=t||0;var a=e.length-t,n=new Array(a);while(a--)n[a]=e[a+t];return n}function D(e,t){for(var a in t)e[a]=t[a];return e}function C(e){for(var t={},a=0;a<e.length;a++)e[a]&&D(t,e[a]);return t}function j(e,t,a){}var x=function(e,t,a){return!1},B=function(e){return e};function Y(e){return e.reduce((function(e,t){return e.concat(t.staticKeys||[])}),[]).join(",")}function P(e,t){if(e===t)return!0;var a=l(e),n=l(t);if(!a||!n)return!a&&!n&&String(e)===String(t);try{var r=Array.isArray(e),i=Array.isArray(t);if(r&&i)return e.length===t.length&&e.every((function(e,a){return P(e,t[a])}));if(e instanceof Date&&t instanceof Date)return e.getTime()===t.getTime();if(r||i)return!1;var o=Object.keys(e),s=Object.keys(t);return o.length===s.length&&o.every((function(a){return P(e[a],t[a])}))}catch(c){return!1}}function I(e,t){for(var a=0;a<e.length;a++)if(P(e[a],t))return a;return-1}function F(e){var t=!1;return function(){t||(t=!0,e.apply(this,arguments))}}var E="data-server-rendered",$=["component","directive","filter"],R=["beforeCreate","created","beforeMount","mounted","beforeUpdate","updated","beforeDestroy","destroyed","activated","deactivated","errorCaptured","serverPrefetch"],N={optionMergeStrategies:Object.create(null),silent:!1,productionTip:!1,devtools:!1,performance:!1,errorHandler:null,warnHandler:null,ignoredElements:[],keyCodes:Object.create(null),isReservedTag:x,isReservedAttr:x,isUnknownElement:x,getTagNamespace:j,parsePlatformTagName:B,mustUseProp:x,async:!0,_lifecycleHooks:R},W=/a-zA-Z\u00B7\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u037D\u037F-\u1FFF\u200C-\u200D\u203F-\u2040\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD/;function U(e){var t=(e+"").charCodeAt(0);return 36===t||95===t}function G(e,t,a,n){Object.defineProperty(e,t,{value:a,enumerable:!!n,writable:!0,configurable:!0})}var q=new RegExp("[^"+W.source+".$_\\d]");function J(e){if(!q.test(e)){var t=e.split(".");return function(e){for(var a=0;a<t.length;a++){if(!e)return;e=e[t[a]]}return e}}}var K,X="__proto__"in{},Z="undefined"!==typeof window,Q="undefined"!==typeof WXEnvironment&&!!WXEnvironment.platform,ee=Q&&WXEnvironment.platform.toLowerCase(),te=Z&&window.navigator.userAgent.toLowerCase(),ae=te&&/msie|trident/.test(te),ne=te&&te.indexOf("msie 9.0")>0,re=te&&te.indexOf("edge/")>0,ie=(te&&te.indexOf("android"),te&&/iphone|ipad|ipod|ios/.test(te)||"ios"===ee),oe=(te&&/chrome\/\d+/.test(te),te&&/phantomjs/.test(te),te&&te.match(/firefox\/(\d+)/)),se={}.watch,le=!1;if(Z)try{var ce={};Object.defineProperty(ce,"passive",{get:function(){le=!0}}),window.addEventListener("test-passive",null,ce)}catch(Zc){}var ue=function(){return void 0===K&&(K=!Z&&!Q&&"undefined"!==typeof e&&(e["process"]&&"server"===e["process"].env.VUE_ENV)),K},de=Z&&window.__VUE_DEVTOOLS_GLOBAL_HOOK__;function he(e){return"function"===typeof e&&/native code/.test(e.toString())}var fe,pe="undefined"!==typeof Symbol&&he(Symbol)&&"undefined"!==typeof Reflect&&he(Reflect.ownKeys);fe="undefined"!==typeof Set&&he(Set)?Set:function(){function e(){this.set=Object.create(null)}return e.prototype.has=function(e){return!0===this.set[e]},e.prototype.add=function(e){this.set[e]=!0},e.prototype.clear=function(){this.set=Object.create(null)},e}();var me=j,ve=0,_e=function(){this.id=ve++,this.subs=[]};_e.prototype.addSub=function(e){this.subs.push(e)},_e.prototype.removeSub=function(e){y(this.subs,e)},_e.prototype.depend=function(){_e.target&&_e.target.addDep(this)},_e.prototype.notify=function(){var e=this.subs.slice();for(var t=0,a=e.length;t<a;t++)e[t].update()},_e.target=null;var ge=[];function ye(e){ge.push(e),_e.target=e}function be(){ge.pop(),_e.target=ge[ge.length-1]}var Me=function(e,t,a,n,r,i,o,s){this.tag=e,this.data=t,this.children=a,this.text=n,this.elm=r,this.ns=void 0,this.context=i,this.fnContext=void 0,this.fnOptions=void 0,this.fnScopeId=void 0,this.key=t&&t.key,this.componentOptions=o,this.componentInstance=void 0,this.parent=void 0,this.raw=!1,this.isStatic=!1,this.isRootInsert=!0,this.isComment=!1,this.isCloned=!1,this.isOnce=!1,this.asyncFactory=s,this.asyncMeta=void 0,this.isAsyncPlaceholder=!1},ze={child:{configurable:!0}};ze.child.get=function(){return this.componentInstance},Object.defineProperties(Me.prototype,ze);var Le=function(e){void 0===e&&(e="");var t=new Me;return t.text=e,t.isComment=!0,t};function we(e){return new Me(void 0,void 0,void 0,String(e))}function He(e){var t=new Me(e.tag,e.data,e.children&&e.children.slice(),e.text,e.elm,e.context,e.componentOptions,e.asyncFactory);return t.ns=e.ns,t.isStatic=e.isStatic,t.key=e.key,t.isComment=e.isComment,t.fnContext=e.fnContext,t.fnOptions=e.fnOptions,t.fnScopeId=e.fnScopeId,t.asyncMeta=e.asyncMeta,t.isCloned=!0,t}var ke=Array.prototype,Ae=Object.create(ke),Se=["push","pop","shift","unshift","splice","sort","reverse"];Se.forEach((function(e){var t=ke[e];G(Ae,e,(function(){var a=[],n=arguments.length;while(n--)a[n]=arguments[n];var r,i=t.apply(this,a),o=this.__ob__;switch(e){case"push":case"unshift":r=a;break;case"splice":r=a.slice(2);break}return r&&o.observeArray(r),o.dep.notify(),i}))}));var Ve=Object.getOwnPropertyNames(Ae),Oe=!0;function Te(e){Oe=e}var De=function(e){this.value=e,this.dep=new _e,this.vmCount=0,G(e,"__ob__",this),Array.isArray(e)?(X?Ce(e,Ae):je(e,Ae,Ve),this.observeArray(e)):this.walk(e)};function Ce(e,t){e.__proto__=t}function je(e,t,a){for(var n=0,r=a.length;n<r;n++){var i=a[n];G(e,i,t[i])}}function xe(e,t){var a;if(l(e)&&!(e instanceof Me))return M(e,"__ob__")&&e.__ob__ instanceof De?a=e.__ob__:Oe&&!ue()&&(Array.isArray(e)||u(e))&&Object.isExtensible(e)&&!e._isVue&&(a=new De(e)),t&&a&&a.vmCount++,a}function Be(e,t,a,n,r){var i=new _e,o=Object.getOwnPropertyDescriptor(e,t);if(!o||!1!==o.configurable){var s=o&&o.get,l=o&&o.set;s&&!l||2!==arguments.length||(a=e[t]);var c=!r&&xe(a);Object.defineProperty(e,t,{enumerable:!0,configurable:!0,get:function(){var t=s?s.call(e):a;return _e.target&&(i.depend(),c&&(c.dep.depend(),Array.isArray(t)&&Ie(t))),t},set:function(t){var n=s?s.call(e):a;t===n||t!==t&&n!==n||s&&!l||(l?l.call(e,t):a=t,c=!r&&xe(t),i.notify())}})}}function Ye(e,t,a){if(Array.isArray(e)&&h(t))return e.length=Math.max(e.length,t),e.splice(t,1,a),a;if(t in e&&!(t in Object.prototype))return e[t]=a,a;var n=e.__ob__;return e._isVue||n&&n.vmCount?a:n?(Be(n.value,t,a),n.dep.notify(),a):(e[t]=a,a)}function Pe(e,t){if(Array.isArray(e)&&h(t))e.splice(t,1);else{var a=e.__ob__;e._isVue||a&&a.vmCount||M(e,t)&&(delete e[t],a&&a.dep.notify())}}function Ie(e){for(var t=void 0,a=0,n=e.length;a<n;a++)t=e[a],t&&t.__ob__&&t.__ob__.dep.depend(),Array.isArray(t)&&Ie(t)}De.prototype.walk=function(e){for(var t=Object.keys(e),a=0;a<t.length;a++)Be(e,t[a])},De.prototype.observeArray=function(e){for(var t=0,a=e.length;t<a;t++)xe(e[t])};var Fe=N.optionMergeStrategies;function Ee(e,t){if(!t)return e;for(var a,n,r,i=pe?Reflect.ownKeys(t):Object.keys(t),o=0;o<i.length;o++)a=i[o],"__ob__"!==a&&(n=e[a],r=t[a],M(e,a)?n!==r&&u(n)&&u(r)&&Ee(n,r):Ye(e,a,r));return e}function $e(e,t,a){return a?function(){var n="function"===typeof t?t.call(a,a):t,r="function"===typeof e?e.call(a,a):e;return n?Ee(n,r):r}:t?e?function(){return Ee("function"===typeof t?t.call(this,this):t,"function"===typeof e?e.call(this,this):e)}:t:e}function Re(e,t){var a=t?e?e.concat(t):Array.isArray(t)?t:[t]:e;return a?Ne(a):a}function Ne(e){for(var t=[],a=0;a<e.length;a++)-1===t.indexOf(e[a])&&t.push(e[a]);return t}function We(e,t,a,n){var r=Object.create(e||null);return t?D(r,t):r}Fe.data=function(e,t,a){return a?$e(e,t,a):t&&"function"!==typeof t?e:$e(e,t)},R.forEach((function(e){Fe[e]=Re})),$.forEach((function(e){Fe[e+"s"]=We})),Fe.watch=function(e,t,a,n){if(e===se&&(e=void 0),t===se&&(t=void 0),!t)return Object.create(e||null);if(!e)return t;var r={};for(var i in D(r,e),t){var o=r[i],s=t[i];o&&!Array.isArray(o)&&(o=[o]),r[i]=o?o.concat(s):Array.isArray(s)?s:[s]}return r},Fe.props=Fe.methods=Fe.inject=Fe.computed=function(e,t,a,n){if(!e)return t;var r=Object.create(null);return D(r,e),t&&D(r,t),r},Fe.provide=$e;var Ue=function(e,t){return void 0===t?e:t};function Ge(e,t){var a=e.props;if(a){var n,r,i,o={};if(Array.isArray(a)){n=a.length;while(n--)r=a[n],"string"===typeof r&&(i=w(r),o[i]={type:null})}else if(u(a))for(var s in a)r=a[s],i=w(s),o[i]=u(r)?r:{type:r};else 0;e.props=o}}function qe(e,t){var a=e.inject;if(a){var n=e.inject={};if(Array.isArray(a))for(var r=0;r<a.length;r++)n[a[r]]={from:a[r]};else if(u(a))for(var i in a){var o=a[i];n[i]=u(o)?D({from:i},o):{from:o}}else 0}}function Je(e){var t=e.directives;if(t)for(var a in t){var n=t[a];"function"===typeof n&&(t[a]={bind:n,update:n})}}function Ke(e,t,a){if("function"===typeof t&&(t=t.options),Ge(t,a),qe(t,a),Je(t),!t._base&&(t.extends&&(e=Ke(e,t.extends,a)),t.mixins))for(var n=0,r=t.mixins.length;n<r;n++)e=Ke(e,t.mixins[n],a);var i,o={};for(i in e)s(i);for(i in t)M(e,i)||s(i);function s(n){var r=Fe[n]||Ue;o[n]=r(e[n],t[n],a,n)}return o}function Xe(e,t,a,n){if("string"===typeof a){var r=e[t];if(M(r,a))return r[a];var i=w(a);if(M(r,i))return r[i];var o=H(i);if(M(r,o))return r[o];var s=r[a]||r[i]||r[o];return s}}function Ze(e,t,a,n){var r=t[e],i=!M(a,e),o=a[e],s=nt(Boolean,r.type);if(s>-1)if(i&&!M(r,"default"))o=!1;else if(""===o||o===A(e)){var l=nt(String,r.type);(l<0||s<l)&&(o=!0)}if(void 0===o){o=Qe(n,r,e);var c=Oe;Te(!0),xe(o),Te(c)}return o}function Qe(e,t,a){if(M(t,"default")){var n=t.default;return e&&e.$options.propsData&&void 0===e.$options.propsData[a]&&void 0!==e._props[a]?e._props[a]:"function"===typeof n&&"Function"!==tt(t.type)?n.call(e):n}}var et=/^\s*function (\w+)/;function tt(e){var t=e&&e.toString().match(et);return t?t[1]:""}function at(e,t){return tt(e)===tt(t)}function nt(e,t){if(!Array.isArray(t))return at(t,e)?0:-1;for(var a=0,n=t.length;a<n;a++)if(at(t[a],e))return a;return-1}function rt(e,t,a){ye();try{if(t){var n=t;while(n=n.$parent){var r=n.$options.errorCaptured;if(r)for(var i=0;i<r.length;i++)try{var o=!1===r[i].call(n,e,t,a);if(o)return}catch(Zc){ot(Zc,n,"errorCaptured hook")}}}ot(e,t,a)}finally{be()}}function it(e,t,a,n,r){var i;try{i=a?e.apply(t,a):e.call(t),i&&!i._isVue&&f(i)&&!i._handled&&(i.catch((function(e){return rt(e,n,r+" (Promise/async)")})),i._handled=!0)}catch(Zc){rt(Zc,n,r)}return i}function ot(e,t,a){if(N.errorHandler)try{return N.errorHandler.call(null,e,t,a)}catch(Zc){Zc!==e&&st(Zc,null,"config.errorHandler")}st(e,t,a)}function st(e,t,a){if(!Z&&!Q||"undefined"===typeof console)throw e;console.error(e)}var lt,ct=!1,ut=[],dt=!1;function ht(){dt=!1;var e=ut.slice(0);ut.length=0;for(var t=0;t<e.length;t++)e[t]()}if("undefined"!==typeof Promise&&he(Promise)){var ft=Promise.resolve();lt=function(){ft.then(ht),ie&&setTimeout(j)},ct=!0}else if(ae||"undefined"===typeof MutationObserver||!he(MutationObserver)&&"[object MutationObserverConstructor]"!==MutationObserver.toString())lt="undefined"!==typeof setImmediate&&he(setImmediate)?function(){setImmediate(ht)}:function(){setTimeout(ht,0)};else{var pt=1,mt=new MutationObserver(ht),vt=document.createTextNode(String(pt));mt.observe(vt,{characterData:!0}),lt=function(){pt=(pt+1)%2,vt.data=String(pt)},ct=!0}function _t(e,t){var a;if(ut.push((function(){if(e)try{e.call(t)}catch(Zc){rt(Zc,t,"nextTick")}else a&&a(t)})),dt||(dt=!0,lt()),!e&&"undefined"!==typeof Promise)return new Promise((function(e){a=e}))}var gt=new fe;function yt(e){bt(e,gt),gt.clear()}function bt(e,t){var a,n,r=Array.isArray(e);if(!(!r&&!l(e)||Object.isFrozen(e)||e instanceof Me)){if(e.__ob__){var i=e.__ob__.dep.id;if(t.has(i))return;t.add(i)}if(r){a=e.length;while(a--)bt(e[a],t)}else{n=Object.keys(e),a=n.length;while(a--)bt(e[n[a]],t)}}}var Mt=z((function(e){var t="&"===e.charAt(0);e=t?e.slice(1):e;var a="~"===e.charAt(0);e=a?e.slice(1):e;var n="!"===e.charAt(0);return e=n?e.slice(1):e,{name:e,once:a,capture:n,passive:t}}));function zt(e,t){function a(){var e=arguments,n=a.fns;if(!Array.isArray(n))return it(n,null,arguments,t,"v-on handler");for(var r=n.slice(),i=0;i<r.length;i++)it(r[i],null,e,t,"v-on handler")}return a.fns=e,a}function Lt(e,t,a,r,o,s){var l,c,u,d;for(l in e)c=e[l],u=t[l],d=Mt(l),n(c)||(n(u)?(n(c.fns)&&(c=e[l]=zt(c,s)),i(d.once)&&(c=e[l]=o(d.name,c,d.capture)),a(d.name,c,d.capture,d.passive,d.params)):c!==u&&(u.fns=c,e[l]=u));for(l in t)n(e[l])&&(d=Mt(l),r(d.name,t[l],d.capture))}function wt(e,t,a){var o;e instanceof Me&&(e=e.data.hook||(e.data.hook={}));var s=e[t];function l(){a.apply(this,arguments),y(o.fns,l)}n(s)?o=zt([l]):r(s.fns)&&i(s.merged)?(o=s,o.fns.push(l)):o=zt([s,l]),o.merged=!0,e[t]=o}function Ht(e,t,a){var i=t.options.props;if(!n(i)){var o={},s=e.attrs,l=e.props;if(r(s)||r(l))for(var c in i){var u=A(c);kt(o,l,c,u,!0)||kt(o,s,c,u,!1)}return o}}function kt(e,t,a,n,i){if(r(t)){if(M(t,a))return e[a]=t[a],i||delete t[a],!0;if(M(t,n))return e[a]=t[n],i||delete t[n],!0}return!1}function At(e){for(var t=0;t<e.length;t++)if(Array.isArray(e[t]))return Array.prototype.concat.apply([],e);return e}function St(e){return s(e)?[we(e)]:Array.isArray(e)?Ot(e):void 0}function Vt(e){return r(e)&&r(e.text)&&o(e.isComment)}function Ot(e,t){var a,o,l,c,u=[];for(a=0;a<e.length;a++)o=e[a],n(o)||"boolean"===typeof o||(l=u.length-1,c=u[l],Array.isArray(o)?o.length>0&&(o=Ot(o,(t||"")+"_"+a),Vt(o[0])&&Vt(c)&&(u[l]=we(c.text+o[0].text),o.shift()),u.push.apply(u,o)):s(o)?Vt(c)?u[l]=we(c.text+o):""!==o&&u.push(we(o)):Vt(o)&&Vt(c)?u[l]=we(c.text+o.text):(i(e._isVList)&&r(o.tag)&&n(o.key)&&r(t)&&(o.key="__vlist"+t+"_"+a+"__"),u.push(o)));return u}function Tt(e){var t=e.$options.provide;t&&(e._provided="function"===typeof t?t.call(e):t)}function Dt(e){var t=Ct(e.$options.inject,e);t&&(Te(!1),Object.keys(t).forEach((function(a){Be(e,a,t[a])})),Te(!0))}function Ct(e,t){if(e){for(var a=Object.create(null),n=pe?Reflect.ownKeys(e):Object.keys(e),r=0;r<n.length;r++){var i=n[r];if("__ob__"!==i){var o=e[i].from,s=t;while(s){if(s._provided&&M(s._provided,o)){a[i]=s._provided[o];break}s=s.$parent}if(!s)if("default"in e[i]){var l=e[i].default;a[i]="function"===typeof l?l.call(t):l}else 0}}return a}}function jt(e,t){if(!e||!e.length)return{};for(var a={},n=0,r=e.length;n<r;n++){var i=e[n],o=i.data;if(o&&o.attrs&&o.attrs.slot&&delete o.attrs.slot,i.context!==t&&i.fnContext!==t||!o||null==o.slot)(a.default||(a.default=[])).push(i);else{var s=o.slot,l=a[s]||(a[s]=[]);"template"===i.tag?l.push.apply(l,i.children||[]):l.push(i)}}for(var c in a)a[c].every(xt)&&delete a[c];return a}function xt(e){return e.isComment&&!e.asyncFactory||" "===e.text}function Bt(e){return e.isComment&&e.asyncFactory}function Yt(e,t,n){var r,i=Object.keys(t).length>0,o=e?!!e.$stable:!i,s=e&&e.$key;if(e){if(e._normalized)return e._normalized;if(o&&n&&n!==a&&s===n.$key&&!i&&!n.$hasNormal)return n;for(var l in r={},e)e[l]&&"$"!==l[0]&&(r[l]=Pt(t,l,e[l]))}else r={};for(var c in t)c in r||(r[c]=It(t,c));return e&&Object.isExtensible(e)&&(e._normalized=r),G(r,"$stable",o),G(r,"$key",s),G(r,"$hasNormal",i),r}function Pt(e,t,a){var n=function(){var e=arguments.length?a.apply(null,arguments):a({});e=e&&"object"===typeof e&&!Array.isArray(e)?[e]:St(e);var t=e&&e[0];return e&&(!t||1===e.length&&t.isComment&&!Bt(t))?void 0:e};return a.proxy&&Object.defineProperty(e,t,{get:n,enumerable:!0,configurable:!0}),n}function It(e,t){return function(){return e[t]}}function Ft(e,t){var a,n,i,o,s;if(Array.isArray(e)||"string"===typeof e)for(a=new Array(e.length),n=0,i=e.length;n<i;n++)a[n]=t(e[n],n);else if("number"===typeof e)for(a=new Array(e),n=0;n<e;n++)a[n]=t(n+1,n);else if(l(e))if(pe&&e[Symbol.iterator]){a=[];var c=e[Symbol.iterator](),u=c.next();while(!u.done)a.push(t(u.value,a.length)),u=c.next()}else for(o=Object.keys(e),a=new Array(o.length),n=0,i=o.length;n<i;n++)s=o[n],a[n]=t(e[s],s,n);return r(a)||(a=[]),a._isVList=!0,a}function Et(e,t,a,n){var r,i=this.$scopedSlots[e];i?(a=a||{},n&&(a=D(D({},n),a)),r=i(a)||("function"===typeof t?t():t)):r=this.$slots[e]||("function"===typeof t?t():t);var o=a&&a.slot;return o?this.$createElement("template",{slot:o},r):r}function $t(e){return Xe(this.$options,"filters",e,!0)||B}function Rt(e,t){return Array.isArray(e)?-1===e.indexOf(t):e!==t}function Nt(e,t,a,n,r){var i=N.keyCodes[t]||a;return r&&n&&!N.keyCodes[t]?Rt(r,n):i?Rt(i,e):n?A(n)!==t:void 0===e}function Wt(e,t,a,n,r){if(a)if(l(a)){var i;Array.isArray(a)&&(a=C(a));var o=function(o){if("class"===o||"style"===o||g(o))i=e;else{var s=e.attrs&&e.attrs.type;i=n||N.mustUseProp(t,s,o)?e.domProps||(e.domProps={}):e.attrs||(e.attrs={})}var l=w(o),c=A(o);if(!(l in i)&&!(c in i)&&(i[o]=a[o],r)){var u=e.on||(e.on={});u["update:"+o]=function(e){a[o]=e}}};for(var s in a)o(s)}else;return e}function Ut(e,t){var a=this._staticTrees||(this._staticTrees=[]),n=a[e];return n&&!t||(n=a[e]=this.$options.staticRenderFns[e].call(this._renderProxy,null,this),qt(n,"__static__"+e,!1)),n}function Gt(e,t,a){return qt(e,"__once__"+t+(a?"_"+a:""),!0),e}function qt(e,t,a){if(Array.isArray(e))for(var n=0;n<e.length;n++)e[n]&&"string"!==typeof e[n]&&Jt(e[n],t+"_"+n,a);else Jt(e,t,a)}function Jt(e,t,a){e.isStatic=!0,e.key=t,e.isOnce=a}function Kt(e,t){if(t)if(u(t)){var a=e.on=e.on?D({},e.on):{};for(var n in t){var r=a[n],i=t[n];a[n]=r?[].concat(r,i):i}}else;return e}function Xt(e,t,a,n){t=t||{$stable:!a};for(var r=0;r<e.length;r++){var i=e[r];Array.isArray(i)?Xt(i,t,a):i&&(i.proxy&&(i.fn.proxy=!0),t[i.key]=i.fn)}return n&&(t.$key=n),t}function Zt(e,t){for(var a=0;a<t.length;a+=2){var n=t[a];"string"===typeof n&&n&&(e[t[a]]=t[a+1])}return e}function Qt(e,t){return"string"===typeof e?t+e:e}function ea(e){e._o=Gt,e._n=m,e._s=p,e._l=Ft,e._t=Et,e._q=P,e._i=I,e._m=Ut,e._f=$t,e._k=Nt,e._b=Wt,e._v=we,e._e=Le,e._u=Xt,e._g=Kt,e._d=Zt,e._p=Qt}function ta(e,t,n,r,o){var s,l=this,c=o.options;M(r,"_uid")?(s=Object.create(r),s._original=r):(s=r,r=r._original);var u=i(c._compiled),d=!u;this.data=e,this.props=t,this.children=n,this.parent=r,this.listeners=e.on||a,this.injections=Ct(c.inject,r),this.slots=function(){return l.$slots||Yt(e.scopedSlots,l.$slots=jt(n,r)),l.$slots},Object.defineProperty(this,"scopedSlots",{enumerable:!0,get:function(){return Yt(e.scopedSlots,this.slots())}}),u&&(this.$options=c,this.$slots=this.slots(),this.$scopedSlots=Yt(e.scopedSlots,this.$slots)),c._scopeId?this._c=function(e,t,a,n){var i=pa(s,e,t,a,n,d);return i&&!Array.isArray(i)&&(i.fnScopeId=c._scopeId,i.fnContext=r),i}:this._c=function(e,t,a,n){return pa(s,e,t,a,n,d)}}function aa(e,t,n,i,o){var s=e.options,l={},c=s.props;if(r(c))for(var u in c)l[u]=Ze(u,c,t||a);else r(n.attrs)&&ra(l,n.attrs),r(n.props)&&ra(l,n.props);var d=new ta(n,l,o,i,e),h=s.render.call(null,d._c,d);if(h instanceof Me)return na(h,n,d.parent,s,d);if(Array.isArray(h)){for(var f=St(h)||[],p=new Array(f.length),m=0;m<f.length;m++)p[m]=na(f[m],n,d.parent,s,d);return p}}function na(e,t,a,n,r){var i=He(e);return i.fnContext=a,i.fnOptions=n,t.slot&&((i.data||(i.data={})).slot=t.slot),i}function ra(e,t){for(var a in t)e[w(a)]=t[a]}ea(ta.prototype);var ia={init:function(e,t){if(e.componentInstance&&!e.componentInstance._isDestroyed&&e.data.keepAlive){var a=e;ia.prepatch(a,a)}else{var n=e.componentInstance=la(e,Da);n.$mount(t?e.elm:void 0,t)}},prepatch:function(e,t){var a=t.componentOptions,n=t.componentInstance=e.componentInstance;Ya(n,a.propsData,a.listeners,t,a.children)},insert:function(e){var t=e.context,a=e.componentInstance;a._isMounted||(a._isMounted=!0,Ea(a,"mounted")),e.data.keepAlive&&(t._isMounted?en(a):Ia(a,!0))},destroy:function(e){var t=e.componentInstance;t._isDestroyed||(e.data.keepAlive?Fa(t,!0):t.$destroy())}},oa=Object.keys(ia);function sa(e,t,a,o,s){if(!n(e)){var c=a.$options._base;if(l(e)&&(e=c.extend(e)),"function"===typeof e){var u;if(n(e.cid)&&(u=e,e=wa(u,c),void 0===e))return La(u,t,a,o,s);t=t||{},wn(e),r(t.model)&&da(e.options,t);var d=Ht(t,e,s);if(i(e.options.functional))return aa(e,d,t,a,o);var h=t.on;if(t.on=t.nativeOn,i(e.options.abstract)){var f=t.slot;t={},f&&(t.slot=f)}ca(t);var p=e.options.name||s,m=new Me("vue-component-"+e.cid+(p?"-"+p:""),t,void 0,void 0,void 0,a,{Ctor:e,propsData:d,listeners:h,tag:s,children:o},u);return m}}}function la(e,t){var a={_isComponent:!0,_parentVnode:e,parent:t},n=e.data.inlineTemplate;return r(n)&&(a.render=n.render,a.staticRenderFns=n.staticRenderFns),new e.componentOptions.Ctor(a)}function ca(e){for(var t=e.hook||(e.hook={}),a=0;a<oa.length;a++){var n=oa[a],r=t[n],i=ia[n];r===i||r&&r._merged||(t[n]=r?ua(i,r):i)}}function ua(e,t){var a=function(a,n){e(a,n),t(a,n)};return a._merged=!0,a}function da(e,t){var a=e.model&&e.model.prop||"value",n=e.model&&e.model.event||"input";(t.attrs||(t.attrs={}))[a]=t.model.value;var i=t.on||(t.on={}),o=i[n],s=t.model.callback;r(o)?(Array.isArray(o)?-1===o.indexOf(s):o!==s)&&(i[n]=[s].concat(o)):i[n]=s}var ha=1,fa=2;function pa(e,t,a,n,r,o){return(Array.isArray(a)||s(a))&&(r=n,n=a,a=void 0),i(o)&&(r=fa),ma(e,t,a,n,r)}function ma(e,t,a,n,i){if(r(a)&&r(a.__ob__))return Le();if(r(a)&&r(a.is)&&(t=a.is),!t)return Le();var o,s,l;(Array.isArray(n)&&"function"===typeof n[0]&&(a=a||{},a.scopedSlots={default:n[0]},n.length=0),i===fa?n=St(n):i===ha&&(n=At(n)),"string"===typeof t)?(s=e.$vnode&&e.$vnode.ns||N.getTagNamespace(t),o=N.isReservedTag(t)?new Me(N.parsePlatformTagName(t),a,n,void 0,void 0,e):a&&a.pre||!r(l=Xe(e.$options,"components",t))?new Me(t,a,n,void 0,void 0,e):sa(l,a,e,n,t)):o=sa(t,a,e,n);return Array.isArray(o)?o:r(o)?(r(s)&&va(o,s),r(a)&&_a(a),o):Le()}function va(e,t,a){if(e.ns=t,"foreignObject"===e.tag&&(t=void 0,a=!0),r(e.children))for(var o=0,s=e.children.length;o<s;o++){var l=e.children[o];r(l.tag)&&(n(l.ns)||i(a)&&"svg"!==l.tag)&&va(l,t,a)}}function _a(e){l(e.style)&&yt(e.style),l(e.class)&&yt(e.class)}function ga(e){e._vnode=null,e._staticTrees=null;var t=e.$options,n=e.$vnode=t._parentVnode,r=n&&n.context;e.$slots=jt(t._renderChildren,r),e.$scopedSlots=a,e._c=function(t,a,n,r){return pa(e,t,a,n,r,!1)},e.$createElement=function(t,a,n,r){return pa(e,t,a,n,r,!0)};var i=n&&n.data;Be(e,"$attrs",i&&i.attrs||a,null,!0),Be(e,"$listeners",t._parentListeners||a,null,!0)}var ya,ba=null;function Ma(e){ea(e.prototype),e.prototype.$nextTick=function(e){return _t(e,this)},e.prototype._render=function(){var e,t=this,a=t.$options,n=a.render,r=a._parentVnode;r&&(t.$scopedSlots=Yt(r.data.scopedSlots,t.$slots,t.$scopedSlots)),t.$vnode=r;try{ba=t,e=n.call(t._renderProxy,t.$createElement)}catch(Zc){rt(Zc,t,"render"),e=t._vnode}finally{ba=null}return Array.isArray(e)&&1===e.length&&(e=e[0]),e instanceof Me||(e=Le()),e.parent=r,e}}function za(e,t){return(e.__esModule||pe&&"Module"===e[Symbol.toStringTag])&&(e=e.default),l(e)?t.extend(e):e}function La(e,t,a,n,r){var i=Le();return i.asyncFactory=e,i.asyncMeta={data:t,context:a,children:n,tag:r},i}function wa(e,t){if(i(e.error)&&r(e.errorComp))return e.errorComp;if(r(e.resolved))return e.resolved;var a=ba;if(a&&r(e.owners)&&-1===e.owners.indexOf(a)&&e.owners.push(a),i(e.loading)&&r(e.loadingComp))return e.loadingComp;if(a&&!r(e.owners)){var o=e.owners=[a],s=!0,c=null,u=null;a.$on("hook:destroyed",(function(){return y(o,a)}));var d=function(e){for(var t=0,a=o.length;t<a;t++)o[t].$forceUpdate();e&&(o.length=0,null!==c&&(clearTimeout(c),c=null),null!==u&&(clearTimeout(u),u=null))},h=F((function(a){e.resolved=za(a,t),s?o.length=0:d(!0)})),p=F((function(t){r(e.errorComp)&&(e.error=!0,d(!0))})),m=e(h,p);return l(m)&&(f(m)?n(e.resolved)&&m.then(h,p):f(m.component)&&(m.component.then(h,p),r(m.error)&&(e.errorComp=za(m.error,t)),r(m.loading)&&(e.loadingComp=za(m.loading,t),0===m.delay?e.loading=!0:c=setTimeout((function(){c=null,n(e.resolved)&&n(e.error)&&(e.loading=!0,d(!1))}),m.delay||200)),r(m.timeout)&&(u=setTimeout((function(){u=null,n(e.resolved)&&p(null)}),m.timeout)))),s=!1,e.loading?e.loadingComp:e.resolved}}function Ha(e){if(Array.isArray(e))for(var t=0;t<e.length;t++){var a=e[t];if(r(a)&&(r(a.componentOptions)||Bt(a)))return a}}function ka(e){e._events=Object.create(null),e._hasHookEvent=!1;var t=e.$options._parentListeners;t&&Oa(e,t)}function Aa(e,t){ya.$on(e,t)}function Sa(e,t){ya.$off(e,t)}function Va(e,t){var a=ya;return function n(){var r=t.apply(null,arguments);null!==r&&a.$off(e,n)}}function Oa(e,t,a){ya=e,Lt(t,a||{},Aa,Sa,Va,e),ya=void 0}function Ta(e){var t=/^hook:/;e.prototype.$on=function(e,a){var n=this;if(Array.isArray(e))for(var r=0,i=e.length;r<i;r++)n.$on(e[r],a);else(n._events[e]||(n._events[e]=[])).push(a),t.test(e)&&(n._hasHookEvent=!0);return n},e.prototype.$once=function(e,t){var a=this;function n(){a.$off(e,n),t.apply(a,arguments)}return n.fn=t,a.$on(e,n),a},e.prototype.$off=function(e,t){var a=this;if(!arguments.length)return a._events=Object.create(null),a;if(Array.isArray(e)){for(var n=0,r=e.length;n<r;n++)a.$off(e[n],t);return a}var i,o=a._events[e];if(!o)return a;if(!t)return a._events[e]=null,a;var s=o.length;while(s--)if(i=o[s],i===t||i.fn===t){o.splice(s,1);break}return a},e.prototype.$emit=function(e){var t=this,a=t._events[e];if(a){a=a.length>1?T(a):a;for(var n=T(arguments,1),r='event handler for "'+e+'"',i=0,o=a.length;i<o;i++)it(a[i],t,n,t,r)}return t}}var Da=null;function Ca(e){var t=Da;return Da=e,function(){Da=t}}function ja(e){var t=e.$options,a=t.parent;if(a&&!t.abstract){while(a.$options.abstract&&a.$parent)a=a.$parent;a.$children.push(e)}e.$parent=a,e.$root=a?a.$root:e,e.$children=[],e.$refs={},e._watcher=null,e._inactive=null,e._directInactive=!1,e._isMounted=!1,e._isDestroyed=!1,e._isBeingDestroyed=!1}function xa(e){e.prototype._update=function(e,t){var a=this,n=a.$el,r=a._vnode,i=Ca(a);a._vnode=e,a.$el=r?a.__patch__(r,e):a.__patch__(a.$el,e,t,!1),i(),n&&(n.__vue__=null),a.$el&&(a.$el.__vue__=a),a.$vnode&&a.$parent&&a.$vnode===a.$parent._vnode&&(a.$parent.$el=a.$el)},e.prototype.$forceUpdate=function(){var e=this;e._watcher&&e._watcher.update()},e.prototype.$destroy=function(){var e=this;if(!e._isBeingDestroyed){Ea(e,"beforeDestroy"),e._isBeingDestroyed=!0;var t=e.$parent;!t||t._isBeingDestroyed||e.$options.abstract||y(t.$children,e),e._watcher&&e._watcher.teardown();var a=e._watchers.length;while(a--)e._watchers[a].teardown();e._data.__ob__&&e._data.__ob__.vmCount--,e._isDestroyed=!0,e.__patch__(e._vnode,null),Ea(e,"destroyed"),e.$off(),e.$el&&(e.$el.__vue__=null),e.$vnode&&(e.$vnode.parent=null)}}}function Ba(e,t,a){var n;return e.$el=t,e.$options.render||(e.$options.render=Le),Ea(e,"beforeMount"),n=function(){e._update(e._render(),a)},new rn(e,n,j,{before:function(){e._isMounted&&!e._isDestroyed&&Ea(e,"beforeUpdate")}},!0),a=!1,null==e.$vnode&&(e._isMounted=!0,Ea(e,"mounted")),e}function Ya(e,t,n,r,i){var o=r.data.scopedSlots,s=e.$scopedSlots,l=!!(o&&!o.$stable||s!==a&&!s.$stable||o&&e.$scopedSlots.$key!==o.$key||!o&&e.$scopedSlots.$key),c=!!(i||e.$options._renderChildren||l);if(e.$options._parentVnode=r,e.$vnode=r,e._vnode&&(e._vnode.parent=r),e.$options._renderChildren=i,e.$attrs=r.data.attrs||a,e.$listeners=n||a,t&&e.$options.props){Te(!1);for(var u=e._props,d=e.$options._propKeys||[],h=0;h<d.length;h++){var f=d[h],p=e.$options.props;u[f]=Ze(f,p,t,e)}Te(!0),e.$options.propsData=t}n=n||a;var m=e.$options._parentListeners;e.$options._parentListeners=n,Oa(e,n,m),c&&(e.$slots=jt(i,r.context),e.$forceUpdate())}function Pa(e){while(e&&(e=e.$parent))if(e._inactive)return!0;return!1}function Ia(e,t){if(t){if(e._directInactive=!1,Pa(e))return}else if(e._directInactive)return;if(e._inactive||null===e._inactive){e._inactive=!1;for(var a=0;a<e.$children.length;a++)Ia(e.$children[a]);Ea(e,"activated")}}function Fa(e,t){if((!t||(e._directInactive=!0,!Pa(e)))&&!e._inactive){e._inactive=!0;for(var a=0;a<e.$children.length;a++)Fa(e.$children[a]);Ea(e,"deactivated")}}function Ea(e,t){ye();var a=e.$options[t],n=t+" hook";if(a)for(var r=0,i=a.length;r<i;r++)it(a[r],e,null,e,n);e._hasHookEvent&&e.$emit("hook:"+t),be()}var $a=[],Ra=[],Na={},Wa=!1,Ua=!1,Ga=0;function qa(){Ga=$a.length=Ra.length=0,Na={},Wa=Ua=!1}var Ja=0,Ka=Date.now;if(Z&&!ae){var Xa=window.performance;Xa&&"function"===typeof Xa.now&&Ka()>document.createEvent("Event").timeStamp&&(Ka=function(){return Xa.now()})}function Za(){var e,t;for(Ja=Ka(),Ua=!0,$a.sort((function(e,t){return e.id-t.id})),Ga=0;Ga<$a.length;Ga++)e=$a[Ga],e.before&&e.before(),t=e.id,Na[t]=null,e.run();var a=Ra.slice(),n=$a.slice();qa(),tn(a),Qa(n),de&&N.devtools&&de.emit("flush")}function Qa(e){var t=e.length;while(t--){var a=e[t],n=a.vm;n._watcher===a&&n._isMounted&&!n._isDestroyed&&Ea(n,"updated")}}function en(e){e._inactive=!1,Ra.push(e)}function tn(e){for(var t=0;t<e.length;t++)e[t]._inactive=!0,Ia(e[t],!0)}function an(e){var t=e.id;if(null==Na[t]){if(Na[t]=!0,Ua){var a=$a.length-1;while(a>Ga&&$a[a].id>e.id)a--;$a.splice(a+1,0,e)}else $a.push(e);Wa||(Wa=!0,_t(Za))}}var nn=0,rn=function(e,t,a,n,r){this.vm=e,r&&(e._watcher=this),e._watchers.push(this),n?(this.deep=!!n.deep,this.user=!!n.user,this.lazy=!!n.lazy,this.sync=!!n.sync,this.before=n.before):this.deep=this.user=this.lazy=this.sync=!1,this.cb=a,this.id=++nn,this.active=!0,this.dirty=this.lazy,this.deps=[],this.newDeps=[],this.depIds=new fe,this.newDepIds=new fe,this.expression="","function"===typeof t?this.getter=t:(this.getter=J(t),this.getter||(this.getter=j)),this.value=this.lazy?void 0:this.get()};rn.prototype.get=function(){var e;ye(this);var t=this.vm;try{e=this.getter.call(t,t)}catch(Zc){if(!this.user)throw Zc;rt(Zc,t,'getter for watcher "'+this.expression+'"')}finally{this.deep&&yt(e),be(),this.cleanupDeps()}return e},rn.prototype.addDep=function(e){var t=e.id;this.newDepIds.has(t)||(this.newDepIds.add(t),this.newDeps.push(e),this.depIds.has(t)||e.addSub(this))},rn.prototype.cleanupDeps=function(){var e=this.deps.length;while(e--){var t=this.deps[e];this.newDepIds.has(t.id)||t.removeSub(this)}var a=this.depIds;this.depIds=this.newDepIds,this.newDepIds=a,this.newDepIds.clear(),a=this.deps,this.deps=this.newDeps,this.newDeps=a,this.newDeps.length=0},rn.prototype.update=function(){this.lazy?this.dirty=!0:this.sync?this.run():an(this)},rn.prototype.run=function(){if(this.active){var e=this.get();if(e!==this.value||l(e)||this.deep){var t=this.value;if(this.value=e,this.user){var a='callback for watcher "'+this.expression+'"';it(this.cb,this.vm,[e,t],this.vm,a)}else this.cb.call(this.vm,e,t)}}},rn.prototype.evaluate=function(){this.value=this.get(),this.dirty=!1},rn.prototype.depend=function(){var e=this.deps.length;while(e--)this.deps[e].depend()},rn.prototype.teardown=function(){if(this.active){this.vm._isBeingDestroyed||y(this.vm._watchers,this);var e=this.deps.length;while(e--)this.deps[e].removeSub(this);this.active=!1}};var on={enumerable:!0,configurable:!0,get:j,set:j};function sn(e,t,a){on.get=function(){return this[t][a]},on.set=function(e){this[t][a]=e},Object.defineProperty(e,a,on)}function ln(e){e._watchers=[];var t=e.$options;t.props&&cn(e,t.props),t.methods&&_n(e,t.methods),t.data?un(e):xe(e._data={},!0),t.computed&&fn(e,t.computed),t.watch&&t.watch!==se&&gn(e,t.watch)}function cn(e,t){var a=e.$options.propsData||{},n=e._props={},r=e.$options._propKeys=[],i=!e.$parent;i||Te(!1);var o=function(i){r.push(i);var o=Ze(i,t,a,e);Be(n,i,o),i in e||sn(e,"_props",i)};for(var s in t)o(s);Te(!0)}function un(e){var t=e.$options.data;t=e._data="function"===typeof t?dn(t,e):t||{},u(t)||(t={});var a=Object.keys(t),n=e.$options.props,r=(e.$options.methods,a.length);while(r--){var i=a[r];0,n&&M(n,i)||U(i)||sn(e,"_data",i)}xe(t,!0)}function dn(e,t){ye();try{return e.call(t,t)}catch(Zc){return rt(Zc,t,"data()"),{}}finally{be()}}var hn={lazy:!0};function fn(e,t){var a=e._computedWatchers=Object.create(null),n=ue();for(var r in t){var i=t[r],o="function"===typeof i?i:i.get;0,n||(a[r]=new rn(e,o||j,j,hn)),r in e||pn(e,r,i)}}function pn(e,t,a){var n=!ue();"function"===typeof a?(on.get=n?mn(t):vn(a),on.set=j):(on.get=a.get?n&&!1!==a.cache?mn(t):vn(a.get):j,on.set=a.set||j),Object.defineProperty(e,t,on)}function mn(e){return function(){var t=this._computedWatchers&&this._computedWatchers[e];if(t)return t.dirty&&t.evaluate(),_e.target&&t.depend(),t.value}}function vn(e){return function(){return e.call(this,this)}}function _n(e,t){e.$options.props;for(var a in t)e[a]="function"!==typeof t[a]?j:O(t[a],e)}function gn(e,t){for(var a in t){var n=t[a];if(Array.isArray(n))for(var r=0;r<n.length;r++)yn(e,a,n[r]);else yn(e,a,n)}}function yn(e,t,a,n){return u(a)&&(n=a,a=a.handler),"string"===typeof a&&(a=e[a]),e.$watch(t,a,n)}function bn(e){var t={get:function(){return this._data}},a={get:function(){return this._props}};Object.defineProperty(e.prototype,"$data",t),Object.defineProperty(e.prototype,"$props",a),e.prototype.$set=Ye,e.prototype.$delete=Pe,e.prototype.$watch=function(e,t,a){var n=this;if(u(t))return yn(n,e,t,a);a=a||{},a.user=!0;var r=new rn(n,e,t,a);if(a.immediate){var i='callback for immediate watcher "'+r.expression+'"';ye(),it(t,n,[r.value],n,i),be()}return function(){r.teardown()}}}var Mn=0;function zn(e){e.prototype._init=function(e){var t=this;t._uid=Mn++,t._isVue=!0,e&&e._isComponent?Ln(t,e):t.$options=Ke(wn(t.constructor),e||{},t),t._renderProxy=t,t._self=t,ja(t),ka(t),ga(t),Ea(t,"beforeCreate"),Dt(t),ln(t),Tt(t),Ea(t,"created"),t.$options.el&&t.$mount(t.$options.el)}}function Ln(e,t){var a=e.$options=Object.create(e.constructor.options),n=t._parentVnode;a.parent=t.parent,a._parentVnode=n;var r=n.componentOptions;a.propsData=r.propsData,a._parentListeners=r.listeners,a._renderChildren=r.children,a._componentTag=r.tag,t.render&&(a.render=t.render,a.staticRenderFns=t.staticRenderFns)}function wn(e){var t=e.options;if(e.super){var a=wn(e.super),n=e.superOptions;if(a!==n){e.superOptions=a;var r=Hn(e);r&&D(e.extendOptions,r),t=e.options=Ke(a,e.extendOptions),t.name&&(t.components[t.name]=e)}}return t}function Hn(e){var t,a=e.options,n=e.sealedOptions;for(var r in a)a[r]!==n[r]&&(t||(t={}),t[r]=a[r]);return t}function kn(e){this._init(e)}function An(e){e.use=function(e){var t=this._installedPlugins||(this._installedPlugins=[]);if(t.indexOf(e)>-1)return this;var a=T(arguments,1);return a.unshift(this),"function"===typeof e.install?e.install.apply(e,a):"function"===typeof e&&e.apply(null,a),t.push(e),this}}function Sn(e){e.mixin=function(e){return this.options=Ke(this.options,e),this}}function Vn(e){e.cid=0;var t=1;e.extend=function(e){e=e||{};var a=this,n=a.cid,r=e._Ctor||(e._Ctor={});if(r[n])return r[n];var i=e.name||a.options.name;var o=function(e){this._init(e)};return o.prototype=Object.create(a.prototype),o.prototype.constructor=o,o.cid=t++,o.options=Ke(a.options,e),o["super"]=a,o.options.props&&On(o),o.options.computed&&Tn(o),o.extend=a.extend,o.mixin=a.mixin,o.use=a.use,$.forEach((function(e){o[e]=a[e]})),i&&(o.options.components[i]=o),o.superOptions=a.options,o.extendOptions=e,o.sealedOptions=D({},o.options),r[n]=o,o}}function On(e){var t=e.options.props;for(var a in t)sn(e.prototype,"_props",a)}function Tn(e){var t=e.options.computed;for(var a in t)pn(e.prototype,a,t[a])}function Dn(e){$.forEach((function(t){e[t]=function(e,a){return a?("component"===t&&u(a)&&(a.name=a.name||e,a=this.options._base.extend(a)),"directive"===t&&"function"===typeof a&&(a={bind:a,update:a}),this.options[t+"s"][e]=a,a):this.options[t+"s"][e]}}))}function Cn(e){return e&&(e.Ctor.options.name||e.tag)}function jn(e,t){return Array.isArray(e)?e.indexOf(t)>-1:"string"===typeof e?e.split(",").indexOf(t)>-1:!!d(e)&&e.test(t)}function xn(e,t){var a=e.cache,n=e.keys,r=e._vnode;for(var i in a){var o=a[i];if(o){var s=o.name;s&&!t(s)&&Bn(a,i,n,r)}}}function Bn(e,t,a,n){var r=e[t];!r||n&&r.tag===n.tag||r.componentInstance.$destroy(),e[t]=null,y(a,t)}zn(kn),bn(kn),Ta(kn),xa(kn),Ma(kn);var Yn=[String,RegExp,Array],Pn={name:"keep-alive",abstract:!0,props:{include:Yn,exclude:Yn,max:[String,Number]},methods:{cacheVNode:function(){var e=this,t=e.cache,a=e.keys,n=e.vnodeToCache,r=e.keyToCache;if(n){var i=n.tag,o=n.componentInstance,s=n.componentOptions;t[r]={name:Cn(s),tag:i,componentInstance:o},a.push(r),this.max&&a.length>parseInt(this.max)&&Bn(t,a[0],a,this._vnode),this.vnodeToCache=null}}},created:function(){this.cache=Object.create(null),this.keys=[]},destroyed:function(){for(var e in this.cache)Bn(this.cache,e,this.keys)},mounted:function(){var e=this;this.cacheVNode(),this.$watch("include",(function(t){xn(e,(function(e){return jn(t,e)}))})),this.$watch("exclude",(function(t){xn(e,(function(e){return!jn(t,e)}))}))},updated:function(){this.cacheVNode()},render:function(){var e=this.$slots.default,t=Ha(e),a=t&&t.componentOptions;if(a){var n=Cn(a),r=this,i=r.include,o=r.exclude;if(i&&(!n||!jn(i,n))||o&&n&&jn(o,n))return t;var s=this,l=s.cache,c=s.keys,u=null==t.key?a.Ctor.cid+(a.tag?"::"+a.tag:""):t.key;l[u]?(t.componentInstance=l[u].componentInstance,y(c,u),c.push(u)):(this.vnodeToCache=t,this.keyToCache=u),t.data.keepAlive=!0}return t||e&&e[0]}},In={KeepAlive:Pn};function Fn(e){var t={get:function(){return N}};Object.defineProperty(e,"config",t),e.util={warn:me,extend:D,mergeOptions:Ke,defineReactive:Be},e.set=Ye,e.delete=Pe,e.nextTick=_t,e.observable=function(e){return xe(e),e},e.options=Object.create(null),$.forEach((function(t){e.options[t+"s"]=Object.create(null)})),e.options._base=e,D(e.options.components,In),An(e),Sn(e),Vn(e),Dn(e)}Fn(kn),Object.defineProperty(kn.prototype,"$isServer",{get:ue}),Object.defineProperty(kn.prototype,"$ssrContext",{get:function(){return this.$vnode&&this.$vnode.ssrContext}}),Object.defineProperty(kn,"FunctionalRenderContext",{value:ta}),kn.version="2.6.14";var En=v("style,class"),$n=v("input,textarea,option,select,progress"),Rn=function(e,t,a){return"value"===a&&$n(e)&&"button"!==t||"selected"===a&&"option"===e||"checked"===a&&"input"===e||"muted"===a&&"video"===e},Nn=v("contenteditable,draggable,spellcheck"),Wn=v("events,caret,typing,plaintext-only"),Un=function(e,t){return Xn(t)||"false"===t?"false":"contenteditable"===e&&Wn(t)?t:"true"},Gn=v("allowfullscreen,async,autofocus,autoplay,checked,compact,controls,declare,default,defaultchecked,defaultmuted,defaultselected,defer,disabled,enabled,formnovalidate,hidden,indeterminate,inert,ismap,itemscope,loop,multiple,muted,nohref,noresize,noshade,novalidate,nowrap,open,pauseonexit,readonly,required,reversed,scoped,seamless,selected,sortable,truespeed,typemustmatch,visible"),qn="http://www.w3.org/1999/xlink",Jn=function(e){return":"===e.charAt(5)&&"xlink"===e.slice(0,5)},Kn=function(e){return Jn(e)?e.slice(6,e.length):""},Xn=function(e){return null==e||!1===e};function Zn(e){var t=e.data,a=e,n=e;while(r(n.componentInstance))n=n.componentInstance._vnode,n&&n.data&&(t=Qn(n.data,t));while(r(a=a.parent))a&&a.data&&(t=Qn(t,a.data));return er(t.staticClass,t.class)}function Qn(e,t){return{staticClass:tr(e.staticClass,t.staticClass),class:r(e.class)?[e.class,t.class]:t.class}}function er(e,t){return r(e)||r(t)?tr(e,ar(t)):""}function tr(e,t){return e?t?e+" "+t:e:t||""}function ar(e){return Array.isArray(e)?nr(e):l(e)?rr(e):"string"===typeof e?e:""}function nr(e){for(var t,a="",n=0,i=e.length;n<i;n++)r(t=ar(e[n]))&&""!==t&&(a&&(a+=" "),a+=t);return a}function rr(e){var t="";for(var a in e)e[a]&&(t&&(t+=" "),t+=a);return t}var ir={svg:"http://www.w3.org/2000/svg",math:"http://www.w3.org/1998/Math/MathML"},or=v("html,body,base,head,link,meta,style,title,address,article,aside,footer,header,h1,h2,h3,h4,h5,h6,hgroup,nav,section,div,dd,dl,dt,figcaption,figure,picture,hr,img,li,main,ol,p,pre,ul,a,b,abbr,bdi,bdo,br,cite,code,data,dfn,em,i,kbd,mark,q,rp,rt,rtc,ruby,s,samp,small,span,strong,sub,sup,time,u,var,wbr,area,audio,map,track,video,embed,object,param,source,canvas,script,noscript,del,ins,caption,col,colgroup,table,thead,tbody,td,th,tr,button,datalist,fieldset,form,input,label,legend,meter,optgroup,option,output,progress,select,textarea,details,dialog,menu,menuitem,summary,content,element,shadow,template,blockquote,iframe,tfoot"),sr=v("svg,animate,circle,clippath,cursor,defs,desc,ellipse,filter,font-face,foreignobject,g,glyph,image,line,marker,mask,missing-glyph,path,pattern,polygon,polyline,rect,switch,symbol,text,textpath,tspan,use,view",!0),lr=function(e){return"pre"===e},cr=function(e){return or(e)||sr(e)};function ur(e){return sr(e)?"svg":"math"===e?"math":void 0}var dr=Object.create(null);function hr(e){if(!Z)return!0;if(cr(e))return!1;if(e=e.toLowerCase(),null!=dr[e])return dr[e];var t=document.createElement(e);return e.indexOf("-")>-1?dr[e]=t.constructor===window.HTMLUnknownElement||t.constructor===window.HTMLElement:dr[e]=/HTMLUnknownElement/.test(t.toString())}var fr=v("text,number,password,search,email,tel,url");function pr(e){if("string"===typeof e){var t=document.querySelector(e);return t||document.createElement("div")}return e}function mr(e,t){var a=document.createElement(e);return"select"!==e||t.data&&t.data.attrs&&void 0!==t.data.attrs.multiple&&a.setAttribute("multiple","multiple"),a}function vr(e,t){return document.createElementNS(ir[e],t)}function _r(e){return document.createTextNode(e)}function gr(e){return document.createComment(e)}function yr(e,t,a){e.insertBefore(t,a)}function br(e,t){e.removeChild(t)}function Mr(e,t){e.appendChild(t)}function zr(e){return e.parentNode}function Lr(e){return e.nextSibling}function wr(e){return e.tagName}function Hr(e,t){e.textContent=t}function kr(e,t){e.setAttribute(t,"")}var Ar=Object.freeze({createElement:mr,createElementNS:vr,createTextNode:_r,createComment:gr,insertBefore:yr,removeChild:br,appendChild:Mr,parentNode:zr,nextSibling:Lr,tagName:wr,setTextContent:Hr,setStyleScope:kr}),Sr={create:function(e,t){Vr(t)},update:function(e,t){e.data.ref!==t.data.ref&&(Vr(e,!0),Vr(t))},destroy:function(e){Vr(e,!0)}};function Vr(e,t){var a=e.data.ref;if(r(a)){var n=e.context,i=e.componentInstance||e.elm,o=n.$refs;t?Array.isArray(o[a])?y(o[a],i):o[a]===i&&(o[a]=void 0):e.data.refInFor?Array.isArray(o[a])?o[a].indexOf(i)<0&&o[a].push(i):o[a]=[i]:o[a]=i}}var Or=new Me("",{},[]),Tr=["create","activate","update","remove","destroy"];function Dr(e,t){return e.key===t.key&&e.asyncFactory===t.asyncFactory&&(e.tag===t.tag&&e.isComment===t.isComment&&r(e.data)===r(t.data)&&Cr(e,t)||i(e.isAsyncPlaceholder)&&n(t.asyncFactory.error))}function Cr(e,t){if("input"!==e.tag)return!0;var a,n=r(a=e.data)&&r(a=a.attrs)&&a.type,i=r(a=t.data)&&r(a=a.attrs)&&a.type;return n===i||fr(n)&&fr(i)}function jr(e,t,a){var n,i,o={};for(n=t;n<=a;++n)i=e[n].key,r(i)&&(o[i]=n);return o}function xr(e){var t,a,o={},l=e.modules,c=e.nodeOps;for(t=0;t<Tr.length;++t)for(o[Tr[t]]=[],a=0;a<l.length;++a)r(l[a][Tr[t]])&&o[Tr[t]].push(l[a][Tr[t]]);function u(e){return new Me(c.tagName(e).toLowerCase(),{},[],void 0,e)}function d(e,t){function a(){0===--a.listeners&&h(e)}return a.listeners=t,a}function h(e){var t=c.parentNode(e);r(t)&&c.removeChild(t,e)}function f(e,t,a,n,o,s,l){if(r(e.elm)&&r(s)&&(e=s[l]=He(e)),e.isRootInsert=!o,!p(e,t,a,n)){var u=e.data,d=e.children,h=e.tag;r(h)?(e.elm=e.ns?c.createElementNS(e.ns,h):c.createElement(h,e),z(e),y(e,d,t),r(u)&&M(e,t),g(a,e.elm,n)):i(e.isComment)?(e.elm=c.createComment(e.text),g(a,e.elm,n)):(e.elm=c.createTextNode(e.text),g(a,e.elm,n))}}function p(e,t,a,n){var o=e.data;if(r(o)){var s=r(e.componentInstance)&&o.keepAlive;if(r(o=o.hook)&&r(o=o.init)&&o(e,!1),r(e.componentInstance))return m(e,t),g(a,e.elm,n),i(s)&&_(e,t,a,n),!0}}function m(e,t){r(e.data.pendingInsert)&&(t.push.apply(t,e.data.pendingInsert),e.data.pendingInsert=null),e.elm=e.componentInstance.$el,b(e)?(M(e,t),z(e)):(Vr(e),t.push(e))}function _(e,t,a,n){var i,s=e;while(s.componentInstance)if(s=s.componentInstance._vnode,r(i=s.data)&&r(i=i.transition)){for(i=0;i<o.activate.length;++i)o.activate[i](Or,s);t.push(s);break}g(a,e.elm,n)}function g(e,t,a){r(e)&&(r(a)?c.parentNode(a)===e&&c.insertBefore(e,t,a):c.appendChild(e,t))}function y(e,t,a){if(Array.isArray(t)){0;for(var n=0;n<t.length;++n)f(t[n],a,e.elm,null,!0,t,n)}else s(e.text)&&c.appendChild(e.elm,c.createTextNode(String(e.text)))}function b(e){while(e.componentInstance)e=e.componentInstance._vnode;return r(e.tag)}function M(e,a){for(var n=0;n<o.create.length;++n)o.create[n](Or,e);t=e.data.hook,r(t)&&(r(t.create)&&t.create(Or,e),r(t.insert)&&a.push(e))}function z(e){var t;if(r(t=e.fnScopeId))c.setStyleScope(e.elm,t);else{var a=e;while(a)r(t=a.context)&&r(t=t.$options._scopeId)&&c.setStyleScope(e.elm,t),a=a.parent}r(t=Da)&&t!==e.context&&t!==e.fnContext&&r(t=t.$options._scopeId)&&c.setStyleScope(e.elm,t)}function L(e,t,a,n,r,i){for(;n<=r;++n)f(a[n],i,e,t,!1,a,n)}function w(e){var t,a,n=e.data;if(r(n))for(r(t=n.hook)&&r(t=t.destroy)&&t(e),t=0;t<o.destroy.length;++t)o.destroy[t](e);if(r(t=e.children))for(a=0;a<e.children.length;++a)w(e.children[a])}function H(e,t,a){for(;t<=a;++t){var n=e[t];r(n)&&(r(n.tag)?(k(n),w(n)):h(n.elm))}}function k(e,t){if(r(t)||r(e.data)){var a,n=o.remove.length+1;for(r(t)?t.listeners+=n:t=d(e.elm,n),r(a=e.componentInstance)&&r(a=a._vnode)&&r(a.data)&&k(a,t),a=0;a<o.remove.length;++a)o.remove[a](e,t);r(a=e.data.hook)&&r(a=a.remove)?a(e,t):t()}else h(e.elm)}function A(e,t,a,i,o){var s,l,u,d,h=0,p=0,m=t.length-1,v=t[0],_=t[m],g=a.length-1,y=a[0],b=a[g],M=!o;while(h<=m&&p<=g)n(v)?v=t[++h]:n(_)?_=t[--m]:Dr(v,y)?(V(v,y,i,a,p),v=t[++h],y=a[++p]):Dr(_,b)?(V(_,b,i,a,g),_=t[--m],b=a[--g]):Dr(v,b)?(V(v,b,i,a,g),M&&c.insertBefore(e,v.elm,c.nextSibling(_.elm)),v=t[++h],b=a[--g]):Dr(_,y)?(V(_,y,i,a,p),M&&c.insertBefore(e,_.elm,v.elm),_=t[--m],y=a[++p]):(n(s)&&(s=jr(t,h,m)),l=r(y.key)?s[y.key]:S(y,t,h,m),n(l)?f(y,i,e,v.elm,!1,a,p):(u=t[l],Dr(u,y)?(V(u,y,i,a,p),t[l]=void 0,M&&c.insertBefore(e,u.elm,v.elm)):f(y,i,e,v.elm,!1,a,p)),y=a[++p]);h>m?(d=n(a[g+1])?null:a[g+1].elm,L(e,d,a,p,g,i)):p>g&&H(t,h,m)}function S(e,t,a,n){for(var i=a;i<n;i++){var o=t[i];if(r(o)&&Dr(e,o))return i}}function V(e,t,a,s,l,u){if(e!==t){r(t.elm)&&r(s)&&(t=s[l]=He(t));var d=t.elm=e.elm;if(i(e.isAsyncPlaceholder))r(t.asyncFactory.resolved)?D(e.elm,t,a):t.isAsyncPlaceholder=!0;else if(i(t.isStatic)&&i(e.isStatic)&&t.key===e.key&&(i(t.isCloned)||i(t.isOnce)))t.componentInstance=e.componentInstance;else{var h,f=t.data;r(f)&&r(h=f.hook)&&r(h=h.prepatch)&&h(e,t);var p=e.children,m=t.children;if(r(f)&&b(t)){for(h=0;h<o.update.length;++h)o.update[h](e,t);r(h=f.hook)&&r(h=h.update)&&h(e,t)}n(t.text)?r(p)&&r(m)?p!==m&&A(d,p,m,a,u):r(m)?(r(e.text)&&c.setTextContent(d,""),L(d,null,m,0,m.length-1,a)):r(p)?H(p,0,p.length-1):r(e.text)&&c.setTextContent(d,""):e.text!==t.text&&c.setTextContent(d,t.text),r(f)&&r(h=f.hook)&&r(h=h.postpatch)&&h(e,t)}}}function O(e,t,a){if(i(a)&&r(e.parent))e.parent.data.pendingInsert=t;else for(var n=0;n<t.length;++n)t[n].data.hook.insert(t[n])}var T=v("attrs,class,staticClass,staticStyle,key");function D(e,t,a,n){var o,s=t.tag,l=t.data,c=t.children;if(n=n||l&&l.pre,t.elm=e,i(t.isComment)&&r(t.asyncFactory))return t.isAsyncPlaceholder=!0,!0;if(r(l)&&(r(o=l.hook)&&r(o=o.init)&&o(t,!0),r(o=t.componentInstance)))return m(t,a),!0;if(r(s)){if(r(c))if(e.hasChildNodes())if(r(o=l)&&r(o=o.domProps)&&r(o=o.innerHTML)){if(o!==e.innerHTML)return!1}else{for(var u=!0,d=e.firstChild,h=0;h<c.length;h++){if(!d||!D(d,c[h],a,n)){u=!1;break}d=d.nextSibling}if(!u||d)return!1}else y(t,c,a);if(r(l)){var f=!1;for(var p in l)if(!T(p)){f=!0,M(t,a);break}!f&&l["class"]&&yt(l["class"])}}else e.data!==t.text&&(e.data=t.text);return!0}return function(e,t,a,s){if(!n(t)){var l=!1,d=[];if(n(e))l=!0,f(t,d);else{var h=r(e.nodeType);if(!h&&Dr(e,t))V(e,t,d,null,null,s);else{if(h){if(1===e.nodeType&&e.hasAttribute(E)&&(e.removeAttribute(E),a=!0),i(a)&&D(e,t,d))return O(t,d,!0),e;e=u(e)}var p=e.elm,m=c.parentNode(p);if(f(t,d,p._leaveCb?null:m,c.nextSibling(p)),r(t.parent)){var v=t.parent,_=b(t);while(v){for(var g=0;g<o.destroy.length;++g)o.destroy[g](v);if(v.elm=t.elm,_){for(var y=0;y<o.create.length;++y)o.create[y](Or,v);var M=v.data.hook.insert;if(M.merged)for(var z=1;z<M.fns.length;z++)M.fns[z]()}else Vr(v);v=v.parent}}r(m)?H([e],0,0):r(e.tag)&&w(e)}}return O(t,d,l),t.elm}r(e)&&w(e)}}var Br={create:Yr,update:Yr,destroy:function(e){Yr(e,Or)}};function Yr(e,t){(e.data.directives||t.data.directives)&&Pr(e,t)}function Pr(e,t){var a,n,r,i=e===Or,o=t===Or,s=Fr(e.data.directives,e.context),l=Fr(t.data.directives,t.context),c=[],u=[];for(a in l)n=s[a],r=l[a],n?(r.oldValue=n.value,r.oldArg=n.arg,$r(r,"update",t,e),r.def&&r.def.componentUpdated&&u.push(r)):($r(r,"bind",t,e),r.def&&r.def.inserted&&c.push(r));if(c.length){var d=function(){for(var a=0;a<c.length;a++)$r(c[a],"inserted",t,e)};i?wt(t,"insert",d):d()}if(u.length&&wt(t,"postpatch",(function(){for(var a=0;a<u.length;a++)$r(u[a],"componentUpdated",t,e)})),!i)for(a in s)l[a]||$r(s[a],"unbind",e,e,o)}var Ir=Object.create(null);function Fr(e,t){var a,n,r=Object.create(null);if(!e)return r;for(a=0;a<e.length;a++)n=e[a],n.modifiers||(n.modifiers=Ir),r[Er(n)]=n,n.def=Xe(t.$options,"directives",n.name,!0);return r}function Er(e){return e.rawName||e.name+"."+Object.keys(e.modifiers||{}).join(".")}function $r(e,t,a,n,r){var i=e.def&&e.def[t];if(i)try{i(a.elm,e,a,n,r)}catch(Zc){rt(Zc,a.context,"directive "+e.name+" "+t+" hook")}}var Rr=[Sr,Br];function Nr(e,t){var a=t.componentOptions;if((!r(a)||!1!==a.Ctor.options.inheritAttrs)&&(!n(e.data.attrs)||!n(t.data.attrs))){var i,o,s,l=t.elm,c=e.data.attrs||{},u=t.data.attrs||{};for(i in r(u.__ob__)&&(u=t.data.attrs=D({},u)),u)o=u[i],s=c[i],s!==o&&Wr(l,i,o,t.data.pre);for(i in(ae||re)&&u.value!==c.value&&Wr(l,"value",u.value),c)n(u[i])&&(Jn(i)?l.removeAttributeNS(qn,Kn(i)):Nn(i)||l.removeAttribute(i))}}function Wr(e,t,a,n){n||e.tagName.indexOf("-")>-1?Ur(e,t,a):Gn(t)?Xn(a)?e.removeAttribute(t):(a="allowfullscreen"===t&&"EMBED"===e.tagName?"true":t,e.setAttribute(t,a)):Nn(t)?e.setAttribute(t,Un(t,a)):Jn(t)?Xn(a)?e.removeAttributeNS(qn,Kn(t)):e.setAttributeNS(qn,t,a):Ur(e,t,a)}function Ur(e,t,a){if(Xn(a))e.removeAttribute(t);else{if(ae&&!ne&&"TEXTAREA"===e.tagName&&"placeholder"===t&&""!==a&&!e.__ieph){var n=function(t){t.stopImmediatePropagation(),e.removeEventListener("input",n)};e.addEventListener("input",n),e.__ieph=!0}e.setAttribute(t,a)}}var Gr={create:Nr,update:Nr};function qr(e,t){var a=t.elm,i=t.data,o=e.data;if(!(n(i.staticClass)&&n(i.class)&&(n(o)||n(o.staticClass)&&n(o.class)))){var s=Zn(t),l=a._transitionClasses;r(l)&&(s=tr(s,ar(l))),s!==a._prevClass&&(a.setAttribute("class",s),a._prevClass=s)}}var Jr,Kr,Xr,Zr,Qr,ei,ti={create:qr,update:qr},ai=/[\w).+\-_$\]]/;function ni(e){var t,a,n,r,i,o=!1,s=!1,l=!1,c=!1,u=0,d=0,h=0,f=0;for(n=0;n<e.length;n++)if(a=t,t=e.charCodeAt(n),o)39===t&&92!==a&&(o=!1);else if(s)34===t&&92!==a&&(s=!1);else if(l)96===t&&92!==a&&(l=!1);else if(c)47===t&&92!==a&&(c=!1);else if(124!==t||124===e.charCodeAt(n+1)||124===e.charCodeAt(n-1)||u||d||h){switch(t){case 34:s=!0;break;case 39:o=!0;break;case 96:l=!0;break;case 40:h++;break;case 41:h--;break;case 91:d++;break;case 93:d--;break;case 123:u++;break;case 125:u--;break}if(47===t){for(var p=n-1,m=void 0;p>=0;p--)if(m=e.charAt(p)," "!==m)break;m&&ai.test(m)||(c=!0)}}else void 0===r?(f=n+1,r=e.slice(0,n).trim()):v();function v(){(i||(i=[])).push(e.slice(f,n).trim()),f=n+1}if(void 0===r?r=e.slice(0,n).trim():0!==f&&v(),i)for(n=0;n<i.length;n++)r=ri(r,i[n]);return r}function ri(e,t){var a=t.indexOf("(");if(a<0)return'_f("'+t+'")('+e+")";var n=t.slice(0,a),r=t.slice(a+1);return'_f("'+n+'")('+e+(")"!==r?","+r:r)}function ii(e,t){console.error("[Vue compiler]: "+e)}function oi(e,t){return e?e.map((function(e){return e[t]})).filter((function(e){return e})):[]}function si(e,t,a,n,r){(e.props||(e.props=[])).push(_i({name:t,value:a,dynamic:r},n)),e.plain=!1}function li(e,t,a,n,r){var i=r?e.dynamicAttrs||(e.dynamicAttrs=[]):e.attrs||(e.attrs=[]);i.push(_i({name:t,value:a,dynamic:r},n)),e.plain=!1}function ci(e,t,a,n){e.attrsMap[t]=a,e.attrsList.push(_i({name:t,value:a},n))}function ui(e,t,a,n,r,i,o,s){(e.directives||(e.directives=[])).push(_i({name:t,rawName:a,value:n,arg:r,isDynamicArg:i,modifiers:o},s)),e.plain=!1}function di(e,t,a){return a?"_p("+t+',"'+e+'")':e+t}function hi(e,t,n,r,i,o,s,l){var c;r=r||a,r.right?l?t="("+t+")==='click'?'contextmenu':("+t+")":"click"===t&&(t="contextmenu",delete r.right):r.middle&&(l?t="("+t+")==='click'?'mouseup':("+t+")":"click"===t&&(t="mouseup")),r.capture&&(delete r.capture,t=di("!",t,l)),r.once&&(delete r.once,t=di("~",t,l)),r.passive&&(delete r.passive,t=di("&",t,l)),r.native?(delete r.native,c=e.nativeEvents||(e.nativeEvents={})):c=e.events||(e.events={});var u=_i({value:n.trim(),dynamic:l},s);r!==a&&(u.modifiers=r);var d=c[t];Array.isArray(d)?i?d.unshift(u):d.push(u):c[t]=d?i?[u,d]:[d,u]:u,e.plain=!1}function fi(e,t){return e.rawAttrsMap[":"+t]||e.rawAttrsMap["v-bind:"+t]||e.rawAttrsMap[t]}function pi(e,t,a){var n=mi(e,":"+t)||mi(e,"v-bind:"+t);if(null!=n)return ni(n);if(!1!==a){var r=mi(e,t);if(null!=r)return JSON.stringify(r)}}function mi(e,t,a){var n;if(null!=(n=e.attrsMap[t]))for(var r=e.attrsList,i=0,o=r.length;i<o;i++)if(r[i].name===t){r.splice(i,1);break}return a&&delete e.attrsMap[t],n}function vi(e,t){for(var a=e.attrsList,n=0,r=a.length;n<r;n++){var i=a[n];if(t.test(i.name))return a.splice(n,1),i}}function _i(e,t){return t&&(null!=t.start&&(e.start=t.start),null!=t.end&&(e.end=t.end)),e}function gi(e,t,a){var n=a||{},r=n.number,i=n.trim,o="$$v",s=o;i&&(s="(typeof "+o+" === 'string'? "+o+".trim(): "+o+")"),r&&(s="_n("+s+")");var l=yi(t,s);e.model={value:"("+t+")",expression:JSON.stringify(t),callback:"function ("+o+") {"+l+"}"}}function yi(e,t){var a=bi(e);return null===a.key?e+"="+t:"$set("+a.exp+", "+a.key+", "+t+")"}function bi(e){if(e=e.trim(),Jr=e.length,e.indexOf("[")<0||e.lastIndexOf("]")<Jr-1)return Zr=e.lastIndexOf("."),Zr>-1?{exp:e.slice(0,Zr),key:'"'+e.slice(Zr+1)+'"'}:{exp:e,key:null};Kr=e,Zr=Qr=ei=0;while(!zi())Xr=Mi(),Li(Xr)?Hi(Xr):91===Xr&&wi(Xr);return{exp:e.slice(0,Qr),key:e.slice(Qr+1,ei)}}function Mi(){return Kr.charCodeAt(++Zr)}function zi(){return Zr>=Jr}function Li(e){return 34===e||39===e}function wi(e){var t=1;Qr=Zr;while(!zi())if(e=Mi(),Li(e))Hi(e);else if(91===e&&t++,93===e&&t--,0===t){ei=Zr;break}}function Hi(e){var t=e;while(!zi())if(e=Mi(),e===t)break}var ki,Ai="__r",Si="__c";function Vi(e,t,a){a;var n=t.value,r=t.modifiers,i=e.tag,o=e.attrsMap.type;if(e.component)return gi(e,n,r),!1;if("select"===i)Di(e,n,r);else if("input"===i&&"checkbox"===o)Oi(e,n,r);else if("input"===i&&"radio"===o)Ti(e,n,r);else if("input"===i||"textarea"===i)Ci(e,n,r);else{if(!N.isReservedTag(i))return gi(e,n,r),!1}return!0}function Oi(e,t,a){var n=a&&a.number,r=pi(e,"value")||"null",i=pi(e,"true-value")||"true",o=pi(e,"false-value")||"false";si(e,"checked","Array.isArray("+t+")?_i("+t+","+r+")>-1"+("true"===i?":("+t+")":":_q("+t+","+i+")")),hi(e,"change","var $$a="+t+",$$el=$event.target,$$c=$$el.checked?("+i+"):("+o+");if(Array.isArray($$a)){var $$v="+(n?"_n("+r+")":r)+",$$i=_i($$a,$$v);if($$el.checked){$$i<0&&("+yi(t,"$$a.concat([$$v])")+")}else{$$i>-1&&("+yi(t,"$$a.slice(0,$$i).concat($$a.slice($$i+1))")+")}}else{"+yi(t,"$$c")+"}",null,!0)}function Ti(e,t,a){var n=a&&a.number,r=pi(e,"value")||"null";r=n?"_n("+r+")":r,si(e,"checked","_q("+t+","+r+")"),hi(e,"change",yi(t,r),null,!0)}function Di(e,t,a){var n=a&&a.number,r='Array.prototype.filter.call($event.target.options,function(o){return o.selected}).map(function(o){var val = "_value" in o ? o._value : o.value;return '+(n?"_n(val)":"val")+"})",i="$event.target.multiple ? $$selectedVal : $$selectedVal[0]",o="var $$selectedVal = "+r+";";o=o+" "+yi(t,i),hi(e,"change",o,null,!0)}function Ci(e,t,a){var n=e.attrsMap.type,r=a||{},i=r.lazy,o=r.number,s=r.trim,l=!i&&"range"!==n,c=i?"change":"range"===n?Ai:"input",u="$event.target.value";s&&(u="$event.target.value.trim()"),o&&(u="_n("+u+")");var d=yi(t,u);l&&(d="if($event.target.composing)return;"+d),si(e,"value","("+t+")"),hi(e,c,d,null,!0),(s||o)&&hi(e,"blur","$forceUpdate()")}function ji(e){if(r(e[Ai])){var t=ae?"change":"input";e[t]=[].concat(e[Ai],e[t]||[]),delete e[Ai]}r(e[Si])&&(e.change=[].concat(e[Si],e.change||[]),delete e[Si])}function xi(e,t,a){var n=ki;return function r(){var i=t.apply(null,arguments);null!==i&&Pi(e,r,a,n)}}var Bi=ct&&!(oe&&Number(oe[1])<=53);function Yi(e,t,a,n){if(Bi){var r=Ja,i=t;t=i._wrapper=function(e){if(e.target===e.currentTarget||e.timeStamp>=r||e.timeStamp<=0||e.target.ownerDocument!==document)return i.apply(this,arguments)}}ki.addEventListener(e,t,le?{capture:a,passive:n}:a)}function Pi(e,t,a,n){(n||ki).removeEventListener(e,t._wrapper||t,a)}function Ii(e,t){if(!n(e.data.on)||!n(t.data.on)){var a=t.data.on||{},r=e.data.on||{};ki=t.elm,ji(a),Lt(a,r,Yi,Pi,xi,t.context),ki=void 0}}var Fi,Ei={create:Ii,update:Ii};function $i(e,t){if(!n(e.data.domProps)||!n(t.data.domProps)){var a,i,o=t.elm,s=e.data.domProps||{},l=t.data.domProps||{};for(a in r(l.__ob__)&&(l=t.data.domProps=D({},l)),s)a in l||(o[a]="");for(a in l){if(i=l[a],"textContent"===a||"innerHTML"===a){if(t.children&&(t.children.length=0),i===s[a])continue;1===o.childNodes.length&&o.removeChild(o.childNodes[0])}if("value"===a&&"PROGRESS"!==o.tagName){o._value=i;var c=n(i)?"":String(i);Ri(o,c)&&(o.value=c)}else if("innerHTML"===a&&sr(o.tagName)&&n(o.innerHTML)){Fi=Fi||document.createElement("div"),Fi.innerHTML="<svg>"+i+"</svg>";var u=Fi.firstChild;while(o.firstChild)o.removeChild(o.firstChild);while(u.firstChild)o.appendChild(u.firstChild)}else if(i!==s[a])try{o[a]=i}catch(Zc){}}}}function Ri(e,t){return!e.composing&&("OPTION"===e.tagName||Ni(e,t)||Wi(e,t))}function Ni(e,t){var a=!0;try{a=document.activeElement!==e}catch(Zc){}return a&&e.value!==t}function Wi(e,t){var a=e.value,n=e._vModifiers;if(r(n)){if(n.number)return m(a)!==m(t);if(n.trim)return a.trim()!==t.trim()}return a!==t}var Ui={create:$i,update:$i},Gi=z((function(e){var t={},a=/;(?![^(]*\))/g,n=/:(.+)/;return e.split(a).forEach((function(e){if(e){var a=e.split(n);a.length>1&&(t[a[0].trim()]=a[1].trim())}})),t}));function qi(e){var t=Ji(e.style);return e.staticStyle?D(e.staticStyle,t):t}function Ji(e){return Array.isArray(e)?C(e):"string"===typeof e?Gi(e):e}function Ki(e,t){var a,n={};if(t){var r=e;while(r.componentInstance)r=r.componentInstance._vnode,r&&r.data&&(a=qi(r.data))&&D(n,a)}(a=qi(e.data))&&D(n,a);var i=e;while(i=i.parent)i.data&&(a=qi(i.data))&&D(n,a);return n}var Xi,Zi=/^--/,Qi=/\s*!important$/,eo=function(e,t,a){if(Zi.test(t))e.style.setProperty(t,a);else if(Qi.test(a))e.style.setProperty(A(t),a.replace(Qi,""),"important");else{var n=ao(t);if(Array.isArray(a))for(var r=0,i=a.length;r<i;r++)e.style[n]=a[r];else e.style[n]=a}},to=["Webkit","Moz","ms"],ao=z((function(e){if(Xi=Xi||document.createElement("div").style,e=w(e),"filter"!==e&&e in Xi)return e;for(var t=e.charAt(0).toUpperCase()+e.slice(1),a=0;a<to.length;a++){var n=to[a]+t;if(n in Xi)return n}}));function no(e,t){var a=t.data,i=e.data;if(!(n(a.staticStyle)&&n(a.style)&&n(i.staticStyle)&&n(i.style))){var o,s,l=t.elm,c=i.staticStyle,u=i.normalizedStyle||i.style||{},d=c||u,h=Ji(t.data.style)||{};t.data.normalizedStyle=r(h.__ob__)?D({},h):h;var f=Ki(t,!0);for(s in d)n(f[s])&&eo(l,s,"");for(s in f)o=f[s],o!==d[s]&&eo(l,s,null==o?"":o)}}var ro={create:no,update:no},io=/\s+/;function oo(e,t){if(t&&(t=t.trim()))if(e.classList)t.indexOf(" ")>-1?t.split(io).forEach((function(t){return e.classList.add(t)})):e.classList.add(t);else{var a=" "+(e.getAttribute("class")||"")+" ";a.indexOf(" "+t+" ")<0&&e.setAttribute("class",(a+t).trim())}}function so(e,t){if(t&&(t=t.trim()))if(e.classList)t.indexOf(" ")>-1?t.split(io).forEach((function(t){return e.classList.remove(t)})):e.classList.remove(t),e.classList.length||e.removeAttribute("class");else{var a=" "+(e.getAttribute("class")||"")+" ",n=" "+t+" ";while(a.indexOf(n)>=0)a=a.replace(n," ");a=a.trim(),a?e.setAttribute("class",a):e.removeAttribute("class")}}function lo(e){if(e){if("object"===typeof e){var t={};return!1!==e.css&&D(t,co(e.name||"v")),D(t,e),t}return"string"===typeof e?co(e):void 0}}var co=z((function(e){return{enterClass:e+"-enter",enterToClass:e+"-enter-to",enterActiveClass:e+"-enter-active",leaveClass:e+"-leave",leaveToClass:e+"-leave-to",leaveActiveClass:e+"-leave-active"}})),uo=Z&&!ne,ho="transition",fo="animation",po="transition",mo="transitionend",vo="animation",_o="animationend";uo&&(void 0===window.ontransitionend&&void 0!==window.onwebkittransitionend&&(po="WebkitTransition",mo="webkitTransitionEnd"),void 0===window.onanimationend&&void 0!==window.onwebkitanimationend&&(vo="WebkitAnimation",_o="webkitAnimationEnd"));var go=Z?window.requestAnimationFrame?window.requestAnimationFrame.bind(window):setTimeout:function(e){return e()};function yo(e){go((function(){go(e)}))}function bo(e,t){var a=e._transitionClasses||(e._transitionClasses=[]);a.indexOf(t)<0&&(a.push(t),oo(e,t))}function Mo(e,t){e._transitionClasses&&y(e._transitionClasses,t),so(e,t)}function zo(e,t,a){var n=wo(e,t),r=n.type,i=n.timeout,o=n.propCount;if(!r)return a();var s=r===ho?mo:_o,l=0,c=function(){e.removeEventListener(s,u),a()},u=function(t){t.target===e&&++l>=o&&c()};setTimeout((function(){l<o&&c()}),i+1),e.addEventListener(s,u)}var Lo=/\b(transform|all)(,|$)/;function wo(e,t){var a,n=window.getComputedStyle(e),r=(n[po+"Delay"]||"").split(", "),i=(n[po+"Duration"]||"").split(", "),o=Ho(r,i),s=(n[vo+"Delay"]||"").split(", "),l=(n[vo+"Duration"]||"").split(", "),c=Ho(s,l),u=0,d=0;t===ho?o>0&&(a=ho,u=o,d=i.length):t===fo?c>0&&(a=fo,u=c,d=l.length):(u=Math.max(o,c),a=u>0?o>c?ho:fo:null,d=a?a===ho?i.length:l.length:0);var h=a===ho&&Lo.test(n[po+"Property"]);return{type:a,timeout:u,propCount:d,hasTransform:h}}function Ho(e,t){while(e.length<t.length)e=e.concat(e);return Math.max.apply(null,t.map((function(t,a){return ko(t)+ko(e[a])})))}function ko(e){return 1e3*Number(e.slice(0,-1).replace(",","."))}function Ao(e,t){var a=e.elm;r(a._leaveCb)&&(a._leaveCb.cancelled=!0,a._leaveCb());var i=lo(e.data.transition);if(!n(i)&&!r(a._enterCb)&&1===a.nodeType){var o=i.css,s=i.type,c=i.enterClass,u=i.enterToClass,d=i.enterActiveClass,h=i.appearClass,f=i.appearToClass,p=i.appearActiveClass,v=i.beforeEnter,_=i.enter,g=i.afterEnter,y=i.enterCancelled,b=i.beforeAppear,M=i.appear,z=i.afterAppear,L=i.appearCancelled,w=i.duration,H=Da,k=Da.$vnode;while(k&&k.parent)H=k.context,k=k.parent;var A=!H._isMounted||!e.isRootInsert;if(!A||M||""===M){var S=A&&h?h:c,V=A&&p?p:d,O=A&&f?f:u,T=A&&b||v,D=A&&"function"===typeof M?M:_,C=A&&z||g,j=A&&L||y,x=m(l(w)?w.enter:w);0;var B=!1!==o&&!ne,Y=Oo(D),P=a._enterCb=F((function(){B&&(Mo(a,O),Mo(a,V)),P.cancelled?(B&&Mo(a,S),j&&j(a)):C&&C(a),a._enterCb=null}));e.data.show||wt(e,"insert",(function(){var t=a.parentNode,n=t&&t._pending&&t._pending[e.key];n&&n.tag===e.tag&&n.elm._leaveCb&&n.elm._leaveCb(),D&&D(a,P)})),T&&T(a),B&&(bo(a,S),bo(a,V),yo((function(){Mo(a,S),P.cancelled||(bo(a,O),Y||(Vo(x)?setTimeout(P,x):zo(a,s,P)))}))),e.data.show&&(t&&t(),D&&D(a,P)),B||Y||P()}}}function So(e,t){var a=e.elm;r(a._enterCb)&&(a._enterCb.cancelled=!0,a._enterCb());var i=lo(e.data.transition);if(n(i)||1!==a.nodeType)return t();if(!r(a._leaveCb)){var o=i.css,s=i.type,c=i.leaveClass,u=i.leaveToClass,d=i.leaveActiveClass,h=i.beforeLeave,f=i.leave,p=i.afterLeave,v=i.leaveCancelled,_=i.delayLeave,g=i.duration,y=!1!==o&&!ne,b=Oo(f),M=m(l(g)?g.leave:g);0;var z=a._leaveCb=F((function(){a.parentNode&&a.parentNode._pending&&(a.parentNode._pending[e.key]=null),y&&(Mo(a,u),Mo(a,d)),z.cancelled?(y&&Mo(a,c),v&&v(a)):(t(),p&&p(a)),a._leaveCb=null}));_?_(L):L()}function L(){z.cancelled||(!e.data.show&&a.parentNode&&((a.parentNode._pending||(a.parentNode._pending={}))[e.key]=e),h&&h(a),y&&(bo(a,c),bo(a,d),yo((function(){Mo(a,c),z.cancelled||(bo(a,u),b||(Vo(M)?setTimeout(z,M):zo(a,s,z)))}))),f&&f(a,z),y||b||z())}}function Vo(e){return"number"===typeof e&&!isNaN(e)}function Oo(e){if(n(e))return!1;var t=e.fns;return r(t)?Oo(Array.isArray(t)?t[0]:t):(e._length||e.length)>1}function To(e,t){!0!==t.data.show&&Ao(t)}var Do=Z?{create:To,activate:To,remove:function(e,t){!0!==e.data.show?So(e,t):t()}}:{},Co=[Gr,ti,Ei,Ui,ro,Do],jo=Co.concat(Rr),xo=xr({nodeOps:Ar,modules:jo});ne&&document.addEventListener("selectionchange",(function(){var e=document.activeElement;e&&e.vmodel&&Ro(e,"input")}));var Bo={inserted:function(e,t,a,n){"select"===a.tag?(n.elm&&!n.elm._vOptions?wt(a,"postpatch",(function(){Bo.componentUpdated(e,t,a)})):Yo(e,t,a.context),e._vOptions=[].map.call(e.options,Fo)):("textarea"===a.tag||fr(e.type))&&(e._vModifiers=t.modifiers,t.modifiers.lazy||(e.addEventListener("compositionstart",Eo),e.addEventListener("compositionend",$o),e.addEventListener("change",$o),ne&&(e.vmodel=!0)))},componentUpdated:function(e,t,a){if("select"===a.tag){Yo(e,t,a.context);var n=e._vOptions,r=e._vOptions=[].map.call(e.options,Fo);if(r.some((function(e,t){return!P(e,n[t])}))){var i=e.multiple?t.value.some((function(e){return Io(e,r)})):t.value!==t.oldValue&&Io(t.value,r);i&&Ro(e,"change")}}}};function Yo(e,t,a){Po(e,t,a),(ae||re)&&setTimeout((function(){Po(e,t,a)}),0)}function Po(e,t,a){var n=t.value,r=e.multiple;if(!r||Array.isArray(n)){for(var i,o,s=0,l=e.options.length;s<l;s++)if(o=e.options[s],r)i=I(n,Fo(o))>-1,o.selected!==i&&(o.selected=i);else if(P(Fo(o),n))return void(e.selectedIndex!==s&&(e.selectedIndex=s));r||(e.selectedIndex=-1)}}function Io(e,t){return t.every((function(t){return!P(t,e)}))}function Fo(e){return"_value"in e?e._value:e.value}function Eo(e){e.target.composing=!0}function $o(e){e.target.composing&&(e.target.composing=!1,Ro(e.target,"input"))}function Ro(e,t){var a=document.createEvent("HTMLEvents");a.initEvent(t,!0,!0),e.dispatchEvent(a)}function No(e){return!e.componentInstance||e.data&&e.data.transition?e:No(e.componentInstance._vnode)}var Wo={bind:function(e,t,a){var n=t.value;a=No(a);var r=a.data&&a.data.transition,i=e.__vOriginalDisplay="none"===e.style.display?"":e.style.display;n&&r?(a.data.show=!0,Ao(a,(function(){e.style.display=i}))):e.style.display=n?i:"none"},update:function(e,t,a){var n=t.value,r=t.oldValue;if(!n!==!r){a=No(a);var i=a.data&&a.data.transition;i?(a.data.show=!0,n?Ao(a,(function(){e.style.display=e.__vOriginalDisplay})):So(a,(function(){e.style.display="none"}))):e.style.display=n?e.__vOriginalDisplay:"none"}},unbind:function(e,t,a,n,r){r||(e.style.display=e.__vOriginalDisplay)}},Uo={model:Bo,show:Wo},Go={name:String,appear:Boolean,css:Boolean,mode:String,type:String,enterClass:String,leaveClass:String,enterToClass:String,leaveToClass:String,enterActiveClass:String,leaveActiveClass:String,appearClass:String,appearActiveClass:String,appearToClass:String,duration:[Number,String,Object]};function qo(e){var t=e&&e.componentOptions;return t&&t.Ctor.options.abstract?qo(Ha(t.children)):e}function Jo(e){var t={},a=e.$options;for(var n in a.propsData)t[n]=e[n];var r=a._parentListeners;for(var i in r)t[w(i)]=r[i];return t}function Ko(e,t){if(/\d-keep-alive$/.test(t.tag))return e("keep-alive",{props:t.componentOptions.propsData})}function Xo(e){while(e=e.parent)if(e.data.transition)return!0}function Zo(e,t){return t.key===e.key&&t.tag===e.tag}var Qo=function(e){return e.tag||Bt(e)},es=function(e){return"show"===e.name},ts={name:"transition",props:Go,abstract:!0,render:function(e){var t=this,a=this.$slots.default;if(a&&(a=a.filter(Qo),a.length)){0;var n=this.mode;0;var r=a[0];if(Xo(this.$vnode))return r;var i=qo(r);if(!i)return r;if(this._leaving)return Ko(e,r);var o="__transition-"+this._uid+"-";i.key=null==i.key?i.isComment?o+"comment":o+i.tag:s(i.key)?0===String(i.key).indexOf(o)?i.key:o+i.key:i.key;var l=(i.data||(i.data={})).transition=Jo(this),c=this._vnode,u=qo(c);if(i.data.directives&&i.data.directives.some(es)&&(i.data.show=!0),u&&u.data&&!Zo(i,u)&&!Bt(u)&&(!u.componentInstance||!u.componentInstance._vnode.isComment)){var d=u.data.transition=D({},l);if("out-in"===n)return this._leaving=!0,wt(d,"afterLeave",(function(){t._leaving=!1,t.$forceUpdate()})),Ko(e,r);if("in-out"===n){if(Bt(i))return c;var h,f=function(){h()};wt(l,"afterEnter",f),wt(l,"enterCancelled",f),wt(d,"delayLeave",(function(e){h=e}))}}return r}}},as=D({tag:String,moveClass:String},Go);delete as.mode;var ns={props:as,beforeMount:function(){var e=this,t=this._update;this._update=function(a,n){var r=Ca(e);e.__patch__(e._vnode,e.kept,!1,!0),e._vnode=e.kept,r(),t.call(e,a,n)}},render:function(e){for(var t=this.tag||this.$vnode.data.tag||"span",a=Object.create(null),n=this.prevChildren=this.children,r=this.$slots.default||[],i=this.children=[],o=Jo(this),s=0;s<r.length;s++){var l=r[s];if(l.tag)if(null!=l.key&&0!==String(l.key).indexOf("__vlist"))i.push(l),a[l.key]=l,(l.data||(l.data={})).transition=o;else;}if(n){for(var c=[],u=[],d=0;d<n.length;d++){var h=n[d];h.data.transition=o,h.data.pos=h.elm.getBoundingClientRect(),a[h.key]?c.push(h):u.push(h)}this.kept=e(t,null,c),this.removed=u}return e(t,null,i)},updated:function(){var e=this.prevChildren,t=this.moveClass||(this.name||"v")+"-move";e.length&&this.hasMove(e[0].elm,t)&&(e.forEach(rs),e.forEach(is),e.forEach(os),this._reflow=document.body.offsetHeight,e.forEach((function(e){if(e.data.moved){var a=e.elm,n=a.style;bo(a,t),n.transform=n.WebkitTransform=n.transitionDuration="",a.addEventListener(mo,a._moveCb=function e(n){n&&n.target!==a||n&&!/transform$/.test(n.propertyName)||(a.removeEventListener(mo,e),a._moveCb=null,Mo(a,t))})}})))},methods:{hasMove:function(e,t){if(!uo)return!1;if(this._hasMove)return this._hasMove;var a=e.cloneNode();e._transitionClasses&&e._transitionClasses.forEach((function(e){so(a,e)})),oo(a,t),a.style.display="none",this.$el.appendChild(a);var n=wo(a);return this.$el.removeChild(a),this._hasMove=n.hasTransform}}};function rs(e){e.elm._moveCb&&e.elm._moveCb(),e.elm._enterCb&&e.elm._enterCb()}function is(e){e.data.newPos=e.elm.getBoundingClientRect()}function os(e){var t=e.data.pos,a=e.data.newPos,n=t.left-a.left,r=t.top-a.top;if(n||r){e.data.moved=!0;var i=e.elm.style;i.transform=i.WebkitTransform="translate("+n+"px,"+r+"px)",i.transitionDuration="0s"}}var ss={Transition:ts,TransitionGroup:ns};kn.config.mustUseProp=Rn,kn.config.isReservedTag=cr,kn.config.isReservedAttr=En,kn.config.getTagNamespace=ur,kn.config.isUnknownElement=hr,D(kn.options.directives,Uo),D(kn.options.components,ss),kn.prototype.__patch__=Z?xo:j,kn.prototype.$mount=function(e,t){return e=e&&Z?pr(e):void 0,Ba(this,e,t)},Z&&setTimeout((function(){N.devtools&&de&&de.emit("init",kn)}),0);var ls=/\{\{((?:.|\r?\n)+?)\}\}/g,cs=/[-.*+?^${}()|[\]\/\\]/g,us=z((function(e){var t=e[0].replace(cs,"\\$&"),a=e[1].replace(cs,"\\$&");return new RegExp(t+"((?:.|\\n)+?)"+a,"g")}));function ds(e,t){var a=t?us(t):ls;if(a.test(e)){var n,r,i,o=[],s=[],l=a.lastIndex=0;while(n=a.exec(e)){r=n.index,r>l&&(s.push(i=e.slice(l,r)),o.push(JSON.stringify(i)));var c=ni(n[1].trim());o.push("_s("+c+")"),s.push({"@binding":c}),l=r+n[0].length}return l<e.length&&(s.push(i=e.slice(l)),o.push(JSON.stringify(i))),{expression:o.join("+"),tokens:s}}}function hs(e,t){t.warn;var a=mi(e,"class");a&&(e.staticClass=JSON.stringify(a));var n=pi(e,"class",!1);n&&(e.classBinding=n)}function fs(e){var t="";return e.staticClass&&(t+="staticClass:"+e.staticClass+","),e.classBinding&&(t+="class:"+e.classBinding+","),t}var ps={staticKeys:["staticClass"],transformNode:hs,genData:fs};function ms(e,t){t.warn;var a=mi(e,"style");a&&(e.staticStyle=JSON.stringify(Gi(a)));var n=pi(e,"style",!1);n&&(e.styleBinding=n)}function vs(e){var t="";return e.staticStyle&&(t+="staticStyle:"+e.staticStyle+","),e.styleBinding&&(t+="style:("+e.styleBinding+"),"),t}var _s,gs={staticKeys:["staticStyle"],transformNode:ms,genData:vs},ys={decode:function(e){return _s=_s||document.createElement("div"),_s.innerHTML=e,_s.textContent}},bs=v("area,base,br,col,embed,frame,hr,img,input,isindex,keygen,link,meta,param,source,track,wbr"),Ms=v("colgroup,dd,dt,li,options,p,td,tfoot,th,thead,tr,source"),zs=v("address,article,aside,base,blockquote,body,caption,col,colgroup,dd,details,dialog,div,dl,dt,fieldset,figcaption,figure,footer,form,h1,h2,h3,h4,h5,h6,head,header,hgroup,hr,html,legend,li,menuitem,meta,optgroup,option,param,rp,rt,source,style,summary,tbody,td,tfoot,th,thead,title,tr,track"),Ls=/^\s*([^\s"'<>\/=]+)(?:\s*(=)\s*(?:"([^"]*)"+|'([^']*)'+|([^\s"'=<>`]+)))?/,ws=/^\s*((?:v-[\w-]+:|@|:|#)\[[^=]+?\][^\s"'<>\/=]*)(?:\s*(=)\s*(?:"([^"]*)"+|'([^']*)'+|([^\s"'=<>`]+)))?/,Hs="[a-zA-Z_][\\-\\.0-9_a-zA-Z"+W.source+"]*",ks="((?:"+Hs+"\\:)?"+Hs+")",As=new RegExp("^<"+ks),Ss=/^\s*(\/?)>/,Vs=new RegExp("^<\\/"+ks+"[^>]*>"),Os=/^<!DOCTYPE [^>]+>/i,Ts=/^<!\--/,Ds=/^<!\[/,Cs=v("script,style,textarea",!0),js={},xs={"&lt;":"<","&gt;":">","&quot;":'"',"&amp;":"&","&#10;":"\n","&#9;":"\t","&#39;":"'"},Bs=/&(?:lt|gt|quot|amp|#39);/g,Ys=/&(?:lt|gt|quot|amp|#39|#10|#9);/g,Ps=v("pre,textarea",!0),Is=function(e,t){return e&&Ps(e)&&"\n"===t[0]};function Fs(e,t){var a=t?Ys:Bs;return e.replace(a,(function(e){return xs[e]}))}function Es(e,t){var a,n,r=[],i=t.expectHTML,o=t.isUnaryTag||x,s=t.canBeLeftOpenTag||x,l=0;while(e){if(a=e,n&&Cs(n)){var c=0,u=n.toLowerCase(),d=js[u]||(js[u]=new RegExp("([\\s\\S]*?)(</"+u+"[^>]*>)","i")),h=e.replace(d,(function(e,a,n){return c=n.length,Cs(u)||"noscript"===u||(a=a.replace(/<!\--([\s\S]*?)-->/g,"$1").replace(/<!\[CDATA\[([\s\S]*?)]]>/g,"$1")),Is(u,a)&&(a=a.slice(1)),t.chars&&t.chars(a),""}));l+=e.length-h.length,e=h,k(u,l-c,l)}else{var f=e.indexOf("<");if(0===f){if(Ts.test(e)){var p=e.indexOf("--\x3e");if(p>=0){t.shouldKeepComment&&t.comment(e.substring(4,p),l,l+p+3),L(p+3);continue}}if(Ds.test(e)){var m=e.indexOf("]>");if(m>=0){L(m+2);continue}}var v=e.match(Os);if(v){L(v[0].length);continue}var _=e.match(Vs);if(_){var g=l;L(_[0].length),k(_[1],g,l);continue}var y=w();if(y){H(y),Is(y.tagName,e)&&L(1);continue}}var b=void 0,M=void 0,z=void 0;if(f>=0){M=e.slice(f);while(!Vs.test(M)&&!As.test(M)&&!Ts.test(M)&&!Ds.test(M)){if(z=M.indexOf("<",1),z<0)break;f+=z,M=e.slice(f)}b=e.substring(0,f)}f<0&&(b=e),b&&L(b.length),t.chars&&b&&t.chars(b,l-b.length,l)}if(e===a){t.chars&&t.chars(e);break}}function L(t){l+=t,e=e.substring(t)}function w(){var t=e.match(As);if(t){var a,n,r={tagName:t[1],attrs:[],start:l};L(t[0].length);while(!(a=e.match(Ss))&&(n=e.match(ws)||e.match(Ls)))n.start=l,L(n[0].length),n.end=l,r.attrs.push(n);if(a)return r.unarySlash=a[1],L(a[0].length),r.end=l,r}}function H(e){var a=e.tagName,l=e.unarySlash;i&&("p"===n&&zs(a)&&k(n),s(a)&&n===a&&k(a));for(var c=o(a)||!!l,u=e.attrs.length,d=new Array(u),h=0;h<u;h++){var f=e.attrs[h],p=f[3]||f[4]||f[5]||"",m="a"===a&&"href"===f[1]?t.shouldDecodeNewlinesForHref:t.shouldDecodeNewlines;d[h]={name:f[1],value:Fs(p,m)}}c||(r.push({tag:a,lowerCasedTag:a.toLowerCase(),attrs:d,start:e.start,end:e.end}),n=a),t.start&&t.start(a,d,c,e.start,e.end)}function k(e,a,i){var o,s;if(null==a&&(a=l),null==i&&(i=l),e){for(s=e.toLowerCase(),o=r.length-1;o>=0;o--)if(r[o].lowerCasedTag===s)break}else o=0;if(o>=0){for(var c=r.length-1;c>=o;c--)t.end&&t.end(r[c].tag,a,i);r.length=o,n=o&&r[o-1].tag}else"br"===s?t.start&&t.start(e,[],!0,a,i):"p"===s&&(t.start&&t.start(e,[],!1,a,i),t.end&&t.end(e,a,i))}k()}var $s,Rs,Ns,Ws,Us,Gs,qs,Js,Ks=/^@|^v-on:/,Xs=/^v-|^@|^:|^#/,Zs=/([\s\S]*?)\s+(?:in|of)\s+([\s\S]*)/,Qs=/,([^,\}\]]*)(?:,([^,\}\]]*))?$/,el=/^\(|\)$/g,tl=/^\[.*\]$/,al=/:(.*)$/,nl=/^:|^\.|^v-bind:/,rl=/\.[^.\]]+(?=[^\]]*$)/g,il=/^v-slot(:|$)|^#/,ol=/[\r\n]/,sl=/[ \f\t\r\n]+/g,ll=z(ys.decode),cl="_empty_";function ul(e,t,a){return{type:1,tag:e,attrsList:t,attrsMap:Tl(t),rawAttrsMap:{},parent:a,children:[]}}function dl(e,t){$s=t.warn||ii,Gs=t.isPreTag||x,qs=t.mustUseProp||x,Js=t.getTagNamespace||x;var a=t.isReservedTag||x;(function(e){return!(!(e.component||e.attrsMap[":is"]||e.attrsMap["v-bind:is"])&&(e.attrsMap.is?a(e.attrsMap.is):a(e.tag)))}),Ns=oi(t.modules,"transformNode"),Ws=oi(t.modules,"preTransformNode"),Us=oi(t.modules,"postTransformNode"),Rs=t.delimiters;var n,r,i=[],o=!1!==t.preserveWhitespace,s=t.whitespace,l=!1,c=!1;function u(e){if(d(e),l||e.processed||(e=pl(e,t)),i.length||e===n||n.if&&(e.elseif||e.else)&&zl(n,{exp:e.elseif,block:e}),r&&!e.forbidden)if(e.elseif||e.else)bl(e,r);else{if(e.slotScope){var a=e.slotTarget||'"default"';(r.scopedSlots||(r.scopedSlots={}))[a]=e}r.children.push(e),e.parent=r}e.children=e.children.filter((function(e){return!e.slotScope})),d(e),e.pre&&(l=!1),Gs(e.tag)&&(c=!1);for(var o=0;o<Us.length;o++)Us[o](e,t)}function d(e){var t;if(!c)while((t=e.children[e.children.length-1])&&3===t.type&&" "===t.text)e.children.pop()}return Es(e,{warn:$s,expectHTML:t.expectHTML,isUnaryTag:t.isUnaryTag,canBeLeftOpenTag:t.canBeLeftOpenTag,shouldDecodeNewlines:t.shouldDecodeNewlines,shouldDecodeNewlinesForHref:t.shouldDecodeNewlinesForHref,shouldKeepComment:t.comments,outputSourceRange:t.outputSourceRange,start:function(e,a,o,s,d){var h=r&&r.ns||Js(e);ae&&"svg"===h&&(a=Bl(a));var f=ul(e,a,r);h&&(f.ns=h),Cl(f)&&!ue()&&(f.forbidden=!0);for(var p=0;p<Ws.length;p++)f=Ws[p](f,t)||f;l||(hl(f),f.pre&&(l=!0)),Gs(f.tag)&&(c=!0),l?fl(f):f.processed||(_l(f),yl(f),Ll(f)),n||(n=f),o?u(f):(r=f,i.push(f))},end:function(e,t,a){var n=i[i.length-1];i.length-=1,r=i[i.length-1],u(n)},chars:function(e,t,a){if(r&&(!ae||"textarea"!==r.tag||r.attrsMap.placeholder!==e)){var n,i,u=r.children;if(e=c||e.trim()?Dl(r)?e:ll(e):u.length?s?"condense"===s&&ol.test(e)?"":" ":o?" ":"":"",e)c||"condense"!==s||(e=e.replace(sl," ")),!l&&" "!==e&&(n=ds(e,Rs))?i={type:2,expression:n.expression,tokens:n.tokens,text:e}:" "===e&&u.length&&" "===u[u.length-1].text||(i={type:3,text:e}),i&&u.push(i)}},comment:function(e,t,a){if(r){var n={type:3,text:e,isComment:!0};0,r.children.push(n)}}}),n}function hl(e){null!=mi(e,"v-pre")&&(e.pre=!0)}function fl(e){var t=e.attrsList,a=t.length;if(a)for(var n=e.attrs=new Array(a),r=0;r<a;r++)n[r]={name:t[r].name,value:JSON.stringify(t[r].value)},null!=t[r].start&&(n[r].start=t[r].start,n[r].end=t[r].end);else e.pre||(e.plain=!0)}function pl(e,t){ml(e),e.plain=!e.key&&!e.scopedSlots&&!e.attrsList.length,vl(e),wl(e),kl(e),Al(e);for(var a=0;a<Ns.length;a++)e=Ns[a](e,t)||e;return Sl(e),e}function ml(e){var t=pi(e,"key");t&&(e.key=t)}function vl(e){var t=pi(e,"ref");t&&(e.ref=t,e.refInFor=Vl(e))}function _l(e){var t;if(t=mi(e,"v-for")){var a=gl(t);a&&D(e,a)}}function gl(e){var t=e.match(Zs);if(t){var a={};a.for=t[2].trim();var n=t[1].trim().replace(el,""),r=n.match(Qs);return r?(a.alias=n.replace(Qs,"").trim(),a.iterator1=r[1].trim(),r[2]&&(a.iterator2=r[2].trim())):a.alias=n,a}}function yl(e){var t=mi(e,"v-if");if(t)e.if=t,zl(e,{exp:t,block:e});else{null!=mi(e,"v-else")&&(e.else=!0);var a=mi(e,"v-else-if");a&&(e.elseif=a)}}function bl(e,t){var a=Ml(t.children);a&&a.if&&zl(a,{exp:e.elseif,block:e})}function Ml(e){var t=e.length;while(t--){if(1===e[t].type)return e[t];e.pop()}}function zl(e,t){e.ifConditions||(e.ifConditions=[]),e.ifConditions.push(t)}function Ll(e){var t=mi(e,"v-once");null!=t&&(e.once=!0)}function wl(e){var t;"template"===e.tag?(t=mi(e,"scope"),e.slotScope=t||mi(e,"slot-scope")):(t=mi(e,"slot-scope"))&&(e.slotScope=t);var a=pi(e,"slot");if(a&&(e.slotTarget='""'===a?'"default"':a,e.slotTargetDynamic=!(!e.attrsMap[":slot"]&&!e.attrsMap["v-bind:slot"]),"template"===e.tag||e.slotScope||li(e,"slot",a,fi(e,"slot"))),"template"===e.tag){var n=vi(e,il);if(n){0;var r=Hl(n),i=r.name,o=r.dynamic;e.slotTarget=i,e.slotTargetDynamic=o,e.slotScope=n.value||cl}}else{var s=vi(e,il);if(s){0;var l=e.scopedSlots||(e.scopedSlots={}),c=Hl(s),u=c.name,d=c.dynamic,h=l[u]=ul("template",[],e);h.slotTarget=u,h.slotTargetDynamic=d,h.children=e.children.filter((function(e){if(!e.slotScope)return e.parent=h,!0})),h.slotScope=s.value||cl,e.children=[],e.plain=!1}}}function Hl(e){var t=e.name.replace(il,"");return t||"#"!==e.name[0]&&(t="default"),tl.test(t)?{name:t.slice(1,-1),dynamic:!0}:{name:'"'+t+'"',dynamic:!1}}function kl(e){"slot"===e.tag&&(e.slotName=pi(e,"name"))}function Al(e){var t;(t=pi(e,"is"))&&(e.component=t),null!=mi(e,"inline-template")&&(e.inlineTemplate=!0)}function Sl(e){var t,a,n,r,i,o,s,l,c=e.attrsList;for(t=0,a=c.length;t<a;t++){if(n=r=c[t].name,i=c[t].value,Xs.test(n))if(e.hasBindings=!0,o=Ol(n.replace(Xs,"")),o&&(n=n.replace(rl,"")),nl.test(n))n=n.replace(nl,""),i=ni(i),l=tl.test(n),l&&(n=n.slice(1,-1)),o&&(o.prop&&!l&&(n=w(n),"innerHtml"===n&&(n="innerHTML")),o.camel&&!l&&(n=w(n)),o.sync&&(s=yi(i,"$event"),l?hi(e,'"update:"+('+n+")",s,null,!1,$s,c[t],!0):(hi(e,"update:"+w(n),s,null,!1,$s,c[t]),A(n)!==w(n)&&hi(e,"update:"+A(n),s,null,!1,$s,c[t])))),o&&o.prop||!e.component&&qs(e.tag,e.attrsMap.type,n)?si(e,n,i,c[t],l):li(e,n,i,c[t],l);else if(Ks.test(n))n=n.replace(Ks,""),l=tl.test(n),l&&(n=n.slice(1,-1)),hi(e,n,i,o,!1,$s,c[t],l);else{n=n.replace(Xs,"");var u=n.match(al),d=u&&u[1];l=!1,d&&(n=n.slice(0,-(d.length+1)),tl.test(d)&&(d=d.slice(1,-1),l=!0)),ui(e,n,r,i,d,l,o,c[t])}else li(e,n,JSON.stringify(i),c[t]),!e.component&&"muted"===n&&qs(e.tag,e.attrsMap.type,n)&&si(e,n,"true",c[t])}}function Vl(e){var t=e;while(t){if(void 0!==t.for)return!0;t=t.parent}return!1}function Ol(e){var t=e.match(rl);if(t){var a={};return t.forEach((function(e){a[e.slice(1)]=!0})),a}}function Tl(e){for(var t={},a=0,n=e.length;a<n;a++)t[e[a].name]=e[a].value;return t}function Dl(e){return"script"===e.tag||"style"===e.tag}function Cl(e){return"style"===e.tag||"script"===e.tag&&(!e.attrsMap.type||"text/javascript"===e.attrsMap.type)}var jl=/^xmlns:NS\d+/,xl=/^NS\d+:/;function Bl(e){for(var t=[],a=0;a<e.length;a++){var n=e[a];jl.test(n.name)||(n.name=n.name.replace(xl,""),t.push(n))}return t}function Yl(e,t){if("input"===e.tag){var a,n=e.attrsMap;if(!n["v-model"])return;if((n[":type"]||n["v-bind:type"])&&(a=pi(e,"type")),n.type||a||!n["v-bind"]||(a="("+n["v-bind"]+").type"),a){var r=mi(e,"v-if",!0),i=r?"&&("+r+")":"",o=null!=mi(e,"v-else",!0),s=mi(e,"v-else-if",!0),l=Pl(e);_l(l),ci(l,"type","checkbox"),pl(l,t),l.processed=!0,l.if="("+a+")==='checkbox'"+i,zl(l,{exp:l.if,block:l});var c=Pl(e);mi(c,"v-for",!0),ci(c,"type","radio"),pl(c,t),zl(l,{exp:"("+a+")==='radio'"+i,block:c});var u=Pl(e);return mi(u,"v-for",!0),ci(u,":type",a),pl(u,t),zl(l,{exp:r,block:u}),o?l.else=!0:s&&(l.elseif=s),l}}}function Pl(e){return ul(e.tag,e.attrsList.slice(),e.parent)}var Il={preTransformNode:Yl},Fl=[ps,gs,Il];function El(e,t){t.value&&si(e,"textContent","_s("+t.value+")",t)}function $l(e,t){t.value&&si(e,"innerHTML","_s("+t.value+")",t)}var Rl,Nl,Wl={model:Vi,text:El,html:$l},Ul={expectHTML:!0,modules:Fl,directives:Wl,isPreTag:lr,isUnaryTag:bs,mustUseProp:Rn,canBeLeftOpenTag:Ms,isReservedTag:cr,getTagNamespace:ur,staticKeys:Y(Fl)},Gl=z(Jl);function ql(e,t){e&&(Rl=Gl(t.staticKeys||""),Nl=t.isReservedTag||x,Kl(e),Xl(e,!1))}function Jl(e){return v("type,tag,attrsList,attrsMap,plain,parent,children,attrs,start,end,rawAttrsMap"+(e?","+e:""))}function Kl(e){if(e.static=Zl(e),1===e.type){if(!Nl(e.tag)&&"slot"!==e.tag&&null==e.attrsMap["inline-template"])return;for(var t=0,a=e.children.length;t<a;t++){var n=e.children[t];Kl(n),n.static||(e.static=!1)}if(e.ifConditions)for(var r=1,i=e.ifConditions.length;r<i;r++){var o=e.ifConditions[r].block;Kl(o),o.static||(e.static=!1)}}}function Xl(e,t){if(1===e.type){if((e.static||e.once)&&(e.staticInFor=t),e.static&&e.children.length&&(1!==e.children.length||3!==e.children[0].type))return void(e.staticRoot=!0);if(e.staticRoot=!1,e.children)for(var a=0,n=e.children.length;a<n;a++)Xl(e.children[a],t||!!e.for);if(e.ifConditions)for(var r=1,i=e.ifConditions.length;r<i;r++)Xl(e.ifConditions[r].block,t)}}function Zl(e){return 2!==e.type&&(3===e.type||!(!e.pre&&(e.hasBindings||e.if||e.for||_(e.tag)||!Nl(e.tag)||Ql(e)||!Object.keys(e).every(Rl))))}function Ql(e){while(e.parent){if(e=e.parent,"template"!==e.tag)return!1;if(e.for)return!0}return!1}var ec=/^([\w$_]+|\([^)]*?\))\s*=>|^function(?:\s+[\w$]+)?\s*\(/,tc=/\([^)]*?\);*$/,ac=/^[A-Za-z_$][\w$]*(?:\.[A-Za-z_$][\w$]*|\['[^']*?']|\["[^"]*?"]|\[\d+]|\[[A-Za-z_$][\w$]*])*$/,nc={esc:27,tab:9,enter:13,space:32,up:38,left:37,right:39,down:40,delete:[8,46]},rc={esc:["Esc","Escape"],tab:"Tab",enter:"Enter",space:[" ","Spacebar"],up:["Up","ArrowUp"],left:["Left","ArrowLeft"],right:["Right","ArrowRight"],down:["Down","ArrowDown"],delete:["Backspace","Delete","Del"]},ic=function(e){return"if("+e+")return null;"},oc={stop:"$event.stopPropagation();",prevent:"$event.preventDefault();",self:ic("$event.target !== $event.currentTarget"),ctrl:ic("!$event.ctrlKey"),shift:ic("!$event.shiftKey"),alt:ic("!$event.altKey"),meta:ic("!$event.metaKey"),left:ic("'button' in $event && $event.button !== 0"),middle:ic("'button' in $event && $event.button !== 1"),right:ic("'button' in $event && $event.button !== 2")};function sc(e,t){var a=t?"nativeOn:":"on:",n="",r="";for(var i in e){var o=lc(e[i]);e[i]&&e[i].dynamic?r+=i+","+o+",":n+='"'+i+'":'+o+","}return n="{"+n.slice(0,-1)+"}",r?a+"_d("+n+",["+r.slice(0,-1)+"])":a+n}function lc(e){if(!e)return"function(){}";if(Array.isArray(e))return"["+e.map((function(e){return lc(e)})).join(",")+"]";var t=ac.test(e.value),a=ec.test(e.value),n=ac.test(e.value.replace(tc,""));if(e.modifiers){var r="",i="",o=[];for(var s in e.modifiers)if(oc[s])i+=oc[s],nc[s]&&o.push(s);else if("exact"===s){var l=e.modifiers;i+=ic(["ctrl","shift","alt","meta"].filter((function(e){return!l[e]})).map((function(e){return"$event."+e+"Key"})).join("||"))}else o.push(s);o.length&&(r+=cc(o)),i&&(r+=i);var c=t?"return "+e.value+".apply(null, arguments)":a?"return ("+e.value+").apply(null, arguments)":n?"return "+e.value:e.value;return"function($event){"+r+c+"}"}return t||a?e.value:"function($event){"+(n?"return "+e.value:e.value)+"}"}function cc(e){return"if(!$event.type.indexOf('key')&&"+e.map(uc).join("&&")+")return null;"}function uc(e){var t=parseInt(e,10);if(t)return"$event.keyCode!=="+t;var a=nc[e],n=rc[e];return"_k($event.keyCode,"+JSON.stringify(e)+","+JSON.stringify(a)+",$event.key,"+JSON.stringify(n)+")"}function dc(e,t){e.wrapListeners=function(e){return"_g("+e+","+t.value+")"}}function hc(e,t){e.wrapData=function(a){return"_b("+a+",'"+e.tag+"',"+t.value+","+(t.modifiers&&t.modifiers.prop?"true":"false")+(t.modifiers&&t.modifiers.sync?",true":"")+")"}}var fc={on:dc,bind:hc,cloak:j},pc=function(e){this.options=e,this.warn=e.warn||ii,this.transforms=oi(e.modules,"transformCode"),this.dataGenFns=oi(e.modules,"genData"),this.directives=D(D({},fc),e.directives);var t=e.isReservedTag||x;this.maybeComponent=function(e){return!!e.component||!t(e.tag)},this.onceId=0,this.staticRenderFns=[],this.pre=!1};function mc(e,t){var a=new pc(t),n=e?"script"===e.tag?"null":vc(e,a):'_c("div")';return{render:"with(this){return "+n+"}",staticRenderFns:a.staticRenderFns}}function vc(e,t){if(e.parent&&(e.pre=e.pre||e.parent.pre),e.staticRoot&&!e.staticProcessed)return _c(e,t);if(e.once&&!e.onceProcessed)return gc(e,t);if(e.for&&!e.forProcessed)return Mc(e,t);if(e.if&&!e.ifProcessed)return yc(e,t);if("template"!==e.tag||e.slotTarget||t.pre){if("slot"===e.tag)return xc(e,t);var a;if(e.component)a=Bc(e.component,e,t);else{var n;(!e.plain||e.pre&&t.maybeComponent(e))&&(n=zc(e,t));var r=e.inlineTemplate?null:Vc(e,t,!0);a="_c('"+e.tag+"'"+(n?","+n:"")+(r?","+r:"")+")"}for(var i=0;i<t.transforms.length;i++)a=t.transforms[i](e,a);return a}return Vc(e,t)||"void 0"}function _c(e,t){e.staticProcessed=!0;var a=t.pre;return e.pre&&(t.pre=e.pre),t.staticRenderFns.push("with(this){return "+vc(e,t)+"}"),t.pre=a,"_m("+(t.staticRenderFns.length-1)+(e.staticInFor?",true":"")+")"}function gc(e,t){if(e.onceProcessed=!0,e.if&&!e.ifProcessed)return yc(e,t);if(e.staticInFor){var a="",n=e.parent;while(n){if(n.for){a=n.key;break}n=n.parent}return a?"_o("+vc(e,t)+","+t.onceId+++","+a+")":vc(e,t)}return _c(e,t)}function yc(e,t,a,n){return e.ifProcessed=!0,bc(e.ifConditions.slice(),t,a,n)}function bc(e,t,a,n){if(!e.length)return n||"_e()";var r=e.shift();return r.exp?"("+r.exp+")?"+i(r.block)+":"+bc(e,t,a,n):""+i(r.block);function i(e){return a?a(e,t):e.once?gc(e,t):vc(e,t)}}function Mc(e,t,a,n){var r=e.for,i=e.alias,o=e.iterator1?","+e.iterator1:"",s=e.iterator2?","+e.iterator2:"";return e.forProcessed=!0,(n||"_l")+"(("+r+"),function("+i+o+s+"){return "+(a||vc)(e,t)+"})"}function zc(e,t){var a="{",n=Lc(e,t);n&&(a+=n+","),e.key&&(a+="key:"+e.key+","),e.ref&&(a+="ref:"+e.ref+","),e.refInFor&&(a+="refInFor:true,"),e.pre&&(a+="pre:true,"),e.component&&(a+='tag:"'+e.tag+'",');for(var r=0;r<t.dataGenFns.length;r++)a+=t.dataGenFns[r](e);if(e.attrs&&(a+="attrs:"+Yc(e.attrs)+","),e.props&&(a+="domProps:"+Yc(e.props)+","),e.events&&(a+=sc(e.events,!1)+","),e.nativeEvents&&(a+=sc(e.nativeEvents,!0)+","),e.slotTarget&&!e.slotScope&&(a+="slot:"+e.slotTarget+","),e.scopedSlots&&(a+=Hc(e,e.scopedSlots,t)+","),e.model&&(a+="model:{value:"+e.model.value+",callback:"+e.model.callback+",expression:"+e.model.expression+"},"),e.inlineTemplate){var i=wc(e,t);i&&(a+=i+",")}return a=a.replace(/,$/,"")+"}",e.dynamicAttrs&&(a="_b("+a+',"'+e.tag+'",'+Yc(e.dynamicAttrs)+")"),e.wrapData&&(a=e.wrapData(a)),e.wrapListeners&&(a=e.wrapListeners(a)),a}function Lc(e,t){var a=e.directives;if(a){var n,r,i,o,s="directives:[",l=!1;for(n=0,r=a.length;n<r;n++){i=a[n],o=!0;var c=t.directives[i.name];c&&(o=!!c(e,i,t.warn)),o&&(l=!0,s+='{name:"'+i.name+'",rawName:"'+i.rawName+'"'+(i.value?",value:("+i.value+"),expression:"+JSON.stringify(i.value):"")+(i.arg?",arg:"+(i.isDynamicArg?i.arg:'"'+i.arg+'"'):"")+(i.modifiers?",modifiers:"+JSON.stringify(i.modifiers):"")+"},")}return l?s.slice(0,-1)+"]":void 0}}function wc(e,t){var a=e.children[0];if(a&&1===a.type){var n=mc(a,t.options);return"inlineTemplate:{render:function(){"+n.render+"},staticRenderFns:["+n.staticRenderFns.map((function(e){return"function(){"+e+"}"})).join(",")+"]}"}}function Hc(e,t,a){var n=e.for||Object.keys(t).some((function(e){var a=t[e];return a.slotTargetDynamic||a.if||a.for||Ac(a)})),r=!!e.if;if(!n){var i=e.parent;while(i){if(i.slotScope&&i.slotScope!==cl||i.for){n=!0;break}i.if&&(r=!0),i=i.parent}}var o=Object.keys(t).map((function(e){return Sc(t[e],a)})).join(",");return"scopedSlots:_u(["+o+"]"+(n?",null,true":"")+(!n&&r?",null,false,"+kc(o):"")+")"}function kc(e){var t=5381,a=e.length;while(a)t=33*t^e.charCodeAt(--a);return t>>>0}function Ac(e){return 1===e.type&&("slot"===e.tag||e.children.some(Ac))}function Sc(e,t){var a=e.attrsMap["slot-scope"];if(e.if&&!e.ifProcessed&&!a)return yc(e,t,Sc,"null");if(e.for&&!e.forProcessed)return Mc(e,t,Sc);var n=e.slotScope===cl?"":String(e.slotScope),r="function("+n+"){return "+("template"===e.tag?e.if&&a?"("+e.if+")?"+(Vc(e,t)||"undefined")+":undefined":Vc(e,t)||"undefined":vc(e,t))+"}",i=n?"":",proxy:true";return"{key:"+(e.slotTarget||'"default"')+",fn:"+r+i+"}"}function Vc(e,t,a,n,r){var i=e.children;if(i.length){var o=i[0];if(1===i.length&&o.for&&"template"!==o.tag&&"slot"!==o.tag){var s=a?t.maybeComponent(o)?",1":",0":"";return""+(n||vc)(o,t)+s}var l=a?Oc(i,t.maybeComponent):0,c=r||Dc;return"["+i.map((function(e){return c(e,t)})).join(",")+"]"+(l?","+l:"")}}function Oc(e,t){for(var a=0,n=0;n<e.length;n++){var r=e[n];if(1===r.type){if(Tc(r)||r.ifConditions&&r.ifConditions.some((function(e){return Tc(e.block)}))){a=2;break}(t(r)||r.ifConditions&&r.ifConditions.some((function(e){return t(e.block)})))&&(a=1)}}return a}function Tc(e){return void 0!==e.for||"template"===e.tag||"slot"===e.tag}function Dc(e,t){return 1===e.type?vc(e,t):3===e.type&&e.isComment?jc(e):Cc(e)}function Cc(e){return"_v("+(2===e.type?e.expression:Pc(JSON.stringify(e.text)))+")"}function jc(e){return"_e("+JSON.stringify(e.text)+")"}function xc(e,t){var a=e.slotName||'"default"',n=Vc(e,t),r="_t("+a+(n?",function(){return "+n+"}":""),i=e.attrs||e.dynamicAttrs?Yc((e.attrs||[]).concat(e.dynamicAttrs||[]).map((function(e){return{name:w(e.name),value:e.value,dynamic:e.dynamic}}))):null,o=e.attrsMap["v-bind"];return!i&&!o||n||(r+=",null"),i&&(r+=","+i),o&&(r+=(i?"":",null")+","+o),r+")"}function Bc(e,t,a){var n=t.inlineTemplate?null:Vc(t,a,!0);return"_c("+e+","+zc(t,a)+(n?","+n:"")+")"}function Yc(e){for(var t="",a="",n=0;n<e.length;n++){var r=e[n],i=Pc(r.value);r.dynamic?a+=r.name+","+i+",":t+='"'+r.name+'":'+i+","}return t="{"+t.slice(0,-1)+"}",a?"_d("+t+",["+a.slice(0,-1)+"])":t}function Pc(e){return e.replace(/\u2028/g,"\\u2028").replace(/\u2029/g,"\\u2029")}new RegExp("\\b"+"do,if,for,let,new,try,var,case,else,with,await,break,catch,class,const,super,throw,while,yield,delete,export,import,return,switch,default,extends,finally,continue,debugger,function,arguments".split(",").join("\\b|\\b")+"\\b"),new RegExp("\\b"+"delete,typeof,void".split(",").join("\\s*\\([^\\)]*\\)|\\b")+"\\s*\\([^\\)]*\\)");function Ic(e,t){try{return new Function(e)}catch(a){return t.push({err:a,code:e}),j}}function Fc(e){var t=Object.create(null);return function(a,n,r){n=D({},n);n.warn;delete n.warn;var i=n.delimiters?String(n.delimiters)+a:a;if(t[i])return t[i];var o=e(a,n);var s={},l=[];return s.render=Ic(o.render,l),s.staticRenderFns=o.staticRenderFns.map((function(e){return Ic(e,l)})),t[i]=s}}function Ec(e){return function(t){function a(a,n){var r=Object.create(t),i=[],o=[],s=function(e,t,a){(a?o:i).push(e)};if(n)for(var l in n.modules&&(r.modules=(t.modules||[]).concat(n.modules)),n.directives&&(r.directives=D(Object.create(t.directives||null),n.directives)),n)"modules"!==l&&"directives"!==l&&(r[l]=n[l]);r.warn=s;var c=e(a.trim(),r);return c.errors=i,c.tips=o,c}return{compile:a,compileToFunctions:Fc(a)}}}var $c,Rc=Ec((function(e,t){var a=dl(e.trim(),t);!1!==t.optimize&&ql(a,t);var n=mc(a,t);return{ast:a,render:n.render,staticRenderFns:n.staticRenderFns}})),Nc=Rc(Ul),Wc=(Nc.compile,Nc.compileToFunctions);function Uc(e){return $c=$c||document.createElement("div"),$c.innerHTML=e?'<a href="\n"/>':'<div a="\n"/>',$c.innerHTML.indexOf("&#10;")>0}var Gc=!!Z&&Uc(!1),qc=!!Z&&Uc(!0),Jc=z((function(e){var t=pr(e);return t&&t.innerHTML})),Kc=kn.prototype.$mount;function Xc(e){if(e.outerHTML)return e.outerHTML;var t=document.createElement("div");return t.appendChild(e.cloneNode(!0)),t.innerHTML}kn.prototype.$mount=function(e,t){if(e=e&&pr(e),e===document.body||e===document.documentElement)return this;var a=this.$options;if(!a.render){var n=a.template;if(n)if("string"===typeof n)"#"===n.charAt(0)&&(n=Jc(n));else{if(!n.nodeType)return this;n=n.innerHTML}else e&&(n=Xc(e));if(n){0;var r=Wc(n,{outputSourceRange:!1,shouldDecodeNewlines:Gc,shouldDecodeNewlinesForHref:qc,delimiters:a.delimiters,comments:a.comments},this),i=r.render,o=r.staticRenderFns;a.render=i,a.staticRenderFns=o}}return Kc.call(this,e,t)},kn.compile=Wc,t["default"]=kn}.call(this,a("c8ba"))},a04b:function(e,t,a){var n=a("c04e"),r=a("d9b5");e.exports=function(e){var t=n(e,"string");return r(t)?t:t+""}},a356:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},a={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},n=function(e){return function(n,r,i,o){var s=t(n),l=a[e][t(n)];return 2===s&&(l=l[r?0:1]),l.replace(/%d/i,n)}},r=["جانفي","فيفري","مارس","أفريل","ماي","جوان","جويلية","أوت","سبتمبر","أكتوبر","نوفمبر","ديسمبر"],i=e.defineLocale("ar-dz",{months:r,monthsShort:r,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,a){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:n("s"),ss:n("s"),m:n("m"),mm:n("m"),h:n("h"),hh:n("h"),d:n("d"),dd:n("d"),M:n("M"),MM:n("M"),y:n("y"),yy:n("y")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:0,doy:4}});return i}))},a7fa:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("bm",{months:"Zanwuyekalo_Fewuruyekalo_Marisikalo_Awirilikalo_Mɛkalo_Zuwɛnkalo_Zuluyekalo_Utikalo_Sɛtanburukalo_ɔkutɔburukalo_Nowanburukalo_Desanburukalo".split("_"),monthsShort:"Zan_Few_Mar_Awi_Mɛ_Zuw_Zul_Uti_Sɛt_ɔku_Now_Des".split("_"),weekdays:"Kari_Ntɛnɛn_Tarata_Araba_Alamisa_Juma_Sibiri".split("_"),weekdaysShort:"Kar_Ntɛ_Tar_Ara_Ala_Jum_Sib".split("_"),weekdaysMin:"Ka_Nt_Ta_Ar_Al_Ju_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"MMMM [tile] D [san] YYYY",LLL:"MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm",LLLL:"dddd MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm"},calendar:{sameDay:"[Bi lɛrɛ] LT",nextDay:"[Sini lɛrɛ] LT",nextWeek:"dddd [don lɛrɛ] LT",lastDay:"[Kunu lɛrɛ] LT",lastWeek:"dddd [tɛmɛnen lɛrɛ] LT",sameElse:"L"},relativeTime:{future:"%s kɔnɔ",past:"a bɛ %s bɔ",s:"sanga dama dama",ss:"sekondi %d",m:"miniti kelen",mm:"miniti %d",h:"lɛrɛ kelen",hh:"lɛrɛ %d",d:"tile kelen",dd:"tile %d",M:"kalo kelen",MM:"kalo %d",y:"san kelen",yy:"san %d"},week:{dow:1,doy:4}});return t}))},aaf2:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r={s:["थोडया सॅकंडांनी","थोडे सॅकंड"],ss:[e+" सॅकंडांनी",e+" सॅकंड"],m:["एका मिणटान","एक मिनूट"],mm:[e+" मिणटांनी",e+" मिणटां"],h:["एका वरान","एक वर"],hh:[e+" वरांनी",e+" वरां"],d:["एका दिसान","एक दीस"],dd:[e+" दिसांनी",e+" दीस"],M:["एका म्हयन्यान","एक म्हयनो"],MM:[e+" म्हयन्यानी",e+" म्हयने"],y:["एका वर्सान","एक वर्स"],yy:[e+" वर्सांनी",e+" वर्सां"]};return n?r[a][0]:r[a][1]}var a=e.defineLocale("gom-deva",{months:{standalone:"जानेवारी_फेब्रुवारी_मार्च_एप्रील_मे_जून_जुलय_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),format:"जानेवारीच्या_फेब्रुवारीच्या_मार्चाच्या_एप्रीलाच्या_मेयाच्या_जूनाच्या_जुलयाच्या_ऑगस्टाच्या_सप्टेंबराच्या_ऑक्टोबराच्या_नोव्हेंबराच्या_डिसेंबराच्या".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"जाने._फेब्रु._मार्च_एप्री._मे_जून_जुल._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"आयतार_सोमार_मंगळार_बुधवार_बिरेस्तार_सुक्रार_शेनवार".split("_"),weekdaysShort:"आयत._सोम._मंगळ._बुध._ब्रेस्त._सुक्र._शेन.".split("_"),weekdaysMin:"आ_सो_मं_बु_ब्रे_सु_शे".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [वाजतां]",LTS:"A h:mm:ss [वाजतां]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [वाजतां]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [वाजतां]",llll:"ddd, D MMM YYYY, A h:mm [वाजतां]"},calendar:{sameDay:"[आयज] LT",nextDay:"[फाल्यां] LT",nextWeek:"[फुडलो] dddd[,] LT",lastDay:"[काल] LT",lastWeek:"[फाटलो] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s आदीं",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}(वेर)/,ordinal:function(e,t){switch(t){case"D":return e+"वेर";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return e}},week:{dow:0,doy:3},meridiemParse:/राती|सकाळीं|दनपारां|सांजे/,meridiemHour:function(e,t){return 12===e&&(e=0),"राती"===t?e<4?e:e+12:"सकाळीं"===t?e:"दनपारां"===t?e>12?e:e+12:"सांजे"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"राती":e<12?"सकाळीं":e<16?"दनपारां":e<20?"सांजे":"राती"}});return a}))},ada2:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t){var a=e.split("_");return t%10===1&&t%100!==11?a[0]:t%10>=2&&t%10<=4&&(t%100<10||t%100>=20)?a[1]:a[2]}function a(e,a,n){var r={ss:a?"секунда_секунди_секунд":"секунду_секунди_секунд",mm:a?"хвилина_хвилини_хвилин":"хвилину_хвилини_хвилин",hh:a?"година_години_годин":"годину_години_годин",dd:"день_дні_днів",MM:"місяць_місяці_місяців",yy:"рік_роки_років"};return"m"===n?a?"хвилина":"хвилину":"h"===n?a?"година":"годину":e+" "+t(r[n],+e)}function n(e,t){var a,n={nominative:"неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота".split("_"),accusative:"неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу".split("_"),genitive:"неділі_понеділка_вівторка_середи_четверга_п’ятниці_суботи".split("_")};return!0===e?n["nominative"].slice(1,7).concat(n["nominative"].slice(0,1)):e?(a=/(\[[ВвУу]\]) ?dddd/.test(t)?"accusative":/\[?(?:минулої|наступної)? ?\] ?dddd/.test(t)?"genitive":"nominative",n[a][e.day()]):n["nominative"]}function r(e){return function(){return e+"о"+(11===this.hours()?"б":"")+"] LT"}}var i=e.defineLocale("uk",{months:{format:"січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня".split("_"),standalone:"січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень".split("_")},monthsShort:"січ_лют_бер_квіт_трав_черв_лип_серп_вер_жовт_лист_груд".split("_"),weekdays:n,weekdaysShort:"нд_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY р.",LLL:"D MMMM YYYY р., HH:mm",LLLL:"dddd, D MMMM YYYY р., HH:mm"},calendar:{sameDay:r("[Сьогодні "),nextDay:r("[Завтра "),lastDay:r("[Вчора "),nextWeek:r("[У] dddd ["),lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return r("[Минулої] dddd [").call(this);case 1:case 2:case 4:return r("[Минулого] dddd [").call(this)}},sameElse:"L"},relativeTime:{future:"за %s",past:"%s тому",s:"декілька секунд",ss:a,m:a,mm:a,h:"годину",hh:a,d:"день",dd:a,M:"місяць",MM:a,y:"рік",yy:a},meridiemParse:/ночі|ранку|дня|вечора/,isPM:function(e){return/^(дня|вечора)$/.test(e)},meridiem:function(e,t,a){return e<4?"ночі":e<12?"ранку":e<17?"дня":"вечора"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":case"w":case"W":return e+"-й";case"D":return e+"-го";default:return e}},week:{dow:1,doy:7}});return i}))},aed9:function(e,t,a){var n=a("83ab"),r=a("d039");e.exports=n&&r((function(){return 42!=Object.defineProperty((function(){}),"prototype",{value:42,writable:!1}).prototype}))},b29d:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("lo",{months:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),monthsShort:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),weekdays:"ອາທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysShort:"ທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysMin:"ທ_ຈ_ອຄ_ພ_ພຫ_ສກ_ສ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"ວັນdddd D MMMM YYYY HH:mm"},meridiemParse:/ຕອນເຊົ້າ|ຕອນແລງ/,isPM:function(e){return"ຕອນແລງ"===e},meridiem:function(e,t,a){return e<12?"ຕອນເຊົ້າ":"ຕອນແລງ"},calendar:{sameDay:"[ມື້ນີ້ເວລາ] LT",nextDay:"[ມື້ອື່ນເວລາ] LT",nextWeek:"[ວັນ]dddd[ໜ້າເວລາ] LT",lastDay:"[ມື້ວານນີ້ເວລາ] LT",lastWeek:"[ວັນ]dddd[ແລ້ວນີ້ເວລາ] LT",sameElse:"L"},relativeTime:{future:"ອີກ %s",past:"%sຜ່ານມາ",s:"ບໍ່ເທົ່າໃດວິນາທີ",ss:"%d ວິນາທີ",m:"1 ນາທີ",mm:"%d ນາທີ",h:"1 ຊົ່ວໂມງ",hh:"%d ຊົ່ວໂມງ",d:"1 ມື້",dd:"%d ມື້",M:"1 ເດືອນ",MM:"%d ເດືອນ",y:"1 ປີ",yy:"%d ປີ"},dayOfMonthOrdinalParse:/(ທີ່)\d{1,2}/,ordinal:function(e){return"ທີ່"+e}});return t}))},b3eb:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?r[a][0]:r[a][1]}var a=e.defineLocale("de-at",{months:"Jänner_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jän._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a}))},b42e:function(e,t){var a=Math.ceil,n=Math.floor;e.exports=Math.trunc||function(e){var t=+e;return(t>0?n:a)(t)}},b469:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?r[a][0]:r[a][1]}var a=e.defineLocale("de",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a}))},b50d:function(e,t,a){"use strict";var n=a("c532"),r=a("467f"),i=a("7aac"),o=a("30b5"),s=a("83b9"),l=a("c345"),c=a("3934"),u=a("2d83");e.exports=function(e){return new Promise((function(t,a){var d=e.data,h=e.headers,f=e.responseType;n.isFormData(d)&&delete h["Content-Type"];var p=new XMLHttpRequest;if(e.auth){var m=e.auth.username||"",v=e.auth.password?unescape(encodeURIComponent(e.auth.password)):"";h.Authorization="Basic "+btoa(m+":"+v)}var _=s(e.baseURL,e.url);function g(){if(p){var n="getAllResponseHeaders"in p?l(p.getAllResponseHeaders()):null,i=f&&"text"!==f&&"json"!==f?p.response:p.responseText,o={data:i,status:p.status,statusText:p.statusText,headers:n,config:e,request:p};r(t,a,o),p=null}}if(p.open(e.method.toUpperCase(),o(_,e.params,e.paramsSerializer),!0),p.timeout=e.timeout,"onloadend"in p?p.onloadend=g:p.onreadystatechange=function(){p&&4===p.readyState&&(0!==p.status||p.responseURL&&0===p.responseURL.indexOf("file:"))&&setTimeout(g)},p.onabort=function(){p&&(a(u("Request aborted",e,"ECONNABORTED",p)),p=null)},p.onerror=function(){a(u("Network Error",e,null,p)),p=null},p.ontimeout=function(){var t="timeout of "+e.timeout+"ms exceeded";e.timeoutErrorMessage&&(t=e.timeoutErrorMessage),a(u(t,e,e.transitional&&e.transitional.clarifyTimeoutError?"ETIMEDOUT":"ECONNABORTED",p)),p=null},n.isStandardBrowserEnv()){var y=(e.withCredentials||c(_))&&e.xsrfCookieName?i.read(e.xsrfCookieName):void 0;y&&(h[e.xsrfHeaderName]=y)}"setRequestHeader"in p&&n.forEach(h,(function(e,t){"undefined"===typeof d&&"content-type"===t.toLowerCase()?delete h[t]:p.setRequestHeader(t,e)})),n.isUndefined(e.withCredentials)||(p.withCredentials=!!e.withCredentials),f&&"json"!==f&&(p.responseType=e.responseType),"function"===typeof e.onDownloadProgress&&p.addEventListener("progress",e.onDownloadProgress),"function"===typeof e.onUploadProgress&&p.upload&&p.upload.addEventListener("progress",e.onUploadProgress),e.cancelToken&&e.cancelToken.promise.then((function(e){p&&(p.abort(),a(e),p=null)})),d||(d=null),p.send(d)}))}},b53d:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tzm-latn",{months:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),monthsShort:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),weekdays:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysShort:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysMin:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[asdkh g] LT",nextDay:"[aska g] LT",nextWeek:"dddd [g] LT",lastDay:"[assant g] LT",lastWeek:"dddd [g] LT",sameElse:"L"},relativeTime:{future:"dadkh s yan %s",past:"yan %s",s:"imik",ss:"%d imik",m:"minuḍ",mm:"%d minuḍ",h:"saɛa",hh:"%d tassaɛin",d:"ass",dd:"%d ossan",M:"ayowr",MM:"%d iyyirn",y:"asgas",yy:"%d isgasn"},week:{dow:6,doy:12}});return t}))},b540:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("jv",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_Nopember_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nop_Des".split("_"),weekdays:"Minggu_Senen_Seloso_Rebu_Kemis_Jemuwah_Septu".split("_"),weekdaysShort:"Min_Sen_Sel_Reb_Kem_Jem_Sep".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sp".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/enjing|siyang|sonten|ndalu/,meridiemHour:function(e,t){return 12===e&&(e=0),"enjing"===t?e:"siyang"===t?e>=11?e:e+12:"sonten"===t||"ndalu"===t?e+12:void 0},meridiem:function(e,t,a){return e<11?"enjing":e<15?"siyang":e<19?"sonten":"ndalu"},calendar:{sameDay:"[Dinten puniko pukul] LT",nextDay:"[Mbenjang pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kala wingi pukul] LT",lastWeek:"dddd [kepengker pukul] LT",sameElse:"L"},relativeTime:{future:"wonten ing %s",past:"%s ingkang kepengker",s:"sawetawis detik",ss:"%d detik",m:"setunggal menit",mm:"%d menit",h:"setunggal jam",hh:"%d jam",d:"sedinten",dd:"%d dinten",M:"sewulan",MM:"%d wulan",y:"setaun",yy:"%d taun"},week:{dow:1,doy:7}});return t}))},b5b7:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),a="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),n=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],r=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,i=e.defineLocale("es-mx",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,n){return e?/-MMM-/.test(n)?a[e.month()]:t[e.month()]:t},monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:0,doy:4},invalidDate:"Fecha inválida"});return i}))},b622:function(e,t,a){var n=a("da84"),r=a("5692"),i=a("1a2d"),o=a("90e3"),s=a("04f8"),l=a("fdbf"),c=r("wks"),u=n.Symbol,d=u&&u["for"],h=l?u:u&&u.withoutSetter||o;e.exports=function(e){if(!i(c,e)||!s&&"string"!=typeof c[e]){var t="Symbol."+e;s&&i(u,e)?c[e]=u[e]:c[e]=l&&d?d(t):h(t)}return c[e]}},b7e9:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-sg",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:1,doy:4}});return t}))},b84c:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("nn",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"sundag_måndag_tysdag_onsdag_torsdag_fredag_laurdag".split("_"),weekdaysShort:"su._må._ty._on._to._fr._lau.".split("_"),weekdaysMin:"su_må_ty_on_to_fr_la".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[I dag klokka] LT",nextDay:"[I morgon klokka] LT",nextWeek:"dddd [klokka] LT",lastDay:"[I går klokka] LT",lastWeek:"[Føregåande] dddd [klokka] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s sidan",s:"nokre sekund",ss:"%d sekund",m:"eit minutt",mm:"%d minutt",h:"ein time",hh:"%d timar",d:"ein dag",dd:"%d dagar",w:"ei veke",ww:"%d veker",M:"ein månad",MM:"%d månader",y:"eit år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},b97c:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={ss:"sekundes_sekundēm_sekunde_sekundes".split("_"),m:"minūtes_minūtēm_minūte_minūtes".split("_"),mm:"minūtes_minūtēm_minūte_minūtes".split("_"),h:"stundas_stundām_stunda_stundas".split("_"),hh:"stundas_stundām_stunda_stundas".split("_"),d:"dienas_dienām_diena_dienas".split("_"),dd:"dienas_dienām_diena_dienas".split("_"),M:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),MM:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),y:"gada_gadiem_gads_gadi".split("_"),yy:"gada_gadiem_gads_gadi".split("_")};function a(e,t,a){return a?t%10===1&&t%100!==11?e[2]:e[3]:t%10===1&&t%100!==11?e[0]:e[1]}function n(e,n,r){return e+" "+a(t[r],e,n)}function r(e,n,r){return a(t[r],e,n)}function i(e,t){return t?"dažas sekundes":"dažām sekundēm"}var o=e.defineLocale("lv",{months:"janvāris_februāris_marts_aprīlis_maijs_jūnijs_jūlijs_augusts_septembris_oktobris_novembris_decembris".split("_"),monthsShort:"jan_feb_mar_apr_mai_jūn_jūl_aug_sep_okt_nov_dec".split("_"),weekdays:"svētdiena_pirmdiena_otrdiena_trešdiena_ceturtdiena_piektdiena_sestdiena".split("_"),weekdaysShort:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysMin:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY.",LL:"YYYY. [gada] D. MMMM",LLL:"YYYY. [gada] D. MMMM, HH:mm",LLLL:"YYYY. [gada] D. MMMM, dddd, HH:mm"},calendar:{sameDay:"[Šodien pulksten] LT",nextDay:"[Rīt pulksten] LT",nextWeek:"dddd [pulksten] LT",lastDay:"[Vakar pulksten] LT",lastWeek:"[Pagājušā] dddd [pulksten] LT",sameElse:"L"},relativeTime:{future:"pēc %s",past:"pirms %s",s:i,ss:n,m:r,mm:n,h:r,hh:n,d:r,dd:n,M:r,MM:n,y:r,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return o}))},ba4c:function(e,t,a){(function(t){
/*!
 * Vue.js v2.6.14
 * (c) 2014-2021 Evan You
 * Released under the MIT License.
 */
(function(t,a){e.exports=a()})(0,(function(){"use strict";var e=Object.freeze({});function a(e){return void 0===e||null===e}function n(e){return void 0!==e&&null!==e}function r(e){return!0===e}function i(e){return!1===e}function o(e){return"string"===typeof e||"number"===typeof e||"symbol"===typeof e||"boolean"===typeof e}function s(e){return null!==e&&"object"===typeof e}var l=Object.prototype.toString;function c(e){return l.call(e).slice(8,-1)}function u(e){return"[object Object]"===l.call(e)}function d(e){return"[object RegExp]"===l.call(e)}function h(e){var t=parseFloat(String(e));return t>=0&&Math.floor(t)===t&&isFinite(e)}function f(e){return n(e)&&"function"===typeof e.then&&"function"===typeof e.catch}function p(e){return null==e?"":Array.isArray(e)||u(e)&&e.toString===l?JSON.stringify(e,null,2):String(e)}function m(e){var t=parseFloat(e);return isNaN(t)?e:t}function v(e,t){for(var a=Object.create(null),n=e.split(","),r=0;r<n.length;r++)a[n[r]]=!0;return t?function(e){return a[e.toLowerCase()]}:function(e){return a[e]}}var _=v("slot,component",!0),g=v("key,ref,slot,slot-scope,is");function y(e,t){if(e.length){var a=e.indexOf(t);if(a>-1)return e.splice(a,1)}}var b=Object.prototype.hasOwnProperty;function M(e,t){return b.call(e,t)}function z(e){var t=Object.create(null);return function(a){var n=t[a];return n||(t[a]=e(a))}}var L=/-(\w)/g,w=z((function(e){return e.replace(L,(function(e,t){return t?t.toUpperCase():""}))})),H=z((function(e){return e.charAt(0).toUpperCase()+e.slice(1)})),k=/\B([A-Z])/g,A=z((function(e){return e.replace(k,"-$1").toLowerCase()}));function S(e,t){function a(a){var n=arguments.length;return n?n>1?e.apply(t,arguments):e.call(t,a):e.call(t)}return a._length=e.length,a}function V(e,t){return e.bind(t)}var O=Function.prototype.bind?V:S;function T(e,t){t=t||0;var a=e.length-t,n=new Array(a);while(a--)n[a]=e[a+t];return n}function D(e,t){for(var a in t)e[a]=t[a];return e}function C(e){for(var t={},a=0;a<e.length;a++)e[a]&&D(t,e[a]);return t}function j(e,t,a){}var x=function(e,t,a){return!1},B=function(e){return e};function Y(e){return e.reduce((function(e,t){return e.concat(t.staticKeys||[])}),[]).join(",")}function P(e,t){if(e===t)return!0;var a=s(e),n=s(t);if(!a||!n)return!a&&!n&&String(e)===String(t);try{var r=Array.isArray(e),i=Array.isArray(t);if(r&&i)return e.length===t.length&&e.every((function(e,a){return P(e,t[a])}));if(e instanceof Date&&t instanceof Date)return e.getTime()===t.getTime();if(r||i)return!1;var o=Object.keys(e),l=Object.keys(t);return o.length===l.length&&o.every((function(a){return P(e[a],t[a])}))}catch(c){return!1}}function I(e,t){for(var a=0;a<e.length;a++)if(P(e[a],t))return a;return-1}function F(e){var t=!1;return function(){t||(t=!0,e.apply(this,arguments))}}var E="data-server-rendered",$=["component","directive","filter"],R=["beforeCreate","created","beforeMount","mounted","beforeUpdate","updated","beforeDestroy","destroyed","activated","deactivated","errorCaptured","serverPrefetch"],N={optionMergeStrategies:Object.create(null),silent:!1,productionTip:!0,devtools:!0,performance:!1,errorHandler:null,warnHandler:null,ignoredElements:[],keyCodes:Object.create(null),isReservedTag:x,isReservedAttr:x,isUnknownElement:x,getTagNamespace:j,parsePlatformTagName:B,mustUseProp:x,async:!0,_lifecycleHooks:R},W=/a-zA-Z\u00B7\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u037D\u037F-\u1FFF\u200C-\u200D\u203F-\u2040\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD/;function U(e){var t=(e+"").charCodeAt(0);return 36===t||95===t}function G(e,t,a,n){Object.defineProperty(e,t,{value:a,enumerable:!!n,writable:!0,configurable:!0})}var q=new RegExp("[^"+W.source+".$_\\d]");function J(e){if(!q.test(e)){var t=e.split(".");return function(e){for(var a=0;a<t.length;a++){if(!e)return;e=e[t[a]]}return e}}}var K,X="__proto__"in{},Z="undefined"!==typeof window,Q="undefined"!==typeof WXEnvironment&&!!WXEnvironment.platform,ee=Q&&WXEnvironment.platform.toLowerCase(),te=Z&&window.navigator.userAgent.toLowerCase(),ae=te&&/msie|trident/.test(te),ne=te&&te.indexOf("msie 9.0")>0,re=te&&te.indexOf("edge/")>0,ie=(te&&te.indexOf("android"),te&&/iphone|ipad|ipod|ios/.test(te)||"ios"===ee),oe=(te&&/chrome\/\d+/.test(te),te&&/phantomjs/.test(te),te&&te.match(/firefox\/(\d+)/)),se={}.watch,le=!1;if(Z)try{var ce={};Object.defineProperty(ce,"passive",{get:function(){le=!0}}),window.addEventListener("test-passive",null,ce)}catch(qu){}var ue=function(){return void 0===K&&(K=!Z&&!Q&&"undefined"!==typeof t&&(t["process"]&&"server"===t["process"].env.VUE_ENV)),K},de=Z&&window.__VUE_DEVTOOLS_GLOBAL_HOOK__;function he(e){return"function"===typeof e&&/native code/.test(e.toString())}var fe,pe="undefined"!==typeof Symbol&&he(Symbol)&&"undefined"!==typeof Reflect&&he(Reflect.ownKeys);fe="undefined"!==typeof Set&&he(Set)?Set:function(){function e(){this.set=Object.create(null)}return e.prototype.has=function(e){return!0===this.set[e]},e.prototype.add=function(e){this.set[e]=!0},e.prototype.clear=function(){this.set=Object.create(null)},e}();var me=j,ve=j,_e=j,ge=j,ye="undefined"!==typeof console,be=/(?:^|[-_])(\w)/g,Me=function(e){return e.replace(be,(function(e){return e.toUpperCase()})).replace(/[-_]/g,"")};me=function(e,t){var a=t?_e(t):"";N.warnHandler?N.warnHandler.call(null,e,t,a):ye&&!N.silent&&console.error("[Vue warn]: "+e+a)},ve=function(e,t){ye&&!N.silent&&console.warn("[Vue tip]: "+e+(t?_e(t):""))},ge=function(e,t){if(e.$root===e)return"<Root>";var a="function"===typeof e&&null!=e.cid?e.options:e._isVue?e.$options||e.constructor.options:e,n=a.name||a._componentTag,r=a.__file;if(!n&&r){var i=r.match(/([^/\\]+)\.vue$/);n=i&&i[1]}return(n?"<"+Me(n)+">":"<Anonymous>")+(r&&!1!==t?" at "+r:"")};var ze=function(e,t){var a="";while(t)t%2===1&&(a+=e),t>1&&(e+=e),t>>=1;return a};_e=function(e){if(e._isVue&&e.$parent){var t=[],a=0;while(e){if(t.length>0){var n=t[t.length-1];if(n.constructor===e.constructor){a++,e=e.$parent;continue}a>0&&(t[t.length-1]=[n,a],a=0)}t.push(e),e=e.$parent}return"\n\nfound in\n\n"+t.map((function(e,t){return""+(0===t?"---\x3e ":ze(" ",5+2*t))+(Array.isArray(e)?ge(e[0])+"... ("+e[1]+" recursive calls)":ge(e))})).join("\n")}return"\n\n(found in "+ge(e)+")"};var Le=0,we=function(){this.id=Le++,this.subs=[]};we.prototype.addSub=function(e){this.subs.push(e)},we.prototype.removeSub=function(e){y(this.subs,e)},we.prototype.depend=function(){we.target&&we.target.addDep(this)},we.prototype.notify=function(){var e=this.subs.slice();N.async||e.sort((function(e,t){return e.id-t.id}));for(var t=0,a=e.length;t<a;t++)e[t].update()},we.target=null;var He=[];function ke(e){He.push(e),we.target=e}function Ae(){He.pop(),we.target=He[He.length-1]}var Se=function(e,t,a,n,r,i,o,s){this.tag=e,this.data=t,this.children=a,this.text=n,this.elm=r,this.ns=void 0,this.context=i,this.fnContext=void 0,this.fnOptions=void 0,this.fnScopeId=void 0,this.key=t&&t.key,this.componentOptions=o,this.componentInstance=void 0,this.parent=void 0,this.raw=!1,this.isStatic=!1,this.isRootInsert=!0,this.isComment=!1,this.isCloned=!1,this.isOnce=!1,this.asyncFactory=s,this.asyncMeta=void 0,this.isAsyncPlaceholder=!1},Ve={child:{configurable:!0}};Ve.child.get=function(){return this.componentInstance},Object.defineProperties(Se.prototype,Ve);var Oe=function(e){void 0===e&&(e="");var t=new Se;return t.text=e,t.isComment=!0,t};function Te(e){return new Se(void 0,void 0,void 0,String(e))}function De(e){var t=new Se(e.tag,e.data,e.children&&e.children.slice(),e.text,e.elm,e.context,e.componentOptions,e.asyncFactory);return t.ns=e.ns,t.isStatic=e.isStatic,t.key=e.key,t.isComment=e.isComment,t.fnContext=e.fnContext,t.fnOptions=e.fnOptions,t.fnScopeId=e.fnScopeId,t.asyncMeta=e.asyncMeta,t.isCloned=!0,t}var Ce=Array.prototype,je=Object.create(Ce),xe=["push","pop","shift","unshift","splice","sort","reverse"];xe.forEach((function(e){var t=Ce[e];G(je,e,(function(){var a=[],n=arguments.length;while(n--)a[n]=arguments[n];var r,i=t.apply(this,a),o=this.__ob__;switch(e){case"push":case"unshift":r=a;break;case"splice":r=a.slice(2);break}return r&&o.observeArray(r),o.dep.notify(),i}))}));var Be=Object.getOwnPropertyNames(je),Ye=!0;function Pe(e){Ye=e}var Ie=function(e){this.value=e,this.dep=new we,this.vmCount=0,G(e,"__ob__",this),Array.isArray(e)?(X?Fe(e,je):Ee(e,je,Be),this.observeArray(e)):this.walk(e)};function Fe(e,t){e.__proto__=t}function Ee(e,t,a){for(var n=0,r=a.length;n<r;n++){var i=a[n];G(e,i,t[i])}}function $e(e,t){var a;if(s(e)&&!(e instanceof Se))return M(e,"__ob__")&&e.__ob__ instanceof Ie?a=e.__ob__:Ye&&!ue()&&(Array.isArray(e)||u(e))&&Object.isExtensible(e)&&!e._isVue&&(a=new Ie(e)),t&&a&&a.vmCount++,a}function Re(e,t,a,n,r){var i=new we,o=Object.getOwnPropertyDescriptor(e,t);if(!o||!1!==o.configurable){var s=o&&o.get,l=o&&o.set;s&&!l||2!==arguments.length||(a=e[t]);var c=!r&&$e(a);Object.defineProperty(e,t,{enumerable:!0,configurable:!0,get:function(){var t=s?s.call(e):a;return we.target&&(i.depend(),c&&(c.dep.depend(),Array.isArray(t)&&Ue(t))),t},set:function(t){var o=s?s.call(e):a;t===o||t!==t&&o!==o||(n&&n(),s&&!l||(l?l.call(e,t):a=t,c=!r&&$e(t),i.notify()))}})}}function Ne(e,t,n){if((a(e)||o(e))&&me("Cannot set reactive property on undefined, null, or primitive value: "+e),Array.isArray(e)&&h(t))return e.length=Math.max(e.length,t),e.splice(t,1,n),n;if(t in e&&!(t in Object.prototype))return e[t]=n,n;var r=e.__ob__;return e._isVue||r&&r.vmCount?(me("Avoid adding reactive properties to a Vue instance or its root $data at runtime - declare it upfront in the data option."),n):r?(Re(r.value,t,n),r.dep.notify(),n):(e[t]=n,n)}function We(e,t){if((a(e)||o(e))&&me("Cannot delete reactive property on undefined, null, or primitive value: "+e),Array.isArray(e)&&h(t))e.splice(t,1);else{var n=e.__ob__;e._isVue||n&&n.vmCount?me("Avoid deleting properties on a Vue instance or its root $data - just set it to null."):M(e,t)&&(delete e[t],n&&n.dep.notify())}}function Ue(e){for(var t=void 0,a=0,n=e.length;a<n;a++)t=e[a],t&&t.__ob__&&t.__ob__.dep.depend(),Array.isArray(t)&&Ue(t)}Ie.prototype.walk=function(e){for(var t=Object.keys(e),a=0;a<t.length;a++)Re(e,t[a])},Ie.prototype.observeArray=function(e){for(var t=0,a=e.length;t<a;t++)$e(e[t])};var Ge=N.optionMergeStrategies;function qe(e,t){if(!t)return e;for(var a,n,r,i=pe?Reflect.ownKeys(t):Object.keys(t),o=0;o<i.length;o++)a=i[o],"__ob__"!==a&&(n=e[a],r=t[a],M(e,a)?n!==r&&u(n)&&u(r)&&qe(n,r):Ne(e,a,r));return e}function Je(e,t,a){return a?function(){var n="function"===typeof t?t.call(a,a):t,r="function"===typeof e?e.call(a,a):e;return n?qe(n,r):r}:t?e?function(){return qe("function"===typeof t?t.call(this,this):t,"function"===typeof e?e.call(this,this):e)}:t:e}function Ke(e,t){var a=t?e?e.concat(t):Array.isArray(t)?t:[t]:e;return a?Xe(a):a}function Xe(e){for(var t=[],a=0;a<e.length;a++)-1===t.indexOf(e[a])&&t.push(e[a]);return t}function Ze(e,t,a,n){var r=Object.create(e||null);return t?(it(n,t,a),D(r,t)):r}Ge.el=Ge.propsData=function(e,t,a,n){return a||me('option "'+n+'" can only be used during instance creation with the `new` keyword.'),Qe(e,t)},Ge.data=function(e,t,a){return a?Je(e,t,a):t&&"function"!==typeof t?(me('The "data" option should be a function that returns a per-instance value in component definitions.',a),e):Je(e,t)},R.forEach((function(e){Ge[e]=Ke})),$.forEach((function(e){Ge[e+"s"]=Ze})),Ge.watch=function(e,t,a,n){if(e===se&&(e=void 0),t===se&&(t=void 0),!t)return Object.create(e||null);if(it(n,t,a),!e)return t;var r={};for(var i in D(r,e),t){var o=r[i],s=t[i];o&&!Array.isArray(o)&&(o=[o]),r[i]=o?o.concat(s):Array.isArray(s)?s:[s]}return r},Ge.props=Ge.methods=Ge.inject=Ge.computed=function(e,t,a,n){if(t&&it(n,t,a),!e)return t;var r=Object.create(null);return D(r,e),t&&D(r,t),r},Ge.provide=Je;var Qe=function(e,t){return void 0===t?e:t};function et(e){for(var t in e.components)tt(t)}function tt(e){new RegExp("^[a-zA-Z][\\-\\.0-9_"+W.source+"]*$").test(e)||me('Invalid component name: "'+e+'". Component names should conform to valid custom element name in html5 specification.'),(_(e)||N.isReservedTag(e))&&me("Do not use built-in or reserved HTML elements as component id: "+e)}function at(e,t){var a=e.props;if(a){var n,r,i,o={};if(Array.isArray(a)){n=a.length;while(n--)r=a[n],"string"===typeof r?(i=w(r),o[i]={type:null}):me("props must be strings when using array syntax.")}else if(u(a))for(var s in a)r=a[s],i=w(s),o[i]=u(r)?r:{type:r};else me('Invalid value for option "props": expected an Array or an Object, but got '+c(a)+".",t);e.props=o}}function nt(e,t){var a=e.inject;if(a){var n=e.inject={};if(Array.isArray(a))for(var r=0;r<a.length;r++)n[a[r]]={from:a[r]};else if(u(a))for(var i in a){var o=a[i];n[i]=u(o)?D({from:i},o):{from:o}}else me('Invalid value for option "inject": expected an Array or an Object, but got '+c(a)+".",t)}}function rt(e){var t=e.directives;if(t)for(var a in t){var n=t[a];"function"===typeof n&&(t[a]={bind:n,update:n})}}function it(e,t,a){u(t)||me('Invalid value for option "'+e+'": expected an Object, but got '+c(t)+".",a)}function ot(e,t,a){if(et(t),"function"===typeof t&&(t=t.options),at(t,a),nt(t,a),rt(t),!t._base&&(t.extends&&(e=ot(e,t.extends,a)),t.mixins))for(var n=0,r=t.mixins.length;n<r;n++)e=ot(e,t.mixins[n],a);var i,o={};for(i in e)s(i);for(i in t)M(e,i)||s(i);function s(n){var r=Ge[n]||Qe;o[n]=r(e[n],t[n],a,n)}return o}function st(e,t,a,n){if("string"===typeof a){var r=e[t];if(M(r,a))return r[a];var i=w(a);if(M(r,i))return r[i];var o=H(i);if(M(r,o))return r[o];var s=r[a]||r[i]||r[o];return n&&!s&&me("Failed to resolve "+t.slice(0,-1)+": "+a,e),s}}function lt(e,t,a,n){var r=t[e],i=!M(a,e),o=a[e],s=vt(Boolean,r.type);if(s>-1)if(i&&!M(r,"default"))o=!1;else if(""===o||o===A(e)){var l=vt(String,r.type);(l<0||s<l)&&(o=!0)}if(void 0===o){o=ct(n,r,e);var c=Ye;Pe(!0),$e(o),Pe(c)}return ut(r,e,o,n,i),o}function ct(e,t,a){if(M(t,"default")){var n=t.default;return s(n)&&me('Invalid default value for prop "'+a+'": Props with type Object/Array must use a factory function to return the default value.',e),e&&e.$options.propsData&&void 0===e.$options.propsData[a]&&void 0!==e._props[a]?e._props[a]:"function"===typeof n&&"Function"!==pt(t.type)?n.call(e):n}}function ut(e,t,a,n,r){if(e.required&&r)me('Missing required prop: "'+t+'"',n);else if(null!=a||e.required){var i=e.type,o=!i||!0===i,s=[];if(i){Array.isArray(i)||(i=[i]);for(var l=0;l<i.length&&!o;l++){var c=ht(a,i[l],n);s.push(c.expectedType||""),o=c.valid}}var u=s.some((function(e){return e}));if(o||!u){var d=e.validator;d&&(d(a)||me('Invalid prop: custom validator check failed for prop "'+t+'".',n))}else me(_t(t,a,s),n)}}var dt=/^(String|Number|Boolean|Function|Symbol|BigInt)$/;function ht(e,t,a){var n,r=pt(t);if(dt.test(r)){var i=typeof e;n=i===r.toLowerCase(),n||"object"!==i||(n=e instanceof t)}else if("Object"===r)n=u(e);else if("Array"===r)n=Array.isArray(e);else try{n=e instanceof t}catch(qu){me('Invalid prop type: "'+String(t)+'" is not a constructor',a),n=!1}return{valid:n,expectedType:r}}var ft=/^\s*function (\w+)/;function pt(e){var t=e&&e.toString().match(ft);return t?t[1]:""}function mt(e,t){return pt(e)===pt(t)}function vt(e,t){if(!Array.isArray(t))return mt(t,e)?0:-1;for(var a=0,n=t.length;a<n;a++)if(mt(t[a],e))return a;return-1}function _t(e,t,a){var n='Invalid prop: type check failed for prop "'+e+'". Expected '+a.map(H).join(", "),r=a[0],i=c(t);return 1===a.length&&bt(r)&&bt(typeof t)&&!Mt(r,i)&&(n+=" with value "+gt(t,r)),n+=", got "+i+" ",bt(i)&&(n+="with value "+gt(t,i)+"."),n}function gt(e,t){return"String"===t?'"'+e+'"':"Number"===t?""+Number(e):""+e}var yt=["string","number","boolean"];function bt(e){return yt.some((function(t){return e.toLowerCase()===t}))}function Mt(){var e=[],t=arguments.length;while(t--)e[t]=arguments[t];return e.some((function(e){return"boolean"===e.toLowerCase()}))}function zt(e,t,a){ke();try{if(t){var n=t;while(n=n.$parent){var r=n.$options.errorCaptured;if(r)for(var i=0;i<r.length;i++)try{var o=!1===r[i].call(n,e,t,a);if(o)return}catch(qu){wt(qu,n,"errorCaptured hook")}}}wt(e,t,a)}finally{Ae()}}function Lt(e,t,a,n,r){var i;try{i=a?e.apply(t,a):e.call(t),i&&!i._isVue&&f(i)&&!i._handled&&(i.catch((function(e){return zt(e,n,r+" (Promise/async)")})),i._handled=!0)}catch(qu){zt(qu,n,r)}return i}function wt(e,t,a){if(N.errorHandler)try{return N.errorHandler.call(null,e,t,a)}catch(qu){qu!==e&&Ht(qu,null,"config.errorHandler")}Ht(e,t,a)}function Ht(e,t,a){if(me("Error in "+a+': "'+e.toString()+'"',t),!Z&&!Q||"undefined"===typeof console)throw e;console.error(e)}var kt,At,St,Vt=!1,Ot=[],Tt=!1;function Dt(){Tt=!1;var e=Ot.slice(0);Ot.length=0;for(var t=0;t<e.length;t++)e[t]()}if("undefined"!==typeof Promise&&he(Promise)){var Ct=Promise.resolve();kt=function(){Ct.then(Dt),ie&&setTimeout(j)},Vt=!0}else if(ae||"undefined"===typeof MutationObserver||!he(MutationObserver)&&"[object MutationObserverConstructor]"!==MutationObserver.toString())kt="undefined"!==typeof setImmediate&&he(setImmediate)?function(){setImmediate(Dt)}:function(){setTimeout(Dt,0)};else{var jt=1,xt=new MutationObserver(Dt),Bt=document.createTextNode(String(jt));xt.observe(Bt,{characterData:!0}),kt=function(){jt=(jt+1)%2,Bt.data=String(jt)},Vt=!0}function Yt(e,t){var a;if(Ot.push((function(){if(e)try{e.call(t)}catch(qu){zt(qu,t,"nextTick")}else a&&a(t)})),Tt||(Tt=!0,kt()),!e&&"undefined"!==typeof Promise)return new Promise((function(e){a=e}))}var Pt,It=Z&&window.performance;It&&It.mark&&It.measure&&It.clearMarks&&It.clearMeasures&&(At=function(e){return It.mark(e)},St=function(e,t,a){It.measure(e,t,a),It.clearMarks(t),It.clearMarks(a)});var Ft=v("Infinity,undefined,NaN,isFinite,isNaN,parseFloat,parseInt,decodeURI,decodeURIComponent,encodeURI,encodeURIComponent,Math,Number,Date,Array,Object,Boolean,String,RegExp,Map,Set,JSON,Intl,BigInt,require"),Et=function(e,t){me('Property or method "'+t+'" is not defined on the instance but referenced during render. Make sure that this property is reactive, either in the data option, or for class-based components, by initializing the property. See: https://vuejs.org/v2/guide/reactivity.html#Declaring-Reactive-Properties.',e)},$t=function(e,t){me('Property "'+t+'" must be accessed with "$data.'+t+'" because properties starting with "$" or "_" are not proxied in the Vue instance to prevent conflicts with Vue internals. See: https://vuejs.org/v2/api/#data',e)},Rt="undefined"!==typeof Proxy&&he(Proxy);if(Rt){var Nt=v("stop,prevent,self,ctrl,shift,alt,meta,exact");N.keyCodes=new Proxy(N.keyCodes,{set:function(e,t,a){return Nt(t)?(me("Avoid overwriting built-in modifier in config.keyCodes: ."+t),!1):(e[t]=a,!0)}})}var Wt={has:function(e,t){var a=t in e,n=Ft(t)||"string"===typeof t&&"_"===t.charAt(0)&&!(t in e.$data);return a||n||(t in e.$data?$t(e,t):Et(e,t)),a||!n}},Ut={get:function(e,t){return"string"!==typeof t||t in e||(t in e.$data?$t(e,t):Et(e,t)),e[t]}};Pt=function(e){if(Rt){var t=e.$options,a=t.render&&t.render._withStripped?Ut:Wt;e._renderProxy=new Proxy(e,a)}else e._renderProxy=e};var Gt=new fe;function qt(e){Jt(e,Gt),Gt.clear()}function Jt(e,t){var a,n,r=Array.isArray(e);if(!(!r&&!s(e)||Object.isFrozen(e)||e instanceof Se)){if(e.__ob__){var i=e.__ob__.dep.id;if(t.has(i))return;t.add(i)}if(r){a=e.length;while(a--)Jt(e[a],t)}else{n=Object.keys(e),a=n.length;while(a--)Jt(e[n[a]],t)}}}var Kt=z((function(e){var t="&"===e.charAt(0);e=t?e.slice(1):e;var a="~"===e.charAt(0);e=a?e.slice(1):e;var n="!"===e.charAt(0);return e=n?e.slice(1):e,{name:e,once:a,capture:n,passive:t}}));function Xt(e,t){function a(){var e=arguments,n=a.fns;if(!Array.isArray(n))return Lt(n,null,arguments,t,"v-on handler");for(var r=n.slice(),i=0;i<r.length;i++)Lt(r[i],null,e,t,"v-on handler")}return a.fns=e,a}function Zt(e,t,n,i,o,s){var l,c,u,d;for(l in e)c=e[l],u=t[l],d=Kt(l),a(c)?me('Invalid handler for event "'+d.name+'": got '+String(c),s):a(u)?(a(c.fns)&&(c=e[l]=Xt(c,s)),r(d.once)&&(c=e[l]=o(d.name,c,d.capture)),n(d.name,c,d.capture,d.passive,d.params)):c!==u&&(u.fns=c,e[l]=u);for(l in t)a(e[l])&&(d=Kt(l),i(d.name,t[l],d.capture))}function Qt(e,t,i){var o;e instanceof Se&&(e=e.data.hook||(e.data.hook={}));var s=e[t];function l(){i.apply(this,arguments),y(o.fns,l)}a(s)?o=Xt([l]):n(s.fns)&&r(s.merged)?(o=s,o.fns.push(l)):o=Xt([s,l]),o.merged=!0,e[t]=o}function ea(e,t,r){var i=t.options.props;if(!a(i)){var o={},s=e.attrs,l=e.props;if(n(s)||n(l))for(var c in i){var u=A(c),d=c.toLowerCase();c!==d&&s&&M(s,d)&&ve('Prop "'+d+'" is passed to component '+ge(r||t)+', but the declared prop name is "'+c+'". Note that HTML attributes are case-insensitive and camelCased props need to use their kebab-case equivalents when using in-DOM templates. You should probably use "'+u+'" instead of "'+c+'".'),ta(o,l,c,u,!0)||ta(o,s,c,u,!1)}return o}}function ta(e,t,a,r,i){if(n(t)){if(M(t,a))return e[a]=t[a],i||delete t[a],!0;if(M(t,r))return e[a]=t[r],i||delete t[r],!0}return!1}function aa(e){for(var t=0;t<e.length;t++)if(Array.isArray(e[t]))return Array.prototype.concat.apply([],e);return e}function na(e){return o(e)?[Te(e)]:Array.isArray(e)?ia(e):void 0}function ra(e){return n(e)&&n(e.text)&&i(e.isComment)}function ia(e,t){var i,s,l,c,u=[];for(i=0;i<e.length;i++)s=e[i],a(s)||"boolean"===typeof s||(l=u.length-1,c=u[l],Array.isArray(s)?s.length>0&&(s=ia(s,(t||"")+"_"+i),ra(s[0])&&ra(c)&&(u[l]=Te(c.text+s[0].text),s.shift()),u.push.apply(u,s)):o(s)?ra(c)?u[l]=Te(c.text+s):""!==s&&u.push(Te(s)):ra(s)&&ra(c)?u[l]=Te(c.text+s.text):(r(e._isVList)&&n(s.tag)&&a(s.key)&&n(t)&&(s.key="__vlist"+t+"_"+i+"__"),u.push(s)));return u}function oa(e){var t=e.$options.provide;t&&(e._provided="function"===typeof t?t.call(e):t)}function sa(e){var t=la(e.$options.inject,e);t&&(Pe(!1),Object.keys(t).forEach((function(a){Re(e,a,t[a],(function(){me('Avoid mutating an injected value directly since the changes will be overwritten whenever the provided component re-renders. injection being mutated: "'+a+'"',e)}))})),Pe(!0))}function la(e,t){if(e){for(var a=Object.create(null),n=pe?Reflect.ownKeys(e):Object.keys(e),r=0;r<n.length;r++){var i=n[r];if("__ob__"!==i){var o=e[i].from,s=t;while(s){if(s._provided&&M(s._provided,o)){a[i]=s._provided[o];break}s=s.$parent}if(!s)if("default"in e[i]){var l=e[i].default;a[i]="function"===typeof l?l.call(t):l}else me('Injection "'+i+'" not found',t)}}return a}}function ca(e,t){if(!e||!e.length)return{};for(var a={},n=0,r=e.length;n<r;n++){var i=e[n],o=i.data;if(o&&o.attrs&&o.attrs.slot&&delete o.attrs.slot,i.context!==t&&i.fnContext!==t||!o||null==o.slot)(a.default||(a.default=[])).push(i);else{var s=o.slot,l=a[s]||(a[s]=[]);"template"===i.tag?l.push.apply(l,i.children||[]):l.push(i)}}for(var c in a)a[c].every(ua)&&delete a[c];return a}function ua(e){return e.isComment&&!e.asyncFactory||" "===e.text}function da(e){return e.isComment&&e.asyncFactory}function ha(t,a,n){var r,i=Object.keys(a).length>0,o=t?!!t.$stable:!i,s=t&&t.$key;if(t){if(t._normalized)return t._normalized;if(o&&n&&n!==e&&s===n.$key&&!i&&!n.$hasNormal)return n;for(var l in r={},t)t[l]&&"$"!==l[0]&&(r[l]=fa(a,l,t[l]))}else r={};for(var c in a)c in r||(r[c]=pa(a,c));return t&&Object.isExtensible(t)&&(t._normalized=r),G(r,"$stable",o),G(r,"$key",s),G(r,"$hasNormal",i),r}function fa(e,t,a){var n=function(){var e=arguments.length?a.apply(null,arguments):a({});e=e&&"object"===typeof e&&!Array.isArray(e)?[e]:na(e);var t=e&&e[0];return e&&(!t||1===e.length&&t.isComment&&!da(t))?void 0:e};return a.proxy&&Object.defineProperty(e,t,{get:n,enumerable:!0,configurable:!0}),n}function pa(e,t){return function(){return e[t]}}function ma(e,t){var a,r,i,o,l;if(Array.isArray(e)||"string"===typeof e)for(a=new Array(e.length),r=0,i=e.length;r<i;r++)a[r]=t(e[r],r);else if("number"===typeof e)for(a=new Array(e),r=0;r<e;r++)a[r]=t(r+1,r);else if(s(e))if(pe&&e[Symbol.iterator]){a=[];var c=e[Symbol.iterator](),u=c.next();while(!u.done)a.push(t(u.value,a.length)),u=c.next()}else for(o=Object.keys(e),a=new Array(o.length),r=0,i=o.length;r<i;r++)l=o[r],a[r]=t(e[l],l,r);return n(a)||(a=[]),a._isVList=!0,a}function va(e,t,a,n){var r,i=this.$scopedSlots[e];i?(a=a||{},n&&(s(n)||me("slot v-bind without argument expects an Object",this),a=D(D({},n),a)),r=i(a)||("function"===typeof t?t():t)):r=this.$slots[e]||("function"===typeof t?t():t);var o=a&&a.slot;return o?this.$createElement("template",{slot:o},r):r}function _a(e){return st(this.$options,"filters",e,!0)||B}function ga(e,t){return Array.isArray(e)?-1===e.indexOf(t):e!==t}function ya(e,t,a,n,r){var i=N.keyCodes[t]||a;return r&&n&&!N.keyCodes[t]?ga(r,n):i?ga(i,e):n?A(n)!==t:void 0===e}function ba(e,t,a,n,r){if(a)if(s(a)){var i;Array.isArray(a)&&(a=C(a));var o=function(o){if("class"===o||"style"===o||g(o))i=e;else{var s=e.attrs&&e.attrs.type;i=n||N.mustUseProp(t,s,o)?e.domProps||(e.domProps={}):e.attrs||(e.attrs={})}var l=w(o),c=A(o);if(!(l in i)&&!(c in i)&&(i[o]=a[o],r)){var u=e.on||(e.on={});u["update:"+o]=function(e){a[o]=e}}};for(var l in a)o(l)}else me("v-bind without argument expects an Object or Array value",this);return e}function Ma(e,t){var a=this._staticTrees||(this._staticTrees=[]),n=a[e];return n&&!t||(n=a[e]=this.$options.staticRenderFns[e].call(this._renderProxy,null,this),La(n,"__static__"+e,!1)),n}function za(e,t,a){return La(e,"__once__"+t+(a?"_"+a:""),!0),e}function La(e,t,a){if(Array.isArray(e))for(var n=0;n<e.length;n++)e[n]&&"string"!==typeof e[n]&&wa(e[n],t+"_"+n,a);else wa(e,t,a)}function wa(e,t,a){e.isStatic=!0,e.key=t,e.isOnce=a}function Ha(e,t){if(t)if(u(t)){var a=e.on=e.on?D({},e.on):{};for(var n in t){var r=a[n],i=t[n];a[n]=r?[].concat(r,i):i}}else me("v-on without argument expects an Object value",this);return e}function ka(e,t,a,n){t=t||{$stable:!a};for(var r=0;r<e.length;r++){var i=e[r];Array.isArray(i)?ka(i,t,a):i&&(i.proxy&&(i.fn.proxy=!0),t[i.key]=i.fn)}return n&&(t.$key=n),t}function Aa(e,t){for(var a=0;a<t.length;a+=2){var n=t[a];"string"===typeof n&&n?e[t[a]]=t[a+1]:""!==n&&null!==n&&me("Invalid value for dynamic directive argument (expected string or null): "+n,this)}return e}function Sa(e,t){return"string"===typeof e?t+e:e}function Va(e){e._o=za,e._n=m,e._s=p,e._l=ma,e._t=va,e._q=P,e._i=I,e._m=Ma,e._f=_a,e._k=ya,e._b=ba,e._v=Te,e._e=Oe,e._u=ka,e._g=Ha,e._d=Aa,e._p=Sa}function Oa(t,a,n,i,o){var s,l=this,c=o.options;M(i,"_uid")?(s=Object.create(i),s._original=i):(s=i,i=i._original);var u=r(c._compiled),d=!u;this.data=t,this.props=a,this.children=n,this.parent=i,this.listeners=t.on||e,this.injections=la(c.inject,i),this.slots=function(){return l.$slots||ha(t.scopedSlots,l.$slots=ca(n,i)),l.$slots},Object.defineProperty(this,"scopedSlots",{enumerable:!0,get:function(){return ha(t.scopedSlots,this.slots())}}),u&&(this.$options=c,this.$slots=this.slots(),this.$scopedSlots=ha(t.scopedSlots,this.$slots)),c._scopeId?this._c=function(e,t,a,n){var r=Ra(s,e,t,a,n,d);return r&&!Array.isArray(r)&&(r.fnScopeId=c._scopeId,r.fnContext=i),r}:this._c=function(e,t,a,n){return Ra(s,e,t,a,n,d)}}function Ta(t,a,r,i,o){var s=t.options,l={},c=s.props;if(n(c))for(var u in c)l[u]=lt(u,c,a||e);else n(r.attrs)&&Ca(l,r.attrs),n(r.props)&&Ca(l,r.props);var d=new Oa(r,l,o,i,t),h=s.render.call(null,d._c,d);if(h instanceof Se)return Da(h,r,d.parent,s,d);if(Array.isArray(h)){for(var f=na(h)||[],p=new Array(f.length),m=0;m<f.length;m++)p[m]=Da(f[m],r,d.parent,s,d);return p}}function Da(e,t,a,n,r){var i=De(e);return i.fnContext=a,i.fnOptions=n,(i.devtoolsMeta=i.devtoolsMeta||{}).renderContext=r,t.slot&&((i.data||(i.data={})).slot=t.slot),i}function Ca(e,t){for(var a in t)e[w(a)]=t[a]}Va(Oa.prototype);var ja={init:function(e,t){if(e.componentInstance&&!e.componentInstance._isDestroyed&&e.data.keepAlive){var a=e;ja.prepatch(a,a)}else{var n=e.componentInstance=Ya(e,ln);n.$mount(t?e.elm:void 0,t)}},prepatch:function(e,t){var a=t.componentOptions,n=t.componentInstance=e.componentInstance;pn(n,a.propsData,a.listeners,t,a.children)},insert:function(e){var t=e.context,a=e.componentInstance;a._isMounted||(a._isMounted=!0,gn(a,"mounted")),e.data.keepAlive&&(t._isMounted?Cn(a):vn(a,!0))},destroy:function(e){var t=e.componentInstance;t._isDestroyed||(e.data.keepAlive?_n(t,!0):t.$destroy())}},xa=Object.keys(ja);function Ba(e,t,i,o,l){if(!a(e)){var c=i.$options._base;if(s(e)&&(e=c.extend(e)),"function"===typeof e){var u;if(a(e.cid)&&(u=e,e=Qa(u,c),void 0===e))return Za(u,t,i,o,l);t=t||{},ar(e),n(t.model)&&Fa(e.options,t);var d=ea(t,e,l);if(r(e.options.functional))return Ta(e,d,t,i,o);var h=t.on;if(t.on=t.nativeOn,r(e.options.abstract)){var f=t.slot;t={},f&&(t.slot=f)}Pa(t);var p=e.options.name||l,m=new Se("vue-component-"+e.cid+(p?"-"+p:""),t,void 0,void 0,void 0,i,{Ctor:e,propsData:d,listeners:h,tag:l,children:o},u);return m}me("Invalid Component definition: "+String(e),i)}}function Ya(e,t){var a={_isComponent:!0,_parentVnode:e,parent:t},r=e.data.inlineTemplate;return n(r)&&(a.render=r.render,a.staticRenderFns=r.staticRenderFns),new e.componentOptions.Ctor(a)}function Pa(e){for(var t=e.hook||(e.hook={}),a=0;a<xa.length;a++){var n=xa[a],r=t[n],i=ja[n];r===i||r&&r._merged||(t[n]=r?Ia(i,r):i)}}function Ia(e,t){var a=function(a,n){e(a,n),t(a,n)};return a._merged=!0,a}function Fa(e,t){var a=e.model&&e.model.prop||"value",r=e.model&&e.model.event||"input";(t.attrs||(t.attrs={}))[a]=t.model.value;var i=t.on||(t.on={}),o=i[r],s=t.model.callback;n(o)?(Array.isArray(o)?-1===o.indexOf(s):o!==s)&&(i[r]=[s].concat(o)):i[r]=s}var Ea=1,$a=2;function Ra(e,t,a,n,i,s){return(Array.isArray(a)||o(a))&&(i=n,n=a,a=void 0),r(s)&&(i=$a),Na(e,t,a,n,i)}function Na(e,t,a,r,i){if(n(a)&&n(a.__ob__))return me("Avoid using observed data object as vnode data: "+JSON.stringify(a)+"\nAlways create fresh vnode data objects in each render!",e),Oe();if(n(a)&&n(a.is)&&(t=a.is),!t)return Oe();var s,l,c;(n(a)&&n(a.key)&&!o(a.key)&&me("Avoid using non-primitive value as key, use string/number value instead.",e),Array.isArray(r)&&"function"===typeof r[0]&&(a=a||{},a.scopedSlots={default:r[0]},r.length=0),i===$a?r=na(r):i===Ea&&(r=aa(r)),"string"===typeof t)?(l=e.$vnode&&e.$vnode.ns||N.getTagNamespace(t),N.isReservedTag(t)?(n(a)&&n(a.nativeOn)&&"component"!==a.tag&&me("The .native modifier for v-on is only valid on components but it was used on <"+t+">.",e),s=new Se(N.parsePlatformTagName(t),a,r,void 0,void 0,e)):s=a&&a.pre||!n(c=st(e.$options,"components",t))?new Se(t,a,r,void 0,void 0,e):Ba(c,a,e,r,t)):s=Ba(t,a,e,r);return Array.isArray(s)?s:n(s)?(n(l)&&Wa(s,l),n(a)&&Ua(a),s):Oe()}function Wa(e,t,i){if(e.ns=t,"foreignObject"===e.tag&&(t=void 0,i=!0),n(e.children))for(var o=0,s=e.children.length;o<s;o++){var l=e.children[o];n(l.tag)&&(a(l.ns)||r(i)&&"svg"!==l.tag)&&Wa(l,t,i)}}function Ua(e){s(e.style)&&qt(e.style),s(e.class)&&qt(e.class)}function Ga(t){t._vnode=null,t._staticTrees=null;var a=t.$options,n=t.$vnode=a._parentVnode,r=n&&n.context;t.$slots=ca(a._renderChildren,r),t.$scopedSlots=e,t._c=function(e,a,n,r){return Ra(t,e,a,n,r,!1)},t.$createElement=function(e,a,n,r){return Ra(t,e,a,n,r,!0)};var i=n&&n.data;Re(t,"$attrs",i&&i.attrs||e,(function(){!cn&&me("$attrs is readonly.",t)}),!0),Re(t,"$listeners",a._parentListeners||e,(function(){!cn&&me("$listeners is readonly.",t)}),!0)}var qa,Ja=null;function Ka(e){Va(e.prototype),e.prototype.$nextTick=function(e){return Yt(e,this)},e.prototype._render=function(){var e,t=this,a=t.$options,n=a.render,r=a._parentVnode;r&&(t.$scopedSlots=ha(r.data.scopedSlots,t.$slots,t.$scopedSlots)),t.$vnode=r;try{Ja=t,e=n.call(t._renderProxy,t.$createElement)}catch(qu){if(zt(qu,t,"render"),t.$options.renderError)try{e=t.$options.renderError.call(t._renderProxy,t.$createElement,qu)}catch(qu){zt(qu,t,"renderError"),e=t._vnode}else e=t._vnode}finally{Ja=null}return Array.isArray(e)&&1===e.length&&(e=e[0]),e instanceof Se||(Array.isArray(e)&&me("Multiple root nodes returned from render function. Render function should return a single root node.",t),e=Oe()),e.parent=r,e}}function Xa(e,t){return(e.__esModule||pe&&"Module"===e[Symbol.toStringTag])&&(e=e.default),s(e)?t.extend(e):e}function Za(e,t,a,n,r){var i=Oe();return i.asyncFactory=e,i.asyncMeta={data:t,context:a,children:n,tag:r},i}function Qa(e,t){if(r(e.error)&&n(e.errorComp))return e.errorComp;if(n(e.resolved))return e.resolved;var i=Ja;if(i&&n(e.owners)&&-1===e.owners.indexOf(i)&&e.owners.push(i),r(e.loading)&&n(e.loadingComp))return e.loadingComp;if(i&&!n(e.owners)){var o=e.owners=[i],l=!0,c=null,u=null;i.$on("hook:destroyed",(function(){return y(o,i)}));var d=function(e){for(var t=0,a=o.length;t<a;t++)o[t].$forceUpdate();e&&(o.length=0,null!==c&&(clearTimeout(c),c=null),null!==u&&(clearTimeout(u),u=null))},h=F((function(a){e.resolved=Xa(a,t),l?o.length=0:d(!0)})),p=F((function(t){me("Failed to resolve async component: "+String(e)+(t?"\nReason: "+t:"")),n(e.errorComp)&&(e.error=!0,d(!0))})),m=e(h,p);return s(m)&&(f(m)?a(e.resolved)&&m.then(h,p):f(m.component)&&(m.component.then(h,p),n(m.error)&&(e.errorComp=Xa(m.error,t)),n(m.loading)&&(e.loadingComp=Xa(m.loading,t),0===m.delay?e.loading=!0:c=setTimeout((function(){c=null,a(e.resolved)&&a(e.error)&&(e.loading=!0,d(!1))}),m.delay||200)),n(m.timeout)&&(u=setTimeout((function(){u=null,a(e.resolved)&&p("timeout ("+m.timeout+"ms)")}),m.timeout)))),l=!1,e.loading?e.loadingComp:e.resolved}}function en(e){if(Array.isArray(e))for(var t=0;t<e.length;t++){var a=e[t];if(n(a)&&(n(a.componentOptions)||da(a)))return a}}function tn(e){e._events=Object.create(null),e._hasHookEvent=!1;var t=e.$options._parentListeners;t&&on(e,t)}function an(e,t){qa.$on(e,t)}function nn(e,t){qa.$off(e,t)}function rn(e,t){var a=qa;return function n(){var r=t.apply(null,arguments);null!==r&&a.$off(e,n)}}function on(e,t,a){qa=e,Zt(t,a||{},an,nn,rn,e),qa=void 0}function sn(e){var t=/^hook:/;e.prototype.$on=function(e,a){var n=this;if(Array.isArray(e))for(var r=0,i=e.length;r<i;r++)n.$on(e[r],a);else(n._events[e]||(n._events[e]=[])).push(a),t.test(e)&&(n._hasHookEvent=!0);return n},e.prototype.$once=function(e,t){var a=this;function n(){a.$off(e,n),t.apply(a,arguments)}return n.fn=t,a.$on(e,n),a},e.prototype.$off=function(e,t){var a=this;if(!arguments.length)return a._events=Object.create(null),a;if(Array.isArray(e)){for(var n=0,r=e.length;n<r;n++)a.$off(e[n],t);return a}var i,o=a._events[e];if(!o)return a;if(!t)return a._events[e]=null,a;var s=o.length;while(s--)if(i=o[s],i===t||i.fn===t){o.splice(s,1);break}return a},e.prototype.$emit=function(e){var t=this,a=e.toLowerCase();a!==e&&t._events[a]&&ve('Event "'+a+'" is emitted in component '+ge(t)+' but the handler is registered for "'+e+'". Note that HTML attributes are case-insensitive and you cannot use v-on to listen to camelCase events when using in-DOM templates. You should probably use "'+A(e)+'" instead of "'+e+'".');var n=t._events[e];if(n){n=n.length>1?T(n):n;for(var r=T(arguments,1),i='event handler for "'+e+'"',o=0,s=n.length;o<s;o++)Lt(n[o],t,r,t,i)}return t}}var ln=null,cn=!1;function un(e){var t=ln;return ln=e,function(){ln=t}}function dn(e){var t=e.$options,a=t.parent;if(a&&!t.abstract){while(a.$options.abstract&&a.$parent)a=a.$parent;a.$children.push(e)}e.$parent=a,e.$root=a?a.$root:e,e.$children=[],e.$refs={},e._watcher=null,e._inactive=null,e._directInactive=!1,e._isMounted=!1,e._isDestroyed=!1,e._isBeingDestroyed=!1}function hn(e){e.prototype._update=function(e,t){var a=this,n=a.$el,r=a._vnode,i=un(a);a._vnode=e,a.$el=r?a.__patch__(r,e):a.__patch__(a.$el,e,t,!1),i(),n&&(n.__vue__=null),a.$el&&(a.$el.__vue__=a),a.$vnode&&a.$parent&&a.$vnode===a.$parent._vnode&&(a.$parent.$el=a.$el)},e.prototype.$forceUpdate=function(){var e=this;e._watcher&&e._watcher.update()},e.prototype.$destroy=function(){var e=this;if(!e._isBeingDestroyed){gn(e,"beforeDestroy"),e._isBeingDestroyed=!0;var t=e.$parent;!t||t._isBeingDestroyed||e.$options.abstract||y(t.$children,e),e._watcher&&e._watcher.teardown();var a=e._watchers.length;while(a--)e._watchers[a].teardown();e._data.__ob__&&e._data.__ob__.vmCount--,e._isDestroyed=!0,e.__patch__(e._vnode,null),gn(e,"destroyed"),e.$off(),e.$el&&(e.$el.__vue__=null),e.$vnode&&(e.$vnode.parent=null)}}}function fn(e,t,a){var n;return e.$el=t,e.$options.render||(e.$options.render=Oe,e.$options.template&&"#"!==e.$options.template.charAt(0)||e.$options.el||t?me("You are using the runtime-only build of Vue where the template compiler is not available. Either pre-compile the templates into render functions, or use the compiler-included build.",e):me("Failed to mount component: template or render function not defined.",e)),gn(e,"beforeMount"),n=N.performance&&At?function(){var t=e._name,n=e._uid,r="vue-perf-start:"+n,i="vue-perf-end:"+n;At(r);var o=e._render();At(i),St("vue "+t+" render",r,i),At(r),e._update(o,a),At(i),St("vue "+t+" patch",r,i)}:function(){e._update(e._render(),a)},new Yn(e,n,j,{before:function(){e._isMounted&&!e._isDestroyed&&gn(e,"beforeUpdate")}},!0),a=!1,null==e.$vnode&&(e._isMounted=!0,gn(e,"mounted")),e}function pn(t,a,n,r,i){cn=!0;var o=r.data.scopedSlots,s=t.$scopedSlots,l=!!(o&&!o.$stable||s!==e&&!s.$stable||o&&t.$scopedSlots.$key!==o.$key||!o&&t.$scopedSlots.$key),c=!!(i||t.$options._renderChildren||l);if(t.$options._parentVnode=r,t.$vnode=r,t._vnode&&(t._vnode.parent=r),t.$options._renderChildren=i,t.$attrs=r.data.attrs||e,t.$listeners=n||e,a&&t.$options.props){Pe(!1);for(var u=t._props,d=t.$options._propKeys||[],h=0;h<d.length;h++){var f=d[h],p=t.$options.props;u[f]=lt(f,p,a,t)}Pe(!0),t.$options.propsData=a}n=n||e;var m=t.$options._parentListeners;t.$options._parentListeners=n,on(t,n,m),c&&(t.$slots=ca(i,r.context),t.$forceUpdate()),cn=!1}function mn(e){while(e&&(e=e.$parent))if(e._inactive)return!0;return!1}function vn(e,t){if(t){if(e._directInactive=!1,mn(e))return}else if(e._directInactive)return;if(e._inactive||null===e._inactive){e._inactive=!1;for(var a=0;a<e.$children.length;a++)vn(e.$children[a]);gn(e,"activated")}}function _n(e,t){if((!t||(e._directInactive=!0,!mn(e)))&&!e._inactive){e._inactive=!0;for(var a=0;a<e.$children.length;a++)_n(e.$children[a]);gn(e,"deactivated")}}function gn(e,t){ke();var a=e.$options[t],n=t+" hook";if(a)for(var r=0,i=a.length;r<i;r++)Lt(a[r],e,null,e,n);e._hasHookEvent&&e.$emit("hook:"+t),Ae()}var yn=100,bn=[],Mn=[],zn={},Ln={},wn=!1,Hn=!1,kn=0;function An(){kn=bn.length=Mn.length=0,zn={},Ln={},wn=Hn=!1}var Sn=0,Vn=Date.now;if(Z&&!ae){var On=window.performance;On&&"function"===typeof On.now&&Vn()>document.createEvent("Event").timeStamp&&(Vn=function(){return On.now()})}function Tn(){var e,t;for(Sn=Vn(),Hn=!0,bn.sort((function(e,t){return e.id-t.id})),kn=0;kn<bn.length;kn++)if(e=bn[kn],e.before&&e.before(),t=e.id,zn[t]=null,e.run(),null!=zn[t]&&(Ln[t]=(Ln[t]||0)+1,Ln[t]>yn)){me("You may have an infinite update loop "+(e.user?'in watcher with expression "'+e.expression+'"':"in a component render function."),e.vm);break}var a=Mn.slice(),n=bn.slice();An(),jn(a),Dn(n),de&&N.devtools&&de.emit("flush")}function Dn(e){var t=e.length;while(t--){var a=e[t],n=a.vm;n._watcher===a&&n._isMounted&&!n._isDestroyed&&gn(n,"updated")}}function Cn(e){e._inactive=!1,Mn.push(e)}function jn(e){for(var t=0;t<e.length;t++)e[t]._inactive=!0,vn(e[t],!0)}function xn(e){var t=e.id;if(null==zn[t]){if(zn[t]=!0,Hn){var a=bn.length-1;while(a>kn&&bn[a].id>e.id)a--;bn.splice(a+1,0,e)}else bn.push(e);if(!wn){if(wn=!0,!N.async)return void Tn();Yt(Tn)}}}var Bn=0,Yn=function(e,t,a,n,r){this.vm=e,r&&(e._watcher=this),e._watchers.push(this),n?(this.deep=!!n.deep,this.user=!!n.user,this.lazy=!!n.lazy,this.sync=!!n.sync,this.before=n.before):this.deep=this.user=this.lazy=this.sync=!1,this.cb=a,this.id=++Bn,this.active=!0,this.dirty=this.lazy,this.deps=[],this.newDeps=[],this.depIds=new fe,this.newDepIds=new fe,this.expression=t.toString(),"function"===typeof t?this.getter=t:(this.getter=J(t),this.getter||(this.getter=j,me('Failed watching path: "'+t+'" Watcher only accepts simple dot-delimited paths. For full control, use a function instead.',e))),this.value=this.lazy?void 0:this.get()};Yn.prototype.get=function(){var e;ke(this);var t=this.vm;try{e=this.getter.call(t,t)}catch(qu){if(!this.user)throw qu;zt(qu,t,'getter for watcher "'+this.expression+'"')}finally{this.deep&&qt(e),Ae(),this.cleanupDeps()}return e},Yn.prototype.addDep=function(e){var t=e.id;this.newDepIds.has(t)||(this.newDepIds.add(t),this.newDeps.push(e),this.depIds.has(t)||e.addSub(this))},Yn.prototype.cleanupDeps=function(){var e=this.deps.length;while(e--){var t=this.deps[e];this.newDepIds.has(t.id)||t.removeSub(this)}var a=this.depIds;this.depIds=this.newDepIds,this.newDepIds=a,this.newDepIds.clear(),a=this.deps,this.deps=this.newDeps,this.newDeps=a,this.newDeps.length=0},Yn.prototype.update=function(){this.lazy?this.dirty=!0:this.sync?this.run():xn(this)},Yn.prototype.run=function(){if(this.active){var e=this.get();if(e!==this.value||s(e)||this.deep){var t=this.value;if(this.value=e,this.user){var a='callback for watcher "'+this.expression+'"';Lt(this.cb,this.vm,[e,t],this.vm,a)}else this.cb.call(this.vm,e,t)}}},Yn.prototype.evaluate=function(){this.value=this.get(),this.dirty=!1},Yn.prototype.depend=function(){var e=this.deps.length;while(e--)this.deps[e].depend()},Yn.prototype.teardown=function(){if(this.active){this.vm._isBeingDestroyed||y(this.vm._watchers,this);var e=this.deps.length;while(e--)this.deps[e].removeSub(this);this.active=!1}};var Pn={enumerable:!0,configurable:!0,get:j,set:j};function In(e,t,a){Pn.get=function(){return this[t][a]},Pn.set=function(e){this[t][a]=e},Object.defineProperty(e,a,Pn)}function Fn(e){e._watchers=[];var t=e.$options;t.props&&En(e,t.props),t.methods&&Jn(e,t.methods),t.data?$n(e):$e(e._data={},!0),t.computed&&Wn(e,t.computed),t.watch&&t.watch!==se&&Kn(e,t.watch)}function En(e,t){var a=e.$options.propsData||{},n=e._props={},r=e.$options._propKeys=[],i=!e.$parent;i||Pe(!1);var o=function(o){r.push(o);var s=lt(o,t,a,e),l=A(o);(g(l)||N.isReservedAttr(l))&&me('"'+l+'" is a reserved attribute and cannot be used as component prop.',e),Re(n,o,s,(function(){i||cn||me("Avoid mutating a prop directly since the value will be overwritten whenever the parent component re-renders. Instead, use a data or computed property based on the prop's value. Prop being mutated: \""+o+'"',e)})),o in e||In(e,"_props",o)};for(var s in t)o(s);Pe(!0)}function $n(e){var t=e.$options.data;t=e._data="function"===typeof t?Rn(t,e):t||{},u(t)||(t={},me("data functions should return an object:\nhttps://vuejs.org/v2/guide/components.html#data-Must-Be-a-Function",e));var a=Object.keys(t),n=e.$options.props,r=e.$options.methods,i=a.length;while(i--){var o=a[i];r&&M(r,o)&&me('Method "'+o+'" has already been defined as a data property.',e),n&&M(n,o)?me('The data property "'+o+'" is already declared as a prop. Use prop default value instead.',e):U(o)||In(e,"_data",o)}$e(t,!0)}function Rn(e,t){ke();try{return e.call(t,t)}catch(qu){return zt(qu,t,"data()"),{}}finally{Ae()}}var Nn={lazy:!0};function Wn(e,t){var a=e._computedWatchers=Object.create(null),n=ue();for(var r in t){var i=t[r],o="function"===typeof i?i:i.get;null==o&&me('Getter is missing for computed property "'+r+'".',e),n||(a[r]=new Yn(e,o||j,j,Nn)),r in e?r in e.$data?me('The computed property "'+r+'" is already defined in data.',e):e.$options.props&&r in e.$options.props?me('The computed property "'+r+'" is already defined as a prop.',e):e.$options.methods&&r in e.$options.methods&&me('The computed property "'+r+'" is already defined as a method.',e):Un(e,r,i)}}function Un(e,t,a){var n=!ue();"function"===typeof a?(Pn.get=n?Gn(t):qn(a),Pn.set=j):(Pn.get=a.get?n&&!1!==a.cache?Gn(t):qn(a.get):j,Pn.set=a.set||j),Pn.set===j&&(Pn.set=function(){me('Computed property "'+t+'" was assigned to but it has no setter.',this)}),Object.defineProperty(e,t,Pn)}function Gn(e){return function(){var t=this._computedWatchers&&this._computedWatchers[e];if(t)return t.dirty&&t.evaluate(),we.target&&t.depend(),t.value}}function qn(e){return function(){return e.call(this,this)}}function Jn(e,t){var a=e.$options.props;for(var n in t)"function"!==typeof t[n]&&me('Method "'+n+'" has type "'+typeof t[n]+'" in the component definition. Did you reference the function correctly?',e),a&&M(a,n)&&me('Method "'+n+'" has already been defined as a prop.',e),n in e&&U(n)&&me('Method "'+n+'" conflicts with an existing Vue instance method. Avoid defining component methods that start with _ or $.'),e[n]="function"!==typeof t[n]?j:O(t[n],e)}function Kn(e,t){for(var a in t){var n=t[a];if(Array.isArray(n))for(var r=0;r<n.length;r++)Xn(e,a,n[r]);else Xn(e,a,n)}}function Xn(e,t,a,n){return u(a)&&(n=a,a=a.handler),"string"===typeof a&&(a=e[a]),e.$watch(t,a,n)}function Zn(e){var t={get:function(){return this._data}},a={get:function(){return this._props}};t.set=function(){me("Avoid replacing instance root $data. Use nested data properties instead.",this)},a.set=function(){me("$props is readonly.",this)},Object.defineProperty(e.prototype,"$data",t),Object.defineProperty(e.prototype,"$props",a),e.prototype.$set=Ne,e.prototype.$delete=We,e.prototype.$watch=function(e,t,a){var n=this;if(u(t))return Xn(n,e,t,a);a=a||{},a.user=!0;var r=new Yn(n,e,t,a);if(a.immediate){var i='callback for immediate watcher "'+r.expression+'"';ke(),Lt(t,n,[r.value],n,i),Ae()}return function(){r.teardown()}}}var Qn=0;function er(e){e.prototype._init=function(e){var t,a,n=this;n._uid=Qn++,N.performance&&At&&(t="vue-perf-start:"+n._uid,a="vue-perf-end:"+n._uid,At(t)),n._isVue=!0,e&&e._isComponent?tr(n,e):n.$options=ot(ar(n.constructor),e||{},n),Pt(n),n._self=n,dn(n),tn(n),Ga(n),gn(n,"beforeCreate"),sa(n),Fn(n),oa(n),gn(n,"created"),N.performance&&At&&(n._name=ge(n,!1),At(a),St("vue "+n._name+" init",t,a)),n.$options.el&&n.$mount(n.$options.el)}}function tr(e,t){var a=e.$options=Object.create(e.constructor.options),n=t._parentVnode;a.parent=t.parent,a._parentVnode=n;var r=n.componentOptions;a.propsData=r.propsData,a._parentListeners=r.listeners,a._renderChildren=r.children,a._componentTag=r.tag,t.render&&(a.render=t.render,a.staticRenderFns=t.staticRenderFns)}function ar(e){var t=e.options;if(e.super){var a=ar(e.super),n=e.superOptions;if(a!==n){e.superOptions=a;var r=nr(e);r&&D(e.extendOptions,r),t=e.options=ot(a,e.extendOptions),t.name&&(t.components[t.name]=e)}}return t}function nr(e){var t,a=e.options,n=e.sealedOptions;for(var r in a)a[r]!==n[r]&&(t||(t={}),t[r]=a[r]);return t}function rr(e){this instanceof rr||me("Vue is a constructor and should be called with the `new` keyword"),this._init(e)}function ir(e){e.use=function(e){var t=this._installedPlugins||(this._installedPlugins=[]);if(t.indexOf(e)>-1)return this;var a=T(arguments,1);return a.unshift(this),"function"===typeof e.install?e.install.apply(e,a):"function"===typeof e&&e.apply(null,a),t.push(e),this}}function or(e){e.mixin=function(e){return this.options=ot(this.options,e),this}}function sr(e){e.cid=0;var t=1;e.extend=function(e){e=e||{};var a=this,n=a.cid,r=e._Ctor||(e._Ctor={});if(r[n])return r[n];var i=e.name||a.options.name;i&&tt(i);var o=function(e){this._init(e)};return o.prototype=Object.create(a.prototype),o.prototype.constructor=o,o.cid=t++,o.options=ot(a.options,e),o["super"]=a,o.options.props&&lr(o),o.options.computed&&cr(o),o.extend=a.extend,o.mixin=a.mixin,o.use=a.use,$.forEach((function(e){o[e]=a[e]})),i&&(o.options.components[i]=o),o.superOptions=a.options,o.extendOptions=e,o.sealedOptions=D({},o.options),r[n]=o,o}}function lr(e){var t=e.options.props;for(var a in t)In(e.prototype,"_props",a)}function cr(e){var t=e.options.computed;for(var a in t)Un(e.prototype,a,t[a])}function ur(e){$.forEach((function(t){e[t]=function(e,a){return a?("component"===t&&tt(e),"component"===t&&u(a)&&(a.name=a.name||e,a=this.options._base.extend(a)),"directive"===t&&"function"===typeof a&&(a={bind:a,update:a}),this.options[t+"s"][e]=a,a):this.options[t+"s"][e]}}))}function dr(e){return e&&(e.Ctor.options.name||e.tag)}function hr(e,t){return Array.isArray(e)?e.indexOf(t)>-1:"string"===typeof e?e.split(",").indexOf(t)>-1:!!d(e)&&e.test(t)}function fr(e,t){var a=e.cache,n=e.keys,r=e._vnode;for(var i in a){var o=a[i];if(o){var s=o.name;s&&!t(s)&&pr(a,i,n,r)}}}function pr(e,t,a,n){var r=e[t];!r||n&&r.tag===n.tag||r.componentInstance.$destroy(),e[t]=null,y(a,t)}er(rr),Zn(rr),sn(rr),hn(rr),Ka(rr);var mr=[String,RegExp,Array],vr={name:"keep-alive",abstract:!0,props:{include:mr,exclude:mr,max:[String,Number]},methods:{cacheVNode:function(){var e=this,t=e.cache,a=e.keys,n=e.vnodeToCache,r=e.keyToCache;if(n){var i=n.tag,o=n.componentInstance,s=n.componentOptions;t[r]={name:dr(s),tag:i,componentInstance:o},a.push(r),this.max&&a.length>parseInt(this.max)&&pr(t,a[0],a,this._vnode),this.vnodeToCache=null}}},created:function(){this.cache=Object.create(null),this.keys=[]},destroyed:function(){for(var e in this.cache)pr(this.cache,e,this.keys)},mounted:function(){var e=this;this.cacheVNode(),this.$watch("include",(function(t){fr(e,(function(e){return hr(t,e)}))})),this.$watch("exclude",(function(t){fr(e,(function(e){return!hr(t,e)}))}))},updated:function(){this.cacheVNode()},render:function(){var e=this.$slots.default,t=en(e),a=t&&t.componentOptions;if(a){var n=dr(a),r=this,i=r.include,o=r.exclude;if(i&&(!n||!hr(i,n))||o&&n&&hr(o,n))return t;var s=this,l=s.cache,c=s.keys,u=null==t.key?a.Ctor.cid+(a.tag?"::"+a.tag:""):t.key;l[u]?(t.componentInstance=l[u].componentInstance,y(c,u),c.push(u)):(this.vnodeToCache=t,this.keyToCache=u),t.data.keepAlive=!0}return t||e&&e[0]}},_r={KeepAlive:vr};function gr(e){var t={get:function(){return N},set:function(){me("Do not replace the Vue.config object, set individual fields instead.")}};Object.defineProperty(e,"config",t),e.util={warn:me,extend:D,mergeOptions:ot,defineReactive:Re},e.set=Ne,e.delete=We,e.nextTick=Yt,e.observable=function(e){return $e(e),e},e.options=Object.create(null),$.forEach((function(t){e.options[t+"s"]=Object.create(null)})),e.options._base=e,D(e.options.components,_r),ir(e),or(e),sr(e),ur(e)}gr(rr),Object.defineProperty(rr.prototype,"$isServer",{get:ue}),Object.defineProperty(rr.prototype,"$ssrContext",{get:function(){return this.$vnode&&this.$vnode.ssrContext}}),Object.defineProperty(rr,"FunctionalRenderContext",{value:Oa}),rr.version="2.6.14";var yr=v("style,class"),br=v("input,textarea,option,select,progress"),Mr=function(e,t,a){return"value"===a&&br(e)&&"button"!==t||"selected"===a&&"option"===e||"checked"===a&&"input"===e||"muted"===a&&"video"===e},zr=v("contenteditable,draggable,spellcheck"),Lr=v("events,caret,typing,plaintext-only"),wr=function(e,t){return Vr(t)||"false"===t?"false":"contenteditable"===e&&Lr(t)?t:"true"},Hr=v("allowfullscreen,async,autofocus,autoplay,checked,compact,controls,declare,default,defaultchecked,defaultmuted,defaultselected,defer,disabled,enabled,formnovalidate,hidden,indeterminate,inert,ismap,itemscope,loop,multiple,muted,nohref,noresize,noshade,novalidate,nowrap,open,pauseonexit,readonly,required,reversed,scoped,seamless,selected,sortable,truespeed,typemustmatch,visible"),kr="http://www.w3.org/1999/xlink",Ar=function(e){return":"===e.charAt(5)&&"xlink"===e.slice(0,5)},Sr=function(e){return Ar(e)?e.slice(6,e.length):""},Vr=function(e){return null==e||!1===e};function Or(e){var t=e.data,a=e,r=e;while(n(r.componentInstance))r=r.componentInstance._vnode,r&&r.data&&(t=Tr(r.data,t));while(n(a=a.parent))a&&a.data&&(t=Tr(t,a.data));return Dr(t.staticClass,t.class)}function Tr(e,t){return{staticClass:Cr(e.staticClass,t.staticClass),class:n(e.class)?[e.class,t.class]:t.class}}function Dr(e,t){return n(e)||n(t)?Cr(e,jr(t)):""}function Cr(e,t){return e?t?e+" "+t:e:t||""}function jr(e){return Array.isArray(e)?xr(e):s(e)?Br(e):"string"===typeof e?e:""}function xr(e){for(var t,a="",r=0,i=e.length;r<i;r++)n(t=jr(e[r]))&&""!==t&&(a&&(a+=" "),a+=t);return a}function Br(e){var t="";for(var a in e)e[a]&&(t&&(t+=" "),t+=a);return t}var Yr={svg:"http://www.w3.org/2000/svg",math:"http://www.w3.org/1998/Math/MathML"},Pr=v("html,body,base,head,link,meta,style,title,address,article,aside,footer,header,h1,h2,h3,h4,h5,h6,hgroup,nav,section,div,dd,dl,dt,figcaption,figure,picture,hr,img,li,main,ol,p,pre,ul,a,b,abbr,bdi,bdo,br,cite,code,data,dfn,em,i,kbd,mark,q,rp,rt,rtc,ruby,s,samp,small,span,strong,sub,sup,time,u,var,wbr,area,audio,map,track,video,embed,object,param,source,canvas,script,noscript,del,ins,caption,col,colgroup,table,thead,tbody,td,th,tr,button,datalist,fieldset,form,input,label,legend,meter,optgroup,option,output,progress,select,textarea,details,dialog,menu,menuitem,summary,content,element,shadow,template,blockquote,iframe,tfoot"),Ir=v("svg,animate,circle,clippath,cursor,defs,desc,ellipse,filter,font-face,foreignobject,g,glyph,image,line,marker,mask,missing-glyph,path,pattern,polygon,polyline,rect,switch,symbol,text,textpath,tspan,use,view",!0),Fr=function(e){return"pre"===e},Er=function(e){return Pr(e)||Ir(e)};function $r(e){return Ir(e)?"svg":"math"===e?"math":void 0}var Rr=Object.create(null);function Nr(e){if(!Z)return!0;if(Er(e))return!1;if(e=e.toLowerCase(),null!=Rr[e])return Rr[e];var t=document.createElement(e);return e.indexOf("-")>-1?Rr[e]=t.constructor===window.HTMLUnknownElement||t.constructor===window.HTMLElement:Rr[e]=/HTMLUnknownElement/.test(t.toString())}var Wr=v("text,number,password,search,email,tel,url");function Ur(e){if("string"===typeof e){var t=document.querySelector(e);return t||(me("Cannot find element: "+e),document.createElement("div"))}return e}function Gr(e,t){var a=document.createElement(e);return"select"!==e||t.data&&t.data.attrs&&void 0!==t.data.attrs.multiple&&a.setAttribute("multiple","multiple"),a}function qr(e,t){return document.createElementNS(Yr[e],t)}function Jr(e){return document.createTextNode(e)}function Kr(e){return document.createComment(e)}function Xr(e,t,a){e.insertBefore(t,a)}function Zr(e,t){e.removeChild(t)}function Qr(e,t){e.appendChild(t)}function ei(e){return e.parentNode}function ti(e){return e.nextSibling}function ai(e){return e.tagName}function ni(e,t){e.textContent=t}function ri(e,t){e.setAttribute(t,"")}var ii=Object.freeze({createElement:Gr,createElementNS:qr,createTextNode:Jr,createComment:Kr,insertBefore:Xr,removeChild:Zr,appendChild:Qr,parentNode:ei,nextSibling:ti,tagName:ai,setTextContent:ni,setStyleScope:ri}),oi={create:function(e,t){si(t)},update:function(e,t){e.data.ref!==t.data.ref&&(si(e,!0),si(t))},destroy:function(e){si(e,!0)}};function si(e,t){var a=e.data.ref;if(n(a)){var r=e.context,i=e.componentInstance||e.elm,o=r.$refs;t?Array.isArray(o[a])?y(o[a],i):o[a]===i&&(o[a]=void 0):e.data.refInFor?Array.isArray(o[a])?o[a].indexOf(i)<0&&o[a].push(i):o[a]=[i]:o[a]=i}}var li=new Se("",{},[]),ci=["create","activate","update","remove","destroy"];function ui(e,t){return e.key===t.key&&e.asyncFactory===t.asyncFactory&&(e.tag===t.tag&&e.isComment===t.isComment&&n(e.data)===n(t.data)&&di(e,t)||r(e.isAsyncPlaceholder)&&a(t.asyncFactory.error))}function di(e,t){if("input"!==e.tag)return!0;var a,r=n(a=e.data)&&n(a=a.attrs)&&a.type,i=n(a=t.data)&&n(a=a.attrs)&&a.type;return r===i||Wr(r)&&Wr(i)}function hi(e,t,a){var r,i,o={};for(r=t;r<=a;++r)i=e[r].key,n(i)&&(o[i]=r);return o}function fi(e){var t,i,s={},l=e.modules,c=e.nodeOps;for(t=0;t<ci.length;++t)for(s[ci[t]]=[],i=0;i<l.length;++i)n(l[i][ci[t]])&&s[ci[t]].push(l[i][ci[t]]);function u(e){return new Se(c.tagName(e).toLowerCase(),{},[],void 0,e)}function h(e,t){function a(){0===--a.listeners&&f(e)}return a.listeners=t,a}function f(e){var t=c.parentNode(e);n(t)&&c.removeChild(t,e)}function p(e,t){return!t&&!e.ns&&!(N.ignoredElements.length&&N.ignoredElements.some((function(t){return d(t)?t.test(e.tag):t===e.tag})))&&N.isUnknownElement(e.tag)}var m=0;function _(e,t,a,i,o,s,l){if(n(e.elm)&&n(s)&&(e=s[l]=De(e)),e.isRootInsert=!o,!g(e,t,a,i)){var u=e.data,d=e.children,h=e.tag;n(h)?(u&&u.pre&&m++,p(e,m)&&me("Unknown custom element: <"+h+'> - did you register the component correctly? For recursive components, make sure to provide the "name" option.',e.context),e.elm=e.ns?c.createElementNS(e.ns,h):c.createElement(h,e),H(e),z(e,d,t),n(u)&&w(e,t),M(a,e.elm,i),u&&u.pre&&m--):r(e.isComment)?(e.elm=c.createComment(e.text),M(a,e.elm,i)):(e.elm=c.createTextNode(e.text),M(a,e.elm,i))}}function g(e,t,a,i){var o=e.data;if(n(o)){var s=n(e.componentInstance)&&o.keepAlive;if(n(o=o.hook)&&n(o=o.init)&&o(e,!1),n(e.componentInstance))return y(e,t),M(a,e.elm,i),r(s)&&b(e,t,a,i),!0}}function y(e,t){n(e.data.pendingInsert)&&(t.push.apply(t,e.data.pendingInsert),e.data.pendingInsert=null),e.elm=e.componentInstance.$el,L(e)?(w(e,t),H(e)):(si(e),t.push(e))}function b(e,t,a,r){var i,o=e;while(o.componentInstance)if(o=o.componentInstance._vnode,n(i=o.data)&&n(i=i.transition)){for(i=0;i<s.activate.length;++i)s.activate[i](li,o);t.push(o);break}M(a,e.elm,r)}function M(e,t,a){n(e)&&(n(a)?c.parentNode(a)===e&&c.insertBefore(e,t,a):c.appendChild(e,t))}function z(e,t,a){if(Array.isArray(t)){T(t);for(var n=0;n<t.length;++n)_(t[n],a,e.elm,null,!0,t,n)}else o(e.text)&&c.appendChild(e.elm,c.createTextNode(String(e.text)))}function L(e){while(e.componentInstance)e=e.componentInstance._vnode;return n(e.tag)}function w(e,a){for(var r=0;r<s.create.length;++r)s.create[r](li,e);t=e.data.hook,n(t)&&(n(t.create)&&t.create(li,e),n(t.insert)&&a.push(e))}function H(e){var t;if(n(t=e.fnScopeId))c.setStyleScope(e.elm,t);else{var a=e;while(a)n(t=a.context)&&n(t=t.$options._scopeId)&&c.setStyleScope(e.elm,t),a=a.parent}n(t=ln)&&t!==e.context&&t!==e.fnContext&&n(t=t.$options._scopeId)&&c.setStyleScope(e.elm,t)}function k(e,t,a,n,r,i){for(;n<=r;++n)_(a[n],i,e,t,!1,a,n)}function A(e){var t,a,r=e.data;if(n(r))for(n(t=r.hook)&&n(t=t.destroy)&&t(e),t=0;t<s.destroy.length;++t)s.destroy[t](e);if(n(t=e.children))for(a=0;a<e.children.length;++a)A(e.children[a])}function S(e,t,a){for(;t<=a;++t){var r=e[t];n(r)&&(n(r.tag)?(V(r),A(r)):f(r.elm))}}function V(e,t){if(n(t)||n(e.data)){var a,r=s.remove.length+1;for(n(t)?t.listeners+=r:t=h(e.elm,r),n(a=e.componentInstance)&&n(a=a._vnode)&&n(a.data)&&V(a,t),a=0;a<s.remove.length;++a)s.remove[a](e,t);n(a=e.data.hook)&&n(a=a.remove)?a(e,t):t()}else f(e.elm)}function O(e,t,r,i,o){var s,l,u,d,h=0,f=0,p=t.length-1,m=t[0],v=t[p],g=r.length-1,y=r[0],b=r[g],M=!o;T(r);while(h<=p&&f<=g)a(m)?m=t[++h]:a(v)?v=t[--p]:ui(m,y)?(C(m,y,i,r,f),m=t[++h],y=r[++f]):ui(v,b)?(C(v,b,i,r,g),v=t[--p],b=r[--g]):ui(m,b)?(C(m,b,i,r,g),M&&c.insertBefore(e,m.elm,c.nextSibling(v.elm)),m=t[++h],b=r[--g]):ui(v,y)?(C(v,y,i,r,f),M&&c.insertBefore(e,v.elm,m.elm),v=t[--p],y=r[++f]):(a(s)&&(s=hi(t,h,p)),l=n(y.key)?s[y.key]:D(y,t,h,p),a(l)?_(y,i,e,m.elm,!1,r,f):(u=t[l],ui(u,y)?(C(u,y,i,r,f),t[l]=void 0,M&&c.insertBefore(e,u.elm,m.elm)):_(y,i,e,m.elm,!1,r,f)),y=r[++f]);h>p?(d=a(r[g+1])?null:r[g+1].elm,k(e,d,r,f,g,i)):f>g&&S(t,h,p)}function T(e){for(var t={},a=0;a<e.length;a++){var r=e[a],i=r.key;n(i)&&(t[i]?me("Duplicate keys detected: '"+i+"'. This may cause an update error.",r.context):t[i]=!0)}}function D(e,t,a,r){for(var i=a;i<r;i++){var o=t[i];if(n(o)&&ui(e,o))return i}}function C(e,t,i,o,l,u){if(e!==t){n(t.elm)&&n(o)&&(t=o[l]=De(t));var d=t.elm=e.elm;if(r(e.isAsyncPlaceholder))n(t.asyncFactory.resolved)?Y(e.elm,t,i):t.isAsyncPlaceholder=!0;else if(r(t.isStatic)&&r(e.isStatic)&&t.key===e.key&&(r(t.isCloned)||r(t.isOnce)))t.componentInstance=e.componentInstance;else{var h,f=t.data;n(f)&&n(h=f.hook)&&n(h=h.prepatch)&&h(e,t);var p=e.children,m=t.children;if(n(f)&&L(t)){for(h=0;h<s.update.length;++h)s.update[h](e,t);n(h=f.hook)&&n(h=h.update)&&h(e,t)}a(t.text)?n(p)&&n(m)?p!==m&&O(d,p,m,i,u):n(m)?(T(m),n(e.text)&&c.setTextContent(d,""),k(d,null,m,0,m.length-1,i)):n(p)?S(p,0,p.length-1):n(e.text)&&c.setTextContent(d,""):e.text!==t.text&&c.setTextContent(d,t.text),n(f)&&n(h=f.hook)&&n(h=h.postpatch)&&h(e,t)}}}function j(e,t,a){if(r(a)&&n(e.parent))e.parent.data.pendingInsert=t;else for(var i=0;i<t.length;++i)t[i].data.hook.insert(t[i])}var x=!1,B=v("attrs,class,staticClass,staticStyle,key");function Y(e,t,a,i){var o,s=t.tag,l=t.data,c=t.children;if(i=i||l&&l.pre,t.elm=e,r(t.isComment)&&n(t.asyncFactory))return t.isAsyncPlaceholder=!0,!0;if(!P(e,t,i))return!1;if(n(l)&&(n(o=l.hook)&&n(o=o.init)&&o(t,!0),n(o=t.componentInstance)))return y(t,a),!0;if(n(s)){if(n(c))if(e.hasChildNodes())if(n(o=l)&&n(o=o.domProps)&&n(o=o.innerHTML)){if(o!==e.innerHTML)return"undefined"===typeof console||x||(x=!0,console.warn("Parent: ",e),console.warn("server innerHTML: ",o),console.warn("client innerHTML: ",e.innerHTML)),!1}else{for(var u=!0,d=e.firstChild,h=0;h<c.length;h++){if(!d||!Y(d,c[h],a,i)){u=!1;break}d=d.nextSibling}if(!u||d)return"undefined"===typeof console||x||(x=!0,console.warn("Parent: ",e),console.warn("Mismatching childNodes vs. VNodes: ",e.childNodes,c)),!1}else z(t,c,a);if(n(l)){var f=!1;for(var p in l)if(!B(p)){f=!0,w(t,a);break}!f&&l["class"]&&qt(l["class"])}}else e.data!==t.text&&(e.data=t.text);return!0}function P(e,t,a){return n(t.tag)?0===t.tag.indexOf("vue-component")||!p(t,a)&&t.tag.toLowerCase()===(e.tagName&&e.tagName.toLowerCase()):e.nodeType===(t.isComment?8:3)}return function(e,t,i,o){if(!a(t)){var l=!1,d=[];if(a(e))l=!0,_(t,d);else{var h=n(e.nodeType);if(!h&&ui(e,t))C(e,t,d,null,null,o);else{if(h){if(1===e.nodeType&&e.hasAttribute(E)&&(e.removeAttribute(E),i=!0),r(i)){if(Y(e,t,d))return j(t,d,!0),e;me("The client-side rendered virtual DOM tree is not matching server-rendered content. This is likely caused by incorrect HTML markup, for example nesting block-level elements inside <p>, or missing <tbody>. Bailing hydration and performing full client-side render.")}e=u(e)}var f=e.elm,p=c.parentNode(f);if(_(t,d,f._leaveCb?null:p,c.nextSibling(f)),n(t.parent)){var m=t.parent,v=L(t);while(m){for(var g=0;g<s.destroy.length;++g)s.destroy[g](m);if(m.elm=t.elm,v){for(var y=0;y<s.create.length;++y)s.create[y](li,m);var b=m.data.hook.insert;if(b.merged)for(var M=1;M<b.fns.length;M++)b.fns[M]()}else si(m);m=m.parent}}n(p)?S([e],0,0):n(e.tag)&&A(e)}}return j(t,d,l),t.elm}n(e)&&A(e)}}var pi={create:mi,update:mi,destroy:function(e){mi(e,li)}};function mi(e,t){(e.data.directives||t.data.directives)&&vi(e,t)}function vi(e,t){var a,n,r,i=e===li,o=t===li,s=gi(e.data.directives,e.context),l=gi(t.data.directives,t.context),c=[],u=[];for(a in l)n=s[a],r=l[a],n?(r.oldValue=n.value,r.oldArg=n.arg,bi(r,"update",t,e),r.def&&r.def.componentUpdated&&u.push(r)):(bi(r,"bind",t,e),r.def&&r.def.inserted&&c.push(r));if(c.length){var d=function(){for(var a=0;a<c.length;a++)bi(c[a],"inserted",t,e)};i?Qt(t,"insert",d):d()}if(u.length&&Qt(t,"postpatch",(function(){for(var a=0;a<u.length;a++)bi(u[a],"componentUpdated",t,e)})),!i)for(a in s)l[a]||bi(s[a],"unbind",e,e,o)}var _i=Object.create(null);function gi(e,t){var a,n,r=Object.create(null);if(!e)return r;for(a=0;a<e.length;a++)n=e[a],n.modifiers||(n.modifiers=_i),r[yi(n)]=n,n.def=st(t.$options,"directives",n.name,!0);return r}function yi(e){return e.rawName||e.name+"."+Object.keys(e.modifiers||{}).join(".")}function bi(e,t,a,n,r){var i=e.def&&e.def[t];if(i)try{i(a.elm,e,a,n,r)}catch(qu){zt(qu,a.context,"directive "+e.name+" "+t+" hook")}}var Mi=[oi,pi];function zi(e,t){var r=t.componentOptions;if((!n(r)||!1!==r.Ctor.options.inheritAttrs)&&(!a(e.data.attrs)||!a(t.data.attrs))){var i,o,s,l=t.elm,c=e.data.attrs||{},u=t.data.attrs||{};for(i in n(u.__ob__)&&(u=t.data.attrs=D({},u)),u)o=u[i],s=c[i],s!==o&&Li(l,i,o,t.data.pre);for(i in(ae||re)&&u.value!==c.value&&Li(l,"value",u.value),c)a(u[i])&&(Ar(i)?l.removeAttributeNS(kr,Sr(i)):zr(i)||l.removeAttribute(i))}}function Li(e,t,a,n){n||e.tagName.indexOf("-")>-1?wi(e,t,a):Hr(t)?Vr(a)?e.removeAttribute(t):(a="allowfullscreen"===t&&"EMBED"===e.tagName?"true":t,e.setAttribute(t,a)):zr(t)?e.setAttribute(t,wr(t,a)):Ar(t)?Vr(a)?e.removeAttributeNS(kr,Sr(t)):e.setAttributeNS(kr,t,a):wi(e,t,a)}function wi(e,t,a){if(Vr(a))e.removeAttribute(t);else{if(ae&&!ne&&"TEXTAREA"===e.tagName&&"placeholder"===t&&""!==a&&!e.__ieph){var n=function(t){t.stopImmediatePropagation(),e.removeEventListener("input",n)};e.addEventListener("input",n),e.__ieph=!0}e.setAttribute(t,a)}}var Hi={create:zi,update:zi};function ki(e,t){var r=t.elm,i=t.data,o=e.data;if(!(a(i.staticClass)&&a(i.class)&&(a(o)||a(o.staticClass)&&a(o.class)))){var s=Or(t),l=r._transitionClasses;n(l)&&(s=Cr(s,jr(l))),s!==r._prevClass&&(r.setAttribute("class",s),r._prevClass=s)}}var Ai,Si,Vi,Oi,Ti,Di,Ci,ji={create:ki,update:ki},xi=/[\w).+\-_$\]]/;function Bi(e){var t,a,n,r,i,o=!1,s=!1,l=!1,c=!1,u=0,d=0,h=0,f=0;for(n=0;n<e.length;n++)if(a=t,t=e.charCodeAt(n),o)39===t&&92!==a&&(o=!1);else if(s)34===t&&92!==a&&(s=!1);else if(l)96===t&&92!==a&&(l=!1);else if(c)47===t&&92!==a&&(c=!1);else if(124!==t||124===e.charCodeAt(n+1)||124===e.charCodeAt(n-1)||u||d||h){switch(t){case 34:s=!0;break;case 39:o=!0;break;case 96:l=!0;break;case 40:h++;break;case 41:h--;break;case 91:d++;break;case 93:d--;break;case 123:u++;break;case 125:u--;break}if(47===t){for(var p=n-1,m=void 0;p>=0;p--)if(m=e.charAt(p)," "!==m)break;m&&xi.test(m)||(c=!0)}}else void 0===r?(f=n+1,r=e.slice(0,n).trim()):v();function v(){(i||(i=[])).push(e.slice(f,n).trim()),f=n+1}if(void 0===r?r=e.slice(0,n).trim():0!==f&&v(),i)for(n=0;n<i.length;n++)r=Yi(r,i[n]);return r}function Yi(e,t){var a=t.indexOf("(");if(a<0)return'_f("'+t+'")('+e+")";var n=t.slice(0,a),r=t.slice(a+1);return'_f("'+n+'")('+e+(")"!==r?","+r:r)}function Pi(e,t){console.error("[Vue compiler]: "+e)}function Ii(e,t){return e?e.map((function(e){return e[t]})).filter((function(e){return e})):[]}function Fi(e,t,a,n,r){(e.props||(e.props=[])).push(Ki({name:t,value:a,dynamic:r},n)),e.plain=!1}function Ei(e,t,a,n,r){var i=r?e.dynamicAttrs||(e.dynamicAttrs=[]):e.attrs||(e.attrs=[]);i.push(Ki({name:t,value:a,dynamic:r},n)),e.plain=!1}function $i(e,t,a,n){e.attrsMap[t]=a,e.attrsList.push(Ki({name:t,value:a},n))}function Ri(e,t,a,n,r,i,o,s){(e.directives||(e.directives=[])).push(Ki({name:t,rawName:a,value:n,arg:r,isDynamicArg:i,modifiers:o},s)),e.plain=!1}function Ni(e,t,a){return a?"_p("+t+',"'+e+'")':e+t}function Wi(t,a,n,r,i,o,s,l){var c;r=r||e,o&&r.prevent&&r.passive&&o("passive and prevent can't be used together. Passive handler can't prevent default event.",s),r.right?l?a="("+a+")==='click'?'contextmenu':("+a+")":"click"===a&&(a="contextmenu",delete r.right):r.middle&&(l?a="("+a+")==='click'?'mouseup':("+a+")":"click"===a&&(a="mouseup")),r.capture&&(delete r.capture,a=Ni("!",a,l)),r.once&&(delete r.once,a=Ni("~",a,l)),r.passive&&(delete r.passive,a=Ni("&",a,l)),r.native?(delete r.native,c=t.nativeEvents||(t.nativeEvents={})):c=t.events||(t.events={});var u=Ki({value:n.trim(),dynamic:l},s);r!==e&&(u.modifiers=r);var d=c[a];Array.isArray(d)?i?d.unshift(u):d.push(u):c[a]=d?i?[u,d]:[d,u]:u,t.plain=!1}function Ui(e,t){return e.rawAttrsMap[":"+t]||e.rawAttrsMap["v-bind:"+t]||e.rawAttrsMap[t]}function Gi(e,t,a){var n=qi(e,":"+t)||qi(e,"v-bind:"+t);if(null!=n)return Bi(n);if(!1!==a){var r=qi(e,t);if(null!=r)return JSON.stringify(r)}}function qi(e,t,a){var n;if(null!=(n=e.attrsMap[t]))for(var r=e.attrsList,i=0,o=r.length;i<o;i++)if(r[i].name===t){r.splice(i,1);break}return a&&delete e.attrsMap[t],n}function Ji(e,t){for(var a=e.attrsList,n=0,r=a.length;n<r;n++){var i=a[n];if(t.test(i.name))return a.splice(n,1),i}}function Ki(e,t){return t&&(null!=t.start&&(e.start=t.start),null!=t.end&&(e.end=t.end)),e}function Xi(e,t,a){var n=a||{},r=n.number,i=n.trim,o="$$v",s=o;i&&(s="(typeof "+o+" === 'string'? "+o+".trim(): "+o+")"),r&&(s="_n("+s+")");var l=Zi(t,s);e.model={value:"("+t+")",expression:JSON.stringify(t),callback:"function ("+o+") {"+l+"}"}}function Zi(e,t){var a=Qi(e);return null===a.key?e+"="+t:"$set("+a.exp+", "+a.key+", "+t+")"}function Qi(e){if(e=e.trim(),Ai=e.length,e.indexOf("[")<0||e.lastIndexOf("]")<Ai-1)return Oi=e.lastIndexOf("."),Oi>-1?{exp:e.slice(0,Oi),key:'"'+e.slice(Oi+1)+'"'}:{exp:e,key:null};Si=e,Oi=Ti=Di=0;while(!to())Vi=eo(),ao(Vi)?ro(Vi):91===Vi&&no(Vi);return{exp:e.slice(0,Ti),key:e.slice(Ti+1,Di)}}function eo(){return Si.charCodeAt(++Oi)}function to(){return Oi>=Ai}function ao(e){return 34===e||39===e}function no(e){var t=1;Ti=Oi;while(!to())if(e=eo(),ao(e))ro(e);else if(91===e&&t++,93===e&&t--,0===t){Di=Oi;break}}function ro(e){var t=e;while(!to())if(e=eo(),e===t)break}var io,oo="__r",so="__c";function lo(e,t,a){Ci=a;var n=t.value,r=t.modifiers,i=e.tag,o=e.attrsMap.type;if("input"===i&&"file"===o&&Ci("<"+e.tag+' v-model="'+n+'" type="file">:\nFile inputs are read only. Use a v-on:change listener instead.',e.rawAttrsMap["v-model"]),e.component)return Xi(e,n,r),!1;if("select"===i)ho(e,n,r);else if("input"===i&&"checkbox"===o)co(e,n,r);else if("input"===i&&"radio"===o)uo(e,n,r);else if("input"===i||"textarea"===i)fo(e,n,r);else{if(!N.isReservedTag(i))return Xi(e,n,r),!1;Ci("<"+e.tag+' v-model="'+n+"\">: v-model is not supported on this element type. If you are working with contenteditable, it's recommended to wrap a library dedicated for that purpose inside a custom component.",e.rawAttrsMap["v-model"])}return!0}function co(e,t,a){var n=a&&a.number,r=Gi(e,"value")||"null",i=Gi(e,"true-value")||"true",o=Gi(e,"false-value")||"false";Fi(e,"checked","Array.isArray("+t+")?_i("+t+","+r+")>-1"+("true"===i?":("+t+")":":_q("+t+","+i+")")),Wi(e,"change","var $$a="+t+",$$el=$event.target,$$c=$$el.checked?("+i+"):("+o+");if(Array.isArray($$a)){var $$v="+(n?"_n("+r+")":r)+",$$i=_i($$a,$$v);if($$el.checked){$$i<0&&("+Zi(t,"$$a.concat([$$v])")+")}else{$$i>-1&&("+Zi(t,"$$a.slice(0,$$i).concat($$a.slice($$i+1))")+")}}else{"+Zi(t,"$$c")+"}",null,!0)}function uo(e,t,a){var n=a&&a.number,r=Gi(e,"value")||"null";r=n?"_n("+r+")":r,Fi(e,"checked","_q("+t+","+r+")"),Wi(e,"change",Zi(t,r),null,!0)}function ho(e,t,a){var n=a&&a.number,r='Array.prototype.filter.call($event.target.options,function(o){return o.selected}).map(function(o){var val = "_value" in o ? o._value : o.value;return '+(n?"_n(val)":"val")+"})",i="$event.target.multiple ? $$selectedVal : $$selectedVal[0]",o="var $$selectedVal = "+r+";";o=o+" "+Zi(t,i),Wi(e,"change",o,null,!0)}function fo(e,t,a){var n=e.attrsMap.type,r=e.attrsMap["v-bind:value"]||e.attrsMap[":value"],i=e.attrsMap["v-bind:type"]||e.attrsMap[":type"];if(r&&!i){var o=e.attrsMap["v-bind:value"]?"v-bind:value":":value";Ci(o+'="'+r+'" conflicts with v-model on the same element because the latter already expands to a value binding internally',e.rawAttrsMap[o])}var s=a||{},l=s.lazy,c=s.number,u=s.trim,d=!l&&"range"!==n,h=l?"change":"range"===n?oo:"input",f="$event.target.value";u&&(f="$event.target.value.trim()"),c&&(f="_n("+f+")");var p=Zi(t,f);d&&(p="if($event.target.composing)return;"+p),Fi(e,"value","("+t+")"),Wi(e,h,p,null,!0),(u||c)&&Wi(e,"blur","$forceUpdate()")}function po(e){if(n(e[oo])){var t=ae?"change":"input";e[t]=[].concat(e[oo],e[t]||[]),delete e[oo]}n(e[so])&&(e.change=[].concat(e[so],e.change||[]),delete e[so])}function mo(e,t,a){var n=io;return function r(){var i=t.apply(null,arguments);null!==i&&go(e,r,a,n)}}var vo=Vt&&!(oe&&Number(oe[1])<=53);function _o(e,t,a,n){if(vo){var r=Sn,i=t;t=i._wrapper=function(e){if(e.target===e.currentTarget||e.timeStamp>=r||e.timeStamp<=0||e.target.ownerDocument!==document)return i.apply(this,arguments)}}io.addEventListener(e,t,le?{capture:a,passive:n}:a)}function go(e,t,a,n){(n||io).removeEventListener(e,t._wrapper||t,a)}function yo(e,t){if(!a(e.data.on)||!a(t.data.on)){var n=t.data.on||{},r=e.data.on||{};io=t.elm,po(n),Zt(n,r,_o,go,mo,t.context),io=void 0}}var bo,Mo={create:yo,update:yo};function zo(e,t){if(!a(e.data.domProps)||!a(t.data.domProps)){var r,i,o=t.elm,s=e.data.domProps||{},l=t.data.domProps||{};for(r in n(l.__ob__)&&(l=t.data.domProps=D({},l)),s)r in l||(o[r]="");for(r in l){if(i=l[r],"textContent"===r||"innerHTML"===r){if(t.children&&(t.children.length=0),i===s[r])continue;1===o.childNodes.length&&o.removeChild(o.childNodes[0])}if("value"===r&&"PROGRESS"!==o.tagName){o._value=i;var c=a(i)?"":String(i);Lo(o,c)&&(o.value=c)}else if("innerHTML"===r&&Ir(o.tagName)&&a(o.innerHTML)){bo=bo||document.createElement("div"),bo.innerHTML="<svg>"+i+"</svg>";var u=bo.firstChild;while(o.firstChild)o.removeChild(o.firstChild);while(u.firstChild)o.appendChild(u.firstChild)}else if(i!==s[r])try{o[r]=i}catch(qu){}}}}function Lo(e,t){return!e.composing&&("OPTION"===e.tagName||wo(e,t)||Ho(e,t))}function wo(e,t){var a=!0;try{a=document.activeElement!==e}catch(qu){}return a&&e.value!==t}function Ho(e,t){var a=e.value,r=e._vModifiers;if(n(r)){if(r.number)return m(a)!==m(t);if(r.trim)return a.trim()!==t.trim()}return a!==t}var ko={create:zo,update:zo},Ao=z((function(e){var t={},a=/;(?![^(]*\))/g,n=/:(.+)/;return e.split(a).forEach((function(e){if(e){var a=e.split(n);a.length>1&&(t[a[0].trim()]=a[1].trim())}})),t}));function So(e){var t=Vo(e.style);return e.staticStyle?D(e.staticStyle,t):t}function Vo(e){return Array.isArray(e)?C(e):"string"===typeof e?Ao(e):e}function Oo(e,t){var a,n={};if(t){var r=e;while(r.componentInstance)r=r.componentInstance._vnode,r&&r.data&&(a=So(r.data))&&D(n,a)}(a=So(e.data))&&D(n,a);var i=e;while(i=i.parent)i.data&&(a=So(i.data))&&D(n,a);return n}var To,Do=/^--/,Co=/\s*!important$/,jo=function(e,t,a){if(Do.test(t))e.style.setProperty(t,a);else if(Co.test(a))e.style.setProperty(A(t),a.replace(Co,""),"important");else{var n=Bo(t);if(Array.isArray(a))for(var r=0,i=a.length;r<i;r++)e.style[n]=a[r];else e.style[n]=a}},xo=["Webkit","Moz","ms"],Bo=z((function(e){if(To=To||document.createElement("div").style,e=w(e),"filter"!==e&&e in To)return e;for(var t=e.charAt(0).toUpperCase()+e.slice(1),a=0;a<xo.length;a++){var n=xo[a]+t;if(n in To)return n}}));function Yo(e,t){var r=t.data,i=e.data;if(!(a(r.staticStyle)&&a(r.style)&&a(i.staticStyle)&&a(i.style))){var o,s,l=t.elm,c=i.staticStyle,u=i.normalizedStyle||i.style||{},d=c||u,h=Vo(t.data.style)||{};t.data.normalizedStyle=n(h.__ob__)?D({},h):h;var f=Oo(t,!0);for(s in d)a(f[s])&&jo(l,s,"");for(s in f)o=f[s],o!==d[s]&&jo(l,s,null==o?"":o)}}var Po={create:Yo,update:Yo},Io=/\s+/;function Fo(e,t){if(t&&(t=t.trim()))if(e.classList)t.indexOf(" ")>-1?t.split(Io).forEach((function(t){return e.classList.add(t)})):e.classList.add(t);else{var a=" "+(e.getAttribute("class")||"")+" ";a.indexOf(" "+t+" ")<0&&e.setAttribute("class",(a+t).trim())}}function Eo(e,t){if(t&&(t=t.trim()))if(e.classList)t.indexOf(" ")>-1?t.split(Io).forEach((function(t){return e.classList.remove(t)})):e.classList.remove(t),e.classList.length||e.removeAttribute("class");else{var a=" "+(e.getAttribute("class")||"")+" ",n=" "+t+" ";while(a.indexOf(n)>=0)a=a.replace(n," ");a=a.trim(),a?e.setAttribute("class",a):e.removeAttribute("class")}}function $o(e){if(e){if("object"===typeof e){var t={};return!1!==e.css&&D(t,Ro(e.name||"v")),D(t,e),t}return"string"===typeof e?Ro(e):void 0}}var Ro=z((function(e){return{enterClass:e+"-enter",enterToClass:e+"-enter-to",enterActiveClass:e+"-enter-active",leaveClass:e+"-leave",leaveToClass:e+"-leave-to",leaveActiveClass:e+"-leave-active"}})),No=Z&&!ne,Wo="transition",Uo="animation",Go="transition",qo="transitionend",Jo="animation",Ko="animationend";No&&(void 0===window.ontransitionend&&void 0!==window.onwebkittransitionend&&(Go="WebkitTransition",qo="webkitTransitionEnd"),void 0===window.onanimationend&&void 0!==window.onwebkitanimationend&&(Jo="WebkitAnimation",Ko="webkitAnimationEnd"));var Xo=Z?window.requestAnimationFrame?window.requestAnimationFrame.bind(window):setTimeout:function(e){return e()};function Zo(e){Xo((function(){Xo(e)}))}function Qo(e,t){var a=e._transitionClasses||(e._transitionClasses=[]);a.indexOf(t)<0&&(a.push(t),Fo(e,t))}function es(e,t){e._transitionClasses&&y(e._transitionClasses,t),Eo(e,t)}function ts(e,t,a){var n=ns(e,t),r=n.type,i=n.timeout,o=n.propCount;if(!r)return a();var s=r===Wo?qo:Ko,l=0,c=function(){e.removeEventListener(s,u),a()},u=function(t){t.target===e&&++l>=o&&c()};setTimeout((function(){l<o&&c()}),i+1),e.addEventListener(s,u)}var as=/\b(transform|all)(,|$)/;function ns(e,t){var a,n=window.getComputedStyle(e),r=(n[Go+"Delay"]||"").split(", "),i=(n[Go+"Duration"]||"").split(", "),o=rs(r,i),s=(n[Jo+"Delay"]||"").split(", "),l=(n[Jo+"Duration"]||"").split(", "),c=rs(s,l),u=0,d=0;t===Wo?o>0&&(a=Wo,u=o,d=i.length):t===Uo?c>0&&(a=Uo,u=c,d=l.length):(u=Math.max(o,c),a=u>0?o>c?Wo:Uo:null,d=a?a===Wo?i.length:l.length:0);var h=a===Wo&&as.test(n[Go+"Property"]);return{type:a,timeout:u,propCount:d,hasTransform:h}}function rs(e,t){while(e.length<t.length)e=e.concat(e);return Math.max.apply(null,t.map((function(t,a){return is(t)+is(e[a])})))}function is(e){return 1e3*Number(e.slice(0,-1).replace(",","."))}function os(e,t){var r=e.elm;n(r._leaveCb)&&(r._leaveCb.cancelled=!0,r._leaveCb());var i=$o(e.data.transition);if(!a(i)&&!n(r._enterCb)&&1===r.nodeType){var o=i.css,l=i.type,c=i.enterClass,u=i.enterToClass,d=i.enterActiveClass,h=i.appearClass,f=i.appearToClass,p=i.appearActiveClass,v=i.beforeEnter,_=i.enter,g=i.afterEnter,y=i.enterCancelled,b=i.beforeAppear,M=i.appear,z=i.afterAppear,L=i.appearCancelled,w=i.duration,H=ln,k=ln.$vnode;while(k&&k.parent)H=k.context,k=k.parent;var A=!H._isMounted||!e.isRootInsert;if(!A||M||""===M){var S=A&&h?h:c,V=A&&p?p:d,O=A&&f?f:u,T=A&&b||v,D=A&&"function"===typeof M?M:_,C=A&&z||g,j=A&&L||y,x=m(s(w)?w.enter:w);null!=x&&ls(x,"enter",e);var B=!1!==o&&!ne,Y=us(D),P=r._enterCb=F((function(){B&&(es(r,O),es(r,V)),P.cancelled?(B&&es(r,S),j&&j(r)):C&&C(r),r._enterCb=null}));e.data.show||Qt(e,"insert",(function(){var t=r.parentNode,a=t&&t._pending&&t._pending[e.key];a&&a.tag===e.tag&&a.elm._leaveCb&&a.elm._leaveCb(),D&&D(r,P)})),T&&T(r),B&&(Qo(r,S),Qo(r,V),Zo((function(){es(r,S),P.cancelled||(Qo(r,O),Y||(cs(x)?setTimeout(P,x):ts(r,l,P)))}))),e.data.show&&(t&&t(),D&&D(r,P)),B||Y||P()}}}function ss(e,t){var r=e.elm;n(r._enterCb)&&(r._enterCb.cancelled=!0,r._enterCb());var i=$o(e.data.transition);if(a(i)||1!==r.nodeType)return t();if(!n(r._leaveCb)){var o=i.css,l=i.type,c=i.leaveClass,u=i.leaveToClass,d=i.leaveActiveClass,h=i.beforeLeave,f=i.leave,p=i.afterLeave,v=i.leaveCancelled,_=i.delayLeave,g=i.duration,y=!1!==o&&!ne,b=us(f),M=m(s(g)?g.leave:g);n(M)&&ls(M,"leave",e);var z=r._leaveCb=F((function(){r.parentNode&&r.parentNode._pending&&(r.parentNode._pending[e.key]=null),y&&(es(r,u),es(r,d)),z.cancelled?(y&&es(r,c),v&&v(r)):(t(),p&&p(r)),r._leaveCb=null}));_?_(L):L()}function L(){z.cancelled||(!e.data.show&&r.parentNode&&((r.parentNode._pending||(r.parentNode._pending={}))[e.key]=e),h&&h(r),y&&(Qo(r,c),Qo(r,d),Zo((function(){es(r,c),z.cancelled||(Qo(r,u),b||(cs(M)?setTimeout(z,M):ts(r,l,z)))}))),f&&f(r,z),y||b||z())}}function ls(e,t,a){"number"!==typeof e?me("<transition> explicit "+t+" duration is not a valid number - got "+JSON.stringify(e)+".",a.context):isNaN(e)&&me("<transition> explicit "+t+" duration is NaN - the duration expression might be incorrect.",a.context)}function cs(e){return"number"===typeof e&&!isNaN(e)}function us(e){if(a(e))return!1;var t=e.fns;return n(t)?us(Array.isArray(t)?t[0]:t):(e._length||e.length)>1}function ds(e,t){!0!==t.data.show&&os(t)}var hs=Z?{create:ds,activate:ds,remove:function(e,t){!0!==e.data.show?ss(e,t):t()}}:{},fs=[Hi,ji,Mo,ko,Po,hs],ps=fs.concat(Mi),ms=fi({nodeOps:ii,modules:ps});ne&&document.addEventListener("selectionchange",(function(){var e=document.activeElement;e&&e.vmodel&&Ls(e,"input")}));var vs={inserted:function(e,t,a,n){"select"===a.tag?(n.elm&&!n.elm._vOptions?Qt(a,"postpatch",(function(){vs.componentUpdated(e,t,a)})):_s(e,t,a.context),e._vOptions=[].map.call(e.options,bs)):("textarea"===a.tag||Wr(e.type))&&(e._vModifiers=t.modifiers,t.modifiers.lazy||(e.addEventListener("compositionstart",Ms),e.addEventListener("compositionend",zs),e.addEventListener("change",zs),ne&&(e.vmodel=!0)))},componentUpdated:function(e,t,a){if("select"===a.tag){_s(e,t,a.context);var n=e._vOptions,r=e._vOptions=[].map.call(e.options,bs);if(r.some((function(e,t){return!P(e,n[t])}))){var i=e.multiple?t.value.some((function(e){return ys(e,r)})):t.value!==t.oldValue&&ys(t.value,r);i&&Ls(e,"change")}}}};function _s(e,t,a){gs(e,t,a),(ae||re)&&setTimeout((function(){gs(e,t,a)}),0)}function gs(e,t,a){var n=t.value,r=e.multiple;if(!r||Array.isArray(n)){for(var i,o,s=0,l=e.options.length;s<l;s++)if(o=e.options[s],r)i=I(n,bs(o))>-1,o.selected!==i&&(o.selected=i);else if(P(bs(o),n))return void(e.selectedIndex!==s&&(e.selectedIndex=s));r||(e.selectedIndex=-1)}else me('<select multiple v-model="'+t.expression+'"> expects an Array value for its binding, but got '+Object.prototype.toString.call(n).slice(8,-1),a)}function ys(e,t){return t.every((function(t){return!P(t,e)}))}function bs(e){return"_value"in e?e._value:e.value}function Ms(e){e.target.composing=!0}function zs(e){e.target.composing&&(e.target.composing=!1,Ls(e.target,"input"))}function Ls(e,t){var a=document.createEvent("HTMLEvents");a.initEvent(t,!0,!0),e.dispatchEvent(a)}function ws(e){return!e.componentInstance||e.data&&e.data.transition?e:ws(e.componentInstance._vnode)}var Hs={bind:function(e,t,a){var n=t.value;a=ws(a);var r=a.data&&a.data.transition,i=e.__vOriginalDisplay="none"===e.style.display?"":e.style.display;n&&r?(a.data.show=!0,os(a,(function(){e.style.display=i}))):e.style.display=n?i:"none"},update:function(e,t,a){var n=t.value,r=t.oldValue;if(!n!==!r){a=ws(a);var i=a.data&&a.data.transition;i?(a.data.show=!0,n?os(a,(function(){e.style.display=e.__vOriginalDisplay})):ss(a,(function(){e.style.display="none"}))):e.style.display=n?e.__vOriginalDisplay:"none"}},unbind:function(e,t,a,n,r){r||(e.style.display=e.__vOriginalDisplay)}},ks={model:vs,show:Hs},As={name:String,appear:Boolean,css:Boolean,mode:String,type:String,enterClass:String,leaveClass:String,enterToClass:String,leaveToClass:String,enterActiveClass:String,leaveActiveClass:String,appearClass:String,appearActiveClass:String,appearToClass:String,duration:[Number,String,Object]};function Ss(e){var t=e&&e.componentOptions;return t&&t.Ctor.options.abstract?Ss(en(t.children)):e}function Vs(e){var t={},a=e.$options;for(var n in a.propsData)t[n]=e[n];var r=a._parentListeners;for(var i in r)t[w(i)]=r[i];return t}function Os(e,t){if(/\d-keep-alive$/.test(t.tag))return e("keep-alive",{props:t.componentOptions.propsData})}function Ts(e){while(e=e.parent)if(e.data.transition)return!0}function Ds(e,t){return t.key===e.key&&t.tag===e.tag}var Cs=function(e){return e.tag||da(e)},js=function(e){return"show"===e.name},xs={name:"transition",props:As,abstract:!0,render:function(e){var t=this,a=this.$slots.default;if(a&&(a=a.filter(Cs),a.length)){a.length>1&&me("<transition> can only be used on a single element. Use <transition-group> for lists.",this.$parent);var n=this.mode;n&&"in-out"!==n&&"out-in"!==n&&me("invalid <transition> mode: "+n,this.$parent);var r=a[0];if(Ts(this.$vnode))return r;var i=Ss(r);if(!i)return r;if(this._leaving)return Os(e,r);var s="__transition-"+this._uid+"-";i.key=null==i.key?i.isComment?s+"comment":s+i.tag:o(i.key)?0===String(i.key).indexOf(s)?i.key:s+i.key:i.key;var l=(i.data||(i.data={})).transition=Vs(this),c=this._vnode,u=Ss(c);if(i.data.directives&&i.data.directives.some(js)&&(i.data.show=!0),u&&u.data&&!Ds(i,u)&&!da(u)&&(!u.componentInstance||!u.componentInstance._vnode.isComment)){var d=u.data.transition=D({},l);if("out-in"===n)return this._leaving=!0,Qt(d,"afterLeave",(function(){t._leaving=!1,t.$forceUpdate()})),Os(e,r);if("in-out"===n){if(da(i))return c;var h,f=function(){h()};Qt(l,"afterEnter",f),Qt(l,"enterCancelled",f),Qt(d,"delayLeave",(function(e){h=e}))}}return r}}},Bs=D({tag:String,moveClass:String},As);delete Bs.mode;var Ys={props:Bs,beforeMount:function(){var e=this,t=this._update;this._update=function(a,n){var r=un(e);e.__patch__(e._vnode,e.kept,!1,!0),e._vnode=e.kept,r(),t.call(e,a,n)}},render:function(e){for(var t=this.tag||this.$vnode.data.tag||"span",a=Object.create(null),n=this.prevChildren=this.children,r=this.$slots.default||[],i=this.children=[],o=Vs(this),s=0;s<r.length;s++){var l=r[s];if(l.tag)if(null!=l.key&&0!==String(l.key).indexOf("__vlist"))i.push(l),a[l.key]=l,(l.data||(l.data={})).transition=o;else{var c=l.componentOptions,u=c?c.Ctor.options.name||c.tag||"":l.tag;me("<transition-group> children must be keyed: <"+u+">")}}if(n){for(var d=[],h=[],f=0;f<n.length;f++){var p=n[f];p.data.transition=o,p.data.pos=p.elm.getBoundingClientRect(),a[p.key]?d.push(p):h.push(p)}this.kept=e(t,null,d),this.removed=h}return e(t,null,i)},updated:function(){var e=this.prevChildren,t=this.moveClass||(this.name||"v")+"-move";e.length&&this.hasMove(e[0].elm,t)&&(e.forEach(Ps),e.forEach(Is),e.forEach(Fs),this._reflow=document.body.offsetHeight,e.forEach((function(e){if(e.data.moved){var a=e.elm,n=a.style;Qo(a,t),n.transform=n.WebkitTransform=n.transitionDuration="",a.addEventListener(qo,a._moveCb=function e(n){n&&n.target!==a||n&&!/transform$/.test(n.propertyName)||(a.removeEventListener(qo,e),a._moveCb=null,es(a,t))})}})))},methods:{hasMove:function(e,t){if(!No)return!1;if(this._hasMove)return this._hasMove;var a=e.cloneNode();e._transitionClasses&&e._transitionClasses.forEach((function(e){Eo(a,e)})),Fo(a,t),a.style.display="none",this.$el.appendChild(a);var n=ns(a);return this.$el.removeChild(a),this._hasMove=n.hasTransform}}};function Ps(e){e.elm._moveCb&&e.elm._moveCb(),e.elm._enterCb&&e.elm._enterCb()}function Is(e){e.data.newPos=e.elm.getBoundingClientRect()}function Fs(e){var t=e.data.pos,a=e.data.newPos,n=t.left-a.left,r=t.top-a.top;if(n||r){e.data.moved=!0;var i=e.elm.style;i.transform=i.WebkitTransform="translate("+n+"px,"+r+"px)",i.transitionDuration="0s"}}var Es={Transition:xs,TransitionGroup:Ys};rr.config.mustUseProp=Mr,rr.config.isReservedTag=Er,rr.config.isReservedAttr=yr,rr.config.getTagNamespace=$r,rr.config.isUnknownElement=Nr,D(rr.options.directives,ks),D(rr.options.components,Es),rr.prototype.__patch__=Z?ms:j,rr.prototype.$mount=function(e,t){return e=e&&Z?Ur(e):void 0,fn(this,e,t)},Z&&setTimeout((function(){N.devtools&&(de?de.emit("init",rr):console[console.info?"info":"log"]("Download the Vue Devtools extension for a better development experience:\nhttps://github.com/vuejs/vue-devtools")),!1!==N.productionTip&&"undefined"!==typeof console&&console[console.info?"info":"log"]("You are running Vue in development mode.\nMake sure to turn on production mode when deploying for production.\nSee more tips at https://vuejs.org/guide/deployment.html")}),0);var $s=/\{\{((?:.|\r?\n)+?)\}\}/g,Rs=/[-.*+?^${}()|[\]\/\\]/g,Ns=z((function(e){var t=e[0].replace(Rs,"\\$&"),a=e[1].replace(Rs,"\\$&");return new RegExp(t+"((?:.|\\n)+?)"+a,"g")}));function Ws(e,t){var a=t?Ns(t):$s;if(a.test(e)){var n,r,i,o=[],s=[],l=a.lastIndex=0;while(n=a.exec(e)){r=n.index,r>l&&(s.push(i=e.slice(l,r)),o.push(JSON.stringify(i)));var c=Bi(n[1].trim());o.push("_s("+c+")"),s.push({"@binding":c}),l=r+n[0].length}return l<e.length&&(s.push(i=e.slice(l)),o.push(JSON.stringify(i))),{expression:o.join("+"),tokens:s}}}function Us(e,t){var a=t.warn||Pi,n=qi(e,"class");if(n){var r=Ws(n,t.delimiters);r&&a('class="'+n+'": Interpolation inside attributes has been removed. Use v-bind or the colon shorthand instead. For example, instead of <div class="{{ val }}">, use <div :class="val">.',e.rawAttrsMap["class"])}n&&(e.staticClass=JSON.stringify(n));var i=Gi(e,"class",!1);i&&(e.classBinding=i)}function Gs(e){var t="";return e.staticClass&&(t+="staticClass:"+e.staticClass+","),e.classBinding&&(t+="class:"+e.classBinding+","),t}var qs={staticKeys:["staticClass"],transformNode:Us,genData:Gs};function Js(e,t){var a=t.warn||Pi,n=qi(e,"style");if(n){var r=Ws(n,t.delimiters);r&&a('style="'+n+'": Interpolation inside attributes has been removed. Use v-bind or the colon shorthand instead. For example, instead of <div style="{{ val }}">, use <div :style="val">.',e.rawAttrsMap["style"]),e.staticStyle=JSON.stringify(Ao(n))}var i=Gi(e,"style",!1);i&&(e.styleBinding=i)}function Ks(e){var t="";return e.staticStyle&&(t+="staticStyle:"+e.staticStyle+","),e.styleBinding&&(t+="style:("+e.styleBinding+"),"),t}var Xs,Zs={staticKeys:["staticStyle"],transformNode:Js,genData:Ks},Qs={decode:function(e){return Xs=Xs||document.createElement("div"),Xs.innerHTML=e,Xs.textContent}},el=v("area,base,br,col,embed,frame,hr,img,input,isindex,keygen,link,meta,param,source,track,wbr"),tl=v("colgroup,dd,dt,li,options,p,td,tfoot,th,thead,tr,source"),al=v("address,article,aside,base,blockquote,body,caption,col,colgroup,dd,details,dialog,div,dl,dt,fieldset,figcaption,figure,footer,form,h1,h2,h3,h4,h5,h6,head,header,hgroup,hr,html,legend,li,menuitem,meta,optgroup,option,param,rp,rt,source,style,summary,tbody,td,tfoot,th,thead,title,tr,track"),nl=/^\s*([^\s"'<>\/=]+)(?:\s*(=)\s*(?:"([^"]*)"+|'([^']*)'+|([^\s"'=<>`]+)))?/,rl=/^\s*((?:v-[\w-]+:|@|:|#)\[[^=]+?\][^\s"'<>\/=]*)(?:\s*(=)\s*(?:"([^"]*)"+|'([^']*)'+|([^\s"'=<>`]+)))?/,il="[a-zA-Z_][\\-\\.0-9_a-zA-Z"+W.source+"]*",ol="((?:"+il+"\\:)?"+il+")",sl=new RegExp("^<"+ol),ll=/^\s*(\/?)>/,cl=new RegExp("^<\\/"+ol+"[^>]*>"),ul=/^<!DOCTYPE [^>]+>/i,dl=/^<!\--/,hl=/^<!\[/,fl=v("script,style,textarea",!0),pl={},ml={"&lt;":"<","&gt;":">","&quot;":'"',"&amp;":"&","&#10;":"\n","&#9;":"\t","&#39;":"'"},vl=/&(?:lt|gt|quot|amp|#39);/g,_l=/&(?:lt|gt|quot|amp|#39|#10|#9);/g,gl=v("pre,textarea",!0),yl=function(e,t){return e&&gl(e)&&"\n"===t[0]};function bl(e,t){var a=t?_l:vl;return e.replace(a,(function(e){return ml[e]}))}function Ml(e,t){var a,n,r=[],i=t.expectHTML,o=t.isUnaryTag||x,s=t.canBeLeftOpenTag||x,l=0;while(e){if(a=e,n&&fl(n)){var c=0,u=n.toLowerCase(),d=pl[u]||(pl[u]=new RegExp("([\\s\\S]*?)(</"+u+"[^>]*>)","i")),h=e.replace(d,(function(e,a,n){return c=n.length,fl(u)||"noscript"===u||(a=a.replace(/<!\--([\s\S]*?)-->/g,"$1").replace(/<!\[CDATA\[([\s\S]*?)]]>/g,"$1")),yl(u,a)&&(a=a.slice(1)),t.chars&&t.chars(a),""}));l+=e.length-h.length,e=h,k(u,l-c,l)}else{var f=e.indexOf("<");if(0===f){if(dl.test(e)){var p=e.indexOf("--\x3e");if(p>=0){t.shouldKeepComment&&t.comment(e.substring(4,p),l,l+p+3),L(p+3);continue}}if(hl.test(e)){var m=e.indexOf("]>");if(m>=0){L(m+2);continue}}var v=e.match(ul);if(v){L(v[0].length);continue}var _=e.match(cl);if(_){var g=l;L(_[0].length),k(_[1],g,l);continue}var y=w();if(y){H(y),yl(y.tagName,e)&&L(1);continue}}var b=void 0,M=void 0,z=void 0;if(f>=0){M=e.slice(f);while(!cl.test(M)&&!sl.test(M)&&!dl.test(M)&&!hl.test(M)){if(z=M.indexOf("<",1),z<0)break;f+=z,M=e.slice(f)}b=e.substring(0,f)}f<0&&(b=e),b&&L(b.length),t.chars&&b&&t.chars(b,l-b.length,l)}if(e===a){t.chars&&t.chars(e),!r.length&&t.warn&&t.warn('Mal-formatted tag at end of template: "'+e+'"',{start:l+e.length});break}}function L(t){l+=t,e=e.substring(t)}function w(){var t=e.match(sl);if(t){var a,n,r={tagName:t[1],attrs:[],start:l};L(t[0].length);while(!(a=e.match(ll))&&(n=e.match(rl)||e.match(nl)))n.start=l,L(n[0].length),n.end=l,r.attrs.push(n);if(a)return r.unarySlash=a[1],L(a[0].length),r.end=l,r}}function H(e){var a=e.tagName,l=e.unarySlash;i&&("p"===n&&al(a)&&k(n),s(a)&&n===a&&k(a));for(var c=o(a)||!!l,u=e.attrs.length,d=new Array(u),h=0;h<u;h++){var f=e.attrs[h],p=f[3]||f[4]||f[5]||"",m="a"===a&&"href"===f[1]?t.shouldDecodeNewlinesForHref:t.shouldDecodeNewlines;d[h]={name:f[1],value:bl(p,m)},t.outputSourceRange&&(d[h].start=f.start+f[0].match(/^\s*/).length,d[h].end=f.end)}c||(r.push({tag:a,lowerCasedTag:a.toLowerCase(),attrs:d,start:e.start,end:e.end}),n=a),t.start&&t.start(a,d,c,e.start,e.end)}function k(e,a,i){var o,s;if(null==a&&(a=l),null==i&&(i=l),e){for(s=e.toLowerCase(),o=r.length-1;o>=0;o--)if(r[o].lowerCasedTag===s)break}else o=0;if(o>=0){for(var c=r.length-1;c>=o;c--)(c>o||!e&&t.warn)&&t.warn("tag <"+r[c].tag+"> has no matching end tag.",{start:r[c].start,end:r[c].end}),t.end&&t.end(r[c].tag,a,i);r.length=o,n=o&&r[o-1].tag}else"br"===s?t.start&&t.start(e,[],!0,a,i):"p"===s&&(t.start&&t.start(e,[],!1,a,i),t.end&&t.end(e,a,i))}k()}var zl,Ll,wl,Hl,kl,Al,Sl,Vl,Ol,Tl=/^@|^v-on:/,Dl=/^v-|^@|^:|^#/,Cl=/([\s\S]*?)\s+(?:in|of)\s+([\s\S]*)/,jl=/,([^,\}\]]*)(?:,([^,\}\]]*))?$/,xl=/^\(|\)$/g,Bl=/^\[.*\]$/,Yl=/:(.*)$/,Pl=/^:|^\.|^v-bind:/,Il=/\.[^.\]]+(?=[^\]]*$)/g,Fl=/^v-slot(:|$)|^#/,El=/[\r\n]/,$l=/[ \f\t\r\n]+/g,Rl=/[\s"'<>\/=]/,Nl=z(Qs.decode),Wl="_empty_";function Ul(e,t,a){return{type:1,tag:e,attrsList:t,attrsMap:fc(t),rawAttrsMap:{},parent:a,children:[]}}function Gl(e,t){zl=t.warn||Pi,Al=t.isPreTag||x,Sl=t.mustUseProp||x,Vl=t.getTagNamespace||x;var a=t.isReservedTag||x;Ol=function(e){return!(!(e.component||e.attrsMap[":is"]||e.attrsMap["v-bind:is"])&&(e.attrsMap.is?a(e.attrsMap.is):a(e.tag)))},wl=Ii(t.modules,"transformNode"),Hl=Ii(t.modules,"preTransformNode"),kl=Ii(t.modules,"postTransformNode"),Ll=t.delimiters;var n,r,i=[],o=!1!==t.preserveWhitespace,s=t.whitespace,l=!1,c=!1,u=!1;function d(e,t){u||(u=!0,zl(e,t))}function h(e){if(f(e),l||e.processed||(e=Kl(e,t)),i.length||e===n||(n.if&&(e.elseif||e.else)?(p(e),rc(n,{exp:e.elseif,block:e})):d("Component template should contain exactly one root element. If you are using v-if on multiple elements, use v-else-if to chain them instead.",{start:e.start})),r&&!e.forbidden)if(e.elseif||e.else)ac(e,r);else{if(e.slotScope){var a=e.slotTarget||'"default"';(r.scopedSlots||(r.scopedSlots={}))[a]=e}r.children.push(e),e.parent=r}e.children=e.children.filter((function(e){return!e.slotScope})),f(e),e.pre&&(l=!1),Al(e.tag)&&(c=!1);for(var o=0;o<kl.length;o++)kl[o](e,t)}function f(e){var t;if(!c)while((t=e.children[e.children.length-1])&&3===t.type&&" "===t.text)e.children.pop()}function p(e){"slot"!==e.tag&&"template"!==e.tag||d("Cannot use <"+e.tag+"> as component root element because it may contain multiple nodes.",{start:e.start}),e.attrsMap.hasOwnProperty("v-for")&&d("Cannot use v-for on stateful component root element because it renders multiple elements.",e.rawAttrsMap["v-for"])}return Ml(e,{warn:zl,expectHTML:t.expectHTML,isUnaryTag:t.isUnaryTag,canBeLeftOpenTag:t.canBeLeftOpenTag,shouldDecodeNewlines:t.shouldDecodeNewlines,shouldDecodeNewlinesForHref:t.shouldDecodeNewlinesForHref,shouldKeepComment:t.comments,outputSourceRange:t.outputSourceRange,start:function(e,a,o,s,u){var d=r&&r.ns||Vl(e);ae&&"svg"===d&&(a=gc(a));var f=Ul(e,a,r);d&&(f.ns=d),t.outputSourceRange&&(f.start=s,f.end=u,f.rawAttrsMap=f.attrsList.reduce((function(e,t){return e[t.name]=t,e}),{})),a.forEach((function(e){Rl.test(e.name)&&zl("Invalid dynamic argument expression: attribute names cannot contain spaces, quotes, <, >, / or =.",{start:e.start+e.name.indexOf("["),end:e.start+e.name.length})})),mc(f)&&!ue()&&(f.forbidden=!0,zl("Templates should only be responsible for mapping the state to the UI. Avoid placing tags with side-effects in your templates, such as <"+e+">, as they will not be parsed.",{start:f.start}));for(var m=0;m<Hl.length;m++)f=Hl[m](f,t)||f;l||(ql(f),f.pre&&(l=!0)),Al(f.tag)&&(c=!0),l?Jl(f):f.processed||(Ql(f),tc(f),ic(f)),n||(n=f,p(n)),o?h(f):(r=f,i.push(f))},end:function(e,a,n){var o=i[i.length-1];i.length-=1,r=i[i.length-1],t.outputSourceRange&&(o.end=n),h(o)},chars:function(a,n,i){if(r){if(!ae||"textarea"!==r.tag||r.attrsMap.placeholder!==a){var u,h,f=r.children;if(a=c||a.trim()?pc(r)?a:Nl(a):f.length?s?"condense"===s&&El.test(a)?"":" ":o?" ":"":"",a)c||"condense"!==s||(a=a.replace($l," ")),!l&&" "!==a&&(u=Ws(a,Ll))?h={type:2,expression:u.expression,tokens:u.tokens,text:a}:" "===a&&f.length&&" "===f[f.length-1].text||(h={type:3,text:a}),h&&(t.outputSourceRange&&(h.start=n,h.end=i),f.push(h))}}else a===e?d("Component template requires a root element, rather than just text.",{start:n}):(a=a.trim())&&d('text "'+a+'" outside root element will be ignored.',{start:n})},comment:function(e,a,n){if(r){var i={type:3,text:e,isComment:!0};t.outputSourceRange&&(i.start=a,i.end=n),r.children.push(i)}}}),n}function ql(e){null!=qi(e,"v-pre")&&(e.pre=!0)}function Jl(e){var t=e.attrsList,a=t.length;if(a)for(var n=e.attrs=new Array(a),r=0;r<a;r++)n[r]={name:t[r].name,value:JSON.stringify(t[r].value)},null!=t[r].start&&(n[r].start=t[r].start,n[r].end=t[r].end);else e.pre||(e.plain=!0)}function Kl(e,t){Xl(e),e.plain=!e.key&&!e.scopedSlots&&!e.attrsList.length,Zl(e),oc(e),lc(e),cc(e);for(var a=0;a<wl.length;a++)e=wl[a](e,t)||e;return uc(e),e}function Xl(e){var t=Gi(e,"key");if(t){if("template"===e.tag&&zl("<template> cannot be keyed. Place the key on real elements instead.",Ui(e,"key")),e.for){var a=e.iterator2||e.iterator1,n=e.parent;a&&a===t&&n&&"transition-group"===n.tag&&zl("Do not use v-for index as key on <transition-group> children, this is the same as not using keys.",Ui(e,"key"),!0)}e.key=t}}function Zl(e){var t=Gi(e,"ref");t&&(e.ref=t,e.refInFor=dc(e))}function Ql(e){var t;if(t=qi(e,"v-for")){var a=ec(t);a?D(e,a):zl("Invalid v-for expression: "+t,e.rawAttrsMap["v-for"])}}function ec(e){var t=e.match(Cl);if(t){var a={};a.for=t[2].trim();var n=t[1].trim().replace(xl,""),r=n.match(jl);return r?(a.alias=n.replace(jl,"").trim(),a.iterator1=r[1].trim(),r[2]&&(a.iterator2=r[2].trim())):a.alias=n,a}}function tc(e){var t=qi(e,"v-if");if(t)e.if=t,rc(e,{exp:t,block:e});else{null!=qi(e,"v-else")&&(e.else=!0);var a=qi(e,"v-else-if");a&&(e.elseif=a)}}function ac(e,t){var a=nc(t.children);a&&a.if?rc(a,{exp:e.elseif,block:e}):zl("v-"+(e.elseif?'else-if="'+e.elseif+'"':"else")+" used on element <"+e.tag+"> without corresponding v-if.",e.rawAttrsMap[e.elseif?"v-else-if":"v-else"])}function nc(e){var t=e.length;while(t--){if(1===e[t].type)return e[t];" "!==e[t].text&&zl('text "'+e[t].text.trim()+'" between v-if and v-else(-if) will be ignored.',e[t]),e.pop()}}function rc(e,t){e.ifConditions||(e.ifConditions=[]),e.ifConditions.push(t)}function ic(e){var t=qi(e,"v-once");null!=t&&(e.once=!0)}function oc(e){var t;"template"===e.tag?(t=qi(e,"scope"),t&&zl('the "scope" attribute for scoped slots have been deprecated and replaced by "slot-scope" since 2.5. The new "slot-scope" attribute can also be used on plain elements in addition to <template> to denote scoped slots.',e.rawAttrsMap["scope"],!0),e.slotScope=t||qi(e,"slot-scope")):(t=qi(e,"slot-scope"))&&(e.attrsMap["v-for"]&&zl("Ambiguous combined usage of slot-scope and v-for on <"+e.tag+"> (v-for takes higher priority). Use a wrapper <template> for the scoped slot to make it clearer.",e.rawAttrsMap["slot-scope"],!0),e.slotScope=t);var a=Gi(e,"slot");if(a&&(e.slotTarget='""'===a?'"default"':a,e.slotTargetDynamic=!(!e.attrsMap[":slot"]&&!e.attrsMap["v-bind:slot"]),"template"===e.tag||e.slotScope||Ei(e,"slot",a,Ui(e,"slot"))),"template"===e.tag){var n=Ji(e,Fl);if(n){(e.slotTarget||e.slotScope)&&zl("Unexpected mixed usage of different slot syntaxes.",e),e.parent&&!Ol(e.parent)&&zl("<template v-slot> can only appear at the root level inside the receiving component",e);var r=sc(n),i=r.name,o=r.dynamic;e.slotTarget=i,e.slotTargetDynamic=o,e.slotScope=n.value||Wl}}else{var s=Ji(e,Fl);if(s){Ol(e)||zl("v-slot can only be used on components or <template>.",s),(e.slotScope||e.slotTarget)&&zl("Unexpected mixed usage of different slot syntaxes.",e),e.scopedSlots&&zl("To avoid scope ambiguity, the default slot should also use <template> syntax when there are other named slots.",s);var l=e.scopedSlots||(e.scopedSlots={}),c=sc(s),u=c.name,d=c.dynamic,h=l[u]=Ul("template",[],e);h.slotTarget=u,h.slotTargetDynamic=d,h.children=e.children.filter((function(e){if(!e.slotScope)return e.parent=h,!0})),h.slotScope=s.value||Wl,e.children=[],e.plain=!1}}}function sc(e){var t=e.name.replace(Fl,"");return t||("#"!==e.name[0]?t="default":zl("v-slot shorthand syntax requires a slot name.",e)),Bl.test(t)?{name:t.slice(1,-1),dynamic:!0}:{name:'"'+t+'"',dynamic:!1}}function lc(e){"slot"===e.tag&&(e.slotName=Gi(e,"name"),e.key&&zl("`key` does not work on <slot> because slots are abstract outlets and can possibly expand into multiple elements. Use the key on a wrapping element instead.",Ui(e,"key")))}function cc(e){var t;(t=Gi(e,"is"))&&(e.component=t),null!=qi(e,"inline-template")&&(e.inlineTemplate=!0)}function uc(e){var t,a,n,r,i,o,s,l,c=e.attrsList;for(t=0,a=c.length;t<a;t++)if(n=r=c[t].name,i=c[t].value,Dl.test(n))if(e.hasBindings=!0,o=hc(n.replace(Dl,"")),o&&(n=n.replace(Il,"")),Pl.test(n))n=n.replace(Pl,""),i=Bi(i),l=Bl.test(n),l&&(n=n.slice(1,-1)),0===i.trim().length&&zl('The value for a v-bind expression cannot be empty. Found in "v-bind:'+n+'"'),o&&(o.prop&&!l&&(n=w(n),"innerHtml"===n&&(n="innerHTML")),o.camel&&!l&&(n=w(n)),o.sync&&(s=Zi(i,"$event"),l?Wi(e,'"update:"+('+n+")",s,null,!1,zl,c[t],!0):(Wi(e,"update:"+w(n),s,null,!1,zl,c[t]),A(n)!==w(n)&&Wi(e,"update:"+A(n),s,null,!1,zl,c[t])))),o&&o.prop||!e.component&&Sl(e.tag,e.attrsMap.type,n)?Fi(e,n,i,c[t],l):Ei(e,n,i,c[t],l);else if(Tl.test(n))n=n.replace(Tl,""),l=Bl.test(n),l&&(n=n.slice(1,-1)),Wi(e,n,i,o,!1,zl,c[t],l);else{n=n.replace(Dl,"");var u=n.match(Yl),d=u&&u[1];l=!1,d&&(n=n.slice(0,-(d.length+1)),Bl.test(d)&&(d=d.slice(1,-1),l=!0)),Ri(e,n,r,i,d,l,o,c[t]),"model"===n&&yc(e,i)}else{var h=Ws(i,Ll);h&&zl(n+'="'+i+'": Interpolation inside attributes has been removed. Use v-bind or the colon shorthand instead. For example, instead of <div id="{{ val }}">, use <div :id="val">.',c[t]),Ei(e,n,JSON.stringify(i),c[t]),!e.component&&"muted"===n&&Sl(e.tag,e.attrsMap.type,n)&&Fi(e,n,"true",c[t])}}function dc(e){var t=e;while(t){if(void 0!==t.for)return!0;t=t.parent}return!1}function hc(e){var t=e.match(Il);if(t){var a={};return t.forEach((function(e){a[e.slice(1)]=!0})),a}}function fc(e){for(var t={},a=0,n=e.length;a<n;a++)!t[e[a].name]||ae||re||zl("duplicate attribute: "+e[a].name,e[a]),t[e[a].name]=e[a].value;return t}function pc(e){return"script"===e.tag||"style"===e.tag}function mc(e){return"style"===e.tag||"script"===e.tag&&(!e.attrsMap.type||"text/javascript"===e.attrsMap.type)}var vc=/^xmlns:NS\d+/,_c=/^NS\d+:/;function gc(e){for(var t=[],a=0;a<e.length;a++){var n=e[a];vc.test(n.name)||(n.name=n.name.replace(_c,""),t.push(n))}return t}function yc(e,t){var a=e;while(a)a.for&&a.alias===t&&zl("<"+e.tag+' v-model="'+t+'">: You are binding v-model directly to a v-for iteration alias. This will not be able to modify the v-for source array because writing to the alias is like modifying a function local variable. Consider using an array of objects and use v-model on an object property instead.',e.rawAttrsMap["v-model"]),a=a.parent}function bc(e,t){if("input"===e.tag){var a,n=e.attrsMap;if(!n["v-model"])return;if((n[":type"]||n["v-bind:type"])&&(a=Gi(e,"type")),n.type||a||!n["v-bind"]||(a="("+n["v-bind"]+").type"),a){var r=qi(e,"v-if",!0),i=r?"&&("+r+")":"",o=null!=qi(e,"v-else",!0),s=qi(e,"v-else-if",!0),l=Mc(e);Ql(l),$i(l,"type","checkbox"),Kl(l,t),l.processed=!0,l.if="("+a+")==='checkbox'"+i,rc(l,{exp:l.if,block:l});var c=Mc(e);qi(c,"v-for",!0),$i(c,"type","radio"),Kl(c,t),rc(l,{exp:"("+a+")==='radio'"+i,block:c});var u=Mc(e);return qi(u,"v-for",!0),$i(u,":type",a),Kl(u,t),rc(l,{exp:r,block:u}),o?l.else=!0:s&&(l.elseif=s),l}}}function Mc(e){return Ul(e.tag,e.attrsList.slice(),e.parent)}var zc={preTransformNode:bc},Lc=[qs,Zs,zc];function wc(e,t){t.value&&Fi(e,"textContent","_s("+t.value+")",t)}function Hc(e,t){t.value&&Fi(e,"innerHTML","_s("+t.value+")",t)}var kc,Ac,Sc={model:lo,text:wc,html:Hc},Vc={expectHTML:!0,modules:Lc,directives:Sc,isPreTag:Fr,isUnaryTag:el,mustUseProp:Mr,canBeLeftOpenTag:tl,isReservedTag:Er,getTagNamespace:$r,staticKeys:Y(Lc)},Oc=z(Dc);function Tc(e,t){e&&(kc=Oc(t.staticKeys||""),Ac=t.isReservedTag||x,Cc(e),jc(e,!1))}function Dc(e){return v("type,tag,attrsList,attrsMap,plain,parent,children,attrs,start,end,rawAttrsMap"+(e?","+e:""))}function Cc(e){if(e.static=xc(e),1===e.type){if(!Ac(e.tag)&&"slot"!==e.tag&&null==e.attrsMap["inline-template"])return;for(var t=0,a=e.children.length;t<a;t++){var n=e.children[t];Cc(n),n.static||(e.static=!1)}if(e.ifConditions)for(var r=1,i=e.ifConditions.length;r<i;r++){var o=e.ifConditions[r].block;Cc(o),o.static||(e.static=!1)}}}function jc(e,t){if(1===e.type){if((e.static||e.once)&&(e.staticInFor=t),e.static&&e.children.length&&(1!==e.children.length||3!==e.children[0].type))return void(e.staticRoot=!0);if(e.staticRoot=!1,e.children)for(var a=0,n=e.children.length;a<n;a++)jc(e.children[a],t||!!e.for);if(e.ifConditions)for(var r=1,i=e.ifConditions.length;r<i;r++)jc(e.ifConditions[r].block,t)}}function xc(e){return 2!==e.type&&(3===e.type||!(!e.pre&&(e.hasBindings||e.if||e.for||_(e.tag)||!Ac(e.tag)||Bc(e)||!Object.keys(e).every(kc))))}function Bc(e){while(e.parent){if(e=e.parent,"template"!==e.tag)return!1;if(e.for)return!0}return!1}var Yc=/^([\w$_]+|\([^)]*?\))\s*=>|^function(?:\s+[\w$]+)?\s*\(/,Pc=/\([^)]*?\);*$/,Ic=/^[A-Za-z_$][\w$]*(?:\.[A-Za-z_$][\w$]*|\['[^']*?']|\["[^"]*?"]|\[\d+]|\[[A-Za-z_$][\w$]*])*$/,Fc={esc:27,tab:9,enter:13,space:32,up:38,left:37,right:39,down:40,delete:[8,46]},Ec={esc:["Esc","Escape"],tab:"Tab",enter:"Enter",space:[" ","Spacebar"],up:["Up","ArrowUp"],left:["Left","ArrowLeft"],right:["Right","ArrowRight"],down:["Down","ArrowDown"],delete:["Backspace","Delete","Del"]},$c=function(e){return"if("+e+")return null;"},Rc={stop:"$event.stopPropagation();",prevent:"$event.preventDefault();",self:$c("$event.target !== $event.currentTarget"),ctrl:$c("!$event.ctrlKey"),shift:$c("!$event.shiftKey"),alt:$c("!$event.altKey"),meta:$c("!$event.metaKey"),left:$c("'button' in $event && $event.button !== 0"),middle:$c("'button' in $event && $event.button !== 1"),right:$c("'button' in $event && $event.button !== 2")};function Nc(e,t){var a=t?"nativeOn:":"on:",n="",r="";for(var i in e){var o=Wc(e[i]);e[i]&&e[i].dynamic?r+=i+","+o+",":n+='"'+i+'":'+o+","}return n="{"+n.slice(0,-1)+"}",r?a+"_d("+n+",["+r.slice(0,-1)+"])":a+n}function Wc(e){if(!e)return"function(){}";if(Array.isArray(e))return"["+e.map((function(e){return Wc(e)})).join(",")+"]";var t=Ic.test(e.value),a=Yc.test(e.value),n=Ic.test(e.value.replace(Pc,""));if(e.modifiers){var r="",i="",o=[];for(var s in e.modifiers)if(Rc[s])i+=Rc[s],Fc[s]&&o.push(s);else if("exact"===s){var l=e.modifiers;i+=$c(["ctrl","shift","alt","meta"].filter((function(e){return!l[e]})).map((function(e){return"$event."+e+"Key"})).join("||"))}else o.push(s);o.length&&(r+=Uc(o)),i&&(r+=i);var c=t?"return "+e.value+".apply(null, arguments)":a?"return ("+e.value+").apply(null, arguments)":n?"return "+e.value:e.value;return"function($event){"+r+c+"}"}return t||a?e.value:"function($event){"+(n?"return "+e.value:e.value)+"}"}function Uc(e){return"if(!$event.type.indexOf('key')&&"+e.map(Gc).join("&&")+")return null;"}function Gc(e){var t=parseInt(e,10);if(t)return"$event.keyCode!=="+t;var a=Fc[e],n=Ec[e];return"_k($event.keyCode,"+JSON.stringify(e)+","+JSON.stringify(a)+",$event.key,"+JSON.stringify(n)+")"}function qc(e,t){t.modifiers&&me("v-on without argument does not support modifiers."),e.wrapListeners=function(e){return"_g("+e+","+t.value+")"}}function Jc(e,t){e.wrapData=function(a){return"_b("+a+",'"+e.tag+"',"+t.value+","+(t.modifiers&&t.modifiers.prop?"true":"false")+(t.modifiers&&t.modifiers.sync?",true":"")+")"}}var Kc={on:qc,bind:Jc,cloak:j},Xc=function(e){this.options=e,this.warn=e.warn||Pi,this.transforms=Ii(e.modules,"transformCode"),this.dataGenFns=Ii(e.modules,"genData"),this.directives=D(D({},Kc),e.directives);var t=e.isReservedTag||x;this.maybeComponent=function(e){return!!e.component||!t(e.tag)},this.onceId=0,this.staticRenderFns=[],this.pre=!1};function Zc(e,t){var a=new Xc(t),n=e?"script"===e.tag?"null":Qc(e,a):'_c("div")';return{render:"with(this){return "+n+"}",staticRenderFns:a.staticRenderFns}}function Qc(e,t){if(e.parent&&(e.pre=e.pre||e.parent.pre),e.staticRoot&&!e.staticProcessed)return eu(e,t);if(e.once&&!e.onceProcessed)return tu(e,t);if(e.for&&!e.forProcessed)return ru(e,t);if(e.if&&!e.ifProcessed)return au(e,t);if("template"!==e.tag||e.slotTarget||t.pre){if("slot"===e.tag)return gu(e,t);var a;if(e.component)a=yu(e.component,e,t);else{var n;(!e.plain||e.pre&&t.maybeComponent(e))&&(n=iu(e,t));var r=e.inlineTemplate?null:hu(e,t,!0);a="_c('"+e.tag+"'"+(n?","+n:"")+(r?","+r:"")+")"}for(var i=0;i<t.transforms.length;i++)a=t.transforms[i](e,a);return a}return hu(e,t)||"void 0"}function eu(e,t){e.staticProcessed=!0;var a=t.pre;return e.pre&&(t.pre=e.pre),t.staticRenderFns.push("with(this){return "+Qc(e,t)+"}"),t.pre=a,"_m("+(t.staticRenderFns.length-1)+(e.staticInFor?",true":"")+")"}function tu(e,t){if(e.onceProcessed=!0,e.if&&!e.ifProcessed)return au(e,t);if(e.staticInFor){var a="",n=e.parent;while(n){if(n.for){a=n.key;break}n=n.parent}return a?"_o("+Qc(e,t)+","+t.onceId+++","+a+")":(t.warn("v-once can only be used inside v-for that is keyed. ",e.rawAttrsMap["v-once"]),Qc(e,t))}return eu(e,t)}function au(e,t,a,n){return e.ifProcessed=!0,nu(e.ifConditions.slice(),t,a,n)}function nu(e,t,a,n){if(!e.length)return n||"_e()";var r=e.shift();return r.exp?"("+r.exp+")?"+i(r.block)+":"+nu(e,t,a,n):""+i(r.block);function i(e){return a?a(e,t):e.once?tu(e,t):Qc(e,t)}}function ru(e,t,a,n){var r=e.for,i=e.alias,o=e.iterator1?","+e.iterator1:"",s=e.iterator2?","+e.iterator2:"";return t.maybeComponent(e)&&"slot"!==e.tag&&"template"!==e.tag&&!e.key&&t.warn("<"+e.tag+' v-for="'+i+" in "+r+'">: component lists rendered with v-for should have explicit keys. See https://vuejs.org/guide/list.html#key for more info.',e.rawAttrsMap["v-for"],!0),e.forProcessed=!0,(n||"_l")+"(("+r+"),function("+i+o+s+"){return "+(a||Qc)(e,t)+"})"}function iu(e,t){var a="{",n=ou(e,t);n&&(a+=n+","),e.key&&(a+="key:"+e.key+","),e.ref&&(a+="ref:"+e.ref+","),e.refInFor&&(a+="refInFor:true,"),e.pre&&(a+="pre:true,"),e.component&&(a+='tag:"'+e.tag+'",');for(var r=0;r<t.dataGenFns.length;r++)a+=t.dataGenFns[r](e);if(e.attrs&&(a+="attrs:"+bu(e.attrs)+","),e.props&&(a+="domProps:"+bu(e.props)+","),e.events&&(a+=Nc(e.events,!1)+","),e.nativeEvents&&(a+=Nc(e.nativeEvents,!0)+","),e.slotTarget&&!e.slotScope&&(a+="slot:"+e.slotTarget+","),e.scopedSlots&&(a+=lu(e,e.scopedSlots,t)+","),e.model&&(a+="model:{value:"+e.model.value+",callback:"+e.model.callback+",expression:"+e.model.expression+"},"),e.inlineTemplate){var i=su(e,t);i&&(a+=i+",")}return a=a.replace(/,$/,"")+"}",e.dynamicAttrs&&(a="_b("+a+',"'+e.tag+'",'+bu(e.dynamicAttrs)+")"),e.wrapData&&(a=e.wrapData(a)),e.wrapListeners&&(a=e.wrapListeners(a)),a}function ou(e,t){var a=e.directives;if(a){var n,r,i,o,s="directives:[",l=!1;for(n=0,r=a.length;n<r;n++){i=a[n],o=!0;var c=t.directives[i.name];c&&(o=!!c(e,i,t.warn)),o&&(l=!0,s+='{name:"'+i.name+'",rawName:"'+i.rawName+'"'+(i.value?",value:("+i.value+"),expression:"+JSON.stringify(i.value):"")+(i.arg?",arg:"+(i.isDynamicArg?i.arg:'"'+i.arg+'"'):"")+(i.modifiers?",modifiers:"+JSON.stringify(i.modifiers):"")+"},")}return l?s.slice(0,-1)+"]":void 0}}function su(e,t){var a=e.children[0];if(1===e.children.length&&1===a.type||t.warn("Inline-template components must have exactly one child element.",{start:e.start}),a&&1===a.type){var n=Zc(a,t.options);return"inlineTemplate:{render:function(){"+n.render+"},staticRenderFns:["+n.staticRenderFns.map((function(e){return"function(){"+e+"}"})).join(",")+"]}"}}function lu(e,t,a){var n=e.for||Object.keys(t).some((function(e){var a=t[e];return a.slotTargetDynamic||a.if||a.for||uu(a)})),r=!!e.if;if(!n){var i=e.parent;while(i){if(i.slotScope&&i.slotScope!==Wl||i.for){n=!0;break}i.if&&(r=!0),i=i.parent}}var o=Object.keys(t).map((function(e){return du(t[e],a)})).join(",");return"scopedSlots:_u(["+o+"]"+(n?",null,true":"")+(!n&&r?",null,false,"+cu(o):"")+")"}function cu(e){var t=5381,a=e.length;while(a)t=33*t^e.charCodeAt(--a);return t>>>0}function uu(e){return 1===e.type&&("slot"===e.tag||e.children.some(uu))}function du(e,t){var a=e.attrsMap["slot-scope"];if(e.if&&!e.ifProcessed&&!a)return au(e,t,du,"null");if(e.for&&!e.forProcessed)return ru(e,t,du);var n=e.slotScope===Wl?"":String(e.slotScope),r="function("+n+"){return "+("template"===e.tag?e.if&&a?"("+e.if+")?"+(hu(e,t)||"undefined")+":undefined":hu(e,t)||"undefined":Qc(e,t))+"}",i=n?"":",proxy:true";return"{key:"+(e.slotTarget||'"default"')+",fn:"+r+i+"}"}function hu(e,t,a,n,r){var i=e.children;if(i.length){var o=i[0];if(1===i.length&&o.for&&"template"!==o.tag&&"slot"!==o.tag){var s=a?t.maybeComponent(o)?",1":",0":"";return""+(n||Qc)(o,t)+s}var l=a?fu(i,t.maybeComponent):0,c=r||mu;return"["+i.map((function(e){return c(e,t)})).join(",")+"]"+(l?","+l:"")}}function fu(e,t){for(var a=0,n=0;n<e.length;n++){var r=e[n];if(1===r.type){if(pu(r)||r.ifConditions&&r.ifConditions.some((function(e){return pu(e.block)}))){a=2;break}(t(r)||r.ifConditions&&r.ifConditions.some((function(e){return t(e.block)})))&&(a=1)}}return a}function pu(e){return void 0!==e.for||"template"===e.tag||"slot"===e.tag}function mu(e,t){return 1===e.type?Qc(e,t):3===e.type&&e.isComment?_u(e):vu(e)}function vu(e){return"_v("+(2===e.type?e.expression:Mu(JSON.stringify(e.text)))+")"}function _u(e){return"_e("+JSON.stringify(e.text)+")"}function gu(e,t){var a=e.slotName||'"default"',n=hu(e,t),r="_t("+a+(n?",function(){return "+n+"}":""),i=e.attrs||e.dynamicAttrs?bu((e.attrs||[]).concat(e.dynamicAttrs||[]).map((function(e){return{name:w(e.name),value:e.value,dynamic:e.dynamic}}))):null,o=e.attrsMap["v-bind"];return!i&&!o||n||(r+=",null"),i&&(r+=","+i),o&&(r+=(i?"":",null")+","+o),r+")"}function yu(e,t,a){var n=t.inlineTemplate?null:hu(t,a,!0);return"_c("+e+","+iu(t,a)+(n?","+n:"")+")"}function bu(e){for(var t="",a="",n=0;n<e.length;n++){var r=e[n],i=Mu(r.value);r.dynamic?a+=r.name+","+i+",":t+='"'+r.name+'":'+i+","}return t="{"+t.slice(0,-1)+"}",a?"_d("+t+",["+a.slice(0,-1)+"])":t}function Mu(e){return e.replace(/\u2028/g,"\\u2028").replace(/\u2029/g,"\\u2029")}var zu=new RegExp("\\b"+"do,if,for,let,new,try,var,case,else,with,await,break,catch,class,const,super,throw,while,yield,delete,export,import,return,switch,default,extends,finally,continue,debugger,function,arguments".split(",").join("\\b|\\b")+"\\b"),Lu=new RegExp("\\b"+"delete,typeof,void".split(",").join("\\s*\\([^\\)]*\\)|\\b")+"\\s*\\([^\\)]*\\)"),wu=/'(?:[^'\\]|\\.)*'|"(?:[^"\\]|\\.)*"|`(?:[^`\\]|\\.)*\$\{|\}(?:[^`\\]|\\.)*`|`(?:[^`\\]|\\.)*`/g;function Hu(e,t){e&&ku(e,t)}function ku(e,t){if(1===e.type){for(var a in e.attrsMap)if(Dl.test(a)){var n=e.attrsMap[a];if(n){var r=e.rawAttrsMap[a];"v-for"===a?Su(e,'v-for="'+n+'"',t,r):"v-slot"===a||"#"===a[0]?Tu(n,a+'="'+n+'"',t,r):Tl.test(a)?Au(n,a+'="'+n+'"',t,r):Ou(n,a+'="'+n+'"',t,r)}}if(e.children)for(var i=0;i<e.children.length;i++)ku(e.children[i],t)}else 2===e.type&&Ou(e.expression,e.text,t,e)}function Au(e,t,a,n){var r=e.replace(wu,""),i=r.match(Lu);i&&"$"!==r.charAt(i.index-1)&&a('avoid using JavaScript unary operator as property name: "'+i[0]+'" in expression '+t.trim(),n),Ou(e,t,a,n)}function Su(e,t,a,n){Ou(e.for||"",t,a,n),Vu(e.alias,"v-for alias",t,a,n),Vu(e.iterator1,"v-for iterator",t,a,n),Vu(e.iterator2,"v-for iterator",t,a,n)}function Vu(e,t,a,n,r){if("string"===typeof e)try{new Function("var "+e+"=_")}catch(qu){n("invalid "+t+' "'+e+'" in expression: '+a.trim(),r)}}function Ou(e,t,a,n){try{new Function("return "+e)}catch(qu){var r=e.replace(wu,"").match(zu);a(r?'avoid using JavaScript keyword as property name: "'+r[0]+'"\n  Raw expression: '+t.trim():"invalid expression: "+qu.message+" in\n\n    "+e+"\n\n  Raw expression: "+t.trim()+"\n",n)}}function Tu(e,t,a,n){try{new Function(e,"")}catch(qu){a("invalid function parameter expression: "+qu.message+" in\n\n    "+e+"\n\n  Raw expression: "+t.trim()+"\n",n)}}var Du=2;function Cu(e,t,a){void 0===t&&(t=0),void 0===a&&(a=e.length);for(var n=e.split(/\r?\n/),r=0,i=[],o=0;o<n.length;o++)if(r+=n[o].length+1,r>=t){for(var s=o-Du;s<=o+Du||a>r;s++)if(!(s<0||s>=n.length)){i.push(""+(s+1)+ju(" ",3-String(s+1).length)+"|  "+n[s]);var l=n[s].length;if(s===o){var c=t-(r-l)+1,u=a>r?l-c:a-t;i.push("   |  "+ju(" ",c)+ju("^",u))}else if(s>o){if(a>r){var d=Math.min(a-r,l);i.push("   |  "+ju("^",d))}r+=l+1}}break}return i.join("\n")}function ju(e,t){var a="";if(t>0)while(1){if(1&t&&(a+=e),t>>>=1,t<=0)break;e+=e}return a}function xu(e,t){try{return new Function(e)}catch(a){return t.push({err:a,code:e}),j}}function Bu(e){var t=Object.create(null);return function(a,n,r){n=D({},n);var i=n.warn||me;delete n.warn;try{new Function("return 1")}catch(qu){qu.toString().match(/unsafe-eval|CSP/)&&i("It seems you are using the standalone build of Vue.js in an environment with Content Security Policy that prohibits unsafe-eval. The template compiler cannot work in this environment. Consider relaxing the policy to allow unsafe-eval or pre-compiling your templates into render functions.")}var o=n.delimiters?String(n.delimiters)+a:a;if(t[o])return t[o];var s=e(a,n);s.errors&&s.errors.length&&(n.outputSourceRange?s.errors.forEach((function(e){i("Error compiling template:\n\n"+e.msg+"\n\n"+Cu(a,e.start,e.end),r)})):i("Error compiling template:\n\n"+a+"\n\n"+s.errors.map((function(e){return"- "+e})).join("\n")+"\n",r)),s.tips&&s.tips.length&&(n.outputSourceRange?s.tips.forEach((function(e){return ve(e.msg,r)})):s.tips.forEach((function(e){return ve(e,r)})));var l={},c=[];return l.render=xu(s.render,c),l.staticRenderFns=s.staticRenderFns.map((function(e){return xu(e,c)})),s.errors&&s.errors.length||!c.length||i("Failed to generate render function:\n\n"+c.map((function(e){var t=e.err,a=e.code;return t.toString()+" in\n\n"+a+"\n"})).join("\n"),r),t[o]=l}}function Yu(e){return function(t){function a(a,n){var r=Object.create(t),i=[],o=[],s=function(e,t,a){(a?o:i).push(e)};if(n){if(n.outputSourceRange){var l=a.match(/^\s*/)[0].length;s=function(e,t,a){var n={msg:e};t&&(null!=t.start&&(n.start=t.start+l),null!=t.end&&(n.end=t.end+l)),(a?o:i).push(n)}}for(var c in n.modules&&(r.modules=(t.modules||[]).concat(n.modules)),n.directives&&(r.directives=D(Object.create(t.directives||null),n.directives)),n)"modules"!==c&&"directives"!==c&&(r[c]=n[c])}r.warn=s;var u=e(a.trim(),r);return Hu(u.ast,s),u.errors=i,u.tips=o,u}return{compile:a,compileToFunctions:Bu(a)}}}var Pu,Iu=Yu((function(e,t){var a=Gl(e.trim(),t);!1!==t.optimize&&Tc(a,t);var n=Zc(a,t);return{ast:a,render:n.render,staticRenderFns:n.staticRenderFns}})),Fu=Iu(Vc),Eu=(Fu.compile,Fu.compileToFunctions);function $u(e){return Pu=Pu||document.createElement("div"),Pu.innerHTML=e?'<a href="\n"/>':'<div a="\n"/>',Pu.innerHTML.indexOf("&#10;")>0}var Ru=!!Z&&$u(!1),Nu=!!Z&&$u(!0),Wu=z((function(e){var t=Ur(e);return t&&t.innerHTML})),Uu=rr.prototype.$mount;function Gu(e){if(e.outerHTML)return e.outerHTML;var t=document.createElement("div");return t.appendChild(e.cloneNode(!0)),t.innerHTML}return rr.prototype.$mount=function(e,t){if(e=e&&Ur(e),e===document.body||e===document.documentElement)return me("Do not mount Vue to <html> or <body> - mount to normal elements instead."),this;var a=this.$options;if(!a.render){var n=a.template;if(n)if("string"===typeof n)"#"===n.charAt(0)&&(n=Wu(n),n||me("Template element not found or is empty: "+a.template,this));else{if(!n.nodeType)return me("invalid template option:"+n,this),this;n=n.innerHTML}else e&&(n=Gu(e));if(n){N.performance&&At&&At("compile");var r=Eu(n,{outputSourceRange:!0,shouldDecodeNewlines:Ru,shouldDecodeNewlinesForHref:Nu,delimiters:a.delimiters,comments:a.comments},this),i=r.render,o=r.staticRenderFns;a.render=i,a.staticRenderFns=o,N.performance&&At&&(At("compile end"),St("vue "+this._name+" compile","compile","compile end"))}}return Uu.call(this,e,t)},rr.compile=Eu,rr}))}).call(this,a("c8ba"))},bb71:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?r[a][0]:r[a][1]}var a=e.defineLocale("de-ch",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a}))},bc3a:function(e,t,a){e.exports=a("cee4")},bf37:function(e,t,a){},c04e:function(e,t,a){var n=a("c65b"),r=a("861d"),i=a("d9b5"),o=a("dc4a"),s=a("485a"),l=a("b622"),c=TypeError,u=l("toPrimitive");e.exports=function(e,t){if(!r(e)||i(e))return e;var a,l=o(e,u);if(l){if(void 0===t&&(t="default"),a=n(l,e,t),!r(a)||i(a))return a;throw c("Can't convert object to primitive value")}return void 0===t&&(t="number"),s(e,t)}},c109:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tzm",{months:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),monthsShort:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),weekdays:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysShort:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysMin:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ⴰⵙⴷⵅ ⴴ] LT",nextDay:"[ⴰⵙⴽⴰ ⴴ] LT",nextWeek:"dddd [ⴴ] LT",lastDay:"[ⴰⵚⴰⵏⵜ ⴴ] LT",lastWeek:"dddd [ⴴ] LT",sameElse:"L"},relativeTime:{future:"ⴷⴰⴷⵅ ⵙ ⵢⴰⵏ %s",past:"ⵢⴰⵏ %s",s:"ⵉⵎⵉⴽ",ss:"%d ⵉⵎⵉⴽ",m:"ⵎⵉⵏⵓⴺ",mm:"%d ⵎⵉⵏⵓⴺ",h:"ⵙⴰⵄⴰ",hh:"%d ⵜⴰⵙⵙⴰⵄⵉⵏ",d:"ⴰⵙⵙ",dd:"%d oⵙⵙⴰⵏ",M:"ⴰⵢoⵓⵔ",MM:"%d ⵉⵢⵢⵉⵔⵏ",y:"ⴰⵙⴳⴰⵙ",yy:"%d ⵉⵙⴳⴰⵙⵏ"},week:{dow:6,doy:12}});return t}))},c1df:function(e,t,a){(function(e){var t;//! moment.js
//! version : 2.29.4
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com
(function(t,a){e.exports=a()})(0,(function(){"use strict";var n,r;function i(){return n.apply(null,arguments)}function o(e){n=e}function s(e){return e instanceof Array||"[object Array]"===Object.prototype.toString.call(e)}function l(e){return null!=e&&"[object Object]"===Object.prototype.toString.call(e)}function c(e,t){return Object.prototype.hasOwnProperty.call(e,t)}function u(e){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(e).length;var t;for(t in e)if(c(e,t))return!1;return!0}function d(e){return void 0===e}function h(e){return"number"===typeof e||"[object Number]"===Object.prototype.toString.call(e)}function f(e){return e instanceof Date||"[object Date]"===Object.prototype.toString.call(e)}function p(e,t){var a,n=[],r=e.length;for(a=0;a<r;++a)n.push(t(e[a],a));return n}function m(e,t){for(var a in t)c(t,a)&&(e[a]=t[a]);return c(t,"toString")&&(e.toString=t.toString),c(t,"valueOf")&&(e.valueOf=t.valueOf),e}function v(e,t,a,n){return Ja(e,t,a,n,!0).utc()}function _(){return{empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidEra:null,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],era:null,meridiem:null,rfc2822:!1,weekdayMismatch:!1}}function g(e){return null==e._pf&&(e._pf=_()),e._pf}function y(e){if(null==e._isValid){var t=g(e),a=r.call(t.parsedDateParts,(function(e){return null!=e})),n=!isNaN(e._d.getTime())&&t.overflow<0&&!t.empty&&!t.invalidEra&&!t.invalidMonth&&!t.invalidWeekday&&!t.weekdayMismatch&&!t.nullInput&&!t.invalidFormat&&!t.userInvalidated&&(!t.meridiem||t.meridiem&&a);if(e._strict&&(n=n&&0===t.charsLeftOver&&0===t.unusedTokens.length&&void 0===t.bigHour),null!=Object.isFrozen&&Object.isFrozen(e))return n;e._isValid=n}return e._isValid}function b(e){var t=v(NaN);return null!=e?m(g(t),e):g(t).userInvalidated=!0,t}r=Array.prototype.some?Array.prototype.some:function(e){var t,a=Object(this),n=a.length>>>0;for(t=0;t<n;t++)if(t in a&&e.call(this,a[t],t,a))return!0;return!1};var M=i.momentProperties=[],z=!1;function L(e,t){var a,n,r,i=M.length;if(d(t._isAMomentObject)||(e._isAMomentObject=t._isAMomentObject),d(t._i)||(e._i=t._i),d(t._f)||(e._f=t._f),d(t._l)||(e._l=t._l),d(t._strict)||(e._strict=t._strict),d(t._tzm)||(e._tzm=t._tzm),d(t._isUTC)||(e._isUTC=t._isUTC),d(t._offset)||(e._offset=t._offset),d(t._pf)||(e._pf=g(t)),d(t._locale)||(e._locale=t._locale),i>0)for(a=0;a<i;a++)n=M[a],r=t[n],d(r)||(e[n]=r);return e}function w(e){L(this,e),this._d=new Date(null!=e._d?e._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===z&&(z=!0,i.updateOffset(this),z=!1)}function H(e){return e instanceof w||null!=e&&null!=e._isAMomentObject}function k(e){!1===i.suppressDeprecationWarnings&&"undefined"!==typeof console&&console.warn&&console.warn("Deprecation warning: "+e)}function A(e,t){var a=!0;return m((function(){if(null!=i.deprecationHandler&&i.deprecationHandler(null,e),a){var n,r,o,s=[],l=arguments.length;for(r=0;r<l;r++){if(n="","object"===typeof arguments[r]){for(o in n+="\n["+r+"] ",arguments[0])c(arguments[0],o)&&(n+=o+": "+arguments[0][o]+", ");n=n.slice(0,-2)}else n=arguments[r];s.push(n)}k(e+"\nArguments: "+Array.prototype.slice.call(s).join("")+"\n"+(new Error).stack),a=!1}return t.apply(this,arguments)}),t)}var S,V={};function O(e,t){null!=i.deprecationHandler&&i.deprecationHandler(e,t),V[e]||(k(t),V[e]=!0)}function T(e){return"undefined"!==typeof Function&&e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}function D(e){var t,a;for(a in e)c(e,a)&&(t=e[a],T(t)?this[a]=t:this["_"+a]=t);this._config=e,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)}function C(e,t){var a,n=m({},e);for(a in t)c(t,a)&&(l(e[a])&&l(t[a])?(n[a]={},m(n[a],e[a]),m(n[a],t[a])):null!=t[a]?n[a]=t[a]:delete n[a]);for(a in e)c(e,a)&&!c(t,a)&&l(e[a])&&(n[a]=m({},n[a]));return n}function j(e){null!=e&&this.set(e)}i.suppressDeprecationWarnings=!1,i.deprecationHandler=null,S=Object.keys?Object.keys:function(e){var t,a=[];for(t in e)c(e,t)&&a.push(t);return a};var x={sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"};function B(e,t,a){var n=this._calendar[e]||this._calendar["sameElse"];return T(n)?n.call(t,a):n}function Y(e,t,a){var n=""+Math.abs(e),r=t-n.length,i=e>=0;return(i?a?"+":"":"-")+Math.pow(10,Math.max(0,r)).toString().substr(1)+n}var P=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,I=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,F={},E={};function $(e,t,a,n){var r=n;"string"===typeof n&&(r=function(){return this[n]()}),e&&(E[e]=r),t&&(E[t[0]]=function(){return Y(r.apply(this,arguments),t[1],t[2])}),a&&(E[a]=function(){return this.localeData().ordinal(r.apply(this,arguments),e)})}function R(e){return e.match(/\[[\s\S]/)?e.replace(/^\[|\]$/g,""):e.replace(/\\/g,"")}function N(e){var t,a,n=e.match(P);for(t=0,a=n.length;t<a;t++)E[n[t]]?n[t]=E[n[t]]:n[t]=R(n[t]);return function(t){var r,i="";for(r=0;r<a;r++)i+=T(n[r])?n[r].call(t,e):n[r];return i}}function W(e,t){return e.isValid()?(t=U(t,e.localeData()),F[t]=F[t]||N(t),F[t](e)):e.localeData().invalidDate()}function U(e,t){var a=5;function n(e){return t.longDateFormat(e)||e}I.lastIndex=0;while(a>=0&&I.test(e))e=e.replace(I,n),I.lastIndex=0,a-=1;return e}var G={LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"};function q(e){var t=this._longDateFormat[e],a=this._longDateFormat[e.toUpperCase()];return t||!a?t:(this._longDateFormat[e]=a.match(P).map((function(e){return"MMMM"===e||"MM"===e||"DD"===e||"dddd"===e?e.slice(1):e})).join(""),this._longDateFormat[e])}var J="Invalid date";function K(){return this._invalidDate}var X="%d",Z=/\d{1,2}/;function Q(e){return this._ordinal.replace("%d",e)}var ee={future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",w:"a week",ww:"%d weeks",M:"a month",MM:"%d months",y:"a year",yy:"%d years"};function te(e,t,a,n){var r=this._relativeTime[a];return T(r)?r(e,t,a,n):r.replace(/%d/i,e)}function ae(e,t){var a=this._relativeTime[e>0?"future":"past"];return T(a)?a(t):a.replace(/%s/i,t)}var ne={};function re(e,t){var a=e.toLowerCase();ne[a]=ne[a+"s"]=ne[t]=e}function ie(e){return"string"===typeof e?ne[e]||ne[e.toLowerCase()]:void 0}function oe(e){var t,a,n={};for(a in e)c(e,a)&&(t=ie(a),t&&(n[t]=e[a]));return n}var se={};function le(e,t){se[e]=t}function ce(e){var t,a=[];for(t in e)c(e,t)&&a.push({unit:t,priority:se[t]});return a.sort((function(e,t){return e.priority-t.priority})),a}function ue(e){return e%4===0&&e%100!==0||e%400===0}function de(e){return e<0?Math.ceil(e)||0:Math.floor(e)}function he(e){var t=+e,a=0;return 0!==t&&isFinite(t)&&(a=de(t)),a}function fe(e,t){return function(a){return null!=a?(me(this,e,a),i.updateOffset(this,t),this):pe(this,e)}}function pe(e,t){return e.isValid()?e._d["get"+(e._isUTC?"UTC":"")+t]():NaN}function me(e,t,a){e.isValid()&&!isNaN(a)&&("FullYear"===t&&ue(e.year())&&1===e.month()&&29===e.date()?(a=he(a),e._d["set"+(e._isUTC?"UTC":"")+t](a,e.month(),tt(a,e.month()))):e._d["set"+(e._isUTC?"UTC":"")+t](a))}function ve(e){return e=ie(e),T(this[e])?this[e]():this}function _e(e,t){if("object"===typeof e){e=oe(e);var a,n=ce(e),r=n.length;for(a=0;a<r;a++)this[n[a].unit](e[n[a].unit])}else if(e=ie(e),T(this[e]))return this[e](t);return this}var ge,ye=/\d/,be=/\d\d/,Me=/\d{3}/,ze=/\d{4}/,Le=/[+-]?\d{6}/,we=/\d\d?/,He=/\d\d\d\d?/,ke=/\d\d\d\d\d\d?/,Ae=/\d{1,3}/,Se=/\d{1,4}/,Ve=/[+-]?\d{1,6}/,Oe=/\d+/,Te=/[+-]?\d+/,De=/Z|[+-]\d\d:?\d\d/gi,Ce=/Z|[+-]\d\d(?::?\d\d)?/gi,je=/[+-]?\d+(\.\d{1,3})?/,xe=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i;function Be(e,t,a){ge[e]=T(t)?t:function(e,n){return e&&a?a:t}}function Ye(e,t){return c(ge,e)?ge[e](t._strict,t._locale):new RegExp(Pe(e))}function Pe(e){return Ie(e.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,(function(e,t,a,n,r){return t||a||n||r})))}function Ie(e){return e.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}ge={};var Fe={};function Ee(e,t){var a,n,r=t;for("string"===typeof e&&(e=[e]),h(t)&&(r=function(e,a){a[t]=he(e)}),n=e.length,a=0;a<n;a++)Fe[e[a]]=r}function $e(e,t){Ee(e,(function(e,a,n,r){n._w=n._w||{},t(e,n._w,n,r)}))}function Re(e,t,a){null!=t&&c(Fe,e)&&Fe[e](t,a._a,a,e)}var Ne,We=0,Ue=1,Ge=2,qe=3,Je=4,Ke=5,Xe=6,Ze=7,Qe=8;function et(e,t){return(e%t+t)%t}function tt(e,t){if(isNaN(e)||isNaN(t))return NaN;var a=et(t,12);return e+=(t-a)/12,1===a?ue(e)?29:28:31-a%7%2}Ne=Array.prototype.indexOf?Array.prototype.indexOf:function(e){var t;for(t=0;t<this.length;++t)if(this[t]===e)return t;return-1},$("M",["MM",2],"Mo",(function(){return this.month()+1})),$("MMM",0,0,(function(e){return this.localeData().monthsShort(this,e)})),$("MMMM",0,0,(function(e){return this.localeData().months(this,e)})),re("month","M"),le("month",8),Be("M",we),Be("MM",we,be),Be("MMM",(function(e,t){return t.monthsShortRegex(e)})),Be("MMMM",(function(e,t){return t.monthsRegex(e)})),Ee(["M","MM"],(function(e,t){t[Ue]=he(e)-1})),Ee(["MMM","MMMM"],(function(e,t,a,n){var r=a._locale.monthsParse(e,n,a._strict);null!=r?t[Ue]=r:g(a).invalidMonth=e}));var at="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),nt="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),rt=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,it=xe,ot=xe;function st(e,t){return e?s(this._months)?this._months[e.month()]:this._months[(this._months.isFormat||rt).test(t)?"format":"standalone"][e.month()]:s(this._months)?this._months:this._months["standalone"]}function lt(e,t){return e?s(this._monthsShort)?this._monthsShort[e.month()]:this._monthsShort[rt.test(t)?"format":"standalone"][e.month()]:s(this._monthsShort)?this._monthsShort:this._monthsShort["standalone"]}function ct(e,t,a){var n,r,i,o=e.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],n=0;n<12;++n)i=v([2e3,n]),this._shortMonthsParse[n]=this.monthsShort(i,"").toLocaleLowerCase(),this._longMonthsParse[n]=this.months(i,"").toLocaleLowerCase();return a?"MMM"===t?(r=Ne.call(this._shortMonthsParse,o),-1!==r?r:null):(r=Ne.call(this._longMonthsParse,o),-1!==r?r:null):"MMM"===t?(r=Ne.call(this._shortMonthsParse,o),-1!==r?r:(r=Ne.call(this._longMonthsParse,o),-1!==r?r:null)):(r=Ne.call(this._longMonthsParse,o),-1!==r?r:(r=Ne.call(this._shortMonthsParse,o),-1!==r?r:null))}function ut(e,t,a){var n,r,i;if(this._monthsParseExact)return ct.call(this,e,t,a);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),n=0;n<12;n++){if(r=v([2e3,n]),a&&!this._longMonthsParse[n]&&(this._longMonthsParse[n]=new RegExp("^"+this.months(r,"").replace(".","")+"$","i"),this._shortMonthsParse[n]=new RegExp("^"+this.monthsShort(r,"").replace(".","")+"$","i")),a||this._monthsParse[n]||(i="^"+this.months(r,"")+"|^"+this.monthsShort(r,""),this._monthsParse[n]=new RegExp(i.replace(".",""),"i")),a&&"MMMM"===t&&this._longMonthsParse[n].test(e))return n;if(a&&"MMM"===t&&this._shortMonthsParse[n].test(e))return n;if(!a&&this._monthsParse[n].test(e))return n}}function dt(e,t){var a;if(!e.isValid())return e;if("string"===typeof t)if(/^\d+$/.test(t))t=he(t);else if(t=e.localeData().monthsParse(t),!h(t))return e;return a=Math.min(e.date(),tt(e.year(),t)),e._d["set"+(e._isUTC?"UTC":"")+"Month"](t,a),e}function ht(e){return null!=e?(dt(this,e),i.updateOffset(this,!0),this):pe(this,"Month")}function ft(){return tt(this.year(),this.month())}function pt(e){return this._monthsParseExact?(c(this,"_monthsRegex")||vt.call(this),e?this._monthsShortStrictRegex:this._monthsShortRegex):(c(this,"_monthsShortRegex")||(this._monthsShortRegex=it),this._monthsShortStrictRegex&&e?this._monthsShortStrictRegex:this._monthsShortRegex)}function mt(e){return this._monthsParseExact?(c(this,"_monthsRegex")||vt.call(this),e?this._monthsStrictRegex:this._monthsRegex):(c(this,"_monthsRegex")||(this._monthsRegex=ot),this._monthsStrictRegex&&e?this._monthsStrictRegex:this._monthsRegex)}function vt(){function e(e,t){return t.length-e.length}var t,a,n=[],r=[],i=[];for(t=0;t<12;t++)a=v([2e3,t]),n.push(this.monthsShort(a,"")),r.push(this.months(a,"")),i.push(this.months(a,"")),i.push(this.monthsShort(a,""));for(n.sort(e),r.sort(e),i.sort(e),t=0;t<12;t++)n[t]=Ie(n[t]),r[t]=Ie(r[t]);for(t=0;t<24;t++)i[t]=Ie(i[t]);this._monthsRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+r.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+n.join("|")+")","i")}function _t(e){return ue(e)?366:365}$("Y",0,0,(function(){var e=this.year();return e<=9999?Y(e,4):"+"+e})),$(0,["YY",2],0,(function(){return this.year()%100})),$(0,["YYYY",4],0,"year"),$(0,["YYYYY",5],0,"year"),$(0,["YYYYYY",6,!0],0,"year"),re("year","y"),le("year",1),Be("Y",Te),Be("YY",we,be),Be("YYYY",Se,ze),Be("YYYYY",Ve,Le),Be("YYYYYY",Ve,Le),Ee(["YYYYY","YYYYYY"],We),Ee("YYYY",(function(e,t){t[We]=2===e.length?i.parseTwoDigitYear(e):he(e)})),Ee("YY",(function(e,t){t[We]=i.parseTwoDigitYear(e)})),Ee("Y",(function(e,t){t[We]=parseInt(e,10)})),i.parseTwoDigitYear=function(e){return he(e)+(he(e)>68?1900:2e3)};var gt=fe("FullYear",!0);function yt(){return ue(this.year())}function bt(e,t,a,n,r,i,o){var s;return e<100&&e>=0?(s=new Date(e+400,t,a,n,r,i,o),isFinite(s.getFullYear())&&s.setFullYear(e)):s=new Date(e,t,a,n,r,i,o),s}function Mt(e){var t,a;return e<100&&e>=0?(a=Array.prototype.slice.call(arguments),a[0]=e+400,t=new Date(Date.UTC.apply(null,a)),isFinite(t.getUTCFullYear())&&t.setUTCFullYear(e)):t=new Date(Date.UTC.apply(null,arguments)),t}function zt(e,t,a){var n=7+t-a,r=(7+Mt(e,0,n).getUTCDay()-t)%7;return-r+n-1}function Lt(e,t,a,n,r){var i,o,s=(7+a-n)%7,l=zt(e,n,r),c=1+7*(t-1)+s+l;return c<=0?(i=e-1,o=_t(i)+c):c>_t(e)?(i=e+1,o=c-_t(e)):(i=e,o=c),{year:i,dayOfYear:o}}function wt(e,t,a){var n,r,i=zt(e.year(),t,a),o=Math.floor((e.dayOfYear()-i-1)/7)+1;return o<1?(r=e.year()-1,n=o+Ht(r,t,a)):o>Ht(e.year(),t,a)?(n=o-Ht(e.year(),t,a),r=e.year()+1):(r=e.year(),n=o),{week:n,year:r}}function Ht(e,t,a){var n=zt(e,t,a),r=zt(e+1,t,a);return(_t(e)-n+r)/7}function kt(e){return wt(e,this._week.dow,this._week.doy).week}$("w",["ww",2],"wo","week"),$("W",["WW",2],"Wo","isoWeek"),re("week","w"),re("isoWeek","W"),le("week",5),le("isoWeek",5),Be("w",we),Be("ww",we,be),Be("W",we),Be("WW",we,be),$e(["w","ww","W","WW"],(function(e,t,a,n){t[n.substr(0,1)]=he(e)}));var At={dow:0,doy:6};function St(){return this._week.dow}function Vt(){return this._week.doy}function Ot(e){var t=this.localeData().week(this);return null==e?t:this.add(7*(e-t),"d")}function Tt(e){var t=wt(this,1,4).week;return null==e?t:this.add(7*(e-t),"d")}function Dt(e,t){return"string"!==typeof e?e:isNaN(e)?(e=t.weekdaysParse(e),"number"===typeof e?e:null):parseInt(e,10)}function Ct(e,t){return"string"===typeof e?t.weekdaysParse(e)%7||7:isNaN(e)?null:e}function jt(e,t){return e.slice(t,7).concat(e.slice(0,t))}$("d",0,"do","day"),$("dd",0,0,(function(e){return this.localeData().weekdaysMin(this,e)})),$("ddd",0,0,(function(e){return this.localeData().weekdaysShort(this,e)})),$("dddd",0,0,(function(e){return this.localeData().weekdays(this,e)})),$("e",0,0,"weekday"),$("E",0,0,"isoWeekday"),re("day","d"),re("weekday","e"),re("isoWeekday","E"),le("day",11),le("weekday",11),le("isoWeekday",11),Be("d",we),Be("e",we),Be("E",we),Be("dd",(function(e,t){return t.weekdaysMinRegex(e)})),Be("ddd",(function(e,t){return t.weekdaysShortRegex(e)})),Be("dddd",(function(e,t){return t.weekdaysRegex(e)})),$e(["dd","ddd","dddd"],(function(e,t,a,n){var r=a._locale.weekdaysParse(e,n,a._strict);null!=r?t.d=r:g(a).invalidWeekday=e})),$e(["d","e","E"],(function(e,t,a,n){t[n]=he(e)}));var xt="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),Bt="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),Yt="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),Pt=xe,It=xe,Ft=xe;function Et(e,t){var a=s(this._weekdays)?this._weekdays:this._weekdays[e&&!0!==e&&this._weekdays.isFormat.test(t)?"format":"standalone"];return!0===e?jt(a,this._week.dow):e?a[e.day()]:a}function $t(e){return!0===e?jt(this._weekdaysShort,this._week.dow):e?this._weekdaysShort[e.day()]:this._weekdaysShort}function Rt(e){return!0===e?jt(this._weekdaysMin,this._week.dow):e?this._weekdaysMin[e.day()]:this._weekdaysMin}function Nt(e,t,a){var n,r,i,o=e.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],n=0;n<7;++n)i=v([2e3,1]).day(n),this._minWeekdaysParse[n]=this.weekdaysMin(i,"").toLocaleLowerCase(),this._shortWeekdaysParse[n]=this.weekdaysShort(i,"").toLocaleLowerCase(),this._weekdaysParse[n]=this.weekdays(i,"").toLocaleLowerCase();return a?"dddd"===t?(r=Ne.call(this._weekdaysParse,o),-1!==r?r:null):"ddd"===t?(r=Ne.call(this._shortWeekdaysParse,o),-1!==r?r:null):(r=Ne.call(this._minWeekdaysParse,o),-1!==r?r:null):"dddd"===t?(r=Ne.call(this._weekdaysParse,o),-1!==r?r:(r=Ne.call(this._shortWeekdaysParse,o),-1!==r?r:(r=Ne.call(this._minWeekdaysParse,o),-1!==r?r:null))):"ddd"===t?(r=Ne.call(this._shortWeekdaysParse,o),-1!==r?r:(r=Ne.call(this._weekdaysParse,o),-1!==r?r:(r=Ne.call(this._minWeekdaysParse,o),-1!==r?r:null))):(r=Ne.call(this._minWeekdaysParse,o),-1!==r?r:(r=Ne.call(this._weekdaysParse,o),-1!==r?r:(r=Ne.call(this._shortWeekdaysParse,o),-1!==r?r:null)))}function Wt(e,t,a){var n,r,i;if(this._weekdaysParseExact)return Nt.call(this,e,t,a);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),n=0;n<7;n++){if(r=v([2e3,1]).day(n),a&&!this._fullWeekdaysParse[n]&&(this._fullWeekdaysParse[n]=new RegExp("^"+this.weekdays(r,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[n]=new RegExp("^"+this.weekdaysShort(r,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[n]=new RegExp("^"+this.weekdaysMin(r,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[n]||(i="^"+this.weekdays(r,"")+"|^"+this.weekdaysShort(r,"")+"|^"+this.weekdaysMin(r,""),this._weekdaysParse[n]=new RegExp(i.replace(".",""),"i")),a&&"dddd"===t&&this._fullWeekdaysParse[n].test(e))return n;if(a&&"ddd"===t&&this._shortWeekdaysParse[n].test(e))return n;if(a&&"dd"===t&&this._minWeekdaysParse[n].test(e))return n;if(!a&&this._weekdaysParse[n].test(e))return n}}function Ut(e){if(!this.isValid())return null!=e?this:NaN;var t=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=e?(e=Dt(e,this.localeData()),this.add(e-t,"d")):t}function Gt(e){if(!this.isValid())return null!=e?this:NaN;var t=(this.day()+7-this.localeData()._week.dow)%7;return null==e?t:this.add(e-t,"d")}function qt(e){if(!this.isValid())return null!=e?this:NaN;if(null!=e){var t=Ct(e,this.localeData());return this.day(this.day()%7?t:t-7)}return this.day()||7}function Jt(e){return this._weekdaysParseExact?(c(this,"_weekdaysRegex")||Zt.call(this),e?this._weekdaysStrictRegex:this._weekdaysRegex):(c(this,"_weekdaysRegex")||(this._weekdaysRegex=Pt),this._weekdaysStrictRegex&&e?this._weekdaysStrictRegex:this._weekdaysRegex)}function Kt(e){return this._weekdaysParseExact?(c(this,"_weekdaysRegex")||Zt.call(this),e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(c(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=It),this._weekdaysShortStrictRegex&&e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)}function Xt(e){return this._weekdaysParseExact?(c(this,"_weekdaysRegex")||Zt.call(this),e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(c(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=Ft),this._weekdaysMinStrictRegex&&e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)}function Zt(){function e(e,t){return t.length-e.length}var t,a,n,r,i,o=[],s=[],l=[],c=[];for(t=0;t<7;t++)a=v([2e3,1]).day(t),n=Ie(this.weekdaysMin(a,"")),r=Ie(this.weekdaysShort(a,"")),i=Ie(this.weekdays(a,"")),o.push(n),s.push(r),l.push(i),c.push(n),c.push(r),c.push(i);o.sort(e),s.sort(e),l.sort(e),c.sort(e),this._weekdaysRegex=new RegExp("^("+c.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+s.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+o.join("|")+")","i")}function Qt(){return this.hours()%12||12}function ea(){return this.hours()||24}function ta(e,t){$(e,0,0,(function(){return this.localeData().meridiem(this.hours(),this.minutes(),t)}))}function aa(e,t){return t._meridiemParse}function na(e){return"p"===(e+"").toLowerCase().charAt(0)}$("H",["HH",2],0,"hour"),$("h",["hh",2],0,Qt),$("k",["kk",2],0,ea),$("hmm",0,0,(function(){return""+Qt.apply(this)+Y(this.minutes(),2)})),$("hmmss",0,0,(function(){return""+Qt.apply(this)+Y(this.minutes(),2)+Y(this.seconds(),2)})),$("Hmm",0,0,(function(){return""+this.hours()+Y(this.minutes(),2)})),$("Hmmss",0,0,(function(){return""+this.hours()+Y(this.minutes(),2)+Y(this.seconds(),2)})),ta("a",!0),ta("A",!1),re("hour","h"),le("hour",13),Be("a",aa),Be("A",aa),Be("H",we),Be("h",we),Be("k",we),Be("HH",we,be),Be("hh",we,be),Be("kk",we,be),Be("hmm",He),Be("hmmss",ke),Be("Hmm",He),Be("Hmmss",ke),Ee(["H","HH"],qe),Ee(["k","kk"],(function(e,t,a){var n=he(e);t[qe]=24===n?0:n})),Ee(["a","A"],(function(e,t,a){a._isPm=a._locale.isPM(e),a._meridiem=e})),Ee(["h","hh"],(function(e,t,a){t[qe]=he(e),g(a).bigHour=!0})),Ee("hmm",(function(e,t,a){var n=e.length-2;t[qe]=he(e.substr(0,n)),t[Je]=he(e.substr(n)),g(a).bigHour=!0})),Ee("hmmss",(function(e,t,a){var n=e.length-4,r=e.length-2;t[qe]=he(e.substr(0,n)),t[Je]=he(e.substr(n,2)),t[Ke]=he(e.substr(r)),g(a).bigHour=!0})),Ee("Hmm",(function(e,t,a){var n=e.length-2;t[qe]=he(e.substr(0,n)),t[Je]=he(e.substr(n))})),Ee("Hmmss",(function(e,t,a){var n=e.length-4,r=e.length-2;t[qe]=he(e.substr(0,n)),t[Je]=he(e.substr(n,2)),t[Ke]=he(e.substr(r))}));var ra=/[ap]\.?m?\.?/i,ia=fe("Hours",!0);function oa(e,t,a){return e>11?a?"pm":"PM":a?"am":"AM"}var sa,la={calendar:x,longDateFormat:G,invalidDate:J,ordinal:X,dayOfMonthOrdinalParse:Z,relativeTime:ee,months:at,monthsShort:nt,week:At,weekdays:xt,weekdaysMin:Yt,weekdaysShort:Bt,meridiemParse:ra},ca={},ua={};function da(e,t){var a,n=Math.min(e.length,t.length);for(a=0;a<n;a+=1)if(e[a]!==t[a])return a;return n}function ha(e){return e?e.toLowerCase().replace("_","-"):e}function fa(e){var t,a,n,r,i=0;while(i<e.length){r=ha(e[i]).split("-"),t=r.length,a=ha(e[i+1]),a=a?a.split("-"):null;while(t>0){if(n=ma(r.slice(0,t).join("-")),n)return n;if(a&&a.length>=t&&da(r,a)>=t-1)break;t--}i++}return sa}function pa(e){return null!=e.match("^[^/\\\\]*$")}function ma(n){var r=null;if(void 0===ca[n]&&"undefined"!==typeof e&&e&&e.exports&&pa(n))try{r=sa._abbr,t,a("4678")("./"+n),va(r)}catch(i){ca[n]=null}return ca[n]}function va(e,t){var a;return e&&(a=d(t)?ya(e):_a(e,t),a?sa=a:"undefined"!==typeof console&&console.warn&&console.warn("Locale "+e+" not found. Did you forget to load it?")),sa._abbr}function _a(e,t){if(null!==t){var a,n=la;if(t.abbr=e,null!=ca[e])O("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),n=ca[e]._config;else if(null!=t.parentLocale)if(null!=ca[t.parentLocale])n=ca[t.parentLocale]._config;else{if(a=ma(t.parentLocale),null==a)return ua[t.parentLocale]||(ua[t.parentLocale]=[]),ua[t.parentLocale].push({name:e,config:t}),null;n=a._config}return ca[e]=new j(C(n,t)),ua[e]&&ua[e].forEach((function(e){_a(e.name,e.config)})),va(e),ca[e]}return delete ca[e],null}function ga(e,t){if(null!=t){var a,n,r=la;null!=ca[e]&&null!=ca[e].parentLocale?ca[e].set(C(ca[e]._config,t)):(n=ma(e),null!=n&&(r=n._config),t=C(r,t),null==n&&(t.abbr=e),a=new j(t),a.parentLocale=ca[e],ca[e]=a),va(e)}else null!=ca[e]&&(null!=ca[e].parentLocale?(ca[e]=ca[e].parentLocale,e===va()&&va(e)):null!=ca[e]&&delete ca[e]);return ca[e]}function ya(e){var t;if(e&&e._locale&&e._locale._abbr&&(e=e._locale._abbr),!e)return sa;if(!s(e)){if(t=ma(e),t)return t;e=[e]}return fa(e)}function ba(){return S(ca)}function Ma(e){var t,a=e._a;return a&&-2===g(e).overflow&&(t=a[Ue]<0||a[Ue]>11?Ue:a[Ge]<1||a[Ge]>tt(a[We],a[Ue])?Ge:a[qe]<0||a[qe]>24||24===a[qe]&&(0!==a[Je]||0!==a[Ke]||0!==a[Xe])?qe:a[Je]<0||a[Je]>59?Je:a[Ke]<0||a[Ke]>59?Ke:a[Xe]<0||a[Xe]>999?Xe:-1,g(e)._overflowDayOfYear&&(t<We||t>Ge)&&(t=Ge),g(e)._overflowWeeks&&-1===t&&(t=Ze),g(e)._overflowWeekday&&-1===t&&(t=Qe),g(e).overflow=t),e}var za=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,La=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,wa=/Z|[+-]\d\d(?::?\d\d)?/,Ha=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/],["YYYYMM",/\d{6}/,!1],["YYYY",/\d{4}/,!1]],ka=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],Aa=/^\/?Date\((-?\d+)/i,Sa=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,Va={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function Oa(e){var t,a,n,r,i,o,s=e._i,l=za.exec(s)||La.exec(s),c=Ha.length,u=ka.length;if(l){for(g(e).iso=!0,t=0,a=c;t<a;t++)if(Ha[t][1].exec(l[1])){r=Ha[t][0],n=!1!==Ha[t][2];break}if(null==r)return void(e._isValid=!1);if(l[3]){for(t=0,a=u;t<a;t++)if(ka[t][1].exec(l[3])){i=(l[2]||" ")+ka[t][0];break}if(null==i)return void(e._isValid=!1)}if(!n&&null!=i)return void(e._isValid=!1);if(l[4]){if(!wa.exec(l[4]))return void(e._isValid=!1);o="Z"}e._f=r+(i||"")+(o||""),$a(e)}else e._isValid=!1}function Ta(e,t,a,n,r,i){var o=[Da(e),nt.indexOf(t),parseInt(a,10),parseInt(n,10),parseInt(r,10)];return i&&o.push(parseInt(i,10)),o}function Da(e){var t=parseInt(e,10);return t<=49?2e3+t:t<=999?1900+t:t}function Ca(e){return e.replace(/\([^()]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,"")}function ja(e,t,a){if(e){var n=Bt.indexOf(e),r=new Date(t[0],t[1],t[2]).getDay();if(n!==r)return g(a).weekdayMismatch=!0,a._isValid=!1,!1}return!0}function xa(e,t,a){if(e)return Va[e];if(t)return 0;var n=parseInt(a,10),r=n%100,i=(n-r)/100;return 60*i+r}function Ba(e){var t,a=Sa.exec(Ca(e._i));if(a){if(t=Ta(a[4],a[3],a[2],a[5],a[6],a[7]),!ja(a[1],t,e))return;e._a=t,e._tzm=xa(a[8],a[9],a[10]),e._d=Mt.apply(null,e._a),e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),g(e).rfc2822=!0}else e._isValid=!1}function Ya(e){var t=Aa.exec(e._i);null===t?(Oa(e),!1===e._isValid&&(delete e._isValid,Ba(e),!1===e._isValid&&(delete e._isValid,e._strict?e._isValid=!1:i.createFromInputFallback(e)))):e._d=new Date(+t[1])}function Pa(e,t,a){return null!=e?e:null!=t?t:a}function Ia(e){var t=new Date(i.now());return e._useUTC?[t.getUTCFullYear(),t.getUTCMonth(),t.getUTCDate()]:[t.getFullYear(),t.getMonth(),t.getDate()]}function Fa(e){var t,a,n,r,i,o=[];if(!e._d){for(n=Ia(e),e._w&&null==e._a[Ge]&&null==e._a[Ue]&&Ea(e),null!=e._dayOfYear&&(i=Pa(e._a[We],n[We]),(e._dayOfYear>_t(i)||0===e._dayOfYear)&&(g(e)._overflowDayOfYear=!0),a=Mt(i,0,e._dayOfYear),e._a[Ue]=a.getUTCMonth(),e._a[Ge]=a.getUTCDate()),t=0;t<3&&null==e._a[t];++t)e._a[t]=o[t]=n[t];for(;t<7;t++)e._a[t]=o[t]=null==e._a[t]?2===t?1:0:e._a[t];24===e._a[qe]&&0===e._a[Je]&&0===e._a[Ke]&&0===e._a[Xe]&&(e._nextDay=!0,e._a[qe]=0),e._d=(e._useUTC?Mt:bt).apply(null,o),r=e._useUTC?e._d.getUTCDay():e._d.getDay(),null!=e._tzm&&e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),e._nextDay&&(e._a[qe]=24),e._w&&"undefined"!==typeof e._w.d&&e._w.d!==r&&(g(e).weekdayMismatch=!0)}}function Ea(e){var t,a,n,r,i,o,s,l,c;t=e._w,null!=t.GG||null!=t.W||null!=t.E?(i=1,o=4,a=Pa(t.GG,e._a[We],wt(Ka(),1,4).year),n=Pa(t.W,1),r=Pa(t.E,1),(r<1||r>7)&&(l=!0)):(i=e._locale._week.dow,o=e._locale._week.doy,c=wt(Ka(),i,o),a=Pa(t.gg,e._a[We],c.year),n=Pa(t.w,c.week),null!=t.d?(r=t.d,(r<0||r>6)&&(l=!0)):null!=t.e?(r=t.e+i,(t.e<0||t.e>6)&&(l=!0)):r=i),n<1||n>Ht(a,i,o)?g(e)._overflowWeeks=!0:null!=l?g(e)._overflowWeekday=!0:(s=Lt(a,n,r,i,o),e._a[We]=s.year,e._dayOfYear=s.dayOfYear)}function $a(e){if(e._f!==i.ISO_8601)if(e._f!==i.RFC_2822){e._a=[],g(e).empty=!0;var t,a,n,r,o,s,l,c=""+e._i,u=c.length,d=0;for(n=U(e._f,e._locale).match(P)||[],l=n.length,t=0;t<l;t++)r=n[t],a=(c.match(Ye(r,e))||[])[0],a&&(o=c.substr(0,c.indexOf(a)),o.length>0&&g(e).unusedInput.push(o),c=c.slice(c.indexOf(a)+a.length),d+=a.length),E[r]?(a?g(e).empty=!1:g(e).unusedTokens.push(r),Re(r,a,e)):e._strict&&!a&&g(e).unusedTokens.push(r);g(e).charsLeftOver=u-d,c.length>0&&g(e).unusedInput.push(c),e._a[qe]<=12&&!0===g(e).bigHour&&e._a[qe]>0&&(g(e).bigHour=void 0),g(e).parsedDateParts=e._a.slice(0),g(e).meridiem=e._meridiem,e._a[qe]=Ra(e._locale,e._a[qe],e._meridiem),s=g(e).era,null!==s&&(e._a[We]=e._locale.erasConvertYear(s,e._a[We])),Fa(e),Ma(e)}else Ba(e);else Oa(e)}function Ra(e,t,a){var n;return null==a?t:null!=e.meridiemHour?e.meridiemHour(t,a):null!=e.isPM?(n=e.isPM(a),n&&t<12&&(t+=12),n||12!==t||(t=0),t):t}function Na(e){var t,a,n,r,i,o,s=!1,l=e._f.length;if(0===l)return g(e).invalidFormat=!0,void(e._d=new Date(NaN));for(r=0;r<l;r++)i=0,o=!1,t=L({},e),null!=e._useUTC&&(t._useUTC=e._useUTC),t._f=e._f[r],$a(t),y(t)&&(o=!0),i+=g(t).charsLeftOver,i+=10*g(t).unusedTokens.length,g(t).score=i,s?i<n&&(n=i,a=t):(null==n||i<n||o)&&(n=i,a=t,o&&(s=!0));m(e,a||t)}function Wa(e){if(!e._d){var t=oe(e._i),a=void 0===t.day?t.date:t.day;e._a=p([t.year,t.month,a,t.hour,t.minute,t.second,t.millisecond],(function(e){return e&&parseInt(e,10)})),Fa(e)}}function Ua(e){var t=new w(Ma(Ga(e)));return t._nextDay&&(t.add(1,"d"),t._nextDay=void 0),t}function Ga(e){var t=e._i,a=e._f;return e._locale=e._locale||ya(e._l),null===t||void 0===a&&""===t?b({nullInput:!0}):("string"===typeof t&&(e._i=t=e._locale.preparse(t)),H(t)?new w(Ma(t)):(f(t)?e._d=t:s(a)?Na(e):a?$a(e):qa(e),y(e)||(e._d=null),e))}function qa(e){var t=e._i;d(t)?e._d=new Date(i.now()):f(t)?e._d=new Date(t.valueOf()):"string"===typeof t?Ya(e):s(t)?(e._a=p(t.slice(0),(function(e){return parseInt(e,10)})),Fa(e)):l(t)?Wa(e):h(t)?e._d=new Date(t):i.createFromInputFallback(e)}function Ja(e,t,a,n,r){var i={};return!0!==t&&!1!==t||(n=t,t=void 0),!0!==a&&!1!==a||(n=a,a=void 0),(l(e)&&u(e)||s(e)&&0===e.length)&&(e=void 0),i._isAMomentObject=!0,i._useUTC=i._isUTC=r,i._l=a,i._i=e,i._f=t,i._strict=n,Ua(i)}function Ka(e,t,a,n){return Ja(e,t,a,n,!1)}i.createFromInputFallback=A("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",(function(e){e._d=new Date(e._i+(e._useUTC?" UTC":""))})),i.ISO_8601=function(){},i.RFC_2822=function(){};var Xa=A("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=Ka.apply(null,arguments);return this.isValid()&&e.isValid()?e<this?this:e:b()})),Za=A("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=Ka.apply(null,arguments);return this.isValid()&&e.isValid()?e>this?this:e:b()}));function Qa(e,t){var a,n;if(1===t.length&&s(t[0])&&(t=t[0]),!t.length)return Ka();for(a=t[0],n=1;n<t.length;++n)t[n].isValid()&&!t[n][e](a)||(a=t[n]);return a}function en(){var e=[].slice.call(arguments,0);return Qa("isBefore",e)}function tn(){var e=[].slice.call(arguments,0);return Qa("isAfter",e)}var an=function(){return Date.now?Date.now():+new Date},nn=["year","quarter","month","week","day","hour","minute","second","millisecond"];function rn(e){var t,a,n=!1,r=nn.length;for(t in e)if(c(e,t)&&(-1===Ne.call(nn,t)||null!=e[t]&&isNaN(e[t])))return!1;for(a=0;a<r;++a)if(e[nn[a]]){if(n)return!1;parseFloat(e[nn[a]])!==he(e[nn[a]])&&(n=!0)}return!0}function on(){return this._isValid}function sn(){return On(NaN)}function ln(e){var t=oe(e),a=t.year||0,n=t.quarter||0,r=t.month||0,i=t.week||t.isoWeek||0,o=t.day||0,s=t.hour||0,l=t.minute||0,c=t.second||0,u=t.millisecond||0;this._isValid=rn(t),this._milliseconds=+u+1e3*c+6e4*l+1e3*s*60*60,this._days=+o+7*i,this._months=+r+3*n+12*a,this._data={},this._locale=ya(),this._bubble()}function cn(e){return e instanceof ln}function un(e){return e<0?-1*Math.round(-1*e):Math.round(e)}function dn(e,t,a){var n,r=Math.min(e.length,t.length),i=Math.abs(e.length-t.length),o=0;for(n=0;n<r;n++)(a&&e[n]!==t[n]||!a&&he(e[n])!==he(t[n]))&&o++;return o+i}function hn(e,t){$(e,0,0,(function(){var e=this.utcOffset(),a="+";return e<0&&(e=-e,a="-"),a+Y(~~(e/60),2)+t+Y(~~e%60,2)}))}hn("Z",":"),hn("ZZ",""),Be("Z",Ce),Be("ZZ",Ce),Ee(["Z","ZZ"],(function(e,t,a){a._useUTC=!0,a._tzm=pn(Ce,e)}));var fn=/([\+\-]|\d\d)/gi;function pn(e,t){var a,n,r,i=(t||"").match(e);return null===i?null:(a=i[i.length-1]||[],n=(a+"").match(fn)||["-",0,0],r=60*n[1]+he(n[2]),0===r?0:"+"===n[0]?r:-r)}function mn(e,t){var a,n;return t._isUTC?(a=t.clone(),n=(H(e)||f(e)?e.valueOf():Ka(e).valueOf())-a.valueOf(),a._d.setTime(a._d.valueOf()+n),i.updateOffset(a,!1),a):Ka(e).local()}function vn(e){return-Math.round(e._d.getTimezoneOffset())}function _n(e,t,a){var n,r=this._offset||0;if(!this.isValid())return null!=e?this:NaN;if(null!=e){if("string"===typeof e){if(e=pn(Ce,e),null===e)return this}else Math.abs(e)<16&&!a&&(e*=60);return!this._isUTC&&t&&(n=vn(this)),this._offset=e,this._isUTC=!0,null!=n&&this.add(n,"m"),r!==e&&(!t||this._changeInProgress?xn(this,On(e-r,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,i.updateOffset(this,!0),this._changeInProgress=null)),this}return this._isUTC?r:vn(this)}function gn(e,t){return null!=e?("string"!==typeof e&&(e=-e),this.utcOffset(e,t),this):-this.utcOffset()}function yn(e){return this.utcOffset(0,e)}function bn(e){return this._isUTC&&(this.utcOffset(0,e),this._isUTC=!1,e&&this.subtract(vn(this),"m")),this}function Mn(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"===typeof this._i){var e=pn(De,this._i);null!=e?this.utcOffset(e):this.utcOffset(0,!0)}return this}function zn(e){return!!this.isValid()&&(e=e?Ka(e).utcOffset():0,(this.utcOffset()-e)%60===0)}function Ln(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()}function wn(){if(!d(this._isDSTShifted))return this._isDSTShifted;var e,t={};return L(t,this),t=Ga(t),t._a?(e=t._isUTC?v(t._a):Ka(t._a),this._isDSTShifted=this.isValid()&&dn(t._a,e.toArray())>0):this._isDSTShifted=!1,this._isDSTShifted}function Hn(){return!!this.isValid()&&!this._isUTC}function kn(){return!!this.isValid()&&this._isUTC}function An(){return!!this.isValid()&&(this._isUTC&&0===this._offset)}i.updateOffset=function(){};var Sn=/^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,Vn=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function On(e,t){var a,n,r,i=e,o=null;return cn(e)?i={ms:e._milliseconds,d:e._days,M:e._months}:h(e)||!isNaN(+e)?(i={},t?i[t]=+e:i.milliseconds=+e):(o=Sn.exec(e))?(a="-"===o[1]?-1:1,i={y:0,d:he(o[Ge])*a,h:he(o[qe])*a,m:he(o[Je])*a,s:he(o[Ke])*a,ms:he(un(1e3*o[Xe]))*a}):(o=Vn.exec(e))?(a="-"===o[1]?-1:1,i={y:Tn(o[2],a),M:Tn(o[3],a),w:Tn(o[4],a),d:Tn(o[5],a),h:Tn(o[6],a),m:Tn(o[7],a),s:Tn(o[8],a)}):null==i?i={}:"object"===typeof i&&("from"in i||"to"in i)&&(r=Cn(Ka(i.from),Ka(i.to)),i={},i.ms=r.milliseconds,i.M=r.months),n=new ln(i),cn(e)&&c(e,"_locale")&&(n._locale=e._locale),cn(e)&&c(e,"_isValid")&&(n._isValid=e._isValid),n}function Tn(e,t){var a=e&&parseFloat(e.replace(",","."));return(isNaN(a)?0:a)*t}function Dn(e,t){var a={};return a.months=t.month()-e.month()+12*(t.year()-e.year()),e.clone().add(a.months,"M").isAfter(t)&&--a.months,a.milliseconds=+t-+e.clone().add(a.months,"M"),a}function Cn(e,t){var a;return e.isValid()&&t.isValid()?(t=mn(t,e),e.isBefore(t)?a=Dn(e,t):(a=Dn(t,e),a.milliseconds=-a.milliseconds,a.months=-a.months),a):{milliseconds:0,months:0}}function jn(e,t){return function(a,n){var r,i;return null===n||isNaN(+n)||(O(t,"moment()."+t+"(period, number) is deprecated. Please use moment()."+t+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),i=a,a=n,n=i),r=On(a,n),xn(this,r,e),this}}function xn(e,t,a,n){var r=t._milliseconds,o=un(t._days),s=un(t._months);e.isValid()&&(n=null==n||n,s&&dt(e,pe(e,"Month")+s*a),o&&me(e,"Date",pe(e,"Date")+o*a),r&&e._d.setTime(e._d.valueOf()+r*a),n&&i.updateOffset(e,o||s))}On.fn=ln.prototype,On.invalid=sn;var Bn=jn(1,"add"),Yn=jn(-1,"subtract");function Pn(e){return"string"===typeof e||e instanceof String}function In(e){return H(e)||f(e)||Pn(e)||h(e)||En(e)||Fn(e)||null===e||void 0===e}function Fn(e){var t,a,n=l(e)&&!u(e),r=!1,i=["years","year","y","months","month","M","days","day","d","dates","date","D","hours","hour","h","minutes","minute","m","seconds","second","s","milliseconds","millisecond","ms"],o=i.length;for(t=0;t<o;t+=1)a=i[t],r=r||c(e,a);return n&&r}function En(e){var t=s(e),a=!1;return t&&(a=0===e.filter((function(t){return!h(t)&&Pn(e)})).length),t&&a}function $n(e){var t,a,n=l(e)&&!u(e),r=!1,i=["sameDay","nextDay","lastDay","nextWeek","lastWeek","sameElse"];for(t=0;t<i.length;t+=1)a=i[t],r=r||c(e,a);return n&&r}function Rn(e,t){var a=e.diff(t,"days",!0);return a<-6?"sameElse":a<-1?"lastWeek":a<0?"lastDay":a<1?"sameDay":a<2?"nextDay":a<7?"nextWeek":"sameElse"}function Nn(e,t){1===arguments.length&&(arguments[0]?In(arguments[0])?(e=arguments[0],t=void 0):$n(arguments[0])&&(t=arguments[0],e=void 0):(e=void 0,t=void 0));var a=e||Ka(),n=mn(a,this).startOf("day"),r=i.calendarFormat(this,n)||"sameElse",o=t&&(T(t[r])?t[r].call(this,a):t[r]);return this.format(o||this.localeData().calendar(r,this,Ka(a)))}function Wn(){return new w(this)}function Un(e,t){var a=H(e)?e:Ka(e);return!(!this.isValid()||!a.isValid())&&(t=ie(t)||"millisecond","millisecond"===t?this.valueOf()>a.valueOf():a.valueOf()<this.clone().startOf(t).valueOf())}function Gn(e,t){var a=H(e)?e:Ka(e);return!(!this.isValid()||!a.isValid())&&(t=ie(t)||"millisecond","millisecond"===t?this.valueOf()<a.valueOf():this.clone().endOf(t).valueOf()<a.valueOf())}function qn(e,t,a,n){var r=H(e)?e:Ka(e),i=H(t)?t:Ka(t);return!!(this.isValid()&&r.isValid()&&i.isValid())&&(n=n||"()",("("===n[0]?this.isAfter(r,a):!this.isBefore(r,a))&&(")"===n[1]?this.isBefore(i,a):!this.isAfter(i,a)))}function Jn(e,t){var a,n=H(e)?e:Ka(e);return!(!this.isValid()||!n.isValid())&&(t=ie(t)||"millisecond","millisecond"===t?this.valueOf()===n.valueOf():(a=n.valueOf(),this.clone().startOf(t).valueOf()<=a&&a<=this.clone().endOf(t).valueOf()))}function Kn(e,t){return this.isSame(e,t)||this.isAfter(e,t)}function Xn(e,t){return this.isSame(e,t)||this.isBefore(e,t)}function Zn(e,t,a){var n,r,i;if(!this.isValid())return NaN;if(n=mn(e,this),!n.isValid())return NaN;switch(r=6e4*(n.utcOffset()-this.utcOffset()),t=ie(t),t){case"year":i=Qn(this,n)/12;break;case"month":i=Qn(this,n);break;case"quarter":i=Qn(this,n)/3;break;case"second":i=(this-n)/1e3;break;case"minute":i=(this-n)/6e4;break;case"hour":i=(this-n)/36e5;break;case"day":i=(this-n-r)/864e5;break;case"week":i=(this-n-r)/6048e5;break;default:i=this-n}return a?i:de(i)}function Qn(e,t){if(e.date()<t.date())return-Qn(t,e);var a,n,r=12*(t.year()-e.year())+(t.month()-e.month()),i=e.clone().add(r,"months");return t-i<0?(a=e.clone().add(r-1,"months"),n=(t-i)/(i-a)):(a=e.clone().add(r+1,"months"),n=(t-i)/(a-i)),-(r+n)||0}function er(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")}function tr(e){if(!this.isValid())return null;var t=!0!==e,a=t?this.clone().utc():this;return a.year()<0||a.year()>9999?W(a,t?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):T(Date.prototype.toISOString)?t?this.toDate().toISOString():new Date(this.valueOf()+60*this.utcOffset()*1e3).toISOString().replace("Z",W(a,"Z")):W(a,t?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")}function ar(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var e,t,a,n,r="moment",i="";return this.isLocal()||(r=0===this.utcOffset()?"moment.utc":"moment.parseZone",i="Z"),e="["+r+'("]',t=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",a="-MM-DD[T]HH:mm:ss.SSS",n=i+'[")]',this.format(e+t+a+n)}function nr(e){e||(e=this.isUtc()?i.defaultFormatUtc:i.defaultFormat);var t=W(this,e);return this.localeData().postformat(t)}function rr(e,t){return this.isValid()&&(H(e)&&e.isValid()||Ka(e).isValid())?On({to:this,from:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()}function ir(e){return this.from(Ka(),e)}function or(e,t){return this.isValid()&&(H(e)&&e.isValid()||Ka(e).isValid())?On({from:this,to:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()}function sr(e){return this.to(Ka(),e)}function lr(e){var t;return void 0===e?this._locale._abbr:(t=ya(e),null!=t&&(this._locale=t),this)}i.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",i.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var cr=A("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",(function(e){return void 0===e?this.localeData():this.locale(e)}));function ur(){return this._locale}var dr=1e3,hr=60*dr,fr=60*hr,pr=3506328*fr;function mr(e,t){return(e%t+t)%t}function vr(e,t,a){return e<100&&e>=0?new Date(e+400,t,a)-pr:new Date(e,t,a).valueOf()}function _r(e,t,a){return e<100&&e>=0?Date.UTC(e+400,t,a)-pr:Date.UTC(e,t,a)}function gr(e){var t,a;if(e=ie(e),void 0===e||"millisecond"===e||!this.isValid())return this;switch(a=this._isUTC?_r:vr,e){case"year":t=a(this.year(),0,1);break;case"quarter":t=a(this.year(),this.month()-this.month()%3,1);break;case"month":t=a(this.year(),this.month(),1);break;case"week":t=a(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":t=a(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":t=a(this.year(),this.month(),this.date());break;case"hour":t=this._d.valueOf(),t-=mr(t+(this._isUTC?0:this.utcOffset()*hr),fr);break;case"minute":t=this._d.valueOf(),t-=mr(t,hr);break;case"second":t=this._d.valueOf(),t-=mr(t,dr);break}return this._d.setTime(t),i.updateOffset(this,!0),this}function yr(e){var t,a;if(e=ie(e),void 0===e||"millisecond"===e||!this.isValid())return this;switch(a=this._isUTC?_r:vr,e){case"year":t=a(this.year()+1,0,1)-1;break;case"quarter":t=a(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":t=a(this.year(),this.month()+1,1)-1;break;case"week":t=a(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":t=a(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":t=a(this.year(),this.month(),this.date()+1)-1;break;case"hour":t=this._d.valueOf(),t+=fr-mr(t+(this._isUTC?0:this.utcOffset()*hr),fr)-1;break;case"minute":t=this._d.valueOf(),t+=hr-mr(t,hr)-1;break;case"second":t=this._d.valueOf(),t+=dr-mr(t,dr)-1;break}return this._d.setTime(t),i.updateOffset(this,!0),this}function br(){return this._d.valueOf()-6e4*(this._offset||0)}function Mr(){return Math.floor(this.valueOf()/1e3)}function zr(){return new Date(this.valueOf())}function Lr(){var e=this;return[e.year(),e.month(),e.date(),e.hour(),e.minute(),e.second(),e.millisecond()]}function wr(){var e=this;return{years:e.year(),months:e.month(),date:e.date(),hours:e.hours(),minutes:e.minutes(),seconds:e.seconds(),milliseconds:e.milliseconds()}}function Hr(){return this.isValid()?this.toISOString():null}function kr(){return y(this)}function Ar(){return m({},g(this))}function Sr(){return g(this).overflow}function Vr(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}}function Or(e,t){var a,n,r,o=this._eras||ya("en")._eras;for(a=0,n=o.length;a<n;++a){switch(typeof o[a].since){case"string":r=i(o[a].since).startOf("day"),o[a].since=r.valueOf();break}switch(typeof o[a].until){case"undefined":o[a].until=1/0;break;case"string":r=i(o[a].until).startOf("day").valueOf(),o[a].until=r.valueOf();break}}return o}function Tr(e,t,a){var n,r,i,o,s,l=this.eras();for(e=e.toUpperCase(),n=0,r=l.length;n<r;++n)if(i=l[n].name.toUpperCase(),o=l[n].abbr.toUpperCase(),s=l[n].narrow.toUpperCase(),a)switch(t){case"N":case"NN":case"NNN":if(o===e)return l[n];break;case"NNNN":if(i===e)return l[n];break;case"NNNNN":if(s===e)return l[n];break}else if([i,o,s].indexOf(e)>=0)return l[n]}function Dr(e,t){var a=e.since<=e.until?1:-1;return void 0===t?i(e.since).year():i(e.since).year()+(t-e.offset)*a}function Cr(){var e,t,a,n=this.localeData().eras();for(e=0,t=n.length;e<t;++e){if(a=this.clone().startOf("day").valueOf(),n[e].since<=a&&a<=n[e].until)return n[e].name;if(n[e].until<=a&&a<=n[e].since)return n[e].name}return""}function jr(){var e,t,a,n=this.localeData().eras();for(e=0,t=n.length;e<t;++e){if(a=this.clone().startOf("day").valueOf(),n[e].since<=a&&a<=n[e].until)return n[e].narrow;if(n[e].until<=a&&a<=n[e].since)return n[e].narrow}return""}function xr(){var e,t,a,n=this.localeData().eras();for(e=0,t=n.length;e<t;++e){if(a=this.clone().startOf("day").valueOf(),n[e].since<=a&&a<=n[e].until)return n[e].abbr;if(n[e].until<=a&&a<=n[e].since)return n[e].abbr}return""}function Br(){var e,t,a,n,r=this.localeData().eras();for(e=0,t=r.length;e<t;++e)if(a=r[e].since<=r[e].until?1:-1,n=this.clone().startOf("day").valueOf(),r[e].since<=n&&n<=r[e].until||r[e].until<=n&&n<=r[e].since)return(this.year()-i(r[e].since).year())*a+r[e].offset;return this.year()}function Yr(e){return c(this,"_erasNameRegex")||Nr.call(this),e?this._erasNameRegex:this._erasRegex}function Pr(e){return c(this,"_erasAbbrRegex")||Nr.call(this),e?this._erasAbbrRegex:this._erasRegex}function Ir(e){return c(this,"_erasNarrowRegex")||Nr.call(this),e?this._erasNarrowRegex:this._erasRegex}function Fr(e,t){return t.erasAbbrRegex(e)}function Er(e,t){return t.erasNameRegex(e)}function $r(e,t){return t.erasNarrowRegex(e)}function Rr(e,t){return t._eraYearOrdinalRegex||Oe}function Nr(){var e,t,a=[],n=[],r=[],i=[],o=this.eras();for(e=0,t=o.length;e<t;++e)n.push(Ie(o[e].name)),a.push(Ie(o[e].abbr)),r.push(Ie(o[e].narrow)),i.push(Ie(o[e].name)),i.push(Ie(o[e].abbr)),i.push(Ie(o[e].narrow));this._erasRegex=new RegExp("^("+i.join("|")+")","i"),this._erasNameRegex=new RegExp("^("+n.join("|")+")","i"),this._erasAbbrRegex=new RegExp("^("+a.join("|")+")","i"),this._erasNarrowRegex=new RegExp("^("+r.join("|")+")","i")}function Wr(e,t){$(0,[e,e.length],0,t)}function Ur(e){return Zr.call(this,e,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)}function Gr(e){return Zr.call(this,e,this.isoWeek(),this.isoWeekday(),1,4)}function qr(){return Ht(this.year(),1,4)}function Jr(){return Ht(this.isoWeekYear(),1,4)}function Kr(){var e=this.localeData()._week;return Ht(this.year(),e.dow,e.doy)}function Xr(){var e=this.localeData()._week;return Ht(this.weekYear(),e.dow,e.doy)}function Zr(e,t,a,n,r){var i;return null==e?wt(this,n,r).year:(i=Ht(e,n,r),t>i&&(t=i),Qr.call(this,e,t,a,n,r))}function Qr(e,t,a,n,r){var i=Lt(e,t,a,n,r),o=Mt(i.year,0,i.dayOfYear);return this.year(o.getUTCFullYear()),this.month(o.getUTCMonth()),this.date(o.getUTCDate()),this}function ei(e){return null==e?Math.ceil((this.month()+1)/3):this.month(3*(e-1)+this.month()%3)}$("N",0,0,"eraAbbr"),$("NN",0,0,"eraAbbr"),$("NNN",0,0,"eraAbbr"),$("NNNN",0,0,"eraName"),$("NNNNN",0,0,"eraNarrow"),$("y",["y",1],"yo","eraYear"),$("y",["yy",2],0,"eraYear"),$("y",["yyy",3],0,"eraYear"),$("y",["yyyy",4],0,"eraYear"),Be("N",Fr),Be("NN",Fr),Be("NNN",Fr),Be("NNNN",Er),Be("NNNNN",$r),Ee(["N","NN","NNN","NNNN","NNNNN"],(function(e,t,a,n){var r=a._locale.erasParse(e,n,a._strict);r?g(a).era=r:g(a).invalidEra=e})),Be("y",Oe),Be("yy",Oe),Be("yyy",Oe),Be("yyyy",Oe),Be("yo",Rr),Ee(["y","yy","yyy","yyyy"],We),Ee(["yo"],(function(e,t,a,n){var r;a._locale._eraYearOrdinalRegex&&(r=e.match(a._locale._eraYearOrdinalRegex)),a._locale.eraYearOrdinalParse?t[We]=a._locale.eraYearOrdinalParse(e,r):t[We]=parseInt(e,10)})),$(0,["gg",2],0,(function(){return this.weekYear()%100})),$(0,["GG",2],0,(function(){return this.isoWeekYear()%100})),Wr("gggg","weekYear"),Wr("ggggg","weekYear"),Wr("GGGG","isoWeekYear"),Wr("GGGGG","isoWeekYear"),re("weekYear","gg"),re("isoWeekYear","GG"),le("weekYear",1),le("isoWeekYear",1),Be("G",Te),Be("g",Te),Be("GG",we,be),Be("gg",we,be),Be("GGGG",Se,ze),Be("gggg",Se,ze),Be("GGGGG",Ve,Le),Be("ggggg",Ve,Le),$e(["gggg","ggggg","GGGG","GGGGG"],(function(e,t,a,n){t[n.substr(0,2)]=he(e)})),$e(["gg","GG"],(function(e,t,a,n){t[n]=i.parseTwoDigitYear(e)})),$("Q",0,"Qo","quarter"),re("quarter","Q"),le("quarter",7),Be("Q",ye),Ee("Q",(function(e,t){t[Ue]=3*(he(e)-1)})),$("D",["DD",2],"Do","date"),re("date","D"),le("date",9),Be("D",we),Be("DD",we,be),Be("Do",(function(e,t){return e?t._dayOfMonthOrdinalParse||t._ordinalParse:t._dayOfMonthOrdinalParseLenient})),Ee(["D","DD"],Ge),Ee("Do",(function(e,t){t[Ge]=he(e.match(we)[0])}));var ti=fe("Date",!0);function ai(e){var t=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==e?t:this.add(e-t,"d")}$("DDD",["DDDD",3],"DDDo","dayOfYear"),re("dayOfYear","DDD"),le("dayOfYear",4),Be("DDD",Ae),Be("DDDD",Me),Ee(["DDD","DDDD"],(function(e,t,a){a._dayOfYear=he(e)})),$("m",["mm",2],0,"minute"),re("minute","m"),le("minute",14),Be("m",we),Be("mm",we,be),Ee(["m","mm"],Je);var ni=fe("Minutes",!1);$("s",["ss",2],0,"second"),re("second","s"),le("second",15),Be("s",we),Be("ss",we,be),Ee(["s","ss"],Ke);var ri,ii,oi=fe("Seconds",!1);for($("S",0,0,(function(){return~~(this.millisecond()/100)})),$(0,["SS",2],0,(function(){return~~(this.millisecond()/10)})),$(0,["SSS",3],0,"millisecond"),$(0,["SSSS",4],0,(function(){return 10*this.millisecond()})),$(0,["SSSSS",5],0,(function(){return 100*this.millisecond()})),$(0,["SSSSSS",6],0,(function(){return 1e3*this.millisecond()})),$(0,["SSSSSSS",7],0,(function(){return 1e4*this.millisecond()})),$(0,["SSSSSSSS",8],0,(function(){return 1e5*this.millisecond()})),$(0,["SSSSSSSSS",9],0,(function(){return 1e6*this.millisecond()})),re("millisecond","ms"),le("millisecond",16),Be("S",Ae,ye),Be("SS",Ae,be),Be("SSS",Ae,Me),ri="SSSS";ri.length<=9;ri+="S")Be(ri,Oe);function si(e,t){t[Xe]=he(1e3*("0."+e))}for(ri="S";ri.length<=9;ri+="S")Ee(ri,si);function li(){return this._isUTC?"UTC":""}function ci(){return this._isUTC?"Coordinated Universal Time":""}ii=fe("Milliseconds",!1),$("z",0,0,"zoneAbbr"),$("zz",0,0,"zoneName");var ui=w.prototype;function di(e){return Ka(1e3*e)}function hi(){return Ka.apply(null,arguments).parseZone()}function fi(e){return e}ui.add=Bn,ui.calendar=Nn,ui.clone=Wn,ui.diff=Zn,ui.endOf=yr,ui.format=nr,ui.from=rr,ui.fromNow=ir,ui.to=or,ui.toNow=sr,ui.get=ve,ui.invalidAt=Sr,ui.isAfter=Un,ui.isBefore=Gn,ui.isBetween=qn,ui.isSame=Jn,ui.isSameOrAfter=Kn,ui.isSameOrBefore=Xn,ui.isValid=kr,ui.lang=cr,ui.locale=lr,ui.localeData=ur,ui.max=Za,ui.min=Xa,ui.parsingFlags=Ar,ui.set=_e,ui.startOf=gr,ui.subtract=Yn,ui.toArray=Lr,ui.toObject=wr,ui.toDate=zr,ui.toISOString=tr,ui.inspect=ar,"undefined"!==typeof Symbol&&null!=Symbol.for&&(ui[Symbol.for("nodejs.util.inspect.custom")]=function(){return"Moment<"+this.format()+">"}),ui.toJSON=Hr,ui.toString=er,ui.unix=Mr,ui.valueOf=br,ui.creationData=Vr,ui.eraName=Cr,ui.eraNarrow=jr,ui.eraAbbr=xr,ui.eraYear=Br,ui.year=gt,ui.isLeapYear=yt,ui.weekYear=Ur,ui.isoWeekYear=Gr,ui.quarter=ui.quarters=ei,ui.month=ht,ui.daysInMonth=ft,ui.week=ui.weeks=Ot,ui.isoWeek=ui.isoWeeks=Tt,ui.weeksInYear=Kr,ui.weeksInWeekYear=Xr,ui.isoWeeksInYear=qr,ui.isoWeeksInISOWeekYear=Jr,ui.date=ti,ui.day=ui.days=Ut,ui.weekday=Gt,ui.isoWeekday=qt,ui.dayOfYear=ai,ui.hour=ui.hours=ia,ui.minute=ui.minutes=ni,ui.second=ui.seconds=oi,ui.millisecond=ui.milliseconds=ii,ui.utcOffset=_n,ui.utc=yn,ui.local=bn,ui.parseZone=Mn,ui.hasAlignedHourOffset=zn,ui.isDST=Ln,ui.isLocal=Hn,ui.isUtcOffset=kn,ui.isUtc=An,ui.isUTC=An,ui.zoneAbbr=li,ui.zoneName=ci,ui.dates=A("dates accessor is deprecated. Use date instead.",ti),ui.months=A("months accessor is deprecated. Use month instead",ht),ui.years=A("years accessor is deprecated. Use year instead",gt),ui.zone=A("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",gn),ui.isDSTShifted=A("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",wn);var pi=j.prototype;function mi(e,t,a,n){var r=ya(),i=v().set(n,t);return r[a](i,e)}function vi(e,t,a){if(h(e)&&(t=e,e=void 0),e=e||"",null!=t)return mi(e,t,a,"month");var n,r=[];for(n=0;n<12;n++)r[n]=mi(e,n,a,"month");return r}function _i(e,t,a,n){"boolean"===typeof e?(h(t)&&(a=t,t=void 0),t=t||""):(t=e,a=t,e=!1,h(t)&&(a=t,t=void 0),t=t||"");var r,i=ya(),o=e?i._week.dow:0,s=[];if(null!=a)return mi(t,(a+o)%7,n,"day");for(r=0;r<7;r++)s[r]=mi(t,(r+o)%7,n,"day");return s}function gi(e,t){return vi(e,t,"months")}function yi(e,t){return vi(e,t,"monthsShort")}function bi(e,t,a){return _i(e,t,a,"weekdays")}function Mi(e,t,a){return _i(e,t,a,"weekdaysShort")}function zi(e,t,a){return _i(e,t,a,"weekdaysMin")}pi.calendar=B,pi.longDateFormat=q,pi.invalidDate=K,pi.ordinal=Q,pi.preparse=fi,pi.postformat=fi,pi.relativeTime=te,pi.pastFuture=ae,pi.set=D,pi.eras=Or,pi.erasParse=Tr,pi.erasConvertYear=Dr,pi.erasAbbrRegex=Pr,pi.erasNameRegex=Yr,pi.erasNarrowRegex=Ir,pi.months=st,pi.monthsShort=lt,pi.monthsParse=ut,pi.monthsRegex=mt,pi.monthsShortRegex=pt,pi.week=kt,pi.firstDayOfYear=Vt,pi.firstDayOfWeek=St,pi.weekdays=Et,pi.weekdaysMin=Rt,pi.weekdaysShort=$t,pi.weekdaysParse=Wt,pi.weekdaysRegex=Jt,pi.weekdaysShortRegex=Kt,pi.weekdaysMinRegex=Xt,pi.isPM=na,pi.meridiem=oa,va("en",{eras:[{since:"0001-01-01",until:1/0,offset:1,name:"Anno Domini",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"Before Christ",narrow:"BC",abbr:"BC"}],dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10,a=1===he(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a}}),i.lang=A("moment.lang is deprecated. Use moment.locale instead.",va),i.langData=A("moment.langData is deprecated. Use moment.localeData instead.",ya);var Li=Math.abs;function wi(){var e=this._data;return this._milliseconds=Li(this._milliseconds),this._days=Li(this._days),this._months=Li(this._months),e.milliseconds=Li(e.milliseconds),e.seconds=Li(e.seconds),e.minutes=Li(e.minutes),e.hours=Li(e.hours),e.months=Li(e.months),e.years=Li(e.years),this}function Hi(e,t,a,n){var r=On(t,a);return e._milliseconds+=n*r._milliseconds,e._days+=n*r._days,e._months+=n*r._months,e._bubble()}function ki(e,t){return Hi(this,e,t,1)}function Ai(e,t){return Hi(this,e,t,-1)}function Si(e){return e<0?Math.floor(e):Math.ceil(e)}function Vi(){var e,t,a,n,r,i=this._milliseconds,o=this._days,s=this._months,l=this._data;return i>=0&&o>=0&&s>=0||i<=0&&o<=0&&s<=0||(i+=864e5*Si(Ti(s)+o),o=0,s=0),l.milliseconds=i%1e3,e=de(i/1e3),l.seconds=e%60,t=de(e/60),l.minutes=t%60,a=de(t/60),l.hours=a%24,o+=de(a/24),r=de(Oi(o)),s+=r,o-=Si(Ti(r)),n=de(s/12),s%=12,l.days=o,l.months=s,l.years=n,this}function Oi(e){return 4800*e/146097}function Ti(e){return 146097*e/4800}function Di(e){if(!this.isValid())return NaN;var t,a,n=this._milliseconds;if(e=ie(e),"month"===e||"quarter"===e||"year"===e)switch(t=this._days+n/864e5,a=this._months+Oi(t),e){case"month":return a;case"quarter":return a/3;case"year":return a/12}else switch(t=this._days+Math.round(Ti(this._months)),e){case"week":return t/7+n/6048e5;case"day":return t+n/864e5;case"hour":return 24*t+n/36e5;case"minute":return 1440*t+n/6e4;case"second":return 86400*t+n/1e3;case"millisecond":return Math.floor(864e5*t)+n;default:throw new Error("Unknown unit "+e)}}function Ci(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*he(this._months/12):NaN}function ji(e){return function(){return this.as(e)}}var xi=ji("ms"),Bi=ji("s"),Yi=ji("m"),Pi=ji("h"),Ii=ji("d"),Fi=ji("w"),Ei=ji("M"),$i=ji("Q"),Ri=ji("y");function Ni(){return On(this)}function Wi(e){return e=ie(e),this.isValid()?this[e+"s"]():NaN}function Ui(e){return function(){return this.isValid()?this._data[e]:NaN}}var Gi=Ui("milliseconds"),qi=Ui("seconds"),Ji=Ui("minutes"),Ki=Ui("hours"),Xi=Ui("days"),Zi=Ui("months"),Qi=Ui("years");function eo(){return de(this.days()/7)}var to=Math.round,ao={ss:44,s:45,m:45,h:22,d:26,w:null,M:11};function no(e,t,a,n,r){return r.relativeTime(t||1,!!a,e,n)}function ro(e,t,a,n){var r=On(e).abs(),i=to(r.as("s")),o=to(r.as("m")),s=to(r.as("h")),l=to(r.as("d")),c=to(r.as("M")),u=to(r.as("w")),d=to(r.as("y")),h=i<=a.ss&&["s",i]||i<a.s&&["ss",i]||o<=1&&["m"]||o<a.m&&["mm",o]||s<=1&&["h"]||s<a.h&&["hh",s]||l<=1&&["d"]||l<a.d&&["dd",l];return null!=a.w&&(h=h||u<=1&&["w"]||u<a.w&&["ww",u]),h=h||c<=1&&["M"]||c<a.M&&["MM",c]||d<=1&&["y"]||["yy",d],h[2]=t,h[3]=+e>0,h[4]=n,no.apply(null,h)}function io(e){return void 0===e?to:"function"===typeof e&&(to=e,!0)}function oo(e,t){return void 0!==ao[e]&&(void 0===t?ao[e]:(ao[e]=t,"s"===e&&(ao.ss=t-1),!0))}function so(e,t){if(!this.isValid())return this.localeData().invalidDate();var a,n,r=!1,i=ao;return"object"===typeof e&&(t=e,e=!1),"boolean"===typeof e&&(r=e),"object"===typeof t&&(i=Object.assign({},ao,t),null!=t.s&&null==t.ss&&(i.ss=t.s-1)),a=this.localeData(),n=ro(this,!r,i,a),r&&(n=a.pastFuture(+this,n)),a.postformat(n)}var lo=Math.abs;function co(e){return(e>0)-(e<0)||+e}function uo(){if(!this.isValid())return this.localeData().invalidDate();var e,t,a,n,r,i,o,s,l=lo(this._milliseconds)/1e3,c=lo(this._days),u=lo(this._months),d=this.asSeconds();return d?(e=de(l/60),t=de(e/60),l%=60,e%=60,a=de(u/12),u%=12,n=l?l.toFixed(3).replace(/\.?0+$/,""):"",r=d<0?"-":"",i=co(this._months)!==co(d)?"-":"",o=co(this._days)!==co(d)?"-":"",s=co(this._milliseconds)!==co(d)?"-":"",r+"P"+(a?i+a+"Y":"")+(u?i+u+"M":"")+(c?o+c+"D":"")+(t||e||l?"T":"")+(t?s+t+"H":"")+(e?s+e+"M":"")+(l?s+n+"S":"")):"P0D"}var ho=ln.prototype;return ho.isValid=on,ho.abs=wi,ho.add=ki,ho.subtract=Ai,ho.as=Di,ho.asMilliseconds=xi,ho.asSeconds=Bi,ho.asMinutes=Yi,ho.asHours=Pi,ho.asDays=Ii,ho.asWeeks=Fi,ho.asMonths=Ei,ho.asQuarters=$i,ho.asYears=Ri,ho.valueOf=Ci,ho._bubble=Vi,ho.clone=Ni,ho.get=Wi,ho.milliseconds=Gi,ho.seconds=qi,ho.minutes=Ji,ho.hours=Ki,ho.days=Xi,ho.weeks=eo,ho.months=Zi,ho.years=Qi,ho.humanize=so,ho.toISOString=uo,ho.toString=uo,ho.toJSON=uo,ho.locale=lr,ho.localeData=ur,ho.toIsoString=A("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",uo),ho.lang=cr,$("X",0,0,"unix"),$("x",0,0,"valueOf"),Be("x",Te),Be("X",je),Ee("X",(function(e,t,a){a._d=new Date(1e3*parseFloat(e))})),Ee("x",(function(e,t,a){a._d=new Date(he(e))})),
//! moment.js
i.version="2.29.4",o(Ka),i.fn=ui,i.min=en,i.max=tn,i.now=an,i.utc=v,i.unix=di,i.months=gi,i.isDate=f,i.locale=va,i.invalid=b,i.duration=On,i.isMoment=H,i.weekdays=bi,i.parseZone=hi,i.localeData=ya,i.isDuration=cn,i.monthsShort=yi,i.weekdaysMin=zi,i.defineLocale=_a,i.updateLocale=ga,i.locales=ba,i.weekdaysShort=Mi,i.normalizeUnits=ie,i.relativeTimeRounding=io,i.relativeTimeThreshold=oo,i.calendarFormat=Rn,i.prototype=ui,i.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},i}))}).call(this,a("62e4")(e))},c345:function(e,t,a){"use strict";var n=a("c532"),r=["age","authorization","content-length","content-type","etag","expires","from","host","if-modified-since","if-unmodified-since","last-modified","location","max-forwards","proxy-authorization","referer","retry-after","user-agent"];e.exports=function(e){var t,a,i,o={};return e?(n.forEach(e.split("\n"),(function(e){if(i=e.indexOf(":"),t=n.trim(e.substr(0,i)).toLowerCase(),a=n.trim(e.substr(i+1)),t){if(o[t]&&r.indexOf(t)>=0)return;o[t]="set-cookie"===t?(o[t]?o[t]:[]).concat([a]):o[t]?o[t]+", "+a:a}})),o):o}},c401:function(e,t,a){"use strict";var n=a("c532"),r=a("2444");e.exports=function(e,t,a){var i=this||r;return n.forEach(a,(function(a){e=a.call(i,e,t)})),e}},c430:function(e,t){e.exports=!1},c532:function(e,t,a){"use strict";var n=a("1d2b"),r=Object.prototype.toString;function i(e){return"[object Array]"===r.call(e)}function o(e){return"undefined"===typeof e}function s(e){return null!==e&&!o(e)&&null!==e.constructor&&!o(e.constructor)&&"function"===typeof e.constructor.isBuffer&&e.constructor.isBuffer(e)}function l(e){return"[object ArrayBuffer]"===r.call(e)}function c(e){return"undefined"!==typeof FormData&&e instanceof FormData}function u(e){var t;return t="undefined"!==typeof ArrayBuffer&&ArrayBuffer.isView?ArrayBuffer.isView(e):e&&e.buffer&&e.buffer instanceof ArrayBuffer,t}function d(e){return"string"===typeof e}function h(e){return"number"===typeof e}function f(e){return null!==e&&"object"===typeof e}function p(e){if("[object Object]"!==r.call(e))return!1;var t=Object.getPrototypeOf(e);return null===t||t===Object.prototype}function m(e){return"[object Date]"===r.call(e)}function v(e){return"[object File]"===r.call(e)}function _(e){return"[object Blob]"===r.call(e)}function g(e){return"[object Function]"===r.call(e)}function y(e){return f(e)&&g(e.pipe)}function b(e){return"undefined"!==typeof URLSearchParams&&e instanceof URLSearchParams}function M(e){return e.trim?e.trim():e.replace(/^\s+|\s+$/g,"")}function z(){return("undefined"===typeof navigator||"ReactNative"!==navigator.product&&"NativeScript"!==navigator.product&&"NS"!==navigator.product)&&("undefined"!==typeof window&&"undefined"!==typeof document)}function L(e,t){if(null!==e&&"undefined"!==typeof e)if("object"!==typeof e&&(e=[e]),i(e))for(var a=0,n=e.length;a<n;a++)t.call(null,e[a],a,e);else for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.call(null,e[r],r,e)}function w(){var e={};function t(t,a){p(e[a])&&p(t)?e[a]=w(e[a],t):p(t)?e[a]=w({},t):i(t)?e[a]=t.slice():e[a]=t}for(var a=0,n=arguments.length;a<n;a++)L(arguments[a],t);return e}function H(e,t,a){return L(t,(function(t,r){e[r]=a&&"function"===typeof t?n(t,a):t})),e}function k(e){return 65279===e.charCodeAt(0)&&(e=e.slice(1)),e}e.exports={isArray:i,isArrayBuffer:l,isBuffer:s,isFormData:c,isArrayBufferView:u,isString:d,isNumber:h,isObject:f,isPlainObject:p,isUndefined:o,isDate:m,isFile:v,isBlob:_,isFunction:g,isStream:y,isURLSearchParams:b,isStandardBrowserEnv:z,forEach:L,merge:w,extend:H,trim:M,stripBOM:k}},c65b:function(e,t,a){var n=a("40d5"),r=Function.prototype.call;e.exports=n?r.bind(r):function(){return r.apply(r,arguments)}},c6b6:function(e,t,a){var n=a("7d7e"),r=n({}.toString),i=n("".slice);e.exports=function(e){return i(r(e),8,-1)}},c6cd:function(e,t,a){var n=a("da84"),r=a("6374"),i="__core-js_shared__",o=n[i]||r(i,{});e.exports=o},c7aa:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("he",{months:"ינואר_פברואר_מרץ_אפריל_מאי_יוני_יולי_אוגוסט_ספטמבר_אוקטובר_נובמבר_דצמבר".split("_"),monthsShort:"ינו׳_פבר׳_מרץ_אפר׳_מאי_יוני_יולי_אוג׳_ספט׳_אוק׳_נוב׳_דצמ׳".split("_"),weekdays:"ראשון_שני_שלישי_רביעי_חמישי_שישי_שבת".split("_"),weekdaysShort:"א׳_ב׳_ג׳_ד׳_ה׳_ו׳_ש׳".split("_"),weekdaysMin:"א_ב_ג_ד_ה_ו_ש".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [ב]MMMM YYYY",LLL:"D [ב]MMMM YYYY HH:mm",LLLL:"dddd, D [ב]MMMM YYYY HH:mm",l:"D/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[היום ב־]LT",nextDay:"[מחר ב־]LT",nextWeek:"dddd [בשעה] LT",lastDay:"[אתמול ב־]LT",lastWeek:"[ביום] dddd [האחרון בשעה] LT",sameElse:"L"},relativeTime:{future:"בעוד %s",past:"לפני %s",s:"מספר שניות",ss:"%d שניות",m:"דקה",mm:"%d דקות",h:"שעה",hh:function(e){return 2===e?"שעתיים":e+" שעות"},d:"יום",dd:function(e){return 2===e?"יומיים":e+" ימים"},M:"חודש",MM:function(e){return 2===e?"חודשיים":e+" חודשים"},y:"שנה",yy:function(e){return 2===e?"שנתיים":e%10===0&&10!==e?e+" שנה":e+" שנים"}},meridiemParse:/אחה"צ|לפנה"צ|אחרי הצהריים|לפני הצהריים|לפנות בוקר|בבוקר|בערב/i,isPM:function(e){return/^(אחה"צ|אחרי הצהריים|בערב)$/.test(e)},meridiem:function(e,t,a){return e<5?"לפנות בוקר":e<10?"בבוקר":e<12?a?'לפנה"צ':"לפני הצהריים":e<18?a?'אחה"צ':"אחרי הצהריים":"בערב"}});return t}))},c8af:function(e,t,a){"use strict";var n=a("c532");e.exports=function(e,t){n.forEach(e,(function(a,n){n!==t&&n.toUpperCase()===t.toUpperCase()&&(e[t]=a,delete e[n])}))}},c8ba:function(e,t){var a;a=function(){return this}();try{a=a||new Function("return this")()}catch(n){"object"===typeof window&&(a=window)}e.exports=a},c8f3:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("sq",{months:"Janar_Shkurt_Mars_Prill_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_Nëntor_Dhjetor".split("_"),monthsShort:"Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_Nën_Dhj".split("_"),weekdays:"E Diel_E Hënë_E Martë_E Mërkurë_E Enjte_E Premte_E Shtunë".split("_"),weekdaysShort:"Die_Hën_Mar_Mër_Enj_Pre_Sht".split("_"),weekdaysMin:"D_H_Ma_Më_E_P_Sh".split("_"),weekdaysParseExact:!0,meridiemParse:/PD|MD/,isPM:function(e){return"M"===e.charAt(0)},meridiem:function(e,t,a){return e<12?"PD":"MD"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Sot në] LT",nextDay:"[Nesër në] LT",nextWeek:"dddd [në] LT",lastDay:"[Dje në] LT",lastWeek:"dddd [e kaluar në] LT",sameElse:"L"},relativeTime:{future:"në %s",past:"%s më parë",s:"disa sekonda",ss:"%d sekonda",m:"një minutë",mm:"%d minuta",h:"një orë",hh:"%d orë",d:"një ditë",dd:"%d ditë",M:"një muaj",MM:"%d muaj",y:"një vit",yy:"%d vite"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},ca84:function(e,t,a){var n=a("e330"),r=a("1a2d"),i=a("fc6a"),o=a("4d64").indexOf,s=a("d012"),l=n([].push);e.exports=function(e,t){var a,n=i(e),c=0,u=[];for(a in n)!r(s,a)&&r(n,a)&&l(u,a);while(t.length>c)r(n,a=t[c++])&&(~o(u,a)||l(u,a));return u}},cb2d:function(e,t,a){var n=a("1626"),r=a("9bf2"),i=a("13d2"),o=a("6374");e.exports=function(e,t,a,s){s||(s={});var l=s.enumerable,c=void 0!==s.name?s.name:t;if(n(a)&&i(a,c,s),s.global)l?e[t]=a:o(t,a);else{try{s.unsafe?e[t]&&(l=!0):delete e[t]}catch(u){}l?e[t]=a:r.f(e,t,{value:a,enumerable:!1,configurable:!s.nonConfigurable,writable:!s.nonWritable})}return e}},cc12:function(e,t,a){var n=a("da84"),r=a("861d"),i=n.document,o=r(i)&&r(i.createElement);e.exports=function(e){return o?i.createElement(e):{}}},cdce:function(e,t,a){var n=a("da84"),r=a("1626"),i=n.WeakMap;e.exports=r(i)&&/native code/.test(String(i))},cee4:function(e,t,a){"use strict";var n=a("c532"),r=a("1d2b"),i=a("0a06"),o=a("4a7b"),s=a("2444");function l(e){var t=new i(e),a=r(i.prototype.request,t);return n.extend(a,i.prototype,t),n.extend(a,t),a}var c=l(s);c.Axios=i,c.create=function(e){return l(o(c.defaults,e))},c.Cancel=a("7a77"),c.CancelToken=a("8df4b"),c.isCancel=a("2e67"),c.all=function(e){return Promise.all(e)},c.spread=a("0df6"),c.isAxiosError=a("5f02"),e.exports=c,e.exports.default=c},cf1e:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["sekunda","sekunde","sekundi"],m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],d:["jedan dan","jednog dana"],dd:["dan","dana","dana"],M:["jedan mesec","jednog meseca"],MM:["mesec","meseca","meseci"],y:["jednu godinu","jedne godine"],yy:["godinu","godine","godina"]},correctGrammaticalCase:function(e,t){return e%10>=1&&e%10<=4&&(e%100<10||e%100>=20)?e%10===1?t[0]:t[1]:t[2]},translate:function(e,a,n,r){var i,o=t.words[n];return 1===n.length?"y"===n&&a?"jedna godina":r||a?o[0]:o[1]:(i=t.correctGrammaticalCase(e,o),"yy"===n&&a&&"godinu"===i?e+" godina":e+" "+i)}},a=e.defineLocale("sr",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljak_utorak_sreda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sre._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedelju] [u] LT";case 3:return"[u] [sredu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){var e=["[prošle] [nedelje] [u] LT","[prošlog] [ponedeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"];return e[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"pre %s",s:"nekoliko sekundi",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:t.translate,dd:t.translate,M:t.translate,MM:t.translate,y:t.translate,yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return a}))},cf51:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tzl",{months:"Januar_Fevraglh_Març_Avrïu_Mai_Gün_Julia_Guscht_Setemvar_Listopäts_Noemvar_Zecemvar".split("_"),monthsShort:"Jan_Fev_Mar_Avr_Mai_Gün_Jul_Gus_Set_Lis_Noe_Zec".split("_"),weekdays:"Súladi_Lúneçi_Maitzi_Márcuri_Xhúadi_Viénerçi_Sáturi".split("_"),weekdaysShort:"Súl_Lún_Mai_Már_Xhú_Vié_Sát".split("_"),weekdaysMin:"Sú_Lú_Ma_Má_Xh_Vi_Sá".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"D. MMMM [dallas] YYYY",LLL:"D. MMMM [dallas] YYYY HH.mm",LLLL:"dddd, [li] D. MMMM [dallas] YYYY HH.mm"},meridiemParse:/d\'o|d\'a/i,isPM:function(e){return"d'o"===e.toLowerCase()},meridiem:function(e,t,a){return e>11?a?"d'o":"D'O":a?"d'a":"D'A"},calendar:{sameDay:"[oxhi à] LT",nextDay:"[demà à] LT",nextWeek:"dddd [à] LT",lastDay:"[ieiri à] LT",lastWeek:"[sür el] dddd [lasteu à] LT",sameElse:"L"},relativeTime:{future:"osprei %s",past:"ja%s",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});function a(e,t,a,n){var r={s:["viensas secunds","'iensas secunds"],ss:[e+" secunds",e+" secunds"],m:["'n míut","'iens míut"],mm:[e+" míuts",e+" míuts"],h:["'n þora","'iensa þora"],hh:[e+" þoras",e+" þoras"],d:["'n ziua","'iensa ziua"],dd:[e+" ziuas",e+" ziuas"],M:["'n mes","'iens mes"],MM:[e+" mesen",e+" mesen"],y:["'n ar","'iens ar"],yy:[e+" ars",e+" ars"]};return n||t?r[a][0]:r[a][1]}return t}))},cf75:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="pagh_wa’_cha’_wej_loS_vagh_jav_Soch_chorgh_Hut".split("_");function a(e){var t=e;return t=-1!==e.indexOf("jaj")?t.slice(0,-3)+"leS":-1!==e.indexOf("jar")?t.slice(0,-3)+"waQ":-1!==e.indexOf("DIS")?t.slice(0,-3)+"nem":t+" pIq",t}function n(e){var t=e;return t=-1!==e.indexOf("jaj")?t.slice(0,-3)+"Hu’":-1!==e.indexOf("jar")?t.slice(0,-3)+"wen":-1!==e.indexOf("DIS")?t.slice(0,-3)+"ben":t+" ret",t}function r(e,t,a,n){var r=i(e);switch(a){case"ss":return r+" lup";case"mm":return r+" tup";case"hh":return r+" rep";case"dd":return r+" jaj";case"MM":return r+" jar";case"yy":return r+" DIS"}}function i(e){var a=Math.floor(e%1e3/100),n=Math.floor(e%100/10),r=e%10,i="";return a>0&&(i+=t[a]+"vatlh"),n>0&&(i+=(""!==i?" ":"")+t[n]+"maH"),r>0&&(i+=(""!==i?" ":"")+t[r]),""===i?"pagh":i}var o=e.defineLocale("tlh",{months:"tera’ jar wa’_tera’ jar cha’_tera’ jar wej_tera’ jar loS_tera’ jar vagh_tera’ jar jav_tera’ jar Soch_tera’ jar chorgh_tera’ jar Hut_tera’ jar wa’maH_tera’ jar wa’maH wa’_tera’ jar wa’maH cha’".split("_"),monthsShort:"jar wa’_jar cha’_jar wej_jar loS_jar vagh_jar jav_jar Soch_jar chorgh_jar Hut_jar wa’maH_jar wa’maH wa’_jar wa’maH cha’".split("_"),monthsParseExact:!0,weekdays:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysShort:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysMin:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[DaHjaj] LT",nextDay:"[wa’leS] LT",nextWeek:"LLL",lastDay:"[wa’Hu’] LT",lastWeek:"LLL",sameElse:"L"},relativeTime:{future:a,past:n,s:"puS lup",ss:r,m:"wa’ tup",mm:r,h:"wa’ rep",hh:r,d:"wa’ jaj",dd:r,M:"wa’ jar",MM:r,y:"wa’ DIS",yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return o}))},d012:function(e,t){e.exports={}},d039:function(e,t){e.exports=function(e){try{return!!e()}catch(t){return!0}}},d066:function(e,t,a){var n=a("da84"),r=a("1626"),i=function(e){return r(e)?e:void 0};e.exports=function(e,t){return arguments.length<2?i(n[e]):n[e]&&n[e][t]}},d099:function(e,t,a){(function(t,a){e.exports=a()})("undefined"!==typeof self&&self,(function(){return function(e){var t={};function a(n){if(t[n])return t[n].exports;var r=t[n]={i:n,l:!1,exports:{}};return e[n].call(r.exports,r,r.exports,a),r.l=!0,r.exports}return a.m=e,a.c=t,a.d=function(e,t,n){a.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:n})},a.r=function(e){"undefined"!==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},a.t=function(e,t){if(1&t&&(e=a(e)),8&t)return e;if(4&t&&"object"===typeof e&&e&&e.__esModule)return e;var n=Object.create(null);if(a.r(n),Object.defineProperty(n,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var r in e)a.d(n,r,function(t){return e[t]}.bind(null,r));return n},a.n=function(e){var t=e&&e.__esModule?function(){return e["default"]}:function(){return e};return a.d(t,"a",t),t},a.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},a.p="",a(a.s="fae3")}({"0a06":function(e,t,a){"use strict";var n=a("c532"),r=a("30b5"),i=a("f6b4"),o=a("5270"),s=a("4a7b"),l=a("848b"),c=l.validators;function u(e){this.defaults=e,this.interceptors={request:new i,response:new i}}u.prototype.request=function(e){"string"===typeof e?(e=arguments[1]||{},e.url=arguments[0]):e=e||{},e=s(this.defaults,e),e.method?e.method=e.method.toLowerCase():this.defaults.method?e.method=this.defaults.method.toLowerCase():e.method="get";var t=e.transitional;void 0!==t&&l.assertOptions(t,{silentJSONParsing:c.transitional(c.boolean,"1.0.0"),forcedJSONParsing:c.transitional(c.boolean,"1.0.0"),clarifyTimeoutError:c.transitional(c.boolean,"1.0.0")},!1);var a=[],n=!0;this.interceptors.request.forEach((function(t){"function"===typeof t.runWhen&&!1===t.runWhen(e)||(n=n&&t.synchronous,a.unshift(t.fulfilled,t.rejected))}));var r,i=[];if(this.interceptors.response.forEach((function(e){i.push(e.fulfilled,e.rejected)})),!n){var u=[o,void 0];Array.prototype.unshift.apply(u,a),u=u.concat(i),r=Promise.resolve(e);while(u.length)r=r.then(u.shift(),u.shift());return r}var d=e;while(a.length){var h=a.shift(),f=a.shift();try{d=h(d)}catch(p){f(p);break}}try{r=o(d)}catch(p){return Promise.reject(p)}while(i.length)r=r.then(i.shift(),i.shift());return r},u.prototype.getUri=function(e){return e=s(this.defaults,e),r(e.url,e.params,e.paramsSerializer).replace(/^\?/,"")},n.forEach(["delete","get","head","options"],(function(e){u.prototype[e]=function(t,a){return this.request(s(a||{},{method:e,url:t,data:(a||{}).data}))}})),n.forEach(["post","put","patch"],(function(e){u.prototype[e]=function(t,a,n){return this.request(s(n||{},{method:e,url:t,data:a}))}})),e.exports=u},"0df6":function(e,t,a){"use strict";e.exports=function(e){return function(t){return e.apply(null,t)}}},"1b62":function(e,t,a){"use strict";a.r(t);var n=a("ea46");a.d(t,"getDataMixin",(function(){return n["a"]}))},"1d2b":function(e,t,a){"use strict";e.exports=function(e,t){return function(){for(var a=new Array(arguments.length),n=0;n<a.length;n++)a[n]=arguments[n];return e.apply(t,a)}}},2444:function(e,t,a){"use strict";(function(t){var n=a("c532"),r=a("c8af"),i=a("387f"),o={"Content-Type":"application/x-www-form-urlencoded"};function s(e,t){!n.isUndefined(e)&&n.isUndefined(e["Content-Type"])&&(e["Content-Type"]=t)}function l(){var e;return("undefined"!==typeof XMLHttpRequest||"undefined"!==typeof t&&"[object process]"===Object.prototype.toString.call(t))&&(e=a("b50d")),e}function c(e,t,a){if(n.isString(e))try{return(t||JSON.parse)(e),n.trim(e)}catch(r){if("SyntaxError"!==r.name)throw r}return(a||JSON.stringify)(e)}var u={transitional:{silentJSONParsing:!0,forcedJSONParsing:!0,clarifyTimeoutError:!1},adapter:l(),transformRequest:[function(e,t){return r(t,"Accept"),r(t,"Content-Type"),n.isFormData(e)||n.isArrayBuffer(e)||n.isBuffer(e)||n.isStream(e)||n.isFile(e)||n.isBlob(e)?e:n.isArrayBufferView(e)?e.buffer:n.isURLSearchParams(e)?(s(t,"application/x-www-form-urlencoded;charset=utf-8"),e.toString()):n.isObject(e)||t&&"application/json"===t["Content-Type"]?(s(t,"application/json"),c(e)):e}],transformResponse:[function(e){var t=this.transitional,a=t&&t.silentJSONParsing,r=t&&t.forcedJSONParsing,o=!a&&"json"===this.responseType;if(o||r&&n.isString(e)&&e.length)try{return JSON.parse(e)}catch(s){if(o){if("SyntaxError"===s.name)throw i(s,this,"E_JSON_PARSE");throw s}}return e}],timeout:0,xsrfCookieName:"XSRF-TOKEN",xsrfHeaderName:"X-XSRF-TOKEN",maxContentLength:-1,maxBodyLength:-1,validateStatus:function(e){return e>=200&&e<300},headers:{common:{Accept:"application/json, text/plain, */*"}}};n.forEach(["delete","get","head"],(function(e){u.headers[e]={}})),n.forEach(["post","put","patch"],(function(e){u.headers[e]=n.merge(o)})),e.exports=u}).call(this,a("4362"))},"2af9":function(e,t,a){"use strict";a.r(t),a.d(t,"HelloWorld",(function(){return z})),a.d(t,"TargetList",(function(){return u})),a.d(t,"TargetTable",(function(){return v}));var n=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",{attrs:{id:"target-list"}},[a("ttk-target-table",{attrs:{selectable:e.selectable,targets:e.targets}})],1)},r=[],i=a("1b62"),o={name:"TargetList",components:{},mixins:[i["getDataMixin"]],props:{tomApiBaseUrl:{type:String,required:!0},selectable:{type:Boolean,required:!1,default:!1}},data:function(){return{targets:[]}},methods:{initializeDataEndpoint:function(){return this.tomApiBaseUrl+"/api/targets/"},onSuccessfulRetrieval:function(e){return this.targets=e["data"]["results"],e}}},s=o;function l(e,t,a,n,r,i,o,s){var l,c="function"===typeof e?e.options:e;if(t&&(c.render=t,c.staticRenderFns=a,c._compiled=!0),n&&(c.functional=!0),i&&(c._scopeId="data-v-"+i),o?(l=function(e){e=e||this.$vnode&&this.$vnode.ssrContext||this.parent&&this.parent.$vnode&&this.parent.$vnode.ssrContext,e||"undefined"===typeof __VUE_SSR_CONTEXT__||(e=__VUE_SSR_CONTEXT__),r&&r.call(this,e),e&&e._registeredComponents&&e._registeredComponents.add(o)},c._ssrRegister=l):r&&(l=s?function(){r.call(this,(c.functional?this.parent:this).$root.$options.shadowRoot)}:r),l)if(c.functional){c._injectStyles=l;var u=c.render;c.render=function(e,t){return l.call(t),u(e,t)}}else{var d=c.beforeCreate;c.beforeCreate=d?[].concat(d,l):[l]}return{exports:e,options:c}}a("3e49");var c=l(s,n,r,!1,null,null,null),u=c.exports,d=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("b-container",[a("b-table",{attrs:{fields:e.targetFields,items:e.targets},scopedSlots:e._u([!0===e.selectable?{key:"cell(selected)",fn:function(t){return[a("b-form-checkbox",{on:{change:function(a){return e.$emit("selected-target",t,a)}}})]}}:null],null,!0)})],1)},h=[],f={name:"TargetTable",components:{},props:{selectable:{type:Boolean,required:!1,default:!0},targets:{type:Array,required:!0}},computed:{targetFields(){let e=[];return this.selectable&&e.push({key:"selected",label:""}),e.push({key:"id",sortable:!0},{key:"name",sortable:!0},{key:"ra"},{key:"dec"}),e}}},p=f,m=l(p,d,h,!1,null,null,null),v=m.exports,_=function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("div",{staticClass:"hello"},[a("h1",[e._v(e._s(e.msg))]),e._m(0),a("h3",[e._v("Installed CLI Plugins")]),e._m(1),a("h3",[e._v("Essential Links")]),e._m(2),a("h3",[e._v("Ecosystem")]),e._m(3)])},g=[function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("p",[e._v(" blah blah blah blah "),a("a",{attrs:{href:"https://cli.vuejs.org",target:"_blank",rel:"noopener"}},[e._v("vue-cli documentation")]),e._v(". ")])},function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("ul",[a("li",[a("a",{attrs:{href:"https://github.com/vuejs/vue-cli/tree/dev/packages/%40vue/cli-plugin-babel",target:"_blank",rel:"noopener"}},[e._v("babel")])]),a("li",[a("a",{attrs:{href:"https://github.com/vuejs/vue-cli/tree/dev/packages/%40vue/cli-plugin-eslint",target:"_blank",rel:"noopener"}},[e._v("eslint")])])])},function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("ul",[a("li",[a("a",{attrs:{href:"https://vuejs.org",target:"_blank",rel:"noopener"}},[e._v("Core Docs")])]),a("li",[a("a",{attrs:{href:"https://forum.vuejs.org",target:"_blank",rel:"noopener"}},[e._v("Forum")])]),a("li",[a("a",{attrs:{href:"https://chat.vuejs.org",target:"_blank",rel:"noopener"}},[e._v("Community Chat")])]),a("li",[a("a",{attrs:{href:"https://twitter.com/vuejs",target:"_blank",rel:"noopener"}},[e._v("Twitter")])]),a("li",[a("a",{attrs:{href:"https://news.vuejs.org",target:"_blank",rel:"noopener"}},[e._v("News")])])])},function(){var e=this,t=e.$createElement,a=e._self._c||t;return a("ul",[a("li",[a("a",{attrs:{href:"https://router.vuejs.org",target:"_blank",rel:"noopener"}},[e._v("vue-router")])]),a("li",[a("a",{attrs:{href:"https://vuex.vuejs.org",target:"_blank",rel:"noopener"}},[e._v("vuex")])]),a("li",[a("a",{attrs:{href:"https://github.com/vuejs/vue-devtools#vue-devtools",target:"_blank",rel:"noopener"}},[e._v("vue-devtools")])]),a("li",[a("a",{attrs:{href:"https://vue-loader.vuejs.org",target:"_blank",rel:"noopener"}},[e._v("vue-loader")])]),a("li",[a("a",{attrs:{href:"https://github.com/vuejs/awesome-vue",target:"_blank",rel:"noopener"}},[e._v("awesome-vue")])])])}],y={name:"HelloWorld",props:{msg:{type:String,default:"Hello!"}}},b=y,M=(a("e673"),l(b,_,g,!1,null,"12016ea4",null)),z=M.exports},"2d83":function(e,t,a){"use strict";var n=a("387f");e.exports=function(e,t,a,r,i){var o=new Error(e);return n(o,t,a,r,i)}},"2e67":function(e,t,a){"use strict";e.exports=function(e){return!(!e||!e.__CANCEL__)}},"30b5":function(e,t,a){"use strict";var n=a("c532");function r(e){return encodeURIComponent(e).replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",").replace(/%20/g,"+").replace(/%5B/gi,"[").replace(/%5D/gi,"]")}e.exports=function(e,t,a){if(!t)return e;var i;if(a)i=a(t);else if(n.isURLSearchParams(t))i=t.toString();else{var o=[];n.forEach(t,(function(e,t){null!==e&&"undefined"!==typeof e&&(n.isArray(e)?t+="[]":e=[e],n.forEach(e,(function(e){n.isDate(e)?e=e.toISOString():n.isObject(e)&&(e=JSON.stringify(e)),o.push(r(t)+"="+r(e))})))})),i=o.join("&")}if(i){var s=e.indexOf("#");-1!==s&&(e=e.slice(0,s)),e+=(-1===e.indexOf("?")?"?":"&")+i}return e}},"387f":function(e,t,a){"use strict";e.exports=function(e,t,a,n,r){return e.config=t,a&&(e.code=a),e.request=n,e.response=r,e.isAxiosError=!0,e.toJSON=function(){return{message:this.message,name:this.name,description:this.description,number:this.number,fileName:this.fileName,lineNumber:this.lineNumber,columnNumber:this.columnNumber,stack:this.stack,config:this.config,code:this.code}},e}},3934:function(e,t,a){"use strict";var n=a("c532");e.exports=n.isStandardBrowserEnv()?function(){var e,t=/(msie|trident)/i.test(navigator.userAgent),a=document.createElement("a");function r(e){var n=e;return t&&(a.setAttribute("href",n),n=a.href),a.setAttribute("href",n),{href:a.href,protocol:a.protocol?a.protocol.replace(/:$/,""):"",host:a.host,search:a.search?a.search.replace(/^\?/,""):"",hash:a.hash?a.hash.replace(/^#/,""):"",hostname:a.hostname,port:a.port,pathname:"/"===a.pathname.charAt(0)?a.pathname:"/"+a.pathname}}return e=r(window.location.href),function(t){var a=n.isString(t)?r(t):t;return a.protocol===e.protocol&&a.host===e.host}}():function(){return function(){return!0}}()},"3e49":function(e,t,a){"use strict";a("5a34")},4362:function(e,t,a){t.nextTick=function(e){var t=Array.prototype.slice.call(arguments);t.shift(),setTimeout((function(){e.apply(null,t)}),0)},t.platform=t.arch=t.execPath=t.title="browser",t.pid=1,t.browser=!0,t.env={},t.argv=[],t.binding=function(e){throw new Error("No such module. (Possibly not yet loaded)")},function(){var e,n="/";t.cwd=function(){return n},t.chdir=function(t){e||(e=a("df7c")),n=e.resolve(t,n)}}(),t.exit=t.kill=t.umask=t.dlopen=t.uptime=t.memoryUsage=t.uvCounters=function(){},t.features={}},"467f":function(e,t,a){"use strict";var n=a("2d83");e.exports=function(e,t,a){var r=a.config.validateStatus;a.status&&r&&!r(a.status)?t(n("Request failed with status code "+a.status,a.config,null,a.request,a)):e(a)}},"4a0c":function(e){e.exports=JSON.parse('{"name":"axios","version":"0.21.4","description":"Promise based HTTP client for the browser and node.js","main":"index.js","scripts":{"test":"grunt test","start":"node ./sandbox/server.js","build":"NODE_ENV=production grunt build","preversion":"npm test","version":"npm run build && grunt version && git add -A dist && git add CHANGELOG.md bower.json package.json","postversion":"git push && git push --tags","examples":"node ./examples/server.js","coveralls":"cat coverage/lcov.info | ./node_modules/coveralls/bin/coveralls.js","fix":"eslint --fix lib/**/*.js"},"repository":{"type":"git","url":"https://github.com/axios/axios.git"},"keywords":["xhr","http","ajax","promise","node"],"author":"Matt Zabriskie","license":"MIT","bugs":{"url":"https://github.com/axios/axios/issues"},"homepage":"https://axios-http.com","devDependencies":{"coveralls":"^3.0.0","es6-promise":"^4.2.4","grunt":"^1.3.0","grunt-banner":"^0.6.0","grunt-cli":"^1.2.0","grunt-contrib-clean":"^1.1.0","grunt-contrib-watch":"^1.0.0","grunt-eslint":"^23.0.0","grunt-karma":"^4.0.0","grunt-mocha-test":"^0.13.3","grunt-ts":"^6.0.0-beta.19","grunt-webpack":"^4.0.2","istanbul-instrumenter-loader":"^1.0.0","jasmine-core":"^2.4.1","karma":"^6.3.2","karma-chrome-launcher":"^3.1.0","karma-firefox-launcher":"^2.1.0","karma-jasmine":"^1.1.1","karma-jasmine-ajax":"^0.1.13","karma-safari-launcher":"^1.0.0","karma-sauce-launcher":"^4.3.6","karma-sinon":"^1.0.5","karma-sourcemap-loader":"^0.3.8","karma-webpack":"^4.0.2","load-grunt-tasks":"^3.5.2","minimist":"^1.2.0","mocha":"^8.2.1","sinon":"^4.5.0","terser-webpack-plugin":"^4.2.3","typescript":"^4.0.5","url-search-params":"^0.10.0","webpack":"^4.44.2","webpack-dev-server":"^3.11.0"},"browser":{"./lib/adapters/http.js":"./lib/adapters/xhr.js"},"jsdelivr":"dist/axios.min.js","unpkg":"dist/axios.min.js","typings":"./index.d.ts","dependencies":{"follow-redirects":"^1.14.0"},"bundlesize":[{"path":"./dist/axios.min.js","threshold":"5kB"}]}')},"4a7b":function(e,t,a){"use strict";var n=a("c532");e.exports=function(e,t){t=t||{};var a={},r=["url","method","data"],i=["headers","auth","proxy","params"],o=["baseURL","transformRequest","transformResponse","paramsSerializer","timeout","timeoutMessage","withCredentials","adapter","responseType","xsrfCookieName","xsrfHeaderName","onUploadProgress","onDownloadProgress","decompress","maxContentLength","maxBodyLength","maxRedirects","transport","httpAgent","httpsAgent","cancelToken","socketPath","responseEncoding"],s=["validateStatus"];function l(e,t){return n.isPlainObject(e)&&n.isPlainObject(t)?n.merge(e,t):n.isPlainObject(t)?n.merge({},t):n.isArray(t)?t.slice():t}function c(r){n.isUndefined(t[r])?n.isUndefined(e[r])||(a[r]=l(void 0,e[r])):a[r]=l(e[r],t[r])}n.forEach(r,(function(e){n.isUndefined(t[e])||(a[e]=l(void 0,t[e]))})),n.forEach(i,c),n.forEach(o,(function(r){n.isUndefined(t[r])?n.isUndefined(e[r])||(a[r]=l(void 0,e[r])):a[r]=l(void 0,t[r])})),n.forEach(s,(function(n){n in t?a[n]=l(e[n],t[n]):n in e&&(a[n]=l(void 0,e[n]))}));var u=r.concat(i).concat(o).concat(s),d=Object.keys(e).concat(Object.keys(t)).filter((function(e){return-1===u.indexOf(e)}));return n.forEach(d,c),a}},5270:function(e,t,a){"use strict";var n=a("c532"),r=a("c401"),i=a("2e67"),o=a("2444");function s(e){e.cancelToken&&e.cancelToken.throwIfRequested()}e.exports=function(e){s(e),e.headers=e.headers||{},e.data=r.call(e,e.data,e.headers,e.transformRequest),e.headers=n.merge(e.headers.common||{},e.headers[e.method]||{},e.headers),n.forEach(["delete","get","head","post","put","patch","common"],(function(t){delete e.headers[t]}));var t=e.adapter||o.adapter;return t(e).then((function(t){return s(e),t.data=r.call(e,t.data,t.headers,e.transformResponse),t}),(function(t){return i(t)||(s(e),t&&t.response&&(t.response.data=r.call(e,t.response.data,t.response.headers,e.transformResponse))),Promise.reject(t)}))}},"56d7":function(e,t,a){"use strict";(function(e){a.d(t,"a",(function(){return i}));var n=a("2af9"),r=a("1b62");a.d(t,"b",(function(){return r}));const i={install(e){for(const t in n){const a=n[t];e.component("ttk"+a.name,a)}}};let o=null;"undefined"!==typeof window?o=window.Vue:"undefined"!==typeof e&&(o=e.Vue),o&&o.use(i)}).call(this,a("c8ba"))},"5a34":function(e,t,a){},"5f02":function(e,t,a){"use strict";e.exports=function(e){return"object"===typeof e&&!0===e.isAxiosError}},"7a77":function(e,t,a){"use strict";function n(e){this.message=e}n.prototype.toString=function(){return"Cancel"+(this.message?": "+this.message:"")},n.prototype.__CANCEL__=!0,e.exports=n},"7aac":function(e,t,a){"use strict";var n=a("c532");e.exports=n.isStandardBrowserEnv()?function(){return{write:function(e,t,a,r,i,o){var s=[];s.push(e+"="+encodeURIComponent(t)),n.isNumber(a)&&s.push("expires="+new Date(a).toGMTString()),n.isString(r)&&s.push("path="+r),n.isString(i)&&s.push("domain="+i),!0===o&&s.push("secure"),document.cookie=s.join("; ")},read:function(e){var t=document.cookie.match(new RegExp("(^|;\\s*)("+e+")=([^;]*)"));return t?decodeURIComponent(t[3]):null},remove:function(e){this.write(e,"",Date.now()-864e5)}}}():function(){return{write:function(){},read:function(){return null},remove:function(){}}}()},"83b9":function(e,t,a){"use strict";var n=a("d925"),r=a("e683");e.exports=function(e,t){return e&&!n(t)?r(e,t):t}},"848b":function(e,t,a){"use strict";var n=a("4a0c"),r={};["object","boolean","number","function","string","symbol"].forEach((function(e,t){r[e]=function(a){return typeof a===e||"a"+(t<1?"n ":" ")+e}}));var i={},o=n.version.split(".");function s(e,t){for(var a=t?t.split("."):o,n=e.split("."),r=0;r<3;r++){if(a[r]>n[r])return!0;if(a[r]<n[r])return!1}return!1}function l(e,t,a){if("object"!==typeof e)throw new TypeError("options must be an object");var n=Object.keys(e),r=n.length;while(r-- >0){var i=n[r],o=t[i];if(o){var s=e[i],l=void 0===s||o(s,i,e);if(!0!==l)throw new TypeError("option "+i+" must be "+l)}else if(!0!==a)throw Error("Unknown option "+i)}}r.transitional=function(e,t,a){var r=t&&s(t);function o(e,t){return"[Axios v"+n.version+"] Transitional option '"+e+"'"+t+(a?". "+a:"")}return function(a,n,s){if(!1===e)throw new Error(o(n," has been removed in "+t));return r&&!i[n]&&(i[n]=!0,console.warn(o(n," has been deprecated since v"+t+" and will be removed in the near future"))),!e||e(a,n,s)}},e.exports={isOlderVersion:s,assertOptions:l,validators:r}},"8df4":function(e,t,a){"use strict";var n=a("7a77");function r(e){if("function"!==typeof e)throw new TypeError("executor must be a function.");var t;this.promise=new Promise((function(e){t=e}));var a=this;e((function(e){a.reason||(a.reason=new n(e),t(a.reason))}))}r.prototype.throwIfRequested=function(){if(this.reason)throw this.reason},r.source=function(){var e,t=new r((function(t){e=t}));return{token:t,cancel:e}},e.exports=r},b50d:function(e,t,a){"use strict";var n=a("c532"),r=a("467f"),i=a("7aac"),o=a("30b5"),s=a("83b9"),l=a("c345"),c=a("3934"),u=a("2d83");e.exports=function(e){return new Promise((function(t,a){var d=e.data,h=e.headers,f=e.responseType;n.isFormData(d)&&delete h["Content-Type"];var p=new XMLHttpRequest;if(e.auth){var m=e.auth.username||"",v=e.auth.password?unescape(encodeURIComponent(e.auth.password)):"";h.Authorization="Basic "+btoa(m+":"+v)}var _=s(e.baseURL,e.url);function g(){if(p){var n="getAllResponseHeaders"in p?l(p.getAllResponseHeaders()):null,i=f&&"text"!==f&&"json"!==f?p.response:p.responseText,o={data:i,status:p.status,statusText:p.statusText,headers:n,config:e,request:p};r(t,a,o),p=null}}if(p.open(e.method.toUpperCase(),o(_,e.params,e.paramsSerializer),!0),p.timeout=e.timeout,"onloadend"in p?p.onloadend=g:p.onreadystatechange=function(){p&&4===p.readyState&&(0!==p.status||p.responseURL&&0===p.responseURL.indexOf("file:"))&&setTimeout(g)},p.onabort=function(){p&&(a(u("Request aborted",e,"ECONNABORTED",p)),p=null)},p.onerror=function(){a(u("Network Error",e,null,p)),p=null},p.ontimeout=function(){var t="timeout of "+e.timeout+"ms exceeded";e.timeoutErrorMessage&&(t=e.timeoutErrorMessage),a(u(t,e,e.transitional&&e.transitional.clarifyTimeoutError?"ETIMEDOUT":"ECONNABORTED",p)),p=null},n.isStandardBrowserEnv()){var y=(e.withCredentials||c(_))&&e.xsrfCookieName?i.read(e.xsrfCookieName):void 0;y&&(h[e.xsrfHeaderName]=y)}"setRequestHeader"in p&&n.forEach(h,(function(e,t){"undefined"===typeof d&&"content-type"===t.toLowerCase()?delete h[t]:p.setRequestHeader(t,e)})),n.isUndefined(e.withCredentials)||(p.withCredentials=!!e.withCredentials),f&&"json"!==f&&(p.responseType=e.responseType),"function"===typeof e.onDownloadProgress&&p.addEventListener("progress",e.onDownloadProgress),"function"===typeof e.onUploadProgress&&p.upload&&p.upload.addEventListener("progress",e.onUploadProgress),e.cancelToken&&e.cancelToken.promise.then((function(e){p&&(p.abort(),a(e),p=null)})),d||(d=null),p.send(d)}))}},bc3a:function(e,t,a){e.exports=a("cee4")},c345:function(e,t,a){"use strict";var n=a("c532"),r=["age","authorization","content-length","content-type","etag","expires","from","host","if-modified-since","if-unmodified-since","last-modified","location","max-forwards","proxy-authorization","referer","retry-after","user-agent"];e.exports=function(e){var t,a,i,o={};return e?(n.forEach(e.split("\n"),(function(e){if(i=e.indexOf(":"),t=n.trim(e.substr(0,i)).toLowerCase(),a=n.trim(e.substr(i+1)),t){if(o[t]&&r.indexOf(t)>=0)return;o[t]="set-cookie"===t?(o[t]?o[t]:[]).concat([a]):o[t]?o[t]+", "+a:a}})),o):o}},c401:function(e,t,a){"use strict";var n=a("c532"),r=a("2444");e.exports=function(e,t,a){var i=this||r;return n.forEach(a,(function(a){e=a.call(i,e,t)})),e}},c532:function(e,t,a){"use strict";var n=a("1d2b"),r=Object.prototype.toString;function i(e){return"[object Array]"===r.call(e)}function o(e){return"undefined"===typeof e}function s(e){return null!==e&&!o(e)&&null!==e.constructor&&!o(e.constructor)&&"function"===typeof e.constructor.isBuffer&&e.constructor.isBuffer(e)}function l(e){return"[object ArrayBuffer]"===r.call(e)}function c(e){return"undefined"!==typeof FormData&&e instanceof FormData}function u(e){var t;return t="undefined"!==typeof ArrayBuffer&&ArrayBuffer.isView?ArrayBuffer.isView(e):e&&e.buffer&&e.buffer instanceof ArrayBuffer,t}function d(e){return"string"===typeof e}function h(e){return"number"===typeof e}function f(e){return null!==e&&"object"===typeof e}function p(e){if("[object Object]"!==r.call(e))return!1;var t=Object.getPrototypeOf(e);return null===t||t===Object.prototype}function m(e){return"[object Date]"===r.call(e)}function v(e){return"[object File]"===r.call(e)}function _(e){return"[object Blob]"===r.call(e)}function g(e){return"[object Function]"===r.call(e)}function y(e){return f(e)&&g(e.pipe)}function b(e){return"undefined"!==typeof URLSearchParams&&e instanceof URLSearchParams}function M(e){return e.trim?e.trim():e.replace(/^\s+|\s+$/g,"")}function z(){return("undefined"===typeof navigator||"ReactNative"!==navigator.product&&"NativeScript"!==navigator.product&&"NS"!==navigator.product)&&"undefined"!==typeof window&&"undefined"!==typeof document}function L(e,t){if(null!==e&&"undefined"!==typeof e)if("object"!==typeof e&&(e=[e]),i(e))for(var a=0,n=e.length;a<n;a++)t.call(null,e[a],a,e);else for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.call(null,e[r],r,e)}function w(){var e={};function t(t,a){p(e[a])&&p(t)?e[a]=w(e[a],t):p(t)?e[a]=w({},t):i(t)?e[a]=t.slice():e[a]=t}for(var a=0,n=arguments.length;a<n;a++)L(arguments[a],t);return e}function H(e,t,a){return L(t,(function(t,r){e[r]=a&&"function"===typeof t?n(t,a):t})),e}function k(e){return 65279===e.charCodeAt(0)&&(e=e.slice(1)),e}e.exports={isArray:i,isArrayBuffer:l,isBuffer:s,isFormData:c,isArrayBufferView:u,isString:d,isNumber:h,isObject:f,isPlainObject:p,isUndefined:o,isDate:m,isFile:v,isBlob:_,isFunction:g,isStream:y,isURLSearchParams:b,isStandardBrowserEnv:z,forEach:L,merge:w,extend:H,trim:M,stripBOM:k}},c8af:function(e,t,a){"use strict";var n=a("c532");e.exports=function(e,t){n.forEach(e,(function(a,n){n!==t&&n.toUpperCase()===t.toUpperCase()&&(e[t]=a,delete e[n])}))}},c8ba:function(e,t){var a;a=function(){return this}();try{a=a||new Function("return this")()}catch(n){"object"===typeof window&&(a=window)}e.exports=a},cee4:function(e,t,a){"use strict";var n=a("c532"),r=a("1d2b"),i=a("0a06"),o=a("4a7b"),s=a("2444");function l(e){var t=new i(e),a=r(i.prototype.request,t);return n.extend(a,i.prototype,t),n.extend(a,t),a}var c=l(s);c.Axios=i,c.create=function(e){return l(o(c.defaults,e))},c.Cancel=a("7a77"),c.CancelToken=a("8df4"),c.isCancel=a("2e67"),c.all=function(e){return Promise.all(e)},c.spread=a("0df6"),c.isAxiosError=a("5f02"),e.exports=c,e.exports.default=c},d925:function(e,t,a){"use strict";e.exports=function(e){return/^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(e)}},df7c:function(e,t,a){(function(e){function a(e,t){for(var a=0,n=e.length-1;n>=0;n--){var r=e[n];"."===r?e.splice(n,1):".."===r?(e.splice(n,1),a++):a&&(e.splice(n,1),a--)}if(t)for(;a--;a)e.unshift("..");return e}function n(e){"string"!==typeof e&&(e+="");var t,a=0,n=-1,r=!0;for(t=e.length-1;t>=0;--t)if(47===e.charCodeAt(t)){if(!r){a=t+1;break}}else-1===n&&(r=!1,n=t+1);return-1===n?"":e.slice(a,n)}function r(e,t){if(e.filter)return e.filter(t);for(var a=[],n=0;n<e.length;n++)t(e[n],n,e)&&a.push(e[n]);return a}t.resolve=function(){for(var t="",n=!1,i=arguments.length-1;i>=-1&&!n;i--){var o=i>=0?arguments[i]:e.cwd();if("string"!==typeof o)throw new TypeError("Arguments to path.resolve must be strings");o&&(t=o+"/"+t,n="/"===o.charAt(0))}return t=a(r(t.split("/"),(function(e){return!!e})),!n).join("/"),(n?"/":"")+t||"."},t.normalize=function(e){var n=t.isAbsolute(e),o="/"===i(e,-1);return e=a(r(e.split("/"),(function(e){return!!e})),!n).join("/"),e||n||(e="."),e&&o&&(e+="/"),(n?"/":"")+e},t.isAbsolute=function(e){return"/"===e.charAt(0)},t.join=function(){var e=Array.prototype.slice.call(arguments,0);return t.normalize(r(e,(function(e,t){if("string"!==typeof e)throw new TypeError("Arguments to path.join must be strings");return e})).join("/"))},t.relative=function(e,a){function n(e){for(var t=0;t<e.length;t++)if(""!==e[t])break;for(var a=e.length-1;a>=0;a--)if(""!==e[a])break;return t>a?[]:e.slice(t,a-t+1)}e=t.resolve(e).substr(1),a=t.resolve(a).substr(1);for(var r=n(e.split("/")),i=n(a.split("/")),o=Math.min(r.length,i.length),s=o,l=0;l<o;l++)if(r[l]!==i[l]){s=l;break}var c=[];for(l=s;l<r.length;l++)c.push("..");return c=c.concat(i.slice(s)),c.join("/")},t.sep="/",t.delimiter=":",t.dirname=function(e){if("string"!==typeof e&&(e+=""),0===e.length)return".";for(var t=e.charCodeAt(0),a=47===t,n=-1,r=!0,i=e.length-1;i>=1;--i)if(t=e.charCodeAt(i),47===t){if(!r){n=i;break}}else r=!1;return-1===n?a?"/":".":a&&1===n?"/":e.slice(0,n)},t.basename=function(e,t){var a=n(e);return t&&a.substr(-1*t.length)===t&&(a=a.substr(0,a.length-t.length)),a},t.extname=function(e){"string"!==typeof e&&(e+="");for(var t=-1,a=0,n=-1,r=!0,i=0,o=e.length-1;o>=0;--o){var s=e.charCodeAt(o);if(47!==s)-1===n&&(r=!1,n=o+1),46===s?-1===t?t=o:1!==i&&(i=1):-1!==t&&(i=-1);else if(!r){a=o+1;break}}return-1===t||-1===n||0===i||1===i&&t===n-1&&t===a+1?"":e.slice(t,n)};var i="b"==="ab".substr(-1)?function(e,t,a){return e.substr(t,a)}:function(e,t,a){return t<0&&(t=e.length+t),e.substr(t,a)}}).call(this,a("4362"))},e50c:function(e,t,a){},e673:function(e,t,a){"use strict";a("e50c")},e683:function(e,t,a){"use strict";e.exports=function(e,t){return t?e.replace(/\/+$/,"")+"/"+t.replace(/^\/+/,""):e}},ea46:function(e,t,a){"use strict";(function(e){a.d(t,"a",(function(){return i}));var n=a("bc3a"),r=a.n(n),i={data:function(){const e=this.initializeDataEndpoint();return{dataLoadError:!1,dataLoaded:!1,dataNotFound:!1,data:this.initializeEmptyData(),dataEndpoint:e}},created:function(){this.getData()},methods:{initializeDataEndpoint:function(){return""},setDataEndpoint:function(e){this.dataEndpoint=e},initializeEmptyData:function(){return{}},getData:function(){this.clearLoadStates(),r.a.get(this.dataEndpoint).then(e=>{this.successCallback(e)}).catch(t=>{e.stdout.write("error: "+t+"\n")})},clearLoadStates:function(){this.dataLoaded=!1,this.dataLoadError=!1,this.dataNotFound=!1},failCallback:function(e){404===e.status||403===e.status?this.dataNotFound=!0:this.dataLoadError=!0,this.onDataRetrievalError(e)},successCallback:function(e){this.data=e,this.onSuccessfulRetrieval(e)},onSuccessfulRetrieval:function(e){return e},onDataRetrievalError:function(e){return e}}}}).call(this,a("4362"))},f6b4:function(e,t,a){"use strict";var n=a("c532");function r(){this.handlers=[]}r.prototype.use=function(e,t,a){return this.handlers.push({fulfilled:e,rejected:t,synchronous:!!a&&a.synchronous,runWhen:a?a.runWhen:null}),this.handlers.length-1},r.prototype.eject=function(e){this.handlers[e]&&(this.handlers[e]=null)},r.prototype.forEach=function(e){n.forEach(this.handlers,(function(t){null!==t&&e(t)}))},e.exports=r},fae3:function(e,t,a){"use strict";if(a.r(t),a.d(t,"TOMToolkitComponentLib",(function(){return i["a"]})),a.d(t,"TTKMixin",(function(){return i["b"]})),"undefined"!==typeof window){var n=window.document.currentScript,r=n&&n.src.match(/(.+\/)[^/]+\.js(\?.*)?$/);r&&(a.p=r[1])}var i=a("56d7")}})}))},d1e7:function(e,t,a){"use strict";var n={}.propertyIsEnumerable,r=Object.getOwnPropertyDescriptor,i=r&&!n.call({1:2},1);t.f=i?function(e){var t=r(this,e);return!!t&&t.enumerable}:n},d26a:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"༡",2:"༢",3:"༣",4:"༤",5:"༥",6:"༦",7:"༧",8:"༨",9:"༩",0:"༠"},a={"༡":"1","༢":"2","༣":"3","༤":"4","༥":"5","༦":"6","༧":"7","༨":"8","༩":"9","༠":"0"},n=e.defineLocale("bo",{months:"ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ".split("_"),monthsShort:"ཟླ་1_ཟླ་2_ཟླ་3_ཟླ་4_ཟླ་5_ཟླ་6_ཟླ་7_ཟླ་8_ཟླ་9_ཟླ་10_ཟླ་11_ཟླ་12".split("_"),monthsShortRegex:/^(ཟླ་\d{1,2})/,monthsParseExact:!0,weekdays:"གཟའ་ཉི་མ་_གཟའ་ཟླ་བ་_གཟའ་མིག་དམར་_གཟའ་ལྷག་པ་_གཟའ་ཕུར་བུ_གཟའ་པ་སངས་_གཟའ་སྤེན་པ་".split("_"),weekdaysShort:"ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་".split("_"),weekdaysMin:"ཉི_ཟླ_མིག_ལྷག_ཕུར_སངས_སྤེན".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[དི་རིང] LT",nextDay:"[སང་ཉིན] LT",nextWeek:"[བདུན་ཕྲག་རྗེས་མ], LT",lastDay:"[ཁ་སང] LT",lastWeek:"[བདུན་ཕྲག་མཐའ་མ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ལ་",past:"%s སྔན་ལ",s:"ལམ་སང",ss:"%d སྐར་ཆ།",m:"སྐར་མ་གཅིག",mm:"%d སྐར་མ",h:"ཆུ་ཚོད་གཅིག",hh:"%d ཆུ་ཚོད",d:"ཉིན་གཅིག",dd:"%d ཉིན་",M:"ཟླ་བ་གཅིག",MM:"%d ཟླ་བ",y:"ལོ་གཅིག",yy:"%d ལོ"},preparse:function(e){return e.replace(/[༡༢༣༤༥༦༧༨༩༠]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/མཚན་མོ|ཞོགས་ཀས|ཉིན་གུང|དགོང་དག|མཚན་མོ/,meridiemHour:function(e,t){return 12===e&&(e=0),"མཚན་མོ"===t&&e>=4||"ཉིན་གུང"===t&&e<5||"དགོང་དག"===t?e+12:e},meridiem:function(e,t,a){return e<4?"མཚན་མོ":e<10?"ཞོགས་ཀས":e<17?"ཉིན་གུང":e<20?"དགོང་དག":"མཚན་མོ"},week:{dow:0,doy:6}});return n}))},d2d4:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("pt-br",{months:"janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"domingo_segunda-feira_terça-feira_quarta-feira_quinta-feira_sexta-feira_sábado".split("_"),weekdaysShort:"dom_seg_ter_qua_qui_sex_sáb".split("_"),weekdaysMin:"do_2ª_3ª_4ª_5ª_6ª_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY [às] HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY [às] HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"poucos segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",invalidDate:"Data inválida"});return t}))},d69a:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fil",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}});return t}))},d6b6:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("hy-am",{months:{format:"հունվարի_փետրվարի_մարտի_ապրիլի_մայիսի_հունիսի_հուլիսի_օգոստոսի_սեպտեմբերի_հոկտեմբերի_նոյեմբերի_դեկտեմբերի".split("_"),standalone:"հունվար_փետրվար_մարտ_ապրիլ_մայիս_հունիս_հուլիս_օգոստոս_սեպտեմբեր_հոկտեմբեր_նոյեմբեր_դեկտեմբեր".split("_")},monthsShort:"հնվ_փտր_մրտ_ապր_մյս_հնս_հլս_օգս_սպտ_հկտ_նմբ_դկտ".split("_"),weekdays:"կիրակի_երկուշաբթի_երեքշաբթի_չորեքշաբթի_հինգշաբթի_ուրբաթ_շաբաթ".split("_"),weekdaysShort:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),weekdaysMin:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY թ.",LLL:"D MMMM YYYY թ., HH:mm",LLLL:"dddd, D MMMM YYYY թ., HH:mm"},calendar:{sameDay:"[այսօր] LT",nextDay:"[վաղը] LT",lastDay:"[երեկ] LT",nextWeek:function(){return"dddd [օրը ժամը] LT"},lastWeek:function(){return"[անցած] dddd [օրը ժամը] LT"},sameElse:"L"},relativeTime:{future:"%s հետո",past:"%s առաջ",s:"մի քանի վայրկյան",ss:"%d վայրկյան",m:"րոպե",mm:"%d րոպե",h:"ժամ",hh:"%d ժամ",d:"օր",dd:"%d օր",M:"ամիս",MM:"%d ամիս",y:"տարի",yy:"%d տարի"},meridiemParse:/գիշերվա|առավոտվա|ցերեկվա|երեկոյան/,isPM:function(e){return/^(ցերեկվա|երեկոյան)$/.test(e)},meridiem:function(e){return e<4?"գիշերվա":e<12?"առավոտվա":e<17?"ցերեկվա":"երեկոյան"},dayOfMonthOrdinalParse:/\d{1,2}|\d{1,2}-(ին|րդ)/,ordinal:function(e,t){switch(t){case"DDD":case"w":case"W":case"DDDo":return 1===e?e+"-ին":e+"-րդ";default:return e}},week:{dow:1,doy:7}});return t}))},d716:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ca",{months:{standalone:"gener_febrer_març_abril_maig_juny_juliol_agost_setembre_octubre_novembre_desembre".split("_"),format:"de gener_de febrer_de març_d'abril_de maig_de juny_de juliol_d'agost_de setembre_d'octubre_de novembre_de desembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._març_abr._maig_juny_jul._ag._set._oct._nov._des.".split("_"),monthsParseExact:!0,weekdays:"diumenge_dilluns_dimarts_dimecres_dijous_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dt._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dt_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a les] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a les] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:function(){return"[avui a "+(1!==this.hours()?"les":"la")+"] LT"},nextDay:function(){return"[demà a "+(1!==this.hours()?"les":"la")+"] LT"},nextWeek:function(){return"dddd [a "+(1!==this.hours()?"les":"la")+"] LT"},lastDay:function(){return"[ahir a "+(1!==this.hours()?"les":"la")+"] LT"},lastWeek:function(){return"[el] dddd [passat a "+(1!==this.hours()?"les":"la")+"] LT"},sameElse:"L"},relativeTime:{future:"d'aquí %s",past:"fa %s",s:"uns segons",ss:"%d segons",m:"un minut",mm:"%d minuts",h:"una hora",hh:"%d hores",d:"un dia",dd:"%d dies",M:"un mes",MM:"%d mesos",y:"un any",yy:"%d anys"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(e,t){var a=1===e?"r":2===e?"n":3===e?"r":4===e?"t":"è";return"w"!==t&&"W"!==t||(a="a"),e+a},week:{dow:1,doy:4}});return t}))},d925:function(e,t,a){"use strict";e.exports=function(e){return/^([a-z][a-z\d\+\-\.]*:)?\/\//i.test(e)}},d9b5:function(e,t,a){var n=a("d066"),r=a("1626"),i=a("3a9b"),o=a("fdbf"),s=Object;e.exports=o?function(e){return"symbol"==typeof e}:function(e){var t=n("Symbol");return r(t)&&i(t.prototype,s(e))}},d9f8:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fr-ca",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(e,t){switch(t){default:case"M":case"Q":case"D":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}}});return t}))},da84:function(e,t,a){(function(t){var a=function(e){return e&&e.Math==Math&&e};e.exports=a("object"==typeof globalThis&&globalThis)||a("object"==typeof window&&window)||a("object"==typeof self&&self)||a("object"==typeof t&&t)||function(){return this}()||Function("return this")()}).call(this,a("c8ba"))},db29:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),a="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),n=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],r=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,i=e.defineLocale("nl-be",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(e,n){return e?/-MMM-/.test(n)?a[e.month()]:t[e.month()]:t},monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return i}))},dc4a:function(e,t,a){var n=a("59ed"),r=a("7234");e.exports=function(e,t){var a=e[t];return r(a)?void 0:n(a)}},dc4d:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},a={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},n=[/^जन/i,/^फ़र|फर/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सितं|सित/i,/^अक्टू/i,/^नव|नवं/i,/^दिसं|दिस/i],r=[/^जन/i,/^फ़र/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सित/i,/^अक्टू/i,/^नव/i,/^दिस/i],i=e.defineLocale("hi",{months:{format:"जनवरी_फ़रवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितम्बर_अक्टूबर_नवम्बर_दिसम्बर".split("_"),standalone:"जनवरी_फरवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितंबर_अक्टूबर_नवंबर_दिसंबर".split("_")},monthsShort:"जन._फ़र._मार्च_अप्रै._मई_जून_जुल._अग._सित._अक्टू._नव._दिस.".split("_"),weekdays:"रविवार_सोमवार_मंगलवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगल_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm बजे",LTS:"A h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm बजे",LLLL:"dddd, D MMMM YYYY, A h:mm बजे"},monthsParse:n,longMonthsParse:n,shortMonthsParse:r,monthsRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsShortRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsStrictRegex:/^(जनवरी?|फ़रवरी|फरवरी?|मार्च?|अप्रैल?|मई?|जून?|जुलाई?|अगस्त?|सितम्बर|सितंबर|सित?\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर?|दिसम्बर|दिसंबर?)/i,monthsShortStrictRegex:/^(जन\.?|फ़र\.?|मार्च?|अप्रै\.?|मई?|जून?|जुल\.?|अग\.?|सित\.?|अक्टू\.?|नव\.?|दिस\.?)/i,calendar:{sameDay:"[आज] LT",nextDay:"[कल] LT",nextWeek:"dddd, LT",lastDay:"[कल] LT",lastWeek:"[पिछले] dddd, LT",sameElse:"L"},relativeTime:{future:"%s में",past:"%s पहले",s:"कुछ ही क्षण",ss:"%d सेकंड",m:"एक मिनट",mm:"%d मिनट",h:"एक घंटा",hh:"%d घंटे",d:"एक दिन",dd:"%d दिन",M:"एक महीने",MM:"%d महीने",y:"एक वर्ष",yy:"%d वर्ष"},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/रात|सुबह|दोपहर|शाम/,meridiemHour:function(e,t){return 12===e&&(e=0),"रात"===t?e<4?e:e+12:"सुबह"===t?e:"दोपहर"===t?e>=10?e:e+12:"शाम"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"रात":e<10?"सुबह":e<17?"दोपहर":e<20?"शाम":"रात"},week:{dow:0,doy:6}});return i}))},df7c:function(e,t,a){(function(e){function a(e,t){for(var a=0,n=e.length-1;n>=0;n--){var r=e[n];"."===r?e.splice(n,1):".."===r?(e.splice(n,1),a++):a&&(e.splice(n,1),a--)}if(t)for(;a--;a)e.unshift("..");return e}function n(e){"string"!==typeof e&&(e+="");var t,a=0,n=-1,r=!0;for(t=e.length-1;t>=0;--t)if(47===e.charCodeAt(t)){if(!r){a=t+1;break}}else-1===n&&(r=!1,n=t+1);return-1===n?"":e.slice(a,n)}function r(e,t){if(e.filter)return e.filter(t);for(var a=[],n=0;n<e.length;n++)t(e[n],n,e)&&a.push(e[n]);return a}t.resolve=function(){for(var t="",n=!1,i=arguments.length-1;i>=-1&&!n;i--){var o=i>=0?arguments[i]:e.cwd();if("string"!==typeof o)throw new TypeError("Arguments to path.resolve must be strings");o&&(t=o+"/"+t,n="/"===o.charAt(0))}return t=a(r(t.split("/"),(function(e){return!!e})),!n).join("/"),(n?"/":"")+t||"."},t.normalize=function(e){var n=t.isAbsolute(e),o="/"===i(e,-1);return e=a(r(e.split("/"),(function(e){return!!e})),!n).join("/"),e||n||(e="."),e&&o&&(e+="/"),(n?"/":"")+e},t.isAbsolute=function(e){return"/"===e.charAt(0)},t.join=function(){var e=Array.prototype.slice.call(arguments,0);return t.normalize(r(e,(function(e,t){if("string"!==typeof e)throw new TypeError("Arguments to path.join must be strings");return e})).join("/"))},t.relative=function(e,a){function n(e){for(var t=0;t<e.length;t++)if(""!==e[t])break;for(var a=e.length-1;a>=0;a--)if(""!==e[a])break;return t>a?[]:e.slice(t,a-t+1)}e=t.resolve(e).substr(1),a=t.resolve(a).substr(1);for(var r=n(e.split("/")),i=n(a.split("/")),o=Math.min(r.length,i.length),s=o,l=0;l<o;l++)if(r[l]!==i[l]){s=l;break}var c=[];for(l=s;l<r.length;l++)c.push("..");return c=c.concat(i.slice(s)),c.join("/")},t.sep="/",t.delimiter=":",t.dirname=function(e){if("string"!==typeof e&&(e+=""),0===e.length)return".";for(var t=e.charCodeAt(0),a=47===t,n=-1,r=!0,i=e.length-1;i>=1;--i)if(t=e.charCodeAt(i),47===t){if(!r){n=i;break}}else r=!1;return-1===n?a?"/":".":a&&1===n?"/":e.slice(0,n)},t.basename=function(e,t){var a=n(e);return t&&a.substr(-1*t.length)===t&&(a=a.substr(0,a.length-t.length)),a},t.extname=function(e){"string"!==typeof e&&(e+="");for(var t=-1,a=0,n=-1,r=!0,i=0,o=e.length-1;o>=0;--o){var s=e.charCodeAt(o);if(47!==s)-1===n&&(r=!1,n=o+1),46===s?-1===t?t=o:1!==i&&(i=1):-1!==t&&(i=-1);else if(!r){a=o+1;break}}return-1===t||-1===n||0===i||1===i&&t===n-1&&t===a+1?"":e.slice(t,n)};var i="b"==="ab".substr(-1)?function(e,t,a){return e.substr(t,a)}:function(e,t,a){return t<0&&(t=e.length+t),e.substr(t,a)}}).call(this,a("4362"))},e0c5:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"૧",2:"૨",3:"૩",4:"૪",5:"૫",6:"૬",7:"૭",8:"૮",9:"૯",0:"૦"},a={"૧":"1","૨":"2","૩":"3","૪":"4","૫":"5","૬":"6","૭":"7","૮":"8","૯":"9","૦":"0"},n=e.defineLocale("gu",{months:"જાન્યુઆરી_ફેબ્રુઆરી_માર્ચ_એપ્રિલ_મે_જૂન_જુલાઈ_ઑગસ્ટ_સપ્ટેમ્બર_ઑક્ટ્બર_નવેમ્બર_ડિસેમ્બર".split("_"),monthsShort:"જાન્યુ._ફેબ્રુ._માર્ચ_એપ્રિ._મે_જૂન_જુલા._ઑગ._સપ્ટે._ઑક્ટ્._નવે._ડિસે.".split("_"),monthsParseExact:!0,weekdays:"રવિવાર_સોમવાર_મંગળવાર_બુધ્વાર_ગુરુવાર_શુક્રવાર_શનિવાર".split("_"),weekdaysShort:"રવિ_સોમ_મંગળ_બુધ્_ગુરુ_શુક્ર_શનિ".split("_"),weekdaysMin:"ર_સો_મં_બુ_ગુ_શુ_શ".split("_"),longDateFormat:{LT:"A h:mm વાગ્યે",LTS:"A h:mm:ss વાગ્યે",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm વાગ્યે",LLLL:"dddd, D MMMM YYYY, A h:mm વાગ્યે"},calendar:{sameDay:"[આજ] LT",nextDay:"[કાલે] LT",nextWeek:"dddd, LT",lastDay:"[ગઇકાલે] LT",lastWeek:"[પાછલા] dddd, LT",sameElse:"L"},relativeTime:{future:"%s મા",past:"%s પહેલા",s:"અમુક પળો",ss:"%d સેકંડ",m:"એક મિનિટ",mm:"%d મિનિટ",h:"એક કલાક",hh:"%d કલાક",d:"એક દિવસ",dd:"%d દિવસ",M:"એક મહિનો",MM:"%d મહિનો",y:"એક વર્ષ",yy:"%d વર્ષ"},preparse:function(e){return e.replace(/[૧૨૩૪૫૬૭૮૯૦]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/રાત|બપોર|સવાર|સાંજ/,meridiemHour:function(e,t){return 12===e&&(e=0),"રાત"===t?e<4?e:e+12:"સવાર"===t?e:"બપોર"===t?e>=10?e:e+12:"સાંજ"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"રાત":e<10?"સવાર":e<17?"બપોર":e<20?"સાંજ":"રાત"},week:{dow:0,doy:6}});return n}))},e1d3:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-ie",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:1,doy:4}});return t}))},e330:function(e,t,a){var n=a("c6b6"),r=a("7d7e");e.exports=function(e){if("Function"===n(e))return r(e)}},e683:function(e,t,a){"use strict";e.exports=function(e,t){return t?e.replace(/\/+$/,"")+"/"+t.replace(/^\/+/,""):e}},e81d:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"១",2:"២",3:"៣",4:"៤",5:"៥",6:"៦",7:"៧",8:"៨",9:"៩",0:"០"},a={"១":"1","២":"2","៣":"3","៤":"4","៥":"5","៦":"6","៧":"7","៨":"8","៩":"9","០":"0"},n=e.defineLocale("km",{months:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),monthsShort:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),weekdays:"អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍".split("_"),weekdaysShort:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysMin:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ព្រឹក|ល្ងាច/,isPM:function(e){return"ល្ងាច"===e},meridiem:function(e,t,a){return e<12?"ព្រឹក":"ល្ងាច"},calendar:{sameDay:"[ថ្ងៃនេះ ម៉ោង] LT",nextDay:"[ស្អែក ម៉ោង] LT",nextWeek:"dddd [ម៉ោង] LT",lastDay:"[ម្សិលមិញ ម៉ោង] LT",lastWeek:"dddd [សប្តាហ៍មុន] [ម៉ោង] LT",sameElse:"L"},relativeTime:{future:"%sទៀត",past:"%sមុន",s:"ប៉ុន្មានវិនាទី",ss:"%d វិនាទី",m:"មួយនាទី",mm:"%d នាទី",h:"មួយម៉ោង",hh:"%d ម៉ោង",d:"មួយថ្ងៃ",dd:"%d ថ្ងៃ",M:"មួយខែ",MM:"%d ខែ",y:"មួយឆ្នាំ",yy:"%d ឆ្នាំ"},dayOfMonthOrdinalParse:/ទី\d{1,2}/,ordinal:"ទី%d",preparse:function(e){return e.replace(/[១២៣៤៥៦៧៨៩០]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},week:{dow:1,doy:4}});return n}))},e893:function(e,t,a){var n=a("1a2d"),r=a("56ef"),i=a("06cf"),o=a("9bf2");e.exports=function(e,t,a){for(var s=r(t),l=o.f,c=i.f,u=0;u<s.length;u++){var d=s[u];n(e,d)||a&&n(a,d)||l(e,d,c(t,d))}}},e8b5:function(e,t,a){var n=a("c6b6");e.exports=Array.isArray||function(e){return"Array"==n(e)}},ebe4:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ms",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"tengahari"===t?e>=11?e:e+12:"petang"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,a){return e<11?"pagi":e<15?"tengahari":e<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}});return t}))},ec18:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,a,n){var r={s:["mõne sekundi","mõni sekund","paar sekundit"],ss:[e+"sekundi",e+"sekundit"],m:["ühe minuti","üks minut"],mm:[e+" minuti",e+" minutit"],h:["ühe tunni","tund aega","üks tund"],hh:[e+" tunni",e+" tundi"],d:["ühe päeva","üks päev"],M:["kuu aja","kuu aega","üks kuu"],MM:[e+" kuu",e+" kuud"],y:["ühe aasta","aasta","üks aasta"],yy:[e+" aasta",e+" aastat"]};return t?r[a][2]?r[a][2]:r[a][1]:n?r[a][0]:r[a][1]}var a=e.defineLocale("et",{months:"jaanuar_veebruar_märts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember".split("_"),monthsShort:"jaan_veebr_märts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets".split("_"),weekdays:"pühapäev_esmaspäev_teisipäev_kolmapäev_neljapäev_reede_laupäev".split("_"),weekdaysShort:"P_E_T_K_N_R_L".split("_"),weekdaysMin:"P_E_T_K_N_R_L".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[Täna,] LT",nextDay:"[Homme,] LT",nextWeek:"[Järgmine] dddd LT",lastDay:"[Eile,] LT",lastWeek:"[Eelmine] dddd LT",sameElse:"L"},relativeTime:{future:"%s pärast",past:"%s tagasi",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:"%d päeva",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a}))},ec2e:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-in",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:0,doy:6}});return t}))},eda5:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("si",{months:"ජනවාරි_පෙබරවාරි_මාර්තු_අප්‍රේල්_මැයි_ජූනි_ජූලි_අගෝස්තු_සැප්තැම්බර්_ඔක්තෝබර්_නොවැම්බර්_දෙසැම්බර්".split("_"),monthsShort:"ජන_පෙබ_මාර්_අප්_මැයි_ජූනි_ජූලි_අගෝ_සැප්_ඔක්_නොවැ_දෙසැ".split("_"),weekdays:"ඉරිදා_සඳුදා_අඟහරුවාදා_බදාදා_බ්‍රහස්පතින්දා_සිකුරාදා_සෙනසුරාදා".split("_"),weekdaysShort:"ඉරි_සඳු_අඟ_බදා_බ්‍රහ_සිකු_සෙන".split("_"),weekdaysMin:"ඉ_ස_අ_බ_බ්‍ර_සි_සෙ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"a h:mm",LTS:"a h:mm:ss",L:"YYYY/MM/DD",LL:"YYYY MMMM D",LLL:"YYYY MMMM D, a h:mm",LLLL:"YYYY MMMM D [වැනි] dddd, a h:mm:ss"},calendar:{sameDay:"[අද] LT[ට]",nextDay:"[හෙට] LT[ට]",nextWeek:"dddd LT[ට]",lastDay:"[ඊයේ] LT[ට]",lastWeek:"[පසුගිය] dddd LT[ට]",sameElse:"L"},relativeTime:{future:"%sකින්",past:"%sකට පෙර",s:"තත්පර කිහිපය",ss:"තත්පර %d",m:"මිනිත්තුව",mm:"මිනිත්තු %d",h:"පැය",hh:"පැය %d",d:"දිනය",dd:"දින %d",M:"මාසය",MM:"මාස %d",y:"වසර",yy:"වසර %d"},dayOfMonthOrdinalParse:/\d{1,2} වැනි/,ordinal:function(e){return e+" වැනි"},meridiemParse:/පෙර වරු|පස් වරු|පෙ.ව|ප.ව./,isPM:function(e){return"ප.ව."===e||"පස් වරු"===e},meridiem:function(e,t,a){return e>11?a?"ප.ව.":"පස් වරු":a?"පෙ.ව.":"පෙර වරු"}});return t}))},f0bd:function(e,t,a){"use strict";(function(e){
/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.16.1
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var a="undefined"!==typeof window&&"undefined"!==typeof document&&"undefined"!==typeof navigator,n=function(){for(var e=["Edge","Trident","Firefox"],t=0;t<e.length;t+=1)if(a&&navigator.userAgent.indexOf(e[t])>=0)return 1;return 0}();function r(e){var t=!1;return function(){t||(t=!0,window.Promise.resolve().then((function(){t=!1,e()})))}}function i(e){var t=!1;return function(){t||(t=!0,setTimeout((function(){t=!1,e()}),n))}}var o=a&&window.Promise,s=o?r:i;function l(e){var t={};return e&&"[object Function]"===t.toString.call(e)}function c(e,t){if(1!==e.nodeType)return[];var a=e.ownerDocument.defaultView,n=a.getComputedStyle(e,null);return t?n[t]:n}function u(e){return"HTML"===e.nodeName?e:e.parentNode||e.host}function d(e){if(!e)return document.body;switch(e.nodeName){case"HTML":case"BODY":return e.ownerDocument.body;case"#document":return e.body}var t=c(e),a=t.overflow,n=t.overflowX,r=t.overflowY;return/(auto|scroll|overlay)/.test(a+r+n)?e:d(u(e))}function h(e){return e&&e.referenceNode?e.referenceNode:e}var f=a&&!(!window.MSInputMethodContext||!document.documentMode),p=a&&/MSIE 10/.test(navigator.userAgent);function m(e){return 11===e?f:10===e?p:f||p}function v(e){if(!e)return document.documentElement;var t=m(10)?document.body:null,a=e.offsetParent||null;while(a===t&&e.nextElementSibling)a=(e=e.nextElementSibling).offsetParent;var n=a&&a.nodeName;return n&&"BODY"!==n&&"HTML"!==n?-1!==["TH","TD","TABLE"].indexOf(a.nodeName)&&"static"===c(a,"position")?v(a):a:e?e.ownerDocument.documentElement:document.documentElement}function _(e){var t=e.nodeName;return"BODY"!==t&&("HTML"===t||v(e.firstElementChild)===e)}function g(e){return null!==e.parentNode?g(e.parentNode):e}function y(e,t){if(!e||!e.nodeType||!t||!t.nodeType)return document.documentElement;var a=e.compareDocumentPosition(t)&Node.DOCUMENT_POSITION_FOLLOWING,n=a?e:t,r=a?t:e,i=document.createRange();i.setStart(n,0),i.setEnd(r,0);var o=i.commonAncestorContainer;if(e!==o&&t!==o||n.contains(r))return _(o)?o:v(o);var s=g(e);return s.host?y(s.host,t):y(e,g(t).host)}function b(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"top",a="top"===t?"scrollTop":"scrollLeft",n=e.nodeName;if("BODY"===n||"HTML"===n){var r=e.ownerDocument.documentElement,i=e.ownerDocument.scrollingElement||r;return i[a]}return e[a]}function M(e,t){var a=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=b(t,"top"),r=b(t,"left"),i=a?-1:1;return e.top+=n*i,e.bottom+=n*i,e.left+=r*i,e.right+=r*i,e}function z(e,t){var a="x"===t?"Left":"Top",n="Left"===a?"Right":"Bottom";return parseFloat(e["border"+a+"Width"])+parseFloat(e["border"+n+"Width"])}function L(e,t,a,n){return Math.max(t["offset"+e],t["scroll"+e],a["client"+e],a["offset"+e],a["scroll"+e],m(10)?parseInt(a["offset"+e])+parseInt(n["margin"+("Height"===e?"Top":"Left")])+parseInt(n["margin"+("Height"===e?"Bottom":"Right")]):0)}function w(e){var t=e.body,a=e.documentElement,n=m(10)&&getComputedStyle(a);return{height:L("Height",t,a,n),width:L("Width",t,a,n)}}var H=function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")},k=function(){function e(e,t){for(var a=0;a<t.length;a++){var n=t[a];n.enumerable=n.enumerable||!1,n.configurable=!0,"value"in n&&(n.writable=!0),Object.defineProperty(e,n.key,n)}}return function(t,a,n){return a&&e(t.prototype,a),n&&e(t,n),t}}(),A=function(e,t,a){return t in e?Object.defineProperty(e,t,{value:a,enumerable:!0,configurable:!0,writable:!0}):e[t]=a,e},S=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var a=arguments[t];for(var n in a)Object.prototype.hasOwnProperty.call(a,n)&&(e[n]=a[n])}return e};function V(e){return S({},e,{right:e.left+e.width,bottom:e.top+e.height})}function O(e){var t={};try{if(m(10)){t=e.getBoundingClientRect();var a=b(e,"top"),n=b(e,"left");t.top+=a,t.left+=n,t.bottom+=a,t.right+=n}else t=e.getBoundingClientRect()}catch(h){}var r={left:t.left,top:t.top,width:t.right-t.left,height:t.bottom-t.top},i="HTML"===e.nodeName?w(e.ownerDocument):{},o=i.width||e.clientWidth||r.width,s=i.height||e.clientHeight||r.height,l=e.offsetWidth-o,u=e.offsetHeight-s;if(l||u){var d=c(e);l-=z(d,"x"),u-=z(d,"y"),r.width-=l,r.height-=u}return V(r)}function T(e,t){var a=arguments.length>2&&void 0!==arguments[2]&&arguments[2],n=m(10),r="HTML"===t.nodeName,i=O(e),o=O(t),s=d(e),l=c(t),u=parseFloat(l.borderTopWidth),h=parseFloat(l.borderLeftWidth);a&&r&&(o.top=Math.max(o.top,0),o.left=Math.max(o.left,0));var f=V({top:i.top-o.top-u,left:i.left-o.left-h,width:i.width,height:i.height});if(f.marginTop=0,f.marginLeft=0,!n&&r){var p=parseFloat(l.marginTop),v=parseFloat(l.marginLeft);f.top-=u-p,f.bottom-=u-p,f.left-=h-v,f.right-=h-v,f.marginTop=p,f.marginLeft=v}return(n&&!a?t.contains(s):t===s&&"BODY"!==s.nodeName)&&(f=M(f,t)),f}function D(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],a=e.ownerDocument.documentElement,n=T(e,a),r=Math.max(a.clientWidth,window.innerWidth||0),i=Math.max(a.clientHeight,window.innerHeight||0),o=t?0:b(a),s=t?0:b(a,"left"),l={top:o-n.top+n.marginTop,left:s-n.left+n.marginLeft,width:r,height:i};return V(l)}function C(e){var t=e.nodeName;if("BODY"===t||"HTML"===t)return!1;if("fixed"===c(e,"position"))return!0;var a=u(e);return!!a&&C(a)}function j(e){if(!e||!e.parentElement||m())return document.documentElement;var t=e.parentElement;while(t&&"none"===c(t,"transform"))t=t.parentElement;return t||document.documentElement}function x(e,t,a,n){var r=arguments.length>4&&void 0!==arguments[4]&&arguments[4],i={top:0,left:0},o=r?j(e):y(e,h(t));if("viewport"===n)i=D(o,r);else{var s=void 0;"scrollParent"===n?(s=d(u(t)),"BODY"===s.nodeName&&(s=e.ownerDocument.documentElement)):s="window"===n?e.ownerDocument.documentElement:n;var l=T(s,o,r);if("HTML"!==s.nodeName||C(o))i=l;else{var c=w(e.ownerDocument),f=c.height,p=c.width;i.top+=l.top-l.marginTop,i.bottom=f+l.top,i.left+=l.left-l.marginLeft,i.right=p+l.left}}a=a||0;var m="number"===typeof a;return i.left+=m?a:a.left||0,i.top+=m?a:a.top||0,i.right-=m?a:a.right||0,i.bottom-=m?a:a.bottom||0,i}function B(e){var t=e.width,a=e.height;return t*a}function Y(e,t,a,n,r){var i=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0;if(-1===e.indexOf("auto"))return e;var o=x(a,n,i,r),s={top:{width:o.width,height:t.top-o.top},right:{width:o.right-t.right,height:o.height},bottom:{width:o.width,height:o.bottom-t.bottom},left:{width:t.left-o.left,height:o.height}},l=Object.keys(s).map((function(e){return S({key:e},s[e],{area:B(s[e])})})).sort((function(e,t){return t.area-e.area})),c=l.filter((function(e){var t=e.width,n=e.height;return t>=a.clientWidth&&n>=a.clientHeight})),u=c.length>0?c[0].key:l[0].key,d=e.split("-")[1];return u+(d?"-"+d:"")}function P(e,t,a){var n=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null,r=n?j(t):y(t,h(a));return T(a,r,n)}function I(e){var t=e.ownerDocument.defaultView,a=t.getComputedStyle(e),n=parseFloat(a.marginTop||0)+parseFloat(a.marginBottom||0),r=parseFloat(a.marginLeft||0)+parseFloat(a.marginRight||0),i={width:e.offsetWidth+r,height:e.offsetHeight+n};return i}function F(e){var t={left:"right",right:"left",bottom:"top",top:"bottom"};return e.replace(/left|right|bottom|top/g,(function(e){return t[e]}))}function E(e,t,a){a=a.split("-")[0];var n=I(e),r={width:n.width,height:n.height},i=-1!==["right","left"].indexOf(a),o=i?"top":"left",s=i?"left":"top",l=i?"height":"width",c=i?"width":"height";return r[o]=t[o]+t[l]/2-n[l]/2,r[s]=a===s?t[s]-n[c]:t[F(s)],r}function $(e,t){return Array.prototype.find?e.find(t):e.filter(t)[0]}function R(e,t,a){if(Array.prototype.findIndex)return e.findIndex((function(e){return e[t]===a}));var n=$(e,(function(e){return e[t]===a}));return e.indexOf(n)}function N(e,t,a){var n=void 0===a?e:e.slice(0,R(e,"name",a));return n.forEach((function(e){e["function"]&&console.warn("`modifier.function` is deprecated, use `modifier.fn`!");var a=e["function"]||e.fn;e.enabled&&l(a)&&(t.offsets.popper=V(t.offsets.popper),t.offsets.reference=V(t.offsets.reference),t=a(t,e))})),t}function W(){if(!this.state.isDestroyed){var e={instance:this,styles:{},arrowStyles:{},attributes:{},flipped:!1,offsets:{}};e.offsets.reference=P(this.state,this.popper,this.reference,this.options.positionFixed),e.placement=Y(this.options.placement,e.offsets.reference,this.popper,this.reference,this.options.modifiers.flip.boundariesElement,this.options.modifiers.flip.padding),e.originalPlacement=e.placement,e.positionFixed=this.options.positionFixed,e.offsets.popper=E(this.popper,e.offsets.reference,e.placement),e.offsets.popper.position=this.options.positionFixed?"fixed":"absolute",e=N(this.modifiers,e),this.state.isCreated?this.options.onUpdate(e):(this.state.isCreated=!0,this.options.onCreate(e))}}function U(e,t){return e.some((function(e){var a=e.name,n=e.enabled;return n&&a===t}))}function G(e){for(var t=[!1,"ms","Webkit","Moz","O"],a=e.charAt(0).toUpperCase()+e.slice(1),n=0;n<t.length;n++){var r=t[n],i=r?""+r+a:e;if("undefined"!==typeof document.body.style[i])return i}return null}function q(){return this.state.isDestroyed=!0,U(this.modifiers,"applyStyle")&&(this.popper.removeAttribute("x-placement"),this.popper.style.position="",this.popper.style.top="",this.popper.style.left="",this.popper.style.right="",this.popper.style.bottom="",this.popper.style.willChange="",this.popper.style[G("transform")]=""),this.disableEventListeners(),this.options.removeOnDestroy&&this.popper.parentNode.removeChild(this.popper),this}function J(e){var t=e.ownerDocument;return t?t.defaultView:window}function K(e,t,a,n){var r="BODY"===e.nodeName,i=r?e.ownerDocument.defaultView:e;i.addEventListener(t,a,{passive:!0}),r||K(d(i.parentNode),t,a,n),n.push(i)}function X(e,t,a,n){a.updateBound=n,J(e).addEventListener("resize",a.updateBound,{passive:!0});var r=d(e);return K(r,"scroll",a.updateBound,a.scrollParents),a.scrollElement=r,a.eventsEnabled=!0,a}function Z(){this.state.eventsEnabled||(this.state=X(this.reference,this.options,this.state,this.scheduleUpdate))}function Q(e,t){return J(e).removeEventListener("resize",t.updateBound),t.scrollParents.forEach((function(e){e.removeEventListener("scroll",t.updateBound)})),t.updateBound=null,t.scrollParents=[],t.scrollElement=null,t.eventsEnabled=!1,t}function ee(){this.state.eventsEnabled&&(cancelAnimationFrame(this.scheduleUpdate),this.state=Q(this.reference,this.state))}function te(e){return""!==e&&!isNaN(parseFloat(e))&&isFinite(e)}function ae(e,t){Object.keys(t).forEach((function(a){var n="";-1!==["width","height","top","right","bottom","left"].indexOf(a)&&te(t[a])&&(n="px"),e.style[a]=t[a]+n}))}function ne(e,t){Object.keys(t).forEach((function(a){var n=t[a];!1!==n?e.setAttribute(a,t[a]):e.removeAttribute(a)}))}function re(e){return ae(e.instance.popper,e.styles),ne(e.instance.popper,e.attributes),e.arrowElement&&Object.keys(e.arrowStyles).length&&ae(e.arrowElement,e.arrowStyles),e}function ie(e,t,a,n,r){var i=P(r,t,e,a.positionFixed),o=Y(a.placement,i,t,e,a.modifiers.flip.boundariesElement,a.modifiers.flip.padding);return t.setAttribute("x-placement",o),ae(t,{position:a.positionFixed?"fixed":"absolute"}),a}function oe(e,t){var a=e.offsets,n=a.popper,r=a.reference,i=Math.round,o=Math.floor,s=function(e){return e},l=i(r.width),c=i(n.width),u=-1!==["left","right"].indexOf(e.placement),d=-1!==e.placement.indexOf("-"),h=l%2===c%2,f=l%2===1&&c%2===1,p=t?u||d||h?i:o:s,m=t?i:s;return{left:p(f&&!d&&t?n.left-1:n.left),top:m(n.top),bottom:m(n.bottom),right:p(n.right)}}var se=a&&/Firefox/i.test(navigator.userAgent);function le(e,t){var a=t.x,n=t.y,r=e.offsets.popper,i=$(e.instance.modifiers,(function(e){return"applyStyle"===e.name})).gpuAcceleration;void 0!==i&&console.warn("WARNING: `gpuAcceleration` option moved to `computeStyle` modifier and will not be supported in future versions of Popper.js!");var o=void 0!==i?i:t.gpuAcceleration,s=v(e.instance.popper),l=O(s),c={position:r.position},u=oe(e,window.devicePixelRatio<2||!se),d="bottom"===a?"top":"bottom",h="right"===n?"left":"right",f=G("transform"),p=void 0,m=void 0;if(m="bottom"===d?"HTML"===s.nodeName?-s.clientHeight+u.bottom:-l.height+u.bottom:u.top,p="right"===h?"HTML"===s.nodeName?-s.clientWidth+u.right:-l.width+u.right:u.left,o&&f)c[f]="translate3d("+p+"px, "+m+"px, 0)",c[d]=0,c[h]=0,c.willChange="transform";else{var _="bottom"===d?-1:1,g="right"===h?-1:1;c[d]=m*_,c[h]=p*g,c.willChange=d+", "+h}var y={"x-placement":e.placement};return e.attributes=S({},y,e.attributes),e.styles=S({},c,e.styles),e.arrowStyles=S({},e.offsets.arrow,e.arrowStyles),e}function ce(e,t,a){var n=$(e,(function(e){var a=e.name;return a===t})),r=!!n&&e.some((function(e){return e.name===a&&e.enabled&&e.order<n.order}));if(!r){var i="`"+t+"`",o="`"+a+"`";console.warn(o+" modifier is required by "+i+" modifier in order to work, be sure to include it before "+i+"!")}return r}function ue(e,t){var a;if(!ce(e.instance.modifiers,"arrow","keepTogether"))return e;var n=t.element;if("string"===typeof n){if(n=e.instance.popper.querySelector(n),!n)return e}else if(!e.instance.popper.contains(n))return console.warn("WARNING: `arrow.element` must be child of its popper element!"),e;var r=e.placement.split("-")[0],i=e.offsets,o=i.popper,s=i.reference,l=-1!==["left","right"].indexOf(r),u=l?"height":"width",d=l?"Top":"Left",h=d.toLowerCase(),f=l?"left":"top",p=l?"bottom":"right",m=I(n)[u];s[p]-m<o[h]&&(e.offsets.popper[h]-=o[h]-(s[p]-m)),s[h]+m>o[p]&&(e.offsets.popper[h]+=s[h]+m-o[p]),e.offsets.popper=V(e.offsets.popper);var v=s[h]+s[u]/2-m/2,_=c(e.instance.popper),g=parseFloat(_["margin"+d]),y=parseFloat(_["border"+d+"Width"]),b=v-e.offsets.popper[h]-g-y;return b=Math.max(Math.min(o[u]-m,b),0),e.arrowElement=n,e.offsets.arrow=(a={},A(a,h,Math.round(b)),A(a,f,""),a),e}function de(e){return"end"===e?"start":"start"===e?"end":e}var he=["auto-start","auto","auto-end","top-start","top","top-end","right-start","right","right-end","bottom-end","bottom","bottom-start","left-end","left","left-start"],fe=he.slice(3);function pe(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],a=fe.indexOf(e),n=fe.slice(a+1).concat(fe.slice(0,a));return t?n.reverse():n}var me={FLIP:"flip",CLOCKWISE:"clockwise",COUNTERCLOCKWISE:"counterclockwise"};function ve(e,t){if(U(e.instance.modifiers,"inner"))return e;if(e.flipped&&e.placement===e.originalPlacement)return e;var a=x(e.instance.popper,e.instance.reference,t.padding,t.boundariesElement,e.positionFixed),n=e.placement.split("-")[0],r=F(n),i=e.placement.split("-")[1]||"",o=[];switch(t.behavior){case me.FLIP:o=[n,r];break;case me.CLOCKWISE:o=pe(n);break;case me.COUNTERCLOCKWISE:o=pe(n,!0);break;default:o=t.behavior}return o.forEach((function(s,l){if(n!==s||o.length===l+1)return e;n=e.placement.split("-")[0],r=F(n);var c=e.offsets.popper,u=e.offsets.reference,d=Math.floor,h="left"===n&&d(c.right)>d(u.left)||"right"===n&&d(c.left)<d(u.right)||"top"===n&&d(c.bottom)>d(u.top)||"bottom"===n&&d(c.top)<d(u.bottom),f=d(c.left)<d(a.left),p=d(c.right)>d(a.right),m=d(c.top)<d(a.top),v=d(c.bottom)>d(a.bottom),_="left"===n&&f||"right"===n&&p||"top"===n&&m||"bottom"===n&&v,g=-1!==["top","bottom"].indexOf(n),y=!!t.flipVariations&&(g&&"start"===i&&f||g&&"end"===i&&p||!g&&"start"===i&&m||!g&&"end"===i&&v),b=!!t.flipVariationsByContent&&(g&&"start"===i&&p||g&&"end"===i&&f||!g&&"start"===i&&v||!g&&"end"===i&&m),M=y||b;(h||_||M)&&(e.flipped=!0,(h||_)&&(n=o[l+1]),M&&(i=de(i)),e.placement=n+(i?"-"+i:""),e.offsets.popper=S({},e.offsets.popper,E(e.instance.popper,e.offsets.reference,e.placement)),e=N(e.instance.modifiers,e,"flip"))})),e}function _e(e){var t=e.offsets,a=t.popper,n=t.reference,r=e.placement.split("-")[0],i=Math.floor,o=-1!==["top","bottom"].indexOf(r),s=o?"right":"bottom",l=o?"left":"top",c=o?"width":"height";return a[s]<i(n[l])&&(e.offsets.popper[l]=i(n[l])-a[c]),a[l]>i(n[s])&&(e.offsets.popper[l]=i(n[s])),e}function ge(e,t,a,n){var r=e.match(/((?:\-|\+)?\d*\.?\d*)(.*)/),i=+r[1],o=r[2];if(!i)return e;if(0===o.indexOf("%")){var s=void 0;switch(o){case"%p":s=a;break;case"%":case"%r":default:s=n}var l=V(s);return l[t]/100*i}if("vh"===o||"vw"===o){var c=void 0;return c="vh"===o?Math.max(document.documentElement.clientHeight,window.innerHeight||0):Math.max(document.documentElement.clientWidth,window.innerWidth||0),c/100*i}return i}function ye(e,t,a,n){var r=[0,0],i=-1!==["right","left"].indexOf(n),o=e.split(/(\+|\-)/).map((function(e){return e.trim()})),s=o.indexOf($(o,(function(e){return-1!==e.search(/,|\s/)})));o[s]&&-1===o[s].indexOf(",")&&console.warn("Offsets separated by white space(s) are deprecated, use a comma (,) instead.");var l=/\s*,\s*|\s+/,c=-1!==s?[o.slice(0,s).concat([o[s].split(l)[0]]),[o[s].split(l)[1]].concat(o.slice(s+1))]:[o];return c=c.map((function(e,n){var r=(1===n?!i:i)?"height":"width",o=!1;return e.reduce((function(e,t){return""===e[e.length-1]&&-1!==["+","-"].indexOf(t)?(e[e.length-1]=t,o=!0,e):o?(e[e.length-1]+=t,o=!1,e):e.concat(t)}),[]).map((function(e){return ge(e,r,t,a)}))})),c.forEach((function(e,t){e.forEach((function(a,n){te(a)&&(r[t]+=a*("-"===e[n-1]?-1:1))}))})),r}function be(e,t){var a=t.offset,n=e.placement,r=e.offsets,i=r.popper,o=r.reference,s=n.split("-")[0],l=void 0;return l=te(+a)?[+a,0]:ye(a,i,o,s),"left"===s?(i.top+=l[0],i.left-=l[1]):"right"===s?(i.top+=l[0],i.left+=l[1]):"top"===s?(i.left+=l[0],i.top-=l[1]):"bottom"===s&&(i.left+=l[0],i.top+=l[1]),e.popper=i,e}function Me(e,t){var a=t.boundariesElement||v(e.instance.popper);e.instance.reference===a&&(a=v(a));var n=G("transform"),r=e.instance.popper.style,i=r.top,o=r.left,s=r[n];r.top="",r.left="",r[n]="";var l=x(e.instance.popper,e.instance.reference,t.padding,a,e.positionFixed);r.top=i,r.left=o,r[n]=s,t.boundaries=l;var c=t.priority,u=e.offsets.popper,d={primary:function(e){var a=u[e];return u[e]<l[e]&&!t.escapeWithReference&&(a=Math.max(u[e],l[e])),A({},e,a)},secondary:function(e){var a="right"===e?"left":"top",n=u[a];return u[e]>l[e]&&!t.escapeWithReference&&(n=Math.min(u[a],l[e]-("right"===e?u.width:u.height))),A({},a,n)}};return c.forEach((function(e){var t=-1!==["left","top"].indexOf(e)?"primary":"secondary";u=S({},u,d[t](e))})),e.offsets.popper=u,e}function ze(e){var t=e.placement,a=t.split("-")[0],n=t.split("-")[1];if(n){var r=e.offsets,i=r.reference,o=r.popper,s=-1!==["bottom","top"].indexOf(a),l=s?"left":"top",c=s?"width":"height",u={start:A({},l,i[l]),end:A({},l,i[l]+i[c]-o[c])};e.offsets.popper=S({},o,u[n])}return e}function Le(e){if(!ce(e.instance.modifiers,"hide","preventOverflow"))return e;var t=e.offsets.reference,a=$(e.instance.modifiers,(function(e){return"preventOverflow"===e.name})).boundaries;if(t.bottom<a.top||t.left>a.right||t.top>a.bottom||t.right<a.left){if(!0===e.hide)return e;e.hide=!0,e.attributes["x-out-of-boundaries"]=""}else{if(!1===e.hide)return e;e.hide=!1,e.attributes["x-out-of-boundaries"]=!1}return e}function we(e){var t=e.placement,a=t.split("-")[0],n=e.offsets,r=n.popper,i=n.reference,o=-1!==["left","right"].indexOf(a),s=-1===["top","left"].indexOf(a);return r[o?"left":"top"]=i[a]-(s?r[o?"width":"height"]:0),e.placement=F(t),e.offsets.popper=V(r),e}var He={shift:{order:100,enabled:!0,fn:ze},offset:{order:200,enabled:!0,fn:be,offset:0},preventOverflow:{order:300,enabled:!0,fn:Me,priority:["left","right","top","bottom"],padding:5,boundariesElement:"scrollParent"},keepTogether:{order:400,enabled:!0,fn:_e},arrow:{order:500,enabled:!0,fn:ue,element:"[x-arrow]"},flip:{order:600,enabled:!0,fn:ve,behavior:"flip",padding:5,boundariesElement:"viewport",flipVariations:!1,flipVariationsByContent:!1},inner:{order:700,enabled:!1,fn:we},hide:{order:800,enabled:!0,fn:Le},computeStyle:{order:850,enabled:!0,fn:le,gpuAcceleration:!0,x:"bottom",y:"right"},applyStyle:{order:900,enabled:!0,fn:re,onLoad:ie,gpuAcceleration:void 0}},ke={placement:"bottom",positionFixed:!1,eventsEnabled:!0,removeOnDestroy:!1,onCreate:function(){},onUpdate:function(){},modifiers:He},Ae=function(){function e(t,a){var n=this,r=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};H(this,e),this.scheduleUpdate=function(){return requestAnimationFrame(n.update)},this.update=s(this.update.bind(this)),this.options=S({},e.Defaults,r),this.state={isDestroyed:!1,isCreated:!1,scrollParents:[]},this.reference=t&&t.jquery?t[0]:t,this.popper=a&&a.jquery?a[0]:a,this.options.modifiers={},Object.keys(S({},e.Defaults.modifiers,r.modifiers)).forEach((function(t){n.options.modifiers[t]=S({},e.Defaults.modifiers[t]||{},r.modifiers?r.modifiers[t]:{})})),this.modifiers=Object.keys(this.options.modifiers).map((function(e){return S({name:e},n.options.modifiers[e])})).sort((function(e,t){return e.order-t.order})),this.modifiers.forEach((function(e){e.enabled&&l(e.onLoad)&&e.onLoad(n.reference,n.popper,n.options,e,n.state)})),this.update();var i=this.options.eventsEnabled;i&&this.enableEventListeners(),this.state.eventsEnabled=i}return k(e,[{key:"update",value:function(){return W.call(this)}},{key:"destroy",value:function(){return q.call(this)}},{key:"enableEventListeners",value:function(){return Z.call(this)}},{key:"disableEventListeners",value:function(){return ee.call(this)}}]),e}();Ae.Utils=("undefined"!==typeof window?window:e).PopperUtils,Ae.placements=he,Ae.Defaults=ke,t["a"]=Ae}).call(this,a("c8ba"))},f260:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("pt",{months:"janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"Domingo_Segunda-feira_Terça-feira_Quarta-feira_Quinta-feira_Sexta-feira_Sábado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_Sáb".split("_"),weekdaysMin:"Do_2ª_3ª_4ª_5ª_6ª_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",w:"uma semana",ww:"%d semanas",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},f3ff:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"੧",2:"੨",3:"੩",4:"੪",5:"੫",6:"੬",7:"੭",8:"੮",9:"੯",0:"੦"},a={"੧":"1","੨":"2","੩":"3","੪":"4","੫":"5","੬":"6","੭":"7","੮":"8","੯":"9","੦":"0"},n=e.defineLocale("pa-in",{months:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),monthsShort:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),weekdays:"ਐਤਵਾਰ_ਸੋਮਵਾਰ_ਮੰਗਲਵਾਰ_ਬੁਧਵਾਰ_ਵੀਰਵਾਰ_ਸ਼ੁੱਕਰਵਾਰ_ਸ਼ਨੀਚਰਵਾਰ".split("_"),weekdaysShort:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),weekdaysMin:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),longDateFormat:{LT:"A h:mm ਵਜੇ",LTS:"A h:mm:ss ਵਜੇ",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm ਵਜੇ",LLLL:"dddd, D MMMM YYYY, A h:mm ਵਜੇ"},calendar:{sameDay:"[ਅਜ] LT",nextDay:"[ਕਲ] LT",nextWeek:"[ਅਗਲਾ] dddd, LT",lastDay:"[ਕਲ] LT",lastWeek:"[ਪਿਛਲੇ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ਵਿੱਚ",past:"%s ਪਿਛਲੇ",s:"ਕੁਝ ਸਕਿੰਟ",ss:"%d ਸਕਿੰਟ",m:"ਇਕ ਮਿੰਟ",mm:"%d ਮਿੰਟ",h:"ਇੱਕ ਘੰਟਾ",hh:"%d ਘੰਟੇ",d:"ਇੱਕ ਦਿਨ",dd:"%d ਦਿਨ",M:"ਇੱਕ ਮਹੀਨਾ",MM:"%d ਮਹੀਨੇ",y:"ਇੱਕ ਸਾਲ",yy:"%d ਸਾਲ"},preparse:function(e){return e.replace(/[੧੨੩੪੫੬੭੮੯੦]/g,(function(e){return a[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/ਰਾਤ|ਸਵੇਰ|ਦੁਪਹਿਰ|ਸ਼ਾਮ/,meridiemHour:function(e,t){return 12===e&&(e=0),"ਰਾਤ"===t?e<4?e:e+12:"ਸਵੇਰ"===t?e:"ਦੁਪਹਿਰ"===t?e>=10?e:e+12:"ਸ਼ਾਮ"===t?e+12:void 0},meridiem:function(e,t,a){return e<4?"ਰਾਤ":e<10?"ਸਵੇਰ":e<17?"ਦੁਪਹਿਰ":e<20?"ਸ਼ਾਮ":"ਰਾਤ"},week:{dow:0,doy:6}});return n}))},f6b4:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["Am Faoilleach","An Gearran","Am Màrt","An Giblean","An Cèitean","An t-Ògmhios","An t-Iuchar","An Lùnastal","An t-Sultain","An Dàmhair","An t-Samhain","An Dùbhlachd"],a=["Faoi","Gear","Màrt","Gibl","Cèit","Ògmh","Iuch","Lùn","Sult","Dàmh","Samh","Dùbh"],n=["Didòmhnaich","Diluain","Dimàirt","Diciadain","Diardaoin","Dihaoine","Disathairne"],r=["Did","Dil","Dim","Dic","Dia","Dih","Dis"],i=["Dò","Lu","Mà","Ci","Ar","Ha","Sa"],o=e.defineLocale("gd",{months:t,monthsShort:a,monthsParseExact:!0,weekdays:n,weekdaysShort:r,weekdaysMin:i,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[An-diugh aig] LT",nextDay:"[A-màireach aig] LT",nextWeek:"dddd [aig] LT",lastDay:"[An-dè aig] LT",lastWeek:"dddd [seo chaidh] [aig] LT",sameElse:"L"},relativeTime:{future:"ann an %s",past:"bho chionn %s",s:"beagan diogan",ss:"%d diogan",m:"mionaid",mm:"%d mionaidean",h:"uair",hh:"%d uairean",d:"latha",dd:"%d latha",M:"mìos",MM:"%d mìosan",y:"bliadhna",yy:"%d bliadhna"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(e){var t=1===e?"d":e%10===2?"na":"mh";return e+t},week:{dow:1,doy:4}});return o}))},f6b49:function(e,t,a){"use strict";var n=a("c532");function r(){this.handlers=[]}r.prototype.use=function(e,t,a){return this.handlers.push({fulfilled:e,rejected:t,synchronous:!!a&&a.synchronous,runWhen:a?a.runWhen:null}),this.handlers.length-1},r.prototype.eject=function(e){this.handlers[e]&&(this.handlers[e]=null)},r.prototype.forEach=function(e){n.forEach(this.handlers,(function(t){null!==t&&e(t)}))},e.exports=r},f772:function(e,t,a){var n=a("5692"),r=a("90e3"),i=n("keys");e.exports=function(e){return i[e]||(i[e]=r(e))}},f9e3:function(e,t,a){},facd:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),a="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),n=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],r=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,i=e.defineLocale("nl",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(e,n){return e?/-MMM-/.test(n)?a[e.month()]:t[e.month()]:t},monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",w:"één week",ww:"%d weken",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return i}))},fc6a:function(e,t,a){var n=a("44ad"),r=a("1d80");e.exports=function(e){return n(r(e))}},fd7e:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("x-pseudo",{months:"J~áñúá~rý_F~ébrú~árý_~Márc~h_Áp~ríl_~Máý_~Júñé~_Júl~ý_Áú~gúst~_Sép~témb~ér_Ó~ctób~ér_Ñ~óvém~bér_~Décé~mbér".split("_"),monthsShort:"J~áñ_~Féb_~Már_~Ápr_~Máý_~Júñ_~Júl_~Áúg_~Sép_~Óct_~Ñóv_~Déc".split("_"),monthsParseExact:!0,weekdays:"S~úñdá~ý_Mó~ñdáý~_Túé~sdáý~_Wéd~ñésd~áý_T~húrs~dáý_~Fríd~áý_S~átúr~dáý".split("_"),weekdaysShort:"S~úñ_~Móñ_~Túé_~Wéd_~Thú_~Frí_~Sát".split("_"),weekdaysMin:"S~ú_Mó~_Tú_~Wé_T~h_Fr~_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[T~ódá~ý át] LT",nextDay:"[T~ómó~rró~w át] LT",nextWeek:"dddd [át] LT",lastDay:"[Ý~ést~érdá~ý át] LT",lastWeek:"[L~ást] dddd [át] LT",sameElse:"L"},relativeTime:{future:"í~ñ %s",past:"%s á~gó",s:"á ~féw ~sécó~ñds",ss:"%d s~écóñ~ds",m:"á ~míñ~úté",mm:"%d m~íñú~tés",h:"á~ñ hó~úr",hh:"%d h~óúrs",d:"á ~dáý",dd:"%d d~áýs",M:"á ~móñ~th",MM:"%d m~óñt~hs",y:"á ~ýéár",yy:"%d ý~éárs"},dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10,a=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+a},week:{dow:1,doy:4}});return t}))},fdbf:function(e,t,a){var n=a("04f8");e.exports=n&&!Symbol.sham&&"symbol"==typeof Symbol.iterator},ffff:function(e,t,a){(function(e,t){t(a("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("se",{months:"ođđajagemánnu_guovvamánnu_njukčamánnu_cuoŋománnu_miessemánnu_geassemánnu_suoidnemánnu_borgemánnu_čakčamánnu_golggotmánnu_skábmamánnu_juovlamánnu".split("_"),monthsShort:"ođđj_guov_njuk_cuo_mies_geas_suoi_borg_čakč_golg_skáb_juov".split("_"),weekdays:"sotnabeaivi_vuossárga_maŋŋebárga_gaskavahkku_duorastat_bearjadat_lávvardat".split("_"),weekdaysShort:"sotn_vuos_maŋ_gask_duor_bear_láv".split("_"),weekdaysMin:"s_v_m_g_d_b_L".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"MMMM D. [b.] YYYY",LLL:"MMMM D. [b.] YYYY [ti.] HH:mm",LLLL:"dddd, MMMM D. [b.] YYYY [ti.] HH:mm"},calendar:{sameDay:"[otne ti] LT",nextDay:"[ihttin ti] LT",nextWeek:"dddd [ti] LT",lastDay:"[ikte ti] LT",lastWeek:"[ovddit] dddd [ti] LT",sameElse:"L"},relativeTime:{future:"%s geažes",past:"maŋit %s",s:"moadde sekunddat",ss:"%d sekunddat",m:"okta minuhta",mm:"%d minuhtat",h:"okta diimmu",hh:"%d diimmut",d:"okta beaivi",dd:"%d beaivvit",M:"okta mánnu",MM:"%d mánut",y:"okta jahki",yy:"%d jagit"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))}});